#!/usr/bin/env node

/*
#-
# Copyright (c) 2012 Ryan Kelker and individual contributors.
# ( https://github.com/runexec/monorail.js ) Monorail.js
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer
# in this position and unchanged.
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
# derived from this software withough specific prior written permission
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES# LOSS OF USE,
# DATA, OR PROFITS# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
var fs = require('fs');
var rl = require('readline');
var crypto = require('crypto');
var https = require('https');
var zip = require('./lib/node-native-zip');
var argv = process.argv;
var routine = argv[2];
var help = [
	'install [mootools | rightjs | jquery | dojo | prototype ] ; Install JS Framework in this project', 
	'start server ; Start project Redis & Express server',
	'reset project ; Removes files from static, models, and views',
	'summary ; Returns a project summary',
	'hashtree ; Return a hash tree of the current project',
	'snapshot [ create | clean ] ; backup your models, views, and routes',
	'new project [project_name] ; Creates project',
	'new page [page_name] ; Creates new project page',
	'new view [view_name] ; Creates view w/ no model',
	'new model [model_name] ; Creates model w/  no view',
	'update ; Download the latest Monorail.js (script only)',
	'generate [model_name] [properties] ; Scaffold modelName name:type name:type etc...'
	].sort();

help = '\n' + help.join('\n\n').replace(/;/g,'\n|_');

var os = require('os').platform();
switch(routine) {
	//
	// Generate
	//
	case "generate":
		var modelName = argv[3];
		var thisModel = [];
		var genKeyStart = 4;
		var genKeys = [];
		var valid = /\w:(^function|string|timestamp|json|boolean|integer|float)$/gi;
		var firstProperty = true;
		// three space tab and not \t
		var tab = '   ';
		var v = 0;


		if(!modelName) {
			throw 'Invalid: model name';
		}
		
		if(!argv[4]) {
			throw 'Invalid: null property';
		}

		console.log('Generating model '+modelName);

		var modelString = [
					"var nohm = require('../lib/nohm').Nohm;",
					"var redis = require('../lib/nohm/node_modules/redis');",
					"var client = redis.createClient();",
					"",
					"nohm.setClient(client);\n"
						].join('\n');

			modelString += "nohm.model('"+modelName+"', {\n";
			modelString += tab+"idGenerator: 'increment',\n";
			modelString += tab+'properties: {\n';

		argv.forEach( function generator(value) {
			if(v >= genKeyStart) {
				if(value.match(valid)) {
					var stub = value.split(':');
					var pName = stub[0];
					var pType = stub[1];

					console.log(pName + '\n|_' + pType);
					
					if(!firstProperty && argv.size != v) {
						modelString += ',\n';
					}

					modelString += tab+tab+pName+': {\n';
					modelString += tab+tab+tab+"type: '"+pType+"',\n";
					if(pType === 'timestamp') {	
						modelString += tab+tab+tab+"defaultValue: new Date()\n";
					}else{
						modelString += tab+tab+tab+"validations: ['notEmpty']\n";
					}
					modelString += tab+tab+'}';
					firstProperty = false;

				}else{ throw 'Generator error: Invalid format for > '+value; }
				// end error
			
			}// end if 
			v++;
		});

		modelString += '\n'+tab+'}\n});';		
		fs.writeFileSync('./models/' + modelName + '.js', modelString);
		break;
	//
	// Snapshot
	//
	case "snapshot":
		var snapshotType = argv[3];
		var snapshotDirs = ['./views/', './models/', './'];
		var snapshotDir = './snapshots'
		
		function statHandler(err, stats) {
			if(err) {
				fs.mkdirSync(snapshotDir);
			}
		}
		fs.lstat(snapshotDir, statHandler);		
		//
		// Snapshot Types
		//
		switch(snapshotType) {
			//
			// Snapshot Create
			//
			case "create":
				var date = new Date();
				var snapshotName = date.toISOString().replace(/:/g,'.').replace(/-/g,'.')
					+ '.zip';
				var snapshotFile = new zip();
				var files = [];

				snapshotDirs.forEach( function loadFiles(dir) {
					fs.readdirSync(dir).forEach( function fileHandler(fp) {
						var realPath = dir+fp;
						if(fs.lstatSync(realPath).isFile()) {
							files.push({
								name: realPath,
								path: realPath
							});
						}	
					});
				});

				snapshotFile.addFiles(files, function zipHandler(err){
					if(err) throw err;
					var buff = snapshotFile.toBuffer();
					fs.writeFile(snapshotDir + '/' + snapshotName, buff, function onEnd(err) {
						if(err) throw err;
						console.log('Saved');
					});
				});
				break;
			//
			// Snapshot Clean
			//
			case "clean":
				var input = rl.createInterface(process.stdin, process.stdout, null);
				var warning = 'WARNING: All snapshots will be DELETED!';
				warning += '\nType continue to continue: ';

				input.question(warning, function handler(answer) {
					if(answer === 'continue') {
						fs.readdirSync(snapshotDir).forEach( function unlinkFile(fp) {
							fs.unlinkSync(snapshotDir + '/' + fp);
						});
						fs.rmdirSync(snapshotDir);
						fs.mkdirSync(snapshotDir);
						console.log('Snapshots deleted.');
						process.exit(0);
					}
				});	
				break;
			
			default: console.log(help);
		}
		break;
	//
	// Update
	//
	case "update":
		var updateURL = '/runexec/Monorail.js/master/monorail.js';
		var details = {host: 'raw.github.com', path: updateURL};

		https.get(details, function(res) {
			var data = '';
			res.setEncoding('utf8');
			save_file = 'new_monorail.js';
			console.log('Saving to '+save_file);

			res.on('data', function(chunk) {
				data += chunk;
				fs.writeFileSync(save_file, data);
			});
		});
		break;
	//
	// Hash Tree
	//
	case "hashtree":
		var hashPaths = ['./views/', './models/', './themes/', './'];
		hashPaths.forEach( function(hp) {
			files = fs.readdirSync(hp);
			files.forEach( function(fp) {
				var actualPath = hp+fp;
				
				function statHandler(err, stats) {
					if(! stats.isDirectory()) {
						var data = fs.readFileSync(actualPath);
						var hash = crypto.createHash('sha1','utf8').update(data).digest('hex');
						
						console.log(actualPath + ': ' + hash);
					}	
				}
				fs.lstat(actualPath, statHandler)
			});
		});
		break;
	//
	// Summary
	//
	case "summary":
		sumPaths = ['./views/', './models/', './themes/', './routes.js'];

		for(p in sumPaths) {
			realPath = sumPaths[p];

			count = 0
			if(fs.lstatSync(realPath).isDirectory()) {
				sumItems = fs.readdirSync(realPath);

				for(s in sumItems) {
					sumItem = realPath+sumItems[s];
					count++;
				}

				attribute = realPath.split('/')[1];
				console.log(attribute + ': ' + count);
			}

		}// end for sumPaths
		break;
	//
	// Reset [sequence]
	//
	case "reset":
		reset_type = argv[3];
		
		switch(reset_type) {
			//
			// Reset Project
			//
			case "project":
				input = rl.createInterface(process.stdin, process.stdout, null);

				function resetHandler(answer) {
					if(answer == 'continue')
					{
						//
						// Delete Project Items
						//
						delPaths = ['./views/', './models/', './static/'];
						
						for(p in delPaths) {
							realPath = delPaths[p];
							deleteItems = fs.readdirSync(realPath);

							for(i in deleteItems) {
								cleaning = deleteItems[i];
								delPath = realPath+cleaning;
								
								if(!fs.lstatSync(delPath).isDirectory()) {
									console.log('Cleaning up ' + delPath + '...');
									fs.unlinkSync(delPath);
								}
							}
						} // end for delPaths
						
						console.log('Your project has been reset :)');
						process.exit(0);
					}
				} // end function resetHandler

				warning = "WARNING: All files in the views, models, and static folders will be DELETED!"
					+ "\nType the word continue to continue: ";

				input.question(warning, resetHandler);
				break;

			default: console.log(help);
		}
		break;
	//
	// Start [sequence]
	//
	case "start":
		start_type = argv[3];

		switch(start_type) {
			//
			// Start Redis & Express
			//
			case "server":
				spawn = require('child_process').spawn;
				
				//
				// Redis
				//
				console.log('Starting Redis...');
				redis = spawn('redis-server', ['redis.config']);
			
				redis.stdout.on('data', function(data) {
					console.log('Redis-Server [pid:'+redis.pid+'] => '+data);	
				});

				redis.stderr.on('data', function(data) {
					console.log('Redis-Server [ERROR]: '+data);
				});

				redis.on('exit', function(code) {
					console.log('Redis-Server Exit: Code('+code+')');
				});

				if(redis.pid === null) { throw 'Failed to start Redis'; }
				
				console.log('Redis Running on Process '+redis.pid);
				//
				// Express
				//
				console.log('Starting Express');
				express = spawn('node', ['./routes.js']);

				express.stdout.on('data', function(data) {
					console.log('Express [pid:'+express.pid+'] => '+data);
				});

				express.stderr.on('data', function(data) {
					console.log('Express [ERROR]: '+data);
				});

				express.on('exit', function(code) {
					console.log('Express Exit: Code('+code+')');
				});
				
				if(express.pid === null) { throw 'Failed to start Express'; }

				console.log('Express Running on Process '+express.pid);
				break;
			//
			// Display Help
			//
			default: console.log(help);
		}
		break;
		// switch start end
	//
	// New [Sequence]
	//
	case "new":
		new_type = argv[3];
		
		switch(new_type) {
		//
		// New View
		//
		case "view":
			view_name = argv[4];

			if(!view_name) {
				console.log(help);
			}else{
				fp = fs.createWriteStream('views/' + view_name + '.xml');
				fp.write("<#CODE#> html_body = 'This code block takes 100% node.js code.'; </#CODE#>");
			}
			break;
		//
		// New Model
		//
		case "model":
			model_name = argv[4];

			if(!model_name) {
				console.log(help);
			}else{
				fp = fs.createWriteStream('models/' + model_name + '.js');		
				var data = [
					"var nohm = require('../lib/nohm').Nohm;",
					"var redis = require('../lib/nohm/node_modules/redis');",
					"var client = redis.createClient();",
					"",
					"nohm.setClient(client);",
					"nohm.model('"+model_name+"',{});"];
				for(i in data) {
					fp.write(data[i]+'\n', function (err) { if (err) throw err; });
				}
			}
			break;
		//
		// New Page
		//
		case "page":
			page_name = argv[4];

			if(!page_name) {
				console.log(help);
			}else{
				page_files = ['views/' + page_name + '.xml',
					'models/' + page_name + '.js'];

				for(i in page_files) {
					page_file = page_files[i];
					fp = fs.createWriteStream(page_file);
					
					data = [];
					if(page_file == 'views/' + page_name + '.xml') {
						data = ["<#CODE#> html_body = 'This code block takes 100% node.js code.'; </#CODE#>"];
					}

					if(page_file == 'models/' + page_name + '.js') {
						//
						// https://github.com/runexec/File2Array
						//
						var data = [
							"var nohm = require('../lib/nohm').Nohm;",
							"var redis = require('../lib/nohm/node_modules/redis');",
							"var client = redis.createClient();",
							"",
							"nohm.setClient(client);",
							"nohm.model('"+page_name+"',{});"];
					}

					for(i in data) {
						fp.write(data[i]+'\n', function (err) { if (err) throw err; });
					}
				} // end for page_files	
			} // end else page_name	
			break;
		//
		// New Project
		//
		case "project":
				project_name = argv[4];

				if(!project_name) { 
					console.log(help); 
				}else{
					console.log('Creating Project '+project_name);
					//
					// Directory Tree
					//
					dir_tree = [project_name,
					    project_name + '/static',
					    project_name + '/static/img',
					    project_name + '/static/js',
					    project_name + '/static/css',
					    project_name + '/themes',
					    project_name + '/themes/classic',
					    project_name + '/themes/classic/img',
					    project_name + '/themes/classic/js',
					    project_name + '/themes/classic/css',
					    project_name + '/models',
					    project_name + '/views'];
	
					
					for(i in dir_tree) {
                        make_dir = dir_tree[i];

						console.log('Making directory '+make_dir);

						fs.mkdir(make_dir, function (err) {
							if (err) { throw err; }
						});
					} // end for 

					//
					// Default Project Files
					//
					file_tree = [project_name + '/monorail.project',
					    project_name + '/routes.js',
						project_name + '/redis.config',
					    project_name + '/themes/classic/classic.html',
						project_name + '/themes/classic/css/classic.css'];

					for(i in file_tree) {
						make_file = file_tree[i];

						console.log('Making file '+make_file);

						data = [];

						//
						// Default Project File Writters
						//
						//
						// monorail.project
						//
						if(make_file == project_name + '/monorail.project') {
							data = ['name : ' + project_name,
								 	'theme : classic',
									'express_port : 8123'];
						}	
						//
						// redis.config
						//
						if(make_file == project_name + '/redis.config') {
							data = ['daemonize no',
									'port 6379',
									'bind 127.0.0.1',
									'timeout 300',
									'# Every minute save the database to harddrive if at least 10 keys changed',
									'save 60 10',
									'dbfilename monorail.'+project_name+'.rdb',
									'# DB directory',
									'dir ./',
									'loglevel debug',
									'logfile monorail.redis.log'];	
						}
						//
						// routes.js
						//
						if(make_file == project_name + '/routes.js') {
						// http://github.com/runexec/File2Array
						
							data = [
								"var fs = require('fs');",
								"var nohm = require('./lib/nohm').Nohm;",
								"var redis = require('./lib/nohm/node_modules/redis');",
								"var express = require('./lib/express');",
								"var app = express.createServer();",
								"var config_data = fs.readFileSync('monorail.project','utf8').split('\\n');",
								"",
								"if(config_data == '') { throw \"Couldn't read or missing file 'monorail.project' !\"; }",
								"",
								"var site_name = '',",
								"	theme = '',",
								" 	theme_body = '',",
								"	theme_file = '',",
								"   express_port = '';",
								"",
								"for(i in config_data) {",
								"	line = config_data[i].split(' : ');",
								"	k = line[0];",
								"	v = line[1];",
								"",
								"	switch(k) {",
								"		// Site name",
								"		case \"name\":",
								"			site_name = v;",
								"			break;",
								"		// Theme",
								"		case \"theme\":",
								"			theme = v;",
								"			break;",
								"		// Express server port",
								"		case \"express_port\":",
								"			express_port = v;",
								"			break;",
								"	}",
								"}",
								"",
								"theme_file = './themes/'+theme+'/'+theme+'.html'; ",
								"theme_body = fs.readFileSync(theme_file);",
								"",
								"if(theme_body == '') { throw \"Couldn't read or missing theme file '\"+theme_file+\"' !\"; }",
								"",
								"// Request Output Builder",
								"function themeResponse(title, body) {",
								"	ret = theme_body.toString();",
								"	ret = ret.replace(/#MONO_RAIL_TITLE#/g, title);",
								"	ret = ret.replace(/#MONO_RAIL_BODY#/g, body);",
								"	return ret;",
								"}",
								"",
								"function railed(title, body) { return themeResponse(title, body); }",
								"",
								"// Serving up static themes",
								"app.use('/themes', express.static(__dirname + '/themes'));",
								"app.use('/static', express.static(__dirname + '/static'));",
								"",
								"// We want to handle POST request",
								"app.use(express.bodyParser());",
								"",
								"// View Handlers ",
								"// All views must return all output to html_body local variable",
								"//",
								"function loadView(view_name, vars) {",
								"	view_path = './views/'+view_name+'.xml';",
								"	view_data = fs.readFileSync(view_path).toString();",
								"	page_code = view_data.split('<#CODE#>')[1].split('</#CODE#>')[0];",
								"",
								"	util = require('util');",
								"	vm = require('vm');",
								"	",
								"	script = vm.createScript(page_code);	",
								"	script.runInNewContext(vars);",
								"	",
								"	return vars['html_body'];",
								"}",
								"",
								"// Model Handlers",
								"function loadModel(model_name) {",
								"	require('./models/'+model_name+'.js');",
								"	return nohm.factory(model_name);",
								"}",
								"",
								"function getModel(model_name,key_id) {",
								"	require('./models/'+model_name+'.js');",
								"	nohm.factory(model_name, key_id, function(err, properties) {",
								"		if(err){ return null; }else{ return properties; }",
								"	});",
								"}",
								"",
								"////////////////////--------------------------------!",
								"//////////////////// CHANGE NOTHING ABOVE THIS LINE !",
								"////////////////////--------------------------------!",
								"",
								"app.get('/', function(req, res) {",
								"	config_view = [];",
								"	config_view.push('Site Name: <br />');",
								"	config_view.push('<input type=\"text\" value=\"'+site_name+'\" /><br />');",
								"	config_view.push('<br />');",
								"	config_view.push('Site Theme: <br />');",
								"	config_view.push('<input type=\"text\" value=\"'+theme+'\" /><br />');",
								"	",
								"	settings = railed(site_name, config_view.join('\\n'));",
								"	res.send(settings);",
								"});",
								"",
								"app.listen(express_port);"
									];
						}

						//
						// /themes/classic/classic.html
						//
						if(make_file == project_name + '/themes/classic/classic.html') {
							data = [
								'<!DOCTYPE html>',
								'<html lang="en">',
								'<head>',
								'	<meta charset="utf-8" />',
								'	<title> Monorail site : #MONO_RAIL_TITLE# </title>',
								'	<link rel="stylesheet" href="/themes/classic/css/classic.css" media="screen" />',
								'</head>',
								'<body>',
								'<header>',
								'	<h3> #MONO_RAIL_TITLE# </h3> ',
								'</header>',
								'#MONO_RAIL_BODY#',
								'<p />',
								'<footer>',
								'	About : Contact : FAQ ',
								'</footer>',
								'</body>',
								'</html>'
									];
						}

						//
						// /themes/classic/css/classic.css
						//
						if(make_file == project_name + '/themes/classic/css/classic.css') {
							data = [
								"header {",
								"	border-style: solid;",
								"	border-width: 0px 0px 2px 0px;",
								"	border-color: #000000;",
								"	margin: 0px auto 10px auto;",
								"	max-width: 80%;	",
								"	text-align: center;",
								"	background-color: #D9DCD7;",
								"}",
								"",
								"h3 {",
								"	text-align:left;",
								"	margin: 20px 20px 0px 20px;",
								"}",
								"body {",
								"	text-align:center;",
								"  	margin: 0px auto 0px auto;",
								"}",
								];
						}

						
						fp = fs.createWriteStream(make_file);
						for(i in data) {
							fp.write(data[i]+'\n', function(err) { if (err) throw err; });
						}
						
					} // end for-loop make_file

					//
					// Required libs
					//
					copy_op = '';

					if(os.indexOf('window') == -1) {
						copy_op = 'cp -vrf ./lib '+ project_name +'/lib';
					}else{
						copy_op = 'XCOPY lib '+ project_name +'\\lib';
					}

					console.log('Copying libs...');
					exec = require('child_process').exec;
					child = exec(copy_op, function (err, stdout, stderr) { if(err) { throw err; } });

				} // end if else
			break;
			// end case project
			
			//
			// Display Help
			//
			default: console.log(help);
		} // switch new_type end
		break;
		// routine new end
	//
	// Install [Sequence]	
	//
	case "install":
		install_type = argv[3];
		switch(install_type) {
			//
			// Install RightJS
			//
			case "rightjs":
				save_file = 'static/js/right-2.3.1.js';
				
				//  Hacking up a mean one liner
				//
				// wget http://site.com/file.js; cat file.js | toBase64 | fold -w 80 > rightjs.blob
				//

				var rightjs_base64_blob = [
				"LyoqCiAqIFJpZ2h0SlMgdjIuMy4xIC0gaHR0cDovL3JpZ2h0anMub3JnCiAqIFJlbGVhc2VkIHVuZGVy",
				"IHRoZSB0ZXJtcyBvZiBNSVQgbGljZW5zZQogKgogKiBDb3B5cmlnaHQgKEMpIDIwMDgtMjAxMiBOaWtv",
				"bGF5IE5lbXNoaWxvdgogKi8KdmFyIFJpZ2h0SlM9ZnVuY3Rpb24oYSxiLGMsZCxlLGYsZyxoLGkpe2Z1",
				"bmN0aW9uIGNMKGEsYixjLGQpe3ZhciBlPXt9LGY9YS5tYXJnaW5MZWZ0LnRvRmxvYXQoKXx8MCxnPWEu",
				"bWFyZ2luVG9wLnRvRmxvYXQoKXx8MCxoPWM9PT0icmlnaHQiLGk9Yz09PSJib3R0b20iLGo9Yz09PSJ0",
				"b3AifHxpO2Q9PT0ib3V0Ij8oZVtqPyJoZWlnaHQiOiJ3aWR0aCJdPSIwcHgiLGg/ZS5tYXJnaW5MZWZ0",
				"PWYrYi54KyJweCI6aSYmKGUubWFyZ2luVG9wPWcrYi55KyJweCIpKTooaj8oZS5oZWlnaHQ9Yi55KyJw",
				"eCIsYS5oZWlnaHQ9IjBweCIpOihlLndpZHRoPWIueCsicHgiLGEud2lkdGg9IjBweCIpLGg/KGUubWFy",
				"Z2luTGVmdD1mKyJweCIsYS5tYXJnaW5MZWZ0PWYrYi54KyJweCIpOmkmJihlLm1hcmdpblRvcD1nKyJw",
				"eCIsYS5tYXJnaW5Ub3A9ZytiLnkrInB4IikpO3JldHVybiBlfWZ1bmN0aW9uIGNLKGEsYixjKXt2YXIg",
				"ZD1hLmNsb25lKCkuc2V0U3R5bGUoInBvc2l0aW9uOmFic29sdXRlO3otaW5kZXg6LTE7dmlzaWJpbGl0",
				"eTpoaWRkZW4iKS5zZXRXaWR0aChhLnNpemUoKS54KS5zZXRTdHlsZShiKSxlO2EucGFyZW50KCkmJmEu",
				"aW5zZXJ0KGQsImJlZm9yZSIpLGU9Y0ooZCxjKSxkLnJlbW92ZSgpO3JldHVybiBlfWZ1bmN0aW9uIGNK",
				"KGEsYil7dmFyIGM9MCxkPWIubGVuZ3RoLGU9YS5jb21wdXRlZFN0eWxlcygpLGY9e30sZztmb3IoO2M8",
				"ZDtjKyspZz1iW2NdLGcgaW4gZSYmKGZbZ109IiIrZVtnXSxnPT09Im9wYWNpdHkiJiYoZltnXT1mW2dd",
				"LnJlcGxhY2UoIiwiLCIuIikpKTtyZXR1cm4gZn1mdW5jdGlvbiBjSShhLGIsYyl7dmFyIGQ7Zm9yKGQg",
				"aW4gYykoZD09IndpZHRoInx8ZD09ImhlaWdodCIpJiZiW2RdPT0iYXV0byImJihiW2RdPWEuX1sib2Zm",
				"c2V0IitkLmNhcGl0YWxpemUoKV0rInB4Iik7YnUmJmMuZmlsdGVyJiYhYi5maWx0ZXImJihiLmZpbHRl",
				"cj0iYWxwaGEob3BhY2l0eT0xMDApIiksY0coYSxiLGMpO2ZvcihkIGluIGMpe2lmKGNbZF0hPT1iW2Rd",
				"JiYvY29sb3IvaS50ZXN0KGQpKXticSYmKGNbZF09Y1tkXS5yZXBsYWNlKC8iL2csIiIpLGJbZF09Yltk",
				"XS5yZXBsYWNlKC8iL2csIiIpKSxjRihjW2RdKXx8KGNbZF09Y1tkXS50b1JnYigpKSxjRihiW2RdKXx8",
				"KGJbZF09YltkXS50b1JnYigpKTtpZighY1tkXXx8IWJbZF0pY1tkXT1iW2RdPSIifS9cZC8udGVzdChj",
				"W2RdKSYmIS9cZC8udGVzdChiW2RdKSYmKGJbZF09Y1tkXS5yZXBsYWNlKC9bXGRcLlwtXSsvZywiMCIp",
				"KTtpZihjW2RdPT09YltkXXx8IS9cZC8udGVzdChiW2RdKXx8IS9cZC8udGVzdChjW2RdKSlkZWxldGUg",
				"Y1tkXSxkZWxldGUgYltkXX19ZnVuY3Rpb24gY0goYSl7dmFyIGI9e30sYz0vW1xkXC5cLV0rL2csZCxl",
				"LGYsZztmb3IoZSBpbiBhKXtkPWFbZV0ubWF0Y2goYyksZj1kLm1hcCgidG9GbG9hdCIpLGYudD1hW2Vd",
				"LnNwbGl0KGMpLGYucj1mLnRbMF09PT0icmdiKCIsZi50Lmxlbmd0aD09MSYmZi50LnVuc2hpZnQoIiIp",
				"O2ZvcihnPTA7ZzxmLmxlbmd0aDtnKyspZi50LnNwbGljZShnKjIrMSwwLGZbZ10pO2JbZV09Zn1yZXR1",
				"cm4gYn1mdW5jdGlvbiBjRyhhLGIsYyl7Zm9yKHZhciBkPTA7ZDw0O2QrKyl7dmFyIGU9ImJvcmRlciIr",
				"Y0NbZF0rIlN0eWxlIixmPSJib3JkZXIiK2NDW2RdKyJXaWR0aCIsZz0iYm9yZGVyIitjQ1tkXSsiQ29s",
				"b3IiO2lmKGUgaW4gYiYmYltlXSE9Y1tlXSl7dmFyIGg9YS5fLnN0eWxlO2JbZV09PSJub25lIiYmKGhb",
				"Zl09IjBweCIpLGhbZV09Y1tlXSxjRihiW2ddKSYmKGhbZ109YS5nZXRTdHlsZSgiQ29sb3IiKSl9fX1m",
				"dW5jdGlvbiBjRihhKXtyZXR1cm4gYT09PSJ0cmFuc3BhcmVudCJ8fGE9PT0icmdiYSgwLCAwLCAwLCAw",
				"KSJ9ZnVuY3Rpb24gY0UoYSl7dmFyIGI9W10sYz1bIlN0eWxlIiwiQ29sb3IiLCJXaWR0aCJdLGQsZSxm",
				"O2ZvcihkIGluIGEpaWYoZC5zdGFydHNXaXRoKCJib3JkZXIiKSlmb3IoZT0wO2U8MztlKyspZm9yKGY9",
				"MDtmPDQ7ZisrKWIucHVzaCgiYm9yZGVyIitjQ1tmXStjW2VdKTtlbHNlIGQ9PT0ibWFyZ2luInx8ZD09",
				"PSJwYWRkaW5nIj9jRChiLGQsY0MpOmQuc3RhcnRzV2l0aCgiYmFja2dyb3VuZCIpP2NEKGIsImJhY2tn",
				"cm91bmQiLFsiQ29sb3IiLCJQb3NpdGlvbiIsIlBvc2l0aW9uWCIsIlBvc2l0aW9uWSJdKTpkPT09Im9w",
				"YWNpdHkiJiZidT9iLnB1c2goImZpbHRlciIpOmIucHVzaChkKTtyZXR1cm4gYn1mdW5jdGlvbiBjRChh",
				"LGIsYyl7Zm9yKHZhciBkPTA7ZDxjLmxlbmd0aDtkKyspYS5wdXNoKGIrY1tkXSl9ZnVuY3Rpb24gY0Io",
				"YSl7ZnVuY3Rpb24gZygpe2Zvcih2YXIgYSBpbiBmKWVbYV09ZlthXX12YXIgYj10aGlzLm9wdGlvbnMs",
				"ZD10aGlzLmVsZW1lbnQsZT1kLl8uc3R5bGUsZj1jLm9ubHkoZC5jb21wdXRlZFN0eWxlcygpLGN4LGN5",
				"LGN6KTt0aGlzLm9uRmluaXNoKGcpLm9uQ2FuY2VsKGZ1bmN0aW9uKCl7ZVtjeF09Im5vbmUiLHNldFRp",
				"bWVvdXQoZywxKX0pLGVbY3hdPSJhbGwiLGVbY3ldPShjZy5EdXJhdGlvbnNbYi5kdXJhdGlvbl18fGIu",
				"ZHVyYXRpb24pKyJtcyIsZVtjel09Y0FbYi50cmFuc2l0aW9uXXx8Yi50cmFuc2l0aW9uLHNldFRpbWVv",
				"dXQoZnVuY3Rpb24oKXtkLnNldFN0eWxlKGEpfSwwKX1mdW5jdGlvbiBjdShhLGIsYyl7dmFyIGQ9Sihj",
				"KS5jb21wYWN0KCksZT1BKGQubGFzdCgpKT9kLnBvcCgpOnt9LGY9bmV3KGNnW2IuY2FwaXRhbGl6ZSgp",
				"XSkoYSxlKTtmLnN0YXJ0LmFwcGx5KGYsZCk7cmV0dXJuIGF9ZnVuY3Rpb24gY3QoYSxiKXtmdW5jdGlv",
				"biBxKGEpe3ZhciBiPWEsYz0wLGQ7d2hpbGUoYzw1KXtkPW4oYiktYTtpZihoLmFicyhkKTwuMDAxKWJy",
				"ZWFrO2I9Yi1kL3AoYiksYysrfXJldHVybiBifWZ1bmN0aW9uIHAoYSl7cmV0dXJuIGQrYSooMiplK2Eq",
				"MypmKSsuMDAxfWZ1bmN0aW9uIG8oYSl7cmV0dXJuIGEqKGcrYSooaSthKmopKX1mdW5jdGlvbiBuKGEp",
				"e3JldHVybiBhKihkK2EqKGUrYSpmKSl9YT1jclthXXx8Y0FbYV18fGEsYT1hLm1hdGNoKC8oW1xkXC5d",
				"KylbXHMsXSsoW1xkXC5dKylbXHMsXSsoW1xkXC5dKylbXHMsXSsoW1xkXC5dKykvKSxhPVswLGFbMV0t",
				"MCxhWzJdLTAsYVszXS0wLGFbNF0tMF07dmFyIGM9YS5qb2luKCIsIikrIiwiK2IsZCxlLGYsZyxpLGos",
				"ayxsLG07aWYoIShjIGluIGNzKSl7ZD0zKmFbMV0sZT0zKihhWzNdLWFbMV0pLWQsZj0xLWQtZSxnPTMq",
				"YVsyXSxpPTMqKGFbNF0tYVsyXSktZyxqPTEtZy1pLGNzW2NdPWs9W10sbT0wLGw9MS9iO3doaWxlKG08",
				"MS4wMDAxKWsucHVzaChvKHEobSkpKSxtKz1sfXJldHVybiBjc1tjXX1mdW5jdGlvbiBjcShhKXthLl90",
				"aW1lciYmY2xlYXJJbnRlcnZhbChhLl90aW1lcil9ZnVuY3Rpb24gY3AoYSl7dmFyIGI9YS5vcHRpb25z",
				"LGM9Y2cuRHVyYXRpb25zW2IuZHVyYXRpb25dfHxiLmR1cmF0aW9uLGQ9aC5jZWlsKGMvMWUzKmIuZnBz",
				"KSxlPWN0KGIudHJhbnNpdGlvbixkKSxmPWgucm91bmQoMWUzL2IuZnBzKSxnPTA7YS5fdGltZXI9c2V0",
				"SW50ZXJ2YWwoZnVuY3Rpb24oKXtnPT09ZD9hLmZpbmlzaCgpOihhLnJlbmRlcihlW2ddKSxnKyspfSxm",
				"KX1mdW5jdGlvbiBjbyhhKXt2YXIgYj1JKGEuXyk7KGNpW2JdfHxbXSkuZWFjaCgiY2FuY2VsIiksKGNo",
				"W2JdfHxbXSkuc3BsaWNlKDApfWZ1bmN0aW9uIGNuKGEpe3ZhciBiPWEuY2gsYz1iLnNoaWZ0KCk7aWYo",
				"Yz1iWzBdKWNbMV0uJGNoPSEwLGNbMV0uc3RhcnQuYXBwbHkoY1sxXSxjWzBdKX1mdW5jdGlvbiBjbShh",
				"KXt2YXIgYj1hLmNyO2ImJmIuc3BsaWNlKGIuaW5kZXhPZihhKSwxKX1mdW5jdGlvbiBjbChhKXthLmNy",
				"JiZhLmNyLnB1c2goYSl9ZnVuY3Rpb24gY2soYSxiKXt2YXIgYz1hLmNoLGQ9YS5vcHRpb25zLnF1ZXVl",
				"O2lmKCFjfHxhLiRjaClyZXR1cm4gYS4kY2g9ITE7ZCYmYy5wdXNoKFtiLGFdKTtyZXR1cm4gZCYmY1sw",
				"XVsxXSE9PWF9ZnVuY3Rpb24gY2ooYSl7dmFyIGI9SSgoYS5lbGVtZW50fHx7fSkuX3x8e30pO2EuY2g9",
				"Y2hbYl09Y2hbYl18fFtdLGEuY3I9Y2lbYl09Y2lbYl18fFtdfWZ1bmN0aW9uIGNmKGEsYil7YS5zdG9w",
				"KCksdGhpcy5zZW5kKGIpfWZ1bmN0aW9uIGNjKGEsYil7dmFyIGQ9YVswXSxlLGYsZz1jYihhKSxoPSFj",
				"LmtleXMoZykubGVuZ3RoO3JldHVybihiLiRsaXN0ZW5lcnN8fFtdKS5maWx0ZXIoZnVuY3Rpb24oYSl7",
				"cmV0dXJuIGEuZHImJmEubj09PWQmJihofHxmdW5jdGlvbigpe2Zvcih2YXIgYiBpbiBnKWlmKGEuZHI9",
				"PT1iKWZvcihlPTAsZj1nW2JdO2U8Zi5sZW5ndGg7ZSsrKWlmKCFmW2VdLmxlbmd0aHx8ZltlXVswXT09",
				"PWEuZGMpcmV0dXJuITA7cmV0dXJuITF9KCkpfSl9ZnVuY3Rpb24gY2IoYSl7dmFyIGI9SihhKSxjPWJb",
				"MV18fHt9LGQ9e30sZTt5KGMpPyhkW2NdPWIuc2xpY2UoMiksQihkW2NdWzBdKSYmKGRbY109ZFtjXVsw",
				"XS5tYXAoTikpKTpkPWM7Zm9yKGUgaW4gZClkW2VdPU4oZFtlXSksZFtlXT1CKGRbZV1bMF0pP2RbZV06",
				"W2RbZV1dO3JldHVybiBkfWZ1bmN0aW9uIGNhKGEsYixjKXt2YXIgZD1KKGIpLGU9ZC5zaGlmdCgpO3Jl",
				"dHVybiBmdW5jdGlvbihiKXt2YXIgYz1iLmZpbmQoYSk7cmV0dXJuIGM9PT1pP2M6dHlwZW9mIGU9PT0i",
				"c3RyaW5nIj9jW2VdLmFwcGx5KGMsZCk6ZS5hcHBseShjLFtiXS5jb25jYXQoZCkpfX1mdW5jdGlvbiBi",
				"Xygpe2JYJiYoYlg9ITEsYnI/KGIuYXR0YWNoRXZlbnQoIm9ubW91c2VvdmVyIixiWiksYS5hdHRhY2hF",
				"dmVudCgiYmx1ciIsYiQpKTooYi5hZGRFdmVudExpc3RlbmVyKCJtb3VzZW92ZXIiLGJaLCExKSxhLmFk",
				"ZEV2ZW50TGlzdGVuZXIoImJsdXIiLGIkLCExKSkpfWZ1bmN0aW9uIGIkKGEpe2JXLmVhY2goZnVuY3Rp",
				"b24oYixjKXtiJiZxW2NdJiZiWShhLHFbY10uXyxjLCExKX0pfWZ1bmN0aW9uIGJaKGEpe3ZhciBiPWEu",
				"dGFyZ2V0fHxhLnNyY0VsZW1lbnQsYz1hLnJlbGF0ZWRUYXJnZXR8fGEuZnJvbUVsZW1lbnQsZD1iLGU9",
				"ITEsZj1bXSxnLGg7d2hpbGUoZC5ub2RlVHlwZT09PTEpZz1JKGQpLGJXW2ddPT09aSYmYlkoYSxkLGcs",
				"YldbZ109ITApLGQ9PT1jJiYoZT0hMCksZi5wdXNoKGQpLGQ9ZC5wYXJlbnROb2RlO2lmKGMmJiFlKXdo",
				"aWxlKGMhPT1udWxsJiZjLm5vZGVUeXBlPT09MSYmZi5pbmRleE9mKGMpPT09LTEpZz1JKGMpLGJXW2dd",
				"IT09aSYmYlkoYSxjLGcsYldbZ109aSksYz1jLnBhcmVudE5vZGV9ZnVuY3Rpb24gYlkoYSxiLGMsZCl7",
				"dmFyIGU9bmV3IGJFKGEpO2UudHlwZT1kPT09ITA/Im1vdXNlZW50ZXIiOiJtb3VzZWxlYXZlIixlLmJ1",
				"YmJsZXM9ITEsZS5zdG9wcGVkPSEwLGUudGFyZ2V0PWJBKGIpLGUuZmluZD1mdW5jdGlvbihhKXtyZXR1",
				"cm4gRihhLCEwKS5pbmRleE9mKHRoaXMudGFyZ2V0Ll8pPT09LTE/aTp0aGlzLnRhcmdldH0sZS50YXJn",
				"ZXQuZmlyZShlKSxiQy5maXJlKGUpfWZ1bmN0aW9uIGJWKGEpe3ZhciBiPW5ldyBiRShhKSxjPWIudGFy",
				"Z2V0LGQ9Yy5wYXJlbnQmJmMucGFyZW50KCk7Yi50eXBlPWEudHlwZT09PSJmb2N1c2luInx8YS50eXBl",
				"PT09ImZvY3VzIj8iZm9jdXMiOiJibHVyIixkJiZkLmZpcmUoYil9ZnVuY3Rpb24gYlMoYSl7YT1IKGEp",
				"LGJGPWJGLmNvbmNhdChhKSxibShiRy5wcm90b3R5cGUsYSksYm0oYkIucHJvdG90eXBlLGEpfWZ1bmN0",
				"aW9uIGJSKGEsYil7Yj1iLmNhbWVsaXplKCk7aWYoYj09PSJvcGFjaXR5IilyZXR1cm4gYnU/KC9vcGFj",
				"aXR5PShcZCspL2kuZXhlYyhhLmZpbHRlcnx8IiIpfHxbIiIsIjEwMCJdKVsxXS50b0ludCgpLzEwMCsi",
				"IjphW2JdLnJlcGxhY2UoIiwiLCIuIik7Yj09PSJmbG9hdCImJihiPWJyPyJzdHlsZUZsb2F0IjoiY3Nz",
				"RmxvYXQiKTt2YXIgYz1hW2JdO2JxJiYvY29sb3IvaS50ZXN0KGIpJiZjJiYoYz1jLnJlcGxhY2UoLyIv",
				"ZywiIikpO3JldHVybiBjfWZ1bmN0aW9uIGJRKGEsYil7aWYodHlwZW9mIGI9PT0ic3RyaW5nIil7dmFy",
				"IGM9YS50YWdOYW1lLGQ9YlAsZT1jIGluIGJOP2JOW2NdOlsiIiwiIiwxXSxmPWVbMl07ZC5pbm5lckhU",
				"TUw9ZVswXSsiPCIrYysiPiIrYisiPC8iK2MrIj4iK2VbMV07d2hpbGUoZi0tIT09MClkPWQuZmlyc3RD",
				"aGlsZDtiPWQuY2hpbGROb2Rlczt3aGlsZShiLmxlbmd0aCE9PTApYk8uYXBwZW5kQ2hpbGQoYlswXSl9",
				"ZWxzZSBmb3IodmFyIGc9MCxoPWIubGVuZ3RoLGk7ZzxoO2crKylpPWJbYi5sZW5ndGg9PT1oP2c6MF0s",
				"Yk8uYXBwZW5kQ2hpbGQoaSBpbnN0YW5jZW9mIGJHP2kuXzppKTtyZXR1cm4gYk99ZnVuY3Rpb24gYkwo",
				"YSxiLGMpe3ZhciBkPWEuXyxlPVtdLGY9MCxnPSFjO3doaWxlKGQ9ZFtiXSlkLm5vZGVUeXBlPT09MSYm",
				"KGd8fGJBKGQpLm1hdGNoKGMpKSYmKGVbZisrXT1iQShkKSk7cmV0dXJuIGV9ZnVuY3Rpb24gYksoYSxi",
				"LGMpe2lmKHR5cGVvZiBiPT09InN0cmluZyIpe2EuXz1iSihiLGMpO2lmKGMhPT1pKWZvcih2YXIgZCBp",
				"biBjKXN3aXRjaChkKXtjYXNlImlkIjphLl8uaWQ9Y1tkXTticmVhaztjYXNlImh0bWwiOmEuXy5pbm5l",
				"ckhUTUw9Y1tkXTticmVhaztjYXNlImNsYXNzIjphLl8uY2xhc3NOYW1lPWNbZF07YnJlYWs7Y2FzZSJv",
				"biI6YS5vbihjW2RdKTticmVhaztkZWZhdWx0OmEuc2V0KGQsY1tkXSl9fWVsc2UgYS5fPWJ9ZnVuY3Rp",
				"b24gYkEoYSl7aWYoYSE9bnVsbCl7dmFyIGI9ciBpbiBhP3FbYVtyXV06aTtpZihiIT09aSlyZXR1cm4g",
				"YjtpZihhLm5vZGVUeXBlPT09MSlyZXR1cm4gbmV3IGJHKGEpO2lmKGEubm9kZVR5cGU9PT05KXJldHVy",
				"biBuZXcgYkIoYSk7aWYoYS53aW5kb3c9PWEpcmV0dXJuIG5ldyBiRChhKTtpZihDKGEudGFyZ2V0KXx8",
				"QyhhLnNyY0VsZW1lbnQpKXJldHVybiBuZXcgYkUoYSl9cmV0dXJuIGF9ZnVuY3Rpb24gYnooYSxiKXt0",
				"eXBlb2YgYT09PSJzdHJpbmciJiYoYT1zKHt0eXBlOmF9LGIpLHRoaXMuc3RvcHBlZD1hLmJ1YmJsZXM9",
				"PT0hMSxBKGIpJiZzKHRoaXMsYikpLHRoaXMuXz1hLHRoaXMudHlwZT1hLnR5cGUsdGhpcy53aGljaD1h",
				"LndoaWNoLHRoaXMua2V5Q29kZT1hLmtleUNvZGUsdGhpcy50YXJnZXQ9YkEoYS50YXJnZXQhPW51bGwm",
				"JiJub2RlVHlwZSJpbiBhLnRhcmdldCYmYS50YXJnZXQubm9kZVR5cGU9PT0zP2EudGFyZ2V0LnBhcmVu",
				"dE5vZGU6YS50YXJnZXQpLHRoaXMuY3VycmVudFRhcmdldD1iQShhLmN1cnJlbnRUYXJnZXQpLHRoaXMu",
				"cmVsYXRlZFRhcmdldD1iQShhLnJlbGF0ZWRUYXJnZXQpLHRoaXMucGFnZVg9YS5wYWdlWCx0aGlzLnBh",
				"Z2VZPWEucGFnZVk7aWYoYnQmJiJzcmNFbGVtZW50ImluIGEpe3RoaXMud2hpY2g9YS5idXR0b249PT0y",
				"PzM6YS5idXR0b249PT00PzI6MSx0aGlzLnRhcmdldD1iQShhLnNyY0VsZW1lbnQpfHxiLHRoaXMucmVs",
				"YXRlZFRhcmdldD10aGlzLnRhcmdldC5fPT09YS5mcm9tRWxlbWVudD9iQShhLnRvRWxlbWVudCk6dGhp",
				"cy50YXJnZXQsdGhpcy5jdXJyZW50VGFyZ2V0PWI7dmFyIGM9dGhpcy50YXJnZXQud2luKCkuc2Nyb2xs",
				"cygpO3RoaXMucGFnZVg9YS5jbGllbnRYK2MueCx0aGlzLnBhZ2VZPWEuY2xpZW50WStjLnl9fWZ1bmN0",
				"aW9uIGJ5KGEsYil7YksodGhpcyxhLGIpO3ZhciBjPXRoaXMsZD1jLl8sZT1idy5DYXN0KGQpLGY9ciBp",
				"biBkP2Rbcl06ZFtyXT1wKys7ZSE9PWkmJihjPW5ldyBlKGQsYiksIiRsaXN0ZW5lcnMiaW4gdGhpcyYm",
				"KGMuJGxpc3RlbmVycz10aGlzLiRsaXN0ZW5lcnMpKSxxW2ZdPWM7cmV0dXJuIGN9ZnVuY3Rpb24gYngo",
				"KXtyZXR1cm4gZnVuY3Rpb24oYSxiKXtiaSh0aGlzKSx0aGlzLmluaXRpYWxpemUuYXBwbHkodGhpcyxh",
				"cmd1bWVudHMpO3ZhciBjPXRoaXMuXyxkPXIgaW4gYz9jW3JdOmNbcl09KGMubm9kZVR5cGU9PT0xPzE6",
				"LTEpKnArKztxW2RdPXRoaXN9fWZ1bmN0aW9uIGJvKGEsYixjLGQpe2lmKEEoYikpZm9yKHZhciBlIGlu",
				"IGIpYS5zdG9wT2JzZXJ2aW5nKGUsYltlXSk7ZWxzZSB5KGIpfHwoYz1iLGI9bnVsbCkseShjKSYmKGM9",
				"YVtjXSksYS4kbGlzdGVuZXJzPShhLiRsaXN0ZW5lcnN8fFtdKS5maWx0ZXIoZnVuY3Rpb24oYSl7dmFy",
				"IGU9YiYmYz9hLmUhPT1ifHxhLmYhPT1jOmI/YS5lIT09YjphLmYhPT1jO2V8fGQoYSk7cmV0dXJuIGV9",
				"KX1mdW5jdGlvbiBibihhLGIsYyl7dmFyIGQ9bi5jYWxsKGIsMiksZT1iWzBdLGY9YlsxXSxnPSExO2lm",
				"KHkoZSkpc3dpdGNoKHR5cGVvZiBmKXtjYXNlInN0cmluZyI6Zz1mLGY9ZiBpbiBhP2FbZl06ZnVuY3Rp",
				"b24oKXt9O2Nhc2UiZnVuY3Rpb24iOigiJGxpc3RlbmVycyJpbiBhP2EuJGxpc3RlbmVyczphLiRsaXN0",
				"ZW5lcnM9W10pLnB1c2goYyh7ZTplLGY6ZixhOmQscjpnfHwhMSx0OmF9KSk7YnJlYWs7ZGVmYXVsdDpp",
				"ZihCKGYpKWZvcih2YXIgaD0wO2g8Zi5sZW5ndGg7aCsrKWEub24uYXBwbHkoYSxbZV0uY29uY2F0KE4o",
				"ZltoXSkpLmNvbmNhdChkKSl9ZWxzZXtkPW4uY2FsbChiLDEpO2ZvcihnIGluIGUpYS5vbi5hcHBseShh",
				"LFtnXS5jb25jYXQoTihlW2ddKSkuY29uY2F0KGQpKX19ZnVuY3Rpb24gYmkoYSl7InByZWJpbmQiaW4g",
				"YSYmQihhLnByZWJpbmQpJiZhLnByZWJpbmQuZWFjaChmdW5jdGlvbihiKXthW2JdPWFbYl0uYmluZChh",
				"KX0pfWZ1bmN0aW9uIGJoKGEsYil7dmFyIGM9Yi50b1VwcGVyQ2FzZSgpLGQ9YS5jb25zdHJ1Y3Rvcixl",
				"PVthLGRdLmNvbmNhdChkLmFuY2VzdG9yc3x8W10pLGY9MDtmb3IobD1lLmxlbmd0aDtmPGw7ZisrKXtp",
				"ZihjIGluIGVbZl0pcmV0dXJuIGVbZl1bY107aWYoYiBpbiBlW2ZdKXJldHVybiBlW2ZdW2JdfXJldHVy",
				"biBudWxsfWZ1bmN0aW9uIGJnKGEsYixjKXsoYltiZVtjPzA6Ml1dfHxiW2JlW2M/MTozXV18fGZ1bmN0",
				"aW9uKCl7fSkuY2FsbChiLGEpfWZ1bmN0aW9uIGJmKGEsYil7cmV0dXJuIGMud2l0aG91dC5hcHBseShj",
				"LFthXS5jb25jYXQoYmUuY29uY2F0KGI/SCgicHJvdG90eXBlIHBhcmVudCBhbmNlc3RvcnMiKTpbImNv",
				"bnN0cnVjdG9yIl0pKSl9ZnVuY3Rpb24gYmIoYSxiKXtyZXR1cm4gYT5iPzE6YTxiPy0xOjB9ZnVuY3Rp",
				"b24gYmEoYSl7cmV0dXJuISFhfWZ1bmN0aW9uIF8oYSxiLGMpe3RyeXtyZXR1cm4gYS5hcHBseShiLFoo",
				"YyxiKSl9Y2F0Y2goZCl7aWYoIShkIGluc3RhbmNlb2YgJCkpdGhyb3cgZH1yZXR1cm4gaX1mdW5jdGlv",
				"biAkKCl7fWZ1bmN0aW9uIFooYSxiKXt2YXIgYz1hWzBdLGQ9bi5jYWxsKGEsMSksZT1iLGY7dHlwZW9m",
				"IGM9PT0ic3RyaW5nIj8oZj1jLGIubGVuZ3RoIT09MCYmdHlwZW9mIGJbMF1bZl09PT0iZnVuY3Rpb24i",
				"P2M9ZnVuY3Rpb24oYSl7cmV0dXJuIGFbZl0uYXBwbHkoYSxkKX06Yz1mdW5jdGlvbihhKXtyZXR1cm4g",
				"YVtmXX0pOmU9ZFswXTtyZXR1cm5bYyxlXX1mdW5jdGlvbiBPKGEsYil7dmFyIGM9W10sZCxlLGY7Zm9y",
				"KGQgaW4gYSl7ZT1hW2RdLGImJihkPWIrIlsiK2QrIl0iKTtpZih0eXBlb2YgZT09PSJvYmplY3QiKXtp",
				"ZihCKGUpKXtkLmVuZHNXaXRoKCJbXSIpfHwoZCs9IltdIik7Zm9yKGY9MDtmPGUubGVuZ3RoO2YrKylj",
				"LnB1c2goW2QsZVtmXV0pfWVsc2UgaWYoZSl7ZT1PKGUsZCk7Zm9yKGY9MDtmPGUubGVuZ3RoO2YrKylj",
				"LnB1c2goZVtmXSl9fWVsc2UgYy5wdXNoKFtkLGVdKX1yZXR1cm4gY31mdW5jdGlvbiBOKGEpe3JldHVy",
				"biBCKGEpP2E6W2FdfWZ1bmN0aW9uIEsoYSl7cmV0dXJuIHMoYSx7TWV0aG9kczp7fSxpbmNsdWRlOmZ1",
				"bmN0aW9uKCl7Zm9yKHZhciBiPTAsYz1hcmd1bWVudHMubGVuZ3RoO2I8YztiKyspQShhcmd1bWVudHNb",
				"Yl0pJiYocyhhLnByb3RvdHlwZSxhcmd1bWVudHNbYl0pLHMoYS5NZXRob2RzLGFyZ3VtZW50c1tiXSkp",
				"fX0pfXZhciBqPWZ1bmN0aW9uKGEpe3JldHVybiBhfTtqLnZlcnNpb249IjIuMy4xIixqLm1vZHVsZXM9",
				"WyJjb3JlIiwiZG9tIiwiZm9ybSIsImV2ZW50cyIsInhociIsImZ4IiwiY29va2llIl07dmFyIGs9ZC5w",
				"cm90b3R5cGUsbT1jLnByb3RvdHlwZS50b1N0cmluZyxuPWsuc2xpY2Usbz1iLmRvY3VtZW50RWxlbWVu",
				"dCxwPTEscT1bXSxyPSJ1bmlxdWVOdW1iZXIiLHM9ai4kZXh0PWZ1bmN0aW9uKGEsYixjKXt2YXIgZD1i",
				"fHx7fSxlO2ZvcihlIGluIGQpaWYoIWN8fCEoZSBpbiBhKSlhW2VdPWRbZV07cmV0dXJuIGF9LHQ9ai4k",
				"ZXZhbD1mdW5jdGlvbihiKXtiJiYoImV4ZWNTY3JpcHQiaW4gYT9iQy53aW4oKS5fLmV4ZWNTY3JpcHQo",
				"Yik6Rygic2NyaXB0Iix7dGV4dDpifSkuaW5zZXJ0VG8obykpfSx1PWouJGJyZWFrPWZ1bmN0aW9uKCl7",
				"dGhyb3cgbmV3ICR9LHY9ai4kYWxpYXM9ZnVuY3Rpb24oYSxiKXtmb3IodmFyIGMgaW4gYilhW2NdPWFb",
				"YltjXV07cmV0dXJuIGF9LHc9ai5kZWZpbmVkPWZ1bmN0aW9uKGEpe3JldHVybiB0eXBlb2YgYSE9PSJ1",
				"bmRlZmluZWQifSx4PWouaXNGdW5jdGlvbj1mdW5jdGlvbihhKXtyZXR1cm4gdHlwZW9mIGE9PT0iZnVu",
				"Y3Rpb24ifSx5PWouaXNTdHJpbmc9ZnVuY3Rpb24oYSl7cmV0dXJuIHR5cGVvZiBhPT09InN0cmluZyJ9",
				"LHo9ai5pc051bWJlcj1mdW5jdGlvbihhKXtyZXR1cm4gdHlwZW9mIGE9PT0ibnVtYmVyIiYmIWlzTmFO",
				"KGEpfSxBPWouaXNIYXNoPWZ1bmN0aW9uKGEpe3JldHVybiBtLmNhbGwoYSk9PT0iW29iamVjdCBPYmpl",
				"Y3RdIn0sQj1qLmlzQXJyYXk9ZnVuY3Rpb24oYSl7cmV0dXJuIG0uY2FsbChhKT09PSJbb2JqZWN0IEFy",
				"cmF5XSJ9LEM9ai5pc0VsZW1lbnQ9ZnVuY3Rpb24oYSl7cmV0dXJuIGEhPW51bGwmJmEubm9kZVR5cGU9",
				"PT0xfSxEPWouaXNOb2RlPWZ1bmN0aW9uKGEpe3JldHVybiBhIT1udWxsJiZhLm5vZGVUeXBlIT1udWxs",
				"fSxFPWouJD1mdW5jdGlvbihhKXtpZihhIGluc3RhbmNlb2YgYncpcmV0dXJuIGE7dHlwZW9mIGE9PT0i",
				"c3RyaW5nIiYmKGE9Yi5nZXRFbGVtZW50QnlJZChhKSk7cmV0dXJuIGJBKGEpfSxGPWouJCQ9ZnVuY3Rp",
				"b24oYSxiKXtyZXR1cm4gYkMuZmluZChhLGIpfSxHPWouJEU9ZnVuY3Rpb24oYSxiKXtyZXR1cm4gbmV3",
				"IGJHKGEsYil9LEg9ai4kdz1mdW5jdGlvbihhKXtyZXR1cm4gYS50cmltKCkuc3BsaXQoL1xzKy8pfSxJ",
				"PWouJHVpZD1mdW5jdGlvbihhKXtyZXR1cm4gciBpbiBhP2Fbcl06YVtyXT1wKyt9LEo9ai4kQT1mdW5j",
				"dGlvbihhKXtyZXR1cm4gbi5jYWxsKGEsMCl9O2subWFwfHwoSj1qLiRBPWZ1bmN0aW9uKGEpe3RyeXty",
				"ZXR1cm4gbi5jYWxsKGEsMCl9Y2F0Y2goYil7Zm9yKHZhciBjPVtdLGQ9MCxlPWEubGVuZ3RoO2Q8ZTtk",
				"KyspY1tkXT1hW2RdO3JldHVybiBjfX0pLEEobykmJihBPWouaXNIYXNoPWZ1bmN0aW9uKGEpe3JldHVy",
				"biBtLmNhbGwoYSk9PT0iW29iamVjdCBPYmplY3RdIiYmYSE9bnVsbCYmYS5oYXNPd25Qcm9wZXJ0eSE9",
				"bnVsbH0pO2Zvcih2YXIgTD0wLE09IkFycmF5IEZ1bmN0aW9uIE51bWJlciBTdHJpbmcgRGF0ZSBSZWdF",
				"eHAiLnNwbGl0KCIgIik7TDxNLmxlbmd0aDtMKyspaltNW0xdXT1LKChuZXcgZigicmV0dXJuICIrTVtM",
				"XSkpKCkpO2ouT2JqZWN0PWMsai5NYXRoPWgscyhjLHtrZXlzOmZ1bmN0aW9uKGEpe3ZhciBiPVtdLGM7",
				"Zm9yKGMgaW4gYSliLnB1c2goYyk7cmV0dXJuIGJ9LHZhbHVlczpmdW5jdGlvbihhKXt2YXIgYj1bXSxj",
				"O2ZvcihjIGluIGEpYi5wdXNoKGFbY10pO3JldHVybiBifSxlYWNoOmZ1bmN0aW9uKGEsYixjKXtmb3Io",
				"dmFyIGQgaW4gYSliLmNhbGwoYyxkLGFbZF0pO3JldHVybiBhfSxlbXB0eTpmdW5jdGlvbihhKXtmb3Io",
				"dmFyIGIgaW4gYSlyZXR1cm4hMTtyZXR1cm4hMH0sY2xvbmU6ZnVuY3Rpb24oYSl7cmV0dXJuIGMubWVy",
				"Z2UoYSl9LHdpdGhvdXQ6ZnVuY3Rpb24oKXt2YXIgYT1KKGFyZ3VtZW50cyksYj1hLnNoaWZ0KCksYz17",
				"fSxkO2ZvcihkIGluIGIpYS5pbmNsdWRlKGQpfHwoY1tkXT1iW2RdKTtyZXR1cm4gY30sb25seTpmdW5j",
				"dGlvbigpe3ZhciBhPUooYXJndW1lbnRzKSxiPWEuc2hpZnQoKSxjPXt9LGQ9MCxlPWEubGVuZ3RoO2Zv",
				"cig7ZDxlO2QrKylhW2RdaW4gYiYmKGNbYVtkXV09YlthW2RdXSk7cmV0dXJuIGN9LG1lcmdlOmZ1bmN0",
				"aW9uKCl7dmFyIGE9e30sYj0wLGQ9YXJndW1lbnRzLGU9ZC5sZW5ndGgsZjtmb3IoO2I8ZTtiKyspaWYo",
				"QShkW2JdKSlmb3IoZiBpbiBkW2JdKWFbZl09QShkW2JdW2ZdKSYmIShkW2JdW2ZdaW5zdGFuY2VvZiBi",
				"Yyk/Yy5tZXJnZShmIGluIGE/YVtmXTp7fSxkW2JdW2ZdKTpkW2JdW2ZdO3JldHVybiBhfSx0b1F1ZXJ5",
				"U3RyaW5nOmZ1bmN0aW9uKGEpe3ZhciBiPU8oYSksYz0wLGQ9W107Zm9yKDtjPGIubGVuZ3RoO2MrKylk",
				"LnB1c2goZW5jb2RlVVJJQ29tcG9uZW50KGJbY11bMF0pKyI9IitlbmNvZGVVUklDb21wb25lbnQoIiIr",
				"YltjXVsxXSkpO3JldHVybiBkLmpvaW4oIiYiKX19LCEwKTt2YXIgUD1oLnJhbmRvbTtoLnJhbmRvbT1m",
				"dW5jdGlvbihhLGIpe2lmKGFyZ3VtZW50cy5sZW5ndGg9PT0wKXJldHVybiBQKCk7YXJndW1lbnRzLmxl",
				"bmd0aD09PTEmJihiPWEsYT0wKTtyZXR1cm5+fihQKCkqKGItYSsxKSt+fmEpfTt2YXIgUT1rLnNvcnQs",
				"Uj1rLmZvckVhY2h8fGZ1bmN0aW9uKGEsYil7Zm9yKHZhciBjPTAsZD10aGlzLmxlbmd0aDtjPGQ7Yysr",
				"KWEuY2FsbChiLHRoaXNbY10sYyx0aGlzKX0sUz1rLmZpbHRlcnx8ZnVuY3Rpb24oYSxiKXtmb3IodmFy",
				"IGM9W10sZD0wLGU9MCxmPXRoaXMubGVuZ3RoO2U8ZjtlKyspYS5jYWxsKGIsdGhpc1tlXSxlLHRoaXMp",
				"JiYoY1tkKytdPXRoaXNbZV0pO3JldHVybiBjfSxUPWZ1bmN0aW9uKGEsYil7Zm9yKHZhciBjPVtdLGQ9",
				"MCxlPTAsZj10aGlzLmxlbmd0aDtlPGY7ZSsrKWEuY2FsbChiLHRoaXNbZV0sZSx0aGlzKXx8KGNbZCsr",
				"XT10aGlzW2VdKTtyZXR1cm4gY30sVT1rLm1hcHx8ZnVuY3Rpb24oYSxiKXtmb3IodmFyIGM9W10sZD0w",
				"LGU9dGhpcy5sZW5ndGg7ZDxlO2QrKyljW2RdPWEuY2FsbChiLHRoaXNbZF0sZCx0aGlzKTtyZXR1cm4g",
				"Y30sVj1rLnNvbWV8fGZ1bmN0aW9uKGEsYil7Zm9yKHZhciBjPTAsZD10aGlzLmxlbmd0aDtjPGQ7Yysr",
				"KWlmKGEuY2FsbChiLHRoaXNbY10sYyx0aGlzKSlyZXR1cm4hMDtyZXR1cm4hMX0sVz1rLmV2ZXJ5fHxm",
				"dW5jdGlvbihhLGIpe2Zvcih2YXIgYz0wLGQ9dGhpcy5sZW5ndGg7YzxkO2MrKylpZighYS5jYWxsKGIs",
				"dGhpc1tjXSxjLHRoaXMpKXJldHVybiExO3JldHVybiEwfSxYPWZ1bmN0aW9uKGEsYil7Zm9yKHZhciBj",
				"PTAsZD10aGlzLmxlbmd0aDtjPGQ7YysrKWlmKGEuY2FsbChiLHRoaXNbY10sYyx0aGlzKSlyZXR1cm4g",
				"dGhpc1tjXTtyZXR1cm4gaX0sWT1mdW5jdGlvbihhLGIpe2Zvcih2YXIgYz10aGlzLmxlbmd0aC0xO2M+",
				"LTE7Yy0tKWlmKGEuY2FsbChiLHRoaXNbY10sYyx0aGlzKSlyZXR1cm4gdGhpc1tjXTtyZXR1cm4gaX07",
				"ZC5pbmNsdWRlKHtpbmRleE9mOmsuaW5kZXhPZnx8ZnVuY3Rpb24oYSxiKXtmb3IodmFyIGM9YjwwP2gu",
				"bWF4KDAsdGhpcy5sZW5ndGgrYik6Ynx8MCxkPXRoaXMubGVuZ3RoO2M8ZDtjKyspaWYodGhpc1tjXT09",
				"PWEpcmV0dXJuIGM7cmV0dXJuLTF9LGxhc3RJbmRleE9mOmsubGFzdEluZGV4T2Z8fGZ1bmN0aW9uKGEp",
				"e2Zvcih2YXIgYj10aGlzLmxlbmd0aC0xO2I+LTE7Yi0tKWlmKHRoaXNbYl09PT1hKXJldHVybiBiO3Jl",
				"dHVybi0xfSxmaXJzdDpmdW5jdGlvbigpe3JldHVybiBhcmd1bWVudHMubGVuZ3RoP18oWCx0aGlzLGFy",
				"Z3VtZW50cyk6dGhpc1swXX0sbGFzdDpmdW5jdGlvbigpe3JldHVybiBhcmd1bWVudHMubGVuZ3RoP18o",
				"WSx0aGlzLGFyZ3VtZW50cyk6dGhpc1t0aGlzLmxlbmd0aC0xXX0scmFuZG9tOmZ1bmN0aW9uKCl7cmV0",
				"dXJuIHRoaXMubGVuZ3RoPT09MD9pOnRoaXNbaC5yYW5kb20odGhpcy5sZW5ndGgtMSldfSxzaXplOmZ1",
				"bmN0aW9uKCl7cmV0dXJuIHRoaXMubGVuZ3RofSxjbGVhbjpmdW5jdGlvbigpe3RoaXMubGVuZ3RoPTA7",
				"cmV0dXJuIHRoaXN9LGVtcHR5OmZ1bmN0aW9uKCl7cmV0dXJuIHRoaXMubGVuZ3RoPT09MH0sY2xvbmU6",
				"ZnVuY3Rpb24oKXtyZXR1cm4gdGhpcy5zbGljZSgwKX0sZWFjaDpmdW5jdGlvbigpe18oUix0aGlzLGFy",
				"Z3VtZW50cyk7cmV0dXJuIHRoaXN9LGZvckVhY2g6UixtYXA6ZnVuY3Rpb24oKXtyZXR1cm4gXyhVLHRo",
				"aXMsYXJndW1lbnRzKX0sZmlsdGVyOmZ1bmN0aW9uKCl7cmV0dXJuIF8oUyx0aGlzLGFyZ3VtZW50cyl9",
				"LHJlamVjdDpmdW5jdGlvbigpe3JldHVybiBfKFQsdGhpcyxhcmd1bWVudHMpfSxzb21lOmZ1bmN0aW9u",
				"KGEpe3JldHVybiBfKFYsdGhpcyxhP2FyZ3VtZW50czpbYmFdKX0sZXZlcnk6ZnVuY3Rpb24oYSl7cmV0",
				"dXJuIF8oVyx0aGlzLGE/YXJndW1lbnRzOltiYV0pfSx3YWxrOmZ1bmN0aW9uKCl7dGhpcy5tYXAuYXBw",
				"bHkodGhpcyxhcmd1bWVudHMpLmZvckVhY2goZnVuY3Rpb24oYSxiKXt0aGlzW2JdPWF9LHRoaXMpO3Jl",
				"dHVybiB0aGlzfSxtZXJnZTpmdW5jdGlvbigpe2Zvcih2YXIgYT10aGlzLmNsb25lKCksYixjPTA7Yzxh",
				"cmd1bWVudHMubGVuZ3RoO2MrKyl7Yj1OKGFyZ3VtZW50c1tjXSk7Zm9yKHZhciBkPTA7ZDxiLmxlbmd0",
				"aDtkKyspYS5pbmRleE9mKGJbZF0pPT0tMSYmYS5wdXNoKGJbZF0pfXJldHVybiBhfSxmbGF0dGVuOmZ1",
				"bmN0aW9uKCl7dmFyIGE9W107dGhpcy5mb3JFYWNoKGZ1bmN0aW9uKGIpe0IoYik/YT1hLmNvbmNhdChi",
				"LmZsYXR0ZW4oKSk6YS5wdXNoKGIpfSk7cmV0dXJuIGF9LGNvbXBhY3Q6ZnVuY3Rpb24oKXtyZXR1cm4g",
				"dGhpcy53aXRob3V0KG51bGwsaSl9LHVuaXE6ZnVuY3Rpb24oKXtyZXR1cm5bXS5tZXJnZSh0aGlzKX0s",
				"aW5jbHVkZXM6ZnVuY3Rpb24oKXtmb3IodmFyIGE9MDthPGFyZ3VtZW50cy5sZW5ndGg7YSsrKWlmKHRo",
				"aXMuaW5kZXhPZihhcmd1bWVudHNbYV0pPT09LTEpcmV0dXJuITE7cmV0dXJuITB9LHdpdGhvdXQ6ZnVu",
				"Y3Rpb24oKXt2YXIgYT1uLmNhbGwoYXJndW1lbnRzKTtyZXR1cm4gdGhpcy5maWx0ZXIoZnVuY3Rpb24o",
				"Yil7cmV0dXJuIGEuaW5kZXhPZihiKT09PS0xfSl9LHNodWZmbGU6ZnVuY3Rpb24oKXt2YXIgYT10aGlz",
				"LmNsb25lKCksYixjLGQ9YS5sZW5ndGg7Zm9yKDtkPjA7Yj1oLnJhbmRvbShkLTEpLGM9YVstLWRdLGFb",
				"ZF09YVtiXSxhW2JdPWMpe31yZXR1cm4gYX0sc29ydDpmdW5jdGlvbihhKXtyZXR1cm4gUS5hcHBseSh0",
				"aGlzLGF8fCF6KHRoaXNbMF0pP2FyZ3VtZW50czpbYmJdKX0sc29ydEJ5OmZ1bmN0aW9uKCl7dmFyIGE9",
				"Wihhcmd1bWVudHMsdGhpcyk7cmV0dXJuIHRoaXMuc29ydChmdW5jdGlvbihiLGMpe3JldHVybiBiYihh",
				"WzBdLmNhbGwoYVsxXSxiKSxhWzBdLmNhbGwoYVsxXSxjKSl9KX0sbWluOmZ1bmN0aW9uKCl7cmV0dXJu",
				"IGgubWluLmFwcGx5KGgsdGhpcyl9LG1heDpmdW5jdGlvbigpe3JldHVybiBoLm1heC5hcHBseShoLHRo",
				"aXMpfSxzdW06ZnVuY3Rpb24oKXtmb3IodmFyIGE9MCxiPTAsYz10aGlzLmxlbmd0aDtiPGM7YSs9dGhp",
				"c1tiKytdKXt9cmV0dXJuIGF9fSksay5pbmNsdWRlPWsuaW5jbHVkZXMsZS5pbmNsdWRlKHtlbXB0eTpm",
				"dW5jdGlvbigpe3JldHVybiB0aGlzPT0iIn0sYmxhbms6ZnVuY3Rpb24oKXtyZXR1cm4gdGhpcz09ITF9",
				"LHRyaW06ZS5wcm90b3R5cGUudHJpbXx8ZnVuY3Rpb24oKXt2YXIgYT10aGlzLnJlcGxhY2UoL15cc1xz",
				"Ki8sIiIpLGI9YS5sZW5ndGg7d2hpbGUoL1xzLy50ZXN0KGEuY2hhckF0KC0tYikpKXt9cmV0dXJuIGEu",
				"c2xpY2UoMCxiKzEpfSxzdHJpcFRhZ3M6ZnVuY3Rpb24oKXtyZXR1cm4gdGhpcy5yZXBsYWNlKC88XC8/",
				"W14+XSs+L2lnLCIiKX0sc3RyaXBTY3JpcHRzOmZ1bmN0aW9uKGEpe3ZhciBiPSIiLGM9dGhpcy5yZXBs",
				"YWNlKC88c2NyaXB0W14+XSo+KFtcc1xTXSo/KTxcL3NjcmlwdD4vaW1nLGZ1bmN0aW9uKGEsYyl7Yis9",
				"YysiXG4iO3JldHVybiIifSk7YT09PSEwP3QoYik6eChhKSYmYShiLGMpO3JldHVybiBjfSxleHRyYWN0",
				"U2NyaXB0czpmdW5jdGlvbigpe3ZhciBhPSIiO3RoaXMuc3RyaXBTY3JpcHRzKGZ1bmN0aW9uKGIpe2E9",
				"Yn0pO3JldHVybiBhfSxldmFsU2NyaXB0czpmdW5jdGlvbigpe3RoaXMuc3RyaXBTY3JpcHRzKCEwKTty",
				"ZXR1cm4gdGhpc30sY2FtZWxpemU6ZnVuY3Rpb24oKXtyZXR1cm4gdGhpcy5yZXBsYWNlKC8oXC18Xykr",
				"KC4pPy9nLGZ1bmN0aW9uKGEsYixjKXtyZXR1cm4gYz9jLnRvVXBwZXJDYXNlKCk6IiJ9KX0sdW5kZXJz",
				"Y29yZWQ6ZnVuY3Rpb24oKXtyZXR1cm4gdGhpcy5yZXBsYWNlKC8oW2EtelxkXSkoW0EtWl0rKS9nLCIk",
				"MV8kMiIpLnJlcGxhY2UoL1wtL2csIl8iKS50b0xvd2VyQ2FzZSgpfSxjYXBpdGFsaXplOmZ1bmN0aW9u",
				"KCl7cmV0dXJuIHRoaXMuY2hhckF0KDApLnRvVXBwZXJDYXNlKCkrdGhpcy5zdWJzdHJpbmcoMSkudG9M",
				"b3dlckNhc2UoKX0sZGFzaGVyaXplOmZ1bmN0aW9uKCl7cmV0dXJuIHRoaXMudW5kZXJzY29yZWQoKS5y",
				"ZXBsYWNlKC9fL2csIi0iKX0saW5jbHVkZXM6ZnVuY3Rpb24oYSl7cmV0dXJuIHRoaXMuaW5kZXhPZihh",
				"KSE9LTF9LHN0YXJ0c1dpdGg6ZnVuY3Rpb24oYSxiKXtyZXR1cm4oYiE9PSEwP3RoaXMuaW5kZXhPZihh",
				"KTp0aGlzLnRvTG93ZXJDYXNlKCkuaW5kZXhPZihhLnRvTG93ZXJDYXNlKCkpKT09PTB9LGVuZHNXaXRo",
				"OmZ1bmN0aW9uKGEsYil7cmV0dXJuIHRoaXMubGVuZ3RoLShiIT09ITA/dGhpcy5sYXN0SW5kZXhPZihh",
				"KTp0aGlzLnRvTG93ZXJDYXNlKCkubGFzdEluZGV4T2YoYS50b0xvd2VyQ2FzZSgpKSk9PT1hLmxlbmd0",
				"aH0sdG9JbnQ6ZnVuY3Rpb24oYSl7cmV0dXJuIHBhcnNlSW50KHRoaXMsYT09PWk/MTA6YSl9LHRvRmxv",
				"YXQ6ZnVuY3Rpb24oYSl7cmV0dXJuIHBhcnNlRmxvYXQoYT09PSEwP3RoaXM6dGhpcy5yZXBsYWNlKCIs",
				"IiwiLiIpLnJlcGxhY2UoLyhcZCktKFxkKS8sIiQxLiQyIikpfX0pLGUucHJvdG90eXBlLmluY2x1ZGU9",
				"ZS5wcm90b3R5cGUuaW5jbHVkZXMsZi5pbmNsdWRlKHtiaW5kOmZ1bmN0aW9uKCl7dmFyIGE9Sihhcmd1",
				"bWVudHMpLGI9YS5zaGlmdCgpLGM9dGhpcztyZXR1cm4gZnVuY3Rpb24oKXtyZXR1cm4gYy5hcHBseShi",
				"LGEubGVuZ3RoIT09MHx8YXJndW1lbnRzLmxlbmd0aCE9PTA/YS5jb25jYXQoSihhcmd1bWVudHMpKTph",
				"KX19LGJpbmRBc0V2ZW50TGlzdGVuZXI6ZnVuY3Rpb24oKXt2YXIgYT1KKGFyZ3VtZW50cyksYj1hLnNo",
				"aWZ0KCksYz10aGlzO3JldHVybiBmdW5jdGlvbihkKXtyZXR1cm4gYy5hcHBseShiLFtkXS5jb25jYXQo",
				"YSkuY29uY2F0KEooYXJndW1lbnRzKSkpfX0sY3Vycnk6ZnVuY3Rpb24oKXtyZXR1cm4gdGhpcy5iaW5k",
				"LmFwcGx5KHRoaXMsW3RoaXNdLmNvbmNhdChKKGFyZ3VtZW50cykpKX0scmN1cnJ5OmZ1bmN0aW9uKCl7",
				"dmFyIGE9Sihhcmd1bWVudHMpLGI9dGhpcztyZXR1cm4gZnVuY3Rpb24oKXtyZXR1cm4gYi5hcHBseShi",
				"LEooYXJndW1lbnRzKS5jb25jYXQoYSkpfX0sZGVsYXk6ZnVuY3Rpb24oKXt2YXIgYT1KKGFyZ3VtZW50",
				"cyksYj1hLnNoaWZ0KCksYz1uZXcgZyhzZXRUaW1lb3V0KHRoaXMuYmluZC5hcHBseSh0aGlzLFt0aGlz",
				"XS5jb25jYXQoYSkpLGIpKTtjLmNhbmNlbD1mdW5jdGlvbigpe2NsZWFyVGltZW91dCh0aGlzKX07cmV0",
				"dXJuIGN9LHBlcmlvZGljYWw6ZnVuY3Rpb24oKXt2YXIgYT1KKGFyZ3VtZW50cyksYj1hLnNoaWZ0KCks",
				"Yz1uZXcgZyhzZXRJbnRlcnZhbCh0aGlzLmJpbmQuYXBwbHkodGhpcyxbdGhpc10uY29uY2F0KGEpKSxi",
				"KSk7Yy5zdG9wPWZ1bmN0aW9uKCl7Y2xlYXJJbnRlcnZhbCh0aGlzKX07cmV0dXJuIGN9LGNoYWluOmZ1",
				"bmN0aW9uKCl7dmFyIGE9Sihhcmd1bWVudHMpLGI9YS5zaGlmdCgpLGM9dGhpcztyZXR1cm4gZnVuY3Rp",
				"b24oKXt2YXIgZD1jLmFwcGx5KGMsYXJndW1lbnRzKTtiLmFwcGx5KGIsYSk7cmV0dXJuIGR9fX0pLGcu",
				"aW5jbHVkZSh7dGltZXM6ZnVuY3Rpb24oYSxiKXtmb3IodmFyIGM9MDtjPHRoaXM7YysrKWEuY2FsbChi",
				"LGMpO3JldHVybiB0aGlzfSx1cHRvOmZ1bmN0aW9uKGEsYixjKXtmb3IodmFyIGQ9dGhpcyswO2Q8PWE7",
				"ZCsrKWIuY2FsbChjLGQpO3JldHVybiB0aGlzfSxkb3dudG86ZnVuY3Rpb24oYSxiLGMpe2Zvcih2YXIg",
				"ZD10aGlzKzA7ZD49YTtkLS0pYi5jYWxsKGMsZCk7cmV0dXJuIHRoaXN9LHRvOmZ1bmN0aW9uKGEsYixj",
				"KXt2YXIgZD10aGlzKzAsZT1hLGY9W10sZz1kO2I9Ynx8ZnVuY3Rpb24oYSl7cmV0dXJuIGF9O2lmKGU+",
				"ZClmb3IoO2c8PWU7ZysrKWYucHVzaChiLmNhbGwoYyxnKSk7ZWxzZSBmb3IoO2c+PWU7Zy0tKWYucHVz",
				"aChiLmNhbGwoYyxnKSk7cmV0dXJuIGZ9LGFiczpmdW5jdGlvbigpe3JldHVybiBoLmFicyh0aGlzKX0s",
				"cm91bmQ6ZnVuY3Rpb24oYSl7cmV0dXJuIGE/cGFyc2VGbG9hdCh0aGlzLnRvRml4ZWQoYSkpOmgucm91",
				"bmQodGhpcyl9LGNlaWw6ZnVuY3Rpb24oKXtyZXR1cm4gaC5jZWlsKHRoaXMpfSxmbG9vcjpmdW5jdGlv",
				"bigpe3JldHVybiBoLmZsb29yKHRoaXMpfSxtaW46ZnVuY3Rpb24oYSl7cmV0dXJuIHRoaXM8YT9hOnRo",
				"aXMrMH0sbWF4OmZ1bmN0aW9uKGEpe3JldHVybiB0aGlzPmE/YTp0aGlzKzB9fSksUmVnRXhwLmVzY2Fw",
				"ZT1mdW5jdGlvbihhKXtyZXR1cm4oIiIrYSkucmVwbGFjZSgvKFsuKis/XF49IToke30oKXxcW1xdXC9c",
				"XF0pL2csIlxcJDEiKX0sYS5KU09OfHwoYS5KU09OPWZ1bmN0aW9uKCl7ZnVuY3Rpb24gZyhhKXtyZXR1",
				"cm4oYTwxMD8iMCI6IiIpK2F9ZnVuY3Rpb24gZChhKXtyZXR1cm4gYS5yZXBsYWNlKGMsZnVuY3Rpb24o",
				"YSl7cmV0dXJuIGJbYV18fCJcXHUiKygiMDAwMCIrYS5jaGFyQ29kZUF0KDApLnRvU3RyaW5nKDE2KSku",
				"c2xpY2UoLTQpfSl9dmFyIGE9L1tcdTAwMDBcdTAwYWRcdTA2MDAtXHUwNjA0XHUwNzBmXHUxN2I0XHUx",
				"N2I1XHUyMDBjLVx1MjAwZlx1MjAyOC1cdTIwMmZcdTIwNjAtXHUyMDZmXHVmZWZmXHVmZmYwLVx1ZmZm",
				"Zl0vZyxiPXsiXGIiOiJcXGIiLCJcdCI6IlxcdCIsIlxuIjoiXFxuIiwiXGYiOiJcXGYiLCJcciI6Ilxc",
				"ciIsJyInOidcXCInLCJcXCI6IlxcXFwifSxjPS9bXFxcIlx4MDAtXHgxZlx4N2YtXHg5Zlx1MDBhZFx1",
				"MDYwMC1cdTA2MDRcdTA3MGZcdTE3YjRcdTE3YjVcdTIwMGMtXHUyMDBmXHUyMDI4LVx1MjAyZlx1MjA2",
				"MC1cdTIwNmZcdWZlZmZcdWZmZjAtXHVmZmZmXS9nO3JldHVybntzdHJpbmdpZnk6ZnVuY3Rpb24oYSl7",
				"c3dpdGNoKHR5cGVvZiBhKXtjYXNlImJvb2xlYW4iOnJldHVybiBlKGEpO2Nhc2UibnVtYmVyIjpyZXR1",
				"cm4gZShhKzApO2Nhc2Uic3RyaW5nIjpyZXR1cm4nIicrZChhKSsnIic7Y2FzZSJvYmplY3QiOmlmKGE9",
				"PT1udWxsKXJldHVybiJudWxsIjtpZihCKGEpKXJldHVybiJbIitKKGEpLm1hcChKU09OLnN0cmluZ2lm",
				"eSkuam9pbigiLCIpKyJdIjtpZihtLmNhbGwoYSk9PT0iW29iamVjdCBEYXRlXSIpcmV0dXJuJyInK2Eu",
				"Z2V0VVRDRnVsbFllYXIoKSsiLSIrZyhhLmdldFVUQ01vbnRoKCkrMSkrIi0iK2coYS5nZXRVVENEYXRl",
				"KCkpKyJUIitnKGEuZ2V0VVRDSG91cnMoKSkrIjoiK2coYS5nZXRVVENNaW51dGVzKCkpKyI6IitnKGEu",
				"Z2V0VVRDU2Vjb25kcygpKSsiLiIrZyhhLmdldE1pbGxpc2Vjb25kcygpKSsnWiInO3ZhciBiPVtdLGM7",
				"Zm9yKGMgaW4gYSliLnB1c2goJyInK2MrJyI6JytKU09OLnN0cmluZ2lmeShhW2NdKSk7cmV0dXJuInsi",
				"K2Iuam9pbigiLCIpKyJ9In19LHBhcnNlOmZ1bmN0aW9uKGIpe2lmKHkoYikmJmIpe2I9Yi5yZXBsYWNl",
				"KGEsZnVuY3Rpb24oYSl7cmV0dXJuIlxcdSIrKCIwMDAwIithLmNoYXJDb2RlQXQoMCkudG9TdHJpbmco",
				"MTYpKS5zbGljZSgtNCl9KTtpZigvXltcXSw6e31cc10qJC8udGVzdChiLnJlcGxhY2UoL1xcKD86WyJc",
				"XFwvYmZucnRdfHVbMC05YS1mQS1GXXs0fSkvZywiQCIpLnJlcGxhY2UoLyJbXiJcXFxuXHJdKiJ8dHJ1",
				"ZXxmYWxzZXxudWxsfC0/XGQrKD86XC5cZCopPyg/OltlRV1bK1wtXT9cZCspPy9nLCJdIikucmVwbGFj",
				"ZSgvKD86Xnw6fCwpKD86XHMqXFspKy9nLCIiKSkpcmV0dXJuKG5ldyBmKCJyZXR1cm4gIitiKSkoKX10",
				"aHJvdyJKU09OIHBhcnNlIGVycm9yOiAiK2J9fX0oKSk7dmFyIGJjPWouQ2xhc3M9ZnVuY3Rpb24oKXt2",
				"YXIgYT1KKGFyZ3VtZW50cykuc2xpY2UoMCwyKSxiPWEucG9wKCl8fHt9LGM9YS5wb3AoKSxkPWFyZ3Vt",
				"ZW50c1syXSxlPWZ1bmN0aW9uKCl7fTshYS5sZW5ndGgmJiFBKGIpJiYoYz1iLGI9e30pLCFkJiZjJiYo",
				"Yz09PWJ3fHxjLmFuY2VzdG9ycy5pbmNsdWRlKGJ3KSkmJihkPWJ4KCkpLGQ9cyhkfHxmdW5jdGlvbigp",
				"e2JpKHRoaXMpO3JldHVybiJpbml0aWFsaXplImluIHRoaXM/dGhpcy5pbml0aWFsaXplLmFwcGx5KHRo",
				"aXMsYXJndW1lbnRzKTp0aGlzfSxiZCksYz1jfHxiYyxlLnByb3RvdHlwZT1jLnByb3RvdHlwZSxkLnBy",
				"b3RvdHlwZT1uZXcgZSxkLnBhcmVudD1jLGQucHJvdG90eXBlLmNvbnN0cnVjdG9yPWQsZC5hbmNlc3Rv",
				"cnM9W107d2hpbGUoYylkLmFuY2VzdG9ycy5wdXNoKGMpLGM9Yy5wYXJlbnQ7WyJleHRlbmQiLCJpbmNs",
				"dWRlIl0uZWFjaChmdW5jdGlvbihhKXthIGluIGImJmRbYV0uYXBwbHkoZCxOKGJbYV0pKX0pO3JldHVy",
				"biBkLmluY2x1ZGUoYil9LGJkPXtleHRlbmQ6ZnVuY3Rpb24oKXtKKGFyZ3VtZW50cykuZmlsdGVyKEEp",
				"LmVhY2goZnVuY3Rpb24oYSl7cyh0aGlzLGJmKGEsITApKSxiZyh0aGlzLGEsITApfSx0aGlzKTtyZXR1",
				"cm4gdGhpc30saW5jbHVkZTpmdW5jdGlvbigpe3ZhciBhPVt0aGlzXS5jb25jYXQodGhpcy5hbmNlc3Rv",
				"cnMpO0ooYXJndW1lbnRzKS5maWx0ZXIoQSkuZWFjaChmdW5jdGlvbihiKXtjLmVhY2goYmYoYiwhMSks",
				"ZnVuY3Rpb24oYixjKXtmb3IodmFyIGQsZT0wLGY9YS5sZW5ndGg7ZTxmO2UrKylpZihiIGluIGFbZV0u",
				"cHJvdG90eXBlKXtkPWFbZV0ucHJvdG90eXBlW2JdO2JyZWFrfXRoaXMucHJvdG90eXBlW2JdPXgoYykm",
				"JngoZCk/ZnVuY3Rpb24oKXt0aGlzLiRzdXBlcj1kO3JldHVybiBjLmFwcGx5KHRoaXMsYXJndW1lbnRz",
				"KX06Y30sdGhpcyksYmcodGhpcyxiLCExKX0sdGhpcyk7cmV0dXJuIHRoaXN9fSxiZT1IKCJzZWxmRXh0",
				"ZW5kZWQgc2VsZl9leHRlbmRlZCBzZWxmSW5jbHVkZWQgc2VsZl9pbmNsdWRlZCBleHRlbmQgaW5jbHVk",
				"ZSIpO3MoYmMsYmQpLGJjLnByb3RvdHlwZS4kc3VwZXI9aTt2YXIgYmo9ai5PcHRpb25zPXtzZXRPcHRp",
				"b25zOmZ1bmN0aW9uKGEpe3ZhciBiPXRoaXMub3B0aW9ucz1zKHMoe30sYy5jbG9uZShiaCh0aGlzLCJP",
				"cHRpb25zIikpKSxhKSxkLGU7aWYoeCh0aGlzLm9uKSlmb3IoZSBpbiBiKWlmKGQ9ZS5tYXRjaCgvb24o",
				"W0EtWl1bQS1aYS16XSspLykpdGhpcy5vbihkWzFdLnRvTG93ZXJDYXNlKCksYltlXSksZGVsZXRlIGJb",
				"ZV07cmV0dXJuIHRoaXN9LGN1dE9wdGlvbnM6ZnVuY3Rpb24oYSl7dmFyIGI9SihhKTt0aGlzLnNldE9w",
				"dGlvbnMoQShiLmxhc3QoKSk/Yi5wb3AoKTp7fSk7cmV0dXJuIGJ9fSxiaz1qLk9ic2VydmVyPW5ldyBi",
				"Yyh7aW5jbHVkZTpiaixpbml0aWFsaXplOmZ1bmN0aW9uKGEpe3RoaXMuc2V0T3B0aW9ucyhhKSxibSh0",
				"aGlzLGJoKHRoaXMsIkV2ZW50cyIpKTtyZXR1cm4gdGhpc30sb246ZnVuY3Rpb24oKXtibih0aGlzLGFy",
				"Z3VtZW50cyxmdW5jdGlvbihhKXtyZXR1cm4gYX0pO3JldHVybiB0aGlzfSxvYnNlcnZlczpmdW5jdGlv",
				"bihhLGIpe3koYSl8fChiPWEsYT1udWxsKSx5KGIpJiYoYj1iIGluIHRoaXM/dGhpc1tiXTpudWxsKTty",
				"ZXR1cm4odGhpcy4kbGlzdGVuZXJzfHxbXSkuc29tZShmdW5jdGlvbihjKXtyZXR1cm4gYSYmYj9jLmU9",
				"PT1hJiZjLmY9PT1iOmE/Yy5lPT09YTpjLmY9PT1ifSl9LHN0b3BPYnNlcnZpbmc6ZnVuY3Rpb24oYSxi",
				"KXtibyh0aGlzLGEsYixmdW5jdGlvbigpe30pO3JldHVybiB0aGlzfSxsaXN0ZW5lcnM6ZnVuY3Rpb24o",
				"YSl7cmV0dXJuKHRoaXMuJGxpc3RlbmVyc3x8W10pLmZpbHRlcihmdW5jdGlvbihiKXtyZXR1cm4hYXx8",
				"Yi5lPT09YX0pLm1hcChmdW5jdGlvbihhKXtyZXR1cm4gYS5mfSkudW5pcSgpfSxmaXJlOmZ1bmN0aW9u",
				"KCl7dmFyIGE9Sihhcmd1bWVudHMpLGI9YS5zaGlmdCgpOyh0aGlzLiRsaXN0ZW5lcnN8fFtdKS5lYWNo",
				"KGZ1bmN0aW9uKGMpe2MuZT09PWImJmMuZi5hcHBseSh0aGlzLGMuYS5jb25jYXQoYSkpfSx0aGlzKTty",
				"ZXR1cm4gdGhpc319KSxibD1iay5jcmVhdGU9ZnVuY3Rpb24oYSxiKXtzKGEsYy53aXRob3V0KGJrLnBy",
				"b3RvdHlwZSwiaW5pdGlhbGl6ZSIsInNldE9wdGlvbnMiKSwhMCk7cmV0dXJuIGJtKGEsYnx8YmgoYSwi",
				"RXZlbnRzIikpfSxibT1iay5jcmVhdGVTaG9ydGN1dHM9ZnVuY3Rpb24oYSxiKXsoYnx8W10pLmVhY2go",
				"ZnVuY3Rpb24oYil7dmFyIGM9Im9uIitiLnJlcGxhY2UoLyhefF98OikoW2Etel0pL2csZnVuY3Rpb24o",
				"YSxiLGMpe3JldHVybiBjLnRvVXBwZXJDYXNlKCl9KTtjIGluIGF8fChhW2NdPWZ1bmN0aW9uKCl7cmV0",
				"dXJuIHRoaXMub24uYXBwbHkodGhpcyxbYl0uY29uY2F0KEooYXJndW1lbnRzKSkpfSl9KTtyZXR1cm4g",
				"YX0sYnA9bmF2aWdhdG9yLnVzZXJBZ2VudCxicT0ib3BlcmEiaW4gYSxicj0iYXR0YWNoRXZlbnQiaW4g",
				"YSYmIWJxLGJzPWouQnJvd3Nlcj17SUU6YnIsT3BlcmE6YnEsV2ViS2l0OmJwLmluY2x1ZGUoIkFwcGxl",
				"V2ViS2l0LyIpLEdlY2tvOmJwLmluY2x1ZGUoIkdlY2tvIikmJiFicC5pbmNsdWRlKCJLSFRNTCIpLE1v",
				"YmlsZVNhZmFyaTovQXBwbGUuKk1vYmlsZS4qU2FmYXJpLy50ZXN0KGJwKSxLb25xdWVyb3I6YnAuaW5j",
				"bHVkZSgiS29ucXVlcm9yIiksT0xEOiFiLnF1ZXJ5U2VsZWN0b3IsSUU4TDohMX0sYnQ9ITEsYnU9ISgi",
				"b3BhY2l0eSJpbiBvLnN0eWxlKSYmImZpbHRlciJpbiBvLnN0eWxlO3RyeXtiLmNyZWF0ZUVsZW1lbnQo",
				"IjxpbnB1dC8+IiksYnMuT0xEPWJzLklFOEw9YnQ9ITB9Y2F0Y2goYnYpe312YXIgYnc9ai5XcmFwcGVy",
				"PW5ldyBiYyh7XzppLGluaXRpYWxpemU6ZnVuY3Rpb24oYSl7dGhpcy5fPWF9fSk7YncuQ2FjaGU9cSxi",
				"dy5DYXN0PWZ1bmN0aW9uKGEpe3JldHVybiBhLnRhZ05hbWUgaW4gYkg/YkhbYS50YWdOYW1lXTppfTt2",
				"YXIgYkI9ai5Eb2N1bWVudD1uZXcgYmMoYncse3dpbjpmdW5jdGlvbigpe3JldHVybiBiQSh0aGlzLl8u",
				"ZGVmYXVsdFZpZXd8fHRoaXMuXy5wYXJlbnRXaW5kb3cpfX0pLGJDPWJBKGIpLGJEPWouV2luZG93PW5l",
				"dyBiYyhidyx7d2luOmZ1bmN0aW9uKCl7cmV0dXJuIHRoaXN9LHNpemU6ZnVuY3Rpb24oKXt2YXIgYT10",
				"aGlzLl8sYj1hLmRvY3VtZW50LmRvY3VtZW50RWxlbWVudDtyZXR1cm4gYS5pbm5lcldpZHRoP3t4OmEu",
				"aW5uZXJXaWR0aCx5OmEuaW5uZXJIZWlnaHR9Ont4OmIuY2xpZW50V2lkdGgseTpiLmNsaWVudEhlaWdo",
				"dH19LHNjcm9sbHM6ZnVuY3Rpb24oKXt2YXIgYT10aGlzLl8sYj1hLmRvY3VtZW50LGM9Yi5ib2R5LGQ9",
				"Yi5kb2N1bWVudEVsZW1lbnQ7cmV0dXJuIGEucGFnZVhPZmZzZXR8fGEucGFnZVlPZmZzZXQ/e3g6YS5w",
				"YWdlWE9mZnNldCx5OmEucGFnZVlPZmZzZXR9OmMmJihjLnNjcm9sbExlZnR8fGMuc2Nyb2xsVG9wKT97",
				"eDpjLnNjcm9sbExlZnQseTpjLnNjcm9sbFRvcH06e3g6ZC5zY3JvbGxMZWZ0LHk6ZC5zY3JvbGxUb3B9",
				"fSxzY3JvbGxUbzpmdW5jdGlvbihhLGIsYyl7dmFyIGQ9YSxlPWIsZj16KGEpP251bGw6RShhKTtmIGlu",
				"c3RhbmNlb2YgYkcmJihhPWYucG9zaXRpb24oKSksQShhKSYmKGU9YS55LGQ9YS54KSxBKGM9Y3x8Yikm",
				"JmouRng/KG5ldyBjZy5TY3JvbGwodGhpcyxjKSkuc3RhcnQoe3g6ZCx5OmV9KTp0aGlzLl8uc2Nyb2xs",
				"VG8oZCxlKTtyZXR1cm4gdGhpc319KSxiRT1qLkV2ZW50PW5ldyBiYyhidyx7dHlwZTpudWxsLHdoaWNo",
				"Om51bGwsa2V5Q29kZTpudWxsLHRhcmdldDpudWxsLGN1cnJlbnRUYXJnZXQ6bnVsbCxyZWxhdGVkVGFy",
				"Z2V0Om51bGwscGFnZVg6bnVsbCxwYWdlWTpudWxsLGluaXRpYWxpemU6Ynosc3RvcFByb3BhZ2F0aW9u",
				"OmZ1bmN0aW9uKCl7dGhpcy5fLnN0b3BQcm9wYWdhdGlvbj90aGlzLl8uc3RvcFByb3BhZ2F0aW9uKCk6",
				"dGhpcy5fLmNhbmNlbEJ1YmJsZT0hMCx0aGlzLnN0b3BwZWQ9ITA7cmV0dXJuIHRoaXN9LHByZXZlbnRE",
				"ZWZhdWx0OmZ1bmN0aW9uKCl7dGhpcy5fLnByZXZlbnREZWZhdWx0P3RoaXMuXy5wcmV2ZW50RGVmYXVs",
				"dCgpOnRoaXMuXy5yZXR1cm5WYWx1ZT0hMTtyZXR1cm4gdGhpc30sc3RvcDpmdW5jdGlvbigpe3JldHVy",
				"biB0aGlzLnN0b3BQcm9wYWdhdGlvbigpLnByZXZlbnREZWZhdWx0KCl9LHBvc2l0aW9uOmZ1bmN0aW9u",
				"KCl7cmV0dXJue3g6dGhpcy5wYWdlWCx5OnRoaXMucGFnZVl9fSxvZmZzZXQ6ZnVuY3Rpb24oKXtpZih0",
				"aGlzLnRhcmdldCBpbnN0YW5jZW9mIGJHKXt2YXIgYT10aGlzLnRhcmdldC5wb3NpdGlvbigpO3JldHVy",
				"bnt4OnRoaXMucGFnZVgtYS54LHk6dGhpcy5wYWdlWS1hLnl9fXJldHVybiBudWxsfSxmaW5kOmZ1bmN0",
				"aW9uKGEpe2lmKHRoaXMudGFyZ2V0IGluc3RhbmNlb2YgYncmJnRoaXMuY3VycmVudFRhcmdldCBpbnN0",
				"YW5jZW9mIGJ3KXt2YXIgYj10aGlzLnRhcmdldC5fLGM9dGhpcy5jdXJyZW50VGFyZ2V0LmZpbmQoYSwh",
				"MCk7d2hpbGUoYil7aWYoYy5pbmRleE9mKGIpIT09LTEpcmV0dXJuIGJBKGIpO2I9Yi5wYXJlbnROb2Rl",
				"fX1yZXR1cm4gaX19LGJ6KSxiRj1bXSxiRz1qLkVsZW1lbnQ9bmV3IGJjKGJ3LHtpbml0aWFsaXplOmZ1",
				"bmN0aW9uKGEsYil7YksodGhpcyxhLGIpfX0sYnkpLGJIPWJHLldyYXBwZXJzPXt9LGJJPXt9LGJKPWZ1",
				"bmN0aW9uKGEsYyl7cmV0dXJuKGEgaW4gYkk/YklbYV06YklbYV09Yi5jcmVhdGVFbGVtZW50KGEpKS5j",
				"bG9uZU5vZGUoITEpfTtidCYmKGJKPWZ1bmN0aW9uKGEsYyl7YyE9PWkmJihhPT09ImlucHV0Inx8YT09",
				"PSJidXR0b24iKSYmKGE9IjwiK2ErJyBuYW1lPSInK2MubmFtZSsnIiB0eXBlPSInK2MudHlwZSsnIicr",
				"KGMuY2hlY2tlZD8iIGNoZWNrZWQiOiIiKSsiIC8+IixkZWxldGUgYy5uYW1lLGRlbGV0ZSBjLnR5cGUp",
				"O3JldHVybiBiLmNyZWF0ZUVsZW1lbnQoYSl9KSxiRy5pbmNsdWRlKHtwYXJlbnQ6ZnVuY3Rpb24oYSl7",
				"dmFyIGI9dGhpcy5fLnBhcmVudE5vZGUsYz1iJiZiLm5vZGVUeXBlO3JldHVybiBhP3RoaXMucGFyZW50",
				"cyhhKVswXTpjPT09MXx8Yz09PTk/YkEoYik6bnVsbH0scGFyZW50czpmdW5jdGlvbihhKXtyZXR1cm4g",
				"YkwodGhpcywicGFyZW50Tm9kZSIsYSl9LGNoaWxkcmVuOmZ1bmN0aW9uKGEpe3JldHVybiB0aGlzLmZp",
				"bmQoYSkuZmlsdGVyKGZ1bmN0aW9uKGEpe3JldHVybiBhLl8ucGFyZW50Tm9kZT09PXRoaXMuX30sdGhp",
				"cyl9LHNpYmxpbmdzOmZ1bmN0aW9uKGEpe3JldHVybiB0aGlzLnByZXZTaWJsaW5ncyhhKS5yZXZlcnNl",
				"KCkuY29uY2F0KHRoaXMubmV4dFNpYmxpbmdzKGEpKX0sbmV4dFNpYmxpbmdzOmZ1bmN0aW9uKGEpe3Jl",
				"dHVybiBiTCh0aGlzLCJuZXh0U2libGluZyIsYSl9LHByZXZTaWJsaW5nczpmdW5jdGlvbihhKXtyZXR1",
				"cm4gYkwodGhpcywicHJldmlvdXNTaWJsaW5nIixhKX0sbmV4dDpmdW5jdGlvbihhKXtyZXR1cm4hYSYm",
				"dGhpcy5fLm5leHRFbGVtZW50U2libGluZyE9PWk/YkEodGhpcy5fLm5leHRFbGVtZW50U2libGluZyk6",
				"dGhpcy5uZXh0U2libGluZ3MoYSlbMF19LHByZXY6ZnVuY3Rpb24oYSl7cmV0dXJuIWEmJnRoaXMuXy5w",
				"cmV2aW91c0VsZW1lbnRTaWJsaW5nIT09aT9iQSh0aGlzLl8ucHJldmlvdXNFbGVtZW50U2libGluZyk6",
				"dGhpcy5wcmV2U2libGluZ3MoYSlbMF19LHJlbW92ZTpmdW5jdGlvbigpe3ZhciBhPXRoaXMuXyxiPWEu",
				"cGFyZW50Tm9kZTtiJiZiLnJlbW92ZUNoaWxkKGEpO3JldHVybiB0aGlzfSxpbnNlcnQ6ZnVuY3Rpb24o",
				"YSxiKXt2YXIgYz1udWxsLGQ9dGhpcy5fO2I9Yj09PWk/ImJvdHRvbSI6Yix0eXBlb2YgYSE9PSJvYmpl",
				"Y3QiP2M9YT0iIithOmEgaW5zdGFuY2VvZiBiRyYmKGE9YS5fKSxiTVtiXShkLGEubm9kZVR5cGU9PT1p",
				"P2JRKGI9PT0iYm90dG9tInx8Yj09PSJ0b3AiP2Q6ZC5wYXJlbnROb2RlLGEpOmEpLGMhPT1udWxsJiZj",
				"LmV2YWxTY3JpcHRzKCk7cmV0dXJuIHRoaXN9LGluc2VydFRvOmZ1bmN0aW9uKGEsYil7RShhKS5pbnNl",
				"cnQodGhpcyxiKTtyZXR1cm4gdGhpc30sYXBwZW5kOmZ1bmN0aW9uKGEpe3JldHVybiB0aGlzLmluc2Vy",
				"dCh5KGEpP0ooYXJndW1lbnRzKS5qb2luKCIiKTphcmd1bWVudHMpfSx1cGRhdGU6ZnVuY3Rpb24oYSl7",
				"aWYodHlwZW9mIGEhPT0ib2JqZWN0Iil7YT0iIithO3RyeXt0aGlzLl8uaW5uZXJIVE1MPWF9Y2F0Y2go",
				"Yil7cmV0dXJuIHRoaXMuY2xlYW4oKS5pbnNlcnQoYSl9YS5ldmFsU2NyaXB0cygpO3JldHVybiB0aGlz",
				"fXJldHVybiB0aGlzLmNsZWFuKCkuaW5zZXJ0KGEpfSxodG1sOmZ1bmN0aW9uKGEpe3JldHVybiBhPT09",
				"aT90aGlzLl8uaW5uZXJIVE1MOnRoaXMudXBkYXRlKGEpfSx0ZXh0OmZ1bmN0aW9uKGEpe3JldHVybiBh",
				"PT09aT90aGlzLl8udGV4dENvbnRlbnQ9PT1pP3RoaXMuXy5pbm5lclRleHQ6dGhpcy5fLnRleHRDb250",
				"ZW50OnRoaXMudXBkYXRlKHRoaXMuZG9jKCkuXy5jcmVhdGVUZXh0Tm9kZShhKSl9LHJlcGxhY2U6ZnVu",
				"Y3Rpb24oYSl7cmV0dXJuIHRoaXMuaW5zZXJ0KGEsImluc3RlYWQiKX0sd3JhcDpmdW5jdGlvbihhKXt2",
				"YXIgYj10aGlzLl8sYz1iLnBhcmVudE5vZGU7YyYmKGE9RShhKS5fLGMucmVwbGFjZUNoaWxkKGEsYiks",
				"YS5hcHBlbmRDaGlsZChiKSk7cmV0dXJuIHRoaXN9LGNsZWFuOmZ1bmN0aW9uKCl7d2hpbGUodGhpcy5f",
				"LmZpcnN0Q2hpbGQpdGhpcy5fLnJlbW92ZUNoaWxkKHRoaXMuXy5maXJzdENoaWxkKTtyZXR1cm4gdGhp",
				"c30sZW1wdHk6ZnVuY3Rpb24oKXtyZXR1cm4gdGhpcy5odG1sKCkuYmxhbmsoKX0sY2xvbmU6ZnVuY3Rp",
				"b24oKXtyZXR1cm4gbmV3IGJHKHRoaXMuXy5jbG9uZU5vZGUoITApKX0saW5kZXg6ZnVuY3Rpb24oKXt2",
				"YXIgYT10aGlzLl8sYj1hLnBhcmVudE5vZGUuZmlyc3RDaGlsZCxjPTA7d2hpbGUoYiE9PWEpYi5ub2Rl",
				"VHlwZT09PTEmJmMrKyxiPWIubmV4dFNpYmxpbmc7cmV0dXJuIGN9fSk7dmFyIGJNPXtib3R0b206ZnVu",
				"Y3Rpb24oYSxiKXthLmFwcGVuZENoaWxkKGIpfSx0b3A6ZnVuY3Rpb24oYSxiKXthLmZpcnN0Q2hpbGQh",
				"PT1udWxsP2EuaW5zZXJ0QmVmb3JlKGIsYS5maXJzdENoaWxkKTphLmFwcGVuZENoaWxkKGIpfSxhZnRl",
				"cjpmdW5jdGlvbihhLGIpe3ZhciBjPWEucGFyZW50Tm9kZSxkPWEubmV4dFNpYmxpbmc7ZCE9PW51bGw/",
				"Yy5pbnNlcnRCZWZvcmUoYixkKTpjLmFwcGVuZENoaWxkKGIpfSxiZWZvcmU6ZnVuY3Rpb24oYSxiKXth",
				"LnBhcmVudE5vZGUuaW5zZXJ0QmVmb3JlKGIsYSl9LGluc3RlYWQ6ZnVuY3Rpb24oYSxiKXthLnBhcmVu",
				"dE5vZGUucmVwbGFjZUNoaWxkKGIsYSl9fSxiTj17VEJPRFk6WyI8VEFCTEU+IiwiPC9UQUJMRT4iLDJd",
				"LFRSOlsiPFRBQkxFPjxUQk9EWT4iLCI8L1RCT0RZPjwvVEFCTEU+IiwzXSxURDpbIjxUQUJMRT48VEJP",
				"RFk+PFRSPiIsIjwvVFI+PC9UQk9EWT48L1RBQkxFPiIsNF0sQ09MOlsiPFRBQkxFPjxDT0xHUk9VUD4i",
				"LCI8L0NPTEdST1VQPjxUQk9EWT48L1RCT0RZPjwvVEFCTEU+IiwyXSxMRUdFTkQ6WyI8RklFTERTRVQ+",
				"IiwiPC9GSUVMRFNFVD4iLDJdLEFSRUE6WyI8bWFwPiIsIjwvbWFwPiIsMl0sT1BUSU9OOlsiPFNFTEVD",
				"VD4iLCI8L1NFTEVDVD4iLDJdfTt2KGJOLHtPUFRHUk9VUDoiT1BUSU9OIixUSEVBRDoiVEJPRFkiLFRG",
				"T09UOiJUQk9EWSIsVEg6IlREIn0pO3ZhciBiTz1iLmNyZWF0ZURvY3VtZW50RnJhZ21lbnQoKSxiUD1i",
				"LmNyZWF0ZUVsZW1lbnQoIkRJViIpO2JHLmluY2x1ZGUoe3NldFN0eWxlOmZ1bmN0aW9uKGEsYil7dmFy",
				"IGMsZCxlPXt9LGY9dGhpcy5fLnN0eWxlO2IhPT1pPyhlW2FdPWIsYT1lKTp5KGEpJiYoYS5zcGxpdCgi",
				"OyIpLmVhY2goZnVuY3Rpb24oYSl7dmFyIGI9YS5zcGxpdCgiOiIpLm1hcCgidHJpbSIpO2JbMF0mJmJb",
				"MV0mJihlW2JbMF1dPWJbMV0pfSksYT1lKTtmb3IoYyBpbiBhKWQ9Yy5pbmRleE9mKCItIik8MD9jOmMu",
				"Y2FtZWxpemUoKSxidSYmYz09PSJvcGFjaXR5Ij9mLmZpbHRlcj0iYWxwaGEob3BhY2l0eT0iK2FbY10q",
				"MTAwKyIpIjpjPT09ImZsb2F0IiYmKGQ9YnI/InN0eWxlRmxvYXQiOiJjc3NGbG9hdCIpLGZbZF09YVtj",
				"XTtyZXR1cm4gdGhpc30sZ2V0U3R5bGU6ZnVuY3Rpb24oYSl7cmV0dXJuIGJSKHRoaXMuXy5zdHlsZSxh",
				"KXx8YlIodGhpcy5jb21wdXRlZFN0eWxlcygpLGEpfSxjb21wdXRlZFN0eWxlczpvLmN1cnJlbnRTdHls",
				"ZT9mdW5jdGlvbigpe3JldHVybiB0aGlzLl8uY3VycmVudFN0eWxlfHx7fX06by5ydW50aW1lU3R5bGU/",
				"ZnVuY3Rpb24oKXtyZXR1cm4gdGhpcy5fLnJ1bnRpbWVTdHlsZXx8e319OmZ1bmN0aW9uKCl7cmV0dXJu",
				"IHRoaXMuXy5vd25lckRvY3VtZW50LmRlZmF1bHRWaWV3LmdldENvbXB1dGVkU3R5bGUodGhpcy5fLG51",
				"bGwpfSxoYXNDbGFzczpmdW5jdGlvbihhKXtyZXR1cm4oIiAiK3RoaXMuXy5jbGFzc05hbWUrIiAiKS5p",
				"bmRleE9mKCIgIithKyIgIikhPS0xfSxzZXRDbGFzczpmdW5jdGlvbihhKXt0aGlzLl8uY2xhc3NOYW1l",
				"PWE7cmV0dXJuIHRoaXN9LGdldENsYXNzOmZ1bmN0aW9uKCl7cmV0dXJuIHRoaXMuXy5jbGFzc05hbWV9",
				"LGFkZENsYXNzOmZ1bmN0aW9uKGEpe3ZhciBiPSIgIit0aGlzLl8uY2xhc3NOYW1lKyIgIjtiLmluZGV4",
				"T2YoIiAiK2ErIiAiKT09LTEmJih0aGlzLl8uY2xhc3NOYW1lKz0oYj09PSIgICI/IiI6IiAiKSthKTty",
				"ZXR1cm4gdGhpc30scmVtb3ZlQ2xhc3M6ZnVuY3Rpb24oYSl7dGhpcy5fLmNsYXNzTmFtZT0oIiAiK3Ro",
				"aXMuXy5jbGFzc05hbWUrIiAiKS5yZXBsYWNlKCIgIithKyIgIiwiICIpLnRyaW0oKTtyZXR1cm4gdGhp",
				"c30sdG9nZ2xlQ2xhc3M6ZnVuY3Rpb24oYSl7cmV0dXJuIHRoaXNbdGhpcy5oYXNDbGFzcyhhKT8icmVt",
				"b3ZlQ2xhc3MiOiJhZGRDbGFzcyJdKGEpfSxyYWRpb0NsYXNzOmZ1bmN0aW9uKGEpe3RoaXMuc2libGlu",
				"Z3MoKS5lYWNoKCJyZW1vdmVDbGFzcyIsYSk7cmV0dXJuIHRoaXMuYWRkQ2xhc3MoYSl9fSksYkcuaW5j",
				"bHVkZSh7c2V0OmZ1bmN0aW9uKGEsYil7aWYodHlwZW9mIGE9PT0ic3RyaW5nIil7dmFyIGM9e307Y1th",
				"XT1iLGE9Y312YXIgZCxlPXRoaXMuXztmb3IoZCBpbiBhKWQ9PT0ic3R5bGUiP3RoaXMuc2V0U3R5bGUo",
				"YVtkXSk6KGQgaW4gZXx8ZS5zZXRBdHRyaWJ1dGUoZCwiIithW2RdKSxkLnN1YnN0cigwLDUpIT09ImRh",
				"dGEtIiYmKGVbZF09YVtkXSkpO3JldHVybiB0aGlzfSxnZXQ6ZnVuY3Rpb24oYSl7dmFyIGI9dGhpcy5f",
				"LGM9YlthXXx8Yi5nZXRBdHRyaWJ1dGUoYSk7cmV0dXJuIGM9PT0iIj9udWxsOmN9LGhhczpmdW5jdGlv",
				"bihhKXtyZXR1cm4gdGhpcy5nZXQoYSkhPT1udWxsfSxlcmFzZTpmdW5jdGlvbihhKXt0aGlzLl8ucmVt",
				"b3ZlQXR0cmlidXRlKGEpO3JldHVybiB0aGlzfSxoaWRkZW46ZnVuY3Rpb24oKXtyZXR1cm4gdGhpcy5n",
				"ZXRTdHlsZSgiZGlzcGxheSIpPT09Im5vbmUifSx2aXNpYmxlOmZ1bmN0aW9uKCl7cmV0dXJuIXRoaXMu",
				"aGlkZGVuKCl9LGhpZGU6ZnVuY3Rpb24oYSxiKXt0aGlzLnZpc2libGUoKSYmKHRoaXMuX2Q9dGhpcy5n",
				"ZXRTdHlsZSgiZGlzcGxheSIpLHRoaXMuXy5zdHlsZS5kaXNwbGF5PSJub25lIik7cmV0dXJuIHRoaXN9",
				"LHNob3c6ZnVuY3Rpb24oKXtpZih0aGlzLmhpZGRlbigpKXt2YXIgYT10aGlzLl8sYj10aGlzLl9kLGM7",
				"aWYoIWJ8fGI9PT0ibm9uZSIpYz1HKGEudGFnTmFtZSkuaW5zZXJ0VG8obyksYj1jLmdldFN0eWxlKCJk",
				"aXNwbGF5IiksYy5yZW1vdmUoKTtiPT09Im5vbmUiJiYoYj0iYmxvY2siKSxhLnN0eWxlLmRpc3BsYXk9",
				"Yn1yZXR1cm4gdGhpc30sdG9nZ2xlOmZ1bmN0aW9uKCl7cmV0dXJuIHRoaXNbdGhpcy52aXNpYmxlKCk/",
				"ImhpZGUiOiJzaG93Il0oKX0scmFkaW86ZnVuY3Rpb24oYSxiKXt0aGlzLnNpYmxpbmdzKCkuZWFjaCgi",
				"aGlkZSIsYSxiKTtyZXR1cm4gdGhpcy5zaG93KCl9LGRhdGE6ZnVuY3Rpb24oYSxiKXt2YXIgYyxkLGUs",
				"ZixnLGg7aWYoQShhKSlmb3IoYyBpbiBhKWI9dGhpcy5kYXRhKGMsYVtjXSk7ZWxzZSBpZihiPT09aSl7",
				"YT0iZGF0YS0iKygiIithKS5kYXNoZXJpemUoKTtmb3IoZD17fSxlPSExLGY9dGhpcy5fLmF0dHJpYnV0",
				"ZXMsaD0wO2g8Zi5sZW5ndGg7aCsrKXtiPWZbaF0udmFsdWU7dHJ5e2I9SlNPTi5wYXJzZShiKX1jYXRj",
				"aChqKXt9aWYoZltoXS5uYW1lPT09YSl7ZD1iLGU9ITA7YnJlYWt9ZltoXS5uYW1lLmluZGV4T2YoYSk9",
				"PT0wJiYoZFtmW2hdLm5hbWUuc3Vic3RyaW5nKGEubGVuZ3RoKzEpLmNhbWVsaXplKCldPWIsZT0hMCl9",
				"Yj1lP2Q6bnVsbH1lbHNle2E9ImRhdGEtIisoIiIrYSkuZGFzaGVyaXplKCksQShiKXx8KGI9eyIiOmJ9",
				"KTtmb3IoYyBpbiBiKWc9Yy5ibGFuaygpP2E6YSsiLSIrYy5kYXNoZXJpemUoKSxiW2NdPT09bnVsbD90",
				"aGlzLl8ucmVtb3ZlQXR0cmlidXRlKGcpOnRoaXMuXy5zZXRBdHRyaWJ1dGUoZyx5KGJbY10pP2JbY106",
				"SlNPTi5zdHJpbmdpZnkoYltjXSkpO2I9dGhpc31yZXR1cm4gYn19KSxiRy5pbmNsdWRlKHtkb2M6ZnVu",
				"Y3Rpb24oKXtyZXR1cm4gYkEodGhpcy5fLm93bmVyRG9jdW1lbnQpfSx3aW46ZnVuY3Rpb24oKXtyZXR1",
				"cm4gdGhpcy5kb2MoKS53aW4oKX0sc2l6ZTpmdW5jdGlvbigpe3JldHVybnt4OnRoaXMuXy5vZmZzZXRX",
				"aWR0aCx5OnRoaXMuXy5vZmZzZXRIZWlnaHR9fSxwb3NpdGlvbjpmdW5jdGlvbigpe3ZhciBhPXRoaXMu",
				"Xy5nZXRCb3VuZGluZ0NsaWVudFJlY3QoKSxiPXRoaXMuZG9jKCkuXy5kb2N1bWVudEVsZW1lbnQsYz10",
				"aGlzLndpbigpLnNjcm9sbHMoKTtyZXR1cm57eDphLmxlZnQrYy54LWIuY2xpZW50TGVmdCx5OmEudG9w",
				"K2MueS1iLmNsaWVudFRvcH19LHNjcm9sbHM6ZnVuY3Rpb24oKXtyZXR1cm57eDp0aGlzLl8uc2Nyb2xs",
				"TGVmdCx5OnRoaXMuXy5zY3JvbGxUb3B9fSxkaW1lbnNpb25zOmZ1bmN0aW9uKCl7dmFyIGE9dGhpcy5z",
				"aXplKCksYj10aGlzLnNjcm9sbHMoKSxjPXRoaXMucG9zaXRpb24oKTtyZXR1cm57dG9wOmMueSxsZWZ0",
				"OmMueCx3aWR0aDphLngsaGVpZ2h0OmEueSxzY3JvbGxMZWZ0OmIueCxzY3JvbGxUb3A6Yi55fX0sb3Zl",
				"cmxhcHM6ZnVuY3Rpb24oYSl7dmFyIGI9dGhpcy5wb3NpdGlvbigpLGM9dGhpcy5zaXplKCk7cmV0dXJu",
				"IGEueD5iLngmJmEueDxiLngrYy54JiZhLnk+Yi55JiZhLnk8Yi55K2MueX0sc2V0V2lkdGg6ZnVuY3Rp",
				"b24oYSl7dmFyIGI9dGhpcy5fLnN0eWxlO2Iud2lkdGg9YSsicHgiLGIud2lkdGg9MiphLXRoaXMuXy5v",
				"ZmZzZXRXaWR0aCsicHgiO3JldHVybiB0aGlzfSxzZXRIZWlnaHQ6ZnVuY3Rpb24oYSl7dmFyIGI9dGhp",
				"cy5fLnN0eWxlO2IuaGVpZ2h0PWErInB4IixiLmhlaWdodD0yKmEtdGhpcy5fLm9mZnNldEhlaWdodCsi",
				"cHgiO3JldHVybiB0aGlzfSxyZXNpemU6ZnVuY3Rpb24oYSxiKXtBKGEpJiYoYj1hLnksYT1hLngpO3Jl",
				"dHVybiB0aGlzLnNldFdpZHRoKGEpLnNldEhlaWdodChiKX0sbW92ZVRvOmZ1bmN0aW9uKGEsYil7QShh",
				"KSYmKGI9YS55LGE9YS54KTtyZXR1cm4gdGhpcy5zZXRTdHlsZSh7bGVmdDphKyJweCIsdG9wOmIrInB4",
				"In0pfSxzY3JvbGxUbzpmdW5jdGlvbihhLGIpe0EoYSkmJihiPWEueSxhPWEueCksdGhpcy5fLnNjcm9s",
				"bExlZnQ9YSx0aGlzLl8uc2Nyb2xsVG9wPWI7cmV0dXJuIHRoaXN9LHNjcm9sbFRoZXJlOmZ1bmN0aW9u",
				"KGEpe3RoaXMud2luKCkuc2Nyb2xsVG8odGhpcyxhKTtyZXR1cm4gdGhpc319KSxbYkcsYkIsYkRdLmVh",
				"Y2goImluY2x1ZGUiLHMoYmwoe30pLHtvbjpmdW5jdGlvbigpe2JuKHRoaXMsYXJndW1lbnRzLGZ1bmN0",
				"aW9uKGEpe2EuZT09PSJtb3VzZWVudGVyInx8YS5lPT09Im1vdXNlbGVhdmUiPyhiXygpLGEubj1hLmUs",
				"YS53PWZ1bmN0aW9uKCl7fSk6KGEuZT09PSJjb250ZXh0bWVudSImJmJzLktvbnF1ZXJvcj9hLm49InJp",
				"Z2h0Y2xpY2siOmEuZT09PSJtb3VzZXdoZWVsIiYmYnMuR2Vja28/YS5uPSJET01Nb3VzZVNjcm9sbCI6",
				"YS5uPWEuZSxhLnc9ZnVuY3Rpb24oYil7Yj1uZXcgYkUoYixhLnQpLGEuZi5hcHBseShhLnQsKGEucj9b",
				"XTpbYl0pLmNvbmNhdChhLmEpKT09PSExJiZiLnN0b3AoKX0sYnQ/YS50Ll8uYXR0YWNoRXZlbnQoIm9u",
				"IithLm4sYS53KTphLnQuXy5hZGRFdmVudExpc3RlbmVyKGEubixhLncsITEpKTtyZXR1cm4gYX0pO3Jl",
				"dHVybiB0aGlzfSxzdG9wT2JzZXJ2aW5nOmZ1bmN0aW9uKGEsYil7Ym8odGhpcyxhLGIsZnVuY3Rpb24o",
				"YSl7YnQ/YS50Ll8uZGV0YWNoRXZlbnQoIm9uIithLm4sYS53KTphLnQuXy5yZW1vdmVFdmVudExpc3Rl",
				"bmVyKGEubixhLncsITEpfSk7cmV0dXJuIHRoaXN9LGZpcmU6ZnVuY3Rpb24oYSxiKXt2YXIgYz10aGlz",
				"LnBhcmVudCYmdGhpcy5wYXJlbnQoKTthIGluc3RhbmNlb2YgYkV8fChhPW5ldyBiRShhLHMoe3Rhcmdl",
				"dDp0aGlzLl99LGIpKSksYS5jdXJyZW50VGFyZ2V0PXRoaXMsKHRoaXMuJGxpc3RlbmVyc3x8W10pLmVh",
				"Y2goZnVuY3Rpb24oYil7Yi5lPT09YS50eXBlJiZiLmYuYXBwbHkodGhpcywoYi5yP1tdOlthXSkuY29u",
				"Y2F0KGIuYSkpPT09ITEmJmEuc3RvcCgpfSx0aGlzKSxjJiZjLmZpcmUmJiFhLnN0b3BwZWQmJmMuZmly",
				"ZShhKTtyZXR1cm4gdGhpc30sc3RvcEV2ZW50OmZ1bmN0aW9uKCl7cmV0dXJuITF9fSkpLGJtKGJELnBy",
				"b3RvdHlwZSxIKCJibHVyIGZvY3VzIHNjcm9sbCByZXNpemUgbG9hZCIpKSxiUygiY2xpY2sgcmlnaHRj",
				"bGljayBjb250ZXh0bWVudSBtb3VzZWRvd24gbW91c2V1cCBtb3VzZW92ZXIgbW91c2VvdXQgbW91c2Vt",
				"b3ZlIGtleXByZXNzIGtleWRvd24ga2V5dXAiKSxbYkcsYkJdLmVhY2goImluY2x1ZGUiLHtmaXJzdDpm",
				"dW5jdGlvbihhKXtyZXR1cm4gYkEoYT09PWkmJnRoaXMuXy5maXJzdEVsZW1lbnRDaGlsZCE9PWk/dGhp",
				"cy5fLmZpcnN0RWxlbWVudENoaWxkOnRoaXMuXy5xdWVyeVNlbGVjdG9yKGF8fCIqIikpfSxmaW5kOmZ1",
				"bmN0aW9uKGEsYil7dmFyIGM9dGhpcy5fLnF1ZXJ5U2VsZWN0b3JBbGwoYXx8IioiKSxkLGU9MCxmPWMu",
				"bGVuZ3RoO2lmKGI9PT0hMClkPUooYyk7ZWxzZSBmb3IoZD1bXTtlPGY7ZSsrKWRbZV09YkEoY1tlXSk7",
				"cmV0dXJuIGR9LG1hdGNoOmZ1bmN0aW9uKGEpe3ZhciBjPXRoaXMuXyxkPWMsZSxmPSExO3doaWxlKGQu",
				"cGFyZW50Tm9kZSE9PW51bGwmJmQucGFyZW50Tm9kZS5ub2RlVHlwZSE9PTExKWQ9ZC5wYXJlbnROb2Rl",
				"O2M9PT1kJiYoZD1iLmNyZWF0ZUVsZW1lbnQoImRpdiIpLGQuYXBwZW5kQ2hpbGQoYyksZj0hMCksZT1i",
				"QShkKS5maW5kKGEsITApLmluZGV4T2YoYykhPT0tMSxmJiZkLnJlbW92ZUNoaWxkKGMpO3JldHVybiBl",
				"fX0pLGJCLmluY2x1ZGUoe29uOmZ1bmN0aW9uKGEpe2lmKGE9PT0icmVhZHkiJiYhdGhpcy5faVIpe3Zh",
				"ciBiPXRoaXMuXyxjPXRoaXMuZmlyZS5iaW5kKHRoaXMsInJlYWR5Iik7InJlYWR5U3RhdGUiaW4gYj9m",
				"dW5jdGlvbigpe1sibG9hZGVkIiwiY29tcGxldGUiXS5pbmNsdWRlKGIucmVhZHlTdGF0ZSk/YygpOmFy",
				"Z3VtZW50cy5jYWxsZWUuZGVsYXkoNTApfSgpOmIuYWRkRXZlbnRMaXN0ZW5lcigiRE9NQ29udGVudExv",
				"YWRlZCIsYywhMSksdGhpcy5faVI9ITB9cmV0dXJuIHRoaXMuJHN1cGVyLmFwcGx5KHRoaXMsYXJndW1l",
				"bnRzKX19KSxibShiQi5wcm90b3R5cGUsWyJyZWFkeSJdKTt2YXIgYlQ9ai5Gb3JtPWJILkZPUk09bmV3",
				"IGJjKGJHLHtpbml0aWFsaXplOmZ1bmN0aW9uKGEpe3ZhciBiPWF8fHt9LGQ9InJlbW90ZSJpbiBiLGU9",
				"YjtBKGIpJiYhQyhiKSYmKGU9ImZvcm0iLGI9Yy53aXRob3V0KGIsInJlbW90ZSIpKSx0aGlzLiRzdXBl",
				"cihlLGIpLGQmJnRoaXMucmVtb3RpemUoKX0sZWxlbWVudHM6ZnVuY3Rpb24oKXtyZXR1cm4gdGhpcy5m",
				"aW5kKCJpbnB1dCxidXR0b24sc2VsZWN0LHRleHRhcmVhIil9LGlucHV0czpmdW5jdGlvbigpe3JldHVy",
				"biB0aGlzLmVsZW1lbnRzKCkuZmlsdGVyKGZ1bmN0aW9uKGEpe3JldHVybiFbInN1Ym1pdCIsImJ1dHRv",
				"biIsInJlc2V0IiwiaW1hZ2UiLG51bGxdLmluY2x1ZGUoYS5fLnR5cGUpfSl9LGlucHV0OmZ1bmN0aW9u",
				"KGEpe3ZhciBiPXRoaXMuX1thXTsidGFnTmFtZSJpbiBiP2I9YkEoYik6Yj1KKGIpLm1hcChiQSk7cmV0",
				"dXJuIGJ9LGZvY3VzOmZ1bmN0aW9uKCl7dmFyIGE9dGhpcy5pbnB1dHMoKS5maXJzdChmdW5jdGlvbihh",
				"KXtyZXR1cm4gYS5fLnR5cGUhPT0iaGlkZGVuIn0pO2EmJmEuZm9jdXMoKTtyZXR1cm4gdGhpc30sYmx1",
				"cjpmdW5jdGlvbigpe3RoaXMuZWxlbWVudHMoKS5lYWNoKCJibHVyIik7cmV0dXJuIHRoaXN9LGRpc2Fi",
				"bGU6ZnVuY3Rpb24oKXt0aGlzLmVsZW1lbnRzKCkuZWFjaCgiZGlzYWJsZSIpO3JldHVybiB0aGlzfSxl",
				"bmFibGU6ZnVuY3Rpb24oKXt0aGlzLmVsZW1lbnRzKCkuZWFjaCgiZW5hYmxlIik7cmV0dXJuIHRoaXN9",
				"LHZhbHVlczpmdW5jdGlvbigpe3ZhciBhPXt9O3RoaXMuaW5wdXRzKCkuZWFjaChmdW5jdGlvbihiKXt2",
				"YXIgYz1iLl8sZD1hLGUsZj1jLm5hbWUubWF0Y2goL1teXFtdKy9nKTtpZighYy5kaXNhYmxlZCYmYy5u",
				"YW1lJiYoYy50eXBlIT09ImNoZWNrYm94IiYmYy50eXBlIT09InJhZGlvInx8Yy5jaGVja2VkKSl7d2hp",
				"bGUoZi5sZW5ndGg+MSllPWYuc2hpZnQoKSxlLmVuZHNXaXRoKCJdIikmJihlPWUuc3Vic3RyKDAsZS5s",
				"ZW5ndGgtMSkpLGRbZV18fChkW2VdPWZbMF09PT0iXSI/W106e30pLGQ9ZFtlXTtlPWYuc2hpZnQoKSxl",
				"LmVuZHNXaXRoKCJdIikmJihlPWUuc3Vic3RyKDAsZS5sZW5ndGgtMSkpLGU9PT0iIj9kLnB1c2goYi52",
				"YWx1ZSgpKTpkW2VdPWIudmFsdWUoKX19KTtyZXR1cm4gYX0sc2VyaWFsaXplOmZ1bmN0aW9uKCl7cmV0",
				"dXJuIGMudG9RdWVyeVN0cmluZyh0aGlzLnZhbHVlcygpKX0sc3VibWl0OmZ1bmN0aW9uKCl7dGhpcy5f",
				"LnN1Ym1pdCgpO3JldHVybiB0aGlzfSxyZXNldDpmdW5jdGlvbigpe3RoaXMuXy5yZXNldCgpO3JldHVy",
				"biB0aGlzfX0pO2JTKCJzdWJtaXQgcmVzZXQgZm9jdXMgYmx1ciBkaXNhYmxlIGVuYWJsZSBjaGFuZ2Ui",
				"KTt2YXIgYlU9ai5JbnB1dD1iSC5JTlBVVD1iSC5CVVRUT049YkguU0VMRUNUPWJILlRFWFRBUkVBPWJI",
				"Lk9QVEdST1VQPW5ldyBiYyhiRyx7aW5pdGlhbGl6ZTpmdW5jdGlvbihhLGIpe2lmKCFhfHxBKGEpJiYh",
				"QyhhKSliPWF8fHt9LC90ZXh0YXJlYXxzZWxlY3QvLnRlc3QoYi50eXBlfHwiIik/KGE9Yi50eXBlLGRl",
				"bGV0ZSBiLnR5cGUpOmE9ImlucHV0Ijt0aGlzLiRzdXBlcihhLGIpfSxmb3JtOmZ1bmN0aW9uKCl7cmV0",
				"dXJuIGJBKHRoaXMuXy5mb3JtKX0saW5zZXJ0OmZ1bmN0aW9uKGEsYil7dGhpcy4kc3VwZXIoYSxiKSx0",
				"aGlzLmZpbmQoIm9wdGlvbiIpLmVhY2goZnVuY3Rpb24oYSl7YS5fLnNlbGVjdGVkPSEhYS5nZXQoInNl",
				"bGVjdGVkIil9KTtyZXR1cm4gdGhpc30sdXBkYXRlOmZ1bmN0aW9uKGEpe3JldHVybiB0aGlzLmNsZWFu",
				"KCkuaW5zZXJ0KGEpfSxnZXRWYWx1ZTpmdW5jdGlvbigpe3JldHVybiB0aGlzLl8udHlwZT09InNlbGVj",
				"dC1tdWx0aXBsZSI/dGhpcy5maW5kKCJvcHRpb24iKS5tYXAoZnVuY3Rpb24oYSl7cmV0dXJuIGEuXy5z",
				"ZWxlY3RlZD9hLl8udmFsdWU6bnVsbH0pLmNvbXBhY3QoKTp0aGlzLl8udmFsdWV9LHNldFZhbHVlOmZ1",
				"bmN0aW9uKGEpe3RoaXMuXy50eXBlPT0ic2VsZWN0LW11bHRpcGxlIj8oYT1OKGEpLm1hcChlKSx0aGlz",
				"LmZpbmQoIm9wdGlvbiIpLmVhY2goZnVuY3Rpb24oYil7Yi5fLnNlbGVjdGVkPWEuaW5jbHVkZShiLl8u",
				"dmFsdWUpfSkpOnRoaXMuXy52YWx1ZT1hO3JldHVybiB0aGlzfSx2YWx1ZTpmdW5jdGlvbihhKXtyZXR1",
				"cm4gdGhpc1thPT09aT8iZ2V0VmFsdWUiOiJzZXRWYWx1ZSJdKGEpfSxmb2N1czpmdW5jdGlvbigpe3Ro",
				"aXMuXy5mb2N1cygpLHRoaXMuZm9jdXNlZD0hMCxiciYmdGhpcy5maXJlKCJmb2N1cyIse2J1YmJsZXM6",
				"ITF9KTtyZXR1cm4gdGhpc30sYmx1cjpmdW5jdGlvbigpe3RoaXMuXy5ibHVyKCksdGhpcy5mb2N1c2Vk",
				"PSExLGJyJiZ0aGlzLmZpcmUoImJsdXIiLHtidWJibGVzOiExfSk7cmV0dXJuIHRoaXN9LHNlbGVjdDpm",
				"dW5jdGlvbigpe3RoaXMuXy5zZWxlY3QoKTtyZXR1cm4gdGhpcy5mb2N1cygpfSxkaXNhYmxlOmZ1bmN0",
				"aW9uKCl7dGhpcy5fLmRpc2FibGVkPSEwO3JldHVybiB0aGlzLmZpcmUoImRpc2FibGUiKX0sZW5hYmxl",
				"OmZ1bmN0aW9uKCl7dGhpcy5fLmRpc2FibGVkPSExO3JldHVybiB0aGlzLmZpcmUoImVuYWJsZSIpfSxk",
				"aXNhYmxlZDpmdW5jdGlvbihhKXtyZXR1cm4gYT09PWk/dGhpcy5fLmRpc2FibGVkOnRoaXNbYT8iZGlz",
				"YWJsZSI6ImVuYWJsZSJdKCl9LGNoZWNrZWQ6ZnVuY3Rpb24oYSl7YT09PWk/YT10aGlzLl8uY2hlY2tl",
				"ZDoodGhpcy5fLmNoZWNrZWQ9YSxhPXRoaXMpO3JldHVybiBhfX0pO2J0PyhiLmF0dGFjaEV2ZW50KCJv",
				"bmZvY3VzaW4iLGJWKSxiLmF0dGFjaEV2ZW50KCJvbmZvY3Vzb3V0IixiVikpOihiLmFkZEV2ZW50TGlz",
				"dGVuZXIoImZvY3VzIixiViwhMCksYi5hZGRFdmVudExpc3RlbmVyKCJibHVyIixiViwhMCkpO3ZhciBi",
				"Vz1bXSxiWD0hMDtiUygibW91c2VlbnRlciBtb3VzZWxlYXZlIiksW2JHLGJCXS5lYWNoKCJpbmNsdWRl",
				"Iix7ZGVsZWdhdGU6ZnVuY3Rpb24oYSl7dmFyIGI9Y2IoYXJndW1lbnRzKSxjLGQsZSxmO2ZvcihjIGlu",
				"IGIpZm9yKGQ9MCxmPWJbY107ZDxmLmxlbmd0aDtkKyspdGhpcy5vbihhLGNhKGMsZltkXSx0aGlzKSks",
				"cyh0aGlzLiRsaXN0ZW5lcnMubGFzdCgpLHtkcjpjLGRjOmZbZF1bMF19KTtyZXR1cm4gdGhpc30sdW5k",
				"ZWxlZ2F0ZTpmdW5jdGlvbihhKXtjYyhhcmd1bWVudHMsdGhpcykuZWFjaChmdW5jdGlvbihhKXt0aGlz",
				"LnN0b3BPYnNlcnZpbmcoYS5uLGEuZil9LHRoaXMpO3JldHVybiB0aGlzfSxkZWxlZ2F0ZXM6ZnVuY3Rp",
				"b24oKXtyZXR1cm4hIWNjKGFyZ3VtZW50cyx0aGlzKS5sZW5ndGh9fSksYy5lYWNoKHtvbjoiZGVsZWdh",
				"dGUiLHN0b3BPYnNlcnZpbmc6InVuZGVsZWdhdGUiLG9ic2VydmVzOiJkZWxlZ2F0ZXMifSxmdW5jdGlv",
				"bihhLGIpe2UucHJvdG90eXBlW2FdPWZ1bmN0aW9uKCl7dmFyIGE9Sihhcmd1bWVudHMpLGM7YS5zcGxp",
				"Y2UoMSwwLCIiK3RoaXMpLGM9YkNbYl0uYXBwbHkoYkMsYSk7cmV0dXJuIGM9PT1iQz90aGlzOmN9fSk7",
				"dmFyIGNkPWUucHJvdG90eXBlLm9uO2UucHJvdG90eXBlLm9uPWZ1bmN0aW9uKGEpe2lmKEEoYSkpZm9y",
				"KHZhciBiIGluIGEpY2QuYXBwbHkodGhpcyxbYl0uY29uY2F0KFthW2JdXSkpO2Vsc2UgY2QuYXBwbHko",
				"dGhpcyxhcmd1bWVudHMpO3JldHVybiB0aGlzfSxiRi5lYWNoKGZ1bmN0aW9uKGEpe2UucHJvdG90eXBl",
				"WyJvbiIrYS5jYXBpdGFsaXplKCldPWZ1bmN0aW9uKCl7cmV0dXJuIHRoaXMub24uYXBwbHkodGhpcyxb",
				"YV0uY29uY2F0KEooYXJndW1lbnRzKSkpfX0pLEgoIkVsZW1lbnQgSW5wdXQgRm9ybSIpLmVhY2goZnVu",
				"Y3Rpb24oYSl7Yy5lYWNoKGEgaW4gaj9qW2FdLnByb3RvdHlwZTp7fSxmdW5jdGlvbihhLGIpe3goYikm",
				"JiEoYSBpbiBlLnByb3RvdHlwZSkmJihlLnByb3RvdHlwZVthXT1mdW5jdGlvbigpe3ZhciBiPUYodGhp",
				"cywhMCksYz0wLGQ9Yi5sZW5ndGgsZT0hMCxmLGc7Zm9yKDtjPGQ7YysrKXtmPWJBKGJbY10pLGc9Zlth",
				"XS5hcHBseShmLGFyZ3VtZW50cyk7aWYoZSl7aWYoZyE9PWYpcmV0dXJuIGc7ZT0hMX19cmV0dXJuIG51",
				"bGx9KX0pfSk7dmFyIGNlPWouWGhyPW5ldyBiYyhiayx7ZXh0ZW5kOntFVkVOVFM6SCgic3VjY2VzcyBm",
				"YWlsdXJlIGNvbXBsZXRlIHJlcXVlc3QgY2FuY2VsIGNyZWF0ZSIpLE9wdGlvbnM6e2hlYWRlcnM6eyJY",
				"LVJlcXVlc3RlZC1XaXRoIjoiWE1MSHR0cFJlcXVlc3QiLEFjY2VwdDoidGV4dC9qYXZhc2NyaXB0LHRl",
				"eHQvaHRtbCxhcHBsaWNhdGlvbi94bWwsdGV4dC94bWwsKi8qIn0sbWV0aG9kOiJwb3N0IixlbmNvZGlu",
				"ZzoidXRmLTgiLGFzeW5jOiEwLGV2YWxTY3JpcHRzOiExLGV2YWxSZXNwb25zZTohMSxldmFsSlM6ITAs",
				"ZXZhbEpTT046ITAsc2VjdXJlSlNPTjohMCx1cmxFbmNvZGVkOiEwLHNwaW5uZXI6bnVsbCxzcGlubmVy",
				"Rng6ImZhZGUiLHBhcmFtczpudWxsLGlmcmFtZWQ6ITEsanNvbnA6ITF9LGxvYWQ6ZnVuY3Rpb24oYSxi",
				"KXtyZXR1cm4obmV3IHRoaXMoYSxzKHttZXRob2Q6ImdldCJ9LGIpKSkuc2VuZCgpfX0saW5pdGlhbGl6",
				"ZTpmdW5jdGlvbihhLGIpe3RoaXMuaW5pdENhbGxiYWNrcygpLHRoaXMudXJsPWEscyh0aGlzLiRzdXBl",
				"cihiKSx0aGlzLm9wdGlvbnMpLHRoaXMucGFyYW1zIT1jZS5PcHRpb25zLnBhcmFtcyYmKHRoaXMucGFy",
				"YW1zPXRoaXMucHJlcGFyZURhdGEoY2UuT3B0aW9ucy5wYXJhbXMsdGhpcy5wYXJhbXMpKSxjZS5PcHRp",
				"b25zLnNwaW5uZXImJkUodGhpcy5zcGlubmVyKT09PUUoY2UuT3B0aW9ucy5zcGlubmVyKSYmKHRoaXMu",
				"c3Bpbm5lcj1udWxsKX0sc2V0SGVhZGVyOmZ1bmN0aW9uKGEsYil7dGhpcy5oZWFkZXJzW2FdPWI7cmV0",
				"dXJuIHRoaXN9LGdldEhlYWRlcjpmdW5jdGlvbihhKXt2YXIgYjt0cnl7Yj10aGlzLnhoci5nZXRSZXNw",
				"b25zZUhlYWRlcihhKX1jYXRjaChjKXt9cmV0dXJuIGJ9LHN1Y2Nlc3NmdWw6ZnVuY3Rpb24oKXtyZXR1",
				"cm4gdGhpcy5zdGF0dXM+PTIwMCYmdGhpcy5zdGF0dXM8MzAwfSxzZW5kOmZ1bmN0aW9uKGEpe3ZhciBi",
				"PXt9LGM9dGhpcy51cmwsZD10aGlzLm1ldGhvZC50b0xvd2VyQ2FzZSgpLGU9dGhpcy5oZWFkZXJzLGYs",
				"ZztpZihkPT0icHV0Inx8ZD09ImRlbGV0ZSIpYi5fbWV0aG9kPWQsZD0icG9zdCI7dmFyIGg9dGhpcy5w",
				"cmVwYXJlRGF0YSh0aGlzLnBhcmFtcyx0aGlzLnByZXBhcmVQYXJhbXMoYSksYik7dGhpcy51cmxFbmNv",
				"ZGVkJiZkPT0icG9zdCImJiFlWyJDb250ZW50LXR5cGUiXSYmdGhpcy5zZXRIZWFkZXIoIkNvbnRlbnQt",
				"dHlwZSIsImFwcGxpY2F0aW9uL3gtd3d3LWZvcm0tdXJsZW5jb2RlZDtjaGFyc2V0PSIrdGhpcy5lbmNv",
				"ZGluZyksZD09ImdldCImJihoJiYoYys9KGMuaW5jbHVkZSgiPyIpPyImIjoiPyIpK2gpLGg9bnVsbCks",
				"Zz10aGlzLnhocj10aGlzLmNyZWF0ZVhocigpLHRoaXMuZmlyZSgiY3JlYXRlIiksZy5vcGVuKGQsYyx0",
				"aGlzLmFzeW5jKSxnLm9ucmVhZHlzdGF0ZWNoYW5nZT10aGlzLnN0YXRlQ2hhbmdlZC5iaW5kKHRoaXMp",
				"O2ZvcihmIGluIGUpZy5zZXRSZXF1ZXN0SGVhZGVyKGYsZVtmXSk7Zy5zZW5kKGgpLHRoaXMuZmlyZSgi",
				"cmVxdWVzdCIpLHRoaXMuYXN5bmN8fHRoaXMuc3RhdGVDaGFuZ2VkKCk7cmV0dXJuIHRoaXN9LHVwZGF0",
				"ZTpmdW5jdGlvbihhLGIpe3JldHVybiB0aGlzLm9uU3VjY2VzcyhmdW5jdGlvbihiKXthLnVwZGF0ZShi",
				"LnRleHQpfSkuc2VuZChiKX0sY2FuY2VsOmZ1bmN0aW9uKCl7dmFyIGE9dGhpcy54aHI7aWYoIWF8fGEu",
				"Y2FuY2VsZWQpcmV0dXJuIHRoaXM7YS5hYm9ydCgpLGEub25yZWFkeXN0YXRlY2hhbmdlPWZ1bmN0aW9u",
				"KCl7fSxhLmNhbmNlbGVkPSEwO3JldHVybiB0aGlzLmZpcmUoImNhbmNlbCIpfSxmaXJlOmZ1bmN0aW9u",
				"KGEpe3JldHVybiB0aGlzLiRzdXBlcihhLHRoaXMsdGhpcy54aHIpfSxjcmVhdGVYaHI6ZnVuY3Rpb24o",
				"KXtyZXR1cm4gdGhpcy5qc29ucD9uZXcgY2UuSlNPTlAodGhpcyk6dGhpcy5mb3JtJiZ0aGlzLmZvcm0u",
				"Zmlyc3QoImlucHV0W3R5cGU9ZmlsZV0iKT9uZXcgY2UuSUZyYW1lZCh0aGlzLmZvcm0pOiJBY3RpdmVY",
				"T2JqZWN0ImluIGE/bmV3IEFjdGl2ZVhPYmplY3QoIk1TWE1MMi5YTUxIVFRQIik6bmV3IFhNTEh0dHBS",
				"ZXF1ZXN0fSxwcmVwYXJlUGFyYW1zOmZ1bmN0aW9uKGEpe3JldHVybiBhfSxwcmVwYXJlRGF0YTpmdW5j",
				"dGlvbigpe3JldHVybiBKKGFyZ3VtZW50cykubWFwKGZ1bmN0aW9uKGEpe3koYSl8fChhPWMudG9RdWVy",
				"eVN0cmluZyhhKSk7cmV0dXJuIGEuYmxhbmsoKT9udWxsOmF9KS5jb21wYWN0KCkuam9pbigiJiIpfSxz",
				"dGF0ZUNoYW5nZWQ6ZnVuY3Rpb24oKXt2YXIgYT10aGlzLnhocjtpZihhLnJlYWR5U3RhdGU9PTQmJiFh",
				"LmNhbmNlbGVkKXt0cnl7dGhpcy5zdGF0dXM9YS5zdGF0dXN9Y2F0Y2goYil7dGhpcy5zdGF0dXM9MH10",
				"aGlzLnRleHQ9dGhpcy5yZXNwb25zZVRleHQ9YS5yZXNwb25zZVRleHQsdGhpcy54bWw9dGhpcy5yZXNw",
				"b25zZVhNTD1hLnJlc3BvbnNlWE1MLHRoaXMuZmlyZSgiY29tcGxldGUiKS5maXJlKHRoaXMuc3VjY2Vz",
				"c2Z1bCgpPyJzdWNjZXNzIjoiZmFpbHVyZSIpfX0sdHJ5U2NyaXB0czpmdW5jdGlvbihhKXt2YXIgYj10",
				"aGlzLmdldEhlYWRlcigiQ29udGVudC10eXBlIiksYz10aGlzLmdldEhlYWRlcigiWC1KU09OIik7YyYm",
				"KHRoaXMuanNvbj10aGlzLnJlc3BvbnNlSlNPTj10aGlzLmhlYWRlckpTT049SlNPTi5wYXJzZShjKSks",
				"dGhpcy5ldmFsUmVzcG9uc2V8fHRoaXMuZXZhbEpTJiYvKGVjbWF8amF2YSlzY3JpcHQvaS50ZXN0KGIp",
				"P3QodGhpcy50ZXh0KTovanNvbi8udGVzdChiKSYmdGhpcy5ldmFsSlNPTj90aGlzLmpzb249dGhpcy5y",
				"ZXNwb25zZUpTT049SlNPTi5wYXJzZSh0aGlzLnRleHQpOnRoaXMuZXZhbFNjcmlwdHMmJnRoaXMudGV4",
				"dC5ldmFsU2NyaXB0cygpfSxpbml0Q2FsbGJhY2tzOmZ1bmN0aW9uKCl7dGhpcy5vbih7Y3JlYXRlOiJz",
				"aG93U3Bpbm5lciIsY29tcGxldGU6ImhpZGVTcGlubmVyIixjYW5jZWw6ImhpZGVTcGlubmVyIn0pLHRo",
				"aXMub24oImNvbXBsZXRlIiwidHJ5U2NyaXB0cyIpLGNlLkVWRU5UUy5lYWNoKGZ1bmN0aW9uKGEpe3Ro",
				"aXMub24oYSxmdW5jdGlvbigpe2NlLmZpcmUoYSx0aGlzLHRoaXMueGhyKX0pfSx0aGlzKX0sc2hvd1Nw",
				"aW5uZXI6ZnVuY3Rpb24oKXtjZS5zaG93U3Bpbm5lci5jYWxsKHRoaXMsdGhpcyl9LGhpZGVTcGlubmVy",
				"OmZ1bmN0aW9uKCl7Y2UuaGlkZVNwaW5uZXIuY2FsbCh0aGlzLHRoaXMpfX0pO3MoYmwoY2UpLHtjb3Vu",
				"dGVyOjAsc2hvd1NwaW5uZXI6ZnVuY3Rpb24oYSl7Y2UudHJ5U3Bpbm5lcihhLCJzaG93Iil9LGhpZGVT",
				"cGlubmVyOmZ1bmN0aW9uKGEpe2NlLnRyeVNwaW5uZXIoYSwiaGlkZSIpfSx0cnlTcGlubmVyOmZ1bmN0",
				"aW9uKGEsYil7dmFyIGM9YXx8Y2UuT3B0aW9ucyxkPUUoYy5zcGlubmVyKTtkJiZkW2JdKGMuc3Bpbm5l",
				"ckZ4LHtkdXJhdGlvbjoxMDB9KX0sY291bnRJbjpmdW5jdGlvbigpe2NlLmNvdW50ZXIrKyxjZS5zaG93",
				"U3Bpbm5lcigpfSxjb3VudE91dDpmdW5jdGlvbigpe2NlLmNvdW50ZXItLSxjZS5jb3VudGVyPDEmJmNl",
				"LmhpZGVTcGlubmVyKCl9fSkub24oe2NyZWF0ZToiY291bnRJbiIsY29tcGxldGU6ImNvdW50T3V0Iixj",
				"YW5jZWw6ImNvdW50T3V0In0pLGJULmluY2x1ZGUoe3NlbmQ6ZnVuY3Rpb24oYSl7YT1hfHx7fSxhLm1l",
				"dGhvZD1hLm1ldGhvZHx8dGhpcy5fLm1ldGhvZHx8InBvc3QiLHRoaXMueGhyPShuZXcgY2UodGhpcy5f",
				"LmFjdGlvbnx8Yi5sb2NhdGlvbi5ocmVmLHMoe3NwaW5uZXI6dGhpcy5maXJzdCgiLnNwaW5uZXIiKX0s",
				"YSkpKS5vbkNvbXBsZXRlKHRoaXMuZW5hYmxlLmJpbmQodGhpcykpLm9uQ2FuY2VsKHRoaXMuZW5hYmxl",
				"LmJpbmQodGhpcykpLnNlbmQodGhpcyksdGhpcy5kaXNhYmxlLmJpbmQodGhpcykuZGVsYXkoMSk7cmV0",
				"dXJuIHRoaXN9LGNhbmNlbFhocjpmdW5jdGlvbigpe3RoaXMueGhyIGluc3RhbmNlb2YgY2UmJnRoaXMu",
				"eGhyLmNhbmNlbCgpO3JldHVybiB0aGlzfSxyZW1vdGl6ZTpmdW5jdGlvbihhKXt0aGlzLnJlbW90ZXx8",
				"KHRoaXMub24oInN1Ym1pdCIsY2YsYSksdGhpcy5yZW1vdGU9ITApO3JldHVybiB0aGlzfSx1bnJlbW90",
				"aXplOmZ1bmN0aW9uKCl7dGhpcy5zdG9wT2JzZXJ2aW5nKCJzdWJtaXQiLGNmKSx0aGlzLnJlbW90ZT0h",
				"MTtyZXR1cm4gdGhpc319KSxjZS5pbmNsdWRlKHtwcmVwYXJlUGFyYW1zOmZ1bmN0aW9uKGEpe2EmJmEg",
				"aW5zdGFuY2VvZiBiVCYmKHRoaXMuZm9ybT1hLGE9YS52YWx1ZXMoKSk7cmV0dXJuIGF9fSksYkcuaW5j",
				"bHVkZSh7bG9hZDpmdW5jdGlvbihhLGIpeyhuZXcgY2UoYSxzKHttZXRob2Q6ImdldCJ9LGIpKSkudXBk",
				"YXRlKHRoaXMpO3JldHVybiB0aGlzfX0pLGNlLkR1bW15PXtvcGVuOmZ1bmN0aW9uKCl7fSxzZXRSZXF1",
				"ZXN0SGVhZGVyOmZ1bmN0aW9uKCl7fSxvbnJlYWR5c3RhdGVjaGFuZ2U6ZnVuY3Rpb24oKXt9fSxjZS5J",
				"RnJhbWVkPW5ldyBiYyh7aW5jbHVkZTpjZS5EdW1teSxpbml0aWFsaXplOmZ1bmN0aW9uKGEpe3RoaXMu",
				"Zm9ybT1hLHRoaXMuaWQ9Inhocl8iKyhuZXcgRGF0ZSkuZ2V0VGltZSgpLHRoaXMuZm9ybS5kb2MoKS5m",
				"aXJzdCgiYm9keSIpLmFwcGVuZCgnPGk+PGlmcmFtZSBuYW1lPSInK3RoaXMuaWQrJyIgaWQ9IicrdGhp",
				"cy5pZCsnIiB3aWR0aD0iMCIgaGVpZ2h0PSIwIiBmcmFtZWJvcmRlcj0iMCIgc3JjPSJhYm91dDpibGFu",
				"ayI+PC9pZnJhbWU+PC9pPicsImFmdGVyIiksRSh0aGlzLmlkKS5vbigibG9hZCIsdGhpcy5vbkxvYWQu",
				"YmluZCh0aGlzKSl9LHNlbmQ6ZnVuY3Rpb24oKXt0aGlzLmZvcm0uc2V0KCJ0YXJnZXQiLHRoaXMuaWQp",
				"LnN1Ym1pdCgpfSxvbkxvYWQ6ZnVuY3Rpb24oKXt0aGlzLnN0YXR1cz0yMDAsdGhpcy5yZWFkeVN0YXRl",
				"PTQsdGhpcy5mb3JtLnNldCgidGFyZ2V0IiwiIik7dHJ5e3RoaXMucmVzcG9uc2VUZXh0PWFbdGhpcy5p",
				"ZF0uZG9jdW1lbnQuZG9jdW1lbnRFbGVtZW50LmlubmVySFRNTH1jYXRjaChiKXt9dGhpcy5vbnJlYWR5",
				"c3RhdGVjaGFuZ2UoKX0sYWJvcnQ6ZnVuY3Rpb24oKXtFKHRoaXMuaWQpLnNldCgic3JjIiwiYWJvdXQ6",
				"YmxhbmsiKX19KSxjZS5KU09OUD1uZXcgYmMoe2luY2x1ZGU6Y2UuRHVtbXkscHJlZml4OiJqc29ucCIs",
				"aW5pdGlhbGl6ZTpmdW5jdGlvbihhKXt0aGlzLnhocj1hLHRoaXMubmFtZT10aGlzLnByZWZpeCsobmV3",
				"IERhdGUpLmdldFRpbWUoKSx0aGlzLnBhcmFtPSh5KGEuanNvbnApP2EuanNvbnA6ImNhbGxiYWNrIikr",
				"Ij0iK3RoaXMubmFtZSx0aGlzLnNjcmlwdD1HKCJzY3JpcHQiLHtjaGFyc2V0OmEuZW5jb2RpbmcsYXN5",
				"bmM6YS5hc3luY30pfSxvcGVuOmZ1bmN0aW9uKGEsYixjKXt0aGlzLnVybD1iLHRoaXMubWV0aG9kPWF9",
				"LHNlbmQ6ZnVuY3Rpb24oYil7YVt0aGlzLm5hbWVdPXRoaXMuZmluaXNoLmJpbmQodGhpcyksdGhpcy5z",
				"Y3JpcHQuc2V0KCJzcmMiLHRoaXMudXJsKyh0aGlzLnVybC5pbmNsdWRlKCI/Iik/IiYiOiI/IikrdGhp",
				"cy5wYXJhbSsiJiIrYikuaW5zZXJ0VG8oRigic2NyaXB0IikubGFzdCgpLCJhZnRlciIpfSxmaW5pc2g6",
				"ZnVuY3Rpb24oYSl7dGhpcy5zdGF0dXM9MjAwLHRoaXMucmVhZHlTdGF0ZT00LHRoaXMueGhyLmpzb249",
				"dGhpcy54aHIucmVzcG9uc2VKU09OPWEsdGhpcy5vbnJlYWR5c3RhdGVjaGFuZ2UoKX0sYWJvcnQ6ZnVu",
				"Y3Rpb24oKXthW3RoaXMubmFtZV09ZnVuY3Rpb24oKXt9fX0pO3ZhciBjZz1qLkZ4PW5ldyBiYyhiayx7",
				"ZXh0ZW5kOntFVkVOVFM6SCgic3RhcnQgZmluaXNoIGNhbmNlbCIpLER1cmF0aW9uczp7InNob3J0Ijoy",
				"MDAsbm9ybWFsOjQwMCwibG9uZyI6ODAwfSxPcHRpb25zOntmcHM6YnQ/NDA6NjAsZHVyYXRpb246Im5v",
				"cm1hbCIsdHJhbnNpdGlvbjoiZGVmYXVsdCIscXVldWU6ITAsZW5naW5lOiJjc3MifX0saW5pdGlhbGl6",
				"ZTpmdW5jdGlvbihhLGIpe3RoaXMuJHN1cGVyKGIpLHRoaXMuZWxlbWVudD1FKGEpLGNqKHRoaXMpfSxz",
				"dGFydDpmdW5jdGlvbigpe2lmKGNrKHRoaXMsYXJndW1lbnRzKSlyZXR1cm4gdGhpcztjbCh0aGlzKSx0",
				"aGlzLnByZXBhcmUuYXBwbHkodGhpcyxhcmd1bWVudHMpLGNwKHRoaXMpO3JldHVybiB0aGlzLmZpcmUo",
				"InN0YXJ0Iix0aGlzKX0sZmluaXNoOmZ1bmN0aW9uKCl7Y3EodGhpcyksY20odGhpcyksdGhpcy5maXJl",
				"KCJmaW5pc2giKSxjbih0aGlzKTtyZXR1cm4gdGhpc30sY2FuY2VsOmZ1bmN0aW9uKCl7Y3EodGhpcyks",
				"Y20odGhpcyk7cmV0dXJuIHRoaXMuZmlyZSgiY2FuY2VsIil9LHByZXBhcmU6ZnVuY3Rpb24oKXt9LHJl",
				"bmRlcjpmdW5jdGlvbigpe319KSxjaD1bXSxjaT1bXSxjcj17ImRlZmF1bHQiOiIoLjI1LC4xLC4yNSwx",
				"KSIsbGluZWFyOiIoMCwwLDEsMSkiLCJlYXNlLWluIjoiKC40MiwwLDEsMSkiLCJlYXNlLW91dCI6Iigw",
				"LDAsLjU4LDEpIiwiZWFzZS1pbi1vdXQiOiIoLjQyLDAsLjU4LDEpIiwiZWFzZS1vdXQtaW4iOiIoMCwu",
				"NDIsMSwuNTgpIn0sY3M9e307ZS5DT0xPUlM9e21hcm9vbjoiIzgwMDAwMCIscmVkOiIjZmYwMDAwIixv",
				"cmFuZ2U6IiNmZkE1MDAiLHllbGxvdzoiI2ZmZmYwMCIsb2xpdmU6IiM4MDgwMDAiLHB1cnBsZToiIzgw",
				"MDA4MCIsZnVjaHNpYToiI2ZmMDBmZiIsd2hpdGU6IiNmZmZmZmYiLGxpbWU6IiMwMGZmMDAiLGdyZWVu",
				"OiIjMDA4MDAwIixuYXZ5OiIjMDAwMDgwIixibHVlOiIjMDAwMGZmIixhcXVhOiIjMDBmZmZmIix0ZWFs",
				"OiIjMDA4MDgwIixibGFjazoiIzAwMDAwMCIsc2lsdmVyOiIjYzBjMGMwIixncmF5OiIjODA4MDgwIixi",
				"cm93bjoiI2E1MmEyYSJ9LGUuaW5jbHVkZSh7dG9IZXg6ZnVuY3Rpb24oKXt2YXIgYT0vXiMoXHcpKFx3",
				"KShcdykkLy5leGVjKHRoaXMpO2E/YT0iIyIrYVsxXSthWzFdK2FbMl0rYVsyXSthWzNdK2FbM106KGE9",
				"L15yZ2JcKChcZCspLFxzKihcZCspLFxzKihcZCspXCkkLy5leGVjKHRoaXMpKT9hPSIjIithLnNsaWNl",
				"KDEpLm1hcChmdW5jdGlvbihhKXthPShhLTApLnRvU3RyaW5nKDE2KTtyZXR1cm4gYS5sZW5ndGg9PTE/",
				"IjAiK2E6YX0pLmpvaW4oIiIpOmE9ZS5DT0xPUlNbdGhpc118fHRoaXM7cmV0dXJuIGF9LHRvUmdiOmZ1",
				"bmN0aW9uKGEpe3ZhciBiPS8jKFthLWZcZF17Mn0pKFthLWZcZF17Mn0pKFthLWZcZF17Mn0pL2kuZXhl",
				"Yyh0aGlzLnRvSGV4KCl8fCIiKTtiJiYoYj1iLnNsaWNlKDEpLm1hcCgidG9JbnQiLDE2KSxiPWE/Yjoi",
				"cmdiKCIrYisiKSIpO3JldHVybiBifX0pLGJHLmluY2x1ZGUoe3N0b3A6ZnVuY3Rpb24oKXtjbyh0aGlz",
				"KTtyZXR1cm4gdGhpc30saGlkZTpmdW5jdGlvbihhLGIpe3JldHVybiBhJiZ0aGlzLnZpc2libGUoKT9j",
				"dSh0aGlzLGEsWyJvdXQiLGJdKTp0aGlzLiRzdXBlcigpfSxzaG93OmZ1bmN0aW9uKGEsYil7cmV0dXJu",
				"IGEmJiF0aGlzLnZpc2libGUoKT9jdSh0aGlzLGEsWyJpbiIsYl0pOnRoaXMuJHN1cGVyKCl9LHRvZ2ds",
				"ZTpmdW5jdGlvbihhLGIpe3JldHVybiBhP2N1KHRoaXMsYSxbInRvZ2dsZSIsYl0pOnRoaXMuJHN1cGVy",
				"KCl9LHJlbW92ZTpmdW5jdGlvbihhLGIpe3JldHVybiBhJiZ0aGlzLnZpc2libGUoKT9jdSh0aGlzLGEs",
				"WyJvdXQiLHMoYnx8e30se29uRmluaXNoOnRoaXMuJHN1cGVyLmJpbmQodGhpcyl9KV0pOnRoaXMuJHN1",
				"cGVyKCl9LG1vcnBoOmZ1bmN0aW9uKGEsYil7cmV0dXJuIGN1KHRoaXMsIm1vcnBoIixbYSxifHx7fV0p",
				"fSxoaWdobGlnaHQ6ZnVuY3Rpb24oKXtyZXR1cm4gY3UodGhpcywiaGlnaGxpZ2h0Iixhcmd1bWVudHMp",
				"fSxmYWRlOmZ1bmN0aW9uKCl7cmV0dXJuIGN1KHRoaXMsImZhZGUiLGFyZ3VtZW50cyl9LHNsaWRlOmZ1",
				"bmN0aW9uKCl7cmV0dXJuIGN1KHRoaXMsInNsaWRlIixhcmd1bWVudHMpfSxzY3JvbGw6ZnVuY3Rpb24o",
				"YSxiKXtyZXR1cm4gY3UodGhpcywic2Nyb2xsIixbYSxifHx7fV0pfSxzY3JvbGxUbzpmdW5jdGlvbihh",
				"LGIpe3JldHVybiBBKGIpP3RoaXMuc2Nyb2xsKGEsYik6dGhpcy4kc3VwZXIuYXBwbHkodGhpcyxhcmd1",
				"bWVudHMpfX0pO3ZhciBjdj1bIldlYmtpdFQiLCJPVCIsIk1velQiLCJNc1QiLCJ0Il0uZmlyc3QoZnVu",
				"Y3Rpb24oYSl7cmV0dXJuIGErInJhbnNpdGlvbiJpbiBvLnN0eWxlfSksY3c9Y3YrInJhbnNpdGlvbiIs",
				"Y3g9Y3crIlByb3BlcnR5IixjeT1jdysiRHVyYXRpb24iLGN6PWN3KyJUaW1pbmdGdW5jdGlvbiIsY0E9",
				"e1NpbjoiY3ViaWMtYmV6aWVyKC4zLDAsLjYsMSkiLENvczoiY3ViaWMtYmV6aWVyKDAsLjMsLjYsMCki",
				"LExvZzoiY3ViaWMtYmV6aWVyKDAsLjYsLjMsLjgpIixFeHA6ImN1YmljLWJlemllciguNiwwLC44LC4z",
				"KSIsTGluOiJjdWJpYy1iZXppZXIoMCwwLDEsMSkifTtjZy5PcHRpb25zLmVuZ2luZT1jdj09PWl8fGJx",
				"PyJqYXZhc2NyaXB0IjoibmF0aXZlIixjZy5Nb3JwaD1uZXcgYmMoY2cse3ByZXBhcmU6ZnVuY3Rpb24o",
				"YSl7aWYodGhpcy5vcHRpb25zLmVuZ2luZT09PSJuYXRpdmUiJiZjdiE9PWkpdGhpcy5yZW5kZXI9dGhp",
				"cy50cmFuc2l0aW9uPWZ1bmN0aW9uKCl7fSxjQi5jYWxsKHRoaXMsYSk7ZWxzZXt2YXIgYj1jRShhKSxj",
				"PWNKKHRoaXMuZWxlbWVudCxiKSxkPWNLKHRoaXMuZWxlbWVudCxhLGIpO2NJKHRoaXMuZWxlbWVudCxj",
				"LGQpLHRoaXMuYmVmb3JlPWNIKGMpLHRoaXMuYWZ0ZXI9Y0goZCl9fSxyZW5kZXI6ZnVuY3Rpb24oYSl7",
				"dmFyIGIsYyxkLGU9dGhpcy5lbGVtZW50Ll8uc3R5bGUsZixnLGk7Zm9yKGYgaW4gdGhpcy5hZnRlcil7",
				"Yj10aGlzLmJlZm9yZVtmXSxjPXRoaXMuYWZ0ZXJbZl07Zm9yKGc9MCxpPWMubGVuZ3RoO2c8aTtnKysp",
				"ZD1iW2ddKyhjW2ddLWJbZ10pKmEsYy5yJiYoZD1oLnJvdW5kKGQpKSxjLnRbZyoyKzFdPWQ7ZVtmXT1j",
				"LnQuam9pbigiIil9fX0pO3ZhciBjQz1IKCJUb3AgTGVmdCBSaWdodCBCb3R0b20iKTtjZy5IaWdobGln",
				"aHQ9bmV3IGJjKGNnLk1vcnBoLHtleHRlbmQ6e09wdGlvbnM6Yy5tZXJnZShjZy5PcHRpb25zLHtjb2xv",
				"cjoiI0ZGOCIsdHJhbnNpdGlvbjoiRXhwIn0pfSxwcmVwYXJlOmZ1bmN0aW9uKGEsYil7dmFyIGM9dGhp",
				"cy5lbGVtZW50LGQ9Yy5fLnN0eWxlLGU9ImJhY2tncm91bmRDb2xvciIsZj1ifHxjLmdldFN0eWxlKGUp",
				"O2NGKGYpJiYodGhpcy5vbkZpbmlzaChmdW5jdGlvbigpe2RbZV09InRyYW5zcGFyZW50In0pLGY9W2Nd",
				"LmNvbmNhdChjLnBhcmVudHMoKSkubWFwKCJnZXRTdHlsZSIsZSkucmVqZWN0KGNGKS5jb21wYWN0KCku",
				"Zmlyc3QoKXx8IiNGRkYiKSxkW2VdPWF8fHRoaXMub3B0aW9ucy5jb2xvcjtyZXR1cm4gdGhpcy4kc3Vw",
				"ZXIoe2JhY2tncm91bmRDb2xvcjpmfSl9fSksY2cuVHdpbj1uZXcgYmMoY2cuTW9ycGgse2ZpbmlzaDpm",
				"dW5jdGlvbigpe3RoaXMuaG93PT09Im91dCImJmJHLnByb3RvdHlwZS5oaWRlLmNhbGwodGhpcy5lbGVt",
				"ZW50KTtyZXR1cm4gdGhpcy4kc3VwZXIoKX0sc2V0SG93OmZ1bmN0aW9uKGEpe3RoaXMuaG93PWF8fCJ0",
				"b2dnbGUiLHRoaXMuaG93PT09InRvZ2dsZSImJih0aGlzLmhvdz10aGlzLmVsZW1lbnQudmlzaWJsZSgp",
				"PyJvdXQiOiJpbiIpfX0pLGNnLlNsaWRlPW5ldyBiYyhjZy5Ud2luLHtleHRlbmQ6e09wdGlvbnM6Yy5t",
				"ZXJnZShjZy5PcHRpb25zLHtkaXJlY3Rpb246InRvcCJ9KX0scHJlcGFyZTpmdW5jdGlvbihhKXtmdW5j",
				"dGlvbiBmKCl7Zm9yKHZhciBhIGluIGUpZFthXT1lW2FdfXRoaXMuc2V0SG93KGEpO3ZhciBiPWJHLnBy",
				"b3RvdHlwZS5zaG93LmNhbGwodGhpcy5lbGVtZW50KSxkPWIuXy5zdHlsZSxlPWMub25seShkLCJvdmVy",
				"ZmxvdyIsIndpZHRoIiwiaGVpZ2h0IiwibWFyZ2luVG9wIiwibWFyZ2luTGVmdCIpO3RoaXMub25GaW5p",
				"c2goZikub25DYW5jZWwoZiksZC5vdmVyZmxvdz0iaGlkZGVuIjtyZXR1cm4gdGhpcy4kc3VwZXIoY0wo",
				"ZCxiLnNpemUoKSx0aGlzLm9wdGlvbnMuZGlyZWN0aW9uLHRoaXMuaG93KSl9fSksY2cuRmFkZT1uZXcg",
				"YmMoY2cuVHdpbix7cHJlcGFyZTpmdW5jdGlvbihhKXt0aGlzLnNldEhvdyhhKSx0aGlzLmhvdz09PSJp",
				"biImJmJHLnByb3RvdHlwZS5zaG93LmNhbGwodGhpcy5lbGVtZW50LnNldFN0eWxlKHtvcGFjaXR5OjB9",
				"KSk7cmV0dXJuIHRoaXMuJHN1cGVyKHtvcGFjaXR5OnRoaXMuaG93PT09ImluIj8xOjB9KX19KSxjZy5B",
				"dHRyPW5ldyBiYyhjZyx7cHJlcGFyZTpmdW5jdGlvbihhKXt0aGlzLmJlZm9yZT17fSx0aGlzLmFmdGVy",
				"PWE7dmFyIGIsYz10aGlzLmVsZW1lbnQuXztmb3IoYiBpbiBhKXRoaXMuYmVmb3JlW2JdPWNbYl19LHJl",
				"bmRlcjpmdW5jdGlvbihhKXt2YXIgYixjPXRoaXMuZWxlbWVudC5fLGQ9dGhpcy5iZWZvcmU7Zm9yKGIg",
				"aW4gZCljW2JdPWRbYl0rKHRoaXMuYWZ0ZXJbYl0tZFtiXSkqYX19KSxjZy5TY3JvbGw9bmV3IGJjKGNn",
				"LkF0dHIse2luaXRpYWxpemU6ZnVuY3Rpb24oYSxiKXthPUUoYSksdGhpcy4kc3VwZXIoYSBpbnN0YW5j",
				"ZW9mIGJEP2EuXy5kb2N1bWVudFticy5XZWJLaXQ/ImJvZHkiOiJkb2N1bWVudEVsZW1lbnQiXTphLGIp",
				"fSxwcmVwYXJlOmZ1bmN0aW9uKGEpe3ZhciBiPXt9OyJ4ImluIGEmJihiLnNjcm9sbExlZnQ9YS54KSwi",
				"eSJpbiBhJiYoYi5zY3JvbGxUb3A9YS55KSx0aGlzLiRzdXBlcihiKX19KTt2YXIgY009ai5Db29raWU9",
				"bmV3IGJjKHtpbmNsdWRlOmJqLGV4dGVuZDp7c2V0OmZ1bmN0aW9uKGEsYixjKXtyZXR1cm4obmV3IHRo",
				"aXMoYSxjKSkuc2V0KGIpfSxnZXQ6ZnVuY3Rpb24oYSxiKXtyZXR1cm4obmV3IHRoaXMoYSxiKSkuZ2V0",
				"KCl9LHJlbW92ZTpmdW5jdGlvbihhLGIpe3JldHVybihuZXcgdGhpcyhhLGIpKS5yZW1vdmUoKX0sZW5h",
				"YmxlZDpmdW5jdGlvbigpe2IuY29va2llPSJfX3Q9MSI7cmV0dXJuIGIuY29va2llLmluZGV4T2YoIl9f",
				"dD0xIikhPS0xfSxPcHRpb25zOntzZWN1cmU6ITEsZG9jdW1lbnQ6Yn19LGluaXRpYWxpemU6ZnVuY3Rp",
				"b24oYSxiKXt0aGlzLm5hbWU9YSx0aGlzLnNldE9wdGlvbnMoYil9LHNldDpmdW5jdGlvbihhKXt5KGEp",
				"fHwoYT1KU09OLnN0cmluZ2lmeShhKSk7dmFyIGI9ZW5jb2RlVVJJQ29tcG9uZW50KGEpLGM9dGhpcy5v",
				"cHRpb25zO2MuZG9tYWluJiYoYis9IjsgZG9tYWluPSIrYy5kb21haW4pLGMucGF0aCYmKGIrPSI7IHBh",
				"dGg9IitjLnBhdGgpO2lmKGMuZHVyYXRpb24pe3ZhciBkPW5ldyBEYXRlO2Quc2V0VGltZShkLmdldFRp",
				"bWUoKStjLmR1cmF0aW9uKjI0KjYwKjYwKjFlMyksYis9IjsgZXhwaXJlcz0iK2QudG9HTVRTdHJpbmco",
				"KX1jLnNlY3VyZSYmKGIrPSI7IHNlY3VyZSIpLGMuZG9jdW1lbnQuY29va2llPXRoaXMubmFtZSsiPSIr",
				"YjtyZXR1cm4gdGhpc30sZ2V0OmZ1bmN0aW9uKCl7dmFyIGE9dGhpcy5vcHRpb25zLmRvY3VtZW50LmNv",
				"b2tpZS5tYXRjaCgiKD86Xnw7KVxccyoiK1JlZ0V4cC5lc2NhcGUodGhpcy5uYW1lKSsiPShbXjtdKiki",
				"KTtpZihhKXthPWRlY29kZVVSSUNvbXBvbmVudChhWzFdKTt0cnl7YT1KU09OLnBhcnNlKGEpfWNhdGNo",
				"KGIpe319cmV0dXJuIGF8fG51bGx9LHJlbW92ZTpmdW5jdGlvbigpe3RoaXMub3B0aW9ucy5kdXJhdGlv",
				"bj0tMTtyZXR1cm4gdGhpcy5zZXQoIiIpfX0pO3MoYSxjLndpdGhvdXQoaiwidmVyc2lvbiIsIm1vZHVs",
				"ZXMiKSk7cmV0dXJuIGp9KHdpbmRvdyxkb2N1bWVudCxPYmplY3QsQXJyYXksU3RyaW5nLEZ1bmN0aW9u",
				"LE51bWJlcixNYXRoKTtSaWdodEpTLkJyb3dzZXIuT0xEJiZmdW5jdGlvbihhKXt2YXIgYj1hLmNyZWF0",
				"ZUVsZW1lbnQoInNjcmlwdCIpLGM9YS5nZXRFbGVtZW50c0J5VGFnTmFtZSgic2NyaXB0IiksZD1jW2Mu",
				"bGVuZ3RoLTFdO2Iuc3JjPWQuc3JjLnJlcGxhY2UoLyhefFwvKShyaWdodCkoW15cL10rKSQvLCIkMSQy",
				"LW9sZHMkMyIpLGQucGFyZW50Tm9kZS5hcHBlbmRDaGlsZChiKX0oZG9jdW1lbnQp"
				].join('');

				buff = new Buffer(rightjs_base64_blob, 'base64').toString('ascii');
				fs.writeFileSync(save_file,buff);				
				break;
			//
			// Install Mootools
			//
			case "mootools":
				save_file = 'static/js/mootools-core-1.4.5-full-compat.js';

					
				var moootools_base64_blob = [
				"LyoKLS0tCk1vb1Rvb2xzOiB0aGUgamF2YXNjcmlwdCBmcmFtZXdvcmsKCndlYiBidWlsZDoKIC0gaHR0",
				"cDovL21vb3Rvb2xzLm5ldC9jb3JlLzc2YmY0NzA2MmQ2YzE5ODNkNjZjZTQ3YWQ2NmFhMGUwCgpwYWNr",
				"YWdlciBidWlsZDoKIC0gcGFja2FnZXIgYnVpbGQgQ29yZS9Db3JlIENvcmUvQXJyYXkgQ29yZS9TdHJp",
				"bmcgQ29yZS9OdW1iZXIgQ29yZS9GdW5jdGlvbiBDb3JlL09iamVjdCBDb3JlL0V2ZW50IENvcmUvQnJv",
				"d3NlciBDb3JlL0NsYXNzIENvcmUvQ2xhc3MuRXh0cmFzIENvcmUvU2xpY2suUGFyc2VyIENvcmUvU2xp",
				"Y2suRmluZGVyIENvcmUvRWxlbWVudCBDb3JlL0VsZW1lbnQuU3R5bGUgQ29yZS9FbGVtZW50LkV2ZW50",
				"IENvcmUvRWxlbWVudC5EZWxlZ2F0aW9uIENvcmUvRWxlbWVudC5EaW1lbnNpb25zIENvcmUvRnggQ29y",
				"ZS9GeC5DU1MgQ29yZS9GeC5Ud2VlbiBDb3JlL0Z4Lk1vcnBoIENvcmUvRnguVHJhbnNpdGlvbnMgQ29y",
				"ZS9SZXF1ZXN0IENvcmUvUmVxdWVzdC5IVE1MIENvcmUvUmVxdWVzdC5KU09OIENvcmUvQ29va2llIENv",
				"cmUvSlNPTiBDb3JlL0RPTVJlYWR5IENvcmUvU3dpZmYKCi4uLgoqLwoKLyoKLS0tCgpuYW1lOiBDb3Jl",
				"CgpkZXNjcmlwdGlvbjogVGhlIGhlYXJ0IG9mIE1vb1Rvb2xzLgoKbGljZW5zZTogTUlULXN0eWxlIGxp",
				"Y2Vuc2UuCgpjb3B5cmlnaHQ6IENvcHlyaWdodCAoYykgMjAwNi0yMDEyIFtWYWxlcmlvIFByb2lldHRp",
				"XShodHRwOi8vbWFkNG1pbGsubmV0LykuCgphdXRob3JzOiBUaGUgTW9vVG9vbHMgcHJvZHVjdGlvbiB0",
				"ZWFtIChodHRwOi8vbW9vdG9vbHMubmV0L2RldmVsb3BlcnMvKQoKaW5zcGlyYXRpb246CiAgLSBDbGFz",
				"cyBpbXBsZW1lbnRhdGlvbiBpbnNwaXJlZCBieSBbQmFzZS5qc10oaHR0cDovL2RlYW4uZWR3YXJkcy5u",
				"YW1lL3dlYmxvZy8yMDA2LzAzL2Jhc2UvKSBDb3B5cmlnaHQgKGMpIDIwMDYgRGVhbiBFZHdhcmRzLCBb",
				"R05VIExlc3NlciBHZW5lcmFsIFB1YmxpYyBMaWNlbnNlXShodHRwOi8vb3BlbnNvdXJjZS5vcmcvbGlj",
				"ZW5zZXMvbGdwbC1saWNlbnNlLnBocCkKICAtIFNvbWUgZnVuY3Rpb25hbGl0eSBpbnNwaXJlZCBieSBb",
				"UHJvdG90eXBlLmpzXShodHRwOi8vcHJvdG90eXBlanMub3JnKSBDb3B5cmlnaHQgKGMpIDIwMDUtMjAw",
				"NyBTYW0gU3RlcGhlbnNvbiwgW01JVCBMaWNlbnNlXShodHRwOi8vb3BlbnNvdXJjZS5vcmcvbGljZW5z",
				"ZXMvbWl0LWxpY2Vuc2UucGhwKQoKcHJvdmlkZXM6IFtDb3JlLCBNb29Ub29scywgVHlwZSwgdHlwZU9m",
				"LCBpbnN0YW5jZU9mLCBOYXRpdmVdCgouLi4KKi8KCihmdW5jdGlvbigpewoKdGhpcy5Nb29Ub29scyA9",
				"IHsKCXZlcnNpb246ICcxLjQuNScsCglidWlsZDogJ2FiOGVhODgyNGRjM2IyNGI2NjY2ODY3YTJjNGVk",
				"NThlYmI3NjJjZjAnCn07CgovLyB0eXBlT2YsIGluc3RhbmNlT2YKCnZhciB0eXBlT2YgPSB0aGlzLnR5",
				"cGVPZiA9IGZ1bmN0aW9uKGl0ZW0pewoJaWYgKGl0ZW0gPT0gbnVsbCkgcmV0dXJuICdudWxsJzsKCWlm",
				"IChpdGVtLiRmYW1pbHkgIT0gbnVsbCkgcmV0dXJuIGl0ZW0uJGZhbWlseSgpOwoKCWlmIChpdGVtLm5v",
				"ZGVOYW1lKXsKCQlpZiAoaXRlbS5ub2RlVHlwZSA9PSAxKSByZXR1cm4gJ2VsZW1lbnQnOwoJCWlmIChp",
				"dGVtLm5vZGVUeXBlID09IDMpIHJldHVybiAoL1xTLykudGVzdChpdGVtLm5vZGVWYWx1ZSkgPyAndGV4",
				"dG5vZGUnIDogJ3doaXRlc3BhY2UnOwoJfSBlbHNlIGlmICh0eXBlb2YgaXRlbS5sZW5ndGggPT0gJ251",
				"bWJlcicpewoJCWlmIChpdGVtLmNhbGxlZSkgcmV0dXJuICdhcmd1bWVudHMnOwoJCWlmICgnaXRlbScg",
				"aW4gaXRlbSkgcmV0dXJuICdjb2xsZWN0aW9uJzsKCX0KCglyZXR1cm4gdHlwZW9mIGl0ZW07Cn07Cgp2",
				"YXIgaW5zdGFuY2VPZiA9IHRoaXMuaW5zdGFuY2VPZiA9IGZ1bmN0aW9uKGl0ZW0sIG9iamVjdCl7Cglp",
				"ZiAoaXRlbSA9PSBudWxsKSByZXR1cm4gZmFsc2U7Cgl2YXIgY29uc3RydWN0b3IgPSBpdGVtLiRjb25z",
				"dHJ1Y3RvciB8fCBpdGVtLmNvbnN0cnVjdG9yOwoJd2hpbGUgKGNvbnN0cnVjdG9yKXsKCQlpZiAoY29u",
				"c3RydWN0b3IgPT09IG9iamVjdCkgcmV0dXJuIHRydWU7CgkJY29uc3RydWN0b3IgPSBjb25zdHJ1Y3Rv",
				"ci5wYXJlbnQ7Cgl9CgkvKjxsdElFOD4qLwoJaWYgKCFpdGVtLmhhc093blByb3BlcnR5KSByZXR1cm4g",
				"ZmFsc2U7CgkvKjwvbHRJRTg+Ki8KCXJldHVybiBpdGVtIGluc3RhbmNlb2Ygb2JqZWN0Owp9OwoKLy8g",
				"RnVuY3Rpb24gb3ZlcmxvYWRpbmcKCnZhciBGdW5jdGlvbiA9IHRoaXMuRnVuY3Rpb247Cgp2YXIgZW51",
				"bWVyYWJsZXMgPSB0cnVlOwpmb3IgKHZhciBpIGluIHt0b1N0cmluZzogMX0pIGVudW1lcmFibGVzID0g",
				"bnVsbDsKaWYgKGVudW1lcmFibGVzKSBlbnVtZXJhYmxlcyA9IFsnaGFzT3duUHJvcGVydHknLCAndmFs",
				"dWVPZicsICdpc1Byb3RvdHlwZU9mJywgJ3Byb3BlcnR5SXNFbnVtZXJhYmxlJywgJ3RvTG9jYWxlU3Ry",
				"aW5nJywgJ3RvU3RyaW5nJywgJ2NvbnN0cnVjdG9yJ107CgpGdW5jdGlvbi5wcm90b3R5cGUub3Zlcmxv",
				"YWRTZXR0ZXIgPSBmdW5jdGlvbih1c2VQbHVyYWwpewoJdmFyIHNlbGYgPSB0aGlzOwoJcmV0dXJuIGZ1",
				"bmN0aW9uKGEsIGIpewoJCWlmIChhID09IG51bGwpIHJldHVybiB0aGlzOwoJCWlmICh1c2VQbHVyYWwg",
				"fHwgdHlwZW9mIGEgIT0gJ3N0cmluZycpewoJCQlmb3IgKHZhciBrIGluIGEpIHNlbGYuY2FsbCh0aGlz",
				"LCBrLCBhW2tdKTsKCQkJaWYgKGVudW1lcmFibGVzKSBmb3IgKHZhciBpID0gZW51bWVyYWJsZXMubGVu",
				"Z3RoOyBpLS07KXsKCQkJCWsgPSBlbnVtZXJhYmxlc1tpXTsKCQkJCWlmIChhLmhhc093blByb3BlcnR5",
				"KGspKSBzZWxmLmNhbGwodGhpcywgaywgYVtrXSk7CgkJCX0KCQl9IGVsc2UgewoJCQlzZWxmLmNhbGwo",
				"dGhpcywgYSwgYik7CgkJfQoJCXJldHVybiB0aGlzOwoJfTsKfTsKCkZ1bmN0aW9uLnByb3RvdHlwZS5v",
				"dmVybG9hZEdldHRlciA9IGZ1bmN0aW9uKHVzZVBsdXJhbCl7Cgl2YXIgc2VsZiA9IHRoaXM7CglyZXR1",
				"cm4gZnVuY3Rpb24oYSl7CgkJdmFyIGFyZ3MsIHJlc3VsdDsKCQlpZiAodHlwZW9mIGEgIT0gJ3N0cmlu",
				"ZycpIGFyZ3MgPSBhOwoJCWVsc2UgaWYgKGFyZ3VtZW50cy5sZW5ndGggPiAxKSBhcmdzID0gYXJndW1l",
				"bnRzOwoJCWVsc2UgaWYgKHVzZVBsdXJhbCkgYXJncyA9IFthXTsKCQlpZiAoYXJncyl7CgkJCXJlc3Vs",
				"dCA9IHt9OwoJCQlmb3IgKHZhciBpID0gMDsgaSA8IGFyZ3MubGVuZ3RoOyBpKyspIHJlc3VsdFthcmdz",
				"W2ldXSA9IHNlbGYuY2FsbCh0aGlzLCBhcmdzW2ldKTsKCQl9IGVsc2UgewoJCQlyZXN1bHQgPSBzZWxm",
				"LmNhbGwodGhpcywgYSk7CgkJfQoJCXJldHVybiByZXN1bHQ7Cgl9Owp9OwoKRnVuY3Rpb24ucHJvdG90",
				"eXBlLmV4dGVuZCA9IGZ1bmN0aW9uKGtleSwgdmFsdWUpewoJdGhpc1trZXldID0gdmFsdWU7Cn0ub3Zl",
				"cmxvYWRTZXR0ZXIoKTsKCkZ1bmN0aW9uLnByb3RvdHlwZS5pbXBsZW1lbnQgPSBmdW5jdGlvbihrZXks",
				"IHZhbHVlKXsKCXRoaXMucHJvdG90eXBlW2tleV0gPSB2YWx1ZTsKfS5vdmVybG9hZFNldHRlcigpOwoK",
				"Ly8gRnJvbQoKdmFyIHNsaWNlID0gQXJyYXkucHJvdG90eXBlLnNsaWNlOwoKRnVuY3Rpb24uZnJvbSA9",
				"IGZ1bmN0aW9uKGl0ZW0pewoJcmV0dXJuICh0eXBlT2YoaXRlbSkgPT0gJ2Z1bmN0aW9uJykgPyBpdGVt",
				"IDogZnVuY3Rpb24oKXsKCQlyZXR1cm4gaXRlbTsKCX07Cn07CgpBcnJheS5mcm9tID0gZnVuY3Rpb24o",
				"aXRlbSl7CglpZiAoaXRlbSA9PSBudWxsKSByZXR1cm4gW107CglyZXR1cm4gKFR5cGUuaXNFbnVtZXJh",
				"YmxlKGl0ZW0pICYmIHR5cGVvZiBpdGVtICE9ICdzdHJpbmcnKSA/ICh0eXBlT2YoaXRlbSkgPT0gJ2Fy",
				"cmF5JykgPyBpdGVtIDogc2xpY2UuY2FsbChpdGVtKSA6IFtpdGVtXTsKfTsKCk51bWJlci5mcm9tID0g",
				"ZnVuY3Rpb24oaXRlbSl7Cgl2YXIgbnVtYmVyID0gcGFyc2VGbG9hdChpdGVtKTsKCXJldHVybiBpc0Zp",
				"bml0ZShudW1iZXIpID8gbnVtYmVyIDogbnVsbDsKfTsKClN0cmluZy5mcm9tID0gZnVuY3Rpb24oaXRl",
				"bSl7CglyZXR1cm4gaXRlbSArICcnOwp9OwoKLy8gaGlkZSwgcHJvdGVjdAoKRnVuY3Rpb24uaW1wbGVt",
				"ZW50KHsKCgloaWRlOiBmdW5jdGlvbigpewoJCXRoaXMuJGhpZGRlbiA9IHRydWU7CgkJcmV0dXJuIHRo",
				"aXM7Cgl9LAoKCXByb3RlY3Q6IGZ1bmN0aW9uKCl7CgkJdGhpcy4kcHJvdGVjdGVkID0gdHJ1ZTsKCQly",
				"ZXR1cm4gdGhpczsKCX0KCn0pOwoKLy8gVHlwZQoKdmFyIFR5cGUgPSB0aGlzLlR5cGUgPSBmdW5jdGlv",
				"bihuYW1lLCBvYmplY3QpewoJaWYgKG5hbWUpewoJCXZhciBsb3dlciA9IG5hbWUudG9Mb3dlckNhc2Uo",
				"KTsKCQl2YXIgdHlwZUNoZWNrID0gZnVuY3Rpb24oaXRlbSl7CgkJCXJldHVybiAodHlwZU9mKGl0ZW0p",
				"ID09IGxvd2VyKTsKCQl9OwoKCQlUeXBlWydpcycgKyBuYW1lXSA9IHR5cGVDaGVjazsKCQlpZiAob2Jq",
				"ZWN0ICE9IG51bGwpewoJCQlvYmplY3QucHJvdG90eXBlLiRmYW1pbHkgPSAoZnVuY3Rpb24oKXsKCQkJ",
				"CXJldHVybiBsb3dlcjsKCQkJfSkuaGlkZSgpOwoJCQkvLzwxLjJjb21wYXQ+CgkJCW9iamVjdC50eXBl",
				"ID0gdHlwZUNoZWNrOwoJCQkvLzwvMS4yY29tcGF0PgoJCX0KCX0KCglpZiAob2JqZWN0ID09IG51bGwp",
				"IHJldHVybiBudWxsOwoKCW9iamVjdC5leHRlbmQodGhpcyk7CglvYmplY3QuJGNvbnN0cnVjdG9yID0g",
				"VHlwZTsKCW9iamVjdC5wcm90b3R5cGUuJGNvbnN0cnVjdG9yID0gb2JqZWN0OwoKCXJldHVybiBvYmpl",
				"Y3Q7Cn07Cgp2YXIgdG9TdHJpbmcgPSBPYmplY3QucHJvdG90eXBlLnRvU3RyaW5nOwoKVHlwZS5pc0Vu",
				"dW1lcmFibGUgPSBmdW5jdGlvbihpdGVtKXsKCXJldHVybiAoaXRlbSAhPSBudWxsICYmIHR5cGVvZiBp",
				"dGVtLmxlbmd0aCA9PSAnbnVtYmVyJyAmJiB0b1N0cmluZy5jYWxsKGl0ZW0pICE9ICdbb2JqZWN0IEZ1",
				"bmN0aW9uXScgKTsKfTsKCnZhciBob29rcyA9IHt9OwoKdmFyIGhvb2tzT2YgPSBmdW5jdGlvbihvYmpl",
				"Y3QpewoJdmFyIHR5cGUgPSB0eXBlT2Yob2JqZWN0LnByb3RvdHlwZSk7CglyZXR1cm4gaG9va3NbdHlw",
				"ZV0gfHwgKGhvb2tzW3R5cGVdID0gW10pOwp9OwoKdmFyIGltcGxlbWVudCA9IGZ1bmN0aW9uKG5hbWUs",
				"IG1ldGhvZCl7CglpZiAobWV0aG9kICYmIG1ldGhvZC4kaGlkZGVuKSByZXR1cm47CgoJdmFyIGhvb2tz",
				"ID0gaG9va3NPZih0aGlzKTsKCglmb3IgKHZhciBpID0gMDsgaSA8IGhvb2tzLmxlbmd0aDsgaSsrKXsK",
				"CQl2YXIgaG9vayA9IGhvb2tzW2ldOwoJCWlmICh0eXBlT2YoaG9vaykgPT0gJ3R5cGUnKSBpbXBsZW1l",
				"bnQuY2FsbChob29rLCBuYW1lLCBtZXRob2QpOwoJCWVsc2UgaG9vay5jYWxsKHRoaXMsIG5hbWUsIG1l",
				"dGhvZCk7Cgl9CgoJdmFyIHByZXZpb3VzID0gdGhpcy5wcm90b3R5cGVbbmFtZV07CglpZiAocHJldmlv",
				"dXMgPT0gbnVsbCB8fCAhcHJldmlvdXMuJHByb3RlY3RlZCkgdGhpcy5wcm90b3R5cGVbbmFtZV0gPSBt",
				"ZXRob2Q7CgoJaWYgKHRoaXNbbmFtZV0gPT0gbnVsbCAmJiB0eXBlT2YobWV0aG9kKSA9PSAnZnVuY3Rp",
				"b24nKSBleHRlbmQuY2FsbCh0aGlzLCBuYW1lLCBmdW5jdGlvbihpdGVtKXsKCQlyZXR1cm4gbWV0aG9k",
				"LmFwcGx5KGl0ZW0sIHNsaWNlLmNhbGwoYXJndW1lbnRzLCAxKSk7Cgl9KTsKfTsKCnZhciBleHRlbmQg",
				"PSBmdW5jdGlvbihuYW1lLCBtZXRob2QpewoJaWYgKG1ldGhvZCAmJiBtZXRob2QuJGhpZGRlbikgcmV0",
				"dXJuOwoJdmFyIHByZXZpb3VzID0gdGhpc1tuYW1lXTsKCWlmIChwcmV2aW91cyA9PSBudWxsIHx8ICFw",
				"cmV2aW91cy4kcHJvdGVjdGVkKSB0aGlzW25hbWVdID0gbWV0aG9kOwp9OwoKVHlwZS5pbXBsZW1lbnQo",
				"ewoKCWltcGxlbWVudDogaW1wbGVtZW50Lm92ZXJsb2FkU2V0dGVyKCksCgoJZXh0ZW5kOiBleHRlbmQu",
				"b3ZlcmxvYWRTZXR0ZXIoKSwKCglhbGlhczogZnVuY3Rpb24obmFtZSwgZXhpc3RpbmcpewoJCWltcGxl",
				"bWVudC5jYWxsKHRoaXMsIG5hbWUsIHRoaXMucHJvdG90eXBlW2V4aXN0aW5nXSk7Cgl9Lm92ZXJsb2Fk",
				"U2V0dGVyKCksCgoJbWlycm9yOiBmdW5jdGlvbihob29rKXsKCQlob29rc09mKHRoaXMpLnB1c2goaG9v",
				"ayk7CgkJcmV0dXJuIHRoaXM7Cgl9Cgp9KTsKCm5ldyBUeXBlKCdUeXBlJywgVHlwZSk7CgovLyBEZWZh",
				"dWx0IFR5cGVzCgp2YXIgZm9yY2UgPSBmdW5jdGlvbihuYW1lLCBvYmplY3QsIG1ldGhvZHMpewoJdmFy",
				"IGlzVHlwZSA9IChvYmplY3QgIT0gT2JqZWN0KSwKCQlwcm90b3R5cGUgPSBvYmplY3QucHJvdG90eXBl",
				"OwoKCWlmIChpc1R5cGUpIG9iamVjdCA9IG5ldyBUeXBlKG5hbWUsIG9iamVjdCk7CgoJZm9yICh2YXIg",
				"aSA9IDAsIGwgPSBtZXRob2RzLmxlbmd0aDsgaSA8IGw7IGkrKyl7CgkJdmFyIGtleSA9IG1ldGhvZHNb",
				"aV0sCgkJCWdlbmVyaWMgPSBvYmplY3Rba2V5XSwKCQkJcHJvdG8gPSBwcm90b3R5cGVba2V5XTsKCgkJ",
				"aWYgKGdlbmVyaWMpIGdlbmVyaWMucHJvdGVjdCgpOwoJCWlmIChpc1R5cGUgJiYgcHJvdG8pIG9iamVj",
				"dC5pbXBsZW1lbnQoa2V5LCBwcm90by5wcm90ZWN0KCkpOwoJfQoKCWlmIChpc1R5cGUpewoJCXZhciBt",
				"ZXRob2RzRW51bWVyYWJsZSA9IHByb3RvdHlwZS5wcm9wZXJ0eUlzRW51bWVyYWJsZShtZXRob2RzWzBd",
				"KTsKCQlvYmplY3QuZm9yRWFjaE1ldGhvZCA9IGZ1bmN0aW9uKGZuKXsKCQkJaWYgKCFtZXRob2RzRW51",
				"bWVyYWJsZSkgZm9yICh2YXIgaSA9IDAsIGwgPSBtZXRob2RzLmxlbmd0aDsgaSA8IGw7IGkrKyl7CgkJ",
				"CQlmbi5jYWxsKHByb3RvdHlwZSwgcHJvdG90eXBlW21ldGhvZHNbaV1dLCBtZXRob2RzW2ldKTsKCQkJ",
				"fQoJCQlmb3IgKHZhciBrZXkgaW4gcHJvdG90eXBlKSBmbi5jYWxsKHByb3RvdHlwZSwgcHJvdG90eXBl",
				"W2tleV0sIGtleSkKCQl9OwoJfQoKCXJldHVybiBmb3JjZTsKfTsKCmZvcmNlKCdTdHJpbmcnLCBTdHJp",
				"bmcsIFsKCSdjaGFyQXQnLCAnY2hhckNvZGVBdCcsICdjb25jYXQnLCAnaW5kZXhPZicsICdsYXN0SW5k",
				"ZXhPZicsICdtYXRjaCcsICdxdW90ZScsICdyZXBsYWNlJywgJ3NlYXJjaCcsCgknc2xpY2UnLCAnc3Bs",
				"aXQnLCAnc3Vic3RyJywgJ3N1YnN0cmluZycsICd0cmltJywgJ3RvTG93ZXJDYXNlJywgJ3RvVXBwZXJD",
				"YXNlJwpdKSgnQXJyYXknLCBBcnJheSwgWwoJJ3BvcCcsICdwdXNoJywgJ3JldmVyc2UnLCAnc2hpZnQn",
				"LCAnc29ydCcsICdzcGxpY2UnLCAndW5zaGlmdCcsICdjb25jYXQnLCAnam9pbicsICdzbGljZScsCgkn",
				"aW5kZXhPZicsICdsYXN0SW5kZXhPZicsICdmaWx0ZXInLCAnZm9yRWFjaCcsICdldmVyeScsICdtYXAn",
				"LCAnc29tZScsICdyZWR1Y2UnLCAncmVkdWNlUmlnaHQnCl0pKCdOdW1iZXInLCBOdW1iZXIsIFsKCSd0",
				"b0V4cG9uZW50aWFsJywgJ3RvRml4ZWQnLCAndG9Mb2NhbGVTdHJpbmcnLCAndG9QcmVjaXNpb24nCl0p",
				"KCdGdW5jdGlvbicsIEZ1bmN0aW9uLCBbCgknYXBwbHknLCAnY2FsbCcsICdiaW5kJwpdKSgnUmVnRXhw",
				"JywgUmVnRXhwLCBbCgknZXhlYycsICd0ZXN0JwpdKSgnT2JqZWN0JywgT2JqZWN0LCBbCgknY3JlYXRl",
				"JywgJ2RlZmluZVByb3BlcnR5JywgJ2RlZmluZVByb3BlcnRpZXMnLCAna2V5cycsCgknZ2V0UHJvdG90",
				"eXBlT2YnLCAnZ2V0T3duUHJvcGVydHlEZXNjcmlwdG9yJywgJ2dldE93blByb3BlcnR5TmFtZXMnLAoJ",
				"J3ByZXZlbnRFeHRlbnNpb25zJywgJ2lzRXh0ZW5zaWJsZScsICdzZWFsJywgJ2lzU2VhbGVkJywgJ2Zy",
				"ZWV6ZScsICdpc0Zyb3plbicKXSkoJ0RhdGUnLCBEYXRlLCBbJ25vdyddKTsKCk9iamVjdC5leHRlbmQg",
				"PSBleHRlbmQub3ZlcmxvYWRTZXR0ZXIoKTsKCkRhdGUuZXh0ZW5kKCdub3cnLCBmdW5jdGlvbigpewoJ",
				"cmV0dXJuICsobmV3IERhdGUpOwp9KTsKCm5ldyBUeXBlKCdCb29sZWFuJywgQm9vbGVhbik7CgovLyBm",
				"aXhlcyBOYU4gcmV0dXJuaW5nIGFzIE51bWJlcgoKTnVtYmVyLnByb3RvdHlwZS4kZmFtaWx5ID0gZnVu",
				"Y3Rpb24oKXsKCXJldHVybiBpc0Zpbml0ZSh0aGlzKSA/ICdudW1iZXInIDogJ251bGwnOwp9LmhpZGUo",
				"KTsKCi8vIE51bWJlci5yYW5kb20KCk51bWJlci5leHRlbmQoJ3JhbmRvbScsIGZ1bmN0aW9uKG1pbiwg",
				"bWF4KXsKCXJldHVybiBNYXRoLmZsb29yKE1hdGgucmFuZG9tKCkgKiAobWF4IC0gbWluICsgMSkgKyBt",
				"aW4pOwp9KTsKCi8vIGZvckVhY2gsIGVhY2gKCnZhciBoYXNPd25Qcm9wZXJ0eSA9IE9iamVjdC5wcm90",
				"b3R5cGUuaGFzT3duUHJvcGVydHk7Ck9iamVjdC5leHRlbmQoJ2ZvckVhY2gnLCBmdW5jdGlvbihvYmpl",
				"Y3QsIGZuLCBiaW5kKXsKCWZvciAodmFyIGtleSBpbiBvYmplY3QpewoJCWlmIChoYXNPd25Qcm9wZXJ0",
				"eS5jYWxsKG9iamVjdCwga2V5KSkgZm4uY2FsbChiaW5kLCBvYmplY3Rba2V5XSwga2V5LCBvYmplY3Qp",
				"OwoJfQp9KTsKCk9iamVjdC5lYWNoID0gT2JqZWN0LmZvckVhY2g7CgpBcnJheS5pbXBsZW1lbnQoewoK",
				"CWZvckVhY2g6IGZ1bmN0aW9uKGZuLCBiaW5kKXsKCQlmb3IgKHZhciBpID0gMCwgbCA9IHRoaXMubGVu",
				"Z3RoOyBpIDwgbDsgaSsrKXsKCQkJaWYgKGkgaW4gdGhpcykgZm4uY2FsbChiaW5kLCB0aGlzW2ldLCBp",
				"LCB0aGlzKTsKCQl9Cgl9LAoKCWVhY2g6IGZ1bmN0aW9uKGZuLCBiaW5kKXsKCQlBcnJheS5mb3JFYWNo",
				"KHRoaXMsIGZuLCBiaW5kKTsKCQlyZXR1cm4gdGhpczsKCX0KCn0pOwoKLy8gQXJyYXkgJiBPYmplY3Qg",
				"Y2xvbmluZywgT2JqZWN0IG1lcmdpbmcgYW5kIGFwcGVuZGluZwoKdmFyIGNsb25lT2YgPSBmdW5jdGlv",
				"bihpdGVtKXsKCXN3aXRjaCAodHlwZU9mKGl0ZW0pKXsKCQljYXNlICdhcnJheSc6IHJldHVybiBpdGVt",
				"LmNsb25lKCk7CgkJY2FzZSAnb2JqZWN0JzogcmV0dXJuIE9iamVjdC5jbG9uZShpdGVtKTsKCQlkZWZh",
				"dWx0OiByZXR1cm4gaXRlbTsKCX0KfTsKCkFycmF5LmltcGxlbWVudCgnY2xvbmUnLCBmdW5jdGlvbigp",
				"ewoJdmFyIGkgPSB0aGlzLmxlbmd0aCwgY2xvbmUgPSBuZXcgQXJyYXkoaSk7Cgl3aGlsZSAoaS0tKSBj",
				"bG9uZVtpXSA9IGNsb25lT2YodGhpc1tpXSk7CglyZXR1cm4gY2xvbmU7Cn0pOwoKdmFyIG1lcmdlT25l",
				"ID0gZnVuY3Rpb24oc291cmNlLCBrZXksIGN1cnJlbnQpewoJc3dpdGNoICh0eXBlT2YoY3VycmVudCkp",
				"ewoJCWNhc2UgJ29iamVjdCc6CgkJCWlmICh0eXBlT2Yoc291cmNlW2tleV0pID09ICdvYmplY3QnKSBP",
				"YmplY3QubWVyZ2Uoc291cmNlW2tleV0sIGN1cnJlbnQpOwoJCQllbHNlIHNvdXJjZVtrZXldID0gT2Jq",
				"ZWN0LmNsb25lKGN1cnJlbnQpOwoJCWJyZWFrOwoJCWNhc2UgJ2FycmF5Jzogc291cmNlW2tleV0gPSBj",
				"dXJyZW50LmNsb25lKCk7IGJyZWFrOwoJCWRlZmF1bHQ6IHNvdXJjZVtrZXldID0gY3VycmVudDsKCX0K",
				"CXJldHVybiBzb3VyY2U7Cn07CgpPYmplY3QuZXh0ZW5kKHsKCgltZXJnZTogZnVuY3Rpb24oc291cmNl",
				"LCBrLCB2KXsKCQlpZiAodHlwZU9mKGspID09ICdzdHJpbmcnKSByZXR1cm4gbWVyZ2VPbmUoc291cmNl",
				"LCBrLCB2KTsKCQlmb3IgKHZhciBpID0gMSwgbCA9IGFyZ3VtZW50cy5sZW5ndGg7IGkgPCBsOyBpKysp",
				"ewoJCQl2YXIgb2JqZWN0ID0gYXJndW1lbnRzW2ldOwoJCQlmb3IgKHZhciBrZXkgaW4gb2JqZWN0KSBt",
				"ZXJnZU9uZShzb3VyY2UsIGtleSwgb2JqZWN0W2tleV0pOwoJCX0KCQlyZXR1cm4gc291cmNlOwoJfSwK",
				"CgljbG9uZTogZnVuY3Rpb24ob2JqZWN0KXsKCQl2YXIgY2xvbmUgPSB7fTsKCQlmb3IgKHZhciBrZXkg",
				"aW4gb2JqZWN0KSBjbG9uZVtrZXldID0gY2xvbmVPZihvYmplY3Rba2V5XSk7CgkJcmV0dXJuIGNsb25l",
				"OwoJfSwKCglhcHBlbmQ6IGZ1bmN0aW9uKG9yaWdpbmFsKXsKCQlmb3IgKHZhciBpID0gMSwgbCA9IGFy",
				"Z3VtZW50cy5sZW5ndGg7IGkgPCBsOyBpKyspewoJCQl2YXIgZXh0ZW5kZWQgPSBhcmd1bWVudHNbaV0g",
				"fHwge307CgkJCWZvciAodmFyIGtleSBpbiBleHRlbmRlZCkgb3JpZ2luYWxba2V5XSA9IGV4dGVuZGVk",
				"W2tleV07CgkJfQoJCXJldHVybiBvcmlnaW5hbDsKCX0KCn0pOwoKLy8gT2JqZWN0LWxlc3MgdHlwZXMK",
				"ClsnT2JqZWN0JywgJ1doaXRlU3BhY2UnLCAnVGV4dE5vZGUnLCAnQ29sbGVjdGlvbicsICdBcmd1bWVu",
				"dHMnXS5lYWNoKGZ1bmN0aW9uKG5hbWUpewoJbmV3IFR5cGUobmFtZSk7Cn0pOwoKLy8gVW5pcXVlIElE",
				"Cgp2YXIgVUlEID0gRGF0ZS5ub3coKTsKClN0cmluZy5leHRlbmQoJ3VuaXF1ZUlEJywgZnVuY3Rpb24o",
				"KXsKCXJldHVybiAoVUlEKyspLnRvU3RyaW5nKDM2KTsKfSk7CgovLzwxLjJjb21wYXQ+Cgp2YXIgSGFz",
				"aCA9IHRoaXMuSGFzaCA9IG5ldyBUeXBlKCdIYXNoJywgZnVuY3Rpb24ob2JqZWN0KXsKCWlmICh0eXBl",
				"T2Yob2JqZWN0KSA9PSAnaGFzaCcpIG9iamVjdCA9IE9iamVjdC5jbG9uZShvYmplY3QuZ2V0Q2xlYW4o",
				"KSk7Cglmb3IgKHZhciBrZXkgaW4gb2JqZWN0KSB0aGlzW2tleV0gPSBvYmplY3Rba2V5XTsKCXJldHVy",
				"biB0aGlzOwp9KTsKCkhhc2guaW1wbGVtZW50KHsKCglmb3JFYWNoOiBmdW5jdGlvbihmbiwgYmluZCl7",
				"CgkJT2JqZWN0LmZvckVhY2godGhpcywgZm4sIGJpbmQpOwoJfSwKCglnZXRDbGVhbjogZnVuY3Rpb24o",
				"KXsKCQl2YXIgY2xlYW4gPSB7fTsKCQlmb3IgKHZhciBrZXkgaW4gdGhpcyl7CgkJCWlmICh0aGlzLmhh",
				"c093blByb3BlcnR5KGtleSkpIGNsZWFuW2tleV0gPSB0aGlzW2tleV07CgkJfQoJCXJldHVybiBjbGVh",
				"bjsKCX0sCgoJZ2V0TGVuZ3RoOiBmdW5jdGlvbigpewoJCXZhciBsZW5ndGggPSAwOwoJCWZvciAodmFy",
				"IGtleSBpbiB0aGlzKXsKCQkJaWYgKHRoaXMuaGFzT3duUHJvcGVydHkoa2V5KSkgbGVuZ3RoKys7CgkJ",
				"fQoJCXJldHVybiBsZW5ndGg7Cgl9Cgp9KTsKCkhhc2guYWxpYXMoJ2VhY2gnLCAnZm9yRWFjaCcpOwoK",
				"T2JqZWN0LnR5cGUgPSBUeXBlLmlzT2JqZWN0OwoKdmFyIE5hdGl2ZSA9IHRoaXMuTmF0aXZlID0gZnVu",
				"Y3Rpb24ocHJvcGVydGllcyl7CglyZXR1cm4gbmV3IFR5cGUocHJvcGVydGllcy5uYW1lLCBwcm9wZXJ0",
				"aWVzLmluaXRpYWxpemUpOwp9OwoKTmF0aXZlLnR5cGUgPSBUeXBlLnR5cGU7CgpOYXRpdmUuaW1wbGVt",
				"ZW50ID0gZnVuY3Rpb24ob2JqZWN0cywgbWV0aG9kcyl7Cglmb3IgKHZhciBpID0gMDsgaSA8IG9iamVj",
				"dHMubGVuZ3RoOyBpKyspIG9iamVjdHNbaV0uaW1wbGVtZW50KG1ldGhvZHMpOwoJcmV0dXJuIE5hdGl2",
				"ZTsKfTsKCnZhciBhcnJheVR5cGUgPSBBcnJheS50eXBlOwpBcnJheS50eXBlID0gZnVuY3Rpb24oaXRl",
				"bSl7CglyZXR1cm4gaW5zdGFuY2VPZihpdGVtLCBBcnJheSkgfHwgYXJyYXlUeXBlKGl0ZW0pOwp9OwoK",
				"dGhpcy4kQSA9IGZ1bmN0aW9uKGl0ZW0pewoJcmV0dXJuIEFycmF5LmZyb20oaXRlbSkuc2xpY2UoKTsK",
				"fTsKCnRoaXMuJGFyZ3VtZW50cyA9IGZ1bmN0aW9uKGkpewoJcmV0dXJuIGZ1bmN0aW9uKCl7CgkJcmV0",
				"dXJuIGFyZ3VtZW50c1tpXTsKCX07Cn07Cgp0aGlzLiRjaGsgPSBmdW5jdGlvbihvYmopewoJcmV0dXJu",
				"ICEhKG9iaiB8fCBvYmogPT09IDApOwp9OwoKdGhpcy4kY2xlYXIgPSBmdW5jdGlvbih0aW1lcil7Cglj",
				"bGVhclRpbWVvdXQodGltZXIpOwoJY2xlYXJJbnRlcnZhbCh0aW1lcik7CglyZXR1cm4gbnVsbDsKfTsK",
				"CnRoaXMuJGRlZmluZWQgPSBmdW5jdGlvbihvYmopewoJcmV0dXJuIChvYmogIT0gbnVsbCk7Cn07Cgp0",
				"aGlzLiRlYWNoID0gZnVuY3Rpb24oaXRlcmFibGUsIGZuLCBiaW5kKXsKCXZhciB0eXBlID0gdHlwZU9m",
				"KGl0ZXJhYmxlKTsKCSgodHlwZSA9PSAnYXJndW1lbnRzJyB8fCB0eXBlID09ICdjb2xsZWN0aW9uJyB8",
				"fCB0eXBlID09ICdhcnJheScgfHwgdHlwZSA9PSAnZWxlbWVudHMnKSA/IEFycmF5IDogT2JqZWN0KS5l",
				"YWNoKGl0ZXJhYmxlLCBmbiwgYmluZCk7Cn07Cgp0aGlzLiRlbXB0eSA9IGZ1bmN0aW9uKCl7fTsKCnRo",
				"aXMuJGV4dGVuZCA9IGZ1bmN0aW9uKG9yaWdpbmFsLCBleHRlbmRlZCl7CglyZXR1cm4gT2JqZWN0LmFw",
				"cGVuZChvcmlnaW5hbCwgZXh0ZW5kZWQpOwp9OwoKdGhpcy4kSCA9IGZ1bmN0aW9uKG9iamVjdCl7Cgly",
				"ZXR1cm4gbmV3IEhhc2gob2JqZWN0KTsKfTsKCnRoaXMuJG1lcmdlID0gZnVuY3Rpb24oKXsKCXZhciBh",
				"cmdzID0gQXJyYXkuc2xpY2UoYXJndW1lbnRzKTsKCWFyZ3MudW5zaGlmdCh7fSk7CglyZXR1cm4gT2Jq",
				"ZWN0Lm1lcmdlLmFwcGx5KG51bGwsIGFyZ3MpOwp9OwoKdGhpcy4kbGFtYmRhID0gRnVuY3Rpb24uZnJv",
				"bTsKdGhpcy4kbWl4aW4gPSBPYmplY3QubWVyZ2U7CnRoaXMuJHJhbmRvbSA9IE51bWJlci5yYW5kb207",
				"CnRoaXMuJHNwbGF0ID0gQXJyYXkuZnJvbTsKdGhpcy4kdGltZSA9IERhdGUubm93OwoKdGhpcy4kdHlw",
				"ZSA9IGZ1bmN0aW9uKG9iamVjdCl7Cgl2YXIgdHlwZSA9IHR5cGVPZihvYmplY3QpOwoJaWYgKHR5cGUg",
				"PT0gJ2VsZW1lbnRzJykgcmV0dXJuICdhcnJheSc7CglyZXR1cm4gKHR5cGUgPT0gJ251bGwnKSA/IGZh",
				"bHNlIDogdHlwZTsKfTsKCnRoaXMuJHVubGluayA9IGZ1bmN0aW9uKG9iamVjdCl7Cglzd2l0Y2ggKHR5",
				"cGVPZihvYmplY3QpKXsKCQljYXNlICdvYmplY3QnOiByZXR1cm4gT2JqZWN0LmNsb25lKG9iamVjdCk7",
				"CgkJY2FzZSAnYXJyYXknOiByZXR1cm4gQXJyYXkuY2xvbmUob2JqZWN0KTsKCQljYXNlICdoYXNoJzog",
				"cmV0dXJuIG5ldyBIYXNoKG9iamVjdCk7CgkJZGVmYXVsdDogcmV0dXJuIG9iamVjdDsKCX0KfTsKCi8v",
				"PC8xLjJjb21wYXQ+Cgp9KSgpOwoKCi8qCi0tLQoKbmFtZTogQXJyYXkKCmRlc2NyaXB0aW9uOiBDb250",
				"YWlucyBBcnJheSBQcm90b3R5cGVzIGxpa2UgZWFjaCwgY29udGFpbnMsIGFuZCBlcmFzZS4KCmxpY2Vu",
				"c2U6IE1JVC1zdHlsZSBsaWNlbnNlLgoKcmVxdWlyZXM6IFR5cGUKCnByb3ZpZGVzOiBBcnJheQoKLi4u",
				"CiovCgpBcnJheS5pbXBsZW1lbnQoewoKCS8qPCFFUzU+Ki8KCWV2ZXJ5OiBmdW5jdGlvbihmbiwgYmlu",
				"ZCl7CgkJZm9yICh2YXIgaSA9IDAsIGwgPSB0aGlzLmxlbmd0aCA+Pj4gMDsgaSA8IGw7IGkrKyl7CgkJ",
				"CWlmICgoaSBpbiB0aGlzKSAmJiAhZm4uY2FsbChiaW5kLCB0aGlzW2ldLCBpLCB0aGlzKSkgcmV0dXJu",
				"IGZhbHNlOwoJCX0KCQlyZXR1cm4gdHJ1ZTsKCX0sCgoJZmlsdGVyOiBmdW5jdGlvbihmbiwgYmluZCl7",
				"CgkJdmFyIHJlc3VsdHMgPSBbXTsKCQlmb3IgKHZhciB2YWx1ZSwgaSA9IDAsIGwgPSB0aGlzLmxlbmd0",
				"aCA+Pj4gMDsgaSA8IGw7IGkrKykgaWYgKGkgaW4gdGhpcyl7CgkJCXZhbHVlID0gdGhpc1tpXTsKCQkJ",
				"aWYgKGZuLmNhbGwoYmluZCwgdmFsdWUsIGksIHRoaXMpKSByZXN1bHRzLnB1c2godmFsdWUpOwoJCX0K",
				"CQlyZXR1cm4gcmVzdWx0czsKCX0sCgoJaW5kZXhPZjogZnVuY3Rpb24oaXRlbSwgZnJvbSl7CgkJdmFy",
				"IGxlbmd0aCA9IHRoaXMubGVuZ3RoID4+PiAwOwoJCWZvciAodmFyIGkgPSAoZnJvbSA8IDApID8gTWF0",
				"aC5tYXgoMCwgbGVuZ3RoICsgZnJvbSkgOiBmcm9tIHx8IDA7IGkgPCBsZW5ndGg7IGkrKyl7CgkJCWlm",
				"ICh0aGlzW2ldID09PSBpdGVtKSByZXR1cm4gaTsKCQl9CgkJcmV0dXJuIC0xOwoJfSwKCgltYXA6IGZ1",
				"bmN0aW9uKGZuLCBiaW5kKXsKCQl2YXIgbGVuZ3RoID0gdGhpcy5sZW5ndGggPj4+IDAsIHJlc3VsdHMg",
				"PSBBcnJheShsZW5ndGgpOwoJCWZvciAodmFyIGkgPSAwOyBpIDwgbGVuZ3RoOyBpKyspewoJCQlpZiAo",
				"aSBpbiB0aGlzKSByZXN1bHRzW2ldID0gZm4uY2FsbChiaW5kLCB0aGlzW2ldLCBpLCB0aGlzKTsKCQl9",
				"CgkJcmV0dXJuIHJlc3VsdHM7Cgl9LAoKCXNvbWU6IGZ1bmN0aW9uKGZuLCBiaW5kKXsKCQlmb3IgKHZh",
				"ciBpID0gMCwgbCA9IHRoaXMubGVuZ3RoID4+PiAwOyBpIDwgbDsgaSsrKXsKCQkJaWYgKChpIGluIHRo",
				"aXMpICYmIGZuLmNhbGwoYmluZCwgdGhpc1tpXSwgaSwgdGhpcykpIHJldHVybiB0cnVlOwoJCX0KCQly",
				"ZXR1cm4gZmFsc2U7Cgl9LAoJLyo8LyFFUzU+Ki8KCgljbGVhbjogZnVuY3Rpb24oKXsKCQlyZXR1cm4g",
				"dGhpcy5maWx0ZXIoZnVuY3Rpb24oaXRlbSl7CgkJCXJldHVybiBpdGVtICE9IG51bGw7CgkJfSk7Cgl9",
				"LAoKCWludm9rZTogZnVuY3Rpb24obWV0aG9kTmFtZSl7CgkJdmFyIGFyZ3MgPSBBcnJheS5zbGljZShh",
				"cmd1bWVudHMsIDEpOwoJCXJldHVybiB0aGlzLm1hcChmdW5jdGlvbihpdGVtKXsKCQkJcmV0dXJuIGl0",
				"ZW1bbWV0aG9kTmFtZV0uYXBwbHkoaXRlbSwgYXJncyk7CgkJfSk7Cgl9LAoKCWFzc29jaWF0ZTogZnVu",
				"Y3Rpb24oa2V5cyl7CgkJdmFyIG9iaiA9IHt9LCBsZW5ndGggPSBNYXRoLm1pbih0aGlzLmxlbmd0aCwg",
				"a2V5cy5sZW5ndGgpOwoJCWZvciAodmFyIGkgPSAwOyBpIDwgbGVuZ3RoOyBpKyspIG9ialtrZXlzW2ld",
				"XSA9IHRoaXNbaV07CgkJcmV0dXJuIG9iajsKCX0sCgoJbGluazogZnVuY3Rpb24ob2JqZWN0KXsKCQl2",
				"YXIgcmVzdWx0ID0ge307CgkJZm9yICh2YXIgaSA9IDAsIGwgPSB0aGlzLmxlbmd0aDsgaSA8IGw7IGkr",
				"Kyl7CgkJCWZvciAodmFyIGtleSBpbiBvYmplY3QpewoJCQkJaWYgKG9iamVjdFtrZXldKHRoaXNbaV0p",
				"KXsKCQkJCQlyZXN1bHRba2V5XSA9IHRoaXNbaV07CgkJCQkJZGVsZXRlIG9iamVjdFtrZXldOwoJCQkJ",
				"CWJyZWFrOwoJCQkJfQoJCQl9CgkJfQoJCXJldHVybiByZXN1bHQ7Cgl9LAoKCWNvbnRhaW5zOiBmdW5j",
				"dGlvbihpdGVtLCBmcm9tKXsKCQlyZXR1cm4gdGhpcy5pbmRleE9mKGl0ZW0sIGZyb20pICE9IC0xOwoJ",
				"fSwKCglhcHBlbmQ6IGZ1bmN0aW9uKGFycmF5KXsKCQl0aGlzLnB1c2guYXBwbHkodGhpcywgYXJyYXkp",
				"OwoJCXJldHVybiB0aGlzOwoJfSwKCglnZXRMYXN0OiBmdW5jdGlvbigpewoJCXJldHVybiAodGhpcy5s",
				"ZW5ndGgpID8gdGhpc1t0aGlzLmxlbmd0aCAtIDFdIDogbnVsbDsKCX0sCgoJZ2V0UmFuZG9tOiBmdW5j",
				"dGlvbigpewoJCXJldHVybiAodGhpcy5sZW5ndGgpID8gdGhpc1tOdW1iZXIucmFuZG9tKDAsIHRoaXMu",
				"bGVuZ3RoIC0gMSldIDogbnVsbDsKCX0sCgoJaW5jbHVkZTogZnVuY3Rpb24oaXRlbSl7CgkJaWYgKCF0",
				"aGlzLmNvbnRhaW5zKGl0ZW0pKSB0aGlzLnB1c2goaXRlbSk7CgkJcmV0dXJuIHRoaXM7Cgl9LAoKCWNv",
				"bWJpbmU6IGZ1bmN0aW9uKGFycmF5KXsKCQlmb3IgKHZhciBpID0gMCwgbCA9IGFycmF5Lmxlbmd0aDsg",
				"aSA8IGw7IGkrKykgdGhpcy5pbmNsdWRlKGFycmF5W2ldKTsKCQlyZXR1cm4gdGhpczsKCX0sCgoJZXJh",
				"c2U6IGZ1bmN0aW9uKGl0ZW0pewoJCWZvciAodmFyIGkgPSB0aGlzLmxlbmd0aDsgaS0tOyl7CgkJCWlm",
				"ICh0aGlzW2ldID09PSBpdGVtKSB0aGlzLnNwbGljZShpLCAxKTsKCQl9CgkJcmV0dXJuIHRoaXM7Cgl9",
				"LAoKCWVtcHR5OiBmdW5jdGlvbigpewoJCXRoaXMubGVuZ3RoID0gMDsKCQlyZXR1cm4gdGhpczsKCX0s",
				"CgoJZmxhdHRlbjogZnVuY3Rpb24oKXsKCQl2YXIgYXJyYXkgPSBbXTsKCQlmb3IgKHZhciBpID0gMCwg",
				"bCA9IHRoaXMubGVuZ3RoOyBpIDwgbDsgaSsrKXsKCQkJdmFyIHR5cGUgPSB0eXBlT2YodGhpc1tpXSk7",
				"CgkJCWlmICh0eXBlID09ICdudWxsJykgY29udGludWU7CgkJCWFycmF5ID0gYXJyYXkuY29uY2F0KCh0",
				"eXBlID09ICdhcnJheScgfHwgdHlwZSA9PSAnY29sbGVjdGlvbicgfHwgdHlwZSA9PSAnYXJndW1lbnRz",
				"JyB8fCBpbnN0YW5jZU9mKHRoaXNbaV0sIEFycmF5KSkgPyBBcnJheS5mbGF0dGVuKHRoaXNbaV0pIDog",
				"dGhpc1tpXSk7CgkJfQoJCXJldHVybiBhcnJheTsKCX0sCgoJcGljazogZnVuY3Rpb24oKXsKCQlmb3Ig",
				"KHZhciBpID0gMCwgbCA9IHRoaXMubGVuZ3RoOyBpIDwgbDsgaSsrKXsKCQkJaWYgKHRoaXNbaV0gIT0g",
				"bnVsbCkgcmV0dXJuIHRoaXNbaV07CgkJfQoJCXJldHVybiBudWxsOwoJfSwKCgloZXhUb1JnYjogZnVu",
				"Y3Rpb24oYXJyYXkpewoJCWlmICh0aGlzLmxlbmd0aCAhPSAzKSByZXR1cm4gbnVsbDsKCQl2YXIgcmdi",
				"ID0gdGhpcy5tYXAoZnVuY3Rpb24odmFsdWUpewoJCQlpZiAodmFsdWUubGVuZ3RoID09IDEpIHZhbHVl",
				"ICs9IHZhbHVlOwoJCQlyZXR1cm4gdmFsdWUudG9JbnQoMTYpOwoJCX0pOwoJCXJldHVybiAoYXJyYXkp",
				"ID8gcmdiIDogJ3JnYignICsgcmdiICsgJyknOwoJfSwKCglyZ2JUb0hleDogZnVuY3Rpb24oYXJyYXkp",
				"ewoJCWlmICh0aGlzLmxlbmd0aCA8IDMpIHJldHVybiBudWxsOwoJCWlmICh0aGlzLmxlbmd0aCA9PSA0",
				"ICYmIHRoaXNbM10gPT0gMCAmJiAhYXJyYXkpIHJldHVybiAndHJhbnNwYXJlbnQnOwoJCXZhciBoZXgg",
				"PSBbXTsKCQlmb3IgKHZhciBpID0gMDsgaSA8IDM7IGkrKyl7CgkJCXZhciBiaXQgPSAodGhpc1tpXSAt",
				"IDApLnRvU3RyaW5nKDE2KTsKCQkJaGV4LnB1c2goKGJpdC5sZW5ndGggPT0gMSkgPyAnMCcgKyBiaXQg",
				"OiBiaXQpOwoJCX0KCQlyZXR1cm4gKGFycmF5KSA/IGhleCA6ICcjJyArIGhleC5qb2luKCcnKTsKCX0K",
				"Cn0pOwoKLy88MS4yY29tcGF0PgoKQXJyYXkuYWxpYXMoJ2V4dGVuZCcsICdhcHBlbmQnKTsKCnZhciAk",
				"cGljayA9IGZ1bmN0aW9uKCl7CglyZXR1cm4gQXJyYXkuZnJvbShhcmd1bWVudHMpLnBpY2soKTsKfTsK",
				"Ci8vPC8xLjJjb21wYXQ+CgoKLyoKLS0tCgpuYW1lOiBTdHJpbmcKCmRlc2NyaXB0aW9uOiBDb250YWlu",
				"cyBTdHJpbmcgUHJvdG90eXBlcyBsaWtlIGNhbWVsQ2FzZSwgY2FwaXRhbGl6ZSwgdGVzdCwgYW5kIHRv",
				"SW50LgoKbGljZW5zZTogTUlULXN0eWxlIGxpY2Vuc2UuCgpyZXF1aXJlczogVHlwZQoKcHJvdmlkZXM6",
				"IFN0cmluZwoKLi4uCiovCgpTdHJpbmcuaW1wbGVtZW50KHsKCgl0ZXN0OiBmdW5jdGlvbihyZWdleCwg",
				"cGFyYW1zKXsKCQlyZXR1cm4gKCh0eXBlT2YocmVnZXgpID09ICdyZWdleHAnKSA/IHJlZ2V4IDogbmV3",
				"IFJlZ0V4cCgnJyArIHJlZ2V4LCBwYXJhbXMpKS50ZXN0KHRoaXMpOwoJfSwKCgljb250YWluczogZnVu",
				"Y3Rpb24oc3RyaW5nLCBzZXBhcmF0b3IpewoJCXJldHVybiAoc2VwYXJhdG9yKSA/IChzZXBhcmF0b3Ig",
				"KyB0aGlzICsgc2VwYXJhdG9yKS5pbmRleE9mKHNlcGFyYXRvciArIHN0cmluZyArIHNlcGFyYXRvcikg",
				"PiAtMSA6IFN0cmluZyh0aGlzKS5pbmRleE9mKHN0cmluZykgPiAtMTsKCX0sCgoJdHJpbTogZnVuY3Rp",
				"b24oKXsKCQlyZXR1cm4gU3RyaW5nKHRoaXMpLnJlcGxhY2UoL15ccyt8XHMrJC9nLCAnJyk7Cgl9LAoK",
				"CWNsZWFuOiBmdW5jdGlvbigpewoJCXJldHVybiBTdHJpbmcodGhpcykucmVwbGFjZSgvXHMrL2csICcg",
				"JykudHJpbSgpOwoJfSwKCgljYW1lbENhc2U6IGZ1bmN0aW9uKCl7CgkJcmV0dXJuIFN0cmluZyh0aGlz",
				"KS5yZXBsYWNlKC8tXEQvZywgZnVuY3Rpb24obWF0Y2gpewoJCQlyZXR1cm4gbWF0Y2guY2hhckF0KDEp",
				"LnRvVXBwZXJDYXNlKCk7CgkJfSk7Cgl9LAoKCWh5cGhlbmF0ZTogZnVuY3Rpb24oKXsKCQlyZXR1cm4g",
				"U3RyaW5nKHRoaXMpLnJlcGxhY2UoL1tBLVpdL2csIGZ1bmN0aW9uKG1hdGNoKXsKCQkJcmV0dXJuICgn",
				"LScgKyBtYXRjaC5jaGFyQXQoMCkudG9Mb3dlckNhc2UoKSk7CgkJfSk7Cgl9LAoKCWNhcGl0YWxpemU6",
				"IGZ1bmN0aW9uKCl7CgkJcmV0dXJuIFN0cmluZyh0aGlzKS5yZXBsYWNlKC9cYlthLXpdL2csIGZ1bmN0",
				"aW9uKG1hdGNoKXsKCQkJcmV0dXJuIG1hdGNoLnRvVXBwZXJDYXNlKCk7CgkJfSk7Cgl9LAoKCWVzY2Fw",
				"ZVJlZ0V4cDogZnVuY3Rpb24oKXsKCQlyZXR1cm4gU3RyaW5nKHRoaXMpLnJlcGxhY2UoLyhbLS4qKz9e",
				"JHt9KCl8W1xdXC9cXF0pL2csICdcXCQxJyk7Cgl9LAoKCXRvSW50OiBmdW5jdGlvbihiYXNlKXsKCQly",
				"ZXR1cm4gcGFyc2VJbnQodGhpcywgYmFzZSB8fCAxMCk7Cgl9LAoKCXRvRmxvYXQ6IGZ1bmN0aW9uKCl7",
				"CgkJcmV0dXJuIHBhcnNlRmxvYXQodGhpcyk7Cgl9LAoKCWhleFRvUmdiOiBmdW5jdGlvbihhcnJheSl7",
				"CgkJdmFyIGhleCA9IFN0cmluZyh0aGlzKS5tYXRjaCgvXiM/KFx3ezEsMn0pKFx3ezEsMn0pKFx3ezEs",
				"Mn0pJC8pOwoJCXJldHVybiAoaGV4KSA/IGhleC5zbGljZSgxKS5oZXhUb1JnYihhcnJheSkgOiBudWxs",
				"OwoJfSwKCglyZ2JUb0hleDogZnVuY3Rpb24oYXJyYXkpewoJCXZhciByZ2IgPSBTdHJpbmcodGhpcyku",
				"bWF0Y2goL1xkezEsM30vZyk7CgkJcmV0dXJuIChyZ2IpID8gcmdiLnJnYlRvSGV4KGFycmF5KSA6IG51",
				"bGw7Cgl9LAoKCXN1YnN0aXR1dGU6IGZ1bmN0aW9uKG9iamVjdCwgcmVnZXhwKXsKCQlyZXR1cm4gU3Ry",
				"aW5nKHRoaXMpLnJlcGxhY2UocmVnZXhwIHx8ICgvXFw/XHsoW157fV0rKVx9L2cpLCBmdW5jdGlvbiht",
				"YXRjaCwgbmFtZSl7CgkJCWlmIChtYXRjaC5jaGFyQXQoMCkgPT0gJ1xcJykgcmV0dXJuIG1hdGNoLnNs",
				"aWNlKDEpOwoJCQlyZXR1cm4gKG9iamVjdFtuYW1lXSAhPSBudWxsKSA/IG9iamVjdFtuYW1lXSA6ICcn",
				"OwoJCX0pOwoJfQoKfSk7CgoKLyoKLS0tCgpuYW1lOiBOdW1iZXIKCmRlc2NyaXB0aW9uOiBDb250YWlu",
				"cyBOdW1iZXIgUHJvdG90eXBlcyBsaWtlIGxpbWl0LCByb3VuZCwgdGltZXMsIGFuZCBjZWlsLgoKbGlj",
				"ZW5zZTogTUlULXN0eWxlIGxpY2Vuc2UuCgpyZXF1aXJlczogVHlwZQoKcHJvdmlkZXM6IE51bWJlcgoK",
				"Li4uCiovCgpOdW1iZXIuaW1wbGVtZW50KHsKCglsaW1pdDogZnVuY3Rpb24obWluLCBtYXgpewoJCXJl",
				"dHVybiBNYXRoLm1pbihtYXgsIE1hdGgubWF4KG1pbiwgdGhpcykpOwoJfSwKCglyb3VuZDogZnVuY3Rp",
				"b24ocHJlY2lzaW9uKXsKCQlwcmVjaXNpb24gPSBNYXRoLnBvdygxMCwgcHJlY2lzaW9uIHx8IDApLnRv",
				"Rml4ZWQocHJlY2lzaW9uIDwgMCA/IC1wcmVjaXNpb24gOiAwKTsKCQlyZXR1cm4gTWF0aC5yb3VuZCh0",
				"aGlzICogcHJlY2lzaW9uKSAvIHByZWNpc2lvbjsKCX0sCgoJdGltZXM6IGZ1bmN0aW9uKGZuLCBiaW5k",
				"KXsKCQlmb3IgKHZhciBpID0gMDsgaSA8IHRoaXM7IGkrKykgZm4uY2FsbChiaW5kLCBpLCB0aGlzKTsK",
				"CX0sCgoJdG9GbG9hdDogZnVuY3Rpb24oKXsKCQlyZXR1cm4gcGFyc2VGbG9hdCh0aGlzKTsKCX0sCgoJ",
				"dG9JbnQ6IGZ1bmN0aW9uKGJhc2UpewoJCXJldHVybiBwYXJzZUludCh0aGlzLCBiYXNlIHx8IDEwKTsK",
				"CX0KCn0pOwoKTnVtYmVyLmFsaWFzKCdlYWNoJywgJ3RpbWVzJyk7CgooZnVuY3Rpb24obWF0aCl7Cgl2",
				"YXIgbWV0aG9kcyA9IHt9OwoJbWF0aC5lYWNoKGZ1bmN0aW9uKG5hbWUpewoJCWlmICghTnVtYmVyW25h",
				"bWVdKSBtZXRob2RzW25hbWVdID0gZnVuY3Rpb24oKXsKCQkJcmV0dXJuIE1hdGhbbmFtZV0uYXBwbHko",
				"bnVsbCwgW3RoaXNdLmNvbmNhdChBcnJheS5mcm9tKGFyZ3VtZW50cykpKTsKCQl9OwoJfSk7CglOdW1i",
				"ZXIuaW1wbGVtZW50KG1ldGhvZHMpOwp9KShbJ2FicycsICdhY29zJywgJ2FzaW4nLCAnYXRhbicsICdh",
				"dGFuMicsICdjZWlsJywgJ2NvcycsICdleHAnLCAnZmxvb3InLCAnbG9nJywgJ21heCcsICdtaW4nLCAn",
				"cG93JywgJ3NpbicsICdzcXJ0JywgJ3RhbiddKTsKCgovKgotLS0KCm5hbWU6IEZ1bmN0aW9uCgpkZXNj",
				"cmlwdGlvbjogQ29udGFpbnMgRnVuY3Rpb24gUHJvdG90eXBlcyBsaWtlIGNyZWF0ZSwgYmluZCwgcGFz",
				"cywgYW5kIGRlbGF5LgoKbGljZW5zZTogTUlULXN0eWxlIGxpY2Vuc2UuCgpyZXF1aXJlczogVHlwZQoK",
				"cHJvdmlkZXM6IEZ1bmN0aW9uCgouLi4KKi8KCkZ1bmN0aW9uLmV4dGVuZCh7CgoJYXR0ZW1wdDogZnVu",
				"Y3Rpb24oKXsKCQlmb3IgKHZhciBpID0gMCwgbCA9IGFyZ3VtZW50cy5sZW5ndGg7IGkgPCBsOyBpKysp",
				"ewoJCQl0cnkgewoJCQkJcmV0dXJuIGFyZ3VtZW50c1tpXSgpOwoJCQl9IGNhdGNoIChlKXt9CgkJfQoJ",
				"CXJldHVybiBudWxsOwoJfQoKfSk7CgpGdW5jdGlvbi5pbXBsZW1lbnQoewoKCWF0dGVtcHQ6IGZ1bmN0",
				"aW9uKGFyZ3MsIGJpbmQpewoJCXRyeSB7CgkJCXJldHVybiB0aGlzLmFwcGx5KGJpbmQsIEFycmF5LmZy",
				"b20oYXJncykpOwoJCX0gY2F0Y2ggKGUpe30KCgkJcmV0dXJuIG51bGw7Cgl9LAoKCS8qPCFFUzUtYmlu",
				"ZD4qLwoJYmluZDogZnVuY3Rpb24odGhhdCl7CgkJdmFyIHNlbGYgPSB0aGlzLAoJCQlhcmdzID0gYXJn",
				"dW1lbnRzLmxlbmd0aCA+IDEgPyBBcnJheS5zbGljZShhcmd1bWVudHMsIDEpIDogbnVsbCwKCQkJRiA9",
				"IGZ1bmN0aW9uKCl7fTsKCgkJdmFyIGJvdW5kID0gZnVuY3Rpb24oKXsKCQkJdmFyIGNvbnRleHQgPSB0",
				"aGF0LCBsZW5ndGggPSBhcmd1bWVudHMubGVuZ3RoOwoJCQlpZiAodGhpcyBpbnN0YW5jZW9mIGJvdW5k",
				"KXsKCQkJCUYucHJvdG90eXBlID0gc2VsZi5wcm90b3R5cGU7CgkJCQljb250ZXh0ID0gbmV3IEY7CgkJ",
				"CX0KCQkJdmFyIHJlc3VsdCA9ICghYXJncyAmJiAhbGVuZ3RoKQoJCQkJPyBzZWxmLmNhbGwoY29udGV4",
				"dCkKCQkJCTogc2VsZi5hcHBseShjb250ZXh0LCBhcmdzICYmIGxlbmd0aCA/IGFyZ3MuY29uY2F0KEFy",
				"cmF5LnNsaWNlKGFyZ3VtZW50cykpIDogYXJncyB8fCBhcmd1bWVudHMpOwoJCQlyZXR1cm4gY29udGV4",
				"dCA9PSB0aGF0ID8gcmVzdWx0IDogY29udGV4dDsKCQl9OwoJCXJldHVybiBib3VuZDsKCX0sCgkvKjwv",
				"IUVTNS1iaW5kPiovCgoJcGFzczogZnVuY3Rpb24oYXJncywgYmluZCl7CgkJdmFyIHNlbGYgPSB0aGlz",
				"OwoJCWlmIChhcmdzICE9IG51bGwpIGFyZ3MgPSBBcnJheS5mcm9tKGFyZ3MpOwoJCXJldHVybiBmdW5j",
				"dGlvbigpewoJCQlyZXR1cm4gc2VsZi5hcHBseShiaW5kLCBhcmdzIHx8IGFyZ3VtZW50cyk7CgkJfTsK",
				"CX0sCgoJZGVsYXk6IGZ1bmN0aW9uKGRlbGF5LCBiaW5kLCBhcmdzKXsKCQlyZXR1cm4gc2V0VGltZW91",
				"dCh0aGlzLnBhc3MoKGFyZ3MgPT0gbnVsbCA/IFtdIDogYXJncyksIGJpbmQpLCBkZWxheSk7Cgl9LAoK",
				"CXBlcmlvZGljYWw6IGZ1bmN0aW9uKHBlcmlvZGljYWwsIGJpbmQsIGFyZ3MpewoJCXJldHVybiBzZXRJ",
				"bnRlcnZhbCh0aGlzLnBhc3MoKGFyZ3MgPT0gbnVsbCA/IFtdIDogYXJncyksIGJpbmQpLCBwZXJpb2Rp",
				"Y2FsKTsKCX0KCn0pOwoKLy88MS4yY29tcGF0PgoKZGVsZXRlIEZ1bmN0aW9uLnByb3RvdHlwZS5iaW5k",
				"OwoKRnVuY3Rpb24uaW1wbGVtZW50KHsKCgljcmVhdGU6IGZ1bmN0aW9uKG9wdGlvbnMpewoJCXZhciBz",
				"ZWxmID0gdGhpczsKCQlvcHRpb25zID0gb3B0aW9ucyB8fCB7fTsKCQlyZXR1cm4gZnVuY3Rpb24oZXZl",
				"bnQpewoJCQl2YXIgYXJncyA9IG9wdGlvbnMuYXJndW1lbnRzOwoJCQlhcmdzID0gKGFyZ3MgIT0gbnVs",
				"bCkgPyBBcnJheS5mcm9tKGFyZ3MpIDogQXJyYXkuc2xpY2UoYXJndW1lbnRzLCAob3B0aW9ucy5ldmVu",
				"dCkgPyAxIDogMCk7CgkJCWlmIChvcHRpb25zLmV2ZW50KSBhcmdzID0gW2V2ZW50IHx8IHdpbmRvdy5l",
				"dmVudF0uZXh0ZW5kKGFyZ3MpOwoJCQl2YXIgcmV0dXJucyA9IGZ1bmN0aW9uKCl7CgkJCQlyZXR1cm4g",
				"c2VsZi5hcHBseShvcHRpb25zLmJpbmQgfHwgbnVsbCwgYXJncyk7CgkJCX07CgkJCWlmIChvcHRpb25z",
				"LmRlbGF5KSByZXR1cm4gc2V0VGltZW91dChyZXR1cm5zLCBvcHRpb25zLmRlbGF5KTsKCQkJaWYgKG9w",
				"dGlvbnMucGVyaW9kaWNhbCkgcmV0dXJuIHNldEludGVydmFsKHJldHVybnMsIG9wdGlvbnMucGVyaW9k",
				"aWNhbCk7CgkJCWlmIChvcHRpb25zLmF0dGVtcHQpIHJldHVybiBGdW5jdGlvbi5hdHRlbXB0KHJldHVy",
				"bnMpOwoJCQlyZXR1cm4gcmV0dXJucygpOwoJCX07Cgl9LAoKCWJpbmQ6IGZ1bmN0aW9uKGJpbmQsIGFy",
				"Z3MpewoJCXZhciBzZWxmID0gdGhpczsKCQlpZiAoYXJncyAhPSBudWxsKSBhcmdzID0gQXJyYXkuZnJv",
				"bShhcmdzKTsKCQlyZXR1cm4gZnVuY3Rpb24oKXsKCQkJcmV0dXJuIHNlbGYuYXBwbHkoYmluZCwgYXJn",
				"cyB8fCBhcmd1bWVudHMpOwoJCX07Cgl9LAoKCWJpbmRXaXRoRXZlbnQ6IGZ1bmN0aW9uKGJpbmQsIGFy",
				"Z3MpewoJCXZhciBzZWxmID0gdGhpczsKCQlpZiAoYXJncyAhPSBudWxsKSBhcmdzID0gQXJyYXkuZnJv",
				"bShhcmdzKTsKCQlyZXR1cm4gZnVuY3Rpb24oZXZlbnQpewoJCQlyZXR1cm4gc2VsZi5hcHBseShiaW5k",
				"LCAoYXJncyA9PSBudWxsKSA/IGFyZ3VtZW50cyA6IFtldmVudF0uY29uY2F0KGFyZ3MpKTsKCQl9OwoJ",
				"fSwKCglydW46IGZ1bmN0aW9uKGFyZ3MsIGJpbmQpewoJCXJldHVybiB0aGlzLmFwcGx5KGJpbmQsIEFy",
				"cmF5LmZyb20oYXJncykpOwoJfQoKfSk7CgppZiAoT2JqZWN0LmNyZWF0ZSA9PSBGdW5jdGlvbi5wcm90",
				"b3R5cGUuY3JlYXRlKSBPYmplY3QuY3JlYXRlID0gbnVsbDsKCnZhciAkdHJ5ID0gRnVuY3Rpb24uYXR0",
				"ZW1wdDsKCi8vPC8xLjJjb21wYXQ+CgoKLyoKLS0tCgpuYW1lOiBPYmplY3QKCmRlc2NyaXB0aW9uOiBP",
				"YmplY3QgZ2VuZXJpYyBtZXRob2RzCgpsaWNlbnNlOiBNSVQtc3R5bGUgbGljZW5zZS4KCnJlcXVpcmVz",
				"OiBUeXBlCgpwcm92aWRlczogW09iamVjdCwgSGFzaF0KCi4uLgoqLwoKKGZ1bmN0aW9uKCl7Cgp2YXIg",
				"aGFzT3duUHJvcGVydHkgPSBPYmplY3QucHJvdG90eXBlLmhhc093blByb3BlcnR5OwoKT2JqZWN0LmV4",
				"dGVuZCh7CgoJc3Vic2V0OiBmdW5jdGlvbihvYmplY3QsIGtleXMpewoJCXZhciByZXN1bHRzID0ge307",
				"CgkJZm9yICh2YXIgaSA9IDAsIGwgPSBrZXlzLmxlbmd0aDsgaSA8IGw7IGkrKyl7CgkJCXZhciBrID0g",
				"a2V5c1tpXTsKCQkJaWYgKGsgaW4gb2JqZWN0KSByZXN1bHRzW2tdID0gb2JqZWN0W2tdOwoJCX0KCQly",
				"ZXR1cm4gcmVzdWx0czsKCX0sCgoJbWFwOiBmdW5jdGlvbihvYmplY3QsIGZuLCBiaW5kKXsKCQl2YXIg",
				"cmVzdWx0cyA9IHt9OwoJCWZvciAodmFyIGtleSBpbiBvYmplY3QpewoJCQlpZiAoaGFzT3duUHJvcGVy",
				"dHkuY2FsbChvYmplY3QsIGtleSkpIHJlc3VsdHNba2V5XSA9IGZuLmNhbGwoYmluZCwgb2JqZWN0W2tl",
				"eV0sIGtleSwgb2JqZWN0KTsKCQl9CgkJcmV0dXJuIHJlc3VsdHM7Cgl9LAoKCWZpbHRlcjogZnVuY3Rp",
				"b24ob2JqZWN0LCBmbiwgYmluZCl7CgkJdmFyIHJlc3VsdHMgPSB7fTsKCQlmb3IgKHZhciBrZXkgaW4g",
				"b2JqZWN0KXsKCQkJdmFyIHZhbHVlID0gb2JqZWN0W2tleV07CgkJCWlmIChoYXNPd25Qcm9wZXJ0eS5j",
				"YWxsKG9iamVjdCwga2V5KSAmJiBmbi5jYWxsKGJpbmQsIHZhbHVlLCBrZXksIG9iamVjdCkpIHJlc3Vs",
				"dHNba2V5XSA9IHZhbHVlOwoJCX0KCQlyZXR1cm4gcmVzdWx0czsKCX0sCgoJZXZlcnk6IGZ1bmN0aW9u",
				"KG9iamVjdCwgZm4sIGJpbmQpewoJCWZvciAodmFyIGtleSBpbiBvYmplY3QpewoJCQlpZiAoaGFzT3du",
				"UHJvcGVydHkuY2FsbChvYmplY3QsIGtleSkgJiYgIWZuLmNhbGwoYmluZCwgb2JqZWN0W2tleV0sIGtl",
				"eSkpIHJldHVybiBmYWxzZTsKCQl9CgkJcmV0dXJuIHRydWU7Cgl9LAoKCXNvbWU6IGZ1bmN0aW9uKG9i",
				"amVjdCwgZm4sIGJpbmQpewoJCWZvciAodmFyIGtleSBpbiBvYmplY3QpewoJCQlpZiAoaGFzT3duUHJv",
				"cGVydHkuY2FsbChvYmplY3QsIGtleSkgJiYgZm4uY2FsbChiaW5kLCBvYmplY3Rba2V5XSwga2V5KSkg",
				"cmV0dXJuIHRydWU7CgkJfQoJCXJldHVybiBmYWxzZTsKCX0sCgoJa2V5czogZnVuY3Rpb24ob2JqZWN0",
				"KXsKCQl2YXIga2V5cyA9IFtdOwoJCWZvciAodmFyIGtleSBpbiBvYmplY3QpewoJCQlpZiAoaGFzT3du",
				"UHJvcGVydHkuY2FsbChvYmplY3QsIGtleSkpIGtleXMucHVzaChrZXkpOwoJCX0KCQlyZXR1cm4ga2V5",
				"czsKCX0sCgoJdmFsdWVzOiBmdW5jdGlvbihvYmplY3QpewoJCXZhciB2YWx1ZXMgPSBbXTsKCQlmb3Ig",
				"KHZhciBrZXkgaW4gb2JqZWN0KXsKCQkJaWYgKGhhc093blByb3BlcnR5LmNhbGwob2JqZWN0LCBrZXkp",
				"KSB2YWx1ZXMucHVzaChvYmplY3Rba2V5XSk7CgkJfQoJCXJldHVybiB2YWx1ZXM7Cgl9LAoKCWdldExl",
				"bmd0aDogZnVuY3Rpb24ob2JqZWN0KXsKCQlyZXR1cm4gT2JqZWN0LmtleXMob2JqZWN0KS5sZW5ndGg7",
				"Cgl9LAoKCWtleU9mOiBmdW5jdGlvbihvYmplY3QsIHZhbHVlKXsKCQlmb3IgKHZhciBrZXkgaW4gb2Jq",
				"ZWN0KXsKCQkJaWYgKGhhc093blByb3BlcnR5LmNhbGwob2JqZWN0LCBrZXkpICYmIG9iamVjdFtrZXld",
				"ID09PSB2YWx1ZSkgcmV0dXJuIGtleTsKCQl9CgkJcmV0dXJuIG51bGw7Cgl9LAoKCWNvbnRhaW5zOiBm",
				"dW5jdGlvbihvYmplY3QsIHZhbHVlKXsKCQlyZXR1cm4gT2JqZWN0LmtleU9mKG9iamVjdCwgdmFsdWUp",
				"ICE9IG51bGw7Cgl9LAoKCXRvUXVlcnlTdHJpbmc6IGZ1bmN0aW9uKG9iamVjdCwgYmFzZSl7CgkJdmFy",
				"IHF1ZXJ5U3RyaW5nID0gW107CgoJCU9iamVjdC5lYWNoKG9iamVjdCwgZnVuY3Rpb24odmFsdWUsIGtl",
				"eSl7CgkJCWlmIChiYXNlKSBrZXkgPSBiYXNlICsgJ1snICsga2V5ICsgJ10nOwoJCQl2YXIgcmVzdWx0",
				"OwoJCQlzd2l0Y2ggKHR5cGVPZih2YWx1ZSkpewoJCQkJY2FzZSAnb2JqZWN0JzogcmVzdWx0ID0gT2Jq",
				"ZWN0LnRvUXVlcnlTdHJpbmcodmFsdWUsIGtleSk7IGJyZWFrOwoJCQkJY2FzZSAnYXJyYXknOgoJCQkJ",
				"CXZhciBxcyA9IHt9OwoJCQkJCXZhbHVlLmVhY2goZnVuY3Rpb24odmFsLCBpKXsKCQkJCQkJcXNbaV0g",
				"PSB2YWw7CgkJCQkJfSk7CgkJCQkJcmVzdWx0ID0gT2JqZWN0LnRvUXVlcnlTdHJpbmcocXMsIGtleSk7",
				"CgkJCQlicmVhazsKCQkJCWRlZmF1bHQ6IHJlc3VsdCA9IGtleSArICc9JyArIGVuY29kZVVSSUNvbXBv",
				"bmVudCh2YWx1ZSk7CgkJCX0KCQkJaWYgKHZhbHVlICE9IG51bGwpIHF1ZXJ5U3RyaW5nLnB1c2gocmVz",
				"dWx0KTsKCQl9KTsKCgkJcmV0dXJuIHF1ZXJ5U3RyaW5nLmpvaW4oJyYnKTsKCX0KCn0pOwoKfSkoKTsK",
				"Ci8vPDEuMmNvbXBhdD4KCkhhc2guaW1wbGVtZW50KHsKCgloYXM6IE9iamVjdC5wcm90b3R5cGUuaGFz",
				"T3duUHJvcGVydHksCgoJa2V5T2Y6IGZ1bmN0aW9uKHZhbHVlKXsKCQlyZXR1cm4gT2JqZWN0LmtleU9m",
				"KHRoaXMsIHZhbHVlKTsKCX0sCgoJaGFzVmFsdWU6IGZ1bmN0aW9uKHZhbHVlKXsKCQlyZXR1cm4gT2Jq",
				"ZWN0LmNvbnRhaW5zKHRoaXMsIHZhbHVlKTsKCX0sCgoJZXh0ZW5kOiBmdW5jdGlvbihwcm9wZXJ0aWVz",
				"KXsKCQlIYXNoLmVhY2gocHJvcGVydGllcyB8fCB7fSwgZnVuY3Rpb24odmFsdWUsIGtleSl7CgkJCUhh",
				"c2guc2V0KHRoaXMsIGtleSwgdmFsdWUpOwoJCX0sIHRoaXMpOwoJCXJldHVybiB0aGlzOwoJfSwKCglj",
				"b21iaW5lOiBmdW5jdGlvbihwcm9wZXJ0aWVzKXsKCQlIYXNoLmVhY2gocHJvcGVydGllcyB8fCB7fSwg",
				"ZnVuY3Rpb24odmFsdWUsIGtleSl7CgkJCUhhc2guaW5jbHVkZSh0aGlzLCBrZXksIHZhbHVlKTsKCQl9",
				"LCB0aGlzKTsKCQlyZXR1cm4gdGhpczsKCX0sCgoJZXJhc2U6IGZ1bmN0aW9uKGtleSl7CgkJaWYgKHRo",
				"aXMuaGFzT3duUHJvcGVydHkoa2V5KSkgZGVsZXRlIHRoaXNba2V5XTsKCQlyZXR1cm4gdGhpczsKCX0s",
				"CgoJZ2V0OiBmdW5jdGlvbihrZXkpewoJCXJldHVybiAodGhpcy5oYXNPd25Qcm9wZXJ0eShrZXkpKSA/",
				"IHRoaXNba2V5XSA6IG51bGw7Cgl9LAoKCXNldDogZnVuY3Rpb24oa2V5LCB2YWx1ZSl7CgkJaWYgKCF0",
				"aGlzW2tleV0gfHwgdGhpcy5oYXNPd25Qcm9wZXJ0eShrZXkpKSB0aGlzW2tleV0gPSB2YWx1ZTsKCQly",
				"ZXR1cm4gdGhpczsKCX0sCgoJZW1wdHk6IGZ1bmN0aW9uKCl7CgkJSGFzaC5lYWNoKHRoaXMsIGZ1bmN0",
				"aW9uKHZhbHVlLCBrZXkpewoJCQlkZWxldGUgdGhpc1trZXldOwoJCX0sIHRoaXMpOwoJCXJldHVybiB0",
				"aGlzOwoJfSwKCglpbmNsdWRlOiBmdW5jdGlvbihrZXksIHZhbHVlKXsKCQlpZiAodGhpc1trZXldID09",
				"IG51bGwpIHRoaXNba2V5XSA9IHZhbHVlOwoJCXJldHVybiB0aGlzOwoJfSwKCgltYXA6IGZ1bmN0aW9u",
				"KGZuLCBiaW5kKXsKCQlyZXR1cm4gbmV3IEhhc2goT2JqZWN0Lm1hcCh0aGlzLCBmbiwgYmluZCkpOwoJ",
				"fSwKCglmaWx0ZXI6IGZ1bmN0aW9uKGZuLCBiaW5kKXsKCQlyZXR1cm4gbmV3IEhhc2goT2JqZWN0LmZp",
				"bHRlcih0aGlzLCBmbiwgYmluZCkpOwoJfSwKCglldmVyeTogZnVuY3Rpb24oZm4sIGJpbmQpewoJCXJl",
				"dHVybiBPYmplY3QuZXZlcnkodGhpcywgZm4sIGJpbmQpOwoJfSwKCglzb21lOiBmdW5jdGlvbihmbiwg",
				"YmluZCl7CgkJcmV0dXJuIE9iamVjdC5zb21lKHRoaXMsIGZuLCBiaW5kKTsKCX0sCgoJZ2V0S2V5czog",
				"ZnVuY3Rpb24oKXsKCQlyZXR1cm4gT2JqZWN0LmtleXModGhpcyk7Cgl9LAoKCWdldFZhbHVlczogZnVu",
				"Y3Rpb24oKXsKCQlyZXR1cm4gT2JqZWN0LnZhbHVlcyh0aGlzKTsKCX0sCgoJdG9RdWVyeVN0cmluZzog",
				"ZnVuY3Rpb24oYmFzZSl7CgkJcmV0dXJuIE9iamVjdC50b1F1ZXJ5U3RyaW5nKHRoaXMsIGJhc2UpOwoJ",
				"fQoKfSk7CgpIYXNoLmV4dGVuZCA9IE9iamVjdC5hcHBlbmQ7CgpIYXNoLmFsaWFzKHtpbmRleE9mOiAn",
				"a2V5T2YnLCBjb250YWluczogJ2hhc1ZhbHVlJ30pOwoKLy88LzEuMmNvbXBhdD4KCgovKgotLS0KCm5h",
				"bWU6IEJyb3dzZXIKCmRlc2NyaXB0aW9uOiBUaGUgQnJvd3NlciBPYmplY3QuIENvbnRhaW5zIEJyb3dz",
				"ZXIgaW5pdGlhbGl6YXRpb24sIFdpbmRvdyBhbmQgRG9jdW1lbnQsIGFuZCB0aGUgQnJvd3NlciBIYXNo",
				"LgoKbGljZW5zZTogTUlULXN0eWxlIGxpY2Vuc2UuCgpyZXF1aXJlczogW0FycmF5LCBGdW5jdGlvbiwg",
				"TnVtYmVyLCBTdHJpbmddCgpwcm92aWRlczogW0Jyb3dzZXIsIFdpbmRvdywgRG9jdW1lbnRdCgouLi4K",
				"Ki8KCihmdW5jdGlvbigpewoKdmFyIGRvY3VtZW50ID0gdGhpcy5kb2N1bWVudDsKdmFyIHdpbmRvdyA9",
				"IGRvY3VtZW50LndpbmRvdyA9IHRoaXM7Cgp2YXIgdWEgPSBuYXZpZ2F0b3IudXNlckFnZW50LnRvTG93",
				"ZXJDYXNlKCksCglwbGF0Zm9ybSA9IG5hdmlnYXRvci5wbGF0Zm9ybS50b0xvd2VyQ2FzZSgpLAoJVUEg",
				"PSB1YS5tYXRjaCgvKG9wZXJhfGllfGZpcmVmb3h8Y2hyb21lfHZlcnNpb24pW1xzXC86XShbXHdcZFwu",
				"XSspPy4qPyhzYWZhcml8dmVyc2lvbltcc1wvOl0oW1x3XGRcLl0rKXwkKS8pIHx8IFtudWxsLCAndW5r",
				"bm93bicsIDBdLAoJbW9kZSA9IFVBWzFdID09ICdpZScgJiYgZG9jdW1lbnQuZG9jdW1lbnRNb2RlOwoK",
				"dmFyIEJyb3dzZXIgPSB0aGlzLkJyb3dzZXIgPSB7CgoJZXh0ZW5kOiBGdW5jdGlvbi5wcm90b3R5cGUu",
				"ZXh0ZW5kLAoKCW5hbWU6IChVQVsxXSA9PSAndmVyc2lvbicpID8gVUFbM10gOiBVQVsxXSwKCgl2ZXJz",
				"aW9uOiBtb2RlIHx8IHBhcnNlRmxvYXQoKFVBWzFdID09ICdvcGVyYScgJiYgVUFbNF0pID8gVUFbNF0g",
				"OiBVQVsyXSksCgoJUGxhdGZvcm06IHsKCQluYW1lOiB1YS5tYXRjaCgvaXAoPzphZHxvZHxob25lKS8p",
				"ID8gJ2lvcycgOiAodWEubWF0Y2goLyg/OndlYm9zfGFuZHJvaWQpLykgfHwgcGxhdGZvcm0ubWF0Y2go",
				"L21hY3x3aW58bGludXgvKSB8fCBbJ290aGVyJ10pWzBdCgl9LAoKCUZlYXR1cmVzOiB7CgkJeHBhdGg6",
				"ICEhKGRvY3VtZW50LmV2YWx1YXRlKSwKCQlhaXI6ICEhKHdpbmRvdy5ydW50aW1lKSwKCQlxdWVyeTog",
				"ISEoZG9jdW1lbnQucXVlcnlTZWxlY3RvciksCgkJanNvbjogISEod2luZG93LkpTT04pCgl9LAoKCVBs",
				"dWdpbnM6IHt9Cgp9OwoKQnJvd3NlcltCcm93c2VyLm5hbWVdID0gdHJ1ZTsKQnJvd3NlcltCcm93c2Vy",
				"Lm5hbWUgKyBwYXJzZUludChCcm93c2VyLnZlcnNpb24sIDEwKV0gPSB0cnVlOwpCcm93c2VyLlBsYXRm",
				"b3JtW0Jyb3dzZXIuUGxhdGZvcm0ubmFtZV0gPSB0cnVlOwoKLy8gUmVxdWVzdAoKQnJvd3Nlci5SZXF1",
				"ZXN0ID0gKGZ1bmN0aW9uKCl7CgoJdmFyIFhNTEhUVFAgPSBmdW5jdGlvbigpewoJCXJldHVybiBuZXcg",
				"WE1MSHR0cFJlcXVlc3QoKTsKCX07CgoJdmFyIE1TWE1MMiA9IGZ1bmN0aW9uKCl7CgkJcmV0dXJuIG5l",
				"dyBBY3RpdmVYT2JqZWN0KCdNU1hNTDIuWE1MSFRUUCcpOwoJfTsKCgl2YXIgTVNYTUwgPSBmdW5jdGlv",
				"bigpewoJCXJldHVybiBuZXcgQWN0aXZlWE9iamVjdCgnTWljcm9zb2Z0LlhNTEhUVFAnKTsKCX07CgoJ",
				"cmV0dXJuIEZ1bmN0aW9uLmF0dGVtcHQoZnVuY3Rpb24oKXsKCQlYTUxIVFRQKCk7CgkJcmV0dXJuIFhN",
				"TEhUVFA7Cgl9LCBmdW5jdGlvbigpewoJCU1TWE1MMigpOwoJCXJldHVybiBNU1hNTDI7Cgl9LCBmdW5j",
				"dGlvbigpewoJCU1TWE1MKCk7CgkJcmV0dXJuIE1TWE1MOwoJfSk7Cgp9KSgpOwoKQnJvd3Nlci5GZWF0",
				"dXJlcy54aHIgPSAhIShCcm93c2VyLlJlcXVlc3QpOwoKLy8gRmxhc2ggZGV0ZWN0aW9uCgp2YXIgdmVy",
				"c2lvbiA9IChGdW5jdGlvbi5hdHRlbXB0KGZ1bmN0aW9uKCl7CglyZXR1cm4gbmF2aWdhdG9yLnBsdWdp",
				"bnNbJ1Nob2Nrd2F2ZSBGbGFzaCddLmRlc2NyaXB0aW9uOwp9LCBmdW5jdGlvbigpewoJcmV0dXJuIG5l",
				"dyBBY3RpdmVYT2JqZWN0KCdTaG9ja3dhdmVGbGFzaC5TaG9ja3dhdmVGbGFzaCcpLkdldFZhcmlhYmxl",
				"KCckdmVyc2lvbicpOwp9KSB8fCAnMCByMCcpLm1hdGNoKC9cZCsvZyk7CgpCcm93c2VyLlBsdWdpbnMu",
				"Rmxhc2ggPSB7Cgl2ZXJzaW9uOiBOdW1iZXIodmVyc2lvblswXSB8fCAnMC4nICsgdmVyc2lvblsxXSkg",
				"fHwgMCwKCWJ1aWxkOiBOdW1iZXIodmVyc2lvblsyXSkgfHwgMAp9OwoKLy8gU3RyaW5nIHNjcmlwdHMK",
				"CkJyb3dzZXIuZXhlYyA9IGZ1bmN0aW9uKHRleHQpewoJaWYgKCF0ZXh0KSByZXR1cm4gdGV4dDsKCWlm",
				"ICh3aW5kb3cuZXhlY1NjcmlwdCl7CgkJd2luZG93LmV4ZWNTY3JpcHQodGV4dCk7Cgl9IGVsc2UgewoJ",
				"CXZhciBzY3JpcHQgPSBkb2N1bWVudC5jcmVhdGVFbGVtZW50KCdzY3JpcHQnKTsKCQlzY3JpcHQuc2V0",
				"QXR0cmlidXRlKCd0eXBlJywgJ3RleHQvamF2YXNjcmlwdCcpOwoJCXNjcmlwdC50ZXh0ID0gdGV4dDsK",
				"CQlkb2N1bWVudC5oZWFkLmFwcGVuZENoaWxkKHNjcmlwdCk7CgkJZG9jdW1lbnQuaGVhZC5yZW1vdmVD",
				"aGlsZChzY3JpcHQpOwoJfQoJcmV0dXJuIHRleHQ7Cn07CgpTdHJpbmcuaW1wbGVtZW50KCdzdHJpcFNj",
				"cmlwdHMnLCBmdW5jdGlvbihleGVjKXsKCXZhciBzY3JpcHRzID0gJyc7Cgl2YXIgdGV4dCA9IHRoaXMu",
				"cmVwbGFjZSgvPHNjcmlwdFtePl0qPihbXHNcU10qPyk8XC9zY3JpcHQ+L2dpLCBmdW5jdGlvbihhbGws",
				"IGNvZGUpewoJCXNjcmlwdHMgKz0gY29kZSArICdcbic7CgkJcmV0dXJuICcnOwoJfSk7CglpZiAoZXhl",
				"YyA9PT0gdHJ1ZSkgQnJvd3Nlci5leGVjKHNjcmlwdHMpOwoJZWxzZSBpZiAodHlwZU9mKGV4ZWMpID09",
				"ICdmdW5jdGlvbicpIGV4ZWMoc2NyaXB0cywgdGV4dCk7CglyZXR1cm4gdGV4dDsKfSk7CgovLyBXaW5k",
				"b3csIERvY3VtZW50CgpCcm93c2VyLmV4dGVuZCh7CglEb2N1bWVudDogdGhpcy5Eb2N1bWVudCwKCVdp",
				"bmRvdzogdGhpcy5XaW5kb3csCglFbGVtZW50OiB0aGlzLkVsZW1lbnQsCglFdmVudDogdGhpcy5FdmVu",
				"dAp9KTsKCnRoaXMuV2luZG93ID0gdGhpcy4kY29uc3RydWN0b3IgPSBuZXcgVHlwZSgnV2luZG93Jywg",
				"ZnVuY3Rpb24oKXt9KTsKCnRoaXMuJGZhbWlseSA9IEZ1bmN0aW9uLmZyb20oJ3dpbmRvdycpLmhpZGUo",
				"KTsKCldpbmRvdy5taXJyb3IoZnVuY3Rpb24obmFtZSwgbWV0aG9kKXsKCXdpbmRvd1tuYW1lXSA9IG1l",
				"dGhvZDsKfSk7Cgp0aGlzLkRvY3VtZW50ID0gZG9jdW1lbnQuJGNvbnN0cnVjdG9yID0gbmV3IFR5cGUo",
				"J0RvY3VtZW50JywgZnVuY3Rpb24oKXt9KTsKCmRvY3VtZW50LiRmYW1pbHkgPSBGdW5jdGlvbi5mcm9t",
				"KCdkb2N1bWVudCcpLmhpZGUoKTsKCkRvY3VtZW50Lm1pcnJvcihmdW5jdGlvbihuYW1lLCBtZXRob2Qp",
				"ewoJZG9jdW1lbnRbbmFtZV0gPSBtZXRob2Q7Cn0pOwoKZG9jdW1lbnQuaHRtbCA9IGRvY3VtZW50LmRv",
				"Y3VtZW50RWxlbWVudDsKaWYgKCFkb2N1bWVudC5oZWFkKSBkb2N1bWVudC5oZWFkID0gZG9jdW1lbnQu",
				"Z2V0RWxlbWVudHNCeVRhZ05hbWUoJ2hlYWQnKVswXTsKCmlmIChkb2N1bWVudC5leGVjQ29tbWFuZCkg",
				"dHJ5IHsKCWRvY3VtZW50LmV4ZWNDb21tYW5kKCJCYWNrZ3JvdW5kSW1hZ2VDYWNoZSIsIGZhbHNlLCB0",
				"cnVlKTsKfSBjYXRjaCAoZSl7fQoKLyo8bHRJRTk+Ki8KaWYgKHRoaXMuYXR0YWNoRXZlbnQgJiYgIXRo",
				"aXMuYWRkRXZlbnRMaXN0ZW5lcil7Cgl2YXIgdW5sb2FkRXZlbnQgPSBmdW5jdGlvbigpewoJCXRoaXMu",
				"ZGV0YWNoRXZlbnQoJ29udW5sb2FkJywgdW5sb2FkRXZlbnQpOwoJCWRvY3VtZW50LmhlYWQgPSBkb2N1",
				"bWVudC5odG1sID0gZG9jdW1lbnQud2luZG93ID0gbnVsbDsKCX07Cgl0aGlzLmF0dGFjaEV2ZW50KCdv",
				"bnVubG9hZCcsIHVubG9hZEV2ZW50KTsKfQoKLy8gSUUgZmFpbHMgb24gY29sbGVjdGlvbnMgYW5kIDxz",
				"ZWxlY3Q+Lm9wdGlvbnMgKHJlZmVycyB0byA8c2VsZWN0PikKdmFyIGFycmF5RnJvbSA9IEFycmF5LmZy",
				"b207CnRyeSB7CglhcnJheUZyb20oZG9jdW1lbnQuaHRtbC5jaGlsZE5vZGVzKTsKfSBjYXRjaChlKXsK",
				"CUFycmF5LmZyb20gPSBmdW5jdGlvbihpdGVtKXsKCQlpZiAodHlwZW9mIGl0ZW0gIT0gJ3N0cmluZycg",
				"JiYgVHlwZS5pc0VudW1lcmFibGUoaXRlbSkgJiYgdHlwZU9mKGl0ZW0pICE9ICdhcnJheScpewoJCQl2",
				"YXIgaSA9IGl0ZW0ubGVuZ3RoLCBhcnJheSA9IG5ldyBBcnJheShpKTsKCQkJd2hpbGUgKGktLSkgYXJy",
				"YXlbaV0gPSBpdGVtW2ldOwoJCQlyZXR1cm4gYXJyYXk7CgkJfQoJCXJldHVybiBhcnJheUZyb20oaXRl",
				"bSk7Cgl9OwoKCXZhciBwcm90b3R5cGUgPSBBcnJheS5wcm90b3R5cGUsCgkJc2xpY2UgPSBwcm90b3R5",
				"cGUuc2xpY2U7CglbJ3BvcCcsICdwdXNoJywgJ3JldmVyc2UnLCAnc2hpZnQnLCAnc29ydCcsICdzcGxp",
				"Y2UnLCAndW5zaGlmdCcsICdjb25jYXQnLCAnam9pbicsICdzbGljZSddLmVhY2goZnVuY3Rpb24obmFt",
				"ZSl7CgkJdmFyIG1ldGhvZCA9IHByb3RvdHlwZVtuYW1lXTsKCQlBcnJheVtuYW1lXSA9IGZ1bmN0aW9u",
				"KGl0ZW0pewoJCQlyZXR1cm4gbWV0aG9kLmFwcGx5KEFycmF5LmZyb20oaXRlbSksIHNsaWNlLmNhbGwo",
				"YXJndW1lbnRzLCAxKSk7CgkJfTsKCX0pOwp9Ci8qPC9sdElFOT4qLwoKLy88MS4yY29tcGF0PgoKaWYg",
				"KEJyb3dzZXIuUGxhdGZvcm0uaW9zKSBCcm93c2VyLlBsYXRmb3JtLmlwb2QgPSB0cnVlOwoKQnJvd3Nl",
				"ci5FbmdpbmUgPSB7fTsKCnZhciBzZXRFbmdpbmUgPSBmdW5jdGlvbihuYW1lLCB2ZXJzaW9uKXsKCUJy",
				"b3dzZXIuRW5naW5lLm5hbWUgPSBuYW1lOwoJQnJvd3Nlci5FbmdpbmVbbmFtZSArIHZlcnNpb25dID0g",
				"dHJ1ZTsKCUJyb3dzZXIuRW5naW5lLnZlcnNpb24gPSB2ZXJzaW9uOwp9OwoKaWYgKEJyb3dzZXIuaWUp",
				"ewoJQnJvd3Nlci5FbmdpbmUudHJpZGVudCA9IHRydWU7CgoJc3dpdGNoIChCcm93c2VyLnZlcnNpb24p",
				"ewoJCWNhc2UgNjogc2V0RW5naW5lKCd0cmlkZW50JywgNCk7IGJyZWFrOwoJCWNhc2UgNzogc2V0RW5n",
				"aW5lKCd0cmlkZW50JywgNSk7IGJyZWFrOwoJCWNhc2UgODogc2V0RW5naW5lKCd0cmlkZW50JywgNik7",
				"Cgl9Cn0KCmlmIChCcm93c2VyLmZpcmVmb3gpewoJQnJvd3Nlci5FbmdpbmUuZ2Vja28gPSB0cnVlOwoK",
				"CWlmIChCcm93c2VyLnZlcnNpb24gPj0gMykgc2V0RW5naW5lKCdnZWNrbycsIDE5KTsKCWVsc2Ugc2V0",
				"RW5naW5lKCdnZWNrbycsIDE4KTsKfQoKaWYgKEJyb3dzZXIuc2FmYXJpIHx8IEJyb3dzZXIuY2hyb21l",
				"KXsKCUJyb3dzZXIuRW5naW5lLndlYmtpdCA9IHRydWU7CgoJc3dpdGNoIChCcm93c2VyLnZlcnNpb24p",
				"ewoJCWNhc2UgMjogc2V0RW5naW5lKCd3ZWJraXQnLCA0MTkpOyBicmVhazsKCQljYXNlIDM6IHNldEVu",
				"Z2luZSgnd2Via2l0JywgNDIwKTsgYnJlYWs7CgkJY2FzZSA0OiBzZXRFbmdpbmUoJ3dlYmtpdCcsIDUy",
				"NSk7Cgl9Cn0KCmlmIChCcm93c2VyLm9wZXJhKXsKCUJyb3dzZXIuRW5naW5lLnByZXN0byA9IHRydWU7",
				"CgoJaWYgKEJyb3dzZXIudmVyc2lvbiA+PSA5LjYpIHNldEVuZ2luZSgncHJlc3RvJywgOTYwKTsKCWVs",
				"c2UgaWYgKEJyb3dzZXIudmVyc2lvbiA+PSA5LjUpIHNldEVuZ2luZSgncHJlc3RvJywgOTUwKTsKCWVs",
				"c2Ugc2V0RW5naW5lKCdwcmVzdG8nLCA5MjUpOwp9CgppZiAoQnJvd3Nlci5uYW1lID09ICd1bmtub3du",
				"Jyl7Cglzd2l0Y2ggKCh1YS5tYXRjaCgvKD86d2Via2l0fGtodG1sfGdlY2tvKS8pIHx8IFtdKVswXSl7",
				"CgkJY2FzZSAnd2Via2l0JzoKCQljYXNlICdraHRtbCc6CgkJCUJyb3dzZXIuRW5naW5lLndlYmtpdCA9",
				"IHRydWU7CgkJYnJlYWs7CgkJY2FzZSAnZ2Vja28nOgoJCQlCcm93c2VyLkVuZ2luZS5nZWNrbyA9IHRy",
				"dWU7Cgl9Cn0KCnRoaXMuJGV4ZWMgPSBCcm93c2VyLmV4ZWM7CgovLzwvMS4yY29tcGF0PgoKfSkoKTsK",
				"CgovKgotLS0KCm5hbWU6IEV2ZW50CgpkZXNjcmlwdGlvbjogQ29udGFpbnMgdGhlIEV2ZW50IFR5cGUs",
				"IHRvIG1ha2UgdGhlIGV2ZW50IG9iamVjdCBjcm9zcy1icm93c2VyLgoKbGljZW5zZTogTUlULXN0eWxl",
				"IGxpY2Vuc2UuCgpyZXF1aXJlczogW1dpbmRvdywgRG9jdW1lbnQsIEFycmF5LCBGdW5jdGlvbiwgU3Ry",
				"aW5nLCBPYmplY3RdCgpwcm92aWRlczogRXZlbnQKCi4uLgoqLwoKKGZ1bmN0aW9uKCkgewoKdmFyIF9r",
				"ZXlzID0ge307Cgp2YXIgRE9NRXZlbnQgPSB0aGlzLkRPTUV2ZW50ID0gbmV3IFR5cGUoJ0RPTUV2ZW50",
				"JywgZnVuY3Rpb24oZXZlbnQsIHdpbil7CglpZiAoIXdpbikgd2luID0gd2luZG93OwoJZXZlbnQgPSBl",
				"dmVudCB8fCB3aW4uZXZlbnQ7CglpZiAoZXZlbnQuJGV4dGVuZGVkKSByZXR1cm4gZXZlbnQ7Cgl0aGlz",
				"LmV2ZW50ID0gZXZlbnQ7Cgl0aGlzLiRleHRlbmRlZCA9IHRydWU7Cgl0aGlzLnNoaWZ0ID0gZXZlbnQu",
				"c2hpZnRLZXk7Cgl0aGlzLmNvbnRyb2wgPSBldmVudC5jdHJsS2V5OwoJdGhpcy5hbHQgPSBldmVudC5h",
				"bHRLZXk7Cgl0aGlzLm1ldGEgPSBldmVudC5tZXRhS2V5OwoJdmFyIHR5cGUgPSB0aGlzLnR5cGUgPSBl",
				"dmVudC50eXBlOwoJdmFyIHRhcmdldCA9IGV2ZW50LnRhcmdldCB8fCBldmVudC5zcmNFbGVtZW50OwoJ",
				"d2hpbGUgKHRhcmdldCAmJiB0YXJnZXQubm9kZVR5cGUgPT0gMykgdGFyZ2V0ID0gdGFyZ2V0LnBhcmVu",
				"dE5vZGU7Cgl0aGlzLnRhcmdldCA9IGRvY3VtZW50LmlkKHRhcmdldCk7CgoJaWYgKHR5cGUuaW5kZXhP",
				"Zigna2V5JykgPT0gMCl7CgkJdmFyIGNvZGUgPSB0aGlzLmNvZGUgPSAoZXZlbnQud2hpY2ggfHwgZXZl",
				"bnQua2V5Q29kZSk7CgkJdGhpcy5rZXkgPSBfa2V5c1tjb2RlXS8qPDEuM2NvbXBhdD4qLyB8fCBPYmpl",
				"Y3Qua2V5T2YoRXZlbnQuS2V5cywgY29kZSkvKjwvMS4zY29tcGF0PiovOwoJCWlmICh0eXBlID09ICdr",
				"ZXlkb3duJyl7CgkJCWlmIChjb2RlID4gMTExICYmIGNvZGUgPCAxMjQpIHRoaXMua2V5ID0gJ2YnICsg",
				"KGNvZGUgLSAxMTEpOwoJCQllbHNlIGlmIChjb2RlID4gOTUgJiYgY29kZSA8IDEwNikgdGhpcy5rZXkg",
				"PSBjb2RlIC0gOTY7CgkJfQoJCWlmICh0aGlzLmtleSA9PSBudWxsKSB0aGlzLmtleSA9IFN0cmluZy5m",
				"cm9tQ2hhckNvZGUoY29kZSkudG9Mb3dlckNhc2UoKTsKCX0gZWxzZSBpZiAodHlwZSA9PSAnY2xpY2sn",
				"IHx8IHR5cGUgPT0gJ2RibGNsaWNrJyB8fCB0eXBlID09ICdjb250ZXh0bWVudScgfHwgdHlwZSA9PSAn",
				"RE9NTW91c2VTY3JvbGwnIHx8IHR5cGUuaW5kZXhPZignbW91c2UnKSA9PSAwKXsKCQl2YXIgZG9jID0g",
				"d2luLmRvY3VtZW50OwoJCWRvYyA9ICghZG9jLmNvbXBhdE1vZGUgfHwgZG9jLmNvbXBhdE1vZGUgPT0g",
				"J0NTUzFDb21wYXQnKSA/IGRvYy5odG1sIDogZG9jLmJvZHk7CgkJdGhpcy5wYWdlID0gewoJCQl4OiAo",
				"ZXZlbnQucGFnZVggIT0gbnVsbCkgPyBldmVudC5wYWdlWCA6IGV2ZW50LmNsaWVudFggKyBkb2Muc2Ny",
				"b2xsTGVmdCwKCQkJeTogKGV2ZW50LnBhZ2VZICE9IG51bGwpID8gZXZlbnQucGFnZVkgOiBldmVudC5j",
				"bGllbnRZICsgZG9jLnNjcm9sbFRvcAoJCX07CgkJdGhpcy5jbGllbnQgPSB7CgkJCXg6IChldmVudC5w",
				"YWdlWCAhPSBudWxsKSA/IGV2ZW50LnBhZ2VYIC0gd2luLnBhZ2VYT2Zmc2V0IDogZXZlbnQuY2xpZW50",
				"WCwKCQkJeTogKGV2ZW50LnBhZ2VZICE9IG51bGwpID8gZXZlbnQucGFnZVkgLSB3aW4ucGFnZVlPZmZz",
				"ZXQgOiBldmVudC5jbGllbnRZCgkJfTsKCQlpZiAodHlwZSA9PSAnRE9NTW91c2VTY3JvbGwnIHx8IHR5",
				"cGUgPT0gJ21vdXNld2hlZWwnKQoJCQl0aGlzLndoZWVsID0gKGV2ZW50LndoZWVsRGVsdGEpID8gZXZl",
				"bnQud2hlZWxEZWx0YSAvIDEyMCA6IC0oZXZlbnQuZGV0YWlsIHx8IDApIC8gMzsKCgkJdGhpcy5yaWdo",
				"dENsaWNrID0gKGV2ZW50LndoaWNoID09IDMgfHwgZXZlbnQuYnV0dG9uID09IDIpOwoJCWlmICh0eXBl",
				"ID09ICdtb3VzZW92ZXInIHx8IHR5cGUgPT0gJ21vdXNlb3V0Jyl7CgkJCXZhciByZWxhdGVkID0gZXZl",
				"bnQucmVsYXRlZFRhcmdldCB8fCBldmVudFsodHlwZSA9PSAnbW91c2VvdmVyJyA/ICdmcm9tJyA6ICd0",
				"bycpICsgJ0VsZW1lbnQnXTsKCQkJd2hpbGUgKHJlbGF0ZWQgJiYgcmVsYXRlZC5ub2RlVHlwZSA9PSAz",
				"KSByZWxhdGVkID0gcmVsYXRlZC5wYXJlbnROb2RlOwoJCQl0aGlzLnJlbGF0ZWRUYXJnZXQgPSBkb2N1",
				"bWVudC5pZChyZWxhdGVkKTsKCQl9Cgl9IGVsc2UgaWYgKHR5cGUuaW5kZXhPZigndG91Y2gnKSA9PSAw",
				"IHx8IHR5cGUuaW5kZXhPZignZ2VzdHVyZScpID09IDApewoJCXRoaXMucm90YXRpb24gPSBldmVudC5y",
				"b3RhdGlvbjsKCQl0aGlzLnNjYWxlID0gZXZlbnQuc2NhbGU7CgkJdGhpcy50YXJnZXRUb3VjaGVzID0g",
				"ZXZlbnQudGFyZ2V0VG91Y2hlczsKCQl0aGlzLmNoYW5nZWRUb3VjaGVzID0gZXZlbnQuY2hhbmdlZFRv",
				"dWNoZXM7CgkJdmFyIHRvdWNoZXMgPSB0aGlzLnRvdWNoZXMgPSBldmVudC50b3VjaGVzOwoJCWlmICh0",
				"b3VjaGVzICYmIHRvdWNoZXNbMF0pewoJCQl2YXIgdG91Y2ggPSB0b3VjaGVzWzBdOwoJCQl0aGlzLnBh",
				"Z2UgPSB7eDogdG91Y2gucGFnZVgsIHk6IHRvdWNoLnBhZ2VZfTsKCQkJdGhpcy5jbGllbnQgPSB7eDog",
				"dG91Y2guY2xpZW50WCwgeTogdG91Y2guY2xpZW50WX07CgkJfQoJfQoKCWlmICghdGhpcy5jbGllbnQp",
				"IHRoaXMuY2xpZW50ID0ge307CglpZiAoIXRoaXMucGFnZSkgdGhpcy5wYWdlID0ge307Cn0pOwoKRE9N",
				"RXZlbnQuaW1wbGVtZW50KHsKCglzdG9wOiBmdW5jdGlvbigpewoJCXJldHVybiB0aGlzLnByZXZlbnRE",
				"ZWZhdWx0KCkuc3RvcFByb3BhZ2F0aW9uKCk7Cgl9LAoKCXN0b3BQcm9wYWdhdGlvbjogZnVuY3Rpb24o",
				"KXsKCQlpZiAodGhpcy5ldmVudC5zdG9wUHJvcGFnYXRpb24pIHRoaXMuZXZlbnQuc3RvcFByb3BhZ2F0",
				"aW9uKCk7CgkJZWxzZSB0aGlzLmV2ZW50LmNhbmNlbEJ1YmJsZSA9IHRydWU7CgkJcmV0dXJuIHRoaXM7",
				"Cgl9LAoKCXByZXZlbnREZWZhdWx0OiBmdW5jdGlvbigpewoJCWlmICh0aGlzLmV2ZW50LnByZXZlbnRE",
				"ZWZhdWx0KSB0aGlzLmV2ZW50LnByZXZlbnREZWZhdWx0KCk7CgkJZWxzZSB0aGlzLmV2ZW50LnJldHVy",
				"blZhbHVlID0gZmFsc2U7CgkJcmV0dXJuIHRoaXM7Cgl9Cgp9KTsKCkRPTUV2ZW50LmRlZmluZUtleSA9",
				"IGZ1bmN0aW9uKGNvZGUsIGtleSl7Cglfa2V5c1tjb2RlXSA9IGtleTsKCXJldHVybiB0aGlzOwp9OwoK",
				"RE9NRXZlbnQuZGVmaW5lS2V5cyA9IERPTUV2ZW50LmRlZmluZUtleS5vdmVybG9hZFNldHRlcih0cnVl",
				"KTsKCkRPTUV2ZW50LmRlZmluZUtleXMoewoJJzM4JzogJ3VwJywgJzQwJzogJ2Rvd24nLCAnMzcnOiAn",
				"bGVmdCcsICczOSc6ICdyaWdodCcsCgknMjcnOiAnZXNjJywgJzMyJzogJ3NwYWNlJywgJzgnOiAnYmFj",
				"a3NwYWNlJywgJzknOiAndGFiJywKCSc0Nic6ICdkZWxldGUnLCAnMTMnOiAnZW50ZXInCn0pOwoKfSko",
				"KTsKCi8qPDEuM2NvbXBhdD4qLwp2YXIgRXZlbnQgPSBET01FdmVudDsKRXZlbnQuS2V5cyA9IHt9Owov",
				"KjwvMS4zY29tcGF0PiovCgovKjwxLjJjb21wYXQ+Ki8KCkV2ZW50LktleXMgPSBuZXcgSGFzaChFdmVu",
				"dC5LZXlzKTsKCi8qPC8xLjJjb21wYXQ+Ki8KCgovKgotLS0KCm5hbWU6IENsYXNzCgpkZXNjcmlwdGlv",
				"bjogQ29udGFpbnMgdGhlIENsYXNzIEZ1bmN0aW9uIGZvciBlYXNpbHkgY3JlYXRpbmcsIGV4dGVuZGlu",
				"ZywgYW5kIGltcGxlbWVudGluZyByZXVzYWJsZSBDbGFzc2VzLgoKbGljZW5zZTogTUlULXN0eWxlIGxp",
				"Y2Vuc2UuCgpyZXF1aXJlczogW0FycmF5LCBTdHJpbmcsIEZ1bmN0aW9uLCBOdW1iZXJdCgpwcm92aWRl",
				"czogQ2xhc3MKCi4uLgoqLwoKKGZ1bmN0aW9uKCl7Cgp2YXIgQ2xhc3MgPSB0aGlzLkNsYXNzID0gbmV3",
				"IFR5cGUoJ0NsYXNzJywgZnVuY3Rpb24ocGFyYW1zKXsKCWlmIChpbnN0YW5jZU9mKHBhcmFtcywgRnVu",
				"Y3Rpb24pKSBwYXJhbXMgPSB7aW5pdGlhbGl6ZTogcGFyYW1zfTsKCgl2YXIgbmV3Q2xhc3MgPSBmdW5j",
				"dGlvbigpewoJCXJlc2V0KHRoaXMpOwoJCWlmIChuZXdDbGFzcy4kcHJvdG90eXBpbmcpIHJldHVybiB0",
				"aGlzOwoJCXRoaXMuJGNhbGxlciA9IG51bGw7CgkJdmFyIHZhbHVlID0gKHRoaXMuaW5pdGlhbGl6ZSkg",
				"PyB0aGlzLmluaXRpYWxpemUuYXBwbHkodGhpcywgYXJndW1lbnRzKSA6IHRoaXM7CgkJdGhpcy4kY2Fs",
				"bGVyID0gdGhpcy5jYWxsZXIgPSBudWxsOwoJCXJldHVybiB2YWx1ZTsKCX0uZXh0ZW5kKHRoaXMpLmlt",
				"cGxlbWVudChwYXJhbXMpOwoKCW5ld0NsYXNzLiRjb25zdHJ1Y3RvciA9IENsYXNzOwoJbmV3Q2xhc3Mu",
				"cHJvdG90eXBlLiRjb25zdHJ1Y3RvciA9IG5ld0NsYXNzOwoJbmV3Q2xhc3MucHJvdG90eXBlLnBhcmVu",
				"dCA9IHBhcmVudDsKCglyZXR1cm4gbmV3Q2xhc3M7Cn0pOwoKdmFyIHBhcmVudCA9IGZ1bmN0aW9uKCl7",
				"CglpZiAoIXRoaXMuJGNhbGxlcikgdGhyb3cgbmV3IEVycm9yKCdUaGUgbWV0aG9kICJwYXJlbnQiIGNh",
				"bm5vdCBiZSBjYWxsZWQuJyk7Cgl2YXIgbmFtZSA9IHRoaXMuJGNhbGxlci4kbmFtZSwKCQlwYXJlbnQg",
				"PSB0aGlzLiRjYWxsZXIuJG93bmVyLnBhcmVudCwKCQlwcmV2aW91cyA9IChwYXJlbnQpID8gcGFyZW50",
				"LnByb3RvdHlwZVtuYW1lXSA6IG51bGw7CglpZiAoIXByZXZpb3VzKSB0aHJvdyBuZXcgRXJyb3IoJ1Ro",
				"ZSBtZXRob2QgIicgKyBuYW1lICsgJyIgaGFzIG5vIHBhcmVudC4nKTsKCXJldHVybiBwcmV2aW91cy5h",
				"cHBseSh0aGlzLCBhcmd1bWVudHMpOwp9OwoKdmFyIHJlc2V0ID0gZnVuY3Rpb24ob2JqZWN0KXsKCWZv",
				"ciAodmFyIGtleSBpbiBvYmplY3QpewoJCXZhciB2YWx1ZSA9IG9iamVjdFtrZXldOwoJCXN3aXRjaCAo",
				"dHlwZU9mKHZhbHVlKSl7CgkJCWNhc2UgJ29iamVjdCc6CgkJCQl2YXIgRiA9IGZ1bmN0aW9uKCl7fTsK",
				"CQkJCUYucHJvdG90eXBlID0gdmFsdWU7CgkJCQlvYmplY3Rba2V5XSA9IHJlc2V0KG5ldyBGKTsKCQkJ",
				"YnJlYWs7CgkJCWNhc2UgJ2FycmF5Jzogb2JqZWN0W2tleV0gPSB2YWx1ZS5jbG9uZSgpOyBicmVhazsK",
				"CQl9Cgl9CglyZXR1cm4gb2JqZWN0Owp9OwoKdmFyIHdyYXAgPSBmdW5jdGlvbihzZWxmLCBrZXksIG1l",
				"dGhvZCl7CglpZiAobWV0aG9kLiRvcmlnaW4pIG1ldGhvZCA9IG1ldGhvZC4kb3JpZ2luOwoJdmFyIHdy",
				"YXBwZXIgPSBmdW5jdGlvbigpewoJCWlmIChtZXRob2QuJHByb3RlY3RlZCAmJiB0aGlzLiRjYWxsZXIg",
				"PT0gbnVsbCkgdGhyb3cgbmV3IEVycm9yKCdUaGUgbWV0aG9kICInICsga2V5ICsgJyIgY2Fubm90IGJl",
				"IGNhbGxlZC4nKTsKCQl2YXIgY2FsbGVyID0gdGhpcy5jYWxsZXIsIGN1cnJlbnQgPSB0aGlzLiRjYWxs",
				"ZXI7CgkJdGhpcy5jYWxsZXIgPSBjdXJyZW50OyB0aGlzLiRjYWxsZXIgPSB3cmFwcGVyOwoJCXZhciBy",
				"ZXN1bHQgPSBtZXRob2QuYXBwbHkodGhpcywgYXJndW1lbnRzKTsKCQl0aGlzLiRjYWxsZXIgPSBjdXJy",
				"ZW50OyB0aGlzLmNhbGxlciA9IGNhbGxlcjsKCQlyZXR1cm4gcmVzdWx0OwoJfS5leHRlbmQoeyRvd25l",
				"cjogc2VsZiwgJG9yaWdpbjogbWV0aG9kLCAkbmFtZToga2V5fSk7CglyZXR1cm4gd3JhcHBlcjsKfTsK",
				"CnZhciBpbXBsZW1lbnQgPSBmdW5jdGlvbihrZXksIHZhbHVlLCByZXRhaW4pewoJaWYgKENsYXNzLk11",
				"dGF0b3JzLmhhc093blByb3BlcnR5KGtleSkpewoJCXZhbHVlID0gQ2xhc3MuTXV0YXRvcnNba2V5XS5j",
				"YWxsKHRoaXMsIHZhbHVlKTsKCQlpZiAodmFsdWUgPT0gbnVsbCkgcmV0dXJuIHRoaXM7Cgl9CgoJaWYg",
				"KHR5cGVPZih2YWx1ZSkgPT0gJ2Z1bmN0aW9uJyl7CgkJaWYgKHZhbHVlLiRoaWRkZW4pIHJldHVybiB0",
				"aGlzOwoJCXRoaXMucHJvdG90eXBlW2tleV0gPSAocmV0YWluKSA/IHZhbHVlIDogd3JhcCh0aGlzLCBr",
				"ZXksIHZhbHVlKTsKCX0gZWxzZSB7CgkJT2JqZWN0Lm1lcmdlKHRoaXMucHJvdG90eXBlLCBrZXksIHZh",
				"bHVlKTsKCX0KCglyZXR1cm4gdGhpczsKfTsKCnZhciBnZXRJbnN0YW5jZSA9IGZ1bmN0aW9uKGtsYXNz",
				"KXsKCWtsYXNzLiRwcm90b3R5cGluZyA9IHRydWU7Cgl2YXIgcHJvdG8gPSBuZXcga2xhc3M7CglkZWxl",
				"dGUga2xhc3MuJHByb3RvdHlwaW5nOwoJcmV0dXJuIHByb3RvOwp9OwoKQ2xhc3MuaW1wbGVtZW50KCdp",
				"bXBsZW1lbnQnLCBpbXBsZW1lbnQub3ZlcmxvYWRTZXR0ZXIoKSk7CgpDbGFzcy5NdXRhdG9ycyA9IHsK",
				"CglFeHRlbmRzOiBmdW5jdGlvbihwYXJlbnQpewoJCXRoaXMucGFyZW50ID0gcGFyZW50OwoJCXRoaXMu",
				"cHJvdG90eXBlID0gZ2V0SW5zdGFuY2UocGFyZW50KTsKCX0sCgoJSW1wbGVtZW50czogZnVuY3Rpb24o",
				"aXRlbXMpewoJCUFycmF5LmZyb20oaXRlbXMpLmVhY2goZnVuY3Rpb24oaXRlbSl7CgkJCXZhciBpbnN0",
				"YW5jZSA9IG5ldyBpdGVtOwoJCQlmb3IgKHZhciBrZXkgaW4gaW5zdGFuY2UpIGltcGxlbWVudC5jYWxs",
				"KHRoaXMsIGtleSwgaW5zdGFuY2Vba2V5XSwgdHJ1ZSk7CgkJfSwgdGhpcyk7Cgl9Cn07Cgp9KSgpOwoK",
				"Ci8qCi0tLQoKbmFtZTogQ2xhc3MuRXh0cmFzCgpkZXNjcmlwdGlvbjogQ29udGFpbnMgVXRpbGl0eSBD",
				"bGFzc2VzIHRoYXQgY2FuIGJlIGltcGxlbWVudGVkIGludG8geW91ciBvd24gQ2xhc3NlcyB0byBlYXNl",
				"IHRoZSBleGVjdXRpb24gb2YgbWFueSBjb21tb24gdGFza3MuCgpsaWNlbnNlOiBNSVQtc3R5bGUgbGlj",
				"ZW5zZS4KCnJlcXVpcmVzOiBDbGFzcwoKcHJvdmlkZXM6IFtDbGFzcy5FeHRyYXMsIENoYWluLCBFdmVu",
				"dHMsIE9wdGlvbnNdCgouLi4KKi8KCihmdW5jdGlvbigpewoKdGhpcy5DaGFpbiA9IG5ldyBDbGFzcyh7",
				"CgoJJGNoYWluOiBbXSwKCgljaGFpbjogZnVuY3Rpb24oKXsKCQl0aGlzLiRjaGFpbi5hcHBlbmQoQXJy",
				"YXkuZmxhdHRlbihhcmd1bWVudHMpKTsKCQlyZXR1cm4gdGhpczsKCX0sCgoJY2FsbENoYWluOiBmdW5j",
				"dGlvbigpewoJCXJldHVybiAodGhpcy4kY2hhaW4ubGVuZ3RoKSA/IHRoaXMuJGNoYWluLnNoaWZ0KCku",
				"YXBwbHkodGhpcywgYXJndW1lbnRzKSA6IGZhbHNlOwoJfSwKCgljbGVhckNoYWluOiBmdW5jdGlvbigp",
				"ewoJCXRoaXMuJGNoYWluLmVtcHR5KCk7CgkJcmV0dXJuIHRoaXM7Cgl9Cgp9KTsKCnZhciByZW1vdmVP",
				"biA9IGZ1bmN0aW9uKHN0cmluZyl7CglyZXR1cm4gc3RyaW5nLnJlcGxhY2UoL15vbihbQS1aXSkvLCBm",
				"dW5jdGlvbihmdWxsLCBmaXJzdCl7CgkJcmV0dXJuIGZpcnN0LnRvTG93ZXJDYXNlKCk7Cgl9KTsKfTsK",
				"CnRoaXMuRXZlbnRzID0gbmV3IENsYXNzKHsKCgkkZXZlbnRzOiB7fSwKCglhZGRFdmVudDogZnVuY3Rp",
				"b24odHlwZSwgZm4sIGludGVybmFsKXsKCQl0eXBlID0gcmVtb3ZlT24odHlwZSk7CgoJCS8qPDEuMmNv",
				"bXBhdD4qLwoJCWlmIChmbiA9PSAkZW1wdHkpIHJldHVybiB0aGlzOwoJCS8qPC8xLjJjb21wYXQ+Ki8K",
				"CgkJdGhpcy4kZXZlbnRzW3R5cGVdID0gKHRoaXMuJGV2ZW50c1t0eXBlXSB8fCBbXSkuaW5jbHVkZShm",
				"bik7CgkJaWYgKGludGVybmFsKSBmbi5pbnRlcm5hbCA9IHRydWU7CgkJcmV0dXJuIHRoaXM7Cgl9LAoK",
				"CWFkZEV2ZW50czogZnVuY3Rpb24oZXZlbnRzKXsKCQlmb3IgKHZhciB0eXBlIGluIGV2ZW50cykgdGhp",
				"cy5hZGRFdmVudCh0eXBlLCBldmVudHNbdHlwZV0pOwoJCXJldHVybiB0aGlzOwoJfSwKCglmaXJlRXZl",
				"bnQ6IGZ1bmN0aW9uKHR5cGUsIGFyZ3MsIGRlbGF5KXsKCQl0eXBlID0gcmVtb3ZlT24odHlwZSk7CgkJ",
				"dmFyIGV2ZW50cyA9IHRoaXMuJGV2ZW50c1t0eXBlXTsKCQlpZiAoIWV2ZW50cykgcmV0dXJuIHRoaXM7",
				"CgkJYXJncyA9IEFycmF5LmZyb20oYXJncyk7CgkJZXZlbnRzLmVhY2goZnVuY3Rpb24oZm4pewoJCQlp",
				"ZiAoZGVsYXkpIGZuLmRlbGF5KGRlbGF5LCB0aGlzLCBhcmdzKTsKCQkJZWxzZSBmbi5hcHBseSh0aGlz",
				"LCBhcmdzKTsKCQl9LCB0aGlzKTsKCQlyZXR1cm4gdGhpczsKCX0sCgoJcmVtb3ZlRXZlbnQ6IGZ1bmN0",
				"aW9uKHR5cGUsIGZuKXsKCQl0eXBlID0gcmVtb3ZlT24odHlwZSk7CgkJdmFyIGV2ZW50cyA9IHRoaXMu",
				"JGV2ZW50c1t0eXBlXTsKCQlpZiAoZXZlbnRzICYmICFmbi5pbnRlcm5hbCl7CgkJCXZhciBpbmRleCA9",
				"ICBldmVudHMuaW5kZXhPZihmbik7CgkJCWlmIChpbmRleCAhPSAtMSkgZGVsZXRlIGV2ZW50c1tpbmRl",
				"eF07CgkJfQoJCXJldHVybiB0aGlzOwoJfSwKCglyZW1vdmVFdmVudHM6IGZ1bmN0aW9uKGV2ZW50cyl7",
				"CgkJdmFyIHR5cGU7CgkJaWYgKHR5cGVPZihldmVudHMpID09ICdvYmplY3QnKXsKCQkJZm9yICh0eXBl",
				"IGluIGV2ZW50cykgdGhpcy5yZW1vdmVFdmVudCh0eXBlLCBldmVudHNbdHlwZV0pOwoJCQlyZXR1cm4g",
				"dGhpczsKCQl9CgkJaWYgKGV2ZW50cykgZXZlbnRzID0gcmVtb3ZlT24oZXZlbnRzKTsKCQlmb3IgKHR5",
				"cGUgaW4gdGhpcy4kZXZlbnRzKXsKCQkJaWYgKGV2ZW50cyAmJiBldmVudHMgIT0gdHlwZSkgY29udGlu",
				"dWU7CgkJCXZhciBmbnMgPSB0aGlzLiRldmVudHNbdHlwZV07CgkJCWZvciAodmFyIGkgPSBmbnMubGVu",
				"Z3RoOyBpLS07KSBpZiAoaSBpbiBmbnMpewoJCQkJdGhpcy5yZW1vdmVFdmVudCh0eXBlLCBmbnNbaV0p",
				"OwoJCQl9CgkJfQoJCXJldHVybiB0aGlzOwoJfQoKfSk7Cgp0aGlzLk9wdGlvbnMgPSBuZXcgQ2xhc3Mo",
				"ewoKCXNldE9wdGlvbnM6IGZ1bmN0aW9uKCl7CgkJdmFyIG9wdGlvbnMgPSB0aGlzLm9wdGlvbnMgPSBP",
				"YmplY3QubWVyZ2UuYXBwbHkobnVsbCwgW3t9LCB0aGlzLm9wdGlvbnNdLmFwcGVuZChhcmd1bWVudHMp",
				"KTsKCQlpZiAodGhpcy5hZGRFdmVudCkgZm9yICh2YXIgb3B0aW9uIGluIG9wdGlvbnMpewoJCQlpZiAo",
				"dHlwZU9mKG9wdGlvbnNbb3B0aW9uXSkgIT0gJ2Z1bmN0aW9uJyB8fCAhKC9eb25bQS1aXS8pLnRlc3Qo",
				"b3B0aW9uKSkgY29udGludWU7CgkJCXRoaXMuYWRkRXZlbnQob3B0aW9uLCBvcHRpb25zW29wdGlvbl0p",
				"OwoJCQlkZWxldGUgb3B0aW9uc1tvcHRpb25dOwoJCX0KCQlyZXR1cm4gdGhpczsKCX0KCn0pOwoKfSko",
				"KTsKCgovKgotLS0KbmFtZTogU2xpY2suUGFyc2VyCmRlc2NyaXB0aW9uOiBTdGFuZGFsb25lIENTUzMg",
				"U2VsZWN0b3IgcGFyc2VyCnByb3ZpZGVzOiBTbGljay5QYXJzZXIKLi4uCiovCgo7KGZ1bmN0aW9uKCl7",
				"Cgp2YXIgcGFyc2VkLAoJc2VwYXJhdG9ySW5kZXgsCgljb21iaW5hdG9ySW5kZXgsCglyZXZlcnNlZCwK",
				"CWNhY2hlID0ge30sCglyZXZlcnNlQ2FjaGUgPSB7fSwKCXJlVW5lc2NhcGUgPSAvXFwvZzsKCnZhciBw",
				"YXJzZSA9IGZ1bmN0aW9uKGV4cHJlc3Npb24sIGlzUmV2ZXJzZWQpewoJaWYgKGV4cHJlc3Npb24gPT0g",
				"bnVsbCkgcmV0dXJuIG51bGw7CglpZiAoZXhwcmVzc2lvbi5TbGljayA9PT0gdHJ1ZSkgcmV0dXJuIGV4",
				"cHJlc3Npb247CglleHByZXNzaW9uID0gKCcnICsgZXhwcmVzc2lvbikucmVwbGFjZSgvXlxzK3xccysk",
				"L2csICcnKTsKCXJldmVyc2VkID0gISFpc1JldmVyc2VkOwoJdmFyIGN1cnJlbnRDYWNoZSA9IChyZXZl",
				"cnNlZCkgPyByZXZlcnNlQ2FjaGUgOiBjYWNoZTsKCWlmIChjdXJyZW50Q2FjaGVbZXhwcmVzc2lvbl0p",
				"IHJldHVybiBjdXJyZW50Q2FjaGVbZXhwcmVzc2lvbl07CglwYXJzZWQgPSB7CgkJU2xpY2s6IHRydWUs",
				"CgkJZXhwcmVzc2lvbnM6IFtdLAoJCXJhdzogZXhwcmVzc2lvbiwKCQlyZXZlcnNlOiBmdW5jdGlvbigp",
				"ewoJCQlyZXR1cm4gcGFyc2UodGhpcy5yYXcsIHRydWUpOwoJCX0KCX07CglzZXBhcmF0b3JJbmRleCA9",
				"IC0xOwoJd2hpbGUgKGV4cHJlc3Npb24gIT0gKGV4cHJlc3Npb24gPSBleHByZXNzaW9uLnJlcGxhY2Uo",
				"cmVnZXhwLCBwYXJzZXIpKSk7CglwYXJzZWQubGVuZ3RoID0gcGFyc2VkLmV4cHJlc3Npb25zLmxlbmd0",
				"aDsKCXJldHVybiBjdXJyZW50Q2FjaGVbcGFyc2VkLnJhd10gPSAocmV2ZXJzZWQpID8gcmV2ZXJzZShw",
				"YXJzZWQpIDogcGFyc2VkOwp9OwoKdmFyIHJldmVyc2VDb21iaW5hdG9yID0gZnVuY3Rpb24oY29tYmlu",
				"YXRvcil7CglpZiAoY29tYmluYXRvciA9PT0gJyEnKSByZXR1cm4gJyAnOwoJZWxzZSBpZiAoY29tYmlu",
				"YXRvciA9PT0gJyAnKSByZXR1cm4gJyEnOwoJZWxzZSBpZiAoKC9eIS8pLnRlc3QoY29tYmluYXRvcikp",
				"IHJldHVybiBjb21iaW5hdG9yLnJlcGxhY2UoL14hLywgJycpOwoJZWxzZSByZXR1cm4gJyEnICsgY29t",
				"YmluYXRvcjsKfTsKCnZhciByZXZlcnNlID0gZnVuY3Rpb24oZXhwcmVzc2lvbil7Cgl2YXIgZXhwcmVz",
				"c2lvbnMgPSBleHByZXNzaW9uLmV4cHJlc3Npb25zOwoJZm9yICh2YXIgaSA9IDA7IGkgPCBleHByZXNz",
				"aW9ucy5sZW5ndGg7IGkrKyl7CgkJdmFyIGV4cCA9IGV4cHJlc3Npb25zW2ldOwoJCXZhciBsYXN0ID0g",
				"e3BhcnRzOiBbXSwgdGFnOiAnKicsIGNvbWJpbmF0b3I6IHJldmVyc2VDb21iaW5hdG9yKGV4cFswXS5j",
				"b21iaW5hdG9yKX07CgoJCWZvciAodmFyIGogPSAwOyBqIDwgZXhwLmxlbmd0aDsgaisrKXsKCQkJdmFy",
				"IGNleHAgPSBleHBbal07CgkJCWlmICghY2V4cC5yZXZlcnNlQ29tYmluYXRvcikgY2V4cC5yZXZlcnNl",
				"Q29tYmluYXRvciA9ICcgJzsKCQkJY2V4cC5jb21iaW5hdG9yID0gY2V4cC5yZXZlcnNlQ29tYmluYXRv",
				"cjsKCQkJZGVsZXRlIGNleHAucmV2ZXJzZUNvbWJpbmF0b3I7CgkJfQoKCQlleHAucmV2ZXJzZSgpLnB1",
				"c2gobGFzdCk7Cgl9CglyZXR1cm4gZXhwcmVzc2lvbjsKfTsKCnZhciBlc2NhcGVSZWdFeHAgPSBmdW5j",
				"dGlvbihzdHJpbmcpey8vIENyZWRpdDogWFJlZ0V4cCAwLjYuMSAoYykgMjAwNy0yMDA4IFN0ZXZlbiBM",
				"ZXZpdGhhbiA8aHR0cDovL3N0ZXZlbmxldml0aGFuLmNvbS9yZWdleC94cmVnZXhwLz4gTUlUIExpY2Vu",
				"c2UKCXJldHVybiBzdHJpbmcucmVwbGFjZSgvWy1bXF17fSgpKis/LlxcXiR8LCNcc10vZywgZnVuY3Rp",
				"b24obWF0Y2gpewoJCXJldHVybiAnXFwnICsgbWF0Y2g7Cgl9KTsKfTsKCnZhciByZWdleHAgPSBuZXcg",
				"UmVnRXhwKAovKgojIS91c3IvYmluL2VudiBydWJ5CnB1dHMgIlx0XHQiICsgREFUQS5yZWFkLmdzdWIo",
				"L1woXD94XCl8XHMrIy4qJHxccyt8XFwkfFxcbi8sJycpCl9fRU5EX18KCSIoP3gpXig/OlwKCSAgXFxz",
				"KiAoICwgKSBcXHMqICAgICAgICAgICAgICAgIyBTZXBhcmF0b3IgICAgICAgICAgXG5cCgl8IFxccyog",
				"KCA8Y29tYmluYXRvcj4rICkgXFxzKiAgICMgQ29tYmluYXRvciAgICAgICAgIFxuXAoJfCAgICAgICgg",
				"XFxzKyApICAgICAgICAgICAgICAgICAjIENvbWJpbmF0b3JDaGlsZHJlbiBcblwKCXwgICAgICAoIDx1",
				"bmljb2RlPisgfCBcXCogKSAgICAgIyBUYWcgICAgICAgICAgICAgICAgXG5cCgl8IFxcIyAgKCA8dW5p",
				"Y29kZT4rICAgICAgICkgICAgICMgSUQgICAgICAgICAgICAgICAgIFxuXAoJfCBcXC4gICggPHVuaWNv",
				"ZGU+KyAgICAgICApICAgICAjIENsYXNzTmFtZSAgICAgICAgICBcblwKCXwgICAgICAgICAgICAgICAg",
				"ICAgICAgICAgICAgICAgIyBBdHRyaWJ1dGUgICAgICAgICAgXG5cCglcXFsgIFwKCQlcXHMqICg8dW5p",
				"Y29kZTE+KykgICg/OiAgXAoJCQlcXHMqIChbKl4kIX58XT89KSAgKD86ICBcCgkJCQlcXHMqICg/OlwK",
				"CQkJCQkoW1wiJ10/KSguKj8pXFw5IFwKCQkJCSlcCgkJCSkgIFwKCQkpPyAgXFxzKiAgXAoJXFxdKD8h",
				"XFxdKSBcblwKCXwgICA6KyAoIDx1bmljb2RlPisgKSg/OlwKCVxcKCAoPzpcCgkJKD86KFtcIiddKShb",
				"XlxcMTJdKilcXDEyKXwoKD86XFwoW14pXStcXCl8W14oKV0qKSspXAoJKSBcXClcCgkpP1wKCSkiCiov",
				"CgkiXig/OlxccyooLClcXHMqfFxccyooPGNvbWJpbmF0b3I+KylcXHMqfChcXHMrKXwoPHVuaWNvZGU+",
				"K3xcXCopfFxcIyg8dW5pY29kZT4rKXxcXC4oPHVuaWNvZGU+Kyl8XFxbXFxzKig8dW5pY29kZTE+Kyko",
				"PzpcXHMqKFsqXiQhfnxdPz0pKD86XFxzKig/OihbXCInXT8pKC4qPylcXDkpKSk/XFxzKlxcXSg/IVxc",
				"XSl8KDorKSg8dW5pY29kZT4rKSg/OlxcKCg/Oig/OihbXCInXSkoW15cXDEzXSopXFwxMyl8KCg/Olxc",
				"KFteKV0rXFwpfFteKCldKikrKSlcXCkpPykiCgkucmVwbGFjZSgvPGNvbWJpbmF0b3I+LywgJ1snICsg",
				"ZXNjYXBlUmVnRXhwKCI+K35gIUAkJV4mPXt9XFw7PC8iKSArICddJykKCS5yZXBsYWNlKC88dW5pY29k",
				"ZT4vZywgJyg/OltcXHdcXHUwMGExLVxcdUZGRkYtXXxcXFxcW15cXHMwLTlhLWZdKScpCgkucmVwbGFj",
				"ZSgvPHVuaWNvZGUxPi9nLCAnKD86WzpcXHdcXHUwMGExLVxcdUZGRkYtXXxcXFxcW15cXHMwLTlhLWZd",
				"KScpCik7CgpmdW5jdGlvbiBwYXJzZXIoCglyYXdNYXRjaCwKCglzZXBhcmF0b3IsCgljb21iaW5hdG9y",
				"LAoJY29tYmluYXRvckNoaWxkcmVuLAoKCXRhZ05hbWUsCglpZCwKCWNsYXNzTmFtZSwKCglhdHRyaWJ1",
				"dGVLZXksCglhdHRyaWJ1dGVPcGVyYXRvciwKCWF0dHJpYnV0ZVF1b3RlLAoJYXR0cmlidXRlVmFsdWUs",
				"CgoJcHNldWRvTWFya2VyLAoJcHNldWRvQ2xhc3MsCglwc2V1ZG9RdW90ZSwKCXBzZXVkb0NsYXNzUXVv",
				"dGVkVmFsdWUsCglwc2V1ZG9DbGFzc1ZhbHVlCil7CglpZiAoc2VwYXJhdG9yIHx8IHNlcGFyYXRvcklu",
				"ZGV4ID09PSAtMSl7CgkJcGFyc2VkLmV4cHJlc3Npb25zWysrc2VwYXJhdG9ySW5kZXhdID0gW107CgkJ",
				"Y29tYmluYXRvckluZGV4ID0gLTE7CgkJaWYgKHNlcGFyYXRvcikgcmV0dXJuICcnOwoJfQoKCWlmIChj",
				"b21iaW5hdG9yIHx8IGNvbWJpbmF0b3JDaGlsZHJlbiB8fCBjb21iaW5hdG9ySW5kZXggPT09IC0xKXsK",
				"CQljb21iaW5hdG9yID0gY29tYmluYXRvciB8fCAnICc7CgkJdmFyIGN1cnJlbnRTZXBhcmF0b3IgPSBw",
				"YXJzZWQuZXhwcmVzc2lvbnNbc2VwYXJhdG9ySW5kZXhdOwoJCWlmIChyZXZlcnNlZCAmJiBjdXJyZW50",
				"U2VwYXJhdG9yW2NvbWJpbmF0b3JJbmRleF0pCgkJCWN1cnJlbnRTZXBhcmF0b3JbY29tYmluYXRvcklu",
				"ZGV4XS5yZXZlcnNlQ29tYmluYXRvciA9IHJldmVyc2VDb21iaW5hdG9yKGNvbWJpbmF0b3IpOwoJCWN1",
				"cnJlbnRTZXBhcmF0b3JbKytjb21iaW5hdG9ySW5kZXhdID0ge2NvbWJpbmF0b3I6IGNvbWJpbmF0b3Is",
				"IHRhZzogJyonfTsKCX0KCgl2YXIgY3VycmVudFBhcnNlZCA9IHBhcnNlZC5leHByZXNzaW9uc1tzZXBh",
				"cmF0b3JJbmRleF1bY29tYmluYXRvckluZGV4XTsKCglpZiAodGFnTmFtZSl7CgkJY3VycmVudFBhcnNl",
				"ZC50YWcgPSB0YWdOYW1lLnJlcGxhY2UocmVVbmVzY2FwZSwgJycpOwoKCX0gZWxzZSBpZiAoaWQpewoJ",
				"CWN1cnJlbnRQYXJzZWQuaWQgPSBpZC5yZXBsYWNlKHJlVW5lc2NhcGUsICcnKTsKCgl9IGVsc2UgaWYg",
				"KGNsYXNzTmFtZSl7CgkJY2xhc3NOYW1lID0gY2xhc3NOYW1lLnJlcGxhY2UocmVVbmVzY2FwZSwgJycp",
				"OwoKCQlpZiAoIWN1cnJlbnRQYXJzZWQuY2xhc3NMaXN0KSBjdXJyZW50UGFyc2VkLmNsYXNzTGlzdCA9",
				"IFtdOwoJCWlmICghY3VycmVudFBhcnNlZC5jbGFzc2VzKSBjdXJyZW50UGFyc2VkLmNsYXNzZXMgPSBb",
				"XTsKCQljdXJyZW50UGFyc2VkLmNsYXNzTGlzdC5wdXNoKGNsYXNzTmFtZSk7CgkJY3VycmVudFBhcnNl",
				"ZC5jbGFzc2VzLnB1c2goewoJCQl2YWx1ZTogY2xhc3NOYW1lLAoJCQlyZWdleHA6IG5ldyBSZWdFeHAo",
				"JyhefFxccyknICsgZXNjYXBlUmVnRXhwKGNsYXNzTmFtZSkgKyAnKFxcc3wkKScpCgkJfSk7CgoJfSBl",
				"bHNlIGlmIChwc2V1ZG9DbGFzcyl7CgkJcHNldWRvQ2xhc3NWYWx1ZSA9IHBzZXVkb0NsYXNzVmFsdWUg",
				"fHwgcHNldWRvQ2xhc3NRdW90ZWRWYWx1ZTsKCQlwc2V1ZG9DbGFzc1ZhbHVlID0gcHNldWRvQ2xhc3NW",
				"YWx1ZSA/IHBzZXVkb0NsYXNzVmFsdWUucmVwbGFjZShyZVVuZXNjYXBlLCAnJykgOiBudWxsOwoKCQlp",
				"ZiAoIWN1cnJlbnRQYXJzZWQucHNldWRvcykgY3VycmVudFBhcnNlZC5wc2V1ZG9zID0gW107CgkJY3Vy",
				"cmVudFBhcnNlZC5wc2V1ZG9zLnB1c2goewoJCQlrZXk6IHBzZXVkb0NsYXNzLnJlcGxhY2UocmVVbmVz",
				"Y2FwZSwgJycpLAoJCQl2YWx1ZTogcHNldWRvQ2xhc3NWYWx1ZSwKCQkJdHlwZTogcHNldWRvTWFya2Vy",
				"Lmxlbmd0aCA9PSAxID8gJ2NsYXNzJyA6ICdlbGVtZW50JwoJCX0pOwoKCX0gZWxzZSBpZiAoYXR0cmli",
				"dXRlS2V5KXsKCQlhdHRyaWJ1dGVLZXkgPSBhdHRyaWJ1dGVLZXkucmVwbGFjZShyZVVuZXNjYXBlLCAn",
				"Jyk7CgkJYXR0cmlidXRlVmFsdWUgPSAoYXR0cmlidXRlVmFsdWUgfHwgJycpLnJlcGxhY2UocmVVbmVz",
				"Y2FwZSwgJycpOwoKCQl2YXIgdGVzdCwgcmVnZXhwOwoKCQlzd2l0Y2ggKGF0dHJpYnV0ZU9wZXJhdG9y",
				"KXsKCQkJY2FzZSAnXj0nIDogcmVnZXhwID0gbmV3IFJlZ0V4cCggICAgICAgJ14nKyBlc2NhcGVSZWdF",
				"eHAoYXR0cmlidXRlVmFsdWUpICAgICAgICAgICAgKTsgYnJlYWs7CgkJCWNhc2UgJyQ9JyA6IHJlZ2V4",
				"cCA9IG5ldyBSZWdFeHAoICAgICAgICAgICAgZXNjYXBlUmVnRXhwKGF0dHJpYnV0ZVZhbHVlKSArJyQn",
				"ICAgICAgICk7IGJyZWFrOwoJCQljYXNlICd+PScgOiByZWdleHAgPSBuZXcgUmVnRXhwKCAnKF58XFxz",
				"KScrIGVzY2FwZVJlZ0V4cChhdHRyaWJ1dGVWYWx1ZSkgKycoXFxzfCQpJyApOyBicmVhazsKCQkJY2Fz",
				"ZSAnfD0nIDogcmVnZXhwID0gbmV3IFJlZ0V4cCggICAgICAgJ14nKyBlc2NhcGVSZWdFeHAoYXR0cmli",
				"dXRlVmFsdWUpICsnKC18JCknICAgKTsgYnJlYWs7CgkJCWNhc2UgICc9JyA6IHRlc3QgPSBmdW5jdGlv",
				"bih2YWx1ZSl7CgkJCQlyZXR1cm4gYXR0cmlidXRlVmFsdWUgPT0gdmFsdWU7CgkJCX07IGJyZWFrOwoJ",
				"CQljYXNlICcqPScgOiB0ZXN0ID0gZnVuY3Rpb24odmFsdWUpewoJCQkJcmV0dXJuIHZhbHVlICYmIHZh",
				"bHVlLmluZGV4T2YoYXR0cmlidXRlVmFsdWUpID4gLTE7CgkJCX07IGJyZWFrOwoJCQljYXNlICchPScg",
				"OiB0ZXN0ID0gZnVuY3Rpb24odmFsdWUpewoJCQkJcmV0dXJuIGF0dHJpYnV0ZVZhbHVlICE9IHZhbHVl",
				"OwoJCQl9OyBicmVhazsKCQkJZGVmYXVsdCAgIDogdGVzdCA9IGZ1bmN0aW9uKHZhbHVlKXsKCQkJCXJl",
				"dHVybiAhIXZhbHVlOwoJCQl9OwoJCX0KCgkJaWYgKGF0dHJpYnV0ZVZhbHVlID09ICcnICYmICgvXlsq",
				"JF5dPSQvKS50ZXN0KGF0dHJpYnV0ZU9wZXJhdG9yKSkgdGVzdCA9IGZ1bmN0aW9uKCl7CgkJCXJldHVy",
				"biBmYWxzZTsKCQl9OwoKCQlpZiAoIXRlc3QpIHRlc3QgPSBmdW5jdGlvbih2YWx1ZSl7CgkJCXJldHVy",
				"biB2YWx1ZSAmJiByZWdleHAudGVzdCh2YWx1ZSk7CgkJfTsKCgkJaWYgKCFjdXJyZW50UGFyc2VkLmF0",
				"dHJpYnV0ZXMpIGN1cnJlbnRQYXJzZWQuYXR0cmlidXRlcyA9IFtdOwoJCWN1cnJlbnRQYXJzZWQuYXR0",
				"cmlidXRlcy5wdXNoKHsKCQkJa2V5OiBhdHRyaWJ1dGVLZXksCgkJCW9wZXJhdG9yOiBhdHRyaWJ1dGVP",
				"cGVyYXRvciwKCQkJdmFsdWU6IGF0dHJpYnV0ZVZhbHVlLAoJCQl0ZXN0OiB0ZXN0CgkJfSk7CgoJfQoK",
				"CXJldHVybiAnJzsKfTsKCi8vIFNsaWNrIE5TCgp2YXIgU2xpY2sgPSAodGhpcy5TbGljayB8fCB7fSk7",
				"CgpTbGljay5wYXJzZSA9IGZ1bmN0aW9uKGV4cHJlc3Npb24pewoJcmV0dXJuIHBhcnNlKGV4cHJlc3Np",
				"b24pOwp9OwoKU2xpY2suZXNjYXBlUmVnRXhwID0gZXNjYXBlUmVnRXhwOwoKaWYgKCF0aGlzLlNsaWNr",
				"KSB0aGlzLlNsaWNrID0gU2xpY2s7Cgp9KS5hcHBseSgvKjxDb21tb25KUz4qLyh0eXBlb2YgZXhwb3J0",
				"cyAhPSAndW5kZWZpbmVkJykgPyBleHBvcnRzIDogLyo8L0NvbW1vbkpTPiovdGhpcyk7CgoKLyoKLS0t",
				"Cm5hbWU6IFNsaWNrLkZpbmRlcgpkZXNjcmlwdGlvbjogVGhlIG5ldywgc3VwZXJmYXN0IGNzcyBzZWxl",
				"Y3RvciBlbmdpbmUuCnByb3ZpZGVzOiBTbGljay5GaW5kZXIKcmVxdWlyZXM6IFNsaWNrLlBhcnNlcgou",
				"Li4KKi8KCjsoZnVuY3Rpb24oKXsKCnZhciBsb2NhbCA9IHt9LAoJZmVhdHVyZXNDYWNoZSA9IHt9LAoJ",
				"dG9TdHJpbmcgPSBPYmplY3QucHJvdG90eXBlLnRvU3RyaW5nOwoKLy8gRmVhdHVyZSAvIEJ1ZyBkZXRl",
				"Y3Rpb24KCmxvY2FsLmlzTmF0aXZlQ29kZSA9IGZ1bmN0aW9uKGZuKXsKCXJldHVybiAoL1x7XHMqXFtu",
				"YXRpdmUgY29kZVxdXHMqXH0vKS50ZXN0KCcnICsgZm4pOwp9OwoKbG9jYWwuaXNYTUwgPSBmdW5jdGlv",
				"bihkb2N1bWVudCl7CglyZXR1cm4gKCEhZG9jdW1lbnQueG1sVmVyc2lvbikgfHwgKCEhZG9jdW1lbnQu",
				"eG1sKSB8fCAodG9TdHJpbmcuY2FsbChkb2N1bWVudCkgPT0gJ1tvYmplY3QgWE1MRG9jdW1lbnRdJykg",
				"fHwKCShkb2N1bWVudC5ub2RlVHlwZSA9PSA5ICYmIGRvY3VtZW50LmRvY3VtZW50RWxlbWVudC5ub2Rl",
				"TmFtZSAhPSAnSFRNTCcpOwp9OwoKbG9jYWwuc2V0RG9jdW1lbnQgPSBmdW5jdGlvbihkb2N1bWVudCl7",
				"CgoJLy8gY29udmVydCBlbGVtZW50cyAvIHdpbmRvdyBhcmd1bWVudHMgdG8gZG9jdW1lbnQuIGlmIGRv",
				"Y3VtZW50IGNhbm5vdCBiZSBleHRyYXBvbGF0ZWQsIHRoZSBmdW5jdGlvbiByZXR1cm5zLgoJdmFyIG5v",
				"ZGVUeXBlID0gZG9jdW1lbnQubm9kZVR5cGU7CglpZiAobm9kZVR5cGUgPT0gOSk7IC8vIGRvY3VtZW50",
				"CgllbHNlIGlmIChub2RlVHlwZSkgZG9jdW1lbnQgPSBkb2N1bWVudC5vd25lckRvY3VtZW50OyAvLyBu",
				"b2RlCgllbHNlIGlmIChkb2N1bWVudC5uYXZpZ2F0b3IpIGRvY3VtZW50ID0gZG9jdW1lbnQuZG9jdW1l",
				"bnQ7IC8vIHdpbmRvdwoJZWxzZSByZXR1cm47CgoJLy8gY2hlY2sgaWYgaXQncyB0aGUgb2xkIGRvY3Vt",
				"ZW50CgoJaWYgKHRoaXMuZG9jdW1lbnQgPT09IGRvY3VtZW50KSByZXR1cm47Cgl0aGlzLmRvY3VtZW50",
				"ID0gZG9jdW1lbnQ7CgoJLy8gY2hlY2sgaWYgd2UgaGF2ZSBkb25lIGZlYXR1cmUgZGV0ZWN0aW9uIG9u",
				"IHRoaXMgZG9jdW1lbnQgYmVmb3JlCgoJdmFyIHJvb3QgPSBkb2N1bWVudC5kb2N1bWVudEVsZW1lbnQs",
				"CgkJcm9vdFVpZCA9IHRoaXMuZ2V0VUlEWE1MKHJvb3QpLAoJCWZlYXR1cmVzID0gZmVhdHVyZXNDYWNo",
				"ZVtyb290VWlkXSwKCQlmZWF0dXJlOwoKCWlmIChmZWF0dXJlcyl7CgkJZm9yIChmZWF0dXJlIGluIGZl",
				"YXR1cmVzKXsKCQkJdGhpc1tmZWF0dXJlXSA9IGZlYXR1cmVzW2ZlYXR1cmVdOwoJCX0KCQlyZXR1cm47",
				"Cgl9CgoJZmVhdHVyZXMgPSBmZWF0dXJlc0NhY2hlW3Jvb3RVaWRdID0ge307CgoJZmVhdHVyZXMucm9v",
				"dCA9IHJvb3Q7CglmZWF0dXJlcy5pc1hNTERvY3VtZW50ID0gdGhpcy5pc1hNTChkb2N1bWVudCk7CgoJ",
				"ZmVhdHVyZXMuYnJva2VuU3RhckdFQlROCgk9IGZlYXR1cmVzLnN0YXJTZWxlY3RzQ2xvc2VkUVNBCgk9",
				"IGZlYXR1cmVzLmlkR2V0c05hbWUKCT0gZmVhdHVyZXMuYnJva2VuTWl4ZWRDYXNlUVNBCgk9IGZlYXR1",
				"cmVzLmJyb2tlbkdFQkNOCgk9IGZlYXR1cmVzLmJyb2tlbkNoZWNrZWRRU0EKCT0gZmVhdHVyZXMuYnJv",
				"a2VuRW1wdHlBdHRyaWJ1dGVRU0EKCT0gZmVhdHVyZXMuaXNIVE1MRG9jdW1lbnQKCT0gZmVhdHVyZXMu",
				"bmF0aXZlTWF0Y2hlc1NlbGVjdG9yCgk9IGZhbHNlOwoKCXZhciBzdGFyU2VsZWN0c0Nsb3NlZCwgc3Rh",
				"clNlbGVjdHNDb21tZW50cywKCQlicm9rZW5TZWNvbmRDbGFzc05hbWVHRUJDTiwgY2FjaGVkR2V0RWxl",
				"bWVudHNCeUNsYXNzTmFtZSwKCQlicm9rZW5Gb3JtQXR0cmlidXRlR2V0dGVyOwoKCXZhciBzZWxlY3Rl",
				"ZCwgaWQgPSAnc2xpY2tfdW5pcXVlaWQnOwoJdmFyIHRlc3ROb2RlID0gZG9jdW1lbnQuY3JlYXRlRWxl",
				"bWVudCgnZGl2Jyk7CgoJdmFyIHRlc3RSb290ID0gZG9jdW1lbnQuYm9keSB8fCBkb2N1bWVudC5nZXRF",
				"bGVtZW50c0J5VGFnTmFtZSgnYm9keScpWzBdIHx8IHJvb3Q7Cgl0ZXN0Um9vdC5hcHBlbmRDaGlsZCh0",
				"ZXN0Tm9kZSk7CgoJLy8gb24gbm9uLUhUTUwgZG9jdW1lbnRzIGlubmVySFRNTCBhbmQgZ2V0RWxlbWVu",
				"dHNCeUlkIGRvZXNudCB3b3JrIHByb3Blcmx5Cgl0cnkgewoJCXRlc3ROb2RlLmlubmVySFRNTCA9ICc8",
				"YSBpZD0iJytpZCsnIj48L2E+JzsKCQlmZWF0dXJlcy5pc0hUTUxEb2N1bWVudCA9ICEhZG9jdW1lbnQu",
				"Z2V0RWxlbWVudEJ5SWQoaWQpOwoJfSBjYXRjaChlKXt9OwoKCWlmIChmZWF0dXJlcy5pc0hUTUxEb2N1",
				"bWVudCl7CgoJCXRlc3ROb2RlLnN0eWxlLmRpc3BsYXkgPSAnbm9uZSc7CgoJCS8vIElFIHJldHVybnMg",
				"Y29tbWVudCBub2RlcyBmb3IgZ2V0RWxlbWVudHNCeVRhZ05hbWUoJyonKSBmb3Igc29tZSBkb2N1bWVu",
				"dHMKCQl0ZXN0Tm9kZS5hcHBlbmRDaGlsZChkb2N1bWVudC5jcmVhdGVDb21tZW50KCcnKSk7CgkJc3Rh",
				"clNlbGVjdHNDb21tZW50cyA9ICh0ZXN0Tm9kZS5nZXRFbGVtZW50c0J5VGFnTmFtZSgnKicpLmxlbmd0",
				"aCA+IDEpOwoKCQkvLyBJRSByZXR1cm5zIGNsb3NlZCBub2RlcyAoRUc6IjwvZm9vPiIpIGZvciBnZXRF",
				"bGVtZW50c0J5VGFnTmFtZSgnKicpIGZvciBzb21lIGRvY3VtZW50cwoJCXRyeSB7CgkJCXRlc3ROb2Rl",
				"LmlubmVySFRNTCA9ICdmb288L2Zvbz4nOwoJCQlzZWxlY3RlZCA9IHRlc3ROb2RlLmdldEVsZW1lbnRz",
				"QnlUYWdOYW1lKCcqJyk7CgkJCXN0YXJTZWxlY3RzQ2xvc2VkID0gKHNlbGVjdGVkICYmICEhc2VsZWN0",
				"ZWQubGVuZ3RoICYmIHNlbGVjdGVkWzBdLm5vZGVOYW1lLmNoYXJBdCgwKSA9PSAnLycpOwoJCX0gY2F0",
				"Y2goZSl7fTsKCgkJZmVhdHVyZXMuYnJva2VuU3RhckdFQlROID0gc3RhclNlbGVjdHNDb21tZW50cyB8",
				"fCBzdGFyU2VsZWN0c0Nsb3NlZDsKCgkJLy8gSUUgcmV0dXJucyBlbGVtZW50cyB3aXRoIHRoZSBuYW1l",
				"IGluc3RlYWQgb2YganVzdCBpZCBmb3IgZ2V0RWxlbWVudHNCeUlkIGZvciBzb21lIGRvY3VtZW50cwoJ",
				"CXRyeSB7CgkJCXRlc3ROb2RlLmlubmVySFRNTCA9ICc8YSBuYW1lPSInKyBpZCArJyI+PC9hPjxiIGlk",
				"PSInKyBpZCArJyI+PC9iPic7CgkJCWZlYXR1cmVzLmlkR2V0c05hbWUgPSBkb2N1bWVudC5nZXRFbGVt",
				"ZW50QnlJZChpZCkgPT09IHRlc3ROb2RlLmZpcnN0Q2hpbGQ7CgkJfSBjYXRjaChlKXt9OwoKCQlpZiAo",
				"dGVzdE5vZGUuZ2V0RWxlbWVudHNCeUNsYXNzTmFtZSl7CgoJCQkvLyBTYWZhcmkgMy4yIGdldEVsZW1l",
				"bnRzQnlDbGFzc05hbWUgY2FjaGVzIHJlc3VsdHMKCQkJdHJ5IHsKCQkJCXRlc3ROb2RlLmlubmVySFRN",
				"TCA9ICc8YSBjbGFzcz0iZiI+PC9hPjxhIGNsYXNzPSJiIj48L2E+JzsKCQkJCXRlc3ROb2RlLmdldEVs",
				"ZW1lbnRzQnlDbGFzc05hbWUoJ2InKS5sZW5ndGg7CgkJCQl0ZXN0Tm9kZS5maXJzdENoaWxkLmNsYXNz",
				"TmFtZSA9ICdiJzsKCQkJCWNhY2hlZEdldEVsZW1lbnRzQnlDbGFzc05hbWUgPSAodGVzdE5vZGUuZ2V0",
				"RWxlbWVudHNCeUNsYXNzTmFtZSgnYicpLmxlbmd0aCAhPSAyKTsKCQkJfSBjYXRjaChlKXt9OwoKCQkJ",
				"Ly8gT3BlcmEgOS42IGdldEVsZW1lbnRzQnlDbGFzc05hbWUgZG9lc250IGRldGVjdHMgdGhlIGNsYXNz",
				"IGlmIGl0cyBub3QgdGhlIGZpcnN0IG9uZQoJCQl0cnkgewoJCQkJdGVzdE5vZGUuaW5uZXJIVE1MID0g",
				"JzxhIGNsYXNzPSJhIj48L2E+PGEgY2xhc3M9ImYgYiBhIj48L2E+JzsKCQkJCWJyb2tlblNlY29uZENs",
				"YXNzTmFtZUdFQkNOID0gKHRlc3ROb2RlLmdldEVsZW1lbnRzQnlDbGFzc05hbWUoJ2EnKS5sZW5ndGgg",
				"IT0gMik7CgkJCX0gY2F0Y2goZSl7fTsKCgkJCWZlYXR1cmVzLmJyb2tlbkdFQkNOID0gY2FjaGVkR2V0",
				"RWxlbWVudHNCeUNsYXNzTmFtZSB8fCBicm9rZW5TZWNvbmRDbGFzc05hbWVHRUJDTjsKCQl9CgoJCWlm",
				"ICh0ZXN0Tm9kZS5xdWVyeVNlbGVjdG9yQWxsKXsKCQkJLy8gSUUgOCByZXR1cm5zIGNsb3NlZCBub2Rl",
				"cyAoRUc6IjwvZm9vPiIpIGZvciBxdWVyeVNlbGVjdG9yQWxsKCcqJykgZm9yIHNvbWUgZG9jdW1lbnRz",
				"CgkJCXRyeSB7CgkJCQl0ZXN0Tm9kZS5pbm5lckhUTUwgPSAnZm9vPC9mb28+JzsKCQkJCXNlbGVjdGVk",
				"ID0gdGVzdE5vZGUucXVlcnlTZWxlY3RvckFsbCgnKicpOwoJCQkJZmVhdHVyZXMuc3RhclNlbGVjdHND",
				"bG9zZWRRU0EgPSAoc2VsZWN0ZWQgJiYgISFzZWxlY3RlZC5sZW5ndGggJiYgc2VsZWN0ZWRbMF0ubm9k",
				"ZU5hbWUuY2hhckF0KDApID09ICcvJyk7CgkJCX0gY2F0Y2goZSl7fTsKCgkJCS8vIFNhZmFyaSAzLjIg",
				"cXVlcnlTZWxlY3RvckFsbCBkb2VzbnQgd29yayB3aXRoIG1peGVkY2FzZSBvbiBxdWlya3Ntb2RlCgkJ",
				"CXRyeSB7CgkJCQl0ZXN0Tm9kZS5pbm5lckhUTUwgPSAnPGEgY2xhc3M9Ik1pWCI+PC9hPic7CgkJCQlm",
				"ZWF0dXJlcy5icm9rZW5NaXhlZENhc2VRU0EgPSAhdGVzdE5vZGUucXVlcnlTZWxlY3RvckFsbCgnLk1p",
				"WCcpLmxlbmd0aDsKCQkJfSBjYXRjaChlKXt9OwoKCQkJLy8gV2Via2l0IGFuZCBPcGVyYSBkb250IHJl",
				"dHVybiBzZWxlY3RlZCBvcHRpb25zIG9uIHF1ZXJ5U2VsZWN0b3JBbGwKCQkJdHJ5IHsKCQkJCXRlc3RO",
				"b2RlLmlubmVySFRNTCA9ICc8c2VsZWN0PjxvcHRpb24gc2VsZWN0ZWQ9InNlbGVjdGVkIj5hPC9vcHRp",
				"b24+PC9zZWxlY3Q+JzsKCQkJCWZlYXR1cmVzLmJyb2tlbkNoZWNrZWRRU0EgPSAodGVzdE5vZGUucXVl",
				"cnlTZWxlY3RvckFsbCgnOmNoZWNrZWQnKS5sZW5ndGggPT0gMCk7CgkJCX0gY2F0Y2goZSl7fTsKCgkJ",
				"CS8vIElFIHJldHVybnMgaW5jb3JyZWN0IHJlc3VsdHMgZm9yIGF0dHJbKl4kXT0iIiBzZWxlY3RvcnMg",
				"b24gcXVlcnlTZWxlY3RvckFsbAoJCQl0cnkgewoJCQkJdGVzdE5vZGUuaW5uZXJIVE1MID0gJzxhIGNs",
				"YXNzPSIiPjwvYT4nOwoJCQkJZmVhdHVyZXMuYnJva2VuRW1wdHlBdHRyaWJ1dGVRU0EgPSAodGVzdE5v",
				"ZGUucXVlcnlTZWxlY3RvckFsbCgnW2NsYXNzKj0iIl0nKS5sZW5ndGggIT0gMCk7CgkJCX0gY2F0Y2go",
				"ZSl7fTsKCgkJfQoKCQkvLyBJRTYtNywgaWYgYSBmb3JtIGhhcyBhbiBpbnB1dCBvZiBpZCB4LCBmb3Jt",
				"LmdldEF0dHJpYnV0ZSh4KSByZXR1cm5zIGEgcmVmZXJlbmNlIHRvIHRoZSBpbnB1dAoJCXRyeSB7CgkJ",
				"CXRlc3ROb2RlLmlubmVySFRNTCA9ICc8Zm9ybSBhY3Rpb249InMiPjxpbnB1dCBpZD0iYWN0aW9uIi8+",
				"PC9mb3JtPic7CgkJCWJyb2tlbkZvcm1BdHRyaWJ1dGVHZXR0ZXIgPSAodGVzdE5vZGUuZmlyc3RDaGls",
				"ZC5nZXRBdHRyaWJ1dGUoJ2FjdGlvbicpICE9ICdzJyk7CgkJfSBjYXRjaChlKXt9OwoKCQkvLyBuYXRp",
				"dmUgbWF0Y2hlc1NlbGVjdG9yIGZ1bmN0aW9uCgoJCWZlYXR1cmVzLm5hdGl2ZU1hdGNoZXNTZWxlY3Rv",
				"ciA9IHJvb3QubWF0Y2hlc1NlbGVjdG9yIHx8IC8qcm9vdC5tc01hdGNoZXNTZWxlY3RvciB8fCovIHJv",
				"b3QubW96TWF0Y2hlc1NlbGVjdG9yIHx8IHJvb3Qud2Via2l0TWF0Y2hlc1NlbGVjdG9yOwoJCWlmIChm",
				"ZWF0dXJlcy5uYXRpdmVNYXRjaGVzU2VsZWN0b3IpIHRyeSB7CgkJCS8vIGlmIG1hdGNoZXNTZWxlY3Rv",
				"ciB0cm93cyBlcnJvcnMgb24gaW5jb3JyZWN0IHNpbnRheGVzIHdlIGNhbiB1c2UgaXQKCQkJZmVhdHVy",
				"ZXMubmF0aXZlTWF0Y2hlc1NlbGVjdG9yLmNhbGwocm9vdCwgJzpzbGljaycpOwoJCQlmZWF0dXJlcy5u",
				"YXRpdmVNYXRjaGVzU2VsZWN0b3IgPSBudWxsOwoJCX0gY2F0Y2goZSl7fTsKCgl9CgoJdHJ5IHsKCQly",
				"b290LnNsaWNrX2V4cGFuZG8gPSAxOwoJCWRlbGV0ZSByb290LnNsaWNrX2V4cGFuZG87CgkJZmVhdHVy",
				"ZXMuZ2V0VUlEID0gdGhpcy5nZXRVSURIVE1MOwoJfSBjYXRjaChlKSB7CgkJZmVhdHVyZXMuZ2V0VUlE",
				"ID0gdGhpcy5nZXRVSURYTUw7Cgl9CgoJdGVzdFJvb3QucmVtb3ZlQ2hpbGQodGVzdE5vZGUpOwoJdGVz",
				"dE5vZGUgPSBzZWxlY3RlZCA9IHRlc3RSb290ID0gbnVsbDsKCgkvLyBnZXRBdHRyaWJ1dGUKCglmZWF0",
				"dXJlcy5nZXRBdHRyaWJ1dGUgPSAoZmVhdHVyZXMuaXNIVE1MRG9jdW1lbnQgJiYgYnJva2VuRm9ybUF0",
				"dHJpYnV0ZUdldHRlcikgPyBmdW5jdGlvbihub2RlLCBuYW1lKXsKCQl2YXIgbWV0aG9kID0gdGhpcy5h",
				"dHRyaWJ1dGVHZXR0ZXJzW25hbWVdOwoJCWlmIChtZXRob2QpIHJldHVybiBtZXRob2QuY2FsbChub2Rl",
				"KTsKCQl2YXIgYXR0cmlidXRlTm9kZSA9IG5vZGUuZ2V0QXR0cmlidXRlTm9kZShuYW1lKTsKCQlyZXR1",
				"cm4gKGF0dHJpYnV0ZU5vZGUpID8gYXR0cmlidXRlTm9kZS5ub2RlVmFsdWUgOiBudWxsOwoJfSA6IGZ1",
				"bmN0aW9uKG5vZGUsIG5hbWUpewoJCXZhciBtZXRob2QgPSB0aGlzLmF0dHJpYnV0ZUdldHRlcnNbbmFt",
				"ZV07CgkJcmV0dXJuIChtZXRob2QpID8gbWV0aG9kLmNhbGwobm9kZSkgOiBub2RlLmdldEF0dHJpYnV0",
				"ZShuYW1lKTsKCX07CgoJLy8gaGFzQXR0cmlidXRlCgoJZmVhdHVyZXMuaGFzQXR0cmlidXRlID0gKHJv",
				"b3QgJiYgdGhpcy5pc05hdGl2ZUNvZGUocm9vdC5oYXNBdHRyaWJ1dGUpKSA/IGZ1bmN0aW9uKG5vZGUs",
				"IGF0dHJpYnV0ZSkgewoJCXJldHVybiBub2RlLmhhc0F0dHJpYnV0ZShhdHRyaWJ1dGUpOwoJfSA6IGZ1",
				"bmN0aW9uKG5vZGUsIGF0dHJpYnV0ZSkgewoJCW5vZGUgPSBub2RlLmdldEF0dHJpYnV0ZU5vZGUoYXR0",
				"cmlidXRlKTsKCQlyZXR1cm4gISEobm9kZSAmJiAobm9kZS5zcGVjaWZpZWQgfHwgbm9kZS5ub2RlVmFs",
				"dWUpKTsKCX07CgoJLy8gY29udGFpbnMKCS8vIEZJWE1FOiBBZGQgc3BlY3M6IGxvY2FsLmNvbnRhaW5z",
				"IHNob3VsZCBiZSBkaWZmZXJlbnQgZm9yIHhtbCBhbmQgaHRtbCBkb2N1bWVudHM/Cgl2YXIgbmF0aXZl",
				"Um9vdENvbnRhaW5zID0gcm9vdCAmJiB0aGlzLmlzTmF0aXZlQ29kZShyb290LmNvbnRhaW5zKSwKCQlu",
				"YXRpdmVEb2N1bWVudENvbnRhaW5zID0gZG9jdW1lbnQgJiYgdGhpcy5pc05hdGl2ZUNvZGUoZG9jdW1l",
				"bnQuY29udGFpbnMpOwoKCWZlYXR1cmVzLmNvbnRhaW5zID0gKG5hdGl2ZVJvb3RDb250YWlucyAmJiBu",
				"YXRpdmVEb2N1bWVudENvbnRhaW5zKSA/IGZ1bmN0aW9uKGNvbnRleHQsIG5vZGUpewoJCXJldHVybiBj",
				"b250ZXh0LmNvbnRhaW5zKG5vZGUpOwoJfSA6IChuYXRpdmVSb290Q29udGFpbnMgJiYgIW5hdGl2ZURv",
				"Y3VtZW50Q29udGFpbnMpID8gZnVuY3Rpb24oY29udGV4dCwgbm9kZSl7CgkJLy8gSUU4IGRvZXMgbm90",
				"IGhhdmUgLmNvbnRhaW5zIG9uIGRvY3VtZW50LgoJCXJldHVybiBjb250ZXh0ID09PSBub2RlIHx8ICgo",
				"Y29udGV4dCA9PT0gZG9jdW1lbnQpID8gZG9jdW1lbnQuZG9jdW1lbnRFbGVtZW50IDogY29udGV4dCku",
				"Y29udGFpbnMobm9kZSk7Cgl9IDogKHJvb3QgJiYgcm9vdC5jb21wYXJlRG9jdW1lbnRQb3NpdGlvbikg",
				"PyBmdW5jdGlvbihjb250ZXh0LCBub2RlKXsKCQlyZXR1cm4gY29udGV4dCA9PT0gbm9kZSB8fCAhIShj",
				"b250ZXh0LmNvbXBhcmVEb2N1bWVudFBvc2l0aW9uKG5vZGUpICYgMTYpOwoJfSA6IGZ1bmN0aW9uKGNv",
				"bnRleHQsIG5vZGUpewoJCWlmIChub2RlKSBkbyB7CgkJCWlmIChub2RlID09PSBjb250ZXh0KSByZXR1",
				"cm4gdHJ1ZTsKCQl9IHdoaWxlICgobm9kZSA9IG5vZGUucGFyZW50Tm9kZSkpOwoJCXJldHVybiBmYWxz",
				"ZTsKCX07CgoJLy8gZG9jdW1lbnQgb3JkZXIgc29ydGluZwoJLy8gY3JlZGl0cyB0byBTaXp6bGUgKGh0",
				"dHA6Ly9zaXp6bGVqcy5jb20vKQoKCWZlYXR1cmVzLmRvY3VtZW50U29ydGVyID0gKHJvb3QuY29tcGFy",
				"ZURvY3VtZW50UG9zaXRpb24pID8gZnVuY3Rpb24oYSwgYil7CgkJaWYgKCFhLmNvbXBhcmVEb2N1bWVu",
				"dFBvc2l0aW9uIHx8ICFiLmNvbXBhcmVEb2N1bWVudFBvc2l0aW9uKSByZXR1cm4gMDsKCQlyZXR1cm4g",
				"YS5jb21wYXJlRG9jdW1lbnRQb3NpdGlvbihiKSAmIDQgPyAtMSA6IGEgPT09IGIgPyAwIDogMTsKCX0g",
				"OiAoJ3NvdXJjZUluZGV4JyBpbiByb290KSA/IGZ1bmN0aW9uKGEsIGIpewoJCWlmICghYS5zb3VyY2VJ",
				"bmRleCB8fCAhYi5zb3VyY2VJbmRleCkgcmV0dXJuIDA7CgkJcmV0dXJuIGEuc291cmNlSW5kZXggLSBi",
				"LnNvdXJjZUluZGV4OwoJfSA6IChkb2N1bWVudC5jcmVhdGVSYW5nZSkgPyBmdW5jdGlvbihhLCBiKXsK",
				"CQlpZiAoIWEub3duZXJEb2N1bWVudCB8fCAhYi5vd25lckRvY3VtZW50KSByZXR1cm4gMDsKCQl2YXIg",
				"YVJhbmdlID0gYS5vd25lckRvY3VtZW50LmNyZWF0ZVJhbmdlKCksIGJSYW5nZSA9IGIub3duZXJEb2N1",
				"bWVudC5jcmVhdGVSYW5nZSgpOwoJCWFSYW5nZS5zZXRTdGFydChhLCAwKTsKCQlhUmFuZ2Uuc2V0RW5k",
				"KGEsIDApOwoJCWJSYW5nZS5zZXRTdGFydChiLCAwKTsKCQliUmFuZ2Uuc2V0RW5kKGIsIDApOwoJCXJl",
				"dHVybiBhUmFuZ2UuY29tcGFyZUJvdW5kYXJ5UG9pbnRzKFJhbmdlLlNUQVJUX1RPX0VORCwgYlJhbmdl",
				"KTsKCX0gOiBudWxsIDsKCglyb290ID0gbnVsbDsKCglmb3IgKGZlYXR1cmUgaW4gZmVhdHVyZXMpewoJ",
				"CXRoaXNbZmVhdHVyZV0gPSBmZWF0dXJlc1tmZWF0dXJlXTsKCX0KfTsKCi8vIE1haW4gTWV0aG9kCgp2",
				"YXIgcmVTaW1wbGVTZWxlY3RvciA9IC9eKFsjLl0/KSgoPzpbXHctXSt8XCopKSQvLAoJcmVFbXB0eUF0",
				"dHJpYnV0ZSA9IC9cWy4rWyokXl09KD86IiJ8JycpP1xdLywKCXFzYUZhaWxFeHBDYWNoZSA9IHt9OwoK",
				"bG9jYWwuc2VhcmNoID0gZnVuY3Rpb24oY29udGV4dCwgZXhwcmVzc2lvbiwgYXBwZW5kLCBmaXJzdCl7",
				"CgoJdmFyIGZvdW5kID0gdGhpcy5mb3VuZCA9IChmaXJzdCkgPyBudWxsIDogKGFwcGVuZCB8fCBbXSk7",
				"CgoJaWYgKCFjb250ZXh0KSByZXR1cm4gZm91bmQ7CgllbHNlIGlmIChjb250ZXh0Lm5hdmlnYXRvcikg",
				"Y29udGV4dCA9IGNvbnRleHQuZG9jdW1lbnQ7IC8vIENvbnZlcnQgdGhlIG5vZGUgZnJvbSBhIHdpbmRv",
				"dyB0byBhIGRvY3VtZW50CgllbHNlIGlmICghY29udGV4dC5ub2RlVHlwZSkgcmV0dXJuIGZvdW5kOwoK",
				"CS8vIHNldHVwCgoJdmFyIHBhcnNlZCwgaSwKCQl1bmlxdWVzID0gdGhpcy51bmlxdWVzID0ge30sCgkJ",
				"aGFzT3RoZXJzID0gISEoYXBwZW5kICYmIGFwcGVuZC5sZW5ndGgpLAoJCWNvbnRleHRJc0RvY3VtZW50",
				"ID0gKGNvbnRleHQubm9kZVR5cGUgPT0gOSk7CgoJaWYgKHRoaXMuZG9jdW1lbnQgIT09IChjb250ZXh0",
				"SXNEb2N1bWVudCA/IGNvbnRleHQgOiBjb250ZXh0Lm93bmVyRG9jdW1lbnQpKSB0aGlzLnNldERvY3Vt",
				"ZW50KGNvbnRleHQpOwoKCS8vIGF2b2lkIGR1cGxpY2F0aW5nIGl0ZW1zIGFscmVhZHkgaW4gdGhlIGFw",
				"cGVuZCBhcnJheQoJaWYgKGhhc090aGVycykgZm9yIChpID0gZm91bmQubGVuZ3RoOyBpLS07KSB1bmlx",
				"dWVzW3RoaXMuZ2V0VUlEKGZvdW5kW2ldKV0gPSB0cnVlOwoKCS8vIGV4cHJlc3Npb24gY2hlY2tzCgoJ",
				"aWYgKHR5cGVvZiBleHByZXNzaW9uID09ICdzdHJpbmcnKXsgLy8gZXhwcmVzc2lvbiBpcyBhIHN0cmlu",
				"ZwoKCQkvKjxzaW1wbGUtc2VsZWN0b3JzLW92ZXJyaWRlPiovCgkJdmFyIHNpbXBsZVNlbGVjdG9yID0g",
				"ZXhwcmVzc2lvbi5tYXRjaChyZVNpbXBsZVNlbGVjdG9yKTsKCQlzaW1wbGVTZWxlY3RvcnM6IGlmIChz",
				"aW1wbGVTZWxlY3RvcikgewoKCQkJdmFyIHN5bWJvbCA9IHNpbXBsZVNlbGVjdG9yWzFdLAoJCQkJbmFt",
				"ZSA9IHNpbXBsZVNlbGVjdG9yWzJdLAoJCQkJbm9kZSwgbm9kZXM7CgoJCQlpZiAoIXN5bWJvbCl7CgoJ",
				"CQkJaWYgKG5hbWUgPT0gJyonICYmIHRoaXMuYnJva2VuU3RhckdFQlROKSBicmVhayBzaW1wbGVTZWxl",
				"Y3RvcnM7CgkJCQlub2RlcyA9IGNvbnRleHQuZ2V0RWxlbWVudHNCeVRhZ05hbWUobmFtZSk7CgkJCQlp",
				"ZiAoZmlyc3QpIHJldHVybiBub2Rlc1swXSB8fCBudWxsOwoJCQkJZm9yIChpID0gMDsgbm9kZSA9IG5v",
				"ZGVzW2krK107KXsKCQkJCQlpZiAoIShoYXNPdGhlcnMgJiYgdW5pcXVlc1t0aGlzLmdldFVJRChub2Rl",
				"KV0pKSBmb3VuZC5wdXNoKG5vZGUpOwoJCQkJfQoKCQkJfSBlbHNlIGlmIChzeW1ib2wgPT0gJyMnKXsK",
				"CgkJCQlpZiAoIXRoaXMuaXNIVE1MRG9jdW1lbnQgfHwgIWNvbnRleHRJc0RvY3VtZW50KSBicmVhayBz",
				"aW1wbGVTZWxlY3RvcnM7CgkJCQlub2RlID0gY29udGV4dC5nZXRFbGVtZW50QnlJZChuYW1lKTsKCQkJ",
				"CWlmICghbm9kZSkgcmV0dXJuIGZvdW5kOwoJCQkJaWYgKHRoaXMuaWRHZXRzTmFtZSAmJiBub2RlLmdl",
				"dEF0dHJpYnV0ZU5vZGUoJ2lkJykubm9kZVZhbHVlICE9IG5hbWUpIGJyZWFrIHNpbXBsZVNlbGVjdG9y",
				"czsKCQkJCWlmIChmaXJzdCkgcmV0dXJuIG5vZGUgfHwgbnVsbDsKCQkJCWlmICghKGhhc090aGVycyAm",
				"JiB1bmlxdWVzW3RoaXMuZ2V0VUlEKG5vZGUpXSkpIGZvdW5kLnB1c2gobm9kZSk7CgoJCQl9IGVsc2Ug",
				"aWYgKHN5bWJvbCA9PSAnLicpewoKCQkJCWlmICghdGhpcy5pc0hUTUxEb2N1bWVudCB8fCAoKCFjb250",
				"ZXh0LmdldEVsZW1lbnRzQnlDbGFzc05hbWUgfHwgdGhpcy5icm9rZW5HRUJDTikgJiYgY29udGV4dC5x",
				"dWVyeVNlbGVjdG9yQWxsKSkgYnJlYWsgc2ltcGxlU2VsZWN0b3JzOwoJCQkJaWYgKGNvbnRleHQuZ2V0",
				"RWxlbWVudHNCeUNsYXNzTmFtZSAmJiAhdGhpcy5icm9rZW5HRUJDTil7CgkJCQkJbm9kZXMgPSBjb250",
				"ZXh0LmdldEVsZW1lbnRzQnlDbGFzc05hbWUobmFtZSk7CgkJCQkJaWYgKGZpcnN0KSByZXR1cm4gbm9k",
				"ZXNbMF0gfHwgbnVsbDsKCQkJCQlmb3IgKGkgPSAwOyBub2RlID0gbm9kZXNbaSsrXTspewoJCQkJCQlp",
				"ZiAoIShoYXNPdGhlcnMgJiYgdW5pcXVlc1t0aGlzLmdldFVJRChub2RlKV0pKSBmb3VuZC5wdXNoKG5v",
				"ZGUpOwoJCQkJCX0KCQkJCX0gZWxzZSB7CgkJCQkJdmFyIG1hdGNoQ2xhc3MgPSBuZXcgUmVnRXhwKCco",
				"XnxcXHMpJysgU2xpY2suZXNjYXBlUmVnRXhwKG5hbWUpICsnKFxcc3wkKScpOwoJCQkJCW5vZGVzID0g",
				"Y29udGV4dC5nZXRFbGVtZW50c0J5VGFnTmFtZSgnKicpOwoJCQkJCWZvciAoaSA9IDA7IG5vZGUgPSBu",
				"b2Rlc1tpKytdOyl7CgkJCQkJCWNsYXNzTmFtZSA9IG5vZGUuY2xhc3NOYW1lOwoJCQkJCQlpZiAoIShj",
				"bGFzc05hbWUgJiYgbWF0Y2hDbGFzcy50ZXN0KGNsYXNzTmFtZSkpKSBjb250aW51ZTsKCQkJCQkJaWYg",
				"KGZpcnN0KSByZXR1cm4gbm9kZTsKCQkJCQkJaWYgKCEoaGFzT3RoZXJzICYmIHVuaXF1ZXNbdGhpcy5n",
				"ZXRVSUQobm9kZSldKSkgZm91bmQucHVzaChub2RlKTsKCQkJCQl9CgkJCQl9CgoJCQl9CgoJCQlpZiAo",
				"aGFzT3RoZXJzKSB0aGlzLnNvcnQoZm91bmQpOwoJCQlyZXR1cm4gKGZpcnN0KSA/IG51bGwgOiBmb3Vu",
				"ZDsKCgkJfQoJCS8qPC9zaW1wbGUtc2VsZWN0b3JzLW92ZXJyaWRlPiovCgoJCS8qPHF1ZXJ5LXNlbGVj",
				"dG9yLW92ZXJyaWRlPiovCgkJcXVlcnlTZWxlY3RvcjogaWYgKGNvbnRleHQucXVlcnlTZWxlY3RvckFs",
				"bCkgewoKCQkJaWYgKCF0aGlzLmlzSFRNTERvY3VtZW50CgkJCQl8fCBxc2FGYWlsRXhwQ2FjaGVbZXhw",
				"cmVzc2lvbl0KCQkJCS8vVE9ETzogb25seSBza2lwIHdoZW4gZXhwcmVzc2lvbiBpcyBhY3R1YWxseSBt",
				"aXhlZCBjYXNlCgkJCQl8fCB0aGlzLmJyb2tlbk1peGVkQ2FzZVFTQQoJCQkJfHwgKHRoaXMuYnJva2Vu",
				"Q2hlY2tlZFFTQSAmJiBleHByZXNzaW9uLmluZGV4T2YoJzpjaGVja2VkJykgPiAtMSkKCQkJCXx8ICh0",
				"aGlzLmJyb2tlbkVtcHR5QXR0cmlidXRlUVNBICYmIHJlRW1wdHlBdHRyaWJ1dGUudGVzdChleHByZXNz",
				"aW9uKSkKCQkJCXx8ICghY29udGV4dElzRG9jdW1lbnQgLy9BYm9ydCB3aGVuICFjb250ZXh0SXNEb2N1",
				"bWVudCBhbmQuLi4KCQkJCQkvLyAgdGhlcmUgYXJlIG11bHRpcGxlIGV4cHJlc3Npb25zIGluIHRoZSBz",
				"ZWxlY3RvcgoJCQkJCS8vICBzaW5jZSB3ZSBjdXJyZW50bHkgb25seSBmaXggbm9uLWRvY3VtZW50IHJv",
				"b3RlZCBRU0EgZm9yIHNpbmdsZSBleHByZXNzaW9uIHNlbGVjdG9ycwoJCQkJCSYmIGV4cHJlc3Npb24u",
				"aW5kZXhPZignLCcpID4gLTEKCQkJCSkKCQkJCXx8IFNsaWNrLmRpc2FibGVRU0EKCQkJKSBicmVhayBx",
				"dWVyeVNlbGVjdG9yOwoKCQkJdmFyIF9leHByZXNzaW9uID0gZXhwcmVzc2lvbiwgX2NvbnRleHQgPSBj",
				"b250ZXh0OwoJCQlpZiAoIWNvbnRleHRJc0RvY3VtZW50KXsKCQkJCS8vIG5vbi1kb2N1bWVudCByb290",
				"ZWQgUVNBCgkJCQkvLyBjcmVkaXRzIHRvIEFuZHJldyBEdXBvbnQKCQkJCXZhciBjdXJyZW50SWQgPSBf",
				"Y29udGV4dC5nZXRBdHRyaWJ1dGUoJ2lkJyksIHNsaWNraWQgPSAnc2xpY2tpZF9fJzsKCQkJCV9jb250",
				"ZXh0LnNldEF0dHJpYnV0ZSgnaWQnLCBzbGlja2lkKTsKCQkJCV9leHByZXNzaW9uID0gJyMnICsgc2xp",
				"Y2tpZCArICcgJyArIF9leHByZXNzaW9uOwoJCQkJY29udGV4dCA9IF9jb250ZXh0LnBhcmVudE5vZGU7",
				"CgkJCX0KCgkJCXRyeSB7CgkJCQlpZiAoZmlyc3QpIHJldHVybiBjb250ZXh0LnF1ZXJ5U2VsZWN0b3Io",
				"X2V4cHJlc3Npb24pIHx8IG51bGw7CgkJCQllbHNlIG5vZGVzID0gY29udGV4dC5xdWVyeVNlbGVjdG9y",
				"QWxsKF9leHByZXNzaW9uKTsKCQkJfSBjYXRjaChlKSB7CgkJCQlxc2FGYWlsRXhwQ2FjaGVbZXhwcmVz",
				"c2lvbl0gPSAxOwoJCQkJYnJlYWsgcXVlcnlTZWxlY3RvcjsKCQkJfSBmaW5hbGx5IHsKCQkJCWlmICgh",
				"Y29udGV4dElzRG9jdW1lbnQpewoJCQkJCWlmIChjdXJyZW50SWQpIF9jb250ZXh0LnNldEF0dHJpYnV0",
				"ZSgnaWQnLCBjdXJyZW50SWQpOwoJCQkJCWVsc2UgX2NvbnRleHQucmVtb3ZlQXR0cmlidXRlKCdpZCcp",
				"OwoJCQkJCWNvbnRleHQgPSBfY29udGV4dDsKCQkJCX0KCQkJfQoKCQkJaWYgKHRoaXMuc3RhclNlbGVj",
				"dHNDbG9zZWRRU0EpIGZvciAoaSA9IDA7IG5vZGUgPSBub2Rlc1tpKytdOyl7CgkJCQlpZiAobm9kZS5u",
				"b2RlTmFtZSA+ICdAJyAmJiAhKGhhc090aGVycyAmJiB1bmlxdWVzW3RoaXMuZ2V0VUlEKG5vZGUpXSkp",
				"IGZvdW5kLnB1c2gobm9kZSk7CgkJCX0gZWxzZSBmb3IgKGkgPSAwOyBub2RlID0gbm9kZXNbaSsrXTsp",
				"ewoJCQkJaWYgKCEoaGFzT3RoZXJzICYmIHVuaXF1ZXNbdGhpcy5nZXRVSUQobm9kZSldKSkgZm91bmQu",
				"cHVzaChub2RlKTsKCQkJfQoKCQkJaWYgKGhhc090aGVycykgdGhpcy5zb3J0KGZvdW5kKTsKCQkJcmV0",
				"dXJuIGZvdW5kOwoKCQl9CgkJLyo8L3F1ZXJ5LXNlbGVjdG9yLW92ZXJyaWRlPiovCgoJCXBhcnNlZCA9",
				"IHRoaXMuU2xpY2sucGFyc2UoZXhwcmVzc2lvbik7CgkJaWYgKCFwYXJzZWQubGVuZ3RoKSByZXR1cm4g",
				"Zm91bmQ7Cgl9IGVsc2UgaWYgKGV4cHJlc3Npb24gPT0gbnVsbCl7IC8vIHRoZXJlIGlzIG5vIGV4cHJl",
				"c3Npb24KCQlyZXR1cm4gZm91bmQ7Cgl9IGVsc2UgaWYgKGV4cHJlc3Npb24uU2xpY2speyAvLyBleHBy",
				"ZXNzaW9uIGlzIGEgcGFyc2VkIFNsaWNrIG9iamVjdAoJCXBhcnNlZCA9IGV4cHJlc3Npb247Cgl9IGVs",
				"c2UgaWYgKHRoaXMuY29udGFpbnMoY29udGV4dC5kb2N1bWVudEVsZW1lbnQgfHwgY29udGV4dCwgZXhw",
				"cmVzc2lvbikpeyAvLyBleHByZXNzaW9uIGlzIGEgbm9kZQoJCShmb3VuZCkgPyBmb3VuZC5wdXNoKGV4",
				"cHJlc3Npb24pIDogZm91bmQgPSBleHByZXNzaW9uOwoJCXJldHVybiBmb3VuZDsKCX0gZWxzZSB7IC8v",
				"IG90aGVyIGp1bmsKCQlyZXR1cm4gZm91bmQ7Cgl9CgoJLyo8cHNldWRvLXNlbGVjdG9ycz4qLy8qPG50",
				"aC1wc2V1ZG8tc2VsZWN0b3JzPiovCgoJLy8gY2FjaGUgZWxlbWVudHMgZm9yIHRoZSBudGggc2VsZWN0",
				"b3JzCgoJdGhpcy5wb3NOVEggPSB7fTsKCXRoaXMucG9zTlRITGFzdCA9IHt9OwoJdGhpcy5wb3NOVEhU",
				"eXBlID0ge307Cgl0aGlzLnBvc05USFR5cGVMYXN0ID0ge307CgoJLyo8L250aC1wc2V1ZG8tc2VsZWN0",
				"b3JzPiovLyo8L3BzZXVkby1zZWxlY3RvcnM+Ki8KCgkvLyBpZiBhcHBlbmQgaXMgbnVsbCBhbmQgdGhl",
				"cmUgaXMgb25seSBhIHNpbmdsZSBzZWxlY3RvciB3aXRoIG9uZSBleHByZXNzaW9uIHVzZSBwdXNoQXJy",
				"YXksIGVsc2UgdXNlIHB1c2hVSUQKCXRoaXMucHVzaCA9ICghaGFzT3RoZXJzICYmIChmaXJzdCB8fCAo",
				"cGFyc2VkLmxlbmd0aCA9PSAxICYmIHBhcnNlZC5leHByZXNzaW9uc1swXS5sZW5ndGggPT0gMSkpKSA/",
				"IHRoaXMucHVzaEFycmF5IDogdGhpcy5wdXNoVUlEOwoKCWlmIChmb3VuZCA9PSBudWxsKSBmb3VuZCA9",
				"IFtdOwoKCS8vIGRlZmF1bHQgZW5naW5lCgoJdmFyIGosIG0sIG47Cgl2YXIgY29tYmluYXRvciwgdGFn",
				"LCBpZCwgY2xhc3NMaXN0LCBjbGFzc2VzLCBhdHRyaWJ1dGVzLCBwc2V1ZG9zOwoJdmFyIGN1cnJlbnRJ",
				"dGVtcywgY3VycmVudEV4cHJlc3Npb24sIGN1cnJlbnRCaXQsIGxhc3RCaXQsIGV4cHJlc3Npb25zID0g",
				"cGFyc2VkLmV4cHJlc3Npb25zOwoKCXNlYXJjaDogZm9yIChpID0gMDsgKGN1cnJlbnRFeHByZXNzaW9u",
				"ID0gZXhwcmVzc2lvbnNbaV0pOyBpKyspIGZvciAoaiA9IDA7IChjdXJyZW50Qml0ID0gY3VycmVudEV4",
				"cHJlc3Npb25bal0pOyBqKyspewoKCQljb21iaW5hdG9yID0gJ2NvbWJpbmF0b3I6JyArIGN1cnJlbnRC",
				"aXQuY29tYmluYXRvcjsKCQlpZiAoIXRoaXNbY29tYmluYXRvcl0pIGNvbnRpbnVlIHNlYXJjaDsKCgkJ",
				"dGFnICAgICAgICA9ICh0aGlzLmlzWE1MRG9jdW1lbnQpID8gY3VycmVudEJpdC50YWcgOiBjdXJyZW50",
				"Qml0LnRhZy50b1VwcGVyQ2FzZSgpOwoJCWlkICAgICAgICAgPSBjdXJyZW50Qml0LmlkOwoJCWNsYXNz",
				"TGlzdCAgPSBjdXJyZW50Qml0LmNsYXNzTGlzdDsKCQljbGFzc2VzICAgID0gY3VycmVudEJpdC5jbGFz",
				"c2VzOwoJCWF0dHJpYnV0ZXMgPSBjdXJyZW50Qml0LmF0dHJpYnV0ZXM7CgkJcHNldWRvcyAgICA9IGN1",
				"cnJlbnRCaXQucHNldWRvczsKCQlsYXN0Qml0ICAgID0gKGogPT09IChjdXJyZW50RXhwcmVzc2lvbi5s",
				"ZW5ndGggLSAxKSk7CgoJCXRoaXMuYml0VW5pcXVlcyA9IHt9OwoKCQlpZiAobGFzdEJpdCl7CgkJCXRo",
				"aXMudW5pcXVlcyA9IHVuaXF1ZXM7CgkJCXRoaXMuZm91bmQgPSBmb3VuZDsKCQl9IGVsc2UgewoJCQl0",
				"aGlzLnVuaXF1ZXMgPSB7fTsKCQkJdGhpcy5mb3VuZCA9IFtdOwoJCX0KCgkJaWYgKGogPT09IDApewoJ",
				"CQl0aGlzW2NvbWJpbmF0b3JdKGNvbnRleHQsIHRhZywgaWQsIGNsYXNzZXMsIGF0dHJpYnV0ZXMsIHBz",
				"ZXVkb3MsIGNsYXNzTGlzdCk7CgkJCWlmIChmaXJzdCAmJiBsYXN0Qml0ICYmIGZvdW5kLmxlbmd0aCkg",
				"YnJlYWsgc2VhcmNoOwoJCX0gZWxzZSB7CgkJCWlmIChmaXJzdCAmJiBsYXN0Qml0KSBmb3IgKG0gPSAw",
				"LCBuID0gY3VycmVudEl0ZW1zLmxlbmd0aDsgbSA8IG47IG0rKyl7CgkJCQl0aGlzW2NvbWJpbmF0b3Jd",
				"KGN1cnJlbnRJdGVtc1ttXSwgdGFnLCBpZCwgY2xhc3NlcywgYXR0cmlidXRlcywgcHNldWRvcywgY2xh",
				"c3NMaXN0KTsKCQkJCWlmIChmb3VuZC5sZW5ndGgpIGJyZWFrIHNlYXJjaDsKCQkJfSBlbHNlIGZvciAo",
				"bSA9IDAsIG4gPSBjdXJyZW50SXRlbXMubGVuZ3RoOyBtIDwgbjsgbSsrKSB0aGlzW2NvbWJpbmF0b3Jd",
				"KGN1cnJlbnRJdGVtc1ttXSwgdGFnLCBpZCwgY2xhc3NlcywgYXR0cmlidXRlcywgcHNldWRvcywgY2xh",
				"c3NMaXN0KTsKCQl9CgoJCWN1cnJlbnRJdGVtcyA9IHRoaXMuZm91bmQ7Cgl9CgoJLy8gc2hvdWxkIHNv",
				"cnQgaWYgdGhlcmUgYXJlIG5vZGVzIGluIGFwcGVuZCBhbmQgaWYgeW91IHBhc3MgbXVsdGlwbGUgZXhw",
				"cmVzc2lvbnMuCglpZiAoaGFzT3RoZXJzIHx8IChwYXJzZWQuZXhwcmVzc2lvbnMubGVuZ3RoID4gMSkp",
				"IHRoaXMuc29ydChmb3VuZCk7CgoJcmV0dXJuIChmaXJzdCkgPyAoZm91bmRbMF0gfHwgbnVsbCkgOiBm",
				"b3VuZDsKfTsKCi8vIFV0aWxzCgpsb2NhbC51aWR4ID0gMTsKbG9jYWwudWlkayA9ICdzbGljay11bmlx",
				"dWVpZCc7Cgpsb2NhbC5nZXRVSURYTUwgPSBmdW5jdGlvbihub2RlKXsKCXZhciB1aWQgPSBub2RlLmdl",
				"dEF0dHJpYnV0ZSh0aGlzLnVpZGspOwoJaWYgKCF1aWQpewoJCXVpZCA9IHRoaXMudWlkeCsrOwoJCW5v",
				"ZGUuc2V0QXR0cmlidXRlKHRoaXMudWlkaywgdWlkKTsKCX0KCXJldHVybiB1aWQ7Cn07Cgpsb2NhbC5n",
				"ZXRVSURIVE1MID0gZnVuY3Rpb24obm9kZSl7CglyZXR1cm4gbm9kZS51bmlxdWVOdW1iZXIgfHwgKG5v",
				"ZGUudW5pcXVlTnVtYmVyID0gdGhpcy51aWR4KyspOwp9OwoKLy8gc29ydCBiYXNlZCBvbiB0aGUgc2V0",
				"RG9jdW1lbnQgZG9jdW1lbnRTb3J0ZXIgbWV0aG9kLgoKbG9jYWwuc29ydCA9IGZ1bmN0aW9uKHJlc3Vs",
				"dHMpewoJaWYgKCF0aGlzLmRvY3VtZW50U29ydGVyKSByZXR1cm4gcmVzdWx0czsKCXJlc3VsdHMuc29y",
				"dCh0aGlzLmRvY3VtZW50U29ydGVyKTsKCXJldHVybiByZXN1bHRzOwp9OwoKLyo8cHNldWRvLXNlbGVj",
				"dG9ycz4qLy8qPG50aC1wc2V1ZG8tc2VsZWN0b3JzPiovCgpsb2NhbC5jYWNoZU5USCA9IHt9OwoKbG9j",
				"YWwubWF0Y2hOVEggPSAvXihbKy1dP1xkKik/KFthLXpdKyk/KFsrLV1cZCspPyQvOwoKbG9jYWwucGFy",
				"c2VOVEhBcmd1bWVudCA9IGZ1bmN0aW9uKGFyZ3VtZW50KXsKCXZhciBwYXJzZWQgPSBhcmd1bWVudC5t",
				"YXRjaCh0aGlzLm1hdGNoTlRIKTsKCWlmICghcGFyc2VkKSByZXR1cm4gZmFsc2U7Cgl2YXIgc3BlY2lh",
				"bCA9IHBhcnNlZFsyXSB8fCBmYWxzZTsKCXZhciBhID0gcGFyc2VkWzFdIHx8IDE7CglpZiAoYSA9PSAn",
				"LScpIGEgPSAtMTsKCXZhciBiID0gK3BhcnNlZFszXSB8fCAwOwoJcGFyc2VkID0KCQkoc3BlY2lhbCA9",
				"PSAnbicpCT8ge2E6IGEsIGI6IGJ9IDoKCQkoc3BlY2lhbCA9PSAnb2RkJykJPyB7YTogMiwgYjogMX0g",
				"OgoJCShzcGVjaWFsID09ICdldmVuJykJPyB7YTogMiwgYjogMH0gOiB7YTogMCwgYjogYX07CgoJcmV0",
				"dXJuICh0aGlzLmNhY2hlTlRIW2FyZ3VtZW50XSA9IHBhcnNlZCk7Cn07Cgpsb2NhbC5jcmVhdGVOVEhQ",
				"c2V1ZG8gPSBmdW5jdGlvbihjaGlsZCwgc2libGluZywgcG9zaXRpb25zLCBvZlR5cGUpewoJcmV0dXJu",
				"IGZ1bmN0aW9uKG5vZGUsIGFyZ3VtZW50KXsKCQl2YXIgdWlkID0gdGhpcy5nZXRVSUQobm9kZSk7CgkJ",
				"aWYgKCF0aGlzW3Bvc2l0aW9uc11bdWlkXSl7CgkJCXZhciBwYXJlbnQgPSBub2RlLnBhcmVudE5vZGU7",
				"CgkJCWlmICghcGFyZW50KSByZXR1cm4gZmFsc2U7CgkJCXZhciBlbCA9IHBhcmVudFtjaGlsZF0sIGNv",
				"dW50ID0gMTsKCQkJaWYgKG9mVHlwZSl7CgkJCQl2YXIgbm9kZU5hbWUgPSBub2RlLm5vZGVOYW1lOwoJ",
				"CQkJZG8gewoJCQkJCWlmIChlbC5ub2RlTmFtZSAhPSBub2RlTmFtZSkgY29udGludWU7CgkJCQkJdGhp",
				"c1twb3NpdGlvbnNdW3RoaXMuZ2V0VUlEKGVsKV0gPSBjb3VudCsrOwoJCQkJfSB3aGlsZSAoKGVsID0g",
				"ZWxbc2libGluZ10pKTsKCQkJfSBlbHNlIHsKCQkJCWRvIHsKCQkJCQlpZiAoZWwubm9kZVR5cGUgIT0g",
				"MSkgY29udGludWU7CgkJCQkJdGhpc1twb3NpdGlvbnNdW3RoaXMuZ2V0VUlEKGVsKV0gPSBjb3VudCsr",
				"OwoJCQkJfSB3aGlsZSAoKGVsID0gZWxbc2libGluZ10pKTsKCQkJfQoJCX0KCQlhcmd1bWVudCA9IGFy",
				"Z3VtZW50IHx8ICduJzsKCQl2YXIgcGFyc2VkID0gdGhpcy5jYWNoZU5USFthcmd1bWVudF0gfHwgdGhp",
				"cy5wYXJzZU5USEFyZ3VtZW50KGFyZ3VtZW50KTsKCQlpZiAoIXBhcnNlZCkgcmV0dXJuIGZhbHNlOwoJ",
				"CXZhciBhID0gcGFyc2VkLmEsIGIgPSBwYXJzZWQuYiwgcG9zID0gdGhpc1twb3NpdGlvbnNdW3VpZF07",
				"CgkJaWYgKGEgPT0gMCkgcmV0dXJuIGIgPT0gcG9zOwoJCWlmIChhID4gMCl7CgkJCWlmIChwb3MgPCBi",
				"KSByZXR1cm4gZmFsc2U7CgkJfSBlbHNlIHsKCQkJaWYgKGIgPCBwb3MpIHJldHVybiBmYWxzZTsKCQl9",
				"CgkJcmV0dXJuICgocG9zIC0gYikgJSBhKSA9PSAwOwoJfTsKfTsKCi8qPC9udGgtcHNldWRvLXNlbGVj",
				"dG9ycz4qLy8qPC9wc2V1ZG8tc2VsZWN0b3JzPiovCgpsb2NhbC5wdXNoQXJyYXkgPSBmdW5jdGlvbihu",
				"b2RlLCB0YWcsIGlkLCBjbGFzc2VzLCBhdHRyaWJ1dGVzLCBwc2V1ZG9zKXsKCWlmICh0aGlzLm1hdGNo",
				"U2VsZWN0b3Iobm9kZSwgdGFnLCBpZCwgY2xhc3NlcywgYXR0cmlidXRlcywgcHNldWRvcykpIHRoaXMu",
				"Zm91bmQucHVzaChub2RlKTsKfTsKCmxvY2FsLnB1c2hVSUQgPSBmdW5jdGlvbihub2RlLCB0YWcsIGlk",
				"LCBjbGFzc2VzLCBhdHRyaWJ1dGVzLCBwc2V1ZG9zKXsKCXZhciB1aWQgPSB0aGlzLmdldFVJRChub2Rl",
				"KTsKCWlmICghdGhpcy51bmlxdWVzW3VpZF0gJiYgdGhpcy5tYXRjaFNlbGVjdG9yKG5vZGUsIHRhZywg",
				"aWQsIGNsYXNzZXMsIGF0dHJpYnV0ZXMsIHBzZXVkb3MpKXsKCQl0aGlzLnVuaXF1ZXNbdWlkXSA9IHRy",
				"dWU7CgkJdGhpcy5mb3VuZC5wdXNoKG5vZGUpOwoJfQp9OwoKbG9jYWwubWF0Y2hOb2RlID0gZnVuY3Rp",
				"b24obm9kZSwgc2VsZWN0b3IpewoJaWYgKHRoaXMuaXNIVE1MRG9jdW1lbnQgJiYgdGhpcy5uYXRpdmVN",
				"YXRjaGVzU2VsZWN0b3IpewoJCXRyeSB7CgkJCXJldHVybiB0aGlzLm5hdGl2ZU1hdGNoZXNTZWxlY3Rv",
				"ci5jYWxsKG5vZGUsIHNlbGVjdG9yLnJlcGxhY2UoL1xbKFtePV0rKT1ccyooW14nIlxdXSs/KVxzKlxd",
				"L2csICdbJDE9IiQyIl0nKSk7CgkJfSBjYXRjaChtYXRjaEVycm9yKSB7fQoJfQoKCXZhciBwYXJzZWQg",
				"PSB0aGlzLlNsaWNrLnBhcnNlKHNlbGVjdG9yKTsKCWlmICghcGFyc2VkKSByZXR1cm4gdHJ1ZTsKCgkv",
				"LyBzaW1wbGUgKHNpbmdsZSkgc2VsZWN0b3JzCgl2YXIgZXhwcmVzc2lvbnMgPSBwYXJzZWQuZXhwcmVz",
				"c2lvbnMsIHNpbXBsZUV4cENvdW50ZXIgPSAwLCBpOwoJZm9yIChpID0gMDsgKGN1cnJlbnRFeHByZXNz",
				"aW9uID0gZXhwcmVzc2lvbnNbaV0pOyBpKyspewoJCWlmIChjdXJyZW50RXhwcmVzc2lvbi5sZW5ndGgg",
				"PT0gMSl7CgkJCXZhciBleHAgPSBjdXJyZW50RXhwcmVzc2lvblswXTsKCQkJaWYgKHRoaXMubWF0Y2hT",
				"ZWxlY3Rvcihub2RlLCAodGhpcy5pc1hNTERvY3VtZW50KSA/IGV4cC50YWcgOiBleHAudGFnLnRvVXBw",
				"ZXJDYXNlKCksIGV4cC5pZCwgZXhwLmNsYXNzZXMsIGV4cC5hdHRyaWJ1dGVzLCBleHAucHNldWRvcykp",
				"IHJldHVybiB0cnVlOwoJCQlzaW1wbGVFeHBDb3VudGVyKys7CgkJfQoJfQoKCWlmIChzaW1wbGVFeHBD",
				"b3VudGVyID09IHBhcnNlZC5sZW5ndGgpIHJldHVybiBmYWxzZTsKCgl2YXIgbm9kZXMgPSB0aGlzLnNl",
				"YXJjaCh0aGlzLmRvY3VtZW50LCBwYXJzZWQpLCBpdGVtOwoJZm9yIChpID0gMDsgaXRlbSA9IG5vZGVz",
				"W2krK107KXsKCQlpZiAoaXRlbSA9PT0gbm9kZSkgcmV0dXJuIHRydWU7Cgl9CglyZXR1cm4gZmFsc2U7",
				"Cn07Cgpsb2NhbC5tYXRjaFBzZXVkbyA9IGZ1bmN0aW9uKG5vZGUsIG5hbWUsIGFyZ3VtZW50KXsKCXZh",
				"ciBwc2V1ZG9OYW1lID0gJ3BzZXVkbzonICsgbmFtZTsKCWlmICh0aGlzW3BzZXVkb05hbWVdKSByZXR1",
				"cm4gdGhpc1twc2V1ZG9OYW1lXShub2RlLCBhcmd1bWVudCk7Cgl2YXIgYXR0cmlidXRlID0gdGhpcy5n",
				"ZXRBdHRyaWJ1dGUobm9kZSwgbmFtZSk7CglyZXR1cm4gKGFyZ3VtZW50KSA/IGFyZ3VtZW50ID09IGF0",
				"dHJpYnV0ZSA6ICEhYXR0cmlidXRlOwp9OwoKbG9jYWwubWF0Y2hTZWxlY3RvciA9IGZ1bmN0aW9uKG5v",
				"ZGUsIHRhZywgaWQsIGNsYXNzZXMsIGF0dHJpYnV0ZXMsIHBzZXVkb3MpewoJaWYgKHRhZyl7CgkJdmFy",
				"IG5vZGVOYW1lID0gKHRoaXMuaXNYTUxEb2N1bWVudCkgPyBub2RlLm5vZGVOYW1lIDogbm9kZS5ub2Rl",
				"TmFtZS50b1VwcGVyQ2FzZSgpOwoJCWlmICh0YWcgPT0gJyonKXsKCQkJaWYgKG5vZGVOYW1lIDwgJ0An",
				"KSByZXR1cm4gZmFsc2U7IC8vIEZpeCBmb3IgY29tbWVudCBub2RlcyBhbmQgY2xvc2VkIG5vZGVzCgkJ",
				"fSBlbHNlIHsKCQkJaWYgKG5vZGVOYW1lICE9IHRhZykgcmV0dXJuIGZhbHNlOwoJCX0KCX0KCglpZiAo",
				"aWQgJiYgbm9kZS5nZXRBdHRyaWJ1dGUoJ2lkJykgIT0gaWQpIHJldHVybiBmYWxzZTsKCgl2YXIgaSwg",
				"cGFydCwgY2xzOwoJaWYgKGNsYXNzZXMpIGZvciAoaSA9IGNsYXNzZXMubGVuZ3RoOyBpLS07KXsKCQlj",
				"bHMgPSB0aGlzLmdldEF0dHJpYnV0ZShub2RlLCAnY2xhc3MnKTsKCQlpZiAoIShjbHMgJiYgY2xhc3Nl",
				"c1tpXS5yZWdleHAudGVzdChjbHMpKSkgcmV0dXJuIGZhbHNlOwoJfQoJaWYgKGF0dHJpYnV0ZXMpIGZv",
				"ciAoaSA9IGF0dHJpYnV0ZXMubGVuZ3RoOyBpLS07KXsKCQlwYXJ0ID0gYXR0cmlidXRlc1tpXTsKCQlp",
				"ZiAocGFydC5vcGVyYXRvciA/ICFwYXJ0LnRlc3QodGhpcy5nZXRBdHRyaWJ1dGUobm9kZSwgcGFydC5r",
				"ZXkpKSA6ICF0aGlzLmhhc0F0dHJpYnV0ZShub2RlLCBwYXJ0LmtleSkpIHJldHVybiBmYWxzZTsKCX0K",
				"CWlmIChwc2V1ZG9zKSBmb3IgKGkgPSBwc2V1ZG9zLmxlbmd0aDsgaS0tOyl7CgkJcGFydCA9IHBzZXVk",
				"b3NbaV07CgkJaWYgKCF0aGlzLm1hdGNoUHNldWRvKG5vZGUsIHBhcnQua2V5LCBwYXJ0LnZhbHVlKSkg",
				"cmV0dXJuIGZhbHNlOwoJfQoJcmV0dXJuIHRydWU7Cn07Cgp2YXIgY29tYmluYXRvcnMgPSB7CgoJJyAn",
				"OiBmdW5jdGlvbihub2RlLCB0YWcsIGlkLCBjbGFzc2VzLCBhdHRyaWJ1dGVzLCBwc2V1ZG9zLCBjbGFz",
				"c0xpc3QpeyAvLyBhbGwgY2hpbGQgbm9kZXMsIGFueSBsZXZlbAoKCQl2YXIgaSwgaXRlbSwgY2hpbGRy",
				"ZW47CgoJCWlmICh0aGlzLmlzSFRNTERvY3VtZW50KXsKCQkJZ2V0QnlJZDogaWYgKGlkKXsKCQkJCWl0",
				"ZW0gPSB0aGlzLmRvY3VtZW50LmdldEVsZW1lbnRCeUlkKGlkKTsKCQkJCWlmICgoIWl0ZW0gJiYgbm9k",
				"ZS5hbGwpIHx8ICh0aGlzLmlkR2V0c05hbWUgJiYgaXRlbSAmJiBpdGVtLmdldEF0dHJpYnV0ZU5vZGUo",
				"J2lkJykubm9kZVZhbHVlICE9IGlkKSl7CgkJCQkJLy8gYWxsW2lkXSByZXR1cm5zIGFsbCB0aGUgZWxl",
				"bWVudHMgd2l0aCB0aGF0IG5hbWUgb3IgaWQgaW5zaWRlIG5vZGUKCQkJCQkvLyBpZiB0aGVyZXMganVz",
				"dCBvbmUgaXQgd2lsbCByZXR1cm4gdGhlIGVsZW1lbnQsIGVsc2UgaXQgd2lsbCBiZSBhIGNvbGxlY3Rp",
				"b24KCQkJCQljaGlsZHJlbiA9IG5vZGUuYWxsW2lkXTsKCQkJCQlpZiAoIWNoaWxkcmVuKSByZXR1cm47",
				"CgkJCQkJaWYgKCFjaGlsZHJlblswXSkgY2hpbGRyZW4gPSBbY2hpbGRyZW5dOwoJCQkJCWZvciAoaSA9",
				"IDA7IGl0ZW0gPSBjaGlsZHJlbltpKytdOyl7CgkJCQkJCXZhciBpZE5vZGUgPSBpdGVtLmdldEF0dHJp",
				"YnV0ZU5vZGUoJ2lkJyk7CgkJCQkJCWlmIChpZE5vZGUgJiYgaWROb2RlLm5vZGVWYWx1ZSA9PSBpZCl7",
				"CgkJCQkJCQl0aGlzLnB1c2goaXRlbSwgdGFnLCBudWxsLCBjbGFzc2VzLCBhdHRyaWJ1dGVzLCBwc2V1",
				"ZG9zKTsKCQkJCQkJCWJyZWFrOwoJCQkJCQl9CgkJCQkJfQoJCQkJCXJldHVybjsKCQkJCX0KCQkJCWlm",
				"ICghaXRlbSl7CgkJCQkJLy8gaWYgdGhlIGNvbnRleHQgaXMgaW4gdGhlIGRvbSB3ZSByZXR1cm4sIGVs",
				"c2Ugd2Ugd2lsbCB0cnkgR0VCVE4sIGJyZWFraW5nIHRoZSBnZXRCeUlkIGxhYmVsCgkJCQkJaWYgKHRo",
				"aXMuY29udGFpbnModGhpcy5yb290LCBub2RlKSkgcmV0dXJuOwoJCQkJCWVsc2UgYnJlYWsgZ2V0QnlJ",
				"ZDsKCQkJCX0gZWxzZSBpZiAodGhpcy5kb2N1bWVudCAhPT0gbm9kZSAmJiAhdGhpcy5jb250YWlucyhu",
				"b2RlLCBpdGVtKSkgcmV0dXJuOwoJCQkJdGhpcy5wdXNoKGl0ZW0sIHRhZywgbnVsbCwgY2xhc3Nlcywg",
				"YXR0cmlidXRlcywgcHNldWRvcyk7CgkJCQlyZXR1cm47CgkJCX0KCQkJZ2V0QnlDbGFzczogaWYgKGNs",
				"YXNzZXMgJiYgbm9kZS5nZXRFbGVtZW50c0J5Q2xhc3NOYW1lICYmICF0aGlzLmJyb2tlbkdFQkNOKXsK",
				"CQkJCWNoaWxkcmVuID0gbm9kZS5nZXRFbGVtZW50c0J5Q2xhc3NOYW1lKGNsYXNzTGlzdC5qb2luKCcg",
				"JykpOwoJCQkJaWYgKCEoY2hpbGRyZW4gJiYgY2hpbGRyZW4ubGVuZ3RoKSkgYnJlYWsgZ2V0QnlDbGFz",
				"czsKCQkJCWZvciAoaSA9IDA7IGl0ZW0gPSBjaGlsZHJlbltpKytdOykgdGhpcy5wdXNoKGl0ZW0sIHRh",
				"ZywgaWQsIG51bGwsIGF0dHJpYnV0ZXMsIHBzZXVkb3MpOwoJCQkJcmV0dXJuOwoJCQl9CgkJfQoJCWdl",
				"dEJ5VGFnOiB7CgkJCWNoaWxkcmVuID0gbm9kZS5nZXRFbGVtZW50c0J5VGFnTmFtZSh0YWcpOwoJCQlp",
				"ZiAoIShjaGlsZHJlbiAmJiBjaGlsZHJlbi5sZW5ndGgpKSBicmVhayBnZXRCeVRhZzsKCQkJaWYgKCF0",
				"aGlzLmJyb2tlblN0YXJHRUJUTikgdGFnID0gbnVsbDsKCQkJZm9yIChpID0gMDsgaXRlbSA9IGNoaWxk",
				"cmVuW2krK107KSB0aGlzLnB1c2goaXRlbSwgdGFnLCBpZCwgY2xhc3NlcywgYXR0cmlidXRlcywgcHNl",
				"dWRvcyk7CgkJfQoJfSwKCgknPic6IGZ1bmN0aW9uKG5vZGUsIHRhZywgaWQsIGNsYXNzZXMsIGF0dHJp",
				"YnV0ZXMsIHBzZXVkb3MpeyAvLyBkaXJlY3QgY2hpbGRyZW4KCQlpZiAoKG5vZGUgPSBub2RlLmZpcnN0",
				"Q2hpbGQpKSBkbyB7CgkJCWlmIChub2RlLm5vZGVUeXBlID09IDEpIHRoaXMucHVzaChub2RlLCB0YWcs",
				"IGlkLCBjbGFzc2VzLCBhdHRyaWJ1dGVzLCBwc2V1ZG9zKTsKCQl9IHdoaWxlICgobm9kZSA9IG5vZGUu",
				"bmV4dFNpYmxpbmcpKTsKCX0sCgoJJysnOiBmdW5jdGlvbihub2RlLCB0YWcsIGlkLCBjbGFzc2VzLCBh",
				"dHRyaWJ1dGVzLCBwc2V1ZG9zKXsgLy8gbmV4dCBzaWJsaW5nCgkJd2hpbGUgKChub2RlID0gbm9kZS5u",
				"ZXh0U2libGluZykpIGlmIChub2RlLm5vZGVUeXBlID09IDEpewoJCQl0aGlzLnB1c2gobm9kZSwgdGFn",
				"LCBpZCwgY2xhc3NlcywgYXR0cmlidXRlcywgcHNldWRvcyk7CgkJCWJyZWFrOwoJCX0KCX0sCgoJJ14n",
				"OiBmdW5jdGlvbihub2RlLCB0YWcsIGlkLCBjbGFzc2VzLCBhdHRyaWJ1dGVzLCBwc2V1ZG9zKXsgLy8g",
				"Zmlyc3QgY2hpbGQKCQlub2RlID0gbm9kZS5maXJzdENoaWxkOwoJCWlmIChub2RlKXsKCQkJaWYgKG5v",
				"ZGUubm9kZVR5cGUgPT0gMSkgdGhpcy5wdXNoKG5vZGUsIHRhZywgaWQsIGNsYXNzZXMsIGF0dHJpYnV0",
				"ZXMsIHBzZXVkb3MpOwoJCQllbHNlIHRoaXNbJ2NvbWJpbmF0b3I6KyddKG5vZGUsIHRhZywgaWQsIGNs",
				"YXNzZXMsIGF0dHJpYnV0ZXMsIHBzZXVkb3MpOwoJCX0KCX0sCgoJJ34nOiBmdW5jdGlvbihub2RlLCB0",
				"YWcsIGlkLCBjbGFzc2VzLCBhdHRyaWJ1dGVzLCBwc2V1ZG9zKXsgLy8gbmV4dCBzaWJsaW5ncwoJCXdo",
				"aWxlICgobm9kZSA9IG5vZGUubmV4dFNpYmxpbmcpKXsKCQkJaWYgKG5vZGUubm9kZVR5cGUgIT0gMSkg",
				"Y29udGludWU7CgkJCXZhciB1aWQgPSB0aGlzLmdldFVJRChub2RlKTsKCQkJaWYgKHRoaXMuYml0VW5p",
				"cXVlc1t1aWRdKSBicmVhazsKCQkJdGhpcy5iaXRVbmlxdWVzW3VpZF0gPSB0cnVlOwoJCQl0aGlzLnB1",
				"c2gobm9kZSwgdGFnLCBpZCwgY2xhc3NlcywgYXR0cmlidXRlcywgcHNldWRvcyk7CgkJfQoJfSwKCgkn",
				"KysnOiBmdW5jdGlvbihub2RlLCB0YWcsIGlkLCBjbGFzc2VzLCBhdHRyaWJ1dGVzLCBwc2V1ZG9zKXsg",
				"Ly8gbmV4dCBzaWJsaW5nIGFuZCBwcmV2aW91cyBzaWJsaW5nCgkJdGhpc1snY29tYmluYXRvcjorJ10o",
				"bm9kZSwgdGFnLCBpZCwgY2xhc3NlcywgYXR0cmlidXRlcywgcHNldWRvcyk7CgkJdGhpc1snY29tYmlu",
				"YXRvcjohKyddKG5vZGUsIHRhZywgaWQsIGNsYXNzZXMsIGF0dHJpYnV0ZXMsIHBzZXVkb3MpOwoJfSwK",
				"Cgknfn4nOiBmdW5jdGlvbihub2RlLCB0YWcsIGlkLCBjbGFzc2VzLCBhdHRyaWJ1dGVzLCBwc2V1ZG9z",
				"KXsgLy8gbmV4dCBzaWJsaW5ncyBhbmQgcHJldmlvdXMgc2libGluZ3MKCQl0aGlzWydjb21iaW5hdG9y",
				"On4nXShub2RlLCB0YWcsIGlkLCBjbGFzc2VzLCBhdHRyaWJ1dGVzLCBwc2V1ZG9zKTsKCQl0aGlzWydj",
				"b21iaW5hdG9yOiF+J10obm9kZSwgdGFnLCBpZCwgY2xhc3NlcywgYXR0cmlidXRlcywgcHNldWRvcyk7",
				"Cgl9LAoKCSchJzogZnVuY3Rpb24obm9kZSwgdGFnLCBpZCwgY2xhc3NlcywgYXR0cmlidXRlcywgcHNl",
				"dWRvcyl7IC8vIGFsbCBwYXJlbnQgbm9kZXMgdXAgdG8gZG9jdW1lbnQKCQl3aGlsZSAoKG5vZGUgPSBu",
				"b2RlLnBhcmVudE5vZGUpKSBpZiAobm9kZSAhPT0gdGhpcy5kb2N1bWVudCkgdGhpcy5wdXNoKG5vZGUs",
				"IHRhZywgaWQsIGNsYXNzZXMsIGF0dHJpYnV0ZXMsIHBzZXVkb3MpOwoJfSwKCgknIT4nOiBmdW5jdGlv",
				"bihub2RlLCB0YWcsIGlkLCBjbGFzc2VzLCBhdHRyaWJ1dGVzLCBwc2V1ZG9zKXsgLy8gZGlyZWN0IHBh",
				"cmVudCAob25lIGxldmVsKQoJCW5vZGUgPSBub2RlLnBhcmVudE5vZGU7CgkJaWYgKG5vZGUgIT09IHRo",
				"aXMuZG9jdW1lbnQpIHRoaXMucHVzaChub2RlLCB0YWcsIGlkLCBjbGFzc2VzLCBhdHRyaWJ1dGVzLCBw",
				"c2V1ZG9zKTsKCX0sCgoJJyErJzogZnVuY3Rpb24obm9kZSwgdGFnLCBpZCwgY2xhc3NlcywgYXR0cmli",
				"dXRlcywgcHNldWRvcyl7IC8vIHByZXZpb3VzIHNpYmxpbmcKCQl3aGlsZSAoKG5vZGUgPSBub2RlLnBy",
				"ZXZpb3VzU2libGluZykpIGlmIChub2RlLm5vZGVUeXBlID09IDEpewoJCQl0aGlzLnB1c2gobm9kZSwg",
				"dGFnLCBpZCwgY2xhc3NlcywgYXR0cmlidXRlcywgcHNldWRvcyk7CgkJCWJyZWFrOwoJCX0KCX0sCgoJ",
				"JyFeJzogZnVuY3Rpb24obm9kZSwgdGFnLCBpZCwgY2xhc3NlcywgYXR0cmlidXRlcywgcHNldWRvcyl7",
				"IC8vIGxhc3QgY2hpbGQKCQlub2RlID0gbm9kZS5sYXN0Q2hpbGQ7CgkJaWYgKG5vZGUpewoJCQlpZiAo",
				"bm9kZS5ub2RlVHlwZSA9PSAxKSB0aGlzLnB1c2gobm9kZSwgdGFnLCBpZCwgY2xhc3NlcywgYXR0cmli",
				"dXRlcywgcHNldWRvcyk7CgkJCWVsc2UgdGhpc1snY29tYmluYXRvcjohKyddKG5vZGUsIHRhZywgaWQs",
				"IGNsYXNzZXMsIGF0dHJpYnV0ZXMsIHBzZXVkb3MpOwoJCX0KCX0sCgoJJyF+JzogZnVuY3Rpb24obm9k",
				"ZSwgdGFnLCBpZCwgY2xhc3NlcywgYXR0cmlidXRlcywgcHNldWRvcyl7IC8vIHByZXZpb3VzIHNpYmxp",
				"bmdzCgkJd2hpbGUgKChub2RlID0gbm9kZS5wcmV2aW91c1NpYmxpbmcpKXsKCQkJaWYgKG5vZGUubm9k",
				"ZVR5cGUgIT0gMSkgY29udGludWU7CgkJCXZhciB1aWQgPSB0aGlzLmdldFVJRChub2RlKTsKCQkJaWYg",
				"KHRoaXMuYml0VW5pcXVlc1t1aWRdKSBicmVhazsKCQkJdGhpcy5iaXRVbmlxdWVzW3VpZF0gPSB0cnVl",
				"OwoJCQl0aGlzLnB1c2gobm9kZSwgdGFnLCBpZCwgY2xhc3NlcywgYXR0cmlidXRlcywgcHNldWRvcyk7",
				"CgkJfQoJfQoKfTsKCmZvciAodmFyIGMgaW4gY29tYmluYXRvcnMpIGxvY2FsWydjb21iaW5hdG9yOicg",
				"KyBjXSA9IGNvbWJpbmF0b3JzW2NdOwoKdmFyIHBzZXVkb3MgPSB7CgoJLyo8cHNldWRvLXNlbGVjdG9y",
				"cz4qLwoKCSdlbXB0eSc6IGZ1bmN0aW9uKG5vZGUpewoJCXZhciBjaGlsZCA9IG5vZGUuZmlyc3RDaGls",
				"ZDsKCQlyZXR1cm4gIShjaGlsZCAmJiBjaGlsZC5ub2RlVHlwZSA9PSAxKSAmJiAhKG5vZGUuaW5uZXJU",
				"ZXh0IHx8IG5vZGUudGV4dENvbnRlbnQgfHwgJycpLmxlbmd0aDsKCX0sCgoJJ25vdCc6IGZ1bmN0aW9u",
				"KG5vZGUsIGV4cHJlc3Npb24pewoJCXJldHVybiAhdGhpcy5tYXRjaE5vZGUobm9kZSwgZXhwcmVzc2lv",
				"bik7Cgl9LAoKCSdjb250YWlucyc6IGZ1bmN0aW9uKG5vZGUsIHRleHQpewoJCXJldHVybiAobm9kZS5p",
				"bm5lclRleHQgfHwgbm9kZS50ZXh0Q29udGVudCB8fCAnJykuaW5kZXhPZih0ZXh0KSA+IC0xOwoJfSwK",
				"CgknZmlyc3QtY2hpbGQnOiBmdW5jdGlvbihub2RlKXsKCQl3aGlsZSAoKG5vZGUgPSBub2RlLnByZXZp",
				"b3VzU2libGluZykpIGlmIChub2RlLm5vZGVUeXBlID09IDEpIHJldHVybiBmYWxzZTsKCQlyZXR1cm4g",
				"dHJ1ZTsKCX0sCgoJJ2xhc3QtY2hpbGQnOiBmdW5jdGlvbihub2RlKXsKCQl3aGlsZSAoKG5vZGUgPSBu",
				"b2RlLm5leHRTaWJsaW5nKSkgaWYgKG5vZGUubm9kZVR5cGUgPT0gMSkgcmV0dXJuIGZhbHNlOwoJCXJl",
				"dHVybiB0cnVlOwoJfSwKCgknb25seS1jaGlsZCc6IGZ1bmN0aW9uKG5vZGUpewoJCXZhciBwcmV2ID0g",
				"bm9kZTsKCQl3aGlsZSAoKHByZXYgPSBwcmV2LnByZXZpb3VzU2libGluZykpIGlmIChwcmV2Lm5vZGVU",
				"eXBlID09IDEpIHJldHVybiBmYWxzZTsKCQl2YXIgbmV4dCA9IG5vZGU7CgkJd2hpbGUgKChuZXh0ID0g",
				"bmV4dC5uZXh0U2libGluZykpIGlmIChuZXh0Lm5vZGVUeXBlID09IDEpIHJldHVybiBmYWxzZTsKCQly",
				"ZXR1cm4gdHJ1ZTsKCX0sCgoJLyo8bnRoLXBzZXVkby1zZWxlY3RvcnM+Ki8KCgknbnRoLWNoaWxkJzog",
				"bG9jYWwuY3JlYXRlTlRIUHNldWRvKCdmaXJzdENoaWxkJywgJ25leHRTaWJsaW5nJywgJ3Bvc05USCcp",
				"LAoKCSdudGgtbGFzdC1jaGlsZCc6IGxvY2FsLmNyZWF0ZU5USFBzZXVkbygnbGFzdENoaWxkJywgJ3By",
				"ZXZpb3VzU2libGluZycsICdwb3NOVEhMYXN0JyksCgoJJ250aC1vZi10eXBlJzogbG9jYWwuY3JlYXRl",
				"TlRIUHNldWRvKCdmaXJzdENoaWxkJywgJ25leHRTaWJsaW5nJywgJ3Bvc05USFR5cGUnLCB0cnVlKSwK",
				"CgknbnRoLWxhc3Qtb2YtdHlwZSc6IGxvY2FsLmNyZWF0ZU5USFBzZXVkbygnbGFzdENoaWxkJywgJ3By",
				"ZXZpb3VzU2libGluZycsICdwb3NOVEhUeXBlTGFzdCcsIHRydWUpLAoKCSdpbmRleCc6IGZ1bmN0aW9u",
				"KG5vZGUsIGluZGV4KXsKCQlyZXR1cm4gdGhpc1sncHNldWRvOm50aC1jaGlsZCddKG5vZGUsICcnICsg",
				"KGluZGV4ICsgMSkpOwoJfSwKCgknZXZlbic6IGZ1bmN0aW9uKG5vZGUpewoJCXJldHVybiB0aGlzWydw",
				"c2V1ZG86bnRoLWNoaWxkJ10obm9kZSwgJzJuJyk7Cgl9LAoKCSdvZGQnOiBmdW5jdGlvbihub2RlKXsK",
				"CQlyZXR1cm4gdGhpc1sncHNldWRvOm50aC1jaGlsZCddKG5vZGUsICcybisxJyk7Cgl9LAoKCS8qPC9u",
				"dGgtcHNldWRvLXNlbGVjdG9ycz4qLwoKCS8qPG9mLXR5cGUtcHNldWRvLXNlbGVjdG9ycz4qLwoKCSdm",
				"aXJzdC1vZi10eXBlJzogZnVuY3Rpb24obm9kZSl7CgkJdmFyIG5vZGVOYW1lID0gbm9kZS5ub2RlTmFt",
				"ZTsKCQl3aGlsZSAoKG5vZGUgPSBub2RlLnByZXZpb3VzU2libGluZykpIGlmIChub2RlLm5vZGVOYW1l",
				"ID09IG5vZGVOYW1lKSByZXR1cm4gZmFsc2U7CgkJcmV0dXJuIHRydWU7Cgl9LAoKCSdsYXN0LW9mLXR5",
				"cGUnOiBmdW5jdGlvbihub2RlKXsKCQl2YXIgbm9kZU5hbWUgPSBub2RlLm5vZGVOYW1lOwoJCXdoaWxl",
				"ICgobm9kZSA9IG5vZGUubmV4dFNpYmxpbmcpKSBpZiAobm9kZS5ub2RlTmFtZSA9PSBub2RlTmFtZSkg",
				"cmV0dXJuIGZhbHNlOwoJCXJldHVybiB0cnVlOwoJfSwKCgknb25seS1vZi10eXBlJzogZnVuY3Rpb24o",
				"bm9kZSl7CgkJdmFyIHByZXYgPSBub2RlLCBub2RlTmFtZSA9IG5vZGUubm9kZU5hbWU7CgkJd2hpbGUg",
				"KChwcmV2ID0gcHJldi5wcmV2aW91c1NpYmxpbmcpKSBpZiAocHJldi5ub2RlTmFtZSA9PSBub2RlTmFt",
				"ZSkgcmV0dXJuIGZhbHNlOwoJCXZhciBuZXh0ID0gbm9kZTsKCQl3aGlsZSAoKG5leHQgPSBuZXh0Lm5l",
				"eHRTaWJsaW5nKSkgaWYgKG5leHQubm9kZU5hbWUgPT0gbm9kZU5hbWUpIHJldHVybiBmYWxzZTsKCQly",
				"ZXR1cm4gdHJ1ZTsKCX0sCgoJLyo8L29mLXR5cGUtcHNldWRvLXNlbGVjdG9ycz4qLwoKCS8vIGN1c3Rv",
				"bSBwc2V1ZG9zCgoJJ2VuYWJsZWQnOiBmdW5jdGlvbihub2RlKXsKCQlyZXR1cm4gIW5vZGUuZGlzYWJs",
				"ZWQ7Cgl9LAoKCSdkaXNhYmxlZCc6IGZ1bmN0aW9uKG5vZGUpewoJCXJldHVybiBub2RlLmRpc2FibGVk",
				"OwoJfSwKCgknY2hlY2tlZCc6IGZ1bmN0aW9uKG5vZGUpewoJCXJldHVybiBub2RlLmNoZWNrZWQgfHwg",
				"bm9kZS5zZWxlY3RlZDsKCX0sCgoJJ2ZvY3VzJzogZnVuY3Rpb24obm9kZSl7CgkJcmV0dXJuIHRoaXMu",
				"aXNIVE1MRG9jdW1lbnQgJiYgdGhpcy5kb2N1bWVudC5hY3RpdmVFbGVtZW50ID09PSBub2RlICYmIChu",
				"b2RlLmhyZWYgfHwgbm9kZS50eXBlIHx8IHRoaXMuaGFzQXR0cmlidXRlKG5vZGUsICd0YWJpbmRleCcp",
				"KTsKCX0sCgoJJ3Jvb3QnOiBmdW5jdGlvbihub2RlKXsKCQlyZXR1cm4gKG5vZGUgPT09IHRoaXMucm9v",
				"dCk7Cgl9LAoKCSdzZWxlY3RlZCc6IGZ1bmN0aW9uKG5vZGUpewoJCXJldHVybiBub2RlLnNlbGVjdGVk",
				"OwoJfQoKCS8qPC9wc2V1ZG8tc2VsZWN0b3JzPiovCn07Cgpmb3IgKHZhciBwIGluIHBzZXVkb3MpIGxv",
				"Y2FsWydwc2V1ZG86JyArIHBdID0gcHNldWRvc1twXTsKCi8vIGF0dHJpYnV0ZXMgbWV0aG9kcwoKdmFy",
				"IGF0dHJpYnV0ZUdldHRlcnMgPSBsb2NhbC5hdHRyaWJ1dGVHZXR0ZXJzID0gewoKCSdmb3InOiBmdW5j",
				"dGlvbigpewoJCXJldHVybiAoJ2h0bWxGb3InIGluIHRoaXMpID8gdGhpcy5odG1sRm9yIDogdGhpcy5n",
				"ZXRBdHRyaWJ1dGUoJ2ZvcicpOwoJfSwKCgknaHJlZic6IGZ1bmN0aW9uKCl7CgkJcmV0dXJuICgnaHJl",
				"ZicgaW4gdGhpcykgPyB0aGlzLmdldEF0dHJpYnV0ZSgnaHJlZicsIDIpIDogdGhpcy5nZXRBdHRyaWJ1",
				"dGUoJ2hyZWYnKTsKCX0sCgoJJ3N0eWxlJzogZnVuY3Rpb24oKXsKCQlyZXR1cm4gKHRoaXMuc3R5bGUp",
				"ID8gdGhpcy5zdHlsZS5jc3NUZXh0IDogdGhpcy5nZXRBdHRyaWJ1dGUoJ3N0eWxlJyk7Cgl9LAoKCSd0",
				"YWJpbmRleCc6IGZ1bmN0aW9uKCl7CgkJdmFyIGF0dHJpYnV0ZU5vZGUgPSB0aGlzLmdldEF0dHJpYnV0",
				"ZU5vZGUoJ3RhYmluZGV4Jyk7CgkJcmV0dXJuIChhdHRyaWJ1dGVOb2RlICYmIGF0dHJpYnV0ZU5vZGUu",
				"c3BlY2lmaWVkKSA/IGF0dHJpYnV0ZU5vZGUubm9kZVZhbHVlIDogbnVsbDsKCX0sCgoJJ3R5cGUnOiBm",
				"dW5jdGlvbigpewoJCXJldHVybiB0aGlzLmdldEF0dHJpYnV0ZSgndHlwZScpOwoJfSwKCgknbWF4bGVu",
				"Z3RoJzogZnVuY3Rpb24oKXsKCQl2YXIgYXR0cmlidXRlTm9kZSA9IHRoaXMuZ2V0QXR0cmlidXRlTm9k",
				"ZSgnbWF4TGVuZ3RoJyk7CgkJcmV0dXJuIChhdHRyaWJ1dGVOb2RlICYmIGF0dHJpYnV0ZU5vZGUuc3Bl",
				"Y2lmaWVkKSA/IGF0dHJpYnV0ZU5vZGUubm9kZVZhbHVlIDogbnVsbDsKCX0KCn07CgphdHRyaWJ1dGVH",
				"ZXR0ZXJzLk1BWExFTkdUSCA9IGF0dHJpYnV0ZUdldHRlcnMubWF4TGVuZ3RoID0gYXR0cmlidXRlR2V0",
				"dGVycy5tYXhsZW5ndGg7CgovLyBTbGljawoKdmFyIFNsaWNrID0gbG9jYWwuU2xpY2sgPSAodGhpcy5T",
				"bGljayB8fCB7fSk7CgpTbGljay52ZXJzaW9uID0gJzEuMS43JzsKCi8vIFNsaWNrIGZpbmRlcgoKU2xp",
				"Y2suc2VhcmNoID0gZnVuY3Rpb24oY29udGV4dCwgZXhwcmVzc2lvbiwgYXBwZW5kKXsKCXJldHVybiBs",
				"b2NhbC5zZWFyY2goY29udGV4dCwgZXhwcmVzc2lvbiwgYXBwZW5kKTsKfTsKClNsaWNrLmZpbmQgPSBm",
				"dW5jdGlvbihjb250ZXh0LCBleHByZXNzaW9uKXsKCXJldHVybiBsb2NhbC5zZWFyY2goY29udGV4dCwg",
				"ZXhwcmVzc2lvbiwgbnVsbCwgdHJ1ZSk7Cn07CgovLyBTbGljayBjb250YWlubWVudCBjaGVja2VyCgpT",
				"bGljay5jb250YWlucyA9IGZ1bmN0aW9uKGNvbnRhaW5lciwgbm9kZSl7Cglsb2NhbC5zZXREb2N1bWVu",
				"dChjb250YWluZXIpOwoJcmV0dXJuIGxvY2FsLmNvbnRhaW5zKGNvbnRhaW5lciwgbm9kZSk7Cn07Cgov",
				"LyBTbGljayBhdHRyaWJ1dGUgZ2V0dGVyCgpTbGljay5nZXRBdHRyaWJ1dGUgPSBmdW5jdGlvbihub2Rl",
				"LCBuYW1lKXsKCWxvY2FsLnNldERvY3VtZW50KG5vZGUpOwoJcmV0dXJuIGxvY2FsLmdldEF0dHJpYnV0",
				"ZShub2RlLCBuYW1lKTsKfTsKClNsaWNrLmhhc0F0dHJpYnV0ZSA9IGZ1bmN0aW9uKG5vZGUsIG5hbWUp",
				"ewoJbG9jYWwuc2V0RG9jdW1lbnQobm9kZSk7CglyZXR1cm4gbG9jYWwuaGFzQXR0cmlidXRlKG5vZGUs",
				"IG5hbWUpOwp9OwoKLy8gU2xpY2sgbWF0Y2hlcgoKU2xpY2subWF0Y2ggPSBmdW5jdGlvbihub2RlLCBz",
				"ZWxlY3Rvcil7CglpZiAoIShub2RlICYmIHNlbGVjdG9yKSkgcmV0dXJuIGZhbHNlOwoJaWYgKCFzZWxl",
				"Y3RvciB8fCBzZWxlY3RvciA9PT0gbm9kZSkgcmV0dXJuIHRydWU7Cglsb2NhbC5zZXREb2N1bWVudChu",
				"b2RlKTsKCXJldHVybiBsb2NhbC5tYXRjaE5vZGUobm9kZSwgc2VsZWN0b3IpOwp9OwoKLy8gU2xpY2sg",
				"YXR0cmlidXRlIGFjY2Vzc29yCgpTbGljay5kZWZpbmVBdHRyaWJ1dGVHZXR0ZXIgPSBmdW5jdGlvbihu",
				"YW1lLCBmbil7Cglsb2NhbC5hdHRyaWJ1dGVHZXR0ZXJzW25hbWVdID0gZm47CglyZXR1cm4gdGhpczsK",
				"fTsKClNsaWNrLmxvb2t1cEF0dHJpYnV0ZUdldHRlciA9IGZ1bmN0aW9uKG5hbWUpewoJcmV0dXJuIGxv",
				"Y2FsLmF0dHJpYnV0ZUdldHRlcnNbbmFtZV07Cn07CgovLyBTbGljayBwc2V1ZG8gYWNjZXNzb3IKClNs",
				"aWNrLmRlZmluZVBzZXVkbyA9IGZ1bmN0aW9uKG5hbWUsIGZuKXsKCWxvY2FsWydwc2V1ZG86JyArIG5h",
				"bWVdID0gZnVuY3Rpb24obm9kZSwgYXJndW1lbnQpewoJCXJldHVybiBmbi5jYWxsKG5vZGUsIGFyZ3Vt",
				"ZW50KTsKCX07CglyZXR1cm4gdGhpczsKfTsKClNsaWNrLmxvb2t1cFBzZXVkbyA9IGZ1bmN0aW9uKG5h",
				"bWUpewoJdmFyIHBzZXVkbyA9IGxvY2FsWydwc2V1ZG86JyArIG5hbWVdOwoJaWYgKHBzZXVkbykgcmV0",
				"dXJuIGZ1bmN0aW9uKGFyZ3VtZW50KXsKCQlyZXR1cm4gcHNldWRvLmNhbGwodGhpcywgYXJndW1lbnQp",
				"OwoJfTsKCXJldHVybiBudWxsOwp9OwoKLy8gU2xpY2sgb3ZlcnJpZGVzIGFjY2Vzc29yCgpTbGljay5v",
				"dmVycmlkZSA9IGZ1bmN0aW9uKHJlZ2V4cCwgZm4pewoJbG9jYWwub3ZlcnJpZGUocmVnZXhwLCBmbik7",
				"CglyZXR1cm4gdGhpczsKfTsKClNsaWNrLmlzWE1MID0gbG9jYWwuaXNYTUw7CgpTbGljay51aWRPZiA9",
				"IGZ1bmN0aW9uKG5vZGUpewoJcmV0dXJuIGxvY2FsLmdldFVJREhUTUwobm9kZSk7Cn07CgppZiAoIXRo",
				"aXMuU2xpY2spIHRoaXMuU2xpY2sgPSBTbGljazsKCn0pLmFwcGx5KC8qPENvbW1vbkpTPiovKHR5cGVv",
				"ZiBleHBvcnRzICE9ICd1bmRlZmluZWQnKSA/IGV4cG9ydHMgOiAvKjwvQ29tbW9uSlM+Ki90aGlzKTsK",
				"CgovKgotLS0KCm5hbWU6IEVsZW1lbnQKCmRlc2NyaXB0aW9uOiBPbmUgb2YgdGhlIG1vc3QgaW1wb3J0",
				"YW50IGl0ZW1zIGluIE1vb1Rvb2xzLiBDb250YWlucyB0aGUgZG9sbGFyIGZ1bmN0aW9uLCB0aGUgZG9s",
				"bGFycyBmdW5jdGlvbiwgYW5kIGFuIGhhbmRmdWwgb2YgY3Jvc3MtYnJvd3NlciwgdGltZS1zYXZlciBt",
				"ZXRob2RzIHRvIGxldCB5b3UgZWFzaWx5IHdvcmsgd2l0aCBIVE1MIEVsZW1lbnRzLgoKbGljZW5zZTog",
				"TUlULXN0eWxlIGxpY2Vuc2UuCgpyZXF1aXJlczogW1dpbmRvdywgRG9jdW1lbnQsIEFycmF5LCBTdHJp",
				"bmcsIEZ1bmN0aW9uLCBPYmplY3QsIE51bWJlciwgU2xpY2suUGFyc2VyLCBTbGljay5GaW5kZXJdCgpw",
				"cm92aWRlczogW0VsZW1lbnQsIEVsZW1lbnRzLCAkLCAkJCwgSWZyYW1lLCBTZWxlY3RvcnNdCgouLi4K",
				"Ki8KCnZhciBFbGVtZW50ID0gZnVuY3Rpb24odGFnLCBwcm9wcyl7Cgl2YXIga29uc3RydWN0b3IgPSBF",
				"bGVtZW50LkNvbnN0cnVjdG9yc1t0YWddOwoJaWYgKGtvbnN0cnVjdG9yKSByZXR1cm4ga29uc3RydWN0",
				"b3IocHJvcHMpOwoJaWYgKHR5cGVvZiB0YWcgIT0gJ3N0cmluZycpIHJldHVybiBkb2N1bWVudC5pZCh0",
				"YWcpLnNldChwcm9wcyk7CgoJaWYgKCFwcm9wcykgcHJvcHMgPSB7fTsKCglpZiAoISgvXltcdy1dKyQv",
				"KS50ZXN0KHRhZykpewoJCXZhciBwYXJzZWQgPSBTbGljay5wYXJzZSh0YWcpLmV4cHJlc3Npb25zWzBd",
				"WzBdOwoJCXRhZyA9IChwYXJzZWQudGFnID09ICcqJykgPyAnZGl2JyA6IHBhcnNlZC50YWc7CgkJaWYg",
				"KHBhcnNlZC5pZCAmJiBwcm9wcy5pZCA9PSBudWxsKSBwcm9wcy5pZCA9IHBhcnNlZC5pZDsKCgkJdmFy",
				"IGF0dHJpYnV0ZXMgPSBwYXJzZWQuYXR0cmlidXRlczsKCQlpZiAoYXR0cmlidXRlcykgZm9yICh2YXIg",
				"YXR0ciwgaSA9IDAsIGwgPSBhdHRyaWJ1dGVzLmxlbmd0aDsgaSA8IGw7IGkrKyl7CgkJCWF0dHIgPSBh",
				"dHRyaWJ1dGVzW2ldOwoJCQlpZiAocHJvcHNbYXR0ci5rZXldICE9IG51bGwpIGNvbnRpbnVlOwoKCQkJ",
				"aWYgKGF0dHIudmFsdWUgIT0gbnVsbCAmJiBhdHRyLm9wZXJhdG9yID09ICc9JykgcHJvcHNbYXR0ci5r",
				"ZXldID0gYXR0ci52YWx1ZTsKCQkJZWxzZSBpZiAoIWF0dHIudmFsdWUgJiYgIWF0dHIub3BlcmF0b3Ip",
				"IHByb3BzW2F0dHIua2V5XSA9IHRydWU7CgkJfQoKCQlpZiAocGFyc2VkLmNsYXNzTGlzdCAmJiBwcm9w",
				"c1snY2xhc3MnXSA9PSBudWxsKSBwcm9wc1snY2xhc3MnXSA9IHBhcnNlZC5jbGFzc0xpc3Quam9pbign",
				"ICcpOwoJfQoKCXJldHVybiBkb2N1bWVudC5uZXdFbGVtZW50KHRhZywgcHJvcHMpOwp9OwoKCmlmIChC",
				"cm93c2VyLkVsZW1lbnQpewoJRWxlbWVudC5wcm90b3R5cGUgPSBCcm93c2VyLkVsZW1lbnQucHJvdG90",
				"eXBlOwoJLy8gSUU4IGFuZCBJRTkgcmVxdWlyZSB0aGUgd3JhcHBpbmcuCglFbGVtZW50LnByb3RvdHlw",
				"ZS5fZmlyZUV2ZW50ID0gKGZ1bmN0aW9uKGZpcmVFdmVudCl7CgkJcmV0dXJuIGZ1bmN0aW9uKHR5cGUs",
				"IGV2ZW50KXsKCQkJcmV0dXJuIGZpcmVFdmVudC5jYWxsKHRoaXMsIHR5cGUsIGV2ZW50KTsKCQl9OwoJ",
				"fSkoRWxlbWVudC5wcm90b3R5cGUuZmlyZUV2ZW50KTsKfQoKbmV3IFR5cGUoJ0VsZW1lbnQnLCBFbGVt",
				"ZW50KS5taXJyb3IoZnVuY3Rpb24obmFtZSl7CglpZiAoQXJyYXkucHJvdG90eXBlW25hbWVdKSByZXR1",
				"cm47CgoJdmFyIG9iaiA9IHt9OwoJb2JqW25hbWVdID0gZnVuY3Rpb24oKXsKCQl2YXIgcmVzdWx0cyA9",
				"IFtdLCBhcmdzID0gYXJndW1lbnRzLCBlbGVtZW50cyA9IHRydWU7CgkJZm9yICh2YXIgaSA9IDAsIGwg",
				"PSB0aGlzLmxlbmd0aDsgaSA8IGw7IGkrKyl7CgkJCXZhciBlbGVtZW50ID0gdGhpc1tpXSwgcmVzdWx0",
				"ID0gcmVzdWx0c1tpXSA9IGVsZW1lbnRbbmFtZV0uYXBwbHkoZWxlbWVudCwgYXJncyk7CgkJCWVsZW1l",
				"bnRzID0gKGVsZW1lbnRzICYmIHR5cGVPZihyZXN1bHQpID09ICdlbGVtZW50Jyk7CgkJfQoJCXJldHVy",
				"biAoZWxlbWVudHMpID8gbmV3IEVsZW1lbnRzKHJlc3VsdHMpIDogcmVzdWx0czsKCX07CgoJRWxlbWVu",
				"dHMuaW1wbGVtZW50KG9iaik7Cn0pOwoKaWYgKCFCcm93c2VyLkVsZW1lbnQpewoJRWxlbWVudC5wYXJl",
				"bnQgPSBPYmplY3Q7CgoJRWxlbWVudC5Qcm90b3R5cGUgPSB7CgkJJyRjb25zdHJ1Y3Rvcic6IEVsZW1l",
				"bnQsCgkJJyRmYW1pbHknOiBGdW5jdGlvbi5mcm9tKCdlbGVtZW50JykuaGlkZSgpCgl9OwoKCUVsZW1l",
				"bnQubWlycm9yKGZ1bmN0aW9uKG5hbWUsIG1ldGhvZCl7CgkJRWxlbWVudC5Qcm90b3R5cGVbbmFtZV0g",
				"PSBtZXRob2Q7Cgl9KTsKfQoKRWxlbWVudC5Db25zdHJ1Y3RvcnMgPSB7fTsKCi8vPDEuMmNvbXBhdD4K",
				"CkVsZW1lbnQuQ29uc3RydWN0b3JzID0gbmV3IEhhc2g7CgovLzwvMS4yY29tcGF0PgoKdmFyIElGcmFt",
				"ZSA9IG5ldyBUeXBlKCdJRnJhbWUnLCBmdW5jdGlvbigpewoJdmFyIHBhcmFtcyA9IEFycmF5Lmxpbmso",
				"YXJndW1lbnRzLCB7CgkJcHJvcGVydGllczogVHlwZS5pc09iamVjdCwKCQlpZnJhbWU6IGZ1bmN0aW9u",
				"KG9iail7CgkJCXJldHVybiAob2JqICE9IG51bGwpOwoJCX0KCX0pOwoKCXZhciBwcm9wcyA9IHBhcmFt",
				"cy5wcm9wZXJ0aWVzIHx8IHt9LCBpZnJhbWU7CglpZiAocGFyYW1zLmlmcmFtZSkgaWZyYW1lID0gZG9j",
				"dW1lbnQuaWQocGFyYW1zLmlmcmFtZSk7Cgl2YXIgb25sb2FkID0gcHJvcHMub25sb2FkIHx8IGZ1bmN0",
				"aW9uKCl7fTsKCWRlbGV0ZSBwcm9wcy5vbmxvYWQ7Cglwcm9wcy5pZCA9IHByb3BzLm5hbWUgPSBbcHJv",
				"cHMuaWQsIHByb3BzLm5hbWUsIGlmcmFtZSA/IChpZnJhbWUuaWQgfHwgaWZyYW1lLm5hbWUpIDogJ0lG",
				"cmFtZV8nICsgU3RyaW5nLnVuaXF1ZUlEKCldLnBpY2soKTsKCWlmcmFtZSA9IG5ldyBFbGVtZW50KGlm",
				"cmFtZSB8fCAnaWZyYW1lJywgcHJvcHMpOwoKCXZhciBvbkxvYWQgPSBmdW5jdGlvbigpewoJCW9ubG9h",
				"ZC5jYWxsKGlmcmFtZS5jb250ZW50V2luZG93KTsKCX07CgoJaWYgKHdpbmRvdy5mcmFtZXNbcHJvcHMu",
				"aWRdKSBvbkxvYWQoKTsKCWVsc2UgaWZyYW1lLmFkZExpc3RlbmVyKCdsb2FkJywgb25Mb2FkKTsKCXJl",
				"dHVybiBpZnJhbWU7Cn0pOwoKdmFyIEVsZW1lbnRzID0gdGhpcy5FbGVtZW50cyA9IGZ1bmN0aW9uKG5v",
				"ZGVzKXsKCWlmIChub2RlcyAmJiBub2Rlcy5sZW5ndGgpewoJCXZhciB1bmlxdWVzID0ge30sIG5vZGU7",
				"CgkJZm9yICh2YXIgaSA9IDA7IG5vZGUgPSBub2Rlc1tpKytdOyl7CgkJCXZhciB1aWQgPSBTbGljay51",
				"aWRPZihub2RlKTsKCQkJaWYgKCF1bmlxdWVzW3VpZF0pewoJCQkJdW5pcXVlc1t1aWRdID0gdHJ1ZTsK",
				"CQkJCXRoaXMucHVzaChub2RlKTsKCQkJfQoJCX0KCX0KfTsKCkVsZW1lbnRzLnByb3RvdHlwZSA9IHts",
				"ZW5ndGg6IDB9OwpFbGVtZW50cy5wYXJlbnQgPSBBcnJheTsKCm5ldyBUeXBlKCdFbGVtZW50cycsIEVs",
				"ZW1lbnRzKS5pbXBsZW1lbnQoewoKCWZpbHRlcjogZnVuY3Rpb24oZmlsdGVyLCBiaW5kKXsKCQlpZiAo",
				"IWZpbHRlcikgcmV0dXJuIHRoaXM7CgkJcmV0dXJuIG5ldyBFbGVtZW50cyhBcnJheS5maWx0ZXIodGhp",
				"cywgKHR5cGVPZihmaWx0ZXIpID09ICdzdHJpbmcnKSA/IGZ1bmN0aW9uKGl0ZW0pewoJCQlyZXR1cm4g",
				"aXRlbS5tYXRjaChmaWx0ZXIpOwoJCX0gOiBmaWx0ZXIsIGJpbmQpKTsKCX0ucHJvdGVjdCgpLAoKCXB1",
				"c2g6IGZ1bmN0aW9uKCl7CgkJdmFyIGxlbmd0aCA9IHRoaXMubGVuZ3RoOwoJCWZvciAodmFyIGkgPSAw",
				"LCBsID0gYXJndW1lbnRzLmxlbmd0aDsgaSA8IGw7IGkrKyl7CgkJCXZhciBpdGVtID0gZG9jdW1lbnQu",
				"aWQoYXJndW1lbnRzW2ldKTsKCQkJaWYgKGl0ZW0pIHRoaXNbbGVuZ3RoKytdID0gaXRlbTsKCQl9CgkJ",
				"cmV0dXJuICh0aGlzLmxlbmd0aCA9IGxlbmd0aCk7Cgl9LnByb3RlY3QoKSwKCgl1bnNoaWZ0OiBmdW5j",
				"dGlvbigpewoJCXZhciBpdGVtcyA9IFtdOwoJCWZvciAodmFyIGkgPSAwLCBsID0gYXJndW1lbnRzLmxl",
				"bmd0aDsgaSA8IGw7IGkrKyl7CgkJCXZhciBpdGVtID0gZG9jdW1lbnQuaWQoYXJndW1lbnRzW2ldKTsK",
				"CQkJaWYgKGl0ZW0pIGl0ZW1zLnB1c2goaXRlbSk7CgkJfQoJCXJldHVybiBBcnJheS5wcm90b3R5cGUu",
				"dW5zaGlmdC5hcHBseSh0aGlzLCBpdGVtcyk7Cgl9LnByb3RlY3QoKSwKCgljb25jYXQ6IGZ1bmN0aW9u",
				"KCl7CgkJdmFyIG5ld0VsZW1lbnRzID0gbmV3IEVsZW1lbnRzKHRoaXMpOwoJCWZvciAodmFyIGkgPSAw",
				"LCBsID0gYXJndW1lbnRzLmxlbmd0aDsgaSA8IGw7IGkrKyl7CgkJCXZhciBpdGVtID0gYXJndW1lbnRz",
				"W2ldOwoJCQlpZiAoVHlwZS5pc0VudW1lcmFibGUoaXRlbSkpIG5ld0VsZW1lbnRzLmFwcGVuZChpdGVt",
				"KTsKCQkJZWxzZSBuZXdFbGVtZW50cy5wdXNoKGl0ZW0pOwoJCX0KCQlyZXR1cm4gbmV3RWxlbWVudHM7",
				"Cgl9LnByb3RlY3QoKSwKCglhcHBlbmQ6IGZ1bmN0aW9uKGNvbGxlY3Rpb24pewoJCWZvciAodmFyIGkg",
				"PSAwLCBsID0gY29sbGVjdGlvbi5sZW5ndGg7IGkgPCBsOyBpKyspIHRoaXMucHVzaChjb2xsZWN0aW9u",
				"W2ldKTsKCQlyZXR1cm4gdGhpczsKCX0ucHJvdGVjdCgpLAoKCWVtcHR5OiBmdW5jdGlvbigpewoJCXdo",
				"aWxlICh0aGlzLmxlbmd0aCkgZGVsZXRlIHRoaXNbLS10aGlzLmxlbmd0aF07CgkJcmV0dXJuIHRoaXM7",
				"Cgl9LnByb3RlY3QoKQoKfSk7CgovLzwxLjJjb21wYXQ+CgpFbGVtZW50cy5hbGlhcygnZXh0ZW5kJywg",
				"J2FwcGVuZCcpOwoKLy88LzEuMmNvbXBhdD4KCihmdW5jdGlvbigpewoKLy8gRkYsIElFCnZhciBzcGxp",
				"Y2UgPSBBcnJheS5wcm90b3R5cGUuc3BsaWNlLCBvYmplY3QgPSB7JzAnOiAwLCAnMSc6IDEsIGxlbmd0",
				"aDogMn07CgpzcGxpY2UuY2FsbChvYmplY3QsIDEsIDEpOwppZiAob2JqZWN0WzFdID09IDEpIEVsZW1l",
				"bnRzLmltcGxlbWVudCgnc3BsaWNlJywgZnVuY3Rpb24oKXsKCXZhciBsZW5ndGggPSB0aGlzLmxlbmd0",
				"aDsKCXZhciByZXN1bHQgPSBzcGxpY2UuYXBwbHkodGhpcywgYXJndW1lbnRzKTsKCXdoaWxlIChsZW5n",
				"dGggPj0gdGhpcy5sZW5ndGgpIGRlbGV0ZSB0aGlzW2xlbmd0aC0tXTsKCXJldHVybiByZXN1bHQ7Cn0u",
				"cHJvdGVjdCgpKTsKCkFycmF5LmZvckVhY2hNZXRob2QoZnVuY3Rpb24obWV0aG9kLCBuYW1lKXsKCUVs",
				"ZW1lbnRzLmltcGxlbWVudChuYW1lLCBtZXRob2QpOwp9KTsKCkFycmF5Lm1pcnJvcihFbGVtZW50cyk7",
				"CgovKjxsdElFOD4qLwp2YXIgY3JlYXRlRWxlbWVudEFjY2VwdHNIVE1MOwp0cnkgewogICAgY3JlYXRl",
				"RWxlbWVudEFjY2VwdHNIVE1MID0gKGRvY3VtZW50LmNyZWF0ZUVsZW1lbnQoJzxpbnB1dCBuYW1lPXg+",
				"JykubmFtZSA9PSAneCcpOwp9IGNhdGNoIChlKXt9Cgp2YXIgZXNjYXBlUXVvdGVzID0gZnVuY3Rpb24o",
				"aHRtbCl7CglyZXR1cm4gKCcnICsgaHRtbCkucmVwbGFjZSgvJi9nLCAnJmFtcDsnKS5yZXBsYWNlKC8i",
				"L2csICcmcXVvdDsnKTsKfTsKLyo8L2x0SUU4PiovCgpEb2N1bWVudC5pbXBsZW1lbnQoewoKCW5ld0Vs",
				"ZW1lbnQ6IGZ1bmN0aW9uKHRhZywgcHJvcHMpewoJCWlmIChwcm9wcyAmJiBwcm9wcy5jaGVja2VkICE9",
				"IG51bGwpIHByb3BzLmRlZmF1bHRDaGVja2VkID0gcHJvcHMuY2hlY2tlZDsKCQkvKjxsdElFOD4qLy8v",
				"IEZpeCBmb3IgcmVhZG9ubHkgbmFtZSBhbmQgdHlwZSBwcm9wZXJ0aWVzIGluIElFIDwgOAoJCWlmIChj",
				"cmVhdGVFbGVtZW50QWNjZXB0c0hUTUwgJiYgcHJvcHMpewoJCQl0YWcgPSAnPCcgKyB0YWc7CgkJCWlm",
				"IChwcm9wcy5uYW1lKSB0YWcgKz0gJyBuYW1lPSInICsgZXNjYXBlUXVvdGVzKHByb3BzLm5hbWUpICsg",
				"JyInOwoJCQlpZiAocHJvcHMudHlwZSkgdGFnICs9ICcgdHlwZT0iJyArIGVzY2FwZVF1b3Rlcyhwcm9w",
				"cy50eXBlKSArICciJzsKCQkJdGFnICs9ICc+JzsKCQkJZGVsZXRlIHByb3BzLm5hbWU7CgkJCWRlbGV0",
				"ZSBwcm9wcy50eXBlOwoJCX0KCQkvKjwvbHRJRTg+Ki8KCQlyZXR1cm4gdGhpcy5pZCh0aGlzLmNyZWF0",
				"ZUVsZW1lbnQodGFnKSkuc2V0KHByb3BzKTsKCX0KCn0pOwoKfSkoKTsKCihmdW5jdGlvbigpewoKU2xp",
				"Y2sudWlkT2Yod2luZG93KTsKU2xpY2sudWlkT2YoZG9jdW1lbnQpOwoKRG9jdW1lbnQuaW1wbGVtZW50",
				"KHsKCgluZXdUZXh0Tm9kZTogZnVuY3Rpb24odGV4dCl7CgkJcmV0dXJuIHRoaXMuY3JlYXRlVGV4dE5v",
				"ZGUodGV4dCk7Cgl9LAoKCWdldERvY3VtZW50OiBmdW5jdGlvbigpewoJCXJldHVybiB0aGlzOwoJfSwK",
				"CglnZXRXaW5kb3c6IGZ1bmN0aW9uKCl7CgkJcmV0dXJuIHRoaXMud2luZG93OwoJfSwKCglpZDogKGZ1",
				"bmN0aW9uKCl7CgoJCXZhciB0eXBlcyA9IHsKCgkJCXN0cmluZzogZnVuY3Rpb24oaWQsIG5vY2FzaCwg",
				"ZG9jKXsKCQkJCWlkID0gU2xpY2suZmluZChkb2MsICcjJyArIGlkLnJlcGxhY2UoLyhcVykvZywgJ1xc",
				"JDEnKSk7CgkJCQlyZXR1cm4gKGlkKSA/IHR5cGVzLmVsZW1lbnQoaWQsIG5vY2FzaCkgOiBudWxsOwoJ",
				"CQl9LAoKCQkJZWxlbWVudDogZnVuY3Rpb24oZWwsIG5vY2FzaCl7CgkJCQlTbGljay51aWRPZihlbCk7",
				"CgkJCQlpZiAoIW5vY2FzaCAmJiAhZWwuJGZhbWlseSAmJiAhKC9eKD86b2JqZWN0fGVtYmVkKSQvaSku",
				"dGVzdChlbC50YWdOYW1lKSl7CgkJCQkJdmFyIGZpcmVFdmVudCA9IGVsLmZpcmVFdmVudDsKCQkJCQkv",
				"LyB3cmFwcGluZyBuZWVkZWQgaW4gSUU3LCBvciBlbHNlIGNyYXNoCgkJCQkJZWwuX2ZpcmVFdmVudCA9",
				"IGZ1bmN0aW9uKHR5cGUsIGV2ZW50KXsKCQkJCQkJcmV0dXJuIGZpcmVFdmVudCh0eXBlLCBldmVudCk7",
				"CgkJCQkJfTsKCQkJCQlPYmplY3QuYXBwZW5kKGVsLCBFbGVtZW50LlByb3RvdHlwZSk7CgkJCQl9CgkJ",
				"CQlyZXR1cm4gZWw7CgkJCX0sCgoJCQlvYmplY3Q6IGZ1bmN0aW9uKG9iaiwgbm9jYXNoLCBkb2MpewoJ",
				"CQkJaWYgKG9iai50b0VsZW1lbnQpIHJldHVybiB0eXBlcy5lbGVtZW50KG9iai50b0VsZW1lbnQoZG9j",
				"KSwgbm9jYXNoKTsKCQkJCXJldHVybiBudWxsOwoJCQl9CgoJCX07CgoJCXR5cGVzLnRleHRub2RlID0g",
				"dHlwZXMud2hpdGVzcGFjZSA9IHR5cGVzLndpbmRvdyA9IHR5cGVzLmRvY3VtZW50ID0gZnVuY3Rpb24o",
				"emVybyl7CgkJCXJldHVybiB6ZXJvOwoJCX07CgoJCXJldHVybiBmdW5jdGlvbihlbCwgbm9jYXNoLCBk",
				"b2MpewoJCQlpZiAoZWwgJiYgZWwuJGZhbWlseSAmJiBlbC51bmlxdWVOdW1iZXIpIHJldHVybiBlbDsK",
				"CQkJdmFyIHR5cGUgPSB0eXBlT2YoZWwpOwoJCQlyZXR1cm4gKHR5cGVzW3R5cGVdKSA/IHR5cGVzW3R5",
				"cGVdKGVsLCBub2Nhc2gsIGRvYyB8fCBkb2N1bWVudCkgOiBudWxsOwoJCX07CgoJfSkoKQoKfSk7Cgpp",
				"ZiAod2luZG93LiQgPT0gbnVsbCkgV2luZG93LmltcGxlbWVudCgnJCcsIGZ1bmN0aW9uKGVsLCBuYyl7",
				"CglyZXR1cm4gZG9jdW1lbnQuaWQoZWwsIG5jLCB0aGlzLmRvY3VtZW50KTsKfSk7CgpXaW5kb3cuaW1w",
				"bGVtZW50KHsKCglnZXREb2N1bWVudDogZnVuY3Rpb24oKXsKCQlyZXR1cm4gdGhpcy5kb2N1bWVudDsK",
				"CX0sCgoJZ2V0V2luZG93OiBmdW5jdGlvbigpewoJCXJldHVybiB0aGlzOwoJfQoKfSk7CgpbRG9jdW1l",
				"bnQsIEVsZW1lbnRdLmludm9rZSgnaW1wbGVtZW50JywgewoKCWdldEVsZW1lbnRzOiBmdW5jdGlvbihl",
				"eHByZXNzaW9uKXsKCQlyZXR1cm4gU2xpY2suc2VhcmNoKHRoaXMsIGV4cHJlc3Npb24sIG5ldyBFbGVt",
				"ZW50cyk7Cgl9LAoKCWdldEVsZW1lbnQ6IGZ1bmN0aW9uKGV4cHJlc3Npb24pewoJCXJldHVybiBkb2N1",
				"bWVudC5pZChTbGljay5maW5kKHRoaXMsIGV4cHJlc3Npb24pKTsKCX0KCn0pOwoKdmFyIGNvbnRhaW5z",
				"ID0ge2NvbnRhaW5zOiBmdW5jdGlvbihlbGVtZW50KXsKCXJldHVybiBTbGljay5jb250YWlucyh0aGlz",
				"LCBlbGVtZW50KTsKfX07CgppZiAoIWRvY3VtZW50LmNvbnRhaW5zKSBEb2N1bWVudC5pbXBsZW1lbnQo",
				"Y29udGFpbnMpOwppZiAoIWRvY3VtZW50LmNyZWF0ZUVsZW1lbnQoJ2RpdicpLmNvbnRhaW5zKSBFbGVt",
				"ZW50LmltcGxlbWVudChjb250YWlucyk7CgovLzwxLjJjb21wYXQ+CgpFbGVtZW50LmltcGxlbWVudCgn",
				"aGFzQ2hpbGQnLCBmdW5jdGlvbihlbGVtZW50KXsKCXJldHVybiB0aGlzICE9PSBlbGVtZW50ICYmIHRo",
				"aXMuY29udGFpbnMoZWxlbWVudCk7Cn0pOwoKKGZ1bmN0aW9uKHNlYXJjaCwgZmluZCwgbWF0Y2gpewoK",
				"CXRoaXMuU2VsZWN0b3JzID0ge307Cgl2YXIgcHNldWRvcyA9IHRoaXMuU2VsZWN0b3JzLlBzZXVkbyA9",
				"IG5ldyBIYXNoKCk7CgoJdmFyIGFkZFNsaWNrUHNldWRvcyA9IGZ1bmN0aW9uKCl7CgkJZm9yICh2YXIg",
				"bmFtZSBpbiBwc2V1ZG9zKSBpZiAocHNldWRvcy5oYXNPd25Qcm9wZXJ0eShuYW1lKSl7CgkJCVNsaWNr",
				"LmRlZmluZVBzZXVkbyhuYW1lLCBwc2V1ZG9zW25hbWVdKTsKCQkJZGVsZXRlIHBzZXVkb3NbbmFtZV07",
				"CgkJfQoJfTsKCglTbGljay5zZWFyY2ggPSBmdW5jdGlvbihjb250ZXh0LCBleHByZXNzaW9uLCBhcHBl",
				"bmQpewoJCWFkZFNsaWNrUHNldWRvcygpOwoJCXJldHVybiBzZWFyY2guY2FsbCh0aGlzLCBjb250ZXh0",
				"LCBleHByZXNzaW9uLCBhcHBlbmQpOwoJfTsKCglTbGljay5maW5kID0gZnVuY3Rpb24oY29udGV4dCwg",
				"ZXhwcmVzc2lvbil7CgkJYWRkU2xpY2tQc2V1ZG9zKCk7CgkJcmV0dXJuIGZpbmQuY2FsbCh0aGlzLCBj",
				"b250ZXh0LCBleHByZXNzaW9uKTsKCX07CgoJU2xpY2subWF0Y2ggPSBmdW5jdGlvbihub2RlLCBzZWxl",
				"Y3Rvcil7CgkJYWRkU2xpY2tQc2V1ZG9zKCk7CgkJcmV0dXJuIG1hdGNoLmNhbGwodGhpcywgbm9kZSwg",
				"c2VsZWN0b3IpOwoJfTsKCn0pKFNsaWNrLnNlYXJjaCwgU2xpY2suZmluZCwgU2xpY2subWF0Y2gpOwoK",
				"Ly88LzEuMmNvbXBhdD4KCi8vIHRyZWUgd2Fsa2luZwoKdmFyIGluamVjdENvbWJpbmF0b3IgPSBmdW5j",
				"dGlvbihleHByZXNzaW9uLCBjb21iaW5hdG9yKXsKCWlmICghZXhwcmVzc2lvbikgcmV0dXJuIGNvbWJp",
				"bmF0b3I7CgoJZXhwcmVzc2lvbiA9IE9iamVjdC5jbG9uZShTbGljay5wYXJzZShleHByZXNzaW9uKSk7",
				"CgoJdmFyIGV4cHJlc3Npb25zID0gZXhwcmVzc2lvbi5leHByZXNzaW9uczsKCWZvciAodmFyIGkgPSBl",
				"eHByZXNzaW9ucy5sZW5ndGg7IGktLTspCgkJZXhwcmVzc2lvbnNbaV1bMF0uY29tYmluYXRvciA9IGNv",
				"bWJpbmF0b3I7CgoJcmV0dXJuIGV4cHJlc3Npb247Cn07CgpPYmplY3QuZm9yRWFjaCh7CglnZXROZXh0",
				"OiAnficsCglnZXRQcmV2aW91czogJyF+JywKCWdldFBhcmVudDogJyEnCn0sIGZ1bmN0aW9uKGNvbWJp",
				"bmF0b3IsIG1ldGhvZCl7CglFbGVtZW50LmltcGxlbWVudChtZXRob2QsIGZ1bmN0aW9uKGV4cHJlc3Np",
				"b24pewoJCXJldHVybiB0aGlzLmdldEVsZW1lbnQoaW5qZWN0Q29tYmluYXRvcihleHByZXNzaW9uLCBj",
				"b21iaW5hdG9yKSk7Cgl9KTsKfSk7CgpPYmplY3QuZm9yRWFjaCh7CglnZXRBbGxOZXh0OiAnficsCgln",
				"ZXRBbGxQcmV2aW91czogJyF+JywKCWdldFNpYmxpbmdzOiAnfn4nLAoJZ2V0Q2hpbGRyZW46ICc+JywK",
				"CWdldFBhcmVudHM6ICchJwp9LCBmdW5jdGlvbihjb21iaW5hdG9yLCBtZXRob2QpewoJRWxlbWVudC5p",
				"bXBsZW1lbnQobWV0aG9kLCBmdW5jdGlvbihleHByZXNzaW9uKXsKCQlyZXR1cm4gdGhpcy5nZXRFbGVt",
				"ZW50cyhpbmplY3RDb21iaW5hdG9yKGV4cHJlc3Npb24sIGNvbWJpbmF0b3IpKTsKCX0pOwp9KTsKCkVs",
				"ZW1lbnQuaW1wbGVtZW50KHsKCglnZXRGaXJzdDogZnVuY3Rpb24oZXhwcmVzc2lvbil7CgkJcmV0dXJu",
				"IGRvY3VtZW50LmlkKFNsaWNrLnNlYXJjaCh0aGlzLCBpbmplY3RDb21iaW5hdG9yKGV4cHJlc3Npb24s",
				"ICc+JykpWzBdKTsKCX0sCgoJZ2V0TGFzdDogZnVuY3Rpb24oZXhwcmVzc2lvbil7CgkJcmV0dXJuIGRv",
				"Y3VtZW50LmlkKFNsaWNrLnNlYXJjaCh0aGlzLCBpbmplY3RDb21iaW5hdG9yKGV4cHJlc3Npb24sICc+",
				"JykpLmdldExhc3QoKSk7Cgl9LAoKCWdldFdpbmRvdzogZnVuY3Rpb24oKXsKCQlyZXR1cm4gdGhpcy5v",
				"d25lckRvY3VtZW50LndpbmRvdzsKCX0sCgoJZ2V0RG9jdW1lbnQ6IGZ1bmN0aW9uKCl7CgkJcmV0dXJu",
				"IHRoaXMub3duZXJEb2N1bWVudDsKCX0sCgoJZ2V0RWxlbWVudEJ5SWQ6IGZ1bmN0aW9uKGlkKXsKCQly",
				"ZXR1cm4gZG9jdW1lbnQuaWQoU2xpY2suZmluZCh0aGlzLCAnIycgKyAoJycgKyBpZCkucmVwbGFjZSgv",
				"KFxXKS9nLCAnXFwkMScpKSk7Cgl9LAoKCW1hdGNoOiBmdW5jdGlvbihleHByZXNzaW9uKXsKCQlyZXR1",
				"cm4gIWV4cHJlc3Npb24gfHwgU2xpY2subWF0Y2godGhpcywgZXhwcmVzc2lvbik7Cgl9Cgp9KTsKCi8v",
				"PDEuMmNvbXBhdD4KCmlmICh3aW5kb3cuJCQgPT0gbnVsbCkgV2luZG93LmltcGxlbWVudCgnJCQnLCBm",
				"dW5jdGlvbihzZWxlY3Rvcil7Cgl2YXIgZWxlbWVudHMgPSBuZXcgRWxlbWVudHM7CglpZiAoYXJndW1l",
				"bnRzLmxlbmd0aCA9PSAxICYmIHR5cGVvZiBzZWxlY3RvciA9PSAnc3RyaW5nJykgcmV0dXJuIFNsaWNr",
				"LnNlYXJjaCh0aGlzLmRvY3VtZW50LCBzZWxlY3RvciwgZWxlbWVudHMpOwoJdmFyIGFyZ3MgPSBBcnJh",
				"eS5mbGF0dGVuKGFyZ3VtZW50cyk7Cglmb3IgKHZhciBpID0gMCwgbCA9IGFyZ3MubGVuZ3RoOyBpIDwg",
				"bDsgaSsrKXsKCQl2YXIgaXRlbSA9IGFyZ3NbaV07CgkJc3dpdGNoICh0eXBlT2YoaXRlbSkpewoJCQlj",
				"YXNlICdlbGVtZW50JzogZWxlbWVudHMucHVzaChpdGVtKTsgYnJlYWs7CgkJCWNhc2UgJ3N0cmluZyc6",
				"IFNsaWNrLnNlYXJjaCh0aGlzLmRvY3VtZW50LCBpdGVtLCBlbGVtZW50cyk7CgkJfQoJfQoJcmV0dXJu",
				"IGVsZW1lbnRzOwp9KTsKCi8vPC8xLjJjb21wYXQ+CgppZiAod2luZG93LiQkID09IG51bGwpIFdpbmRv",
				"dy5pbXBsZW1lbnQoJyQkJywgZnVuY3Rpb24oc2VsZWN0b3IpewoJaWYgKGFyZ3VtZW50cy5sZW5ndGgg",
				"PT0gMSl7CgkJaWYgKHR5cGVvZiBzZWxlY3RvciA9PSAnc3RyaW5nJykgcmV0dXJuIFNsaWNrLnNlYXJj",
				"aCh0aGlzLmRvY3VtZW50LCBzZWxlY3RvciwgbmV3IEVsZW1lbnRzKTsKCQllbHNlIGlmIChUeXBlLmlz",
				"RW51bWVyYWJsZShzZWxlY3RvcikpIHJldHVybiBuZXcgRWxlbWVudHMoc2VsZWN0b3IpOwoJfQoJcmV0",
				"dXJuIG5ldyBFbGVtZW50cyhhcmd1bWVudHMpOwp9KTsKCi8vIEluc2VydGVycwoKdmFyIGluc2VydGVy",
				"cyA9IHsKCgliZWZvcmU6IGZ1bmN0aW9uKGNvbnRleHQsIGVsZW1lbnQpewoJCXZhciBwYXJlbnQgPSBl",
				"bGVtZW50LnBhcmVudE5vZGU7CgkJaWYgKHBhcmVudCkgcGFyZW50Lmluc2VydEJlZm9yZShjb250ZXh0",
				"LCBlbGVtZW50KTsKCX0sCgoJYWZ0ZXI6IGZ1bmN0aW9uKGNvbnRleHQsIGVsZW1lbnQpewoJCXZhciBw",
				"YXJlbnQgPSBlbGVtZW50LnBhcmVudE5vZGU7CgkJaWYgKHBhcmVudCkgcGFyZW50Lmluc2VydEJlZm9y",
				"ZShjb250ZXh0LCBlbGVtZW50Lm5leHRTaWJsaW5nKTsKCX0sCgoJYm90dG9tOiBmdW5jdGlvbihjb250",
				"ZXh0LCBlbGVtZW50KXsKCQllbGVtZW50LmFwcGVuZENoaWxkKGNvbnRleHQpOwoJfSwKCgl0b3A6IGZ1",
				"bmN0aW9uKGNvbnRleHQsIGVsZW1lbnQpewoJCWVsZW1lbnQuaW5zZXJ0QmVmb3JlKGNvbnRleHQsIGVs",
				"ZW1lbnQuZmlyc3RDaGlsZCk7Cgl9Cgp9OwoKaW5zZXJ0ZXJzLmluc2lkZSA9IGluc2VydGVycy5ib3R0",
				"b207CgovLzwxLjJjb21wYXQ+CgpPYmplY3QuZWFjaChpbnNlcnRlcnMsIGZ1bmN0aW9uKGluc2VydGVy",
				"LCB3aGVyZSl7CgoJd2hlcmUgPSB3aGVyZS5jYXBpdGFsaXplKCk7CgoJdmFyIG1ldGhvZHMgPSB7fTsK",
				"CgltZXRob2RzWydpbmplY3QnICsgd2hlcmVdID0gZnVuY3Rpb24oZWwpewoJCWluc2VydGVyKHRoaXMs",
				"IGRvY3VtZW50LmlkKGVsLCB0cnVlKSk7CgkJcmV0dXJuIHRoaXM7Cgl9OwoKCW1ldGhvZHNbJ2dyYWIn",
				"ICsgd2hlcmVdID0gZnVuY3Rpb24oZWwpewoJCWluc2VydGVyKGRvY3VtZW50LmlkKGVsLCB0cnVlKSwg",
				"dGhpcyk7CgkJcmV0dXJuIHRoaXM7Cgl9OwoKCUVsZW1lbnQuaW1wbGVtZW50KG1ldGhvZHMpOwoKfSk7",
				"CgovLzwvMS4yY29tcGF0PgoKLy8gZ2V0UHJvcGVydHkgLyBzZXRQcm9wZXJ0eQoKdmFyIHByb3BlcnR5",
				"R2V0dGVycyA9IHt9LCBwcm9wZXJ0eVNldHRlcnMgPSB7fTsKCi8vIHByb3BlcnRpZXMKCnZhciBwcm9w",
				"ZXJ0aWVzID0ge307CkFycmF5LmZvckVhY2goWwoJJ3R5cGUnLCAndmFsdWUnLCAnZGVmYXVsdFZhbHVl",
				"JywgJ2FjY2Vzc0tleScsICdjZWxsUGFkZGluZycsICdjZWxsU3BhY2luZycsICdjb2xTcGFuJywKCSdm",
				"cmFtZUJvcmRlcicsICdyb3dTcGFuJywgJ3RhYkluZGV4JywgJ3VzZU1hcCcKXSwgZnVuY3Rpb24ocHJv",
				"cGVydHkpewoJcHJvcGVydGllc1twcm9wZXJ0eS50b0xvd2VyQ2FzZSgpXSA9IHByb3BlcnR5Owp9KTsK",
				"CnByb3BlcnRpZXMuaHRtbCA9ICdpbm5lckhUTUwnOwpwcm9wZXJ0aWVzLnRleHQgPSAoZG9jdW1lbnQu",
				"Y3JlYXRlRWxlbWVudCgnZGl2JykudGV4dENvbnRlbnQgPT0gbnVsbCkgPyAnaW5uZXJUZXh0JzogJ3Rl",
				"eHRDb250ZW50JzsKCk9iamVjdC5mb3JFYWNoKHByb3BlcnRpZXMsIGZ1bmN0aW9uKHJlYWwsIGtleSl7",
				"Cglwcm9wZXJ0eVNldHRlcnNba2V5XSA9IGZ1bmN0aW9uKG5vZGUsIHZhbHVlKXsKCQlub2RlW3JlYWxd",
				"ID0gdmFsdWU7Cgl9OwoJcHJvcGVydHlHZXR0ZXJzW2tleV0gPSBmdW5jdGlvbihub2RlKXsKCQlyZXR1",
				"cm4gbm9kZVtyZWFsXTsKCX07Cn0pOwoKLy8gQm9vbGVhbnMKCnZhciBib29scyA9IFsKCSdjb21wYWN0",
				"JywgJ25vd3JhcCcsICdpc21hcCcsICdkZWNsYXJlJywgJ25vc2hhZGUnLCAnY2hlY2tlZCcsCgknZGlz",
				"YWJsZWQnLCAncmVhZE9ubHknLCAnbXVsdGlwbGUnLCAnc2VsZWN0ZWQnLCAnbm9yZXNpemUnLAoJJ2Rl",
				"ZmVyJywgJ2RlZmF1bHRDaGVja2VkJywgJ2F1dG9mb2N1cycsICdjb250cm9scycsICdhdXRvcGxheScs",
				"CgknbG9vcCcKXTsKCnZhciBib29sZWFucyA9IHt9OwpBcnJheS5mb3JFYWNoKGJvb2xzLCBmdW5jdGlv",
				"bihib29sKXsKCXZhciBsb3dlciA9IGJvb2wudG9Mb3dlckNhc2UoKTsKCWJvb2xlYW5zW2xvd2VyXSA9",
				"IGJvb2w7Cglwcm9wZXJ0eVNldHRlcnNbbG93ZXJdID0gZnVuY3Rpb24obm9kZSwgdmFsdWUpewoJCW5v",
				"ZGVbYm9vbF0gPSAhIXZhbHVlOwoJfTsKCXByb3BlcnR5R2V0dGVyc1tsb3dlcl0gPSBmdW5jdGlvbihu",
				"b2RlKXsKCQlyZXR1cm4gISFub2RlW2Jvb2xdOwoJfTsKfSk7CgovLyBTcGVjaWFsIGNhc2VzCgpPYmpl",
				"Y3QuYXBwZW5kKHByb3BlcnR5U2V0dGVycywgewoKCSdjbGFzcyc6IGZ1bmN0aW9uKG5vZGUsIHZhbHVl",
				"KXsKCQkoJ2NsYXNzTmFtZScgaW4gbm9kZSkgPyBub2RlLmNsYXNzTmFtZSA9ICh2YWx1ZSB8fCAnJykg",
				"OiBub2RlLnNldEF0dHJpYnV0ZSgnY2xhc3MnLCB2YWx1ZSk7Cgl9LAoKCSdmb3InOiBmdW5jdGlvbihu",
				"b2RlLCB2YWx1ZSl7CgkJKCdodG1sRm9yJyBpbiBub2RlKSA/IG5vZGUuaHRtbEZvciA9IHZhbHVlIDog",
				"bm9kZS5zZXRBdHRyaWJ1dGUoJ2ZvcicsIHZhbHVlKTsKCX0sCgoJJ3N0eWxlJzogZnVuY3Rpb24obm9k",
				"ZSwgdmFsdWUpewoJCShub2RlLnN0eWxlKSA/IG5vZGUuc3R5bGUuY3NzVGV4dCA9IHZhbHVlIDogbm9k",
				"ZS5zZXRBdHRyaWJ1dGUoJ3N0eWxlJywgdmFsdWUpOwoJfSwKCgkndmFsdWUnOiBmdW5jdGlvbihub2Rl",
				"LCB2YWx1ZSl7CgkJbm9kZS52YWx1ZSA9ICh2YWx1ZSAhPSBudWxsKSA/IHZhbHVlIDogJyc7Cgl9Cgp9",
				"KTsKCnByb3BlcnR5R2V0dGVyc1snY2xhc3MnXSA9IGZ1bmN0aW9uKG5vZGUpewoJcmV0dXJuICgnY2xh",
				"c3NOYW1lJyBpbiBub2RlKSA/IG5vZGUuY2xhc3NOYW1lIHx8IG51bGwgOiBub2RlLmdldEF0dHJpYnV0",
				"ZSgnY2xhc3MnKTsKfTsKCi8qIDx3ZWJraXQ+ICovCnZhciBlbCA9IGRvY3VtZW50LmNyZWF0ZUVsZW1l",
				"bnQoJ2J1dHRvbicpOwovLyBJRSBzZXRzIHR5cGUgYXMgcmVhZG9ubHkgYW5kIHRocm93cwp0cnkgeyBl",
				"bC50eXBlID0gJ2J1dHRvbic7IH0gY2F0Y2goZSl7fQppZiAoZWwudHlwZSAhPSAnYnV0dG9uJykgcHJv",
				"cGVydHlTZXR0ZXJzLnR5cGUgPSBmdW5jdGlvbihub2RlLCB2YWx1ZSl7Cglub2RlLnNldEF0dHJpYnV0",
				"ZSgndHlwZScsIHZhbHVlKTsKfTsKZWwgPSBudWxsOwovKiA8L3dlYmtpdD4gKi8KCi8qPElFPiovCnZh",
				"ciBpbnB1dCA9IGRvY3VtZW50LmNyZWF0ZUVsZW1lbnQoJ2lucHV0Jyk7CmlucHV0LnZhbHVlID0gJ3Qn",
				"OwppbnB1dC50eXBlID0gJ3N1Ym1pdCc7CmlmIChpbnB1dC52YWx1ZSAhPSAndCcpIHByb3BlcnR5U2V0",
				"dGVycy50eXBlID0gZnVuY3Rpb24obm9kZSwgdHlwZSl7Cgl2YXIgdmFsdWUgPSBub2RlLnZhbHVlOwoJ",
				"bm9kZS50eXBlID0gdHlwZTsKCW5vZGUudmFsdWUgPSB2YWx1ZTsKfTsKaW5wdXQgPSBudWxsOwovKjwv",
				"SUU+Ki8KCi8qIGdldFByb3BlcnR5LCBzZXRQcm9wZXJ0eSAqLwoKLyogPGx0SUU5PiAqLwp2YXIgcG9s",
				"bHV0ZXNHZXRBdHRyaWJ1dGUgPSAoZnVuY3Rpb24oZGl2KXsKCWRpdi5yYW5kb20gPSAnYXR0cmlidXRl",
				"JzsKCXJldHVybiAoZGl2LmdldEF0dHJpYnV0ZSgncmFuZG9tJykgPT0gJ2F0dHJpYnV0ZScpOwp9KShk",
				"b2N1bWVudC5jcmVhdGVFbGVtZW50KCdkaXYnKSk7CgovKiA8bHRJRTk+ICovCgpFbGVtZW50LmltcGxl",
				"bWVudCh7CgoJc2V0UHJvcGVydHk6IGZ1bmN0aW9uKG5hbWUsIHZhbHVlKXsKCQl2YXIgc2V0dGVyID0g",
				"cHJvcGVydHlTZXR0ZXJzW25hbWUudG9Mb3dlckNhc2UoKV07CgkJaWYgKHNldHRlcil7CgkJCXNldHRl",
				"cih0aGlzLCB2YWx1ZSk7CgkJfSBlbHNlIHsKCQkJLyogPGx0SUU5PiAqLwoJCQlpZiAocG9sbHV0ZXNH",
				"ZXRBdHRyaWJ1dGUpIHZhciBhdHRyaWJ1dGVXaGl0ZUxpc3QgPSB0aGlzLnJldHJpZXZlKCckYXR0cmli",
				"dXRlV2hpdGVMaXN0Jywge30pOwoJCQkvKiA8L2x0SUU5PiAqLwoKCQkJaWYgKHZhbHVlID09IG51bGwp",
				"ewoJCQkJdGhpcy5yZW1vdmVBdHRyaWJ1dGUobmFtZSk7CgkJCQkvKiA8bHRJRTk+ICovCgkJCQlpZiAo",
				"cG9sbHV0ZXNHZXRBdHRyaWJ1dGUpIGRlbGV0ZSBhdHRyaWJ1dGVXaGl0ZUxpc3RbbmFtZV07CgkJCQkv",
				"KiA8L2x0SUU5PiAqLwoJCQl9IGVsc2UgewoJCQkJdGhpcy5zZXRBdHRyaWJ1dGUobmFtZSwgJycgKyB2",
				"YWx1ZSk7CgkJCQkvKiA8bHRJRTk+ICovCgkJCQlpZiAocG9sbHV0ZXNHZXRBdHRyaWJ1dGUpIGF0dHJp",
				"YnV0ZVdoaXRlTGlzdFtuYW1lXSA9IHRydWU7CgkJCQkvKiA8L2x0SUU5PiAqLwoJCQl9CgkJfQoJCXJl",
				"dHVybiB0aGlzOwoJfSwKCglzZXRQcm9wZXJ0aWVzOiBmdW5jdGlvbihhdHRyaWJ1dGVzKXsKCQlmb3Ig",
				"KHZhciBhdHRyaWJ1dGUgaW4gYXR0cmlidXRlcykgdGhpcy5zZXRQcm9wZXJ0eShhdHRyaWJ1dGUsIGF0",
				"dHJpYnV0ZXNbYXR0cmlidXRlXSk7CgkJcmV0dXJuIHRoaXM7Cgl9LAoKCWdldFByb3BlcnR5OiBmdW5j",
				"dGlvbihuYW1lKXsKCQl2YXIgZ2V0dGVyID0gcHJvcGVydHlHZXR0ZXJzW25hbWUudG9Mb3dlckNhc2Uo",
				"KV07CgkJaWYgKGdldHRlcikgcmV0dXJuIGdldHRlcih0aGlzKTsKCQkvKiA8bHRJRTk+ICovCgkJaWYg",
				"KHBvbGx1dGVzR2V0QXR0cmlidXRlKXsKCQkJdmFyIGF0dHIgPSB0aGlzLmdldEF0dHJpYnV0ZU5vZGUo",
				"bmFtZSksIGF0dHJpYnV0ZVdoaXRlTGlzdCA9IHRoaXMucmV0cmlldmUoJyRhdHRyaWJ1dGVXaGl0ZUxp",
				"c3QnLCB7fSk7CgkJCWlmICghYXR0cikgcmV0dXJuIG51bGw7CgkJCWlmIChhdHRyLmV4cGFuZG8gJiYg",
				"IWF0dHJpYnV0ZVdoaXRlTGlzdFtuYW1lXSl7CgkJCQl2YXIgb3V0ZXIgPSB0aGlzLm91dGVySFRNTDsK",
				"CQkJCS8vIHNlZ21lbnQgYnkgdGhlIG9wZW5pbmcgdGFnIGFuZCBmaW5kIG1lbnRpb24gb2YgYXR0cmli",
				"dXRlIG5hbWUKCQkJCWlmIChvdXRlci5zdWJzdHIoMCwgb3V0ZXIuc2VhcmNoKC9cLz9bJyJdPz4oPyFb",
				"XjxdKjxbJyJdKS8pKS5pbmRleE9mKG5hbWUpIDwgMCkgcmV0dXJuIG51bGw7CgkJCQlhdHRyaWJ1dGVX",
				"aGl0ZUxpc3RbbmFtZV0gPSB0cnVlOwoJCQl9CgkJfQoJCS8qIDwvbHRJRTk+ICovCgkJdmFyIHJlc3Vs",
				"dCA9IFNsaWNrLmdldEF0dHJpYnV0ZSh0aGlzLCBuYW1lKTsKCQlyZXR1cm4gKCFyZXN1bHQgJiYgIVNs",
				"aWNrLmhhc0F0dHJpYnV0ZSh0aGlzLCBuYW1lKSkgPyBudWxsIDogcmVzdWx0OwoJfSwKCglnZXRQcm9w",
				"ZXJ0aWVzOiBmdW5jdGlvbigpewoJCXZhciBhcmdzID0gQXJyYXkuZnJvbShhcmd1bWVudHMpOwoJCXJl",
				"dHVybiBhcmdzLm1hcCh0aGlzLmdldFByb3BlcnR5LCB0aGlzKS5hc3NvY2lhdGUoYXJncyk7Cgl9LAoK",
				"CXJlbW92ZVByb3BlcnR5OiBmdW5jdGlvbihuYW1lKXsKCQlyZXR1cm4gdGhpcy5zZXRQcm9wZXJ0eShu",
				"YW1lLCBudWxsKTsKCX0sCgoJcmVtb3ZlUHJvcGVydGllczogZnVuY3Rpb24oKXsKCQlBcnJheS5lYWNo",
				"KGFyZ3VtZW50cywgdGhpcy5yZW1vdmVQcm9wZXJ0eSwgdGhpcyk7CgkJcmV0dXJuIHRoaXM7Cgl9LAoK",
				"CXNldDogZnVuY3Rpb24ocHJvcCwgdmFsdWUpewoJCXZhciBwcm9wZXJ0eSA9IEVsZW1lbnQuUHJvcGVy",
				"dGllc1twcm9wXTsKCQkocHJvcGVydHkgJiYgcHJvcGVydHkuc2V0KSA/IHByb3BlcnR5LnNldC5jYWxs",
				"KHRoaXMsIHZhbHVlKSA6IHRoaXMuc2V0UHJvcGVydHkocHJvcCwgdmFsdWUpOwoJfS5vdmVybG9hZFNl",
				"dHRlcigpLAoKCWdldDogZnVuY3Rpb24ocHJvcCl7CgkJdmFyIHByb3BlcnR5ID0gRWxlbWVudC5Qcm9w",
				"ZXJ0aWVzW3Byb3BdOwoJCXJldHVybiAocHJvcGVydHkgJiYgcHJvcGVydHkuZ2V0KSA/IHByb3BlcnR5",
				"LmdldC5hcHBseSh0aGlzKSA6IHRoaXMuZ2V0UHJvcGVydHkocHJvcCk7Cgl9Lm92ZXJsb2FkR2V0dGVy",
				"KCksCgoJZXJhc2U6IGZ1bmN0aW9uKHByb3ApewoJCXZhciBwcm9wZXJ0eSA9IEVsZW1lbnQuUHJvcGVy",
				"dGllc1twcm9wXTsKCQkocHJvcGVydHkgJiYgcHJvcGVydHkuZXJhc2UpID8gcHJvcGVydHkuZXJhc2Uu",
				"YXBwbHkodGhpcykgOiB0aGlzLnJlbW92ZVByb3BlcnR5KHByb3ApOwoJCXJldHVybiB0aGlzOwoJfSwK",
				"CgloYXNDbGFzczogZnVuY3Rpb24oY2xhc3NOYW1lKXsKCQlyZXR1cm4gdGhpcy5jbGFzc05hbWUuY2xl",
				"YW4oKS5jb250YWlucyhjbGFzc05hbWUsICcgJyk7Cgl9LAoKCWFkZENsYXNzOiBmdW5jdGlvbihjbGFz",
				"c05hbWUpewoJCWlmICghdGhpcy5oYXNDbGFzcyhjbGFzc05hbWUpKSB0aGlzLmNsYXNzTmFtZSA9ICh0",
				"aGlzLmNsYXNzTmFtZSArICcgJyArIGNsYXNzTmFtZSkuY2xlYW4oKTsKCQlyZXR1cm4gdGhpczsKCX0s",
				"CgoJcmVtb3ZlQ2xhc3M6IGZ1bmN0aW9uKGNsYXNzTmFtZSl7CgkJdGhpcy5jbGFzc05hbWUgPSB0aGlz",
				"LmNsYXNzTmFtZS5yZXBsYWNlKG5ldyBSZWdFeHAoJyhefFxccyknICsgY2xhc3NOYW1lICsgJyg/Olxc",
				"c3wkKScpLCAnJDEnKTsKCQlyZXR1cm4gdGhpczsKCX0sCgoJdG9nZ2xlQ2xhc3M6IGZ1bmN0aW9uKGNs",
				"YXNzTmFtZSwgZm9yY2UpewoJCWlmIChmb3JjZSA9PSBudWxsKSBmb3JjZSA9ICF0aGlzLmhhc0NsYXNz",
				"KGNsYXNzTmFtZSk7CgkJcmV0dXJuIChmb3JjZSkgPyB0aGlzLmFkZENsYXNzKGNsYXNzTmFtZSkgOiB0",
				"aGlzLnJlbW92ZUNsYXNzKGNsYXNzTmFtZSk7Cgl9LAoKCWFkb3B0OiBmdW5jdGlvbigpewoJCXZhciBw",
				"YXJlbnQgPSB0aGlzLCBmcmFnbWVudCwgZWxlbWVudHMgPSBBcnJheS5mbGF0dGVuKGFyZ3VtZW50cyks",
				"IGxlbmd0aCA9IGVsZW1lbnRzLmxlbmd0aDsKCQlpZiAobGVuZ3RoID4gMSkgcGFyZW50ID0gZnJhZ21l",
				"bnQgPSBkb2N1bWVudC5jcmVhdGVEb2N1bWVudEZyYWdtZW50KCk7CgoJCWZvciAodmFyIGkgPSAwOyBp",
				"IDwgbGVuZ3RoOyBpKyspewoJCQl2YXIgZWxlbWVudCA9IGRvY3VtZW50LmlkKGVsZW1lbnRzW2ldLCB0",
				"cnVlKTsKCQkJaWYgKGVsZW1lbnQpIHBhcmVudC5hcHBlbmRDaGlsZChlbGVtZW50KTsKCQl9CgoJCWlm",
				"IChmcmFnbWVudCkgdGhpcy5hcHBlbmRDaGlsZChmcmFnbWVudCk7CgoJCXJldHVybiB0aGlzOwoJfSwK",
				"CglhcHBlbmRUZXh0OiBmdW5jdGlvbih0ZXh0LCB3aGVyZSl7CgkJcmV0dXJuIHRoaXMuZ3JhYih0aGlz",
				"LmdldERvY3VtZW50KCkubmV3VGV4dE5vZGUodGV4dCksIHdoZXJlKTsKCX0sCgoJZ3JhYjogZnVuY3Rp",
				"b24oZWwsIHdoZXJlKXsKCQlpbnNlcnRlcnNbd2hlcmUgfHwgJ2JvdHRvbSddKGRvY3VtZW50LmlkKGVs",
				"LCB0cnVlKSwgdGhpcyk7CgkJcmV0dXJuIHRoaXM7Cgl9LAoKCWluamVjdDogZnVuY3Rpb24oZWwsIHdo",
				"ZXJlKXsKCQlpbnNlcnRlcnNbd2hlcmUgfHwgJ2JvdHRvbSddKHRoaXMsIGRvY3VtZW50LmlkKGVsLCB0",
				"cnVlKSk7CgkJcmV0dXJuIHRoaXM7Cgl9LAoKCXJlcGxhY2VzOiBmdW5jdGlvbihlbCl7CgkJZWwgPSBk",
				"b2N1bWVudC5pZChlbCwgdHJ1ZSk7CgkJZWwucGFyZW50Tm9kZS5yZXBsYWNlQ2hpbGQodGhpcywgZWwp",
				"OwoJCXJldHVybiB0aGlzOwoJfSwKCgl3cmFwczogZnVuY3Rpb24oZWwsIHdoZXJlKXsKCQllbCA9IGRv",
				"Y3VtZW50LmlkKGVsLCB0cnVlKTsKCQlyZXR1cm4gdGhpcy5yZXBsYWNlcyhlbCkuZ3JhYihlbCwgd2hl",
				"cmUpOwoJfSwKCglnZXRTZWxlY3RlZDogZnVuY3Rpb24oKXsKCQl0aGlzLnNlbGVjdGVkSW5kZXg7IC8v",
				"IFNhZmFyaSAzLjIuMQoJCXJldHVybiBuZXcgRWxlbWVudHMoQXJyYXkuZnJvbSh0aGlzLm9wdGlvbnMp",
				"LmZpbHRlcihmdW5jdGlvbihvcHRpb24pewoJCQlyZXR1cm4gb3B0aW9uLnNlbGVjdGVkOwoJCX0pKTsK",
				"CX0sCgoJdG9RdWVyeVN0cmluZzogZnVuY3Rpb24oKXsKCQl2YXIgcXVlcnlTdHJpbmcgPSBbXTsKCQl0",
				"aGlzLmdldEVsZW1lbnRzKCdpbnB1dCwgc2VsZWN0LCB0ZXh0YXJlYScpLmVhY2goZnVuY3Rpb24oZWwp",
				"ewoJCQl2YXIgdHlwZSA9IGVsLnR5cGU7CgkJCWlmICghZWwubmFtZSB8fCBlbC5kaXNhYmxlZCB8fCB0",
				"eXBlID09ICdzdWJtaXQnIHx8IHR5cGUgPT0gJ3Jlc2V0JyB8fCB0eXBlID09ICdmaWxlJyB8fCB0eXBl",
				"ID09ICdpbWFnZScpIHJldHVybjsKCgkJCXZhciB2YWx1ZSA9IChlbC5nZXQoJ3RhZycpID09ICdzZWxl",
				"Y3QnKSA/IGVsLmdldFNlbGVjdGVkKCkubWFwKGZ1bmN0aW9uKG9wdCl7CgkJCQkvLyBJRQoJCQkJcmV0",
				"dXJuIGRvY3VtZW50LmlkKG9wdCkuZ2V0KCd2YWx1ZScpOwoJCQl9KSA6ICgodHlwZSA9PSAncmFkaW8n",
				"IHx8IHR5cGUgPT0gJ2NoZWNrYm94JykgJiYgIWVsLmNoZWNrZWQpID8gbnVsbCA6IGVsLmdldCgndmFs",
				"dWUnKTsKCgkJCUFycmF5LmZyb20odmFsdWUpLmVhY2goZnVuY3Rpb24odmFsKXsKCQkJCWlmICh0eXBl",
				"b2YgdmFsICE9ICd1bmRlZmluZWQnKSBxdWVyeVN0cmluZy5wdXNoKGVuY29kZVVSSUNvbXBvbmVudChl",
				"bC5uYW1lKSArICc9JyArIGVuY29kZVVSSUNvbXBvbmVudCh2YWwpKTsKCQkJfSk7CgkJfSk7CgkJcmV0",
				"dXJuIHF1ZXJ5U3RyaW5nLmpvaW4oJyYnKTsKCX0KCn0pOwoKdmFyIGNvbGxlY3RlZCA9IHt9LCBzdG9y",
				"YWdlID0ge307Cgp2YXIgZ2V0ID0gZnVuY3Rpb24odWlkKXsKCXJldHVybiAoc3RvcmFnZVt1aWRdIHx8",
				"IChzdG9yYWdlW3VpZF0gPSB7fSkpOwp9OwoKdmFyIGNsZWFuID0gZnVuY3Rpb24oaXRlbSl7Cgl2YXIg",
				"dWlkID0gaXRlbS51bmlxdWVOdW1iZXI7CglpZiAoaXRlbS5yZW1vdmVFdmVudHMpIGl0ZW0ucmVtb3Zl",
				"RXZlbnRzKCk7CglpZiAoaXRlbS5jbGVhckF0dHJpYnV0ZXMpIGl0ZW0uY2xlYXJBdHRyaWJ1dGVzKCk7",
				"CglpZiAodWlkICE9IG51bGwpewoJCWRlbGV0ZSBjb2xsZWN0ZWRbdWlkXTsKCQlkZWxldGUgc3RvcmFn",
				"ZVt1aWRdOwoJfQoJcmV0dXJuIGl0ZW07Cn07Cgp2YXIgZm9ybVByb3BzID0ge2lucHV0OiAnY2hlY2tl",
				"ZCcsIG9wdGlvbjogJ3NlbGVjdGVkJywgdGV4dGFyZWE6ICd2YWx1ZSd9OwoKRWxlbWVudC5pbXBsZW1l",
				"bnQoewoKCWRlc3Ryb3k6IGZ1bmN0aW9uKCl7CgkJdmFyIGNoaWxkcmVuID0gY2xlYW4odGhpcykuZ2V0",
				"RWxlbWVudHNCeVRhZ05hbWUoJyonKTsKCQlBcnJheS5lYWNoKGNoaWxkcmVuLCBjbGVhbik7CgkJRWxl",
				"bWVudC5kaXNwb3NlKHRoaXMpOwoJCXJldHVybiBudWxsOwoJfSwKCgllbXB0eTogZnVuY3Rpb24oKXsK",
				"CQlBcnJheS5mcm9tKHRoaXMuY2hpbGROb2RlcykuZWFjaChFbGVtZW50LmRpc3Bvc2UpOwoJCXJldHVy",
				"biB0aGlzOwoJfSwKCglkaXNwb3NlOiBmdW5jdGlvbigpewoJCXJldHVybiAodGhpcy5wYXJlbnROb2Rl",
				"KSA/IHRoaXMucGFyZW50Tm9kZS5yZW1vdmVDaGlsZCh0aGlzKSA6IHRoaXM7Cgl9LAoKCWNsb25lOiBm",
				"dW5jdGlvbihjb250ZW50cywga2VlcGlkKXsKCQljb250ZW50cyA9IGNvbnRlbnRzICE9PSBmYWxzZTsK",
				"CQl2YXIgY2xvbmUgPSB0aGlzLmNsb25lTm9kZShjb250ZW50cyksIGNlID0gW2Nsb25lXSwgdGUgPSBb",
				"dGhpc10sIGk7CgoJCWlmIChjb250ZW50cyl7CgkJCWNlLmFwcGVuZChBcnJheS5mcm9tKGNsb25lLmdl",
				"dEVsZW1lbnRzQnlUYWdOYW1lKCcqJykpKTsKCQkJdGUuYXBwZW5kKEFycmF5LmZyb20odGhpcy5nZXRF",
				"bGVtZW50c0J5VGFnTmFtZSgnKicpKSk7CgkJfQoKCQlmb3IgKGkgPSBjZS5sZW5ndGg7IGktLTspewoJ",
				"CQl2YXIgbm9kZSA9IGNlW2ldLCBlbGVtZW50ID0gdGVbaV07CgkJCWlmICgha2VlcGlkKSBub2RlLnJl",
				"bW92ZUF0dHJpYnV0ZSgnaWQnKTsKCQkJLyo8bHRJRTk+Ki8KCQkJaWYgKG5vZGUuY2xlYXJBdHRyaWJ1",
				"dGVzKXsKCQkJCW5vZGUuY2xlYXJBdHRyaWJ1dGVzKCk7CgkJCQlub2RlLm1lcmdlQXR0cmlidXRlcyhl",
				"bGVtZW50KTsKCQkJCW5vZGUucmVtb3ZlQXR0cmlidXRlKCd1bmlxdWVOdW1iZXInKTsKCQkJCWlmIChu",
				"b2RlLm9wdGlvbnMpewoJCQkJCXZhciBubyA9IG5vZGUub3B0aW9ucywgZW8gPSBlbGVtZW50Lm9wdGlv",
				"bnM7CgkJCQkJZm9yICh2YXIgaiA9IG5vLmxlbmd0aDsgai0tOykgbm9bal0uc2VsZWN0ZWQgPSBlb1tq",
				"XS5zZWxlY3RlZDsKCQkJCX0KCQkJfQoJCQkvKjwvbHRJRTk+Ki8KCQkJdmFyIHByb3AgPSBmb3JtUHJv",
				"cHNbZWxlbWVudC50YWdOYW1lLnRvTG93ZXJDYXNlKCldOwoJCQlpZiAocHJvcCAmJiBlbGVtZW50W3By",
				"b3BdKSBub2RlW3Byb3BdID0gZWxlbWVudFtwcm9wXTsKCQl9CgoJCS8qPGx0SUU5PiovCgkJaWYgKEJy",
				"b3dzZXIuaWUpewoJCQl2YXIgY28gPSBjbG9uZS5nZXRFbGVtZW50c0J5VGFnTmFtZSgnb2JqZWN0Jyks",
				"IHRvID0gdGhpcy5nZXRFbGVtZW50c0J5VGFnTmFtZSgnb2JqZWN0Jyk7CgkJCWZvciAoaSA9IGNvLmxl",
				"bmd0aDsgaS0tOykgY29baV0ub3V0ZXJIVE1MID0gdG9baV0ub3V0ZXJIVE1MOwoJCX0KCQkvKjwvbHRJ",
				"RTk+Ki8KCQlyZXR1cm4gZG9jdW1lbnQuaWQoY2xvbmUpOwoJfQoKfSk7CgpbRWxlbWVudCwgV2luZG93",
				"LCBEb2N1bWVudF0uaW52b2tlKCdpbXBsZW1lbnQnLCB7CgoJYWRkTGlzdGVuZXI6IGZ1bmN0aW9uKHR5",
				"cGUsIGZuKXsKCQlpZiAodHlwZSA9PSAndW5sb2FkJyl7CgkJCXZhciBvbGQgPSBmbiwgc2VsZiA9IHRo",
				"aXM7CgkJCWZuID0gZnVuY3Rpb24oKXsKCQkJCXNlbGYucmVtb3ZlTGlzdGVuZXIoJ3VubG9hZCcsIGZu",
				"KTsKCQkJCW9sZCgpOwoJCQl9OwoJCX0gZWxzZSB7CgkJCWNvbGxlY3RlZFtTbGljay51aWRPZih0aGlz",
				"KV0gPSB0aGlzOwoJCX0KCQlpZiAodGhpcy5hZGRFdmVudExpc3RlbmVyKSB0aGlzLmFkZEV2ZW50TGlz",
				"dGVuZXIodHlwZSwgZm4sICEhYXJndW1lbnRzWzJdKTsKCQllbHNlIHRoaXMuYXR0YWNoRXZlbnQoJ29u",
				"JyArIHR5cGUsIGZuKTsKCQlyZXR1cm4gdGhpczsKCX0sCgoJcmVtb3ZlTGlzdGVuZXI6IGZ1bmN0aW9u",
				"KHR5cGUsIGZuKXsKCQlpZiAodGhpcy5yZW1vdmVFdmVudExpc3RlbmVyKSB0aGlzLnJlbW92ZUV2ZW50",
				"TGlzdGVuZXIodHlwZSwgZm4sICEhYXJndW1lbnRzWzJdKTsKCQllbHNlIHRoaXMuZGV0YWNoRXZlbnQo",
				"J29uJyArIHR5cGUsIGZuKTsKCQlyZXR1cm4gdGhpczsKCX0sCgoJcmV0cmlldmU6IGZ1bmN0aW9uKHBy",
				"b3BlcnR5LCBkZmx0KXsKCQl2YXIgc3RvcmFnZSA9IGdldChTbGljay51aWRPZih0aGlzKSksIHByb3Ag",
				"PSBzdG9yYWdlW3Byb3BlcnR5XTsKCQlpZiAoZGZsdCAhPSBudWxsICYmIHByb3AgPT0gbnVsbCkgcHJv",
				"cCA9IHN0b3JhZ2VbcHJvcGVydHldID0gZGZsdDsKCQlyZXR1cm4gcHJvcCAhPSBudWxsID8gcHJvcCA6",
				"IG51bGw7Cgl9LAoKCXN0b3JlOiBmdW5jdGlvbihwcm9wZXJ0eSwgdmFsdWUpewoJCXZhciBzdG9yYWdl",
				"ID0gZ2V0KFNsaWNrLnVpZE9mKHRoaXMpKTsKCQlzdG9yYWdlW3Byb3BlcnR5XSA9IHZhbHVlOwoJCXJl",
				"dHVybiB0aGlzOwoJfSwKCgllbGltaW5hdGU6IGZ1bmN0aW9uKHByb3BlcnR5KXsKCQl2YXIgc3RvcmFn",
				"ZSA9IGdldChTbGljay51aWRPZih0aGlzKSk7CgkJZGVsZXRlIHN0b3JhZ2VbcHJvcGVydHldOwoJCXJl",
				"dHVybiB0aGlzOwoJfQoKfSk7CgovKjxsdElFOT4qLwppZiAod2luZG93LmF0dGFjaEV2ZW50ICYmICF3",
				"aW5kb3cuYWRkRXZlbnRMaXN0ZW5lcikgd2luZG93LmFkZExpc3RlbmVyKCd1bmxvYWQnLCBmdW5jdGlv",
				"bigpewoJT2JqZWN0LmVhY2goY29sbGVjdGVkLCBjbGVhbik7CglpZiAod2luZG93LkNvbGxlY3RHYXJi",
				"YWdlKSBDb2xsZWN0R2FyYmFnZSgpOwp9KTsKLyo8L2x0SUU5PiovCgpFbGVtZW50LlByb3BlcnRpZXMg",
				"PSB7fTsKCi8vPDEuMmNvbXBhdD4KCkVsZW1lbnQuUHJvcGVydGllcyA9IG5ldyBIYXNoOwoKLy88LzEu",
				"MmNvbXBhdD4KCkVsZW1lbnQuUHJvcGVydGllcy5zdHlsZSA9IHsKCglzZXQ6IGZ1bmN0aW9uKHN0eWxl",
				"KXsKCQl0aGlzLnN0eWxlLmNzc1RleHQgPSBzdHlsZTsKCX0sCgoJZ2V0OiBmdW5jdGlvbigpewoJCXJl",
				"dHVybiB0aGlzLnN0eWxlLmNzc1RleHQ7Cgl9LAoKCWVyYXNlOiBmdW5jdGlvbigpewoJCXRoaXMuc3R5",
				"bGUuY3NzVGV4dCA9ICcnOwoJfQoKfTsKCkVsZW1lbnQuUHJvcGVydGllcy50YWcgPSB7CgoJZ2V0OiBm",
				"dW5jdGlvbigpewoJCXJldHVybiB0aGlzLnRhZ05hbWUudG9Mb3dlckNhc2UoKTsKCX0KCn07CgpFbGVt",
				"ZW50LlByb3BlcnRpZXMuaHRtbCA9IHsKCglzZXQ6IGZ1bmN0aW9uKGh0bWwpewoJCWlmIChodG1sID09",
				"IG51bGwpIGh0bWwgPSAnJzsKCQllbHNlIGlmICh0eXBlT2YoaHRtbCkgPT0gJ2FycmF5JykgaHRtbCA9",
				"IGh0bWwuam9pbignJyk7CgkJdGhpcy5pbm5lckhUTUwgPSBodG1sOwoJfSwKCgllcmFzZTogZnVuY3Rp",
				"b24oKXsKCQl0aGlzLmlubmVySFRNTCA9ICcnOwoJfQoKfTsKCi8qPGx0SUU5PiovCi8vIHRlY2huaXF1",
				"ZSBieSBqZGJhcmxldHQgLSBodHRwOi8vamRiYXJ0bGV0dC5jb20vaW5uZXJzaGl2Lwp2YXIgZGl2ID0g",
				"ZG9jdW1lbnQuY3JlYXRlRWxlbWVudCgnZGl2Jyk7CmRpdi5pbm5lckhUTUwgPSAnPG5hdj48L25hdj4n",
				"Owp2YXIgc3VwcG9ydHNIVE1MNUVsZW1lbnRzID0gKGRpdi5jaGlsZE5vZGVzLmxlbmd0aCA9PSAxKTsK",
				"aWYgKCFzdXBwb3J0c0hUTUw1RWxlbWVudHMpewoJdmFyIHRhZ3MgPSAnYWJiciBhcnRpY2xlIGFzaWRl",
				"IGF1ZGlvIGNhbnZhcyBkYXRhbGlzdCBkZXRhaWxzIGZpZ2NhcHRpb24gZmlndXJlIGZvb3RlciBoZWFk",
				"ZXIgaGdyb3VwIG1hcmsgbWV0ZXIgbmF2IG91dHB1dCBwcm9ncmVzcyBzZWN0aW9uIHN1bW1hcnkgdGlt",
				"ZSB2aWRlbycuc3BsaXQoJyAnKSwKCQlmcmFnbWVudCA9IGRvY3VtZW50LmNyZWF0ZURvY3VtZW50RnJh",
				"Z21lbnQoKSwgbCA9IHRhZ3MubGVuZ3RoOwoJd2hpbGUgKGwtLSkgZnJhZ21lbnQuY3JlYXRlRWxlbWVu",
				"dCh0YWdzW2xdKTsKfQpkaXYgPSBudWxsOwovKjwvbHRJRTk+Ki8KCi8qPElFPiovCnZhciBzdXBwb3J0",
				"c1RhYmxlSW5uZXJIVE1MID0gRnVuY3Rpb24uYXR0ZW1wdChmdW5jdGlvbigpewoJdmFyIHRhYmxlID0g",
				"ZG9jdW1lbnQuY3JlYXRlRWxlbWVudCgndGFibGUnKTsKCXRhYmxlLmlubmVySFRNTCA9ICc8dHI+PHRk",
				"PjwvdGQ+PC90cj4nOwoJcmV0dXJuIHRydWU7Cn0pOwoKLyo8bHRGRjQ+Ki8KdmFyIHRyID0gZG9jdW1l",
				"bnQuY3JlYXRlRWxlbWVudCgndHInKSwgaHRtbCA9ICc8dGQ+PC90ZD4nOwp0ci5pbm5lckhUTUwgPSBo",
				"dG1sOwp2YXIgc3VwcG9ydHNUUklubmVySFRNTCA9ICh0ci5pbm5lckhUTUwgPT0gaHRtbCk7CnRyID0g",
				"bnVsbDsKLyo8L2x0RkY0PiovCgppZiAoIXN1cHBvcnRzVGFibGVJbm5lckhUTUwgfHwgIXN1cHBvcnRz",
				"VFJJbm5lckhUTUwgfHwgIXN1cHBvcnRzSFRNTDVFbGVtZW50cyl7CgoJRWxlbWVudC5Qcm9wZXJ0aWVz",
				"Lmh0bWwuc2V0ID0gKGZ1bmN0aW9uKHNldCl7CgoJCXZhciB0cmFuc2xhdGlvbnMgPSB7CgkJCXRhYmxl",
				"OiBbMSwgJzx0YWJsZT4nLCAnPC90YWJsZT4nXSwKCQkJc2VsZWN0OiBbMSwgJzxzZWxlY3Q+JywgJzwv",
				"c2VsZWN0PiddLAoJCQl0Ym9keTogWzIsICc8dGFibGU+PHRib2R5PicsICc8L3Rib2R5PjwvdGFibGU+",
				"J10sCgkJCXRyOiBbMywgJzx0YWJsZT48dGJvZHk+PHRyPicsICc8L3RyPjwvdGJvZHk+PC90YWJsZT4n",
				"XQoJCX07CgoJCXRyYW5zbGF0aW9ucy50aGVhZCA9IHRyYW5zbGF0aW9ucy50Zm9vdCA9IHRyYW5zbGF0",
				"aW9ucy50Ym9keTsKCgkJcmV0dXJuIGZ1bmN0aW9uKGh0bWwpewoJCQl2YXIgd3JhcCA9IHRyYW5zbGF0",
				"aW9uc1t0aGlzLmdldCgndGFnJyldOwoJCQlpZiAoIXdyYXAgJiYgIXN1cHBvcnRzSFRNTDVFbGVtZW50",
				"cykgd3JhcCA9IFswLCAnJywgJyddOwoJCQlpZiAoIXdyYXApIHJldHVybiBzZXQuY2FsbCh0aGlzLCBo",
				"dG1sKTsKCgkJCXZhciBsZXZlbCA9IHdyYXBbMF0sIHdyYXBwZXIgPSBkb2N1bWVudC5jcmVhdGVFbGVt",
				"ZW50KCdkaXYnKSwgdGFyZ2V0ID0gd3JhcHBlcjsKCQkJaWYgKCFzdXBwb3J0c0hUTUw1RWxlbWVudHMp",
				"IGZyYWdtZW50LmFwcGVuZENoaWxkKHdyYXBwZXIpOwoJCQl3cmFwcGVyLmlubmVySFRNTCA9IFt3cmFw",
				"WzFdLCBodG1sLCB3cmFwWzJdXS5mbGF0dGVuKCkuam9pbignJyk7CgkJCXdoaWxlIChsZXZlbC0tKSB0",
				"YXJnZXQgPSB0YXJnZXQuZmlyc3RDaGlsZDsKCQkJdGhpcy5lbXB0eSgpLmFkb3B0KHRhcmdldC5jaGls",
				"ZE5vZGVzKTsKCQkJaWYgKCFzdXBwb3J0c0hUTUw1RWxlbWVudHMpIGZyYWdtZW50LnJlbW92ZUNoaWxk",
				"KHdyYXBwZXIpOwoJCQl3cmFwcGVyID0gbnVsbDsKCQl9OwoKCX0pKEVsZW1lbnQuUHJvcGVydGllcy5o",
				"dG1sLnNldCk7Cn0KLyo8L0lFPiovCgovKjxsdElFOT4qLwp2YXIgdGVzdEZvcm0gPSBkb2N1bWVudC5j",
				"cmVhdGVFbGVtZW50KCdmb3JtJyk7CnRlc3RGb3JtLmlubmVySFRNTCA9ICc8c2VsZWN0PjxvcHRpb24+",
				"czwvb3B0aW9uPjwvc2VsZWN0Pic7CgppZiAodGVzdEZvcm0uZmlyc3RDaGlsZC52YWx1ZSAhPSAncycp",
				"IEVsZW1lbnQuUHJvcGVydGllcy52YWx1ZSA9IHsKCglzZXQ6IGZ1bmN0aW9uKHZhbHVlKXsKCQl2YXIg",
				"dGFnID0gdGhpcy5nZXQoJ3RhZycpOwoJCWlmICh0YWcgIT0gJ3NlbGVjdCcpIHJldHVybiB0aGlzLnNl",
				"dFByb3BlcnR5KCd2YWx1ZScsIHZhbHVlKTsKCQl2YXIgb3B0aW9ucyA9IHRoaXMuZ2V0RWxlbWVudHMo",
				"J29wdGlvbicpOwoJCWZvciAodmFyIGkgPSAwOyBpIDwgb3B0aW9ucy5sZW5ndGg7IGkrKyl7CgkJCXZh",
				"ciBvcHRpb24gPSBvcHRpb25zW2ldLAoJCQkJYXR0ciA9IG9wdGlvbi5nZXRBdHRyaWJ1dGVOb2RlKCd2",
				"YWx1ZScpLAoJCQkJb3B0aW9uVmFsdWUgPSAoYXR0ciAmJiBhdHRyLnNwZWNpZmllZCkgPyBvcHRpb24u",
				"dmFsdWUgOiBvcHRpb24uZ2V0KCd0ZXh0Jyk7CgkJCWlmIChvcHRpb25WYWx1ZSA9PSB2YWx1ZSkgcmV0",
				"dXJuIG9wdGlvbi5zZWxlY3RlZCA9IHRydWU7CgkJfQoJfSwKCglnZXQ6IGZ1bmN0aW9uKCl7CgkJdmFy",
				"IG9wdGlvbiA9IHRoaXMsIHRhZyA9IG9wdGlvbi5nZXQoJ3RhZycpOwoKCQlpZiAodGFnICE9ICdzZWxl",
				"Y3QnICYmIHRhZyAhPSAnb3B0aW9uJykgcmV0dXJuIHRoaXMuZ2V0UHJvcGVydHkoJ3ZhbHVlJyk7CgoJ",
				"CWlmICh0YWcgPT0gJ3NlbGVjdCcgJiYgIShvcHRpb24gPSBvcHRpb24uZ2V0U2VsZWN0ZWQoKVswXSkp",
				"IHJldHVybiAnJzsKCgkJdmFyIGF0dHIgPSBvcHRpb24uZ2V0QXR0cmlidXRlTm9kZSgndmFsdWUnKTsK",
				"CQlyZXR1cm4gKGF0dHIgJiYgYXR0ci5zcGVjaWZpZWQpID8gb3B0aW9uLnZhbHVlIDogb3B0aW9uLmdl",
				"dCgndGV4dCcpOwoJfQoKfTsKdGVzdEZvcm0gPSBudWxsOwovKjwvbHRJRTk+Ki8KCi8qPElFPiovCmlm",
				"IChkb2N1bWVudC5jcmVhdGVFbGVtZW50KCdkaXYnKS5nZXRBdHRyaWJ1dGVOb2RlKCdpZCcpKSBFbGVt",
				"ZW50LlByb3BlcnRpZXMuaWQgPSB7CglzZXQ6IGZ1bmN0aW9uKGlkKXsKCQl0aGlzLmlkID0gdGhpcy5n",
				"ZXRBdHRyaWJ1dGVOb2RlKCdpZCcpLnZhbHVlID0gaWQ7Cgl9LAoJZ2V0OiBmdW5jdGlvbigpewoJCXJl",
				"dHVybiB0aGlzLmlkIHx8IG51bGw7Cgl9LAoJZXJhc2U6IGZ1bmN0aW9uKCl7CgkJdGhpcy5pZCA9IHRo",
				"aXMuZ2V0QXR0cmlidXRlTm9kZSgnaWQnKS52YWx1ZSA9ICcnOwoJfQp9OwovKjwvSUU+Ki8KCn0pKCk7",
				"CgoKLyoKLS0tCgpuYW1lOiBFbGVtZW50LlN0eWxlCgpkZXNjcmlwdGlvbjogQ29udGFpbnMgbWV0aG9k",
				"cyBmb3IgaW50ZXJhY3Rpbmcgd2l0aCB0aGUgc3R5bGVzIG9mIEVsZW1lbnRzIGluIGEgZmFzaGlvbmFi",
				"bGUgd2F5LgoKbGljZW5zZTogTUlULXN0eWxlIGxpY2Vuc2UuCgpyZXF1aXJlczogRWxlbWVudAoKcHJv",
				"dmlkZXM6IEVsZW1lbnQuU3R5bGUKCi4uLgoqLwoKKGZ1bmN0aW9uKCl7Cgp2YXIgaHRtbCA9IGRvY3Vt",
				"ZW50Lmh0bWw7CgovLzxsdElFOT4KLy8gQ2hlY2sgZm9yIG9sZElFLCB3aGljaCBkb2VzIG5vdCByZW1v",
				"dmUgc3R5bGVzIHdoZW4gdGhleSdyZSBzZXQgdG8gbnVsbAp2YXIgZWwgPSBkb2N1bWVudC5jcmVhdGVF",
				"bGVtZW50KCdkaXYnKTsKZWwuc3R5bGUuY29sb3IgPSAncmVkJzsKZWwuc3R5bGUuY29sb3IgPSBudWxs",
				"Owp2YXIgZG9lc05vdFJlbW92ZVN0eWxlcyA9IGVsLnN0eWxlLmNvbG9yID09ICdyZWQnOwplbCA9IG51",
				"bGw7Ci8vPC9sdElFOT4KCkVsZW1lbnQuUHJvcGVydGllcy5zdHlsZXMgPSB7c2V0OiBmdW5jdGlvbihz",
				"dHlsZXMpewoJdGhpcy5zZXRTdHlsZXMoc3R5bGVzKTsKfX07Cgp2YXIgaGFzT3BhY2l0eSA9IChodG1s",
				"LnN0eWxlLm9wYWNpdHkgIT0gbnVsbCksCgloYXNGaWx0ZXIgPSAoaHRtbC5zdHlsZS5maWx0ZXIgIT0g",
				"bnVsbCksCglyZUFscGhhID0gL2FscGhhXChvcGFjaXR5PShbXGQuXSspXCkvaTsKCnZhciBzZXRWaXNp",
				"YmlsaXR5ID0gZnVuY3Rpb24oZWxlbWVudCwgb3BhY2l0eSl7CgllbGVtZW50LnN0b3JlKCckb3BhY2l0",
				"eScsIG9wYWNpdHkpOwoJZWxlbWVudC5zdHlsZS52aXNpYmlsaXR5ID0gb3BhY2l0eSA+IDAgfHwgb3Bh",
				"Y2l0eSA9PSBudWxsID8gJ3Zpc2libGUnIDogJ2hpZGRlbic7Cn07Cgp2YXIgc2V0T3BhY2l0eSA9ICho",
				"YXNPcGFjaXR5ID8gZnVuY3Rpb24oZWxlbWVudCwgb3BhY2l0eSl7CgllbGVtZW50LnN0eWxlLm9wYWNp",
				"dHkgPSBvcGFjaXR5Owp9IDogKGhhc0ZpbHRlciA/IGZ1bmN0aW9uKGVsZW1lbnQsIG9wYWNpdHkpewoJ",
				"dmFyIHN0eWxlID0gZWxlbWVudC5zdHlsZTsKCWlmICghZWxlbWVudC5jdXJyZW50U3R5bGUgfHwgIWVs",
				"ZW1lbnQuY3VycmVudFN0eWxlLmhhc0xheW91dCkgc3R5bGUuem9vbSA9IDE7CglpZiAob3BhY2l0eSA9",
				"PSBudWxsIHx8IG9wYWNpdHkgPT0gMSkgb3BhY2l0eSA9ICcnOwoJZWxzZSBvcGFjaXR5ID0gJ2FscGhh",
				"KG9wYWNpdHk9JyArIChvcGFjaXR5ICogMTAwKS5saW1pdCgwLCAxMDApLnJvdW5kKCkgKyAnKSc7Cgl2",
				"YXIgZmlsdGVyID0gc3R5bGUuZmlsdGVyIHx8IGVsZW1lbnQuZ2V0Q29tcHV0ZWRTdHlsZSgnZmlsdGVy",
				"JykgfHwgJyc7CglzdHlsZS5maWx0ZXIgPSByZUFscGhhLnRlc3QoZmlsdGVyKSA/IGZpbHRlci5yZXBs",
				"YWNlKHJlQWxwaGEsIG9wYWNpdHkpIDogZmlsdGVyICsgb3BhY2l0eTsKCWlmICghc3R5bGUuZmlsdGVy",
				"KSBzdHlsZS5yZW1vdmVBdHRyaWJ1dGUoJ2ZpbHRlcicpOwp9IDogc2V0VmlzaWJpbGl0eSkpOwoKdmFy",
				"IGdldE9wYWNpdHkgPSAoaGFzT3BhY2l0eSA/IGZ1bmN0aW9uKGVsZW1lbnQpewoJdmFyIG9wYWNpdHkg",
				"PSBlbGVtZW50LnN0eWxlLm9wYWNpdHkgfHwgZWxlbWVudC5nZXRDb21wdXRlZFN0eWxlKCdvcGFjaXR5",
				"Jyk7CglyZXR1cm4gKG9wYWNpdHkgPT0gJycpID8gMSA6IG9wYWNpdHkudG9GbG9hdCgpOwp9IDogKGhh",
				"c0ZpbHRlciA/IGZ1bmN0aW9uKGVsZW1lbnQpewoJdmFyIGZpbHRlciA9IChlbGVtZW50LnN0eWxlLmZp",
				"bHRlciB8fCBlbGVtZW50LmdldENvbXB1dGVkU3R5bGUoJ2ZpbHRlcicpKSwKCQlvcGFjaXR5OwoJaWYg",
				"KGZpbHRlcikgb3BhY2l0eSA9IGZpbHRlci5tYXRjaChyZUFscGhhKTsKCXJldHVybiAob3BhY2l0eSA9",
				"PSBudWxsIHx8IGZpbHRlciA9PSBudWxsKSA/IDEgOiAob3BhY2l0eVsxXSAvIDEwMCk7Cn0gOiBmdW5j",
				"dGlvbihlbGVtZW50KXsKCXZhciBvcGFjaXR5ID0gZWxlbWVudC5yZXRyaWV2ZSgnJG9wYWNpdHknKTsK",
				"CWlmIChvcGFjaXR5ID09IG51bGwpIG9wYWNpdHkgPSAoZWxlbWVudC5zdHlsZS52aXNpYmlsaXR5ID09",
				"ICdoaWRkZW4nID8gMCA6IDEpOwoJcmV0dXJuIG9wYWNpdHk7Cn0pKTsKCnZhciBmbG9hdE5hbWUgPSAo",
				"aHRtbC5zdHlsZS5jc3NGbG9hdCA9PSBudWxsKSA/ICdzdHlsZUZsb2F0JyA6ICdjc3NGbG9hdCc7CgpF",
				"bGVtZW50LmltcGxlbWVudCh7CgoJZ2V0Q29tcHV0ZWRTdHlsZTogZnVuY3Rpb24ocHJvcGVydHkpewoJ",
				"CWlmICh0aGlzLmN1cnJlbnRTdHlsZSkgcmV0dXJuIHRoaXMuY3VycmVudFN0eWxlW3Byb3BlcnR5LmNh",
				"bWVsQ2FzZSgpXTsKCQl2YXIgZGVmYXVsdFZpZXcgPSBFbGVtZW50LmdldERvY3VtZW50KHRoaXMpLmRl",
				"ZmF1bHRWaWV3LAoJCQljb21wdXRlZCA9IGRlZmF1bHRWaWV3ID8gZGVmYXVsdFZpZXcuZ2V0Q29tcHV0",
				"ZWRTdHlsZSh0aGlzLCBudWxsKSA6IG51bGw7CgkJcmV0dXJuIChjb21wdXRlZCkgPyBjb21wdXRlZC5n",
				"ZXRQcm9wZXJ0eVZhbHVlKChwcm9wZXJ0eSA9PSBmbG9hdE5hbWUpID8gJ2Zsb2F0JyA6IHByb3BlcnR5",
				"Lmh5cGhlbmF0ZSgpKSA6IG51bGw7Cgl9LAoKCXNldFN0eWxlOiBmdW5jdGlvbihwcm9wZXJ0eSwgdmFs",
				"dWUpewoJCWlmIChwcm9wZXJ0eSA9PSAnb3BhY2l0eScpewoJCQlpZiAodmFsdWUgIT0gbnVsbCkgdmFs",
				"dWUgPSBwYXJzZUZsb2F0KHZhbHVlKTsKCQkJc2V0T3BhY2l0eSh0aGlzLCB2YWx1ZSk7CgkJCXJldHVy",
				"biB0aGlzOwoJCX0KCQlwcm9wZXJ0eSA9IChwcm9wZXJ0eSA9PSAnZmxvYXQnID8gZmxvYXROYW1lIDog",
				"cHJvcGVydHkpLmNhbWVsQ2FzZSgpOwoJCWlmICh0eXBlT2YodmFsdWUpICE9ICdzdHJpbmcnKXsKCQkJ",
				"dmFyIG1hcCA9IChFbGVtZW50LlN0eWxlc1twcm9wZXJ0eV0gfHwgJ0AnKS5zcGxpdCgnICcpOwoJCQl2",
				"YWx1ZSA9IEFycmF5LmZyb20odmFsdWUpLm1hcChmdW5jdGlvbih2YWwsIGkpewoJCQkJaWYgKCFtYXBb",
				"aV0pIHJldHVybiAnJzsKCQkJCXJldHVybiAodHlwZU9mKHZhbCkgPT0gJ251bWJlcicpID8gbWFwW2ld",
				"LnJlcGxhY2UoJ0AnLCBNYXRoLnJvdW5kKHZhbCkpIDogdmFsOwoJCQl9KS5qb2luKCcgJyk7CgkJfSBl",
				"bHNlIGlmICh2YWx1ZSA9PSBTdHJpbmcoTnVtYmVyKHZhbHVlKSkpewoJCQl2YWx1ZSA9IE1hdGgucm91",
				"bmQodmFsdWUpOwoJCX0KCQl0aGlzLnN0eWxlW3Byb3BlcnR5XSA9IHZhbHVlOwoJCS8vPGx0SUU5PgoJ",
				"CWlmICgodmFsdWUgPT0gJycgfHwgdmFsdWUgPT0gbnVsbCkgJiYgZG9lc05vdFJlbW92ZVN0eWxlcyAm",
				"JiB0aGlzLnN0eWxlLnJlbW92ZUF0dHJpYnV0ZSl7CgkJCXRoaXMuc3R5bGUucmVtb3ZlQXR0cmlidXRl",
				"KHByb3BlcnR5KTsKCQl9CgkJLy88L2x0SUU5PgoJCXJldHVybiB0aGlzOwoJfSwKCglnZXRTdHlsZTog",
				"ZnVuY3Rpb24ocHJvcGVydHkpewoJCWlmIChwcm9wZXJ0eSA9PSAnb3BhY2l0eScpIHJldHVybiBnZXRP",
				"cGFjaXR5KHRoaXMpOwoJCXByb3BlcnR5ID0gKHByb3BlcnR5ID09ICdmbG9hdCcgPyBmbG9hdE5hbWUg",
				"OiBwcm9wZXJ0eSkuY2FtZWxDYXNlKCk7CgkJdmFyIHJlc3VsdCA9IHRoaXMuc3R5bGVbcHJvcGVydHld",
				"OwoJCWlmICghcmVzdWx0IHx8IHByb3BlcnR5ID09ICd6SW5kZXgnKXsKCQkJcmVzdWx0ID0gW107CgkJ",
				"CWZvciAodmFyIHN0eWxlIGluIEVsZW1lbnQuU2hvcnRTdHlsZXMpewoJCQkJaWYgKHByb3BlcnR5ICE9",
				"IHN0eWxlKSBjb250aW51ZTsKCQkJCWZvciAodmFyIHMgaW4gRWxlbWVudC5TaG9ydFN0eWxlc1tzdHls",
				"ZV0pIHJlc3VsdC5wdXNoKHRoaXMuZ2V0U3R5bGUocykpOwoJCQkJcmV0dXJuIHJlc3VsdC5qb2luKCcg",
				"Jyk7CgkJCX0KCQkJcmVzdWx0ID0gdGhpcy5nZXRDb21wdXRlZFN0eWxlKHByb3BlcnR5KTsKCQl9CgkJ",
				"aWYgKHJlc3VsdCl7CgkJCXJlc3VsdCA9IFN0cmluZyhyZXN1bHQpOwoJCQl2YXIgY29sb3IgPSByZXN1",
				"bHQubWF0Y2goL3JnYmE/XChbXGRccyxdK1wpLyk7CgkJCWlmIChjb2xvcikgcmVzdWx0ID0gcmVzdWx0",
				"LnJlcGxhY2UoY29sb3JbMF0sIGNvbG9yWzBdLnJnYlRvSGV4KCkpOwoJCX0KCQlpZiAoQnJvd3Nlci5v",
				"cGVyYSB8fCBCcm93c2VyLmllKXsKCQkJaWYgKCgvXihoZWlnaHR8d2lkdGgpJC8pLnRlc3QocHJvcGVy",
				"dHkpICYmICEoL3B4JC8udGVzdChyZXN1bHQpKSl7CgkJCQl2YXIgdmFsdWVzID0gKHByb3BlcnR5ID09",
				"ICd3aWR0aCcpID8gWydsZWZ0JywgJ3JpZ2h0J10gOiBbJ3RvcCcsICdib3R0b20nXSwgc2l6ZSA9IDA7",
				"CgkJCQl2YWx1ZXMuZWFjaChmdW5jdGlvbih2YWx1ZSl7CgkJCQkJc2l6ZSArPSB0aGlzLmdldFN0eWxl",
				"KCdib3JkZXItJyArIHZhbHVlICsgJy13aWR0aCcpLnRvSW50KCkgKyB0aGlzLmdldFN0eWxlKCdwYWRk",
				"aW5nLScgKyB2YWx1ZSkudG9JbnQoKTsKCQkJCX0sIHRoaXMpOwoJCQkJcmV0dXJuIHRoaXNbJ29mZnNl",
				"dCcgKyBwcm9wZXJ0eS5jYXBpdGFsaXplKCldIC0gc2l6ZSArICdweCc7CgkJCX0KCQkJaWYgKEJyb3dz",
				"ZXIuaWUgJiYgKC9eYm9yZGVyKC4rKVdpZHRofG1hcmdpbnxwYWRkaW5nLykudGVzdChwcm9wZXJ0eSkg",
				"JiYgaXNOYU4ocGFyc2VGbG9hdChyZXN1bHQpKSl7CgkJCQlyZXR1cm4gJzBweCc7CgkJCX0KCQl9CgkJ",
				"cmV0dXJuIHJlc3VsdDsKCX0sCgoJc2V0U3R5bGVzOiBmdW5jdGlvbihzdHlsZXMpewoJCWZvciAodmFy",
				"IHN0eWxlIGluIHN0eWxlcykgdGhpcy5zZXRTdHlsZShzdHlsZSwgc3R5bGVzW3N0eWxlXSk7CgkJcmV0",
				"dXJuIHRoaXM7Cgl9LAoKCWdldFN0eWxlczogZnVuY3Rpb24oKXsKCQl2YXIgcmVzdWx0ID0ge307CgkJ",
				"QXJyYXkuZmxhdHRlbihhcmd1bWVudHMpLmVhY2goZnVuY3Rpb24oa2V5KXsKCQkJcmVzdWx0W2tleV0g",
				"PSB0aGlzLmdldFN0eWxlKGtleSk7CgkJfSwgdGhpcyk7CgkJcmV0dXJuIHJlc3VsdDsKCX0KCn0pOwoK",
				"RWxlbWVudC5TdHlsZXMgPSB7CglsZWZ0OiAnQHB4JywgdG9wOiAnQHB4JywgYm90dG9tOiAnQHB4Jywg",
				"cmlnaHQ6ICdAcHgnLAoJd2lkdGg6ICdAcHgnLCBoZWlnaHQ6ICdAcHgnLCBtYXhXaWR0aDogJ0BweCcs",
				"IG1heEhlaWdodDogJ0BweCcsIG1pbldpZHRoOiAnQHB4JywgbWluSGVpZ2h0OiAnQHB4JywKCWJhY2tn",
				"cm91bmRDb2xvcjogJ3JnYihALCBALCBAKScsIGJhY2tncm91bmRQb3NpdGlvbjogJ0BweCBAcHgnLCBj",
				"b2xvcjogJ3JnYihALCBALCBAKScsCglmb250U2l6ZTogJ0BweCcsIGxldHRlclNwYWNpbmc6ICdAcHgn",
				"LCBsaW5lSGVpZ2h0OiAnQHB4JywgY2xpcDogJ3JlY3QoQHB4IEBweCBAcHggQHB4KScsCgltYXJnaW46",
				"ICdAcHggQHB4IEBweCBAcHgnLCBwYWRkaW5nOiAnQHB4IEBweCBAcHggQHB4JywgYm9yZGVyOiAnQHB4",
				"IEAgcmdiKEAsIEAsIEApIEBweCBAIHJnYihALCBALCBAKSBAcHggQCByZ2IoQCwgQCwgQCknLAoJYm9y",
				"ZGVyV2lkdGg6ICdAcHggQHB4IEBweCBAcHgnLCBib3JkZXJTdHlsZTogJ0AgQCBAIEAnLCBib3JkZXJD",
				"b2xvcjogJ3JnYihALCBALCBAKSByZ2IoQCwgQCwgQCkgcmdiKEAsIEAsIEApIHJnYihALCBALCBAKScs",
				"Cgl6SW5kZXg6ICdAJywgJ3pvb20nOiAnQCcsIGZvbnRXZWlnaHQ6ICdAJywgdGV4dEluZGVudDogJ0Bw",
				"eCcsIG9wYWNpdHk6ICdAJwp9OwoKLy88MS4zY29tcGF0PgoKRWxlbWVudC5pbXBsZW1lbnQoewoKCXNl",
				"dE9wYWNpdHk6IGZ1bmN0aW9uKHZhbHVlKXsKCQlzZXRPcGFjaXR5KHRoaXMsIHZhbHVlKTsKCQlyZXR1",
				"cm4gdGhpczsKCX0sCgoJZ2V0T3BhY2l0eTogZnVuY3Rpb24oKXsKCQlyZXR1cm4gZ2V0T3BhY2l0eSh0",
				"aGlzKTsKCX0KCn0pOwoKRWxlbWVudC5Qcm9wZXJ0aWVzLm9wYWNpdHkgPSB7CgoJc2V0OiBmdW5jdGlv",
				"bihvcGFjaXR5KXsKCQlzZXRPcGFjaXR5KHRoaXMsIG9wYWNpdHkpOwoJCXNldFZpc2liaWxpdHkodGhp",
				"cywgb3BhY2l0eSk7Cgl9LAoKCWdldDogZnVuY3Rpb24oKXsKCQlyZXR1cm4gZ2V0T3BhY2l0eSh0aGlz",
				"KTsKCX0KCn07CgovLzwvMS4zY29tcGF0PgoKLy88MS4yY29tcGF0PgoKRWxlbWVudC5TdHlsZXMgPSBu",
				"ZXcgSGFzaChFbGVtZW50LlN0eWxlcyk7CgovLzwvMS4yY29tcGF0PgoKRWxlbWVudC5TaG9ydFN0eWxl",
				"cyA9IHttYXJnaW46IHt9LCBwYWRkaW5nOiB7fSwgYm9yZGVyOiB7fSwgYm9yZGVyV2lkdGg6IHt9LCBi",
				"b3JkZXJTdHlsZToge30sIGJvcmRlckNvbG9yOiB7fX07CgpbJ1RvcCcsICdSaWdodCcsICdCb3R0b20n",
				"LCAnTGVmdCddLmVhY2goZnVuY3Rpb24oZGlyZWN0aW9uKXsKCXZhciBTaG9ydCA9IEVsZW1lbnQuU2hv",
				"cnRTdHlsZXM7Cgl2YXIgQWxsID0gRWxlbWVudC5TdHlsZXM7CglbJ21hcmdpbicsICdwYWRkaW5nJ10u",
				"ZWFjaChmdW5jdGlvbihzdHlsZSl7CgkJdmFyIHNkID0gc3R5bGUgKyBkaXJlY3Rpb247CgkJU2hvcnRb",
				"c3R5bGVdW3NkXSA9IEFsbFtzZF0gPSAnQHB4JzsKCX0pOwoJdmFyIGJkID0gJ2JvcmRlcicgKyBkaXJl",
				"Y3Rpb247CglTaG9ydC5ib3JkZXJbYmRdID0gQWxsW2JkXSA9ICdAcHggQCByZ2IoQCwgQCwgQCknOwoJ",
				"dmFyIGJkdyA9IGJkICsgJ1dpZHRoJywgYmRzID0gYmQgKyAnU3R5bGUnLCBiZGMgPSBiZCArICdDb2xv",
				"cic7CglTaG9ydFtiZF0gPSB7fTsKCVNob3J0LmJvcmRlcldpZHRoW2Jkd10gPSBTaG9ydFtiZF1bYmR3",
				"XSA9IEFsbFtiZHddID0gJ0BweCc7CglTaG9ydC5ib3JkZXJTdHlsZVtiZHNdID0gU2hvcnRbYmRdW2Jk",
				"c10gPSBBbGxbYmRzXSA9ICdAJzsKCVNob3J0LmJvcmRlckNvbG9yW2JkY10gPSBTaG9ydFtiZF1bYmRj",
				"XSA9IEFsbFtiZGNdID0gJ3JnYihALCBALCBAKSc7Cn0pOwoKfSkoKTsKCgovKgotLS0KCm5hbWU6IEVs",
				"ZW1lbnQuRXZlbnQKCmRlc2NyaXB0aW9uOiBDb250YWlucyBFbGVtZW50IG1ldGhvZHMgZm9yIGRlYWxp",
				"bmcgd2l0aCBldmVudHMuIFRoaXMgZmlsZSBhbHNvIGluY2x1ZGVzIG1vdXNlZW50ZXIgYW5kIG1vdXNl",
				"bGVhdmUgY3VzdG9tIEVsZW1lbnQgRXZlbnRzLCBpZiBuZWNlc3NhcnkuCgpsaWNlbnNlOiBNSVQtc3R5",
				"bGUgbGljZW5zZS4KCnJlcXVpcmVzOiBbRWxlbWVudCwgRXZlbnRdCgpwcm92aWRlczogRWxlbWVudC5F",
				"dmVudAoKLi4uCiovCgooZnVuY3Rpb24oKXsKCkVsZW1lbnQuUHJvcGVydGllcy5ldmVudHMgPSB7c2V0",
				"OiBmdW5jdGlvbihldmVudHMpewoJdGhpcy5hZGRFdmVudHMoZXZlbnRzKTsKfX07CgpbRWxlbWVudCwg",
				"V2luZG93LCBEb2N1bWVudF0uaW52b2tlKCdpbXBsZW1lbnQnLCB7CgoJYWRkRXZlbnQ6IGZ1bmN0aW9u",
				"KHR5cGUsIGZuKXsKCQl2YXIgZXZlbnRzID0gdGhpcy5yZXRyaWV2ZSgnZXZlbnRzJywge30pOwoJCWlm",
				"ICghZXZlbnRzW3R5cGVdKSBldmVudHNbdHlwZV0gPSB7a2V5czogW10sIHZhbHVlczogW119OwoJCWlm",
				"IChldmVudHNbdHlwZV0ua2V5cy5jb250YWlucyhmbikpIHJldHVybiB0aGlzOwoJCWV2ZW50c1t0eXBl",
				"XS5rZXlzLnB1c2goZm4pOwoJCXZhciByZWFsVHlwZSA9IHR5cGUsCgkJCWN1c3RvbSA9IEVsZW1lbnQu",
				"RXZlbnRzW3R5cGVdLAoJCQljb25kaXRpb24gPSBmbiwKCQkJc2VsZiA9IHRoaXM7CgkJaWYgKGN1c3Rv",
				"bSl7CgkJCWlmIChjdXN0b20ub25BZGQpIGN1c3RvbS5vbkFkZC5jYWxsKHRoaXMsIGZuLCB0eXBlKTsK",
				"CQkJaWYgKGN1c3RvbS5jb25kaXRpb24pewoJCQkJY29uZGl0aW9uID0gZnVuY3Rpb24oZXZlbnQpewoJ",
				"CQkJCWlmIChjdXN0b20uY29uZGl0aW9uLmNhbGwodGhpcywgZXZlbnQsIHR5cGUpKSByZXR1cm4gZm4u",
				"Y2FsbCh0aGlzLCBldmVudCk7CgkJCQkJcmV0dXJuIHRydWU7CgkJCQl9OwoJCQl9CgkJCWlmIChjdXN0",
				"b20uYmFzZSkgcmVhbFR5cGUgPSBGdW5jdGlvbi5mcm9tKGN1c3RvbS5iYXNlKS5jYWxsKHRoaXMsIHR5",
				"cGUpOwoJCX0KCQl2YXIgZGVmbiA9IGZ1bmN0aW9uKCl7CgkJCXJldHVybiBmbi5jYWxsKHNlbGYpOwoJ",
				"CX07CgkJdmFyIG5hdGl2ZUV2ZW50ID0gRWxlbWVudC5OYXRpdmVFdmVudHNbcmVhbFR5cGVdOwoJCWlm",
				"IChuYXRpdmVFdmVudCl7CgkJCWlmIChuYXRpdmVFdmVudCA9PSAyKXsKCQkJCWRlZm4gPSBmdW5jdGlv",
				"bihldmVudCl7CgkJCQkJZXZlbnQgPSBuZXcgRE9NRXZlbnQoZXZlbnQsIHNlbGYuZ2V0V2luZG93KCkp",
				"OwoJCQkJCWlmIChjb25kaXRpb24uY2FsbChzZWxmLCBldmVudCkgPT09IGZhbHNlKSBldmVudC5zdG9w",
				"KCk7CgkJCQl9OwoJCQl9CgkJCXRoaXMuYWRkTGlzdGVuZXIocmVhbFR5cGUsIGRlZm4sIGFyZ3VtZW50",
				"c1syXSk7CgkJfQoJCWV2ZW50c1t0eXBlXS52YWx1ZXMucHVzaChkZWZuKTsKCQlyZXR1cm4gdGhpczsK",
				"CX0sCgoJcmVtb3ZlRXZlbnQ6IGZ1bmN0aW9uKHR5cGUsIGZuKXsKCQl2YXIgZXZlbnRzID0gdGhpcy5y",
				"ZXRyaWV2ZSgnZXZlbnRzJyk7CgkJaWYgKCFldmVudHMgfHwgIWV2ZW50c1t0eXBlXSkgcmV0dXJuIHRo",
				"aXM7CgkJdmFyIGxpc3QgPSBldmVudHNbdHlwZV07CgkJdmFyIGluZGV4ID0gbGlzdC5rZXlzLmluZGV4",
				"T2YoZm4pOwoJCWlmIChpbmRleCA9PSAtMSkgcmV0dXJuIHRoaXM7CgkJdmFyIHZhbHVlID0gbGlzdC52",
				"YWx1ZXNbaW5kZXhdOwoJCWRlbGV0ZSBsaXN0LmtleXNbaW5kZXhdOwoJCWRlbGV0ZSBsaXN0LnZhbHVl",
				"c1tpbmRleF07CgkJdmFyIGN1c3RvbSA9IEVsZW1lbnQuRXZlbnRzW3R5cGVdOwoJCWlmIChjdXN0b20p",
				"ewoJCQlpZiAoY3VzdG9tLm9uUmVtb3ZlKSBjdXN0b20ub25SZW1vdmUuY2FsbCh0aGlzLCBmbiwgdHlw",
				"ZSk7CgkJCWlmIChjdXN0b20uYmFzZSkgdHlwZSA9IEZ1bmN0aW9uLmZyb20oY3VzdG9tLmJhc2UpLmNh",
				"bGwodGhpcywgdHlwZSk7CgkJfQoJCXJldHVybiAoRWxlbWVudC5OYXRpdmVFdmVudHNbdHlwZV0pID8g",
				"dGhpcy5yZW1vdmVMaXN0ZW5lcih0eXBlLCB2YWx1ZSwgYXJndW1lbnRzWzJdKSA6IHRoaXM7Cgl9LAoK",
				"CWFkZEV2ZW50czogZnVuY3Rpb24oZXZlbnRzKXsKCQlmb3IgKHZhciBldmVudCBpbiBldmVudHMpIHRo",
				"aXMuYWRkRXZlbnQoZXZlbnQsIGV2ZW50c1tldmVudF0pOwoJCXJldHVybiB0aGlzOwoJfSwKCglyZW1v",
				"dmVFdmVudHM6IGZ1bmN0aW9uKGV2ZW50cyl7CgkJdmFyIHR5cGU7CgkJaWYgKHR5cGVPZihldmVudHMp",
				"ID09ICdvYmplY3QnKXsKCQkJZm9yICh0eXBlIGluIGV2ZW50cykgdGhpcy5yZW1vdmVFdmVudCh0eXBl",
				"LCBldmVudHNbdHlwZV0pOwoJCQlyZXR1cm4gdGhpczsKCQl9CgkJdmFyIGF0dGFjaGVkID0gdGhpcy5y",
				"ZXRyaWV2ZSgnZXZlbnRzJyk7CgkJaWYgKCFhdHRhY2hlZCkgcmV0dXJuIHRoaXM7CgkJaWYgKCFldmVu",
				"dHMpewoJCQlmb3IgKHR5cGUgaW4gYXR0YWNoZWQpIHRoaXMucmVtb3ZlRXZlbnRzKHR5cGUpOwoJCQl0",
				"aGlzLmVsaW1pbmF0ZSgnZXZlbnRzJyk7CgkJfSBlbHNlIGlmIChhdHRhY2hlZFtldmVudHNdKXsKCQkJ",
				"YXR0YWNoZWRbZXZlbnRzXS5rZXlzLmVhY2goZnVuY3Rpb24oZm4pewoJCQkJdGhpcy5yZW1vdmVFdmVu",
				"dChldmVudHMsIGZuKTsKCQkJfSwgdGhpcyk7CgkJCWRlbGV0ZSBhdHRhY2hlZFtldmVudHNdOwoJCX0K",
				"CQlyZXR1cm4gdGhpczsKCX0sCgoJZmlyZUV2ZW50OiBmdW5jdGlvbih0eXBlLCBhcmdzLCBkZWxheSl7",
				"CgkJdmFyIGV2ZW50cyA9IHRoaXMucmV0cmlldmUoJ2V2ZW50cycpOwoJCWlmICghZXZlbnRzIHx8ICFl",
				"dmVudHNbdHlwZV0pIHJldHVybiB0aGlzOwoJCWFyZ3MgPSBBcnJheS5mcm9tKGFyZ3MpOwoKCQlldmVu",
				"dHNbdHlwZV0ua2V5cy5lYWNoKGZ1bmN0aW9uKGZuKXsKCQkJaWYgKGRlbGF5KSBmbi5kZWxheShkZWxh",
				"eSwgdGhpcywgYXJncyk7CgkJCWVsc2UgZm4uYXBwbHkodGhpcywgYXJncyk7CgkJfSwgdGhpcyk7CgkJ",
				"cmV0dXJuIHRoaXM7Cgl9LAoKCWNsb25lRXZlbnRzOiBmdW5jdGlvbihmcm9tLCB0eXBlKXsKCQlmcm9t",
				"ID0gZG9jdW1lbnQuaWQoZnJvbSk7CgkJdmFyIGV2ZW50cyA9IGZyb20ucmV0cmlldmUoJ2V2ZW50cycp",
				"OwoJCWlmICghZXZlbnRzKSByZXR1cm4gdGhpczsKCQlpZiAoIXR5cGUpewoJCQlmb3IgKHZhciBldmVu",
				"dFR5cGUgaW4gZXZlbnRzKSB0aGlzLmNsb25lRXZlbnRzKGZyb20sIGV2ZW50VHlwZSk7CgkJfSBlbHNl",
				"IGlmIChldmVudHNbdHlwZV0pewoJCQlldmVudHNbdHlwZV0ua2V5cy5lYWNoKGZ1bmN0aW9uKGZuKXsK",
				"CQkJCXRoaXMuYWRkRXZlbnQodHlwZSwgZm4pOwoJCQl9LCB0aGlzKTsKCQl9CgkJcmV0dXJuIHRoaXM7",
				"Cgl9Cgp9KTsKCkVsZW1lbnQuTmF0aXZlRXZlbnRzID0gewoJY2xpY2s6IDIsIGRibGNsaWNrOiAyLCBt",
				"b3VzZXVwOiAyLCBtb3VzZWRvd246IDIsIGNvbnRleHRtZW51OiAyLCAvL21vdXNlIGJ1dHRvbnMKCW1v",
				"dXNld2hlZWw6IDIsIERPTU1vdXNlU2Nyb2xsOiAyLCAvL21vdXNlIHdoZWVsCgltb3VzZW92ZXI6IDIs",
				"IG1vdXNlb3V0OiAyLCBtb3VzZW1vdmU6IDIsIHNlbGVjdHN0YXJ0OiAyLCBzZWxlY3RlbmQ6IDIsIC8v",
				"bW91c2UgbW92ZW1lbnQKCWtleWRvd246IDIsIGtleXByZXNzOiAyLCBrZXl1cDogMiwgLy9rZXlib2Fy",
				"ZAoJb3JpZW50YXRpb25jaGFuZ2U6IDIsIC8vIG1vYmlsZQoJdG91Y2hzdGFydDogMiwgdG91Y2htb3Zl",
				"OiAyLCB0b3VjaGVuZDogMiwgdG91Y2hjYW5jZWw6IDIsIC8vIHRvdWNoCglnZXN0dXJlc3RhcnQ6IDIs",
				"IGdlc3R1cmVjaGFuZ2U6IDIsIGdlc3R1cmVlbmQ6IDIsIC8vIGdlc3R1cmUKCWZvY3VzOiAyLCBibHVy",
				"OiAyLCBjaGFuZ2U6IDIsIHJlc2V0OiAyLCBzZWxlY3Q6IDIsIHN1Ym1pdDogMiwgcGFzdGU6IDIsIGlu",
				"cHV0OiAyLCAvL2Zvcm0gZWxlbWVudHMKCWxvYWQ6IDIsIHVubG9hZDogMSwgYmVmb3JldW5sb2FkOiAy",
				"LCByZXNpemU6IDEsIG1vdmU6IDEsIERPTUNvbnRlbnRMb2FkZWQ6IDEsIHJlYWR5c3RhdGVjaGFuZ2U6",
				"IDEsIC8vd2luZG93CgllcnJvcjogMSwgYWJvcnQ6IDEsIHNjcm9sbDogMSAvL21pc2MKfTsKCkVsZW1l",
				"bnQuRXZlbnRzID0ge21vdXNld2hlZWw6IHsKCWJhc2U6IChCcm93c2VyLmZpcmVmb3gpID8gJ0RPTU1v",
				"dXNlU2Nyb2xsJyA6ICdtb3VzZXdoZWVsJwp9fTsKCmlmICgnb25tb3VzZWVudGVyJyBpbiBkb2N1bWVu",
				"dC5kb2N1bWVudEVsZW1lbnQpewoJRWxlbWVudC5OYXRpdmVFdmVudHMubW91c2VlbnRlciA9IEVsZW1l",
				"bnQuTmF0aXZlRXZlbnRzLm1vdXNlbGVhdmUgPSAyOwp9IGVsc2UgewoJdmFyIGNoZWNrID0gZnVuY3Rp",
				"b24oZXZlbnQpewoJCXZhciByZWxhdGVkID0gZXZlbnQucmVsYXRlZFRhcmdldDsKCQlpZiAocmVsYXRl",
				"ZCA9PSBudWxsKSByZXR1cm4gdHJ1ZTsKCQlpZiAoIXJlbGF0ZWQpIHJldHVybiBmYWxzZTsKCQlyZXR1",
				"cm4gKHJlbGF0ZWQgIT0gdGhpcyAmJiByZWxhdGVkLnByZWZpeCAhPSAneHVsJyAmJiB0eXBlT2YodGhp",
				"cykgIT0gJ2RvY3VtZW50JyAmJiAhdGhpcy5jb250YWlucyhyZWxhdGVkKSk7Cgl9OwoKCUVsZW1lbnQu",
				"RXZlbnRzLm1vdXNlZW50ZXIgPSB7CgkJYmFzZTogJ21vdXNlb3ZlcicsCgkJY29uZGl0aW9uOiBjaGVj",
				"awoJfTsKCglFbGVtZW50LkV2ZW50cy5tb3VzZWxlYXZlID0gewoJCWJhc2U6ICdtb3VzZW91dCcsCgkJ",
				"Y29uZGl0aW9uOiBjaGVjawoJfTsKfQoKLyo8bHRJRTk+Ki8KaWYgKCF3aW5kb3cuYWRkRXZlbnRMaXN0",
				"ZW5lcil7CglFbGVtZW50Lk5hdGl2ZUV2ZW50cy5wcm9wZXJ0eWNoYW5nZSA9IDI7CglFbGVtZW50LkV2",
				"ZW50cy5jaGFuZ2UgPSB7CgkJYmFzZTogZnVuY3Rpb24oKXsKCQkJdmFyIHR5cGUgPSB0aGlzLnR5cGU7",
				"CgkJCXJldHVybiAodGhpcy5nZXQoJ3RhZycpID09ICdpbnB1dCcgJiYgKHR5cGUgPT0gJ3JhZGlvJyB8",
				"fCB0eXBlID09ICdjaGVja2JveCcpKSA/ICdwcm9wZXJ0eWNoYW5nZScgOiAnY2hhbmdlJwoJCX0sCgkJ",
				"Y29uZGl0aW9uOiBmdW5jdGlvbihldmVudCl7CgkJCXJldHVybiB0aGlzLnR5cGUgIT0gJ3JhZGlvJyB8",
				"fCAoZXZlbnQuZXZlbnQucHJvcGVydHlOYW1lID09ICdjaGVja2VkJyAmJiB0aGlzLmNoZWNrZWQpOwoJ",
				"CX0KCX0KfQovKjwvbHRJRTk+Ki8KCi8vPDEuMmNvbXBhdD4KCkVsZW1lbnQuRXZlbnRzID0gbmV3IEhh",
				"c2goRWxlbWVudC5FdmVudHMpOwoKLy88LzEuMmNvbXBhdD4KCn0pKCk7CgoKLyoKLS0tCgpuYW1lOiBF",
				"bGVtZW50LkRlbGVnYXRpb24KCmRlc2NyaXB0aW9uOiBFeHRlbmRzIHRoZSBFbGVtZW50IG5hdGl2ZSBv",
				"YmplY3QgdG8gaW5jbHVkZSB0aGUgZGVsZWdhdGUgbWV0aG9kIGZvciBtb3JlIGVmZmljaWVudCBldmVu",
				"dCBtYW5hZ2VtZW50LgoKbGljZW5zZTogTUlULXN0eWxlIGxpY2Vuc2UuCgpyZXF1aXJlczogW0VsZW1l",
				"bnQuRXZlbnRdCgpwcm92aWRlczogW0VsZW1lbnQuRGVsZWdhdGlvbl0KCi4uLgoqLwoKKGZ1bmN0aW9u",
				"KCl7Cgp2YXIgZXZlbnRMaXN0ZW5lclN1cHBvcnQgPSAhIXdpbmRvdy5hZGRFdmVudExpc3RlbmVyOwoK",
				"RWxlbWVudC5OYXRpdmVFdmVudHMuZm9jdXNpbiA9IEVsZW1lbnQuTmF0aXZlRXZlbnRzLmZvY3Vzb3V0",
				"ID0gMjsKCnZhciBidWJibGVVcCA9IGZ1bmN0aW9uKHNlbGYsIG1hdGNoLCBmbiwgZXZlbnQsIHRhcmdl",
				"dCl7Cgl3aGlsZSAodGFyZ2V0ICYmIHRhcmdldCAhPSBzZWxmKXsKCQlpZiAobWF0Y2godGFyZ2V0LCBl",
				"dmVudCkpIHJldHVybiBmbi5jYWxsKHRhcmdldCwgZXZlbnQsIHRhcmdldCk7CgkJdGFyZ2V0ID0gZG9j",
				"dW1lbnQuaWQodGFyZ2V0LnBhcmVudE5vZGUpOwoJfQp9OwoKdmFyIG1hcCA9IHsKCW1vdXNlZW50ZXI6",
				"IHsKCQliYXNlOiAnbW91c2VvdmVyJwoJfSwKCW1vdXNlbGVhdmU6IHsKCQliYXNlOiAnbW91c2VvdXQn",
				"Cgl9LAoJZm9jdXM6IHsKCQliYXNlOiAnZm9jdXMnICsgKGV2ZW50TGlzdGVuZXJTdXBwb3J0ID8gJycg",
				"OiAnaW4nKSwKCQljYXB0dXJlOiB0cnVlCgl9LAoJYmx1cjogewoJCWJhc2U6IGV2ZW50TGlzdGVuZXJT",
				"dXBwb3J0ID8gJ2JsdXInIDogJ2ZvY3Vzb3V0JywKCQljYXB0dXJlOiB0cnVlCgl9Cn07CgovKjxsdElF",
				"OT4qLwp2YXIgX2tleSA9ICckZGVsZWdhdGlvbjonOwp2YXIgZm9ybU9ic2VydmVyID0gZnVuY3Rpb24o",
				"dHlwZSl7CgoJcmV0dXJuIHsKCgkJYmFzZTogJ2ZvY3VzaW4nLAoKCQlyZW1vdmU6IGZ1bmN0aW9uKHNl",
				"bGYsIHVpZCl7CgkJCXZhciBsaXN0ID0gc2VsZi5yZXRyaWV2ZShfa2V5ICsgdHlwZSArICdsaXN0ZW5l",
				"cnMnLCB7fSlbdWlkXTsKCQkJaWYgKGxpc3QgJiYgbGlzdC5mb3JtcykgZm9yICh2YXIgaSA9IGxpc3Qu",
				"Zm9ybXMubGVuZ3RoOyBpLS07KXsKCQkJCWxpc3QuZm9ybXNbaV0ucmVtb3ZlRXZlbnQodHlwZSwgbGlz",
				"dC5mbnNbaV0pOwoJCQl9CgkJfSwKCgkJbGlzdGVuOiBmdW5jdGlvbihzZWxmLCBtYXRjaCwgZm4sIGV2",
				"ZW50LCB0YXJnZXQsIHVpZCl7CgkJCXZhciBmb3JtID0gKHRhcmdldC5nZXQoJ3RhZycpID09ICdmb3Jt",
				"JykgPyB0YXJnZXQgOiBldmVudC50YXJnZXQuZ2V0UGFyZW50KCdmb3JtJyk7CgkJCWlmICghZm9ybSkg",
				"cmV0dXJuOwoKCQkJdmFyIGxpc3RlbmVycyA9IHNlbGYucmV0cmlldmUoX2tleSArIHR5cGUgKyAnbGlz",
				"dGVuZXJzJywge30pLAoJCQkJbGlzdGVuZXIgPSBsaXN0ZW5lcnNbdWlkXSB8fCB7Zm9ybXM6IFtdLCBm",
				"bnM6IFtdfSwKCQkJCWZvcm1zID0gbGlzdGVuZXIuZm9ybXMsIGZucyA9IGxpc3RlbmVyLmZuczsKCgkJ",
				"CWlmIChmb3Jtcy5pbmRleE9mKGZvcm0pICE9IC0xKSByZXR1cm47CgkJCWZvcm1zLnB1c2goZm9ybSk7",
				"CgoJCQl2YXIgX2ZuID0gZnVuY3Rpb24oZXZlbnQpewoJCQkJYnViYmxlVXAoc2VsZiwgbWF0Y2gsIGZu",
				"LCBldmVudCwgdGFyZ2V0KTsKCQkJfTsKCQkJZm9ybS5hZGRFdmVudCh0eXBlLCBfZm4pOwoJCQlmbnMu",
				"cHVzaChfZm4pOwoKCQkJbGlzdGVuZXJzW3VpZF0gPSBsaXN0ZW5lcjsKCQkJc2VsZi5zdG9yZShfa2V5",
				"ICsgdHlwZSArICdsaXN0ZW5lcnMnLCBsaXN0ZW5lcnMpOwoJCX0KCX07Cn07Cgp2YXIgaW5wdXRPYnNl",
				"cnZlciA9IGZ1bmN0aW9uKHR5cGUpewoJcmV0dXJuIHsKCQliYXNlOiAnZm9jdXNpbicsCgkJbGlzdGVu",
				"OiBmdW5jdGlvbihzZWxmLCBtYXRjaCwgZm4sIGV2ZW50LCB0YXJnZXQpewoJCQl2YXIgZXZlbnRzID0g",
				"e2JsdXI6IGZ1bmN0aW9uKCl7CgkJCQl0aGlzLnJlbW92ZUV2ZW50cyhldmVudHMpOwoJCQl9fTsKCQkJ",
				"ZXZlbnRzW3R5cGVdID0gZnVuY3Rpb24oZXZlbnQpewoJCQkJYnViYmxlVXAoc2VsZiwgbWF0Y2gsIGZu",
				"LCBldmVudCwgdGFyZ2V0KTsKCQkJfTsKCQkJZXZlbnQudGFyZ2V0LmFkZEV2ZW50cyhldmVudHMpOwoJ",
				"CX0KCX07Cn07CgppZiAoIWV2ZW50TGlzdGVuZXJTdXBwb3J0KSBPYmplY3QuYXBwZW5kKG1hcCwgewoJ",
				"c3VibWl0OiBmb3JtT2JzZXJ2ZXIoJ3N1Ym1pdCcpLAoJcmVzZXQ6IGZvcm1PYnNlcnZlcigncmVzZXQn",
				"KSwKCWNoYW5nZTogaW5wdXRPYnNlcnZlcignY2hhbmdlJyksCglzZWxlY3Q6IGlucHV0T2JzZXJ2ZXIo",
				"J3NlbGVjdCcpCn0pOwovKjwvbHRJRTk+Ki8KCnZhciBwcm90byA9IEVsZW1lbnQucHJvdG90eXBlLAoJ",
				"YWRkRXZlbnQgPSBwcm90by5hZGRFdmVudCwKCXJlbW92ZUV2ZW50ID0gcHJvdG8ucmVtb3ZlRXZlbnQ7",
				"Cgp2YXIgcmVsYXkgPSBmdW5jdGlvbihvbGQsIG1ldGhvZCl7CglyZXR1cm4gZnVuY3Rpb24odHlwZSwg",
				"Zm4sIHVzZUNhcHR1cmUpewoJCWlmICh0eXBlLmluZGV4T2YoJzpyZWxheScpID09IC0xKSByZXR1cm4g",
				"b2xkLmNhbGwodGhpcywgdHlwZSwgZm4sIHVzZUNhcHR1cmUpOwoJCXZhciBwYXJzZWQgPSBTbGljay5w",
				"YXJzZSh0eXBlKS5leHByZXNzaW9uc1swXVswXTsKCQlpZiAocGFyc2VkLnBzZXVkb3NbMF0ua2V5ICE9",
				"ICdyZWxheScpIHJldHVybiBvbGQuY2FsbCh0aGlzLCB0eXBlLCBmbiwgdXNlQ2FwdHVyZSk7CgkJdmFy",
				"IG5ld1R5cGUgPSBwYXJzZWQudGFnOwoJCXBhcnNlZC5wc2V1ZG9zLnNsaWNlKDEpLmVhY2goZnVuY3Rp",
				"b24ocHNldWRvKXsKCQkJbmV3VHlwZSArPSAnOicgKyBwc2V1ZG8ua2V5ICsgKHBzZXVkby52YWx1ZSA/",
				"ICcoJyArIHBzZXVkby52YWx1ZSArICcpJyA6ICcnKTsKCQl9KTsKCQlvbGQuY2FsbCh0aGlzLCB0eXBl",
				"LCBmbik7CgkJcmV0dXJuIG1ldGhvZC5jYWxsKHRoaXMsIG5ld1R5cGUsIHBhcnNlZC5wc2V1ZG9zWzBd",
				"LnZhbHVlLCBmbik7Cgl9Owp9OwoKdmFyIGRlbGVnYXRpb24gPSB7CgoJYWRkRXZlbnQ6IGZ1bmN0aW9u",
				"KHR5cGUsIG1hdGNoLCBmbil7CgkJdmFyIHN0b3JhZ2UgPSB0aGlzLnJldHJpZXZlKCckZGVsZWdhdGVz",
				"Jywge30pLCBzdG9yZWQgPSBzdG9yYWdlW3R5cGVdOwoJCWlmIChzdG9yZWQpIGZvciAodmFyIF91aWQg",
				"aW4gc3RvcmVkKXsKCQkJaWYgKHN0b3JlZFtfdWlkXS5mbiA9PSBmbiAmJiBzdG9yZWRbX3VpZF0ubWF0",
				"Y2ggPT0gbWF0Y2gpIHJldHVybiB0aGlzOwoJCX0KCgkJdmFyIF90eXBlID0gdHlwZSwgX21hdGNoID0g",
				"bWF0Y2gsIF9mbiA9IGZuLCBfbWFwID0gbWFwW3R5cGVdIHx8IHt9OwoJCXR5cGUgPSBfbWFwLmJhc2Ug",
				"fHwgX3R5cGU7CgoJCW1hdGNoID0gZnVuY3Rpb24odGFyZ2V0KXsKCQkJcmV0dXJuIFNsaWNrLm1hdGNo",
				"KHRhcmdldCwgX21hdGNoKTsKCQl9OwoKCQl2YXIgZWxlbWVudEV2ZW50ID0gRWxlbWVudC5FdmVudHNb",
				"X3R5cGVdOwoJCWlmIChlbGVtZW50RXZlbnQgJiYgZWxlbWVudEV2ZW50LmNvbmRpdGlvbil7CgkJCXZh",
				"ciBfX21hdGNoID0gbWF0Y2gsIGNvbmRpdGlvbiA9IGVsZW1lbnRFdmVudC5jb25kaXRpb247CgkJCW1h",
				"dGNoID0gZnVuY3Rpb24odGFyZ2V0LCBldmVudCl7CgkJCQlyZXR1cm4gX19tYXRjaCh0YXJnZXQsIGV2",
				"ZW50KSAmJiBjb25kaXRpb24uY2FsbCh0YXJnZXQsIGV2ZW50LCB0eXBlKTsKCQkJfTsKCQl9CgoJCXZh",
				"ciBzZWxmID0gdGhpcywgdWlkID0gU3RyaW5nLnVuaXF1ZUlEKCk7CgkJdmFyIGRlbGVnYXRvciA9IF9t",
				"YXAubGlzdGVuID8gZnVuY3Rpb24oZXZlbnQsIHRhcmdldCl7CgkJCWlmICghdGFyZ2V0ICYmIGV2ZW50",
				"ICYmIGV2ZW50LnRhcmdldCkgdGFyZ2V0ID0gZXZlbnQudGFyZ2V0OwoJCQlpZiAodGFyZ2V0KSBfbWFw",
				"Lmxpc3RlbihzZWxmLCBtYXRjaCwgZm4sIGV2ZW50LCB0YXJnZXQsIHVpZCk7CgkJfSA6IGZ1bmN0aW9u",
				"KGV2ZW50LCB0YXJnZXQpewoJCQlpZiAoIXRhcmdldCAmJiBldmVudCAmJiBldmVudC50YXJnZXQpIHRh",
				"cmdldCA9IGV2ZW50LnRhcmdldDsKCQkJaWYgKHRhcmdldCkgYnViYmxlVXAoc2VsZiwgbWF0Y2gsIGZu",
				"LCBldmVudCwgdGFyZ2V0KTsKCQl9OwoKCQlpZiAoIXN0b3JlZCkgc3RvcmVkID0ge307CgkJc3RvcmVk",
				"W3VpZF0gPSB7CgkJCW1hdGNoOiBfbWF0Y2gsCgkJCWZuOiBfZm4sCgkJCWRlbGVnYXRvcjogZGVsZWdh",
				"dG9yCgkJfTsKCQlzdG9yYWdlW190eXBlXSA9IHN0b3JlZDsKCQlyZXR1cm4gYWRkRXZlbnQuY2FsbCh0",
				"aGlzLCB0eXBlLCBkZWxlZ2F0b3IsIF9tYXAuY2FwdHVyZSk7Cgl9LAoKCXJlbW92ZUV2ZW50OiBmdW5j",
				"dGlvbih0eXBlLCBtYXRjaCwgZm4sIF91aWQpewoJCXZhciBzdG9yYWdlID0gdGhpcy5yZXRyaWV2ZSgn",
				"JGRlbGVnYXRlcycsIHt9KSwgc3RvcmVkID0gc3RvcmFnZVt0eXBlXTsKCQlpZiAoIXN0b3JlZCkgcmV0",
				"dXJuIHRoaXM7CgoJCWlmIChfdWlkKXsKCQkJdmFyIF90eXBlID0gdHlwZSwgZGVsZWdhdG9yID0gc3Rv",
				"cmVkW191aWRdLmRlbGVnYXRvciwgX21hcCA9IG1hcFt0eXBlXSB8fCB7fTsKCQkJdHlwZSA9IF9tYXAu",
				"YmFzZSB8fCBfdHlwZTsKCQkJaWYgKF9tYXAucmVtb3ZlKSBfbWFwLnJlbW92ZSh0aGlzLCBfdWlkKTsK",
				"CQkJZGVsZXRlIHN0b3JlZFtfdWlkXTsKCQkJc3RvcmFnZVtfdHlwZV0gPSBzdG9yZWQ7CgkJCXJldHVy",
				"biByZW1vdmVFdmVudC5jYWxsKHRoaXMsIHR5cGUsIGRlbGVnYXRvcik7CgkJfQoKCQl2YXIgX191aWQs",
				"IHM7CgkJaWYgKGZuKSBmb3IgKF9fdWlkIGluIHN0b3JlZCl7CgkJCXMgPSBzdG9yZWRbX191aWRdOwoJ",
				"CQlpZiAocy5tYXRjaCA9PSBtYXRjaCAmJiBzLmZuID09IGZuKSByZXR1cm4gZGVsZWdhdGlvbi5yZW1v",
				"dmVFdmVudC5jYWxsKHRoaXMsIHR5cGUsIG1hdGNoLCBmbiwgX191aWQpOwoJCX0gZWxzZSBmb3IgKF9f",
				"dWlkIGluIHN0b3JlZCl7CgkJCXMgPSBzdG9yZWRbX191aWRdOwoJCQlpZiAocy5tYXRjaCA9PSBtYXRj",
				"aCkgZGVsZWdhdGlvbi5yZW1vdmVFdmVudC5jYWxsKHRoaXMsIHR5cGUsIG1hdGNoLCBzLmZuLCBfX3Vp",
				"ZCk7CgkJfQoJCXJldHVybiB0aGlzOwoJfQoKfTsKCltFbGVtZW50LCBXaW5kb3csIERvY3VtZW50XS5p",
				"bnZva2UoJ2ltcGxlbWVudCcsIHsKCWFkZEV2ZW50OiByZWxheShhZGRFdmVudCwgZGVsZWdhdGlvbi5h",
				"ZGRFdmVudCksCglyZW1vdmVFdmVudDogcmVsYXkocmVtb3ZlRXZlbnQsIGRlbGVnYXRpb24ucmVtb3Zl",
				"RXZlbnQpCn0pOwoKfSkoKTsKCgovKgotLS0KCm5hbWU6IEVsZW1lbnQuRGltZW5zaW9ucwoKZGVzY3Jp",
				"cHRpb246IENvbnRhaW5zIG1ldGhvZHMgdG8gd29yayB3aXRoIHNpemUsIHNjcm9sbCwgb3IgcG9zaXRp",
				"b25pbmcgb2YgRWxlbWVudHMgYW5kIHRoZSB3aW5kb3cgb2JqZWN0LgoKbGljZW5zZTogTUlULXN0eWxl",
				"IGxpY2Vuc2UuCgpjcmVkaXRzOgogIC0gRWxlbWVudCBwb3NpdGlvbmluZyBiYXNlZCBvbiB0aGUgW3Fv",
				"b3hkb29dKGh0dHA6Ly9xb294ZG9vLm9yZy8pIGNvZGUgYW5kIHNtYXJ0IGJyb3dzZXIgZml4ZXMsIFtM",
				"R1BMIExpY2Vuc2VdKGh0dHA6Ly93d3cuZ251Lm9yZy9saWNlbnNlcy9sZ3BsLmh0bWwpLgogIC0gVmll",
				"d3BvcnQgZGltZW5zaW9ucyBiYXNlZCBvbiBbWVVJXShodHRwOi8vZGV2ZWxvcGVyLnlhaG9vLmNvbS95",
				"dWkvKSBjb2RlLCBbQlNEIExpY2Vuc2VdKGh0dHA6Ly9kZXZlbG9wZXIueWFob28uY29tL3l1aS9saWNl",
				"bnNlLmh0bWwpLgoKcmVxdWlyZXM6IFtFbGVtZW50LCBFbGVtZW50LlN0eWxlXQoKcHJvdmlkZXM6IFtF",
				"bGVtZW50LkRpbWVuc2lvbnNdCgouLi4KKi8KCihmdW5jdGlvbigpewoKdmFyIGVsZW1lbnQgPSBkb2N1",
				"bWVudC5jcmVhdGVFbGVtZW50KCdkaXYnKSwKCWNoaWxkID0gZG9jdW1lbnQuY3JlYXRlRWxlbWVudCgn",
				"ZGl2Jyk7CmVsZW1lbnQuc3R5bGUuaGVpZ2h0ID0gJzAnOwplbGVtZW50LmFwcGVuZENoaWxkKGNoaWxk",
				"KTsKdmFyIGJyb2tlbk9mZnNldFBhcmVudCA9IChjaGlsZC5vZmZzZXRQYXJlbnQgPT09IGVsZW1lbnQp",
				"OwplbGVtZW50ID0gY2hpbGQgPSBudWxsOwoKdmFyIGlzT2Zmc2V0ID0gZnVuY3Rpb24oZWwpewoJcmV0",
				"dXJuIHN0eWxlU3RyaW5nKGVsLCAncG9zaXRpb24nKSAhPSAnc3RhdGljJyB8fCBpc0JvZHkoZWwpOwp9",
				"OwoKdmFyIGlzT2Zmc2V0U3RhdGljID0gZnVuY3Rpb24oZWwpewoJcmV0dXJuIGlzT2Zmc2V0KGVsKSB8",
				"fCAoL14oPzp0YWJsZXx0ZHx0aCkkL2kpLnRlc3QoZWwudGFnTmFtZSk7Cn07CgpFbGVtZW50LmltcGxl",
				"bWVudCh7CgoJc2Nyb2xsVG86IGZ1bmN0aW9uKHgsIHkpewoJCWlmIChpc0JvZHkodGhpcykpewoJCQl0",
				"aGlzLmdldFdpbmRvdygpLnNjcm9sbFRvKHgsIHkpOwoJCX0gZWxzZSB7CgkJCXRoaXMuc2Nyb2xsTGVm",
				"dCA9IHg7CgkJCXRoaXMuc2Nyb2xsVG9wID0geTsKCQl9CgkJcmV0dXJuIHRoaXM7Cgl9LAoKCWdldFNp",
				"emU6IGZ1bmN0aW9uKCl7CgkJaWYgKGlzQm9keSh0aGlzKSkgcmV0dXJuIHRoaXMuZ2V0V2luZG93KCku",
				"Z2V0U2l6ZSgpOwoJCXJldHVybiB7eDogdGhpcy5vZmZzZXRXaWR0aCwgeTogdGhpcy5vZmZzZXRIZWln",
				"aHR9OwoJfSwKCglnZXRTY3JvbGxTaXplOiBmdW5jdGlvbigpewoJCWlmIChpc0JvZHkodGhpcykpIHJl",
				"dHVybiB0aGlzLmdldFdpbmRvdygpLmdldFNjcm9sbFNpemUoKTsKCQlyZXR1cm4ge3g6IHRoaXMuc2Ny",
				"b2xsV2lkdGgsIHk6IHRoaXMuc2Nyb2xsSGVpZ2h0fTsKCX0sCgoJZ2V0U2Nyb2xsOiBmdW5jdGlvbigp",
				"ewoJCWlmIChpc0JvZHkodGhpcykpIHJldHVybiB0aGlzLmdldFdpbmRvdygpLmdldFNjcm9sbCgpOwoJ",
				"CXJldHVybiB7eDogdGhpcy5zY3JvbGxMZWZ0LCB5OiB0aGlzLnNjcm9sbFRvcH07Cgl9LAoKCWdldFNj",
				"cm9sbHM6IGZ1bmN0aW9uKCl7CgkJdmFyIGVsZW1lbnQgPSB0aGlzLnBhcmVudE5vZGUsIHBvc2l0aW9u",
				"ID0ge3g6IDAsIHk6IDB9OwoJCXdoaWxlIChlbGVtZW50ICYmICFpc0JvZHkoZWxlbWVudCkpewoJCQlw",
				"b3NpdGlvbi54ICs9IGVsZW1lbnQuc2Nyb2xsTGVmdDsKCQkJcG9zaXRpb24ueSArPSBlbGVtZW50LnNj",
				"cm9sbFRvcDsKCQkJZWxlbWVudCA9IGVsZW1lbnQucGFyZW50Tm9kZTsKCQl9CgkJcmV0dXJuIHBvc2l0",
				"aW9uOwoJfSwKCglnZXRPZmZzZXRQYXJlbnQ6IGJyb2tlbk9mZnNldFBhcmVudCA/IGZ1bmN0aW9uKCl7",
				"CgkJdmFyIGVsZW1lbnQgPSB0aGlzOwoJCWlmIChpc0JvZHkoZWxlbWVudCkgfHwgc3R5bGVTdHJpbmco",
				"ZWxlbWVudCwgJ3Bvc2l0aW9uJykgPT0gJ2ZpeGVkJykgcmV0dXJuIG51bGw7CgoJCXZhciBpc09mZnNl",
				"dENoZWNrID0gKHN0eWxlU3RyaW5nKGVsZW1lbnQsICdwb3NpdGlvbicpID09ICdzdGF0aWMnKSA/IGlz",
				"T2Zmc2V0U3RhdGljIDogaXNPZmZzZXQ7CgkJd2hpbGUgKChlbGVtZW50ID0gZWxlbWVudC5wYXJlbnRO",
				"b2RlKSl7CgkJCWlmIChpc09mZnNldENoZWNrKGVsZW1lbnQpKSByZXR1cm4gZWxlbWVudDsKCQl9CgkJ",
				"cmV0dXJuIG51bGw7Cgl9IDogZnVuY3Rpb24oKXsKCQl2YXIgZWxlbWVudCA9IHRoaXM7CgkJaWYgKGlz",
				"Qm9keShlbGVtZW50KSB8fCBzdHlsZVN0cmluZyhlbGVtZW50LCAncG9zaXRpb24nKSA9PSAnZml4ZWQn",
				"KSByZXR1cm4gbnVsbDsKCgkJdHJ5IHsKCQkJcmV0dXJuIGVsZW1lbnQub2Zmc2V0UGFyZW50OwoJCX0g",
				"Y2F0Y2goZSkge30KCQlyZXR1cm4gbnVsbDsKCX0sCgoJZ2V0T2Zmc2V0czogZnVuY3Rpb24oKXsKCQlp",
				"ZiAodGhpcy5nZXRCb3VuZGluZ0NsaWVudFJlY3QgJiYgIUJyb3dzZXIuUGxhdGZvcm0uaW9zKXsKCQkJ",
				"dmFyIGJvdW5kID0gdGhpcy5nZXRCb3VuZGluZ0NsaWVudFJlY3QoKSwKCQkJCWh0bWwgPSBkb2N1bWVu",
				"dC5pZCh0aGlzLmdldERvY3VtZW50KCkuZG9jdW1lbnRFbGVtZW50KSwKCQkJCWh0bWxTY3JvbGwgPSBo",
				"dG1sLmdldFNjcm9sbCgpLAoJCQkJZWxlbVNjcm9sbHMgPSB0aGlzLmdldFNjcm9sbHMoKSwKCQkJCWlz",
				"Rml4ZWQgPSAoc3R5bGVTdHJpbmcodGhpcywgJ3Bvc2l0aW9uJykgPT0gJ2ZpeGVkJyk7CgoJCQlyZXR1",
				"cm4gewoJCQkJeDogYm91bmQubGVmdC50b0ludCgpICsgZWxlbVNjcm9sbHMueCArICgoaXNGaXhlZCkg",
				"PyAwIDogaHRtbFNjcm9sbC54KSAtIGh0bWwuY2xpZW50TGVmdCwKCQkJCXk6IGJvdW5kLnRvcC50b0lu",
				"dCgpICArIGVsZW1TY3JvbGxzLnkgKyAoKGlzRml4ZWQpID8gMCA6IGh0bWxTY3JvbGwueSkgLSBodG1s",
				"LmNsaWVudFRvcAoJCQl9OwoJCX0KCgkJdmFyIGVsZW1lbnQgPSB0aGlzLCBwb3NpdGlvbiA9IHt4OiAw",
				"LCB5OiAwfTsKCQlpZiAoaXNCb2R5KHRoaXMpKSByZXR1cm4gcG9zaXRpb247CgoJCXdoaWxlIChlbGVt",
				"ZW50ICYmICFpc0JvZHkoZWxlbWVudCkpewoJCQlwb3NpdGlvbi54ICs9IGVsZW1lbnQub2Zmc2V0TGVm",
				"dDsKCQkJcG9zaXRpb24ueSArPSBlbGVtZW50Lm9mZnNldFRvcDsKCgkJCWlmIChCcm93c2VyLmZpcmVm",
				"b3gpewoJCQkJaWYgKCFib3JkZXJCb3goZWxlbWVudCkpewoJCQkJCXBvc2l0aW9uLnggKz0gbGVmdEJv",
				"cmRlcihlbGVtZW50KTsKCQkJCQlwb3NpdGlvbi55ICs9IHRvcEJvcmRlcihlbGVtZW50KTsKCQkJCX0K",
				"CQkJCXZhciBwYXJlbnQgPSBlbGVtZW50LnBhcmVudE5vZGU7CgkJCQlpZiAocGFyZW50ICYmIHN0eWxl",
				"U3RyaW5nKHBhcmVudCwgJ292ZXJmbG93JykgIT0gJ3Zpc2libGUnKXsKCQkJCQlwb3NpdGlvbi54ICs9",
				"IGxlZnRCb3JkZXIocGFyZW50KTsKCQkJCQlwb3NpdGlvbi55ICs9IHRvcEJvcmRlcihwYXJlbnQpOwoJ",
				"CQkJfQoJCQl9IGVsc2UgaWYgKGVsZW1lbnQgIT0gdGhpcyAmJiBCcm93c2VyLnNhZmFyaSl7CgkJCQlw",
				"b3NpdGlvbi54ICs9IGxlZnRCb3JkZXIoZWxlbWVudCk7CgkJCQlwb3NpdGlvbi55ICs9IHRvcEJvcmRl",
				"cihlbGVtZW50KTsKCQkJfQoKCQkJZWxlbWVudCA9IGVsZW1lbnQub2Zmc2V0UGFyZW50OwoJCX0KCQlp",
				"ZiAoQnJvd3Nlci5maXJlZm94ICYmICFib3JkZXJCb3godGhpcykpewoJCQlwb3NpdGlvbi54IC09IGxl",
				"ZnRCb3JkZXIodGhpcyk7CgkJCXBvc2l0aW9uLnkgLT0gdG9wQm9yZGVyKHRoaXMpOwoJCX0KCQlyZXR1",
				"cm4gcG9zaXRpb247Cgl9LAoKCWdldFBvc2l0aW9uOiBmdW5jdGlvbihyZWxhdGl2ZSl7CgkJdmFyIG9m",
				"ZnNldCA9IHRoaXMuZ2V0T2Zmc2V0cygpLAoJCQlzY3JvbGwgPSB0aGlzLmdldFNjcm9sbHMoKTsKCQl2",
				"YXIgcG9zaXRpb24gPSB7CgkJCXg6IG9mZnNldC54IC0gc2Nyb2xsLngsCgkJCXk6IG9mZnNldC55IC0g",
				"c2Nyb2xsLnkKCQl9OwoKCQlpZiAocmVsYXRpdmUgJiYgKHJlbGF0aXZlID0gZG9jdW1lbnQuaWQocmVs",
				"YXRpdmUpKSl7CgkJCXZhciByZWxhdGl2ZVBvc2l0aW9uID0gcmVsYXRpdmUuZ2V0UG9zaXRpb24oKTsK",
				"CQkJcmV0dXJuIHt4OiBwb3NpdGlvbi54IC0gcmVsYXRpdmVQb3NpdGlvbi54IC0gbGVmdEJvcmRlcihy",
				"ZWxhdGl2ZSksIHk6IHBvc2l0aW9uLnkgLSByZWxhdGl2ZVBvc2l0aW9uLnkgLSB0b3BCb3JkZXIocmVs",
				"YXRpdmUpfTsKCQl9CgkJcmV0dXJuIHBvc2l0aW9uOwoJfSwKCglnZXRDb29yZGluYXRlczogZnVuY3Rp",
				"b24oZWxlbWVudCl7CgkJaWYgKGlzQm9keSh0aGlzKSkgcmV0dXJuIHRoaXMuZ2V0V2luZG93KCkuZ2V0",
				"Q29vcmRpbmF0ZXMoKTsKCQl2YXIgcG9zaXRpb24gPSB0aGlzLmdldFBvc2l0aW9uKGVsZW1lbnQpLAoJ",
				"CQlzaXplID0gdGhpcy5nZXRTaXplKCk7CgkJdmFyIG9iaiA9IHsKCQkJbGVmdDogcG9zaXRpb24ueCwK",
				"CQkJdG9wOiBwb3NpdGlvbi55LAoJCQl3aWR0aDogc2l6ZS54LAoJCQloZWlnaHQ6IHNpemUueQoJCX07",
				"CgkJb2JqLnJpZ2h0ID0gb2JqLmxlZnQgKyBvYmoud2lkdGg7CgkJb2JqLmJvdHRvbSA9IG9iai50b3Ag",
				"KyBvYmouaGVpZ2h0OwoJCXJldHVybiBvYmo7Cgl9LAoKCWNvbXB1dGVQb3NpdGlvbjogZnVuY3Rpb24o",
				"b2JqKXsKCQlyZXR1cm4gewoJCQlsZWZ0OiBvYmoueCAtIHN0eWxlTnVtYmVyKHRoaXMsICdtYXJnaW4t",
				"bGVmdCcpLAoJCQl0b3A6IG9iai55IC0gc3R5bGVOdW1iZXIodGhpcywgJ21hcmdpbi10b3AnKQoJCX07",
				"Cgl9LAoKCXNldFBvc2l0aW9uOiBmdW5jdGlvbihvYmopewoJCXJldHVybiB0aGlzLnNldFN0eWxlcyh0",
				"aGlzLmNvbXB1dGVQb3NpdGlvbihvYmopKTsKCX0KCn0pOwoKCltEb2N1bWVudCwgV2luZG93XS5pbnZv",
				"a2UoJ2ltcGxlbWVudCcsIHsKCglnZXRTaXplOiBmdW5jdGlvbigpewoJCXZhciBkb2MgPSBnZXRDb21w",
				"YXRFbGVtZW50KHRoaXMpOwoJCXJldHVybiB7eDogZG9jLmNsaWVudFdpZHRoLCB5OiBkb2MuY2xpZW50",
				"SGVpZ2h0fTsKCX0sCgoJZ2V0U2Nyb2xsOiBmdW5jdGlvbigpewoJCXZhciB3aW4gPSB0aGlzLmdldFdp",
				"bmRvdygpLCBkb2MgPSBnZXRDb21wYXRFbGVtZW50KHRoaXMpOwoJCXJldHVybiB7eDogd2luLnBhZ2VY",
				"T2Zmc2V0IHx8IGRvYy5zY3JvbGxMZWZ0LCB5OiB3aW4ucGFnZVlPZmZzZXQgfHwgZG9jLnNjcm9sbFRv",
				"cH07Cgl9LAoKCWdldFNjcm9sbFNpemU6IGZ1bmN0aW9uKCl7CgkJdmFyIGRvYyA9IGdldENvbXBhdEVs",
				"ZW1lbnQodGhpcyksCgkJCW1pbiA9IHRoaXMuZ2V0U2l6ZSgpLAoJCQlib2R5ID0gdGhpcy5nZXREb2N1",
				"bWVudCgpLmJvZHk7CgoJCXJldHVybiB7eDogTWF0aC5tYXgoZG9jLnNjcm9sbFdpZHRoLCBib2R5LnNj",
				"cm9sbFdpZHRoLCBtaW4ueCksIHk6IE1hdGgubWF4KGRvYy5zY3JvbGxIZWlnaHQsIGJvZHkuc2Nyb2xs",
				"SGVpZ2h0LCBtaW4ueSl9OwoJfSwKCglnZXRQb3NpdGlvbjogZnVuY3Rpb24oKXsKCQlyZXR1cm4ge3g6",
				"IDAsIHk6IDB9OwoJfSwKCglnZXRDb29yZGluYXRlczogZnVuY3Rpb24oKXsKCQl2YXIgc2l6ZSA9IHRo",
				"aXMuZ2V0U2l6ZSgpOwoJCXJldHVybiB7dG9wOiAwLCBsZWZ0OiAwLCBib3R0b206IHNpemUueSwgcmln",
				"aHQ6IHNpemUueCwgaGVpZ2h0OiBzaXplLnksIHdpZHRoOiBzaXplLnh9OwoJfQoKfSk7CgovLyBwcml2",
				"YXRlIG1ldGhvZHMKCnZhciBzdHlsZVN0cmluZyA9IEVsZW1lbnQuZ2V0Q29tcHV0ZWRTdHlsZTsKCmZ1",
				"bmN0aW9uIHN0eWxlTnVtYmVyKGVsZW1lbnQsIHN0eWxlKXsKCXJldHVybiBzdHlsZVN0cmluZyhlbGVt",
				"ZW50LCBzdHlsZSkudG9JbnQoKSB8fCAwOwp9CgpmdW5jdGlvbiBib3JkZXJCb3goZWxlbWVudCl7Cgly",
				"ZXR1cm4gc3R5bGVTdHJpbmcoZWxlbWVudCwgJy1tb3otYm94LXNpemluZycpID09ICdib3JkZXItYm94",
				"JzsKfQoKZnVuY3Rpb24gdG9wQm9yZGVyKGVsZW1lbnQpewoJcmV0dXJuIHN0eWxlTnVtYmVyKGVsZW1l",
				"bnQsICdib3JkZXItdG9wLXdpZHRoJyk7Cn0KCmZ1bmN0aW9uIGxlZnRCb3JkZXIoZWxlbWVudCl7Cgly",
				"ZXR1cm4gc3R5bGVOdW1iZXIoZWxlbWVudCwgJ2JvcmRlci1sZWZ0LXdpZHRoJyk7Cn0KCmZ1bmN0aW9u",
				"IGlzQm9keShlbGVtZW50KXsKCXJldHVybiAoL14oPzpib2R5fGh0bWwpJC9pKS50ZXN0KGVsZW1lbnQu",
				"dGFnTmFtZSk7Cn0KCmZ1bmN0aW9uIGdldENvbXBhdEVsZW1lbnQoZWxlbWVudCl7Cgl2YXIgZG9jID0g",
				"ZWxlbWVudC5nZXREb2N1bWVudCgpOwoJcmV0dXJuICghZG9jLmNvbXBhdE1vZGUgfHwgZG9jLmNvbXBh",
				"dE1vZGUgPT0gJ0NTUzFDb21wYXQnKSA/IGRvYy5odG1sIDogZG9jLmJvZHk7Cn0KCn0pKCk7CgovL2Fs",
				"aWFzZXMKRWxlbWVudC5hbGlhcyh7cG9zaXRpb246ICdzZXRQb3NpdGlvbid9KTsgLy9jb21wYXRhYmls",
				"aXR5CgpbV2luZG93LCBEb2N1bWVudCwgRWxlbWVudF0uaW52b2tlKCdpbXBsZW1lbnQnLCB7CgoJZ2V0",
				"SGVpZ2h0OiBmdW5jdGlvbigpewoJCXJldHVybiB0aGlzLmdldFNpemUoKS55OwoJfSwKCglnZXRXaWR0",
				"aDogZnVuY3Rpb24oKXsKCQlyZXR1cm4gdGhpcy5nZXRTaXplKCkueDsKCX0sCgoJZ2V0U2Nyb2xsVG9w",
				"OiBmdW5jdGlvbigpewoJCXJldHVybiB0aGlzLmdldFNjcm9sbCgpLnk7Cgl9LAoKCWdldFNjcm9sbExl",
				"ZnQ6IGZ1bmN0aW9uKCl7CgkJcmV0dXJuIHRoaXMuZ2V0U2Nyb2xsKCkueDsKCX0sCgoJZ2V0U2Nyb2xs",
				"SGVpZ2h0OiBmdW5jdGlvbigpewoJCXJldHVybiB0aGlzLmdldFNjcm9sbFNpemUoKS55OwoJfSwKCgln",
				"ZXRTY3JvbGxXaWR0aDogZnVuY3Rpb24oKXsKCQlyZXR1cm4gdGhpcy5nZXRTY3JvbGxTaXplKCkueDsK",
				"CX0sCgoJZ2V0VG9wOiBmdW5jdGlvbigpewoJCXJldHVybiB0aGlzLmdldFBvc2l0aW9uKCkueTsKCX0s",
				"CgoJZ2V0TGVmdDogZnVuY3Rpb24oKXsKCQlyZXR1cm4gdGhpcy5nZXRQb3NpdGlvbigpLng7Cgl9Cgp9",
				"KTsKCgovKgotLS0KCm5hbWU6IEZ4CgpkZXNjcmlwdGlvbjogQ29udGFpbnMgdGhlIGJhc2ljIGFuaW1h",
				"dGlvbiBsb2dpYyB0byBiZSBleHRlbmRlZCBieSBhbGwgb3RoZXIgRnggQ2xhc3Nlcy4KCmxpY2Vuc2U6",
				"IE1JVC1zdHlsZSBsaWNlbnNlLgoKcmVxdWlyZXM6IFtDaGFpbiwgRXZlbnRzLCBPcHRpb25zXQoKcHJv",
				"dmlkZXM6IEZ4CgouLi4KKi8KCihmdW5jdGlvbigpewoKdmFyIEZ4ID0gdGhpcy5GeCA9IG5ldyBDbGFz",
				"cyh7CgoJSW1wbGVtZW50czogW0NoYWluLCBFdmVudHMsIE9wdGlvbnNdLAoKCW9wdGlvbnM6IHsKCQkv",
				"KgoJCW9uU3RhcnQ6IG5pbCwKCQlvbkNhbmNlbDogbmlsLAoJCW9uQ29tcGxldGU6IG5pbCwKCQkqLwoJ",
				"CWZwczogNjAsCgkJdW5pdDogZmFsc2UsCgkJZHVyYXRpb246IDUwMCwKCQlmcmFtZXM6IG51bGwsCgkJ",
				"ZnJhbWVTa2lwOiB0cnVlLAoJCWxpbms6ICdpZ25vcmUnCgl9LAoKCWluaXRpYWxpemU6IGZ1bmN0aW9u",
				"KG9wdGlvbnMpewoJCXRoaXMuc3ViamVjdCA9IHRoaXMuc3ViamVjdCB8fCB0aGlzOwoJCXRoaXMuc2V0",
				"T3B0aW9ucyhvcHRpb25zKTsKCX0sCgoJZ2V0VHJhbnNpdGlvbjogZnVuY3Rpb24oKXsKCQlyZXR1cm4g",
				"ZnVuY3Rpb24ocCl7CgkJCXJldHVybiAtKE1hdGguY29zKE1hdGguUEkgKiBwKSAtIDEpIC8gMjsKCQl9",
				"OwoJfSwKCglzdGVwOiBmdW5jdGlvbihub3cpewoJCWlmICh0aGlzLm9wdGlvbnMuZnJhbWVTa2lwKXsK",
				"CQkJdmFyIGRpZmYgPSAodGhpcy50aW1lICE9IG51bGwpID8gKG5vdyAtIHRoaXMudGltZSkgOiAwLCBm",
				"cmFtZXMgPSBkaWZmIC8gdGhpcy5mcmFtZUludGVydmFsOwoJCQl0aGlzLnRpbWUgPSBub3c7CgkJCXRo",
				"aXMuZnJhbWUgKz0gZnJhbWVzOwoJCX0gZWxzZSB7CgkJCXRoaXMuZnJhbWUrKzsKCQl9CgoJCWlmICh0",
				"aGlzLmZyYW1lIDwgdGhpcy5mcmFtZXMpewoJCQl2YXIgZGVsdGEgPSB0aGlzLnRyYW5zaXRpb24odGhp",
				"cy5mcmFtZSAvIHRoaXMuZnJhbWVzKTsKCQkJdGhpcy5zZXQodGhpcy5jb21wdXRlKHRoaXMuZnJvbSwg",
				"dGhpcy50bywgZGVsdGEpKTsKCQl9IGVsc2UgewoJCQl0aGlzLmZyYW1lID0gdGhpcy5mcmFtZXM7CgkJ",
				"CXRoaXMuc2V0KHRoaXMuY29tcHV0ZSh0aGlzLmZyb20sIHRoaXMudG8sIDEpKTsKCQkJdGhpcy5zdG9w",
				"KCk7CgkJfQoJfSwKCglzZXQ6IGZ1bmN0aW9uKG5vdyl7CgkJcmV0dXJuIG5vdzsKCX0sCgoJY29tcHV0",
				"ZTogZnVuY3Rpb24oZnJvbSwgdG8sIGRlbHRhKXsKCQlyZXR1cm4gRnguY29tcHV0ZShmcm9tLCB0bywg",
				"ZGVsdGEpOwoJfSwKCgljaGVjazogZnVuY3Rpb24oKXsKCQlpZiAoIXRoaXMuaXNSdW5uaW5nKCkpIHJl",
				"dHVybiB0cnVlOwoJCXN3aXRjaCAodGhpcy5vcHRpb25zLmxpbmspewoJCQljYXNlICdjYW5jZWwnOiB0",
				"aGlzLmNhbmNlbCgpOyByZXR1cm4gdHJ1ZTsKCQkJY2FzZSAnY2hhaW4nOiB0aGlzLmNoYWluKHRoaXMu",
				"Y2FsbGVyLnBhc3MoYXJndW1lbnRzLCB0aGlzKSk7IHJldHVybiBmYWxzZTsKCQl9CgkJcmV0dXJuIGZh",
				"bHNlOwoJfSwKCglzdGFydDogZnVuY3Rpb24oZnJvbSwgdG8pewoJCWlmICghdGhpcy5jaGVjayhmcm9t",
				"LCB0bykpIHJldHVybiB0aGlzOwoJCXRoaXMuZnJvbSA9IGZyb207CgkJdGhpcy50byA9IHRvOwoJCXRo",
				"aXMuZnJhbWUgPSAodGhpcy5vcHRpb25zLmZyYW1lU2tpcCkgPyAwIDogLTE7CgkJdGhpcy50aW1lID0g",
				"bnVsbDsKCQl0aGlzLnRyYW5zaXRpb24gPSB0aGlzLmdldFRyYW5zaXRpb24oKTsKCQl2YXIgZnJhbWVz",
				"ID0gdGhpcy5vcHRpb25zLmZyYW1lcywgZnBzID0gdGhpcy5vcHRpb25zLmZwcywgZHVyYXRpb24gPSB0",
				"aGlzLm9wdGlvbnMuZHVyYXRpb247CgkJdGhpcy5kdXJhdGlvbiA9IEZ4LkR1cmF0aW9uc1tkdXJhdGlv",
				"bl0gfHwgZHVyYXRpb24udG9JbnQoKTsKCQl0aGlzLmZyYW1lSW50ZXJ2YWwgPSAxMDAwIC8gZnBzOwoJ",
				"CXRoaXMuZnJhbWVzID0gZnJhbWVzIHx8IE1hdGgucm91bmQodGhpcy5kdXJhdGlvbiAvIHRoaXMuZnJh",
				"bWVJbnRlcnZhbCk7CgkJdGhpcy5maXJlRXZlbnQoJ3N0YXJ0JywgdGhpcy5zdWJqZWN0KTsKCQlwdXNo",
				"SW5zdGFuY2UuY2FsbCh0aGlzLCBmcHMpOwoJCXJldHVybiB0aGlzOwoJfSwKCglzdG9wOiBmdW5jdGlv",
				"bigpewoJCWlmICh0aGlzLmlzUnVubmluZygpKXsKCQkJdGhpcy50aW1lID0gbnVsbDsKCQkJcHVsbElu",
				"c3RhbmNlLmNhbGwodGhpcywgdGhpcy5vcHRpb25zLmZwcyk7CgkJCWlmICh0aGlzLmZyYW1lcyA9PSB0",
				"aGlzLmZyYW1lKXsKCQkJCXRoaXMuZmlyZUV2ZW50KCdjb21wbGV0ZScsIHRoaXMuc3ViamVjdCk7CgkJ",
				"CQlpZiAoIXRoaXMuY2FsbENoYWluKCkpIHRoaXMuZmlyZUV2ZW50KCdjaGFpbkNvbXBsZXRlJywgdGhp",
				"cy5zdWJqZWN0KTsKCQkJfSBlbHNlIHsKCQkJCXRoaXMuZmlyZUV2ZW50KCdzdG9wJywgdGhpcy5zdWJq",
				"ZWN0KTsKCQkJfQoJCX0KCQlyZXR1cm4gdGhpczsKCX0sCgoJY2FuY2VsOiBmdW5jdGlvbigpewoJCWlm",
				"ICh0aGlzLmlzUnVubmluZygpKXsKCQkJdGhpcy50aW1lID0gbnVsbDsKCQkJcHVsbEluc3RhbmNlLmNh",
				"bGwodGhpcywgdGhpcy5vcHRpb25zLmZwcyk7CgkJCXRoaXMuZnJhbWUgPSB0aGlzLmZyYW1lczsKCQkJ",
				"dGhpcy5maXJlRXZlbnQoJ2NhbmNlbCcsIHRoaXMuc3ViamVjdCkuY2xlYXJDaGFpbigpOwoJCX0KCQly",
				"ZXR1cm4gdGhpczsKCX0sCgoJcGF1c2U6IGZ1bmN0aW9uKCl7CgkJaWYgKHRoaXMuaXNSdW5uaW5nKCkp",
				"ewoJCQl0aGlzLnRpbWUgPSBudWxsOwoJCQlwdWxsSW5zdGFuY2UuY2FsbCh0aGlzLCB0aGlzLm9wdGlv",
				"bnMuZnBzKTsKCQl9CgkJcmV0dXJuIHRoaXM7Cgl9LAoKCXJlc3VtZTogZnVuY3Rpb24oKXsKCQlpZiAo",
				"KHRoaXMuZnJhbWUgPCB0aGlzLmZyYW1lcykgJiYgIXRoaXMuaXNSdW5uaW5nKCkpIHB1c2hJbnN0YW5j",
				"ZS5jYWxsKHRoaXMsIHRoaXMub3B0aW9ucy5mcHMpOwoJCXJldHVybiB0aGlzOwoJfSwKCglpc1J1bm5p",
				"bmc6IGZ1bmN0aW9uKCl7CgkJdmFyIGxpc3QgPSBpbnN0YW5jZXNbdGhpcy5vcHRpb25zLmZwc107CgkJ",
				"cmV0dXJuIGxpc3QgJiYgbGlzdC5jb250YWlucyh0aGlzKTsKCX0KCn0pOwoKRnguY29tcHV0ZSA9IGZ1",
				"bmN0aW9uKGZyb20sIHRvLCBkZWx0YSl7CglyZXR1cm4gKHRvIC0gZnJvbSkgKiBkZWx0YSArIGZyb207",
				"Cn07CgpGeC5EdXJhdGlvbnMgPSB7J3Nob3J0JzogMjUwLCAnbm9ybWFsJzogNTAwLCAnbG9uZyc6IDEw",
				"MDB9OwoKLy8gZ2xvYmFsIHRpbWVycwoKdmFyIGluc3RhbmNlcyA9IHt9LCB0aW1lcnMgPSB7fTsKCnZh",
				"ciBsb29wID0gZnVuY3Rpb24oKXsKCXZhciBub3cgPSBEYXRlLm5vdygpOwoJZm9yICh2YXIgaSA9IHRo",
				"aXMubGVuZ3RoOyBpLS07KXsKCQl2YXIgaW5zdGFuY2UgPSB0aGlzW2ldOwoJCWlmIChpbnN0YW5jZSkg",
				"aW5zdGFuY2Uuc3RlcChub3cpOwoJfQp9OwoKdmFyIHB1c2hJbnN0YW5jZSA9IGZ1bmN0aW9uKGZwcyl7",
				"Cgl2YXIgbGlzdCA9IGluc3RhbmNlc1tmcHNdIHx8IChpbnN0YW5jZXNbZnBzXSA9IFtdKTsKCWxpc3Qu",
				"cHVzaCh0aGlzKTsKCWlmICghdGltZXJzW2Zwc10pIHRpbWVyc1tmcHNdID0gbG9vcC5wZXJpb2RpY2Fs",
				"KE1hdGgucm91bmQoMTAwMCAvIGZwcyksIGxpc3QpOwp9OwoKdmFyIHB1bGxJbnN0YW5jZSA9IGZ1bmN0",
				"aW9uKGZwcyl7Cgl2YXIgbGlzdCA9IGluc3RhbmNlc1tmcHNdOwoJaWYgKGxpc3QpewoJCWxpc3QuZXJh",
				"c2UodGhpcyk7CgkJaWYgKCFsaXN0Lmxlbmd0aCAmJiB0aW1lcnNbZnBzXSl7CgkJCWRlbGV0ZSBpbnN0",
				"YW5jZXNbZnBzXTsKCQkJdGltZXJzW2Zwc10gPSBjbGVhckludGVydmFsKHRpbWVyc1tmcHNdKTsKCQl9",
				"Cgl9Cn07Cgp9KSgpOwoKCi8qCi0tLQoKbmFtZTogRnguQ1NTCgpkZXNjcmlwdGlvbjogQ29udGFpbnMg",
				"dGhlIENTUyBhbmltYXRpb24gbG9naWMuIFVzZWQgYnkgRnguVHdlZW4sIEZ4Lk1vcnBoLCBGeC5FbGVt",
				"ZW50cy4KCmxpY2Vuc2U6IE1JVC1zdHlsZSBsaWNlbnNlLgoKcmVxdWlyZXM6IFtGeCwgRWxlbWVudC5T",
				"dHlsZV0KCnByb3ZpZGVzOiBGeC5DU1MKCi4uLgoqLwoKRnguQ1NTID0gbmV3IENsYXNzKHsKCglFeHRl",
				"bmRzOiBGeCwKCgkvL3ByZXBhcmVzIHRoZSBiYXNlIGZyb20vdG8gb2JqZWN0CgoJcHJlcGFyZTogZnVu",
				"Y3Rpb24oZWxlbWVudCwgcHJvcGVydHksIHZhbHVlcyl7CgkJdmFsdWVzID0gQXJyYXkuZnJvbSh2YWx1",
				"ZXMpOwoJCXZhciBmcm9tID0gdmFsdWVzWzBdLCB0byA9IHZhbHVlc1sxXTsKCQlpZiAodG8gPT0gbnVs",
				"bCl7CgkJCXRvID0gZnJvbTsKCQkJZnJvbSA9IGVsZW1lbnQuZ2V0U3R5bGUocHJvcGVydHkpOwoJCQl2",
				"YXIgdW5pdCA9IHRoaXMub3B0aW9ucy51bml0OwoJCQkvLyBhZGFwdGVkIGZyb206IGh0dHBzOi8vZ2l0",
				"aHViLmNvbS9yeWFubW9yci9meC9ibG9iL21hc3Rlci9meC5qcyNMMjk5CgkJCWlmICh1bml0ICYmIGZy",
				"b20uc2xpY2UoLXVuaXQubGVuZ3RoKSAhPSB1bml0ICYmIHBhcnNlRmxvYXQoZnJvbSkgIT0gMCl7CgkJ",
				"CQllbGVtZW50LnNldFN0eWxlKHByb3BlcnR5LCB0byArIHVuaXQpOwoJCQkJdmFyIHZhbHVlID0gZWxl",
				"bWVudC5nZXRDb21wdXRlZFN0eWxlKHByb3BlcnR5KTsKCQkJCS8vIElFIGFuZCBPcGVyYSBzdXBwb3J0",
				"IHBpeGVsTGVmdCBvciBwaXhlbFdpZHRoCgkJCQlpZiAoISgvcHgkLy50ZXN0KHZhbHVlKSkpewoJCQkJ",
				"CXZhbHVlID0gZWxlbWVudC5zdHlsZVsoJ3BpeGVsLScgKyBwcm9wZXJ0eSkuY2FtZWxDYXNlKCldOwoJ",
				"CQkJCWlmICh2YWx1ZSA9PSBudWxsKXsKCQkJCQkJLy8gYWRhcHRlZCBmcm9tIERlYW4gRWR3YXJkcycg",
				"aHR0cDovL2VyaWsuZWFlLm5ldC9hcmNoaXZlcy8yMDA3LzA3LzI3LzE4LjU0LjE1LyNjb21tZW50LTEw",
				"MjI5MQoJCQkJCQl2YXIgbGVmdCA9IGVsZW1lbnQuc3R5bGUubGVmdDsKCQkJCQkJZWxlbWVudC5zdHls",
				"ZS5sZWZ0ID0gdG8gKyB1bml0OwoJCQkJCQl2YWx1ZSA9IGVsZW1lbnQuc3R5bGUucGl4ZWxMZWZ0OwoJ",
				"CQkJCQllbGVtZW50LnN0eWxlLmxlZnQgPSBsZWZ0OwoJCQkJCX0KCQkJCX0KCQkJCWZyb20gPSAodG8g",
				"fHwgMSkgLyAocGFyc2VGbG9hdCh2YWx1ZSkgfHwgMSkgKiAocGFyc2VGbG9hdChmcm9tKSB8fCAwKTsK",
				"CQkJCWVsZW1lbnQuc2V0U3R5bGUocHJvcGVydHksIGZyb20gKyB1bml0KTsKCQkJfQoJCX0KCQlyZXR1",
				"cm4ge2Zyb206IHRoaXMucGFyc2UoZnJvbSksIHRvOiB0aGlzLnBhcnNlKHRvKX07Cgl9LAoKCS8vcGFy",
				"c2VzIGEgdmFsdWUgaW50byBhbiBhcnJheQoKCXBhcnNlOiBmdW5jdGlvbih2YWx1ZSl7CgkJdmFsdWUg",
				"PSBGdW5jdGlvbi5mcm9tKHZhbHVlKSgpOwoJCXZhbHVlID0gKHR5cGVvZiB2YWx1ZSA9PSAnc3RyaW5n",
				"JykgPyB2YWx1ZS5zcGxpdCgnICcpIDogQXJyYXkuZnJvbSh2YWx1ZSk7CgkJcmV0dXJuIHZhbHVlLm1h",
				"cChmdW5jdGlvbih2YWwpewoJCQl2YWwgPSBTdHJpbmcodmFsKTsKCQkJdmFyIGZvdW5kID0gZmFsc2U7",
				"CgkJCU9iamVjdC5lYWNoKEZ4LkNTUy5QYXJzZXJzLCBmdW5jdGlvbihwYXJzZXIsIGtleSl7CgkJCQlp",
				"ZiAoZm91bmQpIHJldHVybjsKCQkJCXZhciBwYXJzZWQgPSBwYXJzZXIucGFyc2UodmFsKTsKCQkJCWlm",
				"IChwYXJzZWQgfHwgcGFyc2VkID09PSAwKSBmb3VuZCA9IHt2YWx1ZTogcGFyc2VkLCBwYXJzZXI6IHBh",
				"cnNlcn07CgkJCX0pOwoJCQlmb3VuZCA9IGZvdW5kIHx8IHt2YWx1ZTogdmFsLCBwYXJzZXI6IEZ4LkNT",
				"Uy5QYXJzZXJzLlN0cmluZ307CgkJCXJldHVybiBmb3VuZDsKCQl9KTsKCX0sCgoJLy9jb21wdXRlcyBi",
				"eSBhIGZyb20gYW5kIHRvIHByZXBhcmVkIG9iamVjdHMsIHVzaW5nIHRoZWlyIHBhcnNlcnMuCgoJY29t",
				"cHV0ZTogZnVuY3Rpb24oZnJvbSwgdG8sIGRlbHRhKXsKCQl2YXIgY29tcHV0ZWQgPSBbXTsKCQkoTWF0",
				"aC5taW4oZnJvbS5sZW5ndGgsIHRvLmxlbmd0aCkpLnRpbWVzKGZ1bmN0aW9uKGkpewoJCQljb21wdXRl",
				"ZC5wdXNoKHt2YWx1ZTogZnJvbVtpXS5wYXJzZXIuY29tcHV0ZShmcm9tW2ldLnZhbHVlLCB0b1tpXS52",
				"YWx1ZSwgZGVsdGEpLCBwYXJzZXI6IGZyb21baV0ucGFyc2VyfSk7CgkJfSk7CgkJY29tcHV0ZWQuJGZh",
				"bWlseSA9IEZ1bmN0aW9uLmZyb20oJ2Z4OmNzczp2YWx1ZScpOwoJCXJldHVybiBjb21wdXRlZDsKCX0s",
				"CgoJLy9zZXJ2ZXMgdGhlIHZhbHVlIGFzIHNldHRhYmxlCgoJc2VydmU6IGZ1bmN0aW9uKHZhbHVlLCB1",
				"bml0KXsKCQlpZiAodHlwZU9mKHZhbHVlKSAhPSAnZng6Y3NzOnZhbHVlJykgdmFsdWUgPSB0aGlzLnBh",
				"cnNlKHZhbHVlKTsKCQl2YXIgcmV0dXJuZWQgPSBbXTsKCQl2YWx1ZS5lYWNoKGZ1bmN0aW9uKGJpdCl7",
				"CgkJCXJldHVybmVkID0gcmV0dXJuZWQuY29uY2F0KGJpdC5wYXJzZXIuc2VydmUoYml0LnZhbHVlLCB1",
				"bml0KSk7CgkJfSk7CgkJcmV0dXJuIHJldHVybmVkOwoJfSwKCgkvL3JlbmRlcnMgdGhlIGNoYW5nZSB0",
				"byBhbiBlbGVtZW50CgoJcmVuZGVyOiBmdW5jdGlvbihlbGVtZW50LCBwcm9wZXJ0eSwgdmFsdWUsIHVu",
				"aXQpewoJCWVsZW1lbnQuc2V0U3R5bGUocHJvcGVydHksIHRoaXMuc2VydmUodmFsdWUsIHVuaXQpKTsK",
				"CX0sCgoJLy9zZWFyY2hlcyBpbnNpZGUgdGhlIHBhZ2UgY3NzIHRvIGZpbmQgdGhlIHZhbHVlcyBmb3Ig",
				"YSBzZWxlY3RvcgoKCXNlYXJjaDogZnVuY3Rpb24oc2VsZWN0b3IpewoJCWlmIChGeC5DU1MuQ2FjaGVb",
				"c2VsZWN0b3JdKSByZXR1cm4gRnguQ1NTLkNhY2hlW3NlbGVjdG9yXTsKCQl2YXIgdG8gPSB7fSwgc2Vs",
				"ZWN0b3JUZXN0ID0gbmV3IFJlZ0V4cCgnXicgKyBzZWxlY3Rvci5lc2NhcGVSZWdFeHAoKSArICckJyk7",
				"CgkJQXJyYXkuZWFjaChkb2N1bWVudC5zdHlsZVNoZWV0cywgZnVuY3Rpb24oc2hlZXQsIGopewoJCQl2",
				"YXIgaHJlZiA9IHNoZWV0LmhyZWY7CgkJCWlmIChocmVmICYmIGhyZWYuY29udGFpbnMoJzovLycpICYm",
				"ICFocmVmLmNvbnRhaW5zKGRvY3VtZW50LmRvbWFpbikpIHJldHVybjsKCQkJdmFyIHJ1bGVzID0gc2hl",
				"ZXQucnVsZXMgfHwgc2hlZXQuY3NzUnVsZXM7CgkJCUFycmF5LmVhY2gocnVsZXMsIGZ1bmN0aW9uKHJ1",
				"bGUsIGkpewoJCQkJaWYgKCFydWxlLnN0eWxlKSByZXR1cm47CgkJCQl2YXIgc2VsZWN0b3JUZXh0ID0g",
				"KHJ1bGUuc2VsZWN0b3JUZXh0KSA/IHJ1bGUuc2VsZWN0b3JUZXh0LnJlcGxhY2UoL15cdysvLCBmdW5j",
				"dGlvbihtKXsKCQkJCQlyZXR1cm4gbS50b0xvd2VyQ2FzZSgpOwoJCQkJfSkgOiBudWxsOwoJCQkJaWYg",
				"KCFzZWxlY3RvclRleHQgfHwgIXNlbGVjdG9yVGVzdC50ZXN0KHNlbGVjdG9yVGV4dCkpIHJldHVybjsK",
				"CQkJCU9iamVjdC5lYWNoKEVsZW1lbnQuU3R5bGVzLCBmdW5jdGlvbih2YWx1ZSwgc3R5bGUpewoJCQkJ",
				"CWlmICghcnVsZS5zdHlsZVtzdHlsZV0gfHwgRWxlbWVudC5TaG9ydFN0eWxlc1tzdHlsZV0pIHJldHVy",
				"bjsKCQkJCQl2YWx1ZSA9IFN0cmluZyhydWxlLnN0eWxlW3N0eWxlXSk7CgkJCQkJdG9bc3R5bGVdID0g",
				"KCgvXnJnYi8pLnRlc3QodmFsdWUpKSA/IHZhbHVlLnJnYlRvSGV4KCkgOiB2YWx1ZTsKCQkJCX0pOwoJ",
				"CQl9KTsKCQl9KTsKCQlyZXR1cm4gRnguQ1NTLkNhY2hlW3NlbGVjdG9yXSA9IHRvOwoJfQoKfSk7CgpG",
				"eC5DU1MuQ2FjaGUgPSB7fTsKCkZ4LkNTUy5QYXJzZXJzID0gewoKCUNvbG9yOiB7CgkJcGFyc2U6IGZ1",
				"bmN0aW9uKHZhbHVlKXsKCQkJaWYgKHZhbHVlLm1hdGNoKC9eI1swLTlhLWZdezMsNn0kL2kpKSByZXR1",
				"cm4gdmFsdWUuaGV4VG9SZ2IodHJ1ZSk7CgkJCXJldHVybiAoKHZhbHVlID0gdmFsdWUubWF0Y2goLyhc",
				"ZCspLFxzKihcZCspLFxzKihcZCspLykpKSA/IFt2YWx1ZVsxXSwgdmFsdWVbMl0sIHZhbHVlWzNdXSA6",
				"IGZhbHNlOwoJCX0sCgkJY29tcHV0ZTogZnVuY3Rpb24oZnJvbSwgdG8sIGRlbHRhKXsKCQkJcmV0dXJu",
				"IGZyb20ubWFwKGZ1bmN0aW9uKHZhbHVlLCBpKXsKCQkJCXJldHVybiBNYXRoLnJvdW5kKEZ4LmNvbXB1",
				"dGUoZnJvbVtpXSwgdG9baV0sIGRlbHRhKSk7CgkJCX0pOwoJCX0sCgkJc2VydmU6IGZ1bmN0aW9uKHZh",
				"bHVlKXsKCQkJcmV0dXJuIHZhbHVlLm1hcChOdW1iZXIpOwoJCX0KCX0sCgoJTnVtYmVyOiB7CgkJcGFy",
				"c2U6IHBhcnNlRmxvYXQsCgkJY29tcHV0ZTogRnguY29tcHV0ZSwKCQlzZXJ2ZTogZnVuY3Rpb24odmFs",
				"dWUsIHVuaXQpewoJCQlyZXR1cm4gKHVuaXQpID8gdmFsdWUgKyB1bml0IDogdmFsdWU7CgkJfQoJfSwK",
				"CglTdHJpbmc6IHsKCQlwYXJzZTogRnVuY3Rpb24uZnJvbShmYWxzZSksCgkJY29tcHV0ZTogZnVuY3Rp",
				"b24oemVybywgb25lKXsKCQkJcmV0dXJuIG9uZTsKCQl9LAoJCXNlcnZlOiBmdW5jdGlvbih6ZXJvKXsK",
				"CQkJcmV0dXJuIHplcm87CgkJfQoJfQoKfTsKCi8vPDEuMmNvbXBhdD4KCkZ4LkNTUy5QYXJzZXJzID0g",
				"bmV3IEhhc2goRnguQ1NTLlBhcnNlcnMpOwoKLy88LzEuMmNvbXBhdD4KCgovKgotLS0KCm5hbWU6IEZ4",
				"LlR3ZWVuCgpkZXNjcmlwdGlvbjogRm9ybWVybHkgRnguU3R5bGUsIGVmZmVjdCB0byB0cmFuc2l0aW9u",
				"IGFueSBDU1MgcHJvcGVydHkgZm9yIGFuIGVsZW1lbnQuCgpsaWNlbnNlOiBNSVQtc3R5bGUgbGljZW5z",
				"ZS4KCnJlcXVpcmVzOiBGeC5DU1MKCnByb3ZpZGVzOiBbRnguVHdlZW4sIEVsZW1lbnQuZmFkZSwgRWxl",
				"bWVudC5oaWdobGlnaHRdCgouLi4KKi8KCkZ4LlR3ZWVuID0gbmV3IENsYXNzKHsKCglFeHRlbmRzOiBG",
				"eC5DU1MsCgoJaW5pdGlhbGl6ZTogZnVuY3Rpb24oZWxlbWVudCwgb3B0aW9ucyl7CgkJdGhpcy5lbGVt",
				"ZW50ID0gdGhpcy5zdWJqZWN0ID0gZG9jdW1lbnQuaWQoZWxlbWVudCk7CgkJdGhpcy5wYXJlbnQob3B0",
				"aW9ucyk7Cgl9LAoKCXNldDogZnVuY3Rpb24ocHJvcGVydHksIG5vdyl7CgkJaWYgKGFyZ3VtZW50cy5s",
				"ZW5ndGggPT0gMSl7CgkJCW5vdyA9IHByb3BlcnR5OwoJCQlwcm9wZXJ0eSA9IHRoaXMucHJvcGVydHkg",
				"fHwgdGhpcy5vcHRpb25zLnByb3BlcnR5OwoJCX0KCQl0aGlzLnJlbmRlcih0aGlzLmVsZW1lbnQsIHBy",
				"b3BlcnR5LCBub3csIHRoaXMub3B0aW9ucy51bml0KTsKCQlyZXR1cm4gdGhpczsKCX0sCgoJc3RhcnQ6",
				"IGZ1bmN0aW9uKHByb3BlcnR5LCBmcm9tLCB0byl7CgkJaWYgKCF0aGlzLmNoZWNrKHByb3BlcnR5LCBm",
				"cm9tLCB0bykpIHJldHVybiB0aGlzOwoJCXZhciBhcmdzID0gQXJyYXkuZmxhdHRlbihhcmd1bWVudHMp",
				"OwoJCXRoaXMucHJvcGVydHkgPSB0aGlzLm9wdGlvbnMucHJvcGVydHkgfHwgYXJncy5zaGlmdCgpOwoJ",
				"CXZhciBwYXJzZWQgPSB0aGlzLnByZXBhcmUodGhpcy5lbGVtZW50LCB0aGlzLnByb3BlcnR5LCBhcmdz",
				"KTsKCQlyZXR1cm4gdGhpcy5wYXJlbnQocGFyc2VkLmZyb20sIHBhcnNlZC50byk7Cgl9Cgp9KTsKCkVs",
				"ZW1lbnQuUHJvcGVydGllcy50d2VlbiA9IHsKCglzZXQ6IGZ1bmN0aW9uKG9wdGlvbnMpewoJCXRoaXMu",
				"Z2V0KCd0d2VlbicpLmNhbmNlbCgpLnNldE9wdGlvbnMob3B0aW9ucyk7CgkJcmV0dXJuIHRoaXM7Cgl9",
				"LAoKCWdldDogZnVuY3Rpb24oKXsKCQl2YXIgdHdlZW4gPSB0aGlzLnJldHJpZXZlKCd0d2VlbicpOwoJ",
				"CWlmICghdHdlZW4pewoJCQl0d2VlbiA9IG5ldyBGeC5Ud2Vlbih0aGlzLCB7bGluazogJ2NhbmNlbCd9",
				"KTsKCQkJdGhpcy5zdG9yZSgndHdlZW4nLCB0d2Vlbik7CgkJfQoJCXJldHVybiB0d2VlbjsKCX0KCn07",
				"CgpFbGVtZW50LmltcGxlbWVudCh7CgoJdHdlZW46IGZ1bmN0aW9uKHByb3BlcnR5LCBmcm9tLCB0byl7",
				"CgkJdGhpcy5nZXQoJ3R3ZWVuJykuc3RhcnQocHJvcGVydHksIGZyb20sIHRvKTsKCQlyZXR1cm4gdGhp",
				"czsKCX0sCgoJZmFkZTogZnVuY3Rpb24oaG93KXsKCQl2YXIgZmFkZSA9IHRoaXMuZ2V0KCd0d2Vlbicp",
				"LCBtZXRob2QsIGFyZ3MgPSBbJ29wYWNpdHknXS5hcHBlbmQoYXJndW1lbnRzKSwgdG9nZ2xlOwoJCWlm",
				"IChhcmdzWzFdID09IG51bGwpIGFyZ3NbMV0gPSAndG9nZ2xlJzsKCQlzd2l0Y2ggKGFyZ3NbMV0pewoJ",
				"CQljYXNlICdpbic6IG1ldGhvZCA9ICdzdGFydCc7IGFyZ3NbMV0gPSAxOyBicmVhazsKCQkJY2FzZSAn",
				"b3V0JzogbWV0aG9kID0gJ3N0YXJ0JzsgYXJnc1sxXSA9IDA7IGJyZWFrOwoJCQljYXNlICdzaG93Jzog",
				"bWV0aG9kID0gJ3NldCc7IGFyZ3NbMV0gPSAxOyBicmVhazsKCQkJY2FzZSAnaGlkZSc6IG1ldGhvZCA9",
				"ICdzZXQnOyBhcmdzWzFdID0gMDsgYnJlYWs7CgkJCWNhc2UgJ3RvZ2dsZSc6CgkJCQl2YXIgZmxhZyA9",
				"IHRoaXMucmV0cmlldmUoJ2ZhZGU6ZmxhZycsIHRoaXMuZ2V0U3R5bGUoJ29wYWNpdHknKSA9PSAxKTsK",
				"CQkJCW1ldGhvZCA9ICdzdGFydCc7CgkJCQlhcmdzWzFdID0gZmxhZyA/IDAgOiAxOwoJCQkJdGhpcy5z",
				"dG9yZSgnZmFkZTpmbGFnJywgIWZsYWcpOwoJCQkJdG9nZ2xlID0gdHJ1ZTsKCQkJYnJlYWs7CgkJCWRl",
				"ZmF1bHQ6IG1ldGhvZCA9ICdzdGFydCc7CgkJfQoJCWlmICghdG9nZ2xlKSB0aGlzLmVsaW1pbmF0ZSgn",
				"ZmFkZTpmbGFnJyk7CgkJZmFkZVttZXRob2RdLmFwcGx5KGZhZGUsIGFyZ3MpOwoJCXZhciB0byA9IGFy",
				"Z3NbYXJncy5sZW5ndGggLSAxXTsKCQlpZiAobWV0aG9kID09ICdzZXQnIHx8IHRvICE9IDApIHRoaXMu",
				"c2V0U3R5bGUoJ3Zpc2liaWxpdHknLCB0byA9PSAwID8gJ2hpZGRlbicgOiAndmlzaWJsZScpOwoJCWVs",
				"c2UgZmFkZS5jaGFpbihmdW5jdGlvbigpewoJCQl0aGlzLmVsZW1lbnQuc2V0U3R5bGUoJ3Zpc2liaWxp",
				"dHknLCAnaGlkZGVuJyk7CgkJCXRoaXMuY2FsbENoYWluKCk7CgkJfSk7CgkJcmV0dXJuIHRoaXM7Cgl9",
				"LAoKCWhpZ2hsaWdodDogZnVuY3Rpb24oc3RhcnQsIGVuZCl7CgkJaWYgKCFlbmQpewoJCQllbmQgPSB0",
				"aGlzLnJldHJpZXZlKCdoaWdobGlnaHQ6b3JpZ2luYWwnLCB0aGlzLmdldFN0eWxlKCdiYWNrZ3JvdW5k",
				"LWNvbG9yJykpOwoJCQllbmQgPSAoZW5kID09ICd0cmFuc3BhcmVudCcpID8gJyNmZmYnIDogZW5kOwoJ",
				"CX0KCQl2YXIgdHdlZW4gPSB0aGlzLmdldCgndHdlZW4nKTsKCQl0d2Vlbi5zdGFydCgnYmFja2dyb3Vu",
				"ZC1jb2xvcicsIHN0YXJ0IHx8ICcjZmZmZjg4JywgZW5kKS5jaGFpbihmdW5jdGlvbigpewoJCQl0aGlz",
				"LnNldFN0eWxlKCdiYWNrZ3JvdW5kLWNvbG9yJywgdGhpcy5yZXRyaWV2ZSgnaGlnaGxpZ2h0Om9yaWdp",
				"bmFsJykpOwoJCQl0d2Vlbi5jYWxsQ2hhaW4oKTsKCQl9LmJpbmQodGhpcykpOwoJCXJldHVybiB0aGlz",
				"OwoJfQoKfSk7CgoKLyoKLS0tCgpuYW1lOiBGeC5Nb3JwaAoKZGVzY3JpcHRpb246IEZvcm1lcmx5IEZ4",
				"LlN0eWxlcywgZWZmZWN0IHRvIHRyYW5zaXRpb24gYW55IG51bWJlciBvZiBDU1MgcHJvcGVydGllcyBm",
				"b3IgYW4gZWxlbWVudCB1c2luZyBhbiBvYmplY3Qgb2YgcnVsZXMsIG9yIENTUyBiYXNlZCBzZWxlY3Rv",
				"ciBydWxlcy4KCmxpY2Vuc2U6IE1JVC1zdHlsZSBsaWNlbnNlLgoKcmVxdWlyZXM6IEZ4LkNTUwoKcHJv",
				"dmlkZXM6IEZ4Lk1vcnBoCgouLi4KKi8KCkZ4Lk1vcnBoID0gbmV3IENsYXNzKHsKCglFeHRlbmRzOiBG",
				"eC5DU1MsCgoJaW5pdGlhbGl6ZTogZnVuY3Rpb24oZWxlbWVudCwgb3B0aW9ucyl7CgkJdGhpcy5lbGVt",
				"ZW50ID0gdGhpcy5zdWJqZWN0ID0gZG9jdW1lbnQuaWQoZWxlbWVudCk7CgkJdGhpcy5wYXJlbnQob3B0",
				"aW9ucyk7Cgl9LAoKCXNldDogZnVuY3Rpb24obm93KXsKCQlpZiAodHlwZW9mIG5vdyA9PSAnc3RyaW5n",
				"Jykgbm93ID0gdGhpcy5zZWFyY2gobm93KTsKCQlmb3IgKHZhciBwIGluIG5vdykgdGhpcy5yZW5kZXIo",
				"dGhpcy5lbGVtZW50LCBwLCBub3dbcF0sIHRoaXMub3B0aW9ucy51bml0KTsKCQlyZXR1cm4gdGhpczsK",
				"CX0sCgoJY29tcHV0ZTogZnVuY3Rpb24oZnJvbSwgdG8sIGRlbHRhKXsKCQl2YXIgbm93ID0ge307CgkJ",
				"Zm9yICh2YXIgcCBpbiBmcm9tKSBub3dbcF0gPSB0aGlzLnBhcmVudChmcm9tW3BdLCB0b1twXSwgZGVs",
				"dGEpOwoJCXJldHVybiBub3c7Cgl9LAoKCXN0YXJ0OiBmdW5jdGlvbihwcm9wZXJ0aWVzKXsKCQlpZiAo",
				"IXRoaXMuY2hlY2socHJvcGVydGllcykpIHJldHVybiB0aGlzOwoJCWlmICh0eXBlb2YgcHJvcGVydGll",
				"cyA9PSAnc3RyaW5nJykgcHJvcGVydGllcyA9IHRoaXMuc2VhcmNoKHByb3BlcnRpZXMpOwoJCXZhciBm",
				"cm9tID0ge30sIHRvID0ge307CgkJZm9yICh2YXIgcCBpbiBwcm9wZXJ0aWVzKXsKCQkJdmFyIHBhcnNl",
				"ZCA9IHRoaXMucHJlcGFyZSh0aGlzLmVsZW1lbnQsIHAsIHByb3BlcnRpZXNbcF0pOwoJCQlmcm9tW3Bd",
				"ID0gcGFyc2VkLmZyb207CgkJCXRvW3BdID0gcGFyc2VkLnRvOwoJCX0KCQlyZXR1cm4gdGhpcy5wYXJl",
				"bnQoZnJvbSwgdG8pOwoJfQoKfSk7CgpFbGVtZW50LlByb3BlcnRpZXMubW9ycGggPSB7CgoJc2V0OiBm",
				"dW5jdGlvbihvcHRpb25zKXsKCQl0aGlzLmdldCgnbW9ycGgnKS5jYW5jZWwoKS5zZXRPcHRpb25zKG9w",
				"dGlvbnMpOwoJCXJldHVybiB0aGlzOwoJfSwKCglnZXQ6IGZ1bmN0aW9uKCl7CgkJdmFyIG1vcnBoID0g",
				"dGhpcy5yZXRyaWV2ZSgnbW9ycGgnKTsKCQlpZiAoIW1vcnBoKXsKCQkJbW9ycGggPSBuZXcgRnguTW9y",
				"cGgodGhpcywge2xpbms6ICdjYW5jZWwnfSk7CgkJCXRoaXMuc3RvcmUoJ21vcnBoJywgbW9ycGgpOwoJ",
				"CX0KCQlyZXR1cm4gbW9ycGg7Cgl9Cgp9OwoKRWxlbWVudC5pbXBsZW1lbnQoewoKCW1vcnBoOiBmdW5j",
				"dGlvbihwcm9wcyl7CgkJdGhpcy5nZXQoJ21vcnBoJykuc3RhcnQocHJvcHMpOwoJCXJldHVybiB0aGlz",
				"OwoJfQoKfSk7CgoKLyoKLS0tCgpuYW1lOiBGeC5UcmFuc2l0aW9ucwoKZGVzY3JpcHRpb246IENvbnRh",
				"aW5zIGEgc2V0IG9mIGFkdmFuY2VkIHRyYW5zaXRpb25zIHRvIGJlIHVzZWQgd2l0aCBhbnkgb2YgdGhl",
				"IEZ4IENsYXNzZXMuCgpsaWNlbnNlOiBNSVQtc3R5bGUgbGljZW5zZS4KCmNyZWRpdHM6CiAgLSBFYXNp",
				"bmcgRXF1YXRpb25zIGJ5IFJvYmVydCBQZW5uZXIsIDxodHRwOi8vd3d3LnJvYmVydHBlbm5lci5jb20v",
				"ZWFzaW5nLz4sIG1vZGlmaWVkIGFuZCBvcHRpbWl6ZWQgdG8gYmUgdXNlZCB3aXRoIE1vb1Rvb2xzLgoK",
				"cmVxdWlyZXM6IEZ4Cgpwcm92aWRlczogRnguVHJhbnNpdGlvbnMKCi4uLgoqLwoKRnguaW1wbGVtZW50",
				"KHsKCglnZXRUcmFuc2l0aW9uOiBmdW5jdGlvbigpewoJCXZhciB0cmFucyA9IHRoaXMub3B0aW9ucy50",
				"cmFuc2l0aW9uIHx8IEZ4LlRyYW5zaXRpb25zLlNpbmUuZWFzZUluT3V0OwoJCWlmICh0eXBlb2YgdHJh",
				"bnMgPT0gJ3N0cmluZycpewoJCQl2YXIgZGF0YSA9IHRyYW5zLnNwbGl0KCc6Jyk7CgkJCXRyYW5zID0g",
				"RnguVHJhbnNpdGlvbnM7CgkJCXRyYW5zID0gdHJhbnNbZGF0YVswXV0gfHwgdHJhbnNbZGF0YVswXS5j",
				"YXBpdGFsaXplKCldOwoJCQlpZiAoZGF0YVsxXSkgdHJhbnMgPSB0cmFuc1snZWFzZScgKyBkYXRhWzFd",
				"LmNhcGl0YWxpemUoKSArIChkYXRhWzJdID8gZGF0YVsyXS5jYXBpdGFsaXplKCkgOiAnJyldOwoJCX0K",
				"CQlyZXR1cm4gdHJhbnM7Cgl9Cgp9KTsKCkZ4LlRyYW5zaXRpb24gPSBmdW5jdGlvbih0cmFuc2l0aW9u",
				"LCBwYXJhbXMpewoJcGFyYW1zID0gQXJyYXkuZnJvbShwYXJhbXMpOwoJdmFyIGVhc2VJbiA9IGZ1bmN0",
				"aW9uKHBvcyl7CgkJcmV0dXJuIHRyYW5zaXRpb24ocG9zLCBwYXJhbXMpOwoJfTsKCXJldHVybiBPYmpl",
				"Y3QuYXBwZW5kKGVhc2VJbiwgewoJCWVhc2VJbjogZWFzZUluLAoJCWVhc2VPdXQ6IGZ1bmN0aW9uKHBv",
				"cyl7CgkJCXJldHVybiAxIC0gdHJhbnNpdGlvbigxIC0gcG9zLCBwYXJhbXMpOwoJCX0sCgkJZWFzZUlu",
				"T3V0OiBmdW5jdGlvbihwb3MpewoJCQlyZXR1cm4gKHBvcyA8PSAwLjUgPyB0cmFuc2l0aW9uKDIgKiBw",
				"b3MsIHBhcmFtcykgOiAoMiAtIHRyYW5zaXRpb24oMiAqICgxIC0gcG9zKSwgcGFyYW1zKSkpIC8gMjsK",
				"CQl9Cgl9KTsKfTsKCkZ4LlRyYW5zaXRpb25zID0gewoKCWxpbmVhcjogZnVuY3Rpb24oemVybyl7CgkJ",
				"cmV0dXJuIHplcm87Cgl9Cgp9OwoKLy88MS4yY29tcGF0PgoKRnguVHJhbnNpdGlvbnMgPSBuZXcgSGFz",
				"aChGeC5UcmFuc2l0aW9ucyk7CgovLzwvMS4yY29tcGF0PgoKRnguVHJhbnNpdGlvbnMuZXh0ZW5kID0g",
				"ZnVuY3Rpb24odHJhbnNpdGlvbnMpewoJZm9yICh2YXIgdHJhbnNpdGlvbiBpbiB0cmFuc2l0aW9ucykg",
				"RnguVHJhbnNpdGlvbnNbdHJhbnNpdGlvbl0gPSBuZXcgRnguVHJhbnNpdGlvbih0cmFuc2l0aW9uc1t0",
				"cmFuc2l0aW9uXSk7Cn07CgpGeC5UcmFuc2l0aW9ucy5leHRlbmQoewoKCVBvdzogZnVuY3Rpb24ocCwg",
				"eCl7CgkJcmV0dXJuIE1hdGgucG93KHAsIHggJiYgeFswXSB8fCA2KTsKCX0sCgoJRXhwbzogZnVuY3Rp",
				"b24ocCl7CgkJcmV0dXJuIE1hdGgucG93KDIsIDggKiAocCAtIDEpKTsKCX0sCgoJQ2lyYzogZnVuY3Rp",
				"b24ocCl7CgkJcmV0dXJuIDEgLSBNYXRoLnNpbihNYXRoLmFjb3MocCkpOwoJfSwKCglTaW5lOiBmdW5j",
				"dGlvbihwKXsKCQlyZXR1cm4gMSAtIE1hdGguY29zKHAgKiBNYXRoLlBJIC8gMik7Cgl9LAoKCUJhY2s6",
				"IGZ1bmN0aW9uKHAsIHgpewoJCXggPSB4ICYmIHhbMF0gfHwgMS42MTg7CgkJcmV0dXJuIE1hdGgucG93",
				"KHAsIDIpICogKCh4ICsgMSkgKiBwIC0geCk7Cgl9LAoKCUJvdW5jZTogZnVuY3Rpb24ocCl7CgkJdmFy",
				"IHZhbHVlOwoJCWZvciAodmFyIGEgPSAwLCBiID0gMTsgMTsgYSArPSBiLCBiIC89IDIpewoJCQlpZiAo",
				"cCA+PSAoNyAtIDQgKiBhKSAvIDExKXsKCQkJCXZhbHVlID0gYiAqIGIgLSBNYXRoLnBvdygoMTEgLSA2",
				"ICogYSAtIDExICogcCkgLyA0LCAyKTsKCQkJCWJyZWFrOwoJCQl9CgkJfQoJCXJldHVybiB2YWx1ZTsK",
				"CX0sCgoJRWxhc3RpYzogZnVuY3Rpb24ocCwgeCl7CgkJcmV0dXJuIE1hdGgucG93KDIsIDEwICogLS1w",
				"KSAqIE1hdGguY29zKDIwICogcCAqIE1hdGguUEkgKiAoeCAmJiB4WzBdIHx8IDEpIC8gMyk7Cgl9Cgp9",
				"KTsKClsnUXVhZCcsICdDdWJpYycsICdRdWFydCcsICdRdWludCddLmVhY2goZnVuY3Rpb24odHJhbnNp",
				"dGlvbiwgaSl7CglGeC5UcmFuc2l0aW9uc1t0cmFuc2l0aW9uXSA9IG5ldyBGeC5UcmFuc2l0aW9uKGZ1",
				"bmN0aW9uKHApewoJCXJldHVybiBNYXRoLnBvdyhwLCBpICsgMik7Cgl9KTsKfSk7CgoKLyoKLS0tCgpu",
				"YW1lOiBSZXF1ZXN0CgpkZXNjcmlwdGlvbjogUG93ZXJmdWwgYWxsIHB1cnBvc2UgUmVxdWVzdCBDbGFz",
				"cy4gVXNlcyBYTUxIVFRQUmVxdWVzdC4KCmxpY2Vuc2U6IE1JVC1zdHlsZSBsaWNlbnNlLgoKcmVxdWly",
				"ZXM6IFtPYmplY3QsIEVsZW1lbnQsIENoYWluLCBFdmVudHMsIE9wdGlvbnMsIEJyb3dzZXJdCgpwcm92",
				"aWRlczogUmVxdWVzdAoKLi4uCiovCgooZnVuY3Rpb24oKXsKCnZhciBlbXB0eSA9IGZ1bmN0aW9uKCl7",
				"fSwKCXByb2dyZXNzU3VwcG9ydCA9ICgnb25wcm9ncmVzcycgaW4gbmV3IEJyb3dzZXIuUmVxdWVzdCk7",
				"Cgp2YXIgUmVxdWVzdCA9IHRoaXMuUmVxdWVzdCA9IG5ldyBDbGFzcyh7CgoJSW1wbGVtZW50czogW0No",
				"YWluLCBFdmVudHMsIE9wdGlvbnNdLAoKCW9wdGlvbnM6IHsvKgoJCW9uUmVxdWVzdDogZnVuY3Rpb24o",
				"KXt9LAoJCW9uTG9hZHN0YXJ0OiBmdW5jdGlvbihldmVudCwgeGhyKXt9LAoJCW9uUHJvZ3Jlc3M6IGZ1",
				"bmN0aW9uKGV2ZW50LCB4aHIpe30sCgkJb25Db21wbGV0ZTogZnVuY3Rpb24oKXt9LAoJCW9uQ2FuY2Vs",
				"OiBmdW5jdGlvbigpe30sCgkJb25TdWNjZXNzOiBmdW5jdGlvbihyZXNwb25zZVRleHQsIHJlc3BvbnNl",
				"WE1MKXt9LAoJCW9uRmFpbHVyZTogZnVuY3Rpb24oeGhyKXt9LAoJCW9uRXhjZXB0aW9uOiBmdW5jdGlv",
				"bihoZWFkZXJOYW1lLCB2YWx1ZSl7fSwKCQlvblRpbWVvdXQ6IGZ1bmN0aW9uKCl7fSwKCQl1c2VyOiAn",
				"JywKCQlwYXNzd29yZDogJycsKi8KCQl1cmw6ICcnLAoJCWRhdGE6ICcnLAoJCWhlYWRlcnM6IHsKCQkJ",
				"J1gtUmVxdWVzdGVkLVdpdGgnOiAnWE1MSHR0cFJlcXVlc3QnLAoJCQknQWNjZXB0JzogJ3RleHQvamF2",
				"YXNjcmlwdCwgdGV4dC9odG1sLCBhcHBsaWNhdGlvbi94bWwsIHRleHQveG1sLCAqLyonCgkJfSwKCQlh",
				"c3luYzogdHJ1ZSwKCQlmb3JtYXQ6IGZhbHNlLAoJCW1ldGhvZDogJ3Bvc3QnLAoJCWxpbms6ICdpZ25v",
				"cmUnLAoJCWlzU3VjY2VzczogbnVsbCwKCQllbXVsYXRpb246IHRydWUsCgkJdXJsRW5jb2RlZDogdHJ1",
				"ZSwKCQllbmNvZGluZzogJ3V0Zi04JywKCQlldmFsU2NyaXB0czogZmFsc2UsCgkJZXZhbFJlc3BvbnNl",
				"OiBmYWxzZSwKCQl0aW1lb3V0OiAwLAoJCW5vQ2FjaGU6IGZhbHNlCgl9LAoKCWluaXRpYWxpemU6IGZ1",
				"bmN0aW9uKG9wdGlvbnMpewoJCXRoaXMueGhyID0gbmV3IEJyb3dzZXIuUmVxdWVzdCgpOwoJCXRoaXMu",
				"c2V0T3B0aW9ucyhvcHRpb25zKTsKCQl0aGlzLmhlYWRlcnMgPSB0aGlzLm9wdGlvbnMuaGVhZGVyczsK",
				"CX0sCgoJb25TdGF0ZUNoYW5nZTogZnVuY3Rpb24oKXsKCQl2YXIgeGhyID0gdGhpcy54aHI7CgkJaWYg",
				"KHhoci5yZWFkeVN0YXRlICE9IDQgfHwgIXRoaXMucnVubmluZykgcmV0dXJuOwoJCXRoaXMucnVubmlu",
				"ZyA9IGZhbHNlOwoJCXRoaXMuc3RhdHVzID0gMDsKCQlGdW5jdGlvbi5hdHRlbXB0KGZ1bmN0aW9uKCl7",
				"CgkJCXZhciBzdGF0dXMgPSB4aHIuc3RhdHVzOwoJCQl0aGlzLnN0YXR1cyA9IChzdGF0dXMgPT0gMTIy",
				"MykgPyAyMDQgOiBzdGF0dXM7CgkJfS5iaW5kKHRoaXMpKTsKCQl4aHIub25yZWFkeXN0YXRlY2hhbmdl",
				"ID0gZW1wdHk7CgkJaWYgKHByb2dyZXNzU3VwcG9ydCkgeGhyLm9ucHJvZ3Jlc3MgPSB4aHIub25sb2Fk",
				"c3RhcnQgPSBlbXB0eTsKCQljbGVhclRpbWVvdXQodGhpcy50aW1lcik7CgoJCXRoaXMucmVzcG9uc2Ug",
				"PSB7dGV4dDogdGhpcy54aHIucmVzcG9uc2VUZXh0IHx8ICcnLCB4bWw6IHRoaXMueGhyLnJlc3BvbnNl",
				"WE1MfTsKCQlpZiAodGhpcy5vcHRpb25zLmlzU3VjY2Vzcy5jYWxsKHRoaXMsIHRoaXMuc3RhdHVzKSkK",
				"CQkJdGhpcy5zdWNjZXNzKHRoaXMucmVzcG9uc2UudGV4dCwgdGhpcy5yZXNwb25zZS54bWwpOwoJCWVs",
				"c2UKCQkJdGhpcy5mYWlsdXJlKCk7Cgl9LAoKCWlzU3VjY2VzczogZnVuY3Rpb24oKXsKCQl2YXIgc3Rh",
				"dHVzID0gdGhpcy5zdGF0dXM7CgkJcmV0dXJuIChzdGF0dXMgPj0gMjAwICYmIHN0YXR1cyA8IDMwMCk7",
				"Cgl9LAoKCWlzUnVubmluZzogZnVuY3Rpb24oKXsKCQlyZXR1cm4gISF0aGlzLnJ1bm5pbmc7Cgl9LAoK",
				"CXByb2Nlc3NTY3JpcHRzOiBmdW5jdGlvbih0ZXh0KXsKCQlpZiAodGhpcy5vcHRpb25zLmV2YWxSZXNw",
				"b25zZSB8fCAoLyhlY21hfGphdmEpc2NyaXB0LykudGVzdCh0aGlzLmdldEhlYWRlcignQ29udGVudC10",
				"eXBlJykpKSByZXR1cm4gQnJvd3Nlci5leGVjKHRleHQpOwoJCXJldHVybiB0ZXh0LnN0cmlwU2NyaXB0",
				"cyh0aGlzLm9wdGlvbnMuZXZhbFNjcmlwdHMpOwoJfSwKCglzdWNjZXNzOiBmdW5jdGlvbih0ZXh0LCB4",
				"bWwpewoJCXRoaXMub25TdWNjZXNzKHRoaXMucHJvY2Vzc1NjcmlwdHModGV4dCksIHhtbCk7Cgl9LAoK",
				"CW9uU3VjY2VzczogZnVuY3Rpb24oKXsKCQl0aGlzLmZpcmVFdmVudCgnY29tcGxldGUnLCBhcmd1bWVu",
				"dHMpLmZpcmVFdmVudCgnc3VjY2VzcycsIGFyZ3VtZW50cykuY2FsbENoYWluKCk7Cgl9LAoKCWZhaWx1",
				"cmU6IGZ1bmN0aW9uKCl7CgkJdGhpcy5vbkZhaWx1cmUoKTsKCX0sCgoJb25GYWlsdXJlOiBmdW5jdGlv",
				"bigpewoJCXRoaXMuZmlyZUV2ZW50KCdjb21wbGV0ZScpLmZpcmVFdmVudCgnZmFpbHVyZScsIHRoaXMu",
				"eGhyKTsKCX0sCgoJbG9hZHN0YXJ0OiBmdW5jdGlvbihldmVudCl7CgkJdGhpcy5maXJlRXZlbnQoJ2xv",
				"YWRzdGFydCcsIFtldmVudCwgdGhpcy54aHJdKTsKCX0sCgoJcHJvZ3Jlc3M6IGZ1bmN0aW9uKGV2ZW50",
				"KXsKCQl0aGlzLmZpcmVFdmVudCgncHJvZ3Jlc3MnLCBbZXZlbnQsIHRoaXMueGhyXSk7Cgl9LAoKCXRp",
				"bWVvdXQ6IGZ1bmN0aW9uKCl7CgkJdGhpcy5maXJlRXZlbnQoJ3RpbWVvdXQnLCB0aGlzLnhocik7Cgl9",
				"LAoKCXNldEhlYWRlcjogZnVuY3Rpb24obmFtZSwgdmFsdWUpewoJCXRoaXMuaGVhZGVyc1tuYW1lXSA9",
				"IHZhbHVlOwoJCXJldHVybiB0aGlzOwoJfSwKCglnZXRIZWFkZXI6IGZ1bmN0aW9uKG5hbWUpewoJCXJl",
				"dHVybiBGdW5jdGlvbi5hdHRlbXB0KGZ1bmN0aW9uKCl7CgkJCXJldHVybiB0aGlzLnhoci5nZXRSZXNw",
				"b25zZUhlYWRlcihuYW1lKTsKCQl9LmJpbmQodGhpcykpOwoJfSwKCgljaGVjazogZnVuY3Rpb24oKXsK",
				"CQlpZiAoIXRoaXMucnVubmluZykgcmV0dXJuIHRydWU7CgkJc3dpdGNoICh0aGlzLm9wdGlvbnMubGlu",
				"ayl7CgkJCWNhc2UgJ2NhbmNlbCc6IHRoaXMuY2FuY2VsKCk7IHJldHVybiB0cnVlOwoJCQljYXNlICdj",
				"aGFpbic6IHRoaXMuY2hhaW4odGhpcy5jYWxsZXIucGFzcyhhcmd1bWVudHMsIHRoaXMpKTsgcmV0dXJu",
				"IGZhbHNlOwoJCX0KCQlyZXR1cm4gZmFsc2U7Cgl9LAoKCXNlbmQ6IGZ1bmN0aW9uKG9wdGlvbnMpewoJ",
				"CWlmICghdGhpcy5jaGVjayhvcHRpb25zKSkgcmV0dXJuIHRoaXM7CgoJCXRoaXMub3B0aW9ucy5pc1N1",
				"Y2Nlc3MgPSB0aGlzLm9wdGlvbnMuaXNTdWNjZXNzIHx8IHRoaXMuaXNTdWNjZXNzOwoJCXRoaXMucnVu",
				"bmluZyA9IHRydWU7CgoJCXZhciB0eXBlID0gdHlwZU9mKG9wdGlvbnMpOwoJCWlmICh0eXBlID09ICdz",
				"dHJpbmcnIHx8IHR5cGUgPT0gJ2VsZW1lbnQnKSBvcHRpb25zID0ge2RhdGE6IG9wdGlvbnN9OwoKCQl2",
				"YXIgb2xkID0gdGhpcy5vcHRpb25zOwoJCW9wdGlvbnMgPSBPYmplY3QuYXBwZW5kKHtkYXRhOiBvbGQu",
				"ZGF0YSwgdXJsOiBvbGQudXJsLCBtZXRob2Q6IG9sZC5tZXRob2R9LCBvcHRpb25zKTsKCQl2YXIgZGF0",
				"YSA9IG9wdGlvbnMuZGF0YSwgdXJsID0gU3RyaW5nKG9wdGlvbnMudXJsKSwgbWV0aG9kID0gb3B0aW9u",
				"cy5tZXRob2QudG9Mb3dlckNhc2UoKTsKCgkJc3dpdGNoICh0eXBlT2YoZGF0YSkpewoJCQljYXNlICdl",
				"bGVtZW50JzogZGF0YSA9IGRvY3VtZW50LmlkKGRhdGEpLnRvUXVlcnlTdHJpbmcoKTsgYnJlYWs7CgkJ",
				"CWNhc2UgJ29iamVjdCc6IGNhc2UgJ2hhc2gnOiBkYXRhID0gT2JqZWN0LnRvUXVlcnlTdHJpbmcoZGF0",
				"YSk7CgkJfQoKCQlpZiAodGhpcy5vcHRpb25zLmZvcm1hdCl7CgkJCXZhciBmb3JtYXQgPSAnZm9ybWF0",
				"PScgKyB0aGlzLm9wdGlvbnMuZm9ybWF0OwoJCQlkYXRhID0gKGRhdGEpID8gZm9ybWF0ICsgJyYnICsg",
				"ZGF0YSA6IGZvcm1hdDsKCQl9CgoJCWlmICh0aGlzLm9wdGlvbnMuZW11bGF0aW9uICYmICFbJ2dldCcs",
				"ICdwb3N0J10uY29udGFpbnMobWV0aG9kKSl7CgkJCXZhciBfbWV0aG9kID0gJ19tZXRob2Q9JyArIG1l",
				"dGhvZDsKCQkJZGF0YSA9IChkYXRhKSA/IF9tZXRob2QgKyAnJicgKyBkYXRhIDogX21ldGhvZDsKCQkJ",
				"bWV0aG9kID0gJ3Bvc3QnOwoJCX0KCgkJaWYgKHRoaXMub3B0aW9ucy51cmxFbmNvZGVkICYmIFsncG9z",
				"dCcsICdwdXQnXS5jb250YWlucyhtZXRob2QpKXsKCQkJdmFyIGVuY29kaW5nID0gKHRoaXMub3B0aW9u",
				"cy5lbmNvZGluZykgPyAnOyBjaGFyc2V0PScgKyB0aGlzLm9wdGlvbnMuZW5jb2RpbmcgOiAnJzsKCQkJ",
				"dGhpcy5oZWFkZXJzWydDb250ZW50LXR5cGUnXSA9ICdhcHBsaWNhdGlvbi94LXd3dy1mb3JtLXVybGVu",
				"Y29kZWQnICsgZW5jb2Rpbmc7CgkJfQoKCQlpZiAoIXVybCkgdXJsID0gZG9jdW1lbnQubG9jYXRpb24u",
				"cGF0aG5hbWU7CgoJCXZhciB0cmltUG9zaXRpb24gPSB1cmwubGFzdEluZGV4T2YoJy8nKTsKCQlpZiAo",
				"dHJpbVBvc2l0aW9uID4gLTEgJiYgKHRyaW1Qb3NpdGlvbiA9IHVybC5pbmRleE9mKCcjJykpID4gLTEp",
				"IHVybCA9IHVybC5zdWJzdHIoMCwgdHJpbVBvc2l0aW9uKTsKCgkJaWYgKHRoaXMub3B0aW9ucy5ub0Nh",
				"Y2hlKQoJCQl1cmwgKz0gKHVybC5jb250YWlucygnPycpID8gJyYnIDogJz8nKSArIFN0cmluZy51bmlx",
				"dWVJRCgpOwoKCQlpZiAoZGF0YSAmJiBtZXRob2QgPT0gJ2dldCcpewoJCQl1cmwgKz0gKHVybC5jb250",
				"YWlucygnPycpID8gJyYnIDogJz8nKSArIGRhdGE7CgkJCWRhdGEgPSBudWxsOwoJCX0KCgkJdmFyIHho",
				"ciA9IHRoaXMueGhyOwoJCWlmIChwcm9ncmVzc1N1cHBvcnQpewoJCQl4aHIub25sb2Fkc3RhcnQgPSB0",
				"aGlzLmxvYWRzdGFydC5iaW5kKHRoaXMpOwoJCQl4aHIub25wcm9ncmVzcyA9IHRoaXMucHJvZ3Jlc3Mu",
				"YmluZCh0aGlzKTsKCQl9CgoJCXhoci5vcGVuKG1ldGhvZC50b1VwcGVyQ2FzZSgpLCB1cmwsIHRoaXMu",
				"b3B0aW9ucy5hc3luYywgdGhpcy5vcHRpb25zLnVzZXIsIHRoaXMub3B0aW9ucy5wYXNzd29yZCk7CgkJ",
				"aWYgKHRoaXMub3B0aW9ucy51c2VyICYmICd3aXRoQ3JlZGVudGlhbHMnIGluIHhocikgeGhyLndpdGhD",
				"cmVkZW50aWFscyA9IHRydWU7CgoJCXhoci5vbnJlYWR5c3RhdGVjaGFuZ2UgPSB0aGlzLm9uU3RhdGVD",
				"aGFuZ2UuYmluZCh0aGlzKTsKCgkJT2JqZWN0LmVhY2godGhpcy5oZWFkZXJzLCBmdW5jdGlvbih2YWx1",
				"ZSwga2V5KXsKCQkJdHJ5IHsKCQkJCXhoci5zZXRSZXF1ZXN0SGVhZGVyKGtleSwgdmFsdWUpOwoJCQl9",
				"IGNhdGNoIChlKXsKCQkJCXRoaXMuZmlyZUV2ZW50KCdleGNlcHRpb24nLCBba2V5LCB2YWx1ZV0pOwoJ",
				"CQl9CgkJfSwgdGhpcyk7CgoJCXRoaXMuZmlyZUV2ZW50KCdyZXF1ZXN0Jyk7CgkJeGhyLnNlbmQoZGF0",
				"YSk7CgkJaWYgKCF0aGlzLm9wdGlvbnMuYXN5bmMpIHRoaXMub25TdGF0ZUNoYW5nZSgpOwoJCWVsc2Ug",
				"aWYgKHRoaXMub3B0aW9ucy50aW1lb3V0KSB0aGlzLnRpbWVyID0gdGhpcy50aW1lb3V0LmRlbGF5KHRo",
				"aXMub3B0aW9ucy50aW1lb3V0LCB0aGlzKTsKCQlyZXR1cm4gdGhpczsKCX0sCgoJY2FuY2VsOiBmdW5j",
				"dGlvbigpewoJCWlmICghdGhpcy5ydW5uaW5nKSByZXR1cm4gdGhpczsKCQl0aGlzLnJ1bm5pbmcgPSBm",
				"YWxzZTsKCQl2YXIgeGhyID0gdGhpcy54aHI7CgkJeGhyLmFib3J0KCk7CgkJY2xlYXJUaW1lb3V0KHRo",
				"aXMudGltZXIpOwoJCXhoci5vbnJlYWR5c3RhdGVjaGFuZ2UgPSBlbXB0eTsKCQlpZiAocHJvZ3Jlc3NT",
				"dXBwb3J0KSB4aHIub25wcm9ncmVzcyA9IHhoci5vbmxvYWRzdGFydCA9IGVtcHR5OwoJCXRoaXMueGhy",
				"ID0gbmV3IEJyb3dzZXIuUmVxdWVzdCgpOwoJCXRoaXMuZmlyZUV2ZW50KCdjYW5jZWwnKTsKCQlyZXR1",
				"cm4gdGhpczsKCX0KCn0pOwoKdmFyIG1ldGhvZHMgPSB7fTsKWydnZXQnLCAncG9zdCcsICdwdXQnLCAn",
				"ZGVsZXRlJywgJ0dFVCcsICdQT1NUJywgJ1BVVCcsICdERUxFVEUnXS5lYWNoKGZ1bmN0aW9uKG1ldGhv",
				"ZCl7CgltZXRob2RzW21ldGhvZF0gPSBmdW5jdGlvbihkYXRhKXsKCQl2YXIgb2JqZWN0ID0gewoJCQlt",
				"ZXRob2Q6IG1ldGhvZAoJCX07CgkJaWYgKGRhdGEgIT0gbnVsbCkgb2JqZWN0LmRhdGEgPSBkYXRhOwoJ",
				"CXJldHVybiB0aGlzLnNlbmQob2JqZWN0KTsKCX07Cn0pOwoKUmVxdWVzdC5pbXBsZW1lbnQobWV0aG9k",
				"cyk7CgpFbGVtZW50LlByb3BlcnRpZXMuc2VuZCA9IHsKCglzZXQ6IGZ1bmN0aW9uKG9wdGlvbnMpewoJ",
				"CXZhciBzZW5kID0gdGhpcy5nZXQoJ3NlbmQnKS5jYW5jZWwoKTsKCQlzZW5kLnNldE9wdGlvbnMob3B0",
				"aW9ucyk7CgkJcmV0dXJuIHRoaXM7Cgl9LAoKCWdldDogZnVuY3Rpb24oKXsKCQl2YXIgc2VuZCA9IHRo",
				"aXMucmV0cmlldmUoJ3NlbmQnKTsKCQlpZiAoIXNlbmQpewoJCQlzZW5kID0gbmV3IFJlcXVlc3QoewoJ",
				"CQkJZGF0YTogdGhpcywgbGluazogJ2NhbmNlbCcsIG1ldGhvZDogdGhpcy5nZXQoJ21ldGhvZCcpIHx8",
				"ICdwb3N0JywgdXJsOiB0aGlzLmdldCgnYWN0aW9uJykKCQkJfSk7CgkJCXRoaXMuc3RvcmUoJ3NlbmQn",
				"LCBzZW5kKTsKCQl9CgkJcmV0dXJuIHNlbmQ7Cgl9Cgp9OwoKRWxlbWVudC5pbXBsZW1lbnQoewoKCXNl",
				"bmQ6IGZ1bmN0aW9uKHVybCl7CgkJdmFyIHNlbmRlciA9IHRoaXMuZ2V0KCdzZW5kJyk7CgkJc2VuZGVy",
				"LnNlbmQoe2RhdGE6IHRoaXMsIHVybDogdXJsIHx8IHNlbmRlci5vcHRpb25zLnVybH0pOwoJCXJldHVy",
				"biB0aGlzOwoJfQoKfSk7Cgp9KSgpOwoKCi8qCi0tLQoKbmFtZTogUmVxdWVzdC5IVE1MCgpkZXNjcmlw",
				"dGlvbjogRXh0ZW5kcyB0aGUgYmFzaWMgUmVxdWVzdCBDbGFzcyB3aXRoIGFkZGl0aW9uYWwgbWV0aG9k",
				"cyBmb3IgaW50ZXJhY3Rpbmcgd2l0aCBIVE1MIHJlc3BvbnNlcy4KCmxpY2Vuc2U6IE1JVC1zdHlsZSBs",
				"aWNlbnNlLgoKcmVxdWlyZXM6IFtFbGVtZW50LCBSZXF1ZXN0XQoKcHJvdmlkZXM6IFJlcXVlc3QuSFRN",
				"TAoKLi4uCiovCgpSZXF1ZXN0LkhUTUwgPSBuZXcgQ2xhc3MoewoKCUV4dGVuZHM6IFJlcXVlc3QsCgoJ",
				"b3B0aW9uczogewoJCXVwZGF0ZTogZmFsc2UsCgkJYXBwZW5kOiBmYWxzZSwKCQlldmFsU2NyaXB0czog",
				"dHJ1ZSwKCQlmaWx0ZXI6IGZhbHNlLAoJCWhlYWRlcnM6IHsKCQkJQWNjZXB0OiAndGV4dC9odG1sLCBh",
				"cHBsaWNhdGlvbi94bWwsIHRleHQveG1sLCAqLyonCgkJfQoJfSwKCglzdWNjZXNzOiBmdW5jdGlvbih0",
				"ZXh0KXsKCQl2YXIgb3B0aW9ucyA9IHRoaXMub3B0aW9ucywgcmVzcG9uc2UgPSB0aGlzLnJlc3BvbnNl",
				"OwoKCQlyZXNwb25zZS5odG1sID0gdGV4dC5zdHJpcFNjcmlwdHMoZnVuY3Rpb24oc2NyaXB0KXsKCQkJ",
				"cmVzcG9uc2UuamF2YXNjcmlwdCA9IHNjcmlwdDsKCQl9KTsKCgkJdmFyIG1hdGNoID0gcmVzcG9uc2Uu",
				"aHRtbC5tYXRjaCgvPGJvZHlbXj5dKj4oW1xzXFNdKj8pPFwvYm9keT4vaSk7CgkJaWYgKG1hdGNoKSBy",
				"ZXNwb25zZS5odG1sID0gbWF0Y2hbMV07CgkJdmFyIHRlbXAgPSBuZXcgRWxlbWVudCgnZGl2Jykuc2V0",
				"KCdodG1sJywgcmVzcG9uc2UuaHRtbCk7CgoJCXJlc3BvbnNlLnRyZWUgPSB0ZW1wLmNoaWxkTm9kZXM7",
				"CgkJcmVzcG9uc2UuZWxlbWVudHMgPSB0ZW1wLmdldEVsZW1lbnRzKG9wdGlvbnMuZmlsdGVyIHx8ICcq",
				"Jyk7CgoJCWlmIChvcHRpb25zLmZpbHRlcikgcmVzcG9uc2UudHJlZSA9IHJlc3BvbnNlLmVsZW1lbnRz",
				"OwoJCWlmIChvcHRpb25zLnVwZGF0ZSl7CgkJCXZhciB1cGRhdGUgPSBkb2N1bWVudC5pZChvcHRpb25z",
				"LnVwZGF0ZSkuZW1wdHkoKTsKCQkJaWYgKG9wdGlvbnMuZmlsdGVyKSB1cGRhdGUuYWRvcHQocmVzcG9u",
				"c2UuZWxlbWVudHMpOwoJCQllbHNlIHVwZGF0ZS5zZXQoJ2h0bWwnLCByZXNwb25zZS5odG1sKTsKCQl9",
				"IGVsc2UgaWYgKG9wdGlvbnMuYXBwZW5kKXsKCQkJdmFyIGFwcGVuZCA9IGRvY3VtZW50LmlkKG9wdGlv",
				"bnMuYXBwZW5kKTsKCQkJaWYgKG9wdGlvbnMuZmlsdGVyKSByZXNwb25zZS5lbGVtZW50cy5yZXZlcnNl",
				"KCkuaW5qZWN0KGFwcGVuZCk7CgkJCWVsc2UgYXBwZW5kLmFkb3B0KHRlbXAuZ2V0Q2hpbGRyZW4oKSk7",
				"CgkJfQoJCWlmIChvcHRpb25zLmV2YWxTY3JpcHRzKSBCcm93c2VyLmV4ZWMocmVzcG9uc2UuamF2YXNj",
				"cmlwdCk7CgoJCXRoaXMub25TdWNjZXNzKHJlc3BvbnNlLnRyZWUsIHJlc3BvbnNlLmVsZW1lbnRzLCBy",
				"ZXNwb25zZS5odG1sLCByZXNwb25zZS5qYXZhc2NyaXB0KTsKCX0KCn0pOwoKRWxlbWVudC5Qcm9wZXJ0",
				"aWVzLmxvYWQgPSB7CgoJc2V0OiBmdW5jdGlvbihvcHRpb25zKXsKCQl2YXIgbG9hZCA9IHRoaXMuZ2V0",
				"KCdsb2FkJykuY2FuY2VsKCk7CgkJbG9hZC5zZXRPcHRpb25zKG9wdGlvbnMpOwoJCXJldHVybiB0aGlz",
				"OwoJfSwKCglnZXQ6IGZ1bmN0aW9uKCl7CgkJdmFyIGxvYWQgPSB0aGlzLnJldHJpZXZlKCdsb2FkJyk7",
				"CgkJaWYgKCFsb2FkKXsKCQkJbG9hZCA9IG5ldyBSZXF1ZXN0LkhUTUwoe2RhdGE6IHRoaXMsIGxpbms6",
				"ICdjYW5jZWwnLCB1cGRhdGU6IHRoaXMsIG1ldGhvZDogJ2dldCd9KTsKCQkJdGhpcy5zdG9yZSgnbG9h",
				"ZCcsIGxvYWQpOwoJCX0KCQlyZXR1cm4gbG9hZDsKCX0KCn07CgpFbGVtZW50LmltcGxlbWVudCh7CgoJ",
				"bG9hZDogZnVuY3Rpb24oKXsKCQl0aGlzLmdldCgnbG9hZCcpLnNlbmQoQXJyYXkubGluayhhcmd1bWVu",
				"dHMsIHtkYXRhOiBUeXBlLmlzT2JqZWN0LCB1cmw6IFR5cGUuaXNTdHJpbmd9KSk7CgkJcmV0dXJuIHRo",
				"aXM7Cgl9Cgp9KTsKCgovKgotLS0KCm5hbWU6IEpTT04KCmRlc2NyaXB0aW9uOiBKU09OIGVuY29kZXIg",
				"YW5kIGRlY29kZXIuCgpsaWNlbnNlOiBNSVQtc3R5bGUgbGljZW5zZS4KClNlZUFsc286IDxodHRwOi8v",
				"d3d3Lmpzb24ub3JnLz4KCnJlcXVpcmVzOiBbQXJyYXksIFN0cmluZywgTnVtYmVyLCBGdW5jdGlvbl0K",
				"CnByb3ZpZGVzOiBKU09OCgouLi4KKi8KCmlmICh0eXBlb2YgSlNPTiA9PSAndW5kZWZpbmVkJykgdGhp",
				"cy5KU09OID0ge307CgovLzwxLjJjb21wYXQ+CgpKU09OID0gbmV3IEhhc2goewoJc3RyaW5naWZ5OiBK",
				"U09OLnN0cmluZ2lmeSwKCXBhcnNlOiBKU09OLnBhcnNlCn0pOwoKLy88LzEuMmNvbXBhdD4KCihmdW5j",
				"dGlvbigpewoKdmFyIHNwZWNpYWwgPSB7J1xiJzogJ1xcYicsICdcdCc6ICdcXHQnLCAnXG4nOiAnXFxu",
				"JywgJ1xmJzogJ1xcZicsICdccic6ICdcXHInLCAnIicgOiAnXFwiJywgJ1xcJzogJ1xcXFwnfTsKCnZh",
				"ciBlc2NhcGUgPSBmdW5jdGlvbihjaHIpewoJcmV0dXJuIHNwZWNpYWxbY2hyXSB8fCAnXFx1JyArICgn",
				"MDAwMCcgKyBjaHIuY2hhckNvZGVBdCgwKS50b1N0cmluZygxNikpLnNsaWNlKC00KTsKfTsKCkpTT04u",
				"dmFsaWRhdGUgPSBmdW5jdGlvbihzdHJpbmcpewoJc3RyaW5nID0gc3RyaW5nLnJlcGxhY2UoL1xcKD86",
				"WyJcXFwvYmZucnRdfHVbMC05YS1mQS1GXXs0fSkvZywgJ0AnKS4KCQkJCQlyZXBsYWNlKC8iW14iXFxc",
				"blxyXSoifHRydWV8ZmFsc2V8bnVsbHwtP1xkKyg/OlwuXGQqKT8oPzpbZUVdWytcLV0/XGQrKT8vZywg",
				"J10nKS4KCQkJCQlyZXBsYWNlKC8oPzpefDp8LCkoPzpccypcWykrL2csICcnKTsKCglyZXR1cm4gKC9e",
				"W1xdLDp7fVxzXSokLykudGVzdChzdHJpbmcpOwp9OwoKSlNPTi5lbmNvZGUgPSBKU09OLnN0cmluZ2lm",
				"eSA/IGZ1bmN0aW9uKG9iail7CglyZXR1cm4gSlNPTi5zdHJpbmdpZnkob2JqKTsKfSA6IGZ1bmN0aW9u",
				"KG9iail7CglpZiAob2JqICYmIG9iai50b0pTT04pIG9iaiA9IG9iai50b0pTT04oKTsKCglzd2l0Y2gg",
				"KHR5cGVPZihvYmopKXsKCQljYXNlICdzdHJpbmcnOgoJCQlyZXR1cm4gJyInICsgb2JqLnJlcGxhY2Uo",
				"L1tceDAwLVx4MWZcXCJdL2csIGVzY2FwZSkgKyAnIic7CgkJY2FzZSAnYXJyYXknOgoJCQlyZXR1cm4g",
				"J1snICsgb2JqLm1hcChKU09OLmVuY29kZSkuY2xlYW4oKSArICddJzsKCQljYXNlICdvYmplY3QnOiBj",
				"YXNlICdoYXNoJzoKCQkJdmFyIHN0cmluZyA9IFtdOwoJCQlPYmplY3QuZWFjaChvYmosIGZ1bmN0aW9u",
				"KHZhbHVlLCBrZXkpewoJCQkJdmFyIGpzb24gPSBKU09OLmVuY29kZSh2YWx1ZSk7CgkJCQlpZiAoanNv",
				"bikgc3RyaW5nLnB1c2goSlNPTi5lbmNvZGUoa2V5KSArICc6JyArIGpzb24pOwoJCQl9KTsKCQkJcmV0",
				"dXJuICd7JyArIHN0cmluZyArICd9JzsKCQljYXNlICdudW1iZXInOiBjYXNlICdib29sZWFuJzogcmV0",
				"dXJuICcnICsgb2JqOwoJCWNhc2UgJ251bGwnOiByZXR1cm4gJ251bGwnOwoJfQoKCXJldHVybiBudWxs",
				"Owp9OwoKSlNPTi5kZWNvZGUgPSBmdW5jdGlvbihzdHJpbmcsIHNlY3VyZSl7CglpZiAoIXN0cmluZyB8",
				"fCB0eXBlT2Yoc3RyaW5nKSAhPSAnc3RyaW5nJykgcmV0dXJuIG51bGw7CgoJaWYgKHNlY3VyZSB8fCBK",
				"U09OLnNlY3VyZSl7CgkJaWYgKEpTT04ucGFyc2UpIHJldHVybiBKU09OLnBhcnNlKHN0cmluZyk7CgkJ",
				"aWYgKCFKU09OLnZhbGlkYXRlKHN0cmluZykpIHRocm93IG5ldyBFcnJvcignSlNPTiBjb3VsZCBub3Qg",
				"ZGVjb2RlIHRoZSBpbnB1dDsgc2VjdXJpdHkgaXMgZW5hYmxlZCBhbmQgdGhlIHZhbHVlIGlzIG5vdCBz",
				"ZWN1cmUuJyk7Cgl9CgoJcmV0dXJuIGV2YWwoJygnICsgc3RyaW5nICsgJyknKTsKfTsKCn0pKCk7CgoK",
				"LyoKLS0tCgpuYW1lOiBSZXF1ZXN0LkpTT04KCmRlc2NyaXB0aW9uOiBFeHRlbmRzIHRoZSBiYXNpYyBS",
				"ZXF1ZXN0IENsYXNzIHdpdGggYWRkaXRpb25hbCBtZXRob2RzIGZvciBzZW5kaW5nIGFuZCByZWNlaXZp",
				"bmcgSlNPTiBkYXRhLgoKbGljZW5zZTogTUlULXN0eWxlIGxpY2Vuc2UuCgpyZXF1aXJlczogW1JlcXVl",
				"c3QsIEpTT05dCgpwcm92aWRlczogUmVxdWVzdC5KU09OCgouLi4KKi8KClJlcXVlc3QuSlNPTiA9IG5l",
				"dyBDbGFzcyh7CgoJRXh0ZW5kczogUmVxdWVzdCwKCglvcHRpb25zOiB7CgkJLypvbkVycm9yOiBmdW5j",
				"dGlvbih0ZXh0LCBlcnJvcil7fSwqLwoJCXNlY3VyZTogdHJ1ZQoJfSwKCglpbml0aWFsaXplOiBmdW5j",
				"dGlvbihvcHRpb25zKXsKCQl0aGlzLnBhcmVudChvcHRpb25zKTsKCQlPYmplY3QuYXBwZW5kKHRoaXMu",
				"aGVhZGVycywgewoJCQknQWNjZXB0JzogJ2FwcGxpY2F0aW9uL2pzb24nLAoJCQknWC1SZXF1ZXN0Jzog",
				"J0pTT04nCgkJfSk7Cgl9LAoKCXN1Y2Nlc3M6IGZ1bmN0aW9uKHRleHQpewoJCXZhciBqc29uOwoJCXRy",
				"eSB7CgkJCWpzb24gPSB0aGlzLnJlc3BvbnNlLmpzb24gPSBKU09OLmRlY29kZSh0ZXh0LCB0aGlzLm9w",
				"dGlvbnMuc2VjdXJlKTsKCQl9IGNhdGNoIChlcnJvcil7CgkJCXRoaXMuZmlyZUV2ZW50KCdlcnJvcics",
				"IFt0ZXh0LCBlcnJvcl0pOwoJCQlyZXR1cm47CgkJfQoJCWlmIChqc29uID09IG51bGwpIHRoaXMub25G",
				"YWlsdXJlKCk7CgkJZWxzZSB0aGlzLm9uU3VjY2Vzcyhqc29uLCB0ZXh0KTsKCX0KCn0pOwoKCi8qCi0t",
				"LQoKbmFtZTogQ29va2llCgpkZXNjcmlwdGlvbjogQ2xhc3MgZm9yIGNyZWF0aW5nLCByZWFkaW5nLCBh",
				"bmQgZGVsZXRpbmcgYnJvd3NlciBDb29raWVzLgoKbGljZW5zZTogTUlULXN0eWxlIGxpY2Vuc2UuCgpj",
				"cmVkaXRzOgogIC0gQmFzZWQgb24gdGhlIGZ1bmN0aW9ucyBieSBQZXRlci1QYXVsIEtvY2ggKGh0dHA6",
				"Ly9xdWlya3Ntb2RlLm9yZykuCgpyZXF1aXJlczogW09wdGlvbnMsIEJyb3dzZXJdCgpwcm92aWRlczog",
				"Q29va2llCgouLi4KKi8KCnZhciBDb29raWUgPSBuZXcgQ2xhc3MoewoKCUltcGxlbWVudHM6IE9wdGlv",
				"bnMsCgoJb3B0aW9uczogewoJCXBhdGg6ICcvJywKCQlkb21haW46IGZhbHNlLAoJCWR1cmF0aW9uOiBm",
				"YWxzZSwKCQlzZWN1cmU6IGZhbHNlLAoJCWRvY3VtZW50OiBkb2N1bWVudCwKCQllbmNvZGU6IHRydWUK",
				"CX0sCgoJaW5pdGlhbGl6ZTogZnVuY3Rpb24oa2V5LCBvcHRpb25zKXsKCQl0aGlzLmtleSA9IGtleTsK",
				"CQl0aGlzLnNldE9wdGlvbnMob3B0aW9ucyk7Cgl9LAoKCXdyaXRlOiBmdW5jdGlvbih2YWx1ZSl7CgkJ",
				"aWYgKHRoaXMub3B0aW9ucy5lbmNvZGUpIHZhbHVlID0gZW5jb2RlVVJJQ29tcG9uZW50KHZhbHVlKTsK",
				"CQlpZiAodGhpcy5vcHRpb25zLmRvbWFpbikgdmFsdWUgKz0gJzsgZG9tYWluPScgKyB0aGlzLm9wdGlv",
				"bnMuZG9tYWluOwoJCWlmICh0aGlzLm9wdGlvbnMucGF0aCkgdmFsdWUgKz0gJzsgcGF0aD0nICsgdGhp",
				"cy5vcHRpb25zLnBhdGg7CgkJaWYgKHRoaXMub3B0aW9ucy5kdXJhdGlvbil7CgkJCXZhciBkYXRlID0g",
				"bmV3IERhdGUoKTsKCQkJZGF0ZS5zZXRUaW1lKGRhdGUuZ2V0VGltZSgpICsgdGhpcy5vcHRpb25zLmR1",
				"cmF0aW9uICogMjQgKiA2MCAqIDYwICogMTAwMCk7CgkJCXZhbHVlICs9ICc7IGV4cGlyZXM9JyArIGRh",
				"dGUudG9HTVRTdHJpbmcoKTsKCQl9CgkJaWYgKHRoaXMub3B0aW9ucy5zZWN1cmUpIHZhbHVlICs9ICc7",
				"IHNlY3VyZSc7CgkJdGhpcy5vcHRpb25zLmRvY3VtZW50LmNvb2tpZSA9IHRoaXMua2V5ICsgJz0nICsg",
				"dmFsdWU7CgkJcmV0dXJuIHRoaXM7Cgl9LAoKCXJlYWQ6IGZ1bmN0aW9uKCl7CgkJdmFyIHZhbHVlID0g",
				"dGhpcy5vcHRpb25zLmRvY3VtZW50LmNvb2tpZS5tYXRjaCgnKD86Xnw7KVxccyonICsgdGhpcy5rZXku",
				"ZXNjYXBlUmVnRXhwKCkgKyAnPShbXjtdKiknKTsKCQlyZXR1cm4gKHZhbHVlKSA/IGRlY29kZVVSSUNv",
				"bXBvbmVudCh2YWx1ZVsxXSkgOiBudWxsOwoJfSwKCglkaXNwb3NlOiBmdW5jdGlvbigpewoJCW5ldyBD",
				"b29raWUodGhpcy5rZXksIE9iamVjdC5tZXJnZSh7fSwgdGhpcy5vcHRpb25zLCB7ZHVyYXRpb246IC0x",
				"fSkpLndyaXRlKCcnKTsKCQlyZXR1cm4gdGhpczsKCX0KCn0pOwoKQ29va2llLndyaXRlID0gZnVuY3Rp",
				"b24oa2V5LCB2YWx1ZSwgb3B0aW9ucyl7CglyZXR1cm4gbmV3IENvb2tpZShrZXksIG9wdGlvbnMpLndy",
				"aXRlKHZhbHVlKTsKfTsKCkNvb2tpZS5yZWFkID0gZnVuY3Rpb24oa2V5KXsKCXJldHVybiBuZXcgQ29v",
				"a2llKGtleSkucmVhZCgpOwp9OwoKQ29va2llLmRpc3Bvc2UgPSBmdW5jdGlvbihrZXksIG9wdGlvbnMp",
				"ewoJcmV0dXJuIG5ldyBDb29raWUoa2V5LCBvcHRpb25zKS5kaXNwb3NlKCk7Cn07CgoKLyoKLS0tCgpu",
				"YW1lOiBET01SZWFkeQoKZGVzY3JpcHRpb246IENvbnRhaW5zIHRoZSBjdXN0b20gZXZlbnQgZG9tcmVh",
				"ZHkuCgpsaWNlbnNlOiBNSVQtc3R5bGUgbGljZW5zZS4KCnJlcXVpcmVzOiBbQnJvd3NlciwgRWxlbWVu",
				"dCwgRWxlbWVudC5FdmVudF0KCnByb3ZpZGVzOiBbRE9NUmVhZHksIERvbVJlYWR5XQoKLi4uCiovCgoo",
				"ZnVuY3Rpb24od2luZG93LCBkb2N1bWVudCl7Cgp2YXIgcmVhZHksCglsb2FkZWQsCgljaGVja3MgPSBb",
				"XSwKCXNob3VsZFBvbGwsCgl0aW1lciwKCXRlc3RFbGVtZW50ID0gZG9jdW1lbnQuY3JlYXRlRWxlbWVu",
				"dCgnZGl2Jyk7Cgp2YXIgZG9tcmVhZHkgPSBmdW5jdGlvbigpewoJY2xlYXJUaW1lb3V0KHRpbWVyKTsK",
				"CWlmIChyZWFkeSkgcmV0dXJuOwoJQnJvd3Nlci5sb2FkZWQgPSByZWFkeSA9IHRydWU7Cglkb2N1bWVu",
				"dC5yZW1vdmVMaXN0ZW5lcignRE9NQ29udGVudExvYWRlZCcsIGRvbXJlYWR5KS5yZW1vdmVMaXN0ZW5l",
				"cigncmVhZHlzdGF0ZWNoYW5nZScsIGNoZWNrKTsKCglkb2N1bWVudC5maXJlRXZlbnQoJ2RvbXJlYWR5",
				"Jyk7Cgl3aW5kb3cuZmlyZUV2ZW50KCdkb21yZWFkeScpOwp9OwoKdmFyIGNoZWNrID0gZnVuY3Rpb24o",
				"KXsKCWZvciAodmFyIGkgPSBjaGVja3MubGVuZ3RoOyBpLS07KSBpZiAoY2hlY2tzW2ldKCkpewoJCWRv",
				"bXJlYWR5KCk7CgkJcmV0dXJuIHRydWU7Cgl9CglyZXR1cm4gZmFsc2U7Cn07Cgp2YXIgcG9sbCA9IGZ1",
				"bmN0aW9uKCl7CgljbGVhclRpbWVvdXQodGltZXIpOwoJaWYgKCFjaGVjaygpKSB0aW1lciA9IHNldFRp",
				"bWVvdXQocG9sbCwgMTApOwp9OwoKZG9jdW1lbnQuYWRkTGlzdGVuZXIoJ0RPTUNvbnRlbnRMb2FkZWQn",
				"LCBkb21yZWFkeSk7CgovKjxsdElFOD4qLwovLyBkb1Njcm9sbCB0ZWNobmlxdWUgYnkgRGllZ28gUGVy",
				"aW5pIGh0dHA6Ly9qYXZhc2NyaXB0Lm53Ym94LmNvbS9JRUNvbnRlbnRMb2FkZWQvCi8vIHRlc3RFbGVt",
				"ZW50LmRvU2Nyb2xsKCkgdGhyb3dzIHdoZW4gdGhlIERPTSBpcyBub3QgcmVhZHksIG9ubHkgaW4gdGhl",
				"IHRvcCB3aW5kb3cKdmFyIGRvU2Nyb2xsV29ya3MgPSBmdW5jdGlvbigpewoJdHJ5IHsKCQl0ZXN0RWxl",
				"bWVudC5kb1Njcm9sbCgpOwoJCXJldHVybiB0cnVlOwoJfSBjYXRjaCAoZSl7fQoJcmV0dXJuIGZhbHNl",
				"Owp9OwovLyBJZiBkb1Njcm9sbCB3b3JrcyBhbHJlYWR5LCBpdCBjYW4ndCBiZSB1c2VkIHRvIGRldGVy",
				"bWluZSBkb21yZWFkeQovLyAgIGUuZy4gaW4gYW4gaWZyYW1lCmlmICh0ZXN0RWxlbWVudC5kb1Njcm9s",
				"bCAmJiAhZG9TY3JvbGxXb3JrcygpKXsKCWNoZWNrcy5wdXNoKGRvU2Nyb2xsV29ya3MpOwoJc2hvdWxk",
				"UG9sbCA9IHRydWU7Cn0KLyo8L2x0SUU4PiovCgppZiAoZG9jdW1lbnQucmVhZHlTdGF0ZSkgY2hlY2tz",
				"LnB1c2goZnVuY3Rpb24oKXsKCXZhciBzdGF0ZSA9IGRvY3VtZW50LnJlYWR5U3RhdGU7CglyZXR1cm4g",
				"KHN0YXRlID09ICdsb2FkZWQnIHx8IHN0YXRlID09ICdjb21wbGV0ZScpOwp9KTsKCmlmICgnb25yZWFk",
				"eXN0YXRlY2hhbmdlJyBpbiBkb2N1bWVudCkgZG9jdW1lbnQuYWRkTGlzdGVuZXIoJ3JlYWR5c3RhdGVj",
				"aGFuZ2UnLCBjaGVjayk7CmVsc2Ugc2hvdWxkUG9sbCA9IHRydWU7CgppZiAoc2hvdWxkUG9sbCkgcG9s",
				"bCgpOwoKRWxlbWVudC5FdmVudHMuZG9tcmVhZHkgPSB7CglvbkFkZDogZnVuY3Rpb24oZm4pewoJCWlm",
				"IChyZWFkeSkgZm4uY2FsbCh0aGlzKTsKCX0KfTsKCi8vIE1ha2Ugc3VyZSB0aGF0IGRvbXJlYWR5IGZp",
				"cmVzIGJlZm9yZSBsb2FkCkVsZW1lbnQuRXZlbnRzLmxvYWQgPSB7CgliYXNlOiAnbG9hZCcsCglvbkFk",
				"ZDogZnVuY3Rpb24oZm4pewoJCWlmIChsb2FkZWQgJiYgdGhpcyA9PSB3aW5kb3cpIGZuLmNhbGwodGhp",
				"cyk7Cgl9LAoJY29uZGl0aW9uOiBmdW5jdGlvbigpewoJCWlmICh0aGlzID09IHdpbmRvdyl7CgkJCWRv",
				"bXJlYWR5KCk7CgkJCWRlbGV0ZSBFbGVtZW50LkV2ZW50cy5sb2FkOwoJCX0KCQlyZXR1cm4gdHJ1ZTsK",
				"CX0KfTsKCi8vIFRoaXMgaXMgYmFzZWQgb24gdGhlIGN1c3RvbSBsb2FkIGV2ZW50CndpbmRvdy5hZGRF",
				"dmVudCgnbG9hZCcsIGZ1bmN0aW9uKCl7Cglsb2FkZWQgPSB0cnVlOwp9KTsKCn0pKHdpbmRvdywgZG9j",
				"dW1lbnQpOwoKCi8qCi0tLQoKbmFtZTogU3dpZmYKCmRlc2NyaXB0aW9uOiBXcmFwcGVyIGZvciBlbWJl",
				"ZGRpbmcgU1dGIG1vdmllcy4gU3VwcG9ydHMgRXh0ZXJuYWwgSW50ZXJmYWNlIENvbW11bmljYXRpb24u",
				"CgpsaWNlbnNlOiBNSVQtc3R5bGUgbGljZW5zZS4KCmNyZWRpdHM6CiAgLSBGbGFzaCBkZXRlY3Rpb24g",
				"JiBJbnRlcm5ldCBFeHBsb3JlciArIEZsYXNoIFBsYXllciA5IGZpeCBpbnNwaXJlZCBieSBTV0ZPYmpl",
				"Y3QuCgpyZXF1aXJlczogW09wdGlvbnMsIE9iamVjdCwgRWxlbWVudF0KCnByb3ZpZGVzOiBTd2lmZgoK",
				"Li4uCiovCgooZnVuY3Rpb24oKXsKCnZhciBTd2lmZiA9IHRoaXMuU3dpZmYgPSBuZXcgQ2xhc3MoewoK",
				"CUltcGxlbWVudHM6IE9wdGlvbnMsCgoJb3B0aW9uczogewoJCWlkOiBudWxsLAoJCWhlaWdodDogMSwK",
				"CQl3aWR0aDogMSwKCQljb250YWluZXI6IG51bGwsCgkJcHJvcGVydGllczoge30sCgkJcGFyYW1zOiB7",
				"CgkJCXF1YWxpdHk6ICdoaWdoJywKCQkJYWxsb3dTY3JpcHRBY2Nlc3M6ICdhbHdheXMnLAoJCQl3TW9k",
				"ZTogJ3dpbmRvdycsCgkJCXN3TGl2ZUNvbm5lY3Q6IHRydWUKCQl9LAoJCWNhbGxCYWNrczoge30sCgkJ",
				"dmFyczoge30KCX0sCgoJdG9FbGVtZW50OiBmdW5jdGlvbigpewoJCXJldHVybiB0aGlzLm9iamVjdDsK",
				"CX0sCgoJaW5pdGlhbGl6ZTogZnVuY3Rpb24ocGF0aCwgb3B0aW9ucyl7CgkJdGhpcy5pbnN0YW5jZSA9",
				"ICdTd2lmZl8nICsgU3RyaW5nLnVuaXF1ZUlEKCk7CgoJCXRoaXMuc2V0T3B0aW9ucyhvcHRpb25zKTsK",
				"CQlvcHRpb25zID0gdGhpcy5vcHRpb25zOwoJCXZhciBpZCA9IHRoaXMuaWQgPSBvcHRpb25zLmlkIHx8",
				"IHRoaXMuaW5zdGFuY2U7CgkJdmFyIGNvbnRhaW5lciA9IGRvY3VtZW50LmlkKG9wdGlvbnMuY29udGFp",
				"bmVyKTsKCgkJU3dpZmYuQ2FsbEJhY2tzW3RoaXMuaW5zdGFuY2VdID0ge307CgoJCXZhciBwYXJhbXMg",
				"PSBvcHRpb25zLnBhcmFtcywgdmFycyA9IG9wdGlvbnMudmFycywgY2FsbEJhY2tzID0gb3B0aW9ucy5j",
				"YWxsQmFja3M7CgkJdmFyIHByb3BlcnRpZXMgPSBPYmplY3QuYXBwZW5kKHtoZWlnaHQ6IG9wdGlvbnMu",
				"aGVpZ2h0LCB3aWR0aDogb3B0aW9ucy53aWR0aH0sIG9wdGlvbnMucHJvcGVydGllcyk7CgoJCXZhciBz",
				"ZWxmID0gdGhpczsKCgkJZm9yICh2YXIgY2FsbEJhY2sgaW4gY2FsbEJhY2tzKXsKCQkJU3dpZmYuQ2Fs",
				"bEJhY2tzW3RoaXMuaW5zdGFuY2VdW2NhbGxCYWNrXSA9IChmdW5jdGlvbihvcHRpb24pewoJCQkJcmV0",
				"dXJuIGZ1bmN0aW9uKCl7CgkJCQkJcmV0dXJuIG9wdGlvbi5hcHBseShzZWxmLm9iamVjdCwgYXJndW1l",
				"bnRzKTsKCQkJCX07CgkJCX0pKGNhbGxCYWNrc1tjYWxsQmFja10pOwoJCQl2YXJzW2NhbGxCYWNrXSA9",
				"ICdTd2lmZi5DYWxsQmFja3MuJyArIHRoaXMuaW5zdGFuY2UgKyAnLicgKyBjYWxsQmFjazsKCQl9CgoJ",
				"CXBhcmFtcy5mbGFzaFZhcnMgPSBPYmplY3QudG9RdWVyeVN0cmluZyh2YXJzKTsKCQlpZiAoQnJvd3Nl",
				"ci5pZSl7CgkJCXByb3BlcnRpZXMuY2xhc3NpZCA9ICdjbHNpZDpEMjdDREI2RS1BRTZELTExY2YtOTZC",
				"OC00NDQ1NTM1NDAwMDAnOwoJCQlwYXJhbXMubW92aWUgPSBwYXRoOwoJCX0gZWxzZSB7CgkJCXByb3Bl",
				"cnRpZXMudHlwZSA9ICdhcHBsaWNhdGlvbi94LXNob2Nrd2F2ZS1mbGFzaCc7CgkJfQoJCXByb3BlcnRp",
				"ZXMuZGF0YSA9IHBhdGg7CgoJCXZhciBidWlsZCA9ICc8b2JqZWN0IGlkPSInICsgaWQgKyAnIic7CgkJ",
				"Zm9yICh2YXIgcHJvcGVydHkgaW4gcHJvcGVydGllcykgYnVpbGQgKz0gJyAnICsgcHJvcGVydHkgKyAn",
				"PSInICsgcHJvcGVydGllc1twcm9wZXJ0eV0gKyAnIic7CgkJYnVpbGQgKz0gJz4nOwoJCWZvciAodmFy",
				"IHBhcmFtIGluIHBhcmFtcyl7CgkJCWlmIChwYXJhbXNbcGFyYW1dKSBidWlsZCArPSAnPHBhcmFtIG5h",
				"bWU9IicgKyBwYXJhbSArICciIHZhbHVlPSInICsgcGFyYW1zW3BhcmFtXSArICciIC8+JzsKCQl9CgkJ",
				"YnVpbGQgKz0gJzwvb2JqZWN0Pic7CgkJdGhpcy5vYmplY3QgPSAoKGNvbnRhaW5lcikgPyBjb250YWlu",
				"ZXIuZW1wdHkoKSA6IG5ldyBFbGVtZW50KCdkaXYnKSkuc2V0KCdodG1sJywgYnVpbGQpLmZpcnN0Q2hp",
				"bGQ7Cgl9LAoKCXJlcGxhY2VzOiBmdW5jdGlvbihlbGVtZW50KXsKCQllbGVtZW50ID0gZG9jdW1lbnQu",
				"aWQoZWxlbWVudCwgdHJ1ZSk7CgkJZWxlbWVudC5wYXJlbnROb2RlLnJlcGxhY2VDaGlsZCh0aGlzLnRv",
				"RWxlbWVudCgpLCBlbGVtZW50KTsKCQlyZXR1cm4gdGhpczsKCX0sCgoJaW5qZWN0OiBmdW5jdGlvbihl",
				"bGVtZW50KXsKCQlkb2N1bWVudC5pZChlbGVtZW50LCB0cnVlKS5hcHBlbmRDaGlsZCh0aGlzLnRvRWxl",
				"bWVudCgpKTsKCQlyZXR1cm4gdGhpczsKCX0sCgoJcmVtb3RlOiBmdW5jdGlvbigpewoJCXJldHVybiBT",
				"d2lmZi5yZW1vdGUuYXBwbHkoU3dpZmYsIFt0aGlzLnRvRWxlbWVudCgpXS5hcHBlbmQoYXJndW1lbnRz",
				"KSk7Cgl9Cgp9KTsKClN3aWZmLkNhbGxCYWNrcyA9IHt9OwoKU3dpZmYucmVtb3RlID0gZnVuY3Rpb24o",
				"b2JqLCBmbil7Cgl2YXIgcnMgPSBvYmouQ2FsbEZ1bmN0aW9uKCc8aW52b2tlIG5hbWU9IicgKyBmbiAr",
				"ICciIHJldHVybnR5cGU9ImphdmFzY3JpcHQiPicgKyBfX2ZsYXNoX19hcmd1bWVudHNUb1hNTChhcmd1",
				"bWVudHMsIDIpICsgJzwvaW52b2tlPicpOwoJcmV0dXJuIGV2YWwocnMpOwp9OwoKfSkoKTsKCg=="
				].join('');

				buff = new Buffer(moootools_base64_blob, 'base64').toString('ascii');
				fs.writeFileSync(save_file, buff);
				break;

			case "jquery":
				save_file = 'static/js/jquery-1.7.2.min.js';
				
				var jquery_base64_blob = [
				"LyohIGpRdWVyeSB2MS43LjIganF1ZXJ5LmNvbSB8IGpxdWVyeS5vcmcvbGljZW5zZSAqLwooZnVuY3Rp",
				"b24oYSxiKXtmdW5jdGlvbiBjeShhKXtyZXR1cm4gZi5pc1dpbmRvdyhhKT9hOmEubm9kZVR5cGU9PT05",
				"P2EuZGVmYXVsdFZpZXd8fGEucGFyZW50V2luZG93OiExfWZ1bmN0aW9uIGN1KGEpe2lmKCFjalthXSl7",
				"dmFyIGI9Yy5ib2R5LGQ9ZigiPCIrYSsiPiIpLmFwcGVuZFRvKGIpLGU9ZC5jc3MoImRpc3BsYXkiKTtk",
				"LnJlbW92ZSgpO2lmKGU9PT0ibm9uZSJ8fGU9PT0iIil7Y2t8fChjaz1jLmNyZWF0ZUVsZW1lbnQoImlm",
				"cmFtZSIpLGNrLmZyYW1lQm9yZGVyPWNrLndpZHRoPWNrLmhlaWdodD0wKSxiLmFwcGVuZENoaWxkKGNr",
				"KTtpZighY2x8fCFjay5jcmVhdGVFbGVtZW50KWNsPShjay5jb250ZW50V2luZG93fHxjay5jb250ZW50",
				"RG9jdW1lbnQpLmRvY3VtZW50LGNsLndyaXRlKChmLnN1cHBvcnQuYm94TW9kZWw/IjwhZG9jdHlwZSBo",
				"dG1sPiI6IiIpKyI8aHRtbD48Ym9keT4iKSxjbC5jbG9zZSgpO2Q9Y2wuY3JlYXRlRWxlbWVudChhKSxj",
				"bC5ib2R5LmFwcGVuZENoaWxkKGQpLGU9Zi5jc3MoZCwiZGlzcGxheSIpLGIucmVtb3ZlQ2hpbGQoY2sp",
				"fWNqW2FdPWV9cmV0dXJuIGNqW2FdfWZ1bmN0aW9uIGN0KGEsYil7dmFyIGM9e307Zi5lYWNoKGNwLmNv",
				"bmNhdC5hcHBseShbXSxjcC5zbGljZSgwLGIpKSxmdW5jdGlvbigpe2NbdGhpc109YX0pO3JldHVybiBj",
				"fWZ1bmN0aW9uIGNzKCl7Y3E9Yn1mdW5jdGlvbiBjcigpe3NldFRpbWVvdXQoY3MsMCk7cmV0dXJuIGNx",
				"PWYubm93KCl9ZnVuY3Rpb24gY2koKXt0cnl7cmV0dXJuIG5ldyBhLkFjdGl2ZVhPYmplY3QoIk1pY3Jv",
				"c29mdC5YTUxIVFRQIil9Y2F0Y2goYil7fX1mdW5jdGlvbiBjaCgpe3RyeXtyZXR1cm4gbmV3IGEuWE1M",
				"SHR0cFJlcXVlc3R9Y2F0Y2goYil7fX1mdW5jdGlvbiBjYihhLGMpe2EuZGF0YUZpbHRlciYmKGM9YS5k",
				"YXRhRmlsdGVyKGMsYS5kYXRhVHlwZSkpO3ZhciBkPWEuZGF0YVR5cGVzLGU9e30sZyxoLGk9ZC5sZW5n",
				"dGgsaixrPWRbMF0sbCxtLG4sbyxwO2ZvcihnPTE7ZzxpO2crKyl7aWYoZz09PTEpZm9yKGggaW4gYS5j",
				"b252ZXJ0ZXJzKXR5cGVvZiBoPT0ic3RyaW5nIiYmKGVbaC50b0xvd2VyQ2FzZSgpXT1hLmNvbnZlcnRl",
				"cnNbaF0pO2w9ayxrPWRbZ107aWYoaz09PSIqIilrPWw7ZWxzZSBpZihsIT09IioiJiZsIT09ayl7bT1s",
				"KyIgIitrLG49ZVttXXx8ZVsiKiAiK2tdO2lmKCFuKXtwPWI7Zm9yKG8gaW4gZSl7aj1vLnNwbGl0KCIg",
				"Iik7aWYoalswXT09PWx8fGpbMF09PT0iKiIpe3A9ZVtqWzFdKyIgIitrXTtpZihwKXtvPWVbb10sbz09",
				"PSEwP249cDpwPT09ITAmJihuPW8pO2JyZWFrfX19fSFuJiYhcCYmZi5lcnJvcigiTm8gY29udmVyc2lv",
				"biBmcm9tICIrbS5yZXBsYWNlKCIgIiwiIHRvICIpKSxuIT09ITAmJihjPW4/bihjKTpwKG8oYykpKX19",
				"cmV0dXJuIGN9ZnVuY3Rpb24gY2EoYSxjLGQpe3ZhciBlPWEuY29udGVudHMsZj1hLmRhdGFUeXBlcyxn",
				"PWEucmVzcG9uc2VGaWVsZHMsaCxpLGosaztmb3IoaSBpbiBnKWkgaW4gZCYmKGNbZ1tpXV09ZFtpXSk7",
				"d2hpbGUoZlswXT09PSIqIilmLnNoaWZ0KCksaD09PWImJihoPWEubWltZVR5cGV8fGMuZ2V0UmVzcG9u",
				"c2VIZWFkZXIoImNvbnRlbnQtdHlwZSIpKTtpZihoKWZvcihpIGluIGUpaWYoZVtpXSYmZVtpXS50ZXN0",
				"KGgpKXtmLnVuc2hpZnQoaSk7YnJlYWt9aWYoZlswXWluIGQpaj1mWzBdO2Vsc2V7Zm9yKGkgaW4gZCl7",
				"aWYoIWZbMF18fGEuY29udmVydGVyc1tpKyIgIitmWzBdXSl7aj1pO2JyZWFrfWt8fChrPWkpfWo9anx8",
				"a31pZihqKXtqIT09ZlswXSYmZi51bnNoaWZ0KGopO3JldHVybiBkW2pdfX1mdW5jdGlvbiBiXyhhLGIs",
				"YyxkKXtpZihmLmlzQXJyYXkoYikpZi5lYWNoKGIsZnVuY3Rpb24oYixlKXtjfHxiRC50ZXN0KGEpP2Qo",
				"YSxlKTpiXyhhKyJbIisodHlwZW9mIGU9PSJvYmplY3QiP2I6IiIpKyJdIixlLGMsZCl9KTtlbHNlIGlm",
				"KCFjJiZmLnR5cGUoYik9PT0ib2JqZWN0Iilmb3IodmFyIGUgaW4gYiliXyhhKyJbIitlKyJdIixiW2Vd",
				"LGMsZCk7ZWxzZSBkKGEsYil9ZnVuY3Rpb24gYiQoYSxjKXt2YXIgZCxlLGc9Zi5hamF4U2V0dGluZ3Mu",
				"ZmxhdE9wdGlvbnN8fHt9O2ZvcihkIGluIGMpY1tkXSE9PWImJigoZ1tkXT9hOmV8fChlPXt9KSlbZF09",
				"Y1tkXSk7ZSYmZi5leHRlbmQoITAsYSxlKX1mdW5jdGlvbiBiWihhLGMsZCxlLGYsZyl7Zj1mfHxjLmRh",
				"dGFUeXBlc1swXSxnPWd8fHt9LGdbZl09ITA7dmFyIGg9YVtmXSxpPTAsaj1oP2gubGVuZ3RoOjAsaz1h",
				"PT09YlMsbDtmb3IoO2k8aiYmKGt8fCFsKTtpKyspbD1oW2ldKGMsZCxlKSx0eXBlb2YgbD09InN0cmlu",
				"ZyImJigha3x8Z1tsXT9sPWI6KGMuZGF0YVR5cGVzLnVuc2hpZnQobCksbD1iWihhLGMsZCxlLGwsZykp",
				"KTsoa3x8IWwpJiYhZ1siKiJdJiYobD1iWihhLGMsZCxlLCIqIixnKSk7cmV0dXJuIGx9ZnVuY3Rpb24g",
				"YlkoYSl7cmV0dXJuIGZ1bmN0aW9uKGIsYyl7dHlwZW9mIGIhPSJzdHJpbmciJiYoYz1iLGI9IioiKTtp",
				"ZihmLmlzRnVuY3Rpb24oYykpe3ZhciBkPWIudG9Mb3dlckNhc2UoKS5zcGxpdChiTyksZT0wLGc9ZC5s",
				"ZW5ndGgsaCxpLGo7Zm9yKDtlPGc7ZSsrKWg9ZFtlXSxqPS9eXCsvLnRlc3QoaCksaiYmKGg9aC5zdWJz",
				"dHIoMSl8fCIqIiksaT1hW2hdPWFbaF18fFtdLGlbaj8idW5zaGlmdCI6InB1c2giXShjKX19fWZ1bmN0",
				"aW9uIGJCKGEsYixjKXt2YXIgZD1iPT09IndpZHRoIj9hLm9mZnNldFdpZHRoOmEub2Zmc2V0SGVpZ2h0",
				"LGU9Yj09PSJ3aWR0aCI/MTowLGc9NDtpZihkPjApe2lmKGMhPT0iYm9yZGVyIilmb3IoO2U8ZztlKz0y",
				"KWN8fChkLT1wYXJzZUZsb2F0KGYuY3NzKGEsInBhZGRpbmciK2J4W2VdKSl8fDApLGM9PT0ibWFyZ2lu",
				"Ij9kKz1wYXJzZUZsb2F0KGYuY3NzKGEsYytieFtlXSkpfHwwOmQtPXBhcnNlRmxvYXQoZi5jc3MoYSwi",
				"Ym9yZGVyIitieFtlXSsiV2lkdGgiKSl8fDA7cmV0dXJuIGQrInB4In1kPWJ5KGEsYik7aWYoZDwwfHxk",
				"PT1udWxsKWQ9YS5zdHlsZVtiXTtpZihidC50ZXN0KGQpKXJldHVybiBkO2Q9cGFyc2VGbG9hdChkKXx8",
				"MDtpZihjKWZvcig7ZTxnO2UrPTIpZCs9cGFyc2VGbG9hdChmLmNzcyhhLCJwYWRkaW5nIitieFtlXSkp",
				"fHwwLGMhPT0icGFkZGluZyImJihkKz1wYXJzZUZsb2F0KGYuY3NzKGEsImJvcmRlciIrYnhbZV0rIldp",
				"ZHRoIikpfHwwKSxjPT09Im1hcmdpbiImJihkKz1wYXJzZUZsb2F0KGYuY3NzKGEsYytieFtlXSkpfHww",
				"KTtyZXR1cm4gZCsicHgifWZ1bmN0aW9uIGJvKGEpe3ZhciBiPWMuY3JlYXRlRWxlbWVudCgiZGl2Iik7",
				"YmguYXBwZW5kQ2hpbGQoYiksYi5pbm5lckhUTUw9YS5vdXRlckhUTUw7cmV0dXJuIGIuZmlyc3RDaGls",
				"ZH1mdW5jdGlvbiBibihhKXt2YXIgYj0oYS5ub2RlTmFtZXx8IiIpLnRvTG93ZXJDYXNlKCk7Yj09PSJp",
				"bnB1dCI/Ym0oYSk6YiE9PSJzY3JpcHQiJiZ0eXBlb2YgYS5nZXRFbGVtZW50c0J5VGFnTmFtZSE9InVu",
				"ZGVmaW5lZCImJmYuZ3JlcChhLmdldEVsZW1lbnRzQnlUYWdOYW1lKCJpbnB1dCIpLGJtKX1mdW5jdGlv",
				"biBibShhKXtpZihhLnR5cGU9PT0iY2hlY2tib3gifHxhLnR5cGU9PT0icmFkaW8iKWEuZGVmYXVsdENo",
				"ZWNrZWQ9YS5jaGVja2VkfWZ1bmN0aW9uIGJsKGEpe3JldHVybiB0eXBlb2YgYS5nZXRFbGVtZW50c0J5",
				"VGFnTmFtZSE9InVuZGVmaW5lZCI/YS5nZXRFbGVtZW50c0J5VGFnTmFtZSgiKiIpOnR5cGVvZiBhLnF1",
				"ZXJ5U2VsZWN0b3JBbGwhPSJ1bmRlZmluZWQiP2EucXVlcnlTZWxlY3RvckFsbCgiKiIpOltdfWZ1bmN0",
				"aW9uIGJrKGEsYil7dmFyIGM7Yi5ub2RlVHlwZT09PTEmJihiLmNsZWFyQXR0cmlidXRlcyYmYi5jbGVh",
				"ckF0dHJpYnV0ZXMoKSxiLm1lcmdlQXR0cmlidXRlcyYmYi5tZXJnZUF0dHJpYnV0ZXMoYSksYz1iLm5v",
				"ZGVOYW1lLnRvTG93ZXJDYXNlKCksYz09PSJvYmplY3QiP2Iub3V0ZXJIVE1MPWEub3V0ZXJIVE1MOmMh",
				"PT0iaW5wdXQifHxhLnR5cGUhPT0iY2hlY2tib3giJiZhLnR5cGUhPT0icmFkaW8iP2M9PT0ib3B0aW9u",
				"Ij9iLnNlbGVjdGVkPWEuZGVmYXVsdFNlbGVjdGVkOmM9PT0iaW5wdXQifHxjPT09InRleHRhcmVhIj9i",
				"LmRlZmF1bHRWYWx1ZT1hLmRlZmF1bHRWYWx1ZTpjPT09InNjcmlwdCImJmIudGV4dCE9PWEudGV4dCYm",
				"KGIudGV4dD1hLnRleHQpOihhLmNoZWNrZWQmJihiLmRlZmF1bHRDaGVja2VkPWIuY2hlY2tlZD1hLmNo",
				"ZWNrZWQpLGIudmFsdWUhPT1hLnZhbHVlJiYoYi52YWx1ZT1hLnZhbHVlKSksYi5yZW1vdmVBdHRyaWJ1",
				"dGUoZi5leHBhbmRvKSxiLnJlbW92ZUF0dHJpYnV0ZSgiX3N1Ym1pdF9hdHRhY2hlZCIpLGIucmVtb3Zl",
				"QXR0cmlidXRlKCJfY2hhbmdlX2F0dGFjaGVkIikpfWZ1bmN0aW9uIGJqKGEsYil7aWYoYi5ub2RlVHlw",
				"ZT09PTEmJiEhZi5oYXNEYXRhKGEpKXt2YXIgYyxkLGUsZz1mLl9kYXRhKGEpLGg9Zi5fZGF0YShiLGcp",
				"LGk9Zy5ldmVudHM7aWYoaSl7ZGVsZXRlIGguaGFuZGxlLGguZXZlbnRzPXt9O2ZvcihjIGluIGkpZm9y",
				"KGQ9MCxlPWlbY10ubGVuZ3RoO2Q8ZTtkKyspZi5ldmVudC5hZGQoYixjLGlbY11bZF0pfWguZGF0YSYm",
				"KGguZGF0YT1mLmV4dGVuZCh7fSxoLmRhdGEpKX19ZnVuY3Rpb24gYmkoYSxiKXtyZXR1cm4gZi5ub2Rl",
				"TmFtZShhLCJ0YWJsZSIpP2EuZ2V0RWxlbWVudHNCeVRhZ05hbWUoInRib2R5IilbMF18fGEuYXBwZW5k",
				"Q2hpbGQoYS5vd25lckRvY3VtZW50LmNyZWF0ZUVsZW1lbnQoInRib2R5IikpOmF9ZnVuY3Rpb24gVShh",
				"KXt2YXIgYj1WLnNwbGl0KCJ8IiksYz1hLmNyZWF0ZURvY3VtZW50RnJhZ21lbnQoKTtpZihjLmNyZWF0",
				"ZUVsZW1lbnQpd2hpbGUoYi5sZW5ndGgpYy5jcmVhdGVFbGVtZW50KGIucG9wKCkpO3JldHVybiBjfWZ1",
				"bmN0aW9uIFQoYSxiLGMpe2I9Ynx8MDtpZihmLmlzRnVuY3Rpb24oYikpcmV0dXJuIGYuZ3JlcChhLGZ1",
				"bmN0aW9uKGEsZCl7dmFyIGU9ISFiLmNhbGwoYSxkLGEpO3JldHVybiBlPT09Y30pO2lmKGIubm9kZVR5",
				"cGUpcmV0dXJuIGYuZ3JlcChhLGZ1bmN0aW9uKGEsZCl7cmV0dXJuIGE9PT1iPT09Y30pO2lmKHR5cGVv",
				"ZiBiPT0ic3RyaW5nIil7dmFyIGQ9Zi5ncmVwKGEsZnVuY3Rpb24oYSl7cmV0dXJuIGEubm9kZVR5cGU9",
				"PT0xfSk7aWYoTy50ZXN0KGIpKXJldHVybiBmLmZpbHRlcihiLGQsIWMpO2I9Zi5maWx0ZXIoYixkKX1y",
				"ZXR1cm4gZi5ncmVwKGEsZnVuY3Rpb24oYSxkKXtyZXR1cm4gZi5pbkFycmF5KGEsYik+PTA9PT1jfSl9",
				"ZnVuY3Rpb24gUyhhKXtyZXR1cm4hYXx8IWEucGFyZW50Tm9kZXx8YS5wYXJlbnROb2RlLm5vZGVUeXBl",
				"PT09MTF9ZnVuY3Rpb24gSygpe3JldHVybiEwfWZ1bmN0aW9uIEooKXtyZXR1cm4hMX1mdW5jdGlvbiBu",
				"KGEsYixjKXt2YXIgZD1iKyJkZWZlciIsZT1iKyJxdWV1ZSIsZz1iKyJtYXJrIixoPWYuX2RhdGEoYSxk",
				"KTtoJiYoYz09PSJxdWV1ZSJ8fCFmLl9kYXRhKGEsZSkpJiYoYz09PSJtYXJrInx8IWYuX2RhdGEoYSxn",
				"KSkmJnNldFRpbWVvdXQoZnVuY3Rpb24oKXshZi5fZGF0YShhLGUpJiYhZi5fZGF0YShhLGcpJiYoZi5y",
				"ZW1vdmVEYXRhKGEsZCwhMCksaC5maXJlKCkpfSwwKX1mdW5jdGlvbiBtKGEpe2Zvcih2YXIgYiBpbiBh",
				"KXtpZihiPT09ImRhdGEiJiZmLmlzRW1wdHlPYmplY3QoYVtiXSkpY29udGludWU7aWYoYiE9PSJ0b0pT",
				"T04iKXJldHVybiExfXJldHVybiEwfWZ1bmN0aW9uIGwoYSxjLGQpe2lmKGQ9PT1iJiZhLm5vZGVUeXBl",
				"PT09MSl7dmFyIGU9ImRhdGEtIitjLnJlcGxhY2UoaywiLSQxIikudG9Mb3dlckNhc2UoKTtkPWEuZ2V0",
				"QXR0cmlidXRlKGUpO2lmKHR5cGVvZiBkPT0ic3RyaW5nIil7dHJ5e2Q9ZD09PSJ0cnVlIj8hMDpkPT09",
				"ImZhbHNlIj8hMTpkPT09Im51bGwiP251bGw6Zi5pc051bWVyaWMoZCk/K2Q6ai50ZXN0KGQpP2YucGFy",
				"c2VKU09OKGQpOmR9Y2F0Y2goZyl7fWYuZGF0YShhLGMsZCl9ZWxzZSBkPWJ9cmV0dXJuIGR9ZnVuY3Rp",
				"b24gaChhKXt2YXIgYj1nW2FdPXt9LGMsZDthPWEuc3BsaXQoL1xzKy8pO2ZvcihjPTAsZD1hLmxlbmd0",
				"aDtjPGQ7YysrKWJbYVtjXV09ITA7cmV0dXJuIGJ9dmFyIGM9YS5kb2N1bWVudCxkPWEubmF2aWdhdG9y",
				"LGU9YS5sb2NhdGlvbixmPWZ1bmN0aW9uKCl7ZnVuY3Rpb24gSigpe2lmKCFlLmlzUmVhZHkpe3RyeXtj",
				"LmRvY3VtZW50RWxlbWVudC5kb1Njcm9sbCgibGVmdCIpfWNhdGNoKGEpe3NldFRpbWVvdXQoSiwxKTty",
				"ZXR1cm59ZS5yZWFkeSgpfX12YXIgZT1mdW5jdGlvbihhLGIpe3JldHVybiBuZXcgZS5mbi5pbml0KGEs",
				"YixoKX0sZj1hLmpRdWVyeSxnPWEuJCxoLGk9L14oPzpbXiM8XSooPFtcd1xXXSs+KVtePl0qJHwjKFtc",
				"d1wtXSopJCkvLGo9L1xTLyxrPS9eXHMrLyxsPS9ccyskLyxtPS9ePChcdyspXHMqXC8/Pig/OjxcL1wx",
				"Pik/JC8sbj0vXltcXSw6e31cc10qJC8sbz0vXFwoPzpbIlxcXC9iZm5ydF18dVswLTlhLWZBLUZdezR9",
				"KS9nLHA9LyJbXiJcXFxuXHJdKiJ8dHJ1ZXxmYWxzZXxudWxsfC0/XGQrKD86XC5cZCopPyg/OltlRV1b",
				"K1wtXT9cZCspPy9nLHE9Lyg/Ol58OnwsKSg/OlxzKlxbKSsvZyxyPS8od2Via2l0KVsgXC9dKFtcdy5d",
				"KykvLHM9LyhvcGVyYSkoPzouKnZlcnNpb24pP1sgXC9dKFtcdy5dKykvLHQ9Lyhtc2llKSAoW1x3Ll0r",
				"KS8sdT0vKG1vemlsbGEpKD86Lio/IHJ2OihbXHcuXSspKT8vLHY9Ly0oW2Etel18WzAtOV0pL2lnLHc9",
				"L14tbXMtLyx4PWZ1bmN0aW9uKGEsYil7cmV0dXJuKGIrIiIpLnRvVXBwZXJDYXNlKCl9LHk9ZC51c2Vy",
				"QWdlbnQseixBLEIsQz1PYmplY3QucHJvdG90eXBlLnRvU3RyaW5nLEQ9T2JqZWN0LnByb3RvdHlwZS5o",
				"YXNPd25Qcm9wZXJ0eSxFPUFycmF5LnByb3RvdHlwZS5wdXNoLEY9QXJyYXkucHJvdG90eXBlLnNsaWNl",
				"LEc9U3RyaW5nLnByb3RvdHlwZS50cmltLEg9QXJyYXkucHJvdG90eXBlLmluZGV4T2YsST17fTtlLmZu",
				"PWUucHJvdG90eXBlPXtjb25zdHJ1Y3RvcjplLGluaXQ6ZnVuY3Rpb24oYSxkLGYpe3ZhciBnLGgsaixr",
				"O2lmKCFhKXJldHVybiB0aGlzO2lmKGEubm9kZVR5cGUpe3RoaXMuY29udGV4dD10aGlzWzBdPWEsdGhp",
				"cy5sZW5ndGg9MTtyZXR1cm4gdGhpc31pZihhPT09ImJvZHkiJiYhZCYmYy5ib2R5KXt0aGlzLmNvbnRl",
				"eHQ9Yyx0aGlzWzBdPWMuYm9keSx0aGlzLnNlbGVjdG9yPWEsdGhpcy5sZW5ndGg9MTtyZXR1cm4gdGhp",
				"c31pZih0eXBlb2YgYT09InN0cmluZyIpe2EuY2hhckF0KDApIT09IjwifHxhLmNoYXJBdChhLmxlbmd0",
				"aC0xKSE9PSI+Inx8YS5sZW5ndGg8Mz9nPWkuZXhlYyhhKTpnPVtudWxsLGEsbnVsbF07aWYoZyYmKGdb",
				"MV18fCFkKSl7aWYoZ1sxXSl7ZD1kIGluc3RhbmNlb2YgZT9kWzBdOmQsaz1kP2Qub3duZXJEb2N1bWVu",
				"dHx8ZDpjLGo9bS5leGVjKGEpLGo/ZS5pc1BsYWluT2JqZWN0KGQpPyhhPVtjLmNyZWF0ZUVsZW1lbnQo",
				"alsxXSldLGUuZm4uYXR0ci5jYWxsKGEsZCwhMCkpOmE9W2suY3JlYXRlRWxlbWVudChqWzFdKV06KGo9",
				"ZS5idWlsZEZyYWdtZW50KFtnWzFdXSxba10pLGE9KGouY2FjaGVhYmxlP2UuY2xvbmUoai5mcmFnbWVu",
				"dCk6ai5mcmFnbWVudCkuY2hpbGROb2Rlcyk7cmV0dXJuIGUubWVyZ2UodGhpcyxhKX1oPWMuZ2V0RWxl",
				"bWVudEJ5SWQoZ1syXSk7aWYoaCYmaC5wYXJlbnROb2RlKXtpZihoLmlkIT09Z1syXSlyZXR1cm4gZi5m",
				"aW5kKGEpO3RoaXMubGVuZ3RoPTEsdGhpc1swXT1ofXRoaXMuY29udGV4dD1jLHRoaXMuc2VsZWN0b3I9",
				"YTtyZXR1cm4gdGhpc31yZXR1cm4hZHx8ZC5qcXVlcnk/KGR8fGYpLmZpbmQoYSk6dGhpcy5jb25zdHJ1",
				"Y3RvcihkKS5maW5kKGEpfWlmKGUuaXNGdW5jdGlvbihhKSlyZXR1cm4gZi5yZWFkeShhKTthLnNlbGVj",
				"dG9yIT09YiYmKHRoaXMuc2VsZWN0b3I9YS5zZWxlY3Rvcix0aGlzLmNvbnRleHQ9YS5jb250ZXh0KTty",
				"ZXR1cm4gZS5tYWtlQXJyYXkoYSx0aGlzKX0sc2VsZWN0b3I6IiIsanF1ZXJ5OiIxLjcuMiIsbGVuZ3Ro",
				"OjAsc2l6ZTpmdW5jdGlvbigpe3JldHVybiB0aGlzLmxlbmd0aH0sdG9BcnJheTpmdW5jdGlvbigpe3Jl",
				"dHVybiBGLmNhbGwodGhpcywwKX0sZ2V0OmZ1bmN0aW9uKGEpe3JldHVybiBhPT1udWxsP3RoaXMudG9B",
				"cnJheSgpOmE8MD90aGlzW3RoaXMubGVuZ3RoK2FdOnRoaXNbYV19LHB1c2hTdGFjazpmdW5jdGlvbihh",
				"LGIsYyl7dmFyIGQ9dGhpcy5jb25zdHJ1Y3RvcigpO2UuaXNBcnJheShhKT9FLmFwcGx5KGQsYSk6ZS5t",
				"ZXJnZShkLGEpLGQucHJldk9iamVjdD10aGlzLGQuY29udGV4dD10aGlzLmNvbnRleHQsYj09PSJmaW5k",
				"Ij9kLnNlbGVjdG9yPXRoaXMuc2VsZWN0b3IrKHRoaXMuc2VsZWN0b3I/IiAiOiIiKStjOmImJihkLnNl",
				"bGVjdG9yPXRoaXMuc2VsZWN0b3IrIi4iK2IrIigiK2MrIikiKTtyZXR1cm4gZH0sZWFjaDpmdW5jdGlv",
				"bihhLGIpe3JldHVybiBlLmVhY2godGhpcyxhLGIpfSxyZWFkeTpmdW5jdGlvbihhKXtlLmJpbmRSZWFk",
				"eSgpLEEuYWRkKGEpO3JldHVybiB0aGlzfSxlcTpmdW5jdGlvbihhKXthPSthO3JldHVybiBhPT09LTE/",
				"dGhpcy5zbGljZShhKTp0aGlzLnNsaWNlKGEsYSsxKX0sZmlyc3Q6ZnVuY3Rpb24oKXtyZXR1cm4gdGhp",
				"cy5lcSgwKX0sbGFzdDpmdW5jdGlvbigpe3JldHVybiB0aGlzLmVxKC0xKX0sc2xpY2U6ZnVuY3Rpb24o",
				"KXtyZXR1cm4gdGhpcy5wdXNoU3RhY2soRi5hcHBseSh0aGlzLGFyZ3VtZW50cyksInNsaWNlIixGLmNh",
				"bGwoYXJndW1lbnRzKS5qb2luKCIsIikpfSxtYXA6ZnVuY3Rpb24oYSl7cmV0dXJuIHRoaXMucHVzaFN0",
				"YWNrKGUubWFwKHRoaXMsZnVuY3Rpb24oYixjKXtyZXR1cm4gYS5jYWxsKGIsYyxiKX0pKX0sZW5kOmZ1",
				"bmN0aW9uKCl7cmV0dXJuIHRoaXMucHJldk9iamVjdHx8dGhpcy5jb25zdHJ1Y3RvcihudWxsKX0scHVz",
				"aDpFLHNvcnQ6W10uc29ydCxzcGxpY2U6W10uc3BsaWNlfSxlLmZuLmluaXQucHJvdG90eXBlPWUuZm4s",
				"ZS5leHRlbmQ9ZS5mbi5leHRlbmQ9ZnVuY3Rpb24oKXt2YXIgYSxjLGQsZixnLGgsaT1hcmd1bWVudHNb",
				"MF18fHt9LGo9MSxrPWFyZ3VtZW50cy5sZW5ndGgsbD0hMTt0eXBlb2YgaT09ImJvb2xlYW4iJiYobD1p",
				"LGk9YXJndW1lbnRzWzFdfHx7fSxqPTIpLHR5cGVvZiBpIT0ib2JqZWN0IiYmIWUuaXNGdW5jdGlvbihp",
				"KSYmKGk9e30pLGs9PT1qJiYoaT10aGlzLC0taik7Zm9yKDtqPGs7aisrKWlmKChhPWFyZ3VtZW50c1tq",
				"XSkhPW51bGwpZm9yKGMgaW4gYSl7ZD1pW2NdLGY9YVtjXTtpZihpPT09Ziljb250aW51ZTtsJiZmJiYo",
				"ZS5pc1BsYWluT2JqZWN0KGYpfHwoZz1lLmlzQXJyYXkoZikpKT8oZz8oZz0hMSxoPWQmJmUuaXNBcnJh",
				"eShkKT9kOltdKTpoPWQmJmUuaXNQbGFpbk9iamVjdChkKT9kOnt9LGlbY109ZS5leHRlbmQobCxoLGYp",
				"KTpmIT09YiYmKGlbY109Zil9cmV0dXJuIGl9LGUuZXh0ZW5kKHtub0NvbmZsaWN0OmZ1bmN0aW9uKGIp",
				"e2EuJD09PWUmJihhLiQ9ZyksYiYmYS5qUXVlcnk9PT1lJiYoYS5qUXVlcnk9Zik7cmV0dXJuIGV9LGlz",
				"UmVhZHk6ITEscmVhZHlXYWl0OjEsaG9sZFJlYWR5OmZ1bmN0aW9uKGEpe2E/ZS5yZWFkeVdhaXQrKzpl",
				"LnJlYWR5KCEwKX0scmVhZHk6ZnVuY3Rpb24oYSl7aWYoYT09PSEwJiYhLS1lLnJlYWR5V2FpdHx8YSE9",
				"PSEwJiYhZS5pc1JlYWR5KXtpZighYy5ib2R5KXJldHVybiBzZXRUaW1lb3V0KGUucmVhZHksMSk7ZS5p",
				"c1JlYWR5PSEwO2lmKGEhPT0hMCYmLS1lLnJlYWR5V2FpdD4wKXJldHVybjtBLmZpcmVXaXRoKGMsW2Vd",
				"KSxlLmZuLnRyaWdnZXImJmUoYykudHJpZ2dlcigicmVhZHkiKS5vZmYoInJlYWR5Iil9fSxiaW5kUmVh",
				"ZHk6ZnVuY3Rpb24oKXtpZighQSl7QT1lLkNhbGxiYWNrcygib25jZSBtZW1vcnkiKTtpZihjLnJlYWR5",
				"U3RhdGU9PT0iY29tcGxldGUiKXJldHVybiBzZXRUaW1lb3V0KGUucmVhZHksMSk7aWYoYy5hZGRFdmVu",
				"dExpc3RlbmVyKWMuYWRkRXZlbnRMaXN0ZW5lcigiRE9NQ29udGVudExvYWRlZCIsQiwhMSksYS5hZGRF",
				"dmVudExpc3RlbmVyKCJsb2FkIixlLnJlYWR5LCExKTtlbHNlIGlmKGMuYXR0YWNoRXZlbnQpe2MuYXR0",
				"YWNoRXZlbnQoIm9ucmVhZHlzdGF0ZWNoYW5nZSIsQiksYS5hdHRhY2hFdmVudCgib25sb2FkIixlLnJl",
				"YWR5KTt2YXIgYj0hMTt0cnl7Yj1hLmZyYW1lRWxlbWVudD09bnVsbH1jYXRjaChkKXt9Yy5kb2N1bWVu",
				"dEVsZW1lbnQuZG9TY3JvbGwmJmImJkooKX19fSxpc0Z1bmN0aW9uOmZ1bmN0aW9uKGEpe3JldHVybiBl",
				"LnR5cGUoYSk9PT0iZnVuY3Rpb24ifSxpc0FycmF5OkFycmF5LmlzQXJyYXl8fGZ1bmN0aW9uKGEpe3Jl",
				"dHVybiBlLnR5cGUoYSk9PT0iYXJyYXkifSxpc1dpbmRvdzpmdW5jdGlvbihhKXtyZXR1cm4gYSE9bnVs",
				"bCYmYT09YS53aW5kb3d9LGlzTnVtZXJpYzpmdW5jdGlvbihhKXtyZXR1cm4haXNOYU4ocGFyc2VGbG9h",
				"dChhKSkmJmlzRmluaXRlKGEpfSx0eXBlOmZ1bmN0aW9uKGEpe3JldHVybiBhPT1udWxsP1N0cmluZyhh",
				"KTpJW0MuY2FsbChhKV18fCJvYmplY3QifSxpc1BsYWluT2JqZWN0OmZ1bmN0aW9uKGEpe2lmKCFhfHxl",
				"LnR5cGUoYSkhPT0ib2JqZWN0Inx8YS5ub2RlVHlwZXx8ZS5pc1dpbmRvdyhhKSlyZXR1cm4hMTt0cnl7",
				"aWYoYS5jb25zdHJ1Y3RvciYmIUQuY2FsbChhLCJjb25zdHJ1Y3RvciIpJiYhRC5jYWxsKGEuY29uc3Ry",
				"dWN0b3IucHJvdG90eXBlLCJpc1Byb3RvdHlwZU9mIikpcmV0dXJuITF9Y2F0Y2goYyl7cmV0dXJuITF9",
				"dmFyIGQ7Zm9yKGQgaW4gYSk7cmV0dXJuIGQ9PT1ifHxELmNhbGwoYSxkKX0saXNFbXB0eU9iamVjdDpm",
				"dW5jdGlvbihhKXtmb3IodmFyIGIgaW4gYSlyZXR1cm4hMTtyZXR1cm4hMH0sZXJyb3I6ZnVuY3Rpb24o",
				"YSl7dGhyb3cgbmV3IEVycm9yKGEpfSxwYXJzZUpTT046ZnVuY3Rpb24oYil7aWYodHlwZW9mIGIhPSJz",
				"dHJpbmcifHwhYilyZXR1cm4gbnVsbDtiPWUudHJpbShiKTtpZihhLkpTT04mJmEuSlNPTi5wYXJzZSly",
				"ZXR1cm4gYS5KU09OLnBhcnNlKGIpO2lmKG4udGVzdChiLnJlcGxhY2UobywiQCIpLnJlcGxhY2UocCwi",
				"XSIpLnJlcGxhY2UocSwiIikpKXJldHVybihuZXcgRnVuY3Rpb24oInJldHVybiAiK2IpKSgpO2UuZXJy",
				"b3IoIkludmFsaWQgSlNPTjogIitiKX0scGFyc2VYTUw6ZnVuY3Rpb24oYyl7aWYodHlwZW9mIGMhPSJz",
				"dHJpbmcifHwhYylyZXR1cm4gbnVsbDt2YXIgZCxmO3RyeXthLkRPTVBhcnNlcj8oZj1uZXcgRE9NUGFy",
				"c2VyLGQ9Zi5wYXJzZUZyb21TdHJpbmcoYywidGV4dC94bWwiKSk6KGQ9bmV3IEFjdGl2ZVhPYmplY3Qo",
				"Ik1pY3Jvc29mdC5YTUxET00iKSxkLmFzeW5jPSJmYWxzZSIsZC5sb2FkWE1MKGMpKX1jYXRjaChnKXtk",
				"PWJ9KCFkfHwhZC5kb2N1bWVudEVsZW1lbnR8fGQuZ2V0RWxlbWVudHNCeVRhZ05hbWUoInBhcnNlcmVy",
				"cm9yIikubGVuZ3RoKSYmZS5lcnJvcigiSW52YWxpZCBYTUw6ICIrYyk7cmV0dXJuIGR9LG5vb3A6ZnVu",
				"Y3Rpb24oKXt9LGdsb2JhbEV2YWw6ZnVuY3Rpb24oYil7YiYmai50ZXN0KGIpJiYoYS5leGVjU2NyaXB0",
				"fHxmdW5jdGlvbihiKXthLmV2YWwuY2FsbChhLGIpfSkoYil9LGNhbWVsQ2FzZTpmdW5jdGlvbihhKXty",
				"ZXR1cm4gYS5yZXBsYWNlKHcsIm1zLSIpLnJlcGxhY2Uodix4KX0sbm9kZU5hbWU6ZnVuY3Rpb24oYSxi",
				"KXtyZXR1cm4gYS5ub2RlTmFtZSYmYS5ub2RlTmFtZS50b1VwcGVyQ2FzZSgpPT09Yi50b1VwcGVyQ2Fz",
				"ZSgpfSxlYWNoOmZ1bmN0aW9uKGEsYyxkKXt2YXIgZixnPTAsaD1hLmxlbmd0aCxpPWg9PT1ifHxlLmlz",
				"RnVuY3Rpb24oYSk7aWYoZCl7aWYoaSl7Zm9yKGYgaW4gYSlpZihjLmFwcGx5KGFbZl0sZCk9PT0hMSli",
				"cmVha31lbHNlIGZvcig7ZzxoOylpZihjLmFwcGx5KGFbZysrXSxkKT09PSExKWJyZWFrfWVsc2UgaWYo",
				"aSl7Zm9yKGYgaW4gYSlpZihjLmNhbGwoYVtmXSxmLGFbZl0pPT09ITEpYnJlYWt9ZWxzZSBmb3IoO2c8",
				"aDspaWYoYy5jYWxsKGFbZ10sZyxhW2crK10pPT09ITEpYnJlYWs7cmV0dXJuIGF9LHRyaW06Rz9mdW5j",
				"dGlvbihhKXtyZXR1cm4gYT09bnVsbD8iIjpHLmNhbGwoYSl9OmZ1bmN0aW9uKGEpe3JldHVybiBhPT1u",
				"dWxsPyIiOihhKyIiKS5yZXBsYWNlKGssIiIpLnJlcGxhY2UobCwiIil9LG1ha2VBcnJheTpmdW5jdGlv",
				"bihhLGIpe3ZhciBjPWJ8fFtdO2lmKGEhPW51bGwpe3ZhciBkPWUudHlwZShhKTthLmxlbmd0aD09bnVs",
				"bHx8ZD09PSJzdHJpbmcifHxkPT09ImZ1bmN0aW9uInx8ZD09PSJyZWdleHAifHxlLmlzV2luZG93KGEp",
				"P0UuY2FsbChjLGEpOmUubWVyZ2UoYyxhKX1yZXR1cm4gY30saW5BcnJheTpmdW5jdGlvbihhLGIsYyl7",
				"dmFyIGQ7aWYoYil7aWYoSClyZXR1cm4gSC5jYWxsKGIsYSxjKTtkPWIubGVuZ3RoLGM9Yz9jPDA/TWF0",
				"aC5tYXgoMCxkK2MpOmM6MDtmb3IoO2M8ZDtjKyspaWYoYyBpbiBiJiZiW2NdPT09YSlyZXR1cm4gY31y",
				"ZXR1cm4tMX0sbWVyZ2U6ZnVuY3Rpb24oYSxjKXt2YXIgZD1hLmxlbmd0aCxlPTA7aWYodHlwZW9mIGMu",
				"bGVuZ3RoPT0ibnVtYmVyIilmb3IodmFyIGY9Yy5sZW5ndGg7ZTxmO2UrKylhW2QrK109Y1tlXTtlbHNl",
				"IHdoaWxlKGNbZV0hPT1iKWFbZCsrXT1jW2UrK107YS5sZW5ndGg9ZDtyZXR1cm4gYX0sZ3JlcDpmdW5j",
				"dGlvbihhLGIsYyl7dmFyIGQ9W10sZTtjPSEhYztmb3IodmFyIGY9MCxnPWEubGVuZ3RoO2Y8ZztmKysp",
				"ZT0hIWIoYVtmXSxmKSxjIT09ZSYmZC5wdXNoKGFbZl0pO3JldHVybiBkfSxtYXA6ZnVuY3Rpb24oYSxj",
				"LGQpe3ZhciBmLGcsaD1bXSxpPTAsaj1hLmxlbmd0aCxrPWEgaW5zdGFuY2VvZiBlfHxqIT09YiYmdHlw",
				"ZW9mIGo9PSJudW1iZXIiJiYoaj4wJiZhWzBdJiZhW2otMV18fGo9PT0wfHxlLmlzQXJyYXkoYSkpO2lm",
				"KGspZm9yKDtpPGo7aSsrKWY9YyhhW2ldLGksZCksZiE9bnVsbCYmKGhbaC5sZW5ndGhdPWYpO2Vsc2Ug",
				"Zm9yKGcgaW4gYSlmPWMoYVtnXSxnLGQpLGYhPW51bGwmJihoW2gubGVuZ3RoXT1mKTtyZXR1cm4gaC5j",
				"b25jYXQuYXBwbHkoW10saCl9LGd1aWQ6MSxwcm94eTpmdW5jdGlvbihhLGMpe2lmKHR5cGVvZiBjPT0i",
				"c3RyaW5nIil7dmFyIGQ9YVtjXTtjPWEsYT1kfWlmKCFlLmlzRnVuY3Rpb24oYSkpcmV0dXJuIGI7dmFy",
				"IGY9Ri5jYWxsKGFyZ3VtZW50cywyKSxnPWZ1bmN0aW9uKCl7cmV0dXJuIGEuYXBwbHkoYyxmLmNvbmNh",
				"dChGLmNhbGwoYXJndW1lbnRzKSkpfTtnLmd1aWQ9YS5ndWlkPWEuZ3VpZHx8Zy5ndWlkfHxlLmd1aWQr",
				"KztyZXR1cm4gZ30sYWNjZXNzOmZ1bmN0aW9uKGEsYyxkLGYsZyxoLGkpe3ZhciBqLGs9ZD09bnVsbCxs",
				"PTAsbT1hLmxlbmd0aDtpZihkJiZ0eXBlb2YgZD09Im9iamVjdCIpe2ZvcihsIGluIGQpZS5hY2Nlc3Mo",
				"YSxjLGwsZFtsXSwxLGgsZik7Zz0xfWVsc2UgaWYoZiE9PWIpe2o9aT09PWImJmUuaXNGdW5jdGlvbihm",
				"KSxrJiYoaj8oaj1jLGM9ZnVuY3Rpb24oYSxiLGMpe3JldHVybiBqLmNhbGwoZShhKSxjKX0pOihjLmNh",
				"bGwoYSxmKSxjPW51bGwpKTtpZihjKWZvcig7bDxtO2wrKyljKGFbbF0sZCxqP2YuY2FsbChhW2xdLGws",
				"YyhhW2xdLGQpKTpmLGkpO2c9MX1yZXR1cm4gZz9hOms/Yy5jYWxsKGEpOm0/YyhhWzBdLGQpOmh9LG5v",
				"dzpmdW5jdGlvbigpe3JldHVybihuZXcgRGF0ZSkuZ2V0VGltZSgpfSx1YU1hdGNoOmZ1bmN0aW9uKGEp",
				"e2E9YS50b0xvd2VyQ2FzZSgpO3ZhciBiPXIuZXhlYyhhKXx8cy5leGVjKGEpfHx0LmV4ZWMoYSl8fGEu",
				"aW5kZXhPZigiY29tcGF0aWJsZSIpPDAmJnUuZXhlYyhhKXx8W107cmV0dXJue2Jyb3dzZXI6YlsxXXx8",
				"IiIsdmVyc2lvbjpiWzJdfHwiMCJ9fSxzdWI6ZnVuY3Rpb24oKXtmdW5jdGlvbiBhKGIsYyl7cmV0dXJu",
				"IG5ldyBhLmZuLmluaXQoYixjKX1lLmV4dGVuZCghMCxhLHRoaXMpLGEuc3VwZXJjbGFzcz10aGlzLGEu",
				"Zm49YS5wcm90b3R5cGU9dGhpcygpLGEuZm4uY29uc3RydWN0b3I9YSxhLnN1Yj10aGlzLnN1YixhLmZu",
				"LmluaXQ9ZnVuY3Rpb24oZCxmKXtmJiZmIGluc3RhbmNlb2YgZSYmIShmIGluc3RhbmNlb2YgYSkmJihm",
				"PWEoZikpO3JldHVybiBlLmZuLmluaXQuY2FsbCh0aGlzLGQsZixiKX0sYS5mbi5pbml0LnByb3RvdHlw",
				"ZT1hLmZuO3ZhciBiPWEoYyk7cmV0dXJuIGF9LGJyb3dzZXI6e319KSxlLmVhY2goIkJvb2xlYW4gTnVt",
				"YmVyIFN0cmluZyBGdW5jdGlvbiBBcnJheSBEYXRlIFJlZ0V4cCBPYmplY3QiLnNwbGl0KCIgIiksZnVu",
				"Y3Rpb24oYSxiKXtJWyJbb2JqZWN0ICIrYisiXSJdPWIudG9Mb3dlckNhc2UoKX0pLHo9ZS51YU1hdGNo",
				"KHkpLHouYnJvd3NlciYmKGUuYnJvd3Nlclt6LmJyb3dzZXJdPSEwLGUuYnJvd3Nlci52ZXJzaW9uPXou",
				"dmVyc2lvbiksZS5icm93c2VyLndlYmtpdCYmKGUuYnJvd3Nlci5zYWZhcmk9ITApLGoudGVzdCgiwqAi",
				"KSYmKGs9L15bXHNceEEwXSsvLGw9L1tcc1x4QTBdKyQvKSxoPWUoYyksYy5hZGRFdmVudExpc3RlbmVy",
				"P0I9ZnVuY3Rpb24oKXtjLnJlbW92ZUV2ZW50TGlzdGVuZXIoIkRPTUNvbnRlbnRMb2FkZWQiLEIsITEp",
				"LGUucmVhZHkoKX06Yy5hdHRhY2hFdmVudCYmKEI9ZnVuY3Rpb24oKXtjLnJlYWR5U3RhdGU9PT0iY29t",
				"cGxldGUiJiYoYy5kZXRhY2hFdmVudCgib25yZWFkeXN0YXRlY2hhbmdlIixCKSxlLnJlYWR5KCkpfSk7",
				"cmV0dXJuIGV9KCksZz17fTtmLkNhbGxiYWNrcz1mdW5jdGlvbihhKXthPWE/Z1thXXx8aChhKTp7fTt2",
				"YXIgYz1bXSxkPVtdLGUsaSxqLGssbCxtLG49ZnVuY3Rpb24oYil7dmFyIGQsZSxnLGgsaTtmb3IoZD0w",
				"LGU9Yi5sZW5ndGg7ZDxlO2QrKylnPWJbZF0saD1mLnR5cGUoZyksaD09PSJhcnJheSI/bihnKTpoPT09",
				"ImZ1bmN0aW9uIiYmKCFhLnVuaXF1ZXx8IXAuaGFzKGcpKSYmYy5wdXNoKGcpfSxvPWZ1bmN0aW9uKGIs",
				"Zil7Zj1mfHxbXSxlPSFhLm1lbW9yeXx8W2IsZl0saT0hMCxqPSEwLG09a3x8MCxrPTAsbD1jLmxlbmd0",
				"aDtmb3IoO2MmJm08bDttKyspaWYoY1ttXS5hcHBseShiLGYpPT09ITEmJmEuc3RvcE9uRmFsc2Upe2U9",
				"ITA7YnJlYWt9aj0hMSxjJiYoYS5vbmNlP2U9PT0hMD9wLmRpc2FibGUoKTpjPVtdOmQmJmQubGVuZ3Ro",
				"JiYoZT1kLnNoaWZ0KCkscC5maXJlV2l0aChlWzBdLGVbMV0pKSl9LHA9e2FkZDpmdW5jdGlvbigpe2lm",
				"KGMpe3ZhciBhPWMubGVuZ3RoO24oYXJndW1lbnRzKSxqP2w9Yy5sZW5ndGg6ZSYmZSE9PSEwJiYoaz1h",
				"LG8oZVswXSxlWzFdKSl9cmV0dXJuIHRoaXN9LHJlbW92ZTpmdW5jdGlvbigpe2lmKGMpe3ZhciBiPWFy",
				"Z3VtZW50cyxkPTAsZT1iLmxlbmd0aDtmb3IoO2Q8ZTtkKyspZm9yKHZhciBmPTA7ZjxjLmxlbmd0aDtm",
				"KyspaWYoYltkXT09PWNbZl0pe2omJmY8PWwmJihsLS0sZjw9bSYmbS0tKSxjLnNwbGljZShmLS0sMSk7",
				"aWYoYS51bmlxdWUpYnJlYWt9fXJldHVybiB0aGlzfSxoYXM6ZnVuY3Rpb24oYSl7aWYoYyl7dmFyIGI9",
				"MCxkPWMubGVuZ3RoO2Zvcig7YjxkO2IrKylpZihhPT09Y1tiXSlyZXR1cm4hMH1yZXR1cm4hMX0sZW1w",
				"dHk6ZnVuY3Rpb24oKXtjPVtdO3JldHVybiB0aGlzfSxkaXNhYmxlOmZ1bmN0aW9uKCl7Yz1kPWU9Yjty",
				"ZXR1cm4gdGhpc30sZGlzYWJsZWQ6ZnVuY3Rpb24oKXtyZXR1cm4hY30sbG9jazpmdW5jdGlvbigpe2Q9",
				"YiwoIWV8fGU9PT0hMCkmJnAuZGlzYWJsZSgpO3JldHVybiB0aGlzfSxsb2NrZWQ6ZnVuY3Rpb24oKXty",
				"ZXR1cm4hZH0sZmlyZVdpdGg6ZnVuY3Rpb24oYixjKXtkJiYoaj9hLm9uY2V8fGQucHVzaChbYixjXSk6",
				"KCFhLm9uY2V8fCFlKSYmbyhiLGMpKTtyZXR1cm4gdGhpc30sZmlyZTpmdW5jdGlvbigpe3AuZmlyZVdp",
				"dGgodGhpcyxhcmd1bWVudHMpO3JldHVybiB0aGlzfSxmaXJlZDpmdW5jdGlvbigpe3JldHVybiEhaX19",
				"O3JldHVybiBwfTt2YXIgaT1bXS5zbGljZTtmLmV4dGVuZCh7RGVmZXJyZWQ6ZnVuY3Rpb24oYSl7dmFy",
				"IGI9Zi5DYWxsYmFja3MoIm9uY2UgbWVtb3J5IiksYz1mLkNhbGxiYWNrcygib25jZSBtZW1vcnkiKSxk",
				"PWYuQ2FsbGJhY2tzKCJtZW1vcnkiKSxlPSJwZW5kaW5nIixnPXtyZXNvbHZlOmIscmVqZWN0OmMsbm90",
				"aWZ5OmR9LGg9e2RvbmU6Yi5hZGQsZmFpbDpjLmFkZCxwcm9ncmVzczpkLmFkZCxzdGF0ZTpmdW5jdGlv",
				"bigpe3JldHVybiBlfSxpc1Jlc29sdmVkOmIuZmlyZWQsaXNSZWplY3RlZDpjLmZpcmVkLHRoZW46ZnVu",
				"Y3Rpb24oYSxiLGMpe2kuZG9uZShhKS5mYWlsKGIpLnByb2dyZXNzKGMpO3JldHVybiB0aGlzfSxhbHdh",
				"eXM6ZnVuY3Rpb24oKXtpLmRvbmUuYXBwbHkoaSxhcmd1bWVudHMpLmZhaWwuYXBwbHkoaSxhcmd1bWVu",
				"dHMpO3JldHVybiB0aGlzfSxwaXBlOmZ1bmN0aW9uKGEsYixjKXtyZXR1cm4gZi5EZWZlcnJlZChmdW5j",
				"dGlvbihkKXtmLmVhY2goe2RvbmU6W2EsInJlc29sdmUiXSxmYWlsOltiLCJyZWplY3QiXSxwcm9ncmVz",
				"czpbYywibm90aWZ5Il19LGZ1bmN0aW9uKGEsYil7dmFyIGM9YlswXSxlPWJbMV0sZztmLmlzRnVuY3Rp",
				"b24oYyk/aVthXShmdW5jdGlvbigpe2c9Yy5hcHBseSh0aGlzLGFyZ3VtZW50cyksZyYmZi5pc0Z1bmN0",
				"aW9uKGcucHJvbWlzZSk/Zy5wcm9taXNlKCkudGhlbihkLnJlc29sdmUsZC5yZWplY3QsZC5ub3RpZnkp",
				"OmRbZSsiV2l0aCJdKHRoaXM9PT1pP2Q6dGhpcyxbZ10pfSk6aVthXShkW2VdKX0pfSkucHJvbWlzZSgp",
				"fSxwcm9taXNlOmZ1bmN0aW9uKGEpe2lmKGE9PW51bGwpYT1oO2Vsc2UgZm9yKHZhciBiIGluIGgpYVti",
				"XT1oW2JdO3JldHVybiBhfX0saT1oLnByb21pc2Uoe30pLGo7Zm9yKGogaW4gZylpW2pdPWdbal0uZmly",
				"ZSxpW2orIldpdGgiXT1nW2pdLmZpcmVXaXRoO2kuZG9uZShmdW5jdGlvbigpe2U9InJlc29sdmVkIn0s",
				"Yy5kaXNhYmxlLGQubG9jaykuZmFpbChmdW5jdGlvbigpe2U9InJlamVjdGVkIn0sYi5kaXNhYmxlLGQu",
				"bG9jayksYSYmYS5jYWxsKGksaSk7cmV0dXJuIGl9LHdoZW46ZnVuY3Rpb24oYSl7ZnVuY3Rpb24gbShh",
				"KXtyZXR1cm4gZnVuY3Rpb24oYil7ZVthXT1hcmd1bWVudHMubGVuZ3RoPjE/aS5jYWxsKGFyZ3VtZW50",
				"cywwKTpiLGoubm90aWZ5V2l0aChrLGUpfX1mdW5jdGlvbiBsKGEpe3JldHVybiBmdW5jdGlvbihjKXti",
				"W2FdPWFyZ3VtZW50cy5sZW5ndGg+MT9pLmNhbGwoYXJndW1lbnRzLDApOmMsLS1nfHxqLnJlc29sdmVX",
				"aXRoKGosYil9fXZhciBiPWkuY2FsbChhcmd1bWVudHMsMCksYz0wLGQ9Yi5sZW5ndGgsZT1BcnJheShk",
				"KSxnPWQsaD1kLGo9ZDw9MSYmYSYmZi5pc0Z1bmN0aW9uKGEucHJvbWlzZSk/YTpmLkRlZmVycmVkKCks",
				"az1qLnByb21pc2UoKTtpZihkPjEpe2Zvcig7YzxkO2MrKyliW2NdJiZiW2NdLnByb21pc2UmJmYuaXNG",
				"dW5jdGlvbihiW2NdLnByb21pc2UpP2JbY10ucHJvbWlzZSgpLnRoZW4obChjKSxqLnJlamVjdCxtKGMp",
				"KTotLWc7Z3x8ai5yZXNvbHZlV2l0aChqLGIpfWVsc2UgaiE9PWEmJmoucmVzb2x2ZVdpdGgoaixkP1th",
				"XTpbXSk7cmV0dXJuIGt9fSksZi5zdXBwb3J0PWZ1bmN0aW9uKCl7dmFyIGIsZCxlLGcsaCxpLGosayxs",
				"LG0sbixvLHA9Yy5jcmVhdGVFbGVtZW50KCJkaXYiKSxxPWMuZG9jdW1lbnRFbGVtZW50O3Auc2V0QXR0",
				"cmlidXRlKCJjbGFzc05hbWUiLCJ0IikscC5pbm5lckhUTUw9IiAgIDxsaW5rLz48dGFibGU+PC90YWJs",
				"ZT48YSBocmVmPScvYScgc3R5bGU9J3RvcDoxcHg7ZmxvYXQ6bGVmdDtvcGFjaXR5Oi41NTsnPmE8L2E+",
				"PGlucHV0IHR5cGU9J2NoZWNrYm94Jy8+IixkPXAuZ2V0RWxlbWVudHNCeVRhZ05hbWUoIioiKSxlPXAu",
				"Z2V0RWxlbWVudHNCeVRhZ05hbWUoImEiKVswXTtpZighZHx8IWQubGVuZ3RofHwhZSlyZXR1cm57fTtn",
				"PWMuY3JlYXRlRWxlbWVudCgic2VsZWN0IiksaD1nLmFwcGVuZENoaWxkKGMuY3JlYXRlRWxlbWVudCgi",
				"b3B0aW9uIikpLGk9cC5nZXRFbGVtZW50c0J5VGFnTmFtZSgiaW5wdXQiKVswXSxiPXtsZWFkaW5nV2hp",
				"dGVzcGFjZTpwLmZpcnN0Q2hpbGQubm9kZVR5cGU9PT0zLHRib2R5OiFwLmdldEVsZW1lbnRzQnlUYWdO",
				"YW1lKCJ0Ym9keSIpLmxlbmd0aCxodG1sU2VyaWFsaXplOiEhcC5nZXRFbGVtZW50c0J5VGFnTmFtZSgi",
				"bGluayIpLmxlbmd0aCxzdHlsZTovdG9wLy50ZXN0KGUuZ2V0QXR0cmlidXRlKCJzdHlsZSIpKSxocmVm",
				"Tm9ybWFsaXplZDplLmdldEF0dHJpYnV0ZSgiaHJlZiIpPT09Ii9hIixvcGFjaXR5Oi9eMC41NS8udGVz",
				"dChlLnN0eWxlLm9wYWNpdHkpLGNzc0Zsb2F0OiEhZS5zdHlsZS5jc3NGbG9hdCxjaGVja09uOmkudmFs",
				"dWU9PT0ib24iLG9wdFNlbGVjdGVkOmguc2VsZWN0ZWQsZ2V0U2V0QXR0cmlidXRlOnAuY2xhc3NOYW1l",
				"IT09InQiLGVuY3R5cGU6ISFjLmNyZWF0ZUVsZW1lbnQoImZvcm0iKS5lbmN0eXBlLGh0bWw1Q2xvbmU6",
				"Yy5jcmVhdGVFbGVtZW50KCJuYXYiKS5jbG9uZU5vZGUoITApLm91dGVySFRNTCE9PSI8Om5hdj48Lzpu",
				"YXY+IixzdWJtaXRCdWJibGVzOiEwLGNoYW5nZUJ1YmJsZXM6ITAsZm9jdXNpbkJ1YmJsZXM6ITEsZGVs",
				"ZXRlRXhwYW5kbzohMCxub0Nsb25lRXZlbnQ6ITAsaW5saW5lQmxvY2tOZWVkc0xheW91dDohMSxzaHJp",
				"bmtXcmFwQmxvY2tzOiExLHJlbGlhYmxlTWFyZ2luUmlnaHQ6ITAscGl4ZWxNYXJnaW46ITB9LGYuYm94",
				"TW9kZWw9Yi5ib3hNb2RlbD1jLmNvbXBhdE1vZGU9PT0iQ1NTMUNvbXBhdCIsaS5jaGVja2VkPSEwLGIu",
				"bm9DbG9uZUNoZWNrZWQ9aS5jbG9uZU5vZGUoITApLmNoZWNrZWQsZy5kaXNhYmxlZD0hMCxiLm9wdERp",
				"c2FibGVkPSFoLmRpc2FibGVkO3RyeXtkZWxldGUgcC50ZXN0fWNhdGNoKHIpe2IuZGVsZXRlRXhwYW5k",
				"bz0hMX0hcC5hZGRFdmVudExpc3RlbmVyJiZwLmF0dGFjaEV2ZW50JiZwLmZpcmVFdmVudCYmKHAuYXR0",
				"YWNoRXZlbnQoIm9uY2xpY2siLGZ1bmN0aW9uKCl7Yi5ub0Nsb25lRXZlbnQ9ITF9KSxwLmNsb25lTm9k",
				"ZSghMCkuZmlyZUV2ZW50KCJvbmNsaWNrIikpLGk9Yy5jcmVhdGVFbGVtZW50KCJpbnB1dCIpLGkudmFs",
				"dWU9InQiLGkuc2V0QXR0cmlidXRlKCJ0eXBlIiwicmFkaW8iKSxiLnJhZGlvVmFsdWU9aS52YWx1ZT09",
				"PSJ0IixpLnNldEF0dHJpYnV0ZSgiY2hlY2tlZCIsImNoZWNrZWQiKSxpLnNldEF0dHJpYnV0ZSgibmFt",
				"ZSIsInQiKSxwLmFwcGVuZENoaWxkKGkpLGo9Yy5jcmVhdGVEb2N1bWVudEZyYWdtZW50KCksai5hcHBl",
				"bmRDaGlsZChwLmxhc3RDaGlsZCksYi5jaGVja0Nsb25lPWouY2xvbmVOb2RlKCEwKS5jbG9uZU5vZGUo",
				"ITApLmxhc3RDaGlsZC5jaGVja2VkLGIuYXBwZW5kQ2hlY2tlZD1pLmNoZWNrZWQsai5yZW1vdmVDaGls",
				"ZChpKSxqLmFwcGVuZENoaWxkKHApO2lmKHAuYXR0YWNoRXZlbnQpZm9yKG4gaW57c3VibWl0OjEsY2hh",
				"bmdlOjEsZm9jdXNpbjoxfSltPSJvbiIrbixvPW0gaW4gcCxvfHwocC5zZXRBdHRyaWJ1dGUobSwicmV0",
				"dXJuOyIpLG89dHlwZW9mIHBbbV09PSJmdW5jdGlvbiIpLGJbbisiQnViYmxlcyJdPW87ai5yZW1vdmVD",
				"aGlsZChwKSxqPWc9aD1wPWk9bnVsbCxmKGZ1bmN0aW9uKCl7dmFyIGQsZSxnLGgsaSxqLGwsbSxuLHEs",
				"cixzLHQsdT1jLmdldEVsZW1lbnRzQnlUYWdOYW1lKCJib2R5IilbMF07IXV8fChtPTEsdD0icGFkZGlu",
				"ZzowO21hcmdpbjowO2JvcmRlcjoiLHI9InBvc2l0aW9uOmFic29sdXRlO3RvcDowO2xlZnQ6MDt3aWR0",
				"aDoxcHg7aGVpZ2h0OjFweDsiLHM9dCsiMDt2aXNpYmlsaXR5OmhpZGRlbjsiLG49InN0eWxlPSciK3Ir",
				"dCsiNXB4IHNvbGlkICMwMDA7IixxPSI8ZGl2ICIrbisiZGlzcGxheTpibG9jazsnPjxkaXYgc3R5bGU9",
				"JyIrdCsiMDtkaXNwbGF5OmJsb2NrO292ZXJmbG93OmhpZGRlbjsnPjwvZGl2PjwvZGl2PiIrIjx0YWJs",
				"ZSAiK24rIicgY2VsbHBhZGRpbmc9JzAnIGNlbGxzcGFjaW5nPScwJz4iKyI8dHI+PHRkPjwvdGQ+PC90",
				"cj48L3RhYmxlPiIsZD1jLmNyZWF0ZUVsZW1lbnQoImRpdiIpLGQuc3R5bGUuY3NzVGV4dD1zKyJ3aWR0",
				"aDowO2hlaWdodDowO3Bvc2l0aW9uOnN0YXRpYzt0b3A6MDttYXJnaW4tdG9wOiIrbSsicHgiLHUuaW5z",
				"ZXJ0QmVmb3JlKGQsdS5maXJzdENoaWxkKSxwPWMuY3JlYXRlRWxlbWVudCgiZGl2IiksZC5hcHBlbmRD",
				"aGlsZChwKSxwLmlubmVySFRNTD0iPHRhYmxlPjx0cj48dGQgc3R5bGU9JyIrdCsiMDtkaXNwbGF5Om5v",
				"bmUnPjwvdGQ+PHRkPnQ8L3RkPjwvdHI+PC90YWJsZT4iLGs9cC5nZXRFbGVtZW50c0J5VGFnTmFtZSgi",
				"dGQiKSxvPWtbMF0ub2Zmc2V0SGVpZ2h0PT09MCxrWzBdLnN0eWxlLmRpc3BsYXk9IiIsa1sxXS5zdHls",
				"ZS5kaXNwbGF5PSJub25lIixiLnJlbGlhYmxlSGlkZGVuT2Zmc2V0cz1vJiZrWzBdLm9mZnNldEhlaWdo",
				"dD09PTAsYS5nZXRDb21wdXRlZFN0eWxlJiYocC5pbm5lckhUTUw9IiIsbD1jLmNyZWF0ZUVsZW1lbnQo",
				"ImRpdiIpLGwuc3R5bGUud2lkdGg9IjAiLGwuc3R5bGUubWFyZ2luUmlnaHQ9IjAiLHAuc3R5bGUud2lk",
				"dGg9IjJweCIscC5hcHBlbmRDaGlsZChsKSxiLnJlbGlhYmxlTWFyZ2luUmlnaHQ9KHBhcnNlSW50KChh",
				"LmdldENvbXB1dGVkU3R5bGUobCxudWxsKXx8e21hcmdpblJpZ2h0OjB9KS5tYXJnaW5SaWdodCwxMCl8",
				"fDApPT09MCksdHlwZW9mIHAuc3R5bGUuem9vbSE9InVuZGVmaW5lZCImJihwLmlubmVySFRNTD0iIixw",
				"LnN0eWxlLndpZHRoPXAuc3R5bGUucGFkZGluZz0iMXB4IixwLnN0eWxlLmJvcmRlcj0wLHAuc3R5bGUu",
				"b3ZlcmZsb3c9ImhpZGRlbiIscC5zdHlsZS5kaXNwbGF5PSJpbmxpbmUiLHAuc3R5bGUuem9vbT0xLGIu",
				"aW5saW5lQmxvY2tOZWVkc0xheW91dD1wLm9mZnNldFdpZHRoPT09MyxwLnN0eWxlLmRpc3BsYXk9ImJs",
				"b2NrIixwLnN0eWxlLm92ZXJmbG93PSJ2aXNpYmxlIixwLmlubmVySFRNTD0iPGRpdiBzdHlsZT0nd2lk",
				"dGg6NXB4Oyc+PC9kaXY+IixiLnNocmlua1dyYXBCbG9ja3M9cC5vZmZzZXRXaWR0aCE9PTMpLHAuc3R5",
				"bGUuY3NzVGV4dD1yK3MscC5pbm5lckhUTUw9cSxlPXAuZmlyc3RDaGlsZCxnPWUuZmlyc3RDaGlsZCxp",
				"PWUubmV4dFNpYmxpbmcuZmlyc3RDaGlsZC5maXJzdENoaWxkLGo9e2RvZXNOb3RBZGRCb3JkZXI6Zy5v",
				"ZmZzZXRUb3AhPT01LGRvZXNBZGRCb3JkZXJGb3JUYWJsZUFuZENlbGxzOmkub2Zmc2V0VG9wPT09NX0s",
				"Zy5zdHlsZS5wb3NpdGlvbj0iZml4ZWQiLGcuc3R5bGUudG9wPSIyMHB4IixqLmZpeGVkUG9zaXRpb249",
				"Zy5vZmZzZXRUb3A9PT0yMHx8Zy5vZmZzZXRUb3A9PT0xNSxnLnN0eWxlLnBvc2l0aW9uPWcuc3R5bGUu",
				"dG9wPSIiLGUuc3R5bGUub3ZlcmZsb3c9ImhpZGRlbiIsZS5zdHlsZS5wb3NpdGlvbj0icmVsYXRpdmUi",
				"LGouc3VidHJhY3RzQm9yZGVyRm9yT3ZlcmZsb3dOb3RWaXNpYmxlPWcub2Zmc2V0VG9wPT09LTUsai5k",
				"b2VzTm90SW5jbHVkZU1hcmdpbkluQm9keU9mZnNldD11Lm9mZnNldFRvcCE9PW0sYS5nZXRDb21wdXRl",
				"ZFN0eWxlJiYocC5zdHlsZS5tYXJnaW5Ub3A9IjElIixiLnBpeGVsTWFyZ2luPShhLmdldENvbXB1dGVk",
				"U3R5bGUocCxudWxsKXx8e21hcmdpblRvcDowfSkubWFyZ2luVG9wIT09IjElIiksdHlwZW9mIGQuc3R5",
				"bGUuem9vbSE9InVuZGVmaW5lZCImJihkLnN0eWxlLnpvb209MSksdS5yZW1vdmVDaGlsZChkKSxsPXA9",
				"ZD1udWxsLGYuZXh0ZW5kKGIsaikpfSk7cmV0dXJuIGJ9KCk7dmFyIGo9L14oPzpcey4qXH18XFsuKlxd",
				"KSQvLGs9LyhbQS1aXSkvZztmLmV4dGVuZCh7Y2FjaGU6e30sdXVpZDowLGV4cGFuZG86ImpRdWVyeSIr",
				"KGYuZm4uanF1ZXJ5K01hdGgucmFuZG9tKCkpLnJlcGxhY2UoL1xEL2csIiIpLG5vRGF0YTp7ZW1iZWQ6",
				"ITAsb2JqZWN0OiJjbHNpZDpEMjdDREI2RS1BRTZELTExY2YtOTZCOC00NDQ1NTM1NDAwMDAiLGFwcGxl",
				"dDohMH0saGFzRGF0YTpmdW5jdGlvbihhKXthPWEubm9kZVR5cGU/Zi5jYWNoZVthW2YuZXhwYW5kb11d",
				"OmFbZi5leHBhbmRvXTtyZXR1cm4hIWEmJiFtKGEpfSxkYXRhOmZ1bmN0aW9uKGEsYyxkLGUpe2lmKCEh",
				"Zi5hY2NlcHREYXRhKGEpKXt2YXIgZyxoLGksaj1mLmV4cGFuZG8saz10eXBlb2YgYz09InN0cmluZyIs",
				"bD1hLm5vZGVUeXBlLG09bD9mLmNhY2hlOmEsbj1sP2Fbal06YVtqXSYmaixvPWM9PT0iZXZlbnRzIjtp",
				"ZigoIW58fCFtW25dfHwhbyYmIWUmJiFtW25dLmRhdGEpJiZrJiZkPT09YilyZXR1cm47bnx8KGw/YVtq",
				"XT1uPSsrZi51dWlkOm49aiksbVtuXXx8KG1bbl09e30sbHx8KG1bbl0udG9KU09OPWYubm9vcCkpO2lm",
				"KHR5cGVvZiBjPT0ib2JqZWN0Inx8dHlwZW9mIGM9PSJmdW5jdGlvbiIpZT9tW25dPWYuZXh0ZW5kKG1b",
				"bl0sYyk6bVtuXS5kYXRhPWYuZXh0ZW5kKG1bbl0uZGF0YSxjKTtnPWg9bVtuXSxlfHwoaC5kYXRhfHwo",
				"aC5kYXRhPXt9KSxoPWguZGF0YSksZCE9PWImJihoW2YuY2FtZWxDYXNlKGMpXT1kKTtpZihvJiYhaFtj",
				"XSlyZXR1cm4gZy5ldmVudHM7az8oaT1oW2NdLGk9PW51bGwmJihpPWhbZi5jYW1lbENhc2UoYyldKSk6",
				"aT1oO3JldHVybiBpfX0scmVtb3ZlRGF0YTpmdW5jdGlvbihhLGIsYyl7aWYoISFmLmFjY2VwdERhdGEo",
				"YSkpe3ZhciBkLGUsZyxoPWYuZXhwYW5kbyxpPWEubm9kZVR5cGUsaj1pP2YuY2FjaGU6YSxrPWk/YVto",
				"XTpoO2lmKCFqW2tdKXJldHVybjtpZihiKXtkPWM/altrXTpqW2tdLmRhdGE7aWYoZCl7Zi5pc0FycmF5",
				"KGIpfHwoYiBpbiBkP2I9W2JdOihiPWYuY2FtZWxDYXNlKGIpLGIgaW4gZD9iPVtiXTpiPWIuc3BsaXQo",
				"IiAiKSkpO2ZvcihlPTAsZz1iLmxlbmd0aDtlPGc7ZSsrKWRlbGV0ZSBkW2JbZV1dO2lmKCEoYz9tOmYu",
				"aXNFbXB0eU9iamVjdCkoZCkpcmV0dXJufX1pZighYyl7ZGVsZXRlIGpba10uZGF0YTtpZighbShqW2td",
				"KSlyZXR1cm59Zi5zdXBwb3J0LmRlbGV0ZUV4cGFuZG98fCFqLnNldEludGVydmFsP2RlbGV0ZSBqW2td",
				"Ompba109bnVsbCxpJiYoZi5zdXBwb3J0LmRlbGV0ZUV4cGFuZG8/ZGVsZXRlIGFbaF06YS5yZW1vdmVB",
				"dHRyaWJ1dGU/YS5yZW1vdmVBdHRyaWJ1dGUoaCk6YVtoXT1udWxsKX19LF9kYXRhOmZ1bmN0aW9uKGEs",
				"YixjKXtyZXR1cm4gZi5kYXRhKGEsYixjLCEwKX0sYWNjZXB0RGF0YTpmdW5jdGlvbihhKXtpZihhLm5v",
				"ZGVOYW1lKXt2YXIgYj1mLm5vRGF0YVthLm5vZGVOYW1lLnRvTG93ZXJDYXNlKCldO2lmKGIpcmV0dXJu",
				"IGIhPT0hMCYmYS5nZXRBdHRyaWJ1dGUoImNsYXNzaWQiKT09PWJ9cmV0dXJuITB9fSksZi5mbi5leHRl",
				"bmQoe2RhdGE6ZnVuY3Rpb24oYSxjKXt2YXIgZCxlLGcsaCxpLGo9dGhpc1swXSxrPTAsbT1udWxsO2lm",
				"KGE9PT1iKXtpZih0aGlzLmxlbmd0aCl7bT1mLmRhdGEoaik7aWYoai5ub2RlVHlwZT09PTEmJiFmLl9k",
				"YXRhKGosInBhcnNlZEF0dHJzIikpe2c9ai5hdHRyaWJ1dGVzO2ZvcihpPWcubGVuZ3RoO2s8aTtrKysp",
				"aD1nW2tdLm5hbWUsaC5pbmRleE9mKCJkYXRhLSIpPT09MCYmKGg9Zi5jYW1lbENhc2UoaC5zdWJzdHJp",
				"bmcoNSkpLGwoaixoLG1baF0pKTtmLl9kYXRhKGosInBhcnNlZEF0dHJzIiwhMCl9fXJldHVybiBtfWlm",
				"KHR5cGVvZiBhPT0ib2JqZWN0IilyZXR1cm4gdGhpcy5lYWNoKGZ1bmN0aW9uKCl7Zi5kYXRhKHRoaXMs",
				"YSl9KTtkPWEuc3BsaXQoIi4iLDIpLGRbMV09ZFsxXT8iLiIrZFsxXToiIixlPWRbMV0rIiEiO3JldHVy",
				"biBmLmFjY2Vzcyh0aGlzLGZ1bmN0aW9uKGMpe2lmKGM9PT1iKXttPXRoaXMudHJpZ2dlckhhbmRsZXIo",
				"ImdldERhdGEiK2UsW2RbMF1dKSxtPT09YiYmaiYmKG09Zi5kYXRhKGosYSksbT1sKGosYSxtKSk7cmV0",
				"dXJuIG09PT1iJiZkWzFdP3RoaXMuZGF0YShkWzBdKTptfWRbMV09Yyx0aGlzLmVhY2goZnVuY3Rpb24o",
				"KXt2YXIgYj1mKHRoaXMpO2IudHJpZ2dlckhhbmRsZXIoInNldERhdGEiK2UsZCksZi5kYXRhKHRoaXMs",
				"YSxjKSxiLnRyaWdnZXJIYW5kbGVyKCJjaGFuZ2VEYXRhIitlLGQpfSl9LG51bGwsYyxhcmd1bWVudHMu",
				"bGVuZ3RoPjEsbnVsbCwhMSl9LHJlbW92ZURhdGE6ZnVuY3Rpb24oYSl7cmV0dXJuIHRoaXMuZWFjaChm",
				"dW5jdGlvbigpe2YucmVtb3ZlRGF0YSh0aGlzLGEpfSl9fSksZi5leHRlbmQoe19tYXJrOmZ1bmN0aW9u",
				"KGEsYil7YSYmKGI9KGJ8fCJmeCIpKyJtYXJrIixmLl9kYXRhKGEsYiwoZi5fZGF0YShhLGIpfHwwKSsx",
				"KSl9LF91bm1hcms6ZnVuY3Rpb24oYSxiLGMpe2EhPT0hMCYmKGM9YixiPWEsYT0hMSk7aWYoYil7Yz1j",
				"fHwiZngiO3ZhciBkPWMrIm1hcmsiLGU9YT8wOihmLl9kYXRhKGIsZCl8fDEpLTE7ZT9mLl9kYXRhKGIs",
				"ZCxlKTooZi5yZW1vdmVEYXRhKGIsZCwhMCksbihiLGMsIm1hcmsiKSl9fSxxdWV1ZTpmdW5jdGlvbihh",
				"LGIsYyl7dmFyIGQ7aWYoYSl7Yj0oYnx8ImZ4IikrInF1ZXVlIixkPWYuX2RhdGEoYSxiKSxjJiYoIWR8",
				"fGYuaXNBcnJheShjKT9kPWYuX2RhdGEoYSxiLGYubWFrZUFycmF5KGMpKTpkLnB1c2goYykpO3JldHVy",
				"biBkfHxbXX19LGRlcXVldWU6ZnVuY3Rpb24oYSxiKXtiPWJ8fCJmeCI7dmFyIGM9Zi5xdWV1ZShhLGIp",
				"LGQ9Yy5zaGlmdCgpLGU9e307ZD09PSJpbnByb2dyZXNzIiYmKGQ9Yy5zaGlmdCgpKSxkJiYoYj09PSJm",
				"eCImJmMudW5zaGlmdCgiaW5wcm9ncmVzcyIpLGYuX2RhdGEoYSxiKyIucnVuIixlKSxkLmNhbGwoYSxm",
				"dW5jdGlvbigpe2YuZGVxdWV1ZShhLGIpfSxlKSksYy5sZW5ndGh8fChmLnJlbW92ZURhdGEoYSxiKyJx",
				"dWV1ZSAiK2IrIi5ydW4iLCEwKSxuKGEsYiwicXVldWUiKSl9fSksZi5mbi5leHRlbmQoe3F1ZXVlOmZ1",
				"bmN0aW9uKGEsYyl7dmFyIGQ9Mjt0eXBlb2YgYSE9InN0cmluZyImJihjPWEsYT0iZngiLGQtLSk7aWYo",
				"YXJndW1lbnRzLmxlbmd0aDxkKXJldHVybiBmLnF1ZXVlKHRoaXNbMF0sYSk7cmV0dXJuIGM9PT1iP3Ro",
				"aXM6dGhpcy5lYWNoKGZ1bmN0aW9uKCl7dmFyIGI9Zi5xdWV1ZSh0aGlzLGEsYyk7YT09PSJmeCImJmJb",
				"MF0hPT0iaW5wcm9ncmVzcyImJmYuZGVxdWV1ZSh0aGlzLGEpfSl9LGRlcXVldWU6ZnVuY3Rpb24oYSl7",
				"cmV0dXJuIHRoaXMuZWFjaChmdW5jdGlvbigpe2YuZGVxdWV1ZSh0aGlzLGEpfSl9LGRlbGF5OmZ1bmN0",
				"aW9uKGEsYil7YT1mLmZ4P2YuZnguc3BlZWRzW2FdfHxhOmEsYj1ifHwiZngiO3JldHVybiB0aGlzLnF1",
				"ZXVlKGIsZnVuY3Rpb24oYixjKXt2YXIgZD1zZXRUaW1lb3V0KGIsYSk7Yy5zdG9wPWZ1bmN0aW9uKCl7",
				"Y2xlYXJUaW1lb3V0KGQpfX0pfSxjbGVhclF1ZXVlOmZ1bmN0aW9uKGEpe3JldHVybiB0aGlzLnF1ZXVl",
				"KGF8fCJmeCIsW10pfSxwcm9taXNlOmZ1bmN0aW9uKGEsYyl7ZnVuY3Rpb24gbSgpey0taHx8ZC5yZXNv",
				"bHZlV2l0aChlLFtlXSl9dHlwZW9mIGEhPSJzdHJpbmciJiYoYz1hLGE9YiksYT1hfHwiZngiO3ZhciBk",
				"PWYuRGVmZXJyZWQoKSxlPXRoaXMsZz1lLmxlbmd0aCxoPTEsaT1hKyJkZWZlciIsaj1hKyJxdWV1ZSIs",
				"az1hKyJtYXJrIixsO3doaWxlKGctLSlpZihsPWYuZGF0YShlW2ddLGksYiwhMCl8fChmLmRhdGEoZVtn",
				"XSxqLGIsITApfHxmLmRhdGEoZVtnXSxrLGIsITApKSYmZi5kYXRhKGVbZ10saSxmLkNhbGxiYWNrcygi",
				"b25jZSBtZW1vcnkiKSwhMCkpaCsrLGwuYWRkKG0pO20oKTtyZXR1cm4gZC5wcm9taXNlKGMpfX0pO3Zh",
				"ciBvPS9bXG5cdFxyXS9nLHA9L1xzKy8scT0vXHIvZyxyPS9eKD86YnV0dG9ufGlucHV0KSQvaSxzPS9e",
				"KD86YnV0dG9ufGlucHV0fG9iamVjdHxzZWxlY3R8dGV4dGFyZWEpJC9pLHQ9L15hKD86cmVhKT8kL2ks",
				"dT0vXig/OmF1dG9mb2N1c3xhdXRvcGxheXxhc3luY3xjaGVja2VkfGNvbnRyb2xzfGRlZmVyfGRpc2Fi",
				"bGVkfGhpZGRlbnxsb29wfG11bHRpcGxlfG9wZW58cmVhZG9ubHl8cmVxdWlyZWR8c2NvcGVkfHNlbGVj",
				"dGVkKSQvaSx2PWYuc3VwcG9ydC5nZXRTZXRBdHRyaWJ1dGUsdyx4LHk7Zi5mbi5leHRlbmQoe2F0dHI6",
				"ZnVuY3Rpb24oYSxiKXtyZXR1cm4gZi5hY2Nlc3ModGhpcyxmLmF0dHIsYSxiLGFyZ3VtZW50cy5sZW5n",
				"dGg+MSl9LHJlbW92ZUF0dHI6ZnVuY3Rpb24oYSl7cmV0dXJuIHRoaXMuZWFjaChmdW5jdGlvbigpe2Yu",
				"cmVtb3ZlQXR0cih0aGlzLGEpfSl9LHByb3A6ZnVuY3Rpb24oYSxiKXtyZXR1cm4gZi5hY2Nlc3ModGhp",
				"cyxmLnByb3AsYSxiLGFyZ3VtZW50cy5sZW5ndGg+MSl9LHJlbW92ZVByb3A6ZnVuY3Rpb24oYSl7YT1m",
				"LnByb3BGaXhbYV18fGE7cmV0dXJuIHRoaXMuZWFjaChmdW5jdGlvbigpe3RyeXt0aGlzW2FdPWIsZGVs",
				"ZXRlIHRoaXNbYV19Y2F0Y2goYyl7fX0pfSxhZGRDbGFzczpmdW5jdGlvbihhKXt2YXIgYixjLGQsZSxn",
				"LGgsaTtpZihmLmlzRnVuY3Rpb24oYSkpcmV0dXJuIHRoaXMuZWFjaChmdW5jdGlvbihiKXtmKHRoaXMp",
				"LmFkZENsYXNzKGEuY2FsbCh0aGlzLGIsdGhpcy5jbGFzc05hbWUpKX0pO2lmKGEmJnR5cGVvZiBhPT0i",
				"c3RyaW5nIil7Yj1hLnNwbGl0KHApO2ZvcihjPTAsZD10aGlzLmxlbmd0aDtjPGQ7YysrKXtlPXRoaXNb",
				"Y107aWYoZS5ub2RlVHlwZT09PTEpaWYoIWUuY2xhc3NOYW1lJiZiLmxlbmd0aD09PTEpZS5jbGFzc05h",
				"bWU9YTtlbHNle2c9IiAiK2UuY2xhc3NOYW1lKyIgIjtmb3IoaD0wLGk9Yi5sZW5ndGg7aDxpO2grKyl+",
				"Zy5pbmRleE9mKCIgIitiW2hdKyIgIil8fChnKz1iW2hdKyIgIik7ZS5jbGFzc05hbWU9Zi50cmltKGcp",
				"fX19cmV0dXJuIHRoaXN9LHJlbW92ZUNsYXNzOmZ1bmN0aW9uKGEpe3ZhciBjLGQsZSxnLGgsaSxqO2lm",
				"KGYuaXNGdW5jdGlvbihhKSlyZXR1cm4gdGhpcy5lYWNoKGZ1bmN0aW9uKGIpe2YodGhpcykucmVtb3Zl",
				"Q2xhc3MoYS5jYWxsKHRoaXMsYix0aGlzLmNsYXNzTmFtZSkpfSk7aWYoYSYmdHlwZW9mIGE9PSJzdHJp",
				"bmcifHxhPT09Yil7Yz0oYXx8IiIpLnNwbGl0KHApO2ZvcihkPTAsZT10aGlzLmxlbmd0aDtkPGU7ZCsr",
				"KXtnPXRoaXNbZF07aWYoZy5ub2RlVHlwZT09PTEmJmcuY2xhc3NOYW1lKWlmKGEpe2g9KCIgIitnLmNs",
				"YXNzTmFtZSsiICIpLnJlcGxhY2UobywiICIpO2ZvcihpPTAsaj1jLmxlbmd0aDtpPGo7aSsrKWg9aC5y",
				"ZXBsYWNlKCIgIitjW2ldKyIgIiwiICIpO2cuY2xhc3NOYW1lPWYudHJpbShoKX1lbHNlIGcuY2xhc3NO",
				"YW1lPSIifX1yZXR1cm4gdGhpc30sdG9nZ2xlQ2xhc3M6ZnVuY3Rpb24oYSxiKXt2YXIgYz10eXBlb2Yg",
				"YSxkPXR5cGVvZiBiPT0iYm9vbGVhbiI7aWYoZi5pc0Z1bmN0aW9uKGEpKXJldHVybiB0aGlzLmVhY2go",
				"ZnVuY3Rpb24oYyl7Zih0aGlzKS50b2dnbGVDbGFzcyhhLmNhbGwodGhpcyxjLHRoaXMuY2xhc3NOYW1l",
				"LGIpLGIpfSk7cmV0dXJuIHRoaXMuZWFjaChmdW5jdGlvbigpe2lmKGM9PT0ic3RyaW5nIil7dmFyIGUs",
				"Zz0wLGg9Zih0aGlzKSxpPWIsaj1hLnNwbGl0KHApO3doaWxlKGU9altnKytdKWk9ZD9pOiFoLmhhc0Ns",
				"YXNzKGUpLGhbaT8iYWRkQ2xhc3MiOiJyZW1vdmVDbGFzcyJdKGUpfWVsc2UgaWYoYz09PSJ1bmRlZmlu",
				"ZWQifHxjPT09ImJvb2xlYW4iKXRoaXMuY2xhc3NOYW1lJiZmLl9kYXRhKHRoaXMsIl9fY2xhc3NOYW1l",
				"X18iLHRoaXMuY2xhc3NOYW1lKSx0aGlzLmNsYXNzTmFtZT10aGlzLmNsYXNzTmFtZXx8YT09PSExPyIi",
				"OmYuX2RhdGEodGhpcywiX19jbGFzc05hbWVfXyIpfHwiIn0pfSxoYXNDbGFzczpmdW5jdGlvbihhKXt2",
				"YXIgYj0iICIrYSsiICIsYz0wLGQ9dGhpcy5sZW5ndGg7Zm9yKDtjPGQ7YysrKWlmKHRoaXNbY10ubm9k",
				"ZVR5cGU9PT0xJiYoIiAiK3RoaXNbY10uY2xhc3NOYW1lKyIgIikucmVwbGFjZShvLCIgIikuaW5kZXhP",
				"ZihiKT4tMSlyZXR1cm4hMDtyZXR1cm4hMX0sdmFsOmZ1bmN0aW9uKGEpe3ZhciBjLGQsZSxnPXRoaXNb",
				"MF07e2lmKCEhYXJndW1lbnRzLmxlbmd0aCl7ZT1mLmlzRnVuY3Rpb24oYSk7cmV0dXJuIHRoaXMuZWFj",
				"aChmdW5jdGlvbihkKXt2YXIgZz1mKHRoaXMpLGg7aWYodGhpcy5ub2RlVHlwZT09PTEpe2U/aD1hLmNh",
				"bGwodGhpcyxkLGcudmFsKCkpOmg9YSxoPT1udWxsP2g9IiI6dHlwZW9mIGg9PSJudW1iZXIiP2grPSIi",
				"OmYuaXNBcnJheShoKSYmKGg9Zi5tYXAoaCxmdW5jdGlvbihhKXtyZXR1cm4gYT09bnVsbD8iIjphKyIi",
				"fSkpLGM9Zi52YWxIb29rc1t0aGlzLnR5cGVdfHxmLnZhbEhvb2tzW3RoaXMubm9kZU5hbWUudG9Mb3dl",
				"ckNhc2UoKV07aWYoIWN8fCEoInNldCJpbiBjKXx8Yy5zZXQodGhpcyxoLCJ2YWx1ZSIpPT09Yil0aGlz",
				"LnZhbHVlPWh9fSl9aWYoZyl7Yz1mLnZhbEhvb2tzW2cudHlwZV18fGYudmFsSG9va3NbZy5ub2RlTmFt",
				"ZS50b0xvd2VyQ2FzZSgpXTtpZihjJiYiZ2V0ImluIGMmJihkPWMuZ2V0KGcsInZhbHVlIikpIT09Yily",
				"ZXR1cm4gZDtkPWcudmFsdWU7cmV0dXJuIHR5cGVvZiBkPT0ic3RyaW5nIj9kLnJlcGxhY2UocSwiIik6",
				"ZD09bnVsbD8iIjpkfX19fSksZi5leHRlbmQoe3ZhbEhvb2tzOntvcHRpb246e2dldDpmdW5jdGlvbihh",
				"KXt2YXIgYj1hLmF0dHJpYnV0ZXMudmFsdWU7cmV0dXJuIWJ8fGIuc3BlY2lmaWVkP2EudmFsdWU6YS50",
				"ZXh0fX0sc2VsZWN0OntnZXQ6ZnVuY3Rpb24oYSl7dmFyIGIsYyxkLGUsZz1hLnNlbGVjdGVkSW5kZXgs",
				"aD1bXSxpPWEub3B0aW9ucyxqPWEudHlwZT09PSJzZWxlY3Qtb25lIjtpZihnPDApcmV0dXJuIG51bGw7",
				"Yz1qP2c6MCxkPWo/ZysxOmkubGVuZ3RoO2Zvcig7YzxkO2MrKyl7ZT1pW2NdO2lmKGUuc2VsZWN0ZWQm",
				"JihmLnN1cHBvcnQub3B0RGlzYWJsZWQ/IWUuZGlzYWJsZWQ6ZS5nZXRBdHRyaWJ1dGUoImRpc2FibGVk",
				"Iik9PT1udWxsKSYmKCFlLnBhcmVudE5vZGUuZGlzYWJsZWR8fCFmLm5vZGVOYW1lKGUucGFyZW50Tm9k",
				"ZSwib3B0Z3JvdXAiKSkpe2I9ZihlKS52YWwoKTtpZihqKXJldHVybiBiO2gucHVzaChiKX19aWYoaiYm",
				"IWgubGVuZ3RoJiZpLmxlbmd0aClyZXR1cm4gZihpW2ddKS52YWwoKTtyZXR1cm4gaH0sc2V0OmZ1bmN0",
				"aW9uKGEsYil7dmFyIGM9Zi5tYWtlQXJyYXkoYik7ZihhKS5maW5kKCJvcHRpb24iKS5lYWNoKGZ1bmN0",
				"aW9uKCl7dGhpcy5zZWxlY3RlZD1mLmluQXJyYXkoZih0aGlzKS52YWwoKSxjKT49MH0pLGMubGVuZ3Ro",
				"fHwoYS5zZWxlY3RlZEluZGV4PS0xKTtyZXR1cm4gY319fSxhdHRyRm46e3ZhbDohMCxjc3M6ITAsaHRt",
				"bDohMCx0ZXh0OiEwLGRhdGE6ITAsd2lkdGg6ITAsaGVpZ2h0OiEwLG9mZnNldDohMH0sYXR0cjpmdW5j",
				"dGlvbihhLGMsZCxlKXt2YXIgZyxoLGksaj1hLm5vZGVUeXBlO2lmKCEhYSYmaiE9PTMmJmohPT04JiZq",
				"IT09Mil7aWYoZSYmYyBpbiBmLmF0dHJGbilyZXR1cm4gZihhKVtjXShkKTtpZih0eXBlb2YgYS5nZXRB",
				"dHRyaWJ1dGU9PSJ1bmRlZmluZWQiKXJldHVybiBmLnByb3AoYSxjLGQpO2k9aiE9PTF8fCFmLmlzWE1M",
				"RG9jKGEpLGkmJihjPWMudG9Mb3dlckNhc2UoKSxoPWYuYXR0ckhvb2tzW2NdfHwodS50ZXN0KGMpP3g6",
				"dykpO2lmKGQhPT1iKXtpZihkPT09bnVsbCl7Zi5yZW1vdmVBdHRyKGEsYyk7cmV0dXJufWlmKGgmJiJz",
				"ZXQiaW4gaCYmaSYmKGc9aC5zZXQoYSxkLGMpKSE9PWIpcmV0dXJuIGc7YS5zZXRBdHRyaWJ1dGUoYywi",
				"IitkKTtyZXR1cm4gZH1pZihoJiYiZ2V0ImluIGgmJmkmJihnPWguZ2V0KGEsYykpIT09bnVsbClyZXR1",
				"cm4gZztnPWEuZ2V0QXR0cmlidXRlKGMpO3JldHVybiBnPT09bnVsbD9iOmd9fSxyZW1vdmVBdHRyOmZ1",
				"bmN0aW9uKGEsYil7dmFyIGMsZCxlLGcsaCxpPTA7aWYoYiYmYS5ub2RlVHlwZT09PTEpe2Q9Yi50b0xv",
				"d2VyQ2FzZSgpLnNwbGl0KHApLGc9ZC5sZW5ndGg7Zm9yKDtpPGc7aSsrKWU9ZFtpXSxlJiYoYz1mLnBy",
				"b3BGaXhbZV18fGUsaD11LnRlc3QoZSksaHx8Zi5hdHRyKGEsZSwiIiksYS5yZW1vdmVBdHRyaWJ1dGUo",
				"dj9lOmMpLGgmJmMgaW4gYSYmKGFbY109ITEpKX19LGF0dHJIb29rczp7dHlwZTp7c2V0OmZ1bmN0aW9u",
				"KGEsYil7aWYoci50ZXN0KGEubm9kZU5hbWUpJiZhLnBhcmVudE5vZGUpZi5lcnJvcigidHlwZSBwcm9w",
				"ZXJ0eSBjYW4ndCBiZSBjaGFuZ2VkIik7ZWxzZSBpZighZi5zdXBwb3J0LnJhZGlvVmFsdWUmJmI9PT0i",
				"cmFkaW8iJiZmLm5vZGVOYW1lKGEsImlucHV0Iikpe3ZhciBjPWEudmFsdWU7YS5zZXRBdHRyaWJ1dGUo",
				"InR5cGUiLGIpLGMmJihhLnZhbHVlPWMpO3JldHVybiBifX19LHZhbHVlOntnZXQ6ZnVuY3Rpb24oYSxi",
				"KXtpZih3JiZmLm5vZGVOYW1lKGEsImJ1dHRvbiIpKXJldHVybiB3LmdldChhLGIpO3JldHVybiBiIGlu",
				"IGE/YS52YWx1ZTpudWxsfSxzZXQ6ZnVuY3Rpb24oYSxiLGMpe2lmKHcmJmYubm9kZU5hbWUoYSwiYnV0",
				"dG9uIikpcmV0dXJuIHcuc2V0KGEsYixjKTthLnZhbHVlPWJ9fX0scHJvcEZpeDp7dGFiaW5kZXg6InRh",
				"YkluZGV4IixyZWFkb25seToicmVhZE9ubHkiLCJmb3IiOiJodG1sRm9yIiwiY2xhc3MiOiJjbGFzc05h",
				"bWUiLG1heGxlbmd0aDoibWF4TGVuZ3RoIixjZWxsc3BhY2luZzoiY2VsbFNwYWNpbmciLGNlbGxwYWRk",
				"aW5nOiJjZWxsUGFkZGluZyIscm93c3Bhbjoicm93U3BhbiIsY29sc3BhbjoiY29sU3BhbiIsdXNlbWFw",
				"OiJ1c2VNYXAiLGZyYW1lYm9yZGVyOiJmcmFtZUJvcmRlciIsY29udGVudGVkaXRhYmxlOiJjb250ZW50",
				"RWRpdGFibGUifSxwcm9wOmZ1bmN0aW9uKGEsYyxkKXt2YXIgZSxnLGgsaT1hLm5vZGVUeXBlO2lmKCEh",
				"YSYmaSE9PTMmJmkhPT04JiZpIT09Mil7aD1pIT09MXx8IWYuaXNYTUxEb2MoYSksaCYmKGM9Zi5wcm9w",
				"Rml4W2NdfHxjLGc9Zi5wcm9wSG9va3NbY10pO3JldHVybiBkIT09Yj9nJiYic2V0ImluIGcmJihlPWcu",
				"c2V0KGEsZCxjKSkhPT1iP2U6YVtjXT1kOmcmJiJnZXQiaW4gZyYmKGU9Zy5nZXQoYSxjKSkhPT1udWxs",
				"P2U6YVtjXX19LHByb3BIb29rczp7dGFiSW5kZXg6e2dldDpmdW5jdGlvbihhKXt2YXIgYz1hLmdldEF0",
				"dHJpYnV0ZU5vZGUoInRhYmluZGV4Iik7cmV0dXJuIGMmJmMuc3BlY2lmaWVkP3BhcnNlSW50KGMudmFs",
				"dWUsMTApOnMudGVzdChhLm5vZGVOYW1lKXx8dC50ZXN0KGEubm9kZU5hbWUpJiZhLmhyZWY/MDpifX19",
				"fSksZi5hdHRySG9va3MudGFiaW5kZXg9Zi5wcm9wSG9va3MudGFiSW5kZXgseD17Z2V0OmZ1bmN0aW9u",
				"KGEsYyl7dmFyIGQsZT1mLnByb3AoYSxjKTtyZXR1cm4gZT09PSEwfHx0eXBlb2YgZSE9ImJvb2xlYW4i",
				"JiYoZD1hLmdldEF0dHJpYnV0ZU5vZGUoYykpJiZkLm5vZGVWYWx1ZSE9PSExP2MudG9Mb3dlckNhc2Uo",
				"KTpifSxzZXQ6ZnVuY3Rpb24oYSxiLGMpe3ZhciBkO2I9PT0hMT9mLnJlbW92ZUF0dHIoYSxjKTooZD1m",
				"LnByb3BGaXhbY118fGMsZCBpbiBhJiYoYVtkXT0hMCksYS5zZXRBdHRyaWJ1dGUoYyxjLnRvTG93ZXJD",
				"YXNlKCkpKTtyZXR1cm4gY319LHZ8fCh5PXtuYW1lOiEwLGlkOiEwLGNvb3JkczohMH0sdz1mLnZhbEhv",
				"b2tzLmJ1dHRvbj17Z2V0OmZ1bmN0aW9uKGEsYyl7dmFyIGQ7ZD1hLmdldEF0dHJpYnV0ZU5vZGUoYyk7",
				"cmV0dXJuIGQmJih5W2NdP2Qubm9kZVZhbHVlIT09IiI6ZC5zcGVjaWZpZWQpP2Qubm9kZVZhbHVlOmJ9",
				"LHNldDpmdW5jdGlvbihhLGIsZCl7dmFyIGU9YS5nZXRBdHRyaWJ1dGVOb2RlKGQpO2V8fChlPWMuY3Jl",
				"YXRlQXR0cmlidXRlKGQpLGEuc2V0QXR0cmlidXRlTm9kZShlKSk7cmV0dXJuIGUubm9kZVZhbHVlPWIr",
				"IiJ9fSxmLmF0dHJIb29rcy50YWJpbmRleC5zZXQ9dy5zZXQsZi5lYWNoKFsid2lkdGgiLCJoZWlnaHQi",
				"XSxmdW5jdGlvbihhLGIpe2YuYXR0ckhvb2tzW2JdPWYuZXh0ZW5kKGYuYXR0ckhvb2tzW2JdLHtzZXQ6",
				"ZnVuY3Rpb24oYSxjKXtpZihjPT09IiIpe2Euc2V0QXR0cmlidXRlKGIsImF1dG8iKTtyZXR1cm4gY319",
				"fSl9KSxmLmF0dHJIb29rcy5jb250ZW50ZWRpdGFibGU9e2dldDp3LmdldCxzZXQ6ZnVuY3Rpb24oYSxi",
				"LGMpe2I9PT0iIiYmKGI9ImZhbHNlIiksdy5zZXQoYSxiLGMpfX0pLGYuc3VwcG9ydC5ocmVmTm9ybWFs",
				"aXplZHx8Zi5lYWNoKFsiaHJlZiIsInNyYyIsIndpZHRoIiwiaGVpZ2h0Il0sZnVuY3Rpb24oYSxjKXtm",
				"LmF0dHJIb29rc1tjXT1mLmV4dGVuZChmLmF0dHJIb29rc1tjXSx7Z2V0OmZ1bmN0aW9uKGEpe3ZhciBk",
				"PWEuZ2V0QXR0cmlidXRlKGMsMik7cmV0dXJuIGQ9PT1udWxsP2I6ZH19KX0pLGYuc3VwcG9ydC5zdHls",
				"ZXx8KGYuYXR0ckhvb2tzLnN0eWxlPXtnZXQ6ZnVuY3Rpb24oYSl7cmV0dXJuIGEuc3R5bGUuY3NzVGV4",
				"dC50b0xvd2VyQ2FzZSgpfHxifSxzZXQ6ZnVuY3Rpb24oYSxiKXtyZXR1cm4gYS5zdHlsZS5jc3NUZXh0",
				"PSIiK2J9fSksZi5zdXBwb3J0Lm9wdFNlbGVjdGVkfHwoZi5wcm9wSG9va3Muc2VsZWN0ZWQ9Zi5leHRl",
				"bmQoZi5wcm9wSG9va3Muc2VsZWN0ZWQse2dldDpmdW5jdGlvbihhKXt2YXIgYj1hLnBhcmVudE5vZGU7",
				"YiYmKGIuc2VsZWN0ZWRJbmRleCxiLnBhcmVudE5vZGUmJmIucGFyZW50Tm9kZS5zZWxlY3RlZEluZGV4",
				"KTtyZXR1cm4gbnVsbH19KSksZi5zdXBwb3J0LmVuY3R5cGV8fChmLnByb3BGaXguZW5jdHlwZT0iZW5j",
				"b2RpbmciKSxmLnN1cHBvcnQuY2hlY2tPbnx8Zi5lYWNoKFsicmFkaW8iLCJjaGVja2JveCJdLGZ1bmN0",
				"aW9uKCl7Zi52YWxIb29rc1t0aGlzXT17Z2V0OmZ1bmN0aW9uKGEpe3JldHVybiBhLmdldEF0dHJpYnV0",
				"ZSgidmFsdWUiKT09PW51bGw/Im9uIjphLnZhbHVlfX19KSxmLmVhY2goWyJyYWRpbyIsImNoZWNrYm94",
				"Il0sZnVuY3Rpb24oKXtmLnZhbEhvb2tzW3RoaXNdPWYuZXh0ZW5kKGYudmFsSG9va3NbdGhpc10se3Nl",
				"dDpmdW5jdGlvbihhLGIpe2lmKGYuaXNBcnJheShiKSlyZXR1cm4gYS5jaGVja2VkPWYuaW5BcnJheShm",
				"KGEpLnZhbCgpLGIpPj0wfX0pfSk7dmFyIHo9L14oPzp0ZXh0YXJlYXxpbnB1dHxzZWxlY3QpJC9pLEE9",
				"L14oW15cLl0qKT8oPzpcLiguKykpPyQvLEI9Lyg/Ol58XHMpaG92ZXIoXC5cUyspP1xiLyxDPS9ea2V5",
				"LyxEPS9eKD86bW91c2V8Y29udGV4dG1lbnUpfGNsaWNrLyxFPS9eKD86Zm9jdXNpbmZvY3VzfGZvY3Vz",
				"b3V0Ymx1cikkLyxGPS9eKFx3KikoPzojKFtcd1wtXSspKT8oPzpcLihbXHdcLV0rKSk/JC8sRz1mdW5j",
				"dGlvbigKYSl7dmFyIGI9Ri5leGVjKGEpO2ImJihiWzFdPShiWzFdfHwiIikudG9Mb3dlckNhc2UoKSxi",
				"WzNdPWJbM10mJm5ldyBSZWdFeHAoIig/Ol58XFxzKSIrYlszXSsiKD86XFxzfCQpIikpO3JldHVybiBi",
				"fSxIPWZ1bmN0aW9uKGEsYil7dmFyIGM9YS5hdHRyaWJ1dGVzfHx7fTtyZXR1cm4oIWJbMV18fGEubm9k",
				"ZU5hbWUudG9Mb3dlckNhc2UoKT09PWJbMV0pJiYoIWJbMl18fChjLmlkfHx7fSkudmFsdWU9PT1iWzJd",
				"KSYmKCFiWzNdfHxiWzNdLnRlc3QoKGNbImNsYXNzIl18fHt9KS52YWx1ZSkpfSxJPWZ1bmN0aW9uKGEp",
				"e3JldHVybiBmLmV2ZW50LnNwZWNpYWwuaG92ZXI/YTphLnJlcGxhY2UoQiwibW91c2VlbnRlciQxIG1v",
				"dXNlbGVhdmUkMSIpfTtmLmV2ZW50PXthZGQ6ZnVuY3Rpb24oYSxjLGQsZSxnKXt2YXIgaCxpLGosayxs",
				"LG0sbixvLHAscSxyLHM7aWYoIShhLm5vZGVUeXBlPT09M3x8YS5ub2RlVHlwZT09PTh8fCFjfHwhZHx8",
				"IShoPWYuX2RhdGEoYSkpKSl7ZC5oYW5kbGVyJiYocD1kLGQ9cC5oYW5kbGVyLGc9cC5zZWxlY3Rvciks",
				"ZC5ndWlkfHwoZC5ndWlkPWYuZ3VpZCsrKSxqPWguZXZlbnRzLGp8fChoLmV2ZW50cz1qPXt9KSxpPWgu",
				"aGFuZGxlLGl8fChoLmhhbmRsZT1pPWZ1bmN0aW9uKGEpe3JldHVybiB0eXBlb2YgZiE9InVuZGVmaW5l",
				"ZCImJighYXx8Zi5ldmVudC50cmlnZ2VyZWQhPT1hLnR5cGUpP2YuZXZlbnQuZGlzcGF0Y2guYXBwbHko",
				"aS5lbGVtLGFyZ3VtZW50cyk6Yn0saS5lbGVtPWEpLGM9Zi50cmltKEkoYykpLnNwbGl0KCIgIik7Zm9y",
				"KGs9MDtrPGMubGVuZ3RoO2srKyl7bD1BLmV4ZWMoY1trXSl8fFtdLG09bFsxXSxuPShsWzJdfHwiIiku",
				"c3BsaXQoIi4iKS5zb3J0KCkscz1mLmV2ZW50LnNwZWNpYWxbbV18fHt9LG09KGc/cy5kZWxlZ2F0ZVR5",
				"cGU6cy5iaW5kVHlwZSl8fG0scz1mLmV2ZW50LnNwZWNpYWxbbV18fHt9LG89Zi5leHRlbmQoe3R5cGU6",
				"bSxvcmlnVHlwZTpsWzFdLGRhdGE6ZSxoYW5kbGVyOmQsZ3VpZDpkLmd1aWQsc2VsZWN0b3I6ZyxxdWlj",
				"azpnJiZHKGcpLG5hbWVzcGFjZTpuLmpvaW4oIi4iKX0scCkscj1qW21dO2lmKCFyKXtyPWpbbV09W10s",
				"ci5kZWxlZ2F0ZUNvdW50PTA7aWYoIXMuc2V0dXB8fHMuc2V0dXAuY2FsbChhLGUsbixpKT09PSExKWEu",
				"YWRkRXZlbnRMaXN0ZW5lcj9hLmFkZEV2ZW50TGlzdGVuZXIobSxpLCExKTphLmF0dGFjaEV2ZW50JiZh",
				"LmF0dGFjaEV2ZW50KCJvbiIrbSxpKX1zLmFkZCYmKHMuYWRkLmNhbGwoYSxvKSxvLmhhbmRsZXIuZ3Vp",
				"ZHx8KG8uaGFuZGxlci5ndWlkPWQuZ3VpZCkpLGc/ci5zcGxpY2Uoci5kZWxlZ2F0ZUNvdW50KyssMCxv",
				"KTpyLnB1c2gobyksZi5ldmVudC5nbG9iYWxbbV09ITB9YT1udWxsfX0sZ2xvYmFsOnt9LHJlbW92ZTpm",
				"dW5jdGlvbihhLGIsYyxkLGUpe3ZhciBnPWYuaGFzRGF0YShhKSYmZi5fZGF0YShhKSxoLGksaixrLGws",
				"bSxuLG8scCxxLHIscztpZighIWcmJiEhKG89Zy5ldmVudHMpKXtiPWYudHJpbShJKGJ8fCIiKSkuc3Bs",
				"aXQoIiAiKTtmb3IoaD0wO2g8Yi5sZW5ndGg7aCsrKXtpPUEuZXhlYyhiW2hdKXx8W10saj1rPWlbMV0s",
				"bD1pWzJdO2lmKCFqKXtmb3IoaiBpbiBvKWYuZXZlbnQucmVtb3ZlKGEsaitiW2hdLGMsZCwhMCk7Y29u",
				"dGludWV9cD1mLmV2ZW50LnNwZWNpYWxbal18fHt9LGo9KGQ/cC5kZWxlZ2F0ZVR5cGU6cC5iaW5kVHlw",
				"ZSl8fGoscj1vW2pdfHxbXSxtPXIubGVuZ3RoLGw9bD9uZXcgUmVnRXhwKCIoXnxcXC4pIitsLnNwbGl0",
				"KCIuIikuc29ydCgpLmpvaW4oIlxcLig/Oi4qXFwuKT8iKSsiKFxcLnwkKSIpOm51bGw7Zm9yKG49MDtu",
				"PHIubGVuZ3RoO24rKylzPXJbbl0sKGV8fGs9PT1zLm9yaWdUeXBlKSYmKCFjfHxjLmd1aWQ9PT1zLmd1",
				"aWQpJiYoIWx8fGwudGVzdChzLm5hbWVzcGFjZSkpJiYoIWR8fGQ9PT1zLnNlbGVjdG9yfHxkPT09Iioq",
				"IiYmcy5zZWxlY3RvcikmJihyLnNwbGljZShuLS0sMSkscy5zZWxlY3RvciYmci5kZWxlZ2F0ZUNvdW50",
				"LS0scC5yZW1vdmUmJnAucmVtb3ZlLmNhbGwoYSxzKSk7ci5sZW5ndGg9PT0wJiZtIT09ci5sZW5ndGgm",
				"JigoIXAudGVhcmRvd258fHAudGVhcmRvd24uY2FsbChhLGwpPT09ITEpJiZmLnJlbW92ZUV2ZW50KGEs",
				"aixnLmhhbmRsZSksZGVsZXRlIG9bal0pfWYuaXNFbXB0eU9iamVjdChvKSYmKHE9Zy5oYW5kbGUscSYm",
				"KHEuZWxlbT1udWxsKSxmLnJlbW92ZURhdGEoYSxbImV2ZW50cyIsImhhbmRsZSJdLCEwKSl9fSxjdXN0",
				"b21FdmVudDp7Z2V0RGF0YTohMCxzZXREYXRhOiEwLGNoYW5nZURhdGE6ITB9LHRyaWdnZXI6ZnVuY3Rp",
				"b24oYyxkLGUsZyl7aWYoIWV8fGUubm9kZVR5cGUhPT0zJiZlLm5vZGVUeXBlIT09OCl7dmFyIGg9Yy50",
				"eXBlfHxjLGk9W10saixrLGwsbSxuLG8scCxxLHIscztpZihFLnRlc3QoaCtmLmV2ZW50LnRyaWdnZXJl",
				"ZCkpcmV0dXJuO2guaW5kZXhPZigiISIpPj0wJiYoaD1oLnNsaWNlKDAsLTEpLGs9ITApLGguaW5kZXhP",
				"ZigiLiIpPj0wJiYoaT1oLnNwbGl0KCIuIiksaD1pLnNoaWZ0KCksaS5zb3J0KCkpO2lmKCghZXx8Zi5l",
				"dmVudC5jdXN0b21FdmVudFtoXSkmJiFmLmV2ZW50Lmdsb2JhbFtoXSlyZXR1cm47Yz10eXBlb2YgYz09",
				"Im9iamVjdCI/Y1tmLmV4cGFuZG9dP2M6bmV3IGYuRXZlbnQoaCxjKTpuZXcgZi5FdmVudChoKSxjLnR5",
				"cGU9aCxjLmlzVHJpZ2dlcj0hMCxjLmV4Y2x1c2l2ZT1rLGMubmFtZXNwYWNlPWkuam9pbigiLiIpLGMu",
				"bmFtZXNwYWNlX3JlPWMubmFtZXNwYWNlP25ldyBSZWdFeHAoIihefFxcLikiK2kuam9pbigiXFwuKD86",
				"LipcXC4pPyIpKyIoXFwufCQpIik6bnVsbCxvPWguaW5kZXhPZigiOiIpPDA/Im9uIitoOiIiO2lmKCFl",
				"KXtqPWYuY2FjaGU7Zm9yKGwgaW4gailqW2xdLmV2ZW50cyYmaltsXS5ldmVudHNbaF0mJmYuZXZlbnQu",
				"dHJpZ2dlcihjLGQsaltsXS5oYW5kbGUuZWxlbSwhMCk7cmV0dXJufWMucmVzdWx0PWIsYy50YXJnZXR8",
				"fChjLnRhcmdldD1lKSxkPWQhPW51bGw/Zi5tYWtlQXJyYXkoZCk6W10sZC51bnNoaWZ0KGMpLHA9Zi5l",
				"dmVudC5zcGVjaWFsW2hdfHx7fTtpZihwLnRyaWdnZXImJnAudHJpZ2dlci5hcHBseShlLGQpPT09ITEp",
				"cmV0dXJuO3I9W1tlLHAuYmluZFR5cGV8fGhdXTtpZighZyYmIXAubm9CdWJibGUmJiFmLmlzV2luZG93",
				"KGUpKXtzPXAuZGVsZWdhdGVUeXBlfHxoLG09RS50ZXN0KHMraCk/ZTplLnBhcmVudE5vZGUsbj1udWxs",
				"O2Zvcig7bTttPW0ucGFyZW50Tm9kZSlyLnB1c2goW20sc10pLG49bTtuJiZuPT09ZS5vd25lckRvY3Vt",
				"ZW50JiZyLnB1c2goW24uZGVmYXVsdFZpZXd8fG4ucGFyZW50V2luZG93fHxhLHNdKX1mb3IobD0wO2w8",
				"ci5sZW5ndGgmJiFjLmlzUHJvcGFnYXRpb25TdG9wcGVkKCk7bCsrKW09cltsXVswXSxjLnR5cGU9clts",
				"XVsxXSxxPShmLl9kYXRhKG0sImV2ZW50cyIpfHx7fSlbYy50eXBlXSYmZi5fZGF0YShtLCJoYW5kbGUi",
				"KSxxJiZxLmFwcGx5KG0sZCkscT1vJiZtW29dLHEmJmYuYWNjZXB0RGF0YShtKSYmcS5hcHBseShtLGQp",
				"PT09ITEmJmMucHJldmVudERlZmF1bHQoKTtjLnR5cGU9aCwhZyYmIWMuaXNEZWZhdWx0UHJldmVudGVk",
				"KCkmJighcC5fZGVmYXVsdHx8cC5fZGVmYXVsdC5hcHBseShlLm93bmVyRG9jdW1lbnQsZCk9PT0hMSkm",
				"JihoIT09ImNsaWNrInx8IWYubm9kZU5hbWUoZSwiYSIpKSYmZi5hY2NlcHREYXRhKGUpJiZvJiZlW2hd",
				"JiYoaCE9PSJmb2N1cyImJmghPT0iYmx1ciJ8fGMudGFyZ2V0Lm9mZnNldFdpZHRoIT09MCkmJiFmLmlz",
				"V2luZG93KGUpJiYobj1lW29dLG4mJihlW29dPW51bGwpLGYuZXZlbnQudHJpZ2dlcmVkPWgsZVtoXSgp",
				"LGYuZXZlbnQudHJpZ2dlcmVkPWIsbiYmKGVbb109bikpO3JldHVybiBjLnJlc3VsdH19LGRpc3BhdGNo",
				"OmZ1bmN0aW9uKGMpe2M9Zi5ldmVudC5maXgoY3x8YS5ldmVudCk7dmFyIGQ9KGYuX2RhdGEodGhpcywi",
				"ZXZlbnRzIil8fHt9KVtjLnR5cGVdfHxbXSxlPWQuZGVsZWdhdGVDb3VudCxnPVtdLnNsaWNlLmNhbGwo",
				"YXJndW1lbnRzLDApLGg9IWMuZXhjbHVzaXZlJiYhYy5uYW1lc3BhY2UsaT1mLmV2ZW50LnNwZWNpYWxb",
				"Yy50eXBlXXx8e30saj1bXSxrLGwsbSxuLG8scCxxLHIscyx0LHU7Z1swXT1jLGMuZGVsZWdhdGVUYXJn",
				"ZXQ9dGhpcztpZighaS5wcmVEaXNwYXRjaHx8aS5wcmVEaXNwYXRjaC5jYWxsKHRoaXMsYykhPT0hMSl7",
				"aWYoZSYmKCFjLmJ1dHRvbnx8Yy50eXBlIT09ImNsaWNrIikpe249Zih0aGlzKSxuLmNvbnRleHQ9dGhp",
				"cy5vd25lckRvY3VtZW50fHx0aGlzO2ZvcihtPWMudGFyZ2V0O20hPXRoaXM7bT1tLnBhcmVudE5vZGV8",
				"fHRoaXMpaWYobS5kaXNhYmxlZCE9PSEwKXtwPXt9LHI9W10sblswXT1tO2ZvcihrPTA7azxlO2srKylz",
				"PWRba10sdD1zLnNlbGVjdG9yLHBbdF09PT1iJiYocFt0XT1zLnF1aWNrP0gobSxzLnF1aWNrKTpuLmlz",
				"KHQpKSxwW3RdJiZyLnB1c2gocyk7ci5sZW5ndGgmJmoucHVzaCh7ZWxlbTptLG1hdGNoZXM6cn0pfX1k",
				"Lmxlbmd0aD5lJiZqLnB1c2goe2VsZW06dGhpcyxtYXRjaGVzOmQuc2xpY2UoZSl9KTtmb3Ioaz0wO2s8",
				"ai5sZW5ndGgmJiFjLmlzUHJvcGFnYXRpb25TdG9wcGVkKCk7aysrKXtxPWpba10sYy5jdXJyZW50VGFy",
				"Z2V0PXEuZWxlbTtmb3IobD0wO2w8cS5tYXRjaGVzLmxlbmd0aCYmIWMuaXNJbW1lZGlhdGVQcm9wYWdh",
				"dGlvblN0b3BwZWQoKTtsKyspe3M9cS5tYXRjaGVzW2xdO2lmKGh8fCFjLm5hbWVzcGFjZSYmIXMubmFt",
				"ZXNwYWNlfHxjLm5hbWVzcGFjZV9yZSYmYy5uYW1lc3BhY2VfcmUudGVzdChzLm5hbWVzcGFjZSkpYy5k",
				"YXRhPXMuZGF0YSxjLmhhbmRsZU9iaj1zLG89KChmLmV2ZW50LnNwZWNpYWxbcy5vcmlnVHlwZV18fHt9",
				"KS5oYW5kbGV8fHMuaGFuZGxlcikuYXBwbHkocS5lbGVtLGcpLG8hPT1iJiYoYy5yZXN1bHQ9byxvPT09",
				"ITEmJihjLnByZXZlbnREZWZhdWx0KCksYy5zdG9wUHJvcGFnYXRpb24oKSkpfX1pLnBvc3REaXNwYXRj",
				"aCYmaS5wb3N0RGlzcGF0Y2guY2FsbCh0aGlzLGMpO3JldHVybiBjLnJlc3VsdH19LHByb3BzOiJhdHRy",
				"Q2hhbmdlIGF0dHJOYW1lIHJlbGF0ZWROb2RlIHNyY0VsZW1lbnQgYWx0S2V5IGJ1YmJsZXMgY2FuY2Vs",
				"YWJsZSBjdHJsS2V5IGN1cnJlbnRUYXJnZXQgZXZlbnRQaGFzZSBtZXRhS2V5IHJlbGF0ZWRUYXJnZXQg",
				"c2hpZnRLZXkgdGFyZ2V0IHRpbWVTdGFtcCB2aWV3IHdoaWNoIi5zcGxpdCgiICIpLGZpeEhvb2tzOnt9",
				"LGtleUhvb2tzOntwcm9wczoiY2hhciBjaGFyQ29kZSBrZXkga2V5Q29kZSIuc3BsaXQoIiAiKSxmaWx0",
				"ZXI6ZnVuY3Rpb24oYSxiKXthLndoaWNoPT1udWxsJiYoYS53aGljaD1iLmNoYXJDb2RlIT1udWxsP2Iu",
				"Y2hhckNvZGU6Yi5rZXlDb2RlKTtyZXR1cm4gYX19LG1vdXNlSG9va3M6e3Byb3BzOiJidXR0b24gYnV0",
				"dG9ucyBjbGllbnRYIGNsaWVudFkgZnJvbUVsZW1lbnQgb2Zmc2V0WCBvZmZzZXRZIHBhZ2VYIHBhZ2VZ",
				"IHNjcmVlblggc2NyZWVuWSB0b0VsZW1lbnQiLnNwbGl0KCIgIiksZmlsdGVyOmZ1bmN0aW9uKGEsZCl7",
				"dmFyIGUsZixnLGg9ZC5idXR0b24saT1kLmZyb21FbGVtZW50O2EucGFnZVg9PW51bGwmJmQuY2xpZW50",
				"WCE9bnVsbCYmKGU9YS50YXJnZXQub3duZXJEb2N1bWVudHx8YyxmPWUuZG9jdW1lbnRFbGVtZW50LGc9",
				"ZS5ib2R5LGEucGFnZVg9ZC5jbGllbnRYKyhmJiZmLnNjcm9sbExlZnR8fGcmJmcuc2Nyb2xsTGVmdHx8",
				"MCktKGYmJmYuY2xpZW50TGVmdHx8ZyYmZy5jbGllbnRMZWZ0fHwwKSxhLnBhZ2VZPWQuY2xpZW50WSso",
				"ZiYmZi5zY3JvbGxUb3B8fGcmJmcuc2Nyb2xsVG9wfHwwKS0oZiYmZi5jbGllbnRUb3B8fGcmJmcuY2xp",
				"ZW50VG9wfHwwKSksIWEucmVsYXRlZFRhcmdldCYmaSYmKGEucmVsYXRlZFRhcmdldD1pPT09YS50YXJn",
				"ZXQ/ZC50b0VsZW1lbnQ6aSksIWEud2hpY2gmJmghPT1iJiYoYS53aGljaD1oJjE/MTpoJjI/MzpoJjQ/",
				"MjowKTtyZXR1cm4gYX19LGZpeDpmdW5jdGlvbihhKXtpZihhW2YuZXhwYW5kb10pcmV0dXJuIGE7dmFy",
				"IGQsZSxnPWEsaD1mLmV2ZW50LmZpeEhvb2tzW2EudHlwZV18fHt9LGk9aC5wcm9wcz90aGlzLnByb3Bz",
				"LmNvbmNhdChoLnByb3BzKTp0aGlzLnByb3BzO2E9Zi5FdmVudChnKTtmb3IoZD1pLmxlbmd0aDtkOyll",
				"PWlbLS1kXSxhW2VdPWdbZV07YS50YXJnZXR8fChhLnRhcmdldD1nLnNyY0VsZW1lbnR8fGMpLGEudGFy",
				"Z2V0Lm5vZGVUeXBlPT09MyYmKGEudGFyZ2V0PWEudGFyZ2V0LnBhcmVudE5vZGUpLGEubWV0YUtleT09",
				"PWImJihhLm1ldGFLZXk9YS5jdHJsS2V5KTtyZXR1cm4gaC5maWx0ZXI/aC5maWx0ZXIoYSxnKTphfSxz",
				"cGVjaWFsOntyZWFkeTp7c2V0dXA6Zi5iaW5kUmVhZHl9LGxvYWQ6e25vQnViYmxlOiEwfSxmb2N1czp7",
				"ZGVsZWdhdGVUeXBlOiJmb2N1c2luIn0sYmx1cjp7ZGVsZWdhdGVUeXBlOiJmb2N1c291dCJ9LGJlZm9y",
				"ZXVubG9hZDp7c2V0dXA6ZnVuY3Rpb24oYSxiLGMpe2YuaXNXaW5kb3codGhpcykmJih0aGlzLm9uYmVm",
				"b3JldW5sb2FkPWMpfSx0ZWFyZG93bjpmdW5jdGlvbihhLGIpe3RoaXMub25iZWZvcmV1bmxvYWQ9PT1i",
				"JiYodGhpcy5vbmJlZm9yZXVubG9hZD1udWxsKX19fSxzaW11bGF0ZTpmdW5jdGlvbihhLGIsYyxkKXt2",
				"YXIgZT1mLmV4dGVuZChuZXcgZi5FdmVudCxjLHt0eXBlOmEsaXNTaW11bGF0ZWQ6ITAsb3JpZ2luYWxF",
				"dmVudDp7fX0pO2Q/Zi5ldmVudC50cmlnZ2VyKGUsbnVsbCxiKTpmLmV2ZW50LmRpc3BhdGNoLmNhbGwo",
				"YixlKSxlLmlzRGVmYXVsdFByZXZlbnRlZCgpJiZjLnByZXZlbnREZWZhdWx0KCl9fSxmLmV2ZW50Lmhh",
				"bmRsZT1mLmV2ZW50LmRpc3BhdGNoLGYucmVtb3ZlRXZlbnQ9Yy5yZW1vdmVFdmVudExpc3RlbmVyP2Z1",
				"bmN0aW9uKGEsYixjKXthLnJlbW92ZUV2ZW50TGlzdGVuZXImJmEucmVtb3ZlRXZlbnRMaXN0ZW5lcihi",
				"LGMsITEpfTpmdW5jdGlvbihhLGIsYyl7YS5kZXRhY2hFdmVudCYmYS5kZXRhY2hFdmVudCgib24iK2Is",
				"Yyl9LGYuRXZlbnQ9ZnVuY3Rpb24oYSxiKXtpZighKHRoaXMgaW5zdGFuY2VvZiBmLkV2ZW50KSlyZXR1",
				"cm4gbmV3IGYuRXZlbnQoYSxiKTthJiZhLnR5cGU/KHRoaXMub3JpZ2luYWxFdmVudD1hLHRoaXMudHlw",
				"ZT1hLnR5cGUsdGhpcy5pc0RlZmF1bHRQcmV2ZW50ZWQ9YS5kZWZhdWx0UHJldmVudGVkfHxhLnJldHVy",
				"blZhbHVlPT09ITF8fGEuZ2V0UHJldmVudERlZmF1bHQmJmEuZ2V0UHJldmVudERlZmF1bHQoKT9LOkop",
				"OnRoaXMudHlwZT1hLGImJmYuZXh0ZW5kKHRoaXMsYiksdGhpcy50aW1lU3RhbXA9YSYmYS50aW1lU3Rh",
				"bXB8fGYubm93KCksdGhpc1tmLmV4cGFuZG9dPSEwfSxmLkV2ZW50LnByb3RvdHlwZT17cHJldmVudERl",
				"ZmF1bHQ6ZnVuY3Rpb24oKXt0aGlzLmlzRGVmYXVsdFByZXZlbnRlZD1LO3ZhciBhPXRoaXMub3JpZ2lu",
				"YWxFdmVudDshYXx8KGEucHJldmVudERlZmF1bHQ/YS5wcmV2ZW50RGVmYXVsdCgpOmEucmV0dXJuVmFs",
				"dWU9ITEpfSxzdG9wUHJvcGFnYXRpb246ZnVuY3Rpb24oKXt0aGlzLmlzUHJvcGFnYXRpb25TdG9wcGVk",
				"PUs7dmFyIGE9dGhpcy5vcmlnaW5hbEV2ZW50OyFhfHwoYS5zdG9wUHJvcGFnYXRpb24mJmEuc3RvcFBy",
				"b3BhZ2F0aW9uKCksYS5jYW5jZWxCdWJibGU9ITApfSxzdG9wSW1tZWRpYXRlUHJvcGFnYXRpb246ZnVu",
				"Y3Rpb24oKXt0aGlzLmlzSW1tZWRpYXRlUHJvcGFnYXRpb25TdG9wcGVkPUssdGhpcy5zdG9wUHJvcGFn",
				"YXRpb24oKX0saXNEZWZhdWx0UHJldmVudGVkOkosaXNQcm9wYWdhdGlvblN0b3BwZWQ6Sixpc0ltbWVk",
				"aWF0ZVByb3BhZ2F0aW9uU3RvcHBlZDpKfSxmLmVhY2goe21vdXNlZW50ZXI6Im1vdXNlb3ZlciIsbW91",
				"c2VsZWF2ZToibW91c2VvdXQifSxmdW5jdGlvbihhLGIpe2YuZXZlbnQuc3BlY2lhbFthXT17ZGVsZWdh",
				"dGVUeXBlOmIsYmluZFR5cGU6YixoYW5kbGU6ZnVuY3Rpb24oYSl7dmFyIGM9dGhpcyxkPWEucmVsYXRl",
				"ZFRhcmdldCxlPWEuaGFuZGxlT2JqLGc9ZS5zZWxlY3RvcixoO2lmKCFkfHxkIT09YyYmIWYuY29udGFp",
				"bnMoYyxkKSlhLnR5cGU9ZS5vcmlnVHlwZSxoPWUuaGFuZGxlci5hcHBseSh0aGlzLGFyZ3VtZW50cyks",
				"YS50eXBlPWI7cmV0dXJuIGh9fX0pLGYuc3VwcG9ydC5zdWJtaXRCdWJibGVzfHwoZi5ldmVudC5zcGVj",
				"aWFsLnN1Ym1pdD17c2V0dXA6ZnVuY3Rpb24oKXtpZihmLm5vZGVOYW1lKHRoaXMsImZvcm0iKSlyZXR1",
				"cm4hMTtmLmV2ZW50LmFkZCh0aGlzLCJjbGljay5fc3VibWl0IGtleXByZXNzLl9zdWJtaXQiLGZ1bmN0",
				"aW9uKGEpe3ZhciBjPWEudGFyZ2V0LGQ9Zi5ub2RlTmFtZShjLCJpbnB1dCIpfHxmLm5vZGVOYW1lKGMs",
				"ImJ1dHRvbiIpP2MuZm9ybTpiO2QmJiFkLl9zdWJtaXRfYXR0YWNoZWQmJihmLmV2ZW50LmFkZChkLCJz",
				"dWJtaXQuX3N1Ym1pdCIsZnVuY3Rpb24oYSl7YS5fc3VibWl0X2J1YmJsZT0hMH0pLGQuX3N1Ym1pdF9h",
				"dHRhY2hlZD0hMCl9KX0scG9zdERpc3BhdGNoOmZ1bmN0aW9uKGEpe2EuX3N1Ym1pdF9idWJibGUmJihk",
				"ZWxldGUgYS5fc3VibWl0X2J1YmJsZSx0aGlzLnBhcmVudE5vZGUmJiFhLmlzVHJpZ2dlciYmZi5ldmVu",
				"dC5zaW11bGF0ZSgic3VibWl0Iix0aGlzLnBhcmVudE5vZGUsYSwhMCkpfSx0ZWFyZG93bjpmdW5jdGlv",
				"bigpe2lmKGYubm9kZU5hbWUodGhpcywiZm9ybSIpKXJldHVybiExO2YuZXZlbnQucmVtb3ZlKHRoaXMs",
				"Ii5fc3VibWl0Iil9fSksZi5zdXBwb3J0LmNoYW5nZUJ1YmJsZXN8fChmLmV2ZW50LnNwZWNpYWwuY2hh",
				"bmdlPXtzZXR1cDpmdW5jdGlvbigpe2lmKHoudGVzdCh0aGlzLm5vZGVOYW1lKSl7aWYodGhpcy50eXBl",
				"PT09ImNoZWNrYm94Inx8dGhpcy50eXBlPT09InJhZGlvIilmLmV2ZW50LmFkZCh0aGlzLCJwcm9wZXJ0",
				"eWNoYW5nZS5fY2hhbmdlIixmdW5jdGlvbihhKXthLm9yaWdpbmFsRXZlbnQucHJvcGVydHlOYW1lPT09",
				"ImNoZWNrZWQiJiYodGhpcy5fanVzdF9jaGFuZ2VkPSEwKX0pLGYuZXZlbnQuYWRkKHRoaXMsImNsaWNr",
				"Ll9jaGFuZ2UiLGZ1bmN0aW9uKGEpe3RoaXMuX2p1c3RfY2hhbmdlZCYmIWEuaXNUcmlnZ2VyJiYodGhp",
				"cy5fanVzdF9jaGFuZ2VkPSExLGYuZXZlbnQuc2ltdWxhdGUoImNoYW5nZSIsdGhpcyxhLCEwKSl9KTty",
				"ZXR1cm4hMX1mLmV2ZW50LmFkZCh0aGlzLCJiZWZvcmVhY3RpdmF0ZS5fY2hhbmdlIixmdW5jdGlvbihh",
				"KXt2YXIgYj1hLnRhcmdldDt6LnRlc3QoYi5ub2RlTmFtZSkmJiFiLl9jaGFuZ2VfYXR0YWNoZWQmJihm",
				"LmV2ZW50LmFkZChiLCJjaGFuZ2UuX2NoYW5nZSIsZnVuY3Rpb24oYSl7dGhpcy5wYXJlbnROb2RlJiYh",
				"YS5pc1NpbXVsYXRlZCYmIWEuaXNUcmlnZ2VyJiZmLmV2ZW50LnNpbXVsYXRlKCJjaGFuZ2UiLHRoaXMu",
				"cGFyZW50Tm9kZSxhLCEwKX0pLGIuX2NoYW5nZV9hdHRhY2hlZD0hMCl9KX0saGFuZGxlOmZ1bmN0aW9u",
				"KGEpe3ZhciBiPWEudGFyZ2V0O2lmKHRoaXMhPT1ifHxhLmlzU2ltdWxhdGVkfHxhLmlzVHJpZ2dlcnx8",
				"Yi50eXBlIT09InJhZGlvIiYmYi50eXBlIT09ImNoZWNrYm94IilyZXR1cm4gYS5oYW5kbGVPYmouaGFu",
				"ZGxlci5hcHBseSh0aGlzLGFyZ3VtZW50cyl9LHRlYXJkb3duOmZ1bmN0aW9uKCl7Zi5ldmVudC5yZW1v",
				"dmUodGhpcywiLl9jaGFuZ2UiKTtyZXR1cm4gei50ZXN0KHRoaXMubm9kZU5hbWUpfX0pLGYuc3VwcG9y",
				"dC5mb2N1c2luQnViYmxlc3x8Zi5lYWNoKHtmb2N1czoiZm9jdXNpbiIsYmx1cjoiZm9jdXNvdXQifSxm",
				"dW5jdGlvbihhLGIpe3ZhciBkPTAsZT1mdW5jdGlvbihhKXtmLmV2ZW50LnNpbXVsYXRlKGIsYS50YXJn",
				"ZXQsZi5ldmVudC5maXgoYSksITApfTtmLmV2ZW50LnNwZWNpYWxbYl09e3NldHVwOmZ1bmN0aW9uKCl7",
				"ZCsrPT09MCYmYy5hZGRFdmVudExpc3RlbmVyKGEsZSwhMCl9LHRlYXJkb3duOmZ1bmN0aW9uKCl7LS1k",
				"PT09MCYmYy5yZW1vdmVFdmVudExpc3RlbmVyKGEsZSwhMCl9fX0pLGYuZm4uZXh0ZW5kKHtvbjpmdW5j",
				"dGlvbihhLGMsZCxlLGcpe3ZhciBoLGk7aWYodHlwZW9mIGE9PSJvYmplY3QiKXt0eXBlb2YgYyE9InN0",
				"cmluZyImJihkPWR8fGMsYz1iKTtmb3IoaSBpbiBhKXRoaXMub24oaSxjLGQsYVtpXSxnKTtyZXR1cm4g",
				"dGhpc31kPT1udWxsJiZlPT1udWxsPyhlPWMsZD1jPWIpOmU9PW51bGwmJih0eXBlb2YgYz09InN0cmlu",
				"ZyI/KGU9ZCxkPWIpOihlPWQsZD1jLGM9YikpO2lmKGU9PT0hMSllPUo7ZWxzZSBpZighZSlyZXR1cm4g",
				"dGhpcztnPT09MSYmKGg9ZSxlPWZ1bmN0aW9uKGEpe2YoKS5vZmYoYSk7cmV0dXJuIGguYXBwbHkodGhp",
				"cyxhcmd1bWVudHMpfSxlLmd1aWQ9aC5ndWlkfHwoaC5ndWlkPWYuZ3VpZCsrKSk7cmV0dXJuIHRoaXMu",
				"ZWFjaChmdW5jdGlvbigpe2YuZXZlbnQuYWRkKHRoaXMsYSxlLGQsYyl9KX0sb25lOmZ1bmN0aW9uKGEs",
				"YixjLGQpe3JldHVybiB0aGlzLm9uKGEsYixjLGQsMSl9LG9mZjpmdW5jdGlvbihhLGMsZCl7aWYoYSYm",
				"YS5wcmV2ZW50RGVmYXVsdCYmYS5oYW5kbGVPYmope3ZhciBlPWEuaGFuZGxlT2JqO2YoYS5kZWxlZ2F0",
				"ZVRhcmdldCkub2ZmKGUubmFtZXNwYWNlP2Uub3JpZ1R5cGUrIi4iK2UubmFtZXNwYWNlOmUub3JpZ1R5",
				"cGUsZS5zZWxlY3RvcixlLmhhbmRsZXIpO3JldHVybiB0aGlzfWlmKHR5cGVvZiBhPT0ib2JqZWN0Iil7",
				"Zm9yKHZhciBnIGluIGEpdGhpcy5vZmYoZyxjLGFbZ10pO3JldHVybiB0aGlzfWlmKGM9PT0hMXx8dHlw",
				"ZW9mIGM9PSJmdW5jdGlvbiIpZD1jLGM9YjtkPT09ITEmJihkPUopO3JldHVybiB0aGlzLmVhY2goZnVu",
				"Y3Rpb24oKXtmLmV2ZW50LnJlbW92ZSh0aGlzLGEsZCxjKX0pfSxiaW5kOmZ1bmN0aW9uKGEsYixjKXty",
				"ZXR1cm4gdGhpcy5vbihhLG51bGwsYixjKX0sdW5iaW5kOmZ1bmN0aW9uKGEsYil7cmV0dXJuIHRoaXMu",
				"b2ZmKGEsbnVsbCxiKX0sbGl2ZTpmdW5jdGlvbihhLGIsYyl7Zih0aGlzLmNvbnRleHQpLm9uKGEsdGhp",
				"cy5zZWxlY3RvcixiLGMpO3JldHVybiB0aGlzfSxkaWU6ZnVuY3Rpb24oYSxiKXtmKHRoaXMuY29udGV4",
				"dCkub2ZmKGEsdGhpcy5zZWxlY3Rvcnx8IioqIixiKTtyZXR1cm4gdGhpc30sZGVsZWdhdGU6ZnVuY3Rp",
				"b24oYSxiLGMsZCl7cmV0dXJuIHRoaXMub24oYixhLGMsZCl9LHVuZGVsZWdhdGU6ZnVuY3Rpb24oYSxi",
				"LGMpe3JldHVybiBhcmd1bWVudHMubGVuZ3RoPT0xP3RoaXMub2ZmKGEsIioqIik6dGhpcy5vZmYoYixh",
				"LGMpfSx0cmlnZ2VyOmZ1bmN0aW9uKGEsYil7cmV0dXJuIHRoaXMuZWFjaChmdW5jdGlvbigpe2YuZXZl",
				"bnQudHJpZ2dlcihhLGIsdGhpcyl9KX0sdHJpZ2dlckhhbmRsZXI6ZnVuY3Rpb24oYSxiKXtpZih0aGlz",
				"WzBdKXJldHVybiBmLmV2ZW50LnRyaWdnZXIoYSxiLHRoaXNbMF0sITApfSx0b2dnbGU6ZnVuY3Rpb24o",
				"YSl7dmFyIGI9YXJndW1lbnRzLGM9YS5ndWlkfHxmLmd1aWQrKyxkPTAsZT1mdW5jdGlvbihjKXt2YXIg",
				"ZT0oZi5fZGF0YSh0aGlzLCJsYXN0VG9nZ2xlIithLmd1aWQpfHwwKSVkO2YuX2RhdGEodGhpcywibGFz",
				"dFRvZ2dsZSIrYS5ndWlkLGUrMSksYy5wcmV2ZW50RGVmYXVsdCgpO3JldHVybiBiW2VdLmFwcGx5KHRo",
				"aXMsYXJndW1lbnRzKXx8ITF9O2UuZ3VpZD1jO3doaWxlKGQ8Yi5sZW5ndGgpYltkKytdLmd1aWQ9Yzty",
				"ZXR1cm4gdGhpcy5jbGljayhlKX0saG92ZXI6ZnVuY3Rpb24oYSxiKXtyZXR1cm4gdGhpcy5tb3VzZWVu",
				"dGVyKGEpLm1vdXNlbGVhdmUoYnx8YSl9fSksZi5lYWNoKCJibHVyIGZvY3VzIGZvY3VzaW4gZm9jdXNv",
				"dXQgbG9hZCByZXNpemUgc2Nyb2xsIHVubG9hZCBjbGljayBkYmxjbGljayBtb3VzZWRvd24gbW91c2V1",
				"cCBtb3VzZW1vdmUgbW91c2VvdmVyIG1vdXNlb3V0IG1vdXNlZW50ZXIgbW91c2VsZWF2ZSBjaGFuZ2Ug",
				"c2VsZWN0IHN1Ym1pdCBrZXlkb3duIGtleXByZXNzIGtleXVwIGVycm9yIGNvbnRleHRtZW51Ii5zcGxp",
				"dCgiICIpLGZ1bmN0aW9uKGEsYil7Zi5mbltiXT1mdW5jdGlvbihhLGMpe2M9PW51bGwmJihjPWEsYT1u",
				"dWxsKTtyZXR1cm4gYXJndW1lbnRzLmxlbmd0aD4wP3RoaXMub24oYixudWxsLGEsYyk6dGhpcy50cmln",
				"Z2VyKGIpfSxmLmF0dHJGbiYmKGYuYXR0ckZuW2JdPSEwKSxDLnRlc3QoYikmJihmLmV2ZW50LmZpeEhv",
				"b2tzW2JdPWYuZXZlbnQua2V5SG9va3MpLEQudGVzdChiKSYmKGYuZXZlbnQuZml4SG9va3NbYl09Zi5l",
				"dmVudC5tb3VzZUhvb2tzKX0pLGZ1bmN0aW9uKCl7ZnVuY3Rpb24geChhLGIsYyxlLGYsZyl7Zm9yKHZh",
				"ciBoPTAsaT1lLmxlbmd0aDtoPGk7aCsrKXt2YXIgaj1lW2hdO2lmKGope3ZhciBrPSExO2o9althXTt3",
				"aGlsZShqKXtpZihqW2RdPT09Yyl7az1lW2ouc2l6c2V0XTticmVha31pZihqLm5vZGVUeXBlPT09MSl7",
				"Z3x8KGpbZF09YyxqLnNpenNldD1oKTtpZih0eXBlb2YgYiE9InN0cmluZyIpe2lmKGo9PT1iKXtrPSEw",
				"O2JyZWFrfX1lbHNlIGlmKG0uZmlsdGVyKGIsW2pdKS5sZW5ndGg+MCl7az1qO2JyZWFrfX1qPWpbYV19",
				"ZVtoXT1rfX19ZnVuY3Rpb24gdyhhLGIsYyxlLGYsZyl7Zm9yKHZhciBoPTAsaT1lLmxlbmd0aDtoPGk7",
				"aCsrKXt2YXIgaj1lW2hdO2lmKGope3ZhciBrPSExO2o9althXTt3aGlsZShqKXtpZihqW2RdPT09Yyl7",
				"az1lW2ouc2l6c2V0XTticmVha31qLm5vZGVUeXBlPT09MSYmIWcmJihqW2RdPWMsai5zaXpzZXQ9aCk7",
				"aWYoai5ub2RlTmFtZS50b0xvd2VyQ2FzZSgpPT09Yil7az1qO2JyZWFrfWo9althXX1lW2hdPWt9fX12",
				"YXIgYT0vKCg/OlwoKD86XChbXigpXStcKXxbXigpXSspK1wpfFxbKD86XFtbXlxbXF1dKlxdfFsnIl1b",
				"XiciXSpbJyJdfFteXFtcXSciXSspK1xdfFxcLnxbXiA+K34sKFxbXFxdKykrfFs+K35dKShccyosXHMq",
				"KT8oKD86LnxccnxcbikqKS9nLGQ9InNpemNhY2hlIisoTWF0aC5yYW5kb20oKSsiIikucmVwbGFjZSgi",
				"LiIsIiIpLGU9MCxnPU9iamVjdC5wcm90b3R5cGUudG9TdHJpbmcsaD0hMSxpPSEwLGo9L1xcL2csaz0v",
				"XHJcbi9nLGw9L1xXLztbMCwwXS5zb3J0KGZ1bmN0aW9uKCl7aT0hMTtyZXR1cm4gMH0pO3ZhciBtPWZ1",
				"bmN0aW9uKGIsZCxlLGYpe2U9ZXx8W10sZD1kfHxjO3ZhciBoPWQ7aWYoZC5ub2RlVHlwZSE9PTEmJmQu",
				"bm9kZVR5cGUhPT05KXJldHVybltdO2lmKCFifHx0eXBlb2YgYiE9InN0cmluZyIpcmV0dXJuIGU7dmFy",
				"IGksaixrLGwsbixxLHIsdCx1PSEwLHY9bS5pc1hNTChkKSx3PVtdLHg9Yjtkb3thLmV4ZWMoIiIpLGk9",
				"YS5leGVjKHgpO2lmKGkpe3g9aVszXSx3LnB1c2goaVsxXSk7aWYoaVsyXSl7bD1pWzNdO2JyZWFrfX19",
				"d2hpbGUoaSk7aWYody5sZW5ndGg+MSYmcC5leGVjKGIpKWlmKHcubGVuZ3RoPT09MiYmby5yZWxhdGl2",
				"ZVt3WzBdXSlqPXkod1swXSt3WzFdLGQsZik7ZWxzZXtqPW8ucmVsYXRpdmVbd1swXV0/W2RdOm0ody5z",
				"aGlmdCgpLGQpO3doaWxlKHcubGVuZ3RoKWI9dy5zaGlmdCgpLG8ucmVsYXRpdmVbYl0mJihiKz13LnNo",
				"aWZ0KCkpLGo9eShiLGosZil9ZWxzZXshZiYmdy5sZW5ndGg+MSYmZC5ub2RlVHlwZT09PTkmJiF2JiZv",
				"Lm1hdGNoLklELnRlc3Qod1swXSkmJiFvLm1hdGNoLklELnRlc3Qod1t3Lmxlbmd0aC0xXSkmJihuPW0u",
				"ZmluZCh3LnNoaWZ0KCksZCx2KSxkPW4uZXhwcj9tLmZpbHRlcihuLmV4cHIsbi5zZXQpWzBdOm4uc2V0",
				"WzBdKTtpZihkKXtuPWY/e2V4cHI6dy5wb3AoKSxzZXQ6cyhmKX06bS5maW5kKHcucG9wKCksdy5sZW5n",
				"dGg9PT0xJiYod1swXT09PSJ+Inx8d1swXT09PSIrIikmJmQucGFyZW50Tm9kZT9kLnBhcmVudE5vZGU6",
				"ZCx2KSxqPW4uZXhwcj9tLmZpbHRlcihuLmV4cHIsbi5zZXQpOm4uc2V0LHcubGVuZ3RoPjA/az1zKGop",
				"OnU9ITE7d2hpbGUody5sZW5ndGgpcT13LnBvcCgpLHI9cSxvLnJlbGF0aXZlW3FdP3I9dy5wb3AoKTpx",
				"PSIiLHI9PW51bGwmJihyPWQpLG8ucmVsYXRpdmVbcV0oayxyLHYpfWVsc2Ugaz13PVtdfWt8fChrPWop",
				"LGt8fG0uZXJyb3IocXx8Yik7aWYoZy5jYWxsKGspPT09IltvYmplY3QgQXJyYXldIilpZighdSllLnB1",
				"c2guYXBwbHkoZSxrKTtlbHNlIGlmKGQmJmQubm9kZVR5cGU9PT0xKWZvcih0PTA7a1t0XSE9bnVsbDt0",
				"Kyspa1t0XSYmKGtbdF09PT0hMHx8a1t0XS5ub2RlVHlwZT09PTEmJm0uY29udGFpbnMoZCxrW3RdKSkm",
				"JmUucHVzaChqW3RdKTtlbHNlIGZvcih0PTA7a1t0XSE9bnVsbDt0Kyspa1t0XSYma1t0XS5ub2RlVHlw",
				"ZT09PTEmJmUucHVzaChqW3RdKTtlbHNlIHMoayxlKTtsJiYobShsLGgsZSxmKSxtLnVuaXF1ZVNvcnQo",
				"ZSkpO3JldHVybiBlfTttLnVuaXF1ZVNvcnQ9ZnVuY3Rpb24oYSl7aWYodSl7aD1pLGEuc29ydCh1KTtp",
				"ZihoKWZvcih2YXIgYj0xO2I8YS5sZW5ndGg7YisrKWFbYl09PT1hW2ItMV0mJmEuc3BsaWNlKGItLSwx",
				"KX1yZXR1cm4gYX0sbS5tYXRjaGVzPWZ1bmN0aW9uKGEsYil7cmV0dXJuIG0oYSxudWxsLG51bGwsYil9",
				"LG0ubWF0Y2hlc1NlbGVjdG9yPWZ1bmN0aW9uKGEsYil7cmV0dXJuIG0oYixudWxsLG51bGwsW2FdKS5s",
				"ZW5ndGg+MH0sbS5maW5kPWZ1bmN0aW9uKGEsYixjKXt2YXIgZCxlLGYsZyxoLGk7aWYoIWEpcmV0dXJu",
				"W107Zm9yKGU9MCxmPW8ub3JkZXIubGVuZ3RoO2U8ZjtlKyspe2g9by5vcmRlcltlXTtpZihnPW8ubGVm",
				"dE1hdGNoW2hdLmV4ZWMoYSkpe2k9Z1sxXSxnLnNwbGljZSgxLDEpO2lmKGkuc3Vic3RyKGkubGVuZ3Ro",
				"LTEpIT09IlxcIil7Z1sxXT0oZ1sxXXx8IiIpLnJlcGxhY2UoaiwiIiksZD1vLmZpbmRbaF0oZyxiLGMp",
				"O2lmKGQhPW51bGwpe2E9YS5yZXBsYWNlKG8ubWF0Y2hbaF0sIiIpO2JyZWFrfX19fWR8fChkPXR5cGVv",
				"ZiBiLmdldEVsZW1lbnRzQnlUYWdOYW1lIT0idW5kZWZpbmVkIj9iLmdldEVsZW1lbnRzQnlUYWdOYW1l",
				"KCIqIik6W10pO3JldHVybntzZXQ6ZCxleHByOmF9fSxtLmZpbHRlcj1mdW5jdGlvbihhLGMsZCxlKXt2",
				"YXIgZixnLGgsaSxqLGssbCxuLHAscT1hLHI9W10scz1jLHQ9YyYmY1swXSYmbS5pc1hNTChjWzBdKTt3",
				"aGlsZShhJiZjLmxlbmd0aCl7Zm9yKGggaW4gby5maWx0ZXIpaWYoKGY9by5sZWZ0TWF0Y2hbaF0uZXhl",
				"YyhhKSkhPW51bGwmJmZbMl0pe2s9by5maWx0ZXJbaF0sbD1mWzFdLGc9ITEsZi5zcGxpY2UoMSwxKTtp",
				"ZihsLnN1YnN0cihsLmxlbmd0aC0xKT09PSJcXCIpY29udGludWU7cz09PXImJihyPVtdKTtpZihvLnBy",
				"ZUZpbHRlcltoXSl7Zj1vLnByZUZpbHRlcltoXShmLHMsZCxyLGUsdCk7aWYoIWYpZz1pPSEwO2Vsc2Ug",
				"aWYoZj09PSEwKWNvbnRpbnVlfWlmKGYpZm9yKG49MDsoaj1zW25dKSE9bnVsbDtuKyspaiYmKGk9ayhq",
				"LGYsbixzKSxwPWVeaSxkJiZpIT1udWxsP3A/Zz0hMDpzW25dPSExOnAmJihyLnB1c2goaiksZz0hMCkp",
				"O2lmKGkhPT1iKXtkfHwocz1yKSxhPWEucmVwbGFjZShvLm1hdGNoW2hdLCIiKTtpZighZylyZXR1cm5b",
				"XTticmVha319aWYoYT09PXEpaWYoZz09bnVsbCltLmVycm9yKGEpO2Vsc2UgYnJlYWs7cT1hfXJldHVy",
				"biBzfSxtLmVycm9yPWZ1bmN0aW9uKGEpe3Rocm93IG5ldyBFcnJvcigiU3ludGF4IGVycm9yLCB1bnJl",
				"Y29nbml6ZWQgZXhwcmVzc2lvbjogIithKX07dmFyIG49bS5nZXRUZXh0PWZ1bmN0aW9uKGEpe3ZhciBi",
				"LGMsZD1hLm5vZGVUeXBlLGU9IiI7aWYoZCl7aWYoZD09PTF8fGQ9PT05fHxkPT09MTEpe2lmKHR5cGVv",
				"ZiBhLnRleHRDb250ZW50PT0ic3RyaW5nIilyZXR1cm4gYS50ZXh0Q29udGVudDtpZih0eXBlb2YgYS5p",
				"bm5lclRleHQ9PSJzdHJpbmciKXJldHVybiBhLmlubmVyVGV4dC5yZXBsYWNlKGssIiIpO2ZvcihhPWEu",
				"Zmlyc3RDaGlsZDthO2E9YS5uZXh0U2libGluZyllKz1uKGEpfWVsc2UgaWYoZD09PTN8fGQ9PT00KXJl",
				"dHVybiBhLm5vZGVWYWx1ZX1lbHNlIGZvcihiPTA7Yz1hW2JdO2IrKyljLm5vZGVUeXBlIT09OCYmKGUr",
				"PW4oYykpO3JldHVybiBlfSxvPW0uc2VsZWN0b3JzPXtvcmRlcjpbIklEIiwiTkFNRSIsIlRBRyJdLG1h",
				"dGNoOntJRDovIygoPzpbXHdcdTAwYzAtXHVGRkZGXC1dfFxcLikrKS8sQ0xBU1M6L1wuKCg/Oltcd1x1",
				"MDBjMC1cdUZGRkZcLV18XFwuKSspLyxOQU1FOi9cW25hbWU9WyciXSooKD86W1x3XHUwMGMwLVx1RkZG",
				"RlwtXXxcXC4pKylbJyJdKlxdLyxBVFRSOi9cW1xzKigoPzpbXHdcdTAwYzAtXHVGRkZGXC1dfFxcLikr",
				"KVxzKig/OihcUz89KVxzKig/OihbJyJdKSguKj8pXDN8KCM/KD86W1x3XHUwMGMwLVx1RkZGRlwtXXxc",
				"XC4pKil8KXwpXHMqXF0vLFRBRzovXigoPzpbXHdcdTAwYzAtXHVGRkZGXCpcLV18XFwuKSspLyxDSElM",
				"RDovOihvbmx5fG50aHxsYXN0fGZpcnN0KS1jaGlsZCg/OlwoXHMqKGV2ZW58b2RkfCg/OlsrXC1dP1xk",
				"K3woPzpbK1wtXT9cZCopP25ccyooPzpbK1wtXVxzKlxkKyk/KSlccypcKSk/LyxQT1M6LzoobnRofGVx",
				"fGd0fGx0fGZpcnN0fGxhc3R8ZXZlbnxvZGQpKD86XCgoXGQqKVwpKT8oPz1bXlwtXXwkKS8sUFNFVURP",
				"Oi86KCg/Oltcd1x1MDBjMC1cdUZGRkZcLV18XFwuKSspKD86XCgoWyciXT8pKCg/OlwoW15cKV0rXCl8",
				"W15cKFwpXSopKylcMlwpKT8vfSxsZWZ0TWF0Y2g6e30sYXR0ck1hcDp7ImNsYXNzIjoiY2xhc3NOYW1l",
				"IiwiZm9yIjoiaHRtbEZvciJ9LGF0dHJIYW5kbGU6e2hyZWY6ZnVuY3Rpb24oYSl7cmV0dXJuIGEuZ2V0",
				"QXR0cmlidXRlKCJocmVmIil9LHR5cGU6ZnVuY3Rpb24oYSl7cmV0dXJuIGEuZ2V0QXR0cmlidXRlKCJ0",
				"eXBlIil9fSxyZWxhdGl2ZTp7IisiOmZ1bmN0aW9uKGEsYil7dmFyIGM9dHlwZW9mIGI9PSJzdHJpbmci",
				"LGQ9YyYmIWwudGVzdChiKSxlPWMmJiFkO2QmJihiPWIudG9Mb3dlckNhc2UoKSk7Zm9yKHZhciBmPTAs",
				"Zz1hLmxlbmd0aCxoO2Y8ZztmKyspaWYoaD1hW2ZdKXt3aGlsZSgoaD1oLnByZXZpb3VzU2libGluZykm",
				"Jmgubm9kZVR5cGUhPT0xKTthW2ZdPWV8fGgmJmgubm9kZU5hbWUudG9Mb3dlckNhc2UoKT09PWI/aHx8",
				"ITE6aD09PWJ9ZSYmbS5maWx0ZXIoYixhLCEwKX0sIj4iOmZ1bmN0aW9uKGEsYil7dmFyIGMsZD10eXBl",
				"b2YgYj09InN0cmluZyIsZT0wLGY9YS5sZW5ndGg7aWYoZCYmIWwudGVzdChiKSl7Yj1iLnRvTG93ZXJD",
				"YXNlKCk7Zm9yKDtlPGY7ZSsrKXtjPWFbZV07aWYoYyl7dmFyIGc9Yy5wYXJlbnROb2RlO2FbZV09Zy5u",
				"b2RlTmFtZS50b0xvd2VyQ2FzZSgpPT09Yj9nOiExfX19ZWxzZXtmb3IoO2U8ZjtlKyspYz1hW2VdLGMm",
				"JihhW2VdPWQ/Yy5wYXJlbnROb2RlOmMucGFyZW50Tm9kZT09PWIpO2QmJm0uZmlsdGVyKGIsYSwhMCl9",
				"fSwiIjpmdW5jdGlvbihhLGIsYyl7dmFyIGQsZj1lKyssZz14O3R5cGVvZiBiPT0ic3RyaW5nIiYmIWwu",
				"dGVzdChiKSYmKGI9Yi50b0xvd2VyQ2FzZSgpLGQ9YixnPXcpLGcoInBhcmVudE5vZGUiLGIsZixhLGQs",
				"Yyl9LCJ+IjpmdW5jdGlvbihhLGIsYyl7dmFyIGQsZj1lKyssZz14O3R5cGVvZiBiPT0ic3RyaW5nIiYm",
				"IWwudGVzdChiKSYmKGI9Yi50b0xvd2VyQ2FzZSgpLGQ9YixnPXcpLGcoInByZXZpb3VzU2libGluZyIs",
				"YixmLGEsZCxjKX19LGZpbmQ6e0lEOmZ1bmN0aW9uKGEsYixjKXtpZih0eXBlb2YgYi5nZXRFbGVtZW50",
				"QnlJZCE9InVuZGVmaW5lZCImJiFjKXt2YXIgZD1iLmdldEVsZW1lbnRCeUlkKGFbMV0pO3JldHVybiBk",
				"JiZkLnBhcmVudE5vZGU/W2RdOltdfX0sTkFNRTpmdW5jdGlvbihhLGIpe2lmKHR5cGVvZiBiLmdldEVs",
				"ZW1lbnRzQnlOYW1lIT0idW5kZWZpbmVkIil7dmFyIGM9W10sZD1iLmdldEVsZW1lbnRzQnlOYW1lKGFb",
				"MV0pO2Zvcih2YXIgZT0wLGY9ZC5sZW5ndGg7ZTxmO2UrKylkW2VdLmdldEF0dHJpYnV0ZSgibmFtZSIp",
				"PT09YVsxXSYmYy5wdXNoKGRbZV0pO3JldHVybiBjLmxlbmd0aD09PTA/bnVsbDpjfX0sVEFHOmZ1bmN0",
				"aW9uKGEsYil7aWYodHlwZW9mIGIuZ2V0RWxlbWVudHNCeVRhZ05hbWUhPSJ1bmRlZmluZWQiKXJldHVy",
				"biBiLmdldEVsZW1lbnRzQnlUYWdOYW1lKGFbMV0pfX0scHJlRmlsdGVyOntDTEFTUzpmdW5jdGlvbihh",
				"LGIsYyxkLGUsZil7YT0iICIrYVsxXS5yZXBsYWNlKGosIiIpKyIgIjtpZihmKXJldHVybiBhO2Zvcih2",
				"YXIgZz0wLGg7KGg9YltnXSkhPW51bGw7ZysrKWgmJihlXihoLmNsYXNzTmFtZSYmKCIgIitoLmNsYXNz",
				"TmFtZSsiICIpLnJlcGxhY2UoL1tcdFxuXHJdL2csIiAiKS5pbmRleE9mKGEpPj0wKT9jfHxkLnB1c2go",
				"aCk6YyYmKGJbZ109ITEpKTtyZXR1cm4hMX0sSUQ6ZnVuY3Rpb24oYSl7cmV0dXJuIGFbMV0ucmVwbGFj",
				"ZShqLCIiKX0sVEFHOmZ1bmN0aW9uKGEsYil7cmV0dXJuIGFbMV0ucmVwbGFjZShqLCIiKS50b0xvd2Vy",
				"Q2FzZSgpfSxDSElMRDpmdW5jdGlvbihhKXtpZihhWzFdPT09Im50aCIpe2FbMl18fG0uZXJyb3IoYVsw",
				"XSksYVsyXT1hWzJdLnJlcGxhY2UoL15cK3xccyovZywiIik7dmFyIGI9LygtPykoXGQqKSg/Om4oWytc",
				"LV0/XGQqKSk/Ly5leGVjKGFbMl09PT0iZXZlbiImJiIybiJ8fGFbMl09PT0ib2RkIiYmIjJuKzEifHwh",
				"L1xELy50ZXN0KGFbMl0pJiYiMG4rIithWzJdfHxhWzJdKTthWzJdPWJbMV0rKGJbMl18fDEpLTAsYVsz",
				"XT1iWzNdLTB9ZWxzZSBhWzJdJiZtLmVycm9yKGFbMF0pO2FbMF09ZSsrO3JldHVybiBhfSxBVFRSOmZ1",
				"bmN0aW9uKGEsYixjLGQsZSxmKXt2YXIgZz1hWzFdPWFbMV0ucmVwbGFjZShqLCIiKTshZiYmby5hdHRy",
				"TWFwW2ddJiYoYVsxXT1vLmF0dHJNYXBbZ10pLGFbNF09KGFbNF18fGFbNV18fCIiKS5yZXBsYWNlKGos",
				"IiIpLGFbMl09PT0ifj0iJiYoYVs0XT0iICIrYVs0XSsiICIpO3JldHVybiBhfSxQU0VVRE86ZnVuY3Rp",
				"b24oYixjLGQsZSxmKXtpZihiWzFdPT09Im5vdCIpaWYoKGEuZXhlYyhiWzNdKXx8IiIpLmxlbmd0aD4x",
				"fHwvXlx3Ly50ZXN0KGJbM10pKWJbM109bShiWzNdLG51bGwsbnVsbCxjKTtlbHNle3ZhciBnPW0uZmls",
				"dGVyKGJbM10sYyxkLCEwXmYpO2R8fGUucHVzaC5hcHBseShlLGcpO3JldHVybiExfWVsc2UgaWYoby5t",
				"YXRjaC5QT1MudGVzdChiWzBdKXx8by5tYXRjaC5DSElMRC50ZXN0KGJbMF0pKXJldHVybiEwO3JldHVy",
				"biBifSxQT1M6ZnVuY3Rpb24oYSl7YS51bnNoaWZ0KCEwKTtyZXR1cm4gYX19LGZpbHRlcnM6e2VuYWJs",
				"ZWQ6ZnVuY3Rpb24oYSl7cmV0dXJuIGEuZGlzYWJsZWQ9PT0hMSYmYS50eXBlIT09ImhpZGRlbiJ9LGRp",
				"c2FibGVkOmZ1bmN0aW9uKGEpe3JldHVybiBhLmRpc2FibGVkPT09ITB9LGNoZWNrZWQ6ZnVuY3Rpb24o",
				"YSl7cmV0dXJuIGEuY2hlY2tlZD09PSEwfSxzZWxlY3RlZDpmdW5jdGlvbihhKXthLnBhcmVudE5vZGUm",
				"JmEucGFyZW50Tm9kZS5zZWxlY3RlZEluZGV4O3JldHVybiBhLnNlbGVjdGVkPT09ITB9LHBhcmVudDpm",
				"dW5jdGlvbihhKXtyZXR1cm4hIWEuZmlyc3RDaGlsZH0sZW1wdHk6ZnVuY3Rpb24oYSl7cmV0dXJuIWEu",
				"Zmlyc3RDaGlsZH0saGFzOmZ1bmN0aW9uKGEsYixjKXtyZXR1cm4hIW0oY1szXSxhKS5sZW5ndGh9LGhl",
				"YWRlcjpmdW5jdGlvbihhKXtyZXR1cm4vaFxkL2kudGVzdChhLm5vZGVOYW1lKX0sdGV4dDpmdW5jdGlv",
				"bihhKXt2YXIgYj1hLmdldEF0dHJpYnV0ZSgidHlwZSIpLGM9YS50eXBlO3JldHVybiBhLm5vZGVOYW1l",
				"LnRvTG93ZXJDYXNlKCk9PT0iaW5wdXQiJiYidGV4dCI9PT1jJiYoYj09PWN8fGI9PT1udWxsKX0scmFk",
				"aW86ZnVuY3Rpb24oYSl7cmV0dXJuIGEubm9kZU5hbWUudG9Mb3dlckNhc2UoKT09PSJpbnB1dCImJiJy",
				"YWRpbyI9PT1hLnR5cGV9LGNoZWNrYm94OmZ1bmN0aW9uKGEpe3JldHVybiBhLm5vZGVOYW1lLnRvTG93",
				"ZXJDYXNlKCk9PT0iaW5wdXQiJiYiY2hlY2tib3giPT09YS50eXBlfSxmaWxlOmZ1bmN0aW9uKGEpe3Jl",
				"dHVybiBhLm5vZGVOYW1lLnRvTG93ZXJDYXNlKCk9PT0iaW5wdXQiJiYiZmlsZSI9PT1hLnR5cGV9LHBh",
				"c3N3b3JkOmZ1bmN0aW9uKGEpe3JldHVybiBhLm5vZGVOYW1lLnRvTG93ZXJDYXNlKCk9PT0iaW5wdXQi",
				"JiYicGFzc3dvcmQiPT09YS50eXBlfSxzdWJtaXQ6ZnVuY3Rpb24oYSl7dmFyIGI9YS5ub2RlTmFtZS50",
				"b0xvd2VyQ2FzZSgpO3JldHVybihiPT09ImlucHV0Inx8Yj09PSJidXR0b24iKSYmInN1Ym1pdCI9PT1h",
				"LnR5cGV9LGltYWdlOmZ1bmN0aW9uKGEpe3JldHVybiBhLm5vZGVOYW1lLnRvTG93ZXJDYXNlKCk9PT0i",
				"aW5wdXQiJiYiaW1hZ2UiPT09YS50eXBlfSxyZXNldDpmdW5jdGlvbihhKXt2YXIgYj1hLm5vZGVOYW1l",
				"LnRvTG93ZXJDYXNlKCk7cmV0dXJuKGI9PT0iaW5wdXQifHxiPT09ImJ1dHRvbiIpJiYicmVzZXQiPT09",
				"YS50eXBlfSxidXR0b246ZnVuY3Rpb24oYSl7dmFyIGI9YS5ub2RlTmFtZS50b0xvd2VyQ2FzZSgpO3Jl",
				"dHVybiBiPT09ImlucHV0IiYmImJ1dHRvbiI9PT1hLnR5cGV8fGI9PT0iYnV0dG9uIn0saW5wdXQ6ZnVu",
				"Y3Rpb24oYSl7cmV0dXJuL2lucHV0fHNlbGVjdHx0ZXh0YXJlYXxidXR0b24vaS50ZXN0KGEubm9kZU5h",
				"bWUpfSxmb2N1czpmdW5jdGlvbihhKXtyZXR1cm4gYT09PWEub3duZXJEb2N1bWVudC5hY3RpdmVFbGVt",
				"ZW50fX0sc2V0RmlsdGVyczp7Zmlyc3Q6ZnVuY3Rpb24oYSxiKXtyZXR1cm4gYj09PTB9LGxhc3Q6ZnVu",
				"Y3Rpb24oYSxiLGMsZCl7cmV0dXJuIGI9PT1kLmxlbmd0aC0xfSxldmVuOmZ1bmN0aW9uKGEsYil7cmV0",
				"dXJuIGIlMj09PTB9LG9kZDpmdW5jdGlvbihhLGIpe3JldHVybiBiJTI9PT0xfSxsdDpmdW5jdGlvbihh",
				"LGIsYyl7cmV0dXJuIGI8Y1szXS0wfSxndDpmdW5jdGlvbihhLGIsYyl7cmV0dXJuIGI+Y1szXS0wfSxu",
				"dGg6ZnVuY3Rpb24oYSxiLGMpe3JldHVybiBjWzNdLTA9PT1ifSxlcTpmdW5jdGlvbihhLGIsYyl7cmV0",
				"dXJuIGNbM10tMD09PWJ9fSxmaWx0ZXI6e1BTRVVETzpmdW5jdGlvbihhLGIsYyxkKXt2YXIgZT1iWzFd",
				"LGY9by5maWx0ZXJzW2VdO2lmKGYpcmV0dXJuIGYoYSxjLGIsZCk7aWYoZT09PSJjb250YWlucyIpcmV0",
				"dXJuKGEudGV4dENvbnRlbnR8fGEuaW5uZXJUZXh0fHxuKFthXSl8fCIiKS5pbmRleE9mKGJbM10pPj0w",
				"O2lmKGU9PT0ibm90Iil7dmFyIGc9YlszXTtmb3IodmFyIGg9MCxpPWcubGVuZ3RoO2g8aTtoKyspaWYo",
				"Z1toXT09PWEpcmV0dXJuITE7cmV0dXJuITB9bS5lcnJvcihlKX0sQ0hJTEQ6ZnVuY3Rpb24oYSxiKXt2",
				"YXIgYyxlLGYsZyxoLGksaixrPWJbMV0sbD1hO3N3aXRjaChrKXtjYXNlIm9ubHkiOmNhc2UiZmlyc3Qi",
				"OndoaWxlKGw9bC5wcmV2aW91c1NpYmxpbmcpaWYobC5ub2RlVHlwZT09PTEpcmV0dXJuITE7aWYoaz09",
				"PSJmaXJzdCIpcmV0dXJuITA7bD1hO2Nhc2UibGFzdCI6d2hpbGUobD1sLm5leHRTaWJsaW5nKWlmKGwu",
				"bm9kZVR5cGU9PT0xKXJldHVybiExO3JldHVybiEwO2Nhc2UibnRoIjpjPWJbMl0sZT1iWzNdO2lmKGM9",
				"PT0xJiZlPT09MClyZXR1cm4hMDtmPWJbMF0sZz1hLnBhcmVudE5vZGU7aWYoZyYmKGdbZF0hPT1mfHwh",
				"YS5ub2RlSW5kZXgpKXtpPTA7Zm9yKGw9Zy5maXJzdENoaWxkO2w7bD1sLm5leHRTaWJsaW5nKWwubm9k",
				"ZVR5cGU9PT0xJiYobC5ub2RlSW5kZXg9KytpKTtnW2RdPWZ9aj1hLm5vZGVJbmRleC1lO3JldHVybiBj",
				"PT09MD9qPT09MDpqJWM9PT0wJiZqL2M+PTB9fSxJRDpmdW5jdGlvbihhLGIpe3JldHVybiBhLm5vZGVU",
				"eXBlPT09MSYmYS5nZXRBdHRyaWJ1dGUoImlkIik9PT1ifSxUQUc6ZnVuY3Rpb24oYSxiKXtyZXR1cm4g",
				"Yj09PSIqIiYmYS5ub2RlVHlwZT09PTF8fCEhYS5ub2RlTmFtZSYmYS5ub2RlTmFtZS50b0xvd2VyQ2Fz",
				"ZSgpPT09Yn0sQ0xBU1M6ZnVuY3Rpb24oYSxiKXtyZXR1cm4oIiAiKyhhLmNsYXNzTmFtZXx8YS5nZXRB",
				"dHRyaWJ1dGUoImNsYXNzIikpKyIgIikuaW5kZXhPZihiKT4tMX0sQVRUUjpmdW5jdGlvbihhLGIpe3Zh",
				"ciBjPWJbMV0sZD1tLmF0dHI/bS5hdHRyKGEsYyk6by5hdHRySGFuZGxlW2NdP28uYXR0ckhhbmRsZVtj",
				"XShhKTphW2NdIT1udWxsP2FbY106YS5nZXRBdHRyaWJ1dGUoYyksZT1kKyIiLGY9YlsyXSxnPWJbNF07",
				"cmV0dXJuIGQ9PW51bGw/Zj09PSIhPSI6IWYmJm0uYXR0cj9kIT1udWxsOmY9PT0iPSI/ZT09PWc6Zj09",
				"PSIqPSI/ZS5pbmRleE9mKGcpPj0wOmY9PT0ifj0iPygiICIrZSsiICIpLmluZGV4T2YoZyk+PTA6Zz9m",
				"PT09IiE9Ij9lIT09ZzpmPT09Il49Ij9lLmluZGV4T2YoZyk9PT0wOmY9PT0iJD0iP2Uuc3Vic3RyKGUu",
				"bGVuZ3RoLWcubGVuZ3RoKT09PWc6Zj09PSJ8PSI/ZT09PWd8fGUuc3Vic3RyKDAsZy5sZW5ndGgrMSk9",
				"PT1nKyItIjohMTplJiZkIT09ITF9LFBPUzpmdW5jdGlvbihhLGIsYyxkKXt2YXIgZT1iWzJdLGY9by5z",
				"ZXRGaWx0ZXJzW2VdO2lmKGYpcmV0dXJuIGYoYSxjLGIsZCl9fX0scD1vLm1hdGNoLlBPUyxxPWZ1bmN0",
				"aW9uKGEsYil7cmV0dXJuIlxcIisoYi0wKzEpfTtmb3IodmFyIHIgaW4gby5tYXRjaClvLm1hdGNoW3Jd",
				"PW5ldyBSZWdFeHAoby5tYXRjaFtyXS5zb3VyY2UrLyg/IVteXFtdKlxdKSg/IVteXChdKlwpKS8uc291",
				"cmNlKSxvLmxlZnRNYXRjaFtyXT1uZXcgUmVnRXhwKC8oXig/Oi58XHJ8XG4pKj8pLy5zb3VyY2Urby5t",
				"YXRjaFtyXS5zb3VyY2UucmVwbGFjZSgvXFwoXGQrKS9nLHEpKTtvLm1hdGNoLmdsb2JhbFBPUz1wO3Zh",
				"ciBzPWZ1bmN0aW9uKGEsYil7YT1BcnJheS5wcm90b3R5cGUuc2xpY2UuY2FsbChhLDApO2lmKGIpe2Iu",
				"cHVzaC5hcHBseShiLGEpO3JldHVybiBifXJldHVybiBhfTt0cnl7QXJyYXkucHJvdG90eXBlLnNsaWNl",
				"LmNhbGwoYy5kb2N1bWVudEVsZW1lbnQuY2hpbGROb2RlcywwKVswXS5ub2RlVHlwZX1jYXRjaCh0KXtz",
				"PWZ1bmN0aW9uKGEsYil7dmFyIGM9MCxkPWJ8fFtdO2lmKGcuY2FsbChhKT09PSJbb2JqZWN0IEFycmF5",
				"XSIpQXJyYXkucHJvdG90eXBlLnB1c2guYXBwbHkoZCxhKTtlbHNlIGlmKHR5cGVvZiBhLmxlbmd0aD09",
				"Im51bWJlciIpZm9yKHZhciBlPWEubGVuZ3RoO2M8ZTtjKyspZC5wdXNoKGFbY10pO2Vsc2UgZm9yKDth",
				"W2NdO2MrKylkLnB1c2goYVtjXSk7cmV0dXJuIGR9fXZhciB1LHY7Yy5kb2N1bWVudEVsZW1lbnQuY29t",
				"cGFyZURvY3VtZW50UG9zaXRpb24/dT1mdW5jdGlvbihhLGIpe2lmKGE9PT1iKXtoPSEwO3JldHVybiAw",
				"fWlmKCFhLmNvbXBhcmVEb2N1bWVudFBvc2l0aW9ufHwhYi5jb21wYXJlRG9jdW1lbnRQb3NpdGlvbily",
				"ZXR1cm4gYS5jb21wYXJlRG9jdW1lbnRQb3NpdGlvbj8tMToxO3JldHVybiBhLmNvbXBhcmVEb2N1bWVu",
				"dFBvc2l0aW9uKGIpJjQ/LTE6MX06KHU9ZnVuY3Rpb24oYSxiKXtpZihhPT09Yil7aD0hMDtyZXR1cm4g",
				"MH1pZihhLnNvdXJjZUluZGV4JiZiLnNvdXJjZUluZGV4KXJldHVybiBhLnNvdXJjZUluZGV4LWIuc291",
				"cmNlSW5kZXg7dmFyIGMsZCxlPVtdLGY9W10sZz1hLnBhcmVudE5vZGUsaT1iLnBhcmVudE5vZGUsaj1n",
				"O2lmKGc9PT1pKXJldHVybiB2KGEsYik7aWYoIWcpcmV0dXJuLTE7aWYoIWkpcmV0dXJuIDE7d2hpbGUo",
				"aillLnVuc2hpZnQoaiksaj1qLnBhcmVudE5vZGU7aj1pO3doaWxlKGopZi51bnNoaWZ0KGopLGo9ai5w",
				"YXJlbnROb2RlO2M9ZS5sZW5ndGgsZD1mLmxlbmd0aDtmb3IodmFyIGs9MDtrPGMmJms8ZDtrKyspaWYo",
				"ZVtrXSE9PWZba10pcmV0dXJuIHYoZVtrXSxmW2tdKTtyZXR1cm4gaz09PWM/dihhLGZba10sLTEpOnYo",
				"ZVtrXSxiLDEpfSx2PWZ1bmN0aW9uKGEsYixjKXtpZihhPT09YilyZXR1cm4gYzt2YXIgZD1hLm5leHRT",
				"aWJsaW5nO3doaWxlKGQpe2lmKGQ9PT1iKXJldHVybi0xO2Q9ZC5uZXh0U2libGluZ31yZXR1cm4gMX0p",
				"LGZ1bmN0aW9uKCl7dmFyIGE9Yy5jcmVhdGVFbGVtZW50KCJkaXYiKSxkPSJzY3JpcHQiKyhuZXcgRGF0",
				"ZSkuZ2V0VGltZSgpLGU9Yy5kb2N1bWVudEVsZW1lbnQ7YS5pbm5lckhUTUw9IjxhIG5hbWU9JyIrZCsi",
				"Jy8+IixlLmluc2VydEJlZm9yZShhLGUuZmlyc3RDaGlsZCksYy5nZXRFbGVtZW50QnlJZChkKSYmKG8u",
				"ZmluZC5JRD1mdW5jdGlvbihhLGMsZCl7aWYodHlwZW9mIGMuZ2V0RWxlbWVudEJ5SWQhPSJ1bmRlZmlu",
				"ZWQiJiYhZCl7dmFyIGU9Yy5nZXRFbGVtZW50QnlJZChhWzFdKTtyZXR1cm4gZT9lLmlkPT09YVsxXXx8",
				"dHlwZW9mIGUuZ2V0QXR0cmlidXRlTm9kZSE9InVuZGVmaW5lZCImJmUuZ2V0QXR0cmlidXRlTm9kZSgi",
				"aWQiKS5ub2RlVmFsdWU9PT1hWzFdP1tlXTpiOltdfX0sby5maWx0ZXIuSUQ9ZnVuY3Rpb24oYSxiKXt2",
				"YXIgYz10eXBlb2YgYS5nZXRBdHRyaWJ1dGVOb2RlIT0idW5kZWZpbmVkIiYmYS5nZXRBdHRyaWJ1dGVO",
				"b2RlKCJpZCIpO3JldHVybiBhLm5vZGVUeXBlPT09MSYmYyYmYy5ub2RlVmFsdWU9PT1ifSksZS5yZW1v",
				"dmVDaGlsZChhKSxlPWE9bnVsbH0oKSxmdW5jdGlvbigpe3ZhciBhPWMuY3JlYXRlRWxlbWVudCgiZGl2",
				"Iik7YS5hcHBlbmRDaGlsZChjLmNyZWF0ZUNvbW1lbnQoIiIpKSxhLmdldEVsZW1lbnRzQnlUYWdOYW1l",
				"KCIqIikubGVuZ3RoPjAmJihvLmZpbmQuVEFHPWZ1bmN0aW9uKGEsYil7dmFyIGM9Yi5nZXRFbGVtZW50",
				"c0J5VGFnTmFtZShhWzFdKTtpZihhWzFdPT09IioiKXt2YXIgZD1bXTtmb3IodmFyIGU9MDtjW2VdO2Ur",
				"KyljW2VdLm5vZGVUeXBlPT09MSYmZC5wdXNoKGNbZV0pO2M9ZH1yZXR1cm4gY30pLGEuaW5uZXJIVE1M",
				"PSI8YSBocmVmPScjJz48L2E+IixhLmZpcnN0Q2hpbGQmJnR5cGVvZiBhLmZpcnN0Q2hpbGQuZ2V0QXR0",
				"cmlidXRlIT0idW5kZWZpbmVkIiYmYS5maXJzdENoaWxkLmdldEF0dHJpYnV0ZSgiaHJlZiIpIT09IiMi",
				"JiYoby5hdHRySGFuZGxlLmhyZWY9ZnVuY3Rpb24oYSl7cmV0dXJuIGEuZ2V0QXR0cmlidXRlKCJocmVm",
				"IiwyKX0pLGE9bnVsbH0oKSxjLnF1ZXJ5U2VsZWN0b3JBbGwmJmZ1bmN0aW9uKCl7dmFyIGE9bSxiPWMu",
				"Y3JlYXRlRWxlbWVudCgiZGl2IiksZD0iX19zaXp6bGVfXyI7Yi5pbm5lckhUTUw9IjxwIGNsYXNzPSdU",
				"RVNUJz48L3A+IjtpZighYi5xdWVyeVNlbGVjdG9yQWxsfHxiLnF1ZXJ5U2VsZWN0b3JBbGwoIi5URVNU",
				"IikubGVuZ3RoIT09MCl7bT1mdW5jdGlvbihiLGUsZixnKXtlPWV8fGM7aWYoIWcmJiFtLmlzWE1MKGUp",
				"KXt2YXIgaD0vXihcdyskKXxeXC4oW1x3XC1dKyQpfF4jKFtcd1wtXSskKS8uZXhlYyhiKTtpZihoJiYo",
				"ZS5ub2RlVHlwZT09PTF8fGUubm9kZVR5cGU9PT05KSl7aWYoaFsxXSlyZXR1cm4gcyhlLmdldEVsZW1l",
				"bnRzQnlUYWdOYW1lKGIpLGYpO2lmKGhbMl0mJm8uZmluZC5DTEFTUyYmZS5nZXRFbGVtZW50c0J5Q2xh",
				"c3NOYW1lKXJldHVybiBzKGUuZ2V0RWxlbWVudHNCeUNsYXNzTmFtZShoWzJdKSxmKX1pZihlLm5vZGVU",
				"eXBlPT09OSl7aWYoYj09PSJib2R5IiYmZS5ib2R5KXJldHVybiBzKFtlLmJvZHldLGYpO2lmKGgmJmhb",
				"M10pe3ZhciBpPWUuZ2V0RWxlbWVudEJ5SWQoaFszXSk7aWYoIWl8fCFpLnBhcmVudE5vZGUpcmV0dXJu",
				"IHMoW10sZik7aWYoaS5pZD09PWhbM10pcmV0dXJuIHMoW2ldLGYpfXRyeXtyZXR1cm4gcyhlLnF1ZXJ5",
				"U2VsZWN0b3JBbGwoYiksZil9Y2F0Y2goail7fX1lbHNlIGlmKGUubm9kZVR5cGU9PT0xJiZlLm5vZGVO",
				"YW1lLnRvTG93ZXJDYXNlKCkhPT0ib2JqZWN0Iil7dmFyIGs9ZSxsPWUuZ2V0QXR0cmlidXRlKCJpZCIp",
				"LG49bHx8ZCxwPWUucGFyZW50Tm9kZSxxPS9eXHMqWyt+XS8udGVzdChiKTtsP249bi5yZXBsYWNlKC8n",
				"L2csIlxcJCYiKTplLnNldEF0dHJpYnV0ZSgiaWQiLG4pLHEmJnAmJihlPWUucGFyZW50Tm9kZSk7dHJ5",
				"e2lmKCFxfHxwKXJldHVybiBzKGUucXVlcnlTZWxlY3RvckFsbCgiW2lkPSciK24rIiddICIrYiksZil9",
				"Y2F0Y2gocil7fWZpbmFsbHl7bHx8ay5yZW1vdmVBdHRyaWJ1dGUoImlkIil9fX1yZXR1cm4gYShiLGUs",
				"ZixnKX07Zm9yKHZhciBlIGluIGEpbVtlXT1hW2VdO2I9bnVsbH19KCksZnVuY3Rpb24oKXt2YXIgYT1j",
				"LmRvY3VtZW50RWxlbWVudCxiPWEubWF0Y2hlc1NlbGVjdG9yfHxhLm1vek1hdGNoZXNTZWxlY3Rvcnx8",
				"YS53ZWJraXRNYXRjaGVzU2VsZWN0b3J8fGEubXNNYXRjaGVzU2VsZWN0b3I7aWYoYil7dmFyIGQ9IWIu",
				"Y2FsbChjLmNyZWF0ZUVsZW1lbnQoImRpdiIpLCJkaXYiKSxlPSExO3RyeXtiLmNhbGwoYy5kb2N1bWVu",
				"dEVsZW1lbnQsIlt0ZXN0IT0nJ106c2l6emxlIil9Y2F0Y2goZil7ZT0hMH1tLm1hdGNoZXNTZWxlY3Rv",
				"cj1mdW5jdGlvbihhLGMpe2M9Yy5yZXBsYWNlKC9cPVxzKihbXiciXF1dKilccypcXS9nLCI9JyQxJ10i",
				"KTtpZighbS5pc1hNTChhKSl0cnl7aWYoZXx8IW8ubWF0Y2guUFNFVURPLnRlc3QoYykmJiEvIT0vLnRl",
				"c3QoYykpe3ZhciBmPWIuY2FsbChhLGMpO2lmKGZ8fCFkfHxhLmRvY3VtZW50JiZhLmRvY3VtZW50Lm5v",
				"ZGVUeXBlIT09MTEpcmV0dXJuIGZ9fWNhdGNoKGcpe31yZXR1cm4gbShjLG51bGwsbnVsbCxbYV0pLmxl",
				"bmd0aD4wfX19KCksZnVuY3Rpb24oKXt2YXIgYT1jLmNyZWF0ZUVsZW1lbnQoImRpdiIpO2EuaW5uZXJI",
				"VE1MPSI8ZGl2IGNsYXNzPSd0ZXN0IGUnPjwvZGl2PjxkaXYgY2xhc3M9J3Rlc3QnPjwvZGl2PiI7aWYo",
				"ISFhLmdldEVsZW1lbnRzQnlDbGFzc05hbWUmJmEuZ2V0RWxlbWVudHNCeUNsYXNzTmFtZSgiZSIpLmxl",
				"bmd0aCE9PTApe2EubGFzdENoaWxkLmNsYXNzTmFtZT0iZSI7aWYoYS5nZXRFbGVtZW50c0J5Q2xhc3NO",
				"YW1lKCJlIikubGVuZ3RoPT09MSlyZXR1cm47by5vcmRlci5zcGxpY2UoMSwwLCJDTEFTUyIpLG8uZmlu",
				"ZC5DTEFTUz1mdW5jdGlvbihhLGIsYyl7aWYodHlwZW9mIGIuZ2V0RWxlbWVudHNCeUNsYXNzTmFtZSE9",
				"InVuZGVmaW5lZCImJiFjKXJldHVybiBiLmdldEVsZW1lbnRzQnlDbGFzc05hbWUoYVsxXSl9LGE9bnVs",
				"bH19KCksYy5kb2N1bWVudEVsZW1lbnQuY29udGFpbnM/bS5jb250YWlucz1mdW5jdGlvbihhLGIpe3Jl",
				"dHVybiBhIT09YiYmKGEuY29udGFpbnM/YS5jb250YWlucyhiKTohMCl9OmMuZG9jdW1lbnRFbGVtZW50",
				"LmNvbXBhcmVEb2N1bWVudFBvc2l0aW9uP20uY29udGFpbnM9ZnVuY3Rpb24oYSxiKXtyZXR1cm4hIShh",
				"LmNvbXBhcmVEb2N1bWVudFBvc2l0aW9uKGIpJjE2KX06bS5jb250YWlucz1mdW5jdGlvbigpe3JldHVy",
				"biExfSxtLmlzWE1MPWZ1bmN0aW9uKGEpe3ZhciBiPShhP2Eub3duZXJEb2N1bWVudHx8YTowKS5kb2N1",
				"bWVudEVsZW1lbnQ7cmV0dXJuIGI/Yi5ub2RlTmFtZSE9PSJIVE1MIjohMX07dmFyIHk9ZnVuY3Rpb24o",
				"YSxiLGMpe3ZhciBkLGU9W10sZj0iIixnPWIubm9kZVR5cGU/W2JdOmI7d2hpbGUoZD1vLm1hdGNoLlBT",
				"RVVETy5leGVjKGEpKWYrPWRbMF0sYT1hLnJlcGxhY2Uoby5tYXRjaC5QU0VVRE8sIiIpO2E9by5yZWxh",
				"dGl2ZVthXT9hKyIqIjphO2Zvcih2YXIgaD0wLGk9Zy5sZW5ndGg7aDxpO2grKyltKGEsZ1toXSxlLGMp",
				"O3JldHVybiBtLmZpbHRlcihmLGUpfTttLmF0dHI9Zi5hdHRyLG0uc2VsZWN0b3JzLmF0dHJNYXA9e30s",
				"Zi5maW5kPW0sZi5leHByPW0uc2VsZWN0b3JzLGYuZXhwclsiOiJdPWYuZXhwci5maWx0ZXJzLGYudW5p",
				"cXVlPW0udW5pcXVlU29ydCxmLnRleHQ9bS5nZXRUZXh0LGYuaXNYTUxEb2M9bS5pc1hNTCxmLmNvbnRh",
				"aW5zPW0uY29udGFpbnN9KCk7dmFyIEw9L1VudGlsJC8sTT0vXig/OnBhcmVudHN8cHJldlVudGlsfHBy",
				"ZXZBbGwpLyxOPS8sLyxPPS9eLlteOiNcW1wuLF0qJC8sUD1BcnJheS5wcm90b3R5cGUuc2xpY2UsUT1m",
				"LmV4cHIubWF0Y2guZ2xvYmFsUE9TLFI9e2NoaWxkcmVuOiEwLGNvbnRlbnRzOiEwLG5leHQ6ITAscHJl",
				"djohMH07Zi5mbi5leHRlbmQoe2ZpbmQ6ZnVuY3Rpb24oYSl7dmFyIGI9dGhpcyxjLGQ7aWYodHlwZW9m",
				"IGEhPSJzdHJpbmciKXJldHVybiBmKGEpLmZpbHRlcihmdW5jdGlvbigpe2ZvcihjPTAsZD1iLmxlbmd0",
				"aDtjPGQ7YysrKWlmKGYuY29udGFpbnMoYltjXSx0aGlzKSlyZXR1cm4hMH0pO3ZhciBlPXRoaXMucHVz",
				"aFN0YWNrKCIiLCJmaW5kIixhKSxnLGgsaTtmb3IoYz0wLGQ9dGhpcy5sZW5ndGg7YzxkO2MrKyl7Zz1l",
				"Lmxlbmd0aCxmLmZpbmQoYSx0aGlzW2NdLGUpO2lmKGM+MClmb3IoaD1nO2g8ZS5sZW5ndGg7aCsrKWZv",
				"cihpPTA7aTxnO2krKylpZihlW2ldPT09ZVtoXSl7ZS5zcGxpY2UoaC0tLDEpO2JyZWFrfX1yZXR1cm4g",
				"ZX0saGFzOmZ1bmN0aW9uKGEpe3ZhciBiPWYoYSk7cmV0dXJuIHRoaXMuZmlsdGVyKGZ1bmN0aW9uKCl7",
				"Zm9yKHZhciBhPTAsYz1iLmxlbmd0aDthPGM7YSsrKWlmKGYuY29udGFpbnModGhpcyxiW2FdKSlyZXR1",
				"cm4hMH0pfSxub3Q6ZnVuY3Rpb24oYSl7cmV0dXJuIHRoaXMucHVzaFN0YWNrKFQodGhpcyxhLCExKSwi",
				"bm90IixhKX0sZmlsdGVyOmZ1bmN0aW9uKGEpe3JldHVybiB0aGlzLnB1c2hTdGFjayhUKHRoaXMsYSwh",
				"MCksImZpbHRlciIsYSl9LGlzOmZ1bmN0aW9uKGEpe3JldHVybiEhYSYmKHR5cGVvZiBhPT0ic3RyaW5n",
				"Ij9RLnRlc3QoYSk/ZihhLHRoaXMuY29udGV4dCkuaW5kZXgodGhpc1swXSk+PTA6Zi5maWx0ZXIoYSx0",
				"aGlzKS5sZW5ndGg+MDp0aGlzLmZpbHRlcihhKS5sZW5ndGg+MCl9LGNsb3Nlc3Q6ZnVuY3Rpb24oYSxi",
				"KXt2YXIgYz1bXSxkLGUsZz10aGlzWzBdO2lmKGYuaXNBcnJheShhKSl7dmFyIGg9MTt3aGlsZShnJiZn",
				"Lm93bmVyRG9jdW1lbnQmJmchPT1iKXtmb3IoZD0wO2Q8YS5sZW5ndGg7ZCsrKWYoZykuaXMoYVtkXSkm",
				"JmMucHVzaCh7c2VsZWN0b3I6YVtkXSxlbGVtOmcsbGV2ZWw6aH0pO2c9Zy5wYXJlbnROb2RlLGgrK31y",
				"ZXR1cm4gY312YXIgaT1RLnRlc3QoYSl8fHR5cGVvZiBhIT0ic3RyaW5nIj9mKGEsYnx8dGhpcy5jb250",
				"ZXh0KTowO2ZvcihkPTAsZT10aGlzLmxlbmd0aDtkPGU7ZCsrKXtnPXRoaXNbZF07d2hpbGUoZyl7aWYo",
				"aT9pLmluZGV4KGcpPi0xOmYuZmluZC5tYXRjaGVzU2VsZWN0b3IoZyxhKSl7Yy5wdXNoKGcpO2JyZWFr",
				"fWc9Zy5wYXJlbnROb2RlO2lmKCFnfHwhZy5vd25lckRvY3VtZW50fHxnPT09Ynx8Zy5ub2RlVHlwZT09",
				"PTExKWJyZWFrfX1jPWMubGVuZ3RoPjE/Zi51bmlxdWUoYyk6YztyZXR1cm4gdGhpcy5wdXNoU3RhY2so",
				"YywiY2xvc2VzdCIsYSl9LGluZGV4OmZ1bmN0aW9uKGEpe2lmKCFhKXJldHVybiB0aGlzWzBdJiZ0aGlz",
				"WzBdLnBhcmVudE5vZGU/dGhpcy5wcmV2QWxsKCkubGVuZ3RoOi0xO2lmKHR5cGVvZiBhPT0ic3RyaW5n",
				"IilyZXR1cm4gZi5pbkFycmF5KHRoaXNbMF0sZihhKSk7cmV0dXJuIGYuaW5BcnJheShhLmpxdWVyeT9h",
				"WzBdOmEsdGhpcyl9LGFkZDpmdW5jdGlvbihhLGIpe3ZhciBjPXR5cGVvZiBhPT0ic3RyaW5nIj9mKGEs",
				"Yik6Zi5tYWtlQXJyYXkoYSYmYS5ub2RlVHlwZT9bYV06YSksZD1mLm1lcmdlKHRoaXMuZ2V0KCksYyk7",
				"cmV0dXJuIHRoaXMucHVzaFN0YWNrKFMoY1swXSl8fFMoZFswXSk/ZDpmLnVuaXF1ZShkKSl9LGFuZFNl",
				"bGY6ZnVuY3Rpb24oKXtyZXR1cm4gdGhpcy5hZGQodGhpcy5wcmV2T2JqZWN0KX19KSxmLmVhY2goe3Bh",
				"cmVudDpmdW5jdGlvbihhKXt2YXIgYj1hLnBhcmVudE5vZGU7cmV0dXJuIGImJmIubm9kZVR5cGUhPT0x",
				"MT9iOm51bGx9LHBhcmVudHM6ZnVuY3Rpb24oYSl7cmV0dXJuIGYuZGlyKGEsInBhcmVudE5vZGUiKX0s",
				"cGFyZW50c1VudGlsOmZ1bmN0aW9uKGEsYixjKXtyZXR1cm4gZi5kaXIoYSwicGFyZW50Tm9kZSIsYyl9",
				"LG5leHQ6ZnVuY3Rpb24oYSl7cmV0dXJuIGYubnRoKGEsMiwibmV4dFNpYmxpbmciKX0scHJldjpmdW5j",
				"dGlvbihhKXtyZXR1cm4gZi5udGgoYSwyLCJwcmV2aW91c1NpYmxpbmciKX0sbmV4dEFsbDpmdW5jdGlv",
				"bihhKXtyZXR1cm4gZi5kaXIoYSwibmV4dFNpYmxpbmciKX0scHJldkFsbDpmdW5jdGlvbihhKXtyZXR1",
				"cm4gZi5kaXIoYSwicHJldmlvdXNTaWJsaW5nIil9LG5leHRVbnRpbDpmdW5jdGlvbihhLGIsYyl7cmV0",
				"dXJuIGYuZGlyKGEsIm5leHRTaWJsaW5nIixjKX0scHJldlVudGlsOmZ1bmN0aW9uKGEsYixjKXtyZXR1",
				"cm4gZi5kaXIoYSwicHJldmlvdXNTaWJsaW5nIixjKX0sc2libGluZ3M6ZnVuY3Rpb24oYSl7cmV0dXJu",
				"IGYuc2libGluZygoYS5wYXJlbnROb2RlfHx7fSkuZmlyc3RDaGlsZCxhKX0sY2hpbGRyZW46ZnVuY3Rp",
				"b24oYSl7cmV0dXJuIGYuc2libGluZyhhLmZpcnN0Q2hpbGQpfSxjb250ZW50czpmdW5jdGlvbihhKXty",
				"ZXR1cm4gZi5ub2RlTmFtZShhLCJpZnJhbWUiKT9hLmNvbnRlbnREb2N1bWVudHx8YS5jb250ZW50V2lu",
				"ZG93LmRvY3VtZW50OmYubWFrZUFycmF5KGEuY2hpbGROb2Rlcyl9fSxmdW5jdGlvbihhLGIpe2YuZm5b",
				"YV09ZnVuY3Rpb24oYyxkKXt2YXIgZT1mLm1hcCh0aGlzLGIsYyk7TC50ZXN0KGEpfHwoZD1jKSxkJiZ0",
				"eXBlb2YgZD09InN0cmluZyImJihlPWYuZmlsdGVyKGQsZSkpLGU9dGhpcy5sZW5ndGg+MSYmIVJbYV0/",
				"Zi51bmlxdWUoZSk6ZSwodGhpcy5sZW5ndGg+MXx8Ti50ZXN0KGQpKSYmTS50ZXN0KGEpJiYoZT1lLnJl",
				"dmVyc2UoKSk7cmV0dXJuIHRoaXMucHVzaFN0YWNrKGUsYSxQLmNhbGwoYXJndW1lbnRzKS5qb2luKCIs",
				"IikpfX0pLGYuZXh0ZW5kKHtmaWx0ZXI6ZnVuY3Rpb24oYSxiLGMpe2MmJihhPSI6bm90KCIrYSsiKSIp",
				"O3JldHVybiBiLmxlbmd0aD09PTE/Zi5maW5kLm1hdGNoZXNTZWxlY3RvcihiWzBdLGEpP1tiWzBdXTpb",
				"XTpmLmZpbmQubWF0Y2hlcyhhLGIpfSxkaXI6ZnVuY3Rpb24oYSxjLGQpe3ZhciBlPVtdLGc9YVtjXTt3",
				"aGlsZShnJiZnLm5vZGVUeXBlIT09OSYmKGQ9PT1ifHxnLm5vZGVUeXBlIT09MXx8IWYoZykuaXMoZCkp",
				"KWcubm9kZVR5cGU9PT0xJiZlLnB1c2goZyksZz1nW2NdO3JldHVybiBlfSxudGg6ZnVuY3Rpb24oYSxi",
				"LGMsZCl7Yj1ifHwxO3ZhciBlPTA7Zm9yKDthO2E9YVtjXSlpZihhLm5vZGVUeXBlPT09MSYmKytlPT09",
				"YilicmVhaztyZXR1cm4gYX0sc2libGluZzpmdW5jdGlvbihhLGIpe3ZhciBjPVtdO2Zvcig7YTthPWEu",
				"bmV4dFNpYmxpbmcpYS5ub2RlVHlwZT09PTEmJmEhPT1iJiZjLnB1c2goYSk7cmV0dXJuIGN9fSk7dmFy",
				"IFY9ImFiYnJ8YXJ0aWNsZXxhc2lkZXxhdWRpb3xiZGl8Y2FudmFzfGRhdGF8ZGF0YWxpc3R8ZGV0YWls",
				"c3xmaWdjYXB0aW9ufGZpZ3VyZXxmb290ZXJ8aGVhZGVyfGhncm91cHxtYXJrfG1ldGVyfG5hdnxvdXRw",
				"dXR8cHJvZ3Jlc3N8c2VjdGlvbnxzdW1tYXJ5fHRpbWV8dmlkZW8iLFc9LyBqUXVlcnlcZCs9Iig/Olxk",
				"K3xudWxsKSIvZyxYPS9eXHMrLyxZPS88KD8hYXJlYXxicnxjb2x8ZW1iZWR8aHJ8aW1nfGlucHV0fGxp",
				"bmt8bWV0YXxwYXJhbSkoKFtcdzpdKylbXj5dKilcLz4vaWcsWj0vPChbXHc6XSspLywkPS88dGJvZHkv",
				"aSxfPS88fCYjP1x3KzsvLGJhPS88KD86c2NyaXB0fHN0eWxlKS9pLGJiPS88KD86c2NyaXB0fG9iamVj",
				"dHxlbWJlZHxvcHRpb258c3R5bGUpL2ksYmM9bmV3IFJlZ0V4cCgiPCg/OiIrVisiKVtcXHMvPl0iLCJp",
				"IiksYmQ9L2NoZWNrZWRccyooPzpbXj1dfD1ccyouY2hlY2tlZC4pL2ksYmU9L1wvKGphdmF8ZWNtYSlz",
				"Y3JpcHQvaSxiZj0vXlxzKjwhKD86XFtDREFUQVxbfFwtXC0pLyxiZz17b3B0aW9uOlsxLCI8c2VsZWN0",
				"IG11bHRpcGxlPSdtdWx0aXBsZSc+IiwiPC9zZWxlY3Q+Il0sbGVnZW5kOlsxLCI8ZmllbGRzZXQ+Iiwi",
				"PC9maWVsZHNldD4iXSx0aGVhZDpbMSwiPHRhYmxlPiIsIjwvdGFibGU+Il0sdHI6WzIsIjx0YWJsZT48",
				"dGJvZHk+IiwiPC90Ym9keT48L3RhYmxlPiJdLHRkOlszLCI8dGFibGU+PHRib2R5Pjx0cj4iLCI8L3Ry",
				"PjwvdGJvZHk+PC90YWJsZT4iXSxjb2w6WzIsIjx0YWJsZT48dGJvZHk+PC90Ym9keT48Y29sZ3JvdXA+",
				"IiwiPC9jb2xncm91cD48L3RhYmxlPiJdLGFyZWE6WzEsIjxtYXA+IiwiPC9tYXA+Il0sX2RlZmF1bHQ6",
				"WzAsIiIsIiJdfSxiaD1VKGMpO2JnLm9wdGdyb3VwPWJnLm9wdGlvbixiZy50Ym9keT1iZy50Zm9vdD1i",
				"Zy5jb2xncm91cD1iZy5jYXB0aW9uPWJnLnRoZWFkLGJnLnRoPWJnLnRkLGYuc3VwcG9ydC5odG1sU2Vy",
				"aWFsaXplfHwoYmcuX2RlZmF1bHQ9WzEsImRpdjxkaXY+IiwiPC9kaXY+Il0pLGYuZm4uZXh0ZW5kKHt0",
				"ZXh0OmZ1bmN0aW9uKGEpe3JldHVybiBmLmFjY2Vzcyh0aGlzLGZ1bmN0aW9uKGEpe3JldHVybiBhPT09",
				"Yj9mLnRleHQodGhpcyk6dGhpcy5lbXB0eSgpLmFwcGVuZCgodGhpc1swXSYmdGhpc1swXS5vd25lckRv",
				"Y3VtZW50fHxjKS5jcmVhdGVUZXh0Tm9kZShhKSl9LG51bGwsYSxhcmd1bWVudHMubGVuZ3RoKX0sd3Jh",
				"cEFsbDpmdW5jdGlvbihhKXtpZihmLmlzRnVuY3Rpb24oYSkpcmV0dXJuIHRoaXMuZWFjaChmdW5jdGlv",
				"bihiKXtmKHRoaXMpLndyYXBBbGwoYS5jYWxsKHRoaXMsYikpfSk7aWYodGhpc1swXSl7dmFyIGI9Zihh",
				"LHRoaXNbMF0ub3duZXJEb2N1bWVudCkuZXEoMCkuY2xvbmUoITApO3RoaXNbMF0ucGFyZW50Tm9kZSYm",
				"Yi5pbnNlcnRCZWZvcmUodGhpc1swXSksYi5tYXAoZnVuY3Rpb24oKXt2YXIgYT10aGlzO3doaWxlKGEu",
				"Zmlyc3RDaGlsZCYmYS5maXJzdENoaWxkLm5vZGVUeXBlPT09MSlhPWEuZmlyc3RDaGlsZDtyZXR1cm4g",
				"YX0pLmFwcGVuZCh0aGlzKX1yZXR1cm4gdGhpc30sd3JhcElubmVyOmZ1bmN0aW9uKGEpe2lmKGYuaXNG",
				"dW5jdGlvbihhKSlyZXR1cm4gdGhpcy5lYWNoKGZ1bmN0aW9uKGIpe2YodGhpcykud3JhcElubmVyKGEu",
				"Y2FsbCh0aGlzLGIpKX0pO3JldHVybiB0aGlzLmVhY2goZnVuY3Rpb24oKXt2YXIgYj1mKHRoaXMpLGM9",
				"Yi5jb250ZW50cygpO2MubGVuZ3RoP2Mud3JhcEFsbChhKTpiLmFwcGVuZChhKX0pfSx3cmFwOmZ1bmN0",
				"aW9uKGEpe3ZhciBiPWYuaXNGdW5jdGlvbihhKTtyZXR1cm4gdGhpcy5lYWNoKGZ1bmN0aW9uKGMpe2Yo",
				"dGhpcykud3JhcEFsbChiP2EuY2FsbCh0aGlzLGMpOmEpfSl9LHVud3JhcDpmdW5jdGlvbigpe3JldHVy",
				"biB0aGlzLnBhcmVudCgpLmVhY2goZnVuY3Rpb24oKXtmLm5vZGVOYW1lKHRoaXMsImJvZHkiKXx8Zih0",
				"aGlzKS5yZXBsYWNlV2l0aCh0aGlzLmNoaWxkTm9kZXMpfSkuZW5kKCl9LGFwcGVuZDpmdW5jdGlvbigp",
				"e3JldHVybiB0aGlzLmRvbU1hbmlwKGFyZ3VtZW50cywhMCxmdW5jdGlvbihhKXt0aGlzLm5vZGVUeXBl",
				"PT09MSYmdGhpcy5hcHBlbmRDaGlsZChhKX0pfSxwcmVwZW5kOmZ1bmN0aW9uKCl7cmV0dXJuIHRoaXMu",
				"ZG9tTWFuaXAoYXJndW1lbnRzLCEwLGZ1bmN0aW9uKGEpe3RoaXMubm9kZVR5cGU9PT0xJiZ0aGlzLmlu",
				"c2VydEJlZm9yZShhLHRoaXMuZmlyc3RDaGlsZCl9KX0sYmVmb3JlOmZ1bmN0aW9uKCl7aWYodGhpc1sw",
				"XSYmdGhpc1swXS5wYXJlbnROb2RlKXJldHVybiB0aGlzLmRvbU1hbmlwKGFyZ3VtZW50cywhMSxmdW5j",
				"dGlvbihhKXt0aGlzLnBhcmVudE5vZGUuaW5zZXJ0QmVmb3JlKGEsdGhpcyl9KTtpZihhcmd1bWVudHMu",
				"bGVuZ3RoKXt2YXIgYT1mCi5jbGVhbihhcmd1bWVudHMpO2EucHVzaC5hcHBseShhLHRoaXMudG9BcnJh",
				"eSgpKTtyZXR1cm4gdGhpcy5wdXNoU3RhY2soYSwiYmVmb3JlIixhcmd1bWVudHMpfX0sYWZ0ZXI6ZnVu",
				"Y3Rpb24oKXtpZih0aGlzWzBdJiZ0aGlzWzBdLnBhcmVudE5vZGUpcmV0dXJuIHRoaXMuZG9tTWFuaXAo",
				"YXJndW1lbnRzLCExLGZ1bmN0aW9uKGEpe3RoaXMucGFyZW50Tm9kZS5pbnNlcnRCZWZvcmUoYSx0aGlz",
				"Lm5leHRTaWJsaW5nKX0pO2lmKGFyZ3VtZW50cy5sZW5ndGgpe3ZhciBhPXRoaXMucHVzaFN0YWNrKHRo",
				"aXMsImFmdGVyIixhcmd1bWVudHMpO2EucHVzaC5hcHBseShhLGYuY2xlYW4oYXJndW1lbnRzKSk7cmV0",
				"dXJuIGF9fSxyZW1vdmU6ZnVuY3Rpb24oYSxiKXtmb3IodmFyIGM9MCxkOyhkPXRoaXNbY10pIT1udWxs",
				"O2MrKylpZighYXx8Zi5maWx0ZXIoYSxbZF0pLmxlbmd0aCkhYiYmZC5ub2RlVHlwZT09PTEmJihmLmNs",
				"ZWFuRGF0YShkLmdldEVsZW1lbnRzQnlUYWdOYW1lKCIqIikpLGYuY2xlYW5EYXRhKFtkXSkpLGQucGFy",
				"ZW50Tm9kZSYmZC5wYXJlbnROb2RlLnJlbW92ZUNoaWxkKGQpO3JldHVybiB0aGlzfSxlbXB0eTpmdW5j",
				"dGlvbigpe2Zvcih2YXIgYT0wLGI7KGI9dGhpc1thXSkhPW51bGw7YSsrKXtiLm5vZGVUeXBlPT09MSYm",
				"Zi5jbGVhbkRhdGEoYi5nZXRFbGVtZW50c0J5VGFnTmFtZSgiKiIpKTt3aGlsZShiLmZpcnN0Q2hpbGQp",
				"Yi5yZW1vdmVDaGlsZChiLmZpcnN0Q2hpbGQpfXJldHVybiB0aGlzfSxjbG9uZTpmdW5jdGlvbihhLGIp",
				"e2E9YT09bnVsbD8hMTphLGI9Yj09bnVsbD9hOmI7cmV0dXJuIHRoaXMubWFwKGZ1bmN0aW9uKCl7cmV0",
				"dXJuIGYuY2xvbmUodGhpcyxhLGIpfSl9LGh0bWw6ZnVuY3Rpb24oYSl7cmV0dXJuIGYuYWNjZXNzKHRo",
				"aXMsZnVuY3Rpb24oYSl7dmFyIGM9dGhpc1swXXx8e30sZD0wLGU9dGhpcy5sZW5ndGg7aWYoYT09PWIp",
				"cmV0dXJuIGMubm9kZVR5cGU9PT0xP2MuaW5uZXJIVE1MLnJlcGxhY2UoVywiIik6bnVsbDtpZih0eXBl",
				"b2YgYT09InN0cmluZyImJiFiYS50ZXN0KGEpJiYoZi5zdXBwb3J0LmxlYWRpbmdXaGl0ZXNwYWNlfHwh",
				"WC50ZXN0KGEpKSYmIWJnWyhaLmV4ZWMoYSl8fFsiIiwiIl0pWzFdLnRvTG93ZXJDYXNlKCldKXthPWEu",
				"cmVwbGFjZShZLCI8JDE+PC8kMj4iKTt0cnl7Zm9yKDtkPGU7ZCsrKWM9dGhpc1tkXXx8e30sYy5ub2Rl",
				"VHlwZT09PTEmJihmLmNsZWFuRGF0YShjLmdldEVsZW1lbnRzQnlUYWdOYW1lKCIqIikpLGMuaW5uZXJI",
				"VE1MPWEpO2M9MH1jYXRjaChnKXt9fWMmJnRoaXMuZW1wdHkoKS5hcHBlbmQoYSl9LG51bGwsYSxhcmd1",
				"bWVudHMubGVuZ3RoKX0scmVwbGFjZVdpdGg6ZnVuY3Rpb24oYSl7aWYodGhpc1swXSYmdGhpc1swXS5w",
				"YXJlbnROb2RlKXtpZihmLmlzRnVuY3Rpb24oYSkpcmV0dXJuIHRoaXMuZWFjaChmdW5jdGlvbihiKXt2",
				"YXIgYz1mKHRoaXMpLGQ9Yy5odG1sKCk7Yy5yZXBsYWNlV2l0aChhLmNhbGwodGhpcyxiLGQpKX0pO3R5",
				"cGVvZiBhIT0ic3RyaW5nIiYmKGE9ZihhKS5kZXRhY2goKSk7cmV0dXJuIHRoaXMuZWFjaChmdW5jdGlv",
				"bigpe3ZhciBiPXRoaXMubmV4dFNpYmxpbmcsYz10aGlzLnBhcmVudE5vZGU7Zih0aGlzKS5yZW1vdmUo",
				"KSxiP2YoYikuYmVmb3JlKGEpOmYoYykuYXBwZW5kKGEpfSl9cmV0dXJuIHRoaXMubGVuZ3RoP3RoaXMu",
				"cHVzaFN0YWNrKGYoZi5pc0Z1bmN0aW9uKGEpP2EoKTphKSwicmVwbGFjZVdpdGgiLGEpOnRoaXN9LGRl",
				"dGFjaDpmdW5jdGlvbihhKXtyZXR1cm4gdGhpcy5yZW1vdmUoYSwhMCl9LGRvbU1hbmlwOmZ1bmN0aW9u",
				"KGEsYyxkKXt2YXIgZSxnLGgsaSxqPWFbMF0saz1bXTtpZighZi5zdXBwb3J0LmNoZWNrQ2xvbmUmJmFy",
				"Z3VtZW50cy5sZW5ndGg9PT0zJiZ0eXBlb2Ygaj09InN0cmluZyImJmJkLnRlc3QoaikpcmV0dXJuIHRo",
				"aXMuZWFjaChmdW5jdGlvbigpe2YodGhpcykuZG9tTWFuaXAoYSxjLGQsITApfSk7aWYoZi5pc0Z1bmN0",
				"aW9uKGopKXJldHVybiB0aGlzLmVhY2goZnVuY3Rpb24oZSl7dmFyIGc9Zih0aGlzKTthWzBdPWouY2Fs",
				"bCh0aGlzLGUsYz9nLmh0bWwoKTpiKSxnLmRvbU1hbmlwKGEsYyxkKX0pO2lmKHRoaXNbMF0pe2k9aiYm",
				"ai5wYXJlbnROb2RlLGYuc3VwcG9ydC5wYXJlbnROb2RlJiZpJiZpLm5vZGVUeXBlPT09MTEmJmkuY2hp",
				"bGROb2Rlcy5sZW5ndGg9PT10aGlzLmxlbmd0aD9lPXtmcmFnbWVudDppfTplPWYuYnVpbGRGcmFnbWVu",
				"dChhLHRoaXMsayksaD1lLmZyYWdtZW50LGguY2hpbGROb2Rlcy5sZW5ndGg9PT0xP2c9aD1oLmZpcnN0",
				"Q2hpbGQ6Zz1oLmZpcnN0Q2hpbGQ7aWYoZyl7Yz1jJiZmLm5vZGVOYW1lKGcsInRyIik7Zm9yKHZhciBs",
				"PTAsbT10aGlzLmxlbmd0aCxuPW0tMTtsPG07bCsrKWQuY2FsbChjP2JpKHRoaXNbbF0sZyk6dGhpc1ts",
				"XSxlLmNhY2hlYWJsZXx8bT4xJiZsPG4/Zi5jbG9uZShoLCEwLCEwKTpoKX1rLmxlbmd0aCYmZi5lYWNo",
				"KGssZnVuY3Rpb24oYSxiKXtiLnNyYz9mLmFqYXgoe3R5cGU6IkdFVCIsZ2xvYmFsOiExLHVybDpiLnNy",
				"Yyxhc3luYzohMSxkYXRhVHlwZToic2NyaXB0In0pOmYuZ2xvYmFsRXZhbCgoYi50ZXh0fHxiLnRleHRD",
				"b250ZW50fHxiLmlubmVySFRNTHx8IiIpLnJlcGxhY2UoYmYsIi8qJDAqLyIpKSxiLnBhcmVudE5vZGUm",
				"JmIucGFyZW50Tm9kZS5yZW1vdmVDaGlsZChiKX0pfXJldHVybiB0aGlzfX0pLGYuYnVpbGRGcmFnbWVu",
				"dD1mdW5jdGlvbihhLGIsZCl7dmFyIGUsZyxoLGksaj1hWzBdO2ImJmJbMF0mJihpPWJbMF0ub3duZXJE",
				"b2N1bWVudHx8YlswXSksaS5jcmVhdGVEb2N1bWVudEZyYWdtZW50fHwoaT1jKSxhLmxlbmd0aD09PTEm",
				"JnR5cGVvZiBqPT0ic3RyaW5nIiYmai5sZW5ndGg8NTEyJiZpPT09YyYmai5jaGFyQXQoMCk9PT0iPCIm",
				"JiFiYi50ZXN0KGopJiYoZi5zdXBwb3J0LmNoZWNrQ2xvbmV8fCFiZC50ZXN0KGopKSYmKGYuc3VwcG9y",
				"dC5odG1sNUNsb25lfHwhYmMudGVzdChqKSkmJihnPSEwLGg9Zi5mcmFnbWVudHNbal0saCYmaCE9PTEm",
				"JihlPWgpKSxlfHwoZT1pLmNyZWF0ZURvY3VtZW50RnJhZ21lbnQoKSxmLmNsZWFuKGEsaSxlLGQpKSxn",
				"JiYoZi5mcmFnbWVudHNbal09aD9lOjEpO3JldHVybntmcmFnbWVudDplLGNhY2hlYWJsZTpnfX0sZi5m",
				"cmFnbWVudHM9e30sZi5lYWNoKHthcHBlbmRUbzoiYXBwZW5kIixwcmVwZW5kVG86InByZXBlbmQiLGlu",
				"c2VydEJlZm9yZToiYmVmb3JlIixpbnNlcnRBZnRlcjoiYWZ0ZXIiLHJlcGxhY2VBbGw6InJlcGxhY2VX",
				"aXRoIn0sZnVuY3Rpb24oYSxiKXtmLmZuW2FdPWZ1bmN0aW9uKGMpe3ZhciBkPVtdLGU9ZihjKSxnPXRo",
				"aXMubGVuZ3RoPT09MSYmdGhpc1swXS5wYXJlbnROb2RlO2lmKGcmJmcubm9kZVR5cGU9PT0xMSYmZy5j",
				"aGlsZE5vZGVzLmxlbmd0aD09PTEmJmUubGVuZ3RoPT09MSl7ZVtiXSh0aGlzWzBdKTtyZXR1cm4gdGhp",
				"c31mb3IodmFyIGg9MCxpPWUubGVuZ3RoO2g8aTtoKyspe3ZhciBqPShoPjA/dGhpcy5jbG9uZSghMCk6",
				"dGhpcykuZ2V0KCk7ZihlW2hdKVtiXShqKSxkPWQuY29uY2F0KGopfXJldHVybiB0aGlzLnB1c2hTdGFj",
				"ayhkLGEsZS5zZWxlY3Rvcil9fSksZi5leHRlbmQoe2Nsb25lOmZ1bmN0aW9uKGEsYixjKXt2YXIgZCxl",
				"LGcsaD1mLnN1cHBvcnQuaHRtbDVDbG9uZXx8Zi5pc1hNTERvYyhhKXx8IWJjLnRlc3QoIjwiK2Eubm9k",
				"ZU5hbWUrIj4iKT9hLmNsb25lTm9kZSghMCk6Ym8oYSk7aWYoKCFmLnN1cHBvcnQubm9DbG9uZUV2ZW50",
				"fHwhZi5zdXBwb3J0Lm5vQ2xvbmVDaGVja2VkKSYmKGEubm9kZVR5cGU9PT0xfHxhLm5vZGVUeXBlPT09",
				"MTEpJiYhZi5pc1hNTERvYyhhKSl7YmsoYSxoKSxkPWJsKGEpLGU9YmwoaCk7Zm9yKGc9MDtkW2ddOysr",
				"ZyllW2ddJiZiayhkW2ddLGVbZ10pfWlmKGIpe2JqKGEsaCk7aWYoYyl7ZD1ibChhKSxlPWJsKGgpO2Zv",
				"cihnPTA7ZFtnXTsrK2cpYmooZFtnXSxlW2ddKX19ZD1lPW51bGw7cmV0dXJuIGh9LGNsZWFuOmZ1bmN0",
				"aW9uKGEsYixkLGUpe3ZhciBnLGgsaSxqPVtdO2I9Ynx8Yyx0eXBlb2YgYi5jcmVhdGVFbGVtZW50PT0i",
				"dW5kZWZpbmVkIiYmKGI9Yi5vd25lckRvY3VtZW50fHxiWzBdJiZiWzBdLm93bmVyRG9jdW1lbnR8fGMp",
				"O2Zvcih2YXIgaz0wLGw7KGw9YVtrXSkhPW51bGw7aysrKXt0eXBlb2YgbD09Im51bWJlciImJihsKz0i",
				"Iik7aWYoIWwpY29udGludWU7aWYodHlwZW9mIGw9PSJzdHJpbmciKWlmKCFfLnRlc3QobCkpbD1iLmNy",
				"ZWF0ZVRleHROb2RlKGwpO2Vsc2V7bD1sLnJlcGxhY2UoWSwiPCQxPjwvJDI+Iik7dmFyIG09KFouZXhl",
				"YyhsKXx8WyIiLCIiXSlbMV0udG9Mb3dlckNhc2UoKSxuPWJnW21dfHxiZy5fZGVmYXVsdCxvPW5bMF0s",
				"cD1iLmNyZWF0ZUVsZW1lbnQoImRpdiIpLHE9YmguY2hpbGROb2RlcyxyO2I9PT1jP2JoLmFwcGVuZENo",
				"aWxkKHApOlUoYikuYXBwZW5kQ2hpbGQocCkscC5pbm5lckhUTUw9blsxXStsK25bMl07d2hpbGUoby0t",
				"KXA9cC5sYXN0Q2hpbGQ7aWYoIWYuc3VwcG9ydC50Ym9keSl7dmFyIHM9JC50ZXN0KGwpLHQ9bT09PSJ0",
				"YWJsZSImJiFzP3AuZmlyc3RDaGlsZCYmcC5maXJzdENoaWxkLmNoaWxkTm9kZXM6blsxXT09PSI8dGFi",
				"bGU+IiYmIXM/cC5jaGlsZE5vZGVzOltdO2ZvcihpPXQubGVuZ3RoLTE7aT49MDstLWkpZi5ub2RlTmFt",
				"ZSh0W2ldLCJ0Ym9keSIpJiYhdFtpXS5jaGlsZE5vZGVzLmxlbmd0aCYmdFtpXS5wYXJlbnROb2RlLnJl",
				"bW92ZUNoaWxkKHRbaV0pfSFmLnN1cHBvcnQubGVhZGluZ1doaXRlc3BhY2UmJlgudGVzdChsKSYmcC5p",
				"bnNlcnRCZWZvcmUoYi5jcmVhdGVUZXh0Tm9kZShYLmV4ZWMobClbMF0pLHAuZmlyc3RDaGlsZCksbD1w",
				"LmNoaWxkTm9kZXMscCYmKHAucGFyZW50Tm9kZS5yZW1vdmVDaGlsZChwKSxxLmxlbmd0aD4wJiYocj1x",
				"W3EubGVuZ3RoLTFdLHImJnIucGFyZW50Tm9kZSYmci5wYXJlbnROb2RlLnJlbW92ZUNoaWxkKHIpKSl9",
				"dmFyIHU7aWYoIWYuc3VwcG9ydC5hcHBlbmRDaGVja2VkKWlmKGxbMF0mJnR5cGVvZiAodT1sLmxlbmd0",
				"aCk9PSJudW1iZXIiKWZvcihpPTA7aTx1O2krKylibihsW2ldKTtlbHNlIGJuKGwpO2wubm9kZVR5cGU/",
				"ai5wdXNoKGwpOmo9Zi5tZXJnZShqLGwpfWlmKGQpe2c9ZnVuY3Rpb24oYSl7cmV0dXJuIWEudHlwZXx8",
				"YmUudGVzdChhLnR5cGUpfTtmb3Ioaz0wO2pba107aysrKXtoPWpba107aWYoZSYmZi5ub2RlTmFtZSho",
				"LCJzY3JpcHQiKSYmKCFoLnR5cGV8fGJlLnRlc3QoaC50eXBlKSkpZS5wdXNoKGgucGFyZW50Tm9kZT9o",
				"LnBhcmVudE5vZGUucmVtb3ZlQ2hpbGQoaCk6aCk7ZWxzZXtpZihoLm5vZGVUeXBlPT09MSl7dmFyIHY9",
				"Zi5ncmVwKGguZ2V0RWxlbWVudHNCeVRhZ05hbWUoInNjcmlwdCIpLGcpO2ouc3BsaWNlLmFwcGx5KGos",
				"W2srMSwwXS5jb25jYXQodikpfWQuYXBwZW5kQ2hpbGQoaCl9fX1yZXR1cm4gan0sY2xlYW5EYXRhOmZ1",
				"bmN0aW9uKGEpe3ZhciBiLGMsZD1mLmNhY2hlLGU9Zi5ldmVudC5zcGVjaWFsLGc9Zi5zdXBwb3J0LmRl",
				"bGV0ZUV4cGFuZG87Zm9yKHZhciBoPTAsaTsoaT1hW2hdKSE9bnVsbDtoKyspe2lmKGkubm9kZU5hbWUm",
				"JmYubm9EYXRhW2kubm9kZU5hbWUudG9Mb3dlckNhc2UoKV0pY29udGludWU7Yz1pW2YuZXhwYW5kb107",
				"aWYoYyl7Yj1kW2NdO2lmKGImJmIuZXZlbnRzKXtmb3IodmFyIGogaW4gYi5ldmVudHMpZVtqXT9mLmV2",
				"ZW50LnJlbW92ZShpLGopOmYucmVtb3ZlRXZlbnQoaSxqLGIuaGFuZGxlKTtiLmhhbmRsZSYmKGIuaGFu",
				"ZGxlLmVsZW09bnVsbCl9Zz9kZWxldGUgaVtmLmV4cGFuZG9dOmkucmVtb3ZlQXR0cmlidXRlJiZpLnJl",
				"bW92ZUF0dHJpYnV0ZShmLmV4cGFuZG8pLGRlbGV0ZSBkW2NdfX19fSk7dmFyIGJwPS9hbHBoYVwoW14p",
				"XSpcKS9pLGJxPS9vcGFjaXR5PShbXildKikvLGJyPS8oW0EtWl18Xm1zKS9nLGJzPS9eW1wtK10/KD86",
				"XGQqXC4pP1xkKyQvaSxidD0vXi0/KD86XGQqXC4pP1xkKyg/IXB4KVteXGRcc10rJC9pLGJ1PS9eKFtc",
				"LStdKT0oW1wtKy5cZGVdKykvLGJ2PS9ebWFyZ2luLyxidz17cG9zaXRpb246ImFic29sdXRlIix2aXNp",
				"YmlsaXR5OiJoaWRkZW4iLGRpc3BsYXk6ImJsb2NrIn0sYng9WyJUb3AiLCJSaWdodCIsIkJvdHRvbSIs",
				"IkxlZnQiXSxieSxieixiQTtmLmZuLmNzcz1mdW5jdGlvbihhLGMpe3JldHVybiBmLmFjY2Vzcyh0aGlz",
				"LGZ1bmN0aW9uKGEsYyxkKXtyZXR1cm4gZCE9PWI/Zi5zdHlsZShhLGMsZCk6Zi5jc3MoYSxjKX0sYSxj",
				"LGFyZ3VtZW50cy5sZW5ndGg+MSl9LGYuZXh0ZW5kKHtjc3NIb29rczp7b3BhY2l0eTp7Z2V0OmZ1bmN0",
				"aW9uKGEsYil7aWYoYil7dmFyIGM9YnkoYSwib3BhY2l0eSIpO3JldHVybiBjPT09IiI/IjEiOmN9cmV0",
				"dXJuIGEuc3R5bGUub3BhY2l0eX19fSxjc3NOdW1iZXI6e2ZpbGxPcGFjaXR5OiEwLGZvbnRXZWlnaHQ6",
				"ITAsbGluZUhlaWdodDohMCxvcGFjaXR5OiEwLG9ycGhhbnM6ITAsd2lkb3dzOiEwLHpJbmRleDohMCx6",
				"b29tOiEwfSxjc3NQcm9wczp7ImZsb2F0IjpmLnN1cHBvcnQuY3NzRmxvYXQ/ImNzc0Zsb2F0Ijoic3R5",
				"bGVGbG9hdCJ9LHN0eWxlOmZ1bmN0aW9uKGEsYyxkLGUpe2lmKCEhYSYmYS5ub2RlVHlwZSE9PTMmJmEu",
				"bm9kZVR5cGUhPT04JiYhIWEuc3R5bGUpe3ZhciBnLGgsaT1mLmNhbWVsQ2FzZShjKSxqPWEuc3R5bGUs",
				"az1mLmNzc0hvb2tzW2ldO2M9Zi5jc3NQcm9wc1tpXXx8aTtpZihkPT09Yil7aWYoayYmImdldCJpbiBr",
				"JiYoZz1rLmdldChhLCExLGUpKSE9PWIpcmV0dXJuIGc7cmV0dXJuIGpbY119aD10eXBlb2YgZCxoPT09",
				"InN0cmluZyImJihnPWJ1LmV4ZWMoZCkpJiYoZD0rKGdbMV0rMSkqK2dbMl0rcGFyc2VGbG9hdChmLmNz",
				"cyhhLGMpKSxoPSJudW1iZXIiKTtpZihkPT1udWxsfHxoPT09Im51bWJlciImJmlzTmFOKGQpKXJldHVy",
				"bjtoPT09Im51bWJlciImJiFmLmNzc051bWJlcltpXSYmKGQrPSJweCIpO2lmKCFrfHwhKCJzZXQiaW4g",
				"ayl8fChkPWsuc2V0KGEsZCkpIT09Yil0cnl7altjXT1kfWNhdGNoKGwpe319fSxjc3M6ZnVuY3Rpb24o",
				"YSxjLGQpe3ZhciBlLGc7Yz1mLmNhbWVsQ2FzZShjKSxnPWYuY3NzSG9va3NbY10sYz1mLmNzc1Byb3Bz",
				"W2NdfHxjLGM9PT0iY3NzRmxvYXQiJiYoYz0iZmxvYXQiKTtpZihnJiYiZ2V0ImluIGcmJihlPWcuZ2V0",
				"KGEsITAsZCkpIT09YilyZXR1cm4gZTtpZihieSlyZXR1cm4gYnkoYSxjKX0sc3dhcDpmdW5jdGlvbihh",
				"LGIsYyl7dmFyIGQ9e30sZSxmO2ZvcihmIGluIGIpZFtmXT1hLnN0eWxlW2ZdLGEuc3R5bGVbZl09Yltm",
				"XTtlPWMuY2FsbChhKTtmb3IoZiBpbiBiKWEuc3R5bGVbZl09ZFtmXTtyZXR1cm4gZX19KSxmLmN1ckNT",
				"Uz1mLmNzcyxjLmRlZmF1bHRWaWV3JiZjLmRlZmF1bHRWaWV3LmdldENvbXB1dGVkU3R5bGUmJihiej1m",
				"dW5jdGlvbihhLGIpe3ZhciBjLGQsZSxnLGg9YS5zdHlsZTtiPWIucmVwbGFjZShiciwiLSQxIikudG9M",
				"b3dlckNhc2UoKSwoZD1hLm93bmVyRG9jdW1lbnQuZGVmYXVsdFZpZXcpJiYoZT1kLmdldENvbXB1dGVk",
				"U3R5bGUoYSxudWxsKSkmJihjPWUuZ2V0UHJvcGVydHlWYWx1ZShiKSxjPT09IiImJiFmLmNvbnRhaW5z",
				"KGEub3duZXJEb2N1bWVudC5kb2N1bWVudEVsZW1lbnQsYSkmJihjPWYuc3R5bGUoYSxiKSkpLCFmLnN1",
				"cHBvcnQucGl4ZWxNYXJnaW4mJmUmJmJ2LnRlc3QoYikmJmJ0LnRlc3QoYykmJihnPWgud2lkdGgsaC53",
				"aWR0aD1jLGM9ZS53aWR0aCxoLndpZHRoPWcpO3JldHVybiBjfSksYy5kb2N1bWVudEVsZW1lbnQuY3Vy",
				"cmVudFN0eWxlJiYoYkE9ZnVuY3Rpb24oYSxiKXt2YXIgYyxkLGUsZj1hLmN1cnJlbnRTdHlsZSYmYS5j",
				"dXJyZW50U3R5bGVbYl0sZz1hLnN0eWxlO2Y9PW51bGwmJmcmJihlPWdbYl0pJiYoZj1lKSxidC50ZXN0",
				"KGYpJiYoYz1nLmxlZnQsZD1hLnJ1bnRpbWVTdHlsZSYmYS5ydW50aW1lU3R5bGUubGVmdCxkJiYoYS5y",
				"dW50aW1lU3R5bGUubGVmdD1hLmN1cnJlbnRTdHlsZS5sZWZ0KSxnLmxlZnQ9Yj09PSJmb250U2l6ZSI/",
				"IjFlbSI6ZixmPWcucGl4ZWxMZWZ0KyJweCIsZy5sZWZ0PWMsZCYmKGEucnVudGltZVN0eWxlLmxlZnQ9",
				"ZCkpO3JldHVybiBmPT09IiI/ImF1dG8iOmZ9KSxieT1ienx8YkEsZi5lYWNoKFsiaGVpZ2h0Iiwid2lk",
				"dGgiXSxmdW5jdGlvbihhLGIpe2YuY3NzSG9va3NbYl09e2dldDpmdW5jdGlvbihhLGMsZCl7aWYoYyly",
				"ZXR1cm4gYS5vZmZzZXRXaWR0aCE9PTA/YkIoYSxiLGQpOmYuc3dhcChhLGJ3LGZ1bmN0aW9uKCl7cmV0",
				"dXJuIGJCKGEsYixkKX0pfSxzZXQ6ZnVuY3Rpb24oYSxiKXtyZXR1cm4gYnMudGVzdChiKT9iKyJweCI6",
				"Yn19fSksZi5zdXBwb3J0Lm9wYWNpdHl8fChmLmNzc0hvb2tzLm9wYWNpdHk9e2dldDpmdW5jdGlvbihh",
				"LGIpe3JldHVybiBicS50ZXN0KChiJiZhLmN1cnJlbnRTdHlsZT9hLmN1cnJlbnRTdHlsZS5maWx0ZXI6",
				"YS5zdHlsZS5maWx0ZXIpfHwiIik/cGFyc2VGbG9hdChSZWdFeHAuJDEpLzEwMCsiIjpiPyIxIjoiIn0s",
				"c2V0OmZ1bmN0aW9uKGEsYil7dmFyIGM9YS5zdHlsZSxkPWEuY3VycmVudFN0eWxlLGU9Zi5pc051bWVy",
				"aWMoYik/ImFscGhhKG9wYWNpdHk9IitiKjEwMCsiKSI6IiIsZz1kJiZkLmZpbHRlcnx8Yy5maWx0ZXJ8",
				"fCIiO2Muem9vbT0xO2lmKGI+PTEmJmYudHJpbShnLnJlcGxhY2UoYnAsIiIpKT09PSIiKXtjLnJlbW92",
				"ZUF0dHJpYnV0ZSgiZmlsdGVyIik7aWYoZCYmIWQuZmlsdGVyKXJldHVybn1jLmZpbHRlcj1icC50ZXN0",
				"KGcpP2cucmVwbGFjZShicCxlKTpnKyIgIitlfX0pLGYoZnVuY3Rpb24oKXtmLnN1cHBvcnQucmVsaWFi",
				"bGVNYXJnaW5SaWdodHx8KGYuY3NzSG9va3MubWFyZ2luUmlnaHQ9e2dldDpmdW5jdGlvbihhLGIpe3Jl",
				"dHVybiBmLnN3YXAoYSx7ZGlzcGxheToiaW5saW5lLWJsb2NrIn0sZnVuY3Rpb24oKXtyZXR1cm4gYj9i",
				"eShhLCJtYXJnaW4tcmlnaHQiKTphLnN0eWxlLm1hcmdpblJpZ2h0fSl9fSl9KSxmLmV4cHImJmYuZXhw",
				"ci5maWx0ZXJzJiYoZi5leHByLmZpbHRlcnMuaGlkZGVuPWZ1bmN0aW9uKGEpe3ZhciBiPWEub2Zmc2V0",
				"V2lkdGgsYz1hLm9mZnNldEhlaWdodDtyZXR1cm4gYj09PTAmJmM9PT0wfHwhZi5zdXBwb3J0LnJlbGlh",
				"YmxlSGlkZGVuT2Zmc2V0cyYmKGEuc3R5bGUmJmEuc3R5bGUuZGlzcGxheXx8Zi5jc3MoYSwiZGlzcGxh",
				"eSIpKT09PSJub25lIn0sZi5leHByLmZpbHRlcnMudmlzaWJsZT1mdW5jdGlvbihhKXtyZXR1cm4hZi5l",
				"eHByLmZpbHRlcnMuaGlkZGVuKGEpfSksZi5lYWNoKHttYXJnaW46IiIscGFkZGluZzoiIixib3JkZXI6",
				"IldpZHRoIn0sZnVuY3Rpb24oYSxiKXtmLmNzc0hvb2tzW2ErYl09e2V4cGFuZDpmdW5jdGlvbihjKXt2",
				"YXIgZCxlPXR5cGVvZiBjPT0ic3RyaW5nIj9jLnNwbGl0KCIgIik6W2NdLGY9e307Zm9yKGQ9MDtkPDQ7",
				"ZCsrKWZbYStieFtkXStiXT1lW2RdfHxlW2QtMl18fGVbMF07cmV0dXJuIGZ9fX0pO3ZhciBiQz0vJTIw",
				"L2csYkQ9L1xbXF0kLyxiRT0vXHI/XG4vZyxiRj0vIy4qJC8sYkc9L14oLio/KTpbIFx0XSooW15cclxu",
				"XSopXHI/JC9tZyxiSD0vXig/OmNvbG9yfGRhdGV8ZGF0ZXRpbWV8ZGF0ZXRpbWUtbG9jYWx8ZW1haWx8",
				"aGlkZGVufG1vbnRofG51bWJlcnxwYXNzd29yZHxyYW5nZXxzZWFyY2h8dGVsfHRleHR8dGltZXx1cmx8",
				"d2VlaykkL2ksYkk9L14oPzphYm91dHxhcHB8YXBwXC1zdG9yYWdlfC4rXC1leHRlbnNpb258ZmlsZXxy",
				"ZXN8d2lkZ2V0KTokLyxiSj0vXig/OkdFVHxIRUFEKSQvLGJLPS9eXC9cLy8sYkw9L1w/LyxiTT0vPHNj",
				"cmlwdFxiW148XSooPzooPyE8XC9zY3JpcHQ+KTxbXjxdKikqPFwvc2NyaXB0Pi9naSxiTj0vXig/OnNl",
				"bGVjdHx0ZXh0YXJlYSkvaSxiTz0vXHMrLyxiUD0vKFs/Jl0pXz1bXiZdKi8sYlE9L14oW1x3XCtcLlwt",
				"XSs6KSg/OlwvXC8oW15cLz8jOl0qKSg/OjooXGQrKSk/KT8vLGJSPWYuZm4ubG9hZCxiUz17fSxiVD17",
				"fSxiVSxiVixiVz1bIiovIl0rWyIqIl07dHJ5e2JVPWUuaHJlZn1jYXRjaChiWCl7YlU9Yy5jcmVhdGVF",
				"bGVtZW50KCJhIiksYlUuaHJlZj0iIixiVT1iVS5ocmVmfWJWPWJRLmV4ZWMoYlUudG9Mb3dlckNhc2Uo",
				"KSl8fFtdLGYuZm4uZXh0ZW5kKHtsb2FkOmZ1bmN0aW9uKGEsYyxkKXtpZih0eXBlb2YgYSE9InN0cmlu",
				"ZyImJmJSKXJldHVybiBiUi5hcHBseSh0aGlzLGFyZ3VtZW50cyk7aWYoIXRoaXMubGVuZ3RoKXJldHVy",
				"biB0aGlzO3ZhciBlPWEuaW5kZXhPZigiICIpO2lmKGU+PTApe3ZhciBnPWEuc2xpY2UoZSxhLmxlbmd0",
				"aCk7YT1hLnNsaWNlKDAsZSl9dmFyIGg9IkdFVCI7YyYmKGYuaXNGdW5jdGlvbihjKT8oZD1jLGM9Yik6",
				"dHlwZW9mIGM9PSJvYmplY3QiJiYoYz1mLnBhcmFtKGMsZi5hamF4U2V0dGluZ3MudHJhZGl0aW9uYWwp",
				"LGg9IlBPU1QiKSk7dmFyIGk9dGhpcztmLmFqYXgoe3VybDphLHR5cGU6aCxkYXRhVHlwZToiaHRtbCIs",
				"ZGF0YTpjLGNvbXBsZXRlOmZ1bmN0aW9uKGEsYixjKXtjPWEucmVzcG9uc2VUZXh0LGEuaXNSZXNvbHZl",
				"ZCgpJiYoYS5kb25lKGZ1bmN0aW9uKGEpe2M9YX0pLGkuaHRtbChnP2YoIjxkaXY+IikuYXBwZW5kKGMu",
				"cmVwbGFjZShiTSwiIikpLmZpbmQoZyk6YykpLGQmJmkuZWFjaChkLFtjLGIsYV0pfX0pO3JldHVybiB0",
				"aGlzfSxzZXJpYWxpemU6ZnVuY3Rpb24oKXtyZXR1cm4gZi5wYXJhbSh0aGlzLnNlcmlhbGl6ZUFycmF5",
				"KCkpfSxzZXJpYWxpemVBcnJheTpmdW5jdGlvbigpe3JldHVybiB0aGlzLm1hcChmdW5jdGlvbigpe3Jl",
				"dHVybiB0aGlzLmVsZW1lbnRzP2YubWFrZUFycmF5KHRoaXMuZWxlbWVudHMpOnRoaXN9KS5maWx0ZXIo",
				"ZnVuY3Rpb24oKXtyZXR1cm4gdGhpcy5uYW1lJiYhdGhpcy5kaXNhYmxlZCYmKHRoaXMuY2hlY2tlZHx8",
				"Yk4udGVzdCh0aGlzLm5vZGVOYW1lKXx8YkgudGVzdCh0aGlzLnR5cGUpKX0pLm1hcChmdW5jdGlvbihh",
				"LGIpe3ZhciBjPWYodGhpcykudmFsKCk7cmV0dXJuIGM9PW51bGw/bnVsbDpmLmlzQXJyYXkoYyk/Zi5t",
				"YXAoYyxmdW5jdGlvbihhLGMpe3JldHVybntuYW1lOmIubmFtZSx2YWx1ZTphLnJlcGxhY2UoYkUsIlxy",
				"XG4iKX19KTp7bmFtZTpiLm5hbWUsdmFsdWU6Yy5yZXBsYWNlKGJFLCJcclxuIil9fSkuZ2V0KCl9fSks",
				"Zi5lYWNoKCJhamF4U3RhcnQgYWpheFN0b3AgYWpheENvbXBsZXRlIGFqYXhFcnJvciBhamF4U3VjY2Vz",
				"cyBhamF4U2VuZCIuc3BsaXQoIiAiKSxmdW5jdGlvbihhLGIpe2YuZm5bYl09ZnVuY3Rpb24oYSl7cmV0",
				"dXJuIHRoaXMub24oYixhKX19KSxmLmVhY2goWyJnZXQiLCJwb3N0Il0sZnVuY3Rpb24oYSxjKXtmW2Nd",
				"PWZ1bmN0aW9uKGEsZCxlLGcpe2YuaXNGdW5jdGlvbihkKSYmKGc9Z3x8ZSxlPWQsZD1iKTtyZXR1cm4g",
				"Zi5hamF4KHt0eXBlOmMsdXJsOmEsZGF0YTpkLHN1Y2Nlc3M6ZSxkYXRhVHlwZTpnfSl9fSksZi5leHRl",
				"bmQoe2dldFNjcmlwdDpmdW5jdGlvbihhLGMpe3JldHVybiBmLmdldChhLGIsYywic2NyaXB0Iil9LGdl",
				"dEpTT046ZnVuY3Rpb24oYSxiLGMpe3JldHVybiBmLmdldChhLGIsYywianNvbiIpfSxhamF4U2V0dXA6",
				"ZnVuY3Rpb24oYSxiKXtiP2IkKGEsZi5hamF4U2V0dGluZ3MpOihiPWEsYT1mLmFqYXhTZXR0aW5ncyks",
				"YiQoYSxiKTtyZXR1cm4gYX0sYWpheFNldHRpbmdzOnt1cmw6YlUsaXNMb2NhbDpiSS50ZXN0KGJWWzFd",
				"KSxnbG9iYWw6ITAsdHlwZToiR0VUIixjb250ZW50VHlwZToiYXBwbGljYXRpb24veC13d3ctZm9ybS11",
				"cmxlbmNvZGVkOyBjaGFyc2V0PVVURi04Iixwcm9jZXNzRGF0YTohMCxhc3luYzohMCxhY2NlcHRzOnt4",
				"bWw6ImFwcGxpY2F0aW9uL3htbCwgdGV4dC94bWwiLGh0bWw6InRleHQvaHRtbCIsdGV4dDoidGV4dC9w",
				"bGFpbiIsanNvbjoiYXBwbGljYXRpb24vanNvbiwgdGV4dC9qYXZhc2NyaXB0IiwiKiI6Yld9LGNvbnRl",
				"bnRzOnt4bWw6L3htbC8saHRtbDovaHRtbC8sanNvbjovanNvbi99LHJlc3BvbnNlRmllbGRzOnt4bWw6",
				"InJlc3BvbnNlWE1MIix0ZXh0OiJyZXNwb25zZVRleHQifSxjb252ZXJ0ZXJzOnsiKiB0ZXh0IjphLlN0",
				"cmluZywidGV4dCBodG1sIjohMCwidGV4dCBqc29uIjpmLnBhcnNlSlNPTiwidGV4dCB4bWwiOmYucGFy",
				"c2VYTUx9LGZsYXRPcHRpb25zOntjb250ZXh0OiEwLHVybDohMH19LGFqYXhQcmVmaWx0ZXI6YlkoYlMp",
				"LGFqYXhUcmFuc3BvcnQ6YlkoYlQpLGFqYXg6ZnVuY3Rpb24oYSxjKXtmdW5jdGlvbiB3KGEsYyxsLG0p",
				"e2lmKHMhPT0yKXtzPTIscSYmY2xlYXJUaW1lb3V0KHEpLHA9YixuPW18fCIiLHYucmVhZHlTdGF0ZT1h",
				"PjA/NDowO3ZhciBvLHIsdSx3PWMseD1sP2NhKGQsdixsKTpiLHksejtpZihhPj0yMDAmJmE8MzAwfHxh",
				"PT09MzA0KXtpZihkLmlmTW9kaWZpZWQpe2lmKHk9di5nZXRSZXNwb25zZUhlYWRlcigiTGFzdC1Nb2Rp",
				"ZmllZCIpKWYubGFzdE1vZGlmaWVkW2tdPXk7aWYoej12LmdldFJlc3BvbnNlSGVhZGVyKCJFdGFnIikp",
				"Zi5ldGFnW2tdPXp9aWYoYT09PTMwNCl3PSJub3Rtb2RpZmllZCIsbz0hMDtlbHNlIHRyeXtyPWNiKGQs",
				"eCksdz0ic3VjY2VzcyIsbz0hMH1jYXRjaChBKXt3PSJwYXJzZXJlcnJvciIsdT1BfX1lbHNle3U9dztp",
				"Zighd3x8YSl3PSJlcnJvciIsYTwwJiYoYT0wKX12LnN0YXR1cz1hLHYuc3RhdHVzVGV4dD0iIisoY3x8",
				"dyksbz9oLnJlc29sdmVXaXRoKGUsW3Isdyx2XSk6aC5yZWplY3RXaXRoKGUsW3Ysdyx1XSksdi5zdGF0",
				"dXNDb2RlKGopLGo9Yix0JiZnLnRyaWdnZXIoImFqYXgiKyhvPyJTdWNjZXNzIjoiRXJyb3IiKSxbdixk",
				"LG8/cjp1XSksaS5maXJlV2l0aChlLFt2LHddKSx0JiYoZy50cmlnZ2VyKCJhamF4Q29tcGxldGUiLFt2",
				"LGRdKSwtLWYuYWN0aXZlfHxmLmV2ZW50LnRyaWdnZXIoImFqYXhTdG9wIikpfX10eXBlb2YgYT09Im9i",
				"amVjdCImJihjPWEsYT1iKSxjPWN8fHt9O3ZhciBkPWYuYWpheFNldHVwKHt9LGMpLGU9ZC5jb250ZXh0",
				"fHxkLGc9ZSE9PWQmJihlLm5vZGVUeXBlfHxlIGluc3RhbmNlb2YgZik/ZihlKTpmLmV2ZW50LGg9Zi5E",
				"ZWZlcnJlZCgpLGk9Zi5DYWxsYmFja3MoIm9uY2UgbWVtb3J5Iiksaj1kLnN0YXR1c0NvZGV8fHt9LGss",
				"bD17fSxtPXt9LG4sbyxwLHEscixzPTAsdCx1LHY9e3JlYWR5U3RhdGU6MCxzZXRSZXF1ZXN0SGVhZGVy",
				"OmZ1bmN0aW9uKGEsYil7aWYoIXMpe3ZhciBjPWEudG9Mb3dlckNhc2UoKTthPW1bY109bVtjXXx8YSxs",
				"W2FdPWJ9cmV0dXJuIHRoaXN9LGdldEFsbFJlc3BvbnNlSGVhZGVyczpmdW5jdGlvbigpe3JldHVybiBz",
				"PT09Mj9uOm51bGx9LGdldFJlc3BvbnNlSGVhZGVyOmZ1bmN0aW9uKGEpe3ZhciBjO2lmKHM9PT0yKXtp",
				"Zighbyl7bz17fTt3aGlsZShjPWJHLmV4ZWMobikpb1tjWzFdLnRvTG93ZXJDYXNlKCldPWNbMl19Yz1v",
				"W2EudG9Mb3dlckNhc2UoKV19cmV0dXJuIGM9PT1iP251bGw6Y30sb3ZlcnJpZGVNaW1lVHlwZTpmdW5j",
				"dGlvbihhKXtzfHwoZC5taW1lVHlwZT1hKTtyZXR1cm4gdGhpc30sYWJvcnQ6ZnVuY3Rpb24oYSl7YT1h",
				"fHwiYWJvcnQiLHAmJnAuYWJvcnQoYSksdygwLGEpO3JldHVybiB0aGlzfX07aC5wcm9taXNlKHYpLHYu",
				"c3VjY2Vzcz12LmRvbmUsdi5lcnJvcj12LmZhaWwsdi5jb21wbGV0ZT1pLmFkZCx2LnN0YXR1c0NvZGU9",
				"ZnVuY3Rpb24oYSl7aWYoYSl7dmFyIGI7aWYoczwyKWZvcihiIGluIGEpaltiXT1baltiXSxhW2JdXTtl",
				"bHNlIGI9YVt2LnN0YXR1c10sdi50aGVuKGIsYil9cmV0dXJuIHRoaXN9LGQudXJsPSgoYXx8ZC51cmwp",
				"KyIiKS5yZXBsYWNlKGJGLCIiKS5yZXBsYWNlKGJLLGJWWzFdKyIvLyIpLGQuZGF0YVR5cGVzPWYudHJp",
				"bShkLmRhdGFUeXBlfHwiKiIpLnRvTG93ZXJDYXNlKCkuc3BsaXQoYk8pLGQuY3Jvc3NEb21haW49PW51",
				"bGwmJihyPWJRLmV4ZWMoZC51cmwudG9Mb3dlckNhc2UoKSksZC5jcm9zc0RvbWFpbj0hKCFyfHxyWzFd",
				"PT1iVlsxXSYmclsyXT09YlZbMl0mJihyWzNdfHwoclsxXT09PSJodHRwOiI/ODA6NDQzKSk9PShiVlsz",
				"XXx8KGJWWzFdPT09Imh0dHA6Ij84MDo0NDMpKSkpLGQuZGF0YSYmZC5wcm9jZXNzRGF0YSYmdHlwZW9m",
				"IGQuZGF0YSE9InN0cmluZyImJihkLmRhdGE9Zi5wYXJhbShkLmRhdGEsZC50cmFkaXRpb25hbCkpLGJa",
				"KGJTLGQsYyx2KTtpZihzPT09MilyZXR1cm4hMTt0PWQuZ2xvYmFsLGQudHlwZT1kLnR5cGUudG9VcHBl",
				"ckNhc2UoKSxkLmhhc0NvbnRlbnQ9IWJKLnRlc3QoZC50eXBlKSx0JiZmLmFjdGl2ZSsrPT09MCYmZi5l",
				"dmVudC50cmlnZ2VyKCJhamF4U3RhcnQiKTtpZighZC5oYXNDb250ZW50KXtkLmRhdGEmJihkLnVybCs9",
				"KGJMLnRlc3QoZC51cmwpPyImIjoiPyIpK2QuZGF0YSxkZWxldGUgZC5kYXRhKSxrPWQudXJsO2lmKGQu",
				"Y2FjaGU9PT0hMSl7dmFyIHg9Zi5ub3coKSx5PWQudXJsLnJlcGxhY2UoYlAsIiQxXz0iK3gpO2QudXJs",
				"PXkrKHk9PT1kLnVybD8oYkwudGVzdChkLnVybCk/IiYiOiI/IikrIl89Iit4OiIiKX19KGQuZGF0YSYm",
				"ZC5oYXNDb250ZW50JiZkLmNvbnRlbnRUeXBlIT09ITF8fGMuY29udGVudFR5cGUpJiZ2LnNldFJlcXVl",
				"c3RIZWFkZXIoIkNvbnRlbnQtVHlwZSIsZC5jb250ZW50VHlwZSksZC5pZk1vZGlmaWVkJiYoaz1rfHxk",
				"LnVybCxmLmxhc3RNb2RpZmllZFtrXSYmdi5zZXRSZXF1ZXN0SGVhZGVyKCJJZi1Nb2RpZmllZC1TaW5j",
				"ZSIsZi5sYXN0TW9kaWZpZWRba10pLGYuZXRhZ1trXSYmdi5zZXRSZXF1ZXN0SGVhZGVyKCJJZi1Ob25l",
				"LU1hdGNoIixmLmV0YWdba10pKSx2LnNldFJlcXVlc3RIZWFkZXIoIkFjY2VwdCIsZC5kYXRhVHlwZXNb",
				"MF0mJmQuYWNjZXB0c1tkLmRhdGFUeXBlc1swXV0/ZC5hY2NlcHRzW2QuZGF0YVR5cGVzWzBdXSsoZC5k",
				"YXRhVHlwZXNbMF0hPT0iKiI/IiwgIitiVysiOyBxPTAuMDEiOiIiKTpkLmFjY2VwdHNbIioiXSk7Zm9y",
				"KHUgaW4gZC5oZWFkZXJzKXYuc2V0UmVxdWVzdEhlYWRlcih1LGQuaGVhZGVyc1t1XSk7aWYoZC5iZWZv",
				"cmVTZW5kJiYoZC5iZWZvcmVTZW5kLmNhbGwoZSx2LGQpPT09ITF8fHM9PT0yKSl7di5hYm9ydCgpO3Jl",
				"dHVybiExfWZvcih1IGlue3N1Y2Nlc3M6MSxlcnJvcjoxLGNvbXBsZXRlOjF9KXZbdV0oZFt1XSk7cD1i",
				"WihiVCxkLGMsdik7aWYoIXApdygtMSwiTm8gVHJhbnNwb3J0Iik7ZWxzZXt2LnJlYWR5U3RhdGU9MSx0",
				"JiZnLnRyaWdnZXIoImFqYXhTZW5kIixbdixkXSksZC5hc3luYyYmZC50aW1lb3V0PjAmJihxPXNldFRp",
				"bWVvdXQoZnVuY3Rpb24oKXt2LmFib3J0KCJ0aW1lb3V0Iil9LGQudGltZW91dCkpO3RyeXtzPTEscC5z",
				"ZW5kKGwsdyl9Y2F0Y2goeil7aWYoczwyKXcoLTEseik7ZWxzZSB0aHJvdyB6fX1yZXR1cm4gdn0scGFy",
				"YW06ZnVuY3Rpb24oYSxjKXt2YXIgZD1bXSxlPWZ1bmN0aW9uKGEsYil7Yj1mLmlzRnVuY3Rpb24oYik/",
				"YigpOmIsZFtkLmxlbmd0aF09ZW5jb2RlVVJJQ29tcG9uZW50KGEpKyI9IitlbmNvZGVVUklDb21wb25l",
				"bnQoYil9O2M9PT1iJiYoYz1mLmFqYXhTZXR0aW5ncy50cmFkaXRpb25hbCk7aWYoZi5pc0FycmF5KGEp",
				"fHxhLmpxdWVyeSYmIWYuaXNQbGFpbk9iamVjdChhKSlmLmVhY2goYSxmdW5jdGlvbigpe2UodGhpcy5u",
				"YW1lLHRoaXMudmFsdWUpfSk7ZWxzZSBmb3IodmFyIGcgaW4gYSliXyhnLGFbZ10sYyxlKTtyZXR1cm4g",
				"ZC5qb2luKCImIikucmVwbGFjZShiQywiKyIpfX0pLGYuZXh0ZW5kKHthY3RpdmU6MCxsYXN0TW9kaWZp",
				"ZWQ6e30sZXRhZzp7fX0pO3ZhciBjYz1mLm5vdygpLGNkPS8oXD0pXD8oJnwkKXxcP1w/L2k7Zi5hamF4",
				"U2V0dXAoe2pzb25wOiJjYWxsYmFjayIsanNvbnBDYWxsYmFjazpmdW5jdGlvbigpe3JldHVybiBmLmV4",
				"cGFuZG8rIl8iK2NjKyt9fSksZi5hamF4UHJlZmlsdGVyKCJqc29uIGpzb25wIixmdW5jdGlvbihiLGMs",
				"ZCl7dmFyIGU9dHlwZW9mIGIuZGF0YT09InN0cmluZyImJi9eYXBwbGljYXRpb25cL3hcLXd3d1wtZm9y",
				"bVwtdXJsZW5jb2RlZC8udGVzdChiLmNvbnRlbnRUeXBlKTtpZihiLmRhdGFUeXBlc1swXT09PSJqc29u",
				"cCJ8fGIuanNvbnAhPT0hMSYmKGNkLnRlc3QoYi51cmwpfHxlJiZjZC50ZXN0KGIuZGF0YSkpKXt2YXIg",
				"ZyxoPWIuanNvbnBDYWxsYmFjaz1mLmlzRnVuY3Rpb24oYi5qc29ucENhbGxiYWNrKT9iLmpzb25wQ2Fs",
				"bGJhY2soKTpiLmpzb25wQ2FsbGJhY2ssaT1hW2hdLGo9Yi51cmwsaz1iLmRhdGEsbD0iJDEiK2grIiQy",
				"IjtiLmpzb25wIT09ITEmJihqPWoucmVwbGFjZShjZCxsKSxiLnVybD09PWomJihlJiYoaz1rLnJlcGxh",
				"Y2UoY2QsbCkpLGIuZGF0YT09PWsmJihqKz0oL1w/Ly50ZXN0KGopPyImIjoiPyIpK2IuanNvbnArIj0i",
				"K2gpKSksYi51cmw9aixiLmRhdGE9ayxhW2hdPWZ1bmN0aW9uKGEpe2c9W2FdfSxkLmFsd2F5cyhmdW5j",
				"dGlvbigpe2FbaF09aSxnJiZmLmlzRnVuY3Rpb24oaSkmJmFbaF0oZ1swXSl9KSxiLmNvbnZlcnRlcnNb",
				"InNjcmlwdCBqc29uIl09ZnVuY3Rpb24oKXtnfHxmLmVycm9yKGgrIiB3YXMgbm90IGNhbGxlZCIpO3Jl",
				"dHVybiBnWzBdfSxiLmRhdGFUeXBlc1swXT0ianNvbiI7cmV0dXJuInNjcmlwdCJ9fSksZi5hamF4U2V0",
				"dXAoe2FjY2VwdHM6e3NjcmlwdDoidGV4dC9qYXZhc2NyaXB0LCBhcHBsaWNhdGlvbi9qYXZhc2NyaXB0",
				"LCBhcHBsaWNhdGlvbi9lY21hc2NyaXB0LCBhcHBsaWNhdGlvbi94LWVjbWFzY3JpcHQifSxjb250ZW50",
				"czp7c2NyaXB0Oi9qYXZhc2NyaXB0fGVjbWFzY3JpcHQvfSxjb252ZXJ0ZXJzOnsidGV4dCBzY3JpcHQi",
				"OmZ1bmN0aW9uKGEpe2YuZ2xvYmFsRXZhbChhKTtyZXR1cm4gYX19fSksZi5hamF4UHJlZmlsdGVyKCJz",
				"Y3JpcHQiLGZ1bmN0aW9uKGEpe2EuY2FjaGU9PT1iJiYoYS5jYWNoZT0hMSksYS5jcm9zc0RvbWFpbiYm",
				"KGEudHlwZT0iR0VUIixhLmdsb2JhbD0hMSl9KSxmLmFqYXhUcmFuc3BvcnQoInNjcmlwdCIsZnVuY3Rp",
				"b24oYSl7aWYoYS5jcm9zc0RvbWFpbil7dmFyIGQsZT1jLmhlYWR8fGMuZ2V0RWxlbWVudHNCeVRhZ05h",
				"bWUoImhlYWQiKVswXXx8Yy5kb2N1bWVudEVsZW1lbnQ7cmV0dXJue3NlbmQ6ZnVuY3Rpb24oZixnKXtk",
				"PWMuY3JlYXRlRWxlbWVudCgic2NyaXB0IiksZC5hc3luYz0iYXN5bmMiLGEuc2NyaXB0Q2hhcnNldCYm",
				"KGQuY2hhcnNldD1hLnNjcmlwdENoYXJzZXQpLGQuc3JjPWEudXJsLGQub25sb2FkPWQub25yZWFkeXN0",
				"YXRlY2hhbmdlPWZ1bmN0aW9uKGEsYyl7aWYoY3x8IWQucmVhZHlTdGF0ZXx8L2xvYWRlZHxjb21wbGV0",
				"ZS8udGVzdChkLnJlYWR5U3RhdGUpKWQub25sb2FkPWQub25yZWFkeXN0YXRlY2hhbmdlPW51bGwsZSYm",
				"ZC5wYXJlbnROb2RlJiZlLnJlbW92ZUNoaWxkKGQpLGQ9YixjfHxnKDIwMCwic3VjY2VzcyIpfSxlLmlu",
				"c2VydEJlZm9yZShkLGUuZmlyc3RDaGlsZCl9LGFib3J0OmZ1bmN0aW9uKCl7ZCYmZC5vbmxvYWQoMCwx",
				"KX19fX0pO3ZhciBjZT1hLkFjdGl2ZVhPYmplY3Q/ZnVuY3Rpb24oKXtmb3IodmFyIGEgaW4gY2cpY2db",
				"YV0oMCwxKX06ITEsY2Y9MCxjZztmLmFqYXhTZXR0aW5ncy54aHI9YS5BY3RpdmVYT2JqZWN0P2Z1bmN0",
				"aW9uKCl7cmV0dXJuIXRoaXMuaXNMb2NhbCYmY2goKXx8Y2koKX06Y2gsZnVuY3Rpb24oYSl7Zi5leHRl",
				"bmQoZi5zdXBwb3J0LHthamF4OiEhYSxjb3JzOiEhYSYmIndpdGhDcmVkZW50aWFscyJpbiBhfSl9KGYu",
				"YWpheFNldHRpbmdzLnhocigpKSxmLnN1cHBvcnQuYWpheCYmZi5hamF4VHJhbnNwb3J0KGZ1bmN0aW9u",
				"KGMpe2lmKCFjLmNyb3NzRG9tYWlufHxmLnN1cHBvcnQuY29ycyl7dmFyIGQ7cmV0dXJue3NlbmQ6ZnVu",
				"Y3Rpb24oZSxnKXt2YXIgaD1jLnhocigpLGksajtjLnVzZXJuYW1lP2gub3BlbihjLnR5cGUsYy51cmws",
				"Yy5hc3luYyxjLnVzZXJuYW1lLGMucGFzc3dvcmQpOmgub3BlbihjLnR5cGUsYy51cmwsYy5hc3luYyk7",
				"aWYoYy54aHJGaWVsZHMpZm9yKGogaW4gYy54aHJGaWVsZHMpaFtqXT1jLnhockZpZWxkc1tqXTtjLm1p",
				"bWVUeXBlJiZoLm92ZXJyaWRlTWltZVR5cGUmJmgub3ZlcnJpZGVNaW1lVHlwZShjLm1pbWVUeXBlKSwh",
				"Yy5jcm9zc0RvbWFpbiYmIWVbIlgtUmVxdWVzdGVkLVdpdGgiXSYmKGVbIlgtUmVxdWVzdGVkLVdpdGgi",
				"XT0iWE1MSHR0cFJlcXVlc3QiKTt0cnl7Zm9yKGogaW4gZSloLnNldFJlcXVlc3RIZWFkZXIoaixlW2pd",
				"KX1jYXRjaChrKXt9aC5zZW5kKGMuaGFzQ29udGVudCYmYy5kYXRhfHxudWxsKSxkPWZ1bmN0aW9uKGEs",
				"ZSl7dmFyIGosayxsLG0sbjt0cnl7aWYoZCYmKGV8fGgucmVhZHlTdGF0ZT09PTQpKXtkPWIsaSYmKGgu",
				"b25yZWFkeXN0YXRlY2hhbmdlPWYubm9vcCxjZSYmZGVsZXRlIGNnW2ldKTtpZihlKWgucmVhZHlTdGF0",
				"ZSE9PTQmJmguYWJvcnQoKTtlbHNle2o9aC5zdGF0dXMsbD1oLmdldEFsbFJlc3BvbnNlSGVhZGVycygp",
				"LG09e30sbj1oLnJlc3BvbnNlWE1MLG4mJm4uZG9jdW1lbnRFbGVtZW50JiYobS54bWw9bik7dHJ5e20u",
				"dGV4dD1oLnJlc3BvbnNlVGV4dH1jYXRjaChhKXt9dHJ5e2s9aC5zdGF0dXNUZXh0fWNhdGNoKG8pe2s9",
				"IiJ9IWomJmMuaXNMb2NhbCYmIWMuY3Jvc3NEb21haW4/aj1tLnRleHQ/MjAwOjQwNDpqPT09MTIyMyYm",
				"KGo9MjA0KX19fWNhdGNoKHApe2V8fGcoLTEscCl9bSYmZyhqLGssbSxsKX0sIWMuYXN5bmN8fGgucmVh",
				"ZHlTdGF0ZT09PTQ/ZCgpOihpPSsrY2YsY2UmJihjZ3x8KGNnPXt9LGYoYSkudW5sb2FkKGNlKSksY2db",
				"aV09ZCksaC5vbnJlYWR5c3RhdGVjaGFuZ2U9ZCl9LGFib3J0OmZ1bmN0aW9uKCl7ZCYmZCgwLDEpfX19",
				"fSk7dmFyIGNqPXt9LGNrLGNsLGNtPS9eKD86dG9nZ2xlfHNob3d8aGlkZSkkLyxjbj0vXihbK1wtXT0p",
				"PyhbXGQrLlwtXSspKFthLXolXSopJC9pLGNvLGNwPVtbImhlaWdodCIsIm1hcmdpblRvcCIsIm1hcmdp",
				"bkJvdHRvbSIsInBhZGRpbmdUb3AiLCJwYWRkaW5nQm90dG9tIl0sWyJ3aWR0aCIsIm1hcmdpbkxlZnQi",
				"LCJtYXJnaW5SaWdodCIsInBhZGRpbmdMZWZ0IiwicGFkZGluZ1JpZ2h0Il0sWyJvcGFjaXR5Il1dLGNx",
				"O2YuZm4uZXh0ZW5kKHtzaG93OmZ1bmN0aW9uKGEsYixjKXt2YXIgZCxlO2lmKGF8fGE9PT0wKXJldHVy",
				"biB0aGlzLmFuaW1hdGUoY3QoInNob3ciLDMpLGEsYixjKTtmb3IodmFyIGc9MCxoPXRoaXMubGVuZ3Ro",
				"O2c8aDtnKyspZD10aGlzW2ddLGQuc3R5bGUmJihlPWQuc3R5bGUuZGlzcGxheSwhZi5fZGF0YShkLCJv",
				"bGRkaXNwbGF5IikmJmU9PT0ibm9uZSImJihlPWQuc3R5bGUuZGlzcGxheT0iIiksKGU9PT0iIiYmZi5j",
				"c3MoZCwiZGlzcGxheSIpPT09Im5vbmUifHwhZi5jb250YWlucyhkLm93bmVyRG9jdW1lbnQuZG9jdW1l",
				"bnRFbGVtZW50LGQpKSYmZi5fZGF0YShkLCJvbGRkaXNwbGF5IixjdShkLm5vZGVOYW1lKSkpO2Zvcihn",
				"PTA7ZzxoO2crKyl7ZD10aGlzW2ddO2lmKGQuc3R5bGUpe2U9ZC5zdHlsZS5kaXNwbGF5O2lmKGU9PT0i",
				"Inx8ZT09PSJub25lIilkLnN0eWxlLmRpc3BsYXk9Zi5fZGF0YShkLCJvbGRkaXNwbGF5Iil8fCIifX1y",
				"ZXR1cm4gdGhpc30saGlkZTpmdW5jdGlvbihhLGIsYyl7aWYoYXx8YT09PTApcmV0dXJuIHRoaXMuYW5p",
				"bWF0ZShjdCgiaGlkZSIsMyksYSxiLGMpO3ZhciBkLGUsZz0wLGg9dGhpcy5sZW5ndGg7Zm9yKDtnPGg7",
				"ZysrKWQ9dGhpc1tnXSxkLnN0eWxlJiYoZT1mLmNzcyhkLCJkaXNwbGF5IiksZSE9PSJub25lIiYmIWYu",
				"X2RhdGEoZCwib2xkZGlzcGxheSIpJiZmLl9kYXRhKGQsIm9sZGRpc3BsYXkiLGUpKTtmb3IoZz0wO2c8",
				"aDtnKyspdGhpc1tnXS5zdHlsZSYmKHRoaXNbZ10uc3R5bGUuZGlzcGxheT0ibm9uZSIpO3JldHVybiB0",
				"aGlzfSxfdG9nZ2xlOmYuZm4udG9nZ2xlLHRvZ2dsZTpmdW5jdGlvbihhLGIsYyl7dmFyIGQ9dHlwZW9m",
				"IGE9PSJib29sZWFuIjtmLmlzRnVuY3Rpb24oYSkmJmYuaXNGdW5jdGlvbihiKT90aGlzLl90b2dnbGUu",
				"YXBwbHkodGhpcyxhcmd1bWVudHMpOmE9PW51bGx8fGQ/dGhpcy5lYWNoKGZ1bmN0aW9uKCl7dmFyIGI9",
				"ZD9hOmYodGhpcykuaXMoIjpoaWRkZW4iKTtmKHRoaXMpW2I/InNob3ciOiJoaWRlIl0oKX0pOnRoaXMu",
				"YW5pbWF0ZShjdCgidG9nZ2xlIiwzKSxhLGIsYyk7cmV0dXJuIHRoaXN9LGZhZGVUbzpmdW5jdGlvbihh",
				"LGIsYyxkKXtyZXR1cm4gdGhpcy5maWx0ZXIoIjpoaWRkZW4iKS5jc3MoIm9wYWNpdHkiLDApLnNob3co",
				"KS5lbmQoKS5hbmltYXRlKHtvcGFjaXR5OmJ9LGEsYyxkKX0sYW5pbWF0ZTpmdW5jdGlvbihhLGIsYyxk",
				"KXtmdW5jdGlvbiBnKCl7ZS5xdWV1ZT09PSExJiZmLl9tYXJrKHRoaXMpO3ZhciBiPWYuZXh0ZW5kKHt9",
				"LGUpLGM9dGhpcy5ub2RlVHlwZT09PTEsZD1jJiZmKHRoaXMpLmlzKCI6aGlkZGVuIiksZyxoLGksaixr",
				"LGwsbSxuLG8scCxxO2IuYW5pbWF0ZWRQcm9wZXJ0aWVzPXt9O2ZvcihpIGluIGEpe2c9Zi5jYW1lbENh",
				"c2UoaSksaSE9PWcmJihhW2ddPWFbaV0sZGVsZXRlIGFbaV0pO2lmKChrPWYuY3NzSG9va3NbZ10pJiYi",
				"ZXhwYW5kImluIGspe2w9ay5leHBhbmQoYVtnXSksZGVsZXRlIGFbZ107Zm9yKGkgaW4gbClpIGluIGF8",
				"fChhW2ldPWxbaV0pfX1mb3IoZyBpbiBhKXtoPWFbZ10sZi5pc0FycmF5KGgpPyhiLmFuaW1hdGVkUHJv",
				"cGVydGllc1tnXT1oWzFdLGg9YVtnXT1oWzBdKTpiLmFuaW1hdGVkUHJvcGVydGllc1tnXT1iLnNwZWNp",
				"YWxFYXNpbmcmJmIuc3BlY2lhbEVhc2luZ1tnXXx8Yi5lYXNpbmd8fCJzd2luZyI7aWYoaD09PSJoaWRl",
				"IiYmZHx8aD09PSJzaG93IiYmIWQpcmV0dXJuIGIuY29tcGxldGUuY2FsbCh0aGlzKTtjJiYoZz09PSJo",
				"ZWlnaHQifHxnPT09IndpZHRoIikmJihiLm92ZXJmbG93PVt0aGlzLnN0eWxlLm92ZXJmbG93LHRoaXMu",
				"c3R5bGUub3ZlcmZsb3dYLHRoaXMuc3R5bGUub3ZlcmZsb3dZXSxmLmNzcyh0aGlzLCJkaXNwbGF5Iik9",
				"PT0iaW5saW5lIiYmZi5jc3ModGhpcywiZmxvYXQiKT09PSJub25lIiYmKCFmLnN1cHBvcnQuaW5saW5l",
				"QmxvY2tOZWVkc0xheW91dHx8Y3UodGhpcy5ub2RlTmFtZSk9PT0iaW5saW5lIj90aGlzLnN0eWxlLmRp",
				"c3BsYXk9ImlubGluZS1ibG9jayI6dGhpcy5zdHlsZS56b29tPTEpKX1iLm92ZXJmbG93IT1udWxsJiYo",
				"dGhpcy5zdHlsZS5vdmVyZmxvdz0iaGlkZGVuIik7Zm9yKGkgaW4gYSlqPW5ldyBmLmZ4KHRoaXMsYixp",
				"KSxoPWFbaV0sY20udGVzdChoKT8ocT1mLl9kYXRhKHRoaXMsInRvZ2dsZSIraSl8fChoPT09InRvZ2ds",
				"ZSI/ZD8ic2hvdyI6ImhpZGUiOjApLHE/KGYuX2RhdGEodGhpcywidG9nZ2xlIitpLHE9PT0ic2hvdyI/",
				"ImhpZGUiOiJzaG93IiksaltxXSgpKTpqW2hdKCkpOihtPWNuLmV4ZWMoaCksbj1qLmN1cigpLG0/KG89",
				"cGFyc2VGbG9hdChtWzJdKSxwPW1bM118fChmLmNzc051bWJlcltpXT8iIjoicHgiKSxwIT09InB4IiYm",
				"KGYuc3R5bGUodGhpcyxpLChvfHwxKStwKSxuPShvfHwxKS9qLmN1cigpKm4sZi5zdHlsZSh0aGlzLGks",
				"bitwKSksbVsxXSYmKG89KG1bMV09PT0iLT0iPy0xOjEpKm8rbiksai5jdXN0b20obixvLHApKTpqLmN1",
				"c3RvbShuLGgsIiIpKTtyZXR1cm4hMH12YXIgZT1mLnNwZWVkKGIsYyxkKTtpZihmLmlzRW1wdHlPYmpl",
				"Y3QoYSkpcmV0dXJuIHRoaXMuZWFjaChlLmNvbXBsZXRlLFshMV0pO2E9Zi5leHRlbmQoe30sYSk7cmV0",
				"dXJuIGUucXVldWU9PT0hMT90aGlzLmVhY2goZyk6dGhpcy5xdWV1ZShlLnF1ZXVlLGcpfSxzdG9wOmZ1",
				"bmN0aW9uKGEsYyxkKXt0eXBlb2YgYSE9InN0cmluZyImJihkPWMsYz1hLGE9YiksYyYmYSE9PSExJiZ0",
				"aGlzLnF1ZXVlKGF8fCJmeCIsW10pO3JldHVybiB0aGlzLmVhY2goZnVuY3Rpb24oKXtmdW5jdGlvbiBo",
				"KGEsYixjKXt2YXIgZT1iW2NdO2YucmVtb3ZlRGF0YShhLGMsITApLGUuc3RvcChkKX12YXIgYixjPSEx",
				"LGU9Zi50aW1lcnMsZz1mLl9kYXRhKHRoaXMpO2R8fGYuX3VubWFyayghMCx0aGlzKTtpZihhPT1udWxs",
				"KWZvcihiIGluIGcpZ1tiXSYmZ1tiXS5zdG9wJiZiLmluZGV4T2YoIi5ydW4iKT09PWIubGVuZ3RoLTQm",
				"JmgodGhpcyxnLGIpO2Vsc2UgZ1tiPWErIi5ydW4iXSYmZ1tiXS5zdG9wJiZoKHRoaXMsZyxiKTtmb3Io",
				"Yj1lLmxlbmd0aDtiLS07KWVbYl0uZWxlbT09PXRoaXMmJihhPT1udWxsfHxlW2JdLnF1ZXVlPT09YSkm",
				"JihkP2VbYl0oITApOmVbYl0uc2F2ZVN0YXRlKCksYz0hMCxlLnNwbGljZShiLDEpKTsoIWR8fCFjKSYm",
				"Zi5kZXF1ZXVlKHRoaXMsYSl9KX19KSxmLmVhY2goe3NsaWRlRG93bjpjdCgic2hvdyIsMSksc2xpZGVV",
				"cDpjdCgiaGlkZSIsMSksc2xpZGVUb2dnbGU6Y3QoInRvZ2dsZSIsMSksZmFkZUluOntvcGFjaXR5OiJz",
				"aG93In0sZmFkZU91dDp7b3BhY2l0eToiaGlkZSJ9LGZhZGVUb2dnbGU6e29wYWNpdHk6InRvZ2dsZSJ9",
				"fSxmdW5jdGlvbihhLGIpe2YuZm5bYV09ZnVuY3Rpb24oYSxjLGQpe3JldHVybiB0aGlzLmFuaW1hdGUo",
				"YixhLGMsZCl9fSksZi5leHRlbmQoe3NwZWVkOmZ1bmN0aW9uKGEsYixjKXt2YXIgZD1hJiZ0eXBlb2Yg",
				"YT09Im9iamVjdCI/Zi5leHRlbmQoe30sYSk6e2NvbXBsZXRlOmN8fCFjJiZifHxmLmlzRnVuY3Rpb24o",
				"YSkmJmEsZHVyYXRpb246YSxlYXNpbmc6YyYmYnx8YiYmIWYuaXNGdW5jdGlvbihiKSYmYn07ZC5kdXJh",
				"dGlvbj1mLmZ4Lm9mZj8wOnR5cGVvZiBkLmR1cmF0aW9uPT0ibnVtYmVyIj9kLmR1cmF0aW9uOmQuZHVy",
				"YXRpb24gaW4gZi5meC5zcGVlZHM/Zi5meC5zcGVlZHNbZC5kdXJhdGlvbl06Zi5meC5zcGVlZHMuX2Rl",
				"ZmF1bHQ7aWYoZC5xdWV1ZT09bnVsbHx8ZC5xdWV1ZT09PSEwKWQucXVldWU9ImZ4IjtkLm9sZD1kLmNv",
				"bXBsZXRlLGQuY29tcGxldGU9ZnVuY3Rpb24oYSl7Zi5pc0Z1bmN0aW9uKGQub2xkKSYmZC5vbGQuY2Fs",
				"bCh0aGlzKSxkLnF1ZXVlP2YuZGVxdWV1ZSh0aGlzLGQucXVldWUpOmEhPT0hMSYmZi5fdW5tYXJrKHRo",
				"aXMpfTtyZXR1cm4gZH0sZWFzaW5nOntsaW5lYXI6ZnVuY3Rpb24oYSl7cmV0dXJuIGF9LHN3aW5nOmZ1",
				"bmN0aW9uKGEpe3JldHVybi1NYXRoLmNvcyhhKk1hdGguUEkpLzIrLjV9fSx0aW1lcnM6W10sZng6ZnVu",
				"Y3Rpb24oYSxiLGMpe3RoaXMub3B0aW9ucz1iLHRoaXMuZWxlbT1hLHRoaXMucHJvcD1jLGIub3JpZz1i",
				"Lm9yaWd8fHt9fX0pLGYuZngucHJvdG90eXBlPXt1cGRhdGU6ZnVuY3Rpb24oKXt0aGlzLm9wdGlvbnMu",
				"c3RlcCYmdGhpcy5vcHRpb25zLnN0ZXAuY2FsbCh0aGlzLmVsZW0sdGhpcy5ub3csdGhpcyksKGYuZngu",
				"c3RlcFt0aGlzLnByb3BdfHxmLmZ4LnN0ZXAuX2RlZmF1bHQpKHRoaXMpfSxjdXI6ZnVuY3Rpb24oKXtp",
				"Zih0aGlzLmVsZW1bdGhpcy5wcm9wXSE9bnVsbCYmKCF0aGlzLmVsZW0uc3R5bGV8fHRoaXMuZWxlbS5z",
				"dHlsZVt0aGlzLnByb3BdPT1udWxsKSlyZXR1cm4gdGhpcy5lbGVtW3RoaXMucHJvcF07dmFyIGEsYj1m",
				"LmNzcyh0aGlzLmVsZW0sdGhpcy5wcm9wKTtyZXR1cm4gaXNOYU4oYT1wYXJzZUZsb2F0KGIpKT8hYnx8",
				"Yj09PSJhdXRvIj8wOmI6YX0sY3VzdG9tOmZ1bmN0aW9uKGEsYyxkKXtmdW5jdGlvbiBoKGEpe3JldHVy",
				"biBlLnN0ZXAoYSl9dmFyIGU9dGhpcyxnPWYuZng7dGhpcy5zdGFydFRpbWU9Y3F8fGNyKCksdGhpcy5l",
				"bmQ9Yyx0aGlzLm5vdz10aGlzLnN0YXJ0PWEsdGhpcy5wb3M9dGhpcy5zdGF0ZT0wLHRoaXMudW5pdD1k",
				"fHx0aGlzLnVuaXR8fChmLmNzc051bWJlclt0aGlzLnByb3BdPyIiOiJweCIpLGgucXVldWU9dGhpcy5v",
				"cHRpb25zLnF1ZXVlLGguZWxlbT10aGlzLmVsZW0saC5zYXZlU3RhdGU9ZnVuY3Rpb24oKXtmLl9kYXRh",
				"KGUuZWxlbSwiZnhzaG93IitlLnByb3ApPT09YiYmKGUub3B0aW9ucy5oaWRlP2YuX2RhdGEoZS5lbGVt",
				"LCJmeHNob3ciK2UucHJvcCxlLnN0YXJ0KTplLm9wdGlvbnMuc2hvdyYmZi5fZGF0YShlLmVsZW0sImZ4",
				"c2hvdyIrZS5wcm9wLGUuZW5kKSl9LGgoKSYmZi50aW1lcnMucHVzaChoKSYmIWNvJiYoY289c2V0SW50",
				"ZXJ2YWwoZy50aWNrLGcuaW50ZXJ2YWwpKX0sc2hvdzpmdW5jdGlvbigpe3ZhciBhPWYuX2RhdGEodGhp",
				"cy5lbGVtLCJmeHNob3ciK3RoaXMucHJvcCk7dGhpcy5vcHRpb25zLm9yaWdbdGhpcy5wcm9wXT1hfHxm",
				"LnN0eWxlKHRoaXMuZWxlbSx0aGlzLnByb3ApLHRoaXMub3B0aW9ucy5zaG93PSEwLGEhPT1iP3RoaXMu",
				"Y3VzdG9tKHRoaXMuY3VyKCksYSk6dGhpcy5jdXN0b20odGhpcy5wcm9wPT09IndpZHRoInx8dGhpcy5w",
				"cm9wPT09ImhlaWdodCI/MTowLHRoaXMuY3VyKCkpLGYodGhpcy5lbGVtKS5zaG93KCl9LGhpZGU6ZnVu",
				"Y3Rpb24oKXt0aGlzLm9wdGlvbnMub3JpZ1t0aGlzLnByb3BdPWYuX2RhdGEodGhpcy5lbGVtLCJmeHNo",
				"b3ciK3RoaXMucHJvcCl8fGYuc3R5bGUodGhpcy5lbGVtLHRoaXMucHJvcCksdGhpcy5vcHRpb25zLmhp",
				"ZGU9ITAsdGhpcy5jdXN0b20odGhpcy5jdXIoKSwwKX0sc3RlcDpmdW5jdGlvbihhKXt2YXIgYixjLGQs",
				"ZT1jcXx8Y3IoKSxnPSEwLGg9dGhpcy5lbGVtLGk9dGhpcy5vcHRpb25zO2lmKGF8fGU+PWkuZHVyYXRp",
				"b24rdGhpcy5zdGFydFRpbWUpe3RoaXMubm93PXRoaXMuZW5kLHRoaXMucG9zPXRoaXMuc3RhdGU9MSx0",
				"aGlzLnVwZGF0ZSgpLGkuYW5pbWF0ZWRQcm9wZXJ0aWVzW3RoaXMucHJvcF09ITA7Zm9yKGIgaW4gaS5h",
				"bmltYXRlZFByb3BlcnRpZXMpaS5hbmltYXRlZFByb3BlcnRpZXNbYl0hPT0hMCYmKGc9ITEpO2lmKGcp",
				"e2kub3ZlcmZsb3chPW51bGwmJiFmLnN1cHBvcnQuc2hyaW5rV3JhcEJsb2NrcyYmZi5lYWNoKFsiIiwi",
				"WCIsIlkiXSxmdW5jdGlvbihhLGIpe2guc3R5bGVbIm92ZXJmbG93IitiXT1pLm92ZXJmbG93W2FdfSks",
				"aS5oaWRlJiZmKGgpLmhpZGUoKTtpZihpLmhpZGV8fGkuc2hvdylmb3IoYiBpbiBpLmFuaW1hdGVkUHJv",
				"cGVydGllcylmLnN0eWxlKGgsYixpLm9yaWdbYl0pLGYucmVtb3ZlRGF0YShoLCJmeHNob3ciK2IsITAp",
				"LGYucmVtb3ZlRGF0YShoLCJ0b2dnbGUiK2IsITApO2Q9aS5jb21wbGV0ZSxkJiYoaS5jb21wbGV0ZT0h",
				"MSxkLmNhbGwoaCkpfXJldHVybiExfWkuZHVyYXRpb249PUluZmluaXR5P3RoaXMubm93PWU6KGM9ZS10",
				"aGlzLnN0YXJ0VGltZSx0aGlzLnN0YXRlPWMvaS5kdXJhdGlvbix0aGlzLnBvcz1mLmVhc2luZ1tpLmFu",
				"aW1hdGVkUHJvcGVydGllc1t0aGlzLnByb3BdXSh0aGlzLnN0YXRlLGMsMCwxLGkuZHVyYXRpb24pLHRo",
				"aXMubm93PXRoaXMuc3RhcnQrKHRoaXMuZW5kLXRoaXMuc3RhcnQpKnRoaXMucG9zKSx0aGlzLnVwZGF0",
				"ZSgpO3JldHVybiEwfX0sZi5leHRlbmQoZi5meCx7dGljazpmdW5jdGlvbigpe3ZhciBhLGI9Zi50aW1l",
				"cnMsYz0wO2Zvcig7YzxiLmxlbmd0aDtjKyspYT1iW2NdLCFhKCkmJmJbY109PT1hJiZiLnNwbGljZShj",
				"LS0sMSk7Yi5sZW5ndGh8fGYuZnguc3RvcCgpfSxpbnRlcnZhbDoxMyxzdG9wOmZ1bmN0aW9uKCl7Y2xl",
				"YXJJbnRlcnZhbChjbyksY289bnVsbH0sc3BlZWRzOntzbG93OjYwMCxmYXN0OjIwMCxfZGVmYXVsdDo0",
				"MDB9LHN0ZXA6e29wYWNpdHk6ZnVuY3Rpb24oYSl7Zi5zdHlsZShhLmVsZW0sIm9wYWNpdHkiLGEubm93",
				"KX0sX2RlZmF1bHQ6ZnVuY3Rpb24oYSl7YS5lbGVtLnN0eWxlJiZhLmVsZW0uc3R5bGVbYS5wcm9wXSE9",
				"bnVsbD9hLmVsZW0uc3R5bGVbYS5wcm9wXT1hLm5vdythLnVuaXQ6YS5lbGVtW2EucHJvcF09YS5ub3d9",
				"fX0pLGYuZWFjaChjcC5jb25jYXQuYXBwbHkoW10sY3ApLGZ1bmN0aW9uKGEsYil7Yi5pbmRleE9mKCJt",
				"YXJnaW4iKSYmKGYuZnguc3RlcFtiXT1mdW5jdGlvbihhKXtmLnN0eWxlKGEuZWxlbSxiLE1hdGgubWF4",
				"KDAsYS5ub3cpK2EudW5pdCl9KX0pLGYuZXhwciYmZi5leHByLmZpbHRlcnMmJihmLmV4cHIuZmlsdGVy",
				"cy5hbmltYXRlZD1mdW5jdGlvbihhKXtyZXR1cm4gZi5ncmVwKGYudGltZXJzLGZ1bmN0aW9uKGIpe3Jl",
				"dHVybiBhPT09Yi5lbGVtfSkubGVuZ3RofSk7dmFyIGN2LGN3PS9edCg/OmFibGV8ZHxoKSQvaSxjeD0v",
				"Xig/OmJvZHl8aHRtbCkkL2k7ImdldEJvdW5kaW5nQ2xpZW50UmVjdCJpbiBjLmRvY3VtZW50RWxlbWVu",
				"dD9jdj1mdW5jdGlvbihhLGIsYyxkKXt0cnl7ZD1hLmdldEJvdW5kaW5nQ2xpZW50UmVjdCgpfWNhdGNo",
				"KGUpe31pZighZHx8IWYuY29udGFpbnMoYyxhKSlyZXR1cm4gZD97dG9wOmQudG9wLGxlZnQ6ZC5sZWZ0",
				"fTp7dG9wOjAsbGVmdDowfTt2YXIgZz1iLmJvZHksaD1jeShiKSxpPWMuY2xpZW50VG9wfHxnLmNsaWVu",
				"dFRvcHx8MCxqPWMuY2xpZW50TGVmdHx8Zy5jbGllbnRMZWZ0fHwwLGs9aC5wYWdlWU9mZnNldHx8Zi5z",
				"dXBwb3J0LmJveE1vZGVsJiZjLnNjcm9sbFRvcHx8Zy5zY3JvbGxUb3AsbD1oLnBhZ2VYT2Zmc2V0fHxm",
				"LnN1cHBvcnQuYm94TW9kZWwmJmMuc2Nyb2xsTGVmdHx8Zy5zY3JvbGxMZWZ0LG09ZC50b3Aray1pLG49",
				"ZC5sZWZ0K2wtajtyZXR1cm57dG9wOm0sbGVmdDpufX06Y3Y9ZnVuY3Rpb24oYSxiLGMpe3ZhciBkLGU9",
				"YS5vZmZzZXRQYXJlbnQsZz1hLGg9Yi5ib2R5LGk9Yi5kZWZhdWx0VmlldyxqPWk/aS5nZXRDb21wdXRl",
				"ZFN0eWxlKGEsbnVsbCk6YS5jdXJyZW50U3R5bGUsaz1hLm9mZnNldFRvcCxsPWEub2Zmc2V0TGVmdDt3",
				"aGlsZSgoYT1hLnBhcmVudE5vZGUpJiZhIT09aCYmYSE9PWMpe2lmKGYuc3VwcG9ydC5maXhlZFBvc2l0",
				"aW9uJiZqLnBvc2l0aW9uPT09ImZpeGVkIilicmVhaztkPWk/aS5nZXRDb21wdXRlZFN0eWxlKGEsbnVs",
				"bCk6YS5jdXJyZW50U3R5bGUsay09YS5zY3JvbGxUb3AsbC09YS5zY3JvbGxMZWZ0LGE9PT1lJiYoays9",
				"YS5vZmZzZXRUb3AsbCs9YS5vZmZzZXRMZWZ0LGYuc3VwcG9ydC5kb2VzTm90QWRkQm9yZGVyJiYoIWYu",
				"c3VwcG9ydC5kb2VzQWRkQm9yZGVyRm9yVGFibGVBbmRDZWxsc3x8IWN3LnRlc3QoYS5ub2RlTmFtZSkp",
				"JiYoays9cGFyc2VGbG9hdChkLmJvcmRlclRvcFdpZHRoKXx8MCxsKz1wYXJzZUZsb2F0KGQuYm9yZGVy",
				"TGVmdFdpZHRoKXx8MCksZz1lLGU9YS5vZmZzZXRQYXJlbnQpLGYuc3VwcG9ydC5zdWJ0cmFjdHNCb3Jk",
				"ZXJGb3JPdmVyZmxvd05vdFZpc2libGUmJmQub3ZlcmZsb3chPT0idmlzaWJsZSImJihrKz1wYXJzZUZs",
				"b2F0KGQuYm9yZGVyVG9wV2lkdGgpfHwwLGwrPXBhcnNlRmxvYXQoZC5ib3JkZXJMZWZ0V2lkdGgpfHww",
				"KSxqPWR9aWYoai5wb3NpdGlvbj09PSJyZWxhdGl2ZSJ8fGoucG9zaXRpb249PT0ic3RhdGljIilrKz1o",
				"Lm9mZnNldFRvcCxsKz1oLm9mZnNldExlZnQ7Zi5zdXBwb3J0LmZpeGVkUG9zaXRpb24mJmoucG9zaXRp",
				"b249PT0iZml4ZWQiJiYoays9TWF0aC5tYXgoYy5zY3JvbGxUb3AsaC5zY3JvbGxUb3ApLGwrPU1hdGgu",
				"bWF4KGMuc2Nyb2xsTGVmdCxoLnNjcm9sbExlZnQpKTtyZXR1cm57dG9wOmssbGVmdDpsfX0sZi5mbi5v",
				"ZmZzZXQ9ZnVuY3Rpb24oYSl7aWYoYXJndW1lbnRzLmxlbmd0aClyZXR1cm4gYT09PWI/dGhpczp0aGlz",
				"LmVhY2goZnVuY3Rpb24oYil7Zi5vZmZzZXQuc2V0T2Zmc2V0KHRoaXMsYSxiKX0pO3ZhciBjPXRoaXNb",
				"MF0sZD1jJiZjLm93bmVyRG9jdW1lbnQ7aWYoIWQpcmV0dXJuIG51bGw7aWYoYz09PWQuYm9keSlyZXR1",
				"cm4gZi5vZmZzZXQuYm9keU9mZnNldChjKTtyZXR1cm4gY3YoYyxkLGQuZG9jdW1lbnRFbGVtZW50KX0s",
				"Zi5vZmZzZXQ9e2JvZHlPZmZzZXQ6ZnVuY3Rpb24oYSl7dmFyIGI9YS5vZmZzZXRUb3AsYz1hLm9mZnNl",
				"dExlZnQ7Zi5zdXBwb3J0LmRvZXNOb3RJbmNsdWRlTWFyZ2luSW5Cb2R5T2Zmc2V0JiYoYis9cGFyc2VG",
				"bG9hdChmLmNzcyhhLCJtYXJnaW5Ub3AiKSl8fDAsYys9cGFyc2VGbG9hdChmLmNzcyhhLCJtYXJnaW5M",
				"ZWZ0IikpfHwwKTtyZXR1cm57dG9wOmIsbGVmdDpjfX0sc2V0T2Zmc2V0OmZ1bmN0aW9uKGEsYixjKXt2",
				"YXIgZD1mLmNzcyhhLCJwb3NpdGlvbiIpO2Q9PT0ic3RhdGljIiYmKGEuc3R5bGUucG9zaXRpb249InJl",
				"bGF0aXZlIik7dmFyIGU9ZihhKSxnPWUub2Zmc2V0KCksaD1mLmNzcyhhLCJ0b3AiKSxpPWYuY3NzKGEs",
				"ImxlZnQiKSxqPShkPT09ImFic29sdXRlInx8ZD09PSJmaXhlZCIpJiZmLmluQXJyYXkoImF1dG8iLFto",
				"LGldKT4tMSxrPXt9LGw9e30sbSxuO2o/KGw9ZS5wb3NpdGlvbigpLG09bC50b3Asbj1sLmxlZnQpOiht",
				"PXBhcnNlRmxvYXQoaCl8fDAsbj1wYXJzZUZsb2F0KGkpfHwwKSxmLmlzRnVuY3Rpb24oYikmJihiPWIu",
				"Y2FsbChhLGMsZykpLGIudG9wIT1udWxsJiYoay50b3A9Yi50b3AtZy50b3ArbSksYi5sZWZ0IT1udWxs",
				"JiYoay5sZWZ0PWIubGVmdC1nLmxlZnQrbiksInVzaW5nImluIGI/Yi51c2luZy5jYWxsKGEsayk6ZS5j",
				"c3Moayl9fSxmLmZuLmV4dGVuZCh7cG9zaXRpb246ZnVuY3Rpb24oKXtpZighdGhpc1swXSlyZXR1cm4g",
				"bnVsbDt2YXIgYT10aGlzWzBdLGI9dGhpcy5vZmZzZXRQYXJlbnQoKSxjPXRoaXMub2Zmc2V0KCksZD1j",
				"eC50ZXN0KGJbMF0ubm9kZU5hbWUpP3t0b3A6MCxsZWZ0OjB9OmIub2Zmc2V0KCk7Yy50b3AtPXBhcnNl",
				"RmxvYXQoZi5jc3MoYSwibWFyZ2luVG9wIikpfHwwLGMubGVmdC09cGFyc2VGbG9hdChmLmNzcyhhLCJt",
				"YXJnaW5MZWZ0IikpfHwwLGQudG9wKz1wYXJzZUZsb2F0KGYuY3NzKGJbMF0sImJvcmRlclRvcFdpZHRo",
				"IikpfHwwLGQubGVmdCs9cGFyc2VGbG9hdChmLmNzcyhiWzBdLCJib3JkZXJMZWZ0V2lkdGgiKSl8fDA7",
				"cmV0dXJue3RvcDpjLnRvcC1kLnRvcCxsZWZ0OmMubGVmdC1kLmxlZnR9fSxvZmZzZXRQYXJlbnQ6ZnVu",
				"Y3Rpb24oKXtyZXR1cm4gdGhpcy5tYXAoZnVuY3Rpb24oKXt2YXIgYT10aGlzLm9mZnNldFBhcmVudHx8",
				"Yy5ib2R5O3doaWxlKGEmJiFjeC50ZXN0KGEubm9kZU5hbWUpJiZmLmNzcyhhLCJwb3NpdGlvbiIpPT09",
				"InN0YXRpYyIpYT1hLm9mZnNldFBhcmVudDtyZXR1cm4gYX0pfX0pLGYuZWFjaCh7c2Nyb2xsTGVmdDoi",
				"cGFnZVhPZmZzZXQiLHNjcm9sbFRvcDoicGFnZVlPZmZzZXQifSxmdW5jdGlvbihhLGMpe3ZhciBkPS9Z",
				"Ly50ZXN0KGMpO2YuZm5bYV09ZnVuY3Rpb24oZSl7cmV0dXJuIGYuYWNjZXNzKHRoaXMsZnVuY3Rpb24o",
				"YSxlLGcpe3ZhciBoPWN5KGEpO2lmKGc9PT1iKXJldHVybiBoP2MgaW4gaD9oW2NdOmYuc3VwcG9ydC5i",
				"b3hNb2RlbCYmaC5kb2N1bWVudC5kb2N1bWVudEVsZW1lbnRbZV18fGguZG9jdW1lbnQuYm9keVtlXTph",
				"W2VdO2g/aC5zY3JvbGxUbyhkP2YoaCkuc2Nyb2xsTGVmdCgpOmcsZD9nOmYoaCkuc2Nyb2xsVG9wKCkp",
				"OmFbZV09Z30sYSxlLGFyZ3VtZW50cy5sZW5ndGgsbnVsbCl9fSksZi5lYWNoKHtIZWlnaHQ6ImhlaWdo",
				"dCIsV2lkdGg6IndpZHRoIn0sZnVuY3Rpb24oYSxjKXt2YXIgZD0iY2xpZW50IithLGU9InNjcm9sbCIr",
				"YSxnPSJvZmZzZXQiK2E7Zi5mblsiaW5uZXIiK2FdPWZ1bmN0aW9uKCl7dmFyIGE9dGhpc1swXTtyZXR1",
				"cm4gYT9hLnN0eWxlP3BhcnNlRmxvYXQoZi5jc3MoYSxjLCJwYWRkaW5nIikpOnRoaXNbY10oKTpudWxs",
				"fSxmLmZuWyJvdXRlciIrYV09ZnVuY3Rpb24oYSl7dmFyIGI9dGhpc1swXTtyZXR1cm4gYj9iLnN0eWxl",
				"P3BhcnNlRmxvYXQoZi5jc3MoYixjLGE/Im1hcmdpbiI6ImJvcmRlciIpKTp0aGlzW2NdKCk6bnVsbH0s",
				"Zi5mbltjXT1mdW5jdGlvbihhKXtyZXR1cm4gZi5hY2Nlc3ModGhpcyxmdW5jdGlvbihhLGMsaCl7dmFy",
				"IGksaixrLGw7aWYoZi5pc1dpbmRvdyhhKSl7aT1hLmRvY3VtZW50LGo9aS5kb2N1bWVudEVsZW1lbnRb",
				"ZF07cmV0dXJuIGYuc3VwcG9ydC5ib3hNb2RlbCYmanx8aS5ib2R5JiZpLmJvZHlbZF18fGp9aWYoYS5u",
				"b2RlVHlwZT09PTkpe2k9YS5kb2N1bWVudEVsZW1lbnQ7aWYoaVtkXT49aVtlXSlyZXR1cm4gaVtkXTty",
				"ZXR1cm4gTWF0aC5tYXgoYS5ib2R5W2VdLGlbZV0sYS5ib2R5W2ddLGlbZ10pfWlmKGg9PT1iKXtrPWYu",
				"Y3NzKGEsYyksbD1wYXJzZUZsb2F0KGspO3JldHVybiBmLmlzTnVtZXJpYyhsKT9sOmt9ZihhKS5jc3Mo",
				"YyxoKX0sYyxhLGFyZ3VtZW50cy5sZW5ndGgsbnVsbCl9fSksYS5qUXVlcnk9YS4kPWYsdHlwZW9mIGRl",
				"ZmluZT09ImZ1bmN0aW9uIiYmZGVmaW5lLmFtZCYmZGVmaW5lLmFtZC5qUXVlcnkmJmRlZmluZSgianF1",
				"ZXJ5IixbXSxmdW5jdGlvbigpe3JldHVybiBmfSl9KSh3aW5kb3cpOw=="
				].join('');
				
				buff = new Buffer(jquery_base64_blob, 'base64').toString('ascii');
				fs.writeFileSync(save_file,buff);
				break;

			//
			// Install Dojo
			//
			case "dojo":
				save_file = 'static/js/dojo.1.7.2.js';

				var dojo_base64_blob = [
				"LyoKCUNvcHlyaWdodCAoYykgMjAwNC0yMDExLCBUaGUgRG9qbyBGb3VuZGF0aW9uIEFsbCBSaWdodHMg",
				"UmVzZXJ2ZWQuCglBdmFpbGFibGUgdmlhIEFjYWRlbWljIEZyZWUgTGljZW5zZSA+PSAyLjEgT1IgdGhl",
				"IG1vZGlmaWVkIEJTRCBsaWNlbnNlLgoJc2VlOiBodHRwOi8vZG9qb3Rvb2xraXQub3JnL2xpY2Vuc2Ug",
				"Zm9yIGRldGFpbHMKKi8KCi8qCglUaGlzIGlzIGFuIG9wdGltaXplZCB2ZXJzaW9uIG9mIERvam8sIGJ1",
				"aWx0IGZvciBkZXBsb3ltZW50IGFuZCBub3QgZm9yCglkZXZlbG9wbWVudC4gVG8gZ2V0IHNvdXJjZXMg",
				"YW5kIGRvY3VtZW50YXRpb24sIHBsZWFzZSB2aXNpdDoKCgkJaHR0cDovL2Rvam90b29sa2l0Lm9yZwoq",
				"LwoKLy8+PmJ1aWx0CihmdW5jdGlvbihfMSxfMil7dmFyIF8zPWZ1bmN0aW9uKCl7fSxfND1mdW5jdGlv",
				"bihpdCl7Zm9yKHZhciBwIGluIGl0KXtyZXR1cm4gMDt9cmV0dXJuIDE7fSxfNT17fS50b1N0cmluZyxf",
				"Nj1mdW5jdGlvbihpdCl7cmV0dXJuIF81LmNhbGwoaXQpPT0iW29iamVjdCBGdW5jdGlvbl0iO30sXzc9",
				"ZnVuY3Rpb24oaXQpe3JldHVybiBfNS5jYWxsKGl0KT09IltvYmplY3QgU3RyaW5nXSI7fSxfOD1mdW5j",
				"dGlvbihpdCl7cmV0dXJuIF81LmNhbGwoaXQpPT0iW29iamVjdCBBcnJheV0iO30sXzk9ZnVuY3Rpb24o",
				"X2EsX2Ipe2lmKF9hKXtmb3IodmFyIGk9MDtpPF9hLmxlbmd0aDspe19iKF9hW2krK10pO319fSxfYz1m",
				"dW5jdGlvbihfZCxfZSl7Zm9yKHZhciBwIGluIF9lKXtfZFtwXT1fZVtwXTt9cmV0dXJuIF9kO30sX2Y9",
				"ZnVuY3Rpb24oXzEwLF8xMSl7cmV0dXJuIF9jKG5ldyBFcnJvcihfMTApLHtzcmM6ImRvam9Mb2FkZXIi",
				"LGluZm86XzExfSk7fSxfMTI9MSx1aWQ9ZnVuY3Rpb24oKXtyZXR1cm4gIl8iK18xMisrO30scmVxPWZ1",
				"bmN0aW9uKF8xMyxfMTQsXzE1KXtyZXR1cm4gXzE2KF8xMyxfMTQsXzE1LDAscmVxKTt9LF8xNz10aGlz",
				"LGRvYz1fMTcuZG9jdW1lbnQsXzE4PWRvYyYmZG9jLmNyZWF0ZUVsZW1lbnQoIkRpViIpLGhhcz1yZXEu",
				"aGFzPWZ1bmN0aW9uKF8xOSl7cmV0dXJuIF82KF8xYVtfMTldKT8oXzFhW18xOV09XzFhW18xOV0oXzE3",
				"LGRvYyxfMTgpKTpfMWFbXzE5XTt9LF8xYT1oYXMuY2FjaGU9XzIuaGFzQ2FjaGU7aGFzLmFkZD1mdW5j",
				"dGlvbihfMWIsXzFjLG5vdyxfMWQpeyhfMWFbXzFiXT09PXVuZGVmaW5lZHx8XzFkKSYmKF8xYVtfMWJd",
				"PV8xYyk7cmV0dXJuIG5vdyYmaGFzKF8xYik7fTtmYWxzZSYmaGFzLmFkZCgiaG9zdC1ub2RlIix0eXBl",
				"b2YgcHJvY2Vzcz09Im9iamVjdCImJi9ub2RlKFwuZXhlKT8kLy50ZXN0KHByb2Nlc3MuZXhlY1BhdGgp",
				"KTtpZigwKXtyZXF1aXJlKCIuL19iYXNlL2NvbmZpZ05vZGUuanMiKS5jb25maWcoXzIpO18yLmxvYWRl",
				"clBhdGNoLm5vZGVSZXF1aXJlPXJlcXVpcmU7fWZhbHNlJiZoYXMuYWRkKCJob3N0LXJoaW5vIix0eXBl",
				"b2YgbG9hZD09ImZ1bmN0aW9uIiYmKHR5cGVvZiBQYWNrYWdlcz09ImZ1bmN0aW9uInx8dHlwZW9mIFBh",
				"Y2thZ2VzPT0ib2JqZWN0IikpO2lmKDApe2Zvcih2YXIgXzFlPV8xLmJhc2VVcmx8fCIuIixhcmcsXzFm",
				"PXRoaXMuYXJndW1lbnRzLGk9MDtpPF8xZi5sZW5ndGg7KXthcmc9KF8xZltpKytdKyIiKS5zcGxpdCgi",
				"PSIpO2lmKGFyZ1swXT09ImJhc2VVcmwiKXtfMWU9YXJnWzFdO2JyZWFrO319bG9hZChfMWUrIi9fYmFz",
				"ZS9jb25maWdSaGluby5qcyIpO3JoaW5vRG9qb0NvbmZpZyhfMixfMWUsXzFmKTt9Zm9yKHZhciBwIGlu",
				"IF8xLmhhcyl7aGFzLmFkZChwLF8xLmhhc1twXSwwLDEpO312YXIgXzIwPTEsXzIxPTIsXzIyPTMsXzIz",
				"PTQsXzI0PTU7aWYoMCl7XzIwPSJyZXF1ZXN0ZWQiO18yMT0iYXJyaXZlZCI7XzIyPSJub3QtYS1tb2R1",
				"bGUiO18yMz0iZXhlY3V0aW5nIjtfMjQ9ImV4ZWN1dGVkIjt9dmFyIF8yNT0wLF8yNj0ic3luYyIseGQ9",
				"InhkIixfMjc9W10sXzI4PTAsXzI5PV8zLF8yYT1fMyxfMmI7aWYoMSl7cmVxLmlzWGRVcmw9XzM7cmVx",
				"LmluaXRTeW5jTG9hZGVyPWZ1bmN0aW9uKF8yYyxfMmQsXzJlKXtpZighXzI4KXtfMjg9XzJjO18yOT1f",
				"MmQ7XzJhPV8yZTt9cmV0dXJuIHtzeW5jOl8yNix4ZDp4ZCxhcnJpdmVkOl8yMSxub25tb2R1bGU6XzIy",
				"LGV4ZWN1dGluZzpfMjMsZXhlY3V0ZWQ6XzI0LHN5bmNFeGVjU3RhY2s6XzI3LG1vZHVsZXM6XzJmLGV4",
				"ZWNROl8zMCxnZXRNb2R1bGU6XzMxLGluamVjdE1vZHVsZTpfMzIsc2V0QXJyaXZlZDpfMzMsc2lnbmFs",
				"Ol8zNCxmaW5pc2hFeGVjOl8zNSxleGVjTW9kdWxlOl8zNixkb2pvUmVxdWlyZVBsdWdpbjpfMjgsZ2V0",
				"TGVnYWN5TW9kZTpmdW5jdGlvbigpe3JldHVybiBfMjU7fSxob2xkSWRsZTpmdW5jdGlvbigpe183NCsr",
				"O30scmVsZWFzZUlkbGU6ZnVuY3Rpb24oKXtfMzcoKTt9fTt9O2lmKDEpe3ZhciBfMzg9bG9jYXRpb24u",
				"cHJvdG9jb2wsXzM5PWxvY2F0aW9uLmhvc3QsXzNhPSFfMzk7cmVxLmlzWGRVcmw9ZnVuY3Rpb24odXJs",
				"KXtpZihfM2F8fC9eXC4vLnRlc3QodXJsKSl7cmV0dXJuIGZhbHNlO31pZigvXlwvXC8vLnRlc3QodXJs",
				"KSl7cmV0dXJuIHRydWU7fXZhciBfM2I9dXJsLm1hdGNoKC9eKFteXC9cOl0rXDopXC9cLyhbXlwvXSsp",
				"Lyk7cmV0dXJuIF8zYiYmKF8zYlsxXSE9XzM4fHxfM2JbMl0hPV8zOSk7fTt0cnVlfHxoYXMuYWRkKCJk",
				"b2pvLXhoci1mYWN0b3J5IiwxKTtoYXMuYWRkKCJkb2pvLWZvcmNlLWFjdGl2ZXgteGhyIiwxJiYhZG9j",
				"LmFkZEV2ZW50TGlzdGVuZXImJndpbmRvdy5sb2NhdGlvbi5wcm90b2NvbD09ImZpbGU6Iik7aGFzLmFk",
				"ZCgibmF0aXZlLXhociIsdHlwZW9mIFhNTEh0dHBSZXF1ZXN0IT0idW5kZWZpbmVkIik7aWYoaGFzKCJu",
				"YXRpdmUteGhyIikmJiFoYXMoImRvam8tZm9yY2UtYWN0aXZleC14aHIiKSl7XzJiPWZ1bmN0aW9uKCl7",
				"cmV0dXJuIG5ldyBYTUxIdHRwUmVxdWVzdCgpO307fWVsc2V7Zm9yKHZhciBfM2M9WyJNc3htbDIuWE1M",
				"SFRUUCIsIk1pY3Jvc29mdC5YTUxIVFRQIiwiTXN4bWwyLlhNTEhUVFAuNC4wIl0sXzNkLGk9MDtpPDM7",
				"KXt0cnl7XzNkPV8zY1tpKytdO2lmKG5ldyBBY3RpdmVYT2JqZWN0KF8zZCkpe2JyZWFrO319Y2F0Y2go",
				"ZSl7fX1fMmI9ZnVuY3Rpb24oKXtyZXR1cm4gbmV3IEFjdGl2ZVhPYmplY3QoXzNkKTt9O31yZXEuZ2V0",
				"WGhyPV8yYjtoYXMuYWRkKCJkb2pvLWdldHRleHQtYXBpIiwxKTtyZXEuZ2V0VGV4dD1mdW5jdGlvbih1",
				"cmwsXzNlLF8zZil7dmFyIHhocj1fMmIoKTt4aHIub3BlbigiR0VUIixfNDAodXJsKSxmYWxzZSk7eGhy",
				"LnNlbmQobnVsbCk7aWYoeGhyLnN0YXR1cz09MjAwfHwoIWxvY2F0aW9uLmhvc3QmJiF4aHIuc3RhdHVz",
				"KSl7aWYoXzNmKXtfM2YoeGhyLnJlc3BvbnNlVGV4dCxfM2UpO319ZWxzZXt0aHJvdyBfZigieGhyRmFp",
				"bGVkIix4aHIuc3RhdHVzKTt9cmV0dXJuIHhoci5yZXNwb25zZVRleHQ7fTt9fWVsc2V7cmVxLmFzeW5j",
				"PTE7fXZhciBfNDE9bmV3IEZ1bmN0aW9uKCJfX3RleHQiLCJyZXR1cm4gZXZhbChfX3RleHQpOyIpO3Jl",
				"cS5ldmFsPWZ1bmN0aW9uKF80MixfNDMpe3JldHVybiBfNDEoXzQyKyJcclxuLy8vL0Agc291cmNlVVJM",
				"PSIrXzQzKTt9O3ZhciBfNDQ9e30sXzQ1PSJlcnJvciIsXzM0PXJlcS5zaWduYWw9ZnVuY3Rpb24oXzQ2",
				"LF80Nyl7dmFyIF80OD1fNDRbXzQ2XTtfOShfNDgmJl80OC5zbGljZSgwKSxmdW5jdGlvbihfNDkpe180",
				"OS5hcHBseShudWxsLF84KF80Nyk/XzQ3OltfNDddKTt9KTt9LG9uPXJlcS5vbj1mdW5jdGlvbihfNGEs",
				"XzRiKXt2YXIgXzRjPV80NFtfNGFdfHwoXzQ0W180YV09W10pO180Yy5wdXNoKF80Yik7cmV0dXJuIHty",
				"ZW1vdmU6ZnVuY3Rpb24oKXtmb3IodmFyIGk9MDtpPF80Yy5sZW5ndGg7aSsrKXtpZihfNGNbaV09PT1f",
				"NGIpe180Yy5zcGxpY2UoaSwxKTtyZXR1cm47fX19fTt9O3ZhciBfNGQ9W10sXzRlPXt9LF80Zj1bXSxf",
				"NTA9e30sXzUxPXt9LF81Mj1bXSxfMmY9e30sXzUzPSIiLF81ND17fSxfNTU9e30sXzU2PXt9O2lmKDEp",
				"e3ZhciBfNTc9ZnVuY3Rpb24oXzU4KXtmb3IodmFyIHAgaW4gXzU1KXt2YXIgXzU5PXAubWF0Y2goL151",
				"cmxcOiguKykvKTtpZihfNTkpe181NFtfNWEoXzU5WzFdLF81OCldPV81NVtwXTt9ZWxzZXtpZihwIT0i",
				"Km5vcmVmIil7XzU0W181YihwLF81OCkubWlkXT1fNTVbcF07fX19XzU1PXt9O30sXzVjPWZ1bmN0aW9u",
				"KG1hcCxfNWQsXzVlKXtfNWQuc3BsaWNlKDAsXzVkLmxlbmd0aCk7dmFyIHAsaSxfNWYsXzYwPTA7Zm9y",
				"KHAgaW4gbWFwKXtfNWQucHVzaChbcCxtYXBbcF1dKTtpZihtYXBbcF09PV81ZSl7XzYwPXA7fX1fNWQu",
				"c29ydChmdW5jdGlvbihsaHMscmhzKXtyZXR1cm4gcmhzWzBdLmxlbmd0aC1saHNbMF0ubGVuZ3RoO30p",
				"O2ZvcihpPTA7aTxfNWQubGVuZ3RoOyl7XzVmPV81ZFtpKytdO181ZlsyXT1uZXcgUmVnRXhwKCJeIitf",
				"NWZbMF0ucmVwbGFjZSgvKFtcLiQ/Knx7fVwoXClcW1xdXFxcL1wrXl0pL2csZnVuY3Rpb24oYyl7cmV0",
				"dXJuICJcXCIrYzt9KSsiKC98JCkiKTtfNWZbM109XzVmWzBdLmxlbmd0aCsxO31yZXR1cm4gXzYwO30s",
				"XzYxPWZ1bmN0aW9uKF82MixfNjMpe3ZhciBfNjQ9XzYyLm5hbWU7aWYoIV82NCl7XzY0PV82MjtfNjI9",
				"e25hbWU6XzY0fTt9XzYyPV9jKHttYWluOiJtYWluIixtYXBQcm9nOltdfSxfNjIpO182Mi5sb2NhdGlv",
				"bj0oXzYzfHwiIikrKF82Mi5sb2NhdGlvbj9fNjIubG9jYXRpb246XzY0KTtfNjIucmV2ZXJzZU5hbWU9",
				"XzVjKF82Mi5wYWNrYWdlTWFwLF82Mi5tYXBQcm9nLF82NCk7aWYoIV82Mi5tYWluLmluZGV4T2YoIi4v",
				"Iikpe182Mi5tYWluPV82Mi5tYWluLnN1YnN0cmluZygyKTt9X2MoXzRlLF82Mi5wYXRocyk7XzUwW182",
				"NF09XzYyO181MVtfNjRdPV82NDt9LF82NT1mdW5jdGlvbihfNjYsXzY3KXtmb3IodmFyIHAgaW4gXzY2",
				"KXtpZihwPT0id2FpdFNlY29uZHMiKXtyZXEud2FpdG1zPShfNjZbcF18fDApKjEwMDA7fWlmKHA9PSJj",
				"YWNoZUJ1c3QiKXtfNTM9XzY2W3BdPyhfNyhfNjZbcF0pP182NltwXToobmV3IERhdGUoKSkuZ2V0VGlt",
				"ZSgpKyIiKToiIjt9aWYocD09ImJhc2VVcmwifHxwPT0iY29tYm8iKXtyZXFbcF09XzY2W3BdO31pZigx",
				"JiZwPT0iYXN5bmMiKXt2YXIgXzY4PV82NltwXTtyZXEubGVnYWN5TW9kZT1fMjU9KF83KF82OCkmJi9z",
				"eW5jfGxlZ2FjeUFzeW5jLy50ZXN0KF82OCk/XzY4OighXzY4PyJzeW5jIjpmYWxzZSkpO3JlcS5hc3lu",
				"Yz0hXzI1O31pZihfNjZbcF0hPT1fMWEpe3JlcS5yYXdDb25maWdbcF09XzY2W3BdO3AhPSJoYXMiJiZo",
				"YXMuYWRkKCJjb25maWctIitwLF82NltwXSwwLF82Nyk7fX1pZighcmVxLmJhc2VVcmwpe3JlcS5iYXNl",
				"VXJsPSIuLyI7fWlmKCEvXC8kLy50ZXN0KHJlcS5iYXNlVXJsKSl7cmVxLmJhc2VVcmwrPSIvIjt9Zm9y",
				"KHAgaW4gXzY2Lmhhcyl7aGFzLmFkZChwLF82Ni5oYXNbcF0sMCxfNjcpO31fOShfNjYucGFja2FnZXMs",
				"XzYxKTtmb3IoXzFlIGluIF82Ni5wYWNrYWdlUGF0aHMpe185KF82Ni5wYWNrYWdlUGF0aHNbXzFlXSxm",
				"dW5jdGlvbihfNjkpe182MShfNjksXzFlKyIvIik7fSk7fV81YyhfYyhfNGUsXzY2LnBhdGhzKSxfNGYp",
				"O185KF82Ni5hbGlhc2VzLGZ1bmN0aW9uKF82YSl7aWYoXzcoXzZhWzBdKSl7XzZhWzBdPW5ldyBSZWdF",
				"eHAoIl4iK182YVswXS5yZXBsYWNlKC8oW1wuJD8qfHt9XChcKVxbXF1cXFwvXCteXSkvZyxmdW5jdGlv",
				"bihjKXtyZXR1cm4gIlxcIitjO30pKyIkIik7fV80ZC5wdXNoKF82YSk7fSk7XzVjKF9jKF81MSxfNjYu",
				"cGFja2FnZU1hcCksXzUyKTtpZihfNjYuY2FjaGUpe181NygpO181NT1fNjYuY2FjaGU7aWYoXzY2LmNh",
				"Y2hlWyIqbm9yZWYiXSl7XzU3KCk7fX1fMzQoImNvbmZpZyIsW182NixyZXEucmF3Q29uZmlnXSk7fTtp",
				"ZihoYXMoImRvam8tY2RuIil8fDEpe2Zvcih2YXIgXzZiLHNyYyxfNmMsXzZkPWRvYy5nZXRFbGVtZW50",
				"c0J5VGFnTmFtZSgic2NyaXB0IiksaT0wO2k8XzZkLmxlbmd0aCYmIV82YztpKyspe2lmKChzcmM9XzZk",
				"W2ldLmdldEF0dHJpYnV0ZSgic3JjIikpJiYoXzZjPXNyYy5tYXRjaCgvKC4qKVwvP2Rvam9cLmpzKFxX",
				"fCQpL2kpKSl7XzEuYmFzZVVybD1fNmI9XzEuYmFzZVVybHx8XzIuYmFzZVVybHx8XzZjWzFdO3NyYz0o",
				"XzZkW2ldLmdldEF0dHJpYnV0ZSgiZGF0YS1kb2pvLWNvbmZpZyIpfHxfNmRbaV0uZ2V0QXR0cmlidXRl",
				"KCJkakNvbmZpZyIpKTtpZihzcmMpe181Nj1yZXEuZXZhbCgiKHsgIitzcmMrIiB9KSIsImRhdGEtZG9q",
				"by1jb25maWciKTt9aWYoMCl7dmFyIF82ZT1fNmRbaV0uZ2V0QXR0cmlidXRlKCJkYXRhLW1haW4iKTtp",
				"ZihfNmUpe181Ni5kZXBzPV81Ni5kZXBzfHxbXzZlXTt9fX19fWlmKDApe3RyeXtpZih3aW5kb3cucGFy",
				"ZW50IT13aW5kb3cmJndpbmRvdy5wYXJlbnQucmVxdWlyZSl7dmFyIGRvaD13aW5kb3cucGFyZW50LnJl",
				"cXVpcmUoImRvaCIpO2RvaCYmX2MoXzU2LGRvaC50ZXN0Q29uZmlnKTt9fWNhdGNoKGUpe319cmVxLnJh",
				"d0NvbmZpZz17fTtfNjUoXzIsMSk7XzY1KF8xLDEpO182NShfNTYsMSk7aWYoaGFzKCJkb2pvLWNkbiIp",
				"KXtfNTAuZG9qby5sb2NhdGlvbj1fNmI7XzUwLmRpaml0LmxvY2F0aW9uPV82YisiLi4vZGlqaXQvIjtf",
				"NTAuZG9qb3gubG9jYXRpb249XzZiKyIuLi9kb2pveC8iO319ZWxzZXtfNGU9XzIucGF0aHM7XzRmPV8y",
				"LnBhdGhzTWFwUHJvZztfNTA9XzIucGFja3M7XzRkPV8yLmFsaWFzZXM7XzUxPV8yLnBhY2thZ2VNYXA7",
				"XzUyPV8yLnBhY2thZ2VNYXBQcm9nO18yZj1fMi5tb2R1bGVzO181ND1fMi5jYWNoZTtfNTM9XzIuY2Fj",
				"aGVCdXN0O3JlcS5yYXdDb25maWc9XzI7fWlmKDApe3JlcS5jb21ibz1yZXEuY29tYm98fHthZGQ6XzN9",
				"O3ZhciBfNmY9MCxfNzA9W10sXzcxPW51bGw7fXZhciBfNzI9ZnVuY3Rpb24oXzczKXtfNzQrKztfOShf",
				"NzMuZGVwcyxfMzIpO2lmKDAmJl82ZiYmIV83MSl7XzcxPXNldFRpbWVvdXQoZnVuY3Rpb24oKXtfNmY9",
				"MDtfNzE9bnVsbDtyZXEuY29tYm8uZG9uZShmdW5jdGlvbihfNzUsdXJsKXt2YXIgXzc2PWZ1bmN0aW9u",
				"KCl7Xzc3KDAsXzc1KTtfNzgoKTt9O183MC5wdXNoKF83NSk7Xzc5PV83NTtyZXEuaW5qZWN0VXJsKHVy",
				"bCxfNzYsXzc1KTtfNzk9MDt9LHJlcSk7fSwwKTt9XzM3KCk7fSxfMTY9ZnVuY3Rpb24oYTEsYTIsYTMs",
				"XzdhLF83Yil7dmFyIF83YyxfN2Q7aWYoXzcoYTEpKXtfN2M9XzMxKGExLF83YSx0cnVlKTtpZihfN2Mm",
				"Jl83Yy5leGVjdXRlZCl7cmV0dXJuIF83Yy5yZXN1bHQ7fXRocm93IF9mKCJ1bmRlZmluZWRNb2R1bGUi",
				"LGExKTt9aWYoIV84KGExKSl7XzY1KGExKTthMT1hMjthMj1hMzt9aWYoXzgoYTEpKXtpZighYTEubGVu",
				"Z3RoKXthMiYmYTIoKTt9ZWxzZXtfN2Q9InJlcXVpcmUqIit1aWQoKTtmb3IodmFyIG1pZCxfN2U9W10s",
				"aT0wO2k8YTEubGVuZ3RoOyl7bWlkPWExW2krK107aWYobWlkIGluIHtleHBvcnRzOjEsbW9kdWxlOjF9",
				"KXt0aHJvdyBfZigiaWxsZWdhbE1vZHVsZUlkIixtaWQpO31fN2UucHVzaChfMzEobWlkLF83YSkpO31f",
				"N2M9X2MoXzdmKCIiLF83ZCwwLCIiKSx7aW5qZWN0ZWQ6XzIxLGRlcHM6XzdlLGRlZjphMnx8XzMscmVx",
				"dWlyZTpfN2E/XzdhLnJlcXVpcmU6cmVxfSk7XzJmW183Yy5taWRdPV83YztfNzIoXzdjKTt2YXIgXzgw",
				"PV83NCYmcmVxLmFzeW5jO183NCsrO18zNihfN2MsXzgwKTtfMzcoKTtpZighXzdjLmV4ZWN1dGVkKXtf",
				"MzAucHVzaChfN2MpO31fNzgoKTt9fXJldHVybiBfN2I7fSxfODE9ZnVuY3Rpb24oXzgyKXtpZighXzgy",
				"KXtyZXR1cm4gcmVxO312YXIgXzgzPV84Mi5yZXF1aXJlO2lmKCFfODMpe184Mz1mdW5jdGlvbihhMSxh",
				"MixhMyl7cmV0dXJuIF8xNihhMSxhMixhMyxfODIsXzgzKTt9O184Mi5yZXF1aXJlPV9jKF84MyxyZXEp",
				"O184My5tb2R1bGU9XzgyO184My50b1VybD1mdW5jdGlvbihfODQpe3JldHVybiBfNWEoXzg0LF84Mik7",
				"fTtfODMudG9BYnNNaWQ9ZnVuY3Rpb24obWlkKXtyZXR1cm4gX2FmKG1pZCxfODIpO307aWYoMCl7Xzgz",
				"LnVuZGVmPWZ1bmN0aW9uKG1pZCl7cmVxLnVuZGVmKG1pZCxfODIpO307fX1yZXR1cm4gXzgzO30sXzMw",
				"PVtdLF84NT1bXSxfODY9e30sXzg3PWZ1bmN0aW9uKF84OCl7Xzg4LmluamVjdGVkPV8yMDtfODZbXzg4",
				"Lm1pZF09MTtpZihfODgudXJsKXtfODZbXzg4LnVybF09Xzg4LnBhY2t8fDE7fX0sXzMzPWZ1bmN0aW9u",
				"KF84OSl7Xzg5LmluamVjdGVkPV8yMTtkZWxldGUgXzg2W184OS5taWRdO2lmKF84OS51cmwpe2RlbGV0",
				"ZSBfODZbXzg5LnVybF07fWlmKF80KF84Nikpe184YSgpOzEmJl8yNT09eGQmJihfMjU9XzI2KTt9fSxf",
				"OGI9cmVxLmlkbGU9ZnVuY3Rpb24oKXtyZXR1cm4gIV84NS5sZW5ndGgmJl80KF84NikmJiFfMzAubGVu",
				"Z3RoJiYhXzc0O30sXzhjPWZ1bmN0aW9uKF84ZCxtYXApe2Zvcih2YXIgaT0wO2k8bWFwLmxlbmd0aDtp",
				"Kyspe2lmKG1hcFtpXVsyXS50ZXN0KF84ZCkpe3JldHVybiBtYXBbaV07fX1yZXR1cm4gMDt9LF84ZT1m",
				"dW5jdGlvbihfOGYpe3ZhciBfOTA9W10sXzkxLF85MjtfOGY9XzhmLnJlcGxhY2UoL1xcL2csIi8iKS5z",
				"cGxpdCgiLyIpO3doaWxlKF84Zi5sZW5ndGgpe185MT1fOGYuc2hpZnQoKTtpZihfOTE9PSIuLiImJl85",
				"MC5sZW5ndGgmJl85MiE9Ii4uIil7XzkwLnBvcCgpO185Mj1fOTBbXzkwLmxlbmd0aC0xXTt9ZWxzZXtp",
				"ZihfOTEhPSIuIil7XzkwLnB1c2goXzkyPV85MSk7fX19cmV0dXJuIF85MC5qb2luKCIvIik7fSxfN2Y9",
				"ZnVuY3Rpb24ocGlkLG1pZCxfOTMsdXJsLF85NCl7aWYoMSl7dmFyIHhkPXJlcS5pc1hkVXJsKHVybCk7",
				"cmV0dXJuIHtwaWQ6cGlkLG1pZDptaWQscGFjazpfOTMsdXJsOnVybCxleGVjdXRlZDowLGRlZjowLGlz",
				"WGQ6eGQsaXNBbWQ6ISEoeGR8fChfNTBbcGlkXSYmXzUwW3BpZF0uaXNBbWQpKSxjYWNoZUlkOl85NH07",
				"fWVsc2V7cmV0dXJuIHtwaWQ6cGlkLG1pZDptaWQscGFjazpfOTMsdXJsOnVybCxleGVjdXRlZDowLGRl",
				"ZjowLGNhY2hlSWQ6Xzk0fTt9fSxfOTU9ZnVuY3Rpb24obWlkLF85NixfOTcsXzk4LF85OSxfOWEsXzli",
				"LF85Yyl7dmFyIHBpZCxfOWQsXzllLF85ZixfYTAsX2ExLHVybCxfYTIsX2EzLF9hNCxfYTU9MDtfYTQ9",
				"bWlkO19hMz0vXlwuLy50ZXN0KG1pZCk7aWYoLyheXC8pfChcOil8KFwuanMkKS8udGVzdChtaWQpfHwo",
				"X2EzJiYhXzk2KSl7cmV0dXJuIF83ZigwLG1pZCwwLG1pZCk7fWVsc2V7bWlkPV84ZShfYTM/KF85Ni5t",
				"aWQrIi8uLi8iK21pZCk6bWlkKTtpZigvXlwuLy50ZXN0KG1pZCkpe3Rocm93IF9mKCJpcnJhdGlvbmFs",
				"UGF0aCIsbWlkKTt9XzlmPV85NiYmXzk2LnBhY2smJl85Ni5wYWNrLm1hcFByb2c7X2EwPShfOWYmJl84",
				"YyhtaWQsXzlmKSl8fF84YyhtaWQsXzlhKTtpZihfYTApe3BpZD1fYTBbMV07bWlkPW1pZC5zdWJzdHJp",
				"bmcoX2EwWzNdKTtfOWQ9Xzk3W3BpZF07aWYoIW1pZCl7bWlkPV85ZC5tYWluO31fOWU9bWlkO19hNT1f",
				"OWQucmV2ZXJzZU5hbWUrIi8iK21pZDttaWQ9cGlkKyIvIittaWQ7fWVsc2V7cGlkPSIiO312YXIgX2E2",
				"PTAsX2E3PTA7XzkoXzRkLGZ1bmN0aW9uKF9hOCl7dmFyIF9hOT1taWQubWF0Y2goX2E4WzBdKTtpZihf",
				"YTkmJl9hOS5sZW5ndGg+X2E2KXtfYTc9XzYoX2E4WzFdKT9taWQucmVwbGFjZShfYThbMF0sX2E4WzFd",
				"KTpfYThbMV07fX0pO2lmKF9hNyl7cmV0dXJuIF85NShfYTcsMCxfOTcsXzk4LF85OSxfOWEsXzliLF85",
				"Yyk7fV9hMj1fOThbbWlkXTtpZihfYTIpe3JldHVybiBfOWM/XzdmKF9hMi5waWQsX2EyLm1pZCxfYTIu",
				"cGFjayxfYTIudXJsLF9hNSk6Xzk4W21pZF07fX1fYTA9XzhjKG1pZCxfOWIpO2lmKF9hMCl7dXJsPV9h",
				"MFsxXSttaWQuc3Vic3RyaW5nKF9hMFszXS0xKTt9ZWxzZXtpZihwaWQpe3VybD1fOWQubG9jYXRpb24r",
				"Ii8iK185ZTt9ZWxzZXtpZihoYXMoImNvbmZpZy10bG1TaWJsaW5nT2ZEb2pvIikpe3VybD0iLi4vIitt",
				"aWQ7fWVsc2V7dXJsPW1pZDt9fX1pZighKC8oXlwvKXwoXDopLy50ZXN0KHVybCkpKXt1cmw9Xzk5K3Vy",
				"bDt9dXJsKz0iLmpzIjtyZXR1cm4gXzdmKHBpZCxtaWQsXzlkLF84ZSh1cmwpLF9hNSk7fSxfNWI9ZnVu",
				"Y3Rpb24obWlkLF9hYSl7cmV0dXJuIF85NShtaWQsX2FhLF81MCxfMmYscmVxLmJhc2VVcmwsXzUyLF80",
				"Zik7fSxfYWI9ZnVuY3Rpb24oX2FjLF9hZCxfYWUpe3JldHVybiBfYWMubm9ybWFsaXplP19hYy5ub3Jt",
				"YWxpemUoX2FkLGZ1bmN0aW9uKG1pZCl7cmV0dXJuIF9hZihtaWQsX2FlKTt9KTpfYWYoX2FkLF9hZSk7",
				"fSxfYjA9MCxfMzE9ZnVuY3Rpb24obWlkLF9iMSxfYjIpe3ZhciBfYjMsX2I0LF9iNSxfYjY7X2IzPW1p",
				"ZC5tYXRjaCgvXiguKz8pXCEoLiopJC8pO2lmKF9iMyl7X2I0PV8zMShfYjNbMV0sX2IxLF9iMik7aWYo",
				"MSYmXzI1PT1fMjYmJiFfYjQuZXhlY3V0ZWQpe18zMihfYjQpO2lmKF9iNC5pbmplY3RlZD09PV8yMSYm",
				"IV9iNC5leGVjdXRlZCl7Xzc0Kys7XzM2KF9iNCk7XzM3KCk7fWlmKF9iNC5leGVjdXRlZCl7X2I3KF9i",
				"NCk7fWVsc2V7XzMwLnVuc2hpZnQoX2I0KTt9fWlmKF9iNC5leGVjdXRlZD09PV8yNCYmIV9iNC5sb2Fk",
				"KXtfYjcoX2I0KTt9aWYoX2I0LmxvYWQpe19iNT1fYWIoX2I0LF9iM1syXSxfYjEpO21pZD0oX2I0Lm1p",
				"ZCsiISIrKF9iNC5keW5hbWljPysrX2IwKyIhIjoiIikrX2I1KTt9ZWxzZXtfYjU9X2IzWzJdO21pZD1f",
				"YjQubWlkKyIhIisoKytfYjApKyIhd2FpdGluZ0ZvclBsdWdpbiI7fV9iNj17cGx1Z2luOl9iNCxtaWQ6",
				"bWlkLHJlcTpfODEoX2IxKSxwcmlkOl9iNX07fWVsc2V7X2I2PV81YihtaWQsX2IxKTt9cmV0dXJuIF8y",
				"ZltfYjYubWlkXXx8KCFfYjImJihfMmZbX2I2Lm1pZF09X2I2KSk7fSxfYWY9cmVxLnRvQWJzTWlkPWZ1",
				"bmN0aW9uKG1pZCxfYjgpe3JldHVybiBfNWIobWlkLF9iOCkubWlkO30sXzVhPXJlcS50b1VybD1mdW5j",
				"dGlvbihfYjksX2JhKXt2YXIgX2JiPV9iOS5tYXRjaCgvKC4rKShcLlteXC9cLl0rPykkLyksX2JjPShf",
				"YmImJl9iYlsxXSl8fF9iOSxleHQ9KF9iYiYmX2JiWzJdKXx8IiIsX2JkPV81YihfYmMsX2JhKSx1cmw9",
				"X2JkLnVybDt1cmw9dHlwZW9mIF9iZC5waWQ9PSJzdHJpbmciP3VybC5zdWJzdHJpbmcoMCx1cmwubGVu",
				"Z3RoLTMpOnVybDtyZXR1cm4gXzQwKHVybCtleHQpO30sX2JlPXtpbmplY3RlZDpfMjEsZXhlY3V0ZWQ6",
				"XzI0LGRlZjpfMjIscmVzdWx0Ol8yMn0sX2JmPWZ1bmN0aW9uKG1pZCl7cmV0dXJuIF8yZlttaWRdPV9j",
				"KHttaWQ6bWlkfSxfYmUpO30sX2MwPV9iZigicmVxdWlyZSIpLF9jMT1fYmYoImV4cG9ydHMiKSxfYzI9",
				"X2JmKCJtb2R1bGUiKSxfYzM9ZnVuY3Rpb24oX2M0LF9jNSl7cmVxLnRyYWNlKCJsb2FkZXItcnVuLWZh",
				"Y3RvcnkiLFtfYzQubWlkXSk7dmFyIF9jNj1fYzQuZGVmLF9jNzsxJiZfMjcudW5zaGlmdChfYzQpO2lm",
				"KGhhcygiY29uZmlnLWRvam8tbG9hZGVyLWNhdGNoZXMiKSl7dHJ5e19jNz1fNihfYzYpP19jNi5hcHBs",
				"eShudWxsLF9jNSk6X2M2O31jYXRjaChlKXtfMzQoXzQ1LF9jNC5yZXN1bHQ9X2YoImZhY3RvcnlUaHJl",
				"dyIsW19jNCxlXSkpO319ZWxzZXtfYzc9XzYoX2M2KT9fYzYuYXBwbHkobnVsbCxfYzUpOl9jNjt9X2M0",
				"LnJlc3VsdD1fYzc9PT11bmRlZmluZWQmJl9jNC5janM/X2M0LmNqcy5leHBvcnRzOl9jNzsxJiZfMjcu",
				"c2hpZnQoX2M0KTt9LF9jOD17fSxfYzk9MCxfYjc9ZnVuY3Rpb24oX2NhKXt2YXIgX2NiPV9jYS5yZXN1",
				"bHQ7X2NhLmR5bmFtaWM9X2NiLmR5bmFtaWM7X2NhLm5vcm1hbGl6ZT1fY2Iubm9ybWFsaXplO19jYS5s",
				"b2FkPV9jYi5sb2FkO3JldHVybiBfY2E7fSxfY2M9ZnVuY3Rpb24oX2NkKXt2YXIgbWFwPXt9O185KF9j",
				"ZC5sb2FkUSxmdW5jdGlvbihfY2Upe3ZhciBfY2Y9X2NlLm1pZCxfZDA9X2FiKF9jZCxfY2UucHJpZCxf",
				"Y2UucmVxLm1vZHVsZSksbWlkPV9jZC5keW5hbWljP19jZS5taWQucmVwbGFjZSgvd2FpdGluZ0ZvclBs",
				"dWdpbiQvLF9kMCk6KF9jZC5taWQrIiEiK19kMCksX2QxPV9jKF9jKHt9LF9jZSkse21pZDptaWQscHJp",
				"ZDpfZDAsaW5qZWN0ZWQ6MH0pO2lmKCFfMmZbbWlkXSl7X2UyKF8yZlttaWRdPV9kMSk7fW1hcFtfY2Uu",
				"bWlkXT1fMmZbbWlkXTtfMzMoX2NlKTtkZWxldGUgXzJmW19jZS5taWRdO30pO19jZC5sb2FkUT0wO3Zh",
				"ciBfZDI9ZnVuY3Rpb24oX2QzKXtmb3IodmFyIF9kNCxfZDU9X2QzLmRlcHN8fFtdLGk9MDtpPF9kNS5s",
				"ZW5ndGg7aSsrKXtfZDQ9bWFwW19kNVtpXS5taWRdO2lmKF9kNCl7X2Q1W2ldPV9kNDt9fX07Zm9yKHZh",
				"ciBwIGluIF8yZil7X2QyKF8yZltwXSk7fV85KF8zMCxfZDIpO30sXzM1PWZ1bmN0aW9uKF9kNil7cmVx",
				"LnRyYWNlKCJsb2FkZXItZmluaXNoLWV4ZWMiLFtfZDYubWlkXSk7X2Q2LmV4ZWN1dGVkPV8yNDtfZDYu",
				"ZGVmT3JkZXI9X2M5Kys7MSYmXzkoX2Q2LnByb3ZpZGVzLGZ1bmN0aW9uKGNiKXtjYigpO30pO2lmKF9k",
				"Ni5sb2FkUSl7X2I3KF9kNik7X2NjKF9kNik7fWZvcihpPTA7aTxfMzAubGVuZ3RoOyl7aWYoXzMwW2ld",
				"PT09X2Q2KXtfMzAuc3BsaWNlKGksMSk7fWVsc2V7aSsrO319fSxfZDc9W10sXzM2PWZ1bmN0aW9uKF9k",
				"OCxfZDkpe2lmKF9kOC5leGVjdXRlZD09PV8yMyl7cmVxLnRyYWNlKCJsb2FkZXItY2lyY3VsYXItZGVw",
				"ZW5kZW5jeSIsW19kNy5jb25jYXQobWlkKS5qb2luKCItPiIpXSk7cmV0dXJuICghX2Q4LmRlZnx8X2Q5",
				"KT9fYzg6KF9kOC5janMmJl9kOC5janMuZXhwb3J0cyk7fWlmKCFfZDguZXhlY3V0ZWQpe2lmKCFfZDgu",
				"ZGVmKXtyZXR1cm4gX2M4O312YXIgbWlkPV9kOC5taWQsX2RhPV9kOC5kZXBzfHxbXSxhcmcsX2RiLF9k",
				"Yz1bXSxpPTA7aWYoMCl7X2Q3LnB1c2gobWlkKTtyZXEudHJhY2UoImxvYWRlci1leGVjLW1vZHVsZSIs",
				"WyJleGVjIixfZDcubGVuZ3RoLG1pZF0pO31fZDguZXhlY3V0ZWQ9XzIzO3doaWxlKGk8X2RhLmxlbmd0",
				"aCl7YXJnPV9kYVtpKytdO19kYj0oKGFyZz09PV9jMCk/XzgxKF9kOCk6KChhcmc9PT1fYzEpP19kOC5j",
				"anMuZXhwb3J0czooKGFyZz09PV9jMik/X2Q4LmNqczpfMzYoYXJnLF9kOSkpKSk7aWYoX2RiPT09X2M4",
				"KXtfZDguZXhlY3V0ZWQ9MDtyZXEudHJhY2UoImxvYWRlci1leGVjLW1vZHVsZSIsWyJhYm9ydCIsbWlk",
				"XSk7MCYmX2Q3LnBvcCgpO3JldHVybiBfYzg7fV9kYy5wdXNoKF9kYik7fV9jMyhfZDgsX2RjKTtfMzUo",
				"X2Q4KTt9MCYmX2Q3LnBvcCgpO3JldHVybiBfZDgucmVzdWx0O30sXzc0PTAsXzc4PWZ1bmN0aW9uKCl7",
				"aWYoXzc0KXtyZXR1cm47fV83NCsrO18yOSgpO2Zvcih2YXIgX2RkLF9kZSxpPTA7aTxfMzAubGVuZ3Ro",
				"Oyl7X2RkPV9jOTtfZGU9XzMwW2ldO18zNihfZGUpO2lmKF9kZCE9X2M5KXtfMjkoKTtpPTA7fWVsc2V7",
				"aSsrO319XzM3KCk7fSxfMzc9ZnVuY3Rpb24oKXtfNzQtLTtpZihfOGIoKSl7XzM0KCJpZGxlIixbXSk7",
				"fX07aWYoMCl7cmVxLnVuZGVmPWZ1bmN0aW9uKF9kZixfZTApe3ZhciBfZTE9XzMxKF9kZixfZTApO18z",
				"MyhfZTEpO2RlbGV0ZSBfMmZbX2UxLm1pZF07fTt9aWYoMSl7aWYoaGFzKCJkb2pvLWxvYWRlci1ldmFs",
				"LWhpbnQtdXJsIik9PT11bmRlZmluZWQpe2hhcy5hZGQoImRvam8tbG9hZGVyLWV2YWwtaGludC11cmwi",
				"LDEpO312YXIgXzQwPWZ1bmN0aW9uKHVybCl7dXJsKz0iIjtyZXR1cm4gdXJsKyhfNTM/KCgvXD8vLnRl",
				"c3QodXJsKT8iJiI6Ij8iKStfNTMpOiIiKTt9LF9lMj1mdW5jdGlvbihfZTMpe3ZhciBfZTQ9X2UzLnBs",
				"dWdpbjtpZihfZTQuZXhlY3V0ZWQ9PT1fMjQmJiFfZTQubG9hZCl7X2I3KF9lNCk7fXZhciBfZTU9ZnVu",
				"Y3Rpb24oZGVmKXtfZTMucmVzdWx0PWRlZjtfMzMoX2UzKTtfMzUoX2UzKTtfNzgoKTt9O184NyhfZTMp",
				"O2lmKF9lNC5sb2FkKXtfZTQubG9hZChfZTMucHJpZCxfZTMucmVxLF9lNSk7fWVsc2V7aWYoX2U0Lmxv",
				"YWRRKXtfZTQubG9hZFEucHVzaChfZTMpO31lbHNle18zMC51bnNoaWZ0KF9lNCk7XzMyKF9lNCk7aWYo",
				"X2U0LmxvYWQpe19lNC5sb2FkKF9lMy5wcmlkLF9lMy5yZXEsX2U1KTt9ZWxzZXtfZTQubG9hZFE9W19l",
				"M107fX19fSxfZTY9MCxfNzk9MCxfZTc9MCxfZTg9ZnVuY3Rpb24oX2U5LF9lYSl7X2U3PTE7aWYoaGFz",
				"KCJjb25maWctZG9qby1sb2FkZXItY2F0Y2hlcyIpKXt0cnl7aWYoX2U5PT09X2U2KXtfZTYuY2FsbChu",
				"dWxsKTt9ZWxzZXtyZXEuZXZhbChfZTksaGFzKCJkb2pvLWxvYWRlci1ldmFsLWhpbnQtdXJsIik/X2Vh",
				"LnVybDpfZWEubWlkKTt9fWNhdGNoKGUpe18zNChfNDUsX2YoImV2YWxNb2R1bGVUaHJldyIsX2VhKSk7",
				"fX1lbHNle2lmKF9lOT09PV9lNil7X2U2LmNhbGwobnVsbCk7fWVsc2V7cmVxLmV2YWwoX2U5LGhhcygi",
				"ZG9qby1sb2FkZXItZXZhbC1oaW50LXVybCIpP19lYS51cmw6X2VhLm1pZCk7fX1fZTc9MDt9LF8zMj1m",
				"dW5jdGlvbihfZWIpe3ZhciBtaWQ9X2ViLm1pZCx1cmw9X2ViLnVybDtpZihfZWIuZXhlY3V0ZWR8fF9l",
				"Yi5pbmplY3RlZHx8Xzg2W21pZF18fChfZWIudXJsJiYoKF9lYi5wYWNrJiZfODZbX2ViLnVybF09PT1f",
				"ZWIucGFjayl8fF84NltfZWIudXJsXT09MSkpKXtyZXR1cm47fWlmKDApe3ZhciBfZWM9MDtpZihfZWIu",
				"cGx1Z2luJiZfZWIucGx1Z2luLmlzQ29tYm8pe3JlcS5jb21iby5hZGQoX2ViLnBsdWdpbi5taWQsX2Vi",
				"LnByaWQsMCxyZXEpO19lYz0xO31lbHNle2lmKCFfZWIucGx1Z2luKXtfZWM9cmVxLmNvbWJvLmFkZCgw",
				"LF9lYi5taWQsX2ViLnVybCxyZXEpO319aWYoX2VjKXtfODcoX2ViKTtfNmY9MTtyZXR1cm47fX1pZihf",
				"ZWIucGx1Z2luKXtfZTIoX2ViKTtyZXR1cm47fV84NyhfZWIpO3ZhciBfZWQ9ZnVuY3Rpb24oKXtfNzco",
				"X2ViKTtpZihfZWIuaW5qZWN0ZWQhPT1fMjEpe18zMyhfZWIpO19jKF9lYixfYmUpO31pZigxJiZfMjUp",
				"eyFfMjcubGVuZ3RoJiZfNzgoKTt9ZWxzZXtfNzgoKTt9fTtfZTY9XzU0W21pZF18fF81NFtfZWIuY2Fj",
				"aGVJZF07aWYoX2U2KXtyZXEudHJhY2UoImxvYWRlci1pbmplY3QiLFsiY2FjaGUiLF9lYi5taWQsdXJs",
				"XSk7X2U4KF9lNixfZWIpO19lZCgpO3JldHVybjt9aWYoMSYmXzI1KXtpZihfZWIuaXNYZCl7XzI1PT1f",
				"MjYmJihfMjU9eGQpO31lbHNle2lmKF9lYi5pc0FtZCYmXzI1IT1fMjYpe31lbHNle3ZhciBfZWU9ZnVu",
				"Y3Rpb24oX2VmKXtpZihfMjU9PV8yNil7XzI3LnVuc2hpZnQoX2ViKTtfZTgoX2VmLF9lYik7XzI3LnNo",
				"aWZ0KCk7Xzc3KF9lYik7aWYoIV9lYi5janMpe18zMyhfZWIpO18zNShfZWIpO31pZihfZWIuZmluaXNo",
				"KXt2YXIgX2YwPW1pZCsiKmZpbmlzaCIsX2YxPV9lYi5maW5pc2g7ZGVsZXRlIF9lYi5maW5pc2g7ZGVm",
				"KF9mMCxbImRvam8iLCgiZG9qby9yZXF1aXJlISIrX2YxLmpvaW4oIiwiKSkucmVwbGFjZSgvXC4vZywi",
				"LyIpXSxmdW5jdGlvbihfZjIpe185KF9mMSxmdW5jdGlvbihtaWQpe19mMi5yZXF1aXJlKG1pZCk7fSk7",
				"fSk7XzMwLnVuc2hpZnQoXzMxKF9mMCkpO31fZWQoKTt9ZWxzZXtfZWY9XzJhKF9lYixfZWYpO2lmKF9l",
				"Zil7X2U4KF9lZixfZWIpO19lZCgpO31lbHNle183OT1fZWI7cmVxLmluamVjdFVybChfNDAodXJsKSxf",
				"ZWQsX2ViKTtfNzk9MDt9fX07cmVxLnRyYWNlKCJsb2FkZXItaW5qZWN0IixbInhociIsX2ViLm1pZCx1",
				"cmwsXzI1IT1fMjZdKTtpZihoYXMoImNvbmZpZy1kb2pvLWxvYWRlci1jYXRjaGVzIikpe3RyeXtyZXEu",
				"Z2V0VGV4dCh1cmwsXzI1IT1fMjYsX2VlKTt9Y2F0Y2goZSl7XzM0KF80NSxfZigieGhySW5qZWN0RmFp",
				"bGVkIixbX2ViLGVdKSk7fX1lbHNle3JlcS5nZXRUZXh0KHVybCxfMjUhPV8yNixfZWUpO31yZXR1cm47",
				"fX19cmVxLnRyYWNlKCJsb2FkZXItaW5qZWN0IixbInNjcmlwdCIsX2ViLm1pZCx1cmxdKTtfNzk9X2Vi",
				"O3JlcS5pbmplY3RVcmwoXzQwKHVybCksX2VkLF9lYik7Xzc5PTA7fSxfZjM9ZnVuY3Rpb24oX2Y0LF9m",
				"NSxkZWYpe3JlcS50cmFjZSgibG9hZGVyLWRlZmluZS1tb2R1bGUiLFtfZjQubWlkLF9mNV0pO2lmKDAm",
				"Jl9mNC5wbHVnaW4mJl9mNC5wbHVnaW4uaXNDb21ibyl7X2Y0LnJlc3VsdD1fNihkZWYpP2RlZigpOmRl",
				"ZjtfMzMoX2Y0KTtfMzUoX2Y0KTtyZXR1cm4gX2Y0O312YXIgbWlkPV9mNC5taWQ7aWYoX2Y0LmluamVj",
				"dGVkPT09XzIxKXtfMzQoXzQ1LF9mKCJtdWx0aXBsZURlZmluZSIsX2Y0KSk7cmV0dXJuIF9mNDt9X2Mo",
				"X2Y0LHtkZXBzOl9mNSxkZWY6ZGVmLGNqczp7aWQ6X2Y0Lm1pZCx1cmk6X2Y0LnVybCxleHBvcnRzOihf",
				"ZjQucmVzdWx0PXt9KSxzZXRFeHBvcnRzOmZ1bmN0aW9uKF9mNil7X2Y0LmNqcy5leHBvcnRzPV9mNjt9",
				"fX0pO2Zvcih2YXIgaT0wO2k8X2Y1Lmxlbmd0aDtpKyspe19mNVtpXT1fMzEoX2Y1W2ldLF9mNCk7fWlm",
				"KDEmJl8yNSYmIV84NlttaWRdKXtfNzIoX2Y0KTtfMzAucHVzaChfZjQpO183OCgpO31fMzMoX2Y0KTtp",
				"ZighXzYoZGVmKSYmIV9mNS5sZW5ndGgpe19mNC5yZXN1bHQ9ZGVmO18zNShfZjQpO31yZXR1cm4gX2Y0",
				"O30sXzc3PWZ1bmN0aW9uKF9mNyxfZjgpe181NyhfZjcpO3ZhciBfZjk9W10sX2ZhLF9mYjt3aGlsZShf",
				"ODUubGVuZ3RoKXtfZmI9Xzg1LnNoaWZ0KCk7X2Y4JiYoX2ZiWzBdPV9mOC5zaGlmdCgpKTtfZmE9X2Zi",
				"WzBdJiZfMzEoX2ZiWzBdKXx8X2Y3O19mOS5wdXNoKF9mMyhfZmEsX2ZiWzFdLF9mYlsyXSkpO31fOShf",
				"ZjksXzcyKTt9O312YXIgX2ZjPTAsXzhhPV8zLF9mZD1fMztpZigxKXtfOGE9ZnVuY3Rpb24oKXtfZmMm",
				"JmNsZWFyVGltZW91dChfZmMpO19mYz0wO30sX2ZkPWZ1bmN0aW9uKCl7XzhhKCk7cmVxLndhaXRtcyYm",
				"KF9mYz1zZXRUaW1lb3V0KGZ1bmN0aW9uKCl7XzhhKCk7XzM0KF80NSxfZigidGltZW91dCIsXzg2KSk7",
				"fSxyZXEud2FpdG1zKSk7fTt9aWYoMSl7aGFzLmFkZCgiaWUtZXZlbnQtYmVoYXZpb3IiLGRvYy5hdHRh",
				"Y2hFdmVudCYmKHR5cGVvZiBvcGVyYT09PSJ1bmRlZmluZWQifHxvcGVyYS50b1N0cmluZygpIT0iW29i",
				"amVjdCBPcGVyYV0iKSk7fWlmKDEmJigxfHwxKSl7dmFyIF9mZT1mdW5jdGlvbihfZmYsXzEwMCxfMTAx",
				"LF8xMDIpe2lmKCFoYXMoImllLWV2ZW50LWJlaGF2aW9yIikpe19mZi5hZGRFdmVudExpc3RlbmVyKF8x",
				"MDAsXzEwMixmYWxzZSk7cmV0dXJuIGZ1bmN0aW9uKCl7X2ZmLnJlbW92ZUV2ZW50TGlzdGVuZXIoXzEw",
				"MCxfMTAyLGZhbHNlKTt9O31lbHNle19mZi5hdHRhY2hFdmVudChfMTAxLF8xMDIpO3JldHVybiBmdW5j",
				"dGlvbigpe19mZi5kZXRhY2hFdmVudChfMTAxLF8xMDIpO307fX0sXzEwMz1fZmUod2luZG93LCJsb2Fk",
				"Iiwib25sb2FkIixmdW5jdGlvbigpe3JlcS5wYWdlTG9hZGVkPTE7ZG9jLnJlYWR5U3RhdGUhPSJjb21w",
				"bGV0ZSImJihkb2MucmVhZHlTdGF0ZT0iY29tcGxldGUiKTtfMTAzKCk7fSk7aWYoMSl7dmFyIF8xMDQ9",
				"ZG9jLmdldEVsZW1lbnRzQnlUYWdOYW1lKCJzY3JpcHQiKVswXSxfMTA1PV8xMDQucGFyZW50Tm9kZTty",
				"ZXEuaW5qZWN0VXJsPWZ1bmN0aW9uKHVybCxfMTA2LF8xMDcpe19mZCgpO3ZhciBub2RlPV8xMDcubm9k",
				"ZT1kb2MuY3JlYXRlRWxlbWVudCgic2NyaXB0IiksXzEwOD1mdW5jdGlvbihlKXtlPWV8fHdpbmRvdy5l",
				"dmVudDt2YXIgbm9kZT1lLnRhcmdldHx8ZS5zcmNFbGVtZW50O2lmKGUudHlwZT09PSJsb2FkInx8L2Nv",
				"bXBsZXRlfGxvYWRlZC8udGVzdChub2RlLnJlYWR5U3RhdGUpKXtfMTA5KCk7XzEwNiYmXzEwNigpO319",
				"LF8xMDk9X2ZlKG5vZGUsImxvYWQiLCJvbnJlYWR5c3RhdGVjaGFuZ2UiLF8xMDgpO25vZGUudHlwZT0i",
				"dGV4dC9qYXZhc2NyaXB0Ijtub2RlLmNoYXJzZXQ9InV0Zi04Ijtub2RlLnNyYz11cmw7XzEwNS5pbnNl",
				"cnRCZWZvcmUobm9kZSxfMTA0KTtyZXR1cm4gbm9kZTt9O319aWYoMSl7cmVxLmxvZz1mdW5jdGlvbigp",
				"e3RyeXtmb3IodmFyIGk9MDtpPGFyZ3VtZW50cy5sZW5ndGg7aSsrKXt9fWNhdGNoKGUpe319O31lbHNl",
				"e3JlcS5sb2c9XzM7fWlmKDApe3ZhciBfMTBhPXJlcS50cmFjZT1mdW5jdGlvbihfMTBiLGFyZ3Mpe2lm",
				"KF8xMGEub24mJl8xMGEuZ3JvdXBbXzEwYl0pe18zNCgidHJhY2UiLFtfMTBiLGFyZ3NdKTtmb3IodmFy",
				"IGFyZyxkdW1wPVtdLHRleHQ9InRyYWNlOiIrXzEwYisoYXJncy5sZW5ndGg/KCI6IithcmdzWzBdKToi",
				"IiksaT0xO2k8YXJncy5sZW5ndGg7KXthcmc9YXJnc1tpKytdO2lmKF83KGFyZykpe3RleHQrPSIsICIr",
				"YXJnO31lbHNle2R1bXAucHVzaChhcmcpO319cmVxLmxvZyh0ZXh0KTtkdW1wLmxlbmd0aCYmZHVtcC5w",
				"dXNoKCIuIik7cmVxLmxvZy5hcHBseShyZXEsZHVtcCk7fX07X2MoXzEwYSx7b246MSxncm91cDp7fSxz",
				"ZXQ6ZnVuY3Rpb24oXzEwYyxfMTBkKXtpZihfNyhfMTBjKSl7XzEwYS5ncm91cFtfMTBjXT1fMTBkO31l",
				"bHNle19jKF8xMGEuZ3JvdXAsXzEwYyk7fX19KTtfMTBhLnNldChfYyhfYyhfYyh7fSxfMi50cmFjZSks",
				"XzEudHJhY2UpLF81Ni50cmFjZSkpO29uKCJjb25maWciLGZ1bmN0aW9uKF8xMGUpe18xMGUudHJhY2Um",
				"Jl8xMGEuc2V0KF8xMGUudHJhY2UpO30pO31lbHNle3JlcS50cmFjZT1fMzt9dmFyIGRlZj1mdW5jdGlv",
				"bihtaWQsXzEwZixfMTEwKXt2YXIgXzExMT1hcmd1bWVudHMubGVuZ3RoLGFyZ3M9MCxfMTEyPVsicmVx",
				"dWlyZSIsImV4cG9ydHMiLCJtb2R1bGUiXTtpZigwKXtpZihfMTExPT0xJiZfNihtaWQpKXtfMTBmPVtd",
				"O21pZC50b1N0cmluZygpLnJlcGxhY2UoLyhcL1wqKFtcc1xTXSo/KVwqXC98XC9cLyguKikkKS9tZywi",
				"IikucmVwbGFjZSgvcmVxdWlyZVwoWyInXShbXHdcIVwtX1wuXC9dKylbIiddXCkvZyxmdW5jdGlvbihf",
				"MTEzLGRlcCl7XzEwZi5wdXNoKGRlcCk7fSk7YXJncz1bMCxfMTEyLmNvbmNhdChfMTBmKSxtaWRdO319",
				"aWYoIWFyZ3Mpe2FyZ3M9XzExMT09MT9bMCxfMTEyLG1pZF06KF8xMTE9PTI/KF84KG1pZCk/WzAsbWlk",
				"LF8xMGZdOihfNihfMTBmKT9bbWlkLF8xMTIsXzEwZl06W21pZCxbXSxfMTBmXSkpOlttaWQsXzEwZixf",
				"MTEwXSk7fXJlcS50cmFjZSgibG9hZGVyLWRlZmluZSIsYXJncy5zbGljZSgwLDIpKTt2YXIgXzExND1h",
				"cmdzWzBdJiZfMzEoYXJnc1swXSksXzExNTtpZihfMTE0JiYhXzg2W18xMTQubWlkXSl7XzcyKF9mMyhf",
				"MTE0LGFyZ3NbMV0sYXJnc1syXSkpO31lbHNle2lmKCFoYXMoImllLWV2ZW50LWJlaGF2aW9yIil8fCEx",
				"fHxfZTcpe184NS5wdXNoKGFyZ3MpO31lbHNle18xMTQ9XzExNHx8Xzc5O2lmKCFfMTE0KXtmb3IobWlk",
				"IGluIF84Nil7XzExNT1fMmZbbWlkXTtpZihfMTE1JiZfMTE1Lm5vZGUmJl8xMTUubm9kZS5yZWFkeVN0",
				"YXRlPT09ImludGVyYWN0aXZlIil7XzExND1fMTE1O2JyZWFrO319aWYoMCYmIV8xMTQpe2Zvcih2YXIg",
				"aT0wO2k8XzcwLmxlbmd0aDtpKyspe18xMTQ9XzcwW2ldO2lmKF8xMTQubm9kZSYmXzExNC5ub2RlLnJl",
				"YWR5U3RhdGU9PT0iaW50ZXJhY3RpdmUiKXticmVhazt9XzExND0wO319fWlmKDAmJl84KF8xMTQpKXtf",
				"NzIoX2YzKF8zMShfMTE0LnNoaWZ0KCkpLGFyZ3NbMV0sYXJnc1syXSkpO2lmKCFfMTE0Lmxlbmd0aCl7",
				"XzcwLnNwbGljZShpLDEpO319ZWxzZXtpZihfMTE0KXtfNTcoXzExNCk7XzcyKF9mMyhfMTE0LGFyZ3Nb",
				"MV0sYXJnc1syXSkpO31lbHNle18zNChfNDUsX2YoImllRGVmaW5lRmFpbGVkIixhcmdzWzBdKSk7fX1f",
				"NzgoKTt9fX07ZGVmLmFtZD17dmVuZG9yOiJkb2pvdG9vbGtpdC5vcmcifTtpZigwKXtyZXEuZGVmPWRl",
				"Zjt9X2MoX2MocmVxLF8yLmxvYWRlclBhdGNoKSxfMS5sb2FkZXJQYXRjaCk7b24oXzQ1LGZ1bmN0aW9u",
				"KGFyZyl7dHJ5e2NvbnNvbGUuZXJyb3IoYXJnKTtpZihhcmcgaW5zdGFuY2VvZiBFcnJvcil7Zm9yKHZh",
				"ciBwIGluIGFyZyl7fX19Y2F0Y2goZSl7fX0pO19jKHJlcSx7dWlkOnVpZCxjYWNoZTpfNTQscGFja3M6",
				"XzUwfSk7aWYoMCl7X2MocmVxLHtwYXRoczpfNGUsYWxpYXNlczpfNGQscGFja2FnZU1hcDpfNTEsbW9k",
				"dWxlczpfMmYsbGVnYWN5TW9kZTpfMjUsZXhlY1E6XzMwLGRlZlE6Xzg1LHdhaXRpbmc6Xzg2LHBhdGhz",
				"TWFwUHJvZzpfNGYscGFja2FnZU1hcFByb2c6XzUyLGxpc3RlbmVyUXVldWVzOl80NCxjb21wdXRlTWFw",
				"UHJvZzpfNWMscnVuTWFwUHJvZzpfOGMsY29tcGFjdFBhdGg6XzhlLGdldE1vZHVsZUluZm86Xzk1fSk7",
				"fWlmKF8xNy5kZWZpbmUpe2lmKDEpe18zNChfNDUsX2YoImRlZmluZUFscmVhZHlEZWZpbmVkIiwwKSk7",
				"fX1lbHNle18xNy5kZWZpbmU9ZGVmO18xNy5yZXF1aXJlPXJlcTt9aWYoMCYmcmVxLmNvbWJvJiZyZXEu",
				"Y29tYm8ucGx1Z2lucyl7dmFyIF8xMTY9cmVxLmNvbWJvLnBsdWdpbnMsXzExNztmb3IoXzExNyBpbiBf",
				"MTE2KXtfYyhfYyhfMzEoXzExNyksXzExNltfMTE3XSkse2lzQ29tYm86MSxleGVjdXRlZDoiZXhlY3V0",
				"ZWQiLGxvYWQ6MX0pO319aWYoMSl7dmFyIF8xMTg9XzIuZGVwc3x8XzEuZGVwc3x8XzU2LmRlcHMsXzEx",
				"OT1fMi5jYWxsYmFja3x8XzEuY2FsbGJhY2t8fF81Ni5jYWxsYmFjaztyZXEuYm9vdD0oXzExOHx8XzEx",
				"OSk/W18xMTh8fFtdLF8xMTldOjA7fWlmKCExKXshcmVxLmFzeW5jJiZyZXEoWyJkb2pvIl0pO3JlcS5i",
				"b290JiZyZXEuYXBwbHkobnVsbCxyZXEuYm9vdCk7fX0pKHRoaXMuZG9qb0NvbmZpZ3x8dGhpcy5kakNv",
				"bmZpZ3x8dGhpcy5yZXF1aXJlfHx7fSx7YXN5bmM6MCxoYXNDYWNoZTp7ImNvbmZpZy1zZWxlY3RvckVu",
				"Z2luZSI6ImFjbWUiLCJjb25maWctdGxtU2libGluZ09mRG9qbyI6MSwiZG9qby1idWlsdCI6MSwiZG9q",
				"by1sb2FkZXIiOjEsZG9tOjEsImhvc3QtYnJvd3NlciI6MX0scGFja2FnZXM6W3tsb2NhdGlvbjoiLi4v",
				"ZGlqaXQiLG5hbWU6ImRpaml0In0se2xvY2F0aW9uOiIuLi9kb2pveCIsbmFtZToiZG9qb3gifSx7bG9j",
				"YXRpb246Ii4iLG5hbWU6ImRvam8ifV19KTtyZXF1aXJlKHtjYWNoZTp7ImRvam8vX2Jhc2UvZngiOmZ1",
				"bmN0aW9uKCl7ZGVmaW5lKFsiLi9rZXJuZWwiLCIuL2xhbmciLCIuLi9FdmVudGVkIiwiLi9Db2xvciIs",
				"Ii4vY29ubmVjdCIsIi4vc25pZmYiLCIuLi9kb20iLCIuLi9kb20tc3R5bGUiXSxmdW5jdGlvbihkb2pv",
				"LGxhbmcsXzExYSxfMTFiLF8xMWMsaGFzLGRvbSxfMTFkKXt2YXIgXzExZT1sYW5nLm1peGluO2Rvam8u",
				"X0xpbmU9ZnVuY3Rpb24oXzExZixlbmQpe3RoaXMuc3RhcnQ9XzExZjt0aGlzLmVuZD1lbmQ7fTtkb2pv",
				"Ll9MaW5lLnByb3RvdHlwZS5nZXRWYWx1ZT1mdW5jdGlvbihuKXtyZXR1cm4gKCh0aGlzLmVuZC10aGlz",
				"LnN0YXJ0KSpuKSt0aGlzLnN0YXJ0O307ZG9qby5BbmltYXRpb249ZnVuY3Rpb24oYXJncyl7XzExZSh0",
				"aGlzLGFyZ3MpO2lmKGxhbmcuaXNBcnJheSh0aGlzLmN1cnZlKSl7dGhpcy5jdXJ2ZT1uZXcgZG9qby5f",
				"TGluZSh0aGlzLmN1cnZlWzBdLHRoaXMuY3VydmVbMV0pO319O2Rvam8uQW5pbWF0aW9uLnByb3RvdHlw",
				"ZT1uZXcgXzExYSgpO2Rvam8uX0FuaW1hdGlvbj1kb2pvLkFuaW1hdGlvbjtsYW5nLmV4dGVuZChkb2pv",
				"LkFuaW1hdGlvbix7ZHVyYXRpb246MzUwLHJlcGVhdDowLHJhdGU6MjAsX3BlcmNlbnQ6MCxfc3RhcnRS",
				"ZXBlYXRDb3VudDowLF9nZXRTdGVwOmZ1bmN0aW9uKCl7dmFyIF8xMjA9dGhpcy5fcGVyY2VudCxfMTIx",
				"PXRoaXMuZWFzaW5nO3JldHVybiBfMTIxP18xMjEoXzEyMCk6XzEyMDt9LF9maXJlOmZ1bmN0aW9uKGV2",
				"dCxhcmdzKXt2YXIgYT1hcmdzfHxbXTtpZih0aGlzW2V2dF0pe2lmKGRvam8uY29uZmlnLmRlYnVnQXRB",
				"bGxDb3N0cyl7dGhpc1tldnRdLmFwcGx5KHRoaXMsYSk7fWVsc2V7dHJ5e3RoaXNbZXZ0XS5hcHBseSh0",
				"aGlzLGEpO31jYXRjaChlKXtjb25zb2xlLmVycm9yKCJleGNlcHRpb24gaW4gYW5pbWF0aW9uIGhhbmRs",
				"ZXIgZm9yOiIsZXZ0KTtjb25zb2xlLmVycm9yKGUpO319fXJldHVybiB0aGlzO30scGxheTpmdW5jdGlv",
				"bihfMTIyLF8xMjMpe3ZhciBfMTI0PXRoaXM7aWYoXzEyNC5fZGVsYXlUaW1lcil7XzEyNC5fY2xlYXJU",
				"aW1lcigpO31pZihfMTIzKXtfMTI0Ll9zdG9wVGltZXIoKTtfMTI0Ll9hY3RpdmU9XzEyNC5fcGF1c2Vk",
				"PWZhbHNlO18xMjQuX3BlcmNlbnQ9MDt9ZWxzZXtpZihfMTI0Ll9hY3RpdmUmJiFfMTI0Ll9wYXVzZWQp",
				"e3JldHVybiBfMTI0O319XzEyNC5fZmlyZSgiYmVmb3JlQmVnaW4iLFtfMTI0Lm5vZGVdKTt2YXIgZGU9",
				"XzEyMnx8XzEyNC5kZWxheSxfMTI1PWxhbmcuaGl0Y2goXzEyNCwiX3BsYXkiLF8xMjMpO2lmKGRlPjAp",
				"e18xMjQuX2RlbGF5VGltZXI9c2V0VGltZW91dChfMTI1LGRlKTtyZXR1cm4gXzEyNDt9XzEyNSgpO3Jl",
				"dHVybiBfMTI0O30sX3BsYXk6ZnVuY3Rpb24oXzEyNil7dmFyIF8xMjc9dGhpcztpZihfMTI3Ll9kZWxh",
				"eVRpbWVyKXtfMTI3Ll9jbGVhclRpbWVyKCk7fV8xMjcuX3N0YXJ0VGltZT1uZXcgRGF0ZSgpLnZhbHVl",
				"T2YoKTtpZihfMTI3Ll9wYXVzZWQpe18xMjcuX3N0YXJ0VGltZS09XzEyNy5kdXJhdGlvbipfMTI3Ll9w",
				"ZXJjZW50O31fMTI3Ll9hY3RpdmU9dHJ1ZTtfMTI3Ll9wYXVzZWQ9ZmFsc2U7dmFyIF8xMjg9XzEyNy5j",
				"dXJ2ZS5nZXRWYWx1ZShfMTI3Ll9nZXRTdGVwKCkpO2lmKCFfMTI3Ll9wZXJjZW50KXtpZighXzEyNy5f",
				"c3RhcnRSZXBlYXRDb3VudCl7XzEyNy5fc3RhcnRSZXBlYXRDb3VudD1fMTI3LnJlcGVhdDt9XzEyNy5f",
				"ZmlyZSgib25CZWdpbiIsW18xMjhdKTt9XzEyNy5fZmlyZSgib25QbGF5IixbXzEyOF0pO18xMjcuX2N5",
				"Y2xlKCk7cmV0dXJuIF8xMjc7fSxwYXVzZTpmdW5jdGlvbigpe3ZhciBfMTI5PXRoaXM7aWYoXzEyOS5f",
				"ZGVsYXlUaW1lcil7XzEyOS5fY2xlYXJUaW1lcigpO31fMTI5Ll9zdG9wVGltZXIoKTtpZighXzEyOS5f",
				"YWN0aXZlKXtyZXR1cm4gXzEyOTt9XzEyOS5fcGF1c2VkPXRydWU7XzEyOS5fZmlyZSgib25QYXVzZSIs",
				"W18xMjkuY3VydmUuZ2V0VmFsdWUoXzEyOS5fZ2V0U3RlcCgpKV0pO3JldHVybiBfMTI5O30sZ290b1Bl",
				"cmNlbnQ6ZnVuY3Rpb24oXzEyYSxfMTJiKXt2YXIgXzEyYz10aGlzO18xMmMuX3N0b3BUaW1lcigpO18x",
				"MmMuX2FjdGl2ZT1fMTJjLl9wYXVzZWQ9dHJ1ZTtfMTJjLl9wZXJjZW50PV8xMmE7aWYoXzEyYil7XzEy",
				"Yy5wbGF5KCk7fXJldHVybiBfMTJjO30sc3RvcDpmdW5jdGlvbihfMTJkKXt2YXIgXzEyZT10aGlzO2lm",
				"KF8xMmUuX2RlbGF5VGltZXIpe18xMmUuX2NsZWFyVGltZXIoKTt9aWYoIV8xMmUuX3RpbWVyKXtyZXR1",
				"cm4gXzEyZTt9XzEyZS5fc3RvcFRpbWVyKCk7aWYoXzEyZCl7XzEyZS5fcGVyY2VudD0xO31fMTJlLl9m",
				"aXJlKCJvblN0b3AiLFtfMTJlLmN1cnZlLmdldFZhbHVlKF8xMmUuX2dldFN0ZXAoKSldKTtfMTJlLl9h",
				"Y3RpdmU9XzEyZS5fcGF1c2VkPWZhbHNlO3JldHVybiBfMTJlO30sc3RhdHVzOmZ1bmN0aW9uKCl7aWYo",
				"dGhpcy5fYWN0aXZlKXtyZXR1cm4gdGhpcy5fcGF1c2VkPyJwYXVzZWQiOiJwbGF5aW5nIjt9cmV0dXJu",
				"ICJzdG9wcGVkIjt9LF9jeWNsZTpmdW5jdGlvbigpe3ZhciBfMTJmPXRoaXM7aWYoXzEyZi5fYWN0aXZl",
				"KXt2YXIgY3Vycj1uZXcgRGF0ZSgpLnZhbHVlT2YoKTt2YXIgc3RlcD0oY3Vyci1fMTJmLl9zdGFydFRp",
				"bWUpLyhfMTJmLmR1cmF0aW9uKTtpZihzdGVwPj0xKXtzdGVwPTE7fV8xMmYuX3BlcmNlbnQ9c3RlcDtp",
				"ZihfMTJmLmVhc2luZyl7c3RlcD1fMTJmLmVhc2luZyhzdGVwKTt9XzEyZi5fZmlyZSgib25BbmltYXRl",
				"IixbXzEyZi5jdXJ2ZS5nZXRWYWx1ZShzdGVwKV0pO2lmKF8xMmYuX3BlcmNlbnQ8MSl7XzEyZi5fc3Rh",
				"cnRUaW1lcigpO31lbHNle18xMmYuX2FjdGl2ZT1mYWxzZTtpZihfMTJmLnJlcGVhdD4wKXtfMTJmLnJl",
				"cGVhdC0tO18xMmYucGxheShudWxsLHRydWUpO31lbHNle2lmKF8xMmYucmVwZWF0PT0tMSl7XzEyZi5w",
				"bGF5KG51bGwsdHJ1ZSk7fWVsc2V7aWYoXzEyZi5fc3RhcnRSZXBlYXRDb3VudCl7XzEyZi5yZXBlYXQ9",
				"XzEyZi5fc3RhcnRSZXBlYXRDb3VudDtfMTJmLl9zdGFydFJlcGVhdENvdW50PTA7fX19XzEyZi5fcGVy",
				"Y2VudD0wO18xMmYuX2ZpcmUoIm9uRW5kIixbXzEyZi5ub2RlXSk7IV8xMmYucmVwZWF0JiZfMTJmLl9z",
				"dG9wVGltZXIoKTt9fXJldHVybiBfMTJmO30sX2NsZWFyVGltZXI6ZnVuY3Rpb24oKXtjbGVhclRpbWVv",
				"dXQodGhpcy5fZGVsYXlUaW1lcik7ZGVsZXRlIHRoaXMuX2RlbGF5VGltZXI7fX0pO3ZhciBjdHI9MCxf",
				"MTMwPW51bGwsXzEzMT17cnVuOmZ1bmN0aW9uKCl7fX07bGFuZy5leHRlbmQoZG9qby5BbmltYXRpb24s",
				"e19zdGFydFRpbWVyOmZ1bmN0aW9uKCl7aWYoIXRoaXMuX3RpbWVyKXt0aGlzLl90aW1lcj1fMTFjLmNv",
				"bm5lY3QoXzEzMSwicnVuIix0aGlzLCJfY3ljbGUiKTtjdHIrKzt9aWYoIV8xMzApe18xMzA9c2V0SW50",
				"ZXJ2YWwobGFuZy5oaXRjaChfMTMxLCJydW4iKSx0aGlzLnJhdGUpO319LF9zdG9wVGltZXI6ZnVuY3Rp",
				"b24oKXtpZih0aGlzLl90aW1lcil7XzExYy5kaXNjb25uZWN0KHRoaXMuX3RpbWVyKTt0aGlzLl90aW1l",
				"cj1udWxsO2N0ci0tO31pZihjdHI8PTApe2NsZWFySW50ZXJ2YWwoXzEzMCk7XzEzMD1udWxsO2N0cj0w",
				"O319fSk7dmFyIF8xMzI9aGFzKCJpZSIpP2Z1bmN0aW9uKG5vZGUpe3ZhciBucz1ub2RlLnN0eWxlO2lm",
				"KCFucy53aWR0aC5sZW5ndGgmJl8xMWQuZ2V0KG5vZGUsIndpZHRoIik9PSJhdXRvIil7bnMud2lkdGg9",
				"ImF1dG8iO319OmZ1bmN0aW9uKCl7fTtkb2pvLl9mYWRlPWZ1bmN0aW9uKGFyZ3Mpe2FyZ3Mubm9kZT1k",
				"b20uYnlJZChhcmdzLm5vZGUpO3ZhciBfMTMzPV8xMWUoe3Byb3BlcnRpZXM6e319LGFyZ3MpLF8xMzQ9",
				"KF8xMzMucHJvcGVydGllcy5vcGFjaXR5PXt9KTtfMTM0LnN0YXJ0PSEoInN0YXJ0IiBpbiBfMTMzKT9m",
				"dW5jdGlvbigpe3JldHVybiArXzExZC5nZXQoXzEzMy5ub2RlLCJvcGFjaXR5Iil8fDA7fTpfMTMzLnN0",
				"YXJ0O18xMzQuZW5kPV8xMzMuZW5kO3ZhciBhbmltPWRvam8uYW5pbWF0ZVByb3BlcnR5KF8xMzMpO18x",
				"MWMuY29ubmVjdChhbmltLCJiZWZvcmVCZWdpbiIsbGFuZy5wYXJ0aWFsKF8xMzIsXzEzMy5ub2RlKSk7",
				"cmV0dXJuIGFuaW07fTtkb2pvLmZhZGVJbj1mdW5jdGlvbihhcmdzKXtyZXR1cm4gZG9qby5fZmFkZShf",
				"MTFlKHtlbmQ6MX0sYXJncykpO307ZG9qby5mYWRlT3V0PWZ1bmN0aW9uKGFyZ3Mpe3JldHVybiBkb2pv",
				"Ll9mYWRlKF8xMWUoe2VuZDowfSxhcmdzKSk7fTtkb2pvLl9kZWZhdWx0RWFzaW5nPWZ1bmN0aW9uKG4p",
				"e3JldHVybiAwLjUrKChNYXRoLnNpbigobisxLjUpKk1hdGguUEkpKS8yKTt9O3ZhciBfMTM1PWZ1bmN0",
				"aW9uKF8xMzYpe3RoaXMuX3Byb3BlcnRpZXM9XzEzNjtmb3IodmFyIHAgaW4gXzEzNil7dmFyIHByb3A9",
				"XzEzNltwXTtpZihwcm9wLnN0YXJ0IGluc3RhbmNlb2YgXzExYil7cHJvcC50ZW1wQ29sb3I9bmV3IF8x",
				"MWIoKTt9fX07XzEzNS5wcm90b3R5cGUuZ2V0VmFsdWU9ZnVuY3Rpb24ocil7dmFyIHJldD17fTtmb3Io",
				"dmFyIHAgaW4gdGhpcy5fcHJvcGVydGllcyl7dmFyIHByb3A9dGhpcy5fcHJvcGVydGllc1twXSxfMTM3",
				"PXByb3Auc3RhcnQ7aWYoXzEzNyBpbnN0YW5jZW9mIF8xMWIpe3JldFtwXT1fMTFiLmJsZW5kQ29sb3Jz",
				"KF8xMzcscHJvcC5lbmQscixwcm9wLnRlbXBDb2xvcikudG9Dc3MoKTt9ZWxzZXtpZighbGFuZy5pc0Fy",
				"cmF5KF8xMzcpKXtyZXRbcF09KChwcm9wLmVuZC1fMTM3KSpyKStfMTM3KyhwIT0ib3BhY2l0eSI/cHJv",
				"cC51bml0c3x8InB4IjowKTt9fX1yZXR1cm4gcmV0O307ZG9qby5hbmltYXRlUHJvcGVydHk9ZnVuY3Rp",
				"b24oYXJncyl7dmFyIG49YXJncy5ub2RlPWRvbS5ieUlkKGFyZ3Mubm9kZSk7aWYoIWFyZ3MuZWFzaW5n",
				"KXthcmdzLmVhc2luZz1kb2pvLl9kZWZhdWx0RWFzaW5nO312YXIgYW5pbT1uZXcgZG9qby5BbmltYXRp",
				"b24oYXJncyk7XzExYy5jb25uZWN0KGFuaW0sImJlZm9yZUJlZ2luIixhbmltLGZ1bmN0aW9uKCl7dmFy",
				"IHBtPXt9O2Zvcih2YXIgcCBpbiB0aGlzLnByb3BlcnRpZXMpe2lmKHA9PSJ3aWR0aCJ8fHA9PSJoZWln",
				"aHQiKXt0aGlzLm5vZGUuZGlzcGxheT0iYmxvY2siO312YXIgcHJvcD10aGlzLnByb3BlcnRpZXNbcF07",
				"aWYobGFuZy5pc0Z1bmN0aW9uKHByb3ApKXtwcm9wPXByb3Aobik7fXByb3A9cG1bcF09XzExZSh7fSwo",
				"bGFuZy5pc09iamVjdChwcm9wKT9wcm9wOntlbmQ6cHJvcH0pKTtpZihsYW5nLmlzRnVuY3Rpb24ocHJv",
				"cC5zdGFydCkpe3Byb3Auc3RhcnQ9cHJvcC5zdGFydChuKTt9aWYobGFuZy5pc0Z1bmN0aW9uKHByb3Au",
				"ZW5kKSl7cHJvcC5lbmQ9cHJvcC5lbmQobik7fXZhciBfMTM4PShwLnRvTG93ZXJDYXNlKCkuaW5kZXhP",
				"ZigiY29sb3IiKT49MCk7ZnVuY3Rpb24gXzEzOShub2RlLHApe3ZhciB2PXtoZWlnaHQ6bm9kZS5vZmZz",
				"ZXRIZWlnaHQsd2lkdGg6bm9kZS5vZmZzZXRXaWR0aH1bcF07aWYodiE9PXVuZGVmaW5lZCl7cmV0dXJu",
				"IHY7fXY9XzExZC5nZXQobm9kZSxwKTtyZXR1cm4gKHA9PSJvcGFjaXR5Iik/K3Y6KF8xMzg/djpwYXJz",
				"ZUZsb2F0KHYpKTt9O2lmKCEoImVuZCIgaW4gcHJvcCkpe3Byb3AuZW5kPV8xMzkobixwKTt9ZWxzZXtp",
				"ZighKCJzdGFydCIgaW4gcHJvcCkpe3Byb3Auc3RhcnQ9XzEzOShuLHApO319aWYoXzEzOCl7cHJvcC5z",
				"dGFydD1uZXcgXzExYihwcm9wLnN0YXJ0KTtwcm9wLmVuZD1uZXcgXzExYihwcm9wLmVuZCk7fWVsc2V7",
				"cHJvcC5zdGFydD0ocD09Im9wYWNpdHkiKT8rcHJvcC5zdGFydDpwYXJzZUZsb2F0KHByb3Auc3RhcnQp",
				"O319dGhpcy5jdXJ2ZT1uZXcgXzEzNShwbSk7fSk7XzExYy5jb25uZWN0KGFuaW0sIm9uQW5pbWF0ZSIs",
				"bGFuZy5oaXRjaChfMTFkLCJzZXQiLGFuaW0ubm9kZSkpO3JldHVybiBhbmltO307ZG9qby5hbmltPWZ1",
				"bmN0aW9uKG5vZGUsXzEzYSxfMTNiLF8xM2MsXzEzZCxfMTNlKXtyZXR1cm4gZG9qby5hbmltYXRlUHJv",
				"cGVydHkoe25vZGU6bm9kZSxkdXJhdGlvbjpfMTNifHxkb2pvLkFuaW1hdGlvbi5wcm90b3R5cGUuZHVy",
				"YXRpb24scHJvcGVydGllczpfMTNhLGVhc2luZzpfMTNjLG9uRW5kOl8xM2R9KS5wbGF5KF8xM2V8fDAp",
				"O307cmV0dXJuIHtfTGluZTpkb2pvLl9MaW5lLEFuaW1hdGlvbjpkb2pvLkFuaW1hdGlvbixfZmFkZTpk",
				"b2pvLl9mYWRlLGZhZGVJbjpkb2pvLmZhZGVJbixmYWRlT3V0OmRvam8uZmFkZU91dCxfZGVmYXVsdEVh",
				"c2luZzpkb2pvLl9kZWZhdWx0RWFzaW5nLGFuaW1hdGVQcm9wZXJ0eTpkb2pvLmFuaW1hdGVQcm9wZXJ0",
				"eSxhbmltOmRvam8uYW5pbX07fSk7fSwiZG9qby9kb20tZm9ybSI6ZnVuY3Rpb24oKXtkZWZpbmUoImRv",
				"am8vZG9tLWZvcm0iLFsiLi9fYmFzZS9sYW5nIiwiLi9kb20iLCIuL2lvLXF1ZXJ5IiwiLi9qc29uIl0s",
				"ZnVuY3Rpb24obGFuZyxkb20saW9xLGpzb24pe2Z1bmN0aW9uIF8xM2Yob2JqLG5hbWUsXzE0MCl7aWYo",
				"XzE0MD09PW51bGwpe3JldHVybjt9dmFyIHZhbD1vYmpbbmFtZV07aWYodHlwZW9mIHZhbD09InN0cmlu",
				"ZyIpe29ialtuYW1lXT1bdmFsLF8xNDBdO31lbHNle2lmKGxhbmcuaXNBcnJheSh2YWwpKXt2YWwucHVz",
				"aChfMTQwKTt9ZWxzZXtvYmpbbmFtZV09XzE0MDt9fX07dmFyIF8xNDE9ImZpbGV8c3VibWl0fGltYWdl",
				"fHJlc2V0fGJ1dHRvbiI7dmFyIGZvcm09e2ZpZWxkVG9PYmplY3Q6ZnVuY3Rpb24gZmllbGRUb09iamVj",
				"dChfMTQyKXt2YXIgcmV0PW51bGw7XzE0Mj1kb20uYnlJZChfMTQyKTtpZihfMTQyKXt2YXIgXzE0Mz1f",
				"MTQyLm5hbWUsdHlwZT0oXzE0Mi50eXBlfHwiIikudG9Mb3dlckNhc2UoKTtpZihfMTQzJiZ0eXBlJiYh",
				"XzE0Mi5kaXNhYmxlZCl7aWYodHlwZT09InJhZGlvInx8dHlwZT09ImNoZWNrYm94Iil7aWYoXzE0Mi5j",
				"aGVja2VkKXtyZXQ9XzE0Mi52YWx1ZTt9fWVsc2V7aWYoXzE0Mi5tdWx0aXBsZSl7cmV0PVtdO3ZhciBf",
				"MTQ0PVtfMTQyLmZpcnN0Q2hpbGRdO3doaWxlKF8xNDQubGVuZ3RoKXtmb3IodmFyIG5vZGU9XzE0NC5w",
				"b3AoKTtub2RlO25vZGU9bm9kZS5uZXh0U2libGluZyl7aWYobm9kZS5ub2RlVHlwZT09MSYmbm9kZS50",
				"YWdOYW1lLnRvTG93ZXJDYXNlKCk9PSJvcHRpb24iKXtpZihub2RlLnNlbGVjdGVkKXtyZXQucHVzaChu",
				"b2RlLnZhbHVlKTt9fWVsc2V7aWYobm9kZS5uZXh0U2libGluZyl7XzE0NC5wdXNoKG5vZGUubmV4dFNp",
				"YmxpbmcpO31pZihub2RlLmZpcnN0Q2hpbGQpe18xNDQucHVzaChub2RlLmZpcnN0Q2hpbGQpO31icmVh",
				"azt9fX19ZWxzZXtyZXQ9XzE0Mi52YWx1ZTt9fX19cmV0dXJuIHJldDt9LHRvT2JqZWN0OmZ1bmN0aW9u",
				"IGZvcm1Ub09iamVjdChfMTQ1KXt2YXIgcmV0PXt9LF8xNDY9ZG9tLmJ5SWQoXzE0NSkuZWxlbWVudHM7",
				"Zm9yKHZhciBpPTAsbD1fMTQ2Lmxlbmd0aDtpPGw7KytpKXt2YXIgaXRlbT1fMTQ2W2ldLF8xNDc9aXRl",
				"bS5uYW1lLHR5cGU9KGl0ZW0udHlwZXx8IiIpLnRvTG93ZXJDYXNlKCk7aWYoXzE0NyYmdHlwZSYmXzE0",
				"MS5pbmRleE9mKHR5cGUpPDAmJiFpdGVtLmRpc2FibGVkKXtfMTNmKHJldCxfMTQ3LGZvcm0uZmllbGRU",
				"b09iamVjdChpdGVtKSk7aWYodHlwZT09ImltYWdlIil7cmV0W18xNDcrIi54Il09cmV0W18xNDcrIi55",
				"Il09cmV0W18xNDddLng9cmV0W18xNDddLnk9MDt9fX1yZXR1cm4gcmV0O30sdG9RdWVyeTpmdW5jdGlv",
				"biBmb3JtVG9RdWVyeShfMTQ4KXtyZXR1cm4gaW9xLm9iamVjdFRvUXVlcnkoZm9ybS50b09iamVjdChf",
				"MTQ4KSk7fSx0b0pzb246ZnVuY3Rpb24gZm9ybVRvSnNvbihfMTQ5LF8xNGEpe3JldHVybiBqc29uLnN0",
				"cmluZ2lmeShmb3JtLnRvT2JqZWN0KF8xNDkpLG51bGwsXzE0YT80OjApO319O3JldHVybiBmb3JtO30p",
				"O30sImRvam8vaTE4biI6ZnVuY3Rpb24oKXtkZWZpbmUoWyIuL19iYXNlL2tlcm5lbCIsInJlcXVpcmUi",
				"LCIuL2hhcyIsIi4vX2Jhc2UvYXJyYXkiLCIuL19iYXNlL2NvbmZpZyIsIi4vX2Jhc2UvbGFuZyIsIi4v",
				"X2Jhc2UveGhyIl0sZnVuY3Rpb24oZG9qbyxfMTRiLGhhcyxfMTRjLF8xNGQsbGFuZyx4aHIpe3ZhciBf",
				"MTRlPWRvam8uaTE4bj17fSxfMTRmPS8oXi4qKF58XC8pbmxzKShcL3wkKShbXlwvXSopXC8/KFteXC9d",
				"KikvLF8xNTA9ZnVuY3Rpb24ocm9vdCxfMTUxLF8xNTIsXzE1Myl7Zm9yKHZhciBfMTU0PVtfMTUyK18x",
				"NTNdLF8xNTU9XzE1MS5zcGxpdCgiLSIpLF8xNTY9IiIsaT0wO2k8XzE1NS5sZW5ndGg7aSsrKXtfMTU2",
				"Kz0oXzE1Nj8iLSI6IiIpK18xNTVbaV07aWYoIXJvb3R8fHJvb3RbXzE1Nl0pe18xNTQucHVzaChfMTUy",
				"K18xNTYrIi8iK18xNTMpO319cmV0dXJuIF8xNTQ7fSxfMTU3PXt9LF8xNTg9ZG9qby5nZXRMMTBuTmFt",
				"ZT1mdW5jdGlvbihfMTU5LF8xNWEsXzE1Yil7XzE1Yj1fMTViP18xNWIudG9Mb3dlckNhc2UoKTpkb2pv",
				"LmxvY2FsZTtfMTU5PSJkb2pvL2kxOG4hIitfMTU5LnJlcGxhY2UoL1wuL2csIi8iKTtfMTVhPV8xNWEu",
				"cmVwbGFjZSgvXC4vZywiLyIpO3JldHVybiAoL3Jvb3QvaS50ZXN0KF8xNWIpKT8oXzE1OSsiL25scy8i",
				"K18xNWEpOihfMTU5KyIvbmxzLyIrXzE1YisiLyIrXzE1YSk7fSxfMTVjPWZ1bmN0aW9uKF8xNWQsXzE1",
				"ZSxfMTVmLF8xNjAsXzE2MSxsb2FkKXtfMTVkKFtfMTVlXSxmdW5jdGlvbihyb290KXt2YXIgXzE2Mj1f",
				"MTU3W18xNWUrIi8iXT1sYW5nLmNsb25lKHJvb3Qucm9vdCksXzE2Mz1fMTUwKCFyb290Ll92MXgmJnJv",
				"b3QsXzE2MSxfMTVmLF8xNjApO18xNWQoXzE2MyxmdW5jdGlvbigpe2Zvcih2YXIgaT0xO2k8XzE2My5s",
				"ZW5ndGg7aSsrKXtfMTU3W18xNjNbaV1dPV8xNjI9bGFuZy5taXhpbihsYW5nLmNsb25lKF8xNjIpLGFy",
				"Z3VtZW50c1tpXSk7fXZhciBfMTY0PV8xNWUrIi8iK18xNjE7XzE1N1tfMTY0XT1fMTYyO2xvYWQmJmxv",
				"YWQobGFuZy5kZWxlZ2F0ZShfMTYyKSk7fSk7fSk7fSxfMTY1PWZ1bmN0aW9uKGlkLF8xNjYpe3ZhciBf",
				"MTY3PV8xNGYuZXhlYyhpZCksXzE2OD1fMTY3WzFdO3JldHVybiAvXlwuLy50ZXN0KF8xNjgpP18xNjYo",
				"XzE2OCkrIi8iK2lkLnN1YnN0cmluZyhfMTY4Lmxlbmd0aCk6aWQ7fSxfMTY5PWZ1bmN0aW9uKCl7fSxs",
				"b2FkPWZ1bmN0aW9uKGlkLF8xNmEsbG9hZCl7dmFyIF8xNmI9XzE0Zi5leGVjKGlkKSxfMTZjPV8xNmJb",
				"MV0rIi8iLF8xNmQ9XzE2Yls1XXx8XzE2Yls0XSxfMTZlPV8xNmMrXzE2ZCxfMTZmPShfMTZiWzVdJiZf",
				"MTZiWzRdKSxfMTcwPV8xNmZ8fGRvam8ubG9jYWxlLF8xNzE9XzE2ZSsiLyIrXzE3MDtpZihfMTZmKXtf",
				"MTY5KF8xNzEpO2lmKF8xNTdbXzE3MV0pe2xvYWQoXzE1N1tfMTcxXSk7fWVsc2V7XzE1YyhfMTZhLF8x",
				"NmUsXzE2YyxfMTZkLF8xNzAsbG9hZCk7fXJldHVybjt9dmFyIF8xNzI9XzE0ZC5leHRyYUxvY2FsZXx8",
				"W107XzE3Mj1sYW5nLmlzQXJyYXkoXzE3Mik/XzE3MjpbXzE3Ml07XzE3Mi5wdXNoKF8xNzApO3ZhciBf",
				"MTczPV8xNzIubGVuZ3RoLF8xNzQ7XzE0Yy5mb3JFYWNoKF8xNzIsZnVuY3Rpb24oXzE3NSl7XzE1Yyhf",
				"MTZhLF8xNmUsXzE2YyxfMTZkLF8xNzUsZnVuY3Rpb24oXzE3Nil7aWYoXzE3NT09XzE3MCl7XzE3ND1f",
				"MTc2O31pZighLS1fMTczKXtsb2FkKF8xNzQpO319KTt9KTt9O2lmKGhhcygiZG9qby11bml0LXRlc3Rz",
				"Iikpe3ZhciBfMTc3PV8xNGUudW5pdFRlc3RzPVtdO310cnVlfHxoYXMuYWRkKCJkb2pvLXYxeC1pMThu",
				"LUFwaSIsMSk7aWYoMSl7dmFyIF8xNzg9e30sXzE3OT1uZXcgRnVuY3Rpb24oImJ1bmRsZSwgX19ldmFs",
				"RXJyb3IiLCJ2YXIgX19hbWRSZXN1bHQsIGRlZmluZSA9IGZ1bmN0aW9uKHgpe19fYW1kUmVzdWx0PSB4",
				"O307IisicmV0dXJuIFsoZnVuY3Rpb24oKXsiKyJ0cnl7ZXZhbChhcmd1bWVudHNbMF0pO31jYXRjaChl",
				"KXt9IisiaWYoX19hbWRSZXN1bHQpcmV0dXJuIDA7IisidHJ5e3JldHVybiBldmFsKCcoJythcmd1bWVu",
				"dHNbMF0rJyknKTt9IisiY2F0Y2goZSl7X19ldmFsRXJyb3IuZSA9IGU7IHJldHVybiBfX2V2YWxFcnJv",
				"cjt9IisifSkoYXJndW1lbnRzWzBdKSAsIF9fYW1kUmVzdWx0XTsiKSxfMTdhPWZ1bmN0aW9uKHVybCxf",
				"MTdiLF8xN2Mpe2lmKF8xN2I9PT1fMTc4KXtjb25zb2xlLmVycm9yKCJmYWlsZWQgdG8gZXZhbHVhdGUg",
				"aTE4biBidW5kbGU7IHVybD0iK3VybCxfMTc4LmUpO3JldHVybiB7fTt9cmV0dXJuIF8xN2I/KC9ubHNc",
				"L1teXC9dK1wvW15cL10rJC8udGVzdCh1cmwpP18xN2I6e3Jvb3Q6XzE3YixfdjF4OjF9KTpfMTdjO30s",
				"XzE3ZD1mdW5jdGlvbihkZXBzLF8xN2Upe3ZhciBfMTdmPVtdO18xNGMuZm9yRWFjaChkZXBzLGZ1bmN0",
				"aW9uKG1pZCl7dmFyIHVybD1fMTRiLnRvVXJsKG1pZCsiLmpzIik7aWYoXzE1N1t1cmxdKXtfMTdmLnB1",
				"c2goXzE1N1t1cmxdKTt9ZWxzZXt0cnl7dmFyIF8xODA9XzE0YihtaWQpO2lmKF8xODApe18xN2YucHVz",
				"aChfMTgwKTtyZXR1cm47fX1jYXRjaChlKXt9eGhyLmdldCh7dXJsOnVybCxzeW5jOnRydWUsbG9hZDpm",
				"dW5jdGlvbih0ZXh0KXt2YXIgXzE4MT1fMTc5KHRleHQsXzE3OCk7XzE3Zi5wdXNoKF8xNTdbdXJsXT1f",
				"MTdhKHVybCxfMTgxWzBdLF8xODFbMV0pKTt9LGVycm9yOmZ1bmN0aW9uKCl7XzE3Zi5wdXNoKF8xNTdb",
				"dXJsXT17fSk7fX0pO319KTtfMTdlJiZfMTdlLmFwcGx5KG51bGwsXzE3Zik7fSxfMTgyPV8xNGUubm9y",
				"bWFsaXplTG9jYWxlPWZ1bmN0aW9uKF8xODMpe3ZhciBfMTg0PV8xODM/XzE4My50b0xvd2VyQ2FzZSgp",
				"OmRvam8ubG9jYWxlO2lmKF8xODQ9PSJyb290Iil7XzE4ND0iUk9PVCI7fXJldHVybiBfMTg0O30sXzE4",
				"NT1mdW5jdGlvbihfMTg2LGZ1bmMpe3ZhciBfMTg3PV8xODYuc3BsaXQoIi0iKTt3aGlsZShfMTg3Lmxl",
				"bmd0aCl7aWYoZnVuYyhfMTg3LmpvaW4oIi0iKSkpe3JldHVybiB0cnVlO31fMTg3LnBvcCgpO31yZXR1",
				"cm4gZnVuYygiUk9PVCIpO307XzE2OT1mdW5jdGlvbihfMTg4KXtmb3IodmFyIF8xODk9XzE4OC5zcGxp",
				"dCgiLyIpLF8xOGE9ZG9qby5nbG9iYWxbXzE4OVswXV0saT0xO18xOGEmJmk8XzE4OS5sZW5ndGg7XzE4",
				"YT1fMThhW18xODlbaSsrXV0pe31pZihfMThhKXtfMTU3W18xODhdPV8xOGE7fX07XzE0ZS5nZXRMb2Nh",
				"bGl6YXRpb249ZnVuY3Rpb24oXzE4YixfMThjLF8xOGQpe3ZhciBfMThlLF8xOGY9XzE1OChfMThiLF8x",
				"OGMsXzE4ZCkuc3Vic3RyaW5nKDEwKTtsb2FkKF8xOGYsKDEmJiFfMTRiLmlzWGRVcmwoXzE0Yi50b1Vy",
				"bChfMThmKyIuanMiKSk/XzE3ZDpfMTRiKSxmdW5jdGlvbihfMTkwKXtfMThlPV8xOTA7fSk7cmV0dXJu",
				"IF8xOGU7fTtfMTRlLl9wcmVsb2FkTG9jYWxpemF0aW9ucz1mdW5jdGlvbihfMTkxLF8xOTIpe2Z1bmN0",
				"aW9uIF8xOTMoXzE5NCl7XzE5ND1fMTgyKF8xOTQpO18xODUoXzE5NCxmdW5jdGlvbihsb2Mpe2Zvcih2",
				"YXIgaT0wO2k8XzE5Mi5sZW5ndGg7aSsrKXtpZihfMTkyW2ldPT1sb2Mpe18xN2QoW18xOTEucmVwbGFj",
				"ZSgvXC4vZywiLyIpKyJfIitsb2NdKTtyZXR1cm4gdHJ1ZTt9fXJldHVybiBmYWxzZTt9KTt9O18xOTMo",
				"KTt2YXIgXzE5NT1kb2pvLmNvbmZpZy5leHRyYUxvY2FsZXx8W107Zm9yKHZhciBpPTA7aTxfMTk1Lmxl",
				"bmd0aDtpKyspe18xOTMoXzE5NVtpXSk7fX07aWYoaGFzKCJkb2pvLXVuaXQtdGVzdHMiKSl7XzE3Ny5w",
				"dXNoKGZ1bmN0aW9uKGRvaCl7ZG9oLnJlZ2lzdGVyKCJ0ZXN0cy5pMThuLnVuaXQiLGZ1bmN0aW9uKHQp",
				"e3ZhciBfMTk2O18xOTY9XzE3OSgie3Byb3A6MX0iLF8xNzgpO3QuaXMoe3Byb3A6MX0sXzE5NlswXSk7",
				"dC5pcyh1bmRlZmluZWQsXzE5NlsxXSk7XzE5Nj1fMTc5KCIoe3Byb3A6MX0pIixfMTc4KTt0LmlzKHtw",
				"cm9wOjF9LF8xOTZbMF0pO3QuaXModW5kZWZpbmVkLF8xOTZbMV0pO18xOTY9XzE3OSgieydwcm9wLXgn",
				"OjF9IixfMTc4KTt0LmlzKHsicHJvcC14IjoxfSxfMTk2WzBdKTt0LmlzKHVuZGVmaW5lZCxfMTk2WzFd",
				"KTtfMTk2PV8xNzkoIih7J3Byb3AteCc6MX0pIixfMTc4KTt0LmlzKHsicHJvcC14IjoxfSxfMTk2WzBd",
				"KTt0LmlzKHVuZGVmaW5lZCxfMTk2WzFdKTtfMTk2PV8xNzkoImRlZmluZSh7J3Byb3AteCc6MX0pIixf",
				"MTc4KTt0LmlzKDAsXzE5NlswXSk7dC5pcyh7InByb3AteCI6MX0sXzE5NlsxXSk7XzE5Nj1fMTc5KCJk",
				"ZWZpbmUoeydwcm9wLXgnOjF9KTsiLF8xNzgpO3QuaXMoMCxfMTk2WzBdKTt0LmlzKHsicHJvcC14Ijox",
				"fSxfMTk2WzFdKTtfMTk2PV8xNzkoInRoaXMgaXMgdG90YWwgbm9uc2Vuc2UgYW5kIHNob3VsZCB0aHJv",
				"dyBhbiBlcnJvciIsXzE3OCk7dC5pcyhfMTc4LF8xOTZbMF0pO3QuaXModW5kZWZpbmVkLF8xOTZbMV0p",
				"O3QuaXMoe30sXzE3YSgic29tZS91cmwiLF8xOTZbMF0sXzE5NlsxXSkpO30pO30pO319cmV0dXJuIGxh",
				"bmcubWl4aW4oXzE0ZSx7ZHluYW1pYzp0cnVlLG5vcm1hbGl6ZTpfMTY1LGxvYWQ6bG9hZCxjYWNoZTpm",
				"dW5jdGlvbihtaWQsXzE5Nyl7XzE1N1ttaWRdPV8xOTc7fX0pO30pO30sImRvam8vX2Jhc2UvaHRtbCI6",
				"ZnVuY3Rpb24oKXtkZWZpbmUoWyIuL2tlcm5lbCIsIi4uL2RvbSIsIi4uL2RvbS1zdHlsZSIsIi4uL2Rv",
				"bS1hdHRyIiwiLi4vZG9tLXByb3AiLCIuLi9kb20tY2xhc3MiLCIuLi9kb20tY29uc3RydWN0IiwiLi4v",
				"ZG9tLWdlb21ldHJ5Il0sZnVuY3Rpb24oZG9qbyxkb20sXzE5OCxhdHRyLHByb3AsY2xzLGN0cixnZW9t",
				"KXtkb2pvLmJ5SWQ9ZG9tLmJ5SWQ7ZG9qby5pc0Rlc2NlbmRhbnQ9ZG9tLmlzRGVzY2VuZGFudDtkb2pv",
				"LnNldFNlbGVjdGFibGU9ZG9tLnNldFNlbGVjdGFibGU7ZG9qby5nZXRBdHRyPWF0dHIuZ2V0O2Rvam8u",
				"c2V0QXR0cj1hdHRyLnNldDtkb2pvLmhhc0F0dHI9YXR0ci5oYXM7ZG9qby5yZW1vdmVBdHRyPWF0dHIu",
				"cmVtb3ZlO2Rvam8uZ2V0Tm9kZVByb3A9YXR0ci5nZXROb2RlUHJvcDtkb2pvLmF0dHI9ZnVuY3Rpb24o",
				"bm9kZSxuYW1lLF8xOTkpe2lmKGFyZ3VtZW50cy5sZW5ndGg9PTIpe3JldHVybiBhdHRyW3R5cGVvZiBu",
				"YW1lPT0ic3RyaW5nIj8iZ2V0Ijoic2V0Il0obm9kZSxuYW1lKTt9cmV0dXJuIGF0dHIuc2V0KG5vZGUs",
				"bmFtZSxfMTk5KTt9O2Rvam8uaGFzQ2xhc3M9Y2xzLmNvbnRhaW5zO2Rvam8uYWRkQ2xhc3M9Y2xzLmFk",
				"ZDtkb2pvLnJlbW92ZUNsYXNzPWNscy5yZW1vdmU7ZG9qby50b2dnbGVDbGFzcz1jbHMudG9nZ2xlO2Rv",
				"am8ucmVwbGFjZUNsYXNzPWNscy5yZXBsYWNlO2Rvam8uX3RvRG9tPWRvam8udG9Eb209Y3RyLnRvRG9t",
				"O2Rvam8ucGxhY2U9Y3RyLnBsYWNlO2Rvam8uY3JlYXRlPWN0ci5jcmVhdGU7ZG9qby5lbXB0eT1mdW5j",
				"dGlvbihub2RlKXtjdHIuZW1wdHkobm9kZSk7fTtkb2pvLl9kZXN0cm95RWxlbWVudD1kb2pvLmRlc3Ry",
				"b3k9ZnVuY3Rpb24obm9kZSl7Y3RyLmRlc3Ryb3kobm9kZSk7fTtkb2pvLl9nZXRQYWRFeHRlbnRzPWRv",
				"am8uZ2V0UGFkRXh0ZW50cz1nZW9tLmdldFBhZEV4dGVudHM7ZG9qby5fZ2V0Qm9yZGVyRXh0ZW50cz1k",
				"b2pvLmdldEJvcmRlckV4dGVudHM9Z2VvbS5nZXRCb3JkZXJFeHRlbnRzO2Rvam8uX2dldFBhZEJvcmRl",
				"ckV4dGVudHM9ZG9qby5nZXRQYWRCb3JkZXJFeHRlbnRzPWdlb20uZ2V0UGFkQm9yZGVyRXh0ZW50cztk",
				"b2pvLl9nZXRNYXJnaW5FeHRlbnRzPWRvam8uZ2V0TWFyZ2luRXh0ZW50cz1nZW9tLmdldE1hcmdpbkV4",
				"dGVudHM7ZG9qby5fZ2V0TWFyZ2luU2l6ZT1kb2pvLmdldE1hcmdpblNpemU9Z2VvbS5nZXRNYXJnaW5T",
				"aXplO2Rvam8uX2dldE1hcmdpbkJveD1kb2pvLmdldE1hcmdpbkJveD1nZW9tLmdldE1hcmdpbkJveDtk",
				"b2pvLnNldE1hcmdpbkJveD1nZW9tLnNldE1hcmdpbkJveDtkb2pvLl9nZXRDb250ZW50Qm94PWRvam8u",
				"Z2V0Q29udGVudEJveD1nZW9tLmdldENvbnRlbnRCb3g7ZG9qby5zZXRDb250ZW50U2l6ZT1nZW9tLnNl",
				"dENvbnRlbnRTaXplO2Rvam8uX2lzQm9keUx0cj1kb2pvLmlzQm9keUx0cj1nZW9tLmlzQm9keUx0cjtk",
				"b2pvLl9kb2NTY3JvbGw9ZG9qby5kb2NTY3JvbGw9Z2VvbS5kb2NTY3JvbGw7ZG9qby5fZ2V0SWVEb2N1",
				"bWVudEVsZW1lbnRPZmZzZXQ9ZG9qby5nZXRJZURvY3VtZW50RWxlbWVudE9mZnNldD1nZW9tLmdldEll",
				"RG9jdW1lbnRFbGVtZW50T2Zmc2V0O2Rvam8uX2ZpeEllQmlEaVNjcm9sbExlZnQ9ZG9qby5maXhJZUJp",
				"RGlTY3JvbGxMZWZ0PWdlb20uZml4SWVCaURpU2Nyb2xsTGVmdDtkb2pvLnBvc2l0aW9uPWdlb20ucG9z",
				"aXRpb247ZG9qby5tYXJnaW5Cb3g9ZnVuY3Rpb24gbWFyZ2luQm94KG5vZGUsYm94KXtyZXR1cm4gYm94",
				"P2dlb20uc2V0TWFyZ2luQm94KG5vZGUsYm94KTpnZW9tLmdldE1hcmdpbkJveChub2RlKTt9O2Rvam8u",
				"Y29udGVudEJveD1mdW5jdGlvbiBjb250ZW50Qm94KG5vZGUsYm94KXtyZXR1cm4gYm94P2dlb20uc2V0",
				"Q29udGVudFNpemUobm9kZSxib3gpOmdlb20uZ2V0Q29udGVudEJveChub2RlKTt9O2Rvam8uY29vcmRz",
				"PWZ1bmN0aW9uKG5vZGUsXzE5YSl7ZG9qby5kZXByZWNhdGVkKCJkb2pvLmNvb3JkcygpIiwiVXNlIGRv",
				"am8ucG9zaXRpb24oKSBvciBkb2pvLm1hcmdpbkJveCgpLiIpO25vZGU9ZG9tLmJ5SWQobm9kZSk7dmFy",
				"IHM9XzE5OC5nZXRDb21wdXRlZFN0eWxlKG5vZGUpLG1iPWdlb20uZ2V0TWFyZ2luQm94KG5vZGUscyk7",
				"dmFyIGFicz1nZW9tLnBvc2l0aW9uKG5vZGUsXzE5YSk7bWIueD1hYnMueDttYi55PWFicy55O3JldHVy",
				"biBtYjt9O2Rvam8uZ2V0UHJvcD1wcm9wLmdldDtkb2pvLnNldFByb3A9cHJvcC5zZXQ7ZG9qby5wcm9w",
				"PWZ1bmN0aW9uKG5vZGUsbmFtZSxfMTliKXtpZihhcmd1bWVudHMubGVuZ3RoPT0yKXtyZXR1cm4gcHJv",
				"cFt0eXBlb2YgbmFtZT09InN0cmluZyI/ImdldCI6InNldCJdKG5vZGUsbmFtZSk7fXJldHVybiBwcm9w",
				"LnNldChub2RlLG5hbWUsXzE5Yik7fTtkb2pvLmdldFN0eWxlPV8xOTguZ2V0O2Rvam8uc2V0U3R5bGU9",
				"XzE5OC5zZXQ7ZG9qby5nZXRDb21wdXRlZFN0eWxlPV8xOTguZ2V0Q29tcHV0ZWRTdHlsZTtkb2pvLl9f",
				"dG9QaXhlbFZhbHVlPWRvam8udG9QaXhlbFZhbHVlPV8xOTgudG9QaXhlbFZhbHVlO2Rvam8uc3R5bGU9",
				"ZnVuY3Rpb24obm9kZSxuYW1lLF8xOWMpe3N3aXRjaChhcmd1bWVudHMubGVuZ3RoKXtjYXNlIDE6cmV0",
				"dXJuIF8xOTguZ2V0KG5vZGUpO2Nhc2UgMjpyZXR1cm4gXzE5OFt0eXBlb2YgbmFtZT09InN0cmluZyI/",
				"ImdldCI6InNldCJdKG5vZGUsbmFtZSk7fXJldHVybiBfMTk4LnNldChub2RlLG5hbWUsXzE5Yyk7fTty",
				"ZXR1cm4gZG9qbzt9KTt9LCJkb2pvL19iYXNlL2tlcm5lbCI6ZnVuY3Rpb24oKXtkZWZpbmUoWyIuLi9o",
				"YXMiLCIuL2NvbmZpZyIsInJlcXVpcmUiLCJtb2R1bGUiXSxmdW5jdGlvbihoYXMsXzE5ZCxfMTllLF8x",
				"OWYpe3ZhciBpLHAsXzFhMD17fSxfMWExPXt9LGRvam89e2NvbmZpZzpfMTlkLGdsb2JhbDp0aGlzLGRp",
				"aml0Ol8xYTAsZG9qb3g6XzFhMX07dmFyIF8xYTI9e2Rvam86WyJkb2pvIixkb2pvXSxkaWppdDpbImRp",
				"aml0IixfMWEwXSxkb2pveDpbImRvam94IixfMWExXX0sXzFhMz0oXzE5ZS5wYWNrcyYmXzE5ZS5wYWNr",
				"c1tfMTlmLmlkLm1hdGNoKC9bXlwvXSsvKVswXV0ucGFja2FnZU1hcCl8fHt9LGl0ZW07Zm9yKHAgaW4g",
				"XzFhMyl7aWYoXzFhMltwXSl7XzFhMltwXVswXT1fMWEzW3BdO31lbHNle18xYTJbcF09W18xYTNbcF0s",
				"e31dO319Zm9yKHAgaW4gXzFhMil7aXRlbT1fMWEyW3BdO2l0ZW1bMV0uX3Njb3BlTmFtZT1pdGVtWzBd",
				"O2lmKCFfMTlkLm5vR2xvYmFscyl7dGhpc1tpdGVtWzBdXT1pdGVtWzFdO319ZG9qby5zY29wZU1hcD1f",
				"MWEyO2Rvam8uYmFzZVVybD1kb2pvLmNvbmZpZy5iYXNlVXJsPV8xOWUuYmFzZVVybDtkb2pvLmlzQXN5",
				"bmM9ITF8fF8xOWUuYXN5bmM7ZG9qby5sb2NhbGU9XzE5ZC5sb2NhbGU7dmFyIHJldj0iJFJldjogMjc5",
				"MTMgJCIubWF0Y2goL1xkKy8pO2Rvam8udmVyc2lvbj17bWFqb3I6MSxtaW5vcjo3LHBhdGNoOjIsZmxh",
				"ZzoiIixyZXZpc2lvbjpyZXY/K3JldlswXTpOYU4sdG9TdHJpbmc6ZnVuY3Rpb24oKXt2YXIgdj1kb2pv",
				"LnZlcnNpb247cmV0dXJuIHYubWFqb3IrIi4iK3YubWlub3IrIi4iK3YucGF0Y2grdi5mbGFnKyIgKCIr",
				"di5yZXZpc2lvbisiKSI7fX07dHJ1ZXx8aGFzLmFkZCgiZXh0ZW5kLWRvam8iLDEpO2Rvam8uZXZhbD1m",
				"dW5jdGlvbihfMWE0KXt9OyhGdW5jdGlvbigiZCIsImQuZXZhbCA9IGZ1bmN0aW9uKCl7cmV0dXJuIGQu",
				"Z2xvYmFsLmV2YWwgPyBkLmdsb2JhbC5ldmFsKGFyZ3VtZW50c1swXSkgOiBldmFsKGFyZ3VtZW50c1sw",
				"XSk7fSIpKShkb2pvKTtpZigwKXtkb2pvLmV4aXQ9ZnVuY3Rpb24oXzFhNSl7cXVpdChfMWE1KTt9O31l",
				"bHNle2Rvam8uZXhpdD1mdW5jdGlvbigpe307fXRydWV8fGhhcy5hZGQoImRvam8tZ3VhcmFudGVlLWNv",
				"bnNvbGUiLDEpO2lmKDEpe3R5cGVvZiBjb25zb2xlIT0idW5kZWZpbmVkInx8KGNvbnNvbGU9e30pO3Zh",
				"ciBjbj1bImFzc2VydCIsImNvdW50IiwiZGVidWciLCJkaXIiLCJkaXJ4bWwiLCJlcnJvciIsImdyb3Vw",
				"IiwiZ3JvdXBFbmQiLCJpbmZvIiwicHJvZmlsZSIsInByb2ZpbGVFbmQiLCJ0aW1lIiwidGltZUVuZCIs",
				"InRyYWNlIiwid2FybiIsImxvZyJdO3ZhciB0bjtpPTA7d2hpbGUoKHRuPWNuW2krK10pKXtpZighY29u",
				"c29sZVt0bl0peyhmdW5jdGlvbigpe3ZhciB0Y249dG4rIiI7Y29uc29sZVt0Y25dPSgibG9nIiBpbiBj",
				"b25zb2xlKT9mdW5jdGlvbigpe3ZhciBhPUFycmF5LmFwcGx5KHt9LGFyZ3VtZW50cyk7YS51bnNoaWZ0",
				"KHRjbisiOiIpO2NvbnNvbGVbImxvZyJdKGEuam9pbigiICIpKTt9OmZ1bmN0aW9uKCl7fTtjb25zb2xl",
				"W3Rjbl0uX2Zha2U9dHJ1ZTt9KSgpO319fWhhcy5hZGQoImRvam8tZGVidWctbWVzc2FnZXMiLCEhXzE5",
				"ZC5pc0RlYnVnKTtpZihoYXMoImRvam8tZGVidWctbWVzc2FnZXMiKSl7ZG9qby5kZXByZWNhdGVkPWZ1",
				"bmN0aW9uKF8xYTYsXzFhNyxfMWE4KXt2YXIgXzFhOT0iREVQUkVDQVRFRDogIitfMWE2O2lmKF8xYTcp",
				"e18xYTkrPSIgIitfMWE3O31pZihfMWE4KXtfMWE5Kz0iIC0tIHdpbGwgYmUgcmVtb3ZlZCBpbiB2ZXJz",
				"aW9uOiAiK18xYTg7fWNvbnNvbGUud2FybihfMWE5KTt9O2Rvam8uZXhwZXJpbWVudGFsPWZ1bmN0aW9u",
				"KF8xYWEsXzFhYil7dmFyIF8xYWM9IkVYUEVSSU1FTlRBTDogIitfMWFhKyIgLS0gQVBJcyBzdWJqZWN0",
				"IHRvIGNoYW5nZSB3aXRob3V0IG5vdGljZS4iO2lmKF8xYWIpe18xYWMrPSIgIitfMWFiO31jb25zb2xl",
				"Lndhcm4oXzFhYyk7fTt9ZWxzZXtkb2pvLmRlcHJlY2F0ZWQ9ZG9qby5leHBlcmltZW50YWw9ZnVuY3Rp",
				"b24oKXt9O310cnVlfHxoYXMuYWRkKCJkb2pvLW1vZHVsZVBhdGhzIiwxKTtpZigxKXtpZihfMTlkLm1v",
				"ZHVsZVBhdGhzKXtkb2pvLmRlcHJlY2F0ZWQoImRvam8ubW9kdWxlUGF0aHMiLCJ1c2UgcGF0aHMgY29u",
				"ZmlndXJhdGlvbiIpO3ZhciBfMWFkPXt9O2ZvcihwIGluIF8xOWQubW9kdWxlUGF0aHMpe18xYWRbcC5y",
				"ZXBsYWNlKC9cLi9nLCIvIildPV8xOWQubW9kdWxlUGF0aHNbcF07fV8xOWUoe3BhdGhzOl8xYWR9KTt9",
				"fXRydWV8fGhhcy5hZGQoImRvam8tbW9kdWxlVXJsIiwxKTtpZigxKXtkb2pvLm1vZHVsZVVybD1mdW5j",
				"dGlvbihfMWFlLHVybCl7ZG9qby5kZXByZWNhdGVkKCJkb2pvLm1vZHVsZVVybCgpIiwidXNlIHJlcXVp",
				"cmUudG9VcmwiLCIyLjAiKTt2YXIgXzFhZj1udWxsO2lmKF8xYWUpe18xYWY9XzE5ZS50b1VybChfMWFl",
				"LnJlcGxhY2UoL1wuL2csIi8iKSsodXJsPygiLyIrdXJsKToiIikrIi8qLioiKS5yZXBsYWNlKC9cL1wq",
				"XC5cKi8sIiIpKyh1cmw/IiI6Ii8iKTt9cmV0dXJuIF8xYWY7fTt9ZG9qby5faGFzUmVzb3VyY2U9e307",
				"cmV0dXJuIGRvam87fSk7fSwiZG9qby9pby1xdWVyeSI6ZnVuY3Rpb24oKXtkZWZpbmUoWyIuL19iYXNl",
				"L2xhbmciXSxmdW5jdGlvbihsYW5nKXt2YXIgXzFiMD17fTtmdW5jdGlvbiBfMWIxKG1hcCl7dmFyIGVu",
				"Yz1lbmNvZGVVUklDb21wb25lbnQsXzFiMj1bXTtmb3IodmFyIG5hbWUgaW4gbWFwKXt2YXIgXzFiMz1t",
				"YXBbbmFtZV07aWYoXzFiMyE9XzFiMFtuYW1lXSl7dmFyIF8xYjQ9ZW5jKG5hbWUpKyI9IjtpZihsYW5n",
				"LmlzQXJyYXkoXzFiMykpe2Zvcih2YXIgaT0wLGw9XzFiMy5sZW5ndGg7aTxsOysraSl7XzFiMi5wdXNo",
				"KF8xYjQrZW5jKF8xYjNbaV0pKTt9fWVsc2V7XzFiMi5wdXNoKF8xYjQrZW5jKF8xYjMpKTt9fX1yZXR1",
				"cm4gXzFiMi5qb2luKCImIik7fTtmdW5jdGlvbiBfMWI1KHN0cil7dmFyIGRlYz1kZWNvZGVVUklDb21w",
				"b25lbnQscXA9c3RyLnNwbGl0KCImIikscmV0PXt9LG5hbWUsdmFsO2Zvcih2YXIgaT0wLGw9cXAubGVu",
				"Z3RoLGl0ZW07aTxsOysraSl7aXRlbT1xcFtpXTtpZihpdGVtLmxlbmd0aCl7dmFyIHM9aXRlbS5pbmRl",
				"eE9mKCI9Iik7aWYoczwwKXtuYW1lPWRlYyhpdGVtKTt2YWw9IiI7fWVsc2V7bmFtZT1kZWMoaXRlbS5z",
				"bGljZSgwLHMpKTt2YWw9ZGVjKGl0ZW0uc2xpY2UocysxKSk7fWlmKHR5cGVvZiByZXRbbmFtZV09PSJz",
				"dHJpbmciKXtyZXRbbmFtZV09W3JldFtuYW1lXV07fWlmKGxhbmcuaXNBcnJheShyZXRbbmFtZV0pKXty",
				"ZXRbbmFtZV0ucHVzaCh2YWwpO31lbHNle3JldFtuYW1lXT12YWw7fX19cmV0dXJuIHJldDt9O3JldHVy",
				"biB7b2JqZWN0VG9RdWVyeTpfMWIxLHF1ZXJ5VG9PYmplY3Q6XzFiNX07fSk7fSwiZG9qby9fYmFzZS9E",
				"ZWZlcnJlZCI6ZnVuY3Rpb24oKXtkZWZpbmUoImRvam8vX2Jhc2UvRGVmZXJyZWQiLFsiLi9rZXJuZWwi",
				"LCIuL2xhbmciXSxmdW5jdGlvbihkb2pvLGxhbmcpe3ZhciBfMWI2PWZ1bmN0aW9uKCl7fTt2YXIgXzFi",
				"Nz1PYmplY3QuZnJlZXplfHxmdW5jdGlvbigpe307ZG9qby5EZWZlcnJlZD1mdW5jdGlvbihfMWI4KXt2",
				"YXIgXzFiOSxfMWJhLF8xYmIsaGVhZCxfMWJjO3ZhciBfMWJkPSh0aGlzLnByb21pc2U9e30pO2Z1bmN0",
				"aW9uIF8xYmUoXzFiZil7aWYoXzFiYSl7dGhyb3cgbmV3IEVycm9yKCJUaGlzIGRlZmVycmVkIGhhcyBh",
				"bHJlYWR5IGJlZW4gcmVzb2x2ZWQiKTt9XzFiOT1fMWJmO18xYmE9dHJ1ZTtfMWMwKCk7fTtmdW5jdGlv",
				"biBfMWMwKCl7dmFyIF8xYzE7d2hpbGUoIV8xYzEmJl8xYmMpe3ZhciBfMWMyPV8xYmM7XzFiYz1fMWJj",
				"Lm5leHQ7aWYoKF8xYzE9KF8xYzIucHJvZ3Jlc3M9PV8xYjYpKSl7XzFiYT1mYWxzZTt9dmFyIGZ1bmM9",
				"KF8xYmI/XzFjMi5lcnJvcjpfMWMyLnJlc29sdmVkKTtpZihmdW5jKXt0cnl7dmFyIF8xYzM9ZnVuYyhf",
				"MWI5KTtpZihfMWMzJiZ0eXBlb2YgXzFjMy50aGVuPT09ImZ1bmN0aW9uIil7XzFjMy50aGVuKGxhbmcu",
				"aGl0Y2goXzFjMi5kZWZlcnJlZCwicmVzb2x2ZSIpLGxhbmcuaGl0Y2goXzFjMi5kZWZlcnJlZCwicmVq",
				"ZWN0IiksbGFuZy5oaXRjaChfMWMyLmRlZmVycmVkLCJwcm9ncmVzcyIpKTtjb250aW51ZTt9dmFyIF8x",
				"YzQ9XzFjMSYmXzFjMz09PXVuZGVmaW5lZDtpZihfMWMxJiYhXzFjNCl7XzFiYj1fMWMzIGluc3RhbmNl",
				"b2YgRXJyb3I7fV8xYzIuZGVmZXJyZWRbXzFjNCYmXzFiYj8icmVqZWN0IjoicmVzb2x2ZSJdKF8xYzQ/",
				"XzFiOTpfMWMzKTt9Y2F0Y2goZSl7XzFjMi5kZWZlcnJlZC5yZWplY3QoZSk7fX1lbHNle2lmKF8xYmIp",
				"e18xYzIuZGVmZXJyZWQucmVqZWN0KF8xYjkpO31lbHNle18xYzIuZGVmZXJyZWQucmVzb2x2ZShfMWI5",
				"KTt9fX19O3RoaXMucmVzb2x2ZT10aGlzLmNhbGxiYWNrPWZ1bmN0aW9uKF8xYzUpe3RoaXMuZmlyZWQ9",
				"MDt0aGlzLnJlc3VsdHM9W18xYzUsbnVsbF07XzFiZShfMWM1KTt9O3RoaXMucmVqZWN0PXRoaXMuZXJy",
				"YmFjaz1mdW5jdGlvbihfMWM2KXtfMWJiPXRydWU7dGhpcy5maXJlZD0xO18xYmUoXzFjNik7dGhpcy5y",
				"ZXN1bHRzPVtudWxsLF8xYzZdO2lmKCFfMWM2fHxfMWM2LmxvZyE9PWZhbHNlKXsoZG9qby5jb25maWcu",
				"ZGVmZXJyZWRPbkVycm9yfHxmdW5jdGlvbih4KXtjb25zb2xlLmVycm9yKHgpO30pKF8xYzYpO319O3Ro",
				"aXMucHJvZ3Jlc3M9ZnVuY3Rpb24oXzFjNyl7dmFyIF8xYzg9XzFiYzt3aGlsZShfMWM4KXt2YXIgXzFj",
				"OT1fMWM4LnByb2dyZXNzO18xYzkmJl8xYzkoXzFjNyk7XzFjOD1fMWM4Lm5leHQ7fX07dGhpcy5hZGRD",
				"YWxsYmFja3M9ZnVuY3Rpb24oXzFjYSxfMWNiKXt0aGlzLnRoZW4oXzFjYSxfMWNiLF8xYjYpO3JldHVy",
				"biB0aGlzO307XzFiZC50aGVuPXRoaXMudGhlbj1mdW5jdGlvbihfMWNjLF8xY2QsXzFjZSl7dmFyIF8x",
				"Y2Y9XzFjZT09XzFiNj90aGlzOm5ldyBkb2pvLkRlZmVycmVkKF8xYmQuY2FuY2VsKTt2YXIgXzFkMD17",
				"cmVzb2x2ZWQ6XzFjYyxlcnJvcjpfMWNkLHByb2dyZXNzOl8xY2UsZGVmZXJyZWQ6XzFjZn07aWYoXzFi",
				"Yyl7aGVhZD1oZWFkLm5leHQ9XzFkMDt9ZWxzZXtfMWJjPWhlYWQ9XzFkMDt9aWYoXzFiYSl7XzFjMCgp",
				"O31yZXR1cm4gXzFjZi5wcm9taXNlO307dmFyIF8xZDE9dGhpcztfMWJkLmNhbmNlbD10aGlzLmNhbmNl",
				"bD1mdW5jdGlvbigpe2lmKCFfMWJhKXt2YXIgXzFkMj1fMWI4JiZfMWI4KF8xZDEpO2lmKCFfMWJhKXtp",
				"ZighKF8xZDIgaW5zdGFuY2VvZiBFcnJvcikpe18xZDI9bmV3IEVycm9yKF8xZDIpO31fMWQyLmxvZz1m",
				"YWxzZTtfMWQxLnJlamVjdChfMWQyKTt9fX07XzFiNyhfMWJkKTt9O2xhbmcuZXh0ZW5kKGRvam8uRGVm",
				"ZXJyZWQse2FkZENhbGxiYWNrOmZ1bmN0aW9uKF8xZDMpe3JldHVybiB0aGlzLmFkZENhbGxiYWNrcyhs",
				"YW5nLmhpdGNoLmFwcGx5KGRvam8sYXJndW1lbnRzKSk7fSxhZGRFcnJiYWNrOmZ1bmN0aW9uKF8xZDQp",
				"e3JldHVybiB0aGlzLmFkZENhbGxiYWNrcyhudWxsLGxhbmcuaGl0Y2guYXBwbHkoZG9qbyxhcmd1bWVu",
				"dHMpKTt9LGFkZEJvdGg6ZnVuY3Rpb24oXzFkNSl7dmFyIF8xZDY9bGFuZy5oaXRjaC5hcHBseShkb2pv",
				"LGFyZ3VtZW50cyk7cmV0dXJuIHRoaXMuYWRkQ2FsbGJhY2tzKF8xZDYsXzFkNik7fSxmaXJlZDotMX0p",
				"O2Rvam8uRGVmZXJyZWQud2hlbj1kb2pvLndoZW49ZnVuY3Rpb24oXzFkNyxfMWQ4LF8xZDksXzFkYSl7",
				"aWYoXzFkNyYmdHlwZW9mIF8xZDcudGhlbj09PSJmdW5jdGlvbiIpe3JldHVybiBfMWQ3LnRoZW4oXzFk",
				"OCxfMWQ5LF8xZGEpO31yZXR1cm4gXzFkOD9fMWQ4KF8xZDcpOl8xZDc7fTtyZXR1cm4gZG9qby5EZWZl",
				"cnJlZDt9KTt9LCJkb2pvL05vZGVMaXN0LWRvbSI6ZnVuY3Rpb24oKXtkZWZpbmUoWyIuL19iYXNlL2tl",
				"cm5lbCIsIi4vcXVlcnkiLCIuL19iYXNlL2FycmF5IiwiLi9fYmFzZS9sYW5nIiwiLi9kb20tY2xhc3Mi",
				"LCIuL2RvbS1jb25zdHJ1Y3QiLCIuL2RvbS1nZW9tZXRyeSIsIi4vZG9tLWF0dHIiLCIuL2RvbS1zdHls",
				"ZSJdLGZ1bmN0aW9uKGRvam8sXzFkYixfMWRjLGxhbmcsXzFkZCxfMWRlLF8xZGYsXzFlMCxfMWUxKXt2",
				"YXIgXzFlMj1mdW5jdGlvbihhKXtyZXR1cm4gYS5sZW5ndGg9PTEmJih0eXBlb2YgYVswXT09InN0cmlu",
				"ZyIpO307dmFyIF8xZTM9ZnVuY3Rpb24obm9kZSl7dmFyIHA9bm9kZS5wYXJlbnROb2RlO2lmKHApe3Au",
				"cmVtb3ZlQ2hpbGQobm9kZSk7fX07dmFyIF8xZTQ9XzFkYi5Ob2RlTGlzdCxhd2M9XzFlNC5fYWRhcHRX",
				"aXRoQ29uZGl0aW9uLGFhZmU9XzFlNC5fYWRhcHRBc0ZvckVhY2gsYWFtPV8xZTQuX2FkYXB0QXNNYXA7",
				"ZnVuY3Rpb24gXzFlNShfMWU2KXtyZXR1cm4gZnVuY3Rpb24obm9kZSxuYW1lLF8xZTcpe2lmKGFyZ3Vt",
				"ZW50cy5sZW5ndGg9PTIpe3JldHVybiBfMWU2W3R5cGVvZiBuYW1lPT0ic3RyaW5nIj8iZ2V0Ijoic2V0",
				"Il0obm9kZSxuYW1lKTt9cmV0dXJuIF8xZTYuc2V0KG5vZGUsbmFtZSxfMWU3KTt9O307bGFuZy5leHRl",
				"bmQoXzFlNCx7X25vcm1hbGl6ZTpmdW5jdGlvbihfMWU4LF8xZTkpe3ZhciBfMWVhPV8xZTgucGFyc2U9",
				"PT10cnVlO2lmKHR5cGVvZiBfMWU4LnRlbXBsYXRlPT0ic3RyaW5nIil7dmFyIF8xZWI9XzFlOC50ZW1w",
				"bGF0ZUZ1bmN8fChkb2pvLnN0cmluZyYmZG9qby5zdHJpbmcuc3Vic3RpdHV0ZSk7XzFlOD1fMWViP18x",
				"ZWIoXzFlOC50ZW1wbGF0ZSxfMWU4KTpfMWU4O312YXIgdHlwZT0odHlwZW9mIF8xZTgpO2lmKHR5cGU9",
				"PSJzdHJpbmcifHx0eXBlPT0ibnVtYmVyIil7XzFlOD1fMWRlLnRvRG9tKF8xZTgsKF8xZTkmJl8xZTku",
				"b3duZXJEb2N1bWVudCkpO2lmKF8xZTgubm9kZVR5cGU9PTExKXtfMWU4PWxhbmcuX3RvQXJyYXkoXzFl",
				"OC5jaGlsZE5vZGVzKTt9ZWxzZXtfMWU4PVtfMWU4XTt9fWVsc2V7aWYoIWxhbmcuaXNBcnJheUxpa2Uo",
				"XzFlOCkpe18xZTg9W18xZThdO31lbHNle2lmKCFsYW5nLmlzQXJyYXkoXzFlOCkpe18xZTg9bGFuZy5f",
				"dG9BcnJheShfMWU4KTt9fX1pZihfMWVhKXtfMWU4Ll9ydW5QYXJzZT10cnVlO31yZXR1cm4gXzFlODt9",
				"LF9jbG9uZU5vZGU6ZnVuY3Rpb24obm9kZSl7cmV0dXJuIG5vZGUuY2xvbmVOb2RlKHRydWUpO30sX3Bs",
				"YWNlOmZ1bmN0aW9uKGFyeSxfMWVjLF8xZWQsXzFlZSl7aWYoXzFlYy5ub2RlVHlwZSE9MSYmXzFlZD09",
				"Im9ubHkiKXtyZXR1cm47fXZhciBfMWVmPV8xZWMsXzFmMDt2YXIgXzFmMT1hcnkubGVuZ3RoO2Zvcih2",
				"YXIgaT1fMWYxLTE7aT49MDtpLS0pe3ZhciBub2RlPShfMWVlP3RoaXMuX2Nsb25lTm9kZShhcnlbaV0p",
				"OmFyeVtpXSk7aWYoYXJ5Ll9ydW5QYXJzZSYmZG9qby5wYXJzZXImJmRvam8ucGFyc2VyLnBhcnNlKXtp",
				"ZighXzFmMCl7XzFmMD1fMWVmLm93bmVyRG9jdW1lbnQuY3JlYXRlRWxlbWVudCgiZGl2Iik7fV8xZjAu",
				"YXBwZW5kQ2hpbGQobm9kZSk7ZG9qby5wYXJzZXIucGFyc2UoXzFmMCk7bm9kZT1fMWYwLmZpcnN0Q2hp",
				"bGQ7d2hpbGUoXzFmMC5maXJzdENoaWxkKXtfMWYwLnJlbW92ZUNoaWxkKF8xZjAuZmlyc3RDaGlsZCk7",
				"fX1pZihpPT1fMWYxLTEpe18xZGUucGxhY2Uobm9kZSxfMWVmLF8xZWQpO31lbHNle18xZWYucGFyZW50",
				"Tm9kZS5pbnNlcnRCZWZvcmUobm9kZSxfMWVmKTt9XzFlZj1ub2RlO319LGF0dHI6YXdjKF8xZTUoXzFl",
				"MCksXzFlMiksc3R5bGU6YXdjKF8xZTUoXzFlMSksXzFlMiksYWRkQ2xhc3M6YWFmZShfMWRkLmFkZCks",
				"cmVtb3ZlQ2xhc3M6YWFmZShfMWRkLnJlbW92ZSkscmVwbGFjZUNsYXNzOmFhZmUoXzFkZC5yZXBsYWNl",
				"KSx0b2dnbGVDbGFzczphYWZlKF8xZGQudG9nZ2xlKSxlbXB0eTphYWZlKF8xZGUuZW1wdHkpLHJlbW92",
				"ZUF0dHI6YWFmZShfMWUwLnJlbW92ZSkscG9zaXRpb246YWFtKF8xZGYucG9zaXRpb24pLG1hcmdpbkJv",
				"eDphYW0oXzFkZi5nZXRNYXJnaW5Cb3gpLHBsYWNlOmZ1bmN0aW9uKF8xZjIsXzFmMyl7dmFyIGl0ZW09",
				"XzFkYihfMWYyKVswXTtyZXR1cm4gdGhpcy5mb3JFYWNoKGZ1bmN0aW9uKG5vZGUpe18xZGUucGxhY2Uo",
				"bm9kZSxpdGVtLF8xZjMpO30pO30sb3JwaGFuOmZ1bmN0aW9uKF8xZjQpe3JldHVybiAoXzFmND9fMWRi",
				"Ll9maWx0ZXJSZXN1bHQodGhpcyxfMWY0KTp0aGlzKS5mb3JFYWNoKF8xZTMpO30sYWRvcHQ6ZnVuY3Rp",
				"b24oXzFmNSxfMWY2KXtyZXR1cm4gXzFkYihfMWY1KS5wbGFjZSh0aGlzWzBdLF8xZjYpLl9zdGFzaCh0",
				"aGlzKTt9LHF1ZXJ5OmZ1bmN0aW9uKF8xZjcpe2lmKCFfMWY3KXtyZXR1cm4gdGhpczt9dmFyIHJldD1u",
				"ZXcgXzFlNDt0aGlzLm1hcChmdW5jdGlvbihub2RlKXtfMWRiKF8xZjcsbm9kZSkuZm9yRWFjaChmdW5j",
				"dGlvbihfMWY4KXtpZihfMWY4IT09dW5kZWZpbmVkKXtyZXQucHVzaChfMWY4KTt9fSk7fSk7cmV0dXJu",
				"IHJldC5fc3Rhc2godGhpcyk7fSxmaWx0ZXI6ZnVuY3Rpb24oXzFmOSl7dmFyIGE9YXJndW1lbnRzLF8x",
				"ZmE9dGhpcyxfMWZiPTA7aWYodHlwZW9mIF8xZjk9PSJzdHJpbmciKXtfMWZhPV8xZGIuX2ZpbHRlclJl",
				"c3VsdCh0aGlzLGFbMF0pO2lmKGEubGVuZ3RoPT0xKXtyZXR1cm4gXzFmYS5fc3Rhc2godGhpcyk7fV8x",
				"ZmI9MTt9cmV0dXJuIHRoaXMuX3dyYXAoXzFkYy5maWx0ZXIoXzFmYSxhW18xZmJdLGFbXzFmYisxXSks",
				"dGhpcyk7fSxhZGRDb250ZW50OmZ1bmN0aW9uKF8xZmMsXzFmZCl7XzFmYz10aGlzLl9ub3JtYWxpemUo",
				"XzFmYyx0aGlzWzBdKTtmb3IodmFyIGk9MCxub2RlOyhub2RlPXRoaXNbaV0pO2krKyl7dGhpcy5fcGxh",
				"Y2UoXzFmYyxub2RlLF8xZmQsaT4wKTt9cmV0dXJuIHRoaXM7fX0pO3JldHVybiBfMWU0O30pO30sImRv",
				"am8vcXVlcnkiOmZ1bmN0aW9uKCl7ZGVmaW5lKFsiLi9fYmFzZS9rZXJuZWwiLCIuL2hhcyIsIi4vZG9t",
				"IiwiLi9vbiIsIi4vX2Jhc2UvYXJyYXkiLCIuL19iYXNlL2xhbmciLCIuL3NlbGVjdG9yL19sb2FkZXIi",
				"LCIuL3NlbGVjdG9yL19sb2FkZXIhZGVmYXVsdCJdLGZ1bmN0aW9uKGRvam8saGFzLGRvbSxvbixfMWZl",
				"LGxhbmcsXzFmZixfMjAwKXsidXNlIHN0cmljdCI7aGFzLmFkZCgiYXJyYXktZXh0ZW5zaWJsZSIsZnVu",
				"Y3Rpb24oKXtyZXR1cm4gbGFuZy5kZWxlZ2F0ZShbXSx7bGVuZ3RoOjF9KS5sZW5ndGg9PTEmJiFoYXMo",
				"ImJ1Zy1mb3ItaW4tc2tpcHMtc2hhZG93ZWQiKTt9KTt2YXIgYXA9QXJyYXkucHJvdG90eXBlLGFwcz1h",
				"cC5zbGljZSxhcGM9YXAuY29uY2F0LF8yMDE9XzFmZS5mb3JFYWNoO3ZhciB0bmw9ZnVuY3Rpb24oYSxf",
				"MjAyLF8yMDMpe3ZhciBfMjA0PW5ldyAoXzIwM3x8dGhpcy5fTm9kZUxpc3RDdG9yfHxubCkoYSk7cmV0",
				"dXJuIF8yMDI/XzIwNC5fc3Rhc2goXzIwMik6XzIwNDt9O3ZhciBfMjA1PWZ1bmN0aW9uKGYsYSxvKXth",
				"PVswXS5jb25jYXQoYXBzLmNhbGwoYSwwKSk7bz1vfHxkb2pvLmdsb2JhbDtyZXR1cm4gZnVuY3Rpb24o",
				"bm9kZSl7YVswXT1ub2RlO3JldHVybiBmLmFwcGx5KG8sYSk7fTt9O3ZhciBfMjA2PWZ1bmN0aW9uKGYs",
				"byl7cmV0dXJuIGZ1bmN0aW9uKCl7dGhpcy5mb3JFYWNoKF8yMDUoZixhcmd1bWVudHMsbykpO3JldHVy",
				"biB0aGlzO307fTt2YXIgXzIwNz1mdW5jdGlvbihmLG8pe3JldHVybiBmdW5jdGlvbigpe3JldHVybiB0",
				"aGlzLm1hcChfMjA1KGYsYXJndW1lbnRzLG8pKTt9O307dmFyIF8yMDg9ZnVuY3Rpb24oZixvKXtyZXR1",
				"cm4gZnVuY3Rpb24oKXtyZXR1cm4gdGhpcy5maWx0ZXIoXzIwNShmLGFyZ3VtZW50cyxvKSk7fTt9O3Zh",
				"ciBfMjA5PWZ1bmN0aW9uKGYsZyxvKXtyZXR1cm4gZnVuY3Rpb24oKXt2YXIgYT1hcmd1bWVudHMsYm9k",
				"eT1fMjA1KGYsYSxvKTtpZihnLmNhbGwob3x8ZG9qby5nbG9iYWwsYSkpe3JldHVybiB0aGlzLm1hcChi",
				"b2R5KTt9dGhpcy5mb3JFYWNoKGJvZHkpO3JldHVybiB0aGlzO307fTt2YXIgXzIwYT1mdW5jdGlvbihf",
				"MjBiKXt2YXIgXzIwYz10aGlzIGluc3RhbmNlb2YgbmwmJmhhcygiYXJyYXktZXh0ZW5zaWJsZSIpO2lm",
				"KHR5cGVvZiBfMjBiPT0ibnVtYmVyIil7XzIwYj1BcnJheShfMjBiKTt9dmFyIF8yMGQ9KF8yMGImJiJs",
				"ZW5ndGgiIGluIF8yMGIpP18yMGI6YXJndW1lbnRzO2lmKF8yMGN8fCFfMjBkLnNvcnQpe3ZhciBfMjBl",
				"PV8yMGM/dGhpczpbXSxsPV8yMGUubGVuZ3RoPV8yMGQubGVuZ3RoO2Zvcih2YXIgaT0wO2k8bDtpKysp",
				"e18yMGVbaV09XzIwZFtpXTt9aWYoXzIwYyl7cmV0dXJuIF8yMGU7fV8yMGQ9XzIwZTt9bGFuZy5fbWl4",
				"aW4oXzIwZCxubHApO18yMGQuX05vZGVMaXN0Q3Rvcj1mdW5jdGlvbihfMjBmKXtyZXR1cm4gbmwoXzIw",
				"Zik7fTtyZXR1cm4gXzIwZDt9O3ZhciBubD1fMjBhLG5scD1ubC5wcm90b3R5cGU9aGFzKCJhcnJheS1l",
				"eHRlbnNpYmxlIik/W106e307bmwuX3dyYXA9bmxwLl93cmFwPXRubDtubC5fYWRhcHRBc01hcD1fMjA3",
				"O25sLl9hZGFwdEFzRm9yRWFjaD1fMjA2O25sLl9hZGFwdEFzRmlsdGVyPV8yMDg7bmwuX2FkYXB0V2l0",
				"aENvbmRpdGlvbj1fMjA5O18yMDEoWyJzbGljZSIsInNwbGljZSJdLGZ1bmN0aW9uKG5hbWUpe3ZhciBm",
				"PWFwW25hbWVdO25scFtuYW1lXT1mdW5jdGlvbigpe3JldHVybiB0aGlzLl93cmFwKGYuYXBwbHkodGhp",
				"cyxhcmd1bWVudHMpLG5hbWU9PSJzbGljZSI/dGhpczpudWxsKTt9O30pO18yMDEoWyJpbmRleE9mIiwi",
				"bGFzdEluZGV4T2YiLCJldmVyeSIsInNvbWUiXSxmdW5jdGlvbihuYW1lKXt2YXIgZj1fMWZlW25hbWVd",
				"O25scFtuYW1lXT1mdW5jdGlvbigpe3JldHVybiBmLmFwcGx5KGRvam8sW3RoaXNdLmNvbmNhdChhcHMu",
				"Y2FsbChhcmd1bWVudHMsMCkpKTt9O30pO2xhbmcuZXh0ZW5kKF8yMGEse2NvbnN0cnVjdG9yOm5sLF9O",
				"b2RlTGlzdEN0b3I6bmwsdG9TdHJpbmc6ZnVuY3Rpb24oKXtyZXR1cm4gdGhpcy5qb2luKCIsIik7fSxf",
				"c3Rhc2g6ZnVuY3Rpb24oXzIxMCl7dGhpcy5fcGFyZW50PV8yMTA7cmV0dXJuIHRoaXM7fSxvbjpmdW5j",
				"dGlvbihfMjExLF8yMTIpe3ZhciBfMjEzPXRoaXMubWFwKGZ1bmN0aW9uKG5vZGUpe3JldHVybiBvbihu",
				"b2RlLF8yMTEsXzIxMik7fSk7XzIxMy5yZW1vdmU9ZnVuY3Rpb24oKXtmb3IodmFyIGk9MDtpPF8yMTMu",
				"bGVuZ3RoO2krKyl7XzIxM1tpXS5yZW1vdmUoKTt9fTtyZXR1cm4gXzIxMzt9LGVuZDpmdW5jdGlvbigp",
				"e2lmKHRoaXMuX3BhcmVudCl7cmV0dXJuIHRoaXMuX3BhcmVudDt9ZWxzZXtyZXR1cm4gbmV3IHRoaXMu",
				"X05vZGVMaXN0Q3RvcigwKTt9fSxjb25jYXQ6ZnVuY3Rpb24oaXRlbSl7dmFyIHQ9bGFuZy5pc0FycmF5",
				"KHRoaXMpP3RoaXM6YXBzLmNhbGwodGhpcywwKSxtPV8xZmUubWFwKGFyZ3VtZW50cyxmdW5jdGlvbihh",
				"KXtyZXR1cm4gYSYmIWxhbmcuaXNBcnJheShhKSYmKHR5cGVvZiBfMjBhIT0idW5kZWZpbmVkIiYmYS5j",
				"b25zdHJ1Y3Rvcj09PV8yMGF8fGEuY29uc3RydWN0b3I9PT10aGlzLl9Ob2RlTGlzdEN0b3IpP2Fwcy5j",
				"YWxsKGEsMCk6YTt9KTtyZXR1cm4gdGhpcy5fd3JhcChhcGMuYXBwbHkodCxtKSx0aGlzKTt9LG1hcDpm",
				"dW5jdGlvbihmdW5jLG9iail7cmV0dXJuIHRoaXMuX3dyYXAoXzFmZS5tYXAodGhpcyxmdW5jLG9iaiks",
				"dGhpcyk7fSxmb3JFYWNoOmZ1bmN0aW9uKF8yMTQsXzIxNSl7XzIwMSh0aGlzLF8yMTQsXzIxNSk7cmV0",
				"dXJuIHRoaXM7fSxmaWx0ZXI6ZnVuY3Rpb24oXzIxNil7dmFyIGE9YXJndW1lbnRzLF8yMTc9dGhpcyxf",
				"MjE4PTA7aWYodHlwZW9mIF8yMTY9PSJzdHJpbmciKXtfMjE3PV8yMTkuX2ZpbHRlclJlc3VsdCh0aGlz",
				"LGFbMF0pO2lmKGEubGVuZ3RoPT0xKXtyZXR1cm4gXzIxNy5fc3Rhc2godGhpcyk7fV8yMTg9MTt9cmV0",
				"dXJuIHRoaXMuX3dyYXAoXzFmZS5maWx0ZXIoXzIxNyxhW18yMThdLGFbXzIxOCsxXSksdGhpcyk7fSxp",
				"bnN0YW50aWF0ZTpmdW5jdGlvbihfMjFhLF8yMWIpe3ZhciBjPWxhbmcuaXNGdW5jdGlvbihfMjFhKT9f",
				"MjFhOmxhbmcuZ2V0T2JqZWN0KF8yMWEpO18yMWI9XzIxYnx8e307cmV0dXJuIHRoaXMuZm9yRWFjaChm",
				"dW5jdGlvbihub2RlKXtuZXcgYyhfMjFiLG5vZGUpO30pO30sYXQ6ZnVuY3Rpb24oKXt2YXIgdD1uZXcg",
				"dGhpcy5fTm9kZUxpc3RDdG9yKDApO18yMDEoYXJndW1lbnRzLGZ1bmN0aW9uKGkpe2lmKGk8MCl7aT10",
				"aGlzLmxlbmd0aCtpO31pZih0aGlzW2ldKXt0LnB1c2godGhpc1tpXSk7fX0sdGhpcyk7cmV0dXJuIHQu",
				"X3N0YXNoKHRoaXMpO319KTtmdW5jdGlvbiBfMjFjKF8yMWQsXzIxZSl7dmFyIF8yMWY9ZnVuY3Rpb24o",
				"XzIyMCxyb290KXtpZih0eXBlb2Ygcm9vdD09InN0cmluZyIpe3Jvb3Q9ZG9tLmJ5SWQocm9vdCk7aWYo",
				"IXJvb3Qpe3JldHVybiBuZXcgXzIxZShbXSk7fX12YXIgXzIyMT10eXBlb2YgXzIyMD09InN0cmluZyI/",
				"XzIxZChfMjIwLHJvb3QpOl8yMjAub3JwaGFuP18yMjA6W18yMjBdO2lmKF8yMjEub3JwaGFuKXtyZXR1",
				"cm4gXzIyMTt9cmV0dXJuIG5ldyBfMjFlKF8yMjEpO307XzIxZi5tYXRjaGVzPV8yMWQubWF0Y2h8fGZ1",
				"bmN0aW9uKG5vZGUsXzIyMixyb290KXtyZXR1cm4gXzIxZi5maWx0ZXIoW25vZGVdLF8yMjIscm9vdCku",
				"bGVuZ3RoPjA7fTtfMjFmLmZpbHRlcj1fMjFkLmZpbHRlcnx8ZnVuY3Rpb24oXzIyMyxfMjI0LHJvb3Qp",
				"e3JldHVybiBfMjFmKF8yMjQscm9vdCkuZmlsdGVyKGZ1bmN0aW9uKG5vZGUpe3JldHVybiBfMWZlLmlu",
				"ZGV4T2YoXzIyMyxub2RlKT4tMTt9KTt9O2lmKHR5cGVvZiBfMjFkIT0iZnVuY3Rpb24iKXt2YXIgXzIy",
				"NT1fMjFkLnNlYXJjaDtfMjFkPWZ1bmN0aW9uKF8yMjYscm9vdCl7cmV0dXJuIF8yMjUocm9vdHx8ZG9j",
				"dW1lbnQsXzIyNik7fTt9cmV0dXJuIF8yMWY7fTt2YXIgXzIxOT1fMjFjKF8yMDAsXzIwYSk7ZG9qby5x",
				"dWVyeT1fMjFjKF8yMDAsZnVuY3Rpb24oXzIyNyl7cmV0dXJuIF8yMGEoXzIyNyk7fSk7XzIxOS5sb2Fk",
				"PWZ1bmN0aW9uKGlkLF8yMjgsXzIyOSxfMjJhKXtfMWZmLmxvYWQoaWQsXzIyOCxmdW5jdGlvbihfMjJi",
				"KXtfMjI5KF8yMWMoXzIyYixfMjBhKSk7fSk7fTtkb2pvLl9maWx0ZXJRdWVyeVJlc3VsdD1fMjE5Ll9m",
				"aWx0ZXJSZXN1bHQ9ZnVuY3Rpb24oXzIyYyxfMjJkLHJvb3Qpe3JldHVybiBuZXcgXzIwYShfMjE5LmZp",
				"bHRlcihfMjJjLF8yMmQscm9vdCkpO307ZG9qby5Ob2RlTGlzdD1fMjE5Lk5vZGVMaXN0PV8yMGE7cmV0",
				"dXJuIF8yMTk7fSk7fSwiZG9qby9oYXMiOmZ1bmN0aW9uKCl7ZGVmaW5lKFsicmVxdWlyZSJdLGZ1bmN0",
				"aW9uKF8yMmUpe3ZhciBoYXM9XzIyZS5oYXN8fGZ1bmN0aW9uKCl7fTtpZighMSl7dmFyIF8yMmY9dHlw",
				"ZW9mIHdpbmRvdyE9InVuZGVmaW5lZCImJnR5cGVvZiBsb2NhdGlvbiE9InVuZGVmaW5lZCImJnR5cGVv",
				"ZiBkb2N1bWVudCE9InVuZGVmaW5lZCImJndpbmRvdy5sb2NhdGlvbj09bG9jYXRpb24mJndpbmRvdy5k",
				"b2N1bWVudD09ZG9jdW1lbnQsXzIzMD10aGlzLGRvYz1fMjJmJiZkb2N1bWVudCxfMjMxPWRvYyYmZG9j",
				"LmNyZWF0ZUVsZW1lbnQoIkRpViIpLF8yMzI9e307aGFzPWZ1bmN0aW9uKG5hbWUpe3JldHVybiB0eXBl",
				"b2YgXzIzMltuYW1lXT09ImZ1bmN0aW9uIj8oXzIzMltuYW1lXT1fMjMyW25hbWVdKF8yMzAsZG9jLF8y",
				"MzEpKTpfMjMyW25hbWVdO307aGFzLmNhY2hlPV8yMzI7aGFzLmFkZD1mdW5jdGlvbihuYW1lLHRlc3Qs",
				"bm93LF8yMzMpeyh0eXBlb2YgXzIzMltuYW1lXT09InVuZGVmaW5lZCJ8fF8yMzMpJiYoXzIzMltuYW1l",
				"XT10ZXN0KTtyZXR1cm4gbm93JiZoYXMobmFtZSk7fTt0cnVlfHxoYXMuYWRkKCJob3N0LWJyb3dzZXIi",
				"LF8yMmYpO3RydWV8fGhhcy5hZGQoImRvbSIsXzIyZik7dHJ1ZXx8aGFzLmFkZCgiZG9qby1kb20tcmVh",
				"ZHktYXBpIiwxKTt0cnVlfHxoYXMuYWRkKCJkb2pvLXNuaWZmIiwxKTt9aWYoMSl7dmFyIF8yMzQ9bmF2",
				"aWdhdG9yLnVzZXJBZ2VudDtoYXMuYWRkKCJkb20tYWRkZXZlbnRsaXN0ZW5lciIsISFkb2N1bWVudC5h",
				"ZGRFdmVudExpc3RlbmVyKTtoYXMuYWRkKCJ0b3VjaCIsIm9udG91Y2hzdGFydCIgaW4gZG9jdW1lbnQp",
				"O2hhcy5hZGQoImRldmljZS13aWR0aCIsc2NyZWVuLmF2YWlsV2lkdGh8fGlubmVyV2lkdGgpO2hhcy5h",
				"ZGQoImFnZW50LWlvcyIsISFfMjM0Lm1hdGNoKC9pUGhvbmV8aVBbYW9dZC8pKTtoYXMuYWRkKCJhZ2Vu",
				"dC1hbmRyb2lkIixfMjM0LmluZGV4T2YoImFuZHJvaWQiKT4xKTt9aGFzLmNsZWFyRWxlbWVudD1mdW5j",
				"dGlvbihfMjM1KXtfMjM1LmlubmVySFRNTD0iIjtyZXR1cm4gXzIzNTt9O2hhcy5ub3JtYWxpemU9ZnVu",
				"Y3Rpb24oaWQsXzIzNil7dmFyIF8yMzc9aWQubWF0Y2goL1tcPzpdfFteOlw/XSovZyksaT0wLGdldD1m",
				"dW5jdGlvbihza2lwKXt2YXIgdGVybT1fMjM3W2krK107aWYodGVybT09IjoiKXtyZXR1cm4gMDt9ZWxz",
				"ZXtpZihfMjM3W2krK109PSI/Iil7aWYoIXNraXAmJmhhcyh0ZXJtKSl7cmV0dXJuIGdldCgpO31lbHNl",
				"e2dldCh0cnVlKTtyZXR1cm4gZ2V0KHNraXApO319cmV0dXJuIHRlcm18fDA7fX07aWQ9Z2V0KCk7cmV0",
				"dXJuIGlkJiZfMjM2KGlkKTt9O2hhcy5sb2FkPWZ1bmN0aW9uKGlkLF8yMzgsXzIzOSl7aWYoaWQpe18y",
				"MzgoW2lkXSxfMjM5KTt9ZWxzZXtfMjM5KCk7fX07cmV0dXJuIGhhczt9KTt9LCJkb2pvL19iYXNlL2xv",
				"YWRlciI6ZnVuY3Rpb24oKXtkZWZpbmUoWyIuL2tlcm5lbCIsIi4uL2hhcyIsInJlcXVpcmUiLCJtb2R1",
				"bGUiLCIuL2pzb24iLCIuL2xhbmciLCIuL2FycmF5Il0sZnVuY3Rpb24oZG9qbyxoYXMsXzIzYSxfMjNi",
				"LGpzb24sbGFuZyxfMjNjKXtpZighMSl7Y29uc29sZS5lcnJvcigiY2Fubm90IGxvYWQgdGhlIERvam8g",
				"djEueCBsb2FkZXIgd2l0aCBhIGZvcmVpZ24gbG9hZGVyIik7cmV0dXJuIDA7fXZhciBfMjNkPWZ1bmN0",
				"aW9uKGlkKXtyZXR1cm4ge3NyYzpfMjNiLmlkLGlkOmlkfTt9LF8yM2U9ZnVuY3Rpb24obmFtZSl7cmV0",
				"dXJuIG5hbWUucmVwbGFjZSgvXC4vZywiLyIpO30sXzIzZj0vXC9cLz4+YnVpbHQvLF8yNDA9W10sXzI0",
				"MT1bXSxfMjQyPWZ1bmN0aW9uKG1pZCxfMjQzLF8yNDQpe18yNDAucHVzaChfMjQ0KTtfMjNjLmZvckVh",
				"Y2gobWlkLnNwbGl0KCIsIiksZnVuY3Rpb24obWlkKXt2YXIgXzI0NT1fMjQ2KG1pZCxfMjQzLm1vZHVs",
				"ZSk7XzI0MS5wdXNoKF8yNDUpO18yNDcoXzI0NSk7fSk7XzI0OCgpO30sXzI0OSxfMjRhPWZ1bmN0aW9u",
				"KG0pe2lmKF8yNDlbbS5taWRdfHwvbG9hZEluaXRcIS8udGVzdChtLm1pZCkpe3JldHVybiB0cnVlO31f",
				"MjQ5W20ubWlkXT0xO2lmKG0uaW5qZWN0ZWQhPT1fMjRiJiYhbS5leGVjdXRlZCl7cmV0dXJuIGZhbHNl",
				"O31mb3IodmFyIGRlcHM9bS5kZXBzfHxbXSxpPTA7aTxkZXBzLmxlbmd0aDtpKyspe2lmKCFfMjRhKGRl",
				"cHNbaV0pKXtyZXR1cm4gZmFsc2U7fX1yZXR1cm4gdHJ1ZTt9LF8yNDg9ZnVuY3Rpb24oKXtfMjQ5PXt9",
				"O18yNDE9XzIzYy5maWx0ZXIoXzI0MSxmdW5jdGlvbihfMjRjKXtyZXR1cm4gIV8yNGEoXzI0Yyk7fSk7",
				"aWYoIV8yNDEubGVuZ3RoKXtfMjRlLmhvbGRJZGxlKCk7dmFyIF8yNGQ9XzI0MDtfMjQwPVtdO18yM2Mu",
				"Zm9yRWFjaChfMjRkLGZ1bmN0aW9uKGNiKXtjYigxKTt9KTtfMjRlLnJlbGVhc2VJZGxlKCk7fX0sXzI0",
				"Zj1mdW5jdGlvbihtaWQsXzI1MCxfMjUxKXtfMjUwKFttaWRdLGZ1bmN0aW9uKF8yNTIpe18yNTAoXzI1",
				"Mi5uYW1lcyxmdW5jdGlvbigpe2Zvcih2YXIgXzI1Mz0iIixhcmdzPVtdLGk9MDtpPGFyZ3VtZW50cy5s",
				"ZW5ndGg7aSsrKXtfMjUzKz0idmFyICIrXzI1Mi5uYW1lc1tpXSsiPSBhcmd1bWVudHNbIitpKyJdOyAi",
				"O2FyZ3MucHVzaChhcmd1bWVudHNbaV0pO31ldmFsKF8yNTMpO3ZhciBfMjU0PV8yNTAubW9kdWxlLGRl",
				"cHM9W10saG9sZD17fSxfMjU1PVtdLHAsXzI1Nj17cHJvdmlkZTpmdW5jdGlvbihfMjU3KXtfMjU3PV8y",
				"M2UoXzI1Nyk7dmFyIF8yNTg9XzI0NihfMjU3LF8yNTQpO2lmKF8yNTghPT1fMjU0KXtfMjdkKF8yNTgp",
				"O319LHJlcXVpcmU6ZnVuY3Rpb24oXzI1OSxfMjVhKXtfMjU5PV8yM2UoXzI1OSk7XzI1YSYmKF8yNDYo",
				"XzI1OSxfMjU0KS5yZXN1bHQ9XzI3Nyk7XzI1NS5wdXNoKF8yNTkpO30scmVxdWlyZUxvY2FsaXphdGlv",
				"bjpmdW5jdGlvbihfMjViLF8yNWMsXzI1ZCl7ZGVwcy5sZW5ndGh8fChkZXBzPVsiZG9qby9pMThuIl0p",
				"O18yNWQ9KF8yNWR8fGRvam8ubG9jYWxlKS50b0xvd2VyQ2FzZSgpO18yNWI9XzIzZShfMjViKSsiL25s",
				"cy8iKygvcm9vdC9pLnRlc3QoXzI1ZCk/IiI6XzI1ZCsiLyIpK18yM2UoXzI1Yyk7aWYoXzI0NihfMjVi",
				"LF8yNTQpLmlzWGQpe2RlcHMucHVzaCgiZG9qby9pMThuISIrXzI1Yik7fX0sbG9hZEluaXQ6ZnVuY3Rp",
				"b24oZil7ZigpO319O3RyeXtmb3IocCBpbiBfMjU2KXtob2xkW3BdPWRvam9bcF07ZG9qb1twXT1fMjU2",
				"W3BdO31fMjUyLmRlZi5hcHBseShudWxsLGFyZ3MpO31jYXRjaChlKXtfMjdlKCJlcnJvciIsW18yM2Qo",
				"ImZhaWxlZERvam9Mb2FkSW5pdCIpLGVdKTt9ZmluYWxseXtmb3IocCBpbiBfMjU2KXtkb2pvW3BdPWhv",
				"bGRbcF07fX1fMjU1Lmxlbmd0aCYmZGVwcy5wdXNoKCJkb2pvL3JlcXVpcmUhIitfMjU1LmpvaW4oIiwi",
				"KSk7XzI0MC5wdXNoKF8yNTEpO18yM2MuZm9yRWFjaChfMjU1LGZ1bmN0aW9uKG1pZCl7dmFyIF8yNWU9",
				"XzI0NihtaWQsXzI1MC5tb2R1bGUpO18yNDEucHVzaChfMjVlKTtfMjQ3KF8yNWUpO30pO18yNDgoKTt9",
				"KTt9KTt9LF8yNWY9ZnVuY3Rpb24odGV4dCxfMjYwLF8yNjEpe3ZhciBfMjYyPS9cKHxcKS9nLF8yNjM9",
				"MSxfMjY0O18yNjIubGFzdEluZGV4PV8yNjA7d2hpbGUoKF8yNjQ9XzI2Mi5leGVjKHRleHQpKSl7aWYo",
				"XzI2NFswXT09IikiKXtfMjYzLT0xO31lbHNle18yNjMrPTE7fWlmKF8yNjM9PTApe2JyZWFrO319aWYo",
				"XzI2MyE9MCl7dGhyb3cgInVubWF0Y2hlZCBwYXJlbiBhcm91bmQgY2hhcmFjdGVyICIrXzI2Mi5sYXN0",
				"SW5kZXgrIiBpbjogIit0ZXh0O31yZXR1cm4gW2Rvam8udHJpbSh0ZXh0LnN1YnN0cmluZyhfMjYxLF8y",
				"NjIubGFzdEluZGV4KSkrIjtcbiIsXzI2Mi5sYXN0SW5kZXhdO30sXzI2NT0vKFwvXCooW1xzXFNdKj8p",
				"XCpcL3xcL1wvKC4qKSQpL21nLF8yNjY9LyhefFxzKWRvam9cLihsb2FkSW5pdHxyZXF1aXJlfHByb3Zp",
				"ZGV8cmVxdWlyZUxvY2FsaXphdGlvbnxyZXF1aXJlSWZ8cmVxdWlyZUFmdGVySWZ8cGxhdGZvcm1SZXF1",
				"aXJlKVxzKlwoL21nLF8yNjc9LyhefFxzKShyZXF1aXJlfGRlZmluZSlccypcKC9tLF8yNjg9ZnVuY3Rp",
				"b24odGV4dCxfMjY5KXt2YXIgXzI2YSxfMjZiLF8yNmMsXzI2ZCxfMjZlPVtdLF8yNmY9W10sXzI3MD1b",
				"XTtfMjY5PV8yNjl8fHRleHQucmVwbGFjZShfMjY1LGZ1bmN0aW9uKF8yNzEpe18yNjYubGFzdEluZGV4",
				"PV8yNjcubGFzdEluZGV4PTA7cmV0dXJuIChfMjY2LnRlc3QoXzI3MSl8fF8yNjcudGVzdChfMjcxKSk/",
				"IiI6XzI3MTt9KTt3aGlsZSgoXzI2YT1fMjY2LmV4ZWMoXzI2OSkpKXtfMjZiPV8yNjYubGFzdEluZGV4",
				"O18yNmM9XzI2Yi1fMjZhWzBdLmxlbmd0aDtfMjZkPV8yNWYoXzI2OSxfMjZiLF8yNmMpO2lmKF8yNmFb",
				"Ml09PSJsb2FkSW5pdCIpe18yNmUucHVzaChfMjZkWzBdKTt9ZWxzZXtfMjZmLnB1c2goXzI2ZFswXSk7",
				"fV8yNjYubGFzdEluZGV4PV8yNmRbMV07fV8yNzA9XzI2ZS5jb25jYXQoXzI2Zik7aWYoXzI3MC5sZW5n",
				"dGh8fCFfMjY3LnRlc3QoXzI2OSkpe3JldHVybiBbdGV4dC5yZXBsYWNlKC8oXnxccylkb2pvXC5sb2Fk",
				"SW5pdFxzKlwoL2csIlxuMCAmJiBkb2pvLmxvYWRJbml0KCIpLF8yNzAuam9pbigiIiksXzI3MF07fWVs",
				"c2V7cmV0dXJuIDA7fX0sXzI3Mj1mdW5jdGlvbihfMjczLHRleHQpe3ZhciBfMjc0LGlkLF8yNzU9W10s",
				"XzI3Nj1bXTtpZihfMjNmLnRlc3QodGV4dCl8fCEoXzI3ND1fMjY4KHRleHQpKSl7cmV0dXJuIDA7fWlk",
				"PV8yNzMubWlkKyItKmxvYWRJbml0Ijtmb3IodmFyIHAgaW4gXzI0NigiZG9qbyIsXzI3MykucmVzdWx0",
				"LnNjb3BlTWFwKXtfMjc1LnB1c2gocCk7XzI3Ni5wdXNoKCJcIiIrcCsiXCIiKTt9cmV0dXJuICIvLyB4",
				"ZG9tYWluIHJld3JpdGUgb2YgIitfMjczLnBhdGgrIlxuIisiZGVmaW5lKCciK2lkKyInLHtcbiIrIlx0",
				"bmFtZXM6Iitkb2pvLnRvSnNvbihfMjc1KSsiLFxuIisiXHRkZWY6ZnVuY3Rpb24oIitfMjc1LmpvaW4o",
				"IiwiKSsiKXsiK18yNzRbMV0rIn0iKyJ9KTtcblxuIisiZGVmaW5lKCIrZG9qby50b0pzb24oXzI3NS5j",
				"b25jYXQoWyJkb2pvL2xvYWRJbml0ISIraWRdKSkrIiwgZnVuY3Rpb24oIitfMjc1LmpvaW4oIiwiKSsi",
				"KXtcbiIrXzI3NFswXSsifSk7Ijt9LF8yNGU9XzIzYS5pbml0U3luY0xvYWRlcihfMjQyLF8yNDgsXzI3",
				"Miksc3luYz1fMjRlLnN5bmMseGQ9XzI0ZS54ZCxfMjRiPV8yNGUuYXJyaXZlZCxfMjc3PV8yNGUubm9u",
				"bW9kdWxlLF8yNzg9XzI0ZS5leGVjdXRpbmcsXzI3OT1fMjRlLmV4ZWN1dGVkLF8yN2E9XzI0ZS5zeW5j",
				"RXhlY1N0YWNrLF8yN2I9XzI0ZS5tb2R1bGVzLF8yN2M9XzI0ZS5leGVjUSxfMjQ2PV8yNGUuZ2V0TW9k",
				"dWxlLF8yNDc9XzI0ZS5pbmplY3RNb2R1bGUsXzI3ZD1fMjRlLnNldEFycml2ZWQsXzI3ZT1fMjRlLnNp",
				"Z25hbCxfMjdmPV8yNGUuZmluaXNoRXhlYyxfMjgwPV8yNGUuZXhlY01vZHVsZSxfMjgxPV8yNGUuZ2V0",
				"TGVnYWN5TW9kZTtkb2pvLnByb3ZpZGU9ZnVuY3Rpb24obWlkKXt2YXIgXzI4Mj1fMjdhWzBdLF8yODM9",
				"bGFuZy5taXhpbihfMjQ2KF8yM2UobWlkKSxfMjNhLm1vZHVsZSkse2V4ZWN1dGVkOl8yNzgscmVzdWx0",
				"OmxhbmcuZ2V0T2JqZWN0KG1pZCx0cnVlKX0pO18yN2QoXzI4Myk7aWYoXzI4Mil7KF8yODIucHJvdmlk",
				"ZXN8fChfMjgyLnByb3ZpZGVzPVtdKSkucHVzaChmdW5jdGlvbigpe18yODMucmVzdWx0PWxhbmcuZ2V0",
				"T2JqZWN0KG1pZCk7ZGVsZXRlIF8yODMucHJvdmlkZXM7XzI4My5leGVjdXRlZCE9PV8yNzkmJl8yN2Yo",
				"XzI4Myk7fSk7fXJldHVybiBfMjgzLnJlc3VsdDt9O2hhcy5hZGQoImNvbmZpZy1wdWJsaXNoUmVxdWly",
				"ZVJlc3VsdCIsMSwwLDApO2Rvam8ucmVxdWlyZT1mdW5jdGlvbihfMjg0LF8yODUpe2Z1bmN0aW9uIF8y",
				"ODYobWlkLF8yODcpe3ZhciBfMjg4PV8yNDYoXzIzZShtaWQpLF8yM2EubW9kdWxlKTtpZihfMjdhLmxl",
				"bmd0aCYmXzI3YVswXS5maW5pc2gpe18yN2FbMF0uZmluaXNoLnB1c2gobWlkKTtyZXR1cm4gdW5kZWZp",
				"bmVkO31pZihfMjg4LmV4ZWN1dGVkKXtyZXR1cm4gXzI4OC5yZXN1bHQ7fV8yODcmJihfMjg4LnJlc3Vs",
				"dD1fMjc3KTt2YXIgXzI4OT1fMjgxKCk7XzI0NyhfMjg4KTtfMjg5PV8yODEoKTtpZihfMjg4LmV4ZWN1",
				"dGVkIT09XzI3OSYmXzI4OC5pbmplY3RlZD09PV8yNGIpe18yNGUuaG9sZElkbGUoKTtfMjgwKF8yODgp",
				"O18yNGUucmVsZWFzZUlkbGUoKTt9aWYoXzI4OC5leGVjdXRlZCl7cmV0dXJuIF8yODgucmVzdWx0O31p",
				"ZihfMjg5PT1zeW5jKXtpZihfMjg4LmNqcyl7XzI3Yy51bnNoaWZ0KF8yODgpO31lbHNle18yN2EubGVu",
				"Z3RoJiYoXzI3YVswXS5maW5pc2g9W21pZF0pO319ZWxzZXtfMjdjLnB1c2goXzI4OCk7fXJldHVybiB1",
				"bmRlZmluZWQ7fTt2YXIgXzI4YT1fMjg2KF8yODQsXzI4NSk7aWYoaGFzKCJjb25maWctcHVibGlzaFJl",
				"cXVpcmVSZXN1bHQiKSYmIWxhbmcuZXhpc3RzKF8yODQpJiZfMjhhIT09dW5kZWZpbmVkKXtsYW5nLnNl",
				"dE9iamVjdChfMjg0LF8yOGEpO31yZXR1cm4gXzI4YTt9O2Rvam8ubG9hZEluaXQ9ZnVuY3Rpb24oZil7",
				"ZigpO307ZG9qby5yZWdpc3Rlck1vZHVsZVBhdGg9ZnVuY3Rpb24oXzI4YixfMjhjKXt2YXIgXzI4ZD17",
				"fTtfMjhkW18yOGIucmVwbGFjZSgvXC4vZywiLyIpXT1fMjhjO18yM2Eoe3BhdGhzOl8yOGR9KTt9O2Rv",
				"am8ucGxhdGZvcm1SZXF1aXJlPWZ1bmN0aW9uKF8yOGUpe3ZhciBfMjhmPShfMjhlLmNvbW1vbnx8W10p",
				"LmNvbmNhdChfMjhlW2Rvam8uX25hbWVdfHxfMjhlWyJkZWZhdWx0Il18fFtdKSx0ZW1wO3doaWxlKF8y",
				"OGYubGVuZ3RoKXtpZihsYW5nLmlzQXJyYXkodGVtcD1fMjhmLnNoaWZ0KCkpKXtkb2pvLnJlcXVpcmUu",
				"YXBwbHkoZG9qbyx0ZW1wKTt9ZWxzZXtkb2pvLnJlcXVpcmUodGVtcCk7fX19O2Rvam8ucmVxdWlyZUlm",
				"PWRvam8ucmVxdWlyZUFmdGVySWY9ZnVuY3Rpb24oXzI5MCxfMjkxLF8yOTIpe2lmKF8yOTApe2Rvam8u",
				"cmVxdWlyZShfMjkxLF8yOTIpO319O2Rvam8ucmVxdWlyZUxvY2FsaXphdGlvbj1mdW5jdGlvbihfMjkz",
				"LF8yOTQsXzI5NSl7XzIzYShbIi4uL2kxOG4iXSxmdW5jdGlvbihpMThuKXtpMThuLmdldExvY2FsaXph",
				"dGlvbihfMjkzLF8yOTQsXzI5NSk7fSk7fTtyZXR1cm4ge2V4dHJhY3RMZWdhY3lBcGlBcHBsaWNhdGlv",
				"bnM6XzI2OCxyZXF1aXJlOl8yNGUuZG9qb1JlcXVpcmVQbHVnaW4sbG9hZEluaXQ6XzI0Zn07fSk7fSwi",
				"ZG9qby9qc29uIjpmdW5jdGlvbigpe2RlZmluZShbIi4vaGFzIl0sZnVuY3Rpb24oaGFzKXsidXNlIHN0",
				"cmljdCI7dmFyIF8yOTY9dHlwZW9mIEpTT04hPSJ1bmRlZmluZWQiO2hhcy5hZGQoImpzb24tcGFyc2Ui",
				"LF8yOTYpO2hhcy5hZGQoImpzb24tc3RyaW5naWZ5IixfMjk2JiZKU09OLnN0cmluZ2lmeSh7YTowfSxm",
				"dW5jdGlvbihrLHYpe3JldHVybiB2fHwxO30pPT0ie1wiYVwiOjF9Iik7aWYoaGFzKCJqc29uLXN0cmlu",
				"Z2lmeSIpKXtyZXR1cm4gSlNPTjt9ZWxzZXt2YXIgXzI5Nz1mdW5jdGlvbihzdHIpe3JldHVybiAoIlwi",
				"IitzdHIucmVwbGFjZSgvKFsiXFxdKS9nLCJcXCQxIikrIlwiIikucmVwbGFjZSgvW1xmXS9nLCJcXGYi",
				"KS5yZXBsYWNlKC9bXGJdL2csIlxcYiIpLnJlcGxhY2UoL1tcbl0vZywiXFxuIikucmVwbGFjZSgvW1x0",
				"XS9nLCJcXHQiKS5yZXBsYWNlKC9bXHJdL2csIlxcciIpO307cmV0dXJuIHtwYXJzZTpoYXMoImpzb24t",
				"cGFyc2UiKT9KU09OLnBhcnNlOmZ1bmN0aW9uKHN0cixfMjk4KXtpZihfMjk4JiYhL14oW1xzXFtce10q",
				"KD86Iig/OlxcLnxbXiJdKSsifC0/XGRbXGRcLl0qKD86W0VlXVsrLV0/XGQrKT98bnVsbHx0cnVlfGZh",
				"bHNlfClbXHNcXVx9XSooPzosfDp8JCkpKyQvLnRlc3Qoc3RyKSl7dGhyb3cgbmV3IFN5bnRheEVycm9y",
				"KCJJbnZhbGlkIGNoYXJhY3RlcnMgaW4gSlNPTiIpO31yZXR1cm4gZXZhbCgiKCIrc3RyKyIpIik7fSxz",
				"dHJpbmdpZnk6ZnVuY3Rpb24oXzI5OSxfMjlhLF8yOWIpe3ZhciBfMjljO2lmKHR5cGVvZiBfMjlhPT0i",
				"c3RyaW5nIil7XzI5Yj1fMjlhO18yOWE9bnVsbDt9ZnVuY3Rpb24gXzI5ZChpdCxfMjllLGtleSl7aWYo",
				"XzI5YSl7aXQ9XzI5YShrZXksaXQpO312YXIgdmFsLF8yOWY9dHlwZW9mIGl0O2lmKF8yOWY9PSJudW1i",
				"ZXIiKXtyZXR1cm4gaXNGaW5pdGUoaXQpP2l0KyIiOiJudWxsIjt9aWYoXzI5Zj09ImJvb2xlYW4iKXty",
				"ZXR1cm4gaXQrIiI7fWlmKGl0PT09bnVsbCl7cmV0dXJuICJudWxsIjt9aWYodHlwZW9mIGl0PT0ic3Ry",
				"aW5nIil7cmV0dXJuIF8yOTcoaXQpO31pZihfMjlmPT0iZnVuY3Rpb24ifHxfMjlmPT0idW5kZWZpbmVk",
				"Iil7cmV0dXJuIF8yOWM7fWlmKHR5cGVvZiBpdC50b0pTT049PSJmdW5jdGlvbiIpe3JldHVybiBfMjlk",
				"KGl0LnRvSlNPTihrZXkpLF8yOWUsa2V5KTt9aWYoaXQgaW5zdGFuY2VvZiBEYXRlKXtyZXR1cm4gIlwi",
				"e0Z1bGxZZWFyfS17TW9udGgrfS17RGF0ZX1Ue0hvdXJzfTp7TWludXRlc306e1NlY29uZHN9WlwiIi5y",
				"ZXBsYWNlKC9ceyhcdyspKFwrKT9cfS9nLGZ1bmN0aW9uKHQscHJvcCxwbHVzKXt2YXIgbnVtPWl0WyJn",
				"ZXRVVEMiK3Byb3BdKCkrKHBsdXM/MTowKTtyZXR1cm4gbnVtPDEwPyIwIitudW06bnVtO30pO31pZihp",
				"dC52YWx1ZU9mKCkhPT1pdCl7cmV0dXJuIF8yOWQoaXQudmFsdWVPZigpLF8yOWUsa2V5KTt9dmFyIF8y",
				"YTA9XzI5Yj8oXzI5ZStfMjliKToiIjt2YXIgc2VwPV8yOWI/IiAiOiIiO3ZhciBfMmExPV8yOWI/Ilxu",
				"IjoiIjtpZihpdCBpbnN0YW5jZW9mIEFycmF5KXt2YXIgaXRsPWl0Lmxlbmd0aCxyZXM9W107Zm9yKGtl",
				"eT0wO2tleTxpdGw7a2V5Kyspe3ZhciBvYmo9aXRba2V5XTt2YWw9XzI5ZChvYmosXzJhMCxrZXkpO2lm",
				"KHR5cGVvZiB2YWwhPSJzdHJpbmciKXt2YWw9Im51bGwiO31yZXMucHVzaChfMmExK18yYTArdmFsKTt9",
				"cmV0dXJuICJbIityZXMuam9pbigiLCIpK18yYTErXzI5ZSsiXSI7fXZhciBfMmEyPVtdO2ZvcihrZXkg",
				"aW4gaXQpe3ZhciBfMmEzO2lmKHR5cGVvZiBrZXk9PSJudW1iZXIiKXtfMmEzPSJcIiIra2V5KyJcIiI7",
				"fWVsc2V7aWYodHlwZW9mIGtleT09InN0cmluZyIpe18yYTM9XzI5NyhrZXkpO31lbHNle2NvbnRpbnVl",
				"O319dmFsPV8yOWQoaXRba2V5XSxfMmEwLGtleSk7aWYodHlwZW9mIHZhbCE9InN0cmluZyIpe2NvbnRp",
				"bnVlO31fMmEyLnB1c2goXzJhMStfMmEwK18yYTMrIjoiK3NlcCt2YWwpO31yZXR1cm4gInsiK18yYTIu",
				"am9pbigiLCIpK18yYTErXzI5ZSsifSI7fTtyZXR1cm4gXzI5ZChfMjk5LCIiLCIiKTt9fTt9fSk7fSwi",
				"ZG9qby9fYmFzZS9kZWNsYXJlIjpmdW5jdGlvbigpe2RlZmluZShbIi4va2VybmVsIiwiLi4vaGFzIiwi",
				"Li9sYW5nIl0sZnVuY3Rpb24oZG9qbyxoYXMsbGFuZyl7dmFyIG1peD1sYW5nLm1peGluLG9wPU9iamVj",
				"dC5wcm90b3R5cGUsb3B0cz1vcC50b1N0cmluZyx4dG9yPW5ldyBGdW5jdGlvbixfMmE0PTAsXzJhNT0i",
				"Y29uc3RydWN0b3IiO2Z1bmN0aW9uIGVycihtc2csY2xzKXt0aHJvdyBuZXcgRXJyb3IoImRlY2xhcmUi",
				"KyhjbHM/IiAiK2NsczoiIikrIjogIittc2cpO307ZnVuY3Rpb24gXzJhNihfMmE3LF8yYTgpe3ZhciBf",
				"MmE5PVtdLF8yYWE9W3tjbHM6MCxyZWZzOltdfV0sXzJhYj17fSxfMmFjPTEsbD1fMmE3Lmxlbmd0aCxp",
				"PTAsaixsaW4sYmFzZSx0b3AsXzJhZCxyZWMsbmFtZSxyZWZzO2Zvcig7aTxsOysraSl7YmFzZT1fMmE3",
				"W2ldO2lmKCFiYXNlKXtlcnIoIm1peGluICMiK2krIiBpcyB1bmtub3duLiBEaWQgeW91IHVzZSBkb2pv",
				"LnJlcXVpcmUgdG8gcHVsbCBpdCBpbj8iLF8yYTgpO31lbHNle2lmKG9wdHMuY2FsbChiYXNlKSE9Iltv",
				"YmplY3QgRnVuY3Rpb25dIil7ZXJyKCJtaXhpbiAjIitpKyIgaXMgbm90IGEgY2FsbGFibGUgY29uc3Ry",
				"dWN0b3IuIixfMmE4KTt9fWxpbj1iYXNlLl9tZXRhP2Jhc2UuX21ldGEuYmFzZXM6W2Jhc2VdO3RvcD0w",
				"O2ZvcihqPWxpbi5sZW5ndGgtMTtqPj0wOy0tail7XzJhZD1saW5bal0ucHJvdG90eXBlO2lmKCFfMmFk",
				"Lmhhc093blByb3BlcnR5KCJkZWNsYXJlZENsYXNzIikpe18yYWQuZGVjbGFyZWRDbGFzcz0idW5pcU5h",
				"bWVfIisoXzJhNCsrKTt9bmFtZT1fMmFkLmRlY2xhcmVkQ2xhc3M7aWYoIV8yYWIuaGFzT3duUHJvcGVy",
				"dHkobmFtZSkpe18yYWJbbmFtZV09e2NvdW50OjAscmVmczpbXSxjbHM6bGluW2pdfTsrK18yYWM7fXJl",
				"Yz1fMmFiW25hbWVdO2lmKHRvcCYmdG9wIT09cmVjKXtyZWMucmVmcy5wdXNoKHRvcCk7Kyt0b3AuY291",
				"bnQ7fXRvcD1yZWM7fSsrdG9wLmNvdW50O18yYWFbMF0ucmVmcy5wdXNoKHRvcCk7fXdoaWxlKF8yYWEu",
				"bGVuZ3RoKXt0b3A9XzJhYS5wb3AoKTtfMmE5LnB1c2godG9wLmNscyk7LS1fMmFjO3doaWxlKHJlZnM9",
				"dG9wLnJlZnMscmVmcy5sZW5ndGg9PTEpe3RvcD1yZWZzWzBdO2lmKCF0b3B8fC0tdG9wLmNvdW50KXt0",
				"b3A9MDticmVhazt9XzJhOS5wdXNoKHRvcC5jbHMpOy0tXzJhYzt9aWYodG9wKXtmb3IoaT0wLGw9cmVm",
				"cy5sZW5ndGg7aTxsOysraSl7dG9wPXJlZnNbaV07aWYoIS0tdG9wLmNvdW50KXtfMmFhLnB1c2godG9w",
				"KTt9fX19aWYoXzJhYyl7ZXJyKCJjYW4ndCBidWlsZCBjb25zaXN0ZW50IGxpbmVhcml6YXRpb24iLF8y",
				"YTgpO31iYXNlPV8yYTdbMF07XzJhOVswXT1iYXNlP2Jhc2UuX21ldGEmJmJhc2U9PT1fMmE5W18yYTku",
				"bGVuZ3RoLWJhc2UuX21ldGEuYmFzZXMubGVuZ3RoXT9iYXNlLl9tZXRhLmJhc2VzLmxlbmd0aDoxOjA7",
				"cmV0dXJuIF8yYTk7fTtmdW5jdGlvbiBfMmFlKGFyZ3MsYSxmKXt2YXIgbmFtZSxfMmFmLF8yYjAsXzJi",
				"MSxtZXRhLGJhc2UsXzJiMixvcGYscG9zLF8yYjM9dGhpcy5faW5oZXJpdGVkPXRoaXMuX2luaGVyaXRl",
				"ZHx8e307aWYodHlwZW9mIGFyZ3M9PSJzdHJpbmciKXtuYW1lPWFyZ3M7YXJncz1hO2E9Zjt9Zj0wO18y",
				"YjE9YXJncy5jYWxsZWU7bmFtZT1uYW1lfHxfMmIxLm5vbTtpZighbmFtZSl7ZXJyKCJjYW4ndCBkZWR1",
				"Y2UgYSBuYW1lIHRvIGNhbGwgaW5oZXJpdGVkKCkiLHRoaXMuZGVjbGFyZWRDbGFzcyk7fW1ldGE9dGhp",
				"cy5jb25zdHJ1Y3Rvci5fbWV0YTtfMmIwPW1ldGEuYmFzZXM7cG9zPV8yYjMucDtpZihuYW1lIT1fMmE1",
				"KXtpZihfMmIzLmMhPT1fMmIxKXtwb3M9MDtiYXNlPV8yYjBbMF07bWV0YT1iYXNlLl9tZXRhO2lmKG1l",
				"dGEuaGlkZGVuW25hbWVdIT09XzJiMSl7XzJhZj1tZXRhLmNoYWlucztpZihfMmFmJiZ0eXBlb2YgXzJh",
				"ZltuYW1lXT09InN0cmluZyIpe2VycigiY2FsbGluZyBjaGFpbmVkIG1ldGhvZCB3aXRoIGluaGVyaXRl",
				"ZDogIituYW1lLHRoaXMuZGVjbGFyZWRDbGFzcyk7fWRve21ldGE9YmFzZS5fbWV0YTtfMmIyPWJhc2Uu",
				"cHJvdG90eXBlO2lmKG1ldGEmJihfMmIyW25hbWVdPT09XzJiMSYmXzJiMi5oYXNPd25Qcm9wZXJ0eShu",
				"YW1lKXx8bWV0YS5oaWRkZW5bbmFtZV09PT1fMmIxKSl7YnJlYWs7fX13aGlsZShiYXNlPV8yYjBbKytw",
				"b3NdKTtwb3M9YmFzZT9wb3M6LTE7fX1iYXNlPV8yYjBbKytwb3NdO2lmKGJhc2Upe18yYjI9YmFzZS5w",
				"cm90b3R5cGU7aWYoYmFzZS5fbWV0YSYmXzJiMi5oYXNPd25Qcm9wZXJ0eShuYW1lKSl7Zj1fMmIyW25h",
				"bWVdO31lbHNle29wZj1vcFtuYW1lXTtkb3tfMmIyPWJhc2UucHJvdG90eXBlO2Y9XzJiMltuYW1lXTtp",
				"ZihmJiYoYmFzZS5fbWV0YT9fMmIyLmhhc093blByb3BlcnR5KG5hbWUpOmYhPT1vcGYpKXticmVhazt9",
				"fXdoaWxlKGJhc2U9XzJiMFsrK3Bvc10pO319Zj1iYXNlJiZmfHxvcFtuYW1lXTt9ZWxzZXtpZihfMmIz",
				"LmMhPT1fMmIxKXtwb3M9MDttZXRhPV8yYjBbMF0uX21ldGE7aWYobWV0YSYmbWV0YS5jdG9yIT09XzJi",
				"MSl7XzJhZj1tZXRhLmNoYWlucztpZighXzJhZnx8XzJhZi5jb25zdHJ1Y3RvciE9PSJtYW51YWwiKXtl",
				"cnIoImNhbGxpbmcgY2hhaW5lZCBjb25zdHJ1Y3RvciB3aXRoIGluaGVyaXRlZCIsdGhpcy5kZWNsYXJl",
				"ZENsYXNzKTt9d2hpbGUoYmFzZT1fMmIwWysrcG9zXSl7bWV0YT1iYXNlLl9tZXRhO2lmKG1ldGEmJm1l",
				"dGEuY3Rvcj09PV8yYjEpe2JyZWFrO319cG9zPWJhc2U/cG9zOi0xO319d2hpbGUoYmFzZT1fMmIwWysr",
				"cG9zXSl7bWV0YT1iYXNlLl9tZXRhO2Y9bWV0YT9tZXRhLmN0b3I6YmFzZTtpZihmKXticmVhazt9fWY9",
				"YmFzZSYmZjt9XzJiMy5jPWY7XzJiMy5wPXBvcztpZihmKXtyZXR1cm4gYT09PXRydWU/ZjpmLmFwcGx5",
				"KHRoaXMsYXx8YXJncyk7fX07ZnVuY3Rpb24gXzJiNChuYW1lLGFyZ3Mpe2lmKHR5cGVvZiBuYW1lPT0i",
				"c3RyaW5nIil7cmV0dXJuIHRoaXMuX19pbmhlcml0ZWQobmFtZSxhcmdzLHRydWUpO31yZXR1cm4gdGhp",
				"cy5fX2luaGVyaXRlZChuYW1lLHRydWUpO307ZnVuY3Rpb24gXzJiNShhcmdzLGExLGEyKXt2YXIgZj10",
				"aGlzLmdldEluaGVyaXRlZChhcmdzLGExKTtpZihmKXtyZXR1cm4gZi5hcHBseSh0aGlzLGEyfHxhMXx8",
				"YXJncyk7fX07dmFyIF8yYjY9ZG9qby5jb25maWcuaXNEZWJ1Zz9fMmI1Ol8yYWU7ZnVuY3Rpb24gXzJi",
				"NyhjbHMpe3ZhciBfMmI4PXRoaXMuY29uc3RydWN0b3IuX21ldGEuYmFzZXM7Zm9yKHZhciBpPTAsbD1f",
				"MmI4Lmxlbmd0aDtpPGw7KytpKXtpZihfMmI4W2ldPT09Y2xzKXtyZXR1cm4gdHJ1ZTt9fXJldHVybiB0",
				"aGlzIGluc3RhbmNlb2YgY2xzO307ZnVuY3Rpb24gXzJiOShfMmJhLF8yYmIpe2Zvcih2YXIgbmFtZSBp",
				"biBfMmJiKXtpZihuYW1lIT1fMmE1JiZfMmJiLmhhc093blByb3BlcnR5KG5hbWUpKXtfMmJhW25hbWVd",
				"PV8yYmJbbmFtZV07fX1pZihoYXMoImJ1Zy1mb3ItaW4tc2tpcHMtc2hhZG93ZWQiKSl7Zm9yKHZhciBf",
				"MmJjPWxhbmcuX2V4dHJhTmFtZXMsaT1fMmJjLmxlbmd0aDtpOyl7bmFtZT1fMmJjWy0taV07aWYobmFt",
				"ZSE9XzJhNSYmXzJiYi5oYXNPd25Qcm9wZXJ0eShuYW1lKSl7XzJiYVtuYW1lXT1fMmJiW25hbWVdO319",
				"fX07ZnVuY3Rpb24gXzJiZChfMmJlLF8yYmYpe3ZhciBuYW1lLHQ7Zm9yKG5hbWUgaW4gXzJiZil7dD1f",
				"MmJmW25hbWVdO2lmKCh0IT09b3BbbmFtZV18fCEobmFtZSBpbiBvcCkpJiZuYW1lIT1fMmE1KXtpZihv",
				"cHRzLmNhbGwodCk9PSJbb2JqZWN0IEZ1bmN0aW9uXSIpe3Qubm9tPW5hbWU7fV8yYmVbbmFtZV09dDt9",
				"fWlmKGhhcygiYnVnLWZvci1pbi1za2lwcy1zaGFkb3dlZCIpKXtmb3IodmFyIF8yYzA9bGFuZy5fZXh0",
				"cmFOYW1lcyxpPV8yYzAubGVuZ3RoO2k7KXtuYW1lPV8yYzBbLS1pXTt0PV8yYmZbbmFtZV07aWYoKHQh",
				"PT1vcFtuYW1lXXx8IShuYW1lIGluIG9wKSkmJm5hbWUhPV8yYTUpe2lmKG9wdHMuY2FsbCh0KT09Iltv",
				"YmplY3QgRnVuY3Rpb25dIil7dC5ub209bmFtZTt9XzJiZVtuYW1lXT10O319fXJldHVybiBfMmJlO307",
				"ZnVuY3Rpb24gXzJjMShfMmMyKXtfMmMzLnNhZmVNaXhpbih0aGlzLnByb3RvdHlwZSxfMmMyKTtyZXR1",
				"cm4gdGhpczt9O2Z1bmN0aW9uIF8yYzQoXzJjNSxfMmM2KXtyZXR1cm4gZnVuY3Rpb24oKXt2YXIgYT1h",
				"cmd1bWVudHMsYXJncz1hLGEwPWFbMF0sZixpLG0sbD1fMmM1Lmxlbmd0aCxfMmM3O2lmKCEodGhpcyBp",
				"bnN0YW5jZW9mIGEuY2FsbGVlKSl7cmV0dXJuIF8yYzgoYSk7fWlmKF8yYzYmJihhMCYmYTAucHJlYW1i",
				"bGV8fHRoaXMucHJlYW1ibGUpKXtfMmM3PW5ldyBBcnJheShfMmM1Lmxlbmd0aCk7XzJjN1swXT1hO2Zv",
				"cihpPTA7Oyl7YTA9YVswXTtpZihhMCl7Zj1hMC5wcmVhbWJsZTtpZihmKXthPWYuYXBwbHkodGhpcyxh",
				"KXx8YTt9fWY9XzJjNVtpXS5wcm90b3R5cGU7Zj1mLmhhc093blByb3BlcnR5KCJwcmVhbWJsZSIpJiZm",
				"LnByZWFtYmxlO2lmKGYpe2E9Zi5hcHBseSh0aGlzLGEpfHxhO31pZigrK2k9PWwpe2JyZWFrO31fMmM3",
				"W2ldPWE7fX1mb3IoaT1sLTE7aT49MDstLWkpe2Y9XzJjNVtpXTttPWYuX21ldGE7Zj1tP20uY3Rvcjpm",
				"O2lmKGYpe2YuYXBwbHkodGhpcyxfMmM3P18yYzdbaV06YSk7fX1mPXRoaXMucG9zdHNjcmlwdDtpZihm",
				"KXtmLmFwcGx5KHRoaXMsYXJncyk7fX07fTtmdW5jdGlvbiBfMmM5KGN0b3IsXzJjYSl7cmV0dXJuIGZ1",
				"bmN0aW9uKCl7dmFyIGE9YXJndW1lbnRzLHQ9YSxhMD1hWzBdLGY7aWYoISh0aGlzIGluc3RhbmNlb2Yg",
				"YS5jYWxsZWUpKXtyZXR1cm4gXzJjOChhKTt9aWYoXzJjYSl7aWYoYTApe2Y9YTAucHJlYW1ibGU7aWYo",
				"Zil7dD1mLmFwcGx5KHRoaXMsdCl8fHQ7fX1mPXRoaXMucHJlYW1ibGU7aWYoZil7Zi5hcHBseSh0aGlz",
				"LHQpO319aWYoY3Rvcil7Y3Rvci5hcHBseSh0aGlzLGEpO31mPXRoaXMucG9zdHNjcmlwdDtpZihmKXtm",
				"LmFwcGx5KHRoaXMsYSk7fX07fTtmdW5jdGlvbiBfMmNiKF8yY2Mpe3JldHVybiBmdW5jdGlvbigpe3Zh",
				"ciBhPWFyZ3VtZW50cyxpPTAsZixtO2lmKCEodGhpcyBpbnN0YW5jZW9mIGEuY2FsbGVlKSl7cmV0dXJu",
				"IF8yYzgoYSk7fWZvcig7Zj1fMmNjW2ldOysraSl7bT1mLl9tZXRhO2Y9bT9tLmN0b3I6ZjtpZihmKXtm",
				"LmFwcGx5KHRoaXMsYSk7YnJlYWs7fX1mPXRoaXMucG9zdHNjcmlwdDtpZihmKXtmLmFwcGx5KHRoaXMs",
				"YSk7fX07fTtmdW5jdGlvbiBfMmNkKG5hbWUsXzJjZSxfMmNmKXtyZXR1cm4gZnVuY3Rpb24oKXt2YXIg",
				"YixtLGYsaT0wLHN0ZXA9MTtpZihfMmNmKXtpPV8yY2UubGVuZ3RoLTE7c3RlcD0tMTt9Zm9yKDtiPV8y",
				"Y2VbaV07aSs9c3RlcCl7bT1iLl9tZXRhO2Y9KG0/bS5oaWRkZW46Yi5wcm90b3R5cGUpW25hbWVdO2lm",
				"KGYpe2YuYXBwbHkodGhpcyxhcmd1bWVudHMpO319fTt9O2Z1bmN0aW9uIF8yZDAoY3Rvcil7eHRvci5w",
				"cm90b3R5cGU9Y3Rvci5wcm90b3R5cGU7dmFyIHQ9bmV3IHh0b3I7eHRvci5wcm90b3R5cGU9bnVsbDty",
				"ZXR1cm4gdDt9O2Z1bmN0aW9uIF8yYzgoYXJncyl7dmFyIGN0b3I9YXJncy5jYWxsZWUsdD1fMmQwKGN0",
				"b3IpO2N0b3IuYXBwbHkodCxhcmdzKTtyZXR1cm4gdDt9O2Z1bmN0aW9uIF8yYzMoXzJkMSxfMmQyLF8y",
				"ZDMpe2lmKHR5cGVvZiBfMmQxIT0ic3RyaW5nIil7XzJkMz1fMmQyO18yZDI9XzJkMTtfMmQxPSIiO31f",
				"MmQzPV8yZDN8fHt9O3ZhciBfMmQ0LGksdCxjdG9yLG5hbWUsXzJkNSxfMmQ2LF8yZDc9MSxfMmQ4PV8y",
				"ZDI7aWYob3B0cy5jYWxsKF8yZDIpPT0iW29iamVjdCBBcnJheV0iKXtfMmQ1PV8yYTYoXzJkMixfMmQx",
				"KTt0PV8yZDVbMF07XzJkNz1fMmQ1Lmxlbmd0aC10O18yZDI9XzJkNVtfMmQ3XTt9ZWxzZXtfMmQ1PVsw",
				"XTtpZihfMmQyKXtpZihvcHRzLmNhbGwoXzJkMik9PSJbb2JqZWN0IEZ1bmN0aW9uXSIpe3Q9XzJkMi5f",
				"bWV0YTtfMmQ1PV8yZDUuY29uY2F0KHQ/dC5iYXNlczpfMmQyKTt9ZWxzZXtlcnIoImJhc2UgY2xhc3Mg",
				"aXMgbm90IGEgY2FsbGFibGUgY29uc3RydWN0b3IuIixfMmQxKTt9fWVsc2V7aWYoXzJkMiE9PW51bGwp",
				"e2VycigidW5rbm93biBiYXNlIGNsYXNzLiBEaWQgeW91IHVzZSBkb2pvLnJlcXVpcmUgdG8gcHVsbCBp",
				"dCBpbj8iLF8yZDEpO319fWlmKF8yZDIpe2ZvcihpPV8yZDctMTs7LS1pKXtfMmQ0PV8yZDAoXzJkMik7",
				"aWYoIWkpe2JyZWFrO310PV8yZDVbaV07KHQuX21ldGE/XzJiOTptaXgpKF8yZDQsdC5wcm90b3R5cGUp",
				"O2N0b3I9bmV3IEZ1bmN0aW9uO2N0b3Iuc3VwZXJjbGFzcz1fMmQyO2N0b3IucHJvdG90eXBlPV8yZDQ7",
				"XzJkMj1fMmQ0LmNvbnN0cnVjdG9yPWN0b3I7fX1lbHNle18yZDQ9e307fV8yYzMuc2FmZU1peGluKF8y",
				"ZDQsXzJkMyk7dD1fMmQzLmNvbnN0cnVjdG9yO2lmKHQhPT1vcC5jb25zdHJ1Y3Rvcil7dC5ub209XzJh",
				"NTtfMmQ0LmNvbnN0cnVjdG9yPXQ7fWZvcihpPV8yZDctMTtpOy0taSl7dD1fMmQ1W2ldLl9tZXRhO2lm",
				"KHQmJnQuY2hhaW5zKXtfMmQ2PW1peChfMmQ2fHx7fSx0LmNoYWlucyk7fX1pZihfMmQ0WyItY2hhaW5z",
				"LSJdKXtfMmQ2PW1peChfMmQ2fHx7fSxfMmQ0WyItY2hhaW5zLSJdKTt9dD0hXzJkNnx8IV8yZDYuaGFz",
				"T3duUHJvcGVydHkoXzJhNSk7XzJkNVswXT1jdG9yPShfMmQ2JiZfMmQ2LmNvbnN0cnVjdG9yPT09Im1h",
				"bnVhbCIpP18yY2IoXzJkNSk6KF8yZDUubGVuZ3RoPT0xP18yYzkoXzJkMy5jb25zdHJ1Y3Rvcix0KTpf",
				"MmM0KF8yZDUsdCkpO2N0b3IuX21ldGE9e2Jhc2VzOl8yZDUsaGlkZGVuOl8yZDMsY2hhaW5zOl8yZDYs",
				"cGFyZW50czpfMmQ4LGN0b3I6XzJkMy5jb25zdHJ1Y3Rvcn07Y3Rvci5zdXBlcmNsYXNzPV8yZDImJl8y",
				"ZDIucHJvdG90eXBlO2N0b3IuZXh0ZW5kPV8yYzE7Y3Rvci5wcm90b3R5cGU9XzJkNDtfMmQ0LmNvbnN0",
				"cnVjdG9yPWN0b3I7XzJkNC5nZXRJbmhlcml0ZWQ9XzJiNDtfMmQ0LmlzSW5zdGFuY2VPZj1fMmI3O18y",
				"ZDQuaW5oZXJpdGVkPV8yYjY7XzJkNC5fX2luaGVyaXRlZD1fMmFlO2lmKF8yZDEpe18yZDQuZGVjbGFy",
				"ZWRDbGFzcz1fMmQxO2xhbmcuc2V0T2JqZWN0KF8yZDEsY3Rvcik7fWlmKF8yZDYpe2ZvcihuYW1lIGlu",
				"IF8yZDYpe2lmKF8yZDRbbmFtZV0mJnR5cGVvZiBfMmQ2W25hbWVdPT0ic3RyaW5nIiYmbmFtZSE9XzJh",
				"NSl7dD1fMmQ0W25hbWVdPV8yY2QobmFtZSxfMmQ1LF8yZDZbbmFtZV09PT0iYWZ0ZXIiKTt0Lm5vbT1u",
				"YW1lO319fXJldHVybiBjdG9yO307ZG9qby5zYWZlTWl4aW49XzJjMy5zYWZlTWl4aW49XzJiZDtkb2pv",
				"LmRlY2xhcmU9XzJjMztyZXR1cm4gXzJjMzt9KTt9LCJkb2pvL2RvbSI6ZnVuY3Rpb24oKXtkZWZpbmUo",
				"WyIuL19iYXNlL3NuaWZmIiwiLi9fYmFzZS9sYW5nIiwiLi9fYmFzZS93aW5kb3ciXSxmdW5jdGlvbiho",
				"YXMsbGFuZyx3aW4pe3RyeXtkb2N1bWVudC5leGVjQ29tbWFuZCgiQmFja2dyb3VuZEltYWdlQ2FjaGUi",
				"LGZhbHNlLHRydWUpO31jYXRjaChlKXt9dmFyIGRvbT17fTtpZihoYXMoImllIikpe2RvbS5ieUlkPWZ1",
				"bmN0aW9uKGlkLGRvYyl7aWYodHlwZW9mIGlkIT0ic3RyaW5nIil7cmV0dXJuIGlkO312YXIgXzJkOT1k",
				"b2N8fHdpbi5kb2MsdGU9aWQmJl8yZDkuZ2V0RWxlbWVudEJ5SWQoaWQpO2lmKHRlJiYodGUuYXR0cmli",
				"dXRlcy5pZC52YWx1ZT09aWR8fHRlLmlkPT1pZCkpe3JldHVybiB0ZTt9ZWxzZXt2YXIgZWxlcz1fMmQ5",
				"LmFsbFtpZF07aWYoIWVsZXN8fGVsZXMubm9kZU5hbWUpe2VsZXM9W2VsZXNdO312YXIgaT0wO3doaWxl",
				"KCh0ZT1lbGVzW2krK10pKXtpZigodGUuYXR0cmlidXRlcyYmdGUuYXR0cmlidXRlcy5pZCYmdGUuYXR0",
				"cmlidXRlcy5pZC52YWx1ZT09aWQpfHx0ZS5pZD09aWQpe3JldHVybiB0ZTt9fX19O31lbHNle2RvbS5i",
				"eUlkPWZ1bmN0aW9uKGlkLGRvYyl7cmV0dXJuICgodHlwZW9mIGlkPT0ic3RyaW5nIik/KGRvY3x8d2lu",
				"LmRvYykuZ2V0RWxlbWVudEJ5SWQoaWQpOmlkKXx8bnVsbDt9O31kb20uaXNEZXNjZW5kYW50PWZ1bmN0",
				"aW9uKG5vZGUsXzJkYSl7dHJ5e25vZGU9ZG9tLmJ5SWQobm9kZSk7XzJkYT1kb20uYnlJZChfMmRhKTt3",
				"aGlsZShub2RlKXtpZihub2RlPT1fMmRhKXtyZXR1cm4gdHJ1ZTt9bm9kZT1ub2RlLnBhcmVudE5vZGU7",
				"fX1jYXRjaChlKXt9cmV0dXJuIGZhbHNlO307ZG9tLnNldFNlbGVjdGFibGU9ZnVuY3Rpb24obm9kZSxf",
				"MmRiKXtub2RlPWRvbS5ieUlkKG5vZGUpO2lmKGhhcygibW96aWxsYSIpKXtub2RlLnN0eWxlLk1velVz",
				"ZXJTZWxlY3Q9XzJkYj8iIjoibm9uZSI7fWVsc2V7aWYoaGFzKCJraHRtbCIpfHxoYXMoIndlYmtpdCIp",
				"KXtub2RlLnN0eWxlLktodG1sVXNlclNlbGVjdD1fMmRiPyJhdXRvIjoibm9uZSI7fWVsc2V7aWYoaGFz",
				"KCJpZSIpKXt2YXIgdj0obm9kZS51bnNlbGVjdGFibGU9XzJkYj8iIjoib24iKSxjcz1ub2RlLmdldEVs",
				"ZW1lbnRzQnlUYWdOYW1lKCIqIiksaT0wLGw9Y3MubGVuZ3RoO2Zvcig7aTxsOysraSl7Y3MuaXRlbShp",
				"KS51bnNlbGVjdGFibGU9djt9fX19fTtyZXR1cm4gZG9tO30pO30sImRvam8vX2Jhc2UvYnJvd3NlciI6",
				"ZnVuY3Rpb24oKXtpZihyZXF1aXJlLmhhcyl7cmVxdWlyZS5oYXMuYWRkKCJjb25maWctc2VsZWN0b3JF",
				"bmdpbmUiLCJhY21lIik7fWRlZmluZSgiZG9qby9fYmFzZS9icm93c2VyIixbIi4uL3JlYWR5IiwiLi9r",
				"ZXJuZWwiLCIuL2Nvbm5lY3QiLCIuL3VubG9hZCIsIi4vd2luZG93IiwiLi9ldmVudCIsIi4vaHRtbCIs",
				"Ii4vTm9kZUxpc3QiLCIuLi9xdWVyeSIsIi4veGhyIiwiLi9meCJdLGZ1bmN0aW9uKGRvam8pe3JldHVy",
				"biBkb2pvO30pO30sImRvam8vc2VsZWN0b3IvYWNtZSI6ZnVuY3Rpb24oKXtkZWZpbmUoWyIuLi9fYmFz",
				"ZS9rZXJuZWwiLCIuLi9oYXMiLCIuLi9kb20iLCIuLi9fYmFzZS9zbmlmZiIsIi4uL19iYXNlL2FycmF5",
				"IiwiLi4vX2Jhc2UvbGFuZyIsIi4uL19iYXNlL3dpbmRvdyJdLGZ1bmN0aW9uKGRvam8saGFzLGRvbSl7",
				"dmFyIHRyaW09ZG9qby50cmltO3ZhciBlYWNoPWRvam8uZm9yRWFjaDt2YXIgXzJkYz1mdW5jdGlvbigp",
				"e3JldHVybiBkb2pvLmRvYzt9O3ZhciBfMmRkPSgoZG9qby5pc1dlYktpdHx8ZG9qby5pc01vemlsbGEp",
				"JiYoKF8yZGMoKS5jb21wYXRNb2RlKT09IkJhY2tDb21wYXQiKSk7dmFyIF8yZGU9Ij5+KyI7dmFyIF8y",
				"ZGY9ZmFsc2U7dmFyIF8yZTA9ZnVuY3Rpb24oKXtyZXR1cm4gdHJ1ZTt9O3ZhciBfMmUxPWZ1bmN0aW9u",
				"KF8yZTIpe2lmKF8yZGUuaW5kZXhPZihfMmUyLnNsaWNlKC0xKSk+PTApe18yZTIrPSIgKiAiO31lbHNl",
				"e18yZTIrPSIgIjt9dmFyIHRzPWZ1bmN0aW9uKHMsZSl7cmV0dXJuIHRyaW0oXzJlMi5zbGljZShzLGUp",
				"KTt9O3ZhciBfMmUzPVtdO3ZhciBfMmU0PS0xLF8yZTU9LTEsXzJlNj0tMSxfMmU3PS0xLF8yZTg9LTEs",
				"aW5JZD0tMSxfMmU5PS0xLGxjPSIiLGNjPSIiLF8yZWE7dmFyIHg9MCxxbD1fMmUyLmxlbmd0aCxfMmVi",
				"PW51bGwsXzJlYz1udWxsO3ZhciBfMmVkPWZ1bmN0aW9uKCl7aWYoXzJlOT49MCl7dmFyIHR2PShfMmU5",
				"PT14KT9udWxsOnRzKF8yZTkseCk7XzJlYlsoXzJkZS5pbmRleE9mKHR2KTwwKT8idGFnIjoib3BlciJd",
				"PXR2O18yZTk9LTE7fX07dmFyIF8yZWU9ZnVuY3Rpb24oKXtpZihpbklkPj0wKXtfMmViLmlkPXRzKGlu",
				"SWQseCkucmVwbGFjZSgvXFwvZywiIik7aW5JZD0tMTt9fTt2YXIgXzJlZj1mdW5jdGlvbigpe2lmKF8y",
				"ZTg+PTApe18yZWIuY2xhc3Nlcy5wdXNoKHRzKF8yZTgrMSx4KS5yZXBsYWNlKC9cXC9nLCIiKSk7XzJl",
				"OD0tMTt9fTt2YXIgXzJmMD1mdW5jdGlvbigpe18yZWUoKTtfMmVkKCk7XzJlZigpO307dmFyIF8yZjE9",
				"ZnVuY3Rpb24oKXtfMmYwKCk7aWYoXzJlNz49MCl7XzJlYi5wc2V1ZG9zLnB1c2goe25hbWU6dHMoXzJl",
				"NysxLHgpfSk7fV8yZWIubG9vcHM9KF8yZWIucHNldWRvcy5sZW5ndGh8fF8yZWIuYXR0cnMubGVuZ3Ro",
				"fHxfMmViLmNsYXNzZXMubGVuZ3RoKTtfMmViLm9xdWVyeT1fMmViLnF1ZXJ5PXRzKF8yZWEseCk7XzJl",
				"Yi5vdGFnPV8yZWIudGFnPShfMmViWyJvcGVyIl0pP251bGw6KF8yZWIudGFnfHwiKiIpO2lmKF8yZWIu",
				"dGFnKXtfMmViLnRhZz1fMmViLnRhZy50b1VwcGVyQ2FzZSgpO31pZihfMmUzLmxlbmd0aCYmKF8yZTNb",
				"XzJlMy5sZW5ndGgtMV0ub3Blcikpe18yZWIuaW5maXhPcGVyPV8yZTMucG9wKCk7XzJlYi5xdWVyeT1f",
				"MmViLmluZml4T3Blci5xdWVyeSsiICIrXzJlYi5xdWVyeTt9XzJlMy5wdXNoKF8yZWIpO18yZWI9bnVs",
				"bDt9O2Zvcig7bGM9Y2MsY2M9XzJlMi5jaGFyQXQoeCkseDxxbDt4Kyspe2lmKGxjPT0iXFwiKXtjb250",
				"aW51ZTt9aWYoIV8yZWIpe18yZWE9eDtfMmViPXtxdWVyeTpudWxsLHBzZXVkb3M6W10sYXR0cnM6W10s",
				"Y2xhc3NlczpbXSx0YWc6bnVsbCxvcGVyOm51bGwsaWQ6bnVsbCxnZXRUYWc6ZnVuY3Rpb24oKXtyZXR1",
				"cm4gKF8yZGYpP3RoaXMub3RhZzp0aGlzLnRhZzt9fTtfMmU5PXg7fWlmKF8yZTQ+PTApe2lmKGNjPT0i",
				"XSIpe2lmKCFfMmVjLmF0dHIpe18yZWMuYXR0cj10cyhfMmU0KzEseCk7fWVsc2V7XzJlYy5tYXRjaEZv",
				"cj10cygoXzJlNnx8XzJlNCsxKSx4KTt9dmFyIGNtZj1fMmVjLm1hdGNoRm9yO2lmKGNtZil7aWYoKGNt",
				"Zi5jaGFyQXQoMCk9PSJcIiIpfHwoY21mLmNoYXJBdCgwKT09IiciKSl7XzJlYy5tYXRjaEZvcj1jbWYu",
				"c2xpY2UoMSwtMSk7fX1fMmViLmF0dHJzLnB1c2goXzJlYyk7XzJlYz1udWxsO18yZTQ9XzJlNj0tMTt9",
				"ZWxzZXtpZihjYz09Ij0iKXt2YXIgXzJmMj0oInx+XiQqIi5pbmRleE9mKGxjKT49MCk/bGM6IiI7XzJl",
				"Yy50eXBlPV8yZjIrY2M7XzJlYy5hdHRyPXRzKF8yZTQrMSx4LV8yZjIubGVuZ3RoKTtfMmU2PXgrMTt9",
				"fX1lbHNle2lmKF8yZTU+PTApe2lmKGNjPT0iKSIpe2lmKF8yZTc+PTApe18yZWMudmFsdWU9dHMoXzJl",
				"NSsxLHgpO31fMmU3PV8yZTU9LTE7fX1lbHNle2lmKGNjPT0iIyIpe18yZjAoKTtpbklkPXgrMTt9ZWxz",
				"ZXtpZihjYz09Ii4iKXtfMmYwKCk7XzJlOD14O31lbHNle2lmKGNjPT0iOiIpe18yZjAoKTtfMmU3PXg7",
				"fWVsc2V7aWYoY2M9PSJbIil7XzJmMCgpO18yZTQ9eDtfMmVjPXt9O31lbHNle2lmKGNjPT0iKCIpe2lm",
				"KF8yZTc+PTApe18yZWM9e25hbWU6dHMoXzJlNysxLHgpLHZhbHVlOm51bGx9O18yZWIucHNldWRvcy5w",
				"dXNoKF8yZWMpO31fMmU1PXg7fWVsc2V7aWYoKGNjPT0iICIpJiYobGMhPWNjKSl7XzJmMSgpO319fX19",
				"fX19fXJldHVybiBfMmUzO307dmFyIF8yZjM9ZnVuY3Rpb24oXzJmNCxfMmY1KXtpZighXzJmNCl7cmV0",
				"dXJuIF8yZjU7fWlmKCFfMmY1KXtyZXR1cm4gXzJmNDt9cmV0dXJuIGZ1bmN0aW9uKCl7cmV0dXJuIF8y",
				"ZjQuYXBwbHkod2luZG93LGFyZ3VtZW50cykmJl8yZjUuYXBwbHkod2luZG93LGFyZ3VtZW50cyk7fTt9",
				"O3ZhciBfMmY2PWZ1bmN0aW9uKGksYXJyKXt2YXIgcj1hcnJ8fFtdO2lmKGkpe3IucHVzaChpKTt9cmV0",
				"dXJuIHI7fTt2YXIgXzJmNz1mdW5jdGlvbihuKXtyZXR1cm4gKDE9PW4ubm9kZVR5cGUpO307dmFyIF8y",
				"Zjg9IiI7dmFyIF8yZjk9ZnVuY3Rpb24oZWxlbSxhdHRyKXtpZighZWxlbSl7cmV0dXJuIF8yZjg7fWlm",
				"KGF0dHI9PSJjbGFzcyIpe3JldHVybiBlbGVtLmNsYXNzTmFtZXx8XzJmODt9aWYoYXR0cj09ImZvciIp",
				"e3JldHVybiBlbGVtLmh0bWxGb3J8fF8yZjg7fWlmKGF0dHI9PSJzdHlsZSIpe3JldHVybiBlbGVtLnN0",
				"eWxlLmNzc1RleHR8fF8yZjg7fXJldHVybiAoXzJkZj9lbGVtLmdldEF0dHJpYnV0ZShhdHRyKTplbGVt",
				"LmdldEF0dHJpYnV0ZShhdHRyLDIpKXx8XzJmODt9O3ZhciBfMmZhPXsiKj0iOmZ1bmN0aW9uKGF0dHIs",
				"XzJmYil7cmV0dXJuIGZ1bmN0aW9uKGVsZW0pe3JldHVybiAoXzJmOShlbGVtLGF0dHIpLmluZGV4T2Yo",
				"XzJmYik+PTApO307fSwiXj0iOmZ1bmN0aW9uKGF0dHIsXzJmYyl7cmV0dXJuIGZ1bmN0aW9uKGVsZW0p",
				"e3JldHVybiAoXzJmOShlbGVtLGF0dHIpLmluZGV4T2YoXzJmYyk9PTApO307fSwiJD0iOmZ1bmN0aW9u",
				"KGF0dHIsXzJmZCl7cmV0dXJuIGZ1bmN0aW9uKGVsZW0pe3ZhciBlYT0iICIrXzJmOShlbGVtLGF0dHIp",
				"O3JldHVybiAoZWEubGFzdEluZGV4T2YoXzJmZCk9PShlYS5sZW5ndGgtXzJmZC5sZW5ndGgpKTt9O30s",
				"In49IjpmdW5jdGlvbihhdHRyLF8yZmUpe3ZhciB0dmFsPSIgIitfMmZlKyIgIjtyZXR1cm4gZnVuY3Rp",
				"b24oZWxlbSl7dmFyIGVhPSIgIitfMmY5KGVsZW0sYXR0cikrIiAiO3JldHVybiAoZWEuaW5kZXhPZih0",
				"dmFsKT49MCk7fTt9LCJ8PSI6ZnVuY3Rpb24oYXR0cixfMmZmKXt2YXIgXzMwMD1fMmZmKyItIjtyZXR1",
				"cm4gZnVuY3Rpb24oZWxlbSl7dmFyIGVhPV8yZjkoZWxlbSxhdHRyKTtyZXR1cm4gKChlYT09XzJmZil8",
				"fChlYS5pbmRleE9mKF8zMDApPT0wKSk7fTt9LCI9IjpmdW5jdGlvbihhdHRyLF8zMDEpe3JldHVybiBm",
				"dW5jdGlvbihlbGVtKXtyZXR1cm4gKF8yZjkoZWxlbSxhdHRyKT09XzMwMSk7fTt9fTt2YXIgXzMwMj0o",
				"dHlwZW9mIF8yZGMoKS5maXJzdENoaWxkLm5leHRFbGVtZW50U2libGluZz09InVuZGVmaW5lZCIpO3Zh",
				"ciBfMzAzPSFfMzAyPyJuZXh0RWxlbWVudFNpYmxpbmciOiJuZXh0U2libGluZyI7dmFyIF8zMDQ9IV8z",
				"MDI/InByZXZpb3VzRWxlbWVudFNpYmxpbmciOiJwcmV2aW91c1NpYmxpbmciO3ZhciBfMzA1PShfMzAy",
				"P18yZjc6XzJlMCk7dmFyIF8zMDY9ZnVuY3Rpb24obm9kZSl7d2hpbGUobm9kZT1ub2RlW18zMDRdKXtp",
				"ZihfMzA1KG5vZGUpKXtyZXR1cm4gZmFsc2U7fX1yZXR1cm4gdHJ1ZTt9O3ZhciBfMzA3PWZ1bmN0aW9u",
				"KG5vZGUpe3doaWxlKG5vZGU9bm9kZVtfMzAzXSl7aWYoXzMwNShub2RlKSl7cmV0dXJuIGZhbHNlO319",
				"cmV0dXJuIHRydWU7fTt2YXIgXzMwOD1mdW5jdGlvbihub2RlKXt2YXIgcm9vdD1ub2RlLnBhcmVudE5v",
				"ZGU7dmFyIGk9MCx0cmV0PXJvb3QuY2hpbGRyZW58fHJvb3QuY2hpbGROb2RlcyxjaT0obm9kZVsiX2ki",
				"XXx8LTEpLGNsPShyb290WyJfbCJdfHwtMSk7aWYoIXRyZXQpe3JldHVybiAtMTt9dmFyIGw9dHJldC5s",
				"ZW5ndGg7aWYoY2w9PWwmJmNpPj0wJiZjbD49MCl7cmV0dXJuIGNpO31yb290WyJfbCJdPWw7Y2k9LTE7",
				"Zm9yKHZhciB0ZT1yb290WyJmaXJzdEVsZW1lbnRDaGlsZCJdfHxyb290WyJmaXJzdENoaWxkIl07dGU7",
				"dGU9dGVbXzMwM10pe2lmKF8zMDUodGUpKXt0ZVsiX2kiXT0rK2k7aWYobm9kZT09PXRlKXtjaT1pO319",
				"fXJldHVybiBjaTt9O3ZhciBfMzA5PWZ1bmN0aW9uKGVsZW0pe3JldHVybiAhKChfMzA4KGVsZW0pKSUy",
				"KTt9O3ZhciBfMzBhPWZ1bmN0aW9uKGVsZW0pe3JldHVybiAoKF8zMDgoZWxlbSkpJTIpO307dmFyIF8z",
				"MGI9eyJjaGVja2VkIjpmdW5jdGlvbihuYW1lLF8zMGMpe3JldHVybiBmdW5jdGlvbihlbGVtKXtyZXR1",
				"cm4gISEoImNoZWNrZWQiIGluIGVsZW0/ZWxlbS5jaGVja2VkOmVsZW0uc2VsZWN0ZWQpO307fSwiZmly",
				"c3QtY2hpbGQiOmZ1bmN0aW9uKCl7cmV0dXJuIF8zMDY7fSwibGFzdC1jaGlsZCI6ZnVuY3Rpb24oKXty",
				"ZXR1cm4gXzMwNzt9LCJvbmx5LWNoaWxkIjpmdW5jdGlvbihuYW1lLF8zMGQpe3JldHVybiBmdW5jdGlv",
				"bihub2RlKXtyZXR1cm4gXzMwNihub2RlKSYmXzMwNyhub2RlKTt9O30sImVtcHR5IjpmdW5jdGlvbihu",
				"YW1lLF8zMGUpe3JldHVybiBmdW5jdGlvbihlbGVtKXt2YXIgY249ZWxlbS5jaGlsZE5vZGVzO3ZhciBj",
				"bmw9ZWxlbS5jaGlsZE5vZGVzLmxlbmd0aDtmb3IodmFyIHg9Y25sLTE7eD49MDt4LS0pe3ZhciBudD1j",
				"blt4XS5ub2RlVHlwZTtpZigobnQ9PT0xKXx8KG50PT0zKSl7cmV0dXJuIGZhbHNlO319cmV0dXJuIHRy",
				"dWU7fTt9LCJjb250YWlucyI6ZnVuY3Rpb24obmFtZSxfMzBmKXt2YXIgY3o9XzMwZi5jaGFyQXQoMCk7",
				"aWYoY3o9PSJcIiJ8fGN6PT0iJyIpe18zMGY9XzMwZi5zbGljZSgxLC0xKTt9cmV0dXJuIGZ1bmN0aW9u",
				"KGVsZW0pe3JldHVybiAoZWxlbS5pbm5lckhUTUwuaW5kZXhPZihfMzBmKT49MCk7fTt9LCJub3QiOmZ1",
				"bmN0aW9uKG5hbWUsXzMxMCl7dmFyIHA9XzJlMShfMzEwKVswXTt2YXIgXzMxMT17ZWw6MX07aWYocC50",
				"YWchPSIqIil7XzMxMS50YWc9MTt9aWYoIXAuY2xhc3Nlcy5sZW5ndGgpe18zMTEuY2xhc3Nlcz0xO312",
				"YXIgbnRmPV8zMTIocCxfMzExKTtyZXR1cm4gZnVuY3Rpb24oZWxlbSl7cmV0dXJuICghbnRmKGVsZW0p",
				"KTt9O30sIm50aC1jaGlsZCI6ZnVuY3Rpb24obmFtZSxfMzEzKXt2YXIgcGk9cGFyc2VJbnQ7aWYoXzMx",
				"Mz09Im9kZCIpe3JldHVybiBfMzBhO31lbHNle2lmKF8zMTM9PSJldmVuIil7cmV0dXJuIF8zMDk7fX1p",
				"ZihfMzEzLmluZGV4T2YoIm4iKSE9LTEpe3ZhciBfMzE0PV8zMTMuc3BsaXQoIm4iLDIpO3ZhciBwcmVk",
				"PV8zMTRbMF0/KChfMzE0WzBdPT0iLSIpPy0xOnBpKF8zMTRbMF0pKToxO3ZhciBpZHg9XzMxNFsxXT9w",
				"aShfMzE0WzFdKTowO3ZhciBsYj0wLHViPS0xO2lmKHByZWQ+MCl7aWYoaWR4PDApe2lkeD0oaWR4JXBy",
				"ZWQpJiYocHJlZCsoaWR4JXByZWQpKTt9ZWxzZXtpZihpZHg+MCl7aWYoaWR4Pj1wcmVkKXtsYj1pZHgt",
				"aWR4JXByZWQ7fWlkeD1pZHglcHJlZDt9fX1lbHNle2lmKHByZWQ8MCl7cHJlZCo9LTE7aWYoaWR4PjAp",
				"e3ViPWlkeDtpZHg9aWR4JXByZWQ7fX19aWYocHJlZD4wKXtyZXR1cm4gZnVuY3Rpb24oZWxlbSl7dmFy",
				"IGk9XzMwOChlbGVtKTtyZXR1cm4gKGk+PWxiKSYmKHViPDB8fGk8PXViKSYmKChpJXByZWQpPT1pZHgp",
				"O307fWVsc2V7XzMxMz1pZHg7fX12YXIgXzMxNT1waShfMzEzKTtyZXR1cm4gZnVuY3Rpb24oZWxlbSl7",
				"cmV0dXJuIChfMzA4KGVsZW0pPT1fMzE1KTt9O319O3ZhciBfMzE2PShkb2pvLmlzSUUmJihkb2pvLmlz",
				"SUU8OXx8ZG9qby5pc1F1aXJrcykpP2Z1bmN0aW9uKGNvbmQpe3ZhciBjbGM9Y29uZC50b0xvd2VyQ2Fz",
				"ZSgpO2lmKGNsYz09ImNsYXNzIil7Y29uZD0iY2xhc3NOYW1lIjt9cmV0dXJuIGZ1bmN0aW9uKGVsZW0p",
				"e3JldHVybiAoXzJkZj9lbGVtLmdldEF0dHJpYnV0ZShjb25kKTplbGVtW2NvbmRdfHxlbGVtW2NsY10p",
				"O307fTpmdW5jdGlvbihjb25kKXtyZXR1cm4gZnVuY3Rpb24oZWxlbSl7cmV0dXJuIChlbGVtJiZlbGVt",
				"LmdldEF0dHJpYnV0ZSYmZWxlbS5oYXNBdHRyaWJ1dGUoY29uZCkpO307fTt2YXIgXzMxMj1mdW5jdGlv",
				"bihfMzE3LF8zMTgpe2lmKCFfMzE3KXtyZXR1cm4gXzJlMDt9XzMxOD1fMzE4fHx7fTt2YXIgZmY9bnVs",
				"bDtpZighKCJlbCIgaW4gXzMxOCkpe2ZmPV8yZjMoZmYsXzJmNyk7fWlmKCEoInRhZyIgaW4gXzMxOCkp",
				"e2lmKF8zMTcudGFnIT0iKiIpe2ZmPV8yZjMoZmYsZnVuY3Rpb24oZWxlbSl7cmV0dXJuIChlbGVtJiYo",
				"ZWxlbS50YWdOYW1lPT1fMzE3LmdldFRhZygpKSk7fSk7fX1pZighKCJjbGFzc2VzIiBpbiBfMzE4KSl7",
				"ZWFjaChfMzE3LmNsYXNzZXMsZnVuY3Rpb24oXzMxOSxpZHgsYXJyKXt2YXIgcmU9bmV3IFJlZ0V4cCgi",
				"KD86XnxcXHMpIitfMzE5KyIoPzpcXHN8JCkiKTtmZj1fMmYzKGZmLGZ1bmN0aW9uKGVsZW0pe3JldHVy",
				"biByZS50ZXN0KGVsZW0uY2xhc3NOYW1lKTt9KTtmZi5jb3VudD1pZHg7fSk7fWlmKCEoInBzZXVkb3Mi",
				"IGluIF8zMTgpKXtlYWNoKF8zMTcucHNldWRvcyxmdW5jdGlvbihfMzFhKXt2YXIgcG49XzMxYS5uYW1l",
				"O2lmKF8zMGJbcG5dKXtmZj1fMmYzKGZmLF8zMGJbcG5dKHBuLF8zMWEudmFsdWUpKTt9fSk7fWlmKCEo",
				"ImF0dHJzIiBpbiBfMzE4KSl7ZWFjaChfMzE3LmF0dHJzLGZ1bmN0aW9uKGF0dHIpe3ZhciBfMzFiO3Zh",
				"ciBhPWF0dHIuYXR0cjtpZihhdHRyLnR5cGUmJl8yZmFbYXR0ci50eXBlXSl7XzMxYj1fMmZhW2F0dHIu",
				"dHlwZV0oYSxhdHRyLm1hdGNoRm9yKTt9ZWxzZXtpZihhLmxlbmd0aCl7XzMxYj1fMzE2KGEpO319aWYo",
				"XzMxYil7ZmY9XzJmMyhmZixfMzFiKTt9fSk7fWlmKCEoImlkIiBpbiBfMzE4KSl7aWYoXzMxNy5pZCl7",
				"ZmY9XzJmMyhmZixmdW5jdGlvbihlbGVtKXtyZXR1cm4gKCEhZWxlbSYmKGVsZW0uaWQ9PV8zMTcuaWQp",
				"KTt9KTt9fWlmKCFmZil7aWYoISgiZGVmYXVsdCIgaW4gXzMxOCkpe2ZmPV8yZTA7fX1yZXR1cm4gZmY7",
				"fTt2YXIgXzMxYz1mdW5jdGlvbihfMzFkKXtyZXR1cm4gZnVuY3Rpb24obm9kZSxyZXQsYmFnKXt3aGls",
				"ZShub2RlPW5vZGVbXzMwM10pe2lmKF8zMDImJighXzJmNyhub2RlKSkpe2NvbnRpbnVlO31pZigoIWJh",
				"Z3x8XzMxZShub2RlLGJhZykpJiZfMzFkKG5vZGUpKXtyZXQucHVzaChub2RlKTt9YnJlYWs7fXJldHVy",
				"biByZXQ7fTt9O3ZhciBfMzFmPWZ1bmN0aW9uKF8zMjApe3JldHVybiBmdW5jdGlvbihyb290LHJldCxi",
				"YWcpe3ZhciB0ZT1yb290W18zMDNdO3doaWxlKHRlKXtpZihfMzA1KHRlKSl7aWYoYmFnJiYhXzMxZSh0",
				"ZSxiYWcpKXticmVhazt9aWYoXzMyMCh0ZSkpe3JldC5wdXNoKHRlKTt9fXRlPXRlW18zMDNdO31yZXR1",
				"cm4gcmV0O307fTt2YXIgXzMyMT1mdW5jdGlvbihfMzIyKXtfMzIyPV8zMjJ8fF8yZTA7cmV0dXJuIGZ1",
				"bmN0aW9uKHJvb3QscmV0LGJhZyl7dmFyIHRlLHg9MCx0cmV0PXJvb3QuY2hpbGRyZW58fHJvb3QuY2hp",
				"bGROb2Rlczt3aGlsZSh0ZT10cmV0W3grK10pe2lmKF8zMDUodGUpJiYoIWJhZ3x8XzMxZSh0ZSxiYWcp",
				"KSYmKF8zMjIodGUseCkpKXtyZXQucHVzaCh0ZSk7fX1yZXR1cm4gcmV0O307fTt2YXIgXzMyMz1mdW5j",
				"dGlvbihub2RlLHJvb3Qpe3ZhciBwbj1ub2RlLnBhcmVudE5vZGU7d2hpbGUocG4pe2lmKHBuPT1yb290",
				"KXticmVhazt9cG49cG4ucGFyZW50Tm9kZTt9cmV0dXJuICEhcG47fTt2YXIgXzMyND17fTt2YXIgXzMy",
				"NT1mdW5jdGlvbihfMzI2KXt2YXIgXzMyNz1fMzI0W18zMjYucXVlcnldO2lmKF8zMjcpe3JldHVybiBf",
				"MzI3O312YXIgaW89XzMyNi5pbmZpeE9wZXI7dmFyIG9wZXI9KGlvP2lvLm9wZXI6IiIpO3ZhciBfMzI4",
				"PV8zMTIoXzMyNix7ZWw6MX0pO3ZhciBxdD1fMzI2LnRhZzt2YXIgXzMyOT0oIioiPT1xdCk7dmFyIGVj",
				"cz1fMmRjKClbImdldEVsZW1lbnRzQnlDbGFzc05hbWUiXTtpZighb3Blcil7aWYoXzMyNi5pZCl7XzMy",
				"OD0oIV8zMjYubG9vcHMmJl8zMjkpP18yZTA6XzMxMihfMzI2LHtlbDoxLGlkOjF9KTtfMzI3PWZ1bmN0",
				"aW9uKHJvb3QsYXJyKXt2YXIgdGU9ZG9tLmJ5SWQoXzMyNi5pZCwocm9vdC5vd25lckRvY3VtZW50fHxy",
				"b290KSk7aWYoIXRlfHwhXzMyOCh0ZSkpe3JldHVybjt9aWYoOT09cm9vdC5ub2RlVHlwZSl7cmV0dXJu",
				"IF8yZjYodGUsYXJyKTt9ZWxzZXtpZihfMzIzKHRlLHJvb3QpKXtyZXR1cm4gXzJmNih0ZSxhcnIpO319",
				"fTt9ZWxzZXtpZihlY3MmJi9ce1xzKlxbbmF0aXZlIGNvZGVcXVxzKlx9Ly50ZXN0KFN0cmluZyhlY3Mp",
				"KSYmXzMyNi5jbGFzc2VzLmxlbmd0aCYmIV8yZGQpe18zMjg9XzMxMihfMzI2LHtlbDoxLGNsYXNzZXM6",
				"MSxpZDoxfSk7dmFyIF8zMmE9XzMyNi5jbGFzc2VzLmpvaW4oIiAiKTtfMzI3PWZ1bmN0aW9uKHJvb3Qs",
				"YXJyLGJhZyl7dmFyIHJldD1fMmY2KDAsYXJyKSx0ZSx4PTA7dmFyIHRyZXQ9cm9vdC5nZXRFbGVtZW50",
				"c0J5Q2xhc3NOYW1lKF8zMmEpO3doaWxlKCh0ZT10cmV0W3grK10pKXtpZihfMzI4KHRlLHJvb3QpJiZf",
				"MzFlKHRlLGJhZykpe3JldC5wdXNoKHRlKTt9fXJldHVybiByZXQ7fTt9ZWxzZXtpZighXzMyOSYmIV8z",
				"MjYubG9vcHMpe18zMjc9ZnVuY3Rpb24ocm9vdCxhcnIsYmFnKXt2YXIgcmV0PV8yZjYoMCxhcnIpLHRl",
				"LHg9MDt2YXIgdHJldD1yb290LmdldEVsZW1lbnRzQnlUYWdOYW1lKF8zMjYuZ2V0VGFnKCkpO3doaWxl",
				"KCh0ZT10cmV0W3grK10pKXtpZihfMzFlKHRlLGJhZykpe3JldC5wdXNoKHRlKTt9fXJldHVybiByZXQ7",
				"fTt9ZWxzZXtfMzI4PV8zMTIoXzMyNix7ZWw6MSx0YWc6MSxpZDoxfSk7XzMyNz1mdW5jdGlvbihyb290",
				"LGFycixiYWcpe3ZhciByZXQ9XzJmNigwLGFyciksdGUseD0wO3ZhciB0cmV0PXJvb3QuZ2V0RWxlbWVu",
				"dHNCeVRhZ05hbWUoXzMyNi5nZXRUYWcoKSk7d2hpbGUoKHRlPXRyZXRbeCsrXSkpe2lmKF8zMjgodGUs",
				"cm9vdCkmJl8zMWUodGUsYmFnKSl7cmV0LnB1c2godGUpO319cmV0dXJuIHJldDt9O319fX1lbHNle3Zh",
				"ciBfMzJiPXtlbDoxfTtpZihfMzI5KXtfMzJiLnRhZz0xO31fMzI4PV8zMTIoXzMyNixfMzJiKTtpZigi",
				"KyI9PW9wZXIpe18zMjc9XzMxYyhfMzI4KTt9ZWxzZXtpZigifiI9PW9wZXIpe18zMjc9XzMxZihfMzI4",
				"KTt9ZWxzZXtpZigiPiI9PW9wZXIpe18zMjc9XzMyMShfMzI4KTt9fX19cmV0dXJuIF8zMjRbXzMyNi5x",
				"dWVyeV09XzMyNzt9O3ZhciBfMzJjPWZ1bmN0aW9uKHJvb3QsXzMyZCl7dmFyIF8zMmU9XzJmNihyb290",
				"KSxxcCx4LHRlLHFwbD1fMzJkLmxlbmd0aCxiYWcscmV0O2Zvcih2YXIgaT0wO2k8cXBsO2krKyl7cmV0",
				"PVtdO3FwPV8zMmRbaV07eD1fMzJlLmxlbmd0aC0xO2lmKHg+MCl7YmFnPXt9O3JldC5ub3ppcD10cnVl",
				"O312YXIgZ2VmPV8zMjUocXApO2Zvcih2YXIgaj0wOyh0ZT1fMzJlW2pdKTtqKyspe2dlZih0ZSxyZXQs",
				"YmFnKTt9aWYoIXJldC5sZW5ndGgpe2JyZWFrO31fMzJlPXJldDt9cmV0dXJuIHJldDt9O3ZhciBfMzJm",
				"PXt9LF8zMzA9e307dmFyIF8zMzE9ZnVuY3Rpb24oXzMzMil7dmFyIF8zMzM9XzJlMSh0cmltKF8zMzIp",
				"KTtpZihfMzMzLmxlbmd0aD09MSl7dmFyIHRlZj1fMzI1KF8zMzNbMF0pO3JldHVybiBmdW5jdGlvbihy",
				"b290KXt2YXIgcj10ZWYocm9vdCxbXSk7aWYocil7ci5ub3ppcD10cnVlO31yZXR1cm4gcjt9O31yZXR1",
				"cm4gZnVuY3Rpb24ocm9vdCl7cmV0dXJuIF8zMmMocm9vdCxfMzMzKTt9O307dmFyIG51YT1uYXZpZ2F0",
				"b3IudXNlckFnZW50O3ZhciB3az0iV2ViS2l0LyI7dmFyIF8zMzQ9KGRvam8uaXNXZWJLaXQmJihudWEu",
				"aW5kZXhPZih3ayk+MCkmJihwYXJzZUZsb2F0KG51YS5zcGxpdCh3aylbMV0pPjUyOCkpO3ZhciBfMzM1",
				"PWRvam8uaXNJRT8iY29tbWVudFN0cmlwIjoibm96aXAiO3ZhciBxc2E9InF1ZXJ5U2VsZWN0b3JBbGwi",
				"O3ZhciBfMzM2PSghIV8yZGMoKVtxc2FdJiYoIWRvam8uaXNTYWZhcml8fChkb2pvLmlzU2FmYXJpPjMu",
				"MSl8fF8zMzQpKTt2YXIgXzMzNz0vblwrXGR8KFteIF0pPyhbPn4rXSkoW14gPV0pPy9nO3ZhciBfMzM4",
				"PWZ1bmN0aW9uKF8zMzkscHJlLGNoLHBvc3Qpe3JldHVybiBjaD8ocHJlP3ByZSsiICI6IiIpK2NoKyhw",
				"b3N0PyIgIitwb3N0OiIiKTpfMzM5O307dmFyIF8zM2E9ZnVuY3Rpb24oXzMzYixfMzNjKXtfMzNiPV8z",
				"M2IucmVwbGFjZShfMzM3LF8zMzgpO2lmKF8zMzYpe3ZhciBfMzNkPV8zMzBbXzMzYl07aWYoXzMzZCYm",
				"IV8zM2Mpe3JldHVybiBfMzNkO319dmFyIF8zM2U9XzMyZltfMzNiXTtpZihfMzNlKXtyZXR1cm4gXzMz",
				"ZTt9dmFyIHFjej1fMzNiLmNoYXJBdCgwKTt2YXIgXzMzZj0oLTE9PV8zM2IuaW5kZXhPZigiICIpKTtp",
				"ZigoXzMzYi5pbmRleE9mKCIjIik+PTApJiYoXzMzZikpe18zM2M9dHJ1ZTt9dmFyIF8zNDA9KF8zMzYm",
				"JighXzMzYykmJihfMmRlLmluZGV4T2YocWN6KT09LTEpJiYoIWRvam8uaXNJRXx8KF8zM2IuaW5kZXhP",
				"ZigiOiIpPT0tMSkpJiYoIShfMmRkJiYoXzMzYi5pbmRleE9mKCIuIik+PTApKSkmJihfMzNiLmluZGV4",
				"T2YoIjpjb250YWlucyIpPT0tMSkmJihfMzNiLmluZGV4T2YoIjpjaGVja2VkIik9PS0xKSYmKF8zM2Iu",
				"aW5kZXhPZigifD0iKT09LTEpKTtpZihfMzQwKXt2YXIgdHE9KF8yZGUuaW5kZXhPZihfMzNiLmNoYXJB",
				"dChfMzNiLmxlbmd0aC0xKSk+PTApPyhfMzNiKyIgKiIpOl8zM2I7cmV0dXJuIF8zMzBbXzMzYl09ZnVu",
				"Y3Rpb24ocm9vdCl7dHJ5e2lmKCEoKDk9PXJvb3Qubm9kZVR5cGUpfHxfMzNmKSl7dGhyb3cgIiI7fXZh",
				"ciByPXJvb3RbcXNhXSh0cSk7cltfMzM1XT10cnVlO3JldHVybiByO31jYXRjaChlKXtyZXR1cm4gXzMz",
				"YShfMzNiLHRydWUpKHJvb3QpO319O31lbHNle3ZhciBfMzQxPV8zM2Iuc3BsaXQoL1xzKixccyovKTty",
				"ZXR1cm4gXzMyZltfMzNiXT0oKF8zNDEubGVuZ3RoPDIpP18zMzEoXzMzYik6ZnVuY3Rpb24ocm9vdCl7",
				"dmFyIF8zNDI9MCxyZXQ9W10sdHA7d2hpbGUoKHRwPV8zNDFbXzM0MisrXSkpe3JldD1yZXQuY29uY2F0",
				"KF8zMzEodHApKHJvb3QpKTt9cmV0dXJuIHJldDt9KTt9fTt2YXIgXzM0Mz0wO3ZhciBfMzQ0PWRvam8u",
				"aXNJRT9mdW5jdGlvbihub2RlKXtpZihfMmRmKXtyZXR1cm4gKG5vZGUuZ2V0QXR0cmlidXRlKCJfdWlk",
				"Iil8fG5vZGUuc2V0QXR0cmlidXRlKCJfdWlkIiwrK18zNDMpfHxfMzQzKTt9ZWxzZXtyZXR1cm4gbm9k",
				"ZS51bmlxdWVJRDt9fTpmdW5jdGlvbihub2RlKXtyZXR1cm4gKG5vZGUuX3VpZHx8KG5vZGUuX3VpZD0r",
				"K18zNDMpKTt9O3ZhciBfMzFlPWZ1bmN0aW9uKG5vZGUsYmFnKXtpZighYmFnKXtyZXR1cm4gMTt9dmFy",
				"IGlkPV8zNDQobm9kZSk7aWYoIWJhZ1tpZF0pe3JldHVybiBiYWdbaWRdPTE7fXJldHVybiAwO307dmFy",
				"IF8zNDU9Il96aXBJZHgiO3ZhciBfMzQ2PWZ1bmN0aW9uKGFycil7aWYoYXJyJiZhcnIubm96aXApe3Jl",
				"dHVybiBhcnI7fXZhciByZXQ9W107aWYoIWFycnx8IWFyci5sZW5ndGgpe3JldHVybiByZXQ7fWlmKGFy",
				"clswXSl7cmV0LnB1c2goYXJyWzBdKTt9aWYoYXJyLmxlbmd0aDwyKXtyZXR1cm4gcmV0O31fMzQzKys7",
				"aWYoZG9qby5pc0lFJiZfMmRmKXt2YXIgXzM0Nz1fMzQzKyIiO2FyclswXS5zZXRBdHRyaWJ1dGUoXzM0",
				"NSxfMzQ3KTtmb3IodmFyIHg9MSx0ZTt0ZT1hcnJbeF07eCsrKXtpZihhcnJbeF0uZ2V0QXR0cmlidXRl",
				"KF8zNDUpIT1fMzQ3KXtyZXQucHVzaCh0ZSk7fXRlLnNldEF0dHJpYnV0ZShfMzQ1LF8zNDcpO319ZWxz",
				"ZXtpZihkb2pvLmlzSUUmJmFyci5jb21tZW50U3RyaXApe3RyeXtmb3IodmFyIHg9MSx0ZTt0ZT1hcnJb",
				"eF07eCsrKXtpZihfMmY3KHRlKSl7cmV0LnB1c2godGUpO319fWNhdGNoKGUpe319ZWxzZXtpZihhcnJb",
				"MF0pe2FyclswXVtfMzQ1XT1fMzQzO31mb3IodmFyIHg9MSx0ZTt0ZT1hcnJbeF07eCsrKXtpZihhcnJb",
				"eF1bXzM0NV0hPV8zNDMpe3JldC5wdXNoKHRlKTt9dGVbXzM0NV09XzM0Mzt9fX1yZXR1cm4gcmV0O307",
				"dmFyIF8zNDg9ZnVuY3Rpb24oXzM0OSxyb290KXtyb290PXJvb3R8fF8yZGMoKTt2YXIgb2Q9cm9vdC5v",
				"d25lckRvY3VtZW50fHxyb290LmRvY3VtZW50RWxlbWVudDtfMmRmPShyb290LmNvbnRlbnRUeXBlJiZy",
				"b290LmNvbnRlbnRUeXBlPT0iYXBwbGljYXRpb24veG1sIil8fChkb2pvLmlzT3BlcmEmJihyb290LmRv",
				"Y3R5cGV8fG9kLnRvU3RyaW5nKCk9PSJbb2JqZWN0IFhNTERvY3VtZW50XSIpKXx8KCEhb2QpJiYoZG9q",
				"by5pc0lFP29kLnhtbDoocm9vdC54bWxWZXJzaW9ufHxvZC54bWxWZXJzaW9uKSk7dmFyIHI9XzMzYShf",
				"MzQ5KShyb290KTtpZihyJiZyLm5vemlwKXtyZXR1cm4gcjt9cmV0dXJuIF8zNDYocik7fTtfMzQ4LmZp",
				"bHRlcj1mdW5jdGlvbihfMzRhLF8zNGIscm9vdCl7dmFyIF8zNGM9W10sXzM0ZD1fMmUxKF8zNGIpLF8z",
				"NGU9KF8zNGQubGVuZ3RoPT0xJiYhL1teXHcjXC5dLy50ZXN0KF8zNGIpKT9fMzEyKF8zNGRbMF0pOmZ1",
				"bmN0aW9uKG5vZGUpe3JldHVybiBkb2pvLnF1ZXJ5KF8zNGIscm9vdCkuaW5kZXhPZihub2RlKSE9LTE7",
				"fTtmb3IodmFyIHg9MCx0ZTt0ZT1fMzRhW3hdO3grKyl7aWYoXzM0ZSh0ZSkpe18zNGMucHVzaCh0ZSk7",
				"fX1yZXR1cm4gXzM0Yzt9O3JldHVybiBfMzQ4O30pO30sImRvam8vZG9tLXN0eWxlIjpmdW5jdGlvbigp",
				"e2RlZmluZShbIi4vX2Jhc2Uvc25pZmYiLCIuL2RvbSJdLGZ1bmN0aW9uKGhhcyxkb20pe3ZhciBfMzRm",
				"LF8zNTA9e307aWYoaGFzKCJ3ZWJraXQiKSl7XzM0Zj1mdW5jdGlvbihub2RlKXt2YXIgcztpZihub2Rl",
				"Lm5vZGVUeXBlPT0xKXt2YXIgZHY9bm9kZS5vd25lckRvY3VtZW50LmRlZmF1bHRWaWV3O3M9ZHYuZ2V0",
				"Q29tcHV0ZWRTdHlsZShub2RlLG51bGwpO2lmKCFzJiZub2RlLnN0eWxlKXtub2RlLnN0eWxlLmRpc3Bs",
				"YXk9IiI7cz1kdi5nZXRDb21wdXRlZFN0eWxlKG5vZGUsbnVsbCk7fX1yZXR1cm4gc3x8e307fTt9ZWxz",
				"ZXtpZihoYXMoImllIikmJihoYXMoImllIik8OXx8aGFzKCJxdWlya3MiKSkpe18zNGY9ZnVuY3Rpb24o",
				"bm9kZSl7cmV0dXJuIG5vZGUubm9kZVR5cGU9PTE/bm9kZS5jdXJyZW50U3R5bGU6e307fTt9ZWxzZXtf",
				"MzRmPWZ1bmN0aW9uKG5vZGUpe3JldHVybiBub2RlLm5vZGVUeXBlPT0xP25vZGUub3duZXJEb2N1bWVu",
				"dC5kZWZhdWx0Vmlldy5nZXRDb21wdXRlZFN0eWxlKG5vZGUsbnVsbCk6e307fTt9fV8zNTAuZ2V0Q29t",
				"cHV0ZWRTdHlsZT1fMzRmO3ZhciBfMzUxO2lmKCFoYXMoImllIikpe18zNTE9ZnVuY3Rpb24oXzM1Mixf",
				"MzUzKXtyZXR1cm4gcGFyc2VGbG9hdChfMzUzKXx8MDt9O31lbHNle18zNTE9ZnVuY3Rpb24oXzM1NCxf",
				"MzU1KXtpZighXzM1NSl7cmV0dXJuIDA7fWlmKF8zNTU9PSJtZWRpdW0iKXtyZXR1cm4gNDt9aWYoXzM1",
				"NS5zbGljZSYmXzM1NS5zbGljZSgtMik9PSJweCIpe3JldHVybiBwYXJzZUZsb2F0KF8zNTUpO312YXIg",
				"cz1fMzU0LnN0eWxlLHJzPV8zNTQucnVudGltZVN0eWxlLGNzPV8zNTQuY3VycmVudFN0eWxlLF8zNTY9",
				"cy5sZWZ0LF8zNTc9cnMubGVmdDtycy5sZWZ0PWNzLmxlZnQ7dHJ5e3MubGVmdD1fMzU1O18zNTU9cy5w",
				"aXhlbExlZnQ7fWNhdGNoKGUpe18zNTU9MDt9cy5sZWZ0PV8zNTY7cnMubGVmdD1fMzU3O3JldHVybiBf",
				"MzU1O307fV8zNTAudG9QaXhlbFZhbHVlPV8zNTE7dmFyIGFzdHI9IkRYSW1hZ2VUcmFuc2Zvcm0uTWlj",
				"cm9zb2Z0LkFscGhhIjt2YXIgYWY9ZnVuY3Rpb24obixmKXt0cnl7cmV0dXJuIG4uZmlsdGVycy5pdGVt",
				"KGFzdHIpO31jYXRjaChlKXtyZXR1cm4gZj97fTpudWxsO319O3ZhciBfMzU4PWhhcygiaWUiKTw5fHwo",
				"aGFzKCJpZSIpJiZoYXMoInF1aXJrcyIpKT9mdW5jdGlvbihub2RlKXt0cnl7cmV0dXJuIGFmKG5vZGUp",
				"Lk9wYWNpdHkvMTAwO31jYXRjaChlKXtyZXR1cm4gMTt9fTpmdW5jdGlvbihub2RlKXtyZXR1cm4gXzM0",
				"Zihub2RlKS5vcGFjaXR5O307dmFyIF8zNTk9aGFzKCJpZSIpPDl8fChoYXMoImllIikmJmhhcygicXVp",
				"cmtzIikpP2Z1bmN0aW9uKG5vZGUsXzM1YSl7dmFyIG92PV8zNWEqMTAwLF8zNWI9XzM1YT09MTtub2Rl",
				"LnN0eWxlLnpvb209XzM1Yj8iIjoxO2lmKCFhZihub2RlKSl7aWYoXzM1Yil7cmV0dXJuIF8zNWE7fW5v",
				"ZGUuc3R5bGUuZmlsdGVyKz0iIHByb2dpZDoiK2FzdHIrIihPcGFjaXR5PSIrb3YrIikiO31lbHNle2Fm",
				"KG5vZGUsMSkuT3BhY2l0eT1vdjt9YWYobm9kZSwxKS5FbmFibGVkPSFfMzViO2lmKG5vZGUudGFnTmFt",
				"ZS50b0xvd2VyQ2FzZSgpPT0idHIiKXtmb3IodmFyIHRkPW5vZGUuZmlyc3RDaGlsZDt0ZDt0ZD10ZC5u",
				"ZXh0U2libGluZyl7aWYodGQudGFnTmFtZS50b0xvd2VyQ2FzZSgpPT0idGQiKXtfMzU5KHRkLF8zNWEp",
				"O319fXJldHVybiBfMzVhO306ZnVuY3Rpb24obm9kZSxfMzVjKXtyZXR1cm4gbm9kZS5zdHlsZS5vcGFj",
				"aXR5PV8zNWM7fTt2YXIgXzM1ZD17bGVmdDp0cnVlLHRvcDp0cnVlfTt2YXIgXzM1ZT0vbWFyZ2lufHBh",
				"ZGRpbmd8d2lkdGh8aGVpZ2h0fG1heHxtaW58b2Zmc2V0LztmdW5jdGlvbiBfMzVmKG5vZGUsdHlwZSxf",
				"MzYwKXt0eXBlPXR5cGUudG9Mb3dlckNhc2UoKTtpZihoYXMoImllIikpe2lmKF8zNjA9PSJhdXRvIil7",
				"aWYodHlwZT09ImhlaWdodCIpe3JldHVybiBub2RlLm9mZnNldEhlaWdodDt9aWYodHlwZT09IndpZHRo",
				"Iil7cmV0dXJuIG5vZGUub2Zmc2V0V2lkdGg7fX1pZih0eXBlPT0iZm9udHdlaWdodCIpe3N3aXRjaChf",
				"MzYwKXtjYXNlIDcwMDpyZXR1cm4gImJvbGQiO2Nhc2UgNDAwOmRlZmF1bHQ6cmV0dXJuICJub3JtYWwi",
				"O319fWlmKCEodHlwZSBpbiBfMzVkKSl7XzM1ZFt0eXBlXT1fMzVlLnRlc3QodHlwZSk7fXJldHVybiBf",
				"MzVkW3R5cGVdP18zNTEobm9kZSxfMzYwKTpfMzYwO307dmFyIF8zNjE9aGFzKCJpZSIpPyJzdHlsZUZs",
				"b2F0IjoiY3NzRmxvYXQiLF8zNjI9eyJjc3NGbG9hdCI6XzM2MSwic3R5bGVGbG9hdCI6XzM2MSwiZmxv",
				"YXQiOl8zNjF9O18zNTAuZ2V0PWZ1bmN0aW9uIGdldFN0eWxlKG5vZGUsbmFtZSl7dmFyIG49ZG9tLmJ5",
				"SWQobm9kZSksbD1hcmd1bWVudHMubGVuZ3RoLG9wPShuYW1lPT0ib3BhY2l0eSIpO2lmKGw9PTImJm9w",
				"KXtyZXR1cm4gXzM1OChuKTt9bmFtZT1fMzYyW25hbWVdfHxuYW1lO3ZhciBzPV8zNTAuZ2V0Q29tcHV0",
				"ZWRTdHlsZShuKTtyZXR1cm4gKGw9PTEpP3M6XzM1ZihuLG5hbWUsc1tuYW1lXXx8bi5zdHlsZVtuYW1l",
				"XSk7fTtfMzUwLnNldD1mdW5jdGlvbiBzZXRTdHlsZShub2RlLG5hbWUsXzM2Myl7dmFyIG49ZG9tLmJ5",
				"SWQobm9kZSksbD1hcmd1bWVudHMubGVuZ3RoLG9wPShuYW1lPT0ib3BhY2l0eSIpO25hbWU9XzM2Mltu",
				"YW1lXXx8bmFtZTtpZihsPT0zKXtyZXR1cm4gb3A/XzM1OShuLF8zNjMpOm4uc3R5bGVbbmFtZV09XzM2",
				"Mzt9Zm9yKHZhciB4IGluIG5hbWUpe18zNTAuc2V0KG5vZGUseCxuYW1lW3hdKTt9cmV0dXJuIF8zNTAu",
				"Z2V0Q29tcHV0ZWRTdHlsZShuKTt9O3JldHVybiBfMzUwO30pO30sImRvam8vZG9tLWdlb21ldHJ5Ijpm",
				"dW5jdGlvbigpe2RlZmluZShbIi4vX2Jhc2Uvc25pZmYiLCIuL19iYXNlL3dpbmRvdyIsIi4vZG9tIiwi",
				"Li9kb20tc3R5bGUiXSxmdW5jdGlvbihoYXMsd2luLGRvbSxfMzY0KXt2YXIgZ2VvbT17fTtnZW9tLmJv",
				"eE1vZGVsPSJjb250ZW50LWJveCI7aWYoaGFzKCJpZSIpKXtnZW9tLmJveE1vZGVsPWRvY3VtZW50LmNv",
				"bXBhdE1vZGU9PSJCYWNrQ29tcGF0Ij8iYm9yZGVyLWJveCI6ImNvbnRlbnQtYm94Ijt9Z2VvbS5nZXRQ",
				"YWRFeHRlbnRzPWZ1bmN0aW9uIGdldFBhZEV4dGVudHMobm9kZSxfMzY1KXtub2RlPWRvbS5ieUlkKG5v",
				"ZGUpO3ZhciBzPV8zNjV8fF8zNjQuZ2V0Q29tcHV0ZWRTdHlsZShub2RlKSxweD1fMzY0LnRvUGl4ZWxW",
				"YWx1ZSxsPXB4KG5vZGUscy5wYWRkaW5nTGVmdCksdD1weChub2RlLHMucGFkZGluZ1RvcCkscj1weChu",
				"b2RlLHMucGFkZGluZ1JpZ2h0KSxiPXB4KG5vZGUscy5wYWRkaW5nQm90dG9tKTtyZXR1cm4ge2w6bCx0",
				"OnQscjpyLGI6Yix3OmwrcixoOnQrYn07fTt2YXIgbm9uZT0ibm9uZSI7Z2VvbS5nZXRCb3JkZXJFeHRl",
				"bnRzPWZ1bmN0aW9uIGdldEJvcmRlckV4dGVudHMobm9kZSxfMzY2KXtub2RlPWRvbS5ieUlkKG5vZGUp",
				"O3ZhciBweD1fMzY0LnRvUGl4ZWxWYWx1ZSxzPV8zNjZ8fF8zNjQuZ2V0Q29tcHV0ZWRTdHlsZShub2Rl",
				"KSxsPXMuYm9yZGVyTGVmdFN0eWxlIT1ub25lP3B4KG5vZGUscy5ib3JkZXJMZWZ0V2lkdGgpOjAsdD1z",
				"LmJvcmRlclRvcFN0eWxlIT1ub25lP3B4KG5vZGUscy5ib3JkZXJUb3BXaWR0aCk6MCxyPXMuYm9yZGVy",
				"UmlnaHRTdHlsZSE9bm9uZT9weChub2RlLHMuYm9yZGVyUmlnaHRXaWR0aCk6MCxiPXMuYm9yZGVyQm90",
				"dG9tU3R5bGUhPW5vbmU/cHgobm9kZSxzLmJvcmRlckJvdHRvbVdpZHRoKTowO3JldHVybiB7bDpsLHQ6",
				"dCxyOnIsYjpiLHc6bCtyLGg6dCtifTt9O2dlb20uZ2V0UGFkQm9yZGVyRXh0ZW50cz1mdW5jdGlvbiBn",
				"ZXRQYWRCb3JkZXJFeHRlbnRzKG5vZGUsXzM2Nyl7bm9kZT1kb20uYnlJZChub2RlKTt2YXIgcz1fMzY3",
				"fHxfMzY0LmdldENvbXB1dGVkU3R5bGUobm9kZSkscD1nZW9tLmdldFBhZEV4dGVudHMobm9kZSxzKSxi",
				"PWdlb20uZ2V0Qm9yZGVyRXh0ZW50cyhub2RlLHMpO3JldHVybiB7bDpwLmwrYi5sLHQ6cC50K2IudCxy",
				"OnAucitiLnIsYjpwLmIrYi5iLHc6cC53K2IudyxoOnAuaCtiLmh9O307Z2VvbS5nZXRNYXJnaW5FeHRl",
				"bnRzPWZ1bmN0aW9uIGdldE1hcmdpbkV4dGVudHMobm9kZSxfMzY4KXtub2RlPWRvbS5ieUlkKG5vZGUp",
				"O3ZhciBzPV8zNjh8fF8zNjQuZ2V0Q29tcHV0ZWRTdHlsZShub2RlKSxweD1fMzY0LnRvUGl4ZWxWYWx1",
				"ZSxsPXB4KG5vZGUscy5tYXJnaW5MZWZ0KSx0PXB4KG5vZGUscy5tYXJnaW5Ub3ApLHI9cHgobm9kZSxz",
				"Lm1hcmdpblJpZ2h0KSxiPXB4KG5vZGUscy5tYXJnaW5Cb3R0b20pO2lmKGhhcygid2Via2l0IikmJihz",
				"LnBvc2l0aW9uIT0iYWJzb2x1dGUiKSl7cj1sO31yZXR1cm4ge2w6bCx0OnQscjpyLGI6Yix3Omwrcixo",
				"OnQrYn07fTtnZW9tLmdldE1hcmdpbkJveD1mdW5jdGlvbiBnZXRNYXJnaW5Cb3gobm9kZSxfMzY5KXtu",
				"b2RlPWRvbS5ieUlkKG5vZGUpO3ZhciBzPV8zNjl8fF8zNjQuZ2V0Q29tcHV0ZWRTdHlsZShub2RlKSxt",
				"ZT1nZW9tLmdldE1hcmdpbkV4dGVudHMobm9kZSxzKSxsPW5vZGUub2Zmc2V0TGVmdC1tZS5sLHQ9bm9k",
				"ZS5vZmZzZXRUb3AtbWUudCxwPW5vZGUucGFyZW50Tm9kZSxweD1fMzY0LnRvUGl4ZWxWYWx1ZSxwY3M7",
				"aWYoaGFzKCJtb3ppbGxhIikpe3ZhciBzbD1wYXJzZUZsb2F0KHMubGVmdCksc3Q9cGFyc2VGbG9hdChz",
				"LnRvcCk7aWYoIWlzTmFOKHNsKSYmIWlzTmFOKHN0KSl7bD1zbCx0PXN0O31lbHNle2lmKHAmJnAuc3R5",
				"bGUpe3Bjcz1fMzY0LmdldENvbXB1dGVkU3R5bGUocCk7aWYocGNzLm92ZXJmbG93IT0idmlzaWJsZSIp",
				"e2wrPXBjcy5ib3JkZXJMZWZ0U3R5bGUhPW5vbmU/cHgobm9kZSxwY3MuYm9yZGVyTGVmdFdpZHRoKTow",
				"O3QrPXBjcy5ib3JkZXJUb3BTdHlsZSE9bm9uZT9weChub2RlLHBjcy5ib3JkZXJUb3BXaWR0aCk6MDt9",
				"fX19ZWxzZXtpZihoYXMoIm9wZXJhIil8fChoYXMoImllIik9PTgmJiFoYXMoInF1aXJrcyIpKSl7aWYo",
				"cCl7cGNzPV8zNjQuZ2V0Q29tcHV0ZWRTdHlsZShwKTtsLT1wY3MuYm9yZGVyTGVmdFN0eWxlIT1ub25l",
				"P3B4KG5vZGUscGNzLmJvcmRlckxlZnRXaWR0aCk6MDt0LT1wY3MuYm9yZGVyVG9wU3R5bGUhPW5vbmU/",
				"cHgobm9kZSxwY3MuYm9yZGVyVG9wV2lkdGgpOjA7fX19cmV0dXJuIHtsOmwsdDp0LHc6bm9kZS5vZmZz",
				"ZXRXaWR0aCttZS53LGg6bm9kZS5vZmZzZXRIZWlnaHQrbWUuaH07fTtnZW9tLmdldENvbnRlbnRCb3g9",
				"ZnVuY3Rpb24gZ2V0Q29udGVudEJveChub2RlLF8zNmEpe25vZGU9ZG9tLmJ5SWQobm9kZSk7dmFyIHM9",
				"XzM2YXx8XzM2NC5nZXRDb21wdXRlZFN0eWxlKG5vZGUpLHc9bm9kZS5jbGllbnRXaWR0aCxoLHBlPWdl",
				"b20uZ2V0UGFkRXh0ZW50cyhub2RlLHMpLGJlPWdlb20uZ2V0Qm9yZGVyRXh0ZW50cyhub2RlLHMpO2lm",
				"KCF3KXt3PW5vZGUub2Zmc2V0V2lkdGg7aD1ub2RlLm9mZnNldEhlaWdodDt9ZWxzZXtoPW5vZGUuY2xp",
				"ZW50SGVpZ2h0O2JlLnc9YmUuaD0wO31pZihoYXMoIm9wZXJhIikpe3BlLmwrPWJlLmw7cGUudCs9YmUu",
				"dDt9cmV0dXJuIHtsOnBlLmwsdDpwZS50LHc6dy1wZS53LWJlLncsaDpoLXBlLmgtYmUuaH07fTtmdW5j",
				"dGlvbiBfMzZiKG5vZGUsbCx0LHcsaCx1KXt1PXV8fCJweCI7dmFyIHM9bm9kZS5zdHlsZTtpZighaXNO",
				"YU4obCkpe3MubGVmdD1sK3U7fWlmKCFpc05hTih0KSl7cy50b3A9dCt1O31pZih3Pj0wKXtzLndpZHRo",
				"PXcrdTt9aWYoaD49MCl7cy5oZWlnaHQ9aCt1O319O2Z1bmN0aW9uIF8zNmMobm9kZSl7cmV0dXJuIG5v",
				"ZGUudGFnTmFtZS50b0xvd2VyQ2FzZSgpPT0iYnV0dG9uInx8bm9kZS50YWdOYW1lLnRvTG93ZXJDYXNl",
				"KCk9PSJpbnB1dCImJihub2RlLmdldEF0dHJpYnV0ZSgidHlwZSIpfHwiIikudG9Mb3dlckNhc2UoKT09",
				"ImJ1dHRvbiI7fTtmdW5jdGlvbiBfMzZkKG5vZGUpe3JldHVybiBnZW9tLmJveE1vZGVsPT0iYm9yZGVy",
				"LWJveCJ8fG5vZGUudGFnTmFtZS50b0xvd2VyQ2FzZSgpPT0idGFibGUifHxfMzZjKG5vZGUpO307Z2Vv",
				"bS5zZXRDb250ZW50U2l6ZT1mdW5jdGlvbiBzZXRDb250ZW50U2l6ZShub2RlLGJveCxfMzZlKXtub2Rl",
				"PWRvbS5ieUlkKG5vZGUpO3ZhciB3PWJveC53LGg9Ym94Lmg7aWYoXzM2ZChub2RlKSl7dmFyIHBiPWdl",
				"b20uZ2V0UGFkQm9yZGVyRXh0ZW50cyhub2RlLF8zNmUpO2lmKHc+PTApe3crPXBiLnc7fWlmKGg+PTAp",
				"e2grPXBiLmg7fX1fMzZiKG5vZGUsTmFOLE5hTix3LGgpO307dmFyIF8zNmY9e2w6MCx0OjAsdzowLGg6",
				"MH07Z2VvbS5zZXRNYXJnaW5Cb3g9ZnVuY3Rpb24gc2V0TWFyZ2luQm94KG5vZGUsYm94LF8zNzApe25v",
				"ZGU9ZG9tLmJ5SWQobm9kZSk7dmFyIHM9XzM3MHx8XzM2NC5nZXRDb21wdXRlZFN0eWxlKG5vZGUpLHc9",
				"Ym94LncsaD1ib3guaCxwYj1fMzZkKG5vZGUpP18zNmY6Z2VvbS5nZXRQYWRCb3JkZXJFeHRlbnRzKG5v",
				"ZGUscyksbWI9Z2VvbS5nZXRNYXJnaW5FeHRlbnRzKG5vZGUscyk7aWYoaGFzKCJ3ZWJraXQiKSl7aWYo",
				"XzM2Yyhub2RlKSl7dmFyIG5zPW5vZGUuc3R5bGU7aWYodz49MCYmIW5zLndpZHRoKXtucy53aWR0aD0i",
				"NHB4Ijt9aWYoaD49MCYmIW5zLmhlaWdodCl7bnMuaGVpZ2h0PSI0cHgiO319fWlmKHc+PTApe3c9TWF0",
				"aC5tYXgody1wYi53LW1iLncsMCk7fWlmKGg+PTApe2g9TWF0aC5tYXgoaC1wYi5oLW1iLmgsMCk7fV8z",
				"NmIobm9kZSxib3gubCxib3gudCx3LGgpO307Z2VvbS5pc0JvZHlMdHI9ZnVuY3Rpb24gaXNCb2R5THRy",
				"KCl7cmV0dXJuICh3aW4uYm9keSgpLmRpcnx8d2luLmRvYy5kb2N1bWVudEVsZW1lbnQuZGlyfHwibHRy",
				"IikudG9Mb3dlckNhc2UoKT09Imx0ciI7fTtnZW9tLmRvY1Njcm9sbD1mdW5jdGlvbiBkb2NTY3JvbGwo",
				"KXt2YXIgbm9kZT13aW4uZG9jLnBhcmVudFdpbmRvd3x8d2luLmRvYy5kZWZhdWx0VmlldztyZXR1cm4g",
				"InBhZ2VYT2Zmc2V0IiBpbiBub2RlP3t4Om5vZGUucGFnZVhPZmZzZXQseTpub2RlLnBhZ2VZT2Zmc2V0",
				"fToobm9kZT1oYXMoInF1aXJrcyIpP3dpbi5ib2R5KCk6d2luLmRvYy5kb2N1bWVudEVsZW1lbnQse3g6",
				"Z2VvbS5maXhJZUJpRGlTY3JvbGxMZWZ0KG5vZGUuc2Nyb2xsTGVmdHx8MCkseTpub2RlLnNjcm9sbFRv",
				"cHx8MH0pO307Z2VvbS5nZXRJZURvY3VtZW50RWxlbWVudE9mZnNldD1mdW5jdGlvbiBnZXRJZURvY3Vt",
				"ZW50RWxlbWVudE9mZnNldCgpe3ZhciBkZT13aW4uZG9jLmRvY3VtZW50RWxlbWVudDtpZihoYXMoImll",
				"Iik8OCl7dmFyIHI9ZGUuZ2V0Qm91bmRpbmdDbGllbnRSZWN0KCksbD1yLmxlZnQsdD1yLnRvcDtpZiho",
				"YXMoImllIik8Nyl7bCs9ZGUuY2xpZW50TGVmdDt0Kz1kZS5jbGllbnRUb3A7fXJldHVybiB7eDpsPDA/",
				"MDpsLHk6dDwwPzA6dH07fWVsc2V7cmV0dXJuIHt4OjAseTowfTt9fTtnZW9tLmZpeEllQmlEaVNjcm9s",
				"bExlZnQ9ZnVuY3Rpb24gZml4SWVCaURpU2Nyb2xsTGVmdChfMzcxKXt2YXIgaWU9aGFzKCJpZSIpO2lm",
				"KGllJiYhZ2VvbS5pc0JvZHlMdHIoKSl7dmFyIHFrPWhhcygicXVpcmtzIiksZGU9cWs/d2luLmJvZHko",
				"KTp3aW4uZG9jLmRvY3VtZW50RWxlbWVudDtpZihpZT09NiYmIXFrJiZ3aW4uZ2xvYmFsLmZyYW1lRWxl",
				"bWVudCYmZGUuc2Nyb2xsSGVpZ2h0PmRlLmNsaWVudEhlaWdodCl7XzM3MSs9ZGUuY2xpZW50TGVmdDt9",
				"cmV0dXJuIChpZTw4fHxxayk/KF8zNzErZGUuY2xpZW50V2lkdGgtZGUuc2Nyb2xsV2lkdGgpOi1fMzcx",
				"O31yZXR1cm4gXzM3MTt9O2dlb20ucG9zaXRpb249ZnVuY3Rpb24obm9kZSxfMzcyKXtub2RlPWRvbS5i",
				"eUlkKG5vZGUpO3ZhciBkYj13aW4uYm9keSgpLGRoPWRiLnBhcmVudE5vZGUscmV0PW5vZGUuZ2V0Qm91",
				"bmRpbmdDbGllbnRSZWN0KCk7cmV0PXt4OnJldC5sZWZ0LHk6cmV0LnRvcCx3OnJldC5yaWdodC1yZXQu",
				"bGVmdCxoOnJldC5ib3R0b20tcmV0LnRvcH07aWYoaGFzKCJpZSIpKXt2YXIgXzM3Mz1nZW9tLmdldEll",
				"RG9jdW1lbnRFbGVtZW50T2Zmc2V0KCk7cmV0LngtPV8zNzMueCsoaGFzKCJxdWlya3MiKT9kYi5jbGll",
				"bnRMZWZ0K2RiLm9mZnNldExlZnQ6MCk7cmV0LnktPV8zNzMueSsoaGFzKCJxdWlya3MiKT9kYi5jbGll",
				"bnRUb3ArZGIub2Zmc2V0VG9wOjApO31lbHNle2lmKGhhcygiZmYiKT09Myl7dmFyIGNzPV8zNjQuZ2V0",
				"Q29tcHV0ZWRTdHlsZShkaCkscHg9XzM2NC50b1BpeGVsVmFsdWU7cmV0LngtPXB4KGRoLGNzLm1hcmdp",
				"bkxlZnQpK3B4KGRoLGNzLmJvcmRlckxlZnRXaWR0aCk7cmV0LnktPXB4KGRoLGNzLm1hcmdpblRvcCkr",
				"cHgoZGgsY3MuYm9yZGVyVG9wV2lkdGgpO319aWYoXzM3Mil7dmFyIF8zNzQ9Z2VvbS5kb2NTY3JvbGwo",
				"KTtyZXQueCs9XzM3NC54O3JldC55Kz1fMzc0Lnk7fXJldHVybiByZXQ7fTtnZW9tLmdldE1hcmdpblNp",
				"emU9ZnVuY3Rpb24gZ2V0TWFyZ2luU2l6ZShub2RlLF8zNzUpe25vZGU9ZG9tLmJ5SWQobm9kZSk7dmFy",
				"IG1lPWdlb20uZ2V0TWFyZ2luRXh0ZW50cyhub2RlLF8zNzV8fF8zNjQuZ2V0Q29tcHV0ZWRTdHlsZShu",
				"b2RlKSk7dmFyIHNpemU9bm9kZS5nZXRCb3VuZGluZ0NsaWVudFJlY3QoKTtyZXR1cm4ge3c6KHNpemUu",
				"cmlnaHQtc2l6ZS5sZWZ0KSttZS53LGg6KHNpemUuYm90dG9tLXNpemUudG9wKSttZS5ofTt9O2dlb20u",
				"bm9ybWFsaXplRXZlbnQ9ZnVuY3Rpb24oXzM3Nil7aWYoISgibGF5ZXJYIiBpbiBfMzc2KSl7XzM3Ni5s",
				"YXllclg9XzM3Ni5vZmZzZXRYO18zNzYubGF5ZXJZPV8zNzYub2Zmc2V0WTt9aWYoIWhhcygiZG9tLWFk",
				"ZGV2ZW50bGlzdGVuZXIiKSl7dmFyIHNlPV8zNzYudGFyZ2V0O3ZhciBkb2M9KHNlJiZzZS5vd25lckRv",
				"Y3VtZW50KXx8ZG9jdW1lbnQ7dmFyIF8zNzc9aGFzKCJxdWlya3MiKT9kb2MuYm9keTpkb2MuZG9jdW1l",
				"bnRFbGVtZW50O3ZhciBfMzc4PWdlb20uZ2V0SWVEb2N1bWVudEVsZW1lbnRPZmZzZXQoKTtfMzc2LnBh",
				"Z2VYPV8zNzYuY2xpZW50WCtnZW9tLmZpeEllQmlEaVNjcm9sbExlZnQoXzM3Ny5zY3JvbGxMZWZ0fHww",
				"KS1fMzc4Lng7XzM3Ni5wYWdlWT1fMzc2LmNsaWVudFkrKF8zNzcuc2Nyb2xsVG9wfHwwKS1fMzc4Lnk7",
				"fX07cmV0dXJuIGdlb207fSk7fSwiZG9qby9kb20tcHJvcCI6ZnVuY3Rpb24oKXtkZWZpbmUoImRvam8v",
				"ZG9tLXByb3AiLFsiZXhwb3J0cyIsIi4vX2Jhc2Uva2VybmVsIiwiLi9fYmFzZS9zbmlmZiIsIi4vX2Jh",
				"c2UvbGFuZyIsIi4vZG9tIiwiLi9kb20tc3R5bGUiLCIuL2RvbS1jb25zdHJ1Y3QiLCIuL19iYXNlL2Nv",
				"bm5lY3QiXSxmdW5jdGlvbihfMzc5LGRvam8saGFzLGxhbmcsZG9tLF8zN2EsY3RyLGNvbm4pe3ZhciBf",
				"MzdiPXt9LF8zN2M9MCxfMzdkPWRvam8uX3Njb3BlTmFtZSsiYXR0cmlkIjt2YXIgXzM3ZT17Y29sOjEs",
				"Y29sZ3JvdXA6MSx0YWJsZToxLHRib2R5OjEsdGZvb3Q6MSx0aGVhZDoxLHRyOjEsdGl0bGU6MX07XzM3",
				"OS5uYW1lcz17ImNsYXNzIjoiY2xhc3NOYW1lIiwiZm9yIjoiaHRtbEZvciIsdGFiaW5kZXg6InRhYklu",
				"ZGV4IixyZWFkb25seToicmVhZE9ubHkiLGNvbHNwYW46ImNvbFNwYW4iLGZyYW1lYm9yZGVyOiJmcmFt",
				"ZUJvcmRlciIscm93c3Bhbjoicm93U3BhbiIsdmFsdWV0eXBlOiJ2YWx1ZVR5cGUifTtfMzc5LmdldD1m",
				"dW5jdGlvbiBnZXRQcm9wKG5vZGUsbmFtZSl7bm9kZT1kb20uYnlJZChub2RlKTt2YXIgbGM9bmFtZS50",
				"b0xvd2VyQ2FzZSgpLF8zN2Y9XzM3OS5uYW1lc1tsY118fG5hbWU7cmV0dXJuIG5vZGVbXzM3Zl07fTtf",
				"Mzc5LnNldD1mdW5jdGlvbiBzZXRQcm9wKG5vZGUsbmFtZSxfMzgwKXtub2RlPWRvbS5ieUlkKG5vZGUp",
				"O3ZhciBsPWFyZ3VtZW50cy5sZW5ndGg7aWYobD09MiYmdHlwZW9mIG5hbWUhPSJzdHJpbmciKXtmb3Io",
				"dmFyIHggaW4gbmFtZSl7XzM3OS5zZXQobm9kZSx4LG5hbWVbeF0pO31yZXR1cm4gbm9kZTt9dmFyIGxj",
				"PW5hbWUudG9Mb3dlckNhc2UoKSxfMzgxPV8zNzkubmFtZXNbbGNdfHxuYW1lO2lmKF8zODE9PSJzdHls",
				"ZSImJnR5cGVvZiBfMzgwIT0ic3RyaW5nIil7XzM3YS5zdHlsZShub2RlLF8zODApO3JldHVybiBub2Rl",
				"O31pZihfMzgxPT0iaW5uZXJIVE1MIil7aWYoaGFzKCJpZSIpJiZub2RlLnRhZ05hbWUudG9Mb3dlckNh",
				"c2UoKSBpbiBfMzdlKXtjdHIuZW1wdHkobm9kZSk7bm9kZS5hcHBlbmRDaGlsZChjdHIudG9Eb20oXzM4",
				"MCxub2RlLm93bmVyRG9jdW1lbnQpKTt9ZWxzZXtub2RlW18zODFdPV8zODA7fXJldHVybiBub2RlO31p",
				"ZihsYW5nLmlzRnVuY3Rpb24oXzM4MCkpe3ZhciBfMzgyPW5vZGVbXzM3ZF07aWYoIV8zODIpe18zODI9",
				"XzM3YysrO25vZGVbXzM3ZF09XzM4Mjt9aWYoIV8zN2JbXzM4Ml0pe18zN2JbXzM4Ml09e307fXZhciBo",
				"PV8zN2JbXzM4Ml1bXzM4MV07aWYoaCl7Y29ubi5kaXNjb25uZWN0KGgpO31lbHNle3RyeXtkZWxldGUg",
				"bm9kZVtfMzgxXTt9Y2F0Y2goZSl7fX1pZihfMzgwKXtfMzdiW18zODJdW18zODFdPWNvbm4uY29ubmVj",
				"dChub2RlLF8zODEsXzM4MCk7fWVsc2V7bm9kZVtfMzgxXT1udWxsO31yZXR1cm4gbm9kZTt9bm9kZVtf",
				"MzgxXT1fMzgwO3JldHVybiBub2RlO307fSk7fSwiZG9qby9kb20tYXR0ciI6ZnVuY3Rpb24oKXtkZWZp",
				"bmUoWyJleHBvcnRzIiwiLi9fYmFzZS9zbmlmZiIsIi4vX2Jhc2UvbGFuZyIsIi4vZG9tIiwiLi9kb20t",
				"c3R5bGUiLCIuL2RvbS1wcm9wIl0sZnVuY3Rpb24oXzM4MyxoYXMsbGFuZyxkb20sXzM4NCxwcm9wKXt2",
				"YXIgXzM4NT17aW5uZXJIVE1MOjEsY2xhc3NOYW1lOjEsaHRtbEZvcjpoYXMoImllIiksdmFsdWU6MX0s",
				"XzM4Nj17Y2xhc3NuYW1lOiJjbGFzcyIsaHRtbGZvcjoiZm9yIix0YWJpbmRleDoidGFiSW5kZXgiLHJl",
				"YWRvbmx5OiJyZWFkT25seSJ9O2Z1bmN0aW9uIF8zODcobm9kZSxuYW1lKXt2YXIgYXR0cj1ub2RlLmdl",
				"dEF0dHJpYnV0ZU5vZGUmJm5vZGUuZ2V0QXR0cmlidXRlTm9kZShuYW1lKTtyZXR1cm4gYXR0ciYmYXR0",
				"ci5zcGVjaWZpZWQ7fTtfMzgzLmhhcz1mdW5jdGlvbiBoYXNBdHRyKG5vZGUsbmFtZSl7dmFyIGxjPW5h",
				"bWUudG9Mb3dlckNhc2UoKTtyZXR1cm4gXzM4NVtwcm9wLm5hbWVzW2xjXXx8bmFtZV18fF8zODcoZG9t",
				"LmJ5SWQobm9kZSksXzM4NltsY118fG5hbWUpO307XzM4My5nZXQ9ZnVuY3Rpb24gZ2V0QXR0cihub2Rl",
				"LG5hbWUpe25vZGU9ZG9tLmJ5SWQobm9kZSk7dmFyIGxjPW5hbWUudG9Mb3dlckNhc2UoKSxfMzg4PXBy",
				"b3AubmFtZXNbbGNdfHxuYW1lLF8zODk9XzM4NVtfMzg4XTt2YWx1ZT1ub2RlW18zODhdO2lmKF8zODkm",
				"JnR5cGVvZiB2YWx1ZSE9InVuZGVmaW5lZCIpe3JldHVybiB2YWx1ZTt9aWYoXzM4OCE9ImhyZWYiJiYo",
				"dHlwZW9mIHZhbHVlPT0iYm9vbGVhbiJ8fGxhbmcuaXNGdW5jdGlvbih2YWx1ZSkpKXtyZXR1cm4gdmFs",
				"dWU7fXZhciBfMzhhPV8zODZbbGNdfHxuYW1lO3JldHVybiBfMzg3KG5vZGUsXzM4YSk/bm9kZS5nZXRB",
				"dHRyaWJ1dGUoXzM4YSk6bnVsbDt9O18zODMuc2V0PWZ1bmN0aW9uIHNldEF0dHIobm9kZSxuYW1lLF8z",
				"OGIpe25vZGU9ZG9tLmJ5SWQobm9kZSk7aWYoYXJndW1lbnRzLmxlbmd0aD09Mil7Zm9yKHZhciB4IGlu",
				"IG5hbWUpe18zODMuc2V0KG5vZGUseCxuYW1lW3hdKTt9cmV0dXJuIG5vZGU7fXZhciBsYz1uYW1lLnRv",
				"TG93ZXJDYXNlKCksXzM4Yz1wcm9wLm5hbWVzW2xjXXx8bmFtZSxfMzhkPV8zODVbXzM4Y107aWYoXzM4",
				"Yz09InN0eWxlIiYmdHlwZW9mIF8zOGIhPSJzdHJpbmciKXtfMzg0LnNldChub2RlLF8zOGIpO3JldHVy",
				"biBub2RlO31pZihfMzhkfHx0eXBlb2YgXzM4Yj09ImJvb2xlYW4ifHxsYW5nLmlzRnVuY3Rpb24oXzM4",
				"Yikpe3JldHVybiBwcm9wLnNldChub2RlLG5hbWUsXzM4Yik7fW5vZGUuc2V0QXR0cmlidXRlKF8zODZb",
				"bGNdfHxuYW1lLF8zOGIpO3JldHVybiBub2RlO307XzM4My5yZW1vdmU9ZnVuY3Rpb24gcmVtb3ZlQXR0",
				"cihub2RlLG5hbWUpe2RvbS5ieUlkKG5vZGUpLnJlbW92ZUF0dHJpYnV0ZShfMzg2W25hbWUudG9Mb3dl",
				"ckNhc2UoKV18fG5hbWUpO307XzM4My5nZXROb2RlUHJvcD1mdW5jdGlvbiBnZXROb2RlUHJvcChub2Rl",
				"LG5hbWUpe25vZGU9ZG9tLmJ5SWQobm9kZSk7dmFyIGxjPW5hbWUudG9Mb3dlckNhc2UoKSxfMzhlPXBy",
				"b3AubmFtZXNbbGNdfHxuYW1lO2lmKChfMzhlIGluIG5vZGUpJiZfMzhlIT0iaHJlZiIpe3JldHVybiBu",
				"b2RlW18zOGVdO312YXIgXzM4Zj1fMzg2W2xjXXx8bmFtZTtyZXR1cm4gXzM4Nyhub2RlLF8zOGYpP25v",
				"ZGUuZ2V0QXR0cmlidXRlKF8zOGYpOm51bGw7fTt9KTt9LCJkb2pvL2RvbS1jb25zdHJ1Y3QiOmZ1bmN0",
				"aW9uKCl7ZGVmaW5lKCJkb2pvL2RvbS1jb25zdHJ1Y3QiLFsiZXhwb3J0cyIsIi4vX2Jhc2Uva2VybmVs",
				"IiwiLi9fYmFzZS9zbmlmZiIsIi4vX2Jhc2Uvd2luZG93IiwiLi9kb20iLCIuL2RvbS1hdHRyIiwiLi9v",
				"biJdLGZ1bmN0aW9uKF8zOTAsZG9qbyxoYXMsd2luLGRvbSxhdHRyLG9uKXt2YXIgXzM5MT17b3B0aW9u",
				"Olsic2VsZWN0Il0sdGJvZHk6WyJ0YWJsZSJdLHRoZWFkOlsidGFibGUiXSx0Zm9vdDpbInRhYmxlIl0s",
				"dHI6WyJ0YWJsZSIsInRib2R5Il0sdGQ6WyJ0YWJsZSIsInRib2R5IiwidHIiXSx0aDpbInRhYmxlIiwi",
				"dGhlYWQiLCJ0ciJdLGxlZ2VuZDpbImZpZWxkc2V0Il0sY2FwdGlvbjpbInRhYmxlIl0sY29sZ3JvdXA6",
				"WyJ0YWJsZSJdLGNvbDpbInRhYmxlIiwiY29sZ3JvdXAiXSxsaTpbInVsIl19LF8zOTI9LzxccyooW1x3",
				"XDpdKykvLF8zOTM9e30sXzM5ND0wLF8zOTU9Il9fIitkb2pvLl9zY29wZU5hbWUrIlRvRG9tSWQiO2Zv",
				"cih2YXIgXzM5NiBpbiBfMzkxKXtpZihfMzkxLmhhc093blByb3BlcnR5KF8zOTYpKXt2YXIgdHc9XzM5",
				"MVtfMzk2XTt0dy5wcmU9XzM5Nj09Im9wdGlvbiI/IjxzZWxlY3QgbXVsdGlwbGU9XCJtdWx0aXBsZVwi",
				"PiI6IjwiK3R3LmpvaW4oIj48IikrIj4iO3R3LnBvc3Q9IjwvIit0dy5yZXZlcnNlKCkuam9pbigiPjwv",
				"IikrIj4iO319ZnVuY3Rpb24gXzM5Nyhub2RlLHJlZil7dmFyIF8zOTg9cmVmLnBhcmVudE5vZGU7aWYo",
				"XzM5OCl7XzM5OC5pbnNlcnRCZWZvcmUobm9kZSxyZWYpO319O2Z1bmN0aW9uIF8zOTkobm9kZSxyZWYp",
				"e3ZhciBfMzlhPXJlZi5wYXJlbnROb2RlO2lmKF8zOWEpe2lmKF8zOWEubGFzdENoaWxkPT1yZWYpe18z",
				"OWEuYXBwZW5kQ2hpbGQobm9kZSk7fWVsc2V7XzM5YS5pbnNlcnRCZWZvcmUobm9kZSxyZWYubmV4dFNp",
				"YmxpbmcpO319fTt2YXIgXzM5Yj1udWxsLF8zOWM7b24od2luZG93LCJ1bmxvYWQiLGZ1bmN0aW9uKCl7",
				"XzM5Yj1udWxsO30pO18zOTAudG9Eb209ZnVuY3Rpb24gdG9Eb20oZnJhZyxkb2Mpe2RvYz1kb2N8fHdp",
				"bi5kb2M7dmFyIF8zOWQ9ZG9jW18zOTVdO2lmKCFfMzlkKXtkb2NbXzM5NV09XzM5ZD0rK18zOTQrIiI7",
				"XzM5M1tfMzlkXT1kb2MuY3JlYXRlRWxlbWVudCgiZGl2Iik7fWZyYWcrPSIiO3ZhciBfMzllPWZyYWcu",
				"bWF0Y2goXzM5MiksdGFnPV8zOWU/XzM5ZVsxXS50b0xvd2VyQ2FzZSgpOiIiLF8zOWY9XzM5M1tfMzlk",
				"XSx3cmFwLGksZmMsZGY7aWYoXzM5ZSYmXzM5MVt0YWddKXt3cmFwPV8zOTFbdGFnXTtfMzlmLmlubmVy",
				"SFRNTD13cmFwLnByZStmcmFnK3dyYXAucG9zdDtmb3IoaT13cmFwLmxlbmd0aDtpOy0taSl7XzM5Zj1f",
				"MzlmLmZpcnN0Q2hpbGQ7fX1lbHNle18zOWYuaW5uZXJIVE1MPWZyYWc7fWlmKF8zOWYuY2hpbGROb2Rl",
				"cy5sZW5ndGg9PTEpe3JldHVybiBfMzlmLnJlbW92ZUNoaWxkKF8zOWYuZmlyc3RDaGlsZCk7fWRmPWRv",
				"Yy5jcmVhdGVEb2N1bWVudEZyYWdtZW50KCk7d2hpbGUoZmM9XzM5Zi5maXJzdENoaWxkKXtkZi5hcHBl",
				"bmRDaGlsZChmYyk7fXJldHVybiBkZjt9O18zOTAucGxhY2U9ZnVuY3Rpb24gcGxhY2Uobm9kZSxfM2Ew",
				"LF8zYTEpe18zYTA9ZG9tLmJ5SWQoXzNhMCk7aWYodHlwZW9mIG5vZGU9PSJzdHJpbmciKXtub2RlPS9e",
				"XHMqPC8udGVzdChub2RlKT9fMzkwLnRvRG9tKG5vZGUsXzNhMC5vd25lckRvY3VtZW50KTpkb20uYnlJ",
				"ZChub2RlKTt9aWYodHlwZW9mIF8zYTE9PSJudW1iZXIiKXt2YXIgY249XzNhMC5jaGlsZE5vZGVzO2lm",
				"KCFjbi5sZW5ndGh8fGNuLmxlbmd0aDw9XzNhMSl7XzNhMC5hcHBlbmRDaGlsZChub2RlKTt9ZWxzZXtf",
				"Mzk3KG5vZGUsY25bXzNhMTwwPzA6XzNhMV0pO319ZWxzZXtzd2l0Y2goXzNhMSl7Y2FzZSAiYmVmb3Jl",
				"IjpfMzk3KG5vZGUsXzNhMCk7YnJlYWs7Y2FzZSAiYWZ0ZXIiOl8zOTkobm9kZSxfM2EwKTticmVhaztj",
				"YXNlICJyZXBsYWNlIjpfM2EwLnBhcmVudE5vZGUucmVwbGFjZUNoaWxkKG5vZGUsXzNhMCk7YnJlYWs7",
				"Y2FzZSAib25seSI6XzM5MC5lbXB0eShfM2EwKTtfM2EwLmFwcGVuZENoaWxkKG5vZGUpO2JyZWFrO2Nh",
				"c2UgImZpcnN0IjppZihfM2EwLmZpcnN0Q2hpbGQpe18zOTcobm9kZSxfM2EwLmZpcnN0Q2hpbGQpO2Jy",
				"ZWFrO31kZWZhdWx0Ol8zYTAuYXBwZW5kQ2hpbGQobm9kZSk7fX1yZXR1cm4gbm9kZTt9O18zOTAuY3Jl",
				"YXRlPWZ1bmN0aW9uIGNyZWF0ZSh0YWcsXzNhMixfM2EzLHBvcyl7dmFyIGRvYz13aW4uZG9jO2lmKF8z",
				"YTMpe18zYTM9ZG9tLmJ5SWQoXzNhMyk7ZG9jPV8zYTMub3duZXJEb2N1bWVudDt9aWYodHlwZW9mIHRh",
				"Zz09InN0cmluZyIpe3RhZz1kb2MuY3JlYXRlRWxlbWVudCh0YWcpO31pZihfM2EyKXthdHRyLnNldCh0",
				"YWcsXzNhMik7fWlmKF8zYTMpe18zOTAucGxhY2UodGFnLF8zYTMscG9zKTt9cmV0dXJuIHRhZzt9O18z",
				"OTAuZW1wdHk9aGFzKCJpZSIpP2Z1bmN0aW9uKG5vZGUpe25vZGU9ZG9tLmJ5SWQobm9kZSk7Zm9yKHZh",
				"ciBjO2M9bm9kZS5sYXN0Q2hpbGQ7KXtfMzkwLmRlc3Ryb3koYyk7fX06ZnVuY3Rpb24obm9kZSl7ZG9t",
				"LmJ5SWQobm9kZSkuaW5uZXJIVE1MPSIiO307XzM5MC5kZXN0cm95PWZ1bmN0aW9uIGRlc3Ryb3kobm9k",
				"ZSl7bm9kZT1kb20uYnlJZChub2RlKTt0cnl7dmFyIGRvYz1ub2RlLm93bmVyRG9jdW1lbnQ7aWYoIV8z",
				"OWJ8fF8zOWMhPWRvYyl7XzM5Yj1kb2MuY3JlYXRlRWxlbWVudCgiZGl2Iik7XzM5Yz1kb2M7fV8zOWIu",
				"YXBwZW5kQ2hpbGQobm9kZS5wYXJlbnROb2RlP25vZGUucGFyZW50Tm9kZS5yZW1vdmVDaGlsZChub2Rl",
				"KTpub2RlKTtfMzliLmlubmVySFRNTD0iIjt9Y2F0Y2goZSl7fX07fSk7fSwiZG9qby90ZXh0IjpmdW5j",
				"dGlvbigpe2RlZmluZShbIi4vX2Jhc2Uva2VybmVsIiwicmVxdWlyZSIsIi4vaGFzIiwiLi9fYmFzZS94",
				"aHIiXSxmdW5jdGlvbihkb2pvLF8zYTQsaGFzLHhocil7dmFyIF8zYTU7aWYoMSl7XzNhNT1mdW5jdGlv",
				"bih1cmwsc3luYyxsb2FkKXt4aHIoIkdFVCIse3VybDp1cmwsc3luYzohIXN5bmMsbG9hZDpsb2FkfSk7",
				"fTt9ZWxzZXtpZihfM2E0LmdldFRleHQpe18zYTU9XzNhNC5nZXRUZXh0O31lbHNle2NvbnNvbGUuZXJy",
				"b3IoImRvam8vdGV4dCBwbHVnaW4gZmFpbGVkIHRvIGxvYWQgYmVjYXVzZSBsb2FkZXIgZG9lcyBub3Qg",
				"c3VwcG9ydCBnZXRUZXh0Iik7fX12YXIgXzNhNj17fSxfM2E3PWZ1bmN0aW9uKHRleHQpe2lmKHRleHQp",
				"e3RleHQ9dGV4dC5yZXBsYWNlKC9eXHMqPFw/eG1sKFxzKSt2ZXJzaW9uPVtcJ1wiXShcZCkqLihcZCkq",
				"W1wnXCJdKFxzKSpcPz4vaW0sIiIpO3ZhciBfM2E4PXRleHQubWF0Y2goLzxib2R5W14+XSo+XHMqKFtc",
				"c1xTXSspXHMqPFwvYm9keT4vaW0pO2lmKF8zYTgpe3RleHQ9XzNhOFsxXTt9fWVsc2V7dGV4dD0iIjt9",
				"cmV0dXJuIHRleHQ7fSxfM2E5PXt9LF8zYWE9e30sXzNhYj17ZHluYW1pYzp0cnVlLG5vcm1hbGl6ZTpm",
				"dW5jdGlvbihpZCxfM2FjKXt2YXIgXzNhZD1pZC5zcGxpdCgiISIpLHVybD1fM2FkWzBdO3JldHVybiAo",
				"L15cLi8udGVzdCh1cmwpP18zYWModXJsKTp1cmwpKyhfM2FkWzFdPyIhIitfM2FkWzFdOiIiKTt9LGxv",
				"YWQ6ZnVuY3Rpb24oaWQsXzNhZSxsb2FkKXt2YXIgXzNhZj1pZC5zcGxpdCgiISIpLF8zYjA9XzNhZi5s",
				"ZW5ndGg+MSxfM2IxPV8zYWZbMF0sdXJsPV8zYWUudG9VcmwoXzNhZlswXSksdGV4dD1fM2E5LF8zYjI9",
				"ZnVuY3Rpb24odGV4dCl7bG9hZChfM2IwP18zYTcodGV4dCk6dGV4dCk7fTtpZihfM2IxIGluIF8zYTYp",
				"e3RleHQ9XzNhNltfM2IxXTt9ZWxzZXtpZih1cmwgaW4gXzNhZS5jYWNoZSl7dGV4dD1fM2FlLmNhY2hl",
				"W3VybF07fWVsc2V7aWYodXJsIGluIF8zYTYpe3RleHQ9XzNhNlt1cmxdO319fWlmKHRleHQ9PT1fM2E5",
				"KXtpZihfM2FhW3VybF0pe18zYWFbdXJsXS5wdXNoKF8zYjIpO31lbHNle3ZhciBfM2IzPV8zYWFbdXJs",
				"XT1bXzNiMl07XzNhNSh1cmwsIV8zYWUuYXN5bmMsZnVuY3Rpb24odGV4dCl7XzNhNltfM2IxXT1fM2E2",
				"W3VybF09dGV4dDtmb3IodmFyIGk9MDtpPF8zYjMubGVuZ3RoOyl7XzNiM1tpKytdKHRleHQpO31kZWxl",
				"dGUgXzNhYVt1cmxdO30pO319ZWxzZXtfM2IyKHRleHQpO319fTtkb2pvLmNhY2hlPWZ1bmN0aW9uKF8z",
				"YjQsdXJsLF8zYjUpe3ZhciBrZXk7aWYodHlwZW9mIF8zYjQ9PSJzdHJpbmciKXtpZigvXC8vLnRlc3Qo",
				"XzNiNCkpe2tleT1fM2I0O18zYjU9dXJsO31lbHNle2tleT1fM2E0LnRvVXJsKF8zYjQucmVwbGFjZSgv",
				"XC4vZywiLyIpKyh1cmw/KCIvIit1cmwpOiIiKSk7fX1lbHNle2tleT1fM2I0KyIiO18zYjU9dXJsO312",
				"YXIgdmFsPShfM2I1IT11bmRlZmluZWQmJnR5cGVvZiBfM2I1IT0ic3RyaW5nIik/XzNiNS52YWx1ZTpf",
				"M2I1LF8zYjY9XzNiNSYmXzNiNS5zYW5pdGl6ZTtpZih0eXBlb2YgdmFsPT0ic3RyaW5nIil7XzNhNltr",
				"ZXldPXZhbDtyZXR1cm4gXzNiNj9fM2E3KHZhbCk6dmFsO31lbHNle2lmKHZhbD09PW51bGwpe2RlbGV0",
				"ZSBfM2E2W2tleV07cmV0dXJuIG51bGw7fWVsc2V7aWYoIShrZXkgaW4gXzNhNikpe18zYTUoa2V5LHRy",
				"dWUsZnVuY3Rpb24odGV4dCl7XzNhNltrZXldPXRleHQ7fSk7fXJldHVybiBfM2I2P18zYTcoXzNhNltr",
				"ZXldKTpfM2E2W2tleV07fX19O3JldHVybiBfM2FiO30pO30sImRvam8va2V5cyI6ZnVuY3Rpb24oKXtk",
				"ZWZpbmUoImRvam8va2V5cyIsWyIuL19iYXNlL2tlcm5lbCIsIi4vX2Jhc2Uvc25pZmYiXSxmdW5jdGlv",
				"bihkb2pvLGhhcyl7cmV0dXJuIGRvam8ua2V5cz17QkFDS1NQQUNFOjgsVEFCOjksQ0xFQVI6MTIsRU5U",
				"RVI6MTMsU0hJRlQ6MTYsQ1RSTDoxNyxBTFQ6MTgsTUVUQTpoYXMoInNhZmFyaSIpPzkxOjIyNCxQQVVT",
				"RToxOSxDQVBTX0xPQ0s6MjAsRVNDQVBFOjI3LFNQQUNFOjMyLFBBR0VfVVA6MzMsUEFHRV9ET1dOOjM0",
				"LEVORDozNSxIT01FOjM2LExFRlRfQVJST1c6MzcsVVBfQVJST1c6MzgsUklHSFRfQVJST1c6MzksRE9X",
				"Tl9BUlJPVzo0MCxJTlNFUlQ6NDUsREVMRVRFOjQ2LEhFTFA6NDcsTEVGVF9XSU5ET1c6OTEsUklHSFRf",
				"V0lORE9XOjkyLFNFTEVDVDo5MyxOVU1QQURfMDo5NixOVU1QQURfMTo5NyxOVU1QQURfMjo5OCxOVU1Q",
				"QURfMzo5OSxOVU1QQURfNDoxMDAsTlVNUEFEXzU6MTAxLE5VTVBBRF82OjEwMixOVU1QQURfNzoxMDMs",
				"TlVNUEFEXzg6MTA0LE5VTVBBRF85OjEwNSxOVU1QQURfTVVMVElQTFk6MTA2LE5VTVBBRF9QTFVTOjEw",
				"NyxOVU1QQURfRU5URVI6MTA4LE5VTVBBRF9NSU5VUzoxMDksTlVNUEFEX1BFUklPRDoxMTAsTlVNUEFE",
				"X0RJVklERToxMTEsRjE6MTEyLEYyOjExMyxGMzoxMTQsRjQ6MTE1LEY1OjExNixGNjoxMTcsRjc6MTE4",
				"LEY4OjExOSxGOToxMjAsRjEwOjEyMSxGMTE6MTIyLEYxMjoxMjMsRjEzOjEyNCxGMTQ6MTI1LEYxNTox",
				"MjYsTlVNX0xPQ0s6MTQ0LFNDUk9MTF9MT0NLOjE0NSxVUF9EUEFEOjE3NSxET1dOX0RQQUQ6MTc2LExF",
				"RlRfRFBBRDoxNzcsUklHSFRfRFBBRDoxNzgsY29weUtleTpoYXMoIm1hYyIpJiYhaGFzKCJhaXIiKT8o",
				"aGFzKCJzYWZhcmkiKT85MToyMjQpOjE3fTt9KTt9LCJkb2pvL2RvbVJlYWR5IjpmdW5jdGlvbigpe2Rl",
				"ZmluZShbIi4vaGFzIl0sZnVuY3Rpb24oaGFzKXt2YXIgXzNiNz10aGlzLGRvYz1kb2N1bWVudCxfM2I4",
				"PXsibG9hZGVkIjoxLCJjb21wbGV0ZSI6MX0sXzNiOT10eXBlb2YgZG9jLnJlYWR5U3RhdGUhPSJzdHJp",
				"bmciLF8zYmE9ISFfM2I4W2RvYy5yZWFkeVN0YXRlXTtpZihfM2I5KXtkb2MucmVhZHlTdGF0ZT0ibG9h",
				"ZGluZyI7fWlmKCFfM2JhKXt2YXIgXzNiYj1bXSxfM2JjPVtdLF8zYmQ9ZnVuY3Rpb24oZXZ0KXtldnQ9",
				"ZXZ0fHxfM2I3LmV2ZW50O2lmKF8zYmF8fChldnQudHlwZT09InJlYWR5c3RhdGVjaGFuZ2UiJiYhXzNi",
				"OFtkb2MucmVhZHlTdGF0ZV0pKXtyZXR1cm47fV8zYmE9MTtpZihfM2I5KXtkb2MucmVhZHlTdGF0ZT0i",
				"Y29tcGxldGUiO313aGlsZShfM2JiLmxlbmd0aCl7KF8zYmIuc2hpZnQoKSkoKTt9fSxvbj1mdW5jdGlv",
				"bihub2RlLF8zYmUpe25vZGUuYWRkRXZlbnRMaXN0ZW5lcihfM2JlLF8zYmQsZmFsc2UpO18zYmIucHVz",
				"aChmdW5jdGlvbigpe25vZGUucmVtb3ZlRXZlbnRMaXN0ZW5lcihfM2JlLF8zYmQsZmFsc2UpO30pO307",
				"aWYoIWhhcygiZG9tLWFkZGV2ZW50bGlzdGVuZXIiKSl7b249ZnVuY3Rpb24obm9kZSxfM2JmKXtfM2Jm",
				"PSJvbiIrXzNiZjtub2RlLmF0dGFjaEV2ZW50KF8zYmYsXzNiZCk7XzNiYi5wdXNoKGZ1bmN0aW9uKCl7",
				"bm9kZS5kZXRhY2hFdmVudChfM2JmLF8zYmQpO30pO307dmFyIGRpdj1kb2MuY3JlYXRlRWxlbWVudCgi",
				"ZGl2Iik7dHJ5e2lmKGRpdi5kb1Njcm9sbCYmXzNiNy5mcmFtZUVsZW1lbnQ9PT1udWxsKXtfM2JjLnB1",
				"c2goZnVuY3Rpb24oKXt0cnl7ZGl2LmRvU2Nyb2xsKCJsZWZ0Iik7cmV0dXJuIDE7fWNhdGNoKGUpe319",
				"KTt9fWNhdGNoKGUpe319b24oZG9jLCJET01Db250ZW50TG9hZGVkIik7b24oXzNiNywibG9hZCIpO2lm",
				"KCJvbnJlYWR5c3RhdGVjaGFuZ2UiIGluIGRvYyl7b24oZG9jLCJyZWFkeXN0YXRlY2hhbmdlIik7fWVs",
				"c2V7aWYoIV8zYjkpe18zYmMucHVzaChmdW5jdGlvbigpe3JldHVybiBfM2I4W2RvYy5yZWFkeVN0YXRl",
				"XTt9KTt9fWlmKF8zYmMubGVuZ3RoKXt2YXIgXzNjMD1mdW5jdGlvbigpe2lmKF8zYmEpe3JldHVybjt9",
				"dmFyIGk9XzNiYy5sZW5ndGg7d2hpbGUoaS0tKXtpZihfM2JjW2ldKCkpe18zYmQoInBvbGxlciIpO3Jl",
				"dHVybjt9fXNldFRpbWVvdXQoXzNjMCwzMCk7fTtfM2MwKCk7fX1mdW5jdGlvbiBfM2MxKF8zYzIpe2lm",
				"KF8zYmEpe18zYzIoMSk7fWVsc2V7XzNiYi5wdXNoKF8zYzIpO319O18zYzEubG9hZD1mdW5jdGlvbihp",
				"ZCxyZXEsbG9hZCl7XzNjMShsb2FkKTt9O3JldHVybiBfM2MxO30pO30sImRvam8vX2Jhc2UvbGFuZyI6",
				"ZnVuY3Rpb24oKXtkZWZpbmUoWyIuL2tlcm5lbCIsIi4uL2hhcyIsIi4vc25pZmYiXSxmdW5jdGlvbihk",
				"b2pvLGhhcyl7aGFzLmFkZCgiYnVnLWZvci1pbi1za2lwcy1zaGFkb3dlZCIsZnVuY3Rpb24oKXtmb3Io",
				"dmFyIGkgaW4ge3RvU3RyaW5nOjF9KXtyZXR1cm4gMDt9cmV0dXJuIDE7fSk7dmFyIF8zYzM9aGFzKCJi",
				"dWctZm9yLWluLXNraXBzLXNoYWRvd2VkIik/Imhhc093blByb3BlcnR5LnZhbHVlT2YuaXNQcm90b3R5",
				"cGVPZi5wcm9wZXJ0eUlzRW51bWVyYWJsZS50b0xvY2FsZVN0cmluZy50b1N0cmluZy5jb25zdHJ1Y3Rv",
				"ciIuc3BsaXQoIi4iKTpbXSxfM2M0PV8zYzMubGVuZ3RoLF8zYzU9ZnVuY3Rpb24oZGVzdCxfM2M2LF8z",
				"Yzcpe3ZhciBuYW1lLHMsaSxfM2M4PXt9O2ZvcihuYW1lIGluIF8zYzYpe3M9XzNjNltuYW1lXTtpZigh",
				"KG5hbWUgaW4gZGVzdCl8fChkZXN0W25hbWVdIT09cyYmKCEobmFtZSBpbiBfM2M4KXx8XzNjOFtuYW1l",
				"XSE9PXMpKSl7ZGVzdFtuYW1lXT1fM2M3P18zYzcocyk6czt9fWlmKGhhcygiYnVnLWZvci1pbi1za2lw",
				"cy1zaGFkb3dlZCIpKXtpZihfM2M2KXtmb3IoaT0wO2k8XzNjNDsrK2kpe25hbWU9XzNjM1tpXTtzPV8z",
				"YzZbbmFtZV07aWYoIShuYW1lIGluIGRlc3QpfHwoZGVzdFtuYW1lXSE9PXMmJighKG5hbWUgaW4gXzNj",
				"OCl8fF8zYzhbbmFtZV0hPT1zKSkpe2Rlc3RbbmFtZV09XzNjNz9fM2M3KHMpOnM7fX19fXJldHVybiBk",
				"ZXN0O30sXzNjOT1mdW5jdGlvbihkZXN0LF8zY2Epe2lmKCFkZXN0KXtkZXN0PXt9O31mb3IodmFyIGk9",
				"MSxsPWFyZ3VtZW50cy5sZW5ndGg7aTxsO2krKyl7bGFuZy5fbWl4aW4oZGVzdCxhcmd1bWVudHNbaV0p",
				"O31yZXR1cm4gZGVzdDt9LF8zY2I9ZnVuY3Rpb24oXzNjYyxfM2NkLF8zY2Upe3ZhciBwLGk9MCxfM2Nm",
				"PWRvam8uZ2xvYmFsO2lmKCFfM2NlKXtpZighXzNjYy5sZW5ndGgpe3JldHVybiBfM2NmO31lbHNle3A9",
				"XzNjY1tpKytdO3RyeXtfM2NlPWRvam8uc2NvcGVNYXBbcF0mJmRvam8uc2NvcGVNYXBbcF1bMV07fWNh",
				"dGNoKGUpe31fM2NlPV8zY2V8fChwIGluIF8zY2Y/XzNjZltwXTooXzNjZD9fM2NmW3BdPXt9OnVuZGVm",
				"aW5lZCkpO319d2hpbGUoXzNjZSYmKHA9XzNjY1tpKytdKSl7XzNjZT0ocCBpbiBfM2NlP18zY2VbcF06",
				"KF8zY2Q/XzNjZVtwXT17fTp1bmRlZmluZWQpKTt9cmV0dXJuIF8zY2U7fSxfM2QwPWZ1bmN0aW9uKG5h",
				"bWUsXzNkMSxfM2QyKXt2YXIgXzNkMz1uYW1lLnNwbGl0KCIuIikscD1fM2QzLnBvcCgpLG9iaj1fM2Ni",
				"KF8zZDMsdHJ1ZSxfM2QyKTtyZXR1cm4gb2JqJiZwPyhvYmpbcF09XzNkMSk6dW5kZWZpbmVkO30sXzNk",
				"ND1mdW5jdGlvbihuYW1lLF8zZDUsXzNkNil7cmV0dXJuIF8zY2IobmFtZS5zcGxpdCgiLiIpLF8zZDUs",
				"XzNkNik7fSxfM2Q3PWZ1bmN0aW9uKG5hbWUsb2JqKXtyZXR1cm4gbGFuZy5nZXRPYmplY3QobmFtZSxm",
				"YWxzZSxvYmopIT09dW5kZWZpbmVkO30sb3B0cz1PYmplY3QucHJvdG90eXBlLnRvU3RyaW5nLF8zZDg9",
				"ZnVuY3Rpb24oaXQpe3JldHVybiAodHlwZW9mIGl0PT0ic3RyaW5nInx8aXQgaW5zdGFuY2VvZiBTdHJp",
				"bmcpO30sXzNkOT1mdW5jdGlvbihpdCl7cmV0dXJuIGl0JiYoaXQgaW5zdGFuY2VvZiBBcnJheXx8dHlw",
				"ZW9mIGl0PT0iYXJyYXkiKTt9LF8zZGE9ZnVuY3Rpb24oaXQpe3JldHVybiBvcHRzLmNhbGwoaXQpPT09",
				"IltvYmplY3QgRnVuY3Rpb25dIjt9LF8zZGI9ZnVuY3Rpb24oaXQpe3JldHVybiBpdCE9PXVuZGVmaW5l",
				"ZCYmKGl0PT09bnVsbHx8dHlwZW9mIGl0PT0ib2JqZWN0Inx8bGFuZy5pc0FycmF5KGl0KXx8bGFuZy5p",
				"c0Z1bmN0aW9uKGl0KSk7fSxfM2RjPWZ1bmN0aW9uKGl0KXtyZXR1cm4gaXQmJml0IT09dW5kZWZpbmVk",
				"JiYhbGFuZy5pc1N0cmluZyhpdCkmJiFsYW5nLmlzRnVuY3Rpb24oaXQpJiYhKGl0LnRhZ05hbWUmJml0",
				"LnRhZ05hbWUudG9Mb3dlckNhc2UoKT09ImZvcm0iKSYmKGxhbmcuaXNBcnJheShpdCl8fGlzRmluaXRl",
				"KGl0Lmxlbmd0aCkpO30sXzNkZD1mdW5jdGlvbihpdCl7cmV0dXJuIGl0JiYhbGFuZy5pc0Z1bmN0aW9u",
				"KGl0KSYmL1x7XHMqXFtuYXRpdmUgY29kZVxdXHMqXH0vLnRlc3QoU3RyaW5nKGl0KSk7fSxfM2RlPWZ1",
				"bmN0aW9uKF8zZGYsXzNlMCl7Zm9yKHZhciBpPTEsbD1hcmd1bWVudHMubGVuZ3RoO2k8bDtpKyspe2xh",
				"bmcuX21peGluKF8zZGYucHJvdG90eXBlLGFyZ3VtZW50c1tpXSk7fXJldHVybiBfM2RmO30sXzNlMT1m",
				"dW5jdGlvbihfM2UyLF8zZTMpe3ZhciBwcmU9XzNlNChhcmd1bWVudHMsMik7dmFyIF8zZTU9bGFuZy5p",
				"c1N0cmluZyhfM2UzKTtyZXR1cm4gZnVuY3Rpb24oKXt2YXIgYXJncz1fM2U0KGFyZ3VtZW50cyk7dmFy",
				"IGY9XzNlNT8oXzNlMnx8ZG9qby5nbG9iYWwpW18zZTNdOl8zZTM7cmV0dXJuIGYmJmYuYXBwbHkoXzNl",
				"Mnx8dGhpcyxwcmUuY29uY2F0KGFyZ3MpKTt9O30sXzNlNj1mdW5jdGlvbihfM2U3LF8zZTgpe2lmKGFy",
				"Z3VtZW50cy5sZW5ndGg+Mil7cmV0dXJuIGxhbmcuX2hpdGNoQXJncy5hcHBseShkb2pvLGFyZ3VtZW50",
				"cyk7fWlmKCFfM2U4KXtfM2U4PV8zZTc7XzNlNz1udWxsO31pZihsYW5nLmlzU3RyaW5nKF8zZTgpKXtf",
				"M2U3PV8zZTd8fGRvam8uZ2xvYmFsO2lmKCFfM2U3W18zZThdKXt0aHJvdyAoWyJkb2pvLmhpdGNoOiBz",
				"Y29wZVtcIiIsXzNlOCwiXCJdIGlzIG51bGwgKHNjb3BlPVwiIixfM2U3LCJcIikiXS5qb2luKCIiKSk7",
				"fXJldHVybiBmdW5jdGlvbigpe3JldHVybiBfM2U3W18zZThdLmFwcGx5KF8zZTcsYXJndW1lbnRzfHxb",
				"XSk7fTt9cmV0dXJuICFfM2U3P18zZTg6ZnVuY3Rpb24oKXtyZXR1cm4gXzNlOC5hcHBseShfM2U3LGFy",
				"Z3VtZW50c3x8W10pO307fSxfM2U5PShmdW5jdGlvbigpe2Z1bmN0aW9uIFRNUCgpe307cmV0dXJuIGZ1",
				"bmN0aW9uKG9iaixfM2VhKXtUTVAucHJvdG90eXBlPW9iajt2YXIgdG1wPW5ldyBUTVAoKTtUTVAucHJv",
				"dG90eXBlPW51bGw7aWYoXzNlYSl7bGFuZy5fbWl4aW4odG1wLF8zZWEpO31yZXR1cm4gdG1wO307fSko",
				"KSxfM2ViPWZ1bmN0aW9uKG9iaixfM2VjLF8zZWQpe3JldHVybiAoXzNlZHx8W10pLmNvbmNhdChBcnJh",
				"eS5wcm90b3R5cGUuc2xpY2UuY2FsbChvYmosXzNlY3x8MCkpO30sXzNlND1oYXMoImllIik/KGZ1bmN0",
				"aW9uKCl7ZnVuY3Rpb24gc2xvdyhvYmosXzNlZSxfM2VmKXt2YXIgYXJyPV8zZWZ8fFtdO2Zvcih2YXIg",
				"eD1fM2VlfHwwO3g8b2JqLmxlbmd0aDt4Kyspe2Fyci5wdXNoKG9ialt4XSk7fXJldHVybiBhcnI7fTty",
				"ZXR1cm4gZnVuY3Rpb24ob2JqKXtyZXR1cm4gKChvYmouaXRlbSk/c2xvdzpfM2ViKS5hcHBseSh0aGlz",
				"LGFyZ3VtZW50cyk7fTt9KSgpOl8zZWIsXzNmMD1mdW5jdGlvbihfM2YxKXt2YXIgYXJyPVtudWxsXTty",
				"ZXR1cm4gbGFuZy5oaXRjaC5hcHBseShkb2pvLGFyci5jb25jYXQobGFuZy5fdG9BcnJheShhcmd1bWVu",
				"dHMpKSk7fSxfM2YyPWZ1bmN0aW9uKHNyYyl7aWYoIXNyY3x8dHlwZW9mIHNyYyE9Im9iamVjdCJ8fGxh",
				"bmcuaXNGdW5jdGlvbihzcmMpKXtyZXR1cm4gc3JjO31pZihzcmMubm9kZVR5cGUmJiJjbG9uZU5vZGUi",
				"IGluIHNyYyl7cmV0dXJuIHNyYy5jbG9uZU5vZGUodHJ1ZSk7fWlmKHNyYyBpbnN0YW5jZW9mIERhdGUp",
				"e3JldHVybiBuZXcgRGF0ZShzcmMuZ2V0VGltZSgpKTt9aWYoc3JjIGluc3RhbmNlb2YgUmVnRXhwKXty",
				"ZXR1cm4gbmV3IFJlZ0V4cChzcmMpO312YXIgcixpLGw7aWYobGFuZy5pc0FycmF5KHNyYykpe3I9W107",
				"Zm9yKGk9MCxsPXNyYy5sZW5ndGg7aTxsOysraSl7aWYoaSBpbiBzcmMpe3IucHVzaChfM2YyKHNyY1tp",
				"XSkpO319fWVsc2V7cj1zcmMuY29uc3RydWN0b3I/bmV3IHNyYy5jb25zdHJ1Y3RvcigpOnt9O31yZXR1",
				"cm4gbGFuZy5fbWl4aW4ocixzcmMsXzNmMik7fSx0cmltPVN0cmluZy5wcm90b3R5cGUudHJpbT9mdW5j",
				"dGlvbihzdHIpe3JldHVybiBzdHIudHJpbSgpO306ZnVuY3Rpb24oc3RyKXtyZXR1cm4gc3RyLnJlcGxh",
				"Y2UoL15cc1xzKi8sIiIpLnJlcGxhY2UoL1xzXHMqJC8sIiIpO30sXzNmMz0vXHsoW15cfV0rKVx9L2cs",
				"XzNmND1mdW5jdGlvbih0bXBsLG1hcCxfM2Y1KXtyZXR1cm4gdG1wbC5yZXBsYWNlKF8zZjV8fF8zZjMs",
				"bGFuZy5pc0Z1bmN0aW9uKG1hcCk/bWFwOmZ1bmN0aW9uKF8zZjYsayl7cmV0dXJuIF8zZDQoayxmYWxz",
				"ZSxtYXApO30pO30sbGFuZz17X2V4dHJhTmFtZXM6XzNjMyxfbWl4aW46XzNjNSxtaXhpbjpfM2M5LHNl",
				"dE9iamVjdDpfM2QwLGdldE9iamVjdDpfM2Q0LGV4aXN0czpfM2Q3LGlzU3RyaW5nOl8zZDgsaXNBcnJh",
				"eTpfM2Q5LGlzRnVuY3Rpb246XzNkYSxpc09iamVjdDpfM2RiLGlzQXJyYXlMaWtlOl8zZGMsaXNBbGll",
				"bjpfM2RkLGV4dGVuZDpfM2RlLF9oaXRjaEFyZ3M6XzNlMSxoaXRjaDpfM2U2LGRlbGVnYXRlOl8zZTks",
				"X3RvQXJyYXk6XzNlNCxwYXJ0aWFsOl8zZjAsY2xvbmU6XzNmMix0cmltOnRyaW0scmVwbGFjZTpfM2Y0",
				"fTsxJiZfM2M5KGRvam8sbGFuZyk7cmV0dXJuIGxhbmc7fSk7fSwiZG9qby9FdmVudGVkIjpmdW5jdGlv",
				"bigpe2RlZmluZShbIi4vYXNwZWN0IiwiLi9vbiJdLGZ1bmN0aW9uKF8zZjcsb24peyJ1c2Ugc3RyaWN0",
				"Ijt2YXIgXzNmOD1fM2Y3LmFmdGVyO2Z1bmN0aW9uIF8zZjkoKXt9O18zZjkucHJvdG90eXBlPXtvbjpm",
				"dW5jdGlvbih0eXBlLF8zZmEpe3JldHVybiBvbi5wYXJzZSh0aGlzLHR5cGUsXzNmYSxmdW5jdGlvbihf",
				"M2ZiLHR5cGUpe3JldHVybiBfM2Y4KF8zZmIsIm9uIit0eXBlLF8zZmEsdHJ1ZSk7fSk7fSxlbWl0OmZ1",
				"bmN0aW9uKHR5cGUsXzNmYyl7dmFyIGFyZ3M9W3RoaXNdO2FyZ3MucHVzaC5hcHBseShhcmdzLGFyZ3Vt",
				"ZW50cyk7cmV0dXJuIG9uLmVtaXQuYXBwbHkob24sYXJncyk7fX07cmV0dXJuIF8zZjk7fSk7fSwiZG9q",
				"by9tb3VzZSI6ZnVuY3Rpb24oKXtkZWZpbmUoWyIuL19iYXNlL2tlcm5lbCIsIi4vb24iLCIuL2hhcyIs",
				"Ii4vZG9tIiwiLi9fYmFzZS93aW5kb3ciXSxmdW5jdGlvbihkb2pvLG9uLGhhcyxkb20sd2luKXtoYXMu",
				"YWRkKCJkb20tcXVpcmtzIix3aW4uZG9jJiZ3aW4uZG9jLmNvbXBhdE1vZGU9PSJCYWNrQ29tcGF0Iik7",
				"aGFzLmFkZCgiZXZlbnRzLW1vdXNlZW50ZXIiLHdpbi5kb2MmJiJvbm1vdXNlZW50ZXIiIGluIHdpbi5k",
				"b2MuY3JlYXRlRWxlbWVudCgiZGl2IikpO3ZhciBfM2ZkO2lmKGhhcygiZG9tLXF1aXJrcyIpfHwhaGFz",
				"KCJkb20tYWRkZXZlbnRsaXN0ZW5lciIpKXtfM2ZkPXtMRUZUOjEsTUlERExFOjQsUklHSFQ6Mixpc0J1",
				"dHRvbjpmdW5jdGlvbihlLF8zZmUpe3JldHVybiBlLmJ1dHRvbiZfM2ZlO30saXNMZWZ0OmZ1bmN0aW9u",
				"KGUpe3JldHVybiBlLmJ1dHRvbiYxO30saXNNaWRkbGU6ZnVuY3Rpb24oZSl7cmV0dXJuIGUuYnV0dG9u",
				"JjQ7fSxpc1JpZ2h0OmZ1bmN0aW9uKGUpe3JldHVybiBlLmJ1dHRvbiYyO319O31lbHNle18zZmQ9e0xF",
				"RlQ6MCxNSURETEU6MSxSSUdIVDoyLGlzQnV0dG9uOmZ1bmN0aW9uKGUsXzNmZil7cmV0dXJuIGUuYnV0",
				"dG9uPT1fM2ZmO30saXNMZWZ0OmZ1bmN0aW9uKGUpe3JldHVybiBlLmJ1dHRvbj09MDt9LGlzTWlkZGxl",
				"OmZ1bmN0aW9uKGUpe3JldHVybiBlLmJ1dHRvbj09MTt9LGlzUmlnaHQ6ZnVuY3Rpb24oZSl7cmV0dXJu",
				"IGUuYnV0dG9uPT0yO319O31kb2pvLm1vdXNlQnV0dG9ucz1fM2ZkO2Z1bmN0aW9uIF80MDAodHlwZSxf",
				"NDAxKXt2YXIgXzQwMj1mdW5jdGlvbihub2RlLF80MDMpe3JldHVybiBvbihub2RlLHR5cGUsZnVuY3Rp",
				"b24oZXZ0KXtpZighZG9tLmlzRGVzY2VuZGFudChldnQucmVsYXRlZFRhcmdldCxfNDAxP2V2dC50YXJn",
				"ZXQ6bm9kZSkpe3JldHVybiBfNDAzLmNhbGwodGhpcyxldnQpO319KTt9O2lmKCFfNDAxKXtfNDAyLmJ1",
				"YmJsZT1fNDAwKHR5cGUsdHJ1ZSk7fXJldHVybiBfNDAyO307cmV0dXJuIHtlbnRlcjpfNDAwKCJtb3Vz",
				"ZW92ZXIiKSxsZWF2ZTpfNDAwKCJtb3VzZW91dCIpLGlzTGVmdDpfM2ZkLmlzTGVmdCxpc01pZGRsZTpf",
				"M2ZkLmlzTWlkZGxlLGlzUmlnaHQ6XzNmZC5pc1JpZ2h0fTt9KTt9LCJkb2pvL3RvcGljIjpmdW5jdGlv",
				"bigpe2RlZmluZShbIi4vRXZlbnRlZCJdLGZ1bmN0aW9uKF80MDQpe3ZhciBodWI9bmV3IF80MDQ7cmV0",
				"dXJuIHtwdWJsaXNoOmZ1bmN0aW9uKF80MDUsXzQwNil7cmV0dXJuIGh1Yi5lbWl0LmFwcGx5KGh1Yixh",
				"cmd1bWVudHMpO30sc3Vic2NyaWJlOmZ1bmN0aW9uKF80MDcsXzQwOCl7cmV0dXJuIGh1Yi5vbi5hcHBs",
				"eShodWIsYXJndW1lbnRzKTt9fTt9KTt9LCJkb2pvL19iYXNlL3hociI6ZnVuY3Rpb24oKXtkZWZpbmUo",
				"WyIuL2tlcm5lbCIsIi4vc25pZmYiLCJyZXF1aXJlIiwiLi4vaW8tcXVlcnkiLCIuLi9kb20iLCIuLi9k",
				"b20tZm9ybSIsIi4vRGVmZXJyZWQiLCIuL2pzb24iLCIuL2xhbmciLCIuL2FycmF5IiwiLi4vb24iXSxm",
				"dW5jdGlvbihkb2pvLGhhcyxfNDA5LGlvcSxkb20sXzQwYSxfNDBiLGpzb24sbGFuZyxfNDBjLG9uKXto",
				"YXMuYWRkKCJuYXRpdmUteGhyIixmdW5jdGlvbigpe3JldHVybiB0eXBlb2YgWE1MSHR0cFJlcXVlc3Qh",
				"PT0idW5kZWZpbmVkIjt9KTtpZigxKXtkb2pvLl94aHJPYmo9XzQwOS5nZXRYaHI7fWVsc2V7aWYoaGFz",
				"KCJuYXRpdmUteGhyIikpe2Rvam8uX3hock9iaj1mdW5jdGlvbigpe3RyeXtyZXR1cm4gbmV3IFhNTEh0",
				"dHBSZXF1ZXN0KCk7fWNhdGNoKGUpe3Rocm93IG5ldyBFcnJvcigiWE1MSFRUUCBub3QgYXZhaWxhYmxl",
				"OiAiK2UpO319O31lbHNle2Zvcih2YXIgXzQwZD1bIk1zeG1sMi5YTUxIVFRQIiwiTWljcm9zb2Z0LlhN",
				"TEhUVFAiLCJNc3htbDIuWE1MSFRUUC40LjAiXSxfNDBlLGk9MDtpPDM7KXt0cnl7XzQwZT1fNDBkW2kr",
				"K107aWYobmV3IEFjdGl2ZVhPYmplY3QoXzQwZSkpe2JyZWFrO319Y2F0Y2goZSl7fX1kb2pvLl94aHJP",
				"Ymo9ZnVuY3Rpb24oKXtyZXR1cm4gbmV3IEFjdGl2ZVhPYmplY3QoXzQwZSk7fTt9fXZhciBjZmc9ZG9q",
				"by5jb25maWc7ZG9qby5vYmplY3RUb1F1ZXJ5PWlvcS5vYmplY3RUb1F1ZXJ5O2Rvam8ucXVlcnlUb09i",
				"amVjdD1pb3EucXVlcnlUb09iamVjdDtkb2pvLmZpZWxkVG9PYmplY3Q9XzQwYS5maWVsZFRvT2JqZWN0",
				"O2Rvam8uZm9ybVRvT2JqZWN0PV80MGEudG9PYmplY3Q7ZG9qby5mb3JtVG9RdWVyeT1fNDBhLnRvUXVl",
				"cnk7ZG9qby5mb3JtVG9Kc29uPV80MGEudG9Kc29uO2Rvam8uX2Jsb2NrQXN5bmM9ZmFsc2U7dmFyIF80",
				"MGY9ZG9qby5fY29udGVudEhhbmRsZXJzPWRvam8uY29udGVudEhhbmRsZXJzPXsidGV4dCI6ZnVuY3Rp",
				"b24oeGhyKXtyZXR1cm4geGhyLnJlc3BvbnNlVGV4dDt9LCJqc29uIjpmdW5jdGlvbih4aHIpe3JldHVy",
				"biBqc29uLmZyb21Kc29uKHhoci5yZXNwb25zZVRleHR8fG51bGwpO30sImpzb24tY29tbWVudC1maWx0",
				"ZXJlZCI6ZnVuY3Rpb24oeGhyKXtpZighZG9qby5jb25maWcudXNlQ29tbWVudGVkSnNvbil7Y29uc29s",
				"ZS53YXJuKCJDb25zaWRlciB1c2luZyB0aGUgc3RhbmRhcmQgbWltZXR5cGU6YXBwbGljYXRpb24vanNv",
				"bi4iKyIganNvbi1jb21tZW50aW5nIGNhbiBpbnRyb2R1Y2Ugc2VjdXJpdHkgaXNzdWVzLiBUbyIrIiBk",
				"ZWNyZWFzZSB0aGUgY2hhbmNlcyBvZiBoaWphY2tpbmcsIHVzZSB0aGUgc3RhbmRhcmQgdGhlICdqc29u",
				"JyBoYW5kbGVyIGFuZCIrIiBwcmVmaXggeW91ciBqc29uIHdpdGg6IHt9JiZcbiIrIlVzZSBkakNvbmZp",
				"Zy51c2VDb21tZW50ZWRKc29uPXRydWUgdG8gdHVybiBvZmYgdGhpcyBtZXNzYWdlLiIpO312YXIgXzQx",
				"MD14aHIucmVzcG9uc2VUZXh0O3ZhciBfNDExPV80MTAuaW5kZXhPZigiLyoiKTt2YXIgXzQxMj1fNDEw",
				"Lmxhc3RJbmRleE9mKCIqLyIpO2lmKF80MTE9PS0xfHxfNDEyPT0tMSl7dGhyb3cgbmV3IEVycm9yKCJK",
				"U09OIHdhcyBub3QgY29tbWVudCBmaWx0ZXJlZCIpO31yZXR1cm4ganNvbi5mcm9tSnNvbihfNDEwLnN1",
				"YnN0cmluZyhfNDExKzIsXzQxMikpO30sImphdmFzY3JpcHQiOmZ1bmN0aW9uKHhocil7cmV0dXJuIGRv",
				"am8uZXZhbCh4aHIucmVzcG9uc2VUZXh0KTt9LCJ4bWwiOmZ1bmN0aW9uKHhocil7dmFyIF80MTM9eGhy",
				"LnJlc3BvbnNlWE1MO2lmKGhhcygiaWUiKSl7aWYoKCFfNDEzfHwhXzQxMy5kb2N1bWVudEVsZW1lbnQp",
				"KXt2YXIgbXM9ZnVuY3Rpb24obil7cmV0dXJuICJNU1hNTCIrbisiLkRPTURvY3VtZW50Ijt9O3ZhciBk",
				"cD1bIk1pY3Jvc29mdC5YTUxET00iLG1zKDYpLG1zKDQpLG1zKDMpLG1zKDIpXTtfNDBjLnNvbWUoZHAs",
				"ZnVuY3Rpb24ocCl7dHJ5e3ZhciBkb209bmV3IEFjdGl2ZVhPYmplY3QocCk7ZG9tLmFzeW5jPWZhbHNl",
				"O2RvbS5sb2FkWE1MKHhoci5yZXNwb25zZVRleHQpO180MTM9ZG9tO31jYXRjaChlKXtyZXR1cm4gZmFs",
				"c2U7fXJldHVybiB0cnVlO30pO319cmV0dXJuIF80MTM7fSwianNvbi1jb21tZW50LW9wdGlvbmFsIjpm",
				"dW5jdGlvbih4aHIpe2lmKHhoci5yZXNwb25zZVRleHQmJi9eW157XFtdKlwvXCovLnRlc3QoeGhyLnJl",
				"c3BvbnNlVGV4dCkpe3JldHVybiBfNDBmWyJqc29uLWNvbW1lbnQtZmlsdGVyZWQiXSh4aHIpO31lbHNl",
				"e3JldHVybiBfNDBmWyJqc29uIl0oeGhyKTt9fX07ZG9qby5faW9TZXRBcmdzPWZ1bmN0aW9uKGFyZ3Ms",
				"XzQxNCxfNDE1LF80MTYpe3ZhciBfNDE3PXthcmdzOmFyZ3MsdXJsOmFyZ3MudXJsfTt2YXIgXzQxOD1u",
				"dWxsO2lmKGFyZ3MuZm9ybSl7dmFyIGZvcm09ZG9tLmJ5SWQoYXJncy5mb3JtKTt2YXIgXzQxOT1mb3Jt",
				"LmdldEF0dHJpYnV0ZU5vZGUoImFjdGlvbiIpO180MTcudXJsPV80MTcudXJsfHwoXzQxOT9fNDE5LnZh",
				"bHVlOm51bGwpO180MTg9XzQwYS50b09iamVjdChmb3JtKTt9dmFyIF80MWE9W3t9XTtpZihfNDE4KXtf",
				"NDFhLnB1c2goXzQxOCk7fWlmKGFyZ3MuY29udGVudCl7XzQxYS5wdXNoKGFyZ3MuY29udGVudCk7fWlm",
				"KGFyZ3MucHJldmVudENhY2hlKXtfNDFhLnB1c2goeyJkb2pvLnByZXZlbnRDYWNoZSI6bmV3IERhdGUo",
				"KS52YWx1ZU9mKCl9KTt9XzQxNy5xdWVyeT1pb3Eub2JqZWN0VG9RdWVyeShsYW5nLm1peGluLmFwcGx5",
				"KG51bGwsXzQxYSkpO180MTcuaGFuZGxlQXM9YXJncy5oYW5kbGVBc3x8InRleHQiO3ZhciBkPW5ldyBf",
				"NDBiKF80MTQpO2QuYWRkQ2FsbGJhY2tzKF80MTUsZnVuY3Rpb24oXzQxYil7cmV0dXJuIF80MTYoXzQx",
				"YixkKTt9KTt2YXIgbGQ9YXJncy5sb2FkO2lmKGxkJiZsYW5nLmlzRnVuY3Rpb24obGQpKXtkLmFkZENh",
				"bGxiYWNrKGZ1bmN0aW9uKF80MWMpe3JldHVybiBsZC5jYWxsKGFyZ3MsXzQxYyxfNDE3KTt9KTt9dmFy",
				"IGVycj1hcmdzLmVycm9yO2lmKGVyciYmbGFuZy5pc0Z1bmN0aW9uKGVycikpe2QuYWRkRXJyYmFjayhm",
				"dW5jdGlvbihfNDFkKXtyZXR1cm4gZXJyLmNhbGwoYXJncyxfNDFkLF80MTcpO30pO312YXIgXzQxZT1h",
				"cmdzLmhhbmRsZTtpZihfNDFlJiZsYW5nLmlzRnVuY3Rpb24oXzQxZSkpe2QuYWRkQm90aChmdW5jdGlv",
				"bihfNDFmKXtyZXR1cm4gXzQxZS5jYWxsKGFyZ3MsXzQxZixfNDE3KTt9KTt9aWYoY2ZnLmlvUHVibGlz",
				"aCYmZG9qby5wdWJsaXNoJiZfNDE3LmFyZ3MuaW9QdWJsaXNoIT09ZmFsc2Upe2QuYWRkQ2FsbGJhY2tz",
				"KGZ1bmN0aW9uKHJlcyl7ZG9qby5wdWJsaXNoKCIvZG9qby9pby9sb2FkIixbZCxyZXNdKTtyZXR1cm4g",
				"cmVzO30sZnVuY3Rpb24ocmVzKXtkb2pvLnB1Ymxpc2goIi9kb2pvL2lvL2Vycm9yIixbZCxyZXNdKTty",
				"ZXR1cm4gcmVzO30pO2QuYWRkQm90aChmdW5jdGlvbihyZXMpe2Rvam8ucHVibGlzaCgiL2Rvam8vaW8v",
				"ZG9uZSIsW2QscmVzXSk7cmV0dXJuIHJlczt9KTt9ZC5pb0FyZ3M9XzQxNztyZXR1cm4gZDt9O3ZhciBf",
				"NDIwPWZ1bmN0aW9uKGRmZCl7ZGZkLmNhbmNlbGVkPXRydWU7dmFyIHhocj1kZmQuaW9BcmdzLnhocjt2",
				"YXIgXzQyMT10eXBlb2YgeGhyLmFib3J0O2lmKF80MjE9PSJmdW5jdGlvbiJ8fF80MjE9PSJvYmplY3Qi",
				"fHxfNDIxPT0idW5rbm93biIpe3hoci5hYm9ydCgpO312YXIgZXJyPWRmZC5pb0FyZ3MuZXJyb3I7aWYo",
				"IWVycil7ZXJyPW5ldyBFcnJvcigieGhyIGNhbmNlbGxlZCIpO2Vyci5kb2pvVHlwZT0iY2FuY2VsIjt9",
				"cmV0dXJuIGVycjt9O3ZhciBfNDIyPWZ1bmN0aW9uKGRmZCl7dmFyIHJldD1fNDBmW2RmZC5pb0FyZ3Mu",
				"aGFuZGxlQXNdKGRmZC5pb0FyZ3MueGhyKTtyZXR1cm4gcmV0PT09dW5kZWZpbmVkP251bGw6cmV0O307",
				"dmFyIF80MjM9ZnVuY3Rpb24oXzQyNCxkZmQpe2lmKCFkZmQuaW9BcmdzLmFyZ3MuZmFpbE9rKXtjb25z",
				"b2xlLmVycm9yKF80MjQpO31yZXR1cm4gXzQyNDt9O3ZhciBfNDI1PW51bGw7dmFyIF80MjY9W107dmFy",
				"IF80Mjc9MDt2YXIgXzQyOD1mdW5jdGlvbihkZmQpe2lmKF80Mjc8PTApe180Mjc9MDtpZihjZmcuaW9Q",
				"dWJsaXNoJiZkb2pvLnB1Ymxpc2gmJighZGZkfHxkZmQmJmRmZC5pb0FyZ3MuYXJncy5pb1B1Ymxpc2gh",
				"PT1mYWxzZSkpe2Rvam8ucHVibGlzaCgiL2Rvam8vaW8vc3RvcCIpO319fTt2YXIgXzQyOT1mdW5jdGlv",
				"bigpe3ZhciBub3c9KG5ldyBEYXRlKCkpLmdldFRpbWUoKTtpZighZG9qby5fYmxvY2tBc3luYyl7Zm9y",
				"KHZhciBpPTAsdGlmO2k8XzQyNi5sZW5ndGgmJih0aWY9XzQyNltpXSk7aSsrKXt2YXIgZGZkPXRpZi5k",
				"ZmQ7dmFyIGZ1bmM9ZnVuY3Rpb24oKXtpZighZGZkfHxkZmQuY2FuY2VsZWR8fCF0aWYudmFsaWRDaGVj",
				"ayhkZmQpKXtfNDI2LnNwbGljZShpLS0sMSk7XzQyNy09MTt9ZWxzZXtpZih0aWYuaW9DaGVjayhkZmQp",
				"KXtfNDI2LnNwbGljZShpLS0sMSk7dGlmLnJlc0hhbmRsZShkZmQpO180MjctPTE7fWVsc2V7aWYoZGZk",
				"LnN0YXJ0VGltZSl7aWYoZGZkLnN0YXJ0VGltZSsoZGZkLmlvQXJncy5hcmdzLnRpbWVvdXR8fDApPG5v",
				"dyl7XzQyNi5zcGxpY2UoaS0tLDEpO3ZhciBlcnI9bmV3IEVycm9yKCJ0aW1lb3V0IGV4Y2VlZGVkIik7",
				"ZXJyLmRvam9UeXBlPSJ0aW1lb3V0IjtkZmQuZXJyYmFjayhlcnIpO2RmZC5jYW5jZWwoKTtfNDI3LT0x",
				"O319fX19O2lmKGRvam8uY29uZmlnLmRlYnVnQXRBbGxDb3N0cyl7ZnVuYy5jYWxsKHRoaXMpO31lbHNl",
				"e2Z1bmMuY2FsbCh0aGlzKTt9fX1fNDI4KGRmZCk7aWYoIV80MjYubGVuZ3RoKXtjbGVhckludGVydmFs",
				"KF80MjUpO180MjU9bnVsbDt9fTtkb2pvLl9pb0NhbmNlbEFsbD1mdW5jdGlvbigpe3RyeXtfNDBjLmZv",
				"ckVhY2goXzQyNixmdW5jdGlvbihpKXt0cnl7aS5kZmQuY2FuY2VsKCk7fWNhdGNoKGUpe319KTt9Y2F0",
				"Y2goZSl7fX07aWYoaGFzKCJpZSIpKXtvbih3aW5kb3csInVubG9hZCIsZG9qby5faW9DYW5jZWxBbGwp",
				"O31kb2pvLl9pb05vdGlmeVN0YXJ0PWZ1bmN0aW9uKGRmZCl7aWYoY2ZnLmlvUHVibGlzaCYmZG9qby5w",
				"dWJsaXNoJiZkZmQuaW9BcmdzLmFyZ3MuaW9QdWJsaXNoIT09ZmFsc2Upe2lmKCFfNDI3KXtkb2pvLnB1",
				"Ymxpc2goIi9kb2pvL2lvL3N0YXJ0Iik7fV80MjcrPTE7ZG9qby5wdWJsaXNoKCIvZG9qby9pby9zZW5k",
				"IixbZGZkXSk7fX07ZG9qby5faW9XYXRjaD1mdW5jdGlvbihkZmQsXzQyYSxfNDJiLF80MmMpe3ZhciBh",
				"cmdzPWRmZC5pb0FyZ3MuYXJncztpZihhcmdzLnRpbWVvdXQpe2RmZC5zdGFydFRpbWU9KG5ldyBEYXRl",
				"KCkpLmdldFRpbWUoKTt9XzQyNi5wdXNoKHtkZmQ6ZGZkLHZhbGlkQ2hlY2s6XzQyYSxpb0NoZWNrOl80",
				"MmIscmVzSGFuZGxlOl80MmN9KTtpZighXzQyNSl7XzQyNT1zZXRJbnRlcnZhbChfNDI5LDUwKTt9aWYo",
				"YXJncy5zeW5jKXtfNDI5KCk7fX07dmFyIF80MmQ9ImFwcGxpY2F0aW9uL3gtd3d3LWZvcm0tdXJsZW5j",
				"b2RlZCI7dmFyIF80MmU9ZnVuY3Rpb24oZGZkKXtyZXR1cm4gZGZkLmlvQXJncy54aHIucmVhZHlTdGF0",
				"ZTt9O3ZhciBfNDJmPWZ1bmN0aW9uKGRmZCl7cmV0dXJuIDQ9PWRmZC5pb0FyZ3MueGhyLnJlYWR5U3Rh",
				"dGU7fTt2YXIgXzQzMD1mdW5jdGlvbihkZmQpe3ZhciB4aHI9ZGZkLmlvQXJncy54aHI7aWYoZG9qby5f",
				"aXNEb2N1bWVudE9rKHhocikpe2RmZC5jYWxsYmFjayhkZmQpO31lbHNle3ZhciBlcnI9bmV3IEVycm9y",
				"KCJVbmFibGUgdG8gbG9hZCAiK2RmZC5pb0FyZ3MudXJsKyIgc3RhdHVzOiIreGhyLnN0YXR1cyk7ZXJy",
				"LnN0YXR1cz14aHIuc3RhdHVzO2Vyci5yZXNwb25zZVRleHQ9eGhyLnJlc3BvbnNlVGV4dDtlcnIueGhy",
				"PXhocjtkZmQuZXJyYmFjayhlcnIpO319O2Rvam8uX2lvQWRkUXVlcnlUb1VybD1mdW5jdGlvbihfNDMx",
				"KXtpZihfNDMxLnF1ZXJ5Lmxlbmd0aCl7XzQzMS51cmwrPShfNDMxLnVybC5pbmRleE9mKCI/Iik9PS0x",
				"PyI/IjoiJiIpK180MzEucXVlcnk7XzQzMS5xdWVyeT1udWxsO319O2Rvam8ueGhyPWZ1bmN0aW9uKF80",
				"MzIsYXJncyxfNDMzKXt2YXIgZGZkPWRvam8uX2lvU2V0QXJncyhhcmdzLF80MjAsXzQyMixfNDIzKTt2",
				"YXIgXzQzND1kZmQuaW9BcmdzO3ZhciB4aHI9XzQzNC54aHI9ZG9qby5feGhyT2JqKF80MzQuYXJncyk7",
				"aWYoIXhocil7ZGZkLmNhbmNlbCgpO3JldHVybiBkZmQ7fWlmKCJwb3N0RGF0YSIgaW4gYXJncyl7XzQz",
				"NC5xdWVyeT1hcmdzLnBvc3REYXRhO31lbHNle2lmKCJwdXREYXRhIiBpbiBhcmdzKXtfNDM0LnF1ZXJ5",
				"PWFyZ3MucHV0RGF0YTt9ZWxzZXtpZigicmF3Qm9keSIgaW4gYXJncyl7XzQzNC5xdWVyeT1hcmdzLnJh",
				"d0JvZHk7fWVsc2V7aWYoKGFyZ3VtZW50cy5sZW5ndGg+MiYmIV80MzMpfHwiUE9TVHxQVVQiLmluZGV4",
				"T2YoXzQzMi50b1VwcGVyQ2FzZSgpKT09LTEpe2Rvam8uX2lvQWRkUXVlcnlUb1VybChfNDM0KTt9fX19",
				"eGhyLm9wZW4oXzQzMixfNDM0LnVybCxhcmdzLnN5bmMhPT10cnVlLGFyZ3MudXNlcnx8dW5kZWZpbmVk",
				"LGFyZ3MucGFzc3dvcmR8fHVuZGVmaW5lZCk7aWYoYXJncy5oZWFkZXJzKXtmb3IodmFyIGhkciBpbiBh",
				"cmdzLmhlYWRlcnMpe2lmKGhkci50b0xvd2VyQ2FzZSgpPT09ImNvbnRlbnQtdHlwZSImJiFhcmdzLmNv",
				"bnRlbnRUeXBlKXthcmdzLmNvbnRlbnRUeXBlPWFyZ3MuaGVhZGVyc1toZHJdO31lbHNle2lmKGFyZ3Mu",
				"aGVhZGVyc1toZHJdKXt4aHIuc2V0UmVxdWVzdEhlYWRlcihoZHIsYXJncy5oZWFkZXJzW2hkcl0pO319",
				"fX1pZihhcmdzLmNvbnRlbnRUeXBlIT09ZmFsc2Upe3hoci5zZXRSZXF1ZXN0SGVhZGVyKCJDb250ZW50",
				"LVR5cGUiLGFyZ3MuY29udGVudFR5cGV8fF80MmQpO31pZighYXJncy5oZWFkZXJzfHwhKCJYLVJlcXVl",
				"c3RlZC1XaXRoIiBpbiBhcmdzLmhlYWRlcnMpKXt4aHIuc2V0UmVxdWVzdEhlYWRlcigiWC1SZXF1ZXN0",
				"ZWQtV2l0aCIsIlhNTEh0dHBSZXF1ZXN0Iik7fWRvam8uX2lvTm90aWZ5U3RhcnQoZGZkKTtpZihkb2pv",
				"LmNvbmZpZy5kZWJ1Z0F0QWxsQ29zdHMpe3hoci5zZW5kKF80MzQucXVlcnkpO31lbHNle3RyeXt4aHIu",
				"c2VuZChfNDM0LnF1ZXJ5KTt9Y2F0Y2goZSl7XzQzNC5lcnJvcj1lO2RmZC5jYW5jZWwoKTt9fWRvam8u",
				"X2lvV2F0Y2goZGZkLF80MmUsXzQyZixfNDMwKTt4aHI9bnVsbDtyZXR1cm4gZGZkO307ZG9qby54aHJH",
				"ZXQ9ZnVuY3Rpb24oYXJncyl7cmV0dXJuIGRvam8ueGhyKCJHRVQiLGFyZ3MpO307ZG9qby5yYXdYaHJQ",
				"b3N0PWRvam8ueGhyUG9zdD1mdW5jdGlvbihhcmdzKXtyZXR1cm4gZG9qby54aHIoIlBPU1QiLGFyZ3Ms",
				"dHJ1ZSk7fTtkb2pvLnJhd1hoclB1dD1kb2pvLnhoclB1dD1mdW5jdGlvbihhcmdzKXtyZXR1cm4gZG9q",
				"by54aHIoIlBVVCIsYXJncyx0cnVlKTt9O2Rvam8ueGhyRGVsZXRlPWZ1bmN0aW9uKGFyZ3Mpe3JldHVy",
				"biBkb2pvLnhocigiREVMRVRFIixhcmdzKTt9O2Rvam8uX2lzRG9jdW1lbnRPaz1mdW5jdGlvbihodHRw",
				"KXt2YXIgc3RhdD1odHRwLnN0YXR1c3x8MDtzdGF0PShzdGF0Pj0yMDAmJnN0YXQ8MzAwKXx8c3RhdD09",
				"MzA0fHxzdGF0PT0xMjIzfHwhc3RhdDtyZXR1cm4gc3RhdDt9O2Rvam8uX2dldFRleHQ9ZnVuY3Rpb24o",
				"dXJsKXt2YXIgXzQzNTtkb2pvLnhockdldCh7dXJsOnVybCxzeW5jOnRydWUsbG9hZDpmdW5jdGlvbih0",
				"ZXh0KXtfNDM1PXRleHQ7fX0pO3JldHVybiBfNDM1O307bGFuZy5taXhpbihkb2pvLnhocix7X3hock9i",
				"ajpkb2pvLl94aHJPYmosZmllbGRUb09iamVjdDpfNDBhLmZpZWxkVG9PYmplY3QsZm9ybVRvT2JqZWN0",
				"Ol80MGEudG9PYmplY3Qsb2JqZWN0VG9RdWVyeTppb3Eub2JqZWN0VG9RdWVyeSxmb3JtVG9RdWVyeTpf",
				"NDBhLnRvUXVlcnksZm9ybVRvSnNvbjpfNDBhLnRvSnNvbixxdWVyeVRvT2JqZWN0OmlvcS5xdWVyeVRv",
				"T2JqZWN0LGNvbnRlbnRIYW5kbGVyczpfNDBmLF9pb1NldEFyZ3M6ZG9qby5faW9TZXRBcmdzLF9pb0Nh",
				"bmNlbEFsbDpkb2pvLl9pb0NhbmNlbEFsbCxfaW9Ob3RpZnlTdGFydDpkb2pvLl9pb05vdGlmeVN0YXJ0",
				"LF9pb1dhdGNoOmRvam8uX2lvV2F0Y2gsX2lvQWRkUXVlcnlUb1VybDpkb2pvLl9pb0FkZFF1ZXJ5VG9V",
				"cmwsX2lzRG9jdW1lbnRPazpkb2pvLl9pc0RvY3VtZW50T2ssX2dldFRleHQ6ZG9qby5fZ2V0VGV4dCxn",
				"ZXQ6ZG9qby54aHJHZXQscG9zdDpkb2pvLnhoclBvc3QscHV0OmRvam8ueGhyUHV0LGRlbDpkb2pvLnho",
				"ckRlbGV0ZX0pO3JldHVybiBkb2pvLnhocjt9KTt9LCJkb2pvL19iYXNlL3VubG9hZCI6ZnVuY3Rpb24o",
				"KXtkZWZpbmUoWyIuL2tlcm5lbCIsIi4vY29ubmVjdCJdLGZ1bmN0aW9uKGRvam8sXzQzNil7dmFyIHdp",
				"bj13aW5kb3c7ZG9qby5hZGRPbldpbmRvd1VubG9hZD1mdW5jdGlvbihvYmosXzQzNyl7aWYoIWRvam8u",
				"d2luZG93VW5sb2FkZWQpe180MzYuY29ubmVjdCh3aW4sInVubG9hZCIsKGRvam8ud2luZG93VW5sb2Fk",
				"ZWQ9ZnVuY3Rpb24oKXt9KSk7fV80MzYuY29ubmVjdCh3aW4sInVubG9hZCIsb2JqLF80MzcpO307ZG9q",
				"by5hZGRPblVubG9hZD1mdW5jdGlvbihvYmosXzQzOCl7XzQzNi5jb25uZWN0KHdpbiwiYmVmb3JldW5s",
				"b2FkIixvYmosXzQzOCk7fTtyZXR1cm4ge2FkZE9uV2luZG93VW5sb2FkOmRvam8uYWRkT25XaW5kb3dV",
				"bmxvYWQsYWRkT25VbmxvYWQ6ZG9qby5hZGRPblVubG9hZH07fSk7fSwiZG9qby9sb2FkSW5pdCI6ZnVu",
				"Y3Rpb24oKXtkZWZpbmUoWyIuL19iYXNlL2xvYWRlciJdLGZ1bmN0aW9uKF80Mzkpe3JldHVybiB7ZHlu",
				"YW1pYzowLG5vcm1hbGl6ZTpmdW5jdGlvbihpZCl7cmV0dXJuIGlkO30sbG9hZDpfNDM5LmxvYWRJbml0",
				"fTt9KTt9LCJkb2pvL19iYXNlL05vZGVMaXN0IjpmdW5jdGlvbigpe2RlZmluZShbIi4va2VybmVsIiwi",
				"Li4vcXVlcnkiLCIuL2FycmF5IiwiLi9odG1sIiwiLi4vTm9kZUxpc3QtZG9tIl0sZnVuY3Rpb24oZG9q",
				"byxfNDNhLF80M2Ipe3ZhciBfNDNjPV80M2EuTm9kZUxpc3Q7dmFyIG5scD1fNDNjLnByb3RvdHlwZTtu",
				"bHAuY29ubmVjdD1fNDNjLl9hZGFwdEFzRm9yRWFjaChmdW5jdGlvbigpe3JldHVybiBkb2pvLmNvbm5l",
				"Y3QuYXBwbHkodGhpcyxhcmd1bWVudHMpO30pO25scC5jb29yZHM9XzQzYy5fYWRhcHRBc01hcChkb2pv",
				"LmNvb3Jkcyk7XzQzYy5ldmVudHM9WyJibHVyIiwiZm9jdXMiLCJjaGFuZ2UiLCJjbGljayIsImVycm9y",
				"Iiwia2V5ZG93biIsImtleXByZXNzIiwia2V5dXAiLCJsb2FkIiwibW91c2Vkb3duIiwibW91c2VlbnRl",
				"ciIsIm1vdXNlbGVhdmUiLCJtb3VzZW1vdmUiLCJtb3VzZW91dCIsIm1vdXNlb3ZlciIsIm1vdXNldXAi",
				"LCJzdWJtaXQiXTtfNDNiLmZvckVhY2goXzQzYy5ldmVudHMsZnVuY3Rpb24oZXZ0KXt2YXIgXzQzZD0i",
				"b24iK2V2dDtubHBbXzQzZF09ZnVuY3Rpb24oYSxiKXtyZXR1cm4gdGhpcy5jb25uZWN0KF80M2QsYSxi",
				"KTt9O30pO2Rvam8uTm9kZUxpc3Q9XzQzYztyZXR1cm4gZG9qby5Ob2RlTGlzdDt9KTt9LCJkb2pvL19i",
				"YXNlL0NvbG9yIjpmdW5jdGlvbigpe2RlZmluZShbIi4va2VybmVsIiwiLi9sYW5nIiwiLi9hcnJheSIs",
				"Ii4vY29uZmlnIl0sZnVuY3Rpb24oZG9qbyxsYW5nLF80M2UsXzQzZil7dmFyIF80NDA9ZG9qby5Db2xv",
				"cj1mdW5jdGlvbihfNDQxKXtpZihfNDQxKXt0aGlzLnNldENvbG9yKF80NDEpO319O180NDAubmFtZWQ9",
				"eyJibGFjayI6WzAsMCwwXSwic2lsdmVyIjpbMTkyLDE5MiwxOTJdLCJncmF5IjpbMTI4LDEyOCwxMjhd",
				"LCJ3aGl0ZSI6WzI1NSwyNTUsMjU1XSwibWFyb29uIjpbMTI4LDAsMF0sInJlZCI6WzI1NSwwLDBdLCJw",
				"dXJwbGUiOlsxMjgsMCwxMjhdLCJmdWNoc2lhIjpbMjU1LDAsMjU1XSwiZ3JlZW4iOlswLDEyOCwwXSwi",
				"bGltZSI6WzAsMjU1LDBdLCJvbGl2ZSI6WzEyOCwxMjgsMF0sInllbGxvdyI6WzI1NSwyNTUsMF0sIm5h",
				"dnkiOlswLDAsMTI4XSwiYmx1ZSI6WzAsMCwyNTVdLCJ0ZWFsIjpbMCwxMjgsMTI4XSwiYXF1YSI6WzAs",
				"MjU1LDI1NV0sInRyYW5zcGFyZW50IjpfNDNmLnRyYW5zcGFyZW50Q29sb3J8fFswLDAsMCwwXX07bGFu",
				"Zy5leHRlbmQoXzQ0MCx7cjoyNTUsZzoyNTUsYjoyNTUsYToxLF9zZXQ6ZnVuY3Rpb24ocixnLGIsYSl7",
				"dmFyIHQ9dGhpczt0LnI9cjt0Lmc9Zzt0LmI9Yjt0LmE9YTt9LHNldENvbG9yOmZ1bmN0aW9uKF80NDIp",
				"e2lmKGxhbmcuaXNTdHJpbmcoXzQ0Mikpe180NDAuZnJvbVN0cmluZyhfNDQyLHRoaXMpO31lbHNle2lm",
				"KGxhbmcuaXNBcnJheShfNDQyKSl7XzQ0MC5mcm9tQXJyYXkoXzQ0Mix0aGlzKTt9ZWxzZXt0aGlzLl9z",
				"ZXQoXzQ0Mi5yLF80NDIuZyxfNDQyLmIsXzQ0Mi5hKTtpZighKF80NDIgaW5zdGFuY2VvZiBfNDQwKSl7",
				"dGhpcy5zYW5pdGl6ZSgpO319fXJldHVybiB0aGlzO30sc2FuaXRpemU6ZnVuY3Rpb24oKXtyZXR1cm4g",
				"dGhpczt9LHRvUmdiOmZ1bmN0aW9uKCl7dmFyIHQ9dGhpcztyZXR1cm4gW3Qucix0LmcsdC5iXTt9LHRv",
				"UmdiYTpmdW5jdGlvbigpe3ZhciB0PXRoaXM7cmV0dXJuIFt0LnIsdC5nLHQuYix0LmFdO30sdG9IZXg6",
				"ZnVuY3Rpb24oKXt2YXIgYXJyPV80M2UubWFwKFsiciIsImciLCJiIl0sZnVuY3Rpb24oeCl7dmFyIHM9",
				"dGhpc1t4XS50b1N0cmluZygxNik7cmV0dXJuIHMubGVuZ3RoPDI/IjAiK3M6czt9LHRoaXMpO3JldHVy",
				"biAiIyIrYXJyLmpvaW4oIiIpO30sdG9Dc3M6ZnVuY3Rpb24oXzQ0Myl7dmFyIHQ9dGhpcyxyZ2I9dC5y",
				"KyIsICIrdC5nKyIsICIrdC5iO3JldHVybiAoXzQ0Mz8icmdiYSgiK3JnYisiLCAiK3QuYToicmdiKCIr",
				"cmdiKSsiKSI7fSx0b1N0cmluZzpmdW5jdGlvbigpe3JldHVybiB0aGlzLnRvQ3NzKHRydWUpO319KTtf",
				"NDQwLmJsZW5kQ29sb3JzPWRvam8uYmxlbmRDb2xvcnM9ZnVuY3Rpb24oXzQ0NCxlbmQsXzQ0NSxvYmop",
				"e3ZhciB0PW9ianx8bmV3IF80NDAoKTtfNDNlLmZvckVhY2goWyJyIiwiZyIsImIiLCJhIl0sZnVuY3Rp",
				"b24oeCl7dFt4XT1fNDQ0W3hdKyhlbmRbeF0tXzQ0NFt4XSkqXzQ0NTtpZih4IT0iYSIpe3RbeF09TWF0",
				"aC5yb3VuZCh0W3hdKTt9fSk7cmV0dXJuIHQuc2FuaXRpemUoKTt9O180NDAuZnJvbVJnYj1kb2pvLmNv",
				"bG9yRnJvbVJnYj1mdW5jdGlvbihfNDQ2LG9iail7dmFyIG09XzQ0Ni50b0xvd2VyQ2FzZSgpLm1hdGNo",
				"KC9ecmdiYT9cKChbXHNcLiwwLTldKylcKS8pO3JldHVybiBtJiZfNDQwLmZyb21BcnJheShtWzFdLnNw",
				"bGl0KC9ccyosXHMqLyksb2JqKTt9O180NDAuZnJvbUhleD1kb2pvLmNvbG9yRnJvbUhleD1mdW5jdGlv",
				"bihfNDQ3LG9iail7dmFyIHQ9b2JqfHxuZXcgXzQ0MCgpLGJpdHM9KF80NDcubGVuZ3RoPT00KT80Ojgs",
				"bWFzaz0oMTw8Yml0cyktMTtfNDQ3PU51bWJlcigiMHgiK180NDcuc3Vic3RyKDEpKTtpZihpc05hTihf",
				"NDQ3KSl7cmV0dXJuIG51bGw7fV80M2UuZm9yRWFjaChbImIiLCJnIiwiciJdLGZ1bmN0aW9uKHgpe3Zh",
				"ciBjPV80NDcmbWFzaztfNDQ3Pj49Yml0czt0W3hdPWJpdHM9PTQ/MTcqYzpjO30pO3QuYT0xO3JldHVy",
				"biB0O307XzQ0MC5mcm9tQXJyYXk9ZG9qby5jb2xvckZyb21BcnJheT1mdW5jdGlvbihhLG9iail7dmFy",
				"IHQ9b2JqfHxuZXcgXzQ0MCgpO3QuX3NldChOdW1iZXIoYVswXSksTnVtYmVyKGFbMV0pLE51bWJlcihh",
				"WzJdKSxOdW1iZXIoYVszXSkpO2lmKGlzTmFOKHQuYSkpe3QuYT0xO31yZXR1cm4gdC5zYW5pdGl6ZSgp",
				"O307XzQ0MC5mcm9tU3RyaW5nPWRvam8uY29sb3JGcm9tU3RyaW5nPWZ1bmN0aW9uKHN0cixvYmope3Zh",
				"ciBhPV80NDAubmFtZWRbc3RyXTtyZXR1cm4gYSYmXzQ0MC5mcm9tQXJyYXkoYSxvYmopfHxfNDQwLmZy",
				"b21SZ2Ioc3RyLG9iail8fF80NDAuZnJvbUhleChzdHIsb2JqKTt9O3JldHVybiBfNDQwO30pO30sImRv",
				"am8vc2VsZWN0b3IvX2xvYWRlciI6ZnVuY3Rpb24oKXtkZWZpbmUoWyIuLi9oYXMiLCJyZXF1aXJlIl0s",
				"ZnVuY3Rpb24oaGFzLF80NDgpeyJ1c2Ugc3RyaWN0Ijt2YXIgXzQ0OT1kb2N1bWVudC5jcmVhdGVFbGVt",
				"ZW50KCJkaXYiKTtoYXMuYWRkKCJkb20tcXNhMi4xIiwhIV80NDkucXVlcnlTZWxlY3RvckFsbCk7aGFz",
				"LmFkZCgiZG9tLXFzYTMiLGZ1bmN0aW9uKCl7dHJ5e180NDkuaW5uZXJIVE1MPSI8cCBjbGFzcz0nVEVT",
				"VCc+PC9wPiI7cmV0dXJuIF80NDkucXVlcnlTZWxlY3RvckFsbCgiLlRFU1Q6ZW1wdHkiKS5sZW5ndGg9",
				"PTE7fWNhdGNoKGUpe319KTt2YXIgXzQ0YTt2YXIgYWNtZT0iLi9hY21lIixsaXRlPSIuL2xpdGUiO3Jl",
				"dHVybiB7bG9hZDpmdW5jdGlvbihpZCxfNDRiLF80NGMsXzQ0ZCl7dmFyIHJlcT1fNDQ4O2lkPWlkPT0i",
				"ZGVmYXVsdCI/aGFzKCJjb25maWctc2VsZWN0b3JFbmdpbmUiKXx8ImNzczMiOmlkO2lkPWlkPT0iY3Nz",
				"MiJ8fGlkPT0ibGl0ZSI/bGl0ZTppZD09ImNzczIuMSI/aGFzKCJkb20tcXNhMi4xIik/bGl0ZTphY21l",
				"OmlkPT0iY3NzMyI/aGFzKCJkb20tcXNhMyIpP2xpdGU6YWNtZTppZD09ImFjbWUiP2FjbWU6KHJlcT1f",
				"NDRiKSYmaWQ7aWYoaWQuY2hhckF0KGlkLmxlbmd0aC0xKT09Ij8iKXtpZD1pZC5zdWJzdHJpbmcoMCxp",
				"ZC5sZW5ndGgtMSk7dmFyIF80NGU9dHJ1ZTt9aWYoXzQ0ZSYmKGhhcygiZG9tLWNvbXBsaWFudC1xc2Ei",
				"KXx8XzQ0YSkpe3JldHVybiBfNDRjKF80NGEpO31yZXEoW2lkXSxmdW5jdGlvbihfNDRmKXtpZihpZCE9",
				"Ii4vbGl0ZSIpe180NGE9XzQ0Zjt9XzQ0YyhfNDRmKTt9KTt9fTt9KTt9LCJkb2pvL29uIjpmdW5jdGlv",
				"bigpe2RlZmluZShbIi4vaGFzIWRvbS1hZGRldmVudGxpc3RlbmVyPzouL2FzcGVjdCIsIi4vX2Jhc2Uv",
				"a2VybmVsIiwiLi9oYXMiXSxmdW5jdGlvbihfNDUwLGRvam8saGFzKXsidXNlIHN0cmljdCI7aWYoMSl7",
				"dmFyIF80NTE9d2luZG93LlNjcmlwdEVuZ2luZU1ham9yVmVyc2lvbjtoYXMuYWRkKCJqc2NyaXB0Iixf",
				"NDUxJiYoXzQ1MSgpK1NjcmlwdEVuZ2luZU1pbm9yVmVyc2lvbigpLzEwKSk7aGFzLmFkZCgiZXZlbnQt",
				"b3JpZW50YXRpb25jaGFuZ2UiLGhhcygidG91Y2giKSYmIWhhcygiYW5kcm9pZCIpKTt9dmFyIG9uPWZ1",
				"bmN0aW9uKF80NTIsdHlwZSxfNDUzLF80NTQpe2lmKF80NTIub24pe3JldHVybiBfNDUyLm9uKHR5cGUs",
				"XzQ1Myk7fXJldHVybiBvbi5wYXJzZShfNDUyLHR5cGUsXzQ1MyxfNDU1LF80NTQsdGhpcyk7fTtvbi5w",
				"YXVzYWJsZT1mdW5jdGlvbihfNDU2LHR5cGUsXzQ1NyxfNDU4KXt2YXIgXzQ1OTt2YXIgXzQ1YT1vbihf",
				"NDU2LHR5cGUsZnVuY3Rpb24oKXtpZighXzQ1OSl7cmV0dXJuIF80NTcuYXBwbHkodGhpcyxhcmd1bWVu",
				"dHMpO319LF80NTgpO180NWEucGF1c2U9ZnVuY3Rpb24oKXtfNDU5PXRydWU7fTtfNDVhLnJlc3VtZT1m",
				"dW5jdGlvbigpe180NTk9ZmFsc2U7fTtyZXR1cm4gXzQ1YTt9O29uLm9uY2U9ZnVuY3Rpb24oXzQ1Yix0",
				"eXBlLF80NWMsXzQ1ZCl7dmFyIF80NWU9b24oXzQ1Yix0eXBlLGZ1bmN0aW9uKCl7XzQ1ZS5yZW1vdmUo",
				"KTtyZXR1cm4gXzQ1Yy5hcHBseSh0aGlzLGFyZ3VtZW50cyk7fSk7cmV0dXJuIF80NWU7fTtvbi5wYXJz",
				"ZT1mdW5jdGlvbihfNDVmLHR5cGUsXzQ2MCxfNDYxLF80NjIsXzQ2Myl7aWYodHlwZS5jYWxsKXtyZXR1",
				"cm4gdHlwZS5jYWxsKF80NjMsXzQ1ZixfNDYwKTt9aWYodHlwZS5pbmRleE9mKCIsIik+LTEpe3ZhciBf",
				"NDY0PXR5cGUuc3BsaXQoL1xzKixccyovKTt2YXIgXzQ2NT1bXTt2YXIgaT0wO3ZhciBfNDY2O3doaWxl",
				"KF80NjY9XzQ2NFtpKytdKXtfNDY1LnB1c2goXzQ2MShfNDVmLF80NjYsXzQ2MCxfNDYyLF80NjMpKTt9",
				"XzQ2NS5yZW1vdmU9ZnVuY3Rpb24oKXtmb3IodmFyIGk9MDtpPF80NjUubGVuZ3RoO2krKyl7XzQ2NVtp",
				"XS5yZW1vdmUoKTt9fTtyZXR1cm4gXzQ2NTt9cmV0dXJuIF80NjEoXzQ1Zix0eXBlLF80NjAsXzQ2Mixf",
				"NDYzKTt9O3ZhciBfNDY3PS9edG91Y2gvO2Z1bmN0aW9uIF80NTUoXzQ2OCx0eXBlLF80NjksXzQ2YSxf",
				"NDZiKXt2YXIgXzQ2Yz10eXBlLm1hdGNoKC8oLiopOiguKikvKTtpZihfNDZjKXt0eXBlPV80NmNbMl07",
				"XzQ2Yz1fNDZjWzFdO3JldHVybiBvbi5zZWxlY3RvcihfNDZjLHR5cGUpLmNhbGwoXzQ2YixfNDY4LF80",
				"NjkpO31pZihoYXMoInRvdWNoIikpe2lmKF80NjcudGVzdCh0eXBlKSl7XzQ2OT1fNDZkKF80NjkpO31p",
				"ZighaGFzKCJldmVudC1vcmllbnRhdGlvbmNoYW5nZSIpJiYodHlwZT09Im9yaWVudGF0aW9uY2hhbmdl",
				"Iikpe3R5cGU9InJlc2l6ZSI7XzQ2OD13aW5kb3c7XzQ2OT1fNDZkKF80NjkpO319aWYoXzQ2OC5hZGRF",
				"dmVudExpc3RlbmVyKXt2YXIgXzQ2ZT10eXBlIGluIF80NmY7XzQ2OC5hZGRFdmVudExpc3RlbmVyKF80",
				"NmU/XzQ2Zlt0eXBlXTp0eXBlLF80NjksXzQ2ZSk7cmV0dXJuIHtyZW1vdmU6ZnVuY3Rpb24oKXtfNDY4",
				"LnJlbW92ZUV2ZW50TGlzdGVuZXIodHlwZSxfNDY5LF80NmUpO319O310eXBlPSJvbiIrdHlwZTtpZihf",
				"NDcwJiZfNDY4LmF0dGFjaEV2ZW50KXtyZXR1cm4gXzQ3MChfNDY4LHR5cGUsXzQ2OSk7fXRocm93IG5l",
				"dyBFcnJvcigiVGFyZ2V0IG11c3QgYmUgYW4gZXZlbnQgZW1pdHRlciIpO307b24uc2VsZWN0b3I9ZnVu",
				"Y3Rpb24oXzQ3MSxfNDcyLF80NzMpe3JldHVybiBmdW5jdGlvbihfNDc0LF80NzUpe3ZhciBfNDc2PXRo",
				"aXM7dmFyIF80Nzc9XzQ3Mi5idWJibGU7aWYoXzQ3Nyl7XzQ3Mj1fNDc3O31lbHNle2lmKF80NzMhPT1m",
				"YWxzZSl7XzQ3Mz10cnVlO319cmV0dXJuIG9uKF80NzQsXzQ3MixmdW5jdGlvbihfNDc4KXt2YXIgXzQ3",
				"OT1fNDc4LnRhcmdldDtfNDc2PV80NzYmJl80NzYubWF0Y2hlcz9fNDc2OmRvam8ucXVlcnk7d2hpbGUo",
				"IV80NzYubWF0Y2hlcyhfNDc5LF80NzEsXzQ3NCkpe2lmKF80Nzk9PV80NzR8fCFfNDczfHwhKF80Nzk9",
				"XzQ3OS5wYXJlbnROb2RlKSl7cmV0dXJuO319cmV0dXJuIF80NzUuY2FsbChfNDc5LF80NzgpO30pO307",
				"fTtmdW5jdGlvbiBfNDdhKCl7dGhpcy5jYW5jZWxhYmxlPWZhbHNlO307ZnVuY3Rpb24gXzQ3Yigpe3Ro",
				"aXMuYnViYmxlcz1mYWxzZTt9O3ZhciBfNDdjPVtdLnNsaWNlLF80N2Q9b24uZW1pdD1mdW5jdGlvbihf",
				"NDdlLHR5cGUsXzQ3Zil7dmFyIGFyZ3M9XzQ3Yy5jYWxsKGFyZ3VtZW50cywyKTt2YXIgXzQ4MD0ib24i",
				"K3R5cGU7aWYoInBhcmVudE5vZGUiIGluIF80N2Upe3ZhciBfNDgxPWFyZ3NbMF09e307Zm9yKHZhciBp",
				"IGluIF80N2Ype180ODFbaV09XzQ3ZltpXTt9XzQ4MS5wcmV2ZW50RGVmYXVsdD1fNDdhO180ODEuc3Rv",
				"cFByb3BhZ2F0aW9uPV80N2I7XzQ4MS50YXJnZXQ9XzQ3ZTtfNDgxLnR5cGU9dHlwZTtfNDdmPV80ODE7",
				"fWRve180N2VbXzQ4MF0mJl80N2VbXzQ4MF0uYXBwbHkoXzQ3ZSxhcmdzKTt9d2hpbGUoXzQ3ZiYmXzQ3",
				"Zi5idWJibGVzJiYoXzQ3ZT1fNDdlLnBhcmVudE5vZGUpKTtyZXR1cm4gXzQ3ZiYmXzQ3Zi5jYW5jZWxh",
				"YmxlJiZfNDdmO307dmFyIF80NmY9e307aWYoaGFzKCJkb20tYWRkZXZlbnRsaXN0ZW5lciIpKXtfNDZm",
				"PXtmb2N1c2luOiJmb2N1cyIsZm9jdXNvdXQ6ImJsdXIifTtpZihoYXMoIm9wZXJhIikpe180NmYua2V5",
				"ZG93bj0ia2V5cHJlc3MiO31vbi5lbWl0PWZ1bmN0aW9uKF80ODIsdHlwZSxfNDgzKXtpZihfNDgyLmRp",
				"c3BhdGNoRXZlbnQmJmRvY3VtZW50LmNyZWF0ZUV2ZW50KXt2YXIgXzQ4ND1kb2N1bWVudC5jcmVhdGVF",
				"dmVudCgiSFRNTEV2ZW50cyIpO180ODQuaW5pdEV2ZW50KHR5cGUsISFfNDgzLmJ1YmJsZXMsISFfNDgz",
				"LmNhbmNlbGFibGUpO2Zvcih2YXIgaSBpbiBfNDgzKXt2YXIgXzQ4NT1fNDgzW2ldO2lmKCEoaSBpbiBf",
				"NDg0KSl7XzQ4NFtpXT1fNDgzW2ldO319cmV0dXJuIF80ODIuZGlzcGF0Y2hFdmVudChfNDg0KSYmXzQ4",
				"NDt9cmV0dXJuIF80N2QuYXBwbHkob24sYXJndW1lbnRzKTt9O31lbHNle29uLl9maXhFdmVudD1mdW5j",
				"dGlvbihldnQsXzQ4Nil7aWYoIWV2dCl7dmFyIHc9XzQ4NiYmKF80ODYub3duZXJEb2N1bWVudHx8XzQ4",
				"Ni5kb2N1bWVudHx8XzQ4NikucGFyZW50V2luZG93fHx3aW5kb3c7ZXZ0PXcuZXZlbnQ7fWlmKCFldnQp",
				"e3JldHVybiAoZXZ0KTt9aWYoIWV2dC50YXJnZXQpe2V2dC50YXJnZXQ9ZXZ0LnNyY0VsZW1lbnQ7ZXZ0",
				"LmN1cnJlbnRUYXJnZXQ9KF80ODZ8fGV2dC5zcmNFbGVtZW50KTtpZihldnQudHlwZT09Im1vdXNlb3Zl",
				"ciIpe2V2dC5yZWxhdGVkVGFyZ2V0PWV2dC5mcm9tRWxlbWVudDt9aWYoZXZ0LnR5cGU9PSJtb3VzZW91",
				"dCIpe2V2dC5yZWxhdGVkVGFyZ2V0PWV2dC50b0VsZW1lbnQ7fWlmKCFldnQuc3RvcFByb3BhZ2F0aW9u",
				"KXtldnQuc3RvcFByb3BhZ2F0aW9uPV80ODc7ZXZ0LnByZXZlbnREZWZhdWx0PV80ODg7fXN3aXRjaChl",
				"dnQudHlwZSl7Y2FzZSAia2V5cHJlc3MiOnZhciBjPSgiY2hhckNvZGUiIGluIGV2dD9ldnQuY2hhckNv",
				"ZGU6ZXZ0LmtleUNvZGUpO2lmKGM9PTEwKXtjPTA7ZXZ0LmtleUNvZGU9MTM7fWVsc2V7aWYoYz09MTN8",
				"fGM9PTI3KXtjPTA7fWVsc2V7aWYoYz09Myl7Yz05OTt9fX1ldnQuY2hhckNvZGU9YztfNDg5KGV2dCk7",
				"YnJlYWs7fX1yZXR1cm4gZXZ0O307dmFyIF80OGE9ZnVuY3Rpb24oXzQ4Yil7dGhpcy5oYW5kbGU9XzQ4",
				"Yjt9O180OGEucHJvdG90eXBlLnJlbW92ZT1mdW5jdGlvbigpe2RlbGV0ZSBfZG9qb0lFTGlzdGVuZXJz",
				"X1t0aGlzLmhhbmRsZV07fTt2YXIgXzQ4Yz1mdW5jdGlvbihfNDhkKXtyZXR1cm4gZnVuY3Rpb24oZXZ0",
				"KXtldnQ9b24uX2ZpeEV2ZW50KGV2dCx0aGlzKTtyZXR1cm4gXzQ4ZC5jYWxsKHRoaXMsZXZ0KTt9O307",
				"dmFyIF80NzA9ZnVuY3Rpb24oXzQ4ZSx0eXBlLF80OGYpe180OGY9XzQ4YyhfNDhmKTtpZigoKF80OGUu",
				"b3duZXJEb2N1bWVudD9fNDhlLm93bmVyRG9jdW1lbnQucGFyZW50V2luZG93Ol80OGUucGFyZW50V2lu",
				"ZG93fHxfNDhlLndpbmRvd3x8d2luZG93KSE9dG9wfHxoYXMoImpzY3JpcHQiKTw1LjgpJiYhaGFzKCJj",
				"b25maWctX2FsbG93X2xlYWtzIikpe2lmKHR5cGVvZiBfZG9qb0lFTGlzdGVuZXJzXz09InVuZGVmaW5l",
				"ZCIpe19kb2pvSUVMaXN0ZW5lcnNfPVtdO312YXIgXzQ5MD1fNDhlW3R5cGVdO2lmKCFfNDkwfHwhXzQ5",
				"MC5saXN0ZW5lcnMpe3ZhciBfNDkxPV80OTA7XzQ4ZVt0eXBlXT1fNDkwPUZ1bmN0aW9uKCJldmVudCIs",
				"InZhciBjYWxsZWUgPSBhcmd1bWVudHMuY2FsbGVlOyBmb3IodmFyIGkgPSAwOyBpPGNhbGxlZS5saXN0",
				"ZW5lcnMubGVuZ3RoOyBpKyspe3ZhciBsaXN0ZW5lciA9IF9kb2pvSUVMaXN0ZW5lcnNfW2NhbGxlZS5s",
				"aXN0ZW5lcnNbaV1dOyBpZihsaXN0ZW5lcil7bGlzdGVuZXIuY2FsbCh0aGlzLGV2ZW50KTt9fSIpO180",
				"OTAubGlzdGVuZXJzPVtdO180OTAuZ2xvYmFsPXRoaXM7aWYoXzQ5MSl7XzQ5MC5saXN0ZW5lcnMucHVz",
				"aChfZG9qb0lFTGlzdGVuZXJzXy5wdXNoKF80OTEpLTEpO319dmFyIF80OTI7XzQ5MC5saXN0ZW5lcnMu",
				"cHVzaChfNDkyPShfNDkwLmdsb2JhbC5fZG9qb0lFTGlzdGVuZXJzXy5wdXNoKF80OGYpLTEpKTtyZXR1",
				"cm4gbmV3IF80OGEoXzQ5Mik7fXJldHVybiBfNDUwLmFmdGVyKF80OGUsdHlwZSxfNDhmLHRydWUpO307",
				"dmFyIF80ODk9ZnVuY3Rpb24oZXZ0KXtldnQua2V5Q2hhcj1ldnQuY2hhckNvZGU/U3RyaW5nLmZyb21D",
				"aGFyQ29kZShldnQuY2hhckNvZGUpOiIiO2V2dC5jaGFyT3JDb2RlPWV2dC5rZXlDaGFyfHxldnQua2V5",
				"Q29kZTt9O3ZhciBfNDg3PWZ1bmN0aW9uKCl7dGhpcy5jYW5jZWxCdWJibGU9dHJ1ZTt9O3ZhciBfNDg4",
				"PW9uLl9wcmV2ZW50RGVmYXVsdD1mdW5jdGlvbigpe3RoaXMuYnViYmxlZEtleUNvZGU9dGhpcy5rZXlD",
				"b2RlO2lmKHRoaXMuY3RybEtleSl7dHJ5e3RoaXMua2V5Q29kZT0wO31jYXRjaChlKXt9fXRoaXMucmV0",
				"dXJuVmFsdWU9ZmFsc2U7fTt9aWYoaGFzKCJ0b3VjaCIpKXt2YXIgXzQ5Mz1mdW5jdGlvbigpe307dmFy",
				"IF80OTQ9d2luZG93Lm9yaWVudGF0aW9uO3ZhciBfNDZkPWZ1bmN0aW9uKF80OTUpe3JldHVybiBmdW5j",
				"dGlvbihfNDk2KXt2YXIgXzQ5Nz1fNDk2LmNvcnJlY3RlZDtpZighXzQ5Nyl7dmFyIHR5cGU9XzQ5Ni50",
				"eXBlO3RyeXtkZWxldGUgXzQ5Ni50eXBlO31jYXRjaChlKXt9aWYoXzQ5Ni50eXBlKXtfNDkzLnByb3Rv",
				"dHlwZT1fNDk2O3ZhciBfNDk3PW5ldyBfNDkzO180OTcucHJldmVudERlZmF1bHQ9ZnVuY3Rpb24oKXtf",
				"NDk2LnByZXZlbnREZWZhdWx0KCk7fTtfNDk3LnN0b3BQcm9wYWdhdGlvbj1mdW5jdGlvbigpe180OTYu",
				"c3RvcFByb3BhZ2F0aW9uKCk7fTt9ZWxzZXtfNDk3PV80OTY7XzQ5Ny50eXBlPXR5cGU7fV80OTYuY29y",
				"cmVjdGVkPV80OTc7aWYodHlwZT09InJlc2l6ZSIpe2lmKF80OTQ9PXdpbmRvdy5vcmllbnRhdGlvbil7",
				"cmV0dXJuIG51bGw7fV80OTQ9d2luZG93Lm9yaWVudGF0aW9uO180OTcudHlwZT0ib3JpZW50YXRpb25j",
				"aGFuZ2UiO3JldHVybiBfNDk1LmNhbGwodGhpcyxfNDk3KTt9aWYoISgicm90YXRpb24iIGluIF80OTcp",
				"KXtfNDk3LnJvdGF0aW9uPTA7XzQ5Ny5zY2FsZT0xO312YXIgXzQ5OD1fNDk3LmNoYW5nZWRUb3VjaGVz",
				"WzBdO2Zvcih2YXIgaSBpbiBfNDk4KXtkZWxldGUgXzQ5N1tpXTtfNDk3W2ldPV80OThbaV07fX1yZXR1",
				"cm4gXzQ5NS5jYWxsKHRoaXMsXzQ5Nyk7fTt9O31yZXR1cm4gb247fSk7fSwiZG9qby9fYmFzZS9zbmlm",
				"ZiI6ZnVuY3Rpb24oKXtkZWZpbmUoWyIuL2tlcm5lbCIsIi4uL2hhcyJdLGZ1bmN0aW9uKGRvam8saGFz",
				"KXtpZighMSl7cmV0dXJuIGhhczt9ZG9qby5pc0Jyb3dzZXI9dHJ1ZSxkb2pvLl9uYW1lPSJicm93c2Vy",
				"Ijt2YXIgXzQ5OT1oYXMuYWRkLG49bmF2aWdhdG9yLGR1YT1uLnVzZXJBZ2VudCxkYXY9bi5hcHBWZXJz",
				"aW9uLHR2PXBhcnNlRmxvYXQoZGF2KSxfNDlhLF80OWIsXzQ5YyxfNDlkLF80OWUsXzQ5ZixfNGEwLF80",
				"YTEsXzRhMixpc0lFLGlzRkYsXzRhMyxfNGE0LF80YTUsXzRhNjtpZihkdWEuaW5kZXhPZigiQWRvYmVB",
				"SVIiKT49MCl7XzQ5Yj0xO31fNDljPShkYXYuaW5kZXhPZigiS29ucXVlcm9yIik+PTApP3R2OjA7XzQ5",
				"ZD1wYXJzZUZsb2F0KGR1YS5zcGxpdCgiV2ViS2l0LyIpWzFdKXx8dW5kZWZpbmVkO180OWU9cGFyc2VG",
				"bG9hdChkdWEuc3BsaXQoIkNocm9tZS8iKVsxXSl8fHVuZGVmaW5lZDtfNDlmPWRhdi5pbmRleE9mKCJN",
				"YWNpbnRvc2giKT49MDtfNGE0PS9pUGhvbmV8aVBvZHxpUGFkLy50ZXN0KGR1YSk7XzRhNT1wYXJzZUZs",
				"b2F0KGR1YS5zcGxpdCgiQW5kcm9pZCAiKVsxXSl8fHVuZGVmaW5lZDtfNGE2PXR5cGVvZiBvcGVyYSE9",
				"InVuZGVmaW5lZCImJm9wZXJhLndpaXJlbW90ZTt2YXIgXzRhNz1NYXRoLm1heChkYXYuaW5kZXhPZigi",
				"V2ViS2l0IiksZGF2LmluZGV4T2YoIlNhZmFyaSIpLDApO2lmKF80YTcmJiFfNDllKXtfNGEwPXBhcnNl",
				"RmxvYXQoZGF2LnNwbGl0KCJWZXJzaW9uLyIpWzFdKTtpZighXzRhMHx8cGFyc2VGbG9hdChkYXYuc3Vi",
				"c3RyKF80YTcrNykpPD00MTkuMyl7XzRhMD0yO319aWYoIWhhcygiZG9qby13ZWJraXQiKSl7aWYoZHVh",
				"LmluZGV4T2YoIk9wZXJhIik+PTApe180OWE9dHY7aWYoXzQ5YT49OS44KXtfNDlhPXBhcnNlRmxvYXQo",
				"ZHVhLnNwbGl0KCJWZXJzaW9uLyIpWzFdKXx8dHY7fX1pZihkdWEuaW5kZXhPZigiR2Vja28iKT49MCYm",
				"IV80OWMmJiFfNDlkKXtfNGExPV80YTI9dHY7fWlmKF80YTIpe2lzRkY9cGFyc2VGbG9hdChkdWEuc3Bs",
				"aXQoIkZpcmVmb3gvIilbMV18fGR1YS5zcGxpdCgiTWluZWZpZWxkLyIpWzFdKXx8dW5kZWZpbmVkO31p",
				"Zihkb2N1bWVudC5hbGwmJiFfNDlhKXtpc0lFPXBhcnNlRmxvYXQoZGF2LnNwbGl0KCJNU0lFICIpWzFd",
				"KXx8dW5kZWZpbmVkO3ZhciBtb2RlPWRvY3VtZW50LmRvY3VtZW50TW9kZTtpZihtb2RlJiZtb2RlIT01",
				"JiZNYXRoLmZsb29yKGlzSUUpIT1tb2RlKXtpc0lFPW1vZGU7fX19XzRhMz1kb2N1bWVudC5jb21wYXRN",
				"b2RlPT0iQmFja0NvbXBhdCI7XzQ5OSgib3BlcmEiLGRvam8uaXNPcGVyYT1fNDlhKTtfNDk5KCJhaXIi",
				"LGRvam8uaXNBSVI9XzQ5Yik7XzQ5OSgia2h0bWwiLGRvam8uaXNLaHRtbD1fNDljKTtfNDk5KCJ3ZWJr",
				"aXQiLGRvam8uaXNXZWJLaXQ9XzQ5ZCk7XzQ5OSgiY2hyb21lIixkb2pvLmlzQ2hyb21lPV80OWUpO180",
				"OTkoIm1hYyIsZG9qby5pc01hYz1fNDlmKTtfNDk5KCJzYWZhcmkiLGRvam8uaXNTYWZhcmk9XzRhMCk7",
				"XzQ5OSgibW96aWxsYSIsZG9qby5pc01vemlsbGE9ZG9qby5pc01vej1fNGExKTtfNDk5KCJpZSIsZG9q",
				"by5pc0lFPWlzSUUpO180OTkoImZmIixkb2pvLmlzRkY9aXNGRik7XzQ5OSgicXVpcmtzIixkb2pvLmlz",
				"UXVpcmtzPV80YTMpO180OTkoImlvcyIsZG9qby5pc0lvcz1fNGE0KTtfNDk5KCJhbmRyb2lkIixkb2pv",
				"LmlzQW5kcm9pZD1fNGE1KTtkb2pvLmxvY2FsZT1kb2pvLmxvY2FsZXx8KGlzSUU/bi51c2VyTGFuZ3Vh",
				"Z2U6bi5sYW5ndWFnZSkudG9Mb3dlckNhc2UoKTtyZXR1cm4gaGFzO30pO30sImRvam8vX2Jhc2UvYXJy",
				"YXkiOmZ1bmN0aW9uKCl7ZGVmaW5lKCJkb2pvL19iYXNlL2FycmF5IixbIi4va2VybmVsIiwiLi4vaGFz",
				"IiwiLi9sYW5nIl0sZnVuY3Rpb24oZG9qbyxoYXMsbGFuZyl7dmFyIF80YTg9e30sdSxfNGE5O2Z1bmN0",
				"aW9uIF80YWEoKXtfNGE4PXt9O307ZnVuY3Rpb24gXzRhYihmbil7cmV0dXJuIF80YThbZm5dPW5ldyBG",
				"dW5jdGlvbigiaXRlbSIsImluZGV4IiwiYXJyYXkiLGZuKTt9O2Z1bmN0aW9uIF80YWMoc29tZSl7dmFy",
				"IF80YWQ9IXNvbWU7cmV0dXJuIGZ1bmN0aW9uKGEsZm4sbyl7dmFyIGk9MCxsPWEmJmEubGVuZ3RofHww",
				"LF80YWU7aWYobCYmdHlwZW9mIGE9PSJzdHJpbmciKXthPWEuc3BsaXQoIiIpO31pZih0eXBlb2YgZm49",
				"PSJzdHJpbmciKXtmbj1fNGE4W2ZuXXx8XzRhYihmbik7fWlmKG8pe2Zvcig7aTxsOysraSl7XzRhZT0h",
				"Zm4uY2FsbChvLGFbaV0saSxhKTtpZihzb21lXl80YWUpe3JldHVybiAhXzRhZTt9fX1lbHNle2Zvcig7",
				"aTxsOysraSl7XzRhZT0hZm4oYVtpXSxpLGEpO2lmKHNvbWVeXzRhZSl7cmV0dXJuICFfNGFlO319fXJl",
				"dHVybiBfNGFkO307fTtmdW5jdGlvbiBfNGFmKHVwKXt2YXIgXzRiMD0xLF80YjE9MCxfNGIyPTA7aWYo",
				"IXVwKXtfNGIwPV80YjE9XzRiMj0tMTt9cmV0dXJuIGZ1bmN0aW9uKGEseCxmcm9tLGxhc3Qpe2lmKGxh",
				"c3QmJl80YjA+MCl7cmV0dXJuIF80YTkubGFzdEluZGV4T2YoYSx4LGZyb20pO312YXIgbD1hJiZhLmxl",
				"bmd0aHx8MCxlbmQ9dXA/bCtfNGIyOl80YjEsaTtpZihmcm9tPT09dSl7aT11cD9fNGIxOmwrXzRiMjt9",
				"ZWxzZXtpZihmcm9tPDApe2k9bCtmcm9tO2lmKGk8MCl7aT1fNGIxO319ZWxzZXtpPWZyb20+PWw/bCtf",
				"NGIyOmZyb207fX1pZihsJiZ0eXBlb2YgYT09InN0cmluZyIpe2E9YS5zcGxpdCgiIik7fWZvcig7aSE9",
				"ZW5kO2krPV80YjApe2lmKGFbaV09PXgpe3JldHVybiBpO319cmV0dXJuIC0xO307fTtmdW5jdGlvbiBf",
				"NGIzKGEsZm4sbyl7dmFyIGk9MCxsPWEmJmEubGVuZ3RofHwwO2lmKGwmJnR5cGVvZiBhPT0ic3RyaW5n",
				"Iil7YT1hLnNwbGl0KCIiKTt9aWYodHlwZW9mIGZuPT0ic3RyaW5nIil7Zm49XzRhOFtmbl18fF80YWIo",
				"Zm4pO31pZihvKXtmb3IoO2k8bDsrK2kpe2ZuLmNhbGwobyxhW2ldLGksYSk7fX1lbHNle2Zvcig7aTxs",
				"OysraSl7Zm4oYVtpXSxpLGEpO319fTtmdW5jdGlvbiBtYXAoYSxmbixvLEN0cil7dmFyIGk9MCxsPWEm",
				"JmEubGVuZ3RofHwwLG91dD1uZXcgKEN0cnx8QXJyYXkpKGwpO2lmKGwmJnR5cGVvZiBhPT0ic3RyaW5n",
				"Iil7YT1hLnNwbGl0KCIiKTt9aWYodHlwZW9mIGZuPT0ic3RyaW5nIil7Zm49XzRhOFtmbl18fF80YWIo",
				"Zm4pO31pZihvKXtmb3IoO2k8bDsrK2kpe291dFtpXT1mbi5jYWxsKG8sYVtpXSxpLGEpO319ZWxzZXtm",
				"b3IoO2k8bDsrK2kpe291dFtpXT1mbihhW2ldLGksYSk7fX1yZXR1cm4gb3V0O307ZnVuY3Rpb24gXzRi",
				"NChhLGZuLG8pe3ZhciBpPTAsbD1hJiZhLmxlbmd0aHx8MCxvdXQ9W10sXzRiNTtpZihsJiZ0eXBlb2Yg",
				"YT09InN0cmluZyIpe2E9YS5zcGxpdCgiIik7fWlmKHR5cGVvZiBmbj09InN0cmluZyIpe2ZuPV80YThb",
				"Zm5dfHxfNGFiKGZuKTt9aWYobyl7Zm9yKDtpPGw7KytpKXtfNGI1PWFbaV07aWYoZm4uY2FsbChvLF80",
				"YjUsaSxhKSl7b3V0LnB1c2goXzRiNSk7fX19ZWxzZXtmb3IoO2k8bDsrK2kpe180YjU9YVtpXTtpZihm",
				"bihfNGI1LGksYSkpe291dC5wdXNoKF80YjUpO319fXJldHVybiBvdXQ7fTtfNGE5PXtldmVyeTpfNGFj",
				"KGZhbHNlKSxzb21lOl80YWModHJ1ZSksaW5kZXhPZjpfNGFmKHRydWUpLGxhc3RJbmRleE9mOl80YWYo",
				"ZmFsc2UpLGZvckVhY2g6XzRiMyxtYXA6bWFwLGZpbHRlcjpfNGI0LGNsZWFyQ2FjaGU6XzRhYX07MSYm",
				"bGFuZy5taXhpbihkb2pvLF80YTkpO3JldHVybiBfNGE5O30pO30sImRvam8vX2Jhc2UvanNvbiI6ZnVu",
				"Y3Rpb24oKXtkZWZpbmUoWyIuL2tlcm5lbCIsIi4uL2pzb24iXSxmdW5jdGlvbihkb2pvLGpzb24pe2Rv",
				"am8uZnJvbUpzb249ZnVuY3Rpb24oanMpe3JldHVybiBldmFsKCIoIitqcysiKSIpO307ZG9qby5fZXNj",
				"YXBlU3RyaW5nPWpzb24uc3RyaW5naWZ5O2Rvam8udG9Kc29uSW5kZW50U3RyPSJcdCI7ZG9qby50b0pz",
				"b249ZnVuY3Rpb24oaXQsXzRiNil7cmV0dXJuIGpzb24uc3RyaW5naWZ5KGl0LGZ1bmN0aW9uKGtleSxf",
				"NGI3KXtpZihfNGI3KXt2YXIgdGY9XzRiNy5fX2pzb25fX3x8XzRiNy5qc29uO2lmKHR5cGVvZiB0Zj09",
				"ImZ1bmN0aW9uIil7cmV0dXJuIHRmLmNhbGwoXzRiNyk7fX1yZXR1cm4gXzRiNzt9LF80YjYmJmRvam8u",
				"dG9Kc29uSW5kZW50U3RyKTt9O3JldHVybiBkb2pvO30pO30sImRvam8vX2Jhc2Uvd2luZG93IjpmdW5j",
				"dGlvbigpe2RlZmluZShbIi4va2VybmVsIiwiLi4vaGFzIiwiLi9zbmlmZiJdLGZ1bmN0aW9uKGRvam8s",
				"aGFzKXtkb2pvLmRvYz10aGlzWyJkb2N1bWVudCJdfHxudWxsO2Rvam8uYm9keT1mdW5jdGlvbigpe3Jl",
				"dHVybiBkb2pvLmRvYy5ib2R5fHxkb2pvLmRvYy5nZXRFbGVtZW50c0J5VGFnTmFtZSgiYm9keSIpWzBd",
				"O307ZG9qby5zZXRDb250ZXh0PWZ1bmN0aW9uKF80YjgsXzRiOSl7ZG9qby5nbG9iYWw9cmV0Lmdsb2Jh",
				"bD1fNGI4O2Rvam8uZG9jPXJldC5kb2M9XzRiOTt9O2Rvam8ud2l0aEdsb2JhbD1mdW5jdGlvbihfNGJh",
				"LF80YmIsXzRiYyxfNGJkKXt2YXIgXzRiZT1kb2pvLmdsb2JhbDt0cnl7ZG9qby5nbG9iYWw9cmV0Lmds",
				"b2JhbD1fNGJhO3JldHVybiBkb2pvLndpdGhEb2MuY2FsbChudWxsLF80YmEuZG9jdW1lbnQsXzRiYixf",
				"NGJjLF80YmQpO31maW5hbGx5e2Rvam8uZ2xvYmFsPXJldC5nbG9iYWw9XzRiZTt9fTtkb2pvLndpdGhE",
				"b2M9ZnVuY3Rpb24oXzRiZixfNGMwLF80YzEsXzRjMil7dmFyIF80YzM9ZG9qby5kb2Msb2xkUT1kb2pv",
				"LmlzUXVpcmtzLF80YzQ9ZG9qby5pc0lFLGlzSUUsbW9kZSxwd2luO3RyeXtkb2pvLmRvYz1yZXQuZG9j",
				"PV80YmY7ZG9qby5pc1F1aXJrcz1oYXMuYWRkKCJxdWlya3MiLGRvam8uZG9jLmNvbXBhdE1vZGU9PSJC",
				"YWNrQ29tcGF0Iix0cnVlLHRydWUpO2lmKGhhcygiaWUiKSl7aWYoKHB3aW49XzRiZi5wYXJlbnRXaW5k",
				"b3cpJiZwd2luLm5hdmlnYXRvcil7aXNJRT1wYXJzZUZsb2F0KHB3aW4ubmF2aWdhdG9yLmFwcFZlcnNp",
				"b24uc3BsaXQoIk1TSUUgIilbMV0pfHx1bmRlZmluZWQ7bW9kZT1fNGJmLmRvY3VtZW50TW9kZTtpZiht",
				"b2RlJiZtb2RlIT01JiZNYXRoLmZsb29yKGlzSUUpIT1tb2RlKXtpc0lFPW1vZGU7fWRvam8uaXNJRT1o",
				"YXMuYWRkKCJpZSIsaXNJRSx0cnVlLHRydWUpO319aWYoXzRjMSYmdHlwZW9mIF80YzA9PSJzdHJpbmci",
				"KXtfNGMwPV80YzFbXzRjMF07fXJldHVybiBfNGMwLmFwcGx5KF80YzEsXzRjMnx8W10pO31maW5hbGx5",
				"e2Rvam8uZG9jPXJldC5kb2M9XzRjMztkb2pvLmlzUXVpcmtzPWhhcy5hZGQoInF1aXJrcyIsb2xkUSx0",
				"cnVlLHRydWUpO2Rvam8uaXNJRT1oYXMuYWRkKCJpZSIsXzRjNCx0cnVlLHRydWUpO319O3ZhciByZXQ9",
				"e2dsb2JhbDpkb2pvLmdsb2JhbCxkb2M6ZG9qby5kb2MsYm9keTpkb2pvLmJvZHksc2V0Q29udGV4dDpk",
				"b2pvLnNldENvbnRleHQsd2l0aEdsb2JhbDpkb2pvLndpdGhHbG9iYWwsd2l0aERvYzpkb2pvLndpdGhE",
				"b2N9O3JldHVybiByZXQ7fSk7fSwiZG9qby9kb20tY2xhc3MiOmZ1bmN0aW9uKCl7ZGVmaW5lKFsiLi9f",
				"YmFzZS9sYW5nIiwiLi9fYmFzZS9hcnJheSIsIi4vZG9tIl0sZnVuY3Rpb24obGFuZyxfNGM1LGRvbSl7",
				"dmFyIF80YzY9ImNsYXNzTmFtZSI7dmFyIGNscyxfNGM3PS9ccysvLGExPVsiIl07ZnVuY3Rpb24gXzRj",
				"OChzKXtpZih0eXBlb2Ygcz09InN0cmluZyJ8fHMgaW5zdGFuY2VvZiBTdHJpbmcpe2lmKHMmJiFfNGM3",
				"LnRlc3Qocykpe2ExWzBdPXM7cmV0dXJuIGExO312YXIgYT1zLnNwbGl0KF80YzcpO2lmKGEubGVuZ3Ro",
				"JiYhYVswXSl7YS5zaGlmdCgpO31pZihhLmxlbmd0aCYmIWFbYS5sZW5ndGgtMV0pe2EucG9wKCk7fXJl",
				"dHVybiBhO31pZighcyl7cmV0dXJuIFtdO31yZXR1cm4gXzRjNS5maWx0ZXIocyxmdW5jdGlvbih4KXty",
				"ZXR1cm4geDt9KTt9O3ZhciBfNGM5PXt9O2Nscz17Y29udGFpbnM6ZnVuY3Rpb24gY29udGFpbnNDbGFz",
				"cyhub2RlLF80Y2Epe3JldHVybiAoKCIgIitkb20uYnlJZChub2RlKVtfNGM2XSsiICIpLmluZGV4T2Yo",
				"IiAiK180Y2ErIiAiKT49MCk7fSxhZGQ6ZnVuY3Rpb24gYWRkQ2xhc3Mobm9kZSxfNGNiKXtub2RlPWRv",
				"bS5ieUlkKG5vZGUpO180Y2I9XzRjOChfNGNiKTt2YXIgY2xzPW5vZGVbXzRjNl0sXzRjYztjbHM9Y2xz",
				"PyIgIitjbHMrIiAiOiIgIjtfNGNjPWNscy5sZW5ndGg7Zm9yKHZhciBpPTAsbGVuPV80Y2IubGVuZ3Ro",
				"LGM7aTxsZW47KytpKXtjPV80Y2JbaV07aWYoYyYmY2xzLmluZGV4T2YoIiAiK2MrIiAiKTwwKXtjbHMr",
				"PWMrIiAiO319aWYoXzRjYzxjbHMubGVuZ3RoKXtub2RlW180YzZdPWNscy5zdWJzdHIoMSxjbHMubGVu",
				"Z3RoLTIpO319LHJlbW92ZTpmdW5jdGlvbiByZW1vdmVDbGFzcyhub2RlLF80Y2Qpe25vZGU9ZG9tLmJ5",
				"SWQobm9kZSk7dmFyIGNscztpZihfNGNkIT09dW5kZWZpbmVkKXtfNGNkPV80YzgoXzRjZCk7Y2xzPSIg",
				"Iitub2RlW180YzZdKyIgIjtmb3IodmFyIGk9MCxsZW49XzRjZC5sZW5ndGg7aTxsZW47KytpKXtjbHM9",
				"Y2xzLnJlcGxhY2UoIiAiK180Y2RbaV0rIiAiLCIgIik7fWNscz1sYW5nLnRyaW0oY2xzKTt9ZWxzZXtj",
				"bHM9IiI7fWlmKG5vZGVbXzRjNl0hPWNscyl7bm9kZVtfNGM2XT1jbHM7fX0scmVwbGFjZTpmdW5jdGlv",
				"biByZXBsYWNlQ2xhc3Mobm9kZSxfNGNlLF80Y2Ype25vZGU9ZG9tLmJ5SWQobm9kZSk7XzRjOVtfNGM2",
				"XT1ub2RlW180YzZdO2Nscy5yZW1vdmUoXzRjOSxfNGNmKTtjbHMuYWRkKF80YzksXzRjZSk7aWYobm9k",
				"ZVtfNGM2XSE9PV80YzlbXzRjNl0pe25vZGVbXzRjNl09XzRjOVtfNGM2XTt9fSx0b2dnbGU6ZnVuY3Rp",
				"b24gdG9nZ2xlQ2xhc3Mobm9kZSxfNGQwLF80ZDEpe25vZGU9ZG9tLmJ5SWQobm9kZSk7aWYoXzRkMT09",
				"PXVuZGVmaW5lZCl7XzRkMD1fNGM4KF80ZDApO2Zvcih2YXIgaT0wLGxlbj1fNGQwLmxlbmd0aCxjO2k8",
				"bGVuOysraSl7Yz1fNGQwW2ldO2Nsc1tjbHMuY29udGFpbnMobm9kZSxjKT8icmVtb3ZlIjoiYWRkIl0o",
				"bm9kZSxjKTt9fWVsc2V7Y2xzW180ZDE/ImFkZCI6InJlbW92ZSJdKG5vZGUsXzRkMCk7fXJldHVybiBf",
				"NGQxO319O3JldHVybiBjbHM7fSk7fSwiZG9qby9fYmFzZS9jb25maWciOmZ1bmN0aW9uKCl7ZGVmaW5l",
				"KFsiLi4vaGFzIiwicmVxdWlyZSJdLGZ1bmN0aW9uKGhhcyxfNGQyKXt2YXIgXzRkMz17fTtpZigxKXt2",
				"YXIgc3JjPV80ZDIucmF3Q29uZmlnLHA7Zm9yKHAgaW4gc3JjKXtfNGQzW3BdPXNyY1twXTt9fWVsc2V7",
				"dmFyIF80ZDQ9ZnVuY3Rpb24oXzRkNSxfNGQ2LF80ZDcpe2ZvcihwIGluIF80ZDUpe3AhPSJoYXMiJiZo",
				"YXMuYWRkKF80ZDYrcCxfNGQ1W3BdLDAsXzRkNyk7fX07XzRkMz0xP180ZDIucmF3Q29uZmlnOnRoaXMu",
				"ZG9qb0NvbmZpZ3x8dGhpcy5kakNvbmZpZ3x8e307XzRkNChfNGQzLCJjb25maWciLDEpO180ZDQoXzRk",
				"My5oYXMsIiIsMSk7fXJldHVybiBfNGQzO30pO30sImRvam8vX2Jhc2UvZXZlbnQiOmZ1bmN0aW9uKCl7",
				"ZGVmaW5lKFsiLi9rZXJuZWwiLCIuLi9vbiIsIi4uL2hhcyIsIi4uL2RvbS1nZW9tZXRyeSJdLGZ1bmN0",
				"aW9uKGRvam8sb24saGFzLGRvbSl7aWYob24uX2ZpeEV2ZW50KXt2YXIgXzRkOD1vbi5fZml4RXZlbnQ7",
				"b24uX2ZpeEV2ZW50PWZ1bmN0aW9uKGV2dCxzZSl7ZXZ0PV80ZDgoZXZ0LHNlKTtpZihldnQpe2RvbS5u",
				"b3JtYWxpemVFdmVudChldnQpO31yZXR1cm4gZXZ0O307fWRvam8uZml4RXZlbnQ9ZnVuY3Rpb24oZXZ0",
				"LF80ZDkpe2lmKG9uLl9maXhFdmVudCl7cmV0dXJuIG9uLl9maXhFdmVudChldnQsXzRkOSk7fXJldHVy",
				"biBldnQ7fTtkb2pvLnN0b3BFdmVudD1mdW5jdGlvbihldnQpe2lmKGhhcygiZG9tLWFkZGV2ZW50bGlz",
				"dGVuZXIiKXx8KGV2dCYmZXZ0LnByZXZlbnREZWZhdWx0KSl7ZXZ0LnByZXZlbnREZWZhdWx0KCk7ZXZ0",
				"LnN0b3BQcm9wYWdhdGlvbigpO31lbHNle2V2dD1ldnR8fHdpbmRvdy5ldmVudDtldnQuY2FuY2VsQnVi",
				"YmxlPXRydWU7b24uX3ByZXZlbnREZWZhdWx0LmNhbGwoZXZ0KTt9fTtyZXR1cm4ge2ZpeDpkb2pvLmZp",
				"eEV2ZW50LHN0b3A6ZG9qby5zdG9wRXZlbnR9O30pO30sImRvam8vbWFpbiI6ZnVuY3Rpb24oKXtkZWZp",
				"bmUoWyIuL19iYXNlL2tlcm5lbCIsIi4vaGFzIiwicmVxdWlyZSIsIi4vX2Jhc2Uvc25pZmYiLCIuL19i",
				"YXNlL2xhbmciLCIuL19iYXNlL2FycmF5IiwiLi9yZWFkeSIsIi4vX2Jhc2UvZGVjbGFyZSIsIi4vX2Jh",
				"c2UvY29ubmVjdCIsIi4vX2Jhc2UvRGVmZXJyZWQiLCIuL19iYXNlL2pzb24iLCIuL19iYXNlL0NvbG9y",
				"IiwiLi9oYXMhZG9qby1maXJlYnVnPy4vX2ZpcmVidWcvZmlyZWJ1ZyIsIi4vX2Jhc2UvYnJvd3NlciIs",
				"Ii4vX2Jhc2UvbG9hZGVyIl0sZnVuY3Rpb24oZG9qbyxoYXMsXzRkYSxfNGRiLGxhbmcsXzRkYyxfNGRk",
				"KXtpZihkb2pvLmNvbmZpZy5pc0RlYnVnKXtfNGRhKFsiLi9fZmlyZWJ1Zy9maXJlYnVnIl0pO310cnVl",
				"fHxoYXMuYWRkKCJkb2pvLWNvbmZpZy1yZXF1aXJlIiwxKTtpZigxKXt2YXIgZGVwcz1kb2pvLmNvbmZp",
				"Zy5yZXF1aXJlO2lmKGRlcHMpe2RlcHM9XzRkYy5tYXAobGFuZy5pc0FycmF5KGRlcHMpP2RlcHM6W2Rl",
				"cHNdLGZ1bmN0aW9uKGl0ZW0pe3JldHVybiBpdGVtLnJlcGxhY2UoL1wuL2csIi8iKTt9KTtpZihkb2pv",
				"LmlzQXN5bmMpe180ZGEoZGVwcyk7fWVsc2V7XzRkZCgxLGZ1bmN0aW9uKCl7XzRkYShkZXBzKTt9KTt9",
				"fX1yZXR1cm4gZG9qbzt9KTt9LCJkb2pvL3JlYWR5IjpmdW5jdGlvbigpe2RlZmluZShbIi4vX2Jhc2Uv",
				"a2VybmVsIiwiLi9oYXMiLCJyZXF1aXJlIiwiLi9kb21SZWFkeSIsIi4vX2Jhc2UvbGFuZyJdLGZ1bmN0",
				"aW9uKGRvam8saGFzLF80ZGUsXzRkZixsYW5nKXt2YXIgXzRlMD0wLF80ZTEsXzRlMj1bXSxfNGUzPTAs",
				"XzRlND1mdW5jdGlvbigpe180ZTA9MTtkb2pvLl9wb3N0TG9hZD1kb2pvLmNvbmZpZy5hZnRlck9uTG9h",
				"ZD10cnVlO2lmKF80ZTIubGVuZ3RoKXtfNGUxKF80ZTUpO319LF80ZTU9ZnVuY3Rpb24oKXtpZihfNGUw",
				"JiYhXzRlMyYmXzRlMi5sZW5ndGgpe180ZTM9MTt2YXIgZj1fNGUyLnNoaWZ0KCk7dHJ5e2YoKTt9Zmlu",
				"YWxseXtfNGUzPTA7fV80ZTM9MDtpZihfNGUyLmxlbmd0aCl7XzRlMShfNGU1KTt9fX07aWYoMSl7XzRk",
				"ZS5vbigiaWRsZSIsXzRlNSk7XzRlMT1mdW5jdGlvbigpe2lmKF80ZGUuaWRsZSgpKXtfNGU1KCk7fX07",
				"fWVsc2V7XzRlMT1mdW5jdGlvbigpe180ZGUucmVhZHkoXzRlNSk7fTt9dmFyIF80ZTY9ZG9qby5yZWFk",
				"eT1kb2pvLmFkZE9uTG9hZD1mdW5jdGlvbihfNGU3LF80ZTgsXzRlOSl7dmFyIF80ZWE9bGFuZy5fdG9B",
				"cnJheShhcmd1bWVudHMpO2lmKHR5cGVvZiBfNGU3IT0ibnVtYmVyIil7XzRlOT1fNGU4O180ZTg9XzRl",
				"NztfNGU3PTEwMDA7fWVsc2V7XzRlYS5zaGlmdCgpO31fNGU5PV80ZTk/bGFuZy5oaXRjaC5hcHBseShk",
				"b2pvLF80ZWEpOmZ1bmN0aW9uKCl7XzRlOCgpO307XzRlOS5wcmlvcml0eT1fNGU3O2Zvcih2YXIgaT0w",
				"O2k8XzRlMi5sZW5ndGgmJl80ZTc+PV80ZTJbaV0ucHJpb3JpdHk7aSsrKXt9XzRlMi5zcGxpY2UoaSww",
				"LF80ZTkpO180ZTEoKTt9O3RydWV8fGhhcy5hZGQoImRvam8tY29uZmlnLWFkZE9uTG9hZCIsMSk7aWYo",
				"MSl7dmFyIGRjYT1kb2pvLmNvbmZpZy5hZGRPbkxvYWQ7aWYoZGNhKXtfNGU2WyhsYW5nLmlzQXJyYXko",
				"ZGNhKT8iYXBwbHkiOiJjYWxsIildKGRvam8sZGNhKTt9fWlmKDEmJmRvam8uY29uZmlnLnBhcnNlT25M",
				"b2FkJiYhZG9qby5pc0FzeW5jKXtfNGU2KDk5LGZ1bmN0aW9uKCl7aWYoIWRvam8ucGFyc2VyKXtkb2pv",
				"LmRlcHJlY2F0ZWQoIkFkZCBleHBsaWNpdCByZXF1aXJlKFsnZG9qby9wYXJzZXInXSk7IiwiIiwiMi4w",
				"Iik7XzRkZShbImRvam8vcGFyc2VyIl0pO319KTt9aWYoMSl7XzRkZihfNGU0KTt9ZWxzZXtfNGU0KCk7",
				"fXJldHVybiBfNGU2O30pO30sImRvam8vYXNwZWN0IjpmdW5jdGlvbigpe2RlZmluZShbXSxmdW5jdGlv",
				"bigpeyJ1c2Ugc3RyaWN0Ijt2YXIgXzRlYj0wO2Z1bmN0aW9uIF80ZWMoXzRlZCx0eXBlLF80ZWUsXzRl",
				"Zil7dmFyIF80ZjA9XzRlZFt0eXBlXTt2YXIgXzRmMT10eXBlPT0iYXJvdW5kIjt2YXIgXzRmMjtpZihf",
				"NGYxKXt2YXIgXzRmMz1fNGVlKGZ1bmN0aW9uKCl7cmV0dXJuIF80ZjAuYWR2aWNlKHRoaXMsYXJndW1l",
				"bnRzKTt9KTtfNGYyPXtyZW1vdmU6ZnVuY3Rpb24oKXtfNGYyLmNhbmNlbGxlZD10cnVlO30sYWR2aWNl",
				"OmZ1bmN0aW9uKF80ZjQsYXJncyl7cmV0dXJuIF80ZjIuY2FuY2VsbGVkP180ZjAuYWR2aWNlKF80ZjQs",
				"YXJncyk6XzRmMy5hcHBseShfNGY0LGFyZ3MpO319O31lbHNle180ZjI9e3JlbW92ZTpmdW5jdGlvbigp",
				"e3ZhciBfNGY1PV80ZjIucHJldmlvdXM7dmFyIG5leHQ9XzRmMi5uZXh0O2lmKCFuZXh0JiYhXzRmNSl7",
				"ZGVsZXRlIF80ZWRbdHlwZV07fWVsc2V7aWYoXzRmNSl7XzRmNS5uZXh0PW5leHQ7fWVsc2V7XzRlZFt0",
				"eXBlXT1uZXh0O31pZihuZXh0KXtuZXh0LnByZXZpb3VzPV80ZjU7fX19LGlkOl80ZWIrKyxhZHZpY2U6",
				"XzRlZSxyZWNlaXZlQXJndW1lbnRzOl80ZWZ9O31pZihfNGYwJiYhXzRmMSl7aWYodHlwZT09ImFmdGVy",
				"Iil7dmFyIG5leHQ9XzRmMDt3aGlsZShuZXh0KXtfNGYwPW5leHQ7bmV4dD1uZXh0Lm5leHQ7fV80ZjAu",
				"bmV4dD1fNGYyO180ZjIucHJldmlvdXM9XzRmMDt9ZWxzZXtpZih0eXBlPT0iYmVmb3JlIil7XzRlZFt0",
				"eXBlXT1fNGYyO180ZjIubmV4dD1fNGYwO180ZjAucHJldmlvdXM9XzRmMjt9fX1lbHNle180ZWRbdHlw",
				"ZV09XzRmMjt9cmV0dXJuIF80ZjI7fTtmdW5jdGlvbiBfNGY2KHR5cGUpe3JldHVybiBmdW5jdGlvbihf",
				"NGY3LF80ZjgsXzRmOSxfNGZhKXt2YXIgXzRmYj1fNGY3W180ZjhdLF80ZmM7aWYoIV80ZmJ8fF80ZmIu",
				"dGFyZ2V0IT1fNGY3KXtfNGY3W180ZjhdPV80ZmM9ZnVuY3Rpb24oKXt2YXIgXzRmZD1fNGViO3ZhciBh",
				"cmdzPWFyZ3VtZW50czt2YXIgXzRmZT1fNGZjLmJlZm9yZTt3aGlsZShfNGZlKXthcmdzPV80ZmUuYWR2",
				"aWNlLmFwcGx5KHRoaXMsYXJncyl8fGFyZ3M7XzRmZT1fNGZlLm5leHQ7fWlmKF80ZmMuYXJvdW5kKXt2",
				"YXIgXzRmZj1fNGZjLmFyb3VuZC5hZHZpY2UodGhpcyxhcmdzKTt9dmFyIF81MDA9XzRmYy5hZnRlcjt3",
				"aGlsZShfNTAwJiZfNTAwLmlkPF80ZmQpe180ZmY9XzUwMC5yZWNlaXZlQXJndW1lbnRzP181MDAuYWR2",
				"aWNlLmFwcGx5KHRoaXMsYXJncyl8fF80ZmY6XzUwMC5hZHZpY2UuY2FsbCh0aGlzLF80ZmYpO181MDA9",
				"XzUwMC5uZXh0O31yZXR1cm4gXzRmZjt9O2lmKF80ZmIpe180ZmMuYXJvdW5kPXthZHZpY2U6ZnVuY3Rp",
				"b24oXzUwMSxhcmdzKXtyZXR1cm4gXzRmYi5hcHBseShfNTAxLGFyZ3MpO319O31fNGZjLnRhcmdldD1f",
				"NGY3O312YXIgXzUwMj1fNGVjKChfNGZjfHxfNGZiKSx0eXBlLF80ZjksXzRmYSk7XzRmOT1udWxsO3Jl",
				"dHVybiBfNTAyO307fTtyZXR1cm4ge2JlZm9yZTpfNGY2KCJiZWZvcmUiKSxhcm91bmQ6XzRmNigiYXJv",
				"dW5kIiksYWZ0ZXI6XzRmNigiYWZ0ZXIiKX07fSk7fSwiZG9qby9fYmFzZS9jb25uZWN0IjpmdW5jdGlv",
				"bigpe2RlZmluZShbIi4va2VybmVsIiwiLi4vb24iLCIuLi90b3BpYyIsIi4uL2FzcGVjdCIsIi4vZXZl",
				"bnQiLCIuLi9tb3VzZSIsIi4vc25pZmYiLCIuL2xhbmciLCIuLi9rZXlzIl0sZnVuY3Rpb24oXzUwMyxv",
				"bixodWIsXzUwNCxfNTA1LF81MDYsaGFzLGxhbmcpe2hhcy5hZGQoImV2ZW50cy1rZXlwcmVzcy10eXBl",
				"ZCIsZnVuY3Rpb24oKXt2YXIgXzUwNz17Y2hhckNvZGU6MH07dHJ5e181MDc9ZG9jdW1lbnQuY3JlYXRl",
				"RXZlbnQoIktleWJvYXJkRXZlbnQiKTsoXzUwNy5pbml0S2V5Ym9hcmRFdmVudHx8XzUwNy5pbml0S2V5",
				"RXZlbnQpLmNhbGwoXzUwNywia2V5cHJlc3MiLHRydWUsdHJ1ZSxudWxsLGZhbHNlLGZhbHNlLGZhbHNl",
				"LGZhbHNlLDksMyk7fWNhdGNoKGUpe31yZXR1cm4gXzUwNy5jaGFyQ29kZT09MCYmIWhhcygib3BlcmEi",
				"KTt9KTtmdW5jdGlvbiBfNTA4KG9iaixfNTA5LF81MGEsXzUwYixfNTBjKXtfNTBiPWxhbmcuaGl0Y2go",
				"XzUwYSxfNTBiKTtpZighb2JqfHwhKG9iai5hZGRFdmVudExpc3RlbmVyfHxvYmouYXR0YWNoRXZlbnQp",
				"KXtyZXR1cm4gXzUwNC5hZnRlcihvYmp8fF81MDMuZ2xvYmFsLF81MDksXzUwYix0cnVlKTt9aWYodHlw",
				"ZW9mIF81MDk9PSJzdHJpbmciJiZfNTA5LnN1YnN0cmluZygwLDIpPT0ib24iKXtfNTA5PV81MDkuc3Vi",
				"c3RyaW5nKDIpO31pZighb2JqKXtvYmo9XzUwMy5nbG9iYWw7fWlmKCFfNTBjKXtzd2l0Y2goXzUwOSl7",
				"Y2FzZSAia2V5cHJlc3MiOl81MDk9XzUwZDticmVhaztjYXNlICJtb3VzZWVudGVyIjpfNTA5PV81MDYu",
				"ZW50ZXI7YnJlYWs7Y2FzZSAibW91c2VsZWF2ZSI6XzUwOT1fNTA2LmxlYXZlO2JyZWFrO319cmV0dXJu",
				"IG9uKG9iaixfNTA5LF81MGIsXzUwYyk7fTt2YXIgXzUwZT17MTA2OjQyLDExMTo0NywxODY6NTksMTg3",
				"OjQzLDE4ODo0NCwxODk6NDUsMTkwOjQ2LDE5MTo0NywxOTI6OTYsMjE5OjkxLDIyMDo5MiwyMjE6OTMs",
				"MjIyOjM5LDIyOToxMTN9O3ZhciBfNTBmPWhhcygibWFjIik/Im1ldGFLZXkiOiJjdHJsS2V5Ijt2YXIg",
				"XzUxMD1mdW5jdGlvbihldnQsXzUxMSl7dmFyIGZhdXg9bGFuZy5taXhpbih7fSxldnQsXzUxMSk7XzUx",
				"MihmYXV4KTtmYXV4LnByZXZlbnREZWZhdWx0PWZ1bmN0aW9uKCl7ZXZ0LnByZXZlbnREZWZhdWx0KCk7",
				"fTtmYXV4LnN0b3BQcm9wYWdhdGlvbj1mdW5jdGlvbigpe2V2dC5zdG9wUHJvcGFnYXRpb24oKTt9O3Jl",
				"dHVybiBmYXV4O307ZnVuY3Rpb24gXzUxMihldnQpe2V2dC5rZXlDaGFyPWV2dC5jaGFyQ29kZT9TdHJp",
				"bmcuZnJvbUNoYXJDb2RlKGV2dC5jaGFyQ29kZSk6IiI7ZXZ0LmNoYXJPckNvZGU9ZXZ0LmtleUNoYXJ8",
				"fGV2dC5rZXlDb2RlO307dmFyIF81MGQ7aWYoaGFzKCJldmVudHMta2V5cHJlc3MtdHlwZWQiKSl7dmFy",
				"IF81MTM9ZnVuY3Rpb24oZSxjb2RlKXt0cnl7cmV0dXJuIChlLmtleUNvZGU9Y29kZSk7fWNhdGNoKGUp",
				"e3JldHVybiAwO319O181MGQ9ZnVuY3Rpb24oXzUxNCxfNTE1KXt2YXIgXzUxNj1vbihfNTE0LCJrZXlk",
				"b3duIixmdW5jdGlvbihldnQpe3ZhciBrPWV2dC5rZXlDb2RlO3ZhciBfNTE3PShrIT0xM3x8KGhhcygi",
				"aWUiKT49OSYmIWhhcygicXVpcmtzIikpKSYmayE9MzImJihrIT0yN3x8IWhhcygiaWUiKSkmJihrPDQ4",
				"fHxrPjkwKSYmKGs8OTZ8fGs+MTExKSYmKGs8MTg2fHxrPjE5MikmJihrPDIxOXx8az4yMjIpJiZrIT0y",
				"Mjk7aWYoXzUxN3x8ZXZ0LmN0cmxLZXkpe3ZhciBjPV81MTc/MDprO2lmKGV2dC5jdHJsS2V5KXtpZihr",
				"PT0zfHxrPT0xMyl7cmV0dXJuIF81MTUuY2FsbChldnQuY3VycmVudFRhcmdldCxldnQpO31lbHNle2lm",
				"KGM+OTUmJmM8MTA2KXtjLT00ODt9ZWxzZXtpZigoIWV2dC5zaGlmdEtleSkmJihjPj02NSYmYzw9OTAp",
				"KXtjKz0zMjt9ZWxzZXtjPV81MGVbY118fGM7fX19fXZhciBmYXV4PV81MTAoZXZ0LHt0eXBlOiJrZXlw",
				"cmVzcyIsZmF1eDp0cnVlLGNoYXJDb2RlOmN9KTtfNTE1LmNhbGwoZXZ0LmN1cnJlbnRUYXJnZXQsZmF1",
				"eCk7aWYoaGFzKCJpZSIpKXtfNTEzKGV2dCxmYXV4LmtleUNvZGUpO319fSk7dmFyIF81MTg9b24oXzUx",
				"NCwia2V5cHJlc3MiLGZ1bmN0aW9uKGV2dCl7dmFyIGM9ZXZ0LmNoYXJDb2RlO2M9Yz49MzI/YzowO2V2",
				"dD1fNTEwKGV2dCx7Y2hhckNvZGU6YyxmYXV4OnRydWV9KTtyZXR1cm4gXzUxNS5jYWxsKHRoaXMsZXZ0",
				"KTt9KTtyZXR1cm4ge3JlbW92ZTpmdW5jdGlvbigpe181MTYucmVtb3ZlKCk7XzUxOC5yZW1vdmUoKTt9",
				"fTt9O31lbHNle2lmKGhhcygib3BlcmEiKSl7XzUwZD1mdW5jdGlvbihfNTE5LF81MWEpe3JldHVybiBv",
				"bihfNTE5LCJrZXlwcmVzcyIsZnVuY3Rpb24oZXZ0KXt2YXIgYz1ldnQud2hpY2g7aWYoYz09Myl7Yz05",
				"OTt9Yz1jPDMyJiYhZXZ0LnNoaWZ0S2V5PzA6YztpZihldnQuY3RybEtleSYmIWV2dC5zaGlmdEtleSYm",
				"Yz49NjUmJmM8PTkwKXtjKz0zMjt9cmV0dXJuIF81MWEuY2FsbCh0aGlzLF81MTAoZXZ0LHtjaGFyQ29k",
				"ZTpjfSkpO30pO307fWVsc2V7XzUwZD1mdW5jdGlvbihfNTFiLF81MWMpe3JldHVybiBvbihfNTFiLCJr",
				"ZXlwcmVzcyIsZnVuY3Rpb24oZXZ0KXtfNTEyKGV2dCk7cmV0dXJuIF81MWMuY2FsbCh0aGlzLGV2dCk7",
				"fSk7fTt9fXZhciBfNTFkPXtfa2V5cHJlc3M6XzUwZCxjb25uZWN0OmZ1bmN0aW9uKG9iaixfNTFlLF81",
				"MWYsXzUyMCxfNTIxKXt2YXIgYT1hcmd1bWVudHMsYXJncz1bXSxpPTA7YXJncy5wdXNoKHR5cGVvZiBh",
				"WzBdPT0ic3RyaW5nIj9udWxsOmFbaSsrXSxhW2krK10pO3ZhciBhMT1hW2krMV07YXJncy5wdXNoKHR5",
				"cGVvZiBhMT09InN0cmluZyJ8fHR5cGVvZiBhMT09ImZ1bmN0aW9uIj9hW2krK106bnVsbCxhW2krK10p",
				"O2Zvcih2YXIgbD1hLmxlbmd0aDtpPGw7aSsrKXthcmdzLnB1c2goYVtpXSk7fXJldHVybiBfNTA4LmFw",
				"cGx5KHRoaXMsYXJncyk7fSxkaXNjb25uZWN0OmZ1bmN0aW9uKF81MjIpe2lmKF81MjIpe181MjIucmVt",
				"b3ZlKCk7fX0sc3Vic2NyaWJlOmZ1bmN0aW9uKF81MjMsXzUyNCxfNTI1KXtyZXR1cm4gaHViLnN1YnNj",
				"cmliZShfNTIzLGxhbmcuaGl0Y2goXzUyNCxfNTI1KSk7fSxwdWJsaXNoOmZ1bmN0aW9uKF81MjYsYXJn",
				"cyl7cmV0dXJuIGh1Yi5wdWJsaXNoLmFwcGx5KGh1YixbXzUyNl0uY29uY2F0KGFyZ3MpKTt9LGNvbm5l",
				"Y3RQdWJsaXNoZXI6ZnVuY3Rpb24oXzUyNyxvYmosXzUyOCl7dmFyIHBmPWZ1bmN0aW9uKCl7XzUxZC5w",
				"dWJsaXNoKF81MjcsYXJndW1lbnRzKTt9O3JldHVybiBfNTI4P181MWQuY29ubmVjdChvYmosXzUyOCxw",
				"Zik6XzUxZC5jb25uZWN0KG9iaixwZik7fSxpc0NvcHlLZXk6ZnVuY3Rpb24oZSl7cmV0dXJuIGVbXzUw",
				"Zl07fX07XzUxZC51bnN1YnNjcmliZT1fNTFkLmRpc2Nvbm5lY3Q7MSYmbGFuZy5taXhpbihfNTAzLF81",
				"MWQpO3JldHVybiBfNTFkO30pO319fSk7KGZ1bmN0aW9uKCl7dmFyIF81Mjk9dGhpcy5yZXF1aXJlO181",
				"Mjkoe2NhY2hlOnt9fSk7IV81MjkuYXN5bmMmJl81MjkoWyJkb2pvIl0pO181MjkuYm9vdCYmXzUyOS5h",
				"cHBseShudWxsLF81MjkuYm9vdCk7fSkoKTs="
				].join('');

				buff = new Buffer(dojo_base64_blob,'base64').toString('ascii');
				fs.writeFileSync(save_file,buff);
				break;
			//
			// Install PrototypeJS
			//
			case "prototype":
				save_file = 'static/js/prototype.1.7.js';

				var prototype_base64_blob = [
				"LyogIFByb3RvdHlwZSBKYXZhU2NyaXB0IGZyYW1ld29yaywgdmVyc2lvbiAxLjcKICogIChjKSAyMDA1",
				"LTIwMTAgU2FtIFN0ZXBoZW5zb24KICoKICogIFByb3RvdHlwZSBpcyBmcmVlbHkgZGlzdHJpYnV0YWJs",
				"ZSB1bmRlciB0aGUgdGVybXMgb2YgYW4gTUlULXN0eWxlIGxpY2Vuc2UuCiAqICBGb3IgZGV0YWlscywg",
				"c2VlIHRoZSBQcm90b3R5cGUgd2ViIHNpdGU6IGh0dHA6Ly93d3cucHJvdG90eXBlanMub3JnLwogKgog",
				"Ki0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0t",
				"LS0tLS0tLS0tLS0tLS0tKi8KCnZhciBQcm90b3R5cGUgPSB7CgogIFZlcnNpb246ICcxLjcnLAoKICBC",
				"cm93c2VyOiAoZnVuY3Rpb24oKXsKICAgIHZhciB1YSA9IG5hdmlnYXRvci51c2VyQWdlbnQ7CiAgICB2",
				"YXIgaXNPcGVyYSA9IE9iamVjdC5wcm90b3R5cGUudG9TdHJpbmcuY2FsbCh3aW5kb3cub3BlcmEpID09",
				"ICdbb2JqZWN0IE9wZXJhXSc7CiAgICByZXR1cm4gewogICAgICBJRTogICAgICAgICAgICAgISF3aW5k",
				"b3cuYXR0YWNoRXZlbnQgJiYgIWlzT3BlcmEsCiAgICAgIE9wZXJhOiAgICAgICAgICBpc09wZXJhLAog",
				"ICAgICBXZWJLaXQ6ICAgICAgICAgdWEuaW5kZXhPZignQXBwbGVXZWJLaXQvJykgPiAtMSwKICAgICAg",
				"R2Vja286ICAgICAgICAgIHVhLmluZGV4T2YoJ0dlY2tvJykgPiAtMSAmJiB1YS5pbmRleE9mKCdLSFRN",
				"TCcpID09PSAtMSwKICAgICAgTW9iaWxlU2FmYXJpOiAgIC9BcHBsZS4qTW9iaWxlLy50ZXN0KHVhKQog",
				"ICAgfQogIH0pKCksCgogIEJyb3dzZXJGZWF0dXJlczogewogICAgWFBhdGg6ICEhZG9jdW1lbnQuZXZh",
				"bHVhdGUsCgogICAgU2VsZWN0b3JzQVBJOiAhIWRvY3VtZW50LnF1ZXJ5U2VsZWN0b3IsCgogICAgRWxl",
				"bWVudEV4dGVuc2lvbnM6IChmdW5jdGlvbigpIHsKICAgICAgdmFyIGNvbnN0cnVjdG9yID0gd2luZG93",
				"LkVsZW1lbnQgfHwgd2luZG93LkhUTUxFbGVtZW50OwogICAgICByZXR1cm4gISEoY29uc3RydWN0b3Ig",
				"JiYgY29uc3RydWN0b3IucHJvdG90eXBlKTsKICAgIH0pKCksCiAgICBTcGVjaWZpY0VsZW1lbnRFeHRl",
				"bnNpb25zOiAoZnVuY3Rpb24oKSB7CiAgICAgIGlmICh0eXBlb2Ygd2luZG93LkhUTUxEaXZFbGVtZW50",
				"ICE9PSAndW5kZWZpbmVkJykKICAgICAgICByZXR1cm4gdHJ1ZTsKCiAgICAgIHZhciBkaXYgPSBkb2N1",
				"bWVudC5jcmVhdGVFbGVtZW50KCdkaXYnKSwKICAgICAgICAgIGZvcm0gPSBkb2N1bWVudC5jcmVhdGVF",
				"bGVtZW50KCdmb3JtJyksCiAgICAgICAgICBpc1N1cHBvcnRlZCA9IGZhbHNlOwoKICAgICAgaWYgKGRp",
				"dlsnX19wcm90b19fJ10gJiYgKGRpdlsnX19wcm90b19fJ10gIT09IGZvcm1bJ19fcHJvdG9fXyddKSkg",
				"ewogICAgICAgIGlzU3VwcG9ydGVkID0gdHJ1ZTsKICAgICAgfQoKICAgICAgZGl2ID0gZm9ybSA9IG51",
				"bGw7CgogICAgICByZXR1cm4gaXNTdXBwb3J0ZWQ7CiAgICB9KSgpCiAgfSwKCiAgU2NyaXB0RnJhZ21l",
				"bnQ6ICc8c2NyaXB0W14+XSo+KFtcXFNcXHNdKj8pPFwvc2NyaXB0PicsCiAgSlNPTkZpbHRlcjogL15c",
				"L1wqLXNlY3VyZS0oW1xzXFNdKilcKlwvXHMqJC8sCgogIGVtcHR5RnVuY3Rpb246IGZ1bmN0aW9uKCkg",
				"eyB9LAoKICBLOiBmdW5jdGlvbih4KSB7IHJldHVybiB4IH0KfTsKCmlmIChQcm90b3R5cGUuQnJvd3Nl",
				"ci5Nb2JpbGVTYWZhcmkpCiAgUHJvdG90eXBlLkJyb3dzZXJGZWF0dXJlcy5TcGVjaWZpY0VsZW1lbnRF",
				"eHRlbnNpb25zID0gZmFsc2U7CgoKdmFyIEFic3RyYWN0ID0geyB9OwoKCnZhciBUcnkgPSB7CiAgdGhl",
				"c2U6IGZ1bmN0aW9uKCkgewogICAgdmFyIHJldHVyblZhbHVlOwoKICAgIGZvciAodmFyIGkgPSAwLCBs",
				"ZW5ndGggPSBhcmd1bWVudHMubGVuZ3RoOyBpIDwgbGVuZ3RoOyBpKyspIHsKICAgICAgdmFyIGxhbWJk",
				"YSA9IGFyZ3VtZW50c1tpXTsKICAgICAgdHJ5IHsKICAgICAgICByZXR1cm5WYWx1ZSA9IGxhbWJkYSgp",
				"OwogICAgICAgIGJyZWFrOwogICAgICB9IGNhdGNoIChlKSB7IH0KICAgIH0KCiAgICByZXR1cm4gcmV0",
				"dXJuVmFsdWU7CiAgfQp9OwoKLyogQmFzZWQgb24gQWxleCBBcm5lbGwncyBpbmhlcml0YW5jZSBpbXBs",
				"ZW1lbnRhdGlvbi4gKi8KCnZhciBDbGFzcyA9IChmdW5jdGlvbigpIHsKCiAgdmFyIElTX0RPTlRFTlVN",
				"X0JVR0dZID0gKGZ1bmN0aW9uKCl7CiAgICBmb3IgKHZhciBwIGluIHsgdG9TdHJpbmc6IDEgfSkgewog",
				"ICAgICBpZiAocCA9PT0gJ3RvU3RyaW5nJykgcmV0dXJuIGZhbHNlOwogICAgfQogICAgcmV0dXJuIHRy",
				"dWU7CiAgfSkoKTsKCiAgZnVuY3Rpb24gc3ViY2xhc3MoKSB7fTsKICBmdW5jdGlvbiBjcmVhdGUoKSB7",
				"CiAgICB2YXIgcGFyZW50ID0gbnVsbCwgcHJvcGVydGllcyA9ICRBKGFyZ3VtZW50cyk7CiAgICBpZiAo",
				"T2JqZWN0LmlzRnVuY3Rpb24ocHJvcGVydGllc1swXSkpCiAgICAgIHBhcmVudCA9IHByb3BlcnRpZXMu",
				"c2hpZnQoKTsKCiAgICBmdW5jdGlvbiBrbGFzcygpIHsKICAgICAgdGhpcy5pbml0aWFsaXplLmFwcGx5",
				"KHRoaXMsIGFyZ3VtZW50cyk7CiAgICB9CgogICAgT2JqZWN0LmV4dGVuZChrbGFzcywgQ2xhc3MuTWV0",
				"aG9kcyk7CiAgICBrbGFzcy5zdXBlcmNsYXNzID0gcGFyZW50OwogICAga2xhc3Muc3ViY2xhc3NlcyA9",
				"IFtdOwoKICAgIGlmIChwYXJlbnQpIHsKICAgICAgc3ViY2xhc3MucHJvdG90eXBlID0gcGFyZW50LnBy",
				"b3RvdHlwZTsKICAgICAga2xhc3MucHJvdG90eXBlID0gbmV3IHN1YmNsYXNzOwogICAgICBwYXJlbnQu",
				"c3ViY2xhc3Nlcy5wdXNoKGtsYXNzKTsKICAgIH0KCiAgICBmb3IgKHZhciBpID0gMCwgbGVuZ3RoID0g",
				"cHJvcGVydGllcy5sZW5ndGg7IGkgPCBsZW5ndGg7IGkrKykKICAgICAga2xhc3MuYWRkTWV0aG9kcyhw",
				"cm9wZXJ0aWVzW2ldKTsKCiAgICBpZiAoIWtsYXNzLnByb3RvdHlwZS5pbml0aWFsaXplKQogICAgICBr",
				"bGFzcy5wcm90b3R5cGUuaW5pdGlhbGl6ZSA9IFByb3RvdHlwZS5lbXB0eUZ1bmN0aW9uOwoKICAgIGts",
				"YXNzLnByb3RvdHlwZS5jb25zdHJ1Y3RvciA9IGtsYXNzOwogICAgcmV0dXJuIGtsYXNzOwogIH0KCiAg",
				"ZnVuY3Rpb24gYWRkTWV0aG9kcyhzb3VyY2UpIHsKICAgIHZhciBhbmNlc3RvciAgID0gdGhpcy5zdXBl",
				"cmNsYXNzICYmIHRoaXMuc3VwZXJjbGFzcy5wcm90b3R5cGUsCiAgICAgICAgcHJvcGVydGllcyA9IE9i",
				"amVjdC5rZXlzKHNvdXJjZSk7CgogICAgaWYgKElTX0RPTlRFTlVNX0JVR0dZKSB7CiAgICAgIGlmIChz",
				"b3VyY2UudG9TdHJpbmcgIT0gT2JqZWN0LnByb3RvdHlwZS50b1N0cmluZykKICAgICAgICBwcm9wZXJ0",
				"aWVzLnB1c2goInRvU3RyaW5nIik7CiAgICAgIGlmIChzb3VyY2UudmFsdWVPZiAhPSBPYmplY3QucHJv",
				"dG90eXBlLnZhbHVlT2YpCiAgICAgICAgcHJvcGVydGllcy5wdXNoKCJ2YWx1ZU9mIik7CiAgICB9Cgog",
				"ICAgZm9yICh2YXIgaSA9IDAsIGxlbmd0aCA9IHByb3BlcnRpZXMubGVuZ3RoOyBpIDwgbGVuZ3RoOyBp",
				"KyspIHsKICAgICAgdmFyIHByb3BlcnR5ID0gcHJvcGVydGllc1tpXSwgdmFsdWUgPSBzb3VyY2VbcHJv",
				"cGVydHldOwogICAgICBpZiAoYW5jZXN0b3IgJiYgT2JqZWN0LmlzRnVuY3Rpb24odmFsdWUpICYmCiAg",
				"ICAgICAgICB2YWx1ZS5hcmd1bWVudE5hbWVzKClbMF0gPT0gIiRzdXBlciIpIHsKICAgICAgICB2YXIg",
				"bWV0aG9kID0gdmFsdWU7CiAgICAgICAgdmFsdWUgPSAoZnVuY3Rpb24obSkgewogICAgICAgICAgcmV0",
				"dXJuIGZ1bmN0aW9uKCkgeyByZXR1cm4gYW5jZXN0b3JbbV0uYXBwbHkodGhpcywgYXJndW1lbnRzKTsg",
				"fTsKICAgICAgICB9KShwcm9wZXJ0eSkud3JhcChtZXRob2QpOwoKICAgICAgICB2YWx1ZS52YWx1ZU9m",
				"ID0gbWV0aG9kLnZhbHVlT2YuYmluZChtZXRob2QpOwogICAgICAgIHZhbHVlLnRvU3RyaW5nID0gbWV0",
				"aG9kLnRvU3RyaW5nLmJpbmQobWV0aG9kKTsKICAgICAgfQogICAgICB0aGlzLnByb3RvdHlwZVtwcm9w",
				"ZXJ0eV0gPSB2YWx1ZTsKICAgIH0KCiAgICByZXR1cm4gdGhpczsKICB9CgogIHJldHVybiB7CiAgICBj",
				"cmVhdGU6IGNyZWF0ZSwKICAgIE1ldGhvZHM6IHsKICAgICAgYWRkTWV0aG9kczogYWRkTWV0aG9kcwog",
				"ICAgfQogIH07Cn0pKCk7CihmdW5jdGlvbigpIHsKCiAgdmFyIF90b1N0cmluZyA9IE9iamVjdC5wcm90",
				"b3R5cGUudG9TdHJpbmcsCiAgICAgIE5VTExfVFlQRSA9ICdOdWxsJywKICAgICAgVU5ERUZJTkVEX1RZ",
				"UEUgPSAnVW5kZWZpbmVkJywKICAgICAgQk9PTEVBTl9UWVBFID0gJ0Jvb2xlYW4nLAogICAgICBOVU1C",
				"RVJfVFlQRSA9ICdOdW1iZXInLAogICAgICBTVFJJTkdfVFlQRSA9ICdTdHJpbmcnLAogICAgICBPQkpF",
				"Q1RfVFlQRSA9ICdPYmplY3QnLAogICAgICBGVU5DVElPTl9DTEFTUyA9ICdbb2JqZWN0IEZ1bmN0aW9u",
				"XScsCiAgICAgIEJPT0xFQU5fQ0xBU1MgPSAnW29iamVjdCBCb29sZWFuXScsCiAgICAgIE5VTUJFUl9D",
				"TEFTUyA9ICdbb2JqZWN0IE51bWJlcl0nLAogICAgICBTVFJJTkdfQ0xBU1MgPSAnW29iamVjdCBTdHJp",
				"bmddJywKICAgICAgQVJSQVlfQ0xBU1MgPSAnW29iamVjdCBBcnJheV0nLAogICAgICBEQVRFX0NMQVNT",
				"ID0gJ1tvYmplY3QgRGF0ZV0nLAogICAgICBOQVRJVkVfSlNPTl9TVFJJTkdJRllfU1VQUE9SVCA9IHdp",
				"bmRvdy5KU09OICYmCiAgICAgICAgdHlwZW9mIEpTT04uc3RyaW5naWZ5ID09PSAnZnVuY3Rpb24nICYm",
				"CiAgICAgICAgSlNPTi5zdHJpbmdpZnkoMCkgPT09ICcwJyAmJgogICAgICAgIHR5cGVvZiBKU09OLnN0",
				"cmluZ2lmeShQcm90b3R5cGUuSykgPT09ICd1bmRlZmluZWQnOwoKICBmdW5jdGlvbiBUeXBlKG8pIHsK",
				"ICAgIHN3aXRjaChvKSB7CiAgICAgIGNhc2UgbnVsbDogcmV0dXJuIE5VTExfVFlQRTsKICAgICAgY2Fz",
				"ZSAodm9pZCAwKTogcmV0dXJuIFVOREVGSU5FRF9UWVBFOwogICAgfQogICAgdmFyIHR5cGUgPSB0eXBl",
				"b2YgbzsKICAgIHN3aXRjaCh0eXBlKSB7CiAgICAgIGNhc2UgJ2Jvb2xlYW4nOiByZXR1cm4gQk9PTEVB",
				"Tl9UWVBFOwogICAgICBjYXNlICdudW1iZXInOiAgcmV0dXJuIE5VTUJFUl9UWVBFOwogICAgICBjYXNl",
				"ICdzdHJpbmcnOiAgcmV0dXJuIFNUUklOR19UWVBFOwogICAgfQogICAgcmV0dXJuIE9CSkVDVF9UWVBF",
				"OwogIH0KCiAgZnVuY3Rpb24gZXh0ZW5kKGRlc3RpbmF0aW9uLCBzb3VyY2UpIHsKICAgIGZvciAodmFy",
				"IHByb3BlcnR5IGluIHNvdXJjZSkKICAgICAgZGVzdGluYXRpb25bcHJvcGVydHldID0gc291cmNlW3By",
				"b3BlcnR5XTsKICAgIHJldHVybiBkZXN0aW5hdGlvbjsKICB9CgogIGZ1bmN0aW9uIGluc3BlY3Qob2Jq",
				"ZWN0KSB7CiAgICB0cnkgewogICAgICBpZiAoaXNVbmRlZmluZWQob2JqZWN0KSkgcmV0dXJuICd1bmRl",
				"ZmluZWQnOwogICAgICBpZiAob2JqZWN0ID09PSBudWxsKSByZXR1cm4gJ251bGwnOwogICAgICByZXR1",
				"cm4gb2JqZWN0Lmluc3BlY3QgPyBvYmplY3QuaW5zcGVjdCgpIDogU3RyaW5nKG9iamVjdCk7CiAgICB9",
				"IGNhdGNoIChlKSB7CiAgICAgIGlmIChlIGluc3RhbmNlb2YgUmFuZ2VFcnJvcikgcmV0dXJuICcuLi4n",
				"OwogICAgICB0aHJvdyBlOwogICAgfQogIH0KCiAgZnVuY3Rpb24gdG9KU09OKHZhbHVlKSB7CiAgICBy",
				"ZXR1cm4gU3RyKCcnLCB7ICcnOiB2YWx1ZSB9LCBbXSk7CiAgfQoKICBmdW5jdGlvbiBTdHIoa2V5LCBo",
				"b2xkZXIsIHN0YWNrKSB7CiAgICB2YXIgdmFsdWUgPSBob2xkZXJba2V5XSwKICAgICAgICB0eXBlID0g",
				"dHlwZW9mIHZhbHVlOwoKICAgIGlmIChUeXBlKHZhbHVlKSA9PT0gT0JKRUNUX1RZUEUgJiYgdHlwZW9m",
				"IHZhbHVlLnRvSlNPTiA9PT0gJ2Z1bmN0aW9uJykgewogICAgICB2YWx1ZSA9IHZhbHVlLnRvSlNPTihr",
				"ZXkpOwogICAgfQoKICAgIHZhciBfY2xhc3MgPSBfdG9TdHJpbmcuY2FsbCh2YWx1ZSk7CgogICAgc3dp",
				"dGNoIChfY2xhc3MpIHsKICAgICAgY2FzZSBOVU1CRVJfQ0xBU1M6CiAgICAgIGNhc2UgQk9PTEVBTl9D",
				"TEFTUzoKICAgICAgY2FzZSBTVFJJTkdfQ0xBU1M6CiAgICAgICAgdmFsdWUgPSB2YWx1ZS52YWx1ZU9m",
				"KCk7CiAgICB9CgogICAgc3dpdGNoICh2YWx1ZSkgewogICAgICBjYXNlIG51bGw6IHJldHVybiAnbnVs",
				"bCc7CiAgICAgIGNhc2UgdHJ1ZTogcmV0dXJuICd0cnVlJzsKICAgICAgY2FzZSBmYWxzZTogcmV0dXJu",
				"ICdmYWxzZSc7CiAgICB9CgogICAgdHlwZSA9IHR5cGVvZiB2YWx1ZTsKICAgIHN3aXRjaCAodHlwZSkg",
				"ewogICAgICBjYXNlICdzdHJpbmcnOgogICAgICAgIHJldHVybiB2YWx1ZS5pbnNwZWN0KHRydWUpOwog",
				"ICAgICBjYXNlICdudW1iZXInOgogICAgICAgIHJldHVybiBpc0Zpbml0ZSh2YWx1ZSkgPyBTdHJpbmco",
				"dmFsdWUpIDogJ251bGwnOwogICAgICBjYXNlICdvYmplY3QnOgoKICAgICAgICBmb3IgKHZhciBpID0g",
				"MCwgbGVuZ3RoID0gc3RhY2subGVuZ3RoOyBpIDwgbGVuZ3RoOyBpKyspIHsKICAgICAgICAgIGlmIChz",
				"dGFja1tpXSA9PT0gdmFsdWUpIHsgdGhyb3cgbmV3IFR5cGVFcnJvcigpOyB9CiAgICAgICAgfQogICAg",
				"ICAgIHN0YWNrLnB1c2godmFsdWUpOwoKICAgICAgICB2YXIgcGFydGlhbCA9IFtdOwogICAgICAgIGlm",
				"IChfY2xhc3MgPT09IEFSUkFZX0NMQVNTKSB7CiAgICAgICAgICBmb3IgKHZhciBpID0gMCwgbGVuZ3Ro",
				"ID0gdmFsdWUubGVuZ3RoOyBpIDwgbGVuZ3RoOyBpKyspIHsKICAgICAgICAgICAgdmFyIHN0ciA9IFN0",
				"cihpLCB2YWx1ZSwgc3RhY2spOwogICAgICAgICAgICBwYXJ0aWFsLnB1c2godHlwZW9mIHN0ciA9PT0g",
				"J3VuZGVmaW5lZCcgPyAnbnVsbCcgOiBzdHIpOwogICAgICAgICAgfQogICAgICAgICAgcGFydGlhbCA9",
				"ICdbJyArIHBhcnRpYWwuam9pbignLCcpICsgJ10nOwogICAgICAgIH0gZWxzZSB7CiAgICAgICAgICB2",
				"YXIga2V5cyA9IE9iamVjdC5rZXlzKHZhbHVlKTsKICAgICAgICAgIGZvciAodmFyIGkgPSAwLCBsZW5n",
				"dGggPSBrZXlzLmxlbmd0aDsgaSA8IGxlbmd0aDsgaSsrKSB7CiAgICAgICAgICAgIHZhciBrZXkgPSBr",
				"ZXlzW2ldLCBzdHIgPSBTdHIoa2V5LCB2YWx1ZSwgc3RhY2spOwogICAgICAgICAgICBpZiAodHlwZW9m",
				"IHN0ciAhPT0gInVuZGVmaW5lZCIpIHsKICAgICAgICAgICAgICAgcGFydGlhbC5wdXNoKGtleS5pbnNw",
				"ZWN0KHRydWUpKyAnOicgKyBzdHIpOwogICAgICAgICAgICAgfQogICAgICAgICAgfQogICAgICAgICAg",
				"cGFydGlhbCA9ICd7JyArIHBhcnRpYWwuam9pbignLCcpICsgJ30nOwogICAgICAgIH0KICAgICAgICBz",
				"dGFjay5wb3AoKTsKICAgICAgICByZXR1cm4gcGFydGlhbDsKICAgIH0KICB9CgogIGZ1bmN0aW9uIHN0",
				"cmluZ2lmeShvYmplY3QpIHsKICAgIHJldHVybiBKU09OLnN0cmluZ2lmeShvYmplY3QpOwogIH0KCiAg",
				"ZnVuY3Rpb24gdG9RdWVyeVN0cmluZyhvYmplY3QpIHsKICAgIHJldHVybiAkSChvYmplY3QpLnRvUXVl",
				"cnlTdHJpbmcoKTsKICB9CgogIGZ1bmN0aW9uIHRvSFRNTChvYmplY3QpIHsKICAgIHJldHVybiBvYmpl",
				"Y3QgJiYgb2JqZWN0LnRvSFRNTCA/IG9iamVjdC50b0hUTUwoKSA6IFN0cmluZy5pbnRlcnByZXQob2Jq",
				"ZWN0KTsKICB9CgogIGZ1bmN0aW9uIGtleXMob2JqZWN0KSB7CiAgICBpZiAoVHlwZShvYmplY3QpICE9",
				"PSBPQkpFQ1RfVFlQRSkgeyB0aHJvdyBuZXcgVHlwZUVycm9yKCk7IH0KICAgIHZhciByZXN1bHRzID0g",
				"W107CiAgICBmb3IgKHZhciBwcm9wZXJ0eSBpbiBvYmplY3QpIHsKICAgICAgaWYgKG9iamVjdC5oYXNP",
				"d25Qcm9wZXJ0eShwcm9wZXJ0eSkpIHsKICAgICAgICByZXN1bHRzLnB1c2gocHJvcGVydHkpOwogICAg",
				"ICB9CiAgICB9CiAgICByZXR1cm4gcmVzdWx0czsKICB9CgogIGZ1bmN0aW9uIHZhbHVlcyhvYmplY3Qp",
				"IHsKICAgIHZhciByZXN1bHRzID0gW107CiAgICBmb3IgKHZhciBwcm9wZXJ0eSBpbiBvYmplY3QpCiAg",
				"ICAgIHJlc3VsdHMucHVzaChvYmplY3RbcHJvcGVydHldKTsKICAgIHJldHVybiByZXN1bHRzOwogIH0K",
				"CiAgZnVuY3Rpb24gY2xvbmUob2JqZWN0KSB7CiAgICByZXR1cm4gZXh0ZW5kKHsgfSwgb2JqZWN0KTsK",
				"ICB9CgogIGZ1bmN0aW9uIGlzRWxlbWVudChvYmplY3QpIHsKICAgIHJldHVybiAhIShvYmplY3QgJiYg",
				"b2JqZWN0Lm5vZGVUeXBlID09IDEpOwogIH0KCiAgZnVuY3Rpb24gaXNBcnJheShvYmplY3QpIHsKICAg",
				"IHJldHVybiBfdG9TdHJpbmcuY2FsbChvYmplY3QpID09PSBBUlJBWV9DTEFTUzsKICB9CgogIHZhciBo",
				"YXNOYXRpdmVJc0FycmF5ID0gKHR5cGVvZiBBcnJheS5pc0FycmF5ID09ICdmdW5jdGlvbicpCiAgICAm",
				"JiBBcnJheS5pc0FycmF5KFtdKSAmJiAhQXJyYXkuaXNBcnJheSh7fSk7CgogIGlmIChoYXNOYXRpdmVJ",
				"c0FycmF5KSB7CiAgICBpc0FycmF5ID0gQXJyYXkuaXNBcnJheTsKICB9CgogIGZ1bmN0aW9uIGlzSGFz",
				"aChvYmplY3QpIHsKICAgIHJldHVybiBvYmplY3QgaW5zdGFuY2VvZiBIYXNoOwogIH0KCiAgZnVuY3Rp",
				"b24gaXNGdW5jdGlvbihvYmplY3QpIHsKICAgIHJldHVybiBfdG9TdHJpbmcuY2FsbChvYmplY3QpID09",
				"PSBGVU5DVElPTl9DTEFTUzsKICB9CgogIGZ1bmN0aW9uIGlzU3RyaW5nKG9iamVjdCkgewogICAgcmV0",
				"dXJuIF90b1N0cmluZy5jYWxsKG9iamVjdCkgPT09IFNUUklOR19DTEFTUzsKICB9CgogIGZ1bmN0aW9u",
				"IGlzTnVtYmVyKG9iamVjdCkgewogICAgcmV0dXJuIF90b1N0cmluZy5jYWxsKG9iamVjdCkgPT09IE5V",
				"TUJFUl9DTEFTUzsKICB9CgogIGZ1bmN0aW9uIGlzRGF0ZShvYmplY3QpIHsKICAgIHJldHVybiBfdG9T",
				"dHJpbmcuY2FsbChvYmplY3QpID09PSBEQVRFX0NMQVNTOwogIH0KCiAgZnVuY3Rpb24gaXNVbmRlZmlu",
				"ZWQob2JqZWN0KSB7CiAgICByZXR1cm4gdHlwZW9mIG9iamVjdCA9PT0gInVuZGVmaW5lZCI7CiAgfQoK",
				"ICBleHRlbmQoT2JqZWN0LCB7CiAgICBleHRlbmQ6ICAgICAgICBleHRlbmQsCiAgICBpbnNwZWN0OiAg",
				"ICAgICBpbnNwZWN0LAogICAgdG9KU09OOiAgICAgICAgTkFUSVZFX0pTT05fU1RSSU5HSUZZX1NVUFBP",
				"UlQgPyBzdHJpbmdpZnkgOiB0b0pTT04sCiAgICB0b1F1ZXJ5U3RyaW5nOiB0b1F1ZXJ5U3RyaW5nLAog",
				"ICAgdG9IVE1MOiAgICAgICAgdG9IVE1MLAogICAga2V5czogICAgICAgICAgT2JqZWN0LmtleXMgfHwg",
				"a2V5cywKICAgIHZhbHVlczogICAgICAgIHZhbHVlcywKICAgIGNsb25lOiAgICAgICAgIGNsb25lLAog",
				"ICAgaXNFbGVtZW50OiAgICAgaXNFbGVtZW50LAogICAgaXNBcnJheTogICAgICAgaXNBcnJheSwKICAg",
				"IGlzSGFzaDogICAgICAgIGlzSGFzaCwKICAgIGlzRnVuY3Rpb246ICAgIGlzRnVuY3Rpb24sCiAgICBp",
				"c1N0cmluZzogICAgICBpc1N0cmluZywKICAgIGlzTnVtYmVyOiAgICAgIGlzTnVtYmVyLAogICAgaXNE",
				"YXRlOiAgICAgICAgaXNEYXRlLAogICAgaXNVbmRlZmluZWQ6ICAgaXNVbmRlZmluZWQKICB9KTsKfSko",
				"KTsKT2JqZWN0LmV4dGVuZChGdW5jdGlvbi5wcm90b3R5cGUsIChmdW5jdGlvbigpIHsKICB2YXIgc2xp",
				"Y2UgPSBBcnJheS5wcm90b3R5cGUuc2xpY2U7CgogIGZ1bmN0aW9uIHVwZGF0ZShhcnJheSwgYXJncykg",
				"ewogICAgdmFyIGFycmF5TGVuZ3RoID0gYXJyYXkubGVuZ3RoLCBsZW5ndGggPSBhcmdzLmxlbmd0aDsK",
				"ICAgIHdoaWxlIChsZW5ndGgtLSkgYXJyYXlbYXJyYXlMZW5ndGggKyBsZW5ndGhdID0gYXJnc1tsZW5n",
				"dGhdOwogICAgcmV0dXJuIGFycmF5OwogIH0KCiAgZnVuY3Rpb24gbWVyZ2UoYXJyYXksIGFyZ3MpIHsK",
				"ICAgIGFycmF5ID0gc2xpY2UuY2FsbChhcnJheSwgMCk7CiAgICByZXR1cm4gdXBkYXRlKGFycmF5LCBh",
				"cmdzKTsKICB9CgogIGZ1bmN0aW9uIGFyZ3VtZW50TmFtZXMoKSB7CiAgICB2YXIgbmFtZXMgPSB0aGlz",
				"LnRvU3RyaW5nKCkubWF0Y2goL15bXHNcKF0qZnVuY3Rpb25bXihdKlwoKFteKV0qKVwpLylbMV0KICAg",
				"ICAgLnJlcGxhY2UoL1wvXC8uKj9bXHJcbl18XC9cKig/Oi58W1xyXG5dKSo/XCpcLy9nLCAnJykKICAg",
				"ICAgLnJlcGxhY2UoL1xzKy9nLCAnJykuc3BsaXQoJywnKTsKICAgIHJldHVybiBuYW1lcy5sZW5ndGgg",
				"PT0gMSAmJiAhbmFtZXNbMF0gPyBbXSA6IG5hbWVzOwogIH0KCiAgZnVuY3Rpb24gYmluZChjb250ZXh0",
				"KSB7CiAgICBpZiAoYXJndW1lbnRzLmxlbmd0aCA8IDIgJiYgT2JqZWN0LmlzVW5kZWZpbmVkKGFyZ3Vt",
				"ZW50c1swXSkpIHJldHVybiB0aGlzOwogICAgdmFyIF9fbWV0aG9kID0gdGhpcywgYXJncyA9IHNsaWNl",
				"LmNhbGwoYXJndW1lbnRzLCAxKTsKICAgIHJldHVybiBmdW5jdGlvbigpIHsKICAgICAgdmFyIGEgPSBt",
				"ZXJnZShhcmdzLCBhcmd1bWVudHMpOwogICAgICByZXR1cm4gX19tZXRob2QuYXBwbHkoY29udGV4dCwg",
				"YSk7CiAgICB9CiAgfQoKICBmdW5jdGlvbiBiaW5kQXNFdmVudExpc3RlbmVyKGNvbnRleHQpIHsKICAg",
				"IHZhciBfX21ldGhvZCA9IHRoaXMsIGFyZ3MgPSBzbGljZS5jYWxsKGFyZ3VtZW50cywgMSk7CiAgICBy",
				"ZXR1cm4gZnVuY3Rpb24oZXZlbnQpIHsKICAgICAgdmFyIGEgPSB1cGRhdGUoW2V2ZW50IHx8IHdpbmRv",
				"dy5ldmVudF0sIGFyZ3MpOwogICAgICByZXR1cm4gX19tZXRob2QuYXBwbHkoY29udGV4dCwgYSk7CiAg",
				"ICB9CiAgfQoKICBmdW5jdGlvbiBjdXJyeSgpIHsKICAgIGlmICghYXJndW1lbnRzLmxlbmd0aCkgcmV0",
				"dXJuIHRoaXM7CiAgICB2YXIgX19tZXRob2QgPSB0aGlzLCBhcmdzID0gc2xpY2UuY2FsbChhcmd1bWVu",
				"dHMsIDApOwogICAgcmV0dXJuIGZ1bmN0aW9uKCkgewogICAgICB2YXIgYSA9IG1lcmdlKGFyZ3MsIGFy",
				"Z3VtZW50cyk7CiAgICAgIHJldHVybiBfX21ldGhvZC5hcHBseSh0aGlzLCBhKTsKICAgIH0KICB9Cgog",
				"IGZ1bmN0aW9uIGRlbGF5KHRpbWVvdXQpIHsKICAgIHZhciBfX21ldGhvZCA9IHRoaXMsIGFyZ3MgPSBz",
				"bGljZS5jYWxsKGFyZ3VtZW50cywgMSk7CiAgICB0aW1lb3V0ID0gdGltZW91dCAqIDEwMDA7CiAgICBy",
				"ZXR1cm4gd2luZG93LnNldFRpbWVvdXQoZnVuY3Rpb24oKSB7CiAgICAgIHJldHVybiBfX21ldGhvZC5h",
				"cHBseShfX21ldGhvZCwgYXJncyk7CiAgICB9LCB0aW1lb3V0KTsKICB9CgogIGZ1bmN0aW9uIGRlZmVy",
				"KCkgewogICAgdmFyIGFyZ3MgPSB1cGRhdGUoWzAuMDFdLCBhcmd1bWVudHMpOwogICAgcmV0dXJuIHRo",
				"aXMuZGVsYXkuYXBwbHkodGhpcywgYXJncyk7CiAgfQoKICBmdW5jdGlvbiB3cmFwKHdyYXBwZXIpIHsK",
				"ICAgIHZhciBfX21ldGhvZCA9IHRoaXM7CiAgICByZXR1cm4gZnVuY3Rpb24oKSB7CiAgICAgIHZhciBh",
				"ID0gdXBkYXRlKFtfX21ldGhvZC5iaW5kKHRoaXMpXSwgYXJndW1lbnRzKTsKICAgICAgcmV0dXJuIHdy",
				"YXBwZXIuYXBwbHkodGhpcywgYSk7CiAgICB9CiAgfQoKICBmdW5jdGlvbiBtZXRob2RpemUoKSB7CiAg",
				"ICBpZiAodGhpcy5fbWV0aG9kaXplZCkgcmV0dXJuIHRoaXMuX21ldGhvZGl6ZWQ7CiAgICB2YXIgX19t",
				"ZXRob2QgPSB0aGlzOwogICAgcmV0dXJuIHRoaXMuX21ldGhvZGl6ZWQgPSBmdW5jdGlvbigpIHsKICAg",
				"ICAgdmFyIGEgPSB1cGRhdGUoW3RoaXNdLCBhcmd1bWVudHMpOwogICAgICByZXR1cm4gX19tZXRob2Qu",
				"YXBwbHkobnVsbCwgYSk7CiAgICB9OwogIH0KCiAgcmV0dXJuIHsKICAgIGFyZ3VtZW50TmFtZXM6ICAg",
				"ICAgIGFyZ3VtZW50TmFtZXMsCiAgICBiaW5kOiAgICAgICAgICAgICAgICBiaW5kLAogICAgYmluZEFz",
				"RXZlbnRMaXN0ZW5lcjogYmluZEFzRXZlbnRMaXN0ZW5lciwKICAgIGN1cnJ5OiAgICAgICAgICAgICAg",
				"IGN1cnJ5LAogICAgZGVsYXk6ICAgICAgICAgICAgICAgZGVsYXksCiAgICBkZWZlcjogICAgICAgICAg",
				"ICAgICBkZWZlciwKICAgIHdyYXA6ICAgICAgICAgICAgICAgIHdyYXAsCiAgICBtZXRob2RpemU6ICAg",
				"ICAgICAgICBtZXRob2RpemUKICB9Cn0pKCkpOwoKCgooZnVuY3Rpb24ocHJvdG8pIHsKCgogIGZ1bmN0",
				"aW9uIHRvSVNPU3RyaW5nKCkgewogICAgcmV0dXJuIHRoaXMuZ2V0VVRDRnVsbFllYXIoKSArICctJyAr",
				"CiAgICAgICh0aGlzLmdldFVUQ01vbnRoKCkgKyAxKS50b1BhZGRlZFN0cmluZygyKSArICctJyArCiAg",
				"ICAgIHRoaXMuZ2V0VVRDRGF0ZSgpLnRvUGFkZGVkU3RyaW5nKDIpICsgJ1QnICsKICAgICAgdGhpcy5n",
				"ZXRVVENIb3VycygpLnRvUGFkZGVkU3RyaW5nKDIpICsgJzonICsKICAgICAgdGhpcy5nZXRVVENNaW51",
				"dGVzKCkudG9QYWRkZWRTdHJpbmcoMikgKyAnOicgKwogICAgICB0aGlzLmdldFVUQ1NlY29uZHMoKS50",
				"b1BhZGRlZFN0cmluZygyKSArICdaJzsKICB9CgoKICBmdW5jdGlvbiB0b0pTT04oKSB7CiAgICByZXR1",
				"cm4gdGhpcy50b0lTT1N0cmluZygpOwogIH0KCiAgaWYgKCFwcm90by50b0lTT1N0cmluZykgcHJvdG8u",
				"dG9JU09TdHJpbmcgPSB0b0lTT1N0cmluZzsKICBpZiAoIXByb3RvLnRvSlNPTikgcHJvdG8udG9KU09O",
				"ID0gdG9KU09OOwoKfSkoRGF0ZS5wcm90b3R5cGUpOwoKClJlZ0V4cC5wcm90b3R5cGUubWF0Y2ggPSBS",
				"ZWdFeHAucHJvdG90eXBlLnRlc3Q7CgpSZWdFeHAuZXNjYXBlID0gZnVuY3Rpb24oc3RyKSB7CiAgcmV0",
				"dXJuIFN0cmluZyhzdHIpLnJlcGxhY2UoLyhbLiorP149IToke30oKXxbXF1cL1xcXSkvZywgJ1xcJDEn",
				"KTsKfTsKdmFyIFBlcmlvZGljYWxFeGVjdXRlciA9IENsYXNzLmNyZWF0ZSh7CiAgaW5pdGlhbGl6ZTog",
				"ZnVuY3Rpb24oY2FsbGJhY2ssIGZyZXF1ZW5jeSkgewogICAgdGhpcy5jYWxsYmFjayA9IGNhbGxiYWNr",
				"OwogICAgdGhpcy5mcmVxdWVuY3kgPSBmcmVxdWVuY3k7CiAgICB0aGlzLmN1cnJlbnRseUV4ZWN1dGlu",
				"ZyA9IGZhbHNlOwoKICAgIHRoaXMucmVnaXN0ZXJDYWxsYmFjaygpOwogIH0sCgogIHJlZ2lzdGVyQ2Fs",
				"bGJhY2s6IGZ1bmN0aW9uKCkgewogICAgdGhpcy50aW1lciA9IHNldEludGVydmFsKHRoaXMub25UaW1l",
				"ckV2ZW50LmJpbmQodGhpcyksIHRoaXMuZnJlcXVlbmN5ICogMTAwMCk7CiAgfSwKCiAgZXhlY3V0ZTog",
				"ZnVuY3Rpb24oKSB7CiAgICB0aGlzLmNhbGxiYWNrKHRoaXMpOwogIH0sCgogIHN0b3A6IGZ1bmN0aW9u",
				"KCkgewogICAgaWYgKCF0aGlzLnRpbWVyKSByZXR1cm47CiAgICBjbGVhckludGVydmFsKHRoaXMudGlt",
				"ZXIpOwogICAgdGhpcy50aW1lciA9IG51bGw7CiAgfSwKCiAgb25UaW1lckV2ZW50OiBmdW5jdGlvbigp",
				"IHsKICAgIGlmICghdGhpcy5jdXJyZW50bHlFeGVjdXRpbmcpIHsKICAgICAgdHJ5IHsKICAgICAgICB0",
				"aGlzLmN1cnJlbnRseUV4ZWN1dGluZyA9IHRydWU7CiAgICAgICAgdGhpcy5leGVjdXRlKCk7CiAgICAg",
				"ICAgdGhpcy5jdXJyZW50bHlFeGVjdXRpbmcgPSBmYWxzZTsKICAgICAgfSBjYXRjaChlKSB7CiAgICAg",
				"ICAgdGhpcy5jdXJyZW50bHlFeGVjdXRpbmcgPSBmYWxzZTsKICAgICAgICB0aHJvdyBlOwogICAgICB9",
				"CiAgICB9CiAgfQp9KTsKT2JqZWN0LmV4dGVuZChTdHJpbmcsIHsKICBpbnRlcnByZXQ6IGZ1bmN0aW9u",
				"KHZhbHVlKSB7CiAgICByZXR1cm4gdmFsdWUgPT0gbnVsbCA/ICcnIDogU3RyaW5nKHZhbHVlKTsKICB9",
				"LAogIHNwZWNpYWxDaGFyOiB7CiAgICAnXGInOiAnXFxiJywKICAgICdcdCc6ICdcXHQnLAogICAgJ1xu",
				"JzogJ1xcbicsCiAgICAnXGYnOiAnXFxmJywKICAgICdccic6ICdcXHInLAogICAgJ1xcJzogJ1xcXFwn",
				"CiAgfQp9KTsKCk9iamVjdC5leHRlbmQoU3RyaW5nLnByb3RvdHlwZSwgKGZ1bmN0aW9uKCkgewogIHZh",
				"ciBOQVRJVkVfSlNPTl9QQVJTRV9TVVBQT1JUID0gd2luZG93LkpTT04gJiYKICAgIHR5cGVvZiBKU09O",
				"LnBhcnNlID09PSAnZnVuY3Rpb24nICYmCiAgICBKU09OLnBhcnNlKCd7InRlc3QiOiB0cnVlfScpLnRl",
				"c3Q7CgogIGZ1bmN0aW9uIHByZXBhcmVSZXBsYWNlbWVudChyZXBsYWNlbWVudCkgewogICAgaWYgKE9i",
				"amVjdC5pc0Z1bmN0aW9uKHJlcGxhY2VtZW50KSkgcmV0dXJuIHJlcGxhY2VtZW50OwogICAgdmFyIHRl",
				"bXBsYXRlID0gbmV3IFRlbXBsYXRlKHJlcGxhY2VtZW50KTsKICAgIHJldHVybiBmdW5jdGlvbihtYXRj",
				"aCkgeyByZXR1cm4gdGVtcGxhdGUuZXZhbHVhdGUobWF0Y2gpIH07CiAgfQoKICBmdW5jdGlvbiBnc3Vi",
				"KHBhdHRlcm4sIHJlcGxhY2VtZW50KSB7CiAgICB2YXIgcmVzdWx0ID0gJycsIHNvdXJjZSA9IHRoaXMs",
				"IG1hdGNoOwogICAgcmVwbGFjZW1lbnQgPSBwcmVwYXJlUmVwbGFjZW1lbnQocmVwbGFjZW1lbnQpOwoK",
				"ICAgIGlmIChPYmplY3QuaXNTdHJpbmcocGF0dGVybikpCiAgICAgIHBhdHRlcm4gPSBSZWdFeHAuZXNj",
				"YXBlKHBhdHRlcm4pOwoKICAgIGlmICghKHBhdHRlcm4ubGVuZ3RoIHx8IHBhdHRlcm4uc291cmNlKSkg",
				"ewogICAgICByZXBsYWNlbWVudCA9IHJlcGxhY2VtZW50KCcnKTsKICAgICAgcmV0dXJuIHJlcGxhY2Vt",
				"ZW50ICsgc291cmNlLnNwbGl0KCcnKS5qb2luKHJlcGxhY2VtZW50KSArIHJlcGxhY2VtZW50OwogICAg",
				"fQoKICAgIHdoaWxlIChzb3VyY2UubGVuZ3RoID4gMCkgewogICAgICBpZiAobWF0Y2ggPSBzb3VyY2Uu",
				"bWF0Y2gocGF0dGVybikpIHsKICAgICAgICByZXN1bHQgKz0gc291cmNlLnNsaWNlKDAsIG1hdGNoLmlu",
				"ZGV4KTsKICAgICAgICByZXN1bHQgKz0gU3RyaW5nLmludGVycHJldChyZXBsYWNlbWVudChtYXRjaCkp",
				"OwogICAgICAgIHNvdXJjZSAgPSBzb3VyY2Uuc2xpY2UobWF0Y2guaW5kZXggKyBtYXRjaFswXS5sZW5n",
				"dGgpOwogICAgICB9IGVsc2UgewogICAgICAgIHJlc3VsdCArPSBzb3VyY2UsIHNvdXJjZSA9ICcnOwog",
				"ICAgICB9CiAgICB9CiAgICByZXR1cm4gcmVzdWx0OwogIH0KCiAgZnVuY3Rpb24gc3ViKHBhdHRlcm4s",
				"IHJlcGxhY2VtZW50LCBjb3VudCkgewogICAgcmVwbGFjZW1lbnQgPSBwcmVwYXJlUmVwbGFjZW1lbnQo",
				"cmVwbGFjZW1lbnQpOwogICAgY291bnQgPSBPYmplY3QuaXNVbmRlZmluZWQoY291bnQpID8gMSA6IGNv",
				"dW50OwoKICAgIHJldHVybiB0aGlzLmdzdWIocGF0dGVybiwgZnVuY3Rpb24obWF0Y2gpIHsKICAgICAg",
				"aWYgKC0tY291bnQgPCAwKSByZXR1cm4gbWF0Y2hbMF07CiAgICAgIHJldHVybiByZXBsYWNlbWVudCht",
				"YXRjaCk7CiAgICB9KTsKICB9CgogIGZ1bmN0aW9uIHNjYW4ocGF0dGVybiwgaXRlcmF0b3IpIHsKICAg",
				"IHRoaXMuZ3N1YihwYXR0ZXJuLCBpdGVyYXRvcik7CiAgICByZXR1cm4gU3RyaW5nKHRoaXMpOwogIH0K",
				"CiAgZnVuY3Rpb24gdHJ1bmNhdGUobGVuZ3RoLCB0cnVuY2F0aW9uKSB7CiAgICBsZW5ndGggPSBsZW5n",
				"dGggfHwgMzA7CiAgICB0cnVuY2F0aW9uID0gT2JqZWN0LmlzVW5kZWZpbmVkKHRydW5jYXRpb24pID8g",
				"Jy4uLicgOiB0cnVuY2F0aW9uOwogICAgcmV0dXJuIHRoaXMubGVuZ3RoID4gbGVuZ3RoID8KICAgICAg",
				"dGhpcy5zbGljZSgwLCBsZW5ndGggLSB0cnVuY2F0aW9uLmxlbmd0aCkgKyB0cnVuY2F0aW9uIDogU3Ry",
				"aW5nKHRoaXMpOwogIH0KCiAgZnVuY3Rpb24gc3RyaXAoKSB7CiAgICByZXR1cm4gdGhpcy5yZXBsYWNl",
				"KC9eXHMrLywgJycpLnJlcGxhY2UoL1xzKyQvLCAnJyk7CiAgfQoKICBmdW5jdGlvbiBzdHJpcFRhZ3Mo",
				"KSB7CiAgICByZXR1cm4gdGhpcy5yZXBsYWNlKC88XHcrKFxzKygiW14iXSoifCdbXiddKid8W14+XSkr",
				"KT8+fDxcL1x3Kz4vZ2ksICcnKTsKICB9CgogIGZ1bmN0aW9uIHN0cmlwU2NyaXB0cygpIHsKICAgIHJl",
				"dHVybiB0aGlzLnJlcGxhY2UobmV3IFJlZ0V4cChQcm90b3R5cGUuU2NyaXB0RnJhZ21lbnQsICdpbWcn",
				"KSwgJycpOwogIH0KCiAgZnVuY3Rpb24gZXh0cmFjdFNjcmlwdHMoKSB7CiAgICB2YXIgbWF0Y2hBbGwg",
				"PSBuZXcgUmVnRXhwKFByb3RvdHlwZS5TY3JpcHRGcmFnbWVudCwgJ2ltZycpLAogICAgICAgIG1hdGNo",
				"T25lID0gbmV3IFJlZ0V4cChQcm90b3R5cGUuU2NyaXB0RnJhZ21lbnQsICdpbScpOwogICAgcmV0dXJu",
				"ICh0aGlzLm1hdGNoKG1hdGNoQWxsKSB8fCBbXSkubWFwKGZ1bmN0aW9uKHNjcmlwdFRhZykgewogICAg",
				"ICByZXR1cm4gKHNjcmlwdFRhZy5tYXRjaChtYXRjaE9uZSkgfHwgWycnLCAnJ10pWzFdOwogICAgfSk7",
				"CiAgfQoKICBmdW5jdGlvbiBldmFsU2NyaXB0cygpIHsKICAgIHJldHVybiB0aGlzLmV4dHJhY3RTY3Jp",
				"cHRzKCkubWFwKGZ1bmN0aW9uKHNjcmlwdCkgeyByZXR1cm4gZXZhbChzY3JpcHQpIH0pOwogIH0KCiAg",
				"ZnVuY3Rpb24gZXNjYXBlSFRNTCgpIHsKICAgIHJldHVybiB0aGlzLnJlcGxhY2UoLyYvZywnJmFtcDsn",
				"KS5yZXBsYWNlKC88L2csJyZsdDsnKS5yZXBsYWNlKC8+L2csJyZndDsnKTsKICB9CgogIGZ1bmN0aW9u",
				"IHVuZXNjYXBlSFRNTCgpIHsKICAgIHJldHVybiB0aGlzLnN0cmlwVGFncygpLnJlcGxhY2UoLyZsdDsv",
				"ZywnPCcpLnJlcGxhY2UoLyZndDsvZywnPicpLnJlcGxhY2UoLyZhbXA7L2csJyYnKTsKICB9CgoKICBm",
				"dW5jdGlvbiB0b1F1ZXJ5UGFyYW1zKHNlcGFyYXRvcikgewogICAgdmFyIG1hdGNoID0gdGhpcy5zdHJp",
				"cCgpLm1hdGNoKC8oW14/I10qKSgjLiopPyQvKTsKICAgIGlmICghbWF0Y2gpIHJldHVybiB7IH07Cgog",
				"ICAgcmV0dXJuIG1hdGNoWzFdLnNwbGl0KHNlcGFyYXRvciB8fCAnJicpLmluamVjdCh7IH0sIGZ1bmN0",
				"aW9uKGhhc2gsIHBhaXIpIHsKICAgICAgaWYgKChwYWlyID0gcGFpci5zcGxpdCgnPScpKVswXSkgewog",
				"ICAgICAgIHZhciBrZXkgPSBkZWNvZGVVUklDb21wb25lbnQocGFpci5zaGlmdCgpKSwKICAgICAgICAg",
				"ICAgdmFsdWUgPSBwYWlyLmxlbmd0aCA+IDEgPyBwYWlyLmpvaW4oJz0nKSA6IHBhaXJbMF07CgogICAg",
				"ICAgIGlmICh2YWx1ZSAhPSB1bmRlZmluZWQpIHZhbHVlID0gZGVjb2RlVVJJQ29tcG9uZW50KHZhbHVl",
				"KTsKCiAgICAgICAgaWYgKGtleSBpbiBoYXNoKSB7CiAgICAgICAgICBpZiAoIU9iamVjdC5pc0FycmF5",
				"KGhhc2hba2V5XSkpIGhhc2hba2V5XSA9IFtoYXNoW2tleV1dOwogICAgICAgICAgaGFzaFtrZXldLnB1",
				"c2godmFsdWUpOwogICAgICAgIH0KICAgICAgICBlbHNlIGhhc2hba2V5XSA9IHZhbHVlOwogICAgICB9",
				"CiAgICAgIHJldHVybiBoYXNoOwogICAgfSk7CiAgfQoKICBmdW5jdGlvbiB0b0FycmF5KCkgewogICAg",
				"cmV0dXJuIHRoaXMuc3BsaXQoJycpOwogIH0KCiAgZnVuY3Rpb24gc3VjYygpIHsKICAgIHJldHVybiB0",
				"aGlzLnNsaWNlKDAsIHRoaXMubGVuZ3RoIC0gMSkgKwogICAgICBTdHJpbmcuZnJvbUNoYXJDb2RlKHRo",
				"aXMuY2hhckNvZGVBdCh0aGlzLmxlbmd0aCAtIDEpICsgMSk7CiAgfQoKICBmdW5jdGlvbiB0aW1lcyhj",
				"b3VudCkgewogICAgcmV0dXJuIGNvdW50IDwgMSA/ICcnIDogbmV3IEFycmF5KGNvdW50ICsgMSkuam9p",
				"bih0aGlzKTsKICB9CgogIGZ1bmN0aW9uIGNhbWVsaXplKCkgewogICAgcmV0dXJuIHRoaXMucmVwbGFj",
				"ZSgvLSsoLik/L2csIGZ1bmN0aW9uKG1hdGNoLCBjaHIpIHsKICAgICAgcmV0dXJuIGNociA/IGNoci50",
				"b1VwcGVyQ2FzZSgpIDogJyc7CiAgICB9KTsKICB9CgogIGZ1bmN0aW9uIGNhcGl0YWxpemUoKSB7CiAg",
				"ICByZXR1cm4gdGhpcy5jaGFyQXQoMCkudG9VcHBlckNhc2UoKSArIHRoaXMuc3Vic3RyaW5nKDEpLnRv",
				"TG93ZXJDYXNlKCk7CiAgfQoKICBmdW5jdGlvbiB1bmRlcnNjb3JlKCkgewogICAgcmV0dXJuIHRoaXMu",
				"cmVwbGFjZSgvOjovZywgJy8nKQogICAgICAgICAgICAgICAucmVwbGFjZSgvKFtBLVpdKykoW0EtWl1b",
				"YS16XSkvZywgJyQxXyQyJykKICAgICAgICAgICAgICAgLnJlcGxhY2UoLyhbYS16XGRdKShbQS1aXSkv",
				"ZywgJyQxXyQyJykKICAgICAgICAgICAgICAgLnJlcGxhY2UoLy0vZywgJ18nKQogICAgICAgICAgICAg",
				"ICAudG9Mb3dlckNhc2UoKTsKICB9CgogIGZ1bmN0aW9uIGRhc2hlcml6ZSgpIHsKICAgIHJldHVybiB0",
				"aGlzLnJlcGxhY2UoL18vZywgJy0nKTsKICB9CgogIGZ1bmN0aW9uIGluc3BlY3QodXNlRG91YmxlUXVv",
				"dGVzKSB7CiAgICB2YXIgZXNjYXBlZFN0cmluZyA9IHRoaXMucmVwbGFjZSgvW1x4MDAtXHgxZlxcXS9n",
				"LCBmdW5jdGlvbihjaGFyYWN0ZXIpIHsKICAgICAgaWYgKGNoYXJhY3RlciBpbiBTdHJpbmcuc3BlY2lh",
				"bENoYXIpIHsKICAgICAgICByZXR1cm4gU3RyaW5nLnNwZWNpYWxDaGFyW2NoYXJhY3Rlcl07CiAgICAg",
				"IH0KICAgICAgcmV0dXJuICdcXHUwMCcgKyBjaGFyYWN0ZXIuY2hhckNvZGVBdCgpLnRvUGFkZGVkU3Ry",
				"aW5nKDIsIDE2KTsKICAgIH0pOwogICAgaWYgKHVzZURvdWJsZVF1b3RlcykgcmV0dXJuICciJyArIGVz",
				"Y2FwZWRTdHJpbmcucmVwbGFjZSgvIi9nLCAnXFwiJykgKyAnIic7CiAgICByZXR1cm4gIiciICsgZXNj",
				"YXBlZFN0cmluZy5yZXBsYWNlKC8nL2csICdcXFwnJykgKyAiJyI7CiAgfQoKICBmdW5jdGlvbiB1bmZp",
				"bHRlckpTT04oZmlsdGVyKSB7CiAgICByZXR1cm4gdGhpcy5yZXBsYWNlKGZpbHRlciB8fCBQcm90b3R5",
				"cGUuSlNPTkZpbHRlciwgJyQxJyk7CiAgfQoKICBmdW5jdGlvbiBpc0pTT04oKSB7CiAgICB2YXIgc3Ry",
				"ID0gdGhpczsKICAgIGlmIChzdHIuYmxhbmsoKSkgcmV0dXJuIGZhbHNlOwogICAgc3RyID0gc3RyLnJl",
				"cGxhY2UoL1xcKD86WyJcXFwvYmZucnRdfHVbMC05YS1mQS1GXXs0fSkvZywgJ0AnKTsKICAgIHN0ciA9",
				"IHN0ci5yZXBsYWNlKC8iW14iXFxcblxyXSoifHRydWV8ZmFsc2V8bnVsbHwtP1xkKyg/OlwuXGQqKT8o",
				"PzpbZUVdWytcLV0/XGQrKT8vZywgJ10nKTsKICAgIHN0ciA9IHN0ci5yZXBsYWNlKC8oPzpefDp8LCko",
				"PzpccypcWykrL2csICcnKTsKICAgIHJldHVybiAoL15bXF0sOnt9XHNdKiQvKS50ZXN0KHN0cik7CiAg",
				"fQoKICBmdW5jdGlvbiBldmFsSlNPTihzYW5pdGl6ZSkgewogICAgdmFyIGpzb24gPSB0aGlzLnVuZmls",
				"dGVySlNPTigpLAogICAgICAgIGN4ID0gL1tcdTAwMDBcdTAwYWRcdTA2MDAtXHUwNjA0XHUwNzBmXHUx",
				"N2I0XHUxN2I1XHUyMDBjLVx1MjAwZlx1MjAyOC1cdTIwMmZcdTIwNjAtXHUyMDZmXHVmZWZmXHVmZmYw",
				"LVx1ZmZmZl0vZzsKICAgIGlmIChjeC50ZXN0KGpzb24pKSB7CiAgICAgIGpzb24gPSBqc29uLnJlcGxh",
				"Y2UoY3gsIGZ1bmN0aW9uIChhKSB7CiAgICAgICAgcmV0dXJuICdcXHUnICsgKCcwMDAwJyArIGEuY2hh",
				"ckNvZGVBdCgwKS50b1N0cmluZygxNikpLnNsaWNlKC00KTsKICAgICAgfSk7CiAgICB9CiAgICB0cnkg",
				"ewogICAgICBpZiAoIXNhbml0aXplIHx8IGpzb24uaXNKU09OKCkpIHJldHVybiBldmFsKCcoJyArIGpz",
				"b24gKyAnKScpOwogICAgfSBjYXRjaCAoZSkgeyB9CiAgICB0aHJvdyBuZXcgU3ludGF4RXJyb3IoJ0Jh",
				"ZGx5IGZvcm1lZCBKU09OIHN0cmluZzogJyArIHRoaXMuaW5zcGVjdCgpKTsKICB9CgogIGZ1bmN0aW9u",
				"IHBhcnNlSlNPTigpIHsKICAgIHZhciBqc29uID0gdGhpcy51bmZpbHRlckpTT04oKTsKICAgIHJldHVy",
				"biBKU09OLnBhcnNlKGpzb24pOwogIH0KCiAgZnVuY3Rpb24gaW5jbHVkZShwYXR0ZXJuKSB7CiAgICBy",
				"ZXR1cm4gdGhpcy5pbmRleE9mKHBhdHRlcm4pID4gLTE7CiAgfQoKICBmdW5jdGlvbiBzdGFydHNXaXRo",
				"KHBhdHRlcm4pIHsKICAgIHJldHVybiB0aGlzLmxhc3RJbmRleE9mKHBhdHRlcm4sIDApID09PSAwOwog",
				"IH0KCiAgZnVuY3Rpb24gZW5kc1dpdGgocGF0dGVybikgewogICAgdmFyIGQgPSB0aGlzLmxlbmd0aCAt",
				"IHBhdHRlcm4ubGVuZ3RoOwogICAgcmV0dXJuIGQgPj0gMCAmJiB0aGlzLmluZGV4T2YocGF0dGVybiwg",
				"ZCkgPT09IGQ7CiAgfQoKICBmdW5jdGlvbiBlbXB0eSgpIHsKICAgIHJldHVybiB0aGlzID09ICcnOwog",
				"IH0KCiAgZnVuY3Rpb24gYmxhbmsoKSB7CiAgICByZXR1cm4gL15ccyokLy50ZXN0KHRoaXMpOwogIH0K",
				"CiAgZnVuY3Rpb24gaW50ZXJwb2xhdGUob2JqZWN0LCBwYXR0ZXJuKSB7CiAgICByZXR1cm4gbmV3IFRl",
				"bXBsYXRlKHRoaXMsIHBhdHRlcm4pLmV2YWx1YXRlKG9iamVjdCk7CiAgfQoKICByZXR1cm4gewogICAg",
				"Z3N1YjogICAgICAgICAgIGdzdWIsCiAgICBzdWI6ICAgICAgICAgICAgc3ViLAogICAgc2NhbjogICAg",
				"ICAgICAgIHNjYW4sCiAgICB0cnVuY2F0ZTogICAgICAgdHJ1bmNhdGUsCiAgICBzdHJpcDogICAgICAg",
				"ICAgU3RyaW5nLnByb3RvdHlwZS50cmltIHx8IHN0cmlwLAogICAgc3RyaXBUYWdzOiAgICAgIHN0cmlw",
				"VGFncywKICAgIHN0cmlwU2NyaXB0czogICBzdHJpcFNjcmlwdHMsCiAgICBleHRyYWN0U2NyaXB0czog",
				"ZXh0cmFjdFNjcmlwdHMsCiAgICBldmFsU2NyaXB0czogICAgZXZhbFNjcmlwdHMsCiAgICBlc2NhcGVI",
				"VE1MOiAgICAgZXNjYXBlSFRNTCwKICAgIHVuZXNjYXBlSFRNTDogICB1bmVzY2FwZUhUTUwsCiAgICB0",
				"b1F1ZXJ5UGFyYW1zOiAgdG9RdWVyeVBhcmFtcywKICAgIHBhcnNlUXVlcnk6ICAgICB0b1F1ZXJ5UGFy",
				"YW1zLAogICAgdG9BcnJheTogICAgICAgIHRvQXJyYXksCiAgICBzdWNjOiAgICAgICAgICAgc3VjYywK",
				"ICAgIHRpbWVzOiAgICAgICAgICB0aW1lcywKICAgIGNhbWVsaXplOiAgICAgICBjYW1lbGl6ZSwKICAg",
				"IGNhcGl0YWxpemU6ICAgICBjYXBpdGFsaXplLAogICAgdW5kZXJzY29yZTogICAgIHVuZGVyc2NvcmUs",
				"CiAgICBkYXNoZXJpemU6ICAgICAgZGFzaGVyaXplLAogICAgaW5zcGVjdDogICAgICAgIGluc3BlY3Qs",
				"CiAgICB1bmZpbHRlckpTT046ICAgdW5maWx0ZXJKU09OLAogICAgaXNKU09OOiAgICAgICAgIGlzSlNP",
				"TiwKICAgIGV2YWxKU09OOiAgICAgICBOQVRJVkVfSlNPTl9QQVJTRV9TVVBQT1JUID8gcGFyc2VKU09O",
				"IDogZXZhbEpTT04sCiAgICBpbmNsdWRlOiAgICAgICAgaW5jbHVkZSwKICAgIHN0YXJ0c1dpdGg6ICAg",
				"ICBzdGFydHNXaXRoLAogICAgZW5kc1dpdGg6ICAgICAgIGVuZHNXaXRoLAogICAgZW1wdHk6ICAgICAg",
				"ICAgIGVtcHR5LAogICAgYmxhbms6ICAgICAgICAgIGJsYW5rLAogICAgaW50ZXJwb2xhdGU6ICAgIGlu",
				"dGVycG9sYXRlCiAgfTsKfSkoKSk7Cgp2YXIgVGVtcGxhdGUgPSBDbGFzcy5jcmVhdGUoewogIGluaXRp",
				"YWxpemU6IGZ1bmN0aW9uKHRlbXBsYXRlLCBwYXR0ZXJuKSB7CiAgICB0aGlzLnRlbXBsYXRlID0gdGVt",
				"cGxhdGUudG9TdHJpbmcoKTsKICAgIHRoaXMucGF0dGVybiA9IHBhdHRlcm4gfHwgVGVtcGxhdGUuUGF0",
				"dGVybjsKICB9LAoKICBldmFsdWF0ZTogZnVuY3Rpb24ob2JqZWN0KSB7CiAgICBpZiAob2JqZWN0ICYm",
				"IE9iamVjdC5pc0Z1bmN0aW9uKG9iamVjdC50b1RlbXBsYXRlUmVwbGFjZW1lbnRzKSkKICAgICAgb2Jq",
				"ZWN0ID0gb2JqZWN0LnRvVGVtcGxhdGVSZXBsYWNlbWVudHMoKTsKCiAgICByZXR1cm4gdGhpcy50ZW1w",
				"bGF0ZS5nc3ViKHRoaXMucGF0dGVybiwgZnVuY3Rpb24obWF0Y2gpIHsKICAgICAgaWYgKG9iamVjdCA9",
				"PSBudWxsKSByZXR1cm4gKG1hdGNoWzFdICsgJycpOwoKICAgICAgdmFyIGJlZm9yZSA9IG1hdGNoWzFd",
				"IHx8ICcnOwogICAgICBpZiAoYmVmb3JlID09ICdcXCcpIHJldHVybiBtYXRjaFsyXTsKCiAgICAgIHZh",
				"ciBjdHggPSBvYmplY3QsIGV4cHIgPSBtYXRjaFszXSwKICAgICAgICAgIHBhdHRlcm4gPSAvXihbXi5b",
				"XSt8XFsoKD86Lio/W15cXF0pPylcXSkoXC58XFt8JCkvOwoKICAgICAgbWF0Y2ggPSBwYXR0ZXJuLmV4",
				"ZWMoZXhwcik7CiAgICAgIGlmIChtYXRjaCA9PSBudWxsKSByZXR1cm4gYmVmb3JlOwoKICAgICAgd2hp",
				"bGUgKG1hdGNoICE9IG51bGwpIHsKICAgICAgICB2YXIgY29tcCA9IG1hdGNoWzFdLnN0YXJ0c1dpdGgo",
				"J1snKSA/IG1hdGNoWzJdLnJlcGxhY2UoL1xcXFxdL2csICddJykgOiBtYXRjaFsxXTsKICAgICAgICBj",
				"dHggPSBjdHhbY29tcF07CiAgICAgICAgaWYgKG51bGwgPT0gY3R4IHx8ICcnID09IG1hdGNoWzNdKSBi",
				"cmVhazsKICAgICAgICBleHByID0gZXhwci5zdWJzdHJpbmcoJ1snID09IG1hdGNoWzNdID8gbWF0Y2hb",
				"MV0ubGVuZ3RoIDogbWF0Y2hbMF0ubGVuZ3RoKTsKICAgICAgICBtYXRjaCA9IHBhdHRlcm4uZXhlYyhl",
				"eHByKTsKICAgICAgfQoKICAgICAgcmV0dXJuIGJlZm9yZSArIFN0cmluZy5pbnRlcnByZXQoY3R4KTsK",
				"ICAgIH0pOwogIH0KfSk7ClRlbXBsYXRlLlBhdHRlcm4gPSAvKF58LnxccnxcbikoI1x7KC4qPylcfSkv",
				"OwoKdmFyICRicmVhayA9IHsgfTsKCnZhciBFbnVtZXJhYmxlID0gKGZ1bmN0aW9uKCkgewogIGZ1bmN0",
				"aW9uIGVhY2goaXRlcmF0b3IsIGNvbnRleHQpIHsKICAgIHZhciBpbmRleCA9IDA7CiAgICB0cnkgewog",
				"ICAgICB0aGlzLl9lYWNoKGZ1bmN0aW9uKHZhbHVlKSB7CiAgICAgICAgaXRlcmF0b3IuY2FsbChjb250",
				"ZXh0LCB2YWx1ZSwgaW5kZXgrKyk7CiAgICAgIH0pOwogICAgfSBjYXRjaCAoZSkgewogICAgICBpZiAo",
				"ZSAhPSAkYnJlYWspIHRocm93IGU7CiAgICB9CiAgICByZXR1cm4gdGhpczsKICB9CgogIGZ1bmN0aW9u",
				"IGVhY2hTbGljZShudW1iZXIsIGl0ZXJhdG9yLCBjb250ZXh0KSB7CiAgICB2YXIgaW5kZXggPSAtbnVt",
				"YmVyLCBzbGljZXMgPSBbXSwgYXJyYXkgPSB0aGlzLnRvQXJyYXkoKTsKICAgIGlmIChudW1iZXIgPCAx",
				"KSByZXR1cm4gYXJyYXk7CiAgICB3aGlsZSAoKGluZGV4ICs9IG51bWJlcikgPCBhcnJheS5sZW5ndGgp",
				"CiAgICAgIHNsaWNlcy5wdXNoKGFycmF5LnNsaWNlKGluZGV4LCBpbmRleCtudW1iZXIpKTsKICAgIHJl",
				"dHVybiBzbGljZXMuY29sbGVjdChpdGVyYXRvciwgY29udGV4dCk7CiAgfQoKICBmdW5jdGlvbiBhbGwo",
				"aXRlcmF0b3IsIGNvbnRleHQpIHsKICAgIGl0ZXJhdG9yID0gaXRlcmF0b3IgfHwgUHJvdG90eXBlLks7",
				"CiAgICB2YXIgcmVzdWx0ID0gdHJ1ZTsKICAgIHRoaXMuZWFjaChmdW5jdGlvbih2YWx1ZSwgaW5kZXgp",
				"IHsKICAgICAgcmVzdWx0ID0gcmVzdWx0ICYmICEhaXRlcmF0b3IuY2FsbChjb250ZXh0LCB2YWx1ZSwg",
				"aW5kZXgpOwogICAgICBpZiAoIXJlc3VsdCkgdGhyb3cgJGJyZWFrOwogICAgfSk7CiAgICByZXR1cm4g",
				"cmVzdWx0OwogIH0KCiAgZnVuY3Rpb24gYW55KGl0ZXJhdG9yLCBjb250ZXh0KSB7CiAgICBpdGVyYXRv",
				"ciA9IGl0ZXJhdG9yIHx8IFByb3RvdHlwZS5LOwogICAgdmFyIHJlc3VsdCA9IGZhbHNlOwogICAgdGhp",
				"cy5lYWNoKGZ1bmN0aW9uKHZhbHVlLCBpbmRleCkgewogICAgICBpZiAocmVzdWx0ID0gISFpdGVyYXRv",
				"ci5jYWxsKGNvbnRleHQsIHZhbHVlLCBpbmRleCkpCiAgICAgICAgdGhyb3cgJGJyZWFrOwogICAgfSk7",
				"CiAgICByZXR1cm4gcmVzdWx0OwogIH0KCiAgZnVuY3Rpb24gY29sbGVjdChpdGVyYXRvciwgY29udGV4",
				"dCkgewogICAgaXRlcmF0b3IgPSBpdGVyYXRvciB8fCBQcm90b3R5cGUuSzsKICAgIHZhciByZXN1bHRz",
				"ID0gW107CiAgICB0aGlzLmVhY2goZnVuY3Rpb24odmFsdWUsIGluZGV4KSB7CiAgICAgIHJlc3VsdHMu",
				"cHVzaChpdGVyYXRvci5jYWxsKGNvbnRleHQsIHZhbHVlLCBpbmRleCkpOwogICAgfSk7CiAgICByZXR1",
				"cm4gcmVzdWx0czsKICB9CgogIGZ1bmN0aW9uIGRldGVjdChpdGVyYXRvciwgY29udGV4dCkgewogICAg",
				"dmFyIHJlc3VsdDsKICAgIHRoaXMuZWFjaChmdW5jdGlvbih2YWx1ZSwgaW5kZXgpIHsKICAgICAgaWYg",
				"KGl0ZXJhdG9yLmNhbGwoY29udGV4dCwgdmFsdWUsIGluZGV4KSkgewogICAgICAgIHJlc3VsdCA9IHZh",
				"bHVlOwogICAgICAgIHRocm93ICRicmVhazsKICAgICAgfQogICAgfSk7CiAgICByZXR1cm4gcmVzdWx0",
				"OwogIH0KCiAgZnVuY3Rpb24gZmluZEFsbChpdGVyYXRvciwgY29udGV4dCkgewogICAgdmFyIHJlc3Vs",
				"dHMgPSBbXTsKICAgIHRoaXMuZWFjaChmdW5jdGlvbih2YWx1ZSwgaW5kZXgpIHsKICAgICAgaWYgKGl0",
				"ZXJhdG9yLmNhbGwoY29udGV4dCwgdmFsdWUsIGluZGV4KSkKICAgICAgICByZXN1bHRzLnB1c2godmFs",
				"dWUpOwogICAgfSk7CiAgICByZXR1cm4gcmVzdWx0czsKICB9CgogIGZ1bmN0aW9uIGdyZXAoZmlsdGVy",
				"LCBpdGVyYXRvciwgY29udGV4dCkgewogICAgaXRlcmF0b3IgPSBpdGVyYXRvciB8fCBQcm90b3R5cGUu",
				"SzsKICAgIHZhciByZXN1bHRzID0gW107CgogICAgaWYgKE9iamVjdC5pc1N0cmluZyhmaWx0ZXIpKQog",
				"ICAgICBmaWx0ZXIgPSBuZXcgUmVnRXhwKFJlZ0V4cC5lc2NhcGUoZmlsdGVyKSk7CgogICAgdGhpcy5l",
				"YWNoKGZ1bmN0aW9uKHZhbHVlLCBpbmRleCkgewogICAgICBpZiAoZmlsdGVyLm1hdGNoKHZhbHVlKSkK",
				"ICAgICAgICByZXN1bHRzLnB1c2goaXRlcmF0b3IuY2FsbChjb250ZXh0LCB2YWx1ZSwgaW5kZXgpKTsK",
				"ICAgIH0pOwogICAgcmV0dXJuIHJlc3VsdHM7CiAgfQoKICBmdW5jdGlvbiBpbmNsdWRlKG9iamVjdCkg",
				"ewogICAgaWYgKE9iamVjdC5pc0Z1bmN0aW9uKHRoaXMuaW5kZXhPZikpCiAgICAgIGlmICh0aGlzLmlu",
				"ZGV4T2Yob2JqZWN0KSAhPSAtMSkgcmV0dXJuIHRydWU7CgogICAgdmFyIGZvdW5kID0gZmFsc2U7CiAg",
				"ICB0aGlzLmVhY2goZnVuY3Rpb24odmFsdWUpIHsKICAgICAgaWYgKHZhbHVlID09IG9iamVjdCkgewog",
				"ICAgICAgIGZvdW5kID0gdHJ1ZTsKICAgICAgICB0aHJvdyAkYnJlYWs7CiAgICAgIH0KICAgIH0pOwog",
				"ICAgcmV0dXJuIGZvdW5kOwogIH0KCiAgZnVuY3Rpb24gaW5Hcm91cHNPZihudW1iZXIsIGZpbGxXaXRo",
				"KSB7CiAgICBmaWxsV2l0aCA9IE9iamVjdC5pc1VuZGVmaW5lZChmaWxsV2l0aCkgPyBudWxsIDogZmls",
				"bFdpdGg7CiAgICByZXR1cm4gdGhpcy5lYWNoU2xpY2UobnVtYmVyLCBmdW5jdGlvbihzbGljZSkgewog",
				"ICAgICB3aGlsZShzbGljZS5sZW5ndGggPCBudW1iZXIpIHNsaWNlLnB1c2goZmlsbFdpdGgpOwogICAg",
				"ICByZXR1cm4gc2xpY2U7CiAgICB9KTsKICB9CgogIGZ1bmN0aW9uIGluamVjdChtZW1vLCBpdGVyYXRv",
				"ciwgY29udGV4dCkgewogICAgdGhpcy5lYWNoKGZ1bmN0aW9uKHZhbHVlLCBpbmRleCkgewogICAgICBt",
				"ZW1vID0gaXRlcmF0b3IuY2FsbChjb250ZXh0LCBtZW1vLCB2YWx1ZSwgaW5kZXgpOwogICAgfSk7CiAg",
				"ICByZXR1cm4gbWVtbzsKICB9CgogIGZ1bmN0aW9uIGludm9rZShtZXRob2QpIHsKICAgIHZhciBhcmdz",
				"ID0gJEEoYXJndW1lbnRzKS5zbGljZSgxKTsKICAgIHJldHVybiB0aGlzLm1hcChmdW5jdGlvbih2YWx1",
				"ZSkgewogICAgICByZXR1cm4gdmFsdWVbbWV0aG9kXS5hcHBseSh2YWx1ZSwgYXJncyk7CiAgICB9KTsK",
				"ICB9CgogIGZ1bmN0aW9uIG1heChpdGVyYXRvciwgY29udGV4dCkgewogICAgaXRlcmF0b3IgPSBpdGVy",
				"YXRvciB8fCBQcm90b3R5cGUuSzsKICAgIHZhciByZXN1bHQ7CiAgICB0aGlzLmVhY2goZnVuY3Rpb24o",
				"dmFsdWUsIGluZGV4KSB7CiAgICAgIHZhbHVlID0gaXRlcmF0b3IuY2FsbChjb250ZXh0LCB2YWx1ZSwg",
				"aW5kZXgpOwogICAgICBpZiAocmVzdWx0ID09IG51bGwgfHwgdmFsdWUgPj0gcmVzdWx0KQogICAgICAg",
				"IHJlc3VsdCA9IHZhbHVlOwogICAgfSk7CiAgICByZXR1cm4gcmVzdWx0OwogIH0KCiAgZnVuY3Rpb24g",
				"bWluKGl0ZXJhdG9yLCBjb250ZXh0KSB7CiAgICBpdGVyYXRvciA9IGl0ZXJhdG9yIHx8IFByb3RvdHlw",
				"ZS5LOwogICAgdmFyIHJlc3VsdDsKICAgIHRoaXMuZWFjaChmdW5jdGlvbih2YWx1ZSwgaW5kZXgpIHsK",
				"ICAgICAgdmFsdWUgPSBpdGVyYXRvci5jYWxsKGNvbnRleHQsIHZhbHVlLCBpbmRleCk7CiAgICAgIGlm",
				"IChyZXN1bHQgPT0gbnVsbCB8fCB2YWx1ZSA8IHJlc3VsdCkKICAgICAgICByZXN1bHQgPSB2YWx1ZTsK",
				"ICAgIH0pOwogICAgcmV0dXJuIHJlc3VsdDsKICB9CgogIGZ1bmN0aW9uIHBhcnRpdGlvbihpdGVyYXRv",
				"ciwgY29udGV4dCkgewogICAgaXRlcmF0b3IgPSBpdGVyYXRvciB8fCBQcm90b3R5cGUuSzsKICAgIHZh",
				"ciB0cnVlcyA9IFtdLCBmYWxzZXMgPSBbXTsKICAgIHRoaXMuZWFjaChmdW5jdGlvbih2YWx1ZSwgaW5k",
				"ZXgpIHsKICAgICAgKGl0ZXJhdG9yLmNhbGwoY29udGV4dCwgdmFsdWUsIGluZGV4KSA/CiAgICAgICAg",
				"dHJ1ZXMgOiBmYWxzZXMpLnB1c2godmFsdWUpOwogICAgfSk7CiAgICByZXR1cm4gW3RydWVzLCBmYWxz",
				"ZXNdOwogIH0KCiAgZnVuY3Rpb24gcGx1Y2socHJvcGVydHkpIHsKICAgIHZhciByZXN1bHRzID0gW107",
				"CiAgICB0aGlzLmVhY2goZnVuY3Rpb24odmFsdWUpIHsKICAgICAgcmVzdWx0cy5wdXNoKHZhbHVlW3By",
				"b3BlcnR5XSk7CiAgICB9KTsKICAgIHJldHVybiByZXN1bHRzOwogIH0KCiAgZnVuY3Rpb24gcmVqZWN0",
				"KGl0ZXJhdG9yLCBjb250ZXh0KSB7CiAgICB2YXIgcmVzdWx0cyA9IFtdOwogICAgdGhpcy5lYWNoKGZ1",
				"bmN0aW9uKHZhbHVlLCBpbmRleCkgewogICAgICBpZiAoIWl0ZXJhdG9yLmNhbGwoY29udGV4dCwgdmFs",
				"dWUsIGluZGV4KSkKICAgICAgICByZXN1bHRzLnB1c2godmFsdWUpOwogICAgfSk7CiAgICByZXR1cm4g",
				"cmVzdWx0czsKICB9CgogIGZ1bmN0aW9uIHNvcnRCeShpdGVyYXRvciwgY29udGV4dCkgewogICAgcmV0",
				"dXJuIHRoaXMubWFwKGZ1bmN0aW9uKHZhbHVlLCBpbmRleCkgewogICAgICByZXR1cm4gewogICAgICAg",
				"IHZhbHVlOiB2YWx1ZSwKICAgICAgICBjcml0ZXJpYTogaXRlcmF0b3IuY2FsbChjb250ZXh0LCB2YWx1",
				"ZSwgaW5kZXgpCiAgICAgIH07CiAgICB9KS5zb3J0KGZ1bmN0aW9uKGxlZnQsIHJpZ2h0KSB7CiAgICAg",
				"IHZhciBhID0gbGVmdC5jcml0ZXJpYSwgYiA9IHJpZ2h0LmNyaXRlcmlhOwogICAgICByZXR1cm4gYSA8",
				"IGIgPyAtMSA6IGEgPiBiID8gMSA6IDA7CiAgICB9KS5wbHVjaygndmFsdWUnKTsKICB9CgogIGZ1bmN0",
				"aW9uIHRvQXJyYXkoKSB7CiAgICByZXR1cm4gdGhpcy5tYXAoKTsKICB9CgogIGZ1bmN0aW9uIHppcCgp",
				"IHsKICAgIHZhciBpdGVyYXRvciA9IFByb3RvdHlwZS5LLCBhcmdzID0gJEEoYXJndW1lbnRzKTsKICAg",
				"IGlmIChPYmplY3QuaXNGdW5jdGlvbihhcmdzLmxhc3QoKSkpCiAgICAgIGl0ZXJhdG9yID0gYXJncy5w",
				"b3AoKTsKCiAgICB2YXIgY29sbGVjdGlvbnMgPSBbdGhpc10uY29uY2F0KGFyZ3MpLm1hcCgkQSk7CiAg",
				"ICByZXR1cm4gdGhpcy5tYXAoZnVuY3Rpb24odmFsdWUsIGluZGV4KSB7CiAgICAgIHJldHVybiBpdGVy",
				"YXRvcihjb2xsZWN0aW9ucy5wbHVjayhpbmRleCkpOwogICAgfSk7CiAgfQoKICBmdW5jdGlvbiBzaXpl",
				"KCkgewogICAgcmV0dXJuIHRoaXMudG9BcnJheSgpLmxlbmd0aDsKICB9CgogIGZ1bmN0aW9uIGluc3Bl",
				"Y3QoKSB7CiAgICByZXR1cm4gJyM8RW51bWVyYWJsZTonICsgdGhpcy50b0FycmF5KCkuaW5zcGVjdCgp",
				"ICsgJz4nOwogIH0KCgoKCgoKCgoKICByZXR1cm4gewogICAgZWFjaDogICAgICAgZWFjaCwKICAgIGVh",
				"Y2hTbGljZTogIGVhY2hTbGljZSwKICAgIGFsbDogICAgICAgIGFsbCwKICAgIGV2ZXJ5OiAgICAgIGFs",
				"bCwKICAgIGFueTogICAgICAgIGFueSwKICAgIHNvbWU6ICAgICAgIGFueSwKICAgIGNvbGxlY3Q6ICAg",
				"IGNvbGxlY3QsCiAgICBtYXA6ICAgICAgICBjb2xsZWN0LAogICAgZGV0ZWN0OiAgICAgZGV0ZWN0LAog",
				"ICAgZmluZEFsbDogICAgZmluZEFsbCwKICAgIHNlbGVjdDogICAgIGZpbmRBbGwsCiAgICBmaWx0ZXI6",
				"ICAgICBmaW5kQWxsLAogICAgZ3JlcDogICAgICAgZ3JlcCwKICAgIGluY2x1ZGU6ICAgIGluY2x1ZGUs",
				"CiAgICBtZW1iZXI6ICAgICBpbmNsdWRlLAogICAgaW5Hcm91cHNPZjogaW5Hcm91cHNPZiwKICAgIGlu",
				"amVjdDogICAgIGluamVjdCwKICAgIGludm9rZTogICAgIGludm9rZSwKICAgIG1heDogICAgICAgIG1h",
				"eCwKICAgIG1pbjogICAgICAgIG1pbiwKICAgIHBhcnRpdGlvbjogIHBhcnRpdGlvbiwKICAgIHBsdWNr",
				"OiAgICAgIHBsdWNrLAogICAgcmVqZWN0OiAgICAgcmVqZWN0LAogICAgc29ydEJ5OiAgICAgc29ydEJ5",
				"LAogICAgdG9BcnJheTogICAgdG9BcnJheSwKICAgIGVudHJpZXM6ICAgIHRvQXJyYXksCiAgICB6aXA6",
				"ICAgICAgICB6aXAsCiAgICBzaXplOiAgICAgICBzaXplLAogICAgaW5zcGVjdDogICAgaW5zcGVjdCwK",
				"ICAgIGZpbmQ6ICAgICAgIGRldGVjdAogIH07Cn0pKCk7CgpmdW5jdGlvbiAkQShpdGVyYWJsZSkgewog",
				"IGlmICghaXRlcmFibGUpIHJldHVybiBbXTsKICBpZiAoJ3RvQXJyYXknIGluIE9iamVjdChpdGVyYWJs",
				"ZSkpIHJldHVybiBpdGVyYWJsZS50b0FycmF5KCk7CiAgdmFyIGxlbmd0aCA9IGl0ZXJhYmxlLmxlbmd0",
				"aCB8fCAwLCByZXN1bHRzID0gbmV3IEFycmF5KGxlbmd0aCk7CiAgd2hpbGUgKGxlbmd0aC0tKSByZXN1",
				"bHRzW2xlbmd0aF0gPSBpdGVyYWJsZVtsZW5ndGhdOwogIHJldHVybiByZXN1bHRzOwp9CgoKZnVuY3Rp",
				"b24gJHcoc3RyaW5nKSB7CiAgaWYgKCFPYmplY3QuaXNTdHJpbmcoc3RyaW5nKSkgcmV0dXJuIFtdOwog",
				"IHN0cmluZyA9IHN0cmluZy5zdHJpcCgpOwogIHJldHVybiBzdHJpbmcgPyBzdHJpbmcuc3BsaXQoL1xz",
				"Ky8pIDogW107Cn0KCkFycmF5LmZyb20gPSAkQTsKCgooZnVuY3Rpb24oKSB7CiAgdmFyIGFycmF5UHJv",
				"dG8gPSBBcnJheS5wcm90b3R5cGUsCiAgICAgIHNsaWNlID0gYXJyYXlQcm90by5zbGljZSwKICAgICAg",
				"X2VhY2ggPSBhcnJheVByb3RvLmZvckVhY2g7IC8vIHVzZSBuYXRpdmUgYnJvd3NlciBKUyAxLjYgaW1w",
				"bGVtZW50YXRpb24gaWYgYXZhaWxhYmxlCgogIGZ1bmN0aW9uIGVhY2goaXRlcmF0b3IsIGNvbnRleHQp",
				"IHsKICAgIGZvciAodmFyIGkgPSAwLCBsZW5ndGggPSB0aGlzLmxlbmd0aCA+Pj4gMDsgaSA8IGxlbmd0",
				"aDsgaSsrKSB7CiAgICAgIGlmIChpIGluIHRoaXMpIGl0ZXJhdG9yLmNhbGwoY29udGV4dCwgdGhpc1tp",
				"XSwgaSwgdGhpcyk7CiAgICB9CiAgfQogIGlmICghX2VhY2gpIF9lYWNoID0gZWFjaDsKCiAgZnVuY3Rp",
				"b24gY2xlYXIoKSB7CiAgICB0aGlzLmxlbmd0aCA9IDA7CiAgICByZXR1cm4gdGhpczsKICB9CgogIGZ1",
				"bmN0aW9uIGZpcnN0KCkgewogICAgcmV0dXJuIHRoaXNbMF07CiAgfQoKICBmdW5jdGlvbiBsYXN0KCkg",
				"ewogICAgcmV0dXJuIHRoaXNbdGhpcy5sZW5ndGggLSAxXTsKICB9CgogIGZ1bmN0aW9uIGNvbXBhY3Qo",
				"KSB7CiAgICByZXR1cm4gdGhpcy5zZWxlY3QoZnVuY3Rpb24odmFsdWUpIHsKICAgICAgcmV0dXJuIHZh",
				"bHVlICE9IG51bGw7CiAgICB9KTsKICB9CgogIGZ1bmN0aW9uIGZsYXR0ZW4oKSB7CiAgICByZXR1cm4g",
				"dGhpcy5pbmplY3QoW10sIGZ1bmN0aW9uKGFycmF5LCB2YWx1ZSkgewogICAgICBpZiAoT2JqZWN0Lmlz",
				"QXJyYXkodmFsdWUpKQogICAgICAgIHJldHVybiBhcnJheS5jb25jYXQodmFsdWUuZmxhdHRlbigpKTsK",
				"ICAgICAgYXJyYXkucHVzaCh2YWx1ZSk7CiAgICAgIHJldHVybiBhcnJheTsKICAgIH0pOwogIH0KCiAg",
				"ZnVuY3Rpb24gd2l0aG91dCgpIHsKICAgIHZhciB2YWx1ZXMgPSBzbGljZS5jYWxsKGFyZ3VtZW50cywg",
				"MCk7CiAgICByZXR1cm4gdGhpcy5zZWxlY3QoZnVuY3Rpb24odmFsdWUpIHsKICAgICAgcmV0dXJuICF2",
				"YWx1ZXMuaW5jbHVkZSh2YWx1ZSk7CiAgICB9KTsKICB9CgogIGZ1bmN0aW9uIHJldmVyc2UoaW5saW5l",
				"KSB7CiAgICByZXR1cm4gKGlubGluZSA9PT0gZmFsc2UgPyB0aGlzLnRvQXJyYXkoKSA6IHRoaXMpLl9y",
				"ZXZlcnNlKCk7CiAgfQoKICBmdW5jdGlvbiB1bmlxKHNvcnRlZCkgewogICAgcmV0dXJuIHRoaXMuaW5q",
				"ZWN0KFtdLCBmdW5jdGlvbihhcnJheSwgdmFsdWUsIGluZGV4KSB7CiAgICAgIGlmICgwID09IGluZGV4",
				"IHx8IChzb3J0ZWQgPyBhcnJheS5sYXN0KCkgIT0gdmFsdWUgOiAhYXJyYXkuaW5jbHVkZSh2YWx1ZSkp",
				"KQogICAgICAgIGFycmF5LnB1c2godmFsdWUpOwogICAgICByZXR1cm4gYXJyYXk7CiAgICB9KTsKICB9",
				"CgogIGZ1bmN0aW9uIGludGVyc2VjdChhcnJheSkgewogICAgcmV0dXJuIHRoaXMudW5pcSgpLmZpbmRB",
				"bGwoZnVuY3Rpb24oaXRlbSkgewogICAgICByZXR1cm4gYXJyYXkuZGV0ZWN0KGZ1bmN0aW9uKHZhbHVl",
				"KSB7IHJldHVybiBpdGVtID09PSB2YWx1ZSB9KTsKICAgIH0pOwogIH0KCgogIGZ1bmN0aW9uIGNsb25l",
				"KCkgewogICAgcmV0dXJuIHNsaWNlLmNhbGwodGhpcywgMCk7CiAgfQoKICBmdW5jdGlvbiBzaXplKCkg",
				"ewogICAgcmV0dXJuIHRoaXMubGVuZ3RoOwogIH0KCiAgZnVuY3Rpb24gaW5zcGVjdCgpIHsKICAgIHJl",
				"dHVybiAnWycgKyB0aGlzLm1hcChPYmplY3QuaW5zcGVjdCkuam9pbignLCAnKSArICddJzsKICB9Cgog",
				"IGZ1bmN0aW9uIGluZGV4T2YoaXRlbSwgaSkgewogICAgaSB8fCAoaSA9IDApOwogICAgdmFyIGxlbmd0",
				"aCA9IHRoaXMubGVuZ3RoOwogICAgaWYgKGkgPCAwKSBpID0gbGVuZ3RoICsgaTsKICAgIGZvciAoOyBp",
				"IDwgbGVuZ3RoOyBpKyspCiAgICAgIGlmICh0aGlzW2ldID09PSBpdGVtKSByZXR1cm4gaTsKICAgIHJl",
				"dHVybiAtMTsKICB9CgogIGZ1bmN0aW9uIGxhc3RJbmRleE9mKGl0ZW0sIGkpIHsKICAgIGkgPSBpc05h",
				"TihpKSA/IHRoaXMubGVuZ3RoIDogKGkgPCAwID8gdGhpcy5sZW5ndGggKyBpIDogaSkgKyAxOwogICAg",
				"dmFyIG4gPSB0aGlzLnNsaWNlKDAsIGkpLnJldmVyc2UoKS5pbmRleE9mKGl0ZW0pOwogICAgcmV0dXJu",
				"IChuIDwgMCkgPyBuIDogaSAtIG4gLSAxOwogIH0KCiAgZnVuY3Rpb24gY29uY2F0KCkgewogICAgdmFy",
				"IGFycmF5ID0gc2xpY2UuY2FsbCh0aGlzLCAwKSwgaXRlbTsKICAgIGZvciAodmFyIGkgPSAwLCBsZW5n",
				"dGggPSBhcmd1bWVudHMubGVuZ3RoOyBpIDwgbGVuZ3RoOyBpKyspIHsKICAgICAgaXRlbSA9IGFyZ3Vt",
				"ZW50c1tpXTsKICAgICAgaWYgKE9iamVjdC5pc0FycmF5KGl0ZW0pICYmICEoJ2NhbGxlZScgaW4gaXRl",
				"bSkpIHsKICAgICAgICBmb3IgKHZhciBqID0gMCwgYXJyYXlMZW5ndGggPSBpdGVtLmxlbmd0aDsgaiA8",
				"IGFycmF5TGVuZ3RoOyBqKyspCiAgICAgICAgICBhcnJheS5wdXNoKGl0ZW1bal0pOwogICAgICB9IGVs",
				"c2UgewogICAgICAgIGFycmF5LnB1c2goaXRlbSk7CiAgICAgIH0KICAgIH0KICAgIHJldHVybiBhcnJh",
				"eTsKICB9CgogIE9iamVjdC5leHRlbmQoYXJyYXlQcm90bywgRW51bWVyYWJsZSk7CgogIGlmICghYXJy",
				"YXlQcm90by5fcmV2ZXJzZSkKICAgIGFycmF5UHJvdG8uX3JldmVyc2UgPSBhcnJheVByb3RvLnJldmVy",
				"c2U7CgogIE9iamVjdC5leHRlbmQoYXJyYXlQcm90bywgewogICAgX2VhY2g6ICAgICBfZWFjaCwKICAg",
				"IGNsZWFyOiAgICAgY2xlYXIsCiAgICBmaXJzdDogICAgIGZpcnN0LAogICAgbGFzdDogICAgICBsYXN0",
				"LAogICAgY29tcGFjdDogICBjb21wYWN0LAogICAgZmxhdHRlbjogICBmbGF0dGVuLAogICAgd2l0aG91",
				"dDogICB3aXRob3V0LAogICAgcmV2ZXJzZTogICByZXZlcnNlLAogICAgdW5pcTogICAgICB1bmlxLAog",
				"ICAgaW50ZXJzZWN0OiBpbnRlcnNlY3QsCiAgICBjbG9uZTogICAgIGNsb25lLAogICAgdG9BcnJheTog",
				"ICBjbG9uZSwKICAgIHNpemU6ICAgICAgc2l6ZSwKICAgIGluc3BlY3Q6ICAgaW5zcGVjdAogIH0pOwoK",
				"ICB2YXIgQ09OQ0FUX0FSR1VNRU5UU19CVUdHWSA9IChmdW5jdGlvbigpIHsKICAgIHJldHVybiBbXS5j",
				"b25jYXQoYXJndW1lbnRzKVswXVswXSAhPT0gMTsKICB9KSgxLDIpCgogIGlmIChDT05DQVRfQVJHVU1F",
				"TlRTX0JVR0dZKSBhcnJheVByb3RvLmNvbmNhdCA9IGNvbmNhdDsKCiAgaWYgKCFhcnJheVByb3RvLmlu",
				"ZGV4T2YpIGFycmF5UHJvdG8uaW5kZXhPZiA9IGluZGV4T2Y7CiAgaWYgKCFhcnJheVByb3RvLmxhc3RJ",
				"bmRleE9mKSBhcnJheVByb3RvLmxhc3RJbmRleE9mID0gbGFzdEluZGV4T2Y7Cn0pKCk7CmZ1bmN0aW9u",
				"ICRIKG9iamVjdCkgewogIHJldHVybiBuZXcgSGFzaChvYmplY3QpOwp9OwoKdmFyIEhhc2ggPSBDbGFz",
				"cy5jcmVhdGUoRW51bWVyYWJsZSwgKGZ1bmN0aW9uKCkgewogIGZ1bmN0aW9uIGluaXRpYWxpemUob2Jq",
				"ZWN0KSB7CiAgICB0aGlzLl9vYmplY3QgPSBPYmplY3QuaXNIYXNoKG9iamVjdCkgPyBvYmplY3QudG9P",
				"YmplY3QoKSA6IE9iamVjdC5jbG9uZShvYmplY3QpOwogIH0KCgogIGZ1bmN0aW9uIF9lYWNoKGl0ZXJh",
				"dG9yKSB7CiAgICBmb3IgKHZhciBrZXkgaW4gdGhpcy5fb2JqZWN0KSB7CiAgICAgIHZhciB2YWx1ZSA9",
				"IHRoaXMuX29iamVjdFtrZXldLCBwYWlyID0gW2tleSwgdmFsdWVdOwogICAgICBwYWlyLmtleSA9IGtl",
				"eTsKICAgICAgcGFpci52YWx1ZSA9IHZhbHVlOwogICAgICBpdGVyYXRvcihwYWlyKTsKICAgIH0KICB9",
				"CgogIGZ1bmN0aW9uIHNldChrZXksIHZhbHVlKSB7CiAgICByZXR1cm4gdGhpcy5fb2JqZWN0W2tleV0g",
				"PSB2YWx1ZTsKICB9CgogIGZ1bmN0aW9uIGdldChrZXkpIHsKICAgIGlmICh0aGlzLl9vYmplY3Rba2V5",
				"XSAhPT0gT2JqZWN0LnByb3RvdHlwZVtrZXldKQogICAgICByZXR1cm4gdGhpcy5fb2JqZWN0W2tleV07",
				"CiAgfQoKICBmdW5jdGlvbiB1bnNldChrZXkpIHsKICAgIHZhciB2YWx1ZSA9IHRoaXMuX29iamVjdFtr",
				"ZXldOwogICAgZGVsZXRlIHRoaXMuX29iamVjdFtrZXldOwogICAgcmV0dXJuIHZhbHVlOwogIH0KCiAg",
				"ZnVuY3Rpb24gdG9PYmplY3QoKSB7CiAgICByZXR1cm4gT2JqZWN0LmNsb25lKHRoaXMuX29iamVjdCk7",
				"CiAgfQoKCgogIGZ1bmN0aW9uIGtleXMoKSB7CiAgICByZXR1cm4gdGhpcy5wbHVjaygna2V5Jyk7CiAg",
				"fQoKICBmdW5jdGlvbiB2YWx1ZXMoKSB7CiAgICByZXR1cm4gdGhpcy5wbHVjaygndmFsdWUnKTsKICB9",
				"CgogIGZ1bmN0aW9uIGluZGV4KHZhbHVlKSB7CiAgICB2YXIgbWF0Y2ggPSB0aGlzLmRldGVjdChmdW5j",
				"dGlvbihwYWlyKSB7CiAgICAgIHJldHVybiBwYWlyLnZhbHVlID09PSB2YWx1ZTsKICAgIH0pOwogICAg",
				"cmV0dXJuIG1hdGNoICYmIG1hdGNoLmtleTsKICB9CgogIGZ1bmN0aW9uIG1lcmdlKG9iamVjdCkgewog",
				"ICAgcmV0dXJuIHRoaXMuY2xvbmUoKS51cGRhdGUob2JqZWN0KTsKICB9CgogIGZ1bmN0aW9uIHVwZGF0",
				"ZShvYmplY3QpIHsKICAgIHJldHVybiBuZXcgSGFzaChvYmplY3QpLmluamVjdCh0aGlzLCBmdW5jdGlv",
				"bihyZXN1bHQsIHBhaXIpIHsKICAgICAgcmVzdWx0LnNldChwYWlyLmtleSwgcGFpci52YWx1ZSk7CiAg",
				"ICAgIHJldHVybiByZXN1bHQ7CiAgICB9KTsKICB9CgogIGZ1bmN0aW9uIHRvUXVlcnlQYWlyKGtleSwg",
				"dmFsdWUpIHsKICAgIGlmIChPYmplY3QuaXNVbmRlZmluZWQodmFsdWUpKSByZXR1cm4ga2V5OwogICAg",
				"cmV0dXJuIGtleSArICc9JyArIGVuY29kZVVSSUNvbXBvbmVudChTdHJpbmcuaW50ZXJwcmV0KHZhbHVl",
				"KSk7CiAgfQoKICBmdW5jdGlvbiB0b1F1ZXJ5U3RyaW5nKCkgewogICAgcmV0dXJuIHRoaXMuaW5qZWN0",
				"KFtdLCBmdW5jdGlvbihyZXN1bHRzLCBwYWlyKSB7CiAgICAgIHZhciBrZXkgPSBlbmNvZGVVUklDb21w",
				"b25lbnQocGFpci5rZXkpLCB2YWx1ZXMgPSBwYWlyLnZhbHVlOwoKICAgICAgaWYgKHZhbHVlcyAmJiB0",
				"eXBlb2YgdmFsdWVzID09ICdvYmplY3QnKSB7CiAgICAgICAgaWYgKE9iamVjdC5pc0FycmF5KHZhbHVl",
				"cykpIHsKICAgICAgICAgIHZhciBxdWVyeVZhbHVlcyA9IFtdOwogICAgICAgICAgZm9yICh2YXIgaSA9",
				"IDAsIGxlbiA9IHZhbHVlcy5sZW5ndGgsIHZhbHVlOyBpIDwgbGVuOyBpKyspIHsKICAgICAgICAgICAg",
				"dmFsdWUgPSB2YWx1ZXNbaV07CiAgICAgICAgICAgIHF1ZXJ5VmFsdWVzLnB1c2godG9RdWVyeVBhaXIo",
				"a2V5LCB2YWx1ZSkpOwogICAgICAgICAgfQogICAgICAgICAgcmV0dXJuIHJlc3VsdHMuY29uY2F0KHF1",
				"ZXJ5VmFsdWVzKTsKICAgICAgICB9CiAgICAgIH0gZWxzZSByZXN1bHRzLnB1c2godG9RdWVyeVBhaXIo",
				"a2V5LCB2YWx1ZXMpKTsKICAgICAgcmV0dXJuIHJlc3VsdHM7CiAgICB9KS5qb2luKCcmJyk7CiAgfQoK",
				"ICBmdW5jdGlvbiBpbnNwZWN0KCkgewogICAgcmV0dXJuICcjPEhhc2g6eycgKyB0aGlzLm1hcChmdW5j",
				"dGlvbihwYWlyKSB7CiAgICAgIHJldHVybiBwYWlyLm1hcChPYmplY3QuaW5zcGVjdCkuam9pbignOiAn",
				"KTsKICAgIH0pLmpvaW4oJywgJykgKyAnfT4nOwogIH0KCiAgZnVuY3Rpb24gY2xvbmUoKSB7CiAgICBy",
				"ZXR1cm4gbmV3IEhhc2godGhpcyk7CiAgfQoKICByZXR1cm4gewogICAgaW5pdGlhbGl6ZTogICAgICAg",
				"ICAgICAgaW5pdGlhbGl6ZSwKICAgIF9lYWNoOiAgICAgICAgICAgICAgICAgIF9lYWNoLAogICAgc2V0",
				"OiAgICAgICAgICAgICAgICAgICAgc2V0LAogICAgZ2V0OiAgICAgICAgICAgICAgICAgICAgZ2V0LAog",
				"ICAgdW5zZXQ6ICAgICAgICAgICAgICAgICAgdW5zZXQsCiAgICB0b09iamVjdDogICAgICAgICAgICAg",
				"ICB0b09iamVjdCwKICAgIHRvVGVtcGxhdGVSZXBsYWNlbWVudHM6IHRvT2JqZWN0LAogICAga2V5czog",
				"ICAgICAgICAgICAgICAgICAga2V5cywKICAgIHZhbHVlczogICAgICAgICAgICAgICAgIHZhbHVlcywK",
				"ICAgIGluZGV4OiAgICAgICAgICAgICAgICAgIGluZGV4LAogICAgbWVyZ2U6ICAgICAgICAgICAgICAg",
				"ICAgbWVyZ2UsCiAgICB1cGRhdGU6ICAgICAgICAgICAgICAgICB1cGRhdGUsCiAgICB0b1F1ZXJ5U3Ry",
				"aW5nOiAgICAgICAgICB0b1F1ZXJ5U3RyaW5nLAogICAgaW5zcGVjdDogICAgICAgICAgICAgICAgaW5z",
				"cGVjdCwKICAgIHRvSlNPTjogICAgICAgICAgICAgICAgIHRvT2JqZWN0LAogICAgY2xvbmU6ICAgICAg",
				"ICAgICAgICAgICAgY2xvbmUKICB9Owp9KSgpKTsKCkhhc2guZnJvbSA9ICRIOwpPYmplY3QuZXh0ZW5k",
				"KE51bWJlci5wcm90b3R5cGUsIChmdW5jdGlvbigpIHsKICBmdW5jdGlvbiB0b0NvbG9yUGFydCgpIHsK",
				"ICAgIHJldHVybiB0aGlzLnRvUGFkZGVkU3RyaW5nKDIsIDE2KTsKICB9CgogIGZ1bmN0aW9uIHN1Y2Mo",
				"KSB7CiAgICByZXR1cm4gdGhpcyArIDE7CiAgfQoKICBmdW5jdGlvbiB0aW1lcyhpdGVyYXRvciwgY29u",
				"dGV4dCkgewogICAgJFIoMCwgdGhpcywgdHJ1ZSkuZWFjaChpdGVyYXRvciwgY29udGV4dCk7CiAgICBy",
				"ZXR1cm4gdGhpczsKICB9CgogIGZ1bmN0aW9uIHRvUGFkZGVkU3RyaW5nKGxlbmd0aCwgcmFkaXgpIHsK",
				"ICAgIHZhciBzdHJpbmcgPSB0aGlzLnRvU3RyaW5nKHJhZGl4IHx8IDEwKTsKICAgIHJldHVybiAnMCcu",
				"dGltZXMobGVuZ3RoIC0gc3RyaW5nLmxlbmd0aCkgKyBzdHJpbmc7CiAgfQoKICBmdW5jdGlvbiBhYnMo",
				"KSB7CiAgICByZXR1cm4gTWF0aC5hYnModGhpcyk7CiAgfQoKICBmdW5jdGlvbiByb3VuZCgpIHsKICAg",
				"IHJldHVybiBNYXRoLnJvdW5kKHRoaXMpOwogIH0KCiAgZnVuY3Rpb24gY2VpbCgpIHsKICAgIHJldHVy",
				"biBNYXRoLmNlaWwodGhpcyk7CiAgfQoKICBmdW5jdGlvbiBmbG9vcigpIHsKICAgIHJldHVybiBNYXRo",
				"LmZsb29yKHRoaXMpOwogIH0KCiAgcmV0dXJuIHsKICAgIHRvQ29sb3JQYXJ0OiAgICB0b0NvbG9yUGFy",
				"dCwKICAgIHN1Y2M6ICAgICAgICAgICBzdWNjLAogICAgdGltZXM6ICAgICAgICAgIHRpbWVzLAogICAg",
				"dG9QYWRkZWRTdHJpbmc6IHRvUGFkZGVkU3RyaW5nLAogICAgYWJzOiAgICAgICAgICAgIGFicywKICAg",
				"IHJvdW5kOiAgICAgICAgICByb3VuZCwKICAgIGNlaWw6ICAgICAgICAgICBjZWlsLAogICAgZmxvb3I6",
				"ICAgICAgICAgIGZsb29yCiAgfTsKfSkoKSk7CgpmdW5jdGlvbiAkUihzdGFydCwgZW5kLCBleGNsdXNp",
				"dmUpIHsKICByZXR1cm4gbmV3IE9iamVjdFJhbmdlKHN0YXJ0LCBlbmQsIGV4Y2x1c2l2ZSk7Cn0KCnZh",
				"ciBPYmplY3RSYW5nZSA9IENsYXNzLmNyZWF0ZShFbnVtZXJhYmxlLCAoZnVuY3Rpb24oKSB7CiAgZnVu",
				"Y3Rpb24gaW5pdGlhbGl6ZShzdGFydCwgZW5kLCBleGNsdXNpdmUpIHsKICAgIHRoaXMuc3RhcnQgPSBz",
				"dGFydDsKICAgIHRoaXMuZW5kID0gZW5kOwogICAgdGhpcy5leGNsdXNpdmUgPSBleGNsdXNpdmU7CiAg",
				"fQoKICBmdW5jdGlvbiBfZWFjaChpdGVyYXRvcikgewogICAgdmFyIHZhbHVlID0gdGhpcy5zdGFydDsK",
				"ICAgIHdoaWxlICh0aGlzLmluY2x1ZGUodmFsdWUpKSB7CiAgICAgIGl0ZXJhdG9yKHZhbHVlKTsKICAg",
				"ICAgdmFsdWUgPSB2YWx1ZS5zdWNjKCk7CiAgICB9CiAgfQoKICBmdW5jdGlvbiBpbmNsdWRlKHZhbHVl",
				"KSB7CiAgICBpZiAodmFsdWUgPCB0aGlzLnN0YXJ0KQogICAgICByZXR1cm4gZmFsc2U7CiAgICBpZiAo",
				"dGhpcy5leGNsdXNpdmUpCiAgICAgIHJldHVybiB2YWx1ZSA8IHRoaXMuZW5kOwogICAgcmV0dXJuIHZh",
				"bHVlIDw9IHRoaXMuZW5kOwogIH0KCiAgcmV0dXJuIHsKICAgIGluaXRpYWxpemU6IGluaXRpYWxpemUs",
				"CiAgICBfZWFjaDogICAgICBfZWFjaCwKICAgIGluY2x1ZGU6ICAgIGluY2x1ZGUKICB9Owp9KSgpKTsK",
				"CgoKdmFyIEFqYXggPSB7CiAgZ2V0VHJhbnNwb3J0OiBmdW5jdGlvbigpIHsKICAgIHJldHVybiBUcnku",
				"dGhlc2UoCiAgICAgIGZ1bmN0aW9uKCkge3JldHVybiBuZXcgWE1MSHR0cFJlcXVlc3QoKX0sCiAgICAg",
				"IGZ1bmN0aW9uKCkge3JldHVybiBuZXcgQWN0aXZlWE9iamVjdCgnTXN4bWwyLlhNTEhUVFAnKX0sCiAg",
				"ICAgIGZ1bmN0aW9uKCkge3JldHVybiBuZXcgQWN0aXZlWE9iamVjdCgnTWljcm9zb2Z0LlhNTEhUVFAn",
				"KX0KICAgICkgfHwgZmFsc2U7CiAgfSwKCiAgYWN0aXZlUmVxdWVzdENvdW50OiAwCn07CgpBamF4LlJl",
				"c3BvbmRlcnMgPSB7CiAgcmVzcG9uZGVyczogW10sCgogIF9lYWNoOiBmdW5jdGlvbihpdGVyYXRvcikg",
				"ewogICAgdGhpcy5yZXNwb25kZXJzLl9lYWNoKGl0ZXJhdG9yKTsKICB9LAoKICByZWdpc3RlcjogZnVu",
				"Y3Rpb24ocmVzcG9uZGVyKSB7CiAgICBpZiAoIXRoaXMuaW5jbHVkZShyZXNwb25kZXIpKQogICAgICB0",
				"aGlzLnJlc3BvbmRlcnMucHVzaChyZXNwb25kZXIpOwogIH0sCgogIHVucmVnaXN0ZXI6IGZ1bmN0aW9u",
				"KHJlc3BvbmRlcikgewogICAgdGhpcy5yZXNwb25kZXJzID0gdGhpcy5yZXNwb25kZXJzLndpdGhvdXQo",
				"cmVzcG9uZGVyKTsKICB9LAoKICBkaXNwYXRjaDogZnVuY3Rpb24oY2FsbGJhY2ssIHJlcXVlc3QsIHRy",
				"YW5zcG9ydCwganNvbikgewogICAgdGhpcy5lYWNoKGZ1bmN0aW9uKHJlc3BvbmRlcikgewogICAgICBp",
				"ZiAoT2JqZWN0LmlzRnVuY3Rpb24ocmVzcG9uZGVyW2NhbGxiYWNrXSkpIHsKICAgICAgICB0cnkgewog",
				"ICAgICAgICAgcmVzcG9uZGVyW2NhbGxiYWNrXS5hcHBseShyZXNwb25kZXIsIFtyZXF1ZXN0LCB0cmFu",
				"c3BvcnQsIGpzb25dKTsKICAgICAgICB9IGNhdGNoIChlKSB7IH0KICAgICAgfQogICAgfSk7CiAgfQp9",
				"OwoKT2JqZWN0LmV4dGVuZChBamF4LlJlc3BvbmRlcnMsIEVudW1lcmFibGUpOwoKQWpheC5SZXNwb25k",
				"ZXJzLnJlZ2lzdGVyKHsKICBvbkNyZWF0ZTogICBmdW5jdGlvbigpIHsgQWpheC5hY3RpdmVSZXF1ZXN0",
				"Q291bnQrKyB9LAogIG9uQ29tcGxldGU6IGZ1bmN0aW9uKCkgeyBBamF4LmFjdGl2ZVJlcXVlc3RDb3Vu",
				"dC0tIH0KfSk7CkFqYXguQmFzZSA9IENsYXNzLmNyZWF0ZSh7CiAgaW5pdGlhbGl6ZTogZnVuY3Rpb24o",
				"b3B0aW9ucykgewogICAgdGhpcy5vcHRpb25zID0gewogICAgICBtZXRob2Q6ICAgICAgICdwb3N0JywK",
				"ICAgICAgYXN5bmNocm9ub3VzOiB0cnVlLAogICAgICBjb250ZW50VHlwZTogICdhcHBsaWNhdGlvbi94",
				"LXd3dy1mb3JtLXVybGVuY29kZWQnLAogICAgICBlbmNvZGluZzogICAgICdVVEYtOCcsCiAgICAgIHBh",
				"cmFtZXRlcnM6ICAgJycsCiAgICAgIGV2YWxKU09OOiAgICAgdHJ1ZSwKICAgICAgZXZhbEpTOiAgICAg",
				"ICB0cnVlCiAgICB9OwogICAgT2JqZWN0LmV4dGVuZCh0aGlzLm9wdGlvbnMsIG9wdGlvbnMgfHwgeyB9",
				"KTsKCiAgICB0aGlzLm9wdGlvbnMubWV0aG9kID0gdGhpcy5vcHRpb25zLm1ldGhvZC50b0xvd2VyQ2Fz",
				"ZSgpOwoKICAgIGlmIChPYmplY3QuaXNIYXNoKHRoaXMub3B0aW9ucy5wYXJhbWV0ZXJzKSkKICAgICAg",
				"dGhpcy5vcHRpb25zLnBhcmFtZXRlcnMgPSB0aGlzLm9wdGlvbnMucGFyYW1ldGVycy50b09iamVjdCgp",
				"OwogIH0KfSk7CkFqYXguUmVxdWVzdCA9IENsYXNzLmNyZWF0ZShBamF4LkJhc2UsIHsKICBfY29tcGxl",
				"dGU6IGZhbHNlLAoKICBpbml0aWFsaXplOiBmdW5jdGlvbigkc3VwZXIsIHVybCwgb3B0aW9ucykgewog",
				"ICAgJHN1cGVyKG9wdGlvbnMpOwogICAgdGhpcy50cmFuc3BvcnQgPSBBamF4LmdldFRyYW5zcG9ydCgp",
				"OwogICAgdGhpcy5yZXF1ZXN0KHVybCk7CiAgfSwKCiAgcmVxdWVzdDogZnVuY3Rpb24odXJsKSB7CiAg",
				"ICB0aGlzLnVybCA9IHVybDsKICAgIHRoaXMubWV0aG9kID0gdGhpcy5vcHRpb25zLm1ldGhvZDsKICAg",
				"IHZhciBwYXJhbXMgPSBPYmplY3QuaXNTdHJpbmcodGhpcy5vcHRpb25zLnBhcmFtZXRlcnMpID8KICAg",
				"ICAgICAgIHRoaXMub3B0aW9ucy5wYXJhbWV0ZXJzIDoKICAgICAgICAgIE9iamVjdC50b1F1ZXJ5U3Ry",
				"aW5nKHRoaXMub3B0aW9ucy5wYXJhbWV0ZXJzKTsKCiAgICBpZiAoIVsnZ2V0JywgJ3Bvc3QnXS5pbmNs",
				"dWRlKHRoaXMubWV0aG9kKSkgewogICAgICBwYXJhbXMgKz0gKHBhcmFtcyA/ICcmJyA6ICcnKSArICJf",
				"bWV0aG9kPSIgKyB0aGlzLm1ldGhvZDsKICAgICAgdGhpcy5tZXRob2QgPSAncG9zdCc7CiAgICB9Cgog",
				"ICAgaWYgKHBhcmFtcyAmJiB0aGlzLm1ldGhvZCA9PT0gJ2dldCcpIHsKICAgICAgdGhpcy51cmwgKz0g",
				"KHRoaXMudXJsLmluY2x1ZGUoJz8nKSA/ICcmJyA6ICc/JykgKyBwYXJhbXM7CiAgICB9CgogICAgdGhp",
				"cy5wYXJhbWV0ZXJzID0gcGFyYW1zLnRvUXVlcnlQYXJhbXMoKTsKCiAgICB0cnkgewogICAgICB2YXIg",
				"cmVzcG9uc2UgPSBuZXcgQWpheC5SZXNwb25zZSh0aGlzKTsKICAgICAgaWYgKHRoaXMub3B0aW9ucy5v",
				"bkNyZWF0ZSkgdGhpcy5vcHRpb25zLm9uQ3JlYXRlKHJlc3BvbnNlKTsKICAgICAgQWpheC5SZXNwb25k",
				"ZXJzLmRpc3BhdGNoKCdvbkNyZWF0ZScsIHRoaXMsIHJlc3BvbnNlKTsKCiAgICAgIHRoaXMudHJhbnNw",
				"b3J0Lm9wZW4odGhpcy5tZXRob2QudG9VcHBlckNhc2UoKSwgdGhpcy51cmwsCiAgICAgICAgdGhpcy5v",
				"cHRpb25zLmFzeW5jaHJvbm91cyk7CgogICAgICBpZiAodGhpcy5vcHRpb25zLmFzeW5jaHJvbm91cykg",
				"dGhpcy5yZXNwb25kVG9SZWFkeVN0YXRlLmJpbmQodGhpcykuZGVmZXIoMSk7CgogICAgICB0aGlzLnRy",
				"YW5zcG9ydC5vbnJlYWR5c3RhdGVjaGFuZ2UgPSB0aGlzLm9uU3RhdGVDaGFuZ2UuYmluZCh0aGlzKTsK",
				"ICAgICAgdGhpcy5zZXRSZXF1ZXN0SGVhZGVycygpOwoKICAgICAgdGhpcy5ib2R5ID0gdGhpcy5tZXRo",
				"b2QgPT0gJ3Bvc3QnID8gKHRoaXMub3B0aW9ucy5wb3N0Qm9keSB8fCBwYXJhbXMpIDogbnVsbDsKICAg",
				"ICAgdGhpcy50cmFuc3BvcnQuc2VuZCh0aGlzLmJvZHkpOwoKICAgICAgLyogRm9yY2UgRmlyZWZveCB0",
				"byBoYW5kbGUgcmVhZHkgc3RhdGUgNCBmb3Igc3luY2hyb25vdXMgcmVxdWVzdHMgKi8KICAgICAgaWYg",
				"KCF0aGlzLm9wdGlvbnMuYXN5bmNocm9ub3VzICYmIHRoaXMudHJhbnNwb3J0Lm92ZXJyaWRlTWltZVR5",
				"cGUpCiAgICAgICAgdGhpcy5vblN0YXRlQ2hhbmdlKCk7CgogICAgfQogICAgY2F0Y2ggKGUpIHsKICAg",
				"ICAgdGhpcy5kaXNwYXRjaEV4Y2VwdGlvbihlKTsKICAgIH0KICB9LAoKICBvblN0YXRlQ2hhbmdlOiBm",
				"dW5jdGlvbigpIHsKICAgIHZhciByZWFkeVN0YXRlID0gdGhpcy50cmFuc3BvcnQucmVhZHlTdGF0ZTsK",
				"ICAgIGlmIChyZWFkeVN0YXRlID4gMSAmJiAhKChyZWFkeVN0YXRlID09IDQpICYmIHRoaXMuX2NvbXBs",
				"ZXRlKSkKICAgICAgdGhpcy5yZXNwb25kVG9SZWFkeVN0YXRlKHRoaXMudHJhbnNwb3J0LnJlYWR5U3Rh",
				"dGUpOwogIH0sCgogIHNldFJlcXVlc3RIZWFkZXJzOiBmdW5jdGlvbigpIHsKICAgIHZhciBoZWFkZXJz",
				"ID0gewogICAgICAnWC1SZXF1ZXN0ZWQtV2l0aCc6ICdYTUxIdHRwUmVxdWVzdCcsCiAgICAgICdYLVBy",
				"b3RvdHlwZS1WZXJzaW9uJzogUHJvdG90eXBlLlZlcnNpb24sCiAgICAgICdBY2NlcHQnOiAndGV4dC9q",
				"YXZhc2NyaXB0LCB0ZXh0L2h0bWwsIGFwcGxpY2F0aW9uL3htbCwgdGV4dC94bWwsICovKicKICAgIH07",
				"CgogICAgaWYgKHRoaXMubWV0aG9kID09ICdwb3N0JykgewogICAgICBoZWFkZXJzWydDb250ZW50LXR5",
				"cGUnXSA9IHRoaXMub3B0aW9ucy5jb250ZW50VHlwZSArCiAgICAgICAgKHRoaXMub3B0aW9ucy5lbmNv",
				"ZGluZyA/ICc7IGNoYXJzZXQ9JyArIHRoaXMub3B0aW9ucy5lbmNvZGluZyA6ICcnKTsKCiAgICAgIC8q",
				"IEZvcmNlICJDb25uZWN0aW9uOiBjbG9zZSIgZm9yIG9sZGVyIE1vemlsbGEgYnJvd3NlcnMgdG8gd29y",
				"awogICAgICAgKiBhcm91bmQgYSBidWcgd2hlcmUgWE1MSHR0cFJlcXVlc3Qgc2VuZHMgYW4gaW5jb3Jy",
				"ZWN0CiAgICAgICAqIENvbnRlbnQtbGVuZ3RoIGhlYWRlci4gU2VlIE1vemlsbGEgQnVnemlsbGEgIzI0",
				"NjY1MS4KICAgICAgICovCiAgICAgIGlmICh0aGlzLnRyYW5zcG9ydC5vdmVycmlkZU1pbWVUeXBlICYm",
				"CiAgICAgICAgICAobmF2aWdhdG9yLnVzZXJBZ2VudC5tYXRjaCgvR2Vja29cLyhcZHs0fSkvKSB8fCBb",
				"MCwyMDA1XSlbMV0gPCAyMDA1KQogICAgICAgICAgICBoZWFkZXJzWydDb25uZWN0aW9uJ10gPSAnY2xv",
				"c2UnOwogICAgfQoKICAgIGlmICh0eXBlb2YgdGhpcy5vcHRpb25zLnJlcXVlc3RIZWFkZXJzID09ICdv",
				"YmplY3QnKSB7CiAgICAgIHZhciBleHRyYXMgPSB0aGlzLm9wdGlvbnMucmVxdWVzdEhlYWRlcnM7Cgog",
				"ICAgICBpZiAoT2JqZWN0LmlzRnVuY3Rpb24oZXh0cmFzLnB1c2gpKQogICAgICAgIGZvciAodmFyIGkg",
				"PSAwLCBsZW5ndGggPSBleHRyYXMubGVuZ3RoOyBpIDwgbGVuZ3RoOyBpICs9IDIpCiAgICAgICAgICBo",
				"ZWFkZXJzW2V4dHJhc1tpXV0gPSBleHRyYXNbaSsxXTsKICAgICAgZWxzZQogICAgICAgICRIKGV4dHJh",
				"cykuZWFjaChmdW5jdGlvbihwYWlyKSB7IGhlYWRlcnNbcGFpci5rZXldID0gcGFpci52YWx1ZSB9KTsK",
				"ICAgIH0KCiAgICBmb3IgKHZhciBuYW1lIGluIGhlYWRlcnMpCiAgICAgIHRoaXMudHJhbnNwb3J0LnNl",
				"dFJlcXVlc3RIZWFkZXIobmFtZSwgaGVhZGVyc1tuYW1lXSk7CiAgfSwKCiAgc3VjY2VzczogZnVuY3Rp",
				"b24oKSB7CiAgICB2YXIgc3RhdHVzID0gdGhpcy5nZXRTdGF0dXMoKTsKICAgIHJldHVybiAhc3RhdHVz",
				"IHx8IChzdGF0dXMgPj0gMjAwICYmIHN0YXR1cyA8IDMwMCkgfHwgc3RhdHVzID09IDMwNDsKICB9LAoK",
				"ICBnZXRTdGF0dXM6IGZ1bmN0aW9uKCkgewogICAgdHJ5IHsKICAgICAgaWYgKHRoaXMudHJhbnNwb3J0",
				"LnN0YXR1cyA9PT0gMTIyMykgcmV0dXJuIDIwNDsKICAgICAgcmV0dXJuIHRoaXMudHJhbnNwb3J0LnN0",
				"YXR1cyB8fCAwOwogICAgfSBjYXRjaCAoZSkgeyByZXR1cm4gMCB9CiAgfSwKCiAgcmVzcG9uZFRvUmVh",
				"ZHlTdGF0ZTogZnVuY3Rpb24ocmVhZHlTdGF0ZSkgewogICAgdmFyIHN0YXRlID0gQWpheC5SZXF1ZXN0",
				"LkV2ZW50c1tyZWFkeVN0YXRlXSwgcmVzcG9uc2UgPSBuZXcgQWpheC5SZXNwb25zZSh0aGlzKTsKCiAg",
				"ICBpZiAoc3RhdGUgPT0gJ0NvbXBsZXRlJykgewogICAgICB0cnkgewogICAgICAgIHRoaXMuX2NvbXBs",
				"ZXRlID0gdHJ1ZTsKICAgICAgICAodGhpcy5vcHRpb25zWydvbicgKyByZXNwb25zZS5zdGF0dXNdCiAg",
				"ICAgICAgIHx8IHRoaXMub3B0aW9uc1snb24nICsgKHRoaXMuc3VjY2VzcygpID8gJ1N1Y2Nlc3MnIDog",
				"J0ZhaWx1cmUnKV0KICAgICAgICAgfHwgUHJvdG90eXBlLmVtcHR5RnVuY3Rpb24pKHJlc3BvbnNlLCBy",
				"ZXNwb25zZS5oZWFkZXJKU09OKTsKICAgICAgfSBjYXRjaCAoZSkgewogICAgICAgIHRoaXMuZGlzcGF0",
				"Y2hFeGNlcHRpb24oZSk7CiAgICAgIH0KCiAgICAgIHZhciBjb250ZW50VHlwZSA9IHJlc3BvbnNlLmdl",
				"dEhlYWRlcignQ29udGVudC10eXBlJyk7CiAgICAgIGlmICh0aGlzLm9wdGlvbnMuZXZhbEpTID09ICdm",
				"b3JjZScKICAgICAgICAgIHx8ICh0aGlzLm9wdGlvbnMuZXZhbEpTICYmIHRoaXMuaXNTYW1lT3JpZ2lu",
				"KCkgJiYgY29udGVudFR5cGUKICAgICAgICAgICYmIGNvbnRlbnRUeXBlLm1hdGNoKC9eXHMqKHRleHR8",
				"YXBwbGljYXRpb24pXC8oeC0pPyhqYXZhfGVjbWEpc2NyaXB0KDsuKik/XHMqJC9pKSkpCiAgICAgICAg",
				"dGhpcy5ldmFsUmVzcG9uc2UoKTsKICAgIH0KCiAgICB0cnkgewogICAgICAodGhpcy5vcHRpb25zWydv",
				"bicgKyBzdGF0ZV0gfHwgUHJvdG90eXBlLmVtcHR5RnVuY3Rpb24pKHJlc3BvbnNlLCByZXNwb25zZS5o",
				"ZWFkZXJKU09OKTsKICAgICAgQWpheC5SZXNwb25kZXJzLmRpc3BhdGNoKCdvbicgKyBzdGF0ZSwgdGhp",
				"cywgcmVzcG9uc2UsIHJlc3BvbnNlLmhlYWRlckpTT04pOwogICAgfSBjYXRjaCAoZSkgewogICAgICB0",
				"aGlzLmRpc3BhdGNoRXhjZXB0aW9uKGUpOwogICAgfQoKICAgIGlmIChzdGF0ZSA9PSAnQ29tcGxldGUn",
				"KSB7CiAgICAgIHRoaXMudHJhbnNwb3J0Lm9ucmVhZHlzdGF0ZWNoYW5nZSA9IFByb3RvdHlwZS5lbXB0",
				"eUZ1bmN0aW9uOwogICAgfQogIH0sCgogIGlzU2FtZU9yaWdpbjogZnVuY3Rpb24oKSB7CiAgICB2YXIg",
				"bSA9IHRoaXMudXJsLm1hdGNoKC9eXHMqaHR0cHM/OlwvXC9bXlwvXSovKTsKICAgIHJldHVybiAhbSB8",
				"fCAobVswXSA9PSAnI3twcm90b2NvbH0vLyN7ZG9tYWlufSN7cG9ydH0nLmludGVycG9sYXRlKHsKICAg",
				"ICAgcHJvdG9jb2w6IGxvY2F0aW9uLnByb3RvY29sLAogICAgICBkb21haW46IGRvY3VtZW50LmRvbWFp",
				"biwKICAgICAgcG9ydDogbG9jYXRpb24ucG9ydCA/ICc6JyArIGxvY2F0aW9uLnBvcnQgOiAnJwogICAg",
				"fSkpOwogIH0sCgogIGdldEhlYWRlcjogZnVuY3Rpb24obmFtZSkgewogICAgdHJ5IHsKICAgICAgcmV0",
				"dXJuIHRoaXMudHJhbnNwb3J0LmdldFJlc3BvbnNlSGVhZGVyKG5hbWUpIHx8IG51bGw7CiAgICB9IGNh",
				"dGNoIChlKSB7IHJldHVybiBudWxsOyB9CiAgfSwKCiAgZXZhbFJlc3BvbnNlOiBmdW5jdGlvbigpIHsK",
				"ICAgIHRyeSB7CiAgICAgIHJldHVybiBldmFsKCh0aGlzLnRyYW5zcG9ydC5yZXNwb25zZVRleHQgfHwg",
				"JycpLnVuZmlsdGVySlNPTigpKTsKICAgIH0gY2F0Y2ggKGUpIHsKICAgICAgdGhpcy5kaXNwYXRjaEV4",
				"Y2VwdGlvbihlKTsKICAgIH0KICB9LAoKICBkaXNwYXRjaEV4Y2VwdGlvbjogZnVuY3Rpb24oZXhjZXB0",
				"aW9uKSB7CiAgICAodGhpcy5vcHRpb25zLm9uRXhjZXB0aW9uIHx8IFByb3RvdHlwZS5lbXB0eUZ1bmN0",
				"aW9uKSh0aGlzLCBleGNlcHRpb24pOwogICAgQWpheC5SZXNwb25kZXJzLmRpc3BhdGNoKCdvbkV4Y2Vw",
				"dGlvbicsIHRoaXMsIGV4Y2VwdGlvbik7CiAgfQp9KTsKCkFqYXguUmVxdWVzdC5FdmVudHMgPQogIFsn",
				"VW5pbml0aWFsaXplZCcsICdMb2FkaW5nJywgJ0xvYWRlZCcsICdJbnRlcmFjdGl2ZScsICdDb21wbGV0",
				"ZSddOwoKCgoKCgoKCkFqYXguUmVzcG9uc2UgPSBDbGFzcy5jcmVhdGUoewogIGluaXRpYWxpemU6IGZ1",
				"bmN0aW9uKHJlcXVlc3QpewogICAgdGhpcy5yZXF1ZXN0ID0gcmVxdWVzdDsKICAgIHZhciB0cmFuc3Bv",
				"cnQgID0gdGhpcy50cmFuc3BvcnQgID0gcmVxdWVzdC50cmFuc3BvcnQsCiAgICAgICAgcmVhZHlTdGF0",
				"ZSA9IHRoaXMucmVhZHlTdGF0ZSA9IHRyYW5zcG9ydC5yZWFkeVN0YXRlOwoKICAgIGlmICgocmVhZHlT",
				"dGF0ZSA+IDIgJiYgIVByb3RvdHlwZS5Ccm93c2VyLklFKSB8fCByZWFkeVN0YXRlID09IDQpIHsKICAg",
				"ICAgdGhpcy5zdGF0dXMgICAgICAgPSB0aGlzLmdldFN0YXR1cygpOwogICAgICB0aGlzLnN0YXR1c1Rl",
				"eHQgICA9IHRoaXMuZ2V0U3RhdHVzVGV4dCgpOwogICAgICB0aGlzLnJlc3BvbnNlVGV4dCA9IFN0cmlu",
				"Zy5pbnRlcnByZXQodHJhbnNwb3J0LnJlc3BvbnNlVGV4dCk7CiAgICAgIHRoaXMuaGVhZGVySlNPTiAg",
				"ID0gdGhpcy5fZ2V0SGVhZGVySlNPTigpOwogICAgfQoKICAgIGlmIChyZWFkeVN0YXRlID09IDQpIHsK",
				"ICAgICAgdmFyIHhtbCA9IHRyYW5zcG9ydC5yZXNwb25zZVhNTDsKICAgICAgdGhpcy5yZXNwb25zZVhN",
				"TCAgPSBPYmplY3QuaXNVbmRlZmluZWQoeG1sKSA/IG51bGwgOiB4bWw7CiAgICAgIHRoaXMucmVzcG9u",
				"c2VKU09OID0gdGhpcy5fZ2V0UmVzcG9uc2VKU09OKCk7CiAgICB9CiAgfSwKCiAgc3RhdHVzOiAgICAg",
				"IDAsCgogIHN0YXR1c1RleHQ6ICcnLAoKICBnZXRTdGF0dXM6IEFqYXguUmVxdWVzdC5wcm90b3R5cGUu",
				"Z2V0U3RhdHVzLAoKICBnZXRTdGF0dXNUZXh0OiBmdW5jdGlvbigpIHsKICAgIHRyeSB7CiAgICAgIHJl",
				"dHVybiB0aGlzLnRyYW5zcG9ydC5zdGF0dXNUZXh0IHx8ICcnOwogICAgfSBjYXRjaCAoZSkgeyByZXR1",
				"cm4gJycgfQogIH0sCgogIGdldEhlYWRlcjogQWpheC5SZXF1ZXN0LnByb3RvdHlwZS5nZXRIZWFkZXIs",
				"CgogIGdldEFsbEhlYWRlcnM6IGZ1bmN0aW9uKCkgewogICAgdHJ5IHsKICAgICAgcmV0dXJuIHRoaXMu",
				"Z2V0QWxsUmVzcG9uc2VIZWFkZXJzKCk7CiAgICB9IGNhdGNoIChlKSB7IHJldHVybiBudWxsIH0KICB9",
				"LAoKICBnZXRSZXNwb25zZUhlYWRlcjogZnVuY3Rpb24obmFtZSkgewogICAgcmV0dXJuIHRoaXMudHJh",
				"bnNwb3J0LmdldFJlc3BvbnNlSGVhZGVyKG5hbWUpOwogIH0sCgogIGdldEFsbFJlc3BvbnNlSGVhZGVy",
				"czogZnVuY3Rpb24oKSB7CiAgICByZXR1cm4gdGhpcy50cmFuc3BvcnQuZ2V0QWxsUmVzcG9uc2VIZWFk",
				"ZXJzKCk7CiAgfSwKCiAgX2dldEhlYWRlckpTT046IGZ1bmN0aW9uKCkgewogICAgdmFyIGpzb24gPSB0",
				"aGlzLmdldEhlYWRlcignWC1KU09OJyk7CiAgICBpZiAoIWpzb24pIHJldHVybiBudWxsOwogICAganNv",
				"biA9IGRlY29kZVVSSUNvbXBvbmVudChlc2NhcGUoanNvbikpOwogICAgdHJ5IHsKICAgICAgcmV0dXJu",
				"IGpzb24uZXZhbEpTT04odGhpcy5yZXF1ZXN0Lm9wdGlvbnMuc2FuaXRpemVKU09OIHx8CiAgICAgICAg",
				"IXRoaXMucmVxdWVzdC5pc1NhbWVPcmlnaW4oKSk7CiAgICB9IGNhdGNoIChlKSB7CiAgICAgIHRoaXMu",
				"cmVxdWVzdC5kaXNwYXRjaEV4Y2VwdGlvbihlKTsKICAgIH0KICB9LAoKICBfZ2V0UmVzcG9uc2VKU09O",
				"OiBmdW5jdGlvbigpIHsKICAgIHZhciBvcHRpb25zID0gdGhpcy5yZXF1ZXN0Lm9wdGlvbnM7CiAgICBp",
				"ZiAoIW9wdGlvbnMuZXZhbEpTT04gfHwgKG9wdGlvbnMuZXZhbEpTT04gIT0gJ2ZvcmNlJyAmJgogICAg",
				"ICAhKHRoaXMuZ2V0SGVhZGVyKCdDb250ZW50LXR5cGUnKSB8fCAnJykuaW5jbHVkZSgnYXBwbGljYXRp",
				"b24vanNvbicpKSB8fAogICAgICAgIHRoaXMucmVzcG9uc2VUZXh0LmJsYW5rKCkpCiAgICAgICAgICBy",
				"ZXR1cm4gbnVsbDsKICAgIHRyeSB7CiAgICAgIHJldHVybiB0aGlzLnJlc3BvbnNlVGV4dC5ldmFsSlNP",
				"TihvcHRpb25zLnNhbml0aXplSlNPTiB8fAogICAgICAgICF0aGlzLnJlcXVlc3QuaXNTYW1lT3JpZ2lu",
				"KCkpOwogICAgfSBjYXRjaCAoZSkgewogICAgICB0aGlzLnJlcXVlc3QuZGlzcGF0Y2hFeGNlcHRpb24o",
				"ZSk7CiAgICB9CiAgfQp9KTsKCkFqYXguVXBkYXRlciA9IENsYXNzLmNyZWF0ZShBamF4LlJlcXVlc3Qs",
				"IHsKICBpbml0aWFsaXplOiBmdW5jdGlvbigkc3VwZXIsIGNvbnRhaW5lciwgdXJsLCBvcHRpb25zKSB7",
				"CiAgICB0aGlzLmNvbnRhaW5lciA9IHsKICAgICAgc3VjY2VzczogKGNvbnRhaW5lci5zdWNjZXNzIHx8",
				"IGNvbnRhaW5lciksCiAgICAgIGZhaWx1cmU6IChjb250YWluZXIuZmFpbHVyZSB8fCAoY29udGFpbmVy",
				"LnN1Y2Nlc3MgPyBudWxsIDogY29udGFpbmVyKSkKICAgIH07CgogICAgb3B0aW9ucyA9IE9iamVjdC5j",
				"bG9uZShvcHRpb25zKTsKICAgIHZhciBvbkNvbXBsZXRlID0gb3B0aW9ucy5vbkNvbXBsZXRlOwogICAg",
				"b3B0aW9ucy5vbkNvbXBsZXRlID0gKGZ1bmN0aW9uKHJlc3BvbnNlLCBqc29uKSB7CiAgICAgIHRoaXMu",
				"dXBkYXRlQ29udGVudChyZXNwb25zZS5yZXNwb25zZVRleHQpOwogICAgICBpZiAoT2JqZWN0LmlzRnVu",
				"Y3Rpb24ob25Db21wbGV0ZSkpIG9uQ29tcGxldGUocmVzcG9uc2UsIGpzb24pOwogICAgfSkuYmluZCh0",
				"aGlzKTsKCiAgICAkc3VwZXIodXJsLCBvcHRpb25zKTsKICB9LAoKICB1cGRhdGVDb250ZW50OiBmdW5j",
				"dGlvbihyZXNwb25zZVRleHQpIHsKICAgIHZhciByZWNlaXZlciA9IHRoaXMuY29udGFpbmVyW3RoaXMu",
				"c3VjY2VzcygpID8gJ3N1Y2Nlc3MnIDogJ2ZhaWx1cmUnXSwKICAgICAgICBvcHRpb25zID0gdGhpcy5v",
				"cHRpb25zOwoKICAgIGlmICghb3B0aW9ucy5ldmFsU2NyaXB0cykgcmVzcG9uc2VUZXh0ID0gcmVzcG9u",
				"c2VUZXh0LnN0cmlwU2NyaXB0cygpOwoKICAgIGlmIChyZWNlaXZlciA9ICQocmVjZWl2ZXIpKSB7CiAg",
				"ICAgIGlmIChvcHRpb25zLmluc2VydGlvbikgewogICAgICAgIGlmIChPYmplY3QuaXNTdHJpbmcob3B0",
				"aW9ucy5pbnNlcnRpb24pKSB7CiAgICAgICAgICB2YXIgaW5zZXJ0aW9uID0geyB9OyBpbnNlcnRpb25b",
				"b3B0aW9ucy5pbnNlcnRpb25dID0gcmVzcG9uc2VUZXh0OwogICAgICAgICAgcmVjZWl2ZXIuaW5zZXJ0",
				"KGluc2VydGlvbik7CiAgICAgICAgfQogICAgICAgIGVsc2Ugb3B0aW9ucy5pbnNlcnRpb24ocmVjZWl2",
				"ZXIsIHJlc3BvbnNlVGV4dCk7CiAgICAgIH0KICAgICAgZWxzZSByZWNlaXZlci51cGRhdGUocmVzcG9u",
				"c2VUZXh0KTsKICAgIH0KICB9Cn0pOwoKQWpheC5QZXJpb2RpY2FsVXBkYXRlciA9IENsYXNzLmNyZWF0",
				"ZShBamF4LkJhc2UsIHsKICBpbml0aWFsaXplOiBmdW5jdGlvbigkc3VwZXIsIGNvbnRhaW5lciwgdXJs",
				"LCBvcHRpb25zKSB7CiAgICAkc3VwZXIob3B0aW9ucyk7CiAgICB0aGlzLm9uQ29tcGxldGUgPSB0aGlz",
				"Lm9wdGlvbnMub25Db21wbGV0ZTsKCiAgICB0aGlzLmZyZXF1ZW5jeSA9ICh0aGlzLm9wdGlvbnMuZnJl",
				"cXVlbmN5IHx8IDIpOwogICAgdGhpcy5kZWNheSA9ICh0aGlzLm9wdGlvbnMuZGVjYXkgfHwgMSk7Cgog",
				"ICAgdGhpcy51cGRhdGVyID0geyB9OwogICAgdGhpcy5jb250YWluZXIgPSBjb250YWluZXI7CiAgICB0",
				"aGlzLnVybCA9IHVybDsKCiAgICB0aGlzLnN0YXJ0KCk7CiAgfSwKCiAgc3RhcnQ6IGZ1bmN0aW9uKCkg",
				"ewogICAgdGhpcy5vcHRpb25zLm9uQ29tcGxldGUgPSB0aGlzLnVwZGF0ZUNvbXBsZXRlLmJpbmQodGhp",
				"cyk7CiAgICB0aGlzLm9uVGltZXJFdmVudCgpOwogIH0sCgogIHN0b3A6IGZ1bmN0aW9uKCkgewogICAg",
				"dGhpcy51cGRhdGVyLm9wdGlvbnMub25Db21wbGV0ZSA9IHVuZGVmaW5lZDsKICAgIGNsZWFyVGltZW91",
				"dCh0aGlzLnRpbWVyKTsKICAgICh0aGlzLm9uQ29tcGxldGUgfHwgUHJvdG90eXBlLmVtcHR5RnVuY3Rp",
				"b24pLmFwcGx5KHRoaXMsIGFyZ3VtZW50cyk7CiAgfSwKCiAgdXBkYXRlQ29tcGxldGU6IGZ1bmN0aW9u",
				"KHJlc3BvbnNlKSB7CiAgICBpZiAodGhpcy5vcHRpb25zLmRlY2F5KSB7CiAgICAgIHRoaXMuZGVjYXkg",
				"PSAocmVzcG9uc2UucmVzcG9uc2VUZXh0ID09IHRoaXMubGFzdFRleHQgPwogICAgICAgIHRoaXMuZGVj",
				"YXkgKiB0aGlzLm9wdGlvbnMuZGVjYXkgOiAxKTsKCiAgICAgIHRoaXMubGFzdFRleHQgPSByZXNwb25z",
				"ZS5yZXNwb25zZVRleHQ7CiAgICB9CiAgICB0aGlzLnRpbWVyID0gdGhpcy5vblRpbWVyRXZlbnQuYmlu",
				"ZCh0aGlzKS5kZWxheSh0aGlzLmRlY2F5ICogdGhpcy5mcmVxdWVuY3kpOwogIH0sCgogIG9uVGltZXJF",
				"dmVudDogZnVuY3Rpb24oKSB7CiAgICB0aGlzLnVwZGF0ZXIgPSBuZXcgQWpheC5VcGRhdGVyKHRoaXMu",
				"Y29udGFpbmVyLCB0aGlzLnVybCwgdGhpcy5vcHRpb25zKTsKICB9Cn0pOwoKCmZ1bmN0aW9uICQoZWxl",
				"bWVudCkgewogIGlmIChhcmd1bWVudHMubGVuZ3RoID4gMSkgewogICAgZm9yICh2YXIgaSA9IDAsIGVs",
				"ZW1lbnRzID0gW10sIGxlbmd0aCA9IGFyZ3VtZW50cy5sZW5ndGg7IGkgPCBsZW5ndGg7IGkrKykKICAg",
				"ICAgZWxlbWVudHMucHVzaCgkKGFyZ3VtZW50c1tpXSkpOwogICAgcmV0dXJuIGVsZW1lbnRzOwogIH0K",
				"ICBpZiAoT2JqZWN0LmlzU3RyaW5nKGVsZW1lbnQpKQogICAgZWxlbWVudCA9IGRvY3VtZW50LmdldEVs",
				"ZW1lbnRCeUlkKGVsZW1lbnQpOwogIHJldHVybiBFbGVtZW50LmV4dGVuZChlbGVtZW50KTsKfQoKaWYg",
				"KFByb3RvdHlwZS5Ccm93c2VyRmVhdHVyZXMuWFBhdGgpIHsKICBkb2N1bWVudC5fZ2V0RWxlbWVudHNC",
				"eVhQYXRoID0gZnVuY3Rpb24oZXhwcmVzc2lvbiwgcGFyZW50RWxlbWVudCkgewogICAgdmFyIHJlc3Vs",
				"dHMgPSBbXTsKICAgIHZhciBxdWVyeSA9IGRvY3VtZW50LmV2YWx1YXRlKGV4cHJlc3Npb24sICQocGFy",
				"ZW50RWxlbWVudCkgfHwgZG9jdW1lbnQsCiAgICAgIG51bGwsIFhQYXRoUmVzdWx0Lk9SREVSRURfTk9E",
				"RV9TTkFQU0hPVF9UWVBFLCBudWxsKTsKICAgIGZvciAodmFyIGkgPSAwLCBsZW5ndGggPSBxdWVyeS5z",
				"bmFwc2hvdExlbmd0aDsgaSA8IGxlbmd0aDsgaSsrKQogICAgICByZXN1bHRzLnB1c2goRWxlbWVudC5l",
				"eHRlbmQocXVlcnkuc25hcHNob3RJdGVtKGkpKSk7CiAgICByZXR1cm4gcmVzdWx0czsKICB9Owp9Cgov",
				"Ki0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0t",
				"LS0tLS0tLS0tLS0tLS0tKi8KCmlmICghTm9kZSkgdmFyIE5vZGUgPSB7IH07CgppZiAoIU5vZGUuRUxF",
				"TUVOVF9OT0RFKSB7CiAgT2JqZWN0LmV4dGVuZChOb2RlLCB7CiAgICBFTEVNRU5UX05PREU6IDEsCiAg",
				"ICBBVFRSSUJVVEVfTk9ERTogMiwKICAgIFRFWFRfTk9ERTogMywKICAgIENEQVRBX1NFQ1RJT05fTk9E",
				"RTogNCwKICAgIEVOVElUWV9SRUZFUkVOQ0VfTk9ERTogNSwKICAgIEVOVElUWV9OT0RFOiA2LAogICAg",
				"UFJPQ0VTU0lOR19JTlNUUlVDVElPTl9OT0RFOiA3LAogICAgQ09NTUVOVF9OT0RFOiA4LAogICAgRE9D",
				"VU1FTlRfTk9ERTogOSwKICAgIERPQ1VNRU5UX1RZUEVfTk9ERTogMTAsCiAgICBET0NVTUVOVF9GUkFH",
				"TUVOVF9OT0RFOiAxMSwKICAgIE5PVEFUSU9OX05PREU6IDEyCiAgfSk7Cn0KCgoKKGZ1bmN0aW9uKGds",
				"b2JhbCkgewogIGZ1bmN0aW9uIHNob3VsZFVzZUNhY2hlKHRhZ05hbWUsIGF0dHJpYnV0ZXMpIHsKICAg",
				"IGlmICh0YWdOYW1lID09PSAnc2VsZWN0JykgcmV0dXJuIGZhbHNlOwogICAgaWYgKCd0eXBlJyBpbiBh",
				"dHRyaWJ1dGVzKSByZXR1cm4gZmFsc2U7CiAgICByZXR1cm4gdHJ1ZTsKICB9CgogIHZhciBIQVNfRVhU",
				"RU5ERURfQ1JFQVRFX0VMRU1FTlRfU1lOVEFYID0gKGZ1bmN0aW9uKCl7CiAgICB0cnkgewogICAgICB2",
				"YXIgZWwgPSBkb2N1bWVudC5jcmVhdGVFbGVtZW50KCc8aW5wdXQgbmFtZT0ieCI+Jyk7CiAgICAgIHJl",
				"dHVybiBlbC50YWdOYW1lLnRvTG93ZXJDYXNlKCkgPT09ICdpbnB1dCcgJiYgZWwubmFtZSA9PT0gJ3gn",
				"OwogICAgfQogICAgY2F0Y2goZXJyKSB7CiAgICAgIHJldHVybiBmYWxzZTsKICAgIH0KICB9KSgpOwoK",
				"ICB2YXIgZWxlbWVudCA9IGdsb2JhbC5FbGVtZW50OwoKICBnbG9iYWwuRWxlbWVudCA9IGZ1bmN0aW9u",
				"KHRhZ05hbWUsIGF0dHJpYnV0ZXMpIHsKICAgIGF0dHJpYnV0ZXMgPSBhdHRyaWJ1dGVzIHx8IHsgfTsK",
				"ICAgIHRhZ05hbWUgPSB0YWdOYW1lLnRvTG93ZXJDYXNlKCk7CiAgICB2YXIgY2FjaGUgPSBFbGVtZW50",
				"LmNhY2hlOwoKICAgIGlmIChIQVNfRVhURU5ERURfQ1JFQVRFX0VMRU1FTlRfU1lOVEFYICYmIGF0dHJp",
				"YnV0ZXMubmFtZSkgewogICAgICB0YWdOYW1lID0gJzwnICsgdGFnTmFtZSArICcgbmFtZT0iJyArIGF0",
				"dHJpYnV0ZXMubmFtZSArICciPic7CiAgICAgIGRlbGV0ZSBhdHRyaWJ1dGVzLm5hbWU7CiAgICAgIHJl",
				"dHVybiBFbGVtZW50LndyaXRlQXR0cmlidXRlKGRvY3VtZW50LmNyZWF0ZUVsZW1lbnQodGFnTmFtZSks",
				"IGF0dHJpYnV0ZXMpOwogICAgfQoKICAgIGlmICghY2FjaGVbdGFnTmFtZV0pIGNhY2hlW3RhZ05hbWVd",
				"ID0gRWxlbWVudC5leHRlbmQoZG9jdW1lbnQuY3JlYXRlRWxlbWVudCh0YWdOYW1lKSk7CgogICAgdmFy",
				"IG5vZGUgPSBzaG91bGRVc2VDYWNoZSh0YWdOYW1lLCBhdHRyaWJ1dGVzKSA/CiAgICAgY2FjaGVbdGFn",
				"TmFtZV0uY2xvbmVOb2RlKGZhbHNlKSA6IGRvY3VtZW50LmNyZWF0ZUVsZW1lbnQodGFnTmFtZSk7Cgog",
				"ICAgcmV0dXJuIEVsZW1lbnQud3JpdGVBdHRyaWJ1dGUobm9kZSwgYXR0cmlidXRlcyk7CiAgfTsKCiAg",
				"T2JqZWN0LmV4dGVuZChnbG9iYWwuRWxlbWVudCwgZWxlbWVudCB8fCB7IH0pOwogIGlmIChlbGVtZW50",
				"KSBnbG9iYWwuRWxlbWVudC5wcm90b3R5cGUgPSBlbGVtZW50LnByb3RvdHlwZTsKCn0pKHRoaXMpOwoK",
				"RWxlbWVudC5pZENvdW50ZXIgPSAxOwpFbGVtZW50LmNhY2hlID0geyB9OwoKRWxlbWVudC5fcHVyZ2VF",
				"bGVtZW50ID0gZnVuY3Rpb24oZWxlbWVudCkgewogIHZhciB1aWQgPSBlbGVtZW50Ll9wcm90b3R5cGVV",
				"SUQ7CiAgaWYgKHVpZCkgewogICAgRWxlbWVudC5zdG9wT2JzZXJ2aW5nKGVsZW1lbnQpOwogICAgZWxl",
				"bWVudC5fcHJvdG90eXBlVUlEID0gdm9pZCAwOwogICAgZGVsZXRlIEVsZW1lbnQuU3RvcmFnZVt1aWRd",
				"OwogIH0KfQoKRWxlbWVudC5NZXRob2RzID0gewogIHZpc2libGU6IGZ1bmN0aW9uKGVsZW1lbnQpIHsK",
				"ICAgIHJldHVybiAkKGVsZW1lbnQpLnN0eWxlLmRpc3BsYXkgIT0gJ25vbmUnOwogIH0sCgogIHRvZ2ds",
				"ZTogZnVuY3Rpb24oZWxlbWVudCkgewogICAgZWxlbWVudCA9ICQoZWxlbWVudCk7CiAgICBFbGVtZW50",
				"W0VsZW1lbnQudmlzaWJsZShlbGVtZW50KSA/ICdoaWRlJyA6ICdzaG93J10oZWxlbWVudCk7CiAgICBy",
				"ZXR1cm4gZWxlbWVudDsKICB9LAoKICBoaWRlOiBmdW5jdGlvbihlbGVtZW50KSB7CiAgICBlbGVtZW50",
				"ID0gJChlbGVtZW50KTsKICAgIGVsZW1lbnQuc3R5bGUuZGlzcGxheSA9ICdub25lJzsKICAgIHJldHVy",
				"biBlbGVtZW50OwogIH0sCgogIHNob3c6IGZ1bmN0aW9uKGVsZW1lbnQpIHsKICAgIGVsZW1lbnQgPSAk",
				"KGVsZW1lbnQpOwogICAgZWxlbWVudC5zdHlsZS5kaXNwbGF5ID0gJyc7CiAgICByZXR1cm4gZWxlbWVu",
				"dDsKICB9LAoKICByZW1vdmU6IGZ1bmN0aW9uKGVsZW1lbnQpIHsKICAgIGVsZW1lbnQgPSAkKGVsZW1l",
				"bnQpOwogICAgZWxlbWVudC5wYXJlbnROb2RlLnJlbW92ZUNoaWxkKGVsZW1lbnQpOwogICAgcmV0dXJu",
				"IGVsZW1lbnQ7CiAgfSwKCiAgdXBkYXRlOiAoZnVuY3Rpb24oKXsKCiAgICB2YXIgU0VMRUNUX0VMRU1F",
				"TlRfSU5ORVJIVE1MX0JVR0dZID0gKGZ1bmN0aW9uKCl7CiAgICAgIHZhciBlbCA9IGRvY3VtZW50LmNy",
				"ZWF0ZUVsZW1lbnQoInNlbGVjdCIpLAogICAgICAgICAgaXNCdWdneSA9IHRydWU7CiAgICAgIGVsLmlu",
				"bmVySFRNTCA9ICI8b3B0aW9uIHZhbHVlPVwidGVzdFwiPnRlc3Q8L29wdGlvbj4iOwogICAgICBpZiAo",
				"ZWwub3B0aW9ucyAmJiBlbC5vcHRpb25zWzBdKSB7CiAgICAgICAgaXNCdWdneSA9IGVsLm9wdGlvbnNb",
				"MF0ubm9kZU5hbWUudG9VcHBlckNhc2UoKSAhPT0gIk9QVElPTiI7CiAgICAgIH0KICAgICAgZWwgPSBu",
				"dWxsOwogICAgICByZXR1cm4gaXNCdWdneTsKICAgIH0pKCk7CgogICAgdmFyIFRBQkxFX0VMRU1FTlRf",
				"SU5ORVJIVE1MX0JVR0dZID0gKGZ1bmN0aW9uKCl7CiAgICAgIHRyeSB7CiAgICAgICAgdmFyIGVsID0g",
				"ZG9jdW1lbnQuY3JlYXRlRWxlbWVudCgidGFibGUiKTsKICAgICAgICBpZiAoZWwgJiYgZWwudEJvZGll",
				"cykgewogICAgICAgICAgZWwuaW5uZXJIVE1MID0gIjx0Ym9keT48dHI+PHRkPnRlc3Q8L3RkPjwvdHI+",
				"PC90Ym9keT4iOwogICAgICAgICAgdmFyIGlzQnVnZ3kgPSB0eXBlb2YgZWwudEJvZGllc1swXSA9PSAi",
				"dW5kZWZpbmVkIjsKICAgICAgICAgIGVsID0gbnVsbDsKICAgICAgICAgIHJldHVybiBpc0J1Z2d5Owog",
				"ICAgICAgIH0KICAgICAgfSBjYXRjaCAoZSkgewogICAgICAgIHJldHVybiB0cnVlOwogICAgICB9CiAg",
				"ICB9KSgpOwoKICAgIHZhciBMSU5LX0VMRU1FTlRfSU5ORVJIVE1MX0JVR0dZID0gKGZ1bmN0aW9uKCkg",
				"ewogICAgICB0cnkgewogICAgICAgIHZhciBlbCA9IGRvY3VtZW50LmNyZWF0ZUVsZW1lbnQoJ2Rpdicp",
				"OwogICAgICAgIGVsLmlubmVySFRNTCA9ICI8bGluaz4iOwogICAgICAgIHZhciBpc0J1Z2d5ID0gKGVs",
				"LmNoaWxkTm9kZXMubGVuZ3RoID09PSAwKTsKICAgICAgICBlbCA9IG51bGw7CiAgICAgICAgcmV0dXJu",
				"IGlzQnVnZ3k7CiAgICAgIH0gY2F0Y2goZSkgewogICAgICAgIHJldHVybiB0cnVlOwogICAgICB9CiAg",
				"ICB9KSgpOwoKICAgIHZhciBBTllfSU5ORVJIVE1MX0JVR0dZID0gU0VMRUNUX0VMRU1FTlRfSU5ORVJI",
				"VE1MX0JVR0dZIHx8CiAgICAgVEFCTEVfRUxFTUVOVF9JTk5FUkhUTUxfQlVHR1kgfHwgTElOS19FTEVN",
				"RU5UX0lOTkVSSFRNTF9CVUdHWTsKCiAgICB2YXIgU0NSSVBUX0VMRU1FTlRfUkVKRUNUU19URVhUTk9E",
				"RV9BUFBFTkRJTkcgPSAoZnVuY3Rpb24gKCkgewogICAgICB2YXIgcyA9IGRvY3VtZW50LmNyZWF0ZUVs",
				"ZW1lbnQoInNjcmlwdCIpLAogICAgICAgICAgaXNCdWdneSA9IGZhbHNlOwogICAgICB0cnkgewogICAg",
				"ICAgIHMuYXBwZW5kQ2hpbGQoZG9jdW1lbnQuY3JlYXRlVGV4dE5vZGUoIiIpKTsKICAgICAgICBpc0J1",
				"Z2d5ID0gIXMuZmlyc3RDaGlsZCB8fAogICAgICAgICAgcy5maXJzdENoaWxkICYmIHMuZmlyc3RDaGls",
				"ZC5ub2RlVHlwZSAhPT0gMzsKICAgICAgfSBjYXRjaCAoZSkgewogICAgICAgIGlzQnVnZ3kgPSB0cnVl",
				"OwogICAgICB9CiAgICAgIHMgPSBudWxsOwogICAgICByZXR1cm4gaXNCdWdneTsKICAgIH0pKCk7CgoK",
				"ICAgIGZ1bmN0aW9uIHVwZGF0ZShlbGVtZW50LCBjb250ZW50KSB7CiAgICAgIGVsZW1lbnQgPSAkKGVs",
				"ZW1lbnQpOwogICAgICB2YXIgcHVyZ2VFbGVtZW50ID0gRWxlbWVudC5fcHVyZ2VFbGVtZW50OwoKICAg",
				"ICAgdmFyIGRlc2NlbmRhbnRzID0gZWxlbWVudC5nZXRFbGVtZW50c0J5VGFnTmFtZSgnKicpLAogICAg",
				"ICAgaSA9IGRlc2NlbmRhbnRzLmxlbmd0aDsKICAgICAgd2hpbGUgKGktLSkgcHVyZ2VFbGVtZW50KGRl",
				"c2NlbmRhbnRzW2ldKTsKCiAgICAgIGlmIChjb250ZW50ICYmIGNvbnRlbnQudG9FbGVtZW50KQogICAg",
				"ICAgIGNvbnRlbnQgPSBjb250ZW50LnRvRWxlbWVudCgpOwoKICAgICAgaWYgKE9iamVjdC5pc0VsZW1l",
				"bnQoY29udGVudCkpCiAgICAgICAgcmV0dXJuIGVsZW1lbnQudXBkYXRlKCkuaW5zZXJ0KGNvbnRlbnQp",
				"OwoKICAgICAgY29udGVudCA9IE9iamVjdC50b0hUTUwoY29udGVudCk7CgogICAgICB2YXIgdGFnTmFt",
				"ZSA9IGVsZW1lbnQudGFnTmFtZS50b1VwcGVyQ2FzZSgpOwoKICAgICAgaWYgKHRhZ05hbWUgPT09ICdT",
				"Q1JJUFQnICYmIFNDUklQVF9FTEVNRU5UX1JFSkVDVFNfVEVYVE5PREVfQVBQRU5ESU5HKSB7CiAgICAg",
				"ICAgZWxlbWVudC50ZXh0ID0gY29udGVudDsKICAgICAgICByZXR1cm4gZWxlbWVudDsKICAgICAgfQoK",
				"ICAgICAgaWYgKEFOWV9JTk5FUkhUTUxfQlVHR1kpIHsKICAgICAgICBpZiAodGFnTmFtZSBpbiBFbGVt",
				"ZW50Ll9pbnNlcnRpb25UcmFuc2xhdGlvbnMudGFncykgewogICAgICAgICAgd2hpbGUgKGVsZW1lbnQu",
				"Zmlyc3RDaGlsZCkgewogICAgICAgICAgICBlbGVtZW50LnJlbW92ZUNoaWxkKGVsZW1lbnQuZmlyc3RD",
				"aGlsZCk7CiAgICAgICAgICB9CiAgICAgICAgICBFbGVtZW50Ll9nZXRDb250ZW50RnJvbUFub255bW91",
				"c0VsZW1lbnQodGFnTmFtZSwgY29udGVudC5zdHJpcFNjcmlwdHMoKSkKICAgICAgICAgICAgLmVhY2go",
				"ZnVuY3Rpb24obm9kZSkgewogICAgICAgICAgICAgIGVsZW1lbnQuYXBwZW5kQ2hpbGQobm9kZSkKICAg",
				"ICAgICAgICAgfSk7CiAgICAgICAgfSBlbHNlIGlmIChMSU5LX0VMRU1FTlRfSU5ORVJIVE1MX0JVR0dZ",
				"ICYmIE9iamVjdC5pc1N0cmluZyhjb250ZW50KSAmJiBjb250ZW50LmluZGV4T2YoJzxsaW5rJykgPiAt",
				"MSkgewogICAgICAgICAgd2hpbGUgKGVsZW1lbnQuZmlyc3RDaGlsZCkgewogICAgICAgICAgICBlbGVt",
				"ZW50LnJlbW92ZUNoaWxkKGVsZW1lbnQuZmlyc3RDaGlsZCk7CiAgICAgICAgICB9CiAgICAgICAgICB2",
				"YXIgbm9kZXMgPSBFbGVtZW50Ll9nZXRDb250ZW50RnJvbUFub255bW91c0VsZW1lbnQodGFnTmFtZSwg",
				"Y29udGVudC5zdHJpcFNjcmlwdHMoKSwgdHJ1ZSk7CiAgICAgICAgICBub2Rlcy5lYWNoKGZ1bmN0aW9u",
				"KG5vZGUpIHsgZWxlbWVudC5hcHBlbmRDaGlsZChub2RlKSB9KTsKICAgICAgICB9CiAgICAgICAgZWxz",
				"ZSB7CiAgICAgICAgICBlbGVtZW50LmlubmVySFRNTCA9IGNvbnRlbnQuc3RyaXBTY3JpcHRzKCk7CiAg",
				"ICAgICAgfQogICAgICB9CiAgICAgIGVsc2UgewogICAgICAgIGVsZW1lbnQuaW5uZXJIVE1MID0gY29u",
				"dGVudC5zdHJpcFNjcmlwdHMoKTsKICAgICAgfQoKICAgICAgY29udGVudC5ldmFsU2NyaXB0cy5iaW5k",
				"KGNvbnRlbnQpLmRlZmVyKCk7CiAgICAgIHJldHVybiBlbGVtZW50OwogICAgfQoKICAgIHJldHVybiB1",
				"cGRhdGU7CiAgfSkoKSwKCiAgcmVwbGFjZTogZnVuY3Rpb24oZWxlbWVudCwgY29udGVudCkgewogICAg",
				"ZWxlbWVudCA9ICQoZWxlbWVudCk7CiAgICBpZiAoY29udGVudCAmJiBjb250ZW50LnRvRWxlbWVudCkg",
				"Y29udGVudCA9IGNvbnRlbnQudG9FbGVtZW50KCk7CiAgICBlbHNlIGlmICghT2JqZWN0LmlzRWxlbWVu",
				"dChjb250ZW50KSkgewogICAgICBjb250ZW50ID0gT2JqZWN0LnRvSFRNTChjb250ZW50KTsKICAgICAg",
				"dmFyIHJhbmdlID0gZWxlbWVudC5vd25lckRvY3VtZW50LmNyZWF0ZVJhbmdlKCk7CiAgICAgIHJhbmdl",
				"LnNlbGVjdE5vZGUoZWxlbWVudCk7CiAgICAgIGNvbnRlbnQuZXZhbFNjcmlwdHMuYmluZChjb250ZW50",
				"KS5kZWZlcigpOwogICAgICBjb250ZW50ID0gcmFuZ2UuY3JlYXRlQ29udGV4dHVhbEZyYWdtZW50KGNv",
				"bnRlbnQuc3RyaXBTY3JpcHRzKCkpOwogICAgfQogICAgZWxlbWVudC5wYXJlbnROb2RlLnJlcGxhY2VD",
				"aGlsZChjb250ZW50LCBlbGVtZW50KTsKICAgIHJldHVybiBlbGVtZW50OwogIH0sCgogIGluc2VydDog",
				"ZnVuY3Rpb24oZWxlbWVudCwgaW5zZXJ0aW9ucykgewogICAgZWxlbWVudCA9ICQoZWxlbWVudCk7Cgog",
				"ICAgaWYgKE9iamVjdC5pc1N0cmluZyhpbnNlcnRpb25zKSB8fCBPYmplY3QuaXNOdW1iZXIoaW5zZXJ0",
				"aW9ucykgfHwKICAgICAgICBPYmplY3QuaXNFbGVtZW50KGluc2VydGlvbnMpIHx8IChpbnNlcnRpb25z",
				"ICYmIChpbnNlcnRpb25zLnRvRWxlbWVudCB8fCBpbnNlcnRpb25zLnRvSFRNTCkpKQogICAgICAgICAg",
				"aW5zZXJ0aW9ucyA9IHtib3R0b206aW5zZXJ0aW9uc307CgogICAgdmFyIGNvbnRlbnQsIGluc2VydCwg",
				"dGFnTmFtZSwgY2hpbGROb2RlczsKCiAgICBmb3IgKHZhciBwb3NpdGlvbiBpbiBpbnNlcnRpb25zKSB7",
				"CiAgICAgIGNvbnRlbnQgID0gaW5zZXJ0aW9uc1twb3NpdGlvbl07CiAgICAgIHBvc2l0aW9uID0gcG9z",
				"aXRpb24udG9Mb3dlckNhc2UoKTsKICAgICAgaW5zZXJ0ID0gRWxlbWVudC5faW5zZXJ0aW9uVHJhbnNs",
				"YXRpb25zW3Bvc2l0aW9uXTsKCiAgICAgIGlmIChjb250ZW50ICYmIGNvbnRlbnQudG9FbGVtZW50KSBj",
				"b250ZW50ID0gY29udGVudC50b0VsZW1lbnQoKTsKICAgICAgaWYgKE9iamVjdC5pc0VsZW1lbnQoY29u",
				"dGVudCkpIHsKICAgICAgICBpbnNlcnQoZWxlbWVudCwgY29udGVudCk7CiAgICAgICAgY29udGludWU7",
				"CiAgICAgIH0KCiAgICAgIGNvbnRlbnQgPSBPYmplY3QudG9IVE1MKGNvbnRlbnQpOwoKICAgICAgdGFn",
				"TmFtZSA9ICgocG9zaXRpb24gPT0gJ2JlZm9yZScgfHwgcG9zaXRpb24gPT0gJ2FmdGVyJykKICAgICAg",
				"ICA/IGVsZW1lbnQucGFyZW50Tm9kZSA6IGVsZW1lbnQpLnRhZ05hbWUudG9VcHBlckNhc2UoKTsKCiAg",
				"ICAgIGNoaWxkTm9kZXMgPSBFbGVtZW50Ll9nZXRDb250ZW50RnJvbUFub255bW91c0VsZW1lbnQodGFn",
				"TmFtZSwgY29udGVudC5zdHJpcFNjcmlwdHMoKSk7CgogICAgICBpZiAocG9zaXRpb24gPT0gJ3RvcCcg",
				"fHwgcG9zaXRpb24gPT0gJ2FmdGVyJykgY2hpbGROb2Rlcy5yZXZlcnNlKCk7CiAgICAgIGNoaWxkTm9k",
				"ZXMuZWFjaChpbnNlcnQuY3VycnkoZWxlbWVudCkpOwoKICAgICAgY29udGVudC5ldmFsU2NyaXB0cy5i",
				"aW5kKGNvbnRlbnQpLmRlZmVyKCk7CiAgICB9CgogICAgcmV0dXJuIGVsZW1lbnQ7CiAgfSwKCiAgd3Jh",
				"cDogZnVuY3Rpb24oZWxlbWVudCwgd3JhcHBlciwgYXR0cmlidXRlcykgewogICAgZWxlbWVudCA9ICQo",
				"ZWxlbWVudCk7CiAgICBpZiAoT2JqZWN0LmlzRWxlbWVudCh3cmFwcGVyKSkKICAgICAgJCh3cmFwcGVy",
				"KS53cml0ZUF0dHJpYnV0ZShhdHRyaWJ1dGVzIHx8IHsgfSk7CiAgICBlbHNlIGlmIChPYmplY3QuaXNT",
				"dHJpbmcod3JhcHBlcikpIHdyYXBwZXIgPSBuZXcgRWxlbWVudCh3cmFwcGVyLCBhdHRyaWJ1dGVzKTsK",
				"ICAgIGVsc2Ugd3JhcHBlciA9IG5ldyBFbGVtZW50KCdkaXYnLCB3cmFwcGVyKTsKICAgIGlmIChlbGVt",
				"ZW50LnBhcmVudE5vZGUpCiAgICAgIGVsZW1lbnQucGFyZW50Tm9kZS5yZXBsYWNlQ2hpbGQod3JhcHBl",
				"ciwgZWxlbWVudCk7CiAgICB3cmFwcGVyLmFwcGVuZENoaWxkKGVsZW1lbnQpOwogICAgcmV0dXJuIHdy",
				"YXBwZXI7CiAgfSwKCiAgaW5zcGVjdDogZnVuY3Rpb24oZWxlbWVudCkgewogICAgZWxlbWVudCA9ICQo",
				"ZWxlbWVudCk7CiAgICB2YXIgcmVzdWx0ID0gJzwnICsgZWxlbWVudC50YWdOYW1lLnRvTG93ZXJDYXNl",
				"KCk7CiAgICAkSCh7J2lkJzogJ2lkJywgJ2NsYXNzTmFtZSc6ICdjbGFzcyd9KS5lYWNoKGZ1bmN0aW9u",
				"KHBhaXIpIHsKICAgICAgdmFyIHByb3BlcnR5ID0gcGFpci5maXJzdCgpLAogICAgICAgICAgYXR0cmli",
				"dXRlID0gcGFpci5sYXN0KCksCiAgICAgICAgICB2YWx1ZSA9IChlbGVtZW50W3Byb3BlcnR5XSB8fCAn",
				"JykudG9TdHJpbmcoKTsKICAgICAgaWYgKHZhbHVlKSByZXN1bHQgKz0gJyAnICsgYXR0cmlidXRlICsg",
				"Jz0nICsgdmFsdWUuaW5zcGVjdCh0cnVlKTsKICAgIH0pOwogICAgcmV0dXJuIHJlc3VsdCArICc+JzsK",
				"ICB9LAoKICByZWN1cnNpdmVseUNvbGxlY3Q6IGZ1bmN0aW9uKGVsZW1lbnQsIHByb3BlcnR5LCBtYXhp",
				"bXVtTGVuZ3RoKSB7CiAgICBlbGVtZW50ID0gJChlbGVtZW50KTsKICAgIG1heGltdW1MZW5ndGggPSBt",
				"YXhpbXVtTGVuZ3RoIHx8IC0xOwogICAgdmFyIGVsZW1lbnRzID0gW107CgogICAgd2hpbGUgKGVsZW1l",
				"bnQgPSBlbGVtZW50W3Byb3BlcnR5XSkgewogICAgICBpZiAoZWxlbWVudC5ub2RlVHlwZSA9PSAxKQog",
				"ICAgICAgIGVsZW1lbnRzLnB1c2goRWxlbWVudC5leHRlbmQoZWxlbWVudCkpOwogICAgICBpZiAoZWxl",
				"bWVudHMubGVuZ3RoID09IG1heGltdW1MZW5ndGgpCiAgICAgICAgYnJlYWs7CiAgICB9CgogICAgcmV0",
				"dXJuIGVsZW1lbnRzOwogIH0sCgogIGFuY2VzdG9yczogZnVuY3Rpb24oZWxlbWVudCkgewogICAgcmV0",
				"dXJuIEVsZW1lbnQucmVjdXJzaXZlbHlDb2xsZWN0KGVsZW1lbnQsICdwYXJlbnROb2RlJyk7CiAgfSwK",
				"CiAgZGVzY2VuZGFudHM6IGZ1bmN0aW9uKGVsZW1lbnQpIHsKICAgIHJldHVybiBFbGVtZW50LnNlbGVj",
				"dChlbGVtZW50LCAiKiIpOwogIH0sCgogIGZpcnN0RGVzY2VuZGFudDogZnVuY3Rpb24oZWxlbWVudCkg",
				"ewogICAgZWxlbWVudCA9ICQoZWxlbWVudCkuZmlyc3RDaGlsZDsKICAgIHdoaWxlIChlbGVtZW50ICYm",
				"IGVsZW1lbnQubm9kZVR5cGUgIT0gMSkgZWxlbWVudCA9IGVsZW1lbnQubmV4dFNpYmxpbmc7CiAgICBy",
				"ZXR1cm4gJChlbGVtZW50KTsKICB9LAoKICBpbW1lZGlhdGVEZXNjZW5kYW50czogZnVuY3Rpb24oZWxl",
				"bWVudCkgewogICAgdmFyIHJlc3VsdHMgPSBbXSwgY2hpbGQgPSAkKGVsZW1lbnQpLmZpcnN0Q2hpbGQ7",
				"CiAgICB3aGlsZSAoY2hpbGQpIHsKICAgICAgaWYgKGNoaWxkLm5vZGVUeXBlID09PSAxKSB7CiAgICAg",
				"ICAgcmVzdWx0cy5wdXNoKEVsZW1lbnQuZXh0ZW5kKGNoaWxkKSk7CiAgICAgIH0KICAgICAgY2hpbGQg",
				"PSBjaGlsZC5uZXh0U2libGluZzsKICAgIH0KICAgIHJldHVybiByZXN1bHRzOwogIH0sCgogIHByZXZp",
				"b3VzU2libGluZ3M6IGZ1bmN0aW9uKGVsZW1lbnQsIG1heGltdW1MZW5ndGgpIHsKICAgIHJldHVybiBF",
				"bGVtZW50LnJlY3Vyc2l2ZWx5Q29sbGVjdChlbGVtZW50LCAncHJldmlvdXNTaWJsaW5nJyk7CiAgfSwK",
				"CiAgbmV4dFNpYmxpbmdzOiBmdW5jdGlvbihlbGVtZW50KSB7CiAgICByZXR1cm4gRWxlbWVudC5yZWN1",
				"cnNpdmVseUNvbGxlY3QoZWxlbWVudCwgJ25leHRTaWJsaW5nJyk7CiAgfSwKCiAgc2libGluZ3M6IGZ1",
				"bmN0aW9uKGVsZW1lbnQpIHsKICAgIGVsZW1lbnQgPSAkKGVsZW1lbnQpOwogICAgcmV0dXJuIEVsZW1l",
				"bnQucHJldmlvdXNTaWJsaW5ncyhlbGVtZW50KS5yZXZlcnNlKCkKICAgICAgLmNvbmNhdChFbGVtZW50",
				"Lm5leHRTaWJsaW5ncyhlbGVtZW50KSk7CiAgfSwKCiAgbWF0Y2g6IGZ1bmN0aW9uKGVsZW1lbnQsIHNl",
				"bGVjdG9yKSB7CiAgICBlbGVtZW50ID0gJChlbGVtZW50KTsKICAgIGlmIChPYmplY3QuaXNTdHJpbmco",
				"c2VsZWN0b3IpKQogICAgICByZXR1cm4gUHJvdG90eXBlLlNlbGVjdG9yLm1hdGNoKGVsZW1lbnQsIHNl",
				"bGVjdG9yKTsKICAgIHJldHVybiBzZWxlY3Rvci5tYXRjaChlbGVtZW50KTsKICB9LAoKICB1cDogZnVu",
				"Y3Rpb24oZWxlbWVudCwgZXhwcmVzc2lvbiwgaW5kZXgpIHsKICAgIGVsZW1lbnQgPSAkKGVsZW1lbnQp",
				"OwogICAgaWYgKGFyZ3VtZW50cy5sZW5ndGggPT0gMSkgcmV0dXJuICQoZWxlbWVudC5wYXJlbnROb2Rl",
				"KTsKICAgIHZhciBhbmNlc3RvcnMgPSBFbGVtZW50LmFuY2VzdG9ycyhlbGVtZW50KTsKICAgIHJldHVy",
				"biBPYmplY3QuaXNOdW1iZXIoZXhwcmVzc2lvbikgPyBhbmNlc3RvcnNbZXhwcmVzc2lvbl0gOgogICAg",
				"ICBQcm90b3R5cGUuU2VsZWN0b3IuZmluZChhbmNlc3RvcnMsIGV4cHJlc3Npb24sIGluZGV4KTsKICB9",
				"LAoKICBkb3duOiBmdW5jdGlvbihlbGVtZW50LCBleHByZXNzaW9uLCBpbmRleCkgewogICAgZWxlbWVu",
				"dCA9ICQoZWxlbWVudCk7CiAgICBpZiAoYXJndW1lbnRzLmxlbmd0aCA9PSAxKSByZXR1cm4gRWxlbWVu",
				"dC5maXJzdERlc2NlbmRhbnQoZWxlbWVudCk7CiAgICByZXR1cm4gT2JqZWN0LmlzTnVtYmVyKGV4cHJl",
				"c3Npb24pID8gRWxlbWVudC5kZXNjZW5kYW50cyhlbGVtZW50KVtleHByZXNzaW9uXSA6CiAgICAgIEVs",
				"ZW1lbnQuc2VsZWN0KGVsZW1lbnQsIGV4cHJlc3Npb24pW2luZGV4IHx8IDBdOwogIH0sCgogIHByZXZp",
				"b3VzOiBmdW5jdGlvbihlbGVtZW50LCBleHByZXNzaW9uLCBpbmRleCkgewogICAgZWxlbWVudCA9ICQo",
				"ZWxlbWVudCk7CiAgICBpZiAoT2JqZWN0LmlzTnVtYmVyKGV4cHJlc3Npb24pKSBpbmRleCA9IGV4cHJl",
				"c3Npb24sIGV4cHJlc3Npb24gPSBmYWxzZTsKICAgIGlmICghT2JqZWN0LmlzTnVtYmVyKGluZGV4KSkg",
				"aW5kZXggPSAwOwoKICAgIGlmIChleHByZXNzaW9uKSB7CiAgICAgIHJldHVybiBQcm90b3R5cGUuU2Vs",
				"ZWN0b3IuZmluZChlbGVtZW50LnByZXZpb3VzU2libGluZ3MoKSwgZXhwcmVzc2lvbiwgaW5kZXgpOwog",
				"ICAgfSBlbHNlIHsKICAgICAgcmV0dXJuIGVsZW1lbnQucmVjdXJzaXZlbHlDb2xsZWN0KCJwcmV2aW91",
				"c1NpYmxpbmciLCBpbmRleCArIDEpW2luZGV4XTsKICAgIH0KICB9LAoKICBuZXh0OiBmdW5jdGlvbihl",
				"bGVtZW50LCBleHByZXNzaW9uLCBpbmRleCkgewogICAgZWxlbWVudCA9ICQoZWxlbWVudCk7CiAgICBp",
				"ZiAoT2JqZWN0LmlzTnVtYmVyKGV4cHJlc3Npb24pKSBpbmRleCA9IGV4cHJlc3Npb24sIGV4cHJlc3Np",
				"b24gPSBmYWxzZTsKICAgIGlmICghT2JqZWN0LmlzTnVtYmVyKGluZGV4KSkgaW5kZXggPSAwOwoKICAg",
				"IGlmIChleHByZXNzaW9uKSB7CiAgICAgIHJldHVybiBQcm90b3R5cGUuU2VsZWN0b3IuZmluZChlbGVt",
				"ZW50Lm5leHRTaWJsaW5ncygpLCBleHByZXNzaW9uLCBpbmRleCk7CiAgICB9IGVsc2UgewogICAgICB2",
				"YXIgbWF4aW11bUxlbmd0aCA9IE9iamVjdC5pc051bWJlcihpbmRleCkgPyBpbmRleCArIDEgOiAxOwog",
				"ICAgICByZXR1cm4gZWxlbWVudC5yZWN1cnNpdmVseUNvbGxlY3QoIm5leHRTaWJsaW5nIiwgaW5kZXgg",
				"KyAxKVtpbmRleF07CiAgICB9CiAgfSwKCgogIHNlbGVjdDogZnVuY3Rpb24oZWxlbWVudCkgewogICAg",
				"ZWxlbWVudCA9ICQoZWxlbWVudCk7CiAgICB2YXIgZXhwcmVzc2lvbnMgPSBBcnJheS5wcm90b3R5cGUu",
				"c2xpY2UuY2FsbChhcmd1bWVudHMsIDEpLmpvaW4oJywgJyk7CiAgICByZXR1cm4gUHJvdG90eXBlLlNl",
				"bGVjdG9yLnNlbGVjdChleHByZXNzaW9ucywgZWxlbWVudCk7CiAgfSwKCiAgYWRqYWNlbnQ6IGZ1bmN0",
				"aW9uKGVsZW1lbnQpIHsKICAgIGVsZW1lbnQgPSAkKGVsZW1lbnQpOwogICAgdmFyIGV4cHJlc3Npb25z",
				"ID0gQXJyYXkucHJvdG90eXBlLnNsaWNlLmNhbGwoYXJndW1lbnRzLCAxKS5qb2luKCcsICcpOwogICAg",
				"cmV0dXJuIFByb3RvdHlwZS5TZWxlY3Rvci5zZWxlY3QoZXhwcmVzc2lvbnMsIGVsZW1lbnQucGFyZW50",
				"Tm9kZSkud2l0aG91dChlbGVtZW50KTsKICB9LAoKICBpZGVudGlmeTogZnVuY3Rpb24oZWxlbWVudCkg",
				"ewogICAgZWxlbWVudCA9ICQoZWxlbWVudCk7CiAgICB2YXIgaWQgPSBFbGVtZW50LnJlYWRBdHRyaWJ1",
				"dGUoZWxlbWVudCwgJ2lkJyk7CiAgICBpZiAoaWQpIHJldHVybiBpZDsKICAgIGRvIHsgaWQgPSAnYW5v",
				"bnltb3VzX2VsZW1lbnRfJyArIEVsZW1lbnQuaWRDb3VudGVyKysgfSB3aGlsZSAoJChpZCkpOwogICAg",
				"RWxlbWVudC53cml0ZUF0dHJpYnV0ZShlbGVtZW50LCAnaWQnLCBpZCk7CiAgICByZXR1cm4gaWQ7CiAg",
				"fSwKCiAgcmVhZEF0dHJpYnV0ZTogZnVuY3Rpb24oZWxlbWVudCwgbmFtZSkgewogICAgZWxlbWVudCA9",
				"ICQoZWxlbWVudCk7CiAgICBpZiAoUHJvdG90eXBlLkJyb3dzZXIuSUUpIHsKICAgICAgdmFyIHQgPSBF",
				"bGVtZW50Ll9hdHRyaWJ1dGVUcmFuc2xhdGlvbnMucmVhZDsKICAgICAgaWYgKHQudmFsdWVzW25hbWVd",
				"KSByZXR1cm4gdC52YWx1ZXNbbmFtZV0oZWxlbWVudCwgbmFtZSk7CiAgICAgIGlmICh0Lm5hbWVzW25h",
				"bWVdKSBuYW1lID0gdC5uYW1lc1tuYW1lXTsKICAgICAgaWYgKG5hbWUuaW5jbHVkZSgnOicpKSB7CiAg",
				"ICAgICAgcmV0dXJuICghZWxlbWVudC5hdHRyaWJ1dGVzIHx8ICFlbGVtZW50LmF0dHJpYnV0ZXNbbmFt",
				"ZV0pID8gbnVsbCA6CiAgICAgICAgIGVsZW1lbnQuYXR0cmlidXRlc1tuYW1lXS52YWx1ZTsKICAgICAg",
				"fQogICAgfQogICAgcmV0dXJuIGVsZW1lbnQuZ2V0QXR0cmlidXRlKG5hbWUpOwogIH0sCgogIHdyaXRl",
				"QXR0cmlidXRlOiBmdW5jdGlvbihlbGVtZW50LCBuYW1lLCB2YWx1ZSkgewogICAgZWxlbWVudCA9ICQo",
				"ZWxlbWVudCk7CiAgICB2YXIgYXR0cmlidXRlcyA9IHsgfSwgdCA9IEVsZW1lbnQuX2F0dHJpYnV0ZVRy",
				"YW5zbGF0aW9ucy53cml0ZTsKCiAgICBpZiAodHlwZW9mIG5hbWUgPT0gJ29iamVjdCcpIGF0dHJpYnV0",
				"ZXMgPSBuYW1lOwogICAgZWxzZSBhdHRyaWJ1dGVzW25hbWVdID0gT2JqZWN0LmlzVW5kZWZpbmVkKHZh",
				"bHVlKSA/IHRydWUgOiB2YWx1ZTsKCiAgICBmb3IgKHZhciBhdHRyIGluIGF0dHJpYnV0ZXMpIHsKICAg",
				"ICAgbmFtZSA9IHQubmFtZXNbYXR0cl0gfHwgYXR0cjsKICAgICAgdmFsdWUgPSBhdHRyaWJ1dGVzW2F0",
				"dHJdOwogICAgICBpZiAodC52YWx1ZXNbYXR0cl0pIG5hbWUgPSB0LnZhbHVlc1thdHRyXShlbGVtZW50",
				"LCB2YWx1ZSk7CiAgICAgIGlmICh2YWx1ZSA9PT0gZmFsc2UgfHwgdmFsdWUgPT09IG51bGwpCiAgICAg",
				"ICAgZWxlbWVudC5yZW1vdmVBdHRyaWJ1dGUobmFtZSk7CiAgICAgIGVsc2UgaWYgKHZhbHVlID09PSB0",
				"cnVlKQogICAgICAgIGVsZW1lbnQuc2V0QXR0cmlidXRlKG5hbWUsIG5hbWUpOwogICAgICBlbHNlIGVs",
				"ZW1lbnQuc2V0QXR0cmlidXRlKG5hbWUsIHZhbHVlKTsKICAgIH0KICAgIHJldHVybiBlbGVtZW50Owog",
				"IH0sCgogIGdldEhlaWdodDogZnVuY3Rpb24oZWxlbWVudCkgewogICAgcmV0dXJuIEVsZW1lbnQuZ2V0",
				"RGltZW5zaW9ucyhlbGVtZW50KS5oZWlnaHQ7CiAgfSwKCiAgZ2V0V2lkdGg6IGZ1bmN0aW9uKGVsZW1l",
				"bnQpIHsKICAgIHJldHVybiBFbGVtZW50LmdldERpbWVuc2lvbnMoZWxlbWVudCkud2lkdGg7CiAgfSwK",
				"CiAgY2xhc3NOYW1lczogZnVuY3Rpb24oZWxlbWVudCkgewogICAgcmV0dXJuIG5ldyBFbGVtZW50LkNs",
				"YXNzTmFtZXMoZWxlbWVudCk7CiAgfSwKCiAgaGFzQ2xhc3NOYW1lOiBmdW5jdGlvbihlbGVtZW50LCBj",
				"bGFzc05hbWUpIHsKICAgIGlmICghKGVsZW1lbnQgPSAkKGVsZW1lbnQpKSkgcmV0dXJuOwogICAgdmFy",
				"IGVsZW1lbnRDbGFzc05hbWUgPSBlbGVtZW50LmNsYXNzTmFtZTsKICAgIHJldHVybiAoZWxlbWVudENs",
				"YXNzTmFtZS5sZW5ndGggPiAwICYmIChlbGVtZW50Q2xhc3NOYW1lID09IGNsYXNzTmFtZSB8fAogICAg",
				"ICBuZXcgUmVnRXhwKCIoXnxcXHMpIiArIGNsYXNzTmFtZSArICIoXFxzfCQpIikudGVzdChlbGVtZW50",
				"Q2xhc3NOYW1lKSkpOwogIH0sCgogIGFkZENsYXNzTmFtZTogZnVuY3Rpb24oZWxlbWVudCwgY2xhc3NO",
				"YW1lKSB7CiAgICBpZiAoIShlbGVtZW50ID0gJChlbGVtZW50KSkpIHJldHVybjsKICAgIGlmICghRWxl",
				"bWVudC5oYXNDbGFzc05hbWUoZWxlbWVudCwgY2xhc3NOYW1lKSkKICAgICAgZWxlbWVudC5jbGFzc05h",
				"bWUgKz0gKGVsZW1lbnQuY2xhc3NOYW1lID8gJyAnIDogJycpICsgY2xhc3NOYW1lOwogICAgcmV0dXJu",
				"IGVsZW1lbnQ7CiAgfSwKCiAgcmVtb3ZlQ2xhc3NOYW1lOiBmdW5jdGlvbihlbGVtZW50LCBjbGFzc05h",
				"bWUpIHsKICAgIGlmICghKGVsZW1lbnQgPSAkKGVsZW1lbnQpKSkgcmV0dXJuOwogICAgZWxlbWVudC5j",
				"bGFzc05hbWUgPSBlbGVtZW50LmNsYXNzTmFtZS5yZXBsYWNlKAogICAgICBuZXcgUmVnRXhwKCIoXnxc",
				"XHMrKSIgKyBjbGFzc05hbWUgKyAiKFxccyt8JCkiKSwgJyAnKS5zdHJpcCgpOwogICAgcmV0dXJuIGVs",
				"ZW1lbnQ7CiAgfSwKCiAgdG9nZ2xlQ2xhc3NOYW1lOiBmdW5jdGlvbihlbGVtZW50LCBjbGFzc05hbWUp",
				"IHsKICAgIGlmICghKGVsZW1lbnQgPSAkKGVsZW1lbnQpKSkgcmV0dXJuOwogICAgcmV0dXJuIEVsZW1l",
				"bnRbRWxlbWVudC5oYXNDbGFzc05hbWUoZWxlbWVudCwgY2xhc3NOYW1lKSA/CiAgICAgICdyZW1vdmVD",
				"bGFzc05hbWUnIDogJ2FkZENsYXNzTmFtZSddKGVsZW1lbnQsIGNsYXNzTmFtZSk7CiAgfSwKCiAgY2xl",
				"YW5XaGl0ZXNwYWNlOiBmdW5jdGlvbihlbGVtZW50KSB7CiAgICBlbGVtZW50ID0gJChlbGVtZW50KTsK",
				"ICAgIHZhciBub2RlID0gZWxlbWVudC5maXJzdENoaWxkOwogICAgd2hpbGUgKG5vZGUpIHsKICAgICAg",
				"dmFyIG5leHROb2RlID0gbm9kZS5uZXh0U2libGluZzsKICAgICAgaWYgKG5vZGUubm9kZVR5cGUgPT0g",
				"MyAmJiAhL1xTLy50ZXN0KG5vZGUubm9kZVZhbHVlKSkKICAgICAgICBlbGVtZW50LnJlbW92ZUNoaWxk",
				"KG5vZGUpOwogICAgICBub2RlID0gbmV4dE5vZGU7CiAgICB9CiAgICByZXR1cm4gZWxlbWVudDsKICB9",
				"LAoKICBlbXB0eTogZnVuY3Rpb24oZWxlbWVudCkgewogICAgcmV0dXJuICQoZWxlbWVudCkuaW5uZXJI",
				"VE1MLmJsYW5rKCk7CiAgfSwKCiAgZGVzY2VuZGFudE9mOiBmdW5jdGlvbihlbGVtZW50LCBhbmNlc3Rv",
				"cikgewogICAgZWxlbWVudCA9ICQoZWxlbWVudCksIGFuY2VzdG9yID0gJChhbmNlc3Rvcik7CgogICAg",
				"aWYgKGVsZW1lbnQuY29tcGFyZURvY3VtZW50UG9zaXRpb24pCiAgICAgIHJldHVybiAoZWxlbWVudC5j",
				"b21wYXJlRG9jdW1lbnRQb3NpdGlvbihhbmNlc3RvcikgJiA4KSA9PT0gODsKCiAgICBpZiAoYW5jZXN0",
				"b3IuY29udGFpbnMpCiAgICAgIHJldHVybiBhbmNlc3Rvci5jb250YWlucyhlbGVtZW50KSAmJiBhbmNl",
				"c3RvciAhPT0gZWxlbWVudDsKCiAgICB3aGlsZSAoZWxlbWVudCA9IGVsZW1lbnQucGFyZW50Tm9kZSkK",
				"ICAgICAgaWYgKGVsZW1lbnQgPT0gYW5jZXN0b3IpIHJldHVybiB0cnVlOwoKICAgIHJldHVybiBmYWxz",
				"ZTsKICB9LAoKICBzY3JvbGxUbzogZnVuY3Rpb24oZWxlbWVudCkgewogICAgZWxlbWVudCA9ICQoZWxl",
				"bWVudCk7CiAgICB2YXIgcG9zID0gRWxlbWVudC5jdW11bGF0aXZlT2Zmc2V0KGVsZW1lbnQpOwogICAg",
				"d2luZG93LnNjcm9sbFRvKHBvc1swXSwgcG9zWzFdKTsKICAgIHJldHVybiBlbGVtZW50OwogIH0sCgog",
				"IGdldFN0eWxlOiBmdW5jdGlvbihlbGVtZW50LCBzdHlsZSkgewogICAgZWxlbWVudCA9ICQoZWxlbWVu",
				"dCk7CiAgICBzdHlsZSA9IHN0eWxlID09ICdmbG9hdCcgPyAnY3NzRmxvYXQnIDogc3R5bGUuY2FtZWxp",
				"emUoKTsKICAgIHZhciB2YWx1ZSA9IGVsZW1lbnQuc3R5bGVbc3R5bGVdOwogICAgaWYgKCF2YWx1ZSB8",
				"fCB2YWx1ZSA9PSAnYXV0bycpIHsKICAgICAgdmFyIGNzcyA9IGRvY3VtZW50LmRlZmF1bHRWaWV3Lmdl",
				"dENvbXB1dGVkU3R5bGUoZWxlbWVudCwgbnVsbCk7CiAgICAgIHZhbHVlID0gY3NzID8gY3NzW3N0eWxl",
				"XSA6IG51bGw7CiAgICB9CiAgICBpZiAoc3R5bGUgPT0gJ29wYWNpdHknKSByZXR1cm4gdmFsdWUgPyBw",
				"YXJzZUZsb2F0KHZhbHVlKSA6IDEuMDsKICAgIHJldHVybiB2YWx1ZSA9PSAnYXV0bycgPyBudWxsIDog",
				"dmFsdWU7CiAgfSwKCiAgZ2V0T3BhY2l0eTogZnVuY3Rpb24oZWxlbWVudCkgewogICAgcmV0dXJuICQo",
				"ZWxlbWVudCkuZ2V0U3R5bGUoJ29wYWNpdHknKTsKICB9LAoKICBzZXRTdHlsZTogZnVuY3Rpb24oZWxl",
				"bWVudCwgc3R5bGVzKSB7CiAgICBlbGVtZW50ID0gJChlbGVtZW50KTsKICAgIHZhciBlbGVtZW50U3R5",
				"bGUgPSBlbGVtZW50LnN0eWxlLCBtYXRjaDsKICAgIGlmIChPYmplY3QuaXNTdHJpbmcoc3R5bGVzKSkg",
				"ewogICAgICBlbGVtZW50LnN0eWxlLmNzc1RleHQgKz0gJzsnICsgc3R5bGVzOwogICAgICByZXR1cm4g",
				"c3R5bGVzLmluY2x1ZGUoJ29wYWNpdHknKSA/CiAgICAgICAgZWxlbWVudC5zZXRPcGFjaXR5KHN0eWxl",
				"cy5tYXRjaCgvb3BhY2l0eTpccyooXGQ/XC4/XGQqKS8pWzFdKSA6IGVsZW1lbnQ7CiAgICB9CiAgICBm",
				"b3IgKHZhciBwcm9wZXJ0eSBpbiBzdHlsZXMpCiAgICAgIGlmIChwcm9wZXJ0eSA9PSAnb3BhY2l0eScp",
				"IGVsZW1lbnQuc2V0T3BhY2l0eShzdHlsZXNbcHJvcGVydHldKTsKICAgICAgZWxzZQogICAgICAgIGVs",
				"ZW1lbnRTdHlsZVsocHJvcGVydHkgPT0gJ2Zsb2F0JyB8fCBwcm9wZXJ0eSA9PSAnY3NzRmxvYXQnKSA/",
				"CiAgICAgICAgICAoT2JqZWN0LmlzVW5kZWZpbmVkKGVsZW1lbnRTdHlsZS5zdHlsZUZsb2F0KSA/ICdj",
				"c3NGbG9hdCcgOiAnc3R5bGVGbG9hdCcpIDoKICAgICAgICAgICAgcHJvcGVydHldID0gc3R5bGVzW3By",
				"b3BlcnR5XTsKCiAgICByZXR1cm4gZWxlbWVudDsKICB9LAoKICBzZXRPcGFjaXR5OiBmdW5jdGlvbihl",
				"bGVtZW50LCB2YWx1ZSkgewogICAgZWxlbWVudCA9ICQoZWxlbWVudCk7CiAgICBlbGVtZW50LnN0eWxl",
				"Lm9wYWNpdHkgPSAodmFsdWUgPT0gMSB8fCB2YWx1ZSA9PT0gJycpID8gJycgOgogICAgICAodmFsdWUg",
				"PCAwLjAwMDAxKSA/IDAgOiB2YWx1ZTsKICAgIHJldHVybiBlbGVtZW50OwogIH0sCgogIG1ha2VQb3Np",
				"dGlvbmVkOiBmdW5jdGlvbihlbGVtZW50KSB7CiAgICBlbGVtZW50ID0gJChlbGVtZW50KTsKICAgIHZh",
				"ciBwb3MgPSBFbGVtZW50LmdldFN0eWxlKGVsZW1lbnQsICdwb3NpdGlvbicpOwogICAgaWYgKHBvcyA9",
				"PSAnc3RhdGljJyB8fCAhcG9zKSB7CiAgICAgIGVsZW1lbnQuX21hZGVQb3NpdGlvbmVkID0gdHJ1ZTsK",
				"ICAgICAgZWxlbWVudC5zdHlsZS5wb3NpdGlvbiA9ICdyZWxhdGl2ZSc7CiAgICAgIGlmIChQcm90b3R5",
				"cGUuQnJvd3Nlci5PcGVyYSkgewogICAgICAgIGVsZW1lbnQuc3R5bGUudG9wID0gMDsKICAgICAgICBl",
				"bGVtZW50LnN0eWxlLmxlZnQgPSAwOwogICAgICB9CiAgICB9CiAgICByZXR1cm4gZWxlbWVudDsKICB9",
				"LAoKICB1bmRvUG9zaXRpb25lZDogZnVuY3Rpb24oZWxlbWVudCkgewogICAgZWxlbWVudCA9ICQoZWxl",
				"bWVudCk7CiAgICBpZiAoZWxlbWVudC5fbWFkZVBvc2l0aW9uZWQpIHsKICAgICAgZWxlbWVudC5fbWFk",
				"ZVBvc2l0aW9uZWQgPSB1bmRlZmluZWQ7CiAgICAgIGVsZW1lbnQuc3R5bGUucG9zaXRpb24gPQogICAg",
				"ICAgIGVsZW1lbnQuc3R5bGUudG9wID0KICAgICAgICBlbGVtZW50LnN0eWxlLmxlZnQgPQogICAgICAg",
				"IGVsZW1lbnQuc3R5bGUuYm90dG9tID0KICAgICAgICBlbGVtZW50LnN0eWxlLnJpZ2h0ID0gJyc7CiAg",
				"ICB9CiAgICByZXR1cm4gZWxlbWVudDsKICB9LAoKICBtYWtlQ2xpcHBpbmc6IGZ1bmN0aW9uKGVsZW1l",
				"bnQpIHsKICAgIGVsZW1lbnQgPSAkKGVsZW1lbnQpOwogICAgaWYgKGVsZW1lbnQuX292ZXJmbG93KSBy",
				"ZXR1cm4gZWxlbWVudDsKICAgIGVsZW1lbnQuX292ZXJmbG93ID0gRWxlbWVudC5nZXRTdHlsZShlbGVt",
				"ZW50LCAnb3ZlcmZsb3cnKSB8fCAnYXV0byc7CiAgICBpZiAoZWxlbWVudC5fb3ZlcmZsb3cgIT09ICdo",
				"aWRkZW4nKQogICAgICBlbGVtZW50LnN0eWxlLm92ZXJmbG93ID0gJ2hpZGRlbic7CiAgICByZXR1cm4g",
				"ZWxlbWVudDsKICB9LAoKICB1bmRvQ2xpcHBpbmc6IGZ1bmN0aW9uKGVsZW1lbnQpIHsKICAgIGVsZW1l",
				"bnQgPSAkKGVsZW1lbnQpOwogICAgaWYgKCFlbGVtZW50Ll9vdmVyZmxvdykgcmV0dXJuIGVsZW1lbnQ7",
				"CiAgICBlbGVtZW50LnN0eWxlLm92ZXJmbG93ID0gZWxlbWVudC5fb3ZlcmZsb3cgPT0gJ2F1dG8nID8g",
				"JycgOiBlbGVtZW50Ll9vdmVyZmxvdzsKICAgIGVsZW1lbnQuX292ZXJmbG93ID0gbnVsbDsKICAgIHJl",
				"dHVybiBlbGVtZW50OwogIH0sCgogIGNsb25lUG9zaXRpb246IGZ1bmN0aW9uKGVsZW1lbnQsIHNvdXJj",
				"ZSkgewogICAgdmFyIG9wdGlvbnMgPSBPYmplY3QuZXh0ZW5kKHsKICAgICAgc2V0TGVmdDogICAgdHJ1",
				"ZSwKICAgICAgc2V0VG9wOiAgICAgdHJ1ZSwKICAgICAgc2V0V2lkdGg6ICAgdHJ1ZSwKICAgICAgc2V0",
				"SGVpZ2h0OiAgdHJ1ZSwKICAgICAgb2Zmc2V0VG9wOiAgMCwKICAgICAgb2Zmc2V0TGVmdDogMAogICAg",
				"fSwgYXJndW1lbnRzWzJdIHx8IHsgfSk7CgogICAgc291cmNlID0gJChzb3VyY2UpOwogICAgdmFyIHAg",
				"PSBFbGVtZW50LnZpZXdwb3J0T2Zmc2V0KHNvdXJjZSksIGRlbHRhID0gWzAsIDBdLCBwYXJlbnQgPSBu",
				"dWxsOwoKICAgIGVsZW1lbnQgPSAkKGVsZW1lbnQpOwoKICAgIGlmIChFbGVtZW50LmdldFN0eWxlKGVs",
				"ZW1lbnQsICdwb3NpdGlvbicpID09ICdhYnNvbHV0ZScpIHsKICAgICAgcGFyZW50ID0gRWxlbWVudC5n",
				"ZXRPZmZzZXRQYXJlbnQoZWxlbWVudCk7CiAgICAgIGRlbHRhID0gRWxlbWVudC52aWV3cG9ydE9mZnNl",
				"dChwYXJlbnQpOwogICAgfQoKICAgIGlmIChwYXJlbnQgPT0gZG9jdW1lbnQuYm9keSkgewogICAgICBk",
				"ZWx0YVswXSAtPSBkb2N1bWVudC5ib2R5Lm9mZnNldExlZnQ7CiAgICAgIGRlbHRhWzFdIC09IGRvY3Vt",
				"ZW50LmJvZHkub2Zmc2V0VG9wOwogICAgfQoKICAgIGlmIChvcHRpb25zLnNldExlZnQpICAgZWxlbWVu",
				"dC5zdHlsZS5sZWZ0ICA9IChwWzBdIC0gZGVsdGFbMF0gKyBvcHRpb25zLm9mZnNldExlZnQpICsgJ3B4",
				"JzsKICAgIGlmIChvcHRpb25zLnNldFRvcCkgICAgZWxlbWVudC5zdHlsZS50b3AgICA9IChwWzFdIC0g",
				"ZGVsdGFbMV0gKyBvcHRpb25zLm9mZnNldFRvcCkgKyAncHgnOwogICAgaWYgKG9wdGlvbnMuc2V0V2lk",
				"dGgpICBlbGVtZW50LnN0eWxlLndpZHRoID0gc291cmNlLm9mZnNldFdpZHRoICsgJ3B4JzsKICAgIGlm",
				"IChvcHRpb25zLnNldEhlaWdodCkgZWxlbWVudC5zdHlsZS5oZWlnaHQgPSBzb3VyY2Uub2Zmc2V0SGVp",
				"Z2h0ICsgJ3B4JzsKICAgIHJldHVybiBlbGVtZW50OwogIH0KfTsKCk9iamVjdC5leHRlbmQoRWxlbWVu",
				"dC5NZXRob2RzLCB7CiAgZ2V0RWxlbWVudHNCeVNlbGVjdG9yOiBFbGVtZW50Lk1ldGhvZHMuc2VsZWN0",
				"LAoKICBjaGlsZEVsZW1lbnRzOiBFbGVtZW50Lk1ldGhvZHMuaW1tZWRpYXRlRGVzY2VuZGFudHMKfSk7",
				"CgpFbGVtZW50Ll9hdHRyaWJ1dGVUcmFuc2xhdGlvbnMgPSB7CiAgd3JpdGU6IHsKICAgIG5hbWVzOiB7",
				"CiAgICAgIGNsYXNzTmFtZTogJ2NsYXNzJywKICAgICAgaHRtbEZvcjogICAnZm9yJwogICAgfSwKICAg",
				"IHZhbHVlczogeyB9CiAgfQp9OwoKaWYgKFByb3RvdHlwZS5Ccm93c2VyLk9wZXJhKSB7CiAgRWxlbWVu",
				"dC5NZXRob2RzLmdldFN0eWxlID0gRWxlbWVudC5NZXRob2RzLmdldFN0eWxlLndyYXAoCiAgICBmdW5j",
				"dGlvbihwcm9jZWVkLCBlbGVtZW50LCBzdHlsZSkgewogICAgICBzd2l0Y2ggKHN0eWxlKSB7CiAgICAg",
				"ICAgY2FzZSAnaGVpZ2h0JzogY2FzZSAnd2lkdGgnOgogICAgICAgICAgaWYgKCFFbGVtZW50LnZpc2li",
				"bGUoZWxlbWVudCkpIHJldHVybiBudWxsOwoKICAgICAgICAgIHZhciBkaW0gPSBwYXJzZUludChwcm9j",
				"ZWVkKGVsZW1lbnQsIHN0eWxlKSwgMTApOwoKICAgICAgICAgIGlmIChkaW0gIT09IGVsZW1lbnRbJ29m",
				"ZnNldCcgKyBzdHlsZS5jYXBpdGFsaXplKCldKQogICAgICAgICAgICByZXR1cm4gZGltICsgJ3B4JzsK",
				"CiAgICAgICAgICB2YXIgcHJvcGVydGllczsKICAgICAgICAgIGlmIChzdHlsZSA9PT0gJ2hlaWdodCcp",
				"IHsKICAgICAgICAgICAgcHJvcGVydGllcyA9IFsnYm9yZGVyLXRvcC13aWR0aCcsICdwYWRkaW5nLXRv",
				"cCcsCiAgICAgICAgICAgICAncGFkZGluZy1ib3R0b20nLCAnYm9yZGVyLWJvdHRvbS13aWR0aCddOwog",
				"ICAgICAgICAgfQogICAgICAgICAgZWxzZSB7CiAgICAgICAgICAgIHByb3BlcnRpZXMgPSBbJ2JvcmRl",
				"ci1sZWZ0LXdpZHRoJywgJ3BhZGRpbmctbGVmdCcsCiAgICAgICAgICAgICAncGFkZGluZy1yaWdodCcs",
				"ICdib3JkZXItcmlnaHQtd2lkdGgnXTsKICAgICAgICAgIH0KICAgICAgICAgIHJldHVybiBwcm9wZXJ0",
				"aWVzLmluamVjdChkaW0sIGZ1bmN0aW9uKG1lbW8sIHByb3BlcnR5KSB7CiAgICAgICAgICAgIHZhciB2",
				"YWwgPSBwcm9jZWVkKGVsZW1lbnQsIHByb3BlcnR5KTsKICAgICAgICAgICAgcmV0dXJuIHZhbCA9PT0g",
				"bnVsbCA/IG1lbW8gOiBtZW1vIC0gcGFyc2VJbnQodmFsLCAxMCk7CiAgICAgICAgICB9KSArICdweCc7",
				"CiAgICAgICAgZGVmYXVsdDogcmV0dXJuIHByb2NlZWQoZWxlbWVudCwgc3R5bGUpOwogICAgICB9CiAg",
				"ICB9CiAgKTsKCiAgRWxlbWVudC5NZXRob2RzLnJlYWRBdHRyaWJ1dGUgPSBFbGVtZW50Lk1ldGhvZHMu",
				"cmVhZEF0dHJpYnV0ZS53cmFwKAogICAgZnVuY3Rpb24ocHJvY2VlZCwgZWxlbWVudCwgYXR0cmlidXRl",
				"KSB7CiAgICAgIGlmIChhdHRyaWJ1dGUgPT09ICd0aXRsZScpIHJldHVybiBlbGVtZW50LnRpdGxlOwog",
				"ICAgICByZXR1cm4gcHJvY2VlZChlbGVtZW50LCBhdHRyaWJ1dGUpOwogICAgfQogICk7Cn0KCmVsc2Ug",
				"aWYgKFByb3RvdHlwZS5Ccm93c2VyLklFKSB7CiAgRWxlbWVudC5NZXRob2RzLmdldFN0eWxlID0gZnVu",
				"Y3Rpb24oZWxlbWVudCwgc3R5bGUpIHsKICAgIGVsZW1lbnQgPSAkKGVsZW1lbnQpOwogICAgc3R5bGUg",
				"PSAoc3R5bGUgPT0gJ2Zsb2F0JyB8fCBzdHlsZSA9PSAnY3NzRmxvYXQnKSA/ICdzdHlsZUZsb2F0JyA6",
				"IHN0eWxlLmNhbWVsaXplKCk7CiAgICB2YXIgdmFsdWUgPSBlbGVtZW50LnN0eWxlW3N0eWxlXTsKICAg",
				"IGlmICghdmFsdWUgJiYgZWxlbWVudC5jdXJyZW50U3R5bGUpIHZhbHVlID0gZWxlbWVudC5jdXJyZW50",
				"U3R5bGVbc3R5bGVdOwoKICAgIGlmIChzdHlsZSA9PSAnb3BhY2l0eScpIHsKICAgICAgaWYgKHZhbHVl",
				"ID0gKGVsZW1lbnQuZ2V0U3R5bGUoJ2ZpbHRlcicpIHx8ICcnKS5tYXRjaCgvYWxwaGFcKG9wYWNpdHk9",
				"KC4qKVwpLykpCiAgICAgICAgaWYgKHZhbHVlWzFdKSByZXR1cm4gcGFyc2VGbG9hdCh2YWx1ZVsxXSkg",
				"LyAxMDA7CiAgICAgIHJldHVybiAxLjA7CiAgICB9CgogICAgaWYgKHZhbHVlID09ICdhdXRvJykgewog",
				"ICAgICBpZiAoKHN0eWxlID09ICd3aWR0aCcgfHwgc3R5bGUgPT0gJ2hlaWdodCcpICYmIChlbGVtZW50",
				"LmdldFN0eWxlKCdkaXNwbGF5JykgIT0gJ25vbmUnKSkKICAgICAgICByZXR1cm4gZWxlbWVudFsnb2Zm",
				"c2V0JyArIHN0eWxlLmNhcGl0YWxpemUoKV0gKyAncHgnOwogICAgICByZXR1cm4gbnVsbDsKICAgIH0K",
				"ICAgIHJldHVybiB2YWx1ZTsKICB9OwoKICBFbGVtZW50Lk1ldGhvZHMuc2V0T3BhY2l0eSA9IGZ1bmN0",
				"aW9uKGVsZW1lbnQsIHZhbHVlKSB7CiAgICBmdW5jdGlvbiBzdHJpcEFscGhhKGZpbHRlcil7CiAgICAg",
				"IHJldHVybiBmaWx0ZXIucmVwbGFjZSgvYWxwaGFcKFteXCldKlwpL2dpLCcnKTsKICAgIH0KICAgIGVs",
				"ZW1lbnQgPSAkKGVsZW1lbnQpOwogICAgdmFyIGN1cnJlbnRTdHlsZSA9IGVsZW1lbnQuY3VycmVudFN0",
				"eWxlOwogICAgaWYgKChjdXJyZW50U3R5bGUgJiYgIWN1cnJlbnRTdHlsZS5oYXNMYXlvdXQpIHx8CiAg",
				"ICAgICghY3VycmVudFN0eWxlICYmIGVsZW1lbnQuc3R5bGUuem9vbSA9PSAnbm9ybWFsJykpCiAgICAg",
				"ICAgZWxlbWVudC5zdHlsZS56b29tID0gMTsKCiAgICB2YXIgZmlsdGVyID0gZWxlbWVudC5nZXRTdHls",
				"ZSgnZmlsdGVyJyksIHN0eWxlID0gZWxlbWVudC5zdHlsZTsKICAgIGlmICh2YWx1ZSA9PSAxIHx8IHZh",
				"bHVlID09PSAnJykgewogICAgICAoZmlsdGVyID0gc3RyaXBBbHBoYShmaWx0ZXIpKSA/CiAgICAgICAg",
				"c3R5bGUuZmlsdGVyID0gZmlsdGVyIDogc3R5bGUucmVtb3ZlQXR0cmlidXRlKCdmaWx0ZXInKTsKICAg",
				"ICAgcmV0dXJuIGVsZW1lbnQ7CiAgICB9IGVsc2UgaWYgKHZhbHVlIDwgMC4wMDAwMSkgdmFsdWUgPSAw",
				"OwogICAgc3R5bGUuZmlsdGVyID0gc3RyaXBBbHBoYShmaWx0ZXIpICsKICAgICAgJ2FscGhhKG9wYWNp",
				"dHk9JyArICh2YWx1ZSAqIDEwMCkgKyAnKSc7CiAgICByZXR1cm4gZWxlbWVudDsKICB9OwoKICBFbGVt",
				"ZW50Ll9hdHRyaWJ1dGVUcmFuc2xhdGlvbnMgPSAoZnVuY3Rpb24oKXsKCiAgICB2YXIgY2xhc3NQcm9w",
				"ID0gJ2NsYXNzTmFtZScsCiAgICAgICAgZm9yUHJvcCA9ICdmb3InLAogICAgICAgIGVsID0gZG9jdW1l",
				"bnQuY3JlYXRlRWxlbWVudCgnZGl2Jyk7CgogICAgZWwuc2V0QXR0cmlidXRlKGNsYXNzUHJvcCwgJ3gn",
				"KTsKCiAgICBpZiAoZWwuY2xhc3NOYW1lICE9PSAneCcpIHsKICAgICAgZWwuc2V0QXR0cmlidXRlKCdj",
				"bGFzcycsICd4Jyk7CiAgICAgIGlmIChlbC5jbGFzc05hbWUgPT09ICd4JykgewogICAgICAgIGNsYXNz",
				"UHJvcCA9ICdjbGFzcyc7CiAgICAgIH0KICAgIH0KICAgIGVsID0gbnVsbDsKCiAgICBlbCA9IGRvY3Vt",
				"ZW50LmNyZWF0ZUVsZW1lbnQoJ2xhYmVsJyk7CiAgICBlbC5zZXRBdHRyaWJ1dGUoZm9yUHJvcCwgJ3gn",
				"KTsKICAgIGlmIChlbC5odG1sRm9yICE9PSAneCcpIHsKICAgICAgZWwuc2V0QXR0cmlidXRlKCdodG1s",
				"Rm9yJywgJ3gnKTsKICAgICAgaWYgKGVsLmh0bWxGb3IgPT09ICd4JykgewogICAgICAgIGZvclByb3Ag",
				"PSAnaHRtbEZvcic7CiAgICAgIH0KICAgIH0KICAgIGVsID0gbnVsbDsKCiAgICByZXR1cm4gewogICAg",
				"ICByZWFkOiB7CiAgICAgICAgbmFtZXM6IHsKICAgICAgICAgICdjbGFzcyc6ICAgICAgY2xhc3NQcm9w",
				"LAogICAgICAgICAgJ2NsYXNzTmFtZSc6ICBjbGFzc1Byb3AsCiAgICAgICAgICAnZm9yJzogICAgICAg",
				"IGZvclByb3AsCiAgICAgICAgICAnaHRtbEZvcic6ICAgIGZvclByb3AKICAgICAgICB9LAogICAgICAg",
				"IHZhbHVlczogewogICAgICAgICAgX2dldEF0dHI6IGZ1bmN0aW9uKGVsZW1lbnQsIGF0dHJpYnV0ZSkg",
				"ewogICAgICAgICAgICByZXR1cm4gZWxlbWVudC5nZXRBdHRyaWJ1dGUoYXR0cmlidXRlKTsKICAgICAg",
				"ICAgIH0sCiAgICAgICAgICBfZ2V0QXR0cjI6IGZ1bmN0aW9uKGVsZW1lbnQsIGF0dHJpYnV0ZSkgewog",
				"ICAgICAgICAgICByZXR1cm4gZWxlbWVudC5nZXRBdHRyaWJ1dGUoYXR0cmlidXRlLCAyKTsKICAgICAg",
				"ICAgIH0sCiAgICAgICAgICBfZ2V0QXR0ck5vZGU6IGZ1bmN0aW9uKGVsZW1lbnQsIGF0dHJpYnV0ZSkg",
				"ewogICAgICAgICAgICB2YXIgbm9kZSA9IGVsZW1lbnQuZ2V0QXR0cmlidXRlTm9kZShhdHRyaWJ1dGUp",
				"OwogICAgICAgICAgICByZXR1cm4gbm9kZSA/IG5vZGUudmFsdWUgOiAiIjsKICAgICAgICAgIH0sCiAg",
				"ICAgICAgICBfZ2V0RXY6IChmdW5jdGlvbigpewoKICAgICAgICAgICAgdmFyIGVsID0gZG9jdW1lbnQu",
				"Y3JlYXRlRWxlbWVudCgnZGl2JyksIGY7CiAgICAgICAgICAgIGVsLm9uY2xpY2sgPSBQcm90b3R5cGUu",
				"ZW1wdHlGdW5jdGlvbjsKICAgICAgICAgICAgdmFyIHZhbHVlID0gZWwuZ2V0QXR0cmlidXRlKCdvbmNs",
				"aWNrJyk7CgogICAgICAgICAgICBpZiAoU3RyaW5nKHZhbHVlKS5pbmRleE9mKCd7JykgPiAtMSkgewog",
				"ICAgICAgICAgICAgIGYgPSBmdW5jdGlvbihlbGVtZW50LCBhdHRyaWJ1dGUpIHsKICAgICAgICAgICAg",
				"ICAgIGF0dHJpYnV0ZSA9IGVsZW1lbnQuZ2V0QXR0cmlidXRlKGF0dHJpYnV0ZSk7CiAgICAgICAgICAg",
				"ICAgICBpZiAoIWF0dHJpYnV0ZSkgcmV0dXJuIG51bGw7CiAgICAgICAgICAgICAgICBhdHRyaWJ1dGUg",
				"PSBhdHRyaWJ1dGUudG9TdHJpbmcoKTsKICAgICAgICAgICAgICAgIGF0dHJpYnV0ZSA9IGF0dHJpYnV0",
				"ZS5zcGxpdCgneycpWzFdOwogICAgICAgICAgICAgICAgYXR0cmlidXRlID0gYXR0cmlidXRlLnNwbGl0",
				"KCd9JylbMF07CiAgICAgICAgICAgICAgICByZXR1cm4gYXR0cmlidXRlLnN0cmlwKCk7CiAgICAgICAg",
				"ICAgICAgfTsKICAgICAgICAgICAgfQogICAgICAgICAgICBlbHNlIGlmICh2YWx1ZSA9PT0gJycpIHsK",
				"ICAgICAgICAgICAgICBmID0gZnVuY3Rpb24oZWxlbWVudCwgYXR0cmlidXRlKSB7CiAgICAgICAgICAg",
				"ICAgICBhdHRyaWJ1dGUgPSBlbGVtZW50LmdldEF0dHJpYnV0ZShhdHRyaWJ1dGUpOwogICAgICAgICAg",
				"ICAgICAgaWYgKCFhdHRyaWJ1dGUpIHJldHVybiBudWxsOwogICAgICAgICAgICAgICAgcmV0dXJuIGF0",
				"dHJpYnV0ZS5zdHJpcCgpOwogICAgICAgICAgICAgIH07CiAgICAgICAgICAgIH0KICAgICAgICAgICAg",
				"ZWwgPSBudWxsOwogICAgICAgICAgICByZXR1cm4gZjsKICAgICAgICAgIH0pKCksCiAgICAgICAgICBf",
				"ZmxhZzogZnVuY3Rpb24oZWxlbWVudCwgYXR0cmlidXRlKSB7CiAgICAgICAgICAgIHJldHVybiAkKGVs",
				"ZW1lbnQpLmhhc0F0dHJpYnV0ZShhdHRyaWJ1dGUpID8gYXR0cmlidXRlIDogbnVsbDsKICAgICAgICAg",
				"IH0sCiAgICAgICAgICBzdHlsZTogZnVuY3Rpb24oZWxlbWVudCkgewogICAgICAgICAgICByZXR1cm4g",
				"ZWxlbWVudC5zdHlsZS5jc3NUZXh0LnRvTG93ZXJDYXNlKCk7CiAgICAgICAgICB9LAogICAgICAgICAg",
				"dGl0bGU6IGZ1bmN0aW9uKGVsZW1lbnQpIHsKICAgICAgICAgICAgcmV0dXJuIGVsZW1lbnQudGl0bGU7",
				"CiAgICAgICAgICB9CiAgICAgICAgfQogICAgICB9CiAgICB9CiAgfSkoKTsKCiAgRWxlbWVudC5fYXR0",
				"cmlidXRlVHJhbnNsYXRpb25zLndyaXRlID0gewogICAgbmFtZXM6IE9iamVjdC5leHRlbmQoewogICAg",
				"ICBjZWxscGFkZGluZzogJ2NlbGxQYWRkaW5nJywKICAgICAgY2VsbHNwYWNpbmc6ICdjZWxsU3BhY2lu",
				"ZycKICAgIH0sIEVsZW1lbnQuX2F0dHJpYnV0ZVRyYW5zbGF0aW9ucy5yZWFkLm5hbWVzKSwKICAgIHZh",
				"bHVlczogewogICAgICBjaGVja2VkOiBmdW5jdGlvbihlbGVtZW50LCB2YWx1ZSkgewogICAgICAgIGVs",
				"ZW1lbnQuY2hlY2tlZCA9ICEhdmFsdWU7CiAgICAgIH0sCgogICAgICBzdHlsZTogZnVuY3Rpb24oZWxl",
				"bWVudCwgdmFsdWUpIHsKICAgICAgICBlbGVtZW50LnN0eWxlLmNzc1RleHQgPSB2YWx1ZSA/IHZhbHVl",
				"IDogJyc7CiAgICAgIH0KICAgIH0KICB9OwoKICBFbGVtZW50Ll9hdHRyaWJ1dGVUcmFuc2xhdGlvbnMu",
				"aGFzID0ge307CgogICR3KCdjb2xTcGFuIHJvd1NwYW4gdkFsaWduIGRhdGVUaW1lIGFjY2Vzc0tleSB0",
				"YWJJbmRleCAnICsKICAgICAgJ2VuY1R5cGUgbWF4TGVuZ3RoIHJlYWRPbmx5IGxvbmdEZXNjIGZyYW1l",
				"Qm9yZGVyJykuZWFjaChmdW5jdGlvbihhdHRyKSB7CiAgICBFbGVtZW50Ll9hdHRyaWJ1dGVUcmFuc2xh",
				"dGlvbnMud3JpdGUubmFtZXNbYXR0ci50b0xvd2VyQ2FzZSgpXSA9IGF0dHI7CiAgICBFbGVtZW50Ll9h",
				"dHRyaWJ1dGVUcmFuc2xhdGlvbnMuaGFzW2F0dHIudG9Mb3dlckNhc2UoKV0gPSBhdHRyOwogIH0pOwoK",
				"ICAoZnVuY3Rpb24odikgewogICAgT2JqZWN0LmV4dGVuZCh2LCB7CiAgICAgIGhyZWY6ICAgICAgICB2",
				"Ll9nZXRBdHRyMiwKICAgICAgc3JjOiAgICAgICAgIHYuX2dldEF0dHIyLAogICAgICB0eXBlOiAgICAg",
				"ICAgdi5fZ2V0QXR0ciwKICAgICAgYWN0aW9uOiAgICAgIHYuX2dldEF0dHJOb2RlLAogICAgICBkaXNh",
				"YmxlZDogICAgdi5fZmxhZywKICAgICAgY2hlY2tlZDogICAgIHYuX2ZsYWcsCiAgICAgIHJlYWRvbmx5",
				"OiAgICB2Ll9mbGFnLAogICAgICBtdWx0aXBsZTogICAgdi5fZmxhZywKICAgICAgb25sb2FkOiAgICAg",
				"IHYuX2dldEV2LAogICAgICBvbnVubG9hZDogICAgdi5fZ2V0RXYsCiAgICAgIG9uY2xpY2s6ICAgICB2",
				"Ll9nZXRFdiwKICAgICAgb25kYmxjbGljazogIHYuX2dldEV2LAogICAgICBvbm1vdXNlZG93bjogdi5f",
				"Z2V0RXYsCiAgICAgIG9ubW91c2V1cDogICB2Ll9nZXRFdiwKICAgICAgb25tb3VzZW92ZXI6IHYuX2dl",
				"dEV2LAogICAgICBvbm1vdXNlbW92ZTogdi5fZ2V0RXYsCiAgICAgIG9ubW91c2VvdXQ6ICB2Ll9nZXRF",
				"diwKICAgICAgb25mb2N1czogICAgIHYuX2dldEV2LAogICAgICBvbmJsdXI6ICAgICAgdi5fZ2V0RXYs",
				"CiAgICAgIG9ua2V5cHJlc3M6ICB2Ll9nZXRFdiwKICAgICAgb25rZXlkb3duOiAgIHYuX2dldEV2LAog",
				"ICAgICBvbmtleXVwOiAgICAgdi5fZ2V0RXYsCiAgICAgIG9uc3VibWl0OiAgICB2Ll9nZXRFdiwKICAg",
				"ICAgb25yZXNldDogICAgIHYuX2dldEV2LAogICAgICBvbnNlbGVjdDogICAgdi5fZ2V0RXYsCiAgICAg",
				"IG9uY2hhbmdlOiAgICB2Ll9nZXRFdgogICAgfSk7CiAgfSkoRWxlbWVudC5fYXR0cmlidXRlVHJhbnNs",
				"YXRpb25zLnJlYWQudmFsdWVzKTsKCiAgaWYgKFByb3RvdHlwZS5Ccm93c2VyRmVhdHVyZXMuRWxlbWVu",
				"dEV4dGVuc2lvbnMpIHsKICAgIChmdW5jdGlvbigpIHsKICAgICAgZnVuY3Rpb24gX2Rlc2NlbmRhbnRz",
				"KGVsZW1lbnQpIHsKICAgICAgICB2YXIgbm9kZXMgPSBlbGVtZW50LmdldEVsZW1lbnRzQnlUYWdOYW1l",
				"KCcqJyksIHJlc3VsdHMgPSBbXTsKICAgICAgICBmb3IgKHZhciBpID0gMCwgbm9kZTsgbm9kZSA9IG5v",
				"ZGVzW2ldOyBpKyspCiAgICAgICAgICBpZiAobm9kZS50YWdOYW1lICE9PSAiISIpIC8vIEZpbHRlciBv",
				"dXQgY29tbWVudCBub2Rlcy4KICAgICAgICAgICAgcmVzdWx0cy5wdXNoKG5vZGUpOwogICAgICAgIHJl",
				"dHVybiByZXN1bHRzOwogICAgICB9CgogICAgICBFbGVtZW50Lk1ldGhvZHMuZG93biA9IGZ1bmN0aW9u",
				"KGVsZW1lbnQsIGV4cHJlc3Npb24sIGluZGV4KSB7CiAgICAgICAgZWxlbWVudCA9ICQoZWxlbWVudCk7",
				"CiAgICAgICAgaWYgKGFyZ3VtZW50cy5sZW5ndGggPT0gMSkgcmV0dXJuIGVsZW1lbnQuZmlyc3REZXNj",
				"ZW5kYW50KCk7CiAgICAgICAgcmV0dXJuIE9iamVjdC5pc051bWJlcihleHByZXNzaW9uKSA/IF9kZXNj",
				"ZW5kYW50cyhlbGVtZW50KVtleHByZXNzaW9uXSA6CiAgICAgICAgICBFbGVtZW50LnNlbGVjdChlbGVt",
				"ZW50LCBleHByZXNzaW9uKVtpbmRleCB8fCAwXTsKICAgICAgfQogICAgfSkoKTsKICB9Cgp9CgplbHNl",
				"IGlmIChQcm90b3R5cGUuQnJvd3Nlci5HZWNrbyAmJiAvcnY6MVwuOFwuMC8udGVzdChuYXZpZ2F0b3Iu",
				"dXNlckFnZW50KSkgewogIEVsZW1lbnQuTWV0aG9kcy5zZXRPcGFjaXR5ID0gZnVuY3Rpb24oZWxlbWVu",
				"dCwgdmFsdWUpIHsKICAgIGVsZW1lbnQgPSAkKGVsZW1lbnQpOwogICAgZWxlbWVudC5zdHlsZS5vcGFj",
				"aXR5ID0gKHZhbHVlID09IDEpID8gMC45OTk5OTkgOgogICAgICAodmFsdWUgPT09ICcnKSA/ICcnIDog",
				"KHZhbHVlIDwgMC4wMDAwMSkgPyAwIDogdmFsdWU7CiAgICByZXR1cm4gZWxlbWVudDsKICB9Owp9Cgpl",
				"bHNlIGlmIChQcm90b3R5cGUuQnJvd3Nlci5XZWJLaXQpIHsKICBFbGVtZW50Lk1ldGhvZHMuc2V0T3Bh",
				"Y2l0eSA9IGZ1bmN0aW9uKGVsZW1lbnQsIHZhbHVlKSB7CiAgICBlbGVtZW50ID0gJChlbGVtZW50KTsK",
				"ICAgIGVsZW1lbnQuc3R5bGUub3BhY2l0eSA9ICh2YWx1ZSA9PSAxIHx8IHZhbHVlID09PSAnJykgPyAn",
				"JyA6CiAgICAgICh2YWx1ZSA8IDAuMDAwMDEpID8gMCA6IHZhbHVlOwoKICAgIGlmICh2YWx1ZSA9PSAx",
				"KQogICAgICBpZiAoZWxlbWVudC50YWdOYW1lLnRvVXBwZXJDYXNlKCkgPT0gJ0lNRycgJiYgZWxlbWVu",
				"dC53aWR0aCkgewogICAgICAgIGVsZW1lbnQud2lkdGgrKzsgZWxlbWVudC53aWR0aC0tOwogICAgICB9",
				"IGVsc2UgdHJ5IHsKICAgICAgICB2YXIgbiA9IGRvY3VtZW50LmNyZWF0ZVRleHROb2RlKCcgJyk7CiAg",
				"ICAgICAgZWxlbWVudC5hcHBlbmRDaGlsZChuKTsKICAgICAgICBlbGVtZW50LnJlbW92ZUNoaWxkKG4p",
				"OwogICAgICB9IGNhdGNoIChlKSB7IH0KCiAgICByZXR1cm4gZWxlbWVudDsKICB9Owp9CgppZiAoJ291",
				"dGVySFRNTCcgaW4gZG9jdW1lbnQuZG9jdW1lbnRFbGVtZW50KSB7CiAgRWxlbWVudC5NZXRob2RzLnJl",
				"cGxhY2UgPSBmdW5jdGlvbihlbGVtZW50LCBjb250ZW50KSB7CiAgICBlbGVtZW50ID0gJChlbGVtZW50",
				"KTsKCiAgICBpZiAoY29udGVudCAmJiBjb250ZW50LnRvRWxlbWVudCkgY29udGVudCA9IGNvbnRlbnQu",
				"dG9FbGVtZW50KCk7CiAgICBpZiAoT2JqZWN0LmlzRWxlbWVudChjb250ZW50KSkgewogICAgICBlbGVt",
				"ZW50LnBhcmVudE5vZGUucmVwbGFjZUNoaWxkKGNvbnRlbnQsIGVsZW1lbnQpOwogICAgICByZXR1cm4g",
				"ZWxlbWVudDsKICAgIH0KCiAgICBjb250ZW50ID0gT2JqZWN0LnRvSFRNTChjb250ZW50KTsKICAgIHZh",
				"ciBwYXJlbnQgPSBlbGVtZW50LnBhcmVudE5vZGUsIHRhZ05hbWUgPSBwYXJlbnQudGFnTmFtZS50b1Vw",
				"cGVyQ2FzZSgpOwoKICAgIGlmIChFbGVtZW50Ll9pbnNlcnRpb25UcmFuc2xhdGlvbnMudGFnc1t0YWdO",
				"YW1lXSkgewogICAgICB2YXIgbmV4dFNpYmxpbmcgPSBlbGVtZW50Lm5leHQoKSwKICAgICAgICAgIGZy",
				"YWdtZW50cyA9IEVsZW1lbnQuX2dldENvbnRlbnRGcm9tQW5vbnltb3VzRWxlbWVudCh0YWdOYW1lLCBj",
				"b250ZW50LnN0cmlwU2NyaXB0cygpKTsKICAgICAgcGFyZW50LnJlbW92ZUNoaWxkKGVsZW1lbnQpOwog",
				"ICAgICBpZiAobmV4dFNpYmxpbmcpCiAgICAgICAgZnJhZ21lbnRzLmVhY2goZnVuY3Rpb24obm9kZSkg",
				"eyBwYXJlbnQuaW5zZXJ0QmVmb3JlKG5vZGUsIG5leHRTaWJsaW5nKSB9KTsKICAgICAgZWxzZQogICAg",
				"ICAgIGZyYWdtZW50cy5lYWNoKGZ1bmN0aW9uKG5vZGUpIHsgcGFyZW50LmFwcGVuZENoaWxkKG5vZGUp",
				"IH0pOwogICAgfQogICAgZWxzZSBlbGVtZW50Lm91dGVySFRNTCA9IGNvbnRlbnQuc3RyaXBTY3JpcHRz",
				"KCk7CgogICAgY29udGVudC5ldmFsU2NyaXB0cy5iaW5kKGNvbnRlbnQpLmRlZmVyKCk7CiAgICByZXR1",
				"cm4gZWxlbWVudDsKICB9Owp9CgpFbGVtZW50Ll9yZXR1cm5PZmZzZXQgPSBmdW5jdGlvbihsLCB0KSB7",
				"CiAgdmFyIHJlc3VsdCA9IFtsLCB0XTsKICByZXN1bHQubGVmdCA9IGw7CiAgcmVzdWx0LnRvcCA9IHQ7",
				"CiAgcmV0dXJuIHJlc3VsdDsKfTsKCkVsZW1lbnQuX2dldENvbnRlbnRGcm9tQW5vbnltb3VzRWxlbWVu",
				"dCA9IGZ1bmN0aW9uKHRhZ05hbWUsIGh0bWwsIGZvcmNlKSB7CiAgdmFyIGRpdiA9IG5ldyBFbGVtZW50",
				"KCdkaXYnKSwKICAgICAgdCA9IEVsZW1lbnQuX2luc2VydGlvblRyYW5zbGF0aW9ucy50YWdzW3RhZ05h",
				"bWVdOwoKICB2YXIgd29ya2Fyb3VuZCA9IGZhbHNlOwogIGlmICh0KSB3b3JrYXJvdW5kID0gdHJ1ZTsK",
				"ICBlbHNlIGlmIChmb3JjZSkgewogICAgd29ya2Fyb3VuZCA9IHRydWU7CiAgICB0ID0gWycnLCAnJywg",
				"MF07CiAgfQoKICBpZiAod29ya2Fyb3VuZCkgewogICAgZGl2LmlubmVySFRNTCA9ICcmbmJzcDsnICsg",
				"dFswXSArIGh0bWwgKyB0WzFdOwogICAgZGl2LnJlbW92ZUNoaWxkKGRpdi5maXJzdENoaWxkKTsKICAg",
				"IGZvciAodmFyIGkgPSB0WzJdOyBpLS07ICkgewogICAgICBkaXYgPSBkaXYuZmlyc3RDaGlsZDsKICAg",
				"IH0KICB9CiAgZWxzZSB7CiAgICBkaXYuaW5uZXJIVE1MID0gaHRtbDsKICB9CiAgcmV0dXJuICRBKGRp",
				"di5jaGlsZE5vZGVzKTsKfTsKCkVsZW1lbnQuX2luc2VydGlvblRyYW5zbGF0aW9ucyA9IHsKICBiZWZv",
				"cmU6IGZ1bmN0aW9uKGVsZW1lbnQsIG5vZGUpIHsKICAgIGVsZW1lbnQucGFyZW50Tm9kZS5pbnNlcnRC",
				"ZWZvcmUobm9kZSwgZWxlbWVudCk7CiAgfSwKICB0b3A6IGZ1bmN0aW9uKGVsZW1lbnQsIG5vZGUpIHsK",
				"ICAgIGVsZW1lbnQuaW5zZXJ0QmVmb3JlKG5vZGUsIGVsZW1lbnQuZmlyc3RDaGlsZCk7CiAgfSwKICBi",
				"b3R0b206IGZ1bmN0aW9uKGVsZW1lbnQsIG5vZGUpIHsKICAgIGVsZW1lbnQuYXBwZW5kQ2hpbGQobm9k",
				"ZSk7CiAgfSwKICBhZnRlcjogZnVuY3Rpb24oZWxlbWVudCwgbm9kZSkgewogICAgZWxlbWVudC5wYXJl",
				"bnROb2RlLmluc2VydEJlZm9yZShub2RlLCBlbGVtZW50Lm5leHRTaWJsaW5nKTsKICB9LAogIHRhZ3M6",
				"IHsKICAgIFRBQkxFOiAgWyc8dGFibGU+JywgICAgICAgICAgICAgICAgJzwvdGFibGU+JywgICAgICAg",
				"ICAgICAgICAgICAgMV0sCiAgICBUQk9EWTogIFsnPHRhYmxlPjx0Ym9keT4nLCAgICAgICAgICc8L3Ri",
				"b2R5PjwvdGFibGU+JywgICAgICAgICAgIDJdLAogICAgVFI6ICAgICBbJzx0YWJsZT48dGJvZHk+PHRy",
				"PicsICAgICAnPC90cj48L3Rib2R5PjwvdGFibGU+JywgICAgICAzXSwKICAgIFREOiAgICAgWyc8dGFi",
				"bGU+PHRib2R5Pjx0cj48dGQ+JywgJzwvdGQ+PC90cj48L3Rib2R5PjwvdGFibGU+JywgNF0sCiAgICBT",
				"RUxFQ1Q6IFsnPHNlbGVjdD4nLCAgICAgICAgICAgICAgICc8L3NlbGVjdD4nLCAgICAgICAgICAgICAg",
				"ICAgIDFdCiAgfQp9OwoKKGZ1bmN0aW9uKCkgewogIHZhciB0YWdzID0gRWxlbWVudC5faW5zZXJ0aW9u",
				"VHJhbnNsYXRpb25zLnRhZ3M7CiAgT2JqZWN0LmV4dGVuZCh0YWdzLCB7CiAgICBUSEVBRDogdGFncy5U",
				"Qk9EWSwKICAgIFRGT09UOiB0YWdzLlRCT0RZLAogICAgVEg6ICAgIHRhZ3MuVEQKICB9KTsKfSkoKTsK",
				"CkVsZW1lbnQuTWV0aG9kcy5TaW11bGF0ZWQgPSB7CiAgaGFzQXR0cmlidXRlOiBmdW5jdGlvbihlbGVt",
				"ZW50LCBhdHRyaWJ1dGUpIHsKICAgIGF0dHJpYnV0ZSA9IEVsZW1lbnQuX2F0dHJpYnV0ZVRyYW5zbGF0",
				"aW9ucy5oYXNbYXR0cmlidXRlXSB8fCBhdHRyaWJ1dGU7CiAgICB2YXIgbm9kZSA9ICQoZWxlbWVudCku",
				"Z2V0QXR0cmlidXRlTm9kZShhdHRyaWJ1dGUpOwogICAgcmV0dXJuICEhKG5vZGUgJiYgbm9kZS5zcGVj",
				"aWZpZWQpOwogIH0KfTsKCkVsZW1lbnQuTWV0aG9kcy5CeVRhZyA9IHsgfTsKCk9iamVjdC5leHRlbmQo",
				"RWxlbWVudCwgRWxlbWVudC5NZXRob2RzKTsKCihmdW5jdGlvbihkaXYpIHsKCiAgaWYgKCFQcm90b3R5",
				"cGUuQnJvd3NlckZlYXR1cmVzLkVsZW1lbnRFeHRlbnNpb25zICYmIGRpdlsnX19wcm90b19fJ10pIHsK",
				"ICAgIHdpbmRvdy5IVE1MRWxlbWVudCA9IHsgfTsKICAgIHdpbmRvdy5IVE1MRWxlbWVudC5wcm90b3R5",
				"cGUgPSBkaXZbJ19fcHJvdG9fXyddOwogICAgUHJvdG90eXBlLkJyb3dzZXJGZWF0dXJlcy5FbGVtZW50",
				"RXh0ZW5zaW9ucyA9IHRydWU7CiAgfQoKICBkaXYgPSBudWxsOwoKfSkoZG9jdW1lbnQuY3JlYXRlRWxl",
				"bWVudCgnZGl2JykpOwoKRWxlbWVudC5leHRlbmQgPSAoZnVuY3Rpb24oKSB7CgogIGZ1bmN0aW9uIGNo",
				"ZWNrRGVmaWNpZW5jeSh0YWdOYW1lKSB7CiAgICBpZiAodHlwZW9mIHdpbmRvdy5FbGVtZW50ICE9ICd1",
				"bmRlZmluZWQnKSB7CiAgICAgIHZhciBwcm90byA9IHdpbmRvdy5FbGVtZW50LnByb3RvdHlwZTsKICAg",
				"ICAgaWYgKHByb3RvKSB7CiAgICAgICAgdmFyIGlkID0gJ18nICsgKE1hdGgucmFuZG9tKCkrJycpLnNs",
				"aWNlKDIpLAogICAgICAgICAgICBlbCA9IGRvY3VtZW50LmNyZWF0ZUVsZW1lbnQodGFnTmFtZSk7CiAg",
				"ICAgICAgcHJvdG9baWRdID0gJ3gnOwogICAgICAgIHZhciBpc0J1Z2d5ID0gKGVsW2lkXSAhPT0gJ3gn",
				"KTsKICAgICAgICBkZWxldGUgcHJvdG9baWRdOwogICAgICAgIGVsID0gbnVsbDsKICAgICAgICByZXR1",
				"cm4gaXNCdWdneTsKICAgICAgfQogICAgfQogICAgcmV0dXJuIGZhbHNlOwogIH0KCiAgZnVuY3Rpb24g",
				"ZXh0ZW5kRWxlbWVudFdpdGgoZWxlbWVudCwgbWV0aG9kcykgewogICAgZm9yICh2YXIgcHJvcGVydHkg",
				"aW4gbWV0aG9kcykgewogICAgICB2YXIgdmFsdWUgPSBtZXRob2RzW3Byb3BlcnR5XTsKICAgICAgaWYg",
				"KE9iamVjdC5pc0Z1bmN0aW9uKHZhbHVlKSAmJiAhKHByb3BlcnR5IGluIGVsZW1lbnQpKQogICAgICAg",
				"IGVsZW1lbnRbcHJvcGVydHldID0gdmFsdWUubWV0aG9kaXplKCk7CiAgICB9CiAgfQoKICB2YXIgSFRN",
				"TE9CSkVDVEVMRU1FTlRfUFJPVE9UWVBFX0JVR0dZID0gY2hlY2tEZWZpY2llbmN5KCdvYmplY3QnKTsK",
				"CiAgaWYgKFByb3RvdHlwZS5Ccm93c2VyRmVhdHVyZXMuU3BlY2lmaWNFbGVtZW50RXh0ZW5zaW9ucykg",
				"ewogICAgaWYgKEhUTUxPQkpFQ1RFTEVNRU5UX1BST1RPVFlQRV9CVUdHWSkgewogICAgICByZXR1cm4g",
				"ZnVuY3Rpb24oZWxlbWVudCkgewogICAgICAgIGlmIChlbGVtZW50ICYmIHR5cGVvZiBlbGVtZW50Ll9l",
				"eHRlbmRlZEJ5UHJvdG90eXBlID09ICd1bmRlZmluZWQnKSB7CiAgICAgICAgICB2YXIgdCA9IGVsZW1l",
				"bnQudGFnTmFtZTsKICAgICAgICAgIGlmICh0ICYmICgvXig/Om9iamVjdHxhcHBsZXR8ZW1iZWQpJC9p",
				"LnRlc3QodCkpKSB7CiAgICAgICAgICAgIGV4dGVuZEVsZW1lbnRXaXRoKGVsZW1lbnQsIEVsZW1lbnQu",
				"TWV0aG9kcyk7CiAgICAgICAgICAgIGV4dGVuZEVsZW1lbnRXaXRoKGVsZW1lbnQsIEVsZW1lbnQuTWV0",
				"aG9kcy5TaW11bGF0ZWQpOwogICAgICAgICAgICBleHRlbmRFbGVtZW50V2l0aChlbGVtZW50LCBFbGVt",
				"ZW50Lk1ldGhvZHMuQnlUYWdbdC50b1VwcGVyQ2FzZSgpXSk7CiAgICAgICAgICB9CiAgICAgICAgfQog",
				"ICAgICAgIHJldHVybiBlbGVtZW50OwogICAgICB9CiAgICB9CiAgICByZXR1cm4gUHJvdG90eXBlLks7",
				"CiAgfQoKICB2YXIgTWV0aG9kcyA9IHsgfSwgQnlUYWcgPSBFbGVtZW50Lk1ldGhvZHMuQnlUYWc7Cgog",
				"IHZhciBleHRlbmQgPSBPYmplY3QuZXh0ZW5kKGZ1bmN0aW9uKGVsZW1lbnQpIHsKICAgIGlmICghZWxl",
				"bWVudCB8fCB0eXBlb2YgZWxlbWVudC5fZXh0ZW5kZWRCeVByb3RvdHlwZSAhPSAndW5kZWZpbmVkJyB8",
				"fAogICAgICAgIGVsZW1lbnQubm9kZVR5cGUgIT0gMSB8fCBlbGVtZW50ID09IHdpbmRvdykgcmV0dXJu",
				"IGVsZW1lbnQ7CgogICAgdmFyIG1ldGhvZHMgPSBPYmplY3QuY2xvbmUoTWV0aG9kcyksCiAgICAgICAg",
				"dGFnTmFtZSA9IGVsZW1lbnQudGFnTmFtZS50b1VwcGVyQ2FzZSgpOwoKICAgIGlmIChCeVRhZ1t0YWdO",
				"YW1lXSkgT2JqZWN0LmV4dGVuZChtZXRob2RzLCBCeVRhZ1t0YWdOYW1lXSk7CgogICAgZXh0ZW5kRWxl",
				"bWVudFdpdGgoZWxlbWVudCwgbWV0aG9kcyk7CgogICAgZWxlbWVudC5fZXh0ZW5kZWRCeVByb3RvdHlw",
				"ZSA9IFByb3RvdHlwZS5lbXB0eUZ1bmN0aW9uOwogICAgcmV0dXJuIGVsZW1lbnQ7CgogIH0sIHsKICAg",
				"IHJlZnJlc2g6IGZ1bmN0aW9uKCkgewogICAgICBpZiAoIVByb3RvdHlwZS5Ccm93c2VyRmVhdHVyZXMu",
				"RWxlbWVudEV4dGVuc2lvbnMpIHsKICAgICAgICBPYmplY3QuZXh0ZW5kKE1ldGhvZHMsIEVsZW1lbnQu",
				"TWV0aG9kcyk7CiAgICAgICAgT2JqZWN0LmV4dGVuZChNZXRob2RzLCBFbGVtZW50Lk1ldGhvZHMuU2lt",
				"dWxhdGVkKTsKICAgICAgfQogICAgfQogIH0pOwoKICBleHRlbmQucmVmcmVzaCgpOwogIHJldHVybiBl",
				"eHRlbmQ7Cn0pKCk7CgppZiAoZG9jdW1lbnQuZG9jdW1lbnRFbGVtZW50Lmhhc0F0dHJpYnV0ZSkgewog",
				"IEVsZW1lbnQuaGFzQXR0cmlidXRlID0gZnVuY3Rpb24oZWxlbWVudCwgYXR0cmlidXRlKSB7CiAgICBy",
				"ZXR1cm4gZWxlbWVudC5oYXNBdHRyaWJ1dGUoYXR0cmlidXRlKTsKICB9Owp9CmVsc2UgewogIEVsZW1l",
				"bnQuaGFzQXR0cmlidXRlID0gRWxlbWVudC5NZXRob2RzLlNpbXVsYXRlZC5oYXNBdHRyaWJ1dGU7Cn0K",
				"CkVsZW1lbnQuYWRkTWV0aG9kcyA9IGZ1bmN0aW9uKG1ldGhvZHMpIHsKICB2YXIgRiA9IFByb3RvdHlw",
				"ZS5Ccm93c2VyRmVhdHVyZXMsIFQgPSBFbGVtZW50Lk1ldGhvZHMuQnlUYWc7CgogIGlmICghbWV0aG9k",
				"cykgewogICAgT2JqZWN0LmV4dGVuZChGb3JtLCBGb3JtLk1ldGhvZHMpOwogICAgT2JqZWN0LmV4dGVu",
				"ZChGb3JtLkVsZW1lbnQsIEZvcm0uRWxlbWVudC5NZXRob2RzKTsKICAgIE9iamVjdC5leHRlbmQoRWxl",
				"bWVudC5NZXRob2RzLkJ5VGFnLCB7CiAgICAgICJGT1JNIjogICAgIE9iamVjdC5jbG9uZShGb3JtLk1l",
				"dGhvZHMpLAogICAgICAiSU5QVVQiOiAgICBPYmplY3QuY2xvbmUoRm9ybS5FbGVtZW50Lk1ldGhvZHMp",
				"LAogICAgICAiU0VMRUNUIjogICBPYmplY3QuY2xvbmUoRm9ybS5FbGVtZW50Lk1ldGhvZHMpLAogICAg",
				"ICAiVEVYVEFSRUEiOiBPYmplY3QuY2xvbmUoRm9ybS5FbGVtZW50Lk1ldGhvZHMpLAogICAgICAiQlVU",
				"VE9OIjogICBPYmplY3QuY2xvbmUoRm9ybS5FbGVtZW50Lk1ldGhvZHMpCiAgICB9KTsKICB9CgogIGlm",
				"IChhcmd1bWVudHMubGVuZ3RoID09IDIpIHsKICAgIHZhciB0YWdOYW1lID0gbWV0aG9kczsKICAgIG1l",
				"dGhvZHMgPSBhcmd1bWVudHNbMV07CiAgfQoKICBpZiAoIXRhZ05hbWUpIE9iamVjdC5leHRlbmQoRWxl",
				"bWVudC5NZXRob2RzLCBtZXRob2RzIHx8IHsgfSk7CiAgZWxzZSB7CiAgICBpZiAoT2JqZWN0LmlzQXJy",
				"YXkodGFnTmFtZSkpIHRhZ05hbWUuZWFjaChleHRlbmQpOwogICAgZWxzZSBleHRlbmQodGFnTmFtZSk7",
				"CiAgfQoKICBmdW5jdGlvbiBleHRlbmQodGFnTmFtZSkgewogICAgdGFnTmFtZSA9IHRhZ05hbWUudG9V",
				"cHBlckNhc2UoKTsKICAgIGlmICghRWxlbWVudC5NZXRob2RzLkJ5VGFnW3RhZ05hbWVdKQogICAgICBF",
				"bGVtZW50Lk1ldGhvZHMuQnlUYWdbdGFnTmFtZV0gPSB7IH07CiAgICBPYmplY3QuZXh0ZW5kKEVsZW1l",
				"bnQuTWV0aG9kcy5CeVRhZ1t0YWdOYW1lXSwgbWV0aG9kcyk7CiAgfQoKICBmdW5jdGlvbiBjb3B5KG1l",
				"dGhvZHMsIGRlc3RpbmF0aW9uLCBvbmx5SWZBYnNlbnQpIHsKICAgIG9ubHlJZkFic2VudCA9IG9ubHlJ",
				"ZkFic2VudCB8fCBmYWxzZTsKICAgIGZvciAodmFyIHByb3BlcnR5IGluIG1ldGhvZHMpIHsKICAgICAg",
				"dmFyIHZhbHVlID0gbWV0aG9kc1twcm9wZXJ0eV07CiAgICAgIGlmICghT2JqZWN0LmlzRnVuY3Rpb24o",
				"dmFsdWUpKSBjb250aW51ZTsKICAgICAgaWYgKCFvbmx5SWZBYnNlbnQgfHwgIShwcm9wZXJ0eSBpbiBk",
				"ZXN0aW5hdGlvbikpCiAgICAgICAgZGVzdGluYXRpb25bcHJvcGVydHldID0gdmFsdWUubWV0aG9kaXpl",
				"KCk7CiAgICB9CiAgfQoKICBmdW5jdGlvbiBmaW5kRE9NQ2xhc3ModGFnTmFtZSkgewogICAgdmFyIGts",
				"YXNzOwogICAgdmFyIHRyYW5zID0gewogICAgICAiT1BUR1JPVVAiOiAiT3B0R3JvdXAiLCAiVEVYVEFS",
				"RUEiOiAiVGV4dEFyZWEiLCAiUCI6ICJQYXJhZ3JhcGgiLAogICAgICAiRklFTERTRVQiOiAiRmllbGRT",
				"ZXQiLCAiVUwiOiAiVUxpc3QiLCAiT0wiOiAiT0xpc3QiLCAiREwiOiAiRExpc3QiLAogICAgICAiRElS",
				"IjogIkRpcmVjdG9yeSIsICJIMSI6ICJIZWFkaW5nIiwgIkgyIjogIkhlYWRpbmciLCAiSDMiOiAiSGVh",
				"ZGluZyIsCiAgICAgICJINCI6ICJIZWFkaW5nIiwgIkg1IjogIkhlYWRpbmciLCAiSDYiOiAiSGVhZGlu",
				"ZyIsICJRIjogIlF1b3RlIiwKICAgICAgIklOUyI6ICJNb2QiLCAiREVMIjogIk1vZCIsICJBIjogIkFu",
				"Y2hvciIsICJJTUciOiAiSW1hZ2UiLCAiQ0FQVElPTiI6CiAgICAgICJUYWJsZUNhcHRpb24iLCAiQ09M",
				"IjogIlRhYmxlQ29sIiwgIkNPTEdST1VQIjogIlRhYmxlQ29sIiwgIlRIRUFEIjoKICAgICAgIlRhYmxl",
				"U2VjdGlvbiIsICJURk9PVCI6ICJUYWJsZVNlY3Rpb24iLCAiVEJPRFkiOiAiVGFibGVTZWN0aW9uIiwg",
				"IlRSIjoKICAgICAgIlRhYmxlUm93IiwgIlRIIjogIlRhYmxlQ2VsbCIsICJURCI6ICJUYWJsZUNlbGwi",
				"LCAiRlJBTUVTRVQiOgogICAgICAiRnJhbWVTZXQiLCAiSUZSQU1FIjogIklGcmFtZSIKICAgIH07CiAg",
				"ICBpZiAodHJhbnNbdGFnTmFtZV0pIGtsYXNzID0gJ0hUTUwnICsgdHJhbnNbdGFnTmFtZV0gKyAnRWxl",
				"bWVudCc7CiAgICBpZiAod2luZG93W2tsYXNzXSkgcmV0dXJuIHdpbmRvd1trbGFzc107CiAgICBrbGFz",
				"cyA9ICdIVE1MJyArIHRhZ05hbWUgKyAnRWxlbWVudCc7CiAgICBpZiAod2luZG93W2tsYXNzXSkgcmV0",
				"dXJuIHdpbmRvd1trbGFzc107CiAgICBrbGFzcyA9ICdIVE1MJyArIHRhZ05hbWUuY2FwaXRhbGl6ZSgp",
				"ICsgJ0VsZW1lbnQnOwogICAgaWYgKHdpbmRvd1trbGFzc10pIHJldHVybiB3aW5kb3dba2xhc3NdOwoK",
				"ICAgIHZhciBlbGVtZW50ID0gZG9jdW1lbnQuY3JlYXRlRWxlbWVudCh0YWdOYW1lKSwKICAgICAgICBw",
				"cm90byA9IGVsZW1lbnRbJ19fcHJvdG9fXyddIHx8IGVsZW1lbnQuY29uc3RydWN0b3IucHJvdG90eXBl",
				"OwoKICAgIGVsZW1lbnQgPSBudWxsOwogICAgcmV0dXJuIHByb3RvOwogIH0KCiAgdmFyIGVsZW1lbnRQ",
				"cm90b3R5cGUgPSB3aW5kb3cuSFRNTEVsZW1lbnQgPyBIVE1MRWxlbWVudC5wcm90b3R5cGUgOgogICBF",
				"bGVtZW50LnByb3RvdHlwZTsKCiAgaWYgKEYuRWxlbWVudEV4dGVuc2lvbnMpIHsKICAgIGNvcHkoRWxl",
				"bWVudC5NZXRob2RzLCBlbGVtZW50UHJvdG90eXBlKTsKICAgIGNvcHkoRWxlbWVudC5NZXRob2RzLlNp",
				"bXVsYXRlZCwgZWxlbWVudFByb3RvdHlwZSwgdHJ1ZSk7CiAgfQoKICBpZiAoRi5TcGVjaWZpY0VsZW1l",
				"bnRFeHRlbnNpb25zKSB7CiAgICBmb3IgKHZhciB0YWcgaW4gRWxlbWVudC5NZXRob2RzLkJ5VGFnKSB7",
				"CiAgICAgIHZhciBrbGFzcyA9IGZpbmRET01DbGFzcyh0YWcpOwogICAgICBpZiAoT2JqZWN0LmlzVW5k",
				"ZWZpbmVkKGtsYXNzKSkgY29udGludWU7CiAgICAgIGNvcHkoVFt0YWddLCBrbGFzcy5wcm90b3R5cGUp",
				"OwogICAgfQogIH0KCiAgT2JqZWN0LmV4dGVuZChFbGVtZW50LCBFbGVtZW50Lk1ldGhvZHMpOwogIGRl",
				"bGV0ZSBFbGVtZW50LkJ5VGFnOwoKICBpZiAoRWxlbWVudC5leHRlbmQucmVmcmVzaCkgRWxlbWVudC5l",
				"eHRlbmQucmVmcmVzaCgpOwogIEVsZW1lbnQuY2FjaGUgPSB7IH07Cn07CgoKZG9jdW1lbnQudmlld3Bv",
				"cnQgPSB7CgogIGdldERpbWVuc2lvbnM6IGZ1bmN0aW9uKCkgewogICAgcmV0dXJuIHsgd2lkdGg6IHRo",
				"aXMuZ2V0V2lkdGgoKSwgaGVpZ2h0OiB0aGlzLmdldEhlaWdodCgpIH07CiAgfSwKCiAgZ2V0U2Nyb2xs",
				"T2Zmc2V0czogZnVuY3Rpb24oKSB7CiAgICByZXR1cm4gRWxlbWVudC5fcmV0dXJuT2Zmc2V0KAogICAg",
				"ICB3aW5kb3cucGFnZVhPZmZzZXQgfHwgZG9jdW1lbnQuZG9jdW1lbnRFbGVtZW50LnNjcm9sbExlZnQg",
				"fHwgZG9jdW1lbnQuYm9keS5zY3JvbGxMZWZ0LAogICAgICB3aW5kb3cucGFnZVlPZmZzZXQgfHwgZG9j",
				"dW1lbnQuZG9jdW1lbnRFbGVtZW50LnNjcm9sbFRvcCAgfHwgZG9jdW1lbnQuYm9keS5zY3JvbGxUb3Ap",
				"OwogIH0KfTsKCihmdW5jdGlvbih2aWV3cG9ydCkgewogIHZhciBCID0gUHJvdG90eXBlLkJyb3dzZXIs",
				"IGRvYyA9IGRvY3VtZW50LCBlbGVtZW50LCBwcm9wZXJ0eSA9IHt9OwoKICBmdW5jdGlvbiBnZXRSb290",
				"RWxlbWVudCgpIHsKICAgIGlmIChCLldlYktpdCAmJiAhZG9jLmV2YWx1YXRlKQogICAgICByZXR1cm4g",
				"ZG9jdW1lbnQ7CgogICAgaWYgKEIuT3BlcmEgJiYgd2luZG93LnBhcnNlRmxvYXQod2luZG93Lm9wZXJh",
				"LnZlcnNpb24oKSkgPCA5LjUpCiAgICAgIHJldHVybiBkb2N1bWVudC5ib2R5OwoKICAgIHJldHVybiBk",
				"b2N1bWVudC5kb2N1bWVudEVsZW1lbnQ7CiAgfQoKICBmdW5jdGlvbiBkZWZpbmUoRCkgewogICAgaWYg",
				"KCFlbGVtZW50KSBlbGVtZW50ID0gZ2V0Um9vdEVsZW1lbnQoKTsKCiAgICBwcm9wZXJ0eVtEXSA9ICdj",
				"bGllbnQnICsgRDsKCiAgICB2aWV3cG9ydFsnZ2V0JyArIERdID0gZnVuY3Rpb24oKSB7IHJldHVybiBl",
				"bGVtZW50W3Byb3BlcnR5W0RdXSB9OwogICAgcmV0dXJuIHZpZXdwb3J0WydnZXQnICsgRF0oKTsKICB9",
				"CgogIHZpZXdwb3J0LmdldFdpZHRoICA9IGRlZmluZS5jdXJyeSgnV2lkdGgnKTsKCiAgdmlld3BvcnQu",
				"Z2V0SGVpZ2h0ID0gZGVmaW5lLmN1cnJ5KCdIZWlnaHQnKTsKfSkoZG9jdW1lbnQudmlld3BvcnQpOwoK",
				"CkVsZW1lbnQuU3RvcmFnZSA9IHsKICBVSUQ6IDEKfTsKCkVsZW1lbnQuYWRkTWV0aG9kcyh7CiAgZ2V0",
				"U3RvcmFnZTogZnVuY3Rpb24oZWxlbWVudCkgewogICAgaWYgKCEoZWxlbWVudCA9ICQoZWxlbWVudCkp",
				"KSByZXR1cm47CgogICAgdmFyIHVpZDsKICAgIGlmIChlbGVtZW50ID09PSB3aW5kb3cpIHsKICAgICAg",
				"dWlkID0gMDsKICAgIH0gZWxzZSB7CiAgICAgIGlmICh0eXBlb2YgZWxlbWVudC5fcHJvdG90eXBlVUlE",
				"ID09PSAidW5kZWZpbmVkIikKICAgICAgICBlbGVtZW50Ll9wcm90b3R5cGVVSUQgPSBFbGVtZW50LlN0",
				"b3JhZ2UuVUlEKys7CiAgICAgIHVpZCA9IGVsZW1lbnQuX3Byb3RvdHlwZVVJRDsKICAgIH0KCiAgICBp",
				"ZiAoIUVsZW1lbnQuU3RvcmFnZVt1aWRdKQogICAgICBFbGVtZW50LlN0b3JhZ2VbdWlkXSA9ICRIKCk7",
				"CgogICAgcmV0dXJuIEVsZW1lbnQuU3RvcmFnZVt1aWRdOwogIH0sCgogIHN0b3JlOiBmdW5jdGlvbihl",
				"bGVtZW50LCBrZXksIHZhbHVlKSB7CiAgICBpZiAoIShlbGVtZW50ID0gJChlbGVtZW50KSkpIHJldHVy",
				"bjsKCiAgICBpZiAoYXJndW1lbnRzLmxlbmd0aCA9PT0gMikgewogICAgICBFbGVtZW50LmdldFN0b3Jh",
				"Z2UoZWxlbWVudCkudXBkYXRlKGtleSk7CiAgICB9IGVsc2UgewogICAgICBFbGVtZW50LmdldFN0b3Jh",
				"Z2UoZWxlbWVudCkuc2V0KGtleSwgdmFsdWUpOwogICAgfQoKICAgIHJldHVybiBlbGVtZW50OwogIH0s",
				"CgogIHJldHJpZXZlOiBmdW5jdGlvbihlbGVtZW50LCBrZXksIGRlZmF1bHRWYWx1ZSkgewogICAgaWYg",
				"KCEoZWxlbWVudCA9ICQoZWxlbWVudCkpKSByZXR1cm47CiAgICB2YXIgaGFzaCA9IEVsZW1lbnQuZ2V0",
				"U3RvcmFnZShlbGVtZW50KSwgdmFsdWUgPSBoYXNoLmdldChrZXkpOwoKICAgIGlmIChPYmplY3QuaXNV",
				"bmRlZmluZWQodmFsdWUpKSB7CiAgICAgIGhhc2guc2V0KGtleSwgZGVmYXVsdFZhbHVlKTsKICAgICAg",
				"dmFsdWUgPSBkZWZhdWx0VmFsdWU7CiAgICB9CgogICAgcmV0dXJuIHZhbHVlOwogIH0sCgogIGNsb25l",
				"OiBmdW5jdGlvbihlbGVtZW50LCBkZWVwKSB7CiAgICBpZiAoIShlbGVtZW50ID0gJChlbGVtZW50KSkp",
				"IHJldHVybjsKICAgIHZhciBjbG9uZSA9IGVsZW1lbnQuY2xvbmVOb2RlKGRlZXApOwogICAgY2xvbmUu",
				"X3Byb3RvdHlwZVVJRCA9IHZvaWQgMDsKICAgIGlmIChkZWVwKSB7CiAgICAgIHZhciBkZXNjZW5kYW50",
				"cyA9IEVsZW1lbnQuc2VsZWN0KGNsb25lLCAnKicpLAogICAgICAgICAgaSA9IGRlc2NlbmRhbnRzLmxl",
				"bmd0aDsKICAgICAgd2hpbGUgKGktLSkgewogICAgICAgIGRlc2NlbmRhbnRzW2ldLl9wcm90b3R5cGVV",
				"SUQgPSB2b2lkIDA7CiAgICAgIH0KICAgIH0KICAgIHJldHVybiBFbGVtZW50LmV4dGVuZChjbG9uZSk7",
				"CiAgfSwKCiAgcHVyZ2U6IGZ1bmN0aW9uKGVsZW1lbnQpIHsKICAgIGlmICghKGVsZW1lbnQgPSAkKGVs",
				"ZW1lbnQpKSkgcmV0dXJuOwogICAgdmFyIHB1cmdlRWxlbWVudCA9IEVsZW1lbnQuX3B1cmdlRWxlbWVu",
				"dDsKCiAgICBwdXJnZUVsZW1lbnQoZWxlbWVudCk7CgogICAgdmFyIGRlc2NlbmRhbnRzID0gZWxlbWVu",
				"dC5nZXRFbGVtZW50c0J5VGFnTmFtZSgnKicpLAogICAgIGkgPSBkZXNjZW5kYW50cy5sZW5ndGg7Cgog",
				"ICAgd2hpbGUgKGktLSkgcHVyZ2VFbGVtZW50KGRlc2NlbmRhbnRzW2ldKTsKCiAgICByZXR1cm4gbnVs",
				"bDsKICB9Cn0pOwoKKGZ1bmN0aW9uKCkgewoKICBmdW5jdGlvbiB0b0RlY2ltYWwocGN0U3RyaW5nKSB7",
				"CiAgICB2YXIgbWF0Y2ggPSBwY3RTdHJpbmcubWF0Y2goL14oXGQrKSU/JC9pKTsKICAgIGlmICghbWF0",
				"Y2gpIHJldHVybiBudWxsOwogICAgcmV0dXJuIChOdW1iZXIobWF0Y2hbMV0pIC8gMTAwKTsKICB9Cgog",
				"IGZ1bmN0aW9uIGdldFBpeGVsVmFsdWUodmFsdWUsIHByb3BlcnR5LCBjb250ZXh0KSB7CiAgICB2YXIg",
				"ZWxlbWVudCA9IG51bGw7CiAgICBpZiAoT2JqZWN0LmlzRWxlbWVudCh2YWx1ZSkpIHsKICAgICAgZWxl",
				"bWVudCA9IHZhbHVlOwogICAgICB2YWx1ZSA9IGVsZW1lbnQuZ2V0U3R5bGUocHJvcGVydHkpOwogICAg",
				"fQoKICAgIGlmICh2YWx1ZSA9PT0gbnVsbCkgewogICAgICByZXR1cm4gbnVsbDsKICAgIH0KCiAgICBp",
				"ZiAoKC9eKD86LSk/XGQrKFwuXGQrKT8ocHgpPyQvaSkudGVzdCh2YWx1ZSkpIHsKICAgICAgcmV0dXJu",
				"IHdpbmRvdy5wYXJzZUZsb2F0KHZhbHVlKTsKICAgIH0KCiAgICB2YXIgaXNQZXJjZW50YWdlID0gdmFs",
				"dWUuaW5jbHVkZSgnJScpLCBpc1ZpZXdwb3J0ID0gKGNvbnRleHQgPT09IGRvY3VtZW50LnZpZXdwb3J0",
				"KTsKCiAgICBpZiAoL1xkLy50ZXN0KHZhbHVlKSAmJiBlbGVtZW50ICYmIGVsZW1lbnQucnVudGltZVN0",
				"eWxlICYmICEoaXNQZXJjZW50YWdlICYmIGlzVmlld3BvcnQpKSB7CiAgICAgIHZhciBzdHlsZSA9IGVs",
				"ZW1lbnQuc3R5bGUubGVmdCwgclN0eWxlID0gZWxlbWVudC5ydW50aW1lU3R5bGUubGVmdDsKICAgICAg",
				"ZWxlbWVudC5ydW50aW1lU3R5bGUubGVmdCA9IGVsZW1lbnQuY3VycmVudFN0eWxlLmxlZnQ7CiAgICAg",
				"IGVsZW1lbnQuc3R5bGUubGVmdCA9IHZhbHVlIHx8IDA7CiAgICAgIHZhbHVlID0gZWxlbWVudC5zdHls",
				"ZS5waXhlbExlZnQ7CiAgICAgIGVsZW1lbnQuc3R5bGUubGVmdCA9IHN0eWxlOwogICAgICBlbGVtZW50",
				"LnJ1bnRpbWVTdHlsZS5sZWZ0ID0gclN0eWxlOwoKICAgICAgcmV0dXJuIHZhbHVlOwogICAgfQoKICAg",
				"IGlmIChlbGVtZW50ICYmIGlzUGVyY2VudGFnZSkgewogICAgICBjb250ZXh0ID0gY29udGV4dCB8fCBl",
				"bGVtZW50LnBhcmVudE5vZGU7CiAgICAgIHZhciBkZWNpbWFsID0gdG9EZWNpbWFsKHZhbHVlKTsKICAg",
				"ICAgdmFyIHdob2xlID0gbnVsbDsKICAgICAgdmFyIHBvc2l0aW9uID0gZWxlbWVudC5nZXRTdHlsZSgn",
				"cG9zaXRpb24nKTsKCiAgICAgIHZhciBpc0hvcml6b250YWwgPSBwcm9wZXJ0eS5pbmNsdWRlKCdsZWZ0",
				"JykgfHwgcHJvcGVydHkuaW5jbHVkZSgncmlnaHQnKSB8fAogICAgICAgcHJvcGVydHkuaW5jbHVkZSgn",
				"d2lkdGgnKTsKCiAgICAgIHZhciBpc1ZlcnRpY2FsID0gIHByb3BlcnR5LmluY2x1ZGUoJ3RvcCcpIHx8",
				"IHByb3BlcnR5LmluY2x1ZGUoJ2JvdHRvbScpIHx8CiAgICAgICAgcHJvcGVydHkuaW5jbHVkZSgnaGVp",
				"Z2h0Jyk7CgogICAgICBpZiAoY29udGV4dCA9PT0gZG9jdW1lbnQudmlld3BvcnQpIHsKICAgICAgICBp",
				"ZiAoaXNIb3Jpem9udGFsKSB7CiAgICAgICAgICB3aG9sZSA9IGRvY3VtZW50LnZpZXdwb3J0LmdldFdp",
				"ZHRoKCk7CiAgICAgICAgfSBlbHNlIGlmIChpc1ZlcnRpY2FsKSB7CiAgICAgICAgICB3aG9sZSA9IGRv",
				"Y3VtZW50LnZpZXdwb3J0LmdldEhlaWdodCgpOwogICAgICAgIH0KICAgICAgfSBlbHNlIHsKICAgICAg",
				"ICBpZiAoaXNIb3Jpem9udGFsKSB7CiAgICAgICAgICB3aG9sZSA9ICQoY29udGV4dCkubWVhc3VyZSgn",
				"d2lkdGgnKTsKICAgICAgICB9IGVsc2UgaWYgKGlzVmVydGljYWwpIHsKICAgICAgICAgIHdob2xlID0g",
				"JChjb250ZXh0KS5tZWFzdXJlKCdoZWlnaHQnKTsKICAgICAgICB9CiAgICAgIH0KCiAgICAgIHJldHVy",
				"biAod2hvbGUgPT09IG51bGwpID8gMCA6IHdob2xlICogZGVjaW1hbDsKICAgIH0KCiAgICByZXR1cm4g",
				"MDsKICB9CgogIGZ1bmN0aW9uIHRvQ1NTUGl4ZWxzKG51bWJlcikgewogICAgaWYgKE9iamVjdC5pc1N0",
				"cmluZyhudW1iZXIpICYmIG51bWJlci5lbmRzV2l0aCgncHgnKSkgewogICAgICByZXR1cm4gbnVtYmVy",
				"OwogICAgfQogICAgcmV0dXJuIG51bWJlciArICdweCc7CiAgfQoKICBmdW5jdGlvbiBpc0Rpc3BsYXll",
				"ZChlbGVtZW50KSB7CiAgICB2YXIgb3JpZ2luYWxFbGVtZW50ID0gZWxlbWVudDsKICAgIHdoaWxlIChl",
				"bGVtZW50ICYmIGVsZW1lbnQucGFyZW50Tm9kZSkgewogICAgICB2YXIgZGlzcGxheSA9IGVsZW1lbnQu",
				"Z2V0U3R5bGUoJ2Rpc3BsYXknKTsKICAgICAgaWYgKGRpc3BsYXkgPT09ICdub25lJykgewogICAgICAg",
				"IHJldHVybiBmYWxzZTsKICAgICAgfQogICAgICBlbGVtZW50ID0gJChlbGVtZW50LnBhcmVudE5vZGUp",
				"OwogICAgfQogICAgcmV0dXJuIHRydWU7CiAgfQoKICB2YXIgaGFzTGF5b3V0ID0gUHJvdG90eXBlLks7",
				"CiAgaWYgKCdjdXJyZW50U3R5bGUnIGluIGRvY3VtZW50LmRvY3VtZW50RWxlbWVudCkgewogICAgaGFz",
				"TGF5b3V0ID0gZnVuY3Rpb24oZWxlbWVudCkgewogICAgICBpZiAoIWVsZW1lbnQuY3VycmVudFN0eWxl",
				"Lmhhc0xheW91dCkgewogICAgICAgIGVsZW1lbnQuc3R5bGUuem9vbSA9IDE7CiAgICAgIH0KICAgICAg",
				"cmV0dXJuIGVsZW1lbnQ7CiAgICB9OwogIH0KCiAgZnVuY3Rpb24gY3NzTmFtZUZvcihrZXkpIHsKICAg",
				"IGlmIChrZXkuaW5jbHVkZSgnYm9yZGVyJykpIGtleSA9IGtleSArICctd2lkdGgnOwogICAgcmV0dXJu",
				"IGtleS5jYW1lbGl6ZSgpOwogIH0KCiAgRWxlbWVudC5MYXlvdXQgPSBDbGFzcy5jcmVhdGUoSGFzaCwg",
				"ewogICAgaW5pdGlhbGl6ZTogZnVuY3Rpb24oJHN1cGVyLCBlbGVtZW50LCBwcmVDb21wdXRlKSB7CiAg",
				"ICAgICRzdXBlcigpOwogICAgICB0aGlzLmVsZW1lbnQgPSAkKGVsZW1lbnQpOwoKICAgICAgRWxlbWVu",
				"dC5MYXlvdXQuUFJPUEVSVElFUy5lYWNoKCBmdW5jdGlvbihwcm9wZXJ0eSkgewogICAgICAgIHRoaXMu",
				"X3NldChwcm9wZXJ0eSwgbnVsbCk7CiAgICAgIH0sIHRoaXMpOwoKICAgICAgaWYgKHByZUNvbXB1dGUp",
				"IHsKICAgICAgICB0aGlzLl9wcmVDb21wdXRpbmcgPSB0cnVlOwogICAgICAgIHRoaXMuX2JlZ2luKCk7",
				"CiAgICAgICAgRWxlbWVudC5MYXlvdXQuUFJPUEVSVElFUy5lYWNoKCB0aGlzLl9jb21wdXRlLCB0aGlz",
				"ICk7CiAgICAgICAgdGhpcy5fZW5kKCk7CiAgICAgICAgdGhpcy5fcHJlQ29tcHV0aW5nID0gZmFsc2U7",
				"CiAgICAgIH0KICAgIH0sCgogICAgX3NldDogZnVuY3Rpb24ocHJvcGVydHksIHZhbHVlKSB7CiAgICAg",
				"IHJldHVybiBIYXNoLnByb3RvdHlwZS5zZXQuY2FsbCh0aGlzLCBwcm9wZXJ0eSwgdmFsdWUpOwogICAg",
				"fSwKCiAgICBzZXQ6IGZ1bmN0aW9uKHByb3BlcnR5LCB2YWx1ZSkgewogICAgICB0aHJvdyAiUHJvcGVy",
				"dGllcyBvZiBFbGVtZW50LkxheW91dCBhcmUgcmVhZC1vbmx5LiI7CiAgICB9LAoKICAgIGdldDogZnVu",
				"Y3Rpb24oJHN1cGVyLCBwcm9wZXJ0eSkgewogICAgICB2YXIgdmFsdWUgPSAkc3VwZXIocHJvcGVydHkp",
				"OwogICAgICByZXR1cm4gdmFsdWUgPT09IG51bGwgPyB0aGlzLl9jb21wdXRlKHByb3BlcnR5KSA6IHZh",
				"bHVlOwogICAgfSwKCiAgICBfYmVnaW46IGZ1bmN0aW9uKCkgewogICAgICBpZiAodGhpcy5fcHJlcGFy",
				"ZWQpIHJldHVybjsKCiAgICAgIHZhciBlbGVtZW50ID0gdGhpcy5lbGVtZW50OwogICAgICBpZiAoaXNE",
				"aXNwbGF5ZWQoZWxlbWVudCkpIHsKICAgICAgICB0aGlzLl9wcmVwYXJlZCA9IHRydWU7CiAgICAgICAg",
				"cmV0dXJuOwogICAgICB9CgogICAgICB2YXIgb3JpZ2luYWxTdHlsZXMgPSB7CiAgICAgICAgcG9zaXRp",
				"b246ICAgZWxlbWVudC5zdHlsZS5wb3NpdGlvbiAgIHx8ICcnLAogICAgICAgIHdpZHRoOiAgICAgIGVs",
				"ZW1lbnQuc3R5bGUud2lkdGggICAgICB8fCAnJywKICAgICAgICB2aXNpYmlsaXR5OiBlbGVtZW50LnN0",
				"eWxlLnZpc2liaWxpdHkgfHwgJycsCiAgICAgICAgZGlzcGxheTogICAgZWxlbWVudC5zdHlsZS5kaXNw",
				"bGF5ICAgIHx8ICcnCiAgICAgIH07CgogICAgICBlbGVtZW50LnN0b3JlKCdwcm90b3R5cGVfb3JpZ2lu",
				"YWxfc3R5bGVzJywgb3JpZ2luYWxTdHlsZXMpOwoKICAgICAgdmFyIHBvc2l0aW9uID0gZWxlbWVudC5n",
				"ZXRTdHlsZSgncG9zaXRpb24nKSwKICAgICAgIHdpZHRoID0gZWxlbWVudC5nZXRTdHlsZSgnd2lkdGgn",
				"KTsKCiAgICAgIGlmICh3aWR0aCA9PT0gIjBweCIgfHwgd2lkdGggPT09IG51bGwpIHsKICAgICAgICBl",
				"bGVtZW50LnN0eWxlLmRpc3BsYXkgPSAnYmxvY2snOwogICAgICAgIHdpZHRoID0gZWxlbWVudC5nZXRT",
				"dHlsZSgnd2lkdGgnKTsKICAgICAgfQoKICAgICAgdmFyIGNvbnRleHQgPSAocG9zaXRpb24gPT09ICdm",
				"aXhlZCcpID8gZG9jdW1lbnQudmlld3BvcnQgOgogICAgICAgZWxlbWVudC5wYXJlbnROb2RlOwoKICAg",
				"ICAgZWxlbWVudC5zZXRTdHlsZSh7CiAgICAgICAgcG9zaXRpb246ICAgJ2Fic29sdXRlJywKICAgICAg",
				"ICB2aXNpYmlsaXR5OiAnaGlkZGVuJywKICAgICAgICBkaXNwbGF5OiAgICAnYmxvY2snCiAgICAgIH0p",
				"OwoKICAgICAgdmFyIHBvc2l0aW9uZWRXaWR0aCA9IGVsZW1lbnQuZ2V0U3R5bGUoJ3dpZHRoJyk7Cgog",
				"ICAgICB2YXIgbmV3V2lkdGg7CiAgICAgIGlmICh3aWR0aCAmJiAocG9zaXRpb25lZFdpZHRoID09PSB3",
				"aWR0aCkpIHsKICAgICAgICBuZXdXaWR0aCA9IGdldFBpeGVsVmFsdWUoZWxlbWVudCwgJ3dpZHRoJywg",
				"Y29udGV4dCk7CiAgICAgIH0gZWxzZSBpZiAocG9zaXRpb24gPT09ICdhYnNvbHV0ZScgfHwgcG9zaXRp",
				"b24gPT09ICdmaXhlZCcpIHsKICAgICAgICBuZXdXaWR0aCA9IGdldFBpeGVsVmFsdWUoZWxlbWVudCwg",
				"J3dpZHRoJywgY29udGV4dCk7CiAgICAgIH0gZWxzZSB7CiAgICAgICAgdmFyIHBhcmVudCA9IGVsZW1l",
				"bnQucGFyZW50Tm9kZSwgcExheW91dCA9ICQocGFyZW50KS5nZXRMYXlvdXQoKTsKCiAgICAgICAgbmV3",
				"V2lkdGggPSBwTGF5b3V0LmdldCgnd2lkdGgnKSAtCiAgICAgICAgIHRoaXMuZ2V0KCdtYXJnaW4tbGVm",
				"dCcpIC0KICAgICAgICAgdGhpcy5nZXQoJ2JvcmRlci1sZWZ0JykgLQogICAgICAgICB0aGlzLmdldCgn",
				"cGFkZGluZy1sZWZ0JykgLQogICAgICAgICB0aGlzLmdldCgncGFkZGluZy1yaWdodCcpIC0KICAgICAg",
				"ICAgdGhpcy5nZXQoJ2JvcmRlci1yaWdodCcpIC0KICAgICAgICAgdGhpcy5nZXQoJ21hcmdpbi1yaWdo",
				"dCcpOwogICAgICB9CgogICAgICBlbGVtZW50LnNldFN0eWxlKHsgd2lkdGg6IG5ld1dpZHRoICsgJ3B4",
				"JyB9KTsKCiAgICAgIHRoaXMuX3ByZXBhcmVkID0gdHJ1ZTsKICAgIH0sCgogICAgX2VuZDogZnVuY3Rp",
				"b24oKSB7CiAgICAgIHZhciBlbGVtZW50ID0gdGhpcy5lbGVtZW50OwogICAgICB2YXIgb3JpZ2luYWxT",
				"dHlsZXMgPSBlbGVtZW50LnJldHJpZXZlKCdwcm90b3R5cGVfb3JpZ2luYWxfc3R5bGVzJyk7CiAgICAg",
				"IGVsZW1lbnQuc3RvcmUoJ3Byb3RvdHlwZV9vcmlnaW5hbF9zdHlsZXMnLCBudWxsKTsKICAgICAgZWxl",
				"bWVudC5zZXRTdHlsZShvcmlnaW5hbFN0eWxlcyk7CiAgICAgIHRoaXMuX3ByZXBhcmVkID0gZmFsc2U7",
				"CiAgICB9LAoKICAgIF9jb21wdXRlOiBmdW5jdGlvbihwcm9wZXJ0eSkgewogICAgICB2YXIgQ09NUFVU",
				"QVRJT05TID0gRWxlbWVudC5MYXlvdXQuQ09NUFVUQVRJT05TOwogICAgICBpZiAoIShwcm9wZXJ0eSBp",
				"biBDT01QVVRBVElPTlMpKSB7CiAgICAgICAgdGhyb3cgIlByb3BlcnR5IG5vdCBmb3VuZC4iOwogICAg",
				"ICB9CgogICAgICByZXR1cm4gdGhpcy5fc2V0KHByb3BlcnR5LCBDT01QVVRBVElPTlNbcHJvcGVydHld",
				"LmNhbGwodGhpcywgdGhpcy5lbGVtZW50KSk7CiAgICB9LAoKICAgIHRvT2JqZWN0OiBmdW5jdGlvbigp",
				"IHsKICAgICAgdmFyIGFyZ3MgPSAkQShhcmd1bWVudHMpOwogICAgICB2YXIga2V5cyA9IChhcmdzLmxl",
				"bmd0aCA9PT0gMCkgPyBFbGVtZW50LkxheW91dC5QUk9QRVJUSUVTIDoKICAgICAgIGFyZ3Muam9pbign",
				"ICcpLnNwbGl0KCcgJyk7CiAgICAgIHZhciBvYmogPSB7fTsKICAgICAga2V5cy5lYWNoKCBmdW5jdGlv",
				"bihrZXkpIHsKICAgICAgICBpZiAoIUVsZW1lbnQuTGF5b3V0LlBST1BFUlRJRVMuaW5jbHVkZShrZXkp",
				"KSByZXR1cm47CiAgICAgICAgdmFyIHZhbHVlID0gdGhpcy5nZXQoa2V5KTsKICAgICAgICBpZiAodmFs",
				"dWUgIT0gbnVsbCkgb2JqW2tleV0gPSB2YWx1ZTsKICAgICAgfSwgdGhpcyk7CiAgICAgIHJldHVybiBv",
				"Ymo7CiAgICB9LAoKICAgIHRvSGFzaDogZnVuY3Rpb24oKSB7CiAgICAgIHZhciBvYmogPSB0aGlzLnRv",
				"T2JqZWN0LmFwcGx5KHRoaXMsIGFyZ3VtZW50cyk7CiAgICAgIHJldHVybiBuZXcgSGFzaChvYmopOwog",
				"ICAgfSwKCiAgICB0b0NTUzogZnVuY3Rpb24oKSB7CiAgICAgIHZhciBhcmdzID0gJEEoYXJndW1lbnRz",
				"KTsKICAgICAgdmFyIGtleXMgPSAoYXJncy5sZW5ndGggPT09IDApID8gRWxlbWVudC5MYXlvdXQuUFJP",
				"UEVSVElFUyA6CiAgICAgICBhcmdzLmpvaW4oJyAnKS5zcGxpdCgnICcpOwogICAgICB2YXIgY3NzID0g",
				"e307CgogICAgICBrZXlzLmVhY2goIGZ1bmN0aW9uKGtleSkgewogICAgICAgIGlmICghRWxlbWVudC5M",
				"YXlvdXQuUFJPUEVSVElFUy5pbmNsdWRlKGtleSkpIHJldHVybjsKICAgICAgICBpZiAoRWxlbWVudC5M",
				"YXlvdXQuQ09NUE9TSVRFX1BST1BFUlRJRVMuaW5jbHVkZShrZXkpKSByZXR1cm47CgogICAgICAgIHZh",
				"ciB2YWx1ZSA9IHRoaXMuZ2V0KGtleSk7CiAgICAgICAgaWYgKHZhbHVlICE9IG51bGwpIGNzc1tjc3NO",
				"YW1lRm9yKGtleSldID0gdmFsdWUgKyAncHgnOwogICAgICB9LCB0aGlzKTsKICAgICAgcmV0dXJuIGNz",
				"czsKICAgIH0sCgogICAgaW5zcGVjdDogZnVuY3Rpb24oKSB7CiAgICAgIHJldHVybiAiIzxFbGVtZW50",
				"LkxheW91dD4iOwogICAgfQogIH0pOwoKICBPYmplY3QuZXh0ZW5kKEVsZW1lbnQuTGF5b3V0LCB7CiAg",
				"ICBQUk9QRVJUSUVTOiAkdygnaGVpZ2h0IHdpZHRoIHRvcCBsZWZ0IHJpZ2h0IGJvdHRvbSBib3JkZXIt",
				"bGVmdCBib3JkZXItcmlnaHQgYm9yZGVyLXRvcCBib3JkZXItYm90dG9tIHBhZGRpbmctbGVmdCBwYWRk",
				"aW5nLXJpZ2h0IHBhZGRpbmctdG9wIHBhZGRpbmctYm90dG9tIG1hcmdpbi10b3AgbWFyZ2luLWJvdHRv",
				"bSBtYXJnaW4tbGVmdCBtYXJnaW4tcmlnaHQgcGFkZGluZy1ib3gtd2lkdGggcGFkZGluZy1ib3gtaGVp",
				"Z2h0IGJvcmRlci1ib3gtd2lkdGggYm9yZGVyLWJveC1oZWlnaHQgbWFyZ2luLWJveC13aWR0aCBtYXJn",
				"aW4tYm94LWhlaWdodCcpLAoKICAgIENPTVBPU0lURV9QUk9QRVJUSUVTOiAkdygncGFkZGluZy1ib3gt",
				"d2lkdGggcGFkZGluZy1ib3gtaGVpZ2h0IG1hcmdpbi1ib3gtd2lkdGggbWFyZ2luLWJveC1oZWlnaHQg",
				"Ym9yZGVyLWJveC13aWR0aCBib3JkZXItYm94LWhlaWdodCcpLAoKICAgIENPTVBVVEFUSU9OUzogewog",
				"ICAgICAnaGVpZ2h0JzogZnVuY3Rpb24oZWxlbWVudCkgewogICAgICAgIGlmICghdGhpcy5fcHJlQ29t",
				"cHV0aW5nKSB0aGlzLl9iZWdpbigpOwoKICAgICAgICB2YXIgYkhlaWdodCA9IHRoaXMuZ2V0KCdib3Jk",
				"ZXItYm94LWhlaWdodCcpOwogICAgICAgIGlmIChiSGVpZ2h0IDw9IDApIHsKICAgICAgICAgIGlmICgh",
				"dGhpcy5fcHJlQ29tcHV0aW5nKSB0aGlzLl9lbmQoKTsKICAgICAgICAgIHJldHVybiAwOwogICAgICAg",
				"IH0KCiAgICAgICAgdmFyIGJUb3AgPSB0aGlzLmdldCgnYm9yZGVyLXRvcCcpLAogICAgICAgICBiQm90",
				"dG9tID0gdGhpcy5nZXQoJ2JvcmRlci1ib3R0b20nKTsKCiAgICAgICAgdmFyIHBUb3AgPSB0aGlzLmdl",
				"dCgncGFkZGluZy10b3AnKSwKICAgICAgICAgcEJvdHRvbSA9IHRoaXMuZ2V0KCdwYWRkaW5nLWJvdHRv",
				"bScpOwoKICAgICAgICBpZiAoIXRoaXMuX3ByZUNvbXB1dGluZykgdGhpcy5fZW5kKCk7CgogICAgICAg",
				"IHJldHVybiBiSGVpZ2h0IC0gYlRvcCAtIGJCb3R0b20gLSBwVG9wIC0gcEJvdHRvbTsKICAgICAgfSwK",
				"CiAgICAgICd3aWR0aCc6IGZ1bmN0aW9uKGVsZW1lbnQpIHsKICAgICAgICBpZiAoIXRoaXMuX3ByZUNv",
				"bXB1dGluZykgdGhpcy5fYmVnaW4oKTsKCiAgICAgICAgdmFyIGJXaWR0aCA9IHRoaXMuZ2V0KCdib3Jk",
				"ZXItYm94LXdpZHRoJyk7CiAgICAgICAgaWYgKGJXaWR0aCA8PSAwKSB7CiAgICAgICAgICBpZiAoIXRo",
				"aXMuX3ByZUNvbXB1dGluZykgdGhpcy5fZW5kKCk7CiAgICAgICAgICByZXR1cm4gMDsKICAgICAgICB9",
				"CgogICAgICAgIHZhciBiTGVmdCA9IHRoaXMuZ2V0KCdib3JkZXItbGVmdCcpLAogICAgICAgICBiUmln",
				"aHQgPSB0aGlzLmdldCgnYm9yZGVyLXJpZ2h0Jyk7CgogICAgICAgIHZhciBwTGVmdCA9IHRoaXMuZ2V0",
				"KCdwYWRkaW5nLWxlZnQnKSwKICAgICAgICAgcFJpZ2h0ID0gdGhpcy5nZXQoJ3BhZGRpbmctcmlnaHQn",
				"KTsKCiAgICAgICAgaWYgKCF0aGlzLl9wcmVDb21wdXRpbmcpIHRoaXMuX2VuZCgpOwoKICAgICAgICBy",
				"ZXR1cm4gYldpZHRoIC0gYkxlZnQgLSBiUmlnaHQgLSBwTGVmdCAtIHBSaWdodDsKICAgICAgfSwKCiAg",
				"ICAgICdwYWRkaW5nLWJveC1oZWlnaHQnOiBmdW5jdGlvbihlbGVtZW50KSB7CiAgICAgICAgdmFyIGhl",
				"aWdodCA9IHRoaXMuZ2V0KCdoZWlnaHQnKSwKICAgICAgICAgcFRvcCA9IHRoaXMuZ2V0KCdwYWRkaW5n",
				"LXRvcCcpLAogICAgICAgICBwQm90dG9tID0gdGhpcy5nZXQoJ3BhZGRpbmctYm90dG9tJyk7CgogICAg",
				"ICAgIHJldHVybiBoZWlnaHQgKyBwVG9wICsgcEJvdHRvbTsKICAgICAgfSwKCiAgICAgICdwYWRkaW5n",
				"LWJveC13aWR0aCc6IGZ1bmN0aW9uKGVsZW1lbnQpIHsKICAgICAgICB2YXIgd2lkdGggPSB0aGlzLmdl",
				"dCgnd2lkdGgnKSwKICAgICAgICAgcExlZnQgPSB0aGlzLmdldCgncGFkZGluZy1sZWZ0JyksCiAgICAg",
				"ICAgIHBSaWdodCA9IHRoaXMuZ2V0KCdwYWRkaW5nLXJpZ2h0Jyk7CgogICAgICAgIHJldHVybiB3aWR0",
				"aCArIHBMZWZ0ICsgcFJpZ2h0OwogICAgICB9LAoKICAgICAgJ2JvcmRlci1ib3gtaGVpZ2h0JzogZnVu",
				"Y3Rpb24oZWxlbWVudCkgewogICAgICAgIGlmICghdGhpcy5fcHJlQ29tcHV0aW5nKSB0aGlzLl9iZWdp",
				"bigpOwogICAgICAgIHZhciBoZWlnaHQgPSBlbGVtZW50Lm9mZnNldEhlaWdodDsKICAgICAgICBpZiAo",
				"IXRoaXMuX3ByZUNvbXB1dGluZykgdGhpcy5fZW5kKCk7CiAgICAgICAgcmV0dXJuIGhlaWdodDsKICAg",
				"ICAgfSwKCiAgICAgICdib3JkZXItYm94LXdpZHRoJzogZnVuY3Rpb24oZWxlbWVudCkgewogICAgICAg",
				"IGlmICghdGhpcy5fcHJlQ29tcHV0aW5nKSB0aGlzLl9iZWdpbigpOwogICAgICAgIHZhciB3aWR0aCA9",
				"IGVsZW1lbnQub2Zmc2V0V2lkdGg7CiAgICAgICAgaWYgKCF0aGlzLl9wcmVDb21wdXRpbmcpIHRoaXMu",
				"X2VuZCgpOwogICAgICAgIHJldHVybiB3aWR0aDsKICAgICAgfSwKCiAgICAgICdtYXJnaW4tYm94LWhl",
				"aWdodCc6IGZ1bmN0aW9uKGVsZW1lbnQpIHsKICAgICAgICB2YXIgYkhlaWdodCA9IHRoaXMuZ2V0KCdi",
				"b3JkZXItYm94LWhlaWdodCcpLAogICAgICAgICBtVG9wID0gdGhpcy5nZXQoJ21hcmdpbi10b3AnKSwK",
				"ICAgICAgICAgbUJvdHRvbSA9IHRoaXMuZ2V0KCdtYXJnaW4tYm90dG9tJyk7CgogICAgICAgIGlmIChi",
				"SGVpZ2h0IDw9IDApIHJldHVybiAwOwoKICAgICAgICByZXR1cm4gYkhlaWdodCArIG1Ub3AgKyBtQm90",
				"dG9tOwogICAgICB9LAoKICAgICAgJ21hcmdpbi1ib3gtd2lkdGgnOiBmdW5jdGlvbihlbGVtZW50KSB7",
				"CiAgICAgICAgdmFyIGJXaWR0aCA9IHRoaXMuZ2V0KCdib3JkZXItYm94LXdpZHRoJyksCiAgICAgICAg",
				"IG1MZWZ0ID0gdGhpcy5nZXQoJ21hcmdpbi1sZWZ0JyksCiAgICAgICAgIG1SaWdodCA9IHRoaXMuZ2V0",
				"KCdtYXJnaW4tcmlnaHQnKTsKCiAgICAgICAgaWYgKGJXaWR0aCA8PSAwKSByZXR1cm4gMDsKCiAgICAg",
				"ICAgcmV0dXJuIGJXaWR0aCArIG1MZWZ0ICsgbVJpZ2h0OwogICAgICB9LAoKICAgICAgJ3RvcCc6IGZ1",
				"bmN0aW9uKGVsZW1lbnQpIHsKICAgICAgICB2YXIgb2Zmc2V0ID0gZWxlbWVudC5wb3NpdGlvbmVkT2Zm",
				"c2V0KCk7CiAgICAgICAgcmV0dXJuIG9mZnNldC50b3A7CiAgICAgIH0sCgogICAgICAnYm90dG9tJzog",
				"ZnVuY3Rpb24oZWxlbWVudCkgewogICAgICAgIHZhciBvZmZzZXQgPSBlbGVtZW50LnBvc2l0aW9uZWRP",
				"ZmZzZXQoKSwKICAgICAgICAgcGFyZW50ID0gZWxlbWVudC5nZXRPZmZzZXRQYXJlbnQoKSwKICAgICAg",
				"ICAgcEhlaWdodCA9IHBhcmVudC5tZWFzdXJlKCdoZWlnaHQnKTsKCiAgICAgICAgdmFyIG1IZWlnaHQg",
				"PSB0aGlzLmdldCgnYm9yZGVyLWJveC1oZWlnaHQnKTsKCiAgICAgICAgcmV0dXJuIHBIZWlnaHQgLSBt",
				"SGVpZ2h0IC0gb2Zmc2V0LnRvcDsKICAgICAgfSwKCiAgICAgICdsZWZ0JzogZnVuY3Rpb24oZWxlbWVu",
				"dCkgewogICAgICAgIHZhciBvZmZzZXQgPSBlbGVtZW50LnBvc2l0aW9uZWRPZmZzZXQoKTsKICAgICAg",
				"ICByZXR1cm4gb2Zmc2V0LmxlZnQ7CiAgICAgIH0sCgogICAgICAncmlnaHQnOiBmdW5jdGlvbihlbGVt",
				"ZW50KSB7CiAgICAgICAgdmFyIG9mZnNldCA9IGVsZW1lbnQucG9zaXRpb25lZE9mZnNldCgpLAogICAg",
				"ICAgICBwYXJlbnQgPSBlbGVtZW50LmdldE9mZnNldFBhcmVudCgpLAogICAgICAgICBwV2lkdGggPSBw",
				"YXJlbnQubWVhc3VyZSgnd2lkdGgnKTsKCiAgICAgICAgdmFyIG1XaWR0aCA9IHRoaXMuZ2V0KCdib3Jk",
				"ZXItYm94LXdpZHRoJyk7CgogICAgICAgIHJldHVybiBwV2lkdGggLSBtV2lkdGggLSBvZmZzZXQubGVm",
				"dDsKICAgICAgfSwKCiAgICAgICdwYWRkaW5nLXRvcCc6IGZ1bmN0aW9uKGVsZW1lbnQpIHsKICAgICAg",
				"ICByZXR1cm4gZ2V0UGl4ZWxWYWx1ZShlbGVtZW50LCAncGFkZGluZ1RvcCcpOwogICAgICB9LAoKICAg",
				"ICAgJ3BhZGRpbmctYm90dG9tJzogZnVuY3Rpb24oZWxlbWVudCkgewogICAgICAgIHJldHVybiBnZXRQ",
				"aXhlbFZhbHVlKGVsZW1lbnQsICdwYWRkaW5nQm90dG9tJyk7CiAgICAgIH0sCgogICAgICAncGFkZGlu",
				"Zy1sZWZ0JzogZnVuY3Rpb24oZWxlbWVudCkgewogICAgICAgIHJldHVybiBnZXRQaXhlbFZhbHVlKGVs",
				"ZW1lbnQsICdwYWRkaW5nTGVmdCcpOwogICAgICB9LAoKICAgICAgJ3BhZGRpbmctcmlnaHQnOiBmdW5j",
				"dGlvbihlbGVtZW50KSB7CiAgICAgICAgcmV0dXJuIGdldFBpeGVsVmFsdWUoZWxlbWVudCwgJ3BhZGRp",
				"bmdSaWdodCcpOwogICAgICB9LAoKICAgICAgJ2JvcmRlci10b3AnOiBmdW5jdGlvbihlbGVtZW50KSB7",
				"CiAgICAgICAgcmV0dXJuIGdldFBpeGVsVmFsdWUoZWxlbWVudCwgJ2JvcmRlclRvcFdpZHRoJyk7CiAg",
				"ICAgIH0sCgogICAgICAnYm9yZGVyLWJvdHRvbSc6IGZ1bmN0aW9uKGVsZW1lbnQpIHsKICAgICAgICBy",
				"ZXR1cm4gZ2V0UGl4ZWxWYWx1ZShlbGVtZW50LCAnYm9yZGVyQm90dG9tV2lkdGgnKTsKICAgICAgfSwK",
				"CiAgICAgICdib3JkZXItbGVmdCc6IGZ1bmN0aW9uKGVsZW1lbnQpIHsKICAgICAgICByZXR1cm4gZ2V0",
				"UGl4ZWxWYWx1ZShlbGVtZW50LCAnYm9yZGVyTGVmdFdpZHRoJyk7CiAgICAgIH0sCgogICAgICAnYm9y",
				"ZGVyLXJpZ2h0JzogZnVuY3Rpb24oZWxlbWVudCkgewogICAgICAgIHJldHVybiBnZXRQaXhlbFZhbHVl",
				"KGVsZW1lbnQsICdib3JkZXJSaWdodFdpZHRoJyk7CiAgICAgIH0sCgogICAgICAnbWFyZ2luLXRvcCc6",
				"IGZ1bmN0aW9uKGVsZW1lbnQpIHsKICAgICAgICByZXR1cm4gZ2V0UGl4ZWxWYWx1ZShlbGVtZW50LCAn",
				"bWFyZ2luVG9wJyk7CiAgICAgIH0sCgogICAgICAnbWFyZ2luLWJvdHRvbSc6IGZ1bmN0aW9uKGVsZW1l",
				"bnQpIHsKICAgICAgICByZXR1cm4gZ2V0UGl4ZWxWYWx1ZShlbGVtZW50LCAnbWFyZ2luQm90dG9tJyk7",
				"CiAgICAgIH0sCgogICAgICAnbWFyZ2luLWxlZnQnOiBmdW5jdGlvbihlbGVtZW50KSB7CiAgICAgICAg",
				"cmV0dXJuIGdldFBpeGVsVmFsdWUoZWxlbWVudCwgJ21hcmdpbkxlZnQnKTsKICAgICAgfSwKCiAgICAg",
				"ICdtYXJnaW4tcmlnaHQnOiBmdW5jdGlvbihlbGVtZW50KSB7CiAgICAgICAgcmV0dXJuIGdldFBpeGVs",
				"VmFsdWUoZWxlbWVudCwgJ21hcmdpblJpZ2h0Jyk7CiAgICAgIH0KICAgIH0KICB9KTsKCiAgaWYgKCdn",
				"ZXRCb3VuZGluZ0NsaWVudFJlY3QnIGluIGRvY3VtZW50LmRvY3VtZW50RWxlbWVudCkgewogICAgT2Jq",
				"ZWN0LmV4dGVuZChFbGVtZW50LkxheW91dC5DT01QVVRBVElPTlMsIHsKICAgICAgJ3JpZ2h0JzogZnVu",
				"Y3Rpb24oZWxlbWVudCkgewogICAgICAgIHZhciBwYXJlbnQgPSBoYXNMYXlvdXQoZWxlbWVudC5nZXRP",
				"ZmZzZXRQYXJlbnQoKSk7CiAgICAgICAgdmFyIHJlY3QgPSBlbGVtZW50LmdldEJvdW5kaW5nQ2xpZW50",
				"UmVjdCgpLAogICAgICAgICBwUmVjdCA9IHBhcmVudC5nZXRCb3VuZGluZ0NsaWVudFJlY3QoKTsKCiAg",
				"ICAgICAgcmV0dXJuIChwUmVjdC5yaWdodCAtIHJlY3QucmlnaHQpLnJvdW5kKCk7CiAgICAgIH0sCgog",
				"ICAgICAnYm90dG9tJzogZnVuY3Rpb24oZWxlbWVudCkgewogICAgICAgIHZhciBwYXJlbnQgPSBoYXNM",
				"YXlvdXQoZWxlbWVudC5nZXRPZmZzZXRQYXJlbnQoKSk7CiAgICAgICAgdmFyIHJlY3QgPSBlbGVtZW50",
				"LmdldEJvdW5kaW5nQ2xpZW50UmVjdCgpLAogICAgICAgICBwUmVjdCA9IHBhcmVudC5nZXRCb3VuZGlu",
				"Z0NsaWVudFJlY3QoKTsKCiAgICAgICAgcmV0dXJuIChwUmVjdC5ib3R0b20gLSByZWN0LmJvdHRvbSku",
				"cm91bmQoKTsKICAgICAgfQogICAgfSk7CiAgfQoKICBFbGVtZW50Lk9mZnNldCA9IENsYXNzLmNyZWF0",
				"ZSh7CiAgICBpbml0aWFsaXplOiBmdW5jdGlvbihsZWZ0LCB0b3ApIHsKICAgICAgdGhpcy5sZWZ0ID0g",
				"bGVmdC5yb3VuZCgpOwogICAgICB0aGlzLnRvcCAgPSB0b3Aucm91bmQoKTsKCiAgICAgIHRoaXNbMF0g",
				"PSB0aGlzLmxlZnQ7CiAgICAgIHRoaXNbMV0gPSB0aGlzLnRvcDsKICAgIH0sCgogICAgcmVsYXRpdmVU",
				"bzogZnVuY3Rpb24ob2Zmc2V0KSB7CiAgICAgIHJldHVybiBuZXcgRWxlbWVudC5PZmZzZXQoCiAgICAg",
				"ICAgdGhpcy5sZWZ0IC0gb2Zmc2V0LmxlZnQsCiAgICAgICAgdGhpcy50b3AgIC0gb2Zmc2V0LnRvcAog",
				"ICAgICApOwogICAgfSwKCiAgICBpbnNwZWN0OiBmdW5jdGlvbigpIHsKICAgICAgcmV0dXJuICIjPEVs",
				"ZW1lbnQuT2Zmc2V0IGxlZnQ6ICN7bGVmdH0gdG9wOiAje3RvcH0+Ii5pbnRlcnBvbGF0ZSh0aGlzKTsK",
				"ICAgIH0sCgogICAgdG9TdHJpbmc6IGZ1bmN0aW9uKCkgewogICAgICByZXR1cm4gIlsje2xlZnR9LCAj",
				"e3RvcH1dIi5pbnRlcnBvbGF0ZSh0aGlzKTsKICAgIH0sCgogICAgdG9BcnJheTogZnVuY3Rpb24oKSB7",
				"CiAgICAgIHJldHVybiBbdGhpcy5sZWZ0LCB0aGlzLnRvcF07CiAgICB9CiAgfSk7CgogIGZ1bmN0aW9u",
				"IGdldExheW91dChlbGVtZW50LCBwcmVDb21wdXRlKSB7CiAgICByZXR1cm4gbmV3IEVsZW1lbnQuTGF5",
				"b3V0KGVsZW1lbnQsIHByZUNvbXB1dGUpOwogIH0KCiAgZnVuY3Rpb24gbWVhc3VyZShlbGVtZW50LCBw",
				"cm9wZXJ0eSkgewogICAgcmV0dXJuICQoZWxlbWVudCkuZ2V0TGF5b3V0KCkuZ2V0KHByb3BlcnR5KTsK",
				"ICB9CgogIGZ1bmN0aW9uIGdldERpbWVuc2lvbnMoZWxlbWVudCkgewogICAgZWxlbWVudCA9ICQoZWxl",
				"bWVudCk7CiAgICB2YXIgZGlzcGxheSA9IEVsZW1lbnQuZ2V0U3R5bGUoZWxlbWVudCwgJ2Rpc3BsYXkn",
				"KTsKCiAgICBpZiAoZGlzcGxheSAmJiBkaXNwbGF5ICE9PSAnbm9uZScpIHsKICAgICAgcmV0dXJuIHsg",
				"d2lkdGg6IGVsZW1lbnQub2Zmc2V0V2lkdGgsIGhlaWdodDogZWxlbWVudC5vZmZzZXRIZWlnaHQgfTsK",
				"ICAgIH0KCiAgICB2YXIgc3R5bGUgPSBlbGVtZW50LnN0eWxlOwogICAgdmFyIG9yaWdpbmFsU3R5bGVz",
				"ID0gewogICAgICB2aXNpYmlsaXR5OiBzdHlsZS52aXNpYmlsaXR5LAogICAgICBwb3NpdGlvbjogICBz",
				"dHlsZS5wb3NpdGlvbiwKICAgICAgZGlzcGxheTogICAgc3R5bGUuZGlzcGxheQogICAgfTsKCiAgICB2",
				"YXIgbmV3U3R5bGVzID0gewogICAgICB2aXNpYmlsaXR5OiAnaGlkZGVuJywKICAgICAgZGlzcGxheTog",
				"ICAgJ2Jsb2NrJwogICAgfTsKCiAgICBpZiAob3JpZ2luYWxTdHlsZXMucG9zaXRpb24gIT09ICdmaXhl",
				"ZCcpCiAgICAgIG5ld1N0eWxlcy5wb3NpdGlvbiA9ICdhYnNvbHV0ZSc7CgogICAgRWxlbWVudC5zZXRT",
				"dHlsZShlbGVtZW50LCBuZXdTdHlsZXMpOwoKICAgIHZhciBkaW1lbnNpb25zID0gewogICAgICB3aWR0",
				"aDogIGVsZW1lbnQub2Zmc2V0V2lkdGgsCiAgICAgIGhlaWdodDogZWxlbWVudC5vZmZzZXRIZWlnaHQK",
				"ICAgIH07CgogICAgRWxlbWVudC5zZXRTdHlsZShlbGVtZW50LCBvcmlnaW5hbFN0eWxlcyk7CgogICAg",
				"cmV0dXJuIGRpbWVuc2lvbnM7CiAgfQoKICBmdW5jdGlvbiBnZXRPZmZzZXRQYXJlbnQoZWxlbWVudCkg",
				"ewogICAgZWxlbWVudCA9ICQoZWxlbWVudCk7CgogICAgaWYgKGlzRG9jdW1lbnQoZWxlbWVudCkgfHwg",
				"aXNEZXRhY2hlZChlbGVtZW50KSB8fCBpc0JvZHkoZWxlbWVudCkgfHwgaXNIdG1sKGVsZW1lbnQpKQog",
				"ICAgICByZXR1cm4gJChkb2N1bWVudC5ib2R5KTsKCiAgICB2YXIgaXNJbmxpbmUgPSAoRWxlbWVudC5n",
				"ZXRTdHlsZShlbGVtZW50LCAnZGlzcGxheScpID09PSAnaW5saW5lJyk7CiAgICBpZiAoIWlzSW5saW5l",
				"ICYmIGVsZW1lbnQub2Zmc2V0UGFyZW50KSByZXR1cm4gJChlbGVtZW50Lm9mZnNldFBhcmVudCk7Cgog",
				"ICAgd2hpbGUgKChlbGVtZW50ID0gZWxlbWVudC5wYXJlbnROb2RlKSAmJiBlbGVtZW50ICE9PSBkb2N1",
				"bWVudC5ib2R5KSB7CiAgICAgIGlmIChFbGVtZW50LmdldFN0eWxlKGVsZW1lbnQsICdwb3NpdGlvbicp",
				"ICE9PSAnc3RhdGljJykgewogICAgICAgIHJldHVybiBpc0h0bWwoZWxlbWVudCkgPyAkKGRvY3VtZW50",
				"LmJvZHkpIDogJChlbGVtZW50KTsKICAgICAgfQogICAgfQoKICAgIHJldHVybiAkKGRvY3VtZW50LmJv",
				"ZHkpOwogIH0KCgogIGZ1bmN0aW9uIGN1bXVsYXRpdmVPZmZzZXQoZWxlbWVudCkgewogICAgZWxlbWVu",
				"dCA9ICQoZWxlbWVudCk7CiAgICB2YXIgdmFsdWVUID0gMCwgdmFsdWVMID0gMDsKICAgIGlmIChlbGVt",
				"ZW50LnBhcmVudE5vZGUpIHsKICAgICAgZG8gewogICAgICAgIHZhbHVlVCArPSBlbGVtZW50Lm9mZnNl",
				"dFRvcCAgfHwgMDsKICAgICAgICB2YWx1ZUwgKz0gZWxlbWVudC5vZmZzZXRMZWZ0IHx8IDA7CiAgICAg",
				"ICAgZWxlbWVudCA9IGVsZW1lbnQub2Zmc2V0UGFyZW50OwogICAgICB9IHdoaWxlIChlbGVtZW50KTsK",
				"ICAgIH0KICAgIHJldHVybiBuZXcgRWxlbWVudC5PZmZzZXQodmFsdWVMLCB2YWx1ZVQpOwogIH0KCiAg",
				"ZnVuY3Rpb24gcG9zaXRpb25lZE9mZnNldChlbGVtZW50KSB7CiAgICBlbGVtZW50ID0gJChlbGVtZW50",
				"KTsKCiAgICB2YXIgbGF5b3V0ID0gZWxlbWVudC5nZXRMYXlvdXQoKTsKCiAgICB2YXIgdmFsdWVUID0g",
				"MCwgdmFsdWVMID0gMDsKICAgIGRvIHsKICAgICAgdmFsdWVUICs9IGVsZW1lbnQub2Zmc2V0VG9wICB8",
				"fCAwOwogICAgICB2YWx1ZUwgKz0gZWxlbWVudC5vZmZzZXRMZWZ0IHx8IDA7CiAgICAgIGVsZW1lbnQg",
				"PSBlbGVtZW50Lm9mZnNldFBhcmVudDsKICAgICAgaWYgKGVsZW1lbnQpIHsKICAgICAgICBpZiAoaXNC",
				"b2R5KGVsZW1lbnQpKSBicmVhazsKICAgICAgICB2YXIgcCA9IEVsZW1lbnQuZ2V0U3R5bGUoZWxlbWVu",
				"dCwgJ3Bvc2l0aW9uJyk7CiAgICAgICAgaWYgKHAgIT09ICdzdGF0aWMnKSBicmVhazsKICAgICAgfQog",
				"ICAgfSB3aGlsZSAoZWxlbWVudCk7CgogICAgdmFsdWVMIC09IGxheW91dC5nZXQoJ21hcmdpbi10b3An",
				"KTsKICAgIHZhbHVlVCAtPSBsYXlvdXQuZ2V0KCdtYXJnaW4tbGVmdCcpOwoKICAgIHJldHVybiBuZXcg",
				"RWxlbWVudC5PZmZzZXQodmFsdWVMLCB2YWx1ZVQpOwogIH0KCiAgZnVuY3Rpb24gY3VtdWxhdGl2ZVNj",
				"cm9sbE9mZnNldChlbGVtZW50KSB7CiAgICB2YXIgdmFsdWVUID0gMCwgdmFsdWVMID0gMDsKICAgIGRv",
				"IHsKICAgICAgdmFsdWVUICs9IGVsZW1lbnQuc2Nyb2xsVG9wICB8fCAwOwogICAgICB2YWx1ZUwgKz0g",
				"ZWxlbWVudC5zY3JvbGxMZWZ0IHx8IDA7CiAgICAgIGVsZW1lbnQgPSBlbGVtZW50LnBhcmVudE5vZGU7",
				"CiAgICB9IHdoaWxlIChlbGVtZW50KTsKICAgIHJldHVybiBuZXcgRWxlbWVudC5PZmZzZXQodmFsdWVM",
				"LCB2YWx1ZVQpOwogIH0KCiAgZnVuY3Rpb24gdmlld3BvcnRPZmZzZXQoZm9yRWxlbWVudCkgewogICAg",
				"ZWxlbWVudCA9ICQoZWxlbWVudCk7CiAgICB2YXIgdmFsdWVUID0gMCwgdmFsdWVMID0gMCwgZG9jQm9k",
				"eSA9IGRvY3VtZW50LmJvZHk7CgogICAgdmFyIGVsZW1lbnQgPSBmb3JFbGVtZW50OwogICAgZG8gewog",
				"ICAgICB2YWx1ZVQgKz0gZWxlbWVudC5vZmZzZXRUb3AgIHx8IDA7CiAgICAgIHZhbHVlTCArPSBlbGVt",
				"ZW50Lm9mZnNldExlZnQgfHwgMDsKICAgICAgaWYgKGVsZW1lbnQub2Zmc2V0UGFyZW50ID09IGRvY0Jv",
				"ZHkgJiYKICAgICAgICBFbGVtZW50LmdldFN0eWxlKGVsZW1lbnQsICdwb3NpdGlvbicpID09ICdhYnNv",
				"bHV0ZScpIGJyZWFrOwogICAgfSB3aGlsZSAoZWxlbWVudCA9IGVsZW1lbnQub2Zmc2V0UGFyZW50KTsK",
				"CiAgICBlbGVtZW50ID0gZm9yRWxlbWVudDsKICAgIGRvIHsKICAgICAgaWYgKGVsZW1lbnQgIT0gZG9j",
				"Qm9keSkgewogICAgICAgIHZhbHVlVCAtPSBlbGVtZW50LnNjcm9sbFRvcCAgfHwgMDsKICAgICAgICB2",
				"YWx1ZUwgLT0gZWxlbWVudC5zY3JvbGxMZWZ0IHx8IDA7CiAgICAgIH0KICAgIH0gd2hpbGUgKGVsZW1l",
				"bnQgPSBlbGVtZW50LnBhcmVudE5vZGUpOwogICAgcmV0dXJuIG5ldyBFbGVtZW50Lk9mZnNldCh2YWx1",
				"ZUwsIHZhbHVlVCk7CiAgfQoKICBmdW5jdGlvbiBhYnNvbHV0aXplKGVsZW1lbnQpIHsKICAgIGVsZW1l",
				"bnQgPSAkKGVsZW1lbnQpOwoKICAgIGlmIChFbGVtZW50LmdldFN0eWxlKGVsZW1lbnQsICdwb3NpdGlv",
				"bicpID09PSAnYWJzb2x1dGUnKSB7CiAgICAgIHJldHVybiBlbGVtZW50OwogICAgfQoKICAgIHZhciBv",
				"ZmZzZXRQYXJlbnQgPSBnZXRPZmZzZXRQYXJlbnQoZWxlbWVudCk7CiAgICB2YXIgZU9mZnNldCA9IGVs",
				"ZW1lbnQudmlld3BvcnRPZmZzZXQoKSwKICAgICBwT2Zmc2V0ID0gb2Zmc2V0UGFyZW50LnZpZXdwb3J0",
				"T2Zmc2V0KCk7CgogICAgdmFyIG9mZnNldCA9IGVPZmZzZXQucmVsYXRpdmVUbyhwT2Zmc2V0KTsKICAg",
				"IHZhciBsYXlvdXQgPSBlbGVtZW50LmdldExheW91dCgpOwoKICAgIGVsZW1lbnQuc3RvcmUoJ3Byb3Rv",
				"dHlwZV9hYnNvbHV0aXplX29yaWdpbmFsX3N0eWxlcycsIHsKICAgICAgbGVmdDogICBlbGVtZW50Lmdl",
				"dFN0eWxlKCdsZWZ0JyksCiAgICAgIHRvcDogICAgZWxlbWVudC5nZXRTdHlsZSgndG9wJyksCiAgICAg",
				"IHdpZHRoOiAgZWxlbWVudC5nZXRTdHlsZSgnd2lkdGgnKSwKICAgICAgaGVpZ2h0OiBlbGVtZW50Lmdl",
				"dFN0eWxlKCdoZWlnaHQnKQogICAgfSk7CgogICAgZWxlbWVudC5zZXRTdHlsZSh7CiAgICAgIHBvc2l0",
				"aW9uOiAnYWJzb2x1dGUnLAogICAgICB0b3A6ICAgIG9mZnNldC50b3AgKyAncHgnLAogICAgICBsZWZ0",
				"OiAgIG9mZnNldC5sZWZ0ICsgJ3B4JywKICAgICAgd2lkdGg6ICBsYXlvdXQuZ2V0KCd3aWR0aCcpICsg",
				"J3B4JywKICAgICAgaGVpZ2h0OiBsYXlvdXQuZ2V0KCdoZWlnaHQnKSArICdweCcKICAgIH0pOwoKICAg",
				"IHJldHVybiBlbGVtZW50OwogIH0KCiAgZnVuY3Rpb24gcmVsYXRpdml6ZShlbGVtZW50KSB7CiAgICBl",
				"bGVtZW50ID0gJChlbGVtZW50KTsKICAgIGlmIChFbGVtZW50LmdldFN0eWxlKGVsZW1lbnQsICdwb3Np",
				"dGlvbicpID09PSAncmVsYXRpdmUnKSB7CiAgICAgIHJldHVybiBlbGVtZW50OwogICAgfQoKICAgIHZh",
				"ciBvcmlnaW5hbFN0eWxlcyA9CiAgICAgZWxlbWVudC5yZXRyaWV2ZSgncHJvdG90eXBlX2Fic29sdXRp",
				"emVfb3JpZ2luYWxfc3R5bGVzJyk7CgogICAgaWYgKG9yaWdpbmFsU3R5bGVzKSBlbGVtZW50LnNldFN0",
				"eWxlKG9yaWdpbmFsU3R5bGVzKTsKICAgIHJldHVybiBlbGVtZW50OwogIH0KCiAgaWYgKFByb3RvdHlw",
				"ZS5Ccm93c2VyLklFKSB7CiAgICBnZXRPZmZzZXRQYXJlbnQgPSBnZXRPZmZzZXRQYXJlbnQud3JhcCgK",
				"ICAgICAgZnVuY3Rpb24ocHJvY2VlZCwgZWxlbWVudCkgewogICAgICAgIGVsZW1lbnQgPSAkKGVsZW1l",
				"bnQpOwoKICAgICAgICBpZiAoaXNEb2N1bWVudChlbGVtZW50KSB8fCBpc0RldGFjaGVkKGVsZW1lbnQp",
				"IHx8IGlzQm9keShlbGVtZW50KSB8fCBpc0h0bWwoZWxlbWVudCkpCiAgICAgICAgICByZXR1cm4gJChk",
				"b2N1bWVudC5ib2R5KTsKCiAgICAgICAgdmFyIHBvc2l0aW9uID0gZWxlbWVudC5nZXRTdHlsZSgncG9z",
				"aXRpb24nKTsKICAgICAgICBpZiAocG9zaXRpb24gIT09ICdzdGF0aWMnKSByZXR1cm4gcHJvY2VlZChl",
				"bGVtZW50KTsKCiAgICAgICAgZWxlbWVudC5zZXRTdHlsZSh7IHBvc2l0aW9uOiAncmVsYXRpdmUnIH0p",
				"OwogICAgICAgIHZhciB2YWx1ZSA9IHByb2NlZWQoZWxlbWVudCk7CiAgICAgICAgZWxlbWVudC5zZXRT",
				"dHlsZSh7IHBvc2l0aW9uOiBwb3NpdGlvbiB9KTsKICAgICAgICByZXR1cm4gdmFsdWU7CiAgICAgIH0K",
				"ICAgICk7CgogICAgcG9zaXRpb25lZE9mZnNldCA9IHBvc2l0aW9uZWRPZmZzZXQud3JhcChmdW5jdGlv",
				"bihwcm9jZWVkLCBlbGVtZW50KSB7CiAgICAgIGVsZW1lbnQgPSAkKGVsZW1lbnQpOwogICAgICBpZiAo",
				"IWVsZW1lbnQucGFyZW50Tm9kZSkgcmV0dXJuIG5ldyBFbGVtZW50Lk9mZnNldCgwLCAwKTsKICAgICAg",
				"dmFyIHBvc2l0aW9uID0gZWxlbWVudC5nZXRTdHlsZSgncG9zaXRpb24nKTsKICAgICAgaWYgKHBvc2l0",
				"aW9uICE9PSAnc3RhdGljJykgcmV0dXJuIHByb2NlZWQoZWxlbWVudCk7CgogICAgICB2YXIgb2Zmc2V0",
				"UGFyZW50ID0gZWxlbWVudC5nZXRPZmZzZXRQYXJlbnQoKTsKICAgICAgaWYgKG9mZnNldFBhcmVudCAm",
				"JiBvZmZzZXRQYXJlbnQuZ2V0U3R5bGUoJ3Bvc2l0aW9uJykgPT09ICdmaXhlZCcpCiAgICAgICAgaGFz",
				"TGF5b3V0KG9mZnNldFBhcmVudCk7CgogICAgICBlbGVtZW50LnNldFN0eWxlKHsgcG9zaXRpb246ICdy",
				"ZWxhdGl2ZScgfSk7CiAgICAgIHZhciB2YWx1ZSA9IHByb2NlZWQoZWxlbWVudCk7CiAgICAgIGVsZW1l",
				"bnQuc2V0U3R5bGUoeyBwb3NpdGlvbjogcG9zaXRpb24gfSk7CiAgICAgIHJldHVybiB2YWx1ZTsKICAg",
				"IH0pOwogIH0gZWxzZSBpZiAoUHJvdG90eXBlLkJyb3dzZXIuV2Via2l0KSB7CiAgICBjdW11bGF0aXZl",
				"T2Zmc2V0ID0gZnVuY3Rpb24oZWxlbWVudCkgewogICAgICBlbGVtZW50ID0gJChlbGVtZW50KTsKICAg",
				"ICAgdmFyIHZhbHVlVCA9IDAsIHZhbHVlTCA9IDA7CiAgICAgIGRvIHsKICAgICAgICB2YWx1ZVQgKz0g",
				"ZWxlbWVudC5vZmZzZXRUb3AgIHx8IDA7CiAgICAgICAgdmFsdWVMICs9IGVsZW1lbnQub2Zmc2V0TGVm",
				"dCB8fCAwOwogICAgICAgIGlmIChlbGVtZW50Lm9mZnNldFBhcmVudCA9PSBkb2N1bWVudC5ib2R5KQog",
				"ICAgICAgICAgaWYgKEVsZW1lbnQuZ2V0U3R5bGUoZWxlbWVudCwgJ3Bvc2l0aW9uJykgPT0gJ2Fic29s",
				"dXRlJykgYnJlYWs7CgogICAgICAgIGVsZW1lbnQgPSBlbGVtZW50Lm9mZnNldFBhcmVudDsKICAgICAg",
				"fSB3aGlsZSAoZWxlbWVudCk7CgogICAgICByZXR1cm4gbmV3IEVsZW1lbnQuT2Zmc2V0KHZhbHVlTCwg",
				"dmFsdWVUKTsKICAgIH07CiAgfQoKCiAgRWxlbWVudC5hZGRNZXRob2RzKHsKICAgIGdldExheW91dDog",
				"ICAgICAgICAgICAgIGdldExheW91dCwKICAgIG1lYXN1cmU6ICAgICAgICAgICAgICAgIG1lYXN1cmUs",
				"CiAgICBnZXREaW1lbnNpb25zOiAgICAgICAgICBnZXREaW1lbnNpb25zLAogICAgZ2V0T2Zmc2V0UGFy",
				"ZW50OiAgICAgICAgZ2V0T2Zmc2V0UGFyZW50LAogICAgY3VtdWxhdGl2ZU9mZnNldDogICAgICAgY3Vt",
				"dWxhdGl2ZU9mZnNldCwKICAgIHBvc2l0aW9uZWRPZmZzZXQ6ICAgICAgIHBvc2l0aW9uZWRPZmZzZXQs",
				"CiAgICBjdW11bGF0aXZlU2Nyb2xsT2Zmc2V0OiBjdW11bGF0aXZlU2Nyb2xsT2Zmc2V0LAogICAgdmll",
				"d3BvcnRPZmZzZXQ6ICAgICAgICAgdmlld3BvcnRPZmZzZXQsCiAgICBhYnNvbHV0aXplOiAgICAgICAg",
				"ICAgICBhYnNvbHV0aXplLAogICAgcmVsYXRpdml6ZTogICAgICAgICAgICAgcmVsYXRpdml6ZQogIH0p",
				"OwoKICBmdW5jdGlvbiBpc0JvZHkoZWxlbWVudCkgewogICAgcmV0dXJuIGVsZW1lbnQubm9kZU5hbWUu",
				"dG9VcHBlckNhc2UoKSA9PT0gJ0JPRFknOwogIH0KCiAgZnVuY3Rpb24gaXNIdG1sKGVsZW1lbnQpIHsK",
				"ICAgIHJldHVybiBlbGVtZW50Lm5vZGVOYW1lLnRvVXBwZXJDYXNlKCkgPT09ICdIVE1MJzsKICB9Cgog",
				"IGZ1bmN0aW9uIGlzRG9jdW1lbnQoZWxlbWVudCkgewogICAgcmV0dXJuIGVsZW1lbnQubm9kZVR5cGUg",
				"PT09IE5vZGUuRE9DVU1FTlRfTk9ERTsKICB9CgogIGZ1bmN0aW9uIGlzRGV0YWNoZWQoZWxlbWVudCkg",
				"ewogICAgcmV0dXJuIGVsZW1lbnQgIT09IGRvY3VtZW50LmJvZHkgJiYKICAgICAhRWxlbWVudC5kZXNj",
				"ZW5kYW50T2YoZWxlbWVudCwgZG9jdW1lbnQuYm9keSk7CiAgfQoKICBpZiAoJ2dldEJvdW5kaW5nQ2xp",
				"ZW50UmVjdCcgaW4gZG9jdW1lbnQuZG9jdW1lbnRFbGVtZW50KSB7CiAgICBFbGVtZW50LmFkZE1ldGhv",
				"ZHMoewogICAgICB2aWV3cG9ydE9mZnNldDogZnVuY3Rpb24oZWxlbWVudCkgewogICAgICAgIGVsZW1l",
				"bnQgPSAkKGVsZW1lbnQpOwogICAgICAgIGlmIChpc0RldGFjaGVkKGVsZW1lbnQpKSByZXR1cm4gbmV3",
				"IEVsZW1lbnQuT2Zmc2V0KDAsIDApOwoKICAgICAgICB2YXIgcmVjdCA9IGVsZW1lbnQuZ2V0Qm91bmRp",
				"bmdDbGllbnRSZWN0KCksCiAgICAgICAgIGRvY0VsID0gZG9jdW1lbnQuZG9jdW1lbnRFbGVtZW50Owog",
				"ICAgICAgIHJldHVybiBuZXcgRWxlbWVudC5PZmZzZXQocmVjdC5sZWZ0IC0gZG9jRWwuY2xpZW50TGVm",
				"dCwKICAgICAgICAgcmVjdC50b3AgLSBkb2NFbC5jbGllbnRUb3ApOwogICAgICB9CiAgICB9KTsKICB9",
				"Cn0pKCk7CndpbmRvdy4kJCA9IGZ1bmN0aW9uKCkgewogIHZhciBleHByZXNzaW9uID0gJEEoYXJndW1l",
				"bnRzKS5qb2luKCcsICcpOwogIHJldHVybiBQcm90b3R5cGUuU2VsZWN0b3Iuc2VsZWN0KGV4cHJlc3Np",
				"b24sIGRvY3VtZW50KTsKfTsKClByb3RvdHlwZS5TZWxlY3RvciA9IChmdW5jdGlvbigpIHsKCiAgZnVu",
				"Y3Rpb24gc2VsZWN0KCkgewogICAgdGhyb3cgbmV3IEVycm9yKCdNZXRob2QgIlByb3RvdHlwZS5TZWxl",
				"Y3Rvci5zZWxlY3QiIG11c3QgYmUgZGVmaW5lZC4nKTsKICB9CgogIGZ1bmN0aW9uIG1hdGNoKCkgewog",
				"ICAgdGhyb3cgbmV3IEVycm9yKCdNZXRob2QgIlByb3RvdHlwZS5TZWxlY3Rvci5tYXRjaCIgbXVzdCBi",
				"ZSBkZWZpbmVkLicpOwogIH0KCiAgZnVuY3Rpb24gZmluZChlbGVtZW50cywgZXhwcmVzc2lvbiwgaW5k",
				"ZXgpIHsKICAgIGluZGV4ID0gaW5kZXggfHwgMDsKICAgIHZhciBtYXRjaCA9IFByb3RvdHlwZS5TZWxl",
				"Y3Rvci5tYXRjaCwgbGVuZ3RoID0gZWxlbWVudHMubGVuZ3RoLCBtYXRjaEluZGV4ID0gMCwgaTsKCiAg",
				"ICBmb3IgKGkgPSAwOyBpIDwgbGVuZ3RoOyBpKyspIHsKICAgICAgaWYgKG1hdGNoKGVsZW1lbnRzW2ld",
				"LCBleHByZXNzaW9uKSAmJiBpbmRleCA9PSBtYXRjaEluZGV4KyspIHsKICAgICAgICByZXR1cm4gRWxl",
				"bWVudC5leHRlbmQoZWxlbWVudHNbaV0pOwogICAgICB9CiAgICB9CiAgfQoKICBmdW5jdGlvbiBleHRl",
				"bmRFbGVtZW50cyhlbGVtZW50cykgewogICAgZm9yICh2YXIgaSA9IDAsIGxlbmd0aCA9IGVsZW1lbnRz",
				"Lmxlbmd0aDsgaSA8IGxlbmd0aDsgaSsrKSB7CiAgICAgIEVsZW1lbnQuZXh0ZW5kKGVsZW1lbnRzW2ld",
				"KTsKICAgIH0KICAgIHJldHVybiBlbGVtZW50czsKICB9CgoKICB2YXIgSyA9IFByb3RvdHlwZS5LOwoK",
				"ICByZXR1cm4gewogICAgc2VsZWN0OiBzZWxlY3QsCiAgICBtYXRjaDogbWF0Y2gsCiAgICBmaW5kOiBm",
				"aW5kLAogICAgZXh0ZW5kRWxlbWVudHM6IChFbGVtZW50LmV4dGVuZCA9PT0gSykgPyBLIDogZXh0ZW5k",
				"RWxlbWVudHMsCiAgICBleHRlbmRFbGVtZW50OiBFbGVtZW50LmV4dGVuZAogIH07Cn0pKCk7ClByb3Rv",
				"dHlwZS5fb3JpZ2luYWxfcHJvcGVydHkgPSB3aW5kb3cuU2l6emxlOwovKiEKICogU2l6emxlIENTUyBT",
				"ZWxlY3RvciBFbmdpbmUgLSB2MS4wCiAqICBDb3B5cmlnaHQgMjAwOSwgVGhlIERvam8gRm91bmRhdGlv",
				"bgogKiAgUmVsZWFzZWQgdW5kZXIgdGhlIE1JVCwgQlNELCBhbmQgR1BMIExpY2Vuc2VzLgogKiAgTW9y",
				"ZSBpbmZvcm1hdGlvbjogaHR0cDovL3NpenpsZWpzLmNvbS8KICovCihmdW5jdGlvbigpewoKdmFyIGNo",
				"dW5rZXIgPSAvKCg/OlwoKD86XChbXigpXStcKXxbXigpXSspK1wpfFxbKD86XFtbXltcXV0qXF18Wyci",
				"XVteJyJdKlsnIl18W15bXF0nIl0rKStcXXxcXC58W14gPit+LChcW1xcXSspK3xbPit+XSkoXHMqLFxz",
				"Kik/KCg/Oi58XHJ8XG4pKikvZywKCWRvbmUgPSAwLAoJdG9TdHJpbmcgPSBPYmplY3QucHJvdG90eXBl",
				"LnRvU3RyaW5nLAoJaGFzRHVwbGljYXRlID0gZmFsc2UsCgliYXNlSGFzRHVwbGljYXRlID0gdHJ1ZTsK",
				"ClswLCAwXS5zb3J0KGZ1bmN0aW9uKCl7CgliYXNlSGFzRHVwbGljYXRlID0gZmFsc2U7CglyZXR1cm4g",
				"MDsKfSk7Cgp2YXIgU2l6emxlID0gZnVuY3Rpb24oc2VsZWN0b3IsIGNvbnRleHQsIHJlc3VsdHMsIHNl",
				"ZWQpIHsKCXJlc3VsdHMgPSByZXN1bHRzIHx8IFtdOwoJdmFyIG9yaWdDb250ZXh0ID0gY29udGV4dCA9",
				"IGNvbnRleHQgfHwgZG9jdW1lbnQ7CgoJaWYgKCBjb250ZXh0Lm5vZGVUeXBlICE9PSAxICYmIGNvbnRl",
				"eHQubm9kZVR5cGUgIT09IDkgKSB7CgkJcmV0dXJuIFtdOwoJfQoKCWlmICggIXNlbGVjdG9yIHx8IHR5",
				"cGVvZiBzZWxlY3RvciAhPT0gInN0cmluZyIgKSB7CgkJcmV0dXJuIHJlc3VsdHM7Cgl9CgoJdmFyIHBh",
				"cnRzID0gW10sIG0sIHNldCwgY2hlY2tTZXQsIGNoZWNrLCBtb2RlLCBleHRyYSwgcHJ1bmUgPSB0cnVl",
				"LCBjb250ZXh0WE1MID0gaXNYTUwoY29udGV4dCksCgkJc29GYXIgPSBzZWxlY3RvcjsKCgl3aGlsZSAo",
				"IChjaHVua2VyLmV4ZWMoIiIpLCBtID0gY2h1bmtlci5leGVjKHNvRmFyKSkgIT09IG51bGwgKSB7CgkJ",
				"c29GYXIgPSBtWzNdOwoKCQlwYXJ0cy5wdXNoKCBtWzFdICk7CgoJCWlmICggbVsyXSApIHsKCQkJZXh0",
				"cmEgPSBtWzNdOwoJCQlicmVhazsKCQl9Cgl9CgoJaWYgKCBwYXJ0cy5sZW5ndGggPiAxICYmIG9yaWdQ",
				"T1MuZXhlYyggc2VsZWN0b3IgKSApIHsKCQlpZiAoIHBhcnRzLmxlbmd0aCA9PT0gMiAmJiBFeHByLnJl",
				"bGF0aXZlWyBwYXJ0c1swXSBdICkgewoJCQlzZXQgPSBwb3NQcm9jZXNzKCBwYXJ0c1swXSArIHBhcnRz",
				"WzFdLCBjb250ZXh0ICk7CgkJfSBlbHNlIHsKCQkJc2V0ID0gRXhwci5yZWxhdGl2ZVsgcGFydHNbMF0g",
				"XSA/CgkJCQlbIGNvbnRleHQgXSA6CgkJCQlTaXp6bGUoIHBhcnRzLnNoaWZ0KCksIGNvbnRleHQgKTsK",
				"CgkJCXdoaWxlICggcGFydHMubGVuZ3RoICkgewoJCQkJc2VsZWN0b3IgPSBwYXJ0cy5zaGlmdCgpOwoK",
				"CQkJCWlmICggRXhwci5yZWxhdGl2ZVsgc2VsZWN0b3IgXSApCgkJCQkJc2VsZWN0b3IgKz0gcGFydHMu",
				"c2hpZnQoKTsKCgkJCQlzZXQgPSBwb3NQcm9jZXNzKCBzZWxlY3Rvciwgc2V0ICk7CgkJCX0KCQl9Cgl9",
				"IGVsc2UgewoJCWlmICggIXNlZWQgJiYgcGFydHMubGVuZ3RoID4gMSAmJiBjb250ZXh0Lm5vZGVUeXBl",
				"ID09PSA5ICYmICFjb250ZXh0WE1MICYmCgkJCQlFeHByLm1hdGNoLklELnRlc3QocGFydHNbMF0pICYm",
				"ICFFeHByLm1hdGNoLklELnRlc3QocGFydHNbcGFydHMubGVuZ3RoIC0gMV0pICkgewoJCQl2YXIgcmV0",
				"ID0gU2l6emxlLmZpbmQoIHBhcnRzLnNoaWZ0KCksIGNvbnRleHQsIGNvbnRleHRYTUwgKTsKCQkJY29u",
				"dGV4dCA9IHJldC5leHByID8gU2l6emxlLmZpbHRlciggcmV0LmV4cHIsIHJldC5zZXQgKVswXSA6IHJl",
				"dC5zZXRbMF07CgkJfQoKCQlpZiAoIGNvbnRleHQgKSB7CgkJCXZhciByZXQgPSBzZWVkID8KCQkJCXsg",
				"ZXhwcjogcGFydHMucG9wKCksIHNldDogbWFrZUFycmF5KHNlZWQpIH0gOgoJCQkJU2l6emxlLmZpbmQo",
				"IHBhcnRzLnBvcCgpLCBwYXJ0cy5sZW5ndGggPT09IDEgJiYgKHBhcnRzWzBdID09PSAifiIgfHwgcGFy",
				"dHNbMF0gPT09ICIrIikgJiYgY29udGV4dC5wYXJlbnROb2RlID8gY29udGV4dC5wYXJlbnROb2RlIDog",
				"Y29udGV4dCwgY29udGV4dFhNTCApOwoJCQlzZXQgPSByZXQuZXhwciA/IFNpenpsZS5maWx0ZXIoIHJl",
				"dC5leHByLCByZXQuc2V0ICkgOiByZXQuc2V0OwoKCQkJaWYgKCBwYXJ0cy5sZW5ndGggPiAwICkgewoJ",
				"CQkJY2hlY2tTZXQgPSBtYWtlQXJyYXkoc2V0KTsKCQkJfSBlbHNlIHsKCQkJCXBydW5lID0gZmFsc2U7",
				"CgkJCX0KCgkJCXdoaWxlICggcGFydHMubGVuZ3RoICkgewoJCQkJdmFyIGN1ciA9IHBhcnRzLnBvcCgp",
				"LCBwb3AgPSBjdXI7CgoJCQkJaWYgKCAhRXhwci5yZWxhdGl2ZVsgY3VyIF0gKSB7CgkJCQkJY3VyID0g",
				"IiI7CgkJCQl9IGVsc2UgewoJCQkJCXBvcCA9IHBhcnRzLnBvcCgpOwoJCQkJfQoKCQkJCWlmICggcG9w",
				"ID09IG51bGwgKSB7CgkJCQkJcG9wID0gY29udGV4dDsKCQkJCX0KCgkJCQlFeHByLnJlbGF0aXZlWyBj",
				"dXIgXSggY2hlY2tTZXQsIHBvcCwgY29udGV4dFhNTCApOwoJCQl9CgkJfSBlbHNlIHsKCQkJY2hlY2tT",
				"ZXQgPSBwYXJ0cyA9IFtdOwoJCX0KCX0KCglpZiAoICFjaGVja1NldCApIHsKCQljaGVja1NldCA9IHNl",
				"dDsKCX0KCglpZiAoICFjaGVja1NldCApIHsKCQl0aHJvdyAiU3ludGF4IGVycm9yLCB1bnJlY29nbml6",
				"ZWQgZXhwcmVzc2lvbjogIiArIChjdXIgfHwgc2VsZWN0b3IpOwoJfQoKCWlmICggdG9TdHJpbmcuY2Fs",
				"bChjaGVja1NldCkgPT09ICJbb2JqZWN0IEFycmF5XSIgKSB7CgkJaWYgKCAhcHJ1bmUgKSB7CgkJCXJl",
				"c3VsdHMucHVzaC5hcHBseSggcmVzdWx0cywgY2hlY2tTZXQgKTsKCQl9IGVsc2UgaWYgKCBjb250ZXh0",
				"ICYmIGNvbnRleHQubm9kZVR5cGUgPT09IDEgKSB7CgkJCWZvciAoIHZhciBpID0gMDsgY2hlY2tTZXRb",
				"aV0gIT0gbnVsbDsgaSsrICkgewoJCQkJaWYgKCBjaGVja1NldFtpXSAmJiAoY2hlY2tTZXRbaV0gPT09",
				"IHRydWUgfHwgY2hlY2tTZXRbaV0ubm9kZVR5cGUgPT09IDEgJiYgY29udGFpbnMoY29udGV4dCwgY2hl",
				"Y2tTZXRbaV0pKSApIHsKCQkJCQlyZXN1bHRzLnB1c2goIHNldFtpXSApOwoJCQkJfQoJCQl9CgkJfSBl",
				"bHNlIHsKCQkJZm9yICggdmFyIGkgPSAwOyBjaGVja1NldFtpXSAhPSBudWxsOyBpKysgKSB7CgkJCQlp",
				"ZiAoIGNoZWNrU2V0W2ldICYmIGNoZWNrU2V0W2ldLm5vZGVUeXBlID09PSAxICkgewoJCQkJCXJlc3Vs",
				"dHMucHVzaCggc2V0W2ldICk7CgkJCQl9CgkJCX0KCQl9Cgl9IGVsc2UgewoJCW1ha2VBcnJheSggY2hl",
				"Y2tTZXQsIHJlc3VsdHMgKTsKCX0KCglpZiAoIGV4dHJhICkgewoJCVNpenpsZSggZXh0cmEsIG9yaWdD",
				"b250ZXh0LCByZXN1bHRzLCBzZWVkICk7CgkJU2l6emxlLnVuaXF1ZVNvcnQoIHJlc3VsdHMgKTsKCX0K",
				"CglyZXR1cm4gcmVzdWx0czsKfTsKClNpenpsZS51bmlxdWVTb3J0ID0gZnVuY3Rpb24ocmVzdWx0cyl7",
				"CglpZiAoIHNvcnRPcmRlciApIHsKCQloYXNEdXBsaWNhdGUgPSBiYXNlSGFzRHVwbGljYXRlOwoJCXJl",
				"c3VsdHMuc29ydChzb3J0T3JkZXIpOwoKCQlpZiAoIGhhc0R1cGxpY2F0ZSApIHsKCQkJZm9yICggdmFy",
				"IGkgPSAxOyBpIDwgcmVzdWx0cy5sZW5ndGg7IGkrKyApIHsKCQkJCWlmICggcmVzdWx0c1tpXSA9PT0g",
				"cmVzdWx0c1tpLTFdICkgewoJCQkJCXJlc3VsdHMuc3BsaWNlKGktLSwgMSk7CgkJCQl9CgkJCX0KCQl9",
				"Cgl9CgoJcmV0dXJuIHJlc3VsdHM7Cn07CgpTaXp6bGUubWF0Y2hlcyA9IGZ1bmN0aW9uKGV4cHIsIHNl",
				"dCl7CglyZXR1cm4gU2l6emxlKGV4cHIsIG51bGwsIG51bGwsIHNldCk7Cn07CgpTaXp6bGUuZmluZCA9",
				"IGZ1bmN0aW9uKGV4cHIsIGNvbnRleHQsIGlzWE1MKXsKCXZhciBzZXQsIG1hdGNoOwoKCWlmICggIWV4",
				"cHIgKSB7CgkJcmV0dXJuIFtdOwoJfQoKCWZvciAoIHZhciBpID0gMCwgbCA9IEV4cHIub3JkZXIubGVu",
				"Z3RoOyBpIDwgbDsgaSsrICkgewoJCXZhciB0eXBlID0gRXhwci5vcmRlcltpXSwgbWF0Y2g7CgoJCWlm",
				"ICggKG1hdGNoID0gRXhwci5sZWZ0TWF0Y2hbIHR5cGUgXS5leGVjKCBleHByICkpICkgewoJCQl2YXIg",
				"bGVmdCA9IG1hdGNoWzFdOwoJCQltYXRjaC5zcGxpY2UoMSwxKTsKCgkJCWlmICggbGVmdC5zdWJzdHIo",
				"IGxlZnQubGVuZ3RoIC0gMSApICE9PSAiXFwiICkgewoJCQkJbWF0Y2hbMV0gPSAobWF0Y2hbMV0gfHwg",
				"IiIpLnJlcGxhY2UoL1xcL2csICIiKTsKCQkJCXNldCA9IEV4cHIuZmluZFsgdHlwZSBdKCBtYXRjaCwg",
				"Y29udGV4dCwgaXNYTUwgKTsKCQkJCWlmICggc2V0ICE9IG51bGwgKSB7CgkJCQkJZXhwciA9IGV4cHIu",
				"cmVwbGFjZSggRXhwci5tYXRjaFsgdHlwZSBdLCAiIiApOwoJCQkJCWJyZWFrOwoJCQkJfQoJCQl9CgkJ",
				"fQoJfQoKCWlmICggIXNldCApIHsKCQlzZXQgPSBjb250ZXh0LmdldEVsZW1lbnRzQnlUYWdOYW1lKCIq",
				"Iik7Cgl9CgoJcmV0dXJuIHtzZXQ6IHNldCwgZXhwcjogZXhwcn07Cn07CgpTaXp6bGUuZmlsdGVyID0g",
				"ZnVuY3Rpb24oZXhwciwgc2V0LCBpbnBsYWNlLCBub3QpewoJdmFyIG9sZCA9IGV4cHIsIHJlc3VsdCA9",
				"IFtdLCBjdXJMb29wID0gc2V0LCBtYXRjaCwgYW55Rm91bmQsCgkJaXNYTUxGaWx0ZXIgPSBzZXQgJiYg",
				"c2V0WzBdICYmIGlzWE1MKHNldFswXSk7CgoJd2hpbGUgKCBleHByICYmIHNldC5sZW5ndGggKSB7CgkJ",
				"Zm9yICggdmFyIHR5cGUgaW4gRXhwci5maWx0ZXIgKSB7CgkJCWlmICggKG1hdGNoID0gRXhwci5tYXRj",
				"aFsgdHlwZSBdLmV4ZWMoIGV4cHIgKSkgIT0gbnVsbCApIHsKCQkJCXZhciBmaWx0ZXIgPSBFeHByLmZp",
				"bHRlclsgdHlwZSBdLCBmb3VuZCwgaXRlbTsKCQkJCWFueUZvdW5kID0gZmFsc2U7CgoJCQkJaWYgKCBj",
				"dXJMb29wID09IHJlc3VsdCApIHsKCQkJCQlyZXN1bHQgPSBbXTsKCQkJCX0KCgkJCQlpZiAoIEV4cHIu",
				"cHJlRmlsdGVyWyB0eXBlIF0gKSB7CgkJCQkJbWF0Y2ggPSBFeHByLnByZUZpbHRlclsgdHlwZSBdKCBt",
				"YXRjaCwgY3VyTG9vcCwgaW5wbGFjZSwgcmVzdWx0LCBub3QsIGlzWE1MRmlsdGVyICk7CgoJCQkJCWlm",
				"ICggIW1hdGNoICkgewoJCQkJCQlhbnlGb3VuZCA9IGZvdW5kID0gdHJ1ZTsKCQkJCQl9IGVsc2UgaWYg",
				"KCBtYXRjaCA9PT0gdHJ1ZSApIHsKCQkJCQkJY29udGludWU7CgkJCQkJfQoJCQkJfQoKCQkJCWlmICgg",
				"bWF0Y2ggKSB7CgkJCQkJZm9yICggdmFyIGkgPSAwOyAoaXRlbSA9IGN1ckxvb3BbaV0pICE9IG51bGw7",
				"IGkrKyApIHsKCQkJCQkJaWYgKCBpdGVtICkgewoJCQkJCQkJZm91bmQgPSBmaWx0ZXIoIGl0ZW0sIG1h",
				"dGNoLCBpLCBjdXJMb29wICk7CgkJCQkJCQl2YXIgcGFzcyA9IG5vdCBeICEhZm91bmQ7CgoJCQkJCQkJ",
				"aWYgKCBpbnBsYWNlICYmIGZvdW5kICE9IG51bGwgKSB7CgkJCQkJCQkJaWYgKCBwYXNzICkgewoJCQkJ",
				"CQkJCQlhbnlGb3VuZCA9IHRydWU7CgkJCQkJCQkJfSBlbHNlIHsKCQkJCQkJCQkJY3VyTG9vcFtpXSA9",
				"IGZhbHNlOwoJCQkJCQkJCX0KCQkJCQkJCX0gZWxzZSBpZiAoIHBhc3MgKSB7CgkJCQkJCQkJcmVzdWx0",
				"LnB1c2goIGl0ZW0gKTsKCQkJCQkJCQlhbnlGb3VuZCA9IHRydWU7CgkJCQkJCQl9CgkJCQkJCX0KCQkJ",
				"CQl9CgkJCQl9CgoJCQkJaWYgKCBmb3VuZCAhPT0gdW5kZWZpbmVkICkgewoJCQkJCWlmICggIWlucGxh",
				"Y2UgKSB7CgkJCQkJCWN1ckxvb3AgPSByZXN1bHQ7CgkJCQkJfQoKCQkJCQlleHByID0gZXhwci5yZXBs",
				"YWNlKCBFeHByLm1hdGNoWyB0eXBlIF0sICIiICk7CgoJCQkJCWlmICggIWFueUZvdW5kICkgewoJCQkJ",
				"CQlyZXR1cm4gW107CgkJCQkJfQoKCQkJCQlicmVhazsKCQkJCX0KCQkJfQoJCX0KCgkJaWYgKCBleHBy",
				"ID09IG9sZCApIHsKCQkJaWYgKCBhbnlGb3VuZCA9PSBudWxsICkgewoJCQkJdGhyb3cgIlN5bnRheCBl",
				"cnJvciwgdW5yZWNvZ25pemVkIGV4cHJlc3Npb246ICIgKyBleHByOwoJCQl9IGVsc2UgewoJCQkJYnJl",
				"YWs7CgkJCX0KCQl9CgoJCW9sZCA9IGV4cHI7Cgl9CgoJcmV0dXJuIGN1ckxvb3A7Cn07Cgp2YXIgRXhw",
				"ciA9IFNpenpsZS5zZWxlY3RvcnMgPSB7CglvcmRlcjogWyAiSUQiLCAiTkFNRSIsICJUQUciIF0sCglt",
				"YXRjaDogewoJCUlEOiAvIygoPzpbXHdcdTAwYzAtXHVGRkZGLV18XFwuKSspLywKCQlDTEFTUzogL1wu",
				"KCg/Oltcd1x1MDBjMC1cdUZGRkYtXXxcXC4pKykvLAoJCU5BTUU6IC9cW25hbWU9WyciXSooKD86W1x3",
				"XHUwMGMwLVx1RkZGRi1dfFxcLikrKVsnIl0qXF0vLAoJCUFUVFI6IC9cW1xzKigoPzpbXHdcdTAwYzAt",
				"XHVGRkZGLV18XFwuKSspXHMqKD86KFxTPz0pXHMqKFsnIl0qKSguKj8pXDN8KVxzKlxdLywKCQlUQUc6",
				"IC9eKCg/Oltcd1x1MDBjMC1cdUZGRkZcKi1dfFxcLikrKS8sCgkJQ0hJTEQ6IC86KG9ubHl8bnRofGxh",
				"c3R8Zmlyc3QpLWNoaWxkKD86XCgoZXZlbnxvZGR8W1xkbistXSopXCkpPy8sCgkJUE9TOiAvOihudGh8",
				"ZXF8Z3R8bHR8Zmlyc3R8bGFzdHxldmVufG9kZCkoPzpcKChcZCopXCkpPyg/PVteLV18JCkvLAoJCVBT",
				"RVVETzogLzooKD86W1x3XHUwMGMwLVx1RkZGRi1dfFxcLikrKSg/OlwoKFsnIl0qKSgoPzpcKFteXCld",
				"K1wpfFteXDJcKFwpXSopKylcMlwpKT8vCgl9LAoJbGVmdE1hdGNoOiB7fSwKCWF0dHJNYXA6IHsKCQki",
				"Y2xhc3MiOiAiY2xhc3NOYW1lIiwKCQkiZm9yIjogImh0bWxGb3IiCgl9LAoJYXR0ckhhbmRsZTogewoJ",
				"CWhyZWY6IGZ1bmN0aW9uKGVsZW0pewoJCQlyZXR1cm4gZWxlbS5nZXRBdHRyaWJ1dGUoImhyZWYiKTsK",
				"CQl9Cgl9LAoJcmVsYXRpdmU6IHsKCQkiKyI6IGZ1bmN0aW9uKGNoZWNrU2V0LCBwYXJ0LCBpc1hNTCl7",
				"CgkJCXZhciBpc1BhcnRTdHIgPSB0eXBlb2YgcGFydCA9PT0gInN0cmluZyIsCgkJCQlpc1RhZyA9IGlz",
				"UGFydFN0ciAmJiAhL1xXLy50ZXN0KHBhcnQpLAoJCQkJaXNQYXJ0U3RyTm90VGFnID0gaXNQYXJ0U3Ry",
				"ICYmICFpc1RhZzsKCgkJCWlmICggaXNUYWcgJiYgIWlzWE1MICkgewoJCQkJcGFydCA9IHBhcnQudG9V",
				"cHBlckNhc2UoKTsKCQkJfQoKCQkJZm9yICggdmFyIGkgPSAwLCBsID0gY2hlY2tTZXQubGVuZ3RoLCBl",
				"bGVtOyBpIDwgbDsgaSsrICkgewoJCQkJaWYgKCAoZWxlbSA9IGNoZWNrU2V0W2ldKSApIHsKCQkJCQl3",
				"aGlsZSAoIChlbGVtID0gZWxlbS5wcmV2aW91c1NpYmxpbmcpICYmIGVsZW0ubm9kZVR5cGUgIT09IDEg",
				"KSB7fQoKCQkJCQljaGVja1NldFtpXSA9IGlzUGFydFN0ck5vdFRhZyB8fCBlbGVtICYmIGVsZW0ubm9k",
				"ZU5hbWUgPT09IHBhcnQgPwoJCQkJCQllbGVtIHx8IGZhbHNlIDoKCQkJCQkJZWxlbSA9PT0gcGFydDsK",
				"CQkJCX0KCQkJfQoKCQkJaWYgKCBpc1BhcnRTdHJOb3RUYWcgKSB7CgkJCQlTaXp6bGUuZmlsdGVyKCBw",
				"YXJ0LCBjaGVja1NldCwgdHJ1ZSApOwoJCQl9CgkJfSwKCQkiPiI6IGZ1bmN0aW9uKGNoZWNrU2V0LCBw",
				"YXJ0LCBpc1hNTCl7CgkJCXZhciBpc1BhcnRTdHIgPSB0eXBlb2YgcGFydCA9PT0gInN0cmluZyI7CgoJ",
				"CQlpZiAoIGlzUGFydFN0ciAmJiAhL1xXLy50ZXN0KHBhcnQpICkgewoJCQkJcGFydCA9IGlzWE1MID8g",
				"cGFydCA6IHBhcnQudG9VcHBlckNhc2UoKTsKCgkJCQlmb3IgKCB2YXIgaSA9IDAsIGwgPSBjaGVja1Nl",
				"dC5sZW5ndGg7IGkgPCBsOyBpKysgKSB7CgkJCQkJdmFyIGVsZW0gPSBjaGVja1NldFtpXTsKCQkJCQlp",
				"ZiAoIGVsZW0gKSB7CgkJCQkJCXZhciBwYXJlbnQgPSBlbGVtLnBhcmVudE5vZGU7CgkJCQkJCWNoZWNr",
				"U2V0W2ldID0gcGFyZW50Lm5vZGVOYW1lID09PSBwYXJ0ID8gcGFyZW50IDogZmFsc2U7CgkJCQkJfQoJ",
				"CQkJfQoJCQl9IGVsc2UgewoJCQkJZm9yICggdmFyIGkgPSAwLCBsID0gY2hlY2tTZXQubGVuZ3RoOyBp",
				"IDwgbDsgaSsrICkgewoJCQkJCXZhciBlbGVtID0gY2hlY2tTZXRbaV07CgkJCQkJaWYgKCBlbGVtICkg",
				"ewoJCQkJCQljaGVja1NldFtpXSA9IGlzUGFydFN0ciA/CgkJCQkJCQllbGVtLnBhcmVudE5vZGUgOgoJ",
				"CQkJCQkJZWxlbS5wYXJlbnROb2RlID09PSBwYXJ0OwoJCQkJCX0KCQkJCX0KCgkJCQlpZiAoIGlzUGFy",
				"dFN0ciApIHsKCQkJCQlTaXp6bGUuZmlsdGVyKCBwYXJ0LCBjaGVja1NldCwgdHJ1ZSApOwoJCQkJfQoJ",
				"CQl9CgkJfSwKCQkiIjogZnVuY3Rpb24oY2hlY2tTZXQsIHBhcnQsIGlzWE1MKXsKCQkJdmFyIGRvbmVO",
				"YW1lID0gZG9uZSsrLCBjaGVja0ZuID0gZGlyQ2hlY2s7CgoJCQlpZiAoICEvXFcvLnRlc3QocGFydCkg",
				"KSB7CgkJCQl2YXIgbm9kZUNoZWNrID0gcGFydCA9IGlzWE1MID8gcGFydCA6IHBhcnQudG9VcHBlckNh",
				"c2UoKTsKCQkJCWNoZWNrRm4gPSBkaXJOb2RlQ2hlY2s7CgkJCX0KCgkJCWNoZWNrRm4oInBhcmVudE5v",
				"ZGUiLCBwYXJ0LCBkb25lTmFtZSwgY2hlY2tTZXQsIG5vZGVDaGVjaywgaXNYTUwpOwoJCX0sCgkJIn4i",
				"OiBmdW5jdGlvbihjaGVja1NldCwgcGFydCwgaXNYTUwpewoJCQl2YXIgZG9uZU5hbWUgPSBkb25lKyss",
				"IGNoZWNrRm4gPSBkaXJDaGVjazsKCgkJCWlmICggdHlwZW9mIHBhcnQgPT09ICJzdHJpbmciICYmICEv",
				"XFcvLnRlc3QocGFydCkgKSB7CgkJCQl2YXIgbm9kZUNoZWNrID0gcGFydCA9IGlzWE1MID8gcGFydCA6",
				"IHBhcnQudG9VcHBlckNhc2UoKTsKCQkJCWNoZWNrRm4gPSBkaXJOb2RlQ2hlY2s7CgkJCX0KCgkJCWNo",
				"ZWNrRm4oInByZXZpb3VzU2libGluZyIsIHBhcnQsIGRvbmVOYW1lLCBjaGVja1NldCwgbm9kZUNoZWNr",
				"LCBpc1hNTCk7CgkJfQoJfSwKCWZpbmQ6IHsKCQlJRDogZnVuY3Rpb24obWF0Y2gsIGNvbnRleHQsIGlz",
				"WE1MKXsKCQkJaWYgKCB0eXBlb2YgY29udGV4dC5nZXRFbGVtZW50QnlJZCAhPT0gInVuZGVmaW5lZCIg",
				"JiYgIWlzWE1MICkgewoJCQkJdmFyIG0gPSBjb250ZXh0LmdldEVsZW1lbnRCeUlkKG1hdGNoWzFdKTsK",
				"CQkJCXJldHVybiBtID8gW21dIDogW107CgkJCX0KCQl9LAoJCU5BTUU6IGZ1bmN0aW9uKG1hdGNoLCBj",
				"b250ZXh0LCBpc1hNTCl7CgkJCWlmICggdHlwZW9mIGNvbnRleHQuZ2V0RWxlbWVudHNCeU5hbWUgIT09",
				"ICJ1bmRlZmluZWQiICkgewoJCQkJdmFyIHJldCA9IFtdLCByZXN1bHRzID0gY29udGV4dC5nZXRFbGVt",
				"ZW50c0J5TmFtZShtYXRjaFsxXSk7CgoJCQkJZm9yICggdmFyIGkgPSAwLCBsID0gcmVzdWx0cy5sZW5n",
				"dGg7IGkgPCBsOyBpKysgKSB7CgkJCQkJaWYgKCByZXN1bHRzW2ldLmdldEF0dHJpYnV0ZSgibmFtZSIp",
				"ID09PSBtYXRjaFsxXSApIHsKCQkJCQkJcmV0LnB1c2goIHJlc3VsdHNbaV0gKTsKCQkJCQl9CgkJCQl9",
				"CgoJCQkJcmV0dXJuIHJldC5sZW5ndGggPT09IDAgPyBudWxsIDogcmV0OwoJCQl9CgkJfSwKCQlUQUc6",
				"IGZ1bmN0aW9uKG1hdGNoLCBjb250ZXh0KXsKCQkJcmV0dXJuIGNvbnRleHQuZ2V0RWxlbWVudHNCeVRh",
				"Z05hbWUobWF0Y2hbMV0pOwoJCX0KCX0sCglwcmVGaWx0ZXI6IHsKCQlDTEFTUzogZnVuY3Rpb24obWF0",
				"Y2gsIGN1ckxvb3AsIGlucGxhY2UsIHJlc3VsdCwgbm90LCBpc1hNTCl7CgkJCW1hdGNoID0gIiAiICsg",
				"bWF0Y2hbMV0ucmVwbGFjZSgvXFwvZywgIiIpICsgIiAiOwoKCQkJaWYgKCBpc1hNTCApIHsKCQkJCXJl",
				"dHVybiBtYXRjaDsKCQkJfQoKCQkJZm9yICggdmFyIGkgPSAwLCBlbGVtOyAoZWxlbSA9IGN1ckxvb3Bb",
				"aV0pICE9IG51bGw7IGkrKyApIHsKCQkJCWlmICggZWxlbSApIHsKCQkJCQlpZiAoIG5vdCBeIChlbGVt",
				"LmNsYXNzTmFtZSAmJiAoIiAiICsgZWxlbS5jbGFzc05hbWUgKyAiICIpLmluZGV4T2YobWF0Y2gpID49",
				"IDApICkgewoJCQkJCQlpZiAoICFpbnBsYWNlICkKCQkJCQkJCXJlc3VsdC5wdXNoKCBlbGVtICk7CgkJ",
				"CQkJfSBlbHNlIGlmICggaW5wbGFjZSApIHsKCQkJCQkJY3VyTG9vcFtpXSA9IGZhbHNlOwoJCQkJCX0K",
				"CQkJCX0KCQkJfQoKCQkJcmV0dXJuIGZhbHNlOwoJCX0sCgkJSUQ6IGZ1bmN0aW9uKG1hdGNoKXsKCQkJ",
				"cmV0dXJuIG1hdGNoWzFdLnJlcGxhY2UoL1xcL2csICIiKTsKCQl9LAoJCVRBRzogZnVuY3Rpb24obWF0",
				"Y2gsIGN1ckxvb3ApewoJCQlmb3IgKCB2YXIgaSA9IDA7IGN1ckxvb3BbaV0gPT09IGZhbHNlOyBpKysg",
				"KXt9CgkJCXJldHVybiBjdXJMb29wW2ldICYmIGlzWE1MKGN1ckxvb3BbaV0pID8gbWF0Y2hbMV0gOiBt",
				"YXRjaFsxXS50b1VwcGVyQ2FzZSgpOwoJCX0sCgkJQ0hJTEQ6IGZ1bmN0aW9uKG1hdGNoKXsKCQkJaWYg",
				"KCBtYXRjaFsxXSA9PSAibnRoIiApIHsKCQkJCXZhciB0ZXN0ID0gLygtPykoXGQqKW4oKD86XCt8LSk/",
				"XGQqKS8uZXhlYygKCQkJCQltYXRjaFsyXSA9PSAiZXZlbiIgJiYgIjJuIiB8fCBtYXRjaFsyXSA9PSAi",
				"b2RkIiAmJiAiMm4rMSIgfHwKCQkJCQkhL1xELy50ZXN0KCBtYXRjaFsyXSApICYmICIwbisiICsgbWF0",
				"Y2hbMl0gfHwgbWF0Y2hbMl0pOwoKCQkJCW1hdGNoWzJdID0gKHRlc3RbMV0gKyAodGVzdFsyXSB8fCAx",
				"KSkgLSAwOwoJCQkJbWF0Y2hbM10gPSB0ZXN0WzNdIC0gMDsKCQkJfQoKCQkJbWF0Y2hbMF0gPSBkb25l",
				"Kys7CgoJCQlyZXR1cm4gbWF0Y2g7CgkJfSwKCQlBVFRSOiBmdW5jdGlvbihtYXRjaCwgY3VyTG9vcCwg",
				"aW5wbGFjZSwgcmVzdWx0LCBub3QsIGlzWE1MKXsKCQkJdmFyIG5hbWUgPSBtYXRjaFsxXS5yZXBsYWNl",
				"KC9cXC9nLCAiIik7CgoJCQlpZiAoICFpc1hNTCAmJiBFeHByLmF0dHJNYXBbbmFtZV0gKSB7CgkJCQlt",
				"YXRjaFsxXSA9IEV4cHIuYXR0ck1hcFtuYW1lXTsKCQkJfQoKCQkJaWYgKCBtYXRjaFsyXSA9PT0gIn49",
				"IiApIHsKCQkJCW1hdGNoWzRdID0gIiAiICsgbWF0Y2hbNF0gKyAiICI7CgkJCX0KCgkJCXJldHVybiBt",
				"YXRjaDsKCQl9LAoJCVBTRVVETzogZnVuY3Rpb24obWF0Y2gsIGN1ckxvb3AsIGlucGxhY2UsIHJlc3Vs",
				"dCwgbm90KXsKCQkJaWYgKCBtYXRjaFsxXSA9PT0gIm5vdCIgKSB7CgkJCQlpZiAoICggY2h1bmtlci5l",
				"eGVjKG1hdGNoWzNdKSB8fCAiIiApLmxlbmd0aCA+IDEgfHwgL15cdy8udGVzdChtYXRjaFszXSkgKSB7",
				"CgkJCQkJbWF0Y2hbM10gPSBTaXp6bGUobWF0Y2hbM10sIG51bGwsIG51bGwsIGN1ckxvb3ApOwoJCQkJ",
				"fSBlbHNlIHsKCQkJCQl2YXIgcmV0ID0gU2l6emxlLmZpbHRlcihtYXRjaFszXSwgY3VyTG9vcCwgaW5w",
				"bGFjZSwgdHJ1ZSBeIG5vdCk7CgkJCQkJaWYgKCAhaW5wbGFjZSApIHsKCQkJCQkJcmVzdWx0LnB1c2gu",
				"YXBwbHkoIHJlc3VsdCwgcmV0ICk7CgkJCQkJfQoJCQkJCXJldHVybiBmYWxzZTsKCQkJCX0KCQkJfSBl",
				"bHNlIGlmICggRXhwci5tYXRjaC5QT1MudGVzdCggbWF0Y2hbMF0gKSB8fCBFeHByLm1hdGNoLkNISUxE",
				"LnRlc3QoIG1hdGNoWzBdICkgKSB7CgkJCQlyZXR1cm4gdHJ1ZTsKCQkJfQoKCQkJcmV0dXJuIG1hdGNo",
				"OwoJCX0sCgkJUE9TOiBmdW5jdGlvbihtYXRjaCl7CgkJCW1hdGNoLnVuc2hpZnQoIHRydWUgKTsKCQkJ",
				"cmV0dXJuIG1hdGNoOwoJCX0KCX0sCglmaWx0ZXJzOiB7CgkJZW5hYmxlZDogZnVuY3Rpb24oZWxlbSl7",
				"CgkJCXJldHVybiBlbGVtLmRpc2FibGVkID09PSBmYWxzZSAmJiBlbGVtLnR5cGUgIT09ICJoaWRkZW4i",
				"OwoJCX0sCgkJZGlzYWJsZWQ6IGZ1bmN0aW9uKGVsZW0pewoJCQlyZXR1cm4gZWxlbS5kaXNhYmxlZCA9",
				"PT0gdHJ1ZTsKCQl9LAoJCWNoZWNrZWQ6IGZ1bmN0aW9uKGVsZW0pewoJCQlyZXR1cm4gZWxlbS5jaGVj",
				"a2VkID09PSB0cnVlOwoJCX0sCgkJc2VsZWN0ZWQ6IGZ1bmN0aW9uKGVsZW0pewoJCQllbGVtLnBhcmVu",
				"dE5vZGUuc2VsZWN0ZWRJbmRleDsKCQkJcmV0dXJuIGVsZW0uc2VsZWN0ZWQgPT09IHRydWU7CgkJfSwK",
				"CQlwYXJlbnQ6IGZ1bmN0aW9uKGVsZW0pewoJCQlyZXR1cm4gISFlbGVtLmZpcnN0Q2hpbGQ7CgkJfSwK",
				"CQllbXB0eTogZnVuY3Rpb24oZWxlbSl7CgkJCXJldHVybiAhZWxlbS5maXJzdENoaWxkOwoJCX0sCgkJ",
				"aGFzOiBmdW5jdGlvbihlbGVtLCBpLCBtYXRjaCl7CgkJCXJldHVybiAhIVNpenpsZSggbWF0Y2hbM10s",
				"IGVsZW0gKS5sZW5ndGg7CgkJfSwKCQloZWFkZXI6IGZ1bmN0aW9uKGVsZW0pewoJCQlyZXR1cm4gL2hc",
				"ZC9pLnRlc3QoIGVsZW0ubm9kZU5hbWUgKTsKCQl9LAoJCXRleHQ6IGZ1bmN0aW9uKGVsZW0pewoJCQly",
				"ZXR1cm4gInRleHQiID09PSBlbGVtLnR5cGU7CgkJfSwKCQlyYWRpbzogZnVuY3Rpb24oZWxlbSl7CgkJ",
				"CXJldHVybiAicmFkaW8iID09PSBlbGVtLnR5cGU7CgkJfSwKCQljaGVja2JveDogZnVuY3Rpb24oZWxl",
				"bSl7CgkJCXJldHVybiAiY2hlY2tib3giID09PSBlbGVtLnR5cGU7CgkJfSwKCQlmaWxlOiBmdW5jdGlv",
				"bihlbGVtKXsKCQkJcmV0dXJuICJmaWxlIiA9PT0gZWxlbS50eXBlOwoJCX0sCgkJcGFzc3dvcmQ6IGZ1",
				"bmN0aW9uKGVsZW0pewoJCQlyZXR1cm4gInBhc3N3b3JkIiA9PT0gZWxlbS50eXBlOwoJCX0sCgkJc3Vi",
				"bWl0OiBmdW5jdGlvbihlbGVtKXsKCQkJcmV0dXJuICJzdWJtaXQiID09PSBlbGVtLnR5cGU7CgkJfSwK",
				"CQlpbWFnZTogZnVuY3Rpb24oZWxlbSl7CgkJCXJldHVybiAiaW1hZ2UiID09PSBlbGVtLnR5cGU7CgkJ",
				"fSwKCQlyZXNldDogZnVuY3Rpb24oZWxlbSl7CgkJCXJldHVybiAicmVzZXQiID09PSBlbGVtLnR5cGU7",
				"CgkJfSwKCQlidXR0b246IGZ1bmN0aW9uKGVsZW0pewoJCQlyZXR1cm4gImJ1dHRvbiIgPT09IGVsZW0u",
				"dHlwZSB8fCBlbGVtLm5vZGVOYW1lLnRvVXBwZXJDYXNlKCkgPT09ICJCVVRUT04iOwoJCX0sCgkJaW5w",
				"dXQ6IGZ1bmN0aW9uKGVsZW0pewoJCQlyZXR1cm4gL2lucHV0fHNlbGVjdHx0ZXh0YXJlYXxidXR0b24v",
				"aS50ZXN0KGVsZW0ubm9kZU5hbWUpOwoJCX0KCX0sCglzZXRGaWx0ZXJzOiB7CgkJZmlyc3Q6IGZ1bmN0",
				"aW9uKGVsZW0sIGkpewoJCQlyZXR1cm4gaSA9PT0gMDsKCQl9LAoJCWxhc3Q6IGZ1bmN0aW9uKGVsZW0s",
				"IGksIG1hdGNoLCBhcnJheSl7CgkJCXJldHVybiBpID09PSBhcnJheS5sZW5ndGggLSAxOwoJCX0sCgkJ",
				"ZXZlbjogZnVuY3Rpb24oZWxlbSwgaSl7CgkJCXJldHVybiBpICUgMiA9PT0gMDsKCQl9LAoJCW9kZDog",
				"ZnVuY3Rpb24oZWxlbSwgaSl7CgkJCXJldHVybiBpICUgMiA9PT0gMTsKCQl9LAoJCWx0OiBmdW5jdGlv",
				"bihlbGVtLCBpLCBtYXRjaCl7CgkJCXJldHVybiBpIDwgbWF0Y2hbM10gLSAwOwoJCX0sCgkJZ3Q6IGZ1",
				"bmN0aW9uKGVsZW0sIGksIG1hdGNoKXsKCQkJcmV0dXJuIGkgPiBtYXRjaFszXSAtIDA7CgkJfSwKCQlu",
				"dGg6IGZ1bmN0aW9uKGVsZW0sIGksIG1hdGNoKXsKCQkJcmV0dXJuIG1hdGNoWzNdIC0gMCA9PSBpOwoJ",
				"CX0sCgkJZXE6IGZ1bmN0aW9uKGVsZW0sIGksIG1hdGNoKXsKCQkJcmV0dXJuIG1hdGNoWzNdIC0gMCA9",
				"PSBpOwoJCX0KCX0sCglmaWx0ZXI6IHsKCQlQU0VVRE86IGZ1bmN0aW9uKGVsZW0sIG1hdGNoLCBpLCBh",
				"cnJheSl7CgkJCXZhciBuYW1lID0gbWF0Y2hbMV0sIGZpbHRlciA9IEV4cHIuZmlsdGVyc1sgbmFtZSBd",
				"OwoKCQkJaWYgKCBmaWx0ZXIgKSB7CgkJCQlyZXR1cm4gZmlsdGVyKCBlbGVtLCBpLCBtYXRjaCwgYXJy",
				"YXkgKTsKCQkJfSBlbHNlIGlmICggbmFtZSA9PT0gImNvbnRhaW5zIiApIHsKCQkJCXJldHVybiAoZWxl",
				"bS50ZXh0Q29udGVudCB8fCBlbGVtLmlubmVyVGV4dCB8fCAiIikuaW5kZXhPZihtYXRjaFszXSkgPj0g",
				"MDsKCQkJfSBlbHNlIGlmICggbmFtZSA9PT0gIm5vdCIgKSB7CgkJCQl2YXIgbm90ID0gbWF0Y2hbM107",
				"CgoJCQkJZm9yICggdmFyIGkgPSAwLCBsID0gbm90Lmxlbmd0aDsgaSA8IGw7IGkrKyApIHsKCQkJCQlp",
				"ZiAoIG5vdFtpXSA9PT0gZWxlbSApIHsKCQkJCQkJcmV0dXJuIGZhbHNlOwoJCQkJCX0KCQkJCX0KCgkJ",
				"CQlyZXR1cm4gdHJ1ZTsKCQkJfQoJCX0sCgkJQ0hJTEQ6IGZ1bmN0aW9uKGVsZW0sIG1hdGNoKXsKCQkJ",
				"dmFyIHR5cGUgPSBtYXRjaFsxXSwgbm9kZSA9IGVsZW07CgkJCXN3aXRjaCAodHlwZSkgewoJCQkJY2Fz",
				"ZSAnb25seSc6CgkJCQljYXNlICdmaXJzdCc6CgkJCQkJd2hpbGUgKCAobm9kZSA9IG5vZGUucHJldmlv",
				"dXNTaWJsaW5nKSApICB7CgkJCQkJCWlmICggbm9kZS5ub2RlVHlwZSA9PT0gMSApIHJldHVybiBmYWxz",
				"ZTsKCQkJCQl9CgkJCQkJaWYgKCB0eXBlID09ICdmaXJzdCcpIHJldHVybiB0cnVlOwoJCQkJCW5vZGUg",
				"PSBlbGVtOwoJCQkJY2FzZSAnbGFzdCc6CgkJCQkJd2hpbGUgKCAobm9kZSA9IG5vZGUubmV4dFNpYmxp",
				"bmcpICkgIHsKCQkJCQkJaWYgKCBub2RlLm5vZGVUeXBlID09PSAxICkgcmV0dXJuIGZhbHNlOwoJCQkJ",
				"CX0KCQkJCQlyZXR1cm4gdHJ1ZTsKCQkJCWNhc2UgJ250aCc6CgkJCQkJdmFyIGZpcnN0ID0gbWF0Y2hb",
				"Ml0sIGxhc3QgPSBtYXRjaFszXTsKCgkJCQkJaWYgKCBmaXJzdCA9PSAxICYmIGxhc3QgPT0gMCApIHsK",
				"CQkJCQkJcmV0dXJuIHRydWU7CgkJCQkJfQoKCQkJCQl2YXIgZG9uZU5hbWUgPSBtYXRjaFswXSwKCQkJ",
				"CQkJcGFyZW50ID0gZWxlbS5wYXJlbnROb2RlOwoKCQkJCQlpZiAoIHBhcmVudCAmJiAocGFyZW50LnNp",
				"emNhY2hlICE9PSBkb25lTmFtZSB8fCAhZWxlbS5ub2RlSW5kZXgpICkgewoJCQkJCQl2YXIgY291bnQg",
				"PSAwOwoJCQkJCQlmb3IgKCBub2RlID0gcGFyZW50LmZpcnN0Q2hpbGQ7IG5vZGU7IG5vZGUgPSBub2Rl",
				"Lm5leHRTaWJsaW5nICkgewoJCQkJCQkJaWYgKCBub2RlLm5vZGVUeXBlID09PSAxICkgewoJCQkJCQkJ",
				"CW5vZGUubm9kZUluZGV4ID0gKytjb3VudDsKCQkJCQkJCX0KCQkJCQkJfQoJCQkJCQlwYXJlbnQuc2l6",
				"Y2FjaGUgPSBkb25lTmFtZTsKCQkJCQl9CgoJCQkJCXZhciBkaWZmID0gZWxlbS5ub2RlSW5kZXggLSBs",
				"YXN0OwoJCQkJCWlmICggZmlyc3QgPT0gMCApIHsKCQkJCQkJcmV0dXJuIGRpZmYgPT0gMDsKCQkJCQl9",
				"IGVsc2UgewoJCQkJCQlyZXR1cm4gKCBkaWZmICUgZmlyc3QgPT0gMCAmJiBkaWZmIC8gZmlyc3QgPj0g",
				"MCApOwoJCQkJCX0KCQkJfQoJCX0sCgkJSUQ6IGZ1bmN0aW9uKGVsZW0sIG1hdGNoKXsKCQkJcmV0dXJu",
				"IGVsZW0ubm9kZVR5cGUgPT09IDEgJiYgZWxlbS5nZXRBdHRyaWJ1dGUoImlkIikgPT09IG1hdGNoOwoJ",
				"CX0sCgkJVEFHOiBmdW5jdGlvbihlbGVtLCBtYXRjaCl7CgkJCXJldHVybiAobWF0Y2ggPT09ICIqIiAm",
				"JiBlbGVtLm5vZGVUeXBlID09PSAxKSB8fCBlbGVtLm5vZGVOYW1lID09PSBtYXRjaDsKCQl9LAoJCUNM",
				"QVNTOiBmdW5jdGlvbihlbGVtLCBtYXRjaCl7CgkJCXJldHVybiAoIiAiICsgKGVsZW0uY2xhc3NOYW1l",
				"IHx8IGVsZW0uZ2V0QXR0cmlidXRlKCJjbGFzcyIpKSArICIgIikKCQkJCS5pbmRleE9mKCBtYXRjaCAp",
				"ID4gLTE7CgkJfSwKCQlBVFRSOiBmdW5jdGlvbihlbGVtLCBtYXRjaCl7CgkJCXZhciBuYW1lID0gbWF0",
				"Y2hbMV0sCgkJCQlyZXN1bHQgPSBFeHByLmF0dHJIYW5kbGVbIG5hbWUgXSA/CgkJCQkJRXhwci5hdHRy",
				"SGFuZGxlWyBuYW1lIF0oIGVsZW0gKSA6CgkJCQkJZWxlbVsgbmFtZSBdICE9IG51bGwgPwoJCQkJCQll",
				"bGVtWyBuYW1lIF0gOgoJCQkJCQllbGVtLmdldEF0dHJpYnV0ZSggbmFtZSApLAoJCQkJdmFsdWUgPSBy",
				"ZXN1bHQgKyAiIiwKCQkJCXR5cGUgPSBtYXRjaFsyXSwKCQkJCWNoZWNrID0gbWF0Y2hbNF07CgoJCQly",
				"ZXR1cm4gcmVzdWx0ID09IG51bGwgPwoJCQkJdHlwZSA9PT0gIiE9IiA6CgkJCQl0eXBlID09PSAiPSIg",
				"PwoJCQkJdmFsdWUgPT09IGNoZWNrIDoKCQkJCXR5cGUgPT09ICIqPSIgPwoJCQkJdmFsdWUuaW5kZXhP",
				"ZihjaGVjaykgPj0gMCA6CgkJCQl0eXBlID09PSAifj0iID8KCQkJCSgiICIgKyB2YWx1ZSArICIgIiku",
				"aW5kZXhPZihjaGVjaykgPj0gMCA6CgkJCQkhY2hlY2sgPwoJCQkJdmFsdWUgJiYgcmVzdWx0ICE9PSBm",
				"YWxzZSA6CgkJCQl0eXBlID09PSAiIT0iID8KCQkJCXZhbHVlICE9IGNoZWNrIDoKCQkJCXR5cGUgPT09",
				"ICJePSIgPwoJCQkJdmFsdWUuaW5kZXhPZihjaGVjaykgPT09IDAgOgoJCQkJdHlwZSA9PT0gIiQ9IiA/",
				"CgkJCQl2YWx1ZS5zdWJzdHIodmFsdWUubGVuZ3RoIC0gY2hlY2subGVuZ3RoKSA9PT0gY2hlY2sgOgoJ",
				"CQkJdHlwZSA9PT0gInw9IiA/CgkJCQl2YWx1ZSA9PT0gY2hlY2sgfHwgdmFsdWUuc3Vic3RyKDAsIGNo",
				"ZWNrLmxlbmd0aCArIDEpID09PSBjaGVjayArICItIiA6CgkJCQlmYWxzZTsKCQl9LAoJCVBPUzogZnVu",
				"Y3Rpb24oZWxlbSwgbWF0Y2gsIGksIGFycmF5KXsKCQkJdmFyIG5hbWUgPSBtYXRjaFsyXSwgZmlsdGVy",
				"ID0gRXhwci5zZXRGaWx0ZXJzWyBuYW1lIF07CgoJCQlpZiAoIGZpbHRlciApIHsKCQkJCXJldHVybiBm",
				"aWx0ZXIoIGVsZW0sIGksIG1hdGNoLCBhcnJheSApOwoJCQl9CgkJfQoJfQp9OwoKdmFyIG9yaWdQT1Mg",
				"PSBFeHByLm1hdGNoLlBPUzsKCmZvciAoIHZhciB0eXBlIGluIEV4cHIubWF0Y2ggKSB7CglFeHByLm1h",
				"dGNoWyB0eXBlIF0gPSBuZXcgUmVnRXhwKCBFeHByLm1hdGNoWyB0eXBlIF0uc291cmNlICsgLyg/IVte",
				"XFtdKlxdKSg/IVteXChdKlwpKS8uc291cmNlICk7CglFeHByLmxlZnRNYXRjaFsgdHlwZSBdID0gbmV3",
				"IFJlZ0V4cCggLyheKD86LnxccnxcbikqPykvLnNvdXJjZSArIEV4cHIubWF0Y2hbIHR5cGUgXS5zb3Vy",
				"Y2UgKTsKfQoKdmFyIG1ha2VBcnJheSA9IGZ1bmN0aW9uKGFycmF5LCByZXN1bHRzKSB7CglhcnJheSA9",
				"IEFycmF5LnByb3RvdHlwZS5zbGljZS5jYWxsKCBhcnJheSwgMCApOwoKCWlmICggcmVzdWx0cyApIHsK",
				"CQlyZXN1bHRzLnB1c2guYXBwbHkoIHJlc3VsdHMsIGFycmF5ICk7CgkJcmV0dXJuIHJlc3VsdHM7Cgl9",
				"CgoJcmV0dXJuIGFycmF5Owp9OwoKdHJ5IHsKCUFycmF5LnByb3RvdHlwZS5zbGljZS5jYWxsKCBkb2N1",
				"bWVudC5kb2N1bWVudEVsZW1lbnQuY2hpbGROb2RlcywgMCApOwoKfSBjYXRjaChlKXsKCW1ha2VBcnJh",
				"eSA9IGZ1bmN0aW9uKGFycmF5LCByZXN1bHRzKSB7CgkJdmFyIHJldCA9IHJlc3VsdHMgfHwgW107CgoJ",
				"CWlmICggdG9TdHJpbmcuY2FsbChhcnJheSkgPT09ICJbb2JqZWN0IEFycmF5XSIgKSB7CgkJCUFycmF5",
				"LnByb3RvdHlwZS5wdXNoLmFwcGx5KCByZXQsIGFycmF5ICk7CgkJfSBlbHNlIHsKCQkJaWYgKCB0eXBl",
				"b2YgYXJyYXkubGVuZ3RoID09PSAibnVtYmVyIiApIHsKCQkJCWZvciAoIHZhciBpID0gMCwgbCA9IGFy",
				"cmF5Lmxlbmd0aDsgaSA8IGw7IGkrKyApIHsKCQkJCQlyZXQucHVzaCggYXJyYXlbaV0gKTsKCQkJCX0K",
				"CQkJfSBlbHNlIHsKCQkJCWZvciAoIHZhciBpID0gMDsgYXJyYXlbaV07IGkrKyApIHsKCQkJCQlyZXQu",
				"cHVzaCggYXJyYXlbaV0gKTsKCQkJCX0KCQkJfQoJCX0KCgkJcmV0dXJuIHJldDsKCX07Cn0KCnZhciBz",
				"b3J0T3JkZXI7CgppZiAoIGRvY3VtZW50LmRvY3VtZW50RWxlbWVudC5jb21wYXJlRG9jdW1lbnRQb3Np",
				"dGlvbiApIHsKCXNvcnRPcmRlciA9IGZ1bmN0aW9uKCBhLCBiICkgewoJCWlmICggIWEuY29tcGFyZURv",
				"Y3VtZW50UG9zaXRpb24gfHwgIWIuY29tcGFyZURvY3VtZW50UG9zaXRpb24gKSB7CgkJCWlmICggYSA9",
				"PSBiICkgewoJCQkJaGFzRHVwbGljYXRlID0gdHJ1ZTsKCQkJfQoJCQlyZXR1cm4gMDsKCQl9CgoJCXZh",
				"ciByZXQgPSBhLmNvbXBhcmVEb2N1bWVudFBvc2l0aW9uKGIpICYgNCA/IC0xIDogYSA9PT0gYiA/IDAg",
				"OiAxOwoJCWlmICggcmV0ID09PSAwICkgewoJCQloYXNEdXBsaWNhdGUgPSB0cnVlOwoJCX0KCQlyZXR1",
				"cm4gcmV0OwoJfTsKfSBlbHNlIGlmICggInNvdXJjZUluZGV4IiBpbiBkb2N1bWVudC5kb2N1bWVudEVs",
				"ZW1lbnQgKSB7Cglzb3J0T3JkZXIgPSBmdW5jdGlvbiggYSwgYiApIHsKCQlpZiAoICFhLnNvdXJjZUlu",
				"ZGV4IHx8ICFiLnNvdXJjZUluZGV4ICkgewoJCQlpZiAoIGEgPT0gYiApIHsKCQkJCWhhc0R1cGxpY2F0",
				"ZSA9IHRydWU7CgkJCX0KCQkJcmV0dXJuIDA7CgkJfQoKCQl2YXIgcmV0ID0gYS5zb3VyY2VJbmRleCAt",
				"IGIuc291cmNlSW5kZXg7CgkJaWYgKCByZXQgPT09IDAgKSB7CgkJCWhhc0R1cGxpY2F0ZSA9IHRydWU7",
				"CgkJfQoJCXJldHVybiByZXQ7Cgl9Owp9IGVsc2UgaWYgKCBkb2N1bWVudC5jcmVhdGVSYW5nZSApIHsK",
				"CXNvcnRPcmRlciA9IGZ1bmN0aW9uKCBhLCBiICkgewoJCWlmICggIWEub3duZXJEb2N1bWVudCB8fCAh",
				"Yi5vd25lckRvY3VtZW50ICkgewoJCQlpZiAoIGEgPT0gYiApIHsKCQkJCWhhc0R1cGxpY2F0ZSA9IHRy",
				"dWU7CgkJCX0KCQkJcmV0dXJuIDA7CgkJfQoKCQl2YXIgYVJhbmdlID0gYS5vd25lckRvY3VtZW50LmNy",
				"ZWF0ZVJhbmdlKCksIGJSYW5nZSA9IGIub3duZXJEb2N1bWVudC5jcmVhdGVSYW5nZSgpOwoJCWFSYW5n",
				"ZS5zZXRTdGFydChhLCAwKTsKCQlhUmFuZ2Uuc2V0RW5kKGEsIDApOwoJCWJSYW5nZS5zZXRTdGFydChi",
				"LCAwKTsKCQliUmFuZ2Uuc2V0RW5kKGIsIDApOwoJCXZhciByZXQgPSBhUmFuZ2UuY29tcGFyZUJvdW5k",
				"YXJ5UG9pbnRzKFJhbmdlLlNUQVJUX1RPX0VORCwgYlJhbmdlKTsKCQlpZiAoIHJldCA9PT0gMCApIHsK",
				"CQkJaGFzRHVwbGljYXRlID0gdHJ1ZTsKCQl9CgkJcmV0dXJuIHJldDsKCX07Cn0KCihmdW5jdGlvbigp",
				"ewoJdmFyIGZvcm0gPSBkb2N1bWVudC5jcmVhdGVFbGVtZW50KCJkaXYiKSwKCQlpZCA9ICJzY3JpcHQi",
				"ICsgKG5ldyBEYXRlKS5nZXRUaW1lKCk7Cglmb3JtLmlubmVySFRNTCA9ICI8YSBuYW1lPSciICsgaWQg",
				"KyAiJy8+IjsKCgl2YXIgcm9vdCA9IGRvY3VtZW50LmRvY3VtZW50RWxlbWVudDsKCXJvb3QuaW5zZXJ0",
				"QmVmb3JlKCBmb3JtLCByb290LmZpcnN0Q2hpbGQgKTsKCglpZiAoICEhZG9jdW1lbnQuZ2V0RWxlbWVu",
				"dEJ5SWQoIGlkICkgKSB7CgkJRXhwci5maW5kLklEID0gZnVuY3Rpb24obWF0Y2gsIGNvbnRleHQsIGlz",
				"WE1MKXsKCQkJaWYgKCB0eXBlb2YgY29udGV4dC5nZXRFbGVtZW50QnlJZCAhPT0gInVuZGVmaW5lZCIg",
				"JiYgIWlzWE1MICkgewoJCQkJdmFyIG0gPSBjb250ZXh0LmdldEVsZW1lbnRCeUlkKG1hdGNoWzFdKTsK",
				"CQkJCXJldHVybiBtID8gbS5pZCA9PT0gbWF0Y2hbMV0gfHwgdHlwZW9mIG0uZ2V0QXR0cmlidXRlTm9k",
				"ZSAhPT0gInVuZGVmaW5lZCIgJiYgbS5nZXRBdHRyaWJ1dGVOb2RlKCJpZCIpLm5vZGVWYWx1ZSA9PT0g",
				"bWF0Y2hbMV0gPyBbbV0gOiB1bmRlZmluZWQgOiBbXTsKCQkJfQoJCX07CgoJCUV4cHIuZmlsdGVyLklE",
				"ID0gZnVuY3Rpb24oZWxlbSwgbWF0Y2gpewoJCQl2YXIgbm9kZSA9IHR5cGVvZiBlbGVtLmdldEF0dHJp",
				"YnV0ZU5vZGUgIT09ICJ1bmRlZmluZWQiICYmIGVsZW0uZ2V0QXR0cmlidXRlTm9kZSgiaWQiKTsKCQkJ",
				"cmV0dXJuIGVsZW0ubm9kZVR5cGUgPT09IDEgJiYgbm9kZSAmJiBub2RlLm5vZGVWYWx1ZSA9PT0gbWF0",
				"Y2g7CgkJfTsKCX0KCglyb290LnJlbW92ZUNoaWxkKCBmb3JtICk7Cglyb290ID0gZm9ybSA9IG51bGw7",
				"IC8vIHJlbGVhc2UgbWVtb3J5IGluIElFCn0pKCk7CgooZnVuY3Rpb24oKXsKCgl2YXIgZGl2ID0gZG9j",
				"dW1lbnQuY3JlYXRlRWxlbWVudCgiZGl2Iik7CglkaXYuYXBwZW5kQ2hpbGQoIGRvY3VtZW50LmNyZWF0",
				"ZUNvbW1lbnQoIiIpICk7CgoJaWYgKCBkaXYuZ2V0RWxlbWVudHNCeVRhZ05hbWUoIioiKS5sZW5ndGgg",
				"PiAwICkgewoJCUV4cHIuZmluZC5UQUcgPSBmdW5jdGlvbihtYXRjaCwgY29udGV4dCl7CgkJCXZhciBy",
				"ZXN1bHRzID0gY29udGV4dC5nZXRFbGVtZW50c0J5VGFnTmFtZShtYXRjaFsxXSk7CgoJCQlpZiAoIG1h",
				"dGNoWzFdID09PSAiKiIgKSB7CgkJCQl2YXIgdG1wID0gW107CgoJCQkJZm9yICggdmFyIGkgPSAwOyBy",
				"ZXN1bHRzW2ldOyBpKysgKSB7CgkJCQkJaWYgKCByZXN1bHRzW2ldLm5vZGVUeXBlID09PSAxICkgewoJ",
				"CQkJCQl0bXAucHVzaCggcmVzdWx0c1tpXSApOwoJCQkJCX0KCQkJCX0KCgkJCQlyZXN1bHRzID0gdG1w",
				"OwoJCQl9CgoJCQlyZXR1cm4gcmVzdWx0czsKCQl9OwoJfQoKCWRpdi5pbm5lckhUTUwgPSAiPGEgaHJl",
				"Zj0nIyc+PC9hPiI7CglpZiAoIGRpdi5maXJzdENoaWxkICYmIHR5cGVvZiBkaXYuZmlyc3RDaGlsZC5n",
				"ZXRBdHRyaWJ1dGUgIT09ICJ1bmRlZmluZWQiICYmCgkJCWRpdi5maXJzdENoaWxkLmdldEF0dHJpYnV0",
				"ZSgiaHJlZiIpICE9PSAiIyIgKSB7CgkJRXhwci5hdHRySGFuZGxlLmhyZWYgPSBmdW5jdGlvbihlbGVt",
				"KXsKCQkJcmV0dXJuIGVsZW0uZ2V0QXR0cmlidXRlKCJocmVmIiwgMik7CgkJfTsKCX0KCglkaXYgPSBu",
				"dWxsOyAvLyByZWxlYXNlIG1lbW9yeSBpbiBJRQp9KSgpOwoKaWYgKCBkb2N1bWVudC5xdWVyeVNlbGVj",
				"dG9yQWxsICkgKGZ1bmN0aW9uKCl7Cgl2YXIgb2xkU2l6emxlID0gU2l6emxlLCBkaXYgPSBkb2N1bWVu",
				"dC5jcmVhdGVFbGVtZW50KCJkaXYiKTsKCWRpdi5pbm5lckhUTUwgPSAiPHAgY2xhc3M9J1RFU1QnPjwv",
				"cD4iOwoKCWlmICggZGl2LnF1ZXJ5U2VsZWN0b3JBbGwgJiYgZGl2LnF1ZXJ5U2VsZWN0b3JBbGwoIi5U",
				"RVNUIikubGVuZ3RoID09PSAwICkgewoJCXJldHVybjsKCX0KCglTaXp6bGUgPSBmdW5jdGlvbihxdWVy",
				"eSwgY29udGV4dCwgZXh0cmEsIHNlZWQpewoJCWNvbnRleHQgPSBjb250ZXh0IHx8IGRvY3VtZW50OwoK",
				"CQlpZiAoICFzZWVkICYmIGNvbnRleHQubm9kZVR5cGUgPT09IDkgJiYgIWlzWE1MKGNvbnRleHQpICkg",
				"ewoJCQl0cnkgewoJCQkJcmV0dXJuIG1ha2VBcnJheSggY29udGV4dC5xdWVyeVNlbGVjdG9yQWxsKHF1",
				"ZXJ5KSwgZXh0cmEgKTsKCQkJfSBjYXRjaChlKXt9CgkJfQoKCQlyZXR1cm4gb2xkU2l6emxlKHF1ZXJ5",
				"LCBjb250ZXh0LCBleHRyYSwgc2VlZCk7Cgl9OwoKCWZvciAoIHZhciBwcm9wIGluIG9sZFNpenpsZSAp",
				"IHsKCQlTaXp6bGVbIHByb3AgXSA9IG9sZFNpenpsZVsgcHJvcCBdOwoJfQoKCWRpdiA9IG51bGw7IC8v",
				"IHJlbGVhc2UgbWVtb3J5IGluIElFCn0pKCk7CgppZiAoIGRvY3VtZW50LmdldEVsZW1lbnRzQnlDbGFz",
				"c05hbWUgJiYgZG9jdW1lbnQuZG9jdW1lbnRFbGVtZW50LmdldEVsZW1lbnRzQnlDbGFzc05hbWUgKSAo",
				"ZnVuY3Rpb24oKXsKCXZhciBkaXYgPSBkb2N1bWVudC5jcmVhdGVFbGVtZW50KCJkaXYiKTsKCWRpdi5p",
				"bm5lckhUTUwgPSAiPGRpdiBjbGFzcz0ndGVzdCBlJz48L2Rpdj48ZGl2IGNsYXNzPSd0ZXN0Jz48L2Rp",
				"dj4iOwoKCWlmICggZGl2LmdldEVsZW1lbnRzQnlDbGFzc05hbWUoImUiKS5sZW5ndGggPT09IDAgKQoJ",
				"CXJldHVybjsKCglkaXYubGFzdENoaWxkLmNsYXNzTmFtZSA9ICJlIjsKCglpZiAoIGRpdi5nZXRFbGVt",
				"ZW50c0J5Q2xhc3NOYW1lKCJlIikubGVuZ3RoID09PSAxICkKCQlyZXR1cm47CgoJRXhwci5vcmRlci5z",
				"cGxpY2UoMSwgMCwgIkNMQVNTIik7CglFeHByLmZpbmQuQ0xBU1MgPSBmdW5jdGlvbihtYXRjaCwgY29u",
				"dGV4dCwgaXNYTUwpIHsKCQlpZiAoIHR5cGVvZiBjb250ZXh0LmdldEVsZW1lbnRzQnlDbGFzc05hbWUg",
				"IT09ICJ1bmRlZmluZWQiICYmICFpc1hNTCApIHsKCQkJcmV0dXJuIGNvbnRleHQuZ2V0RWxlbWVudHNC",
				"eUNsYXNzTmFtZShtYXRjaFsxXSk7CgkJfQoJfTsKCglkaXYgPSBudWxsOyAvLyByZWxlYXNlIG1lbW9y",
				"eSBpbiBJRQp9KSgpOwoKZnVuY3Rpb24gZGlyTm9kZUNoZWNrKCBkaXIsIGN1ciwgZG9uZU5hbWUsIGNo",
				"ZWNrU2V0LCBub2RlQ2hlY2ssIGlzWE1MICkgewoJdmFyIHNpYkRpciA9IGRpciA9PSAicHJldmlvdXNT",
				"aWJsaW5nIiAmJiAhaXNYTUw7Cglmb3IgKCB2YXIgaSA9IDAsIGwgPSBjaGVja1NldC5sZW5ndGg7IGkg",
				"PCBsOyBpKysgKSB7CgkJdmFyIGVsZW0gPSBjaGVja1NldFtpXTsKCQlpZiAoIGVsZW0gKSB7CgkJCWlm",
				"ICggc2liRGlyICYmIGVsZW0ubm9kZVR5cGUgPT09IDEgKXsKCQkJCWVsZW0uc2l6Y2FjaGUgPSBkb25l",
				"TmFtZTsKCQkJCWVsZW0uc2l6c2V0ID0gaTsKCQkJfQoJCQllbGVtID0gZWxlbVtkaXJdOwoJCQl2YXIg",
				"bWF0Y2ggPSBmYWxzZTsKCgkJCXdoaWxlICggZWxlbSApIHsKCQkJCWlmICggZWxlbS5zaXpjYWNoZSA9",
				"PT0gZG9uZU5hbWUgKSB7CgkJCQkJbWF0Y2ggPSBjaGVja1NldFtlbGVtLnNpenNldF07CgkJCQkJYnJl",
				"YWs7CgkJCQl9CgoJCQkJaWYgKCBlbGVtLm5vZGVUeXBlID09PSAxICYmICFpc1hNTCApewoJCQkJCWVs",
				"ZW0uc2l6Y2FjaGUgPSBkb25lTmFtZTsKCQkJCQllbGVtLnNpenNldCA9IGk7CgkJCQl9CgoJCQkJaWYg",
				"KCBlbGVtLm5vZGVOYW1lID09PSBjdXIgKSB7CgkJCQkJbWF0Y2ggPSBlbGVtOwoJCQkJCWJyZWFrOwoJ",
				"CQkJfQoKCQkJCWVsZW0gPSBlbGVtW2Rpcl07CgkJCX0KCgkJCWNoZWNrU2V0W2ldID0gbWF0Y2g7CgkJ",
				"fQoJfQp9CgpmdW5jdGlvbiBkaXJDaGVjayggZGlyLCBjdXIsIGRvbmVOYW1lLCBjaGVja1NldCwgbm9k",
				"ZUNoZWNrLCBpc1hNTCApIHsKCXZhciBzaWJEaXIgPSBkaXIgPT0gInByZXZpb3VzU2libGluZyIgJiYg",
				"IWlzWE1MOwoJZm9yICggdmFyIGkgPSAwLCBsID0gY2hlY2tTZXQubGVuZ3RoOyBpIDwgbDsgaSsrICkg",
				"ewoJCXZhciBlbGVtID0gY2hlY2tTZXRbaV07CgkJaWYgKCBlbGVtICkgewoJCQlpZiAoIHNpYkRpciAm",
				"JiBlbGVtLm5vZGVUeXBlID09PSAxICkgewoJCQkJZWxlbS5zaXpjYWNoZSA9IGRvbmVOYW1lOwoJCQkJ",
				"ZWxlbS5zaXpzZXQgPSBpOwoJCQl9CgkJCWVsZW0gPSBlbGVtW2Rpcl07CgkJCXZhciBtYXRjaCA9IGZh",
				"bHNlOwoKCQkJd2hpbGUgKCBlbGVtICkgewoJCQkJaWYgKCBlbGVtLnNpemNhY2hlID09PSBkb25lTmFt",
				"ZSApIHsKCQkJCQltYXRjaCA9IGNoZWNrU2V0W2VsZW0uc2l6c2V0XTsKCQkJCQlicmVhazsKCQkJCX0K",
				"CgkJCQlpZiAoIGVsZW0ubm9kZVR5cGUgPT09IDEgKSB7CgkJCQkJaWYgKCAhaXNYTUwgKSB7CgkJCQkJ",
				"CWVsZW0uc2l6Y2FjaGUgPSBkb25lTmFtZTsKCQkJCQkJZWxlbS5zaXpzZXQgPSBpOwoJCQkJCX0KCQkJ",
				"CQlpZiAoIHR5cGVvZiBjdXIgIT09ICJzdHJpbmciICkgewoJCQkJCQlpZiAoIGVsZW0gPT09IGN1ciAp",
				"IHsKCQkJCQkJCW1hdGNoID0gdHJ1ZTsKCQkJCQkJCWJyZWFrOwoJCQkJCQl9CgoJCQkJCX0gZWxzZSBp",
				"ZiAoIFNpenpsZS5maWx0ZXIoIGN1ciwgW2VsZW1dICkubGVuZ3RoID4gMCApIHsKCQkJCQkJbWF0Y2gg",
				"PSBlbGVtOwoJCQkJCQlicmVhazsKCQkJCQl9CgkJCQl9CgoJCQkJZWxlbSA9IGVsZW1bZGlyXTsKCQkJ",
				"fQoKCQkJY2hlY2tTZXRbaV0gPSBtYXRjaDsKCQl9Cgl9Cn0KCnZhciBjb250YWlucyA9IGRvY3VtZW50",
				"LmNvbXBhcmVEb2N1bWVudFBvc2l0aW9uID8gIGZ1bmN0aW9uKGEsIGIpewoJcmV0dXJuIGEuY29tcGFy",
				"ZURvY3VtZW50UG9zaXRpb24oYikgJiAxNjsKfSA6IGZ1bmN0aW9uKGEsIGIpewoJcmV0dXJuIGEgIT09",
				"IGIgJiYgKGEuY29udGFpbnMgPyBhLmNvbnRhaW5zKGIpIDogdHJ1ZSk7Cn07Cgp2YXIgaXNYTUwgPSBm",
				"dW5jdGlvbihlbGVtKXsKCXJldHVybiBlbGVtLm5vZGVUeXBlID09PSA5ICYmIGVsZW0uZG9jdW1lbnRF",
				"bGVtZW50Lm5vZGVOYW1lICE9PSAiSFRNTCIgfHwKCQkhIWVsZW0ub3duZXJEb2N1bWVudCAmJiBlbGVt",
				"Lm93bmVyRG9jdW1lbnQuZG9jdW1lbnRFbGVtZW50Lm5vZGVOYW1lICE9PSAiSFRNTCI7Cn07Cgp2YXIg",
				"cG9zUHJvY2VzcyA9IGZ1bmN0aW9uKHNlbGVjdG9yLCBjb250ZXh0KXsKCXZhciB0bXBTZXQgPSBbXSwg",
				"bGF0ZXIgPSAiIiwgbWF0Y2gsCgkJcm9vdCA9IGNvbnRleHQubm9kZVR5cGUgPyBbY29udGV4dF0gOiBj",
				"b250ZXh0OwoKCXdoaWxlICggKG1hdGNoID0gRXhwci5tYXRjaC5QU0VVRE8uZXhlYyggc2VsZWN0b3Ig",
				"KSkgKSB7CgkJbGF0ZXIgKz0gbWF0Y2hbMF07CgkJc2VsZWN0b3IgPSBzZWxlY3Rvci5yZXBsYWNlKCBF",
				"eHByLm1hdGNoLlBTRVVETywgIiIgKTsKCX0KCglzZWxlY3RvciA9IEV4cHIucmVsYXRpdmVbc2VsZWN0",
				"b3JdID8gc2VsZWN0b3IgKyAiKiIgOiBzZWxlY3RvcjsKCglmb3IgKCB2YXIgaSA9IDAsIGwgPSByb290",
				"Lmxlbmd0aDsgaSA8IGw7IGkrKyApIHsKCQlTaXp6bGUoIHNlbGVjdG9yLCByb290W2ldLCB0bXBTZXQg",
				"KTsKCX0KCglyZXR1cm4gU2l6emxlLmZpbHRlciggbGF0ZXIsIHRtcFNldCApOwp9OwoKCndpbmRvdy5T",
				"aXp6bGUgPSBTaXp6bGU7Cgp9KSgpOwoKOyhmdW5jdGlvbihlbmdpbmUpIHsKICB2YXIgZXh0ZW5kRWxl",
				"bWVudHMgPSBQcm90b3R5cGUuU2VsZWN0b3IuZXh0ZW5kRWxlbWVudHM7CgogIGZ1bmN0aW9uIHNlbGVj",
				"dChzZWxlY3Rvciwgc2NvcGUpIHsKICAgIHJldHVybiBleHRlbmRFbGVtZW50cyhlbmdpbmUoc2VsZWN0",
				"b3IsIHNjb3BlIHx8IGRvY3VtZW50KSk7CiAgfQoKICBmdW5jdGlvbiBtYXRjaChlbGVtZW50LCBzZWxl",
				"Y3RvcikgewogICAgcmV0dXJuIGVuZ2luZS5tYXRjaGVzKHNlbGVjdG9yLCBbZWxlbWVudF0pLmxlbmd0",
				"aCA9PSAxOwogIH0KCiAgUHJvdG90eXBlLlNlbGVjdG9yLmVuZ2luZSA9IGVuZ2luZTsKICBQcm90b3R5",
				"cGUuU2VsZWN0b3Iuc2VsZWN0ID0gc2VsZWN0OwogIFByb3RvdHlwZS5TZWxlY3Rvci5tYXRjaCA9IG1h",
				"dGNoOwp9KShTaXp6bGUpOwoKd2luZG93LlNpenpsZSA9IFByb3RvdHlwZS5fb3JpZ2luYWxfcHJvcGVy",
				"dHk7CmRlbGV0ZSBQcm90b3R5cGUuX29yaWdpbmFsX3Byb3BlcnR5OwoKdmFyIEZvcm0gPSB7CiAgcmVz",
				"ZXQ6IGZ1bmN0aW9uKGZvcm0pIHsKICAgIGZvcm0gPSAkKGZvcm0pOwogICAgZm9ybS5yZXNldCgpOwog",
				"ICAgcmV0dXJuIGZvcm07CiAgfSwKCiAgc2VyaWFsaXplRWxlbWVudHM6IGZ1bmN0aW9uKGVsZW1lbnRz",
				"LCBvcHRpb25zKSB7CiAgICBpZiAodHlwZW9mIG9wdGlvbnMgIT0gJ29iamVjdCcpIG9wdGlvbnMgPSB7",
				"IGhhc2g6ICEhb3B0aW9ucyB9OwogICAgZWxzZSBpZiAoT2JqZWN0LmlzVW5kZWZpbmVkKG9wdGlvbnMu",
				"aGFzaCkpIG9wdGlvbnMuaGFzaCA9IHRydWU7CiAgICB2YXIga2V5LCB2YWx1ZSwgc3VibWl0dGVkID0g",
				"ZmFsc2UsIHN1Ym1pdCA9IG9wdGlvbnMuc3VibWl0LCBhY2N1bXVsYXRvciwgaW5pdGlhbDsKCiAgICBp",
				"ZiAob3B0aW9ucy5oYXNoKSB7CiAgICAgIGluaXRpYWwgPSB7fTsKICAgICAgYWNjdW11bGF0b3IgPSBm",
				"dW5jdGlvbihyZXN1bHQsIGtleSwgdmFsdWUpIHsKICAgICAgICBpZiAoa2V5IGluIHJlc3VsdCkgewog",
				"ICAgICAgICAgaWYgKCFPYmplY3QuaXNBcnJheShyZXN1bHRba2V5XSkpIHJlc3VsdFtrZXldID0gW3Jl",
				"c3VsdFtrZXldXTsKICAgICAgICAgIHJlc3VsdFtrZXldLnB1c2godmFsdWUpOwogICAgICAgIH0gZWxz",
				"ZSByZXN1bHRba2V5XSA9IHZhbHVlOwogICAgICAgIHJldHVybiByZXN1bHQ7CiAgICAgIH07CiAgICB9",
				"IGVsc2UgewogICAgICBpbml0aWFsID0gJyc7CiAgICAgIGFjY3VtdWxhdG9yID0gZnVuY3Rpb24ocmVz",
				"dWx0LCBrZXksIHZhbHVlKSB7CiAgICAgICAgcmV0dXJuIHJlc3VsdCArIChyZXN1bHQgPyAnJicgOiAn",
				"JykgKyBlbmNvZGVVUklDb21wb25lbnQoa2V5KSArICc9JyArIGVuY29kZVVSSUNvbXBvbmVudCh2YWx1",
				"ZSk7CiAgICAgIH0KICAgIH0KCiAgICByZXR1cm4gZWxlbWVudHMuaW5qZWN0KGluaXRpYWwsIGZ1bmN0",
				"aW9uKHJlc3VsdCwgZWxlbWVudCkgewogICAgICBpZiAoIWVsZW1lbnQuZGlzYWJsZWQgJiYgZWxlbWVu",
				"dC5uYW1lKSB7CiAgICAgICAga2V5ID0gZWxlbWVudC5uYW1lOyB2YWx1ZSA9ICQoZWxlbWVudCkuZ2V0",
				"VmFsdWUoKTsKICAgICAgICBpZiAodmFsdWUgIT0gbnVsbCAmJiBlbGVtZW50LnR5cGUgIT0gJ2ZpbGUn",
				"ICYmIChlbGVtZW50LnR5cGUgIT0gJ3N1Ym1pdCcgfHwgKCFzdWJtaXR0ZWQgJiYKICAgICAgICAgICAg",
				"c3VibWl0ICE9PSBmYWxzZSAmJiAoIXN1Ym1pdCB8fCBrZXkgPT0gc3VibWl0KSAmJiAoc3VibWl0dGVk",
				"ID0gdHJ1ZSkpKSkgewogICAgICAgICAgcmVzdWx0ID0gYWNjdW11bGF0b3IocmVzdWx0LCBrZXksIHZh",
				"bHVlKTsKICAgICAgICB9CiAgICAgIH0KICAgICAgcmV0dXJuIHJlc3VsdDsKICAgIH0pOwogIH0KfTsK",
				"CkZvcm0uTWV0aG9kcyA9IHsKICBzZXJpYWxpemU6IGZ1bmN0aW9uKGZvcm0sIG9wdGlvbnMpIHsKICAg",
				"IHJldHVybiBGb3JtLnNlcmlhbGl6ZUVsZW1lbnRzKEZvcm0uZ2V0RWxlbWVudHMoZm9ybSksIG9wdGlv",
				"bnMpOwogIH0sCgogIGdldEVsZW1lbnRzOiBmdW5jdGlvbihmb3JtKSB7CiAgICB2YXIgZWxlbWVudHMg",
				"PSAkKGZvcm0pLmdldEVsZW1lbnRzQnlUYWdOYW1lKCcqJyksCiAgICAgICAgZWxlbWVudCwKICAgICAg",
				"ICBhcnIgPSBbIF0sCiAgICAgICAgc2VyaWFsaXplcnMgPSBGb3JtLkVsZW1lbnQuU2VyaWFsaXplcnM7",
				"CiAgICBmb3IgKHZhciBpID0gMDsgZWxlbWVudCA9IGVsZW1lbnRzW2ldOyBpKyspIHsKICAgICAgYXJy",
				"LnB1c2goZWxlbWVudCk7CiAgICB9CiAgICByZXR1cm4gYXJyLmluamVjdChbXSwgZnVuY3Rpb24oZWxl",
				"bWVudHMsIGNoaWxkKSB7CiAgICAgIGlmIChzZXJpYWxpemVyc1tjaGlsZC50YWdOYW1lLnRvTG93ZXJD",
				"YXNlKCldKQogICAgICAgIGVsZW1lbnRzLnB1c2goRWxlbWVudC5leHRlbmQoY2hpbGQpKTsKICAgICAg",
				"cmV0dXJuIGVsZW1lbnRzOwogICAgfSkKICB9LAoKICBnZXRJbnB1dHM6IGZ1bmN0aW9uKGZvcm0sIHR5",
				"cGVOYW1lLCBuYW1lKSB7CiAgICBmb3JtID0gJChmb3JtKTsKICAgIHZhciBpbnB1dHMgPSBmb3JtLmdl",
				"dEVsZW1lbnRzQnlUYWdOYW1lKCdpbnB1dCcpOwoKICAgIGlmICghdHlwZU5hbWUgJiYgIW5hbWUpIHJl",
				"dHVybiAkQShpbnB1dHMpLm1hcChFbGVtZW50LmV4dGVuZCk7CgogICAgZm9yICh2YXIgaSA9IDAsIG1h",
				"dGNoaW5nSW5wdXRzID0gW10sIGxlbmd0aCA9IGlucHV0cy5sZW5ndGg7IGkgPCBsZW5ndGg7IGkrKykg",
				"ewogICAgICB2YXIgaW5wdXQgPSBpbnB1dHNbaV07CiAgICAgIGlmICgodHlwZU5hbWUgJiYgaW5wdXQu",
				"dHlwZSAhPSB0eXBlTmFtZSkgfHwgKG5hbWUgJiYgaW5wdXQubmFtZSAhPSBuYW1lKSkKICAgICAgICBj",
				"b250aW51ZTsKICAgICAgbWF0Y2hpbmdJbnB1dHMucHVzaChFbGVtZW50LmV4dGVuZChpbnB1dCkpOwog",
				"ICAgfQoKICAgIHJldHVybiBtYXRjaGluZ0lucHV0czsKICB9LAoKICBkaXNhYmxlOiBmdW5jdGlvbihm",
				"b3JtKSB7CiAgICBmb3JtID0gJChmb3JtKTsKICAgIEZvcm0uZ2V0RWxlbWVudHMoZm9ybSkuaW52b2tl",
				"KCdkaXNhYmxlJyk7CiAgICByZXR1cm4gZm9ybTsKICB9LAoKICBlbmFibGU6IGZ1bmN0aW9uKGZvcm0p",
				"IHsKICAgIGZvcm0gPSAkKGZvcm0pOwogICAgRm9ybS5nZXRFbGVtZW50cyhmb3JtKS5pbnZva2UoJ2Vu",
				"YWJsZScpOwogICAgcmV0dXJuIGZvcm07CiAgfSwKCiAgZmluZEZpcnN0RWxlbWVudDogZnVuY3Rpb24o",
				"Zm9ybSkgewogICAgdmFyIGVsZW1lbnRzID0gJChmb3JtKS5nZXRFbGVtZW50cygpLmZpbmRBbGwoZnVu",
				"Y3Rpb24oZWxlbWVudCkgewogICAgICByZXR1cm4gJ2hpZGRlbicgIT0gZWxlbWVudC50eXBlICYmICFl",
				"bGVtZW50LmRpc2FibGVkOwogICAgfSk7CiAgICB2YXIgZmlyc3RCeUluZGV4ID0gZWxlbWVudHMuZmlu",
				"ZEFsbChmdW5jdGlvbihlbGVtZW50KSB7CiAgICAgIHJldHVybiBlbGVtZW50Lmhhc0F0dHJpYnV0ZSgn",
				"dGFiSW5kZXgnKSAmJiBlbGVtZW50LnRhYkluZGV4ID49IDA7CiAgICB9KS5zb3J0QnkoZnVuY3Rpb24o",
				"ZWxlbWVudCkgeyByZXR1cm4gZWxlbWVudC50YWJJbmRleCB9KS5maXJzdCgpOwoKICAgIHJldHVybiBm",
				"aXJzdEJ5SW5kZXggPyBmaXJzdEJ5SW5kZXggOiBlbGVtZW50cy5maW5kKGZ1bmN0aW9uKGVsZW1lbnQp",
				"IHsKICAgICAgcmV0dXJuIC9eKD86aW5wdXR8c2VsZWN0fHRleHRhcmVhKSQvaS50ZXN0KGVsZW1lbnQu",
				"dGFnTmFtZSk7CiAgICB9KTsKICB9LAoKICBmb2N1c0ZpcnN0RWxlbWVudDogZnVuY3Rpb24oZm9ybSkg",
				"ewogICAgZm9ybSA9ICQoZm9ybSk7CiAgICB2YXIgZWxlbWVudCA9IGZvcm0uZmluZEZpcnN0RWxlbWVu",
				"dCgpOwogICAgaWYgKGVsZW1lbnQpIGVsZW1lbnQuYWN0aXZhdGUoKTsKICAgIHJldHVybiBmb3JtOwog",
				"IH0sCgogIHJlcXVlc3Q6IGZ1bmN0aW9uKGZvcm0sIG9wdGlvbnMpIHsKICAgIGZvcm0gPSAkKGZvcm0p",
				"LCBvcHRpb25zID0gT2JqZWN0LmNsb25lKG9wdGlvbnMgfHwgeyB9KTsKCiAgICB2YXIgcGFyYW1zID0g",
				"b3B0aW9ucy5wYXJhbWV0ZXJzLCBhY3Rpb24gPSBmb3JtLnJlYWRBdHRyaWJ1dGUoJ2FjdGlvbicpIHx8",
				"ICcnOwogICAgaWYgKGFjdGlvbi5ibGFuaygpKSBhY3Rpb24gPSB3aW5kb3cubG9jYXRpb24uaHJlZjsK",
				"ICAgIG9wdGlvbnMucGFyYW1ldGVycyA9IGZvcm0uc2VyaWFsaXplKHRydWUpOwoKICAgIGlmIChwYXJh",
				"bXMpIHsKICAgICAgaWYgKE9iamVjdC5pc1N0cmluZyhwYXJhbXMpKSBwYXJhbXMgPSBwYXJhbXMudG9R",
				"dWVyeVBhcmFtcygpOwogICAgICBPYmplY3QuZXh0ZW5kKG9wdGlvbnMucGFyYW1ldGVycywgcGFyYW1z",
				"KTsKICAgIH0KCiAgICBpZiAoZm9ybS5oYXNBdHRyaWJ1dGUoJ21ldGhvZCcpICYmICFvcHRpb25zLm1l",
				"dGhvZCkKICAgICAgb3B0aW9ucy5tZXRob2QgPSBmb3JtLm1ldGhvZDsKCiAgICByZXR1cm4gbmV3IEFq",
				"YXguUmVxdWVzdChhY3Rpb24sIG9wdGlvbnMpOwogIH0KfTsKCi8qLS0tLS0tLS0tLS0tLS0tLS0tLS0t",
				"LS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0qLwoKCkZv",
				"cm0uRWxlbWVudCA9IHsKICBmb2N1czogZnVuY3Rpb24oZWxlbWVudCkgewogICAgJChlbGVtZW50KS5m",
				"b2N1cygpOwogICAgcmV0dXJuIGVsZW1lbnQ7CiAgfSwKCiAgc2VsZWN0OiBmdW5jdGlvbihlbGVtZW50",
				"KSB7CiAgICAkKGVsZW1lbnQpLnNlbGVjdCgpOwogICAgcmV0dXJuIGVsZW1lbnQ7CiAgfQp9OwoKRm9y",
				"bS5FbGVtZW50Lk1ldGhvZHMgPSB7CgogIHNlcmlhbGl6ZTogZnVuY3Rpb24oZWxlbWVudCkgewogICAg",
				"ZWxlbWVudCA9ICQoZWxlbWVudCk7CiAgICBpZiAoIWVsZW1lbnQuZGlzYWJsZWQgJiYgZWxlbWVudC5u",
				"YW1lKSB7CiAgICAgIHZhciB2YWx1ZSA9IGVsZW1lbnQuZ2V0VmFsdWUoKTsKICAgICAgaWYgKHZhbHVl",
				"ICE9IHVuZGVmaW5lZCkgewogICAgICAgIHZhciBwYWlyID0geyB9OwogICAgICAgIHBhaXJbZWxlbWVu",
				"dC5uYW1lXSA9IHZhbHVlOwogICAgICAgIHJldHVybiBPYmplY3QudG9RdWVyeVN0cmluZyhwYWlyKTsK",
				"ICAgICAgfQogICAgfQogICAgcmV0dXJuICcnOwogIH0sCgogIGdldFZhbHVlOiBmdW5jdGlvbihlbGVt",
				"ZW50KSB7CiAgICBlbGVtZW50ID0gJChlbGVtZW50KTsKICAgIHZhciBtZXRob2QgPSBlbGVtZW50LnRh",
				"Z05hbWUudG9Mb3dlckNhc2UoKTsKICAgIHJldHVybiBGb3JtLkVsZW1lbnQuU2VyaWFsaXplcnNbbWV0",
				"aG9kXShlbGVtZW50KTsKICB9LAoKICBzZXRWYWx1ZTogZnVuY3Rpb24oZWxlbWVudCwgdmFsdWUpIHsK",
				"ICAgIGVsZW1lbnQgPSAkKGVsZW1lbnQpOwogICAgdmFyIG1ldGhvZCA9IGVsZW1lbnQudGFnTmFtZS50",
				"b0xvd2VyQ2FzZSgpOwogICAgRm9ybS5FbGVtZW50LlNlcmlhbGl6ZXJzW21ldGhvZF0oZWxlbWVudCwg",
				"dmFsdWUpOwogICAgcmV0dXJuIGVsZW1lbnQ7CiAgfSwKCiAgY2xlYXI6IGZ1bmN0aW9uKGVsZW1lbnQp",
				"IHsKICAgICQoZWxlbWVudCkudmFsdWUgPSAnJzsKICAgIHJldHVybiBlbGVtZW50OwogIH0sCgogIHBy",
				"ZXNlbnQ6IGZ1bmN0aW9uKGVsZW1lbnQpIHsKICAgIHJldHVybiAkKGVsZW1lbnQpLnZhbHVlICE9ICcn",
				"OwogIH0sCgogIGFjdGl2YXRlOiBmdW5jdGlvbihlbGVtZW50KSB7CiAgICBlbGVtZW50ID0gJChlbGVt",
				"ZW50KTsKICAgIHRyeSB7CiAgICAgIGVsZW1lbnQuZm9jdXMoKTsKICAgICAgaWYgKGVsZW1lbnQuc2Vs",
				"ZWN0ICYmIChlbGVtZW50LnRhZ05hbWUudG9Mb3dlckNhc2UoKSAhPSAnaW5wdXQnIHx8CiAgICAgICAg",
				"ICAhKC9eKD86YnV0dG9ufHJlc2V0fHN1Ym1pdCkkL2kudGVzdChlbGVtZW50LnR5cGUpKSkpCiAgICAg",
				"ICAgZWxlbWVudC5zZWxlY3QoKTsKICAgIH0gY2F0Y2ggKGUpIHsgfQogICAgcmV0dXJuIGVsZW1lbnQ7",
				"CiAgfSwKCiAgZGlzYWJsZTogZnVuY3Rpb24oZWxlbWVudCkgewogICAgZWxlbWVudCA9ICQoZWxlbWVu",
				"dCk7CiAgICBlbGVtZW50LmRpc2FibGVkID0gdHJ1ZTsKICAgIHJldHVybiBlbGVtZW50OwogIH0sCgog",
				"IGVuYWJsZTogZnVuY3Rpb24oZWxlbWVudCkgewogICAgZWxlbWVudCA9ICQoZWxlbWVudCk7CiAgICBl",
				"bGVtZW50LmRpc2FibGVkID0gZmFsc2U7CiAgICByZXR1cm4gZWxlbWVudDsKICB9Cn07CgovKi0tLS0t",
				"LS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0t",
				"LS0tLS0tLS0tKi8KCnZhciBGaWVsZCA9IEZvcm0uRWxlbWVudDsKCnZhciAkRiA9IEZvcm0uRWxlbWVu",
				"dC5NZXRob2RzLmdldFZhbHVlOwoKLyotLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0t",
				"LS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLSovCgpGb3JtLkVsZW1lbnQuU2VyaWFs",
				"aXplcnMgPSAoZnVuY3Rpb24oKSB7CiAgZnVuY3Rpb24gaW5wdXQoZWxlbWVudCwgdmFsdWUpIHsKICAg",
				"IHN3aXRjaCAoZWxlbWVudC50eXBlLnRvTG93ZXJDYXNlKCkpIHsKICAgICAgY2FzZSAnY2hlY2tib3gn",
				"OgogICAgICBjYXNlICdyYWRpbyc6CiAgICAgICAgcmV0dXJuIGlucHV0U2VsZWN0b3IoZWxlbWVudCwg",
				"dmFsdWUpOwogICAgICBkZWZhdWx0OgogICAgICAgIHJldHVybiB2YWx1ZVNlbGVjdG9yKGVsZW1lbnQs",
				"IHZhbHVlKTsKICAgIH0KICB9CgogIGZ1bmN0aW9uIGlucHV0U2VsZWN0b3IoZWxlbWVudCwgdmFsdWUp",
				"IHsKICAgIGlmIChPYmplY3QuaXNVbmRlZmluZWQodmFsdWUpKQogICAgICByZXR1cm4gZWxlbWVudC5j",
				"aGVja2VkID8gZWxlbWVudC52YWx1ZSA6IG51bGw7CiAgICBlbHNlIGVsZW1lbnQuY2hlY2tlZCA9ICEh",
				"dmFsdWU7CiAgfQoKICBmdW5jdGlvbiB2YWx1ZVNlbGVjdG9yKGVsZW1lbnQsIHZhbHVlKSB7CiAgICBp",
				"ZiAoT2JqZWN0LmlzVW5kZWZpbmVkKHZhbHVlKSkgcmV0dXJuIGVsZW1lbnQudmFsdWU7CiAgICBlbHNl",
				"IGVsZW1lbnQudmFsdWUgPSB2YWx1ZTsKICB9CgogIGZ1bmN0aW9uIHNlbGVjdChlbGVtZW50LCB2YWx1",
				"ZSkgewogICAgaWYgKE9iamVjdC5pc1VuZGVmaW5lZCh2YWx1ZSkpCiAgICAgIHJldHVybiAoZWxlbWVu",
				"dC50eXBlID09PSAnc2VsZWN0LW9uZScgPyBzZWxlY3RPbmUgOiBzZWxlY3RNYW55KShlbGVtZW50KTsK",
				"CiAgICB2YXIgb3B0LCBjdXJyZW50VmFsdWUsIHNpbmdsZSA9ICFPYmplY3QuaXNBcnJheSh2YWx1ZSk7",
				"CiAgICBmb3IgKHZhciBpID0gMCwgbGVuZ3RoID0gZWxlbWVudC5sZW5ndGg7IGkgPCBsZW5ndGg7IGkr",
				"KykgewogICAgICBvcHQgPSBlbGVtZW50Lm9wdGlvbnNbaV07CiAgICAgIGN1cnJlbnRWYWx1ZSA9IHRo",
				"aXMub3B0aW9uVmFsdWUob3B0KTsKICAgICAgaWYgKHNpbmdsZSkgewogICAgICAgIGlmIChjdXJyZW50",
				"VmFsdWUgPT0gdmFsdWUpIHsKICAgICAgICAgIG9wdC5zZWxlY3RlZCA9IHRydWU7CiAgICAgICAgICBy",
				"ZXR1cm47CiAgICAgICAgfQogICAgICB9CiAgICAgIGVsc2Ugb3B0LnNlbGVjdGVkID0gdmFsdWUuaW5j",
				"bHVkZShjdXJyZW50VmFsdWUpOwogICAgfQogIH0KCiAgZnVuY3Rpb24gc2VsZWN0T25lKGVsZW1lbnQp",
				"IHsKICAgIHZhciBpbmRleCA9IGVsZW1lbnQuc2VsZWN0ZWRJbmRleDsKICAgIHJldHVybiBpbmRleCA+",
				"PSAwID8gb3B0aW9uVmFsdWUoZWxlbWVudC5vcHRpb25zW2luZGV4XSkgOiBudWxsOwogIH0KCiAgZnVu",
				"Y3Rpb24gc2VsZWN0TWFueShlbGVtZW50KSB7CiAgICB2YXIgdmFsdWVzLCBsZW5ndGggPSBlbGVtZW50",
				"Lmxlbmd0aDsKICAgIGlmICghbGVuZ3RoKSByZXR1cm4gbnVsbDsKCiAgICBmb3IgKHZhciBpID0gMCwg",
				"dmFsdWVzID0gW107IGkgPCBsZW5ndGg7IGkrKykgewogICAgICB2YXIgb3B0ID0gZWxlbWVudC5vcHRp",
				"b25zW2ldOwogICAgICBpZiAob3B0LnNlbGVjdGVkKSB2YWx1ZXMucHVzaChvcHRpb25WYWx1ZShvcHQp",
				"KTsKICAgIH0KICAgIHJldHVybiB2YWx1ZXM7CiAgfQoKICBmdW5jdGlvbiBvcHRpb25WYWx1ZShvcHQp",
				"IHsKICAgIHJldHVybiBFbGVtZW50Lmhhc0F0dHJpYnV0ZShvcHQsICd2YWx1ZScpID8gb3B0LnZhbHVl",
				"IDogb3B0LnRleHQ7CiAgfQoKICByZXR1cm4gewogICAgaW5wdXQ6ICAgICAgICAgaW5wdXQsCiAgICBp",
				"bnB1dFNlbGVjdG9yOiBpbnB1dFNlbGVjdG9yLAogICAgdGV4dGFyZWE6ICAgICAgdmFsdWVTZWxlY3Rv",
				"ciwKICAgIHNlbGVjdDogICAgICAgIHNlbGVjdCwKICAgIHNlbGVjdE9uZTogICAgIHNlbGVjdE9uZSwK",
				"ICAgIHNlbGVjdE1hbnk6ICAgIHNlbGVjdE1hbnksCiAgICBvcHRpb25WYWx1ZTogICBvcHRpb25WYWx1",
				"ZSwKICAgIGJ1dHRvbjogICAgICAgIHZhbHVlU2VsZWN0b3IKICB9Owp9KSgpOwoKLyotLS0tLS0tLS0t",
				"LS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0t",
				"LS0tLSovCgoKQWJzdHJhY3QuVGltZWRPYnNlcnZlciA9IENsYXNzLmNyZWF0ZShQZXJpb2RpY2FsRXhl",
				"Y3V0ZXIsIHsKICBpbml0aWFsaXplOiBmdW5jdGlvbigkc3VwZXIsIGVsZW1lbnQsIGZyZXF1ZW5jeSwg",
				"Y2FsbGJhY2spIHsKICAgICRzdXBlcihjYWxsYmFjaywgZnJlcXVlbmN5KTsKICAgIHRoaXMuZWxlbWVu",
				"dCAgID0gJChlbGVtZW50KTsKICAgIHRoaXMubGFzdFZhbHVlID0gdGhpcy5nZXRWYWx1ZSgpOwogIH0s",
				"CgogIGV4ZWN1dGU6IGZ1bmN0aW9uKCkgewogICAgdmFyIHZhbHVlID0gdGhpcy5nZXRWYWx1ZSgpOwog",
				"ICAgaWYgKE9iamVjdC5pc1N0cmluZyh0aGlzLmxhc3RWYWx1ZSkgJiYgT2JqZWN0LmlzU3RyaW5nKHZh",
				"bHVlKSA/CiAgICAgICAgdGhpcy5sYXN0VmFsdWUgIT0gdmFsdWUgOiBTdHJpbmcodGhpcy5sYXN0VmFs",
				"dWUpICE9IFN0cmluZyh2YWx1ZSkpIHsKICAgICAgdGhpcy5jYWxsYmFjayh0aGlzLmVsZW1lbnQsIHZh",
				"bHVlKTsKICAgICAgdGhpcy5sYXN0VmFsdWUgPSB2YWx1ZTsKICAgIH0KICB9Cn0pOwoKRm9ybS5FbGVt",
				"ZW50Lk9ic2VydmVyID0gQ2xhc3MuY3JlYXRlKEFic3RyYWN0LlRpbWVkT2JzZXJ2ZXIsIHsKICBnZXRW",
				"YWx1ZTogZnVuY3Rpb24oKSB7CiAgICByZXR1cm4gRm9ybS5FbGVtZW50LmdldFZhbHVlKHRoaXMuZWxl",
				"bWVudCk7CiAgfQp9KTsKCkZvcm0uT2JzZXJ2ZXIgPSBDbGFzcy5jcmVhdGUoQWJzdHJhY3QuVGltZWRP",
				"YnNlcnZlciwgewogIGdldFZhbHVlOiBmdW5jdGlvbigpIHsKICAgIHJldHVybiBGb3JtLnNlcmlhbGl6",
				"ZSh0aGlzLmVsZW1lbnQpOwogIH0KfSk7CgovKi0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0t",
				"LS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tKi8KCkFic3RyYWN0LkV2ZW50",
				"T2JzZXJ2ZXIgPSBDbGFzcy5jcmVhdGUoewogIGluaXRpYWxpemU6IGZ1bmN0aW9uKGVsZW1lbnQsIGNh",
				"bGxiYWNrKSB7CiAgICB0aGlzLmVsZW1lbnQgID0gJChlbGVtZW50KTsKICAgIHRoaXMuY2FsbGJhY2sg",
				"PSBjYWxsYmFjazsKCiAgICB0aGlzLmxhc3RWYWx1ZSA9IHRoaXMuZ2V0VmFsdWUoKTsKICAgIGlmICh0",
				"aGlzLmVsZW1lbnQudGFnTmFtZS50b0xvd2VyQ2FzZSgpID09ICdmb3JtJykKICAgICAgdGhpcy5yZWdp",
				"c3RlckZvcm1DYWxsYmFja3MoKTsKICAgIGVsc2UKICAgICAgdGhpcy5yZWdpc3RlckNhbGxiYWNrKHRo",
				"aXMuZWxlbWVudCk7CiAgfSwKCiAgb25FbGVtZW50RXZlbnQ6IGZ1bmN0aW9uKCkgewogICAgdmFyIHZh",
				"bHVlID0gdGhpcy5nZXRWYWx1ZSgpOwogICAgaWYgKHRoaXMubGFzdFZhbHVlICE9IHZhbHVlKSB7CiAg",
				"ICAgIHRoaXMuY2FsbGJhY2sodGhpcy5lbGVtZW50LCB2YWx1ZSk7CiAgICAgIHRoaXMubGFzdFZhbHVl",
				"ID0gdmFsdWU7CiAgICB9CiAgfSwKCiAgcmVnaXN0ZXJGb3JtQ2FsbGJhY2tzOiBmdW5jdGlvbigpIHsK",
				"ICAgIEZvcm0uZ2V0RWxlbWVudHModGhpcy5lbGVtZW50KS5lYWNoKHRoaXMucmVnaXN0ZXJDYWxsYmFj",
				"aywgdGhpcyk7CiAgfSwKCiAgcmVnaXN0ZXJDYWxsYmFjazogZnVuY3Rpb24oZWxlbWVudCkgewogICAg",
				"aWYgKGVsZW1lbnQudHlwZSkgewogICAgICBzd2l0Y2ggKGVsZW1lbnQudHlwZS50b0xvd2VyQ2FzZSgp",
				"KSB7CiAgICAgICAgY2FzZSAnY2hlY2tib3gnOgogICAgICAgIGNhc2UgJ3JhZGlvJzoKICAgICAgICAg",
				"IEV2ZW50Lm9ic2VydmUoZWxlbWVudCwgJ2NsaWNrJywgdGhpcy5vbkVsZW1lbnRFdmVudC5iaW5kKHRo",
				"aXMpKTsKICAgICAgICAgIGJyZWFrOwogICAgICAgIGRlZmF1bHQ6CiAgICAgICAgICBFdmVudC5vYnNl",
				"cnZlKGVsZW1lbnQsICdjaGFuZ2UnLCB0aGlzLm9uRWxlbWVudEV2ZW50LmJpbmQodGhpcykpOwogICAg",
				"ICAgICAgYnJlYWs7CiAgICAgIH0KICAgIH0KICB9Cn0pOwoKRm9ybS5FbGVtZW50LkV2ZW50T2JzZXJ2",
				"ZXIgPSBDbGFzcy5jcmVhdGUoQWJzdHJhY3QuRXZlbnRPYnNlcnZlciwgewogIGdldFZhbHVlOiBmdW5j",
				"dGlvbigpIHsKICAgIHJldHVybiBGb3JtLkVsZW1lbnQuZ2V0VmFsdWUodGhpcy5lbGVtZW50KTsKICB9",
				"Cn0pOwoKRm9ybS5FdmVudE9ic2VydmVyID0gQ2xhc3MuY3JlYXRlKEFic3RyYWN0LkV2ZW50T2JzZXJ2",
				"ZXIsIHsKICBnZXRWYWx1ZTogZnVuY3Rpb24oKSB7CiAgICByZXR1cm4gRm9ybS5zZXJpYWxpemUodGhp",
				"cy5lbGVtZW50KTsKICB9Cn0pOwooZnVuY3Rpb24oKSB7CgogIHZhciBFdmVudCA9IHsKICAgIEtFWV9C",
				"QUNLU1BBQ0U6IDgsCiAgICBLRVlfVEFCOiAgICAgICA5LAogICAgS0VZX1JFVFVSTjogICAxMywKICAg",
				"IEtFWV9FU0M6ICAgICAgMjcsCiAgICBLRVlfTEVGVDogICAgIDM3LAogICAgS0VZX1VQOiAgICAgICAz",
				"OCwKICAgIEtFWV9SSUdIVDogICAgMzksCiAgICBLRVlfRE9XTjogICAgIDQwLAogICAgS0VZX0RFTEVU",
				"RTogICA0NiwKICAgIEtFWV9IT01FOiAgICAgMzYsCiAgICBLRVlfRU5EOiAgICAgIDM1LAogICAgS0VZ",
				"X1BBR0VVUDogICAzMywKICAgIEtFWV9QQUdFRE9XTjogMzQsCiAgICBLRVlfSU5TRVJUOiAgIDQ1LAoK",
				"ICAgIGNhY2hlOiB7fQogIH07CgogIHZhciBkb2NFbCA9IGRvY3VtZW50LmRvY3VtZW50RWxlbWVudDsK",
				"ICB2YXIgTU9VU0VFTlRFUl9NT1VTRUxFQVZFX0VWRU5UU19TVVBQT1JURUQgPSAnb25tb3VzZWVudGVy",
				"JyBpbiBkb2NFbAogICAgJiYgJ29ubW91c2VsZWF2ZScgaW4gZG9jRWw7CgoKCiAgdmFyIGlzSUVMZWdh",
				"Y3lFdmVudCA9IGZ1bmN0aW9uKGV2ZW50KSB7IHJldHVybiBmYWxzZTsgfTsKCiAgaWYgKHdpbmRvdy5h",
				"dHRhY2hFdmVudCkgewogICAgaWYgKHdpbmRvdy5hZGRFdmVudExpc3RlbmVyKSB7CiAgICAgIGlzSUVM",
				"ZWdhY3lFdmVudCA9IGZ1bmN0aW9uKGV2ZW50KSB7CiAgICAgICAgcmV0dXJuICEoZXZlbnQgaW5zdGFu",
				"Y2VvZiB3aW5kb3cuRXZlbnQpOwogICAgICB9OwogICAgfSBlbHNlIHsKICAgICAgaXNJRUxlZ2FjeUV2",
				"ZW50ID0gZnVuY3Rpb24oZXZlbnQpIHsgcmV0dXJuIHRydWU7IH07CiAgICB9CiAgfQoKICB2YXIgX2lz",
				"QnV0dG9uOwoKICBmdW5jdGlvbiBfaXNCdXR0b25Gb3JET01FdmVudHMoZXZlbnQsIGNvZGUpIHsKICAg",
				"IHJldHVybiBldmVudC53aGljaCA/IChldmVudC53aGljaCA9PT0gY29kZSArIDEpIDogKGV2ZW50LmJ1",
				"dHRvbiA9PT0gY29kZSk7CiAgfQoKICB2YXIgbGVnYWN5QnV0dG9uTWFwID0geyAwOiAxLCAxOiA0LCAy",
				"OiAyIH07CiAgZnVuY3Rpb24gX2lzQnV0dG9uRm9yTGVnYWN5RXZlbnRzKGV2ZW50LCBjb2RlKSB7CiAg",
				"ICByZXR1cm4gZXZlbnQuYnV0dG9uID09PSBsZWdhY3lCdXR0b25NYXBbY29kZV07CiAgfQoKICBmdW5j",
				"dGlvbiBfaXNCdXR0b25Gb3JXZWJLaXQoZXZlbnQsIGNvZGUpIHsKICAgIHN3aXRjaCAoY29kZSkgewog",
				"ICAgICBjYXNlIDA6IHJldHVybiBldmVudC53aGljaCA9PSAxICYmICFldmVudC5tZXRhS2V5OwogICAg",
				"ICBjYXNlIDE6IHJldHVybiBldmVudC53aGljaCA9PSAyIHx8IChldmVudC53aGljaCA9PSAxICYmIGV2",
				"ZW50Lm1ldGFLZXkpOwogICAgICBjYXNlIDI6IHJldHVybiBldmVudC53aGljaCA9PSAzOwogICAgICBk",
				"ZWZhdWx0OiByZXR1cm4gZmFsc2U7CiAgICB9CiAgfQoKICBpZiAod2luZG93LmF0dGFjaEV2ZW50KSB7",
				"CiAgICBpZiAoIXdpbmRvdy5hZGRFdmVudExpc3RlbmVyKSB7CiAgICAgIF9pc0J1dHRvbiA9IF9pc0J1",
				"dHRvbkZvckxlZ2FjeUV2ZW50czsKICAgIH0gZWxzZSB7CiAgICAgIF9pc0J1dHRvbiA9IGZ1bmN0aW9u",
				"KGV2ZW50LCBjb2RlKSB7CiAgICAgICAgcmV0dXJuIGlzSUVMZWdhY3lFdmVudChldmVudCkgPyBfaXNC",
				"dXR0b25Gb3JMZWdhY3lFdmVudHMoZXZlbnQsIGNvZGUpIDoKICAgICAgICAgX2lzQnV0dG9uRm9yRE9N",
				"RXZlbnRzKGV2ZW50LCBjb2RlKTsKICAgICAgfQogICAgfQogIH0gZWxzZSBpZiAoUHJvdG90eXBlLkJy",
				"b3dzZXIuV2ViS2l0KSB7CiAgICBfaXNCdXR0b24gPSBfaXNCdXR0b25Gb3JXZWJLaXQ7CiAgfSBlbHNl",
				"IHsKICAgIF9pc0J1dHRvbiA9IF9pc0J1dHRvbkZvckRPTUV2ZW50czsKICB9CgogIGZ1bmN0aW9uIGlz",
				"TGVmdENsaWNrKGV2ZW50KSAgIHsgcmV0dXJuIF9pc0J1dHRvbihldmVudCwgMCkgfQoKICBmdW5jdGlv",
				"biBpc01pZGRsZUNsaWNrKGV2ZW50KSB7IHJldHVybiBfaXNCdXR0b24oZXZlbnQsIDEpIH0KCiAgZnVu",
				"Y3Rpb24gaXNSaWdodENsaWNrKGV2ZW50KSAgeyByZXR1cm4gX2lzQnV0dG9uKGV2ZW50LCAyKSB9Cgog",
				"IGZ1bmN0aW9uIGVsZW1lbnQoZXZlbnQpIHsKICAgIGV2ZW50ID0gRXZlbnQuZXh0ZW5kKGV2ZW50KTsK",
				"CiAgICB2YXIgbm9kZSA9IGV2ZW50LnRhcmdldCwgdHlwZSA9IGV2ZW50LnR5cGUsCiAgICAgY3VycmVu",
				"dFRhcmdldCA9IGV2ZW50LmN1cnJlbnRUYXJnZXQ7CgogICAgaWYgKGN1cnJlbnRUYXJnZXQgJiYgY3Vy",
				"cmVudFRhcmdldC50YWdOYW1lKSB7CiAgICAgIGlmICh0eXBlID09PSAnbG9hZCcgfHwgdHlwZSA9PT0g",
				"J2Vycm9yJyB8fAogICAgICAgICh0eXBlID09PSAnY2xpY2snICYmIGN1cnJlbnRUYXJnZXQudGFnTmFt",
				"ZS50b0xvd2VyQ2FzZSgpID09PSAnaW5wdXQnCiAgICAgICAgICAmJiBjdXJyZW50VGFyZ2V0LnR5cGUg",
				"PT09ICdyYWRpbycpKQogICAgICAgICAgICBub2RlID0gY3VycmVudFRhcmdldDsKICAgIH0KCiAgICBp",
				"ZiAobm9kZS5ub2RlVHlwZSA9PSBOb2RlLlRFWFRfTk9ERSkKICAgICAgbm9kZSA9IG5vZGUucGFyZW50",
				"Tm9kZTsKCiAgICByZXR1cm4gRWxlbWVudC5leHRlbmQobm9kZSk7CiAgfQoKICBmdW5jdGlvbiBmaW5k",
				"RWxlbWVudChldmVudCwgZXhwcmVzc2lvbikgewogICAgdmFyIGVsZW1lbnQgPSBFdmVudC5lbGVtZW50",
				"KGV2ZW50KTsKCiAgICBpZiAoIWV4cHJlc3Npb24pIHJldHVybiBlbGVtZW50OwogICAgd2hpbGUgKGVs",
				"ZW1lbnQpIHsKICAgICAgaWYgKE9iamVjdC5pc0VsZW1lbnQoZWxlbWVudCkgJiYgUHJvdG90eXBlLlNl",
				"bGVjdG9yLm1hdGNoKGVsZW1lbnQsIGV4cHJlc3Npb24pKSB7CiAgICAgICAgcmV0dXJuIEVsZW1lbnQu",
				"ZXh0ZW5kKGVsZW1lbnQpOwogICAgICB9CiAgICAgIGVsZW1lbnQgPSBlbGVtZW50LnBhcmVudE5vZGU7",
				"CiAgICB9CiAgfQoKICBmdW5jdGlvbiBwb2ludGVyKGV2ZW50KSB7CiAgICByZXR1cm4geyB4OiBwb2lu",
				"dGVyWChldmVudCksIHk6IHBvaW50ZXJZKGV2ZW50KSB9OwogIH0KCiAgZnVuY3Rpb24gcG9pbnRlclgo",
				"ZXZlbnQpIHsKICAgIHZhciBkb2NFbGVtZW50ID0gZG9jdW1lbnQuZG9jdW1lbnRFbGVtZW50LAogICAg",
				"IGJvZHkgPSBkb2N1bWVudC5ib2R5IHx8IHsgc2Nyb2xsTGVmdDogMCB9OwoKICAgIHJldHVybiBldmVu",
				"dC5wYWdlWCB8fCAoZXZlbnQuY2xpZW50WCArCiAgICAgIChkb2NFbGVtZW50LnNjcm9sbExlZnQgfHwg",
				"Ym9keS5zY3JvbGxMZWZ0KSAtCiAgICAgIChkb2NFbGVtZW50LmNsaWVudExlZnQgfHwgMCkpOwogIH0K",
				"CiAgZnVuY3Rpb24gcG9pbnRlclkoZXZlbnQpIHsKICAgIHZhciBkb2NFbGVtZW50ID0gZG9jdW1lbnQu",
				"ZG9jdW1lbnRFbGVtZW50LAogICAgIGJvZHkgPSBkb2N1bWVudC5ib2R5IHx8IHsgc2Nyb2xsVG9wOiAw",
				"IH07CgogICAgcmV0dXJuICBldmVudC5wYWdlWSB8fCAoZXZlbnQuY2xpZW50WSArCiAgICAgICAoZG9j",
				"RWxlbWVudC5zY3JvbGxUb3AgfHwgYm9keS5zY3JvbGxUb3ApIC0KICAgICAgIChkb2NFbGVtZW50LmNs",
				"aWVudFRvcCB8fCAwKSk7CiAgfQoKCiAgZnVuY3Rpb24gc3RvcChldmVudCkgewogICAgRXZlbnQuZXh0",
				"ZW5kKGV2ZW50KTsKICAgIGV2ZW50LnByZXZlbnREZWZhdWx0KCk7CiAgICBldmVudC5zdG9wUHJvcGFn",
				"YXRpb24oKTsKCiAgICBldmVudC5zdG9wcGVkID0gdHJ1ZTsKICB9CgoKICBFdmVudC5NZXRob2RzID0g",
				"ewogICAgaXNMZWZ0Q2xpY2s6ICAgaXNMZWZ0Q2xpY2ssCiAgICBpc01pZGRsZUNsaWNrOiBpc01pZGRs",
				"ZUNsaWNrLAogICAgaXNSaWdodENsaWNrOiAgaXNSaWdodENsaWNrLAoKICAgIGVsZW1lbnQ6ICAgICBl",
				"bGVtZW50LAogICAgZmluZEVsZW1lbnQ6IGZpbmRFbGVtZW50LAoKICAgIHBvaW50ZXI6ICBwb2ludGVy",
				"LAogICAgcG9pbnRlclg6IHBvaW50ZXJYLAogICAgcG9pbnRlclk6IHBvaW50ZXJZLAoKICAgIHN0b3A6",
				"IHN0b3AKICB9OwoKICB2YXIgbWV0aG9kcyA9IE9iamVjdC5rZXlzKEV2ZW50Lk1ldGhvZHMpLmluamVj",
				"dCh7IH0sIGZ1bmN0aW9uKG0sIG5hbWUpIHsKICAgIG1bbmFtZV0gPSBFdmVudC5NZXRob2RzW25hbWVd",
				"Lm1ldGhvZGl6ZSgpOwogICAgcmV0dXJuIG07CiAgfSk7CgogIGlmICh3aW5kb3cuYXR0YWNoRXZlbnQp",
				"IHsKICAgIGZ1bmN0aW9uIF9yZWxhdGVkVGFyZ2V0KGV2ZW50KSB7CiAgICAgIHZhciBlbGVtZW50Owog",
				"ICAgICBzd2l0Y2ggKGV2ZW50LnR5cGUpIHsKICAgICAgICBjYXNlICdtb3VzZW92ZXInOgogICAgICAg",
				"IGNhc2UgJ21vdXNlZW50ZXInOgogICAgICAgICAgZWxlbWVudCA9IGV2ZW50LmZyb21FbGVtZW50Owog",
				"ICAgICAgICAgYnJlYWs7CiAgICAgICAgY2FzZSAnbW91c2VvdXQnOgogICAgICAgIGNhc2UgJ21vdXNl",
				"bGVhdmUnOgogICAgICAgICAgZWxlbWVudCA9IGV2ZW50LnRvRWxlbWVudDsKICAgICAgICAgIGJyZWFr",
				"OwogICAgICAgIGRlZmF1bHQ6CiAgICAgICAgICByZXR1cm4gbnVsbDsKICAgICAgfQogICAgICByZXR1",
				"cm4gRWxlbWVudC5leHRlbmQoZWxlbWVudCk7CiAgICB9CgogICAgdmFyIGFkZGl0aW9uYWxNZXRob2Rz",
				"ID0gewogICAgICBzdG9wUHJvcGFnYXRpb246IGZ1bmN0aW9uKCkgeyB0aGlzLmNhbmNlbEJ1YmJsZSA9",
				"IHRydWUgfSwKICAgICAgcHJldmVudERlZmF1bHQ6ICBmdW5jdGlvbigpIHsgdGhpcy5yZXR1cm5WYWx1",
				"ZSA9IGZhbHNlIH0sCiAgICAgIGluc3BlY3Q6IGZ1bmN0aW9uKCkgeyByZXR1cm4gJ1tvYmplY3QgRXZl",
				"bnRdJyB9CiAgICB9OwoKICAgIEV2ZW50LmV4dGVuZCA9IGZ1bmN0aW9uKGV2ZW50LCBlbGVtZW50KSB7",
				"CiAgICAgIGlmICghZXZlbnQpIHJldHVybiBmYWxzZTsKCiAgICAgIGlmICghaXNJRUxlZ2FjeUV2ZW50",
				"KGV2ZW50KSkgcmV0dXJuIGV2ZW50OwoKICAgICAgaWYgKGV2ZW50Ll9leHRlbmRlZEJ5UHJvdG90eXBl",
				"KSByZXR1cm4gZXZlbnQ7CiAgICAgIGV2ZW50Ll9leHRlbmRlZEJ5UHJvdG90eXBlID0gUHJvdG90eXBl",
				"LmVtcHR5RnVuY3Rpb247CgogICAgICB2YXIgcG9pbnRlciA9IEV2ZW50LnBvaW50ZXIoZXZlbnQpOwoK",
				"ICAgICAgT2JqZWN0LmV4dGVuZChldmVudCwgewogICAgICAgIHRhcmdldDogZXZlbnQuc3JjRWxlbWVu",
				"dCB8fCBlbGVtZW50LAogICAgICAgIHJlbGF0ZWRUYXJnZXQ6IF9yZWxhdGVkVGFyZ2V0KGV2ZW50KSwK",
				"ICAgICAgICBwYWdlWDogIHBvaW50ZXIueCwKICAgICAgICBwYWdlWTogIHBvaW50ZXIueQogICAgICB9",
				"KTsKCiAgICAgIE9iamVjdC5leHRlbmQoZXZlbnQsIG1ldGhvZHMpOwogICAgICBPYmplY3QuZXh0ZW5k",
				"KGV2ZW50LCBhZGRpdGlvbmFsTWV0aG9kcyk7CgogICAgICByZXR1cm4gZXZlbnQ7CiAgICB9OwogIH0g",
				"ZWxzZSB7CiAgICBFdmVudC5leHRlbmQgPSBQcm90b3R5cGUuSzsKICB9CgogIGlmICh3aW5kb3cuYWRk",
				"RXZlbnRMaXN0ZW5lcikgewogICAgRXZlbnQucHJvdG90eXBlID0gd2luZG93LkV2ZW50LnByb3RvdHlw",
				"ZSB8fCBkb2N1bWVudC5jcmVhdGVFdmVudCgnSFRNTEV2ZW50cycpLl9fcHJvdG9fXzsKICAgIE9iamVj",
				"dC5leHRlbmQoRXZlbnQucHJvdG90eXBlLCBtZXRob2RzKTsKICB9CgogIGZ1bmN0aW9uIF9jcmVhdGVS",
				"ZXNwb25kZXIoZWxlbWVudCwgZXZlbnROYW1lLCBoYW5kbGVyKSB7CiAgICB2YXIgcmVnaXN0cnkgPSBF",
				"bGVtZW50LnJldHJpZXZlKGVsZW1lbnQsICdwcm90b3R5cGVfZXZlbnRfcmVnaXN0cnknKTsKCiAgICBp",
				"ZiAoT2JqZWN0LmlzVW5kZWZpbmVkKHJlZ2lzdHJ5KSkgewogICAgICBDQUNIRS5wdXNoKGVsZW1lbnQp",
				"OwogICAgICByZWdpc3RyeSA9IEVsZW1lbnQucmV0cmlldmUoZWxlbWVudCwgJ3Byb3RvdHlwZV9ldmVu",
				"dF9yZWdpc3RyeScsICRIKCkpOwogICAgfQoKICAgIHZhciByZXNwb25kZXJzRm9yRXZlbnQgPSByZWdp",
				"c3RyeS5nZXQoZXZlbnROYW1lKTsKICAgIGlmIChPYmplY3QuaXNVbmRlZmluZWQocmVzcG9uZGVyc0Zv",
				"ckV2ZW50KSkgewogICAgICByZXNwb25kZXJzRm9yRXZlbnQgPSBbXTsKICAgICAgcmVnaXN0cnkuc2V0",
				"KGV2ZW50TmFtZSwgcmVzcG9uZGVyc0ZvckV2ZW50KTsKICAgIH0KCiAgICBpZiAocmVzcG9uZGVyc0Zv",
				"ckV2ZW50LnBsdWNrKCdoYW5kbGVyJykuaW5jbHVkZShoYW5kbGVyKSkgcmV0dXJuIGZhbHNlOwoKICAg",
				"IHZhciByZXNwb25kZXI7CiAgICBpZiAoZXZlbnROYW1lLmluY2x1ZGUoIjoiKSkgewogICAgICByZXNw",
				"b25kZXIgPSBmdW5jdGlvbihldmVudCkgewogICAgICAgIGlmIChPYmplY3QuaXNVbmRlZmluZWQoZXZl",
				"bnQuZXZlbnROYW1lKSkKICAgICAgICAgIHJldHVybiBmYWxzZTsKCiAgICAgICAgaWYgKGV2ZW50LmV2",
				"ZW50TmFtZSAhPT0gZXZlbnROYW1lKQogICAgICAgICAgcmV0dXJuIGZhbHNlOwoKICAgICAgICBFdmVu",
				"dC5leHRlbmQoZXZlbnQsIGVsZW1lbnQpOwogICAgICAgIGhhbmRsZXIuY2FsbChlbGVtZW50LCBldmVu",
				"dCk7CiAgICAgIH07CiAgICB9IGVsc2UgewogICAgICBpZiAoIU1PVVNFRU5URVJfTU9VU0VMRUFWRV9F",
				"VkVOVFNfU1VQUE9SVEVEICYmCiAgICAgICAoZXZlbnROYW1lID09PSAibW91c2VlbnRlciIgfHwgZXZl",
				"bnROYW1lID09PSAibW91c2VsZWF2ZSIpKSB7CiAgICAgICAgaWYgKGV2ZW50TmFtZSA9PT0gIm1vdXNl",
				"ZW50ZXIiIHx8IGV2ZW50TmFtZSA9PT0gIm1vdXNlbGVhdmUiKSB7CiAgICAgICAgICByZXNwb25kZXIg",
				"PSBmdW5jdGlvbihldmVudCkgewogICAgICAgICAgICBFdmVudC5leHRlbmQoZXZlbnQsIGVsZW1lbnQp",
				"OwoKICAgICAgICAgICAgdmFyIHBhcmVudCA9IGV2ZW50LnJlbGF0ZWRUYXJnZXQ7CiAgICAgICAgICAg",
				"IHdoaWxlIChwYXJlbnQgJiYgcGFyZW50ICE9PSBlbGVtZW50KSB7CiAgICAgICAgICAgICAgdHJ5IHsg",
				"cGFyZW50ID0gcGFyZW50LnBhcmVudE5vZGU7IH0KICAgICAgICAgICAgICBjYXRjaChlKSB7IHBhcmVu",
				"dCA9IGVsZW1lbnQ7IH0KICAgICAgICAgICAgfQoKICAgICAgICAgICAgaWYgKHBhcmVudCA9PT0gZWxl",
				"bWVudCkgcmV0dXJuOwoKICAgICAgICAgICAgaGFuZGxlci5jYWxsKGVsZW1lbnQsIGV2ZW50KTsKICAg",
				"ICAgICAgIH07CiAgICAgICAgfQogICAgICB9IGVsc2UgewogICAgICAgIHJlc3BvbmRlciA9IGZ1bmN0",
				"aW9uKGV2ZW50KSB7CiAgICAgICAgICBFdmVudC5leHRlbmQoZXZlbnQsIGVsZW1lbnQpOwogICAgICAg",
				"ICAgaGFuZGxlci5jYWxsKGVsZW1lbnQsIGV2ZW50KTsKICAgICAgICB9OwogICAgICB9CiAgICB9Cgog",
				"ICAgcmVzcG9uZGVyLmhhbmRsZXIgPSBoYW5kbGVyOwogICAgcmVzcG9uZGVyc0ZvckV2ZW50LnB1c2go",
				"cmVzcG9uZGVyKTsKICAgIHJldHVybiByZXNwb25kZXI7CiAgfQoKICBmdW5jdGlvbiBfZGVzdHJveUNh",
				"Y2hlKCkgewogICAgZm9yICh2YXIgaSA9IDAsIGxlbmd0aCA9IENBQ0hFLmxlbmd0aDsgaSA8IGxlbmd0",
				"aDsgaSsrKSB7CiAgICAgIEV2ZW50LnN0b3BPYnNlcnZpbmcoQ0FDSEVbaV0pOwogICAgICBDQUNIRVtp",
				"XSA9IG51bGw7CiAgICB9CiAgfQoKICB2YXIgQ0FDSEUgPSBbXTsKCiAgaWYgKFByb3RvdHlwZS5Ccm93",
				"c2VyLklFKQogICAgd2luZG93LmF0dGFjaEV2ZW50KCdvbnVubG9hZCcsIF9kZXN0cm95Q2FjaGUpOwoK",
				"ICBpZiAoUHJvdG90eXBlLkJyb3dzZXIuV2ViS2l0KQogICAgd2luZG93LmFkZEV2ZW50TGlzdGVuZXIo",
				"J3VubG9hZCcsIFByb3RvdHlwZS5lbXB0eUZ1bmN0aW9uLCBmYWxzZSk7CgoKICB2YXIgX2dldERPTUV2",
				"ZW50TmFtZSA9IFByb3RvdHlwZS5LLAogICAgICB0cmFuc2xhdGlvbnMgPSB7IG1vdXNlZW50ZXI6ICJt",
				"b3VzZW92ZXIiLCBtb3VzZWxlYXZlOiAibW91c2VvdXQiIH07CgogIGlmICghTU9VU0VFTlRFUl9NT1VT",
				"RUxFQVZFX0VWRU5UU19TVVBQT1JURUQpIHsKICAgIF9nZXRET01FdmVudE5hbWUgPSBmdW5jdGlvbihl",
				"dmVudE5hbWUpIHsKICAgICAgcmV0dXJuICh0cmFuc2xhdGlvbnNbZXZlbnROYW1lXSB8fCBldmVudE5h",
				"bWUpOwogICAgfTsKICB9CgogIGZ1bmN0aW9uIG9ic2VydmUoZWxlbWVudCwgZXZlbnROYW1lLCBoYW5k",
				"bGVyKSB7CiAgICBlbGVtZW50ID0gJChlbGVtZW50KTsKCiAgICB2YXIgcmVzcG9uZGVyID0gX2NyZWF0",
				"ZVJlc3BvbmRlcihlbGVtZW50LCBldmVudE5hbWUsIGhhbmRsZXIpOwoKICAgIGlmICghcmVzcG9uZGVy",
				"KSByZXR1cm4gZWxlbWVudDsKCiAgICBpZiAoZXZlbnROYW1lLmluY2x1ZGUoJzonKSkgewogICAgICBp",
				"ZiAoZWxlbWVudC5hZGRFdmVudExpc3RlbmVyKQogICAgICAgIGVsZW1lbnQuYWRkRXZlbnRMaXN0ZW5l",
				"cigiZGF0YWF2YWlsYWJsZSIsIHJlc3BvbmRlciwgZmFsc2UpOwogICAgICBlbHNlIHsKICAgICAgICBl",
				"bGVtZW50LmF0dGFjaEV2ZW50KCJvbmRhdGFhdmFpbGFibGUiLCByZXNwb25kZXIpOwogICAgICAgIGVs",
				"ZW1lbnQuYXR0YWNoRXZlbnQoIm9ubG9zZWNhcHR1cmUiLCByZXNwb25kZXIpOwogICAgICB9CiAgICB9",
				"IGVsc2UgewogICAgICB2YXIgYWN0dWFsRXZlbnROYW1lID0gX2dldERPTUV2ZW50TmFtZShldmVudE5h",
				"bWUpOwoKICAgICAgaWYgKGVsZW1lbnQuYWRkRXZlbnRMaXN0ZW5lcikKICAgICAgICBlbGVtZW50LmFk",
				"ZEV2ZW50TGlzdGVuZXIoYWN0dWFsRXZlbnROYW1lLCByZXNwb25kZXIsIGZhbHNlKTsKICAgICAgZWxz",
				"ZQogICAgICAgIGVsZW1lbnQuYXR0YWNoRXZlbnQoIm9uIiArIGFjdHVhbEV2ZW50TmFtZSwgcmVzcG9u",
				"ZGVyKTsKICAgIH0KCiAgICByZXR1cm4gZWxlbWVudDsKICB9CgogIGZ1bmN0aW9uIHN0b3BPYnNlcnZp",
				"bmcoZWxlbWVudCwgZXZlbnROYW1lLCBoYW5kbGVyKSB7CiAgICBlbGVtZW50ID0gJChlbGVtZW50KTsK",
				"CiAgICB2YXIgcmVnaXN0cnkgPSBFbGVtZW50LnJldHJpZXZlKGVsZW1lbnQsICdwcm90b3R5cGVfZXZl",
				"bnRfcmVnaXN0cnknKTsKICAgIGlmICghcmVnaXN0cnkpIHJldHVybiBlbGVtZW50OwoKICAgIGlmICgh",
				"ZXZlbnROYW1lKSB7CiAgICAgIHJlZ2lzdHJ5LmVhY2goIGZ1bmN0aW9uKHBhaXIpIHsKICAgICAgICB2",
				"YXIgZXZlbnROYW1lID0gcGFpci5rZXk7CiAgICAgICAgc3RvcE9ic2VydmluZyhlbGVtZW50LCBldmVu",
				"dE5hbWUpOwogICAgICB9KTsKICAgICAgcmV0dXJuIGVsZW1lbnQ7CiAgICB9CgogICAgdmFyIHJlc3Bv",
				"bmRlcnMgPSByZWdpc3RyeS5nZXQoZXZlbnROYW1lKTsKICAgIGlmICghcmVzcG9uZGVycykgcmV0dXJu",
				"IGVsZW1lbnQ7CgogICAgaWYgKCFoYW5kbGVyKSB7CiAgICAgIHJlc3BvbmRlcnMuZWFjaChmdW5jdGlv",
				"bihyKSB7CiAgICAgICAgc3RvcE9ic2VydmluZyhlbGVtZW50LCBldmVudE5hbWUsIHIuaGFuZGxlcik7",
				"CiAgICAgIH0pOwogICAgICByZXR1cm4gZWxlbWVudDsKICAgIH0KCiAgICB2YXIgaSA9IHJlc3BvbmRl",
				"cnMubGVuZ3RoLCByZXNwb25kZXI7CiAgICB3aGlsZSAoaS0tKSB7CiAgICAgIGlmIChyZXNwb25kZXJz",
				"W2ldLmhhbmRsZXIgPT09IGhhbmRsZXIpIHsKICAgICAgICByZXNwb25kZXIgPSByZXNwb25kZXJzW2ld",
				"OwogICAgICAgIGJyZWFrOwogICAgICB9CiAgICB9CiAgICBpZiAoIXJlc3BvbmRlcikgcmV0dXJuIGVs",
				"ZW1lbnQ7CgogICAgaWYgKGV2ZW50TmFtZS5pbmNsdWRlKCc6JykpIHsKICAgICAgaWYgKGVsZW1lbnQu",
				"cmVtb3ZlRXZlbnRMaXN0ZW5lcikKICAgICAgICBlbGVtZW50LnJlbW92ZUV2ZW50TGlzdGVuZXIoImRh",
				"dGFhdmFpbGFibGUiLCByZXNwb25kZXIsIGZhbHNlKTsKICAgICAgZWxzZSB7CiAgICAgICAgZWxlbWVu",
				"dC5kZXRhY2hFdmVudCgib25kYXRhYXZhaWxhYmxlIiwgcmVzcG9uZGVyKTsKICAgICAgICBlbGVtZW50",
				"LmRldGFjaEV2ZW50KCJvbmxvc2VjYXB0dXJlIiwgcmVzcG9uZGVyKTsKICAgICAgfQogICAgfSBlbHNl",
				"IHsKICAgICAgdmFyIGFjdHVhbEV2ZW50TmFtZSA9IF9nZXRET01FdmVudE5hbWUoZXZlbnROYW1lKTsK",
				"ICAgICAgaWYgKGVsZW1lbnQucmVtb3ZlRXZlbnRMaXN0ZW5lcikKICAgICAgICBlbGVtZW50LnJlbW92",
				"ZUV2ZW50TGlzdGVuZXIoYWN0dWFsRXZlbnROYW1lLCByZXNwb25kZXIsIGZhbHNlKTsKICAgICAgZWxz",
				"ZQogICAgICAgIGVsZW1lbnQuZGV0YWNoRXZlbnQoJ29uJyArIGFjdHVhbEV2ZW50TmFtZSwgcmVzcG9u",
				"ZGVyKTsKICAgIH0KCiAgICByZWdpc3RyeS5zZXQoZXZlbnROYW1lLCByZXNwb25kZXJzLndpdGhvdXQo",
				"cmVzcG9uZGVyKSk7CgogICAgcmV0dXJuIGVsZW1lbnQ7CiAgfQoKICBmdW5jdGlvbiBmaXJlKGVsZW1l",
				"bnQsIGV2ZW50TmFtZSwgbWVtbywgYnViYmxlKSB7CiAgICBlbGVtZW50ID0gJChlbGVtZW50KTsKCiAg",
				"ICBpZiAoT2JqZWN0LmlzVW5kZWZpbmVkKGJ1YmJsZSkpCiAgICAgIGJ1YmJsZSA9IHRydWU7CgogICAg",
				"aWYgKGVsZW1lbnQgPT0gZG9jdW1lbnQgJiYgZG9jdW1lbnQuY3JlYXRlRXZlbnQgJiYgIWVsZW1lbnQu",
				"ZGlzcGF0Y2hFdmVudCkKICAgICAgZWxlbWVudCA9IGRvY3VtZW50LmRvY3VtZW50RWxlbWVudDsKCiAg",
				"ICB2YXIgZXZlbnQ7CiAgICBpZiAoZG9jdW1lbnQuY3JlYXRlRXZlbnQpIHsKICAgICAgZXZlbnQgPSBk",
				"b2N1bWVudC5jcmVhdGVFdmVudCgnSFRNTEV2ZW50cycpOwogICAgICBldmVudC5pbml0RXZlbnQoJ2Rh",
				"dGFhdmFpbGFibGUnLCBidWJibGUsIHRydWUpOwogICAgfSBlbHNlIHsKICAgICAgZXZlbnQgPSBkb2N1",
				"bWVudC5jcmVhdGVFdmVudE9iamVjdCgpOwogICAgICBldmVudC5ldmVudFR5cGUgPSBidWJibGUgPyAn",
				"b25kYXRhYXZhaWxhYmxlJyA6ICdvbmxvc2VjYXB0dXJlJzsKICAgIH0KCiAgICBldmVudC5ldmVudE5h",
				"bWUgPSBldmVudE5hbWU7CiAgICBldmVudC5tZW1vID0gbWVtbyB8fCB7IH07CgogICAgaWYgKGRvY3Vt",
				"ZW50LmNyZWF0ZUV2ZW50KQogICAgICBlbGVtZW50LmRpc3BhdGNoRXZlbnQoZXZlbnQpOwogICAgZWxz",
				"ZQogICAgICBlbGVtZW50LmZpcmVFdmVudChldmVudC5ldmVudFR5cGUsIGV2ZW50KTsKCiAgICByZXR1",
				"cm4gRXZlbnQuZXh0ZW5kKGV2ZW50KTsKICB9CgogIEV2ZW50LkhhbmRsZXIgPSBDbGFzcy5jcmVhdGUo",
				"ewogICAgaW5pdGlhbGl6ZTogZnVuY3Rpb24oZWxlbWVudCwgZXZlbnROYW1lLCBzZWxlY3RvciwgY2Fs",
				"bGJhY2spIHsKICAgICAgdGhpcy5lbGVtZW50ICAgPSAkKGVsZW1lbnQpOwogICAgICB0aGlzLmV2ZW50",
				"TmFtZSA9IGV2ZW50TmFtZTsKICAgICAgdGhpcy5zZWxlY3RvciAgPSBzZWxlY3RvcjsKICAgICAgdGhp",
				"cy5jYWxsYmFjayAgPSBjYWxsYmFjazsKICAgICAgdGhpcy5oYW5kbGVyICAgPSB0aGlzLmhhbmRsZUV2",
				"ZW50LmJpbmQodGhpcyk7CiAgICB9LAoKICAgIHN0YXJ0OiBmdW5jdGlvbigpIHsKICAgICAgRXZlbnQu",
				"b2JzZXJ2ZSh0aGlzLmVsZW1lbnQsIHRoaXMuZXZlbnROYW1lLCB0aGlzLmhhbmRsZXIpOwogICAgICBy",
				"ZXR1cm4gdGhpczsKICAgIH0sCgogICAgc3RvcDogZnVuY3Rpb24oKSB7CiAgICAgIEV2ZW50LnN0b3BP",
				"YnNlcnZpbmcodGhpcy5lbGVtZW50LCB0aGlzLmV2ZW50TmFtZSwgdGhpcy5oYW5kbGVyKTsKICAgICAg",
				"cmV0dXJuIHRoaXM7CiAgICB9LAoKICAgIGhhbmRsZUV2ZW50OiBmdW5jdGlvbihldmVudCkgewogICAg",
				"ICB2YXIgZWxlbWVudCA9IEV2ZW50LmZpbmRFbGVtZW50KGV2ZW50LCB0aGlzLnNlbGVjdG9yKTsKICAg",
				"ICAgaWYgKGVsZW1lbnQpIHRoaXMuY2FsbGJhY2suY2FsbCh0aGlzLmVsZW1lbnQsIGV2ZW50LCBlbGVt",
				"ZW50KTsKICAgIH0KICB9KTsKCiAgZnVuY3Rpb24gb24oZWxlbWVudCwgZXZlbnROYW1lLCBzZWxlY3Rv",
				"ciwgY2FsbGJhY2spIHsKICAgIGVsZW1lbnQgPSAkKGVsZW1lbnQpOwogICAgaWYgKE9iamVjdC5pc0Z1",
				"bmN0aW9uKHNlbGVjdG9yKSAmJiBPYmplY3QuaXNVbmRlZmluZWQoY2FsbGJhY2spKSB7CiAgICAgIGNh",
				"bGxiYWNrID0gc2VsZWN0b3IsIHNlbGVjdG9yID0gbnVsbDsKICAgIH0KCiAgICByZXR1cm4gbmV3IEV2",
				"ZW50LkhhbmRsZXIoZWxlbWVudCwgZXZlbnROYW1lLCBzZWxlY3RvciwgY2FsbGJhY2spLnN0YXJ0KCk7",
				"CiAgfQoKICBPYmplY3QuZXh0ZW5kKEV2ZW50LCBFdmVudC5NZXRob2RzKTsKCiAgT2JqZWN0LmV4dGVu",
				"ZChFdmVudCwgewogICAgZmlyZTogICAgICAgICAgZmlyZSwKICAgIG9ic2VydmU6ICAgICAgIG9ic2Vy",
				"dmUsCiAgICBzdG9wT2JzZXJ2aW5nOiBzdG9wT2JzZXJ2aW5nLAogICAgb246ICAgICAgICAgICAgb24K",
				"ICB9KTsKCiAgRWxlbWVudC5hZGRNZXRob2RzKHsKICAgIGZpcmU6ICAgICAgICAgIGZpcmUsCgogICAg",
				"b2JzZXJ2ZTogICAgICAgb2JzZXJ2ZSwKCiAgICBzdG9wT2JzZXJ2aW5nOiBzdG9wT2JzZXJ2aW5nLAoK",
				"ICAgIG9uOiAgICAgICAgICAgIG9uCiAgfSk7CgogIE9iamVjdC5leHRlbmQoZG9jdW1lbnQsIHsKICAg",
				"IGZpcmU6ICAgICAgICAgIGZpcmUubWV0aG9kaXplKCksCgogICAgb2JzZXJ2ZTogICAgICAgb2JzZXJ2",
				"ZS5tZXRob2RpemUoKSwKCiAgICBzdG9wT2JzZXJ2aW5nOiBzdG9wT2JzZXJ2aW5nLm1ldGhvZGl6ZSgp",
				"LAoKICAgIG9uOiAgICAgICAgICAgIG9uLm1ldGhvZGl6ZSgpLAoKICAgIGxvYWRlZDogICAgICAgIGZh",
				"bHNlCiAgfSk7CgogIGlmICh3aW5kb3cuRXZlbnQpIE9iamVjdC5leHRlbmQod2luZG93LkV2ZW50LCBF",
				"dmVudCk7CiAgZWxzZSB3aW5kb3cuRXZlbnQgPSBFdmVudDsKfSkoKTsKCihmdW5jdGlvbigpIHsKICAv",
				"KiBTdXBwb3J0IGZvciB0aGUgRE9NQ29udGVudExvYWRlZCBldmVudCBpcyBiYXNlZCBvbiB3b3JrIGJ5",
				"IERhbiBXZWJiLAogICAgIE1hdHRoaWFzIE1pbGxlciwgRGVhbiBFZHdhcmRzLCBKb2huIFJlc2lnLCBh",
				"bmQgRGllZ28gUGVyaW5pLiAqLwoKICB2YXIgdGltZXI7CgogIGZ1bmN0aW9uIGZpcmVDb250ZW50TG9h",
				"ZGVkRXZlbnQoKSB7CiAgICBpZiAoZG9jdW1lbnQubG9hZGVkKSByZXR1cm47CiAgICBpZiAodGltZXIp",
				"IHdpbmRvdy5jbGVhclRpbWVvdXQodGltZXIpOwogICAgZG9jdW1lbnQubG9hZGVkID0gdHJ1ZTsKICAg",
				"IGRvY3VtZW50LmZpcmUoJ2RvbTpsb2FkZWQnKTsKICB9CgogIGZ1bmN0aW9uIGNoZWNrUmVhZHlTdGF0",
				"ZSgpIHsKICAgIGlmIChkb2N1bWVudC5yZWFkeVN0YXRlID09PSAnY29tcGxldGUnKSB7CiAgICAgIGRv",
				"Y3VtZW50LnN0b3BPYnNlcnZpbmcoJ3JlYWR5c3RhdGVjaGFuZ2UnLCBjaGVja1JlYWR5U3RhdGUpOwog",
				"ICAgICBmaXJlQ29udGVudExvYWRlZEV2ZW50KCk7CiAgICB9CiAgfQoKICBmdW5jdGlvbiBwb2xsRG9T",
				"Y3JvbGwoKSB7CiAgICB0cnkgeyBkb2N1bWVudC5kb2N1bWVudEVsZW1lbnQuZG9TY3JvbGwoJ2xlZnQn",
				"KTsgfQogICAgY2F0Y2goZSkgewogICAgICB0aW1lciA9IHBvbGxEb1Njcm9sbC5kZWZlcigpOwogICAg",
				"ICByZXR1cm47CiAgICB9CiAgICBmaXJlQ29udGVudExvYWRlZEV2ZW50KCk7CiAgfQoKICBpZiAoZG9j",
				"dW1lbnQuYWRkRXZlbnRMaXN0ZW5lcikgewogICAgZG9jdW1lbnQuYWRkRXZlbnRMaXN0ZW5lcignRE9N",
				"Q29udGVudExvYWRlZCcsIGZpcmVDb250ZW50TG9hZGVkRXZlbnQsIGZhbHNlKTsKICB9IGVsc2Ugewog",
				"ICAgZG9jdW1lbnQub2JzZXJ2ZSgncmVhZHlzdGF0ZWNoYW5nZScsIGNoZWNrUmVhZHlTdGF0ZSk7CiAg",
				"ICBpZiAod2luZG93ID09IHRvcCkKICAgICAgdGltZXIgPSBwb2xsRG9TY3JvbGwuZGVmZXIoKTsKICB9",
				"CgogIEV2ZW50Lm9ic2VydmUod2luZG93LCAnbG9hZCcsIGZpcmVDb250ZW50TG9hZGVkRXZlbnQpOwp9",
				"KSgpOwoKRWxlbWVudC5hZGRNZXRob2RzKCk7CgovKi0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0t",
				"LS0gREVQUkVDQVRFRCAtLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tKi8KCkhhc2gudG9RdWVy",
				"eVN0cmluZyA9IE9iamVjdC50b1F1ZXJ5U3RyaW5nOwoKdmFyIFRvZ2dsZSA9IHsgZGlzcGxheTogRWxl",
				"bWVudC50b2dnbGUgfTsKCkVsZW1lbnQuTWV0aG9kcy5jaGlsZE9mID0gRWxlbWVudC5NZXRob2RzLmRl",
				"c2NlbmRhbnRPZjsKCnZhciBJbnNlcnRpb24gPSB7CiAgQmVmb3JlOiBmdW5jdGlvbihlbGVtZW50LCBj",
				"b250ZW50KSB7CiAgICByZXR1cm4gRWxlbWVudC5pbnNlcnQoZWxlbWVudCwge2JlZm9yZTpjb250ZW50",
				"fSk7CiAgfSwKCiAgVG9wOiBmdW5jdGlvbihlbGVtZW50LCBjb250ZW50KSB7CiAgICByZXR1cm4gRWxl",
				"bWVudC5pbnNlcnQoZWxlbWVudCwge3RvcDpjb250ZW50fSk7CiAgfSwKCiAgQm90dG9tOiBmdW5jdGlv",
				"bihlbGVtZW50LCBjb250ZW50KSB7CiAgICByZXR1cm4gRWxlbWVudC5pbnNlcnQoZWxlbWVudCwge2Jv",
				"dHRvbTpjb250ZW50fSk7CiAgfSwKCiAgQWZ0ZXI6IGZ1bmN0aW9uKGVsZW1lbnQsIGNvbnRlbnQpIHsK",
				"ICAgIHJldHVybiBFbGVtZW50Lmluc2VydChlbGVtZW50LCB7YWZ0ZXI6Y29udGVudH0pOwogIH0KfTsK",
				"CnZhciAkY29udGludWUgPSBuZXcgRXJyb3IoJyJ0aHJvdyAkY29udGludWUiIGlzIGRlcHJlY2F0ZWQs",
				"IHVzZSAicmV0dXJuIiBpbnN0ZWFkJyk7Cgp2YXIgUG9zaXRpb24gPSB7CiAgaW5jbHVkZVNjcm9sbE9m",
				"ZnNldHM6IGZhbHNlLAoKICBwcmVwYXJlOiBmdW5jdGlvbigpIHsKICAgIHRoaXMuZGVsdGFYID0gIHdp",
				"bmRvdy5wYWdlWE9mZnNldAogICAgICAgICAgICAgICAgfHwgZG9jdW1lbnQuZG9jdW1lbnRFbGVtZW50",
				"LnNjcm9sbExlZnQKICAgICAgICAgICAgICAgIHx8IGRvY3VtZW50LmJvZHkuc2Nyb2xsTGVmdAogICAg",
				"ICAgICAgICAgICAgfHwgMDsKICAgIHRoaXMuZGVsdGFZID0gIHdpbmRvdy5wYWdlWU9mZnNldAogICAg",
				"ICAgICAgICAgICAgfHwgZG9jdW1lbnQuZG9jdW1lbnRFbGVtZW50LnNjcm9sbFRvcAogICAgICAgICAg",
				"ICAgICAgfHwgZG9jdW1lbnQuYm9keS5zY3JvbGxUb3AKICAgICAgICAgICAgICAgIHx8IDA7CiAgfSwK",
				"CiAgd2l0aGluOiBmdW5jdGlvbihlbGVtZW50LCB4LCB5KSB7CiAgICBpZiAodGhpcy5pbmNsdWRlU2Ny",
				"b2xsT2Zmc2V0cykKICAgICAgcmV0dXJuIHRoaXMud2l0aGluSW5jbHVkaW5nU2Nyb2xsb2Zmc2V0cyhl",
				"bGVtZW50LCB4LCB5KTsKICAgIHRoaXMueGNvbXAgPSB4OwogICAgdGhpcy55Y29tcCA9IHk7CiAgICB0",
				"aGlzLm9mZnNldCA9IEVsZW1lbnQuY3VtdWxhdGl2ZU9mZnNldChlbGVtZW50KTsKCiAgICByZXR1cm4g",
				"KHkgPj0gdGhpcy5vZmZzZXRbMV0gJiYKICAgICAgICAgICAgeSA8ICB0aGlzLm9mZnNldFsxXSArIGVs",
				"ZW1lbnQub2Zmc2V0SGVpZ2h0ICYmCiAgICAgICAgICAgIHggPj0gdGhpcy5vZmZzZXRbMF0gJiYKICAg",
				"ICAgICAgICAgeCA8ICB0aGlzLm9mZnNldFswXSArIGVsZW1lbnQub2Zmc2V0V2lkdGgpOwogIH0sCgog",
				"IHdpdGhpbkluY2x1ZGluZ1Njcm9sbG9mZnNldHM6IGZ1bmN0aW9uKGVsZW1lbnQsIHgsIHkpIHsKICAg",
				"IHZhciBvZmZzZXRjYWNoZSA9IEVsZW1lbnQuY3VtdWxhdGl2ZVNjcm9sbE9mZnNldChlbGVtZW50KTsK",
				"CiAgICB0aGlzLnhjb21wID0geCArIG9mZnNldGNhY2hlWzBdIC0gdGhpcy5kZWx0YVg7CiAgICB0aGlz",
				"Lnljb21wID0geSArIG9mZnNldGNhY2hlWzFdIC0gdGhpcy5kZWx0YVk7CiAgICB0aGlzLm9mZnNldCA9",
				"IEVsZW1lbnQuY3VtdWxhdGl2ZU9mZnNldChlbGVtZW50KTsKCiAgICByZXR1cm4gKHRoaXMueWNvbXAg",
				"Pj0gdGhpcy5vZmZzZXRbMV0gJiYKICAgICAgICAgICAgdGhpcy55Y29tcCA8ICB0aGlzLm9mZnNldFsx",
				"XSArIGVsZW1lbnQub2Zmc2V0SGVpZ2h0ICYmCiAgICAgICAgICAgIHRoaXMueGNvbXAgPj0gdGhpcy5v",
				"ZmZzZXRbMF0gJiYKICAgICAgICAgICAgdGhpcy54Y29tcCA8ICB0aGlzLm9mZnNldFswXSArIGVsZW1l",
				"bnQub2Zmc2V0V2lkdGgpOwogIH0sCgogIG92ZXJsYXA6IGZ1bmN0aW9uKG1vZGUsIGVsZW1lbnQpIHsK",
				"ICAgIGlmICghbW9kZSkgcmV0dXJuIDA7CiAgICBpZiAobW9kZSA9PSAndmVydGljYWwnKQogICAgICBy",
				"ZXR1cm4gKCh0aGlzLm9mZnNldFsxXSArIGVsZW1lbnQub2Zmc2V0SGVpZ2h0KSAtIHRoaXMueWNvbXAp",
				"IC8KICAgICAgICBlbGVtZW50Lm9mZnNldEhlaWdodDsKICAgIGlmIChtb2RlID09ICdob3Jpem9udGFs",
				"JykKICAgICAgcmV0dXJuICgodGhpcy5vZmZzZXRbMF0gKyBlbGVtZW50Lm9mZnNldFdpZHRoKSAtIHRo",
				"aXMueGNvbXApIC8KICAgICAgICBlbGVtZW50Lm9mZnNldFdpZHRoOwogIH0sCgoKICBjdW11bGF0aXZl",
				"T2Zmc2V0OiBFbGVtZW50Lk1ldGhvZHMuY3VtdWxhdGl2ZU9mZnNldCwKCiAgcG9zaXRpb25lZE9mZnNl",
				"dDogRWxlbWVudC5NZXRob2RzLnBvc2l0aW9uZWRPZmZzZXQsCgogIGFic29sdXRpemU6IGZ1bmN0aW9u",
				"KGVsZW1lbnQpIHsKICAgIFBvc2l0aW9uLnByZXBhcmUoKTsKICAgIHJldHVybiBFbGVtZW50LmFic29s",
				"dXRpemUoZWxlbWVudCk7CiAgfSwKCiAgcmVsYXRpdml6ZTogZnVuY3Rpb24oZWxlbWVudCkgewogICAg",
				"UG9zaXRpb24ucHJlcGFyZSgpOwogICAgcmV0dXJuIEVsZW1lbnQucmVsYXRpdml6ZShlbGVtZW50KTsK",
				"ICB9LAoKICByZWFsT2Zmc2V0OiBFbGVtZW50Lk1ldGhvZHMuY3VtdWxhdGl2ZVNjcm9sbE9mZnNldCwK",
				"CiAgb2Zmc2V0UGFyZW50OiBFbGVtZW50Lk1ldGhvZHMuZ2V0T2Zmc2V0UGFyZW50LAoKICBwYWdlOiBF",
				"bGVtZW50Lk1ldGhvZHMudmlld3BvcnRPZmZzZXQsCgogIGNsb25lOiBmdW5jdGlvbihzb3VyY2UsIHRh",
				"cmdldCwgb3B0aW9ucykgewogICAgb3B0aW9ucyA9IG9wdGlvbnMgfHwgeyB9OwogICAgcmV0dXJuIEVs",
				"ZW1lbnQuY2xvbmVQb3NpdGlvbih0YXJnZXQsIHNvdXJjZSwgb3B0aW9ucyk7CiAgfQp9OwoKLyotLS0t",
				"LS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0t",
				"LS0tLS0tLS0tLSovCgppZiAoIWRvY3VtZW50LmdldEVsZW1lbnRzQnlDbGFzc05hbWUpIGRvY3VtZW50",
				"LmdldEVsZW1lbnRzQnlDbGFzc05hbWUgPSBmdW5jdGlvbihpbnN0YW5jZU1ldGhvZHMpewogIGZ1bmN0",
				"aW9uIGl0ZXIobmFtZSkgewogICAgcmV0dXJuIG5hbWUuYmxhbmsoKSA/IG51bGwgOiAiW2NvbnRhaW5z",
				"KGNvbmNhdCgnICcsIEBjbGFzcywgJyAnKSwgJyAiICsgbmFtZSArICIgJyldIjsKICB9CgogIGluc3Rh",
				"bmNlTWV0aG9kcy5nZXRFbGVtZW50c0J5Q2xhc3NOYW1lID0gUHJvdG90eXBlLkJyb3dzZXJGZWF0dXJl",
				"cy5YUGF0aCA/CiAgZnVuY3Rpb24oZWxlbWVudCwgY2xhc3NOYW1lKSB7CiAgICBjbGFzc05hbWUgPSBj",
				"bGFzc05hbWUudG9TdHJpbmcoKS5zdHJpcCgpOwogICAgdmFyIGNvbmQgPSAvXHMvLnRlc3QoY2xhc3NO",
				"YW1lKSA/ICR3KGNsYXNzTmFtZSkubWFwKGl0ZXIpLmpvaW4oJycpIDogaXRlcihjbGFzc05hbWUpOwog",
				"ICAgcmV0dXJuIGNvbmQgPyBkb2N1bWVudC5fZ2V0RWxlbWVudHNCeVhQYXRoKCcuLy8qJyArIGNvbmQs",
				"IGVsZW1lbnQpIDogW107CiAgfSA6IGZ1bmN0aW9uKGVsZW1lbnQsIGNsYXNzTmFtZSkgewogICAgY2xh",
				"c3NOYW1lID0gY2xhc3NOYW1lLnRvU3RyaW5nKCkuc3RyaXAoKTsKICAgIHZhciBlbGVtZW50cyA9IFtd",
				"LCBjbGFzc05hbWVzID0gKC9ccy8udGVzdChjbGFzc05hbWUpID8gJHcoY2xhc3NOYW1lKSA6IG51bGwp",
				"OwogICAgaWYgKCFjbGFzc05hbWVzICYmICFjbGFzc05hbWUpIHJldHVybiBlbGVtZW50czsKCiAgICB2",
				"YXIgbm9kZXMgPSAkKGVsZW1lbnQpLmdldEVsZW1lbnRzQnlUYWdOYW1lKCcqJyk7CiAgICBjbGFzc05h",
				"bWUgPSAnICcgKyBjbGFzc05hbWUgKyAnICc7CgogICAgZm9yICh2YXIgaSA9IDAsIGNoaWxkLCBjbjsg",
				"Y2hpbGQgPSBub2Rlc1tpXTsgaSsrKSB7CiAgICAgIGlmIChjaGlsZC5jbGFzc05hbWUgJiYgKGNuID0g",
				"JyAnICsgY2hpbGQuY2xhc3NOYW1lICsgJyAnKSAmJiAoY24uaW5jbHVkZShjbGFzc05hbWUpIHx8CiAg",
				"ICAgICAgICAoY2xhc3NOYW1lcyAmJiBjbGFzc05hbWVzLmFsbChmdW5jdGlvbihuYW1lKSB7CiAgICAg",
				"ICAgICAgIHJldHVybiAhbmFtZS50b1N0cmluZygpLmJsYW5rKCkgJiYgY24uaW5jbHVkZSgnICcgKyBu",
				"YW1lICsgJyAnKTsKICAgICAgICAgIH0pKSkpCiAgICAgICAgZWxlbWVudHMucHVzaChFbGVtZW50LmV4",
				"dGVuZChjaGlsZCkpOwogICAgfQogICAgcmV0dXJuIGVsZW1lbnRzOwogIH07CgogIHJldHVybiBmdW5j",
				"dGlvbihjbGFzc05hbWUsIHBhcmVudEVsZW1lbnQpIHsKICAgIHJldHVybiAkKHBhcmVudEVsZW1lbnQg",
				"fHwgZG9jdW1lbnQuYm9keSkuZ2V0RWxlbWVudHNCeUNsYXNzTmFtZShjbGFzc05hbWUpOwogIH07Cn0o",
				"RWxlbWVudC5NZXRob2RzKTsKCi8qLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0t",
				"LS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0qLwoKRWxlbWVudC5DbGFzc05hbWVzID0g",
				"Q2xhc3MuY3JlYXRlKCk7CkVsZW1lbnQuQ2xhc3NOYW1lcy5wcm90b3R5cGUgPSB7CiAgaW5pdGlhbGl6",
				"ZTogZnVuY3Rpb24oZWxlbWVudCkgewogICAgdGhpcy5lbGVtZW50ID0gJChlbGVtZW50KTsKICB9LAoK",
				"ICBfZWFjaDogZnVuY3Rpb24oaXRlcmF0b3IpIHsKICAgIHRoaXMuZWxlbWVudC5jbGFzc05hbWUuc3Bs",
				"aXQoL1xzKy8pLnNlbGVjdChmdW5jdGlvbihuYW1lKSB7CiAgICAgIHJldHVybiBuYW1lLmxlbmd0aCA+",
				"IDA7CiAgICB9KS5fZWFjaChpdGVyYXRvcik7CiAgfSwKCiAgc2V0OiBmdW5jdGlvbihjbGFzc05hbWUp",
				"IHsKICAgIHRoaXMuZWxlbWVudC5jbGFzc05hbWUgPSBjbGFzc05hbWU7CiAgfSwKCiAgYWRkOiBmdW5j",
				"dGlvbihjbGFzc05hbWVUb0FkZCkgewogICAgaWYgKHRoaXMuaW5jbHVkZShjbGFzc05hbWVUb0FkZCkp",
				"IHJldHVybjsKICAgIHRoaXMuc2V0KCRBKHRoaXMpLmNvbmNhdChjbGFzc05hbWVUb0FkZCkuam9pbign",
				"ICcpKTsKICB9LAoKICByZW1vdmU6IGZ1bmN0aW9uKGNsYXNzTmFtZVRvUmVtb3ZlKSB7CiAgICBpZiAo",
				"IXRoaXMuaW5jbHVkZShjbGFzc05hbWVUb1JlbW92ZSkpIHJldHVybjsKICAgIHRoaXMuc2V0KCRBKHRo",
				"aXMpLndpdGhvdXQoY2xhc3NOYW1lVG9SZW1vdmUpLmpvaW4oJyAnKSk7CiAgfSwKCiAgdG9TdHJpbmc6",
				"IGZ1bmN0aW9uKCkgewogICAgcmV0dXJuICRBKHRoaXMpLmpvaW4oJyAnKTsKICB9Cn07CgpPYmplY3Qu",
				"ZXh0ZW5kKEVsZW1lbnQuQ2xhc3NOYW1lcy5wcm90b3R5cGUsIEVudW1lcmFibGUpOwoKLyotLS0tLS0t",
				"LS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0t",
				"LS0tLS0tLSovCgooZnVuY3Rpb24oKSB7CiAgd2luZG93LlNlbGVjdG9yID0gQ2xhc3MuY3JlYXRlKHsK",
				"ICAgIGluaXRpYWxpemU6IGZ1bmN0aW9uKGV4cHJlc3Npb24pIHsKICAgICAgdGhpcy5leHByZXNzaW9u",
				"ID0gZXhwcmVzc2lvbi5zdHJpcCgpOwogICAgfSwKCiAgICBmaW5kRWxlbWVudHM6IGZ1bmN0aW9uKHJv",
				"b3RFbGVtZW50KSB7CiAgICAgIHJldHVybiBQcm90b3R5cGUuU2VsZWN0b3Iuc2VsZWN0KHRoaXMuZXhw",
				"cmVzc2lvbiwgcm9vdEVsZW1lbnQpOwogICAgfSwKCiAgICBtYXRjaDogZnVuY3Rpb24oZWxlbWVudCkg",
				"ewogICAgICByZXR1cm4gUHJvdG90eXBlLlNlbGVjdG9yLm1hdGNoKGVsZW1lbnQsIHRoaXMuZXhwcmVz",
				"c2lvbik7CiAgICB9LAoKICAgIHRvU3RyaW5nOiBmdW5jdGlvbigpIHsKICAgICAgcmV0dXJuIHRoaXMu",
				"ZXhwcmVzc2lvbjsKICAgIH0sCgogICAgaW5zcGVjdDogZnVuY3Rpb24oKSB7CiAgICAgIHJldHVybiAi",
				"IzxTZWxlY3RvcjogIiArIHRoaXMuZXhwcmVzc2lvbiArICI+IjsKICAgIH0KICB9KTsKCiAgT2JqZWN0",
				"LmV4dGVuZChTZWxlY3RvciwgewogICAgbWF0Y2hFbGVtZW50czogZnVuY3Rpb24oZWxlbWVudHMsIGV4",
				"cHJlc3Npb24pIHsKICAgICAgdmFyIG1hdGNoID0gUHJvdG90eXBlLlNlbGVjdG9yLm1hdGNoLAogICAg",
				"ICAgICAgcmVzdWx0cyA9IFtdOwoKICAgICAgZm9yICh2YXIgaSA9IDAsIGxlbmd0aCA9IGVsZW1lbnRz",
				"Lmxlbmd0aDsgaSA8IGxlbmd0aDsgaSsrKSB7CiAgICAgICAgdmFyIGVsZW1lbnQgPSBlbGVtZW50c1tp",
				"XTsKICAgICAgICBpZiAobWF0Y2goZWxlbWVudCwgZXhwcmVzc2lvbikpIHsKICAgICAgICAgIHJlc3Vs",
				"dHMucHVzaChFbGVtZW50LmV4dGVuZChlbGVtZW50KSk7CiAgICAgICAgfQogICAgICB9CiAgICAgIHJl",
				"dHVybiByZXN1bHRzOwogICAgfSwKCiAgICBmaW5kRWxlbWVudDogZnVuY3Rpb24oZWxlbWVudHMsIGV4",
				"cHJlc3Npb24sIGluZGV4KSB7CiAgICAgIGluZGV4ID0gaW5kZXggfHwgMDsKICAgICAgdmFyIG1hdGNo",
				"SW5kZXggPSAwLCBlbGVtZW50OwogICAgICBmb3IgKHZhciBpID0gMCwgbGVuZ3RoID0gZWxlbWVudHMu",
				"bGVuZ3RoOyBpIDwgbGVuZ3RoOyBpKyspIHsKICAgICAgICBlbGVtZW50ID0gZWxlbWVudHNbaV07CiAg",
				"ICAgICAgaWYgKFByb3RvdHlwZS5TZWxlY3Rvci5tYXRjaChlbGVtZW50LCBleHByZXNzaW9uKSAmJiBp",
				"bmRleCA9PT0gbWF0Y2hJbmRleCsrKSB7CiAgICAgICAgICByZXR1cm4gRWxlbWVudC5leHRlbmQoZWxl",
				"bWVudCk7CiAgICAgICAgfQogICAgICB9CiAgICB9LAoKICAgIGZpbmRDaGlsZEVsZW1lbnRzOiBmdW5j",
				"dGlvbihlbGVtZW50LCBleHByZXNzaW9ucykgewogICAgICB2YXIgc2VsZWN0b3IgPSBleHByZXNzaW9u",
				"cy50b0FycmF5KCkuam9pbignLCAnKTsKICAgICAgcmV0dXJuIFByb3RvdHlwZS5TZWxlY3Rvci5zZWxl",
				"Y3Qoc2VsZWN0b3IsIGVsZW1lbnQgfHwgZG9jdW1lbnQpOwogICAgfQogIH0pOwp9KSgpOwo="
				].join('\n');

				buff = new Buffer(prototype_base64_blob, 'base64').toString('ascii');
				fs.writeFileSync(save_file, buff);
				break;

			default: console.log(help);
		}
		break;
	default: console.log(help);
}
